/**
 * PathData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.demographics;

import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.general.GeneralInfo;

/**
 * JavaBean that represents a path of categories from a last level category to a
 * root category, crossing categories from every level in its definition. The
 * purpose of this concept is to provide the information to the system about
 * which geographic location should be used in case of an ambiguity.
 */
@Entity
@Table(name = "path", uniqueConstraints = { @UniqueConstraint(columnNames = "path_code") })
public class PathData extends GeneralInfo implements Comparable<PathData> {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/**
	 * This path's representative name.
	 */
	private String name;

	/** The custom code. */
	private String customCode;

	/**
	 * The set of categories that define this path.
	 */
	private Set<CategoryData> categories;

	/**
	 * The set of districts related to this path.
	 */
	private Set<DistrictData> districts;

	/**
	 * <p>
	 * Whether this path is the system's default.
	 * </p>
	 * <p>
	 * The "default path" corresponds to one single category-hierarchy set from
	 * the lowest level to the highest level possible that the application will
	 * use for general geographical location information.
	 * </p>
	 * <p>
	 * Only on default path may be defined in the data.
	 * </p>
	 */
	private boolean defaultPath;

	/**
	 * Getter of <code>code</code> property.
	 * 
	 * @return <code>code</code> property value.
	 * @see #code
	 */
	@Override
	@Id
	@Column(name = "path_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_path")
	@SequenceGenerator(name = "s_path", sequenceName = "s_path", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/**
	 * Getter of <code>name</code> property.
	 * 
	 * @return <code>name</code> property value.
	 * @see #name
	 */
	@Override
	@Column(name = "path_name", nullable = false, length = 640)
	public String getName() {
		return this.name;
	}

	/**
	 * Setter of <code>name</code> property.
	 * 
	 * @param name
	 *            The <code>name</code> to set.
	 * @see #name
	 */
	@Override
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Getter of <code>customCode</code> property.
	 * 
	 * @return <code>customCode</code> property value.
	 * @see #customCode
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "path_custom_code", nullable = false, length = 80)
	public String getCustomCode() {
		return this.customCode;
	}

	/**
	 * Setter of <code>customCode</code> property.
	 * 
	 * @param customCode
	 *            The <code>customCode</code> to set.
	 * @see #customCode
	 */
	@Override
	public void setCustomCode(String customCode) {
		this.customCode = customCode;
	}

	/**
	 * Getter of <code>categories</code> property.
	 * 
	 * @return <code>categories</code> property value.
	 * @see #categories
	 */
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = "category_path", inverseJoinColumns = @JoinColumn(name = "category_code", referencedColumnName = "category_code"), joinColumns = @JoinColumn(name = "path_code", referencedColumnName = "path_code"))
	public Set<CategoryData> getCategories() {
		return this.categories;
	}

	/**
	 * Setter of <code>categories</code> property.
	 * 
	 * @param categories
	 *            The <code>categories</code> to set.
	 * @see #categories
	 */
	public void setCategories(Set<CategoryData> categories) {
		this.categories = categories;
	}

	/**
	 * Getter of <code>districts</code> property.
	 * 
	 * @return <code>districts</code> property value.
	 * @see #districts
	 */

	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = "district_path", inverseJoinColumns = @JoinColumn(name = "district_code", referencedColumnName = "district_code"), joinColumns = @JoinColumn(name = "path_code", referencedColumnName = "path_code"))
	public Set<DistrictData> getDistricts() {
		return this.districts;
	}

	/**
	 * Setter of <code>districts</code> property.
	 * 
	 * @param districts
	 *            The <code>districts</code> to set.
	 * @see #districts
	 */
	public void setDistricts(Set<DistrictData> districts) {
		this.districts = districts;
	}

	/**
	 * Getter of <code>defaultPath</code> property.
	 * 
	 * @return <code>defaultPath</code> property value.
	 * @see #defaultPath
	 */
	@Column(name = "default_path", nullable = false)
	public boolean isDefaultPath() {
		return this.defaultPath;
	}

	/**
	 * Setter of <code>defaultPath</code> property.
	 * 
	 * @param thePath
	 *            The <code>defaultPath</code> to set.
	 * @see #defaultPath
	 */
	public void setDefaultPath(boolean thePath) {
		this.defaultPath = thePath;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.domain.BaseDataObject#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object anObject) {
		if (anObject instanceof PathData) {
			PathData thePathData = (PathData) anObject;
			return thePathData.code == this.code;
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.domain.BaseDataObject#hashCode()
	 */
	@Override
	public int hashCode() {
		return super.hashCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(PathData arg0) {
		return 0;
	}

}
