/*
 * DeviceParameterData.java
 * 
 * Copyright (c) 2022 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.device;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;

import com.smartmatic.waffle.domain.BaseDataObject;
import com.smartmatic.waffle.lang.annotations.basic.EqualsFieldAnnotation;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamConverter;
import com.thoughtworks.xstream.converters.extended.ToAttributedValueConverter;

/**
 * Aspect or feature configurable in Voting Devices.
 * <p>
 * Once established or configured, the value given to a {@link
 * DeviceParameterData DeviceParameter} applies to all deployed
 * Voting Devices during an election event.
 *
 * @author  Carlos Chacon (carlos.chacon@smartmatic.com)
 * @version 1.0.1 (2022/07/13)
 */
@Entity
@Table(name = "device_parameter")
@XStreamAlias("parameter")
@XStreamConverter(value = ToAttributedValueConverter.class, strings = "value")
public class DeviceParameterData extends BaseDataObject {

    /**
     * serialVersionUID.
     */
    private static final long serialVersionUID = 4753714809456354730L;


    /**
     * Name that identifies uniquely this {@link DeviceParameterDate
     * DataDeviceParameter}. 
     */
    @Id
    @Column(name = "name", nullable = false)
    @EqualsFieldAnnotation
    @XStreamAsAttribute
    private String name;

    /**
     * Value associated to this {@link DeviceParameterData DataDeviceParameter}. 
     */
    @Column(name = "value", nullable = false)
    private String value;


    /**
     * No Arguments Class Constructor.
     */
    public DeviceParameterData() {
        super();
    }


    /**
     * Class Constructor.
     *
     * @param name
     *      the name of the parameter
     * @param value
     *      the value associated to the parameter
     */
    public DeviceParameterData(String name, String value) {
        this();
        this.name  = name;
        this.value = value;
    }


    /**
     * Returns a {@code Map} that contains a group of default
     * {@link DeviceParameterData DeviceParameter} instances.
     * <p>
     * By "default" we're referring to a base configuration
     * defined by the System for a given {@code DeviceParameter}.
     *
     * @return
     *      A map of tuples {@code (K,V)} where {@code K} is the name of
     *      {@link DeviceParameterData} and {@code V} is the associated
     *      {@link DeviceParameterData}. This map will never be {@code null}
     */
    public static Map<String, DeviceParameterData> defaultInstances() {
        final Map<String, DeviceParameterData> defs;

        defs = new HashMap<String, DeviceParameterData>();

        defs.put(
            "hand-scanner-use-enabled",
            new DeviceParameterData("hand-scanner-use-enabled", "false")
        );
        return defs;
    }


    /**
     * Returns the parameter's name.
     *
     * @return
     *      {@code String}
     */
    public String getName() {
        return this.name;
    }


    /**
     * Returns the parameter's value.
     *
     * @return
     *      {@code String}
     */
    public String getValue() {
        return this.value;
    }


    /**
     * Returns always zero.
     * <p>
     * This entity does not support such concurrency mechanism.
     */
    @Override
    public int getVersion() {
        return 0;
    }


    /**
     * Configures the name of this {@link DeviceParameterData parameter}.
     *
     * @param name
     *      the parameter's name
     */
    public void setName(String name) {
        this.name = name;
    }


    /**
     * Configures the value of this {@link DeviceParameterData parameter}.
     *
     * @param value
     *      the parameter's value
     */
    public void setValue(String value) {
        this.value = value;
    }



    /**
     * Common {@link Comparator Comparators} for sorting {@link
     * DeviceParameterData DeviceParameters}.
     */
    public static final class Comparators {
        /**
         * Compares {@link DeviceParameterData DeviceParameters} by their
         * {@link DeviceParameterData#getName() names} in ascending order.
         */
        public static final Comparator<DeviceParameterData> NAME_ASC =
            new Comparator<DeviceParameterData>() {

                @Override
                public int compare(
                    DeviceParameterData o1, DeviceParameterData o2
                ) {
                String name1 = o1.getName();
                String name2 = o2.getName();

                return name1.compareTo(name2);
            }
        };

        /**
         * No Arguments Class Constructor.
         */
        private Comparators() {
            super();
        }
    }
}
