/**
 * PostProcessing.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.demographics.VotingBoothData;
import com.smartmatic.waffle.domain.device.LogicalVotingGroupData;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.log.SMTTLogger;

/***
 * Class used to execute the post load process.<br>
 * This class must be used after the load data has been done.
 */
public class PostProcessing {

	/** For log messages purpose. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(PostProcessing.class);

	/**
	 * Method to download ECM report from data base to server file system.
	 * 
	 * @throws BaseException
	 *             The exception is thrown when there is an error executing the
	 *             post load process.
	 */
	public static void downloadReports() throws Exception {
		try {
			logger.info("Generating reports");
			ECMServiceProvider.downloadECMReports();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw e;
		}
	}

	/**
	 * Method to fill the language combinations table.
	 * 
	 * @throws BaseException
	 *             - The exception is thrown when there is an error executing
	 *             the post load process.
	 */
	public static void fillLanguageCombinations() throws Exception {
		try {
			logger.info("Generating language combinations");
			ECMServiceProvider.fillLanguageCombinations();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw e;
		}
	}

	/**
	 * Sets the default application key.
	 *
	 * @throws Exception
	 *             the exception
	 */
	public static void setDefaultApplicationKey() throws Exception {
		try {
			logger.info("Setting the default application key");
			ECMServiceProvider.setDefaultApplicationKey();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw e;
		}
	}

	/**
	 * Method used to generate the logical voting groups based on the data
	 * loaded into the system. The logical voting groups created are saved into
	 * database.
	 * 
	 * @throws Exception
	 *             The exception is thrown when there is an error executing the
	 *             post load process.
	 */
	public static void generateLogicalVotingGroup() throws Exception {

		try {
			List<RegionData> regionList = ECMServiceProvider
					.getLastLevelRegionsLight(null);

			List<Map<String, LogicalVotingGroupData>> logicalVotingGroupsDataMapList = new ArrayList<Map<String, LogicalVotingGroupData>>();

			for (RegionData regionIndex : regionList) {

				logger.info("Generating logical voting groups for: "
						+ regionIndex.getRegionName());

				logicalVotingGroupsDataMapList.add(ECMServiceProvider
						.postLoadProcess(regionIndex));

			}

			Map<String, LogicalVotingGroupData> logicalVotingGroupsFinalMap = new HashMap<String, LogicalVotingGroupData>();

			for (Map<String, LogicalVotingGroupData> map : logicalVotingGroupsDataMapList) {

				Collection<LogicalVotingGroupData> LogicalVotingGroupCollection = map
						.values();

				for (LogicalVotingGroupData logicalVotingGroupData : LogicalVotingGroupCollection) {

					logicalVotingGroupsFinalMap
							.putAll(addLogicalVotingGroup(
									logicalVotingGroupData,
									logicalVotingGroupsFinalMap));
				}
			}

			logger.info("Persisting logical voting groups");

			ECMServiceProvider
					.persistLogicalVotingGroup(logicalVotingGroupsFinalMap);

			logger.info("Logical voting groups generation done");
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw e;
		}
	}

	/**
	 * Adds the logical voting group generated to a map
	 * 
	 * @param logicalVotingGroupData
	 *            the logical voting group generated
	 * @param logicalVotingGroupMap
	 *            map to add the logical voting group generated
	 * @return map with the the logical voting group added
	 */
	private static Map<String, LogicalVotingGroupData> addLogicalVotingGroup(
			LogicalVotingGroupData logicalVotingGroupData,
			Map<String, LogicalVotingGroupData> logicalVotingGroupMap) {

		if (logicalVotingGroupMap.containsKey(logicalVotingGroupData
				.getCustomCode()) == false) {

			Set<VotingBoothData> votingBoothSet = new HashSet<VotingBoothData>();
			votingBoothSet.addAll(logicalVotingGroupData.getVotingBooths());

			logicalVotingGroupData.setVotingBooths(votingBoothSet);

			logicalVotingGroupMap.put(logicalVotingGroupData.getCustomCode(),
					logicalVotingGroupData);
		} else {
			LogicalVotingGroupData newlogicalVotingGroupData = logicalVotingGroupMap
					.get(logicalVotingGroupData.getCustomCode());

			Set<VotingBoothData> votingBoothSet = newlogicalVotingGroupData
					.getVotingBooths();

			votingBoothSet.addAll(logicalVotingGroupData.getVotingBooths());

			newlogicalVotingGroupData.setVotingBooths(votingBoothSet);

			logicalVotingGroupMap.put(logicalVotingGroupData.getCustomCode(),
					newlogicalVotingGroupData);
		}
		return logicalVotingGroupMap;
	}
}
