/*
 * BulkUploadSmttFilesController.java
 *
 * 
 * Copyright (c) 2022 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui;

import java.util.List;

import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener;
import com.smartmatic.waffle.ecm.client.worker.BulkUploadSmttFilesWorker;
import com.smartmatic.waffle.ecm.data.transfer.objects.FileToUploadDTO;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException;

/**
 * Controller class that creates and starts the worker thread to execute the
 * bulk upload process of Smartmatic files.
 */
public class BulkUploadSmttFilesController implements IAwtOperationListener {

    /**
     * Input list with files to load.
     */
    private List<FileToUploadDTO> files;

    /**
     * BulkUploadSmttFilesPanel instance.
     */
    private BulkUploadSmttFilesPanel panel;

    /**
     * Hold the byte array with the resources file.
     */
    private byte[] resources;

    /**
     * BulkUploadSmttFilesWorker instance.
     */
    private BulkUploadSmttFilesWorker worker;


    /**
     * Instantiates a new BulkUploadSmttFilesController.
     * 
     * @param panel
     *      BulkUploadSmttFilesPanel instance
     * @param files
     *      Input list with files to load
     * @param resources
     *      byte array with the resources file
     */
    public BulkUploadSmttFilesController(
        BulkUploadSmttFilesPanel panel, List<FileToUploadDTO> files,
        byte[] resources
    ) {
        super();
        this.files     = files;
        this.panel     = panel;
        this.resources = resources;
    }


    /**
     * Method that starts the worker that does the real backend functionality
     * work.
     */
    public void startWorker() {
        Thread wThread;     // Worker's Thread

        this.worker = new BulkUploadSmttFilesWorker(this.files, this.resources);
        this.worker.addListener(this);

        wThread = new Thread(this.worker, "BulkUploadSmttFilesWorkerThread");
        wThread.start();
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public void awtOperation(AwtOperationType awtOperationType) {
        int processed;      // Amount of processed files
        int total;          // Total of files to process
        String fbp;         // fbp: File being processed

        switch (awtOperationType) {
            case REPORT_ERROR_IN_PROCESS:
                this.panel.showErrorMessageBadlyConfiguredAudioVoting();
                break;

            case SHOW_ERROR_MESSAGE:
                showErrorMessageOperation();
                break;

            case SHOW_RESULT_MESSAGE:
                this.panel.finishBulkUploadProcess(
                    this.worker.getFileUploadedDetailList()
                );
                break;

            case UPDATE_COMPONENTS:
                processed = this.worker.getNumberOfProcessedFiles() + 1;
                total     = this.files.size();
                fbp       = this.worker.getFileBeingProcessed();

                this.panel.updateStatusDuringBulkUploadOperation(
                    processed, total, fbp
                );
                break;

            case UPDATE_PROGRESS_BAR:
                processed = this.worker.getNumberOfProcessedFiles();
                this.panel.updateProgressBar(processed);
                break;

            default:
                break;
        }
    }


    /**
     * Method used to control the logic when an active worker fires
     * the SHOW_ERROR_MESSAGE operation.
     */
    private void showErrorMessageOperation() {
        BulkDataLoadException ex = this.worker.getFailure();
        this.panel.showErrorMessageFor(ex, this.worker.getFileBeingProcessed());
    }
}
