/**
 * IntegrityPanel.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client.ui;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JFileChooser;
import javax.swing.JTable;
import javax.swing.table.TableModel;

import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.jdesktop.application.ResourceMap;

import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.core.pendrive.util.PendriveUtil;
import com.smartmatic.waffle.ecm.data.transfer.objects.QueryResultDTO;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;
import com.smartmatic.waffle.ecm.ui.util.ECMOptionPane;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * The Class is used to load and draw all the necessary components for the
 * Integrity Panel.
 */
public class IntegrityPanel extends javax.swing.JPanel {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -3878215364274631197L;

	/** The language controller. */
	private LanguageController langController = LanguageController
			.getInstance();

	/**
	 * Localized bundle containing the application messages.
	 */
	private ResourceMap resourceMap = LanguageController.getInstance()
			.getResourceMap();

	/** Localized resource bundle containing the application errors. */
	private ResourceBundle errorsBundle = ResourceBundle
			.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMErrors");

	/** The logger. */
	private SMTTLogger logger = SMTTLogger.getLogger(IntegrityPanel.class);

	/** The queries. */
	private List<String> queries;

	/**
	 * Creates a new instance of IntegrityPanel.
	 */
	public IntegrityPanel() {
		initComponents();
		postInit();

	}

	/**
	 * Performs operations after new class instantiation.
	 */
	public void postInit() {
		this.exportResultsButton.setEnabled(false);
		this.queries = new ArrayList<String>();
		this.loadQueries();
	}

	/**
	 * Blocks controls while is performing an operation.
	 */
	public void blockControls() {
		this.executeQueryButton.setEnabled(false);
		this.cleanResultButton.setEnabled(false);
		this.sqlQueryPane.setEnabled(false);
	}

	/**
	 * Unblocks controls once the operation has been completed.
	 */
	public void unBlockControls() {
		this.executeQueryButton.setEnabled(true);
		this.cleanResultButton.setEnabled(true);
		this.sqlQueryPane.setEnabled(true);
	}

	/**
	 * Activates status bar.
	 */
	public void activateStatusBar() {
		this.mainProgressBar.setIndeterminate(true);
	}

	/**
	 * Deactivates status bar.
	 */
	public void deactivateStatusBar() {
		this.mainProgressBar.setIndeterminate(false);
	}

	/**
	 * Shows an information message to the user.
	 * 
	 * @param errorBundleKey
	 *            The error bundle key to fetch the message.
	 */
	public void showMessage(String bundleKey) {

		ECMOptionPane.showMessageDialog(this, this.resourceMap
				.getString(bundleKey), this.resourceMap
				.getString("IntegrityPanel.title.message.warning"),
				ECMOptionPane.INFORMATION_MESSAGE,
				ECMOptionPane.INFORMATION_DIALOG);
	}

	/**
	 * Shows an error message to the user.
	 * 
	 * @param errorBundleKey
	 *            The error bundle key to fetch the message.
	 */
	public void showErrorMessage(String errorBundleKey) {

		ECMOptionPane.showMessageDialog(this, this.errorsBundle
				.getString(errorBundleKey), this.resourceMap
				.getString("IntegrityPanel.title.message.error"),
				ECMOptionPane.ERROR_MESSAGE, ECMOptionPane.ERROR_DIALOG);
	}

	/**
	 * Method used to fill the results table with the query data.
	 * 
	 * @param resultQuery
	 *            The result query object.
	 */
	@SuppressWarnings("serial")
	public void showResult(QueryResultDTO resultQuery) {

		if (resultQuery != null && resultQuery.getRows() != null) {

			this.resultsTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
			this.resultsTable.setModel(new javax.swing.table.DefaultTableModel(
					resultQuery.getRows(), resultQuery.getColumnsNames()) {
				@Override
				public boolean isCellEditable(int rowIndex, int columnIndex) {
					return false;
				}
			});

			this.resultsTable.repaint();
			this.exportResultsButton.setEnabled(true);
		} else {
			showMessage("IntegrityPanel.no_result.message");
			this.exportResultsButton.setEnabled(false);
		}
	}

	/**
	 * Exports the results table.
	 * 
	 * @param table
	 *            The results table to export.
	 * @param file
	 *            The destination file.
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public void exportTable(JTable table, File file) throws IOException {
		FileWriter out = new FileWriter(file);
		try {
			TableModel model = table.getModel();
			for (int i = 0; i < model.getColumnCount(); i++) {
				out.write(model.getColumnName(i) + "\t");
			}
			out.write("\n");

			for (int i = 0; i < model.getRowCount(); i++) {
				for (int j = 0; j < model.getColumnCount(); j++) {
					if (model.getValueAt(i, j) != null) {
						out.write(model.getValueAt(i, j).toString() + "\t");
					} else {
						out.write(" \t");
					}
				}
				out.write("\n");
			}
		} finally {
			out.flush();
			out.close();
		}
	}

	/**
	 * Loads necessaries queries to integrity check from a XML file.
	 */
	public void loadQueries() {
		try {
			SAXReader saxReader = new SAXReader();
			Document document = saxReader.read(SAESCommonDestinations
					.getInstance().integrityFile);
			List<Node> nodes = document.selectNodes("//integrity-check/query");
			List<String> names = new ArrayList<String>();
			names.add("---");
			for (Node indexNode : nodes) {
				LanguageData selectedLanguage = this.langController
						.getSelectedLanguage();
				String language = indexNode.valueOf("@language");
				if ((language != null) && (!language.isEmpty())
						&& (language.equals(selectedLanguage.getLanguageRFC()))) {
					String name = indexNode.valueOf("@name");
					Element elem = (Element) indexNode;
					String data = (String) elem.getData();
					if ((data != null) && (!data.trim().isEmpty())
							&& (name != null) && (!name.trim().isEmpty())) {
						names.add(name);
						this.queries.add(data.trim());
					}
				}
			}
			this.queryComboBox.setModel(new javax.swing.DefaultComboBoxModel(
					names.toArray(new String[names.size()])));
		} catch (Exception e) {
			this.logger.error(e,
					"There is an error loading queries from xml file");
		}
	}

	/**
	 * Sets the selected query.
	 */
	private void setSelectedQuery() {
		int index = this.queryComboBox.getSelectedIndex();
		if (index > 0) {
			String query = this.queries.get(index - 1);
			this.sqlQueryPane.setText(query);
		} else {
			this.sqlQueryPane.setText("");
		}
	}

	/**
	 * Executes the selected query.
	 * 
	 * @param evt
	 *            The Action event that triggers this method.
	 */
	private void executeQuery(java.awt.event.ActionEvent evt) {
		String message;
		String title = this.resourceMap
				.getString("IntegrityPanel.title.message.warning");
		String query = this.sqlQueryPane.getText();

		if (query != null && !query.isEmpty()) {
			message = this.resourceMap
					.getString("IntegrityPanel.execute.query.confirm");
			int option = ECMOptionPane.showConfirmDialog(this, message, title,
					ECMOptionPane.YES_NO_OPTION, ECMOptionPane.CONFIRM_DIALOG);

			if (option == 0) {
				this.cleanTableResult();

				IntegrityValidationExecutorController integrityValidationController = new IntegrityValidationExecutorController(
						this, query);
				integrityValidationController.startWorker();

			}

		} else {
			message = this.resourceMap
					.getString("IntegrityPanel.No.empty.query");
			ECMOptionPane
					.showMessageDialog(this, message, title,
							ECMOptionPane.WARNING_MESSAGE,
							ECMOptionPane.WARNING_DIALOG);

		}
	}

	/**
	 * Exports the results table to a file.
	 * 
	 * @param evt
	 *            The Action event that triggers this method.
	 */
	private void exportFile(java.awt.event.ActionEvent evt) {
		int option = this.exportResultsFileChooser.showSaveDialog(this);

		if (option == JFileChooser.APPROVE_OPTION) {

			ClientCommon
					.saveFileChooserLocationPreference(this.exportResultsFileChooser
							.getCurrentDirectory().getAbsolutePath());

			File fileToSave = this.exportResultsFileChooser.getSelectedFile();

			try {
				this.exportTable(this.resultsTable, fileToSave);

				PendriveUtil.sync();

				ECMOptionPane
						.showMessageDialog(
								IntegrityPanel.this,
								this.resourceMap
										.getString("IntegrityPanel.export.result.successfull"),
								this.resourceMap
										.getString("NormalMessage.title"),
								ECMOptionPane.INFORMATION_MESSAGE,
								ECMOptionPane.INFORMATION_DIALOG);

			} catch (Exception e) {
				ECMOptionPane
						.showMessageDialog(
								IntegrityPanel.this,
								this.resourceMap
										.getString("IntegrityPanel.export.result.error"),
								this.resourceMap
										.getString("ErrorMessage.title"),
								ECMOptionPane.ERROR_MESSAGE,
								ECMOptionPane.ERROR_DIALOG);
				this.logger.error(e,
						"There is an error exporting the result file");
			}
		}

	}

	/**
	 * Selects an item from the query list.
	 * 
	 * @param evt
	 *            The Action event that triggers this method.
	 */
	private void selectItem(java.awt.event.ActionEvent evt) {
		setSelectedQuery();
	}

	/**
	 * Resets the executed query table, query and selected item.
	 * 
	 * @param evt
	 *            The Action event that triggers this method.
	 */
	private void resetQueryComponents(java.awt.event.ActionEvent evt) {
		this.cleanTableResult();
		this.sqlQueryPane.setText(null);
		this.queryComboBox.setSelectedIndex(0);
	}

	/**
	 * Cleans the result table.
	 */
	private void cleanTableResult() {
		this.resultsTable.setModel(new javax.swing.table.DefaultTableModel(
				new Object[][] {

				}, new String[] {

				}));
		this.resultsTable.repaint();
		this.exportResultsButton.setEnabled(false);
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		this.exportResultsFileChooser = new javax.swing.JFileChooser();
		this.mainPanel = new javax.swing.JPanel();
		this.resultsScrollPane = new javax.swing.JScrollPane();
		this.sqlQueryPane = new javax.swing.JTextPane();
		this.executeQueryButton = new javax.swing.JButton();
		this.cleanResultButton = new javax.swing.JButton();
		this.mainScrollPane = new javax.swing.JScrollPane();
		this.resultsTable = new javax.swing.JTable();
		this.mainProgressBar = new javax.swing.JProgressBar();
		this.queryLabel = new javax.swing.JLabel();
		this.queryComboBox = new javax.swing.JComboBox();
		this.exportResultsButton = new javax.swing.JButton();
		this.mainLayeredPane = new javax.swing.JLayeredPane();
		this.functionalityIcon = new javax.swing.JLabel();
		this.functionalityTitle = new javax.swing.JLabel();
		this.functionalityDescriptionLabel = new javax.swing.JLabel();
		this.bannerBackgroundLabel = new javax.swing.JLabel();

		ClientCommon.initFileChooserLocation(this.exportResultsFileChooser);

		this.mainPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder(""));

		this.resultsScrollPane.setViewportView(this.sqlQueryPane);

		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N
		this.executeQueryButton.setText(bundle
				.getString("IntegrityPanel.executeQuery.text")); // NOI18N
		this.executeQueryButton.setName("buttonExecuteQuery");
		this.executeQueryButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						executeQuery(evt);
					}
				});

		this.cleanResultButton.setText(bundle
				.getString("IntegrityPanel.cleanResult.text")); // NOI18N
		this.cleanResultButton.setName("buttonCleanResult");
		this.cleanResultButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						resetQueryComponents(evt);
					}
				});

		this.mainScrollPane.setAutoscrolls(true);

		this.resultsTable.setModel(new javax.swing.table.DefaultTableModel(
				new Object[][] {

				}, new String[] {

				}));
		this.resultsTable
				.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_ALL_COLUMNS);
		this.resultsTable.setFillsViewportHeight(true);
		this.resultsTable.getTableHeader().setReorderingAllowed(false);
		this.mainScrollPane.setViewportView(this.resultsTable);

		this.queryLabel
				.setText(bundle.getString("IntegrityPanel.jLabel1.text")); // NOI18N
		this.queryLabel.setName("labelQuery");

		this.queryComboBox.setModel(new javax.swing.DefaultComboBoxModel(
				new String[] { "----" }));
		this.queryComboBox.setName("comboQuerySelection");
		this.queryComboBox
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						selectItem(evt);
					}
				});

		this.exportResultsButton.setText(bundle
				.getString("IntegrityPanel.jButton1.text")); // NOI18N
		this.exportResultsButton.setName("buttonExport");
		this.exportResultsButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						exportFile(evt);
					}
				});

		javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(
				this.mainPanel);
		this.mainPanel.setLayout(jPanel1Layout);
		jPanel1Layout
				.setHorizontalGroup(jPanel1Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jPanel1Layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												this.resultsScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												924, Short.MAX_VALUE)
										.addContainerGap())
						.addGroup(
								jPanel1Layout
										.createSequentialGroup()
										.addGap(231, 231, 231)
										.addComponent(
												this.executeQueryButton,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												154, Short.MAX_VALUE)
										.addGap(139, 139, 139)
										.addComponent(
												this.cleanResultButton,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												155, Short.MAX_VALUE)
										.addGap(269, 269, 269))
						.addGroup(
								jPanel1Layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												this.mainProgressBar,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												924, Short.MAX_VALUE)
										.addContainerGap())
						.addGroup(
								jPanel1Layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(this.queryComboBox, 0,
												735, Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.queryLabel,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												171,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addGap(18, 18, 18))
						.addGroup(
								jPanel1Layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												this.mainScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												924, Short.MAX_VALUE)
										.addContainerGap())
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								jPanel1Layout
										.createSequentialGroup()
										.addGap(409, 409, 409)
										.addComponent(
												this.exportResultsButton,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.addGap(385, 385, 385)));
		jPanel1Layout
				.setVerticalGroup(jPanel1Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								jPanel1Layout
										.createSequentialGroup()
										.addGap(34, 34, 34)
										.addGroup(
												jPanel1Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.queryComboBox,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.queryLabel,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																28,
																Short.MAX_VALUE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.resultsScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												124, Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												jPanel1Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.executeQueryButton)
														.addComponent(
																this.cleanResultButton))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(
												this.mainProgressBar,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(
												this.mainScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												247, Short.MAX_VALUE)
										.addGap(13, 13, 13)
										.addComponent(
												this.exportResultsButton,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												30, Short.MAX_VALUE)
										.addContainerGap()));

		this.functionalityIcon
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/integrity_check.png"))); // NOI18N
		this.functionalityIcon.setBounds(10, 10, 120, 90);
		this.mainLayeredPane.add(this.functionalityIcon,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.functionalityTitle
				.setFont(new java.awt.Font("DejaVu Sans", 1, 14));
		this.functionalityTitle.setText(bundle
				.getString("IntegrityPanel.file_conversion_title.text")); // NOI18N
		this.functionalityTitle.setBounds(150, 40, 386, 17);
		this.functionalityTitle.setName("labelIntegrityCheckTitle");

		this.mainLayeredPane.add(this.functionalityTitle,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.functionalityDescriptionLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 0, 11));
		this.functionalityDescriptionLabel.setText(bundle
				.getString("IntegrityPanel.file_conversion_desc.text")); // NOI18N
		this.functionalityDescriptionLabel.setBounds(180, 60, 670, 14);
		this.functionalityDescriptionLabel
				.setName("labelIntegrityCheckDescription");

		this.mainLayeredPane.add(this.functionalityDescriptionLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.bannerBackgroundLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/barra titulo subtitulo.jpg"))); // NOI18N
		this.bannerBackgroundLabel.setText(bundle
				.getString("IntegrityPanel.banner_bg.text")); // NOI18N
		this.bannerBackgroundLabel.setPreferredSize(new java.awt.Dimension(
				1440, 120));
		this.bannerBackgroundLabel.setBounds(0, 0, 1440, 120);
		this.bannerBackgroundLabel.setName("labelBannerBG");

		this.mainLayeredPane.add(this.bannerBackgroundLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(this.mainLayeredPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 984,
						Short.MAX_VALUE)
				.addGroup(
						layout.createSequentialGroup()
								.addGap(12, 12, 12)
								.addComponent(this.mainPanel,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE).addGap(12, 12, 12)));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				layout.createSequentialGroup()
						.addComponent(this.mainLayeredPane,
								javax.swing.GroupLayout.PREFERRED_SIZE, 120,
								javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGap(13, 13, 13)
						.addComponent(this.mainPanel,
								javax.swing.GroupLayout.DEFAULT_SIZE,
								javax.swing.GroupLayout.DEFAULT_SIZE,
								Short.MAX_VALUE).addGap(31, 31, 31)));
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables

	private javax.swing.JLabel bannerBackgroundLabel;

	private javax.swing.JButton cleanResultButton;

	private javax.swing.JButton executeQueryButton;

	private javax.swing.JLabel functionalityDescriptionLabel;

	private javax.swing.JLabel functionalityIcon;

	private javax.swing.JLabel functionalityTitle;

	private javax.swing.JButton exportResultsButton;

	private javax.swing.JComboBox queryComboBox;

	private javax.swing.JFileChooser exportResultsFileChooser;

	private javax.swing.JLabel queryLabel;

	private javax.swing.JLayeredPane mainLayeredPane;

	private javax.swing.JPanel mainPanel;

	private javax.swing.JProgressBar mainProgressBar;

	private javax.swing.JScrollPane mainScrollPane;

	private javax.swing.JScrollPane resultsScrollPane;

	private javax.swing.JTable resultsTable;

	private javax.swing.JTextPane sqlQueryPane;

}