package com.smartmatic.waffle.ecm.client.ui;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.Duration;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener;
import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.client.util.FileVerificationUtils;
import com.smartmatic.waffle.ecm.client.worker.ConnectedDriveFinderWorker;
import com.smartmatic.waffle.ecm.client.worker.CopyOSWorker;
import com.smartmatic.waffle.ecm.client.worker.FileIntegrityVerificationWorker;
import com.smartmatic.waffle.ecm.client.worker.UnmountingWorker;
import com.smartmatic.waffle.ecm.client.worker.data.FileBackup;
import com.smartmatic.waffle.ecm.data.transfer.type.USBGenerationType;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager.MountedDrive;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * 
 * Abstract class that holds the common logic among the USB generation
 * controllers.
 */
public abstract class USBGenerationController implements IAwtOperationListener {

	private static SMTTLogger logger = SMTTLogger
			.getLogger(USBGenerationController.class);

	private GUIGenerationInterface generationPanel;

	private ConnectedDriveFinderWorker connectedDriveFinderWorker;

	private UnmountingWorker unmountingWorker;

	private CopyOSWorker copyOSWorker;

	private FileIntegrityVerificationWorker fileIntegrityVerificationWorker;

	/**
	 * Boolean that holds if the generation is being performed in test mode.
	 */
	protected boolean testMode;

	/**
	 * Boolean that holds if the finding drive worker is running.
	 */
	protected boolean isFindingDrive;

	/**
	 * Boolean that holds if the copying OS worker is running.
	 */
	protected boolean isCopyingOS;

	/**
	 * Boolean that holds if the preparing contests worker is running.
	 */
	protected boolean isPreparingContests;

	/**
	 * Boolean that holds if the generating files worker is running.
	 */
	protected boolean isGeneratingFiles;

	/**
	 * Boolean that holds if the files verification worker is running.
	 */
	protected boolean isVerifyingFiles;

	/**
	 * Boolean that holds if drive is Unmounting
	 */
	protected boolean isUnmountingDrive;

	/**
	 * Holds generation start time.
	 */
	protected long generationStartTime;

	/**
	 * Memory mounted path
	 */
	protected String mountedPath;

	/**
	 * The path to the device which will be used for the generation.
	 */
	protected String devicePath;

	public USBGenerationController(GUIGenerationInterface generationPanel) {

		this.generationPanel = generationPanel;
		resetOperationStatus();

		this.testMode = Boolean.getBoolean("test_mode");
	}

	/**
	 * Sets all the operation status to false.
	 */
	protected void resetOperationStatus() {
		this.isFindingDrive = false;
		this.isCopyingOS = false;
		this.isPreparingContests = false;
		this.isGeneratingFiles = false;
		this.isVerifyingFiles = false;
		this.isUnmountingDrive = false;
		FileVerificationUtils.deleteTemporaryPath();
	}

	/**
	 * Method used to start the connected drive finder worker thread.
	 */
	public void startDriveFinderWorker() {

		if (this.testMode) {
			startProcessTestMode();
		} else {
			this.isFindingDrive = true;

			this.connectedDriveFinderWorker = new ConnectedDriveFinderWorker();
			this.connectedDriveFinderWorker.addListener(this);

			Thread connectedDriveFinderWorkerThread = new Thread(
					this.connectedDriveFinderWorker,
					"ConnectedDriveFinderWorkerThread");
			connectedDriveFinderWorkerThread.start();
		}
	}

	/**
	 * Method used to stop the drive finder worker current execution.
	 */
	public void stopDriveFinderWorker() {
		if (this.connectedDriveFinderWorker != null) {
			this.connectedDriveFinderWorker.disableWorker();
		}
		this.isFindingDrive = false;
	}

	/**
	 * Setups the environment to test mode
	 */
	private void startProcessTestMode() {
		String localDirectory = System
				.getProperty("saes_saes_local_copy_directory");
		if (getUSBGenerationType() == USBGenerationType.MASTER_USB) {
			this.mountedPath = localDirectory + "/MasterFolder";
		} else {
			this.mountedPath = localDirectory + "/ScreenShotFolder";
		}

		boolean generateNewInitKey = getPanel().getGenerateNewInitKey();

		int result = getPanel().showConfirmDialogStartGeneration(
				this.mountedPath);
		if (result == 0) {
			stopDriveFinderWorker();
			getPanel().preparePanelToProcess();
			startProcessWorker(this.mountedPath, generateNewInitKey);
		}
	}

	protected GUIGenerationInterface getPanel() {
		return this.generationPanel;
	}

	/**
	 * Method that return the CopyOSWorker instance
	 * 
	 * @return the worker instance
	 */
	protected CopyOSWorker getCopyOSWorker() {
		return this.copyOSWorker;
	}

	/**
	 * Method that return the FileIntegrityVerificationWorker instance
	 * 
	 * @return the worker instance
	 */
	protected FileIntegrityVerificationWorker getFileIntegrityVerificationWorker() {
		return this.fileIntegrityVerificationWorker;
	}

	/**
	 * Method used to start the execution of the file integrity verification
	 * worker.
	 */
	protected void startFileIntegrityVerificationWorker() {
		this.isVerifyingFiles = true;

		this.fileIntegrityVerificationWorker = new FileIntegrityVerificationWorker(
				this.mountedPath, getWrittenFiles(), getUSBGenerationType());
		this.fileIntegrityVerificationWorker.addListener(this);

		Thread fileIntegrityVerificationWorkerThread = new Thread(
				this.fileIntegrityVerificationWorker,
				"FileIntegrityVerificationWorker");
		fileIntegrityVerificationWorkerThread.start();
	}

	/**
	 * Method used to start the execution of the copy OS worker.
	 */
	protected void startCopyOSWorker() {
		this.isCopyingOS = true;
		updateMountedPath();

		this.copyOSWorker = new CopyOSWorker(this.devicePath,
				getUSBGenerationType(), ECMServiceProvider.getApplicationKey());
		this.copyOSWorker.addListener(this);

		Thread copyOSWorkerThread = new Thread(this.copyOSWorker,
				"CopyOSWorkerThread");
		copyOSWorkerThread.start();
	}

	/**
	 * Starts execution of the UnmountingWorker.
	 */
	protected void startUnmountingDrive() {
		this.isUnmountingDrive = true;

		this.unmountingWorker = new UnmountingWorker(this.mountedPath);
		this.unmountingWorker.addListener(this);

		Thread unmountingWorkerThread = new Thread(this.unmountingWorker,
				"unmountingWorker");
		unmountingWorkerThread.start();
	}

	/**
	 * Manage generic error.
	 * 
	 * @param message
	 *            Message related to error
	 * @param e
	 *            Exception
	 */
	private void manageGenericError(String message, Exception e) {
		logger.error(e, StringUtils.defaultString(message));
		getPanel().showErrorMessage("ErrorMessage.GENERIC_ERROR");
	}

	/**
	 * Method used to create the folder structure required locally when
	 * generating in test mode.
	 * 
	 * @param mountedPath
	 *            the base path to create the structure
	 * @throws IOException
	 *             signal that an error has occurred creating the directories.
	 */
	protected void makeDirectoriesForTestMode(String mountedPath)
			throws IOException {

		ClientCommon.createDirectory(mountedPath, "firstboot");
		ClientCommon.createDirectory(mountedPath, "SAES");
		ClientCommon.createDirectory(mountedPath, "SAESLocalUnsigned");
		ClientCommon.createDirectory(mountedPath, "SAESLocal/SAES/Docs");
		ClientCommon.createDirectory(mountedPath, "SAESLocal/SAES/Logs");
		ClientCommon.createDirectory(mountedPath, "SAESLocal/Temp");
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * UPDATE_COMPONENTS operation.
	 */
	protected void updateComponentsOperation() {

		if (this.isFindingDrive) {

			findingDriveOnUpdate();

		} else if (this.isCopyingOS) {

			copyingOSOnUpdate();

		} else if (this.isPreparingContests) {

			preparingContestOnUpdate();

		} else if (this.isGeneratingFiles) {

			generatingFilesOnUpdate();

		} else if (this.isVerifyingFiles) {

			verifyingFilesOnUpdate();

		} else if (this.isUnmountingDrive) {

			unmountingOnUpdate();
		}
	}

	/**
	 * Performs the logic when an update event has been fired by the running
	 * worker and is finding driving operation.
	 */
	protected void findingDriveOnUpdate() {

		List<SAESCommonDestinations> devices = this.connectedDriveFinderWorker
				.getDevicesAllowed();

		if (ECMClientSideConstants.VALIDATE_DRIVES_CONNECTED_AMOUNT
				&& devices.size() > 1) {
			getPanel().showMessageManyDevicesConnected();
		} else if (devices.size() > 0) {

			MountedDrive mountedDrive = devices.get(0).getDeviceInfo();
			String devicePath = mountedDrive.getDeviceName();

			int result = getPanel()
					.showConfirmDialogStartGeneration(devicePath);

			if (result == 0) {
				stopDriveFinderWorker();
				boolean generateNewInitKey = getPanel().getGenerateNewInitKey();
				getPanel().preparePanelToProcess();
				startProcessWorker(devicePath, generateNewInitKey);
			}
		}
	}

	/**
	 * Performs the logic when an update event has been fired by the running
	 * worker and is copying the OS operation.
	 */
	protected void copyingOSOnUpdate() {
		if (getCopyOSWorker().isCopyDone()) {
			this.isCopyingOS = false;
			updateMountedPath();
			startPrepareContestsWorker();
		} else {
			getPanel().setCopyOsIconInPendriveOkLabel();
			getPanel().setPendriveOkLabelVisible(true);
			getPanel().showLabelStatusProccess(true);
			getPanel()
					.setLabelStatusProcess(
							getPanel()
									.getBundle()
									.getString(
											"MasterUsbMergedPanel.status_message.executing_script"));
		}
	}

	/**
	 * Performs the logic when an update event has been fired by the running
	 * worker and is preparing contests operation.
	 */
	protected void preparingContestOnUpdate() {

		try {
			while (!isContestPreparationWorkerDone()) {
				Thread.sleep(200);
			}
		} catch (InterruptedException e) {
			manageGenericError("Error waiting for preparationWorker", e);
			return;
		}

		if (!this.testMode) {
			updateMountedPath();
		}

		this.isPreparingContests = false;

		startFileGenerationWorker();

	}

	/**
	 * Updates the mounted path to use in the generation process.
	 * 
	 * After the OS is copied to the pendrive, this method will find the EXT3
	 * partition where the files must reside.
	 * 
	 */
	private void updateMountedPath() {
		List<SAESCommonDestinations> devices = new ArrayList<SAESCommonDestinations>();

		try {
			SAESLocalPersistenceManager.getInstance().checkMountedDevices();

			devices = SAESLocalPersistenceManager.getInstance()
					.getStoreMedias();
		} catch (Exception e) {
			manageGenericError(
					"Error detecting USB drive with a proper ext partition.", e);
			return;
		}

		logger.info("Looking for devices with an ext3/ext4 file system...");
		for (SAESCommonDestinations device : devices) {
			MountedDrive mountedDrive = device.getDeviceInfo();
			logger.debug("-- " + mountedDrive.mountedPath.getAbsolutePath()
					+ ", " + mountedDrive.getFileSystem());
			if (mountedDrive.mountedPath.getAbsolutePath().contains("/media")
					&& mountedDrive.getFileSystem().contains("ext")) {
				this.mountedPath = mountedDrive.mountedPath.getAbsolutePath();
				break;
			}
		}
		logger.info("Selected device (with ext3/ext4): " + this.mountedPath);
	}

	/**
	 * Performs the logic when an update event has been fired by the running
	 * worker and is verifying files operation.
	 */
	protected void verifyingFilesOnUpdate() {
		if (getFileIntegrityVerificationWorker().isFileVerificationDone()) {
			this.isVerifyingFiles = false;
			if (!this.testMode) {
				startUnmountingDrive();
			} else {
				finishProcess();
			}
		} else {
			getPanel().setDocumentImageLabelVisible(false);
			getPanel().setMainProgressBarVisible(false);
			getPanel().setVerificationIconInPendriveOkLabel();
			getPanel().setPendriveOkLabelVisible(true);
			getPanel().showLabelStatusProccess(true);
			getPanel().setLabelStatusProcess(
					getPanel().getBundle().getString(
							"MasterUsbMergedPanel.cheking.file.integrity"));
		}
	}

	protected void unmountingOnUpdate() {
		if (this.unmountingWorker.isDone()) {
			this.isUnmountingDrive = false;
			finishProcess();
		} else {
			getPanel().setSyncAndUnmountIconInPendriveOkLabel();
			getPanel().setPendriveOkLabelVisible(true);
			getPanel().showLabelStatusProccess(true);
			getPanel()
					.setLabelStatusProcess(
							getPanel()
									.getBundle()
									.getString(
											"FileGenerationPanel.syncing.and.unmounting.drive"));
		}
	}

	/**
	 * Performs the logic when an update event has been fired by the running
	 * worker and is generating files operation.
	 */
	protected void generatingFilesOnUpdate() {
		if (isFileGenerationDone()) {
			startFileIntegrityVerificationWorker();
			this.isGeneratingFiles = false;
		} else {
			getPanel().setPendriveOkLabelVisible(false);
			getPanel().setMainProgressBarVisible(true);
			getPanel().setDocumentImageLabelVisible(true);

			getPanel().setLabelStatusProcess(
					getPanel().getBundle().getString(
							"MasterUsbMergedPanel.status_message.creating_pm"));

		}
	}

	/**
	 * Updates the client status and performs a synchronization and unmount of
	 * the mounted path before result message window is displayed.
	 */
	private void finishProcess() {
		Duration duration = getProcessDuration();
		logger.info("Process finished successful, Duration:"
				+ duration.toString());
		getPanel().finishProcess(duration);
	}

	/**
	 * Gets duration of the process.
	 */
	private Duration getProcessDuration() {

		long start = this.generationStartTime;
		long stop = System.currentTimeMillis();

		final long milisecondsDiff = stop - start;

		Duration duration = null;
		try {
			duration = DatatypeFactory.newInstance().newDuration(
					milisecondsDiff);
		} catch (DatatypeConfigurationException e) {
			logger.warn("It was not possible to calculate the result time", e);
		}
		return duration;
	}

	/**
	 * Method used to start the workers required to the whole Master USB
	 * generation process.
	 * 
	 * @param mountedPath
	 *            The path where is mounted the USB drive.
	 * @param generateApplicationKey
	 *            indicates if new application key should be generated
	 */
	protected void startProcessWorker(String devicePath,
			boolean generateApplicationKey) {

		this.devicePath = devicePath;
		this.generationStartTime = System.currentTimeMillis();

		try {
			if (generateApplicationKey) {
				ECMServiceProvider.generateRandomApplicationKey();
			}

			if (!this.testMode) {
				startCopyOSWorker();
			} else {
				makeDirectoriesForTestMode(devicePath);
				startPrepareContestsWorker();
			}
		} catch (Exception e) {
			manageGenericError("Error starting process worker", e);
		}
	}

	/**
	 * Method used to start the execution of the master USB preparation of
	 * contests worker.
	 */
	protected abstract void startPrepareContestsWorker();

	/**
	 * Method used to start the execution of the file generation worker.
	 * 
	 */
	protected abstract void startFileGenerationWorker();

	/**
	 * Method that returns the USB generation type.
	 * 
	 * @return the generation type
	 */
	protected abstract USBGenerationType getUSBGenerationType();

	/**
	 * Return the set of written files by the file generation workers
	 * 
	 * @return the set of files
	 */
	protected abstract Map<String, FileBackup> getWrittenFiles();

	/**
	 * Return true if the file generation process is done, otherwise false
	 * 
	 * @return true if the process generation is done
	 */
	protected abstract boolean isFileGenerationDone();

	/**
	 * Return true if the contest preparation worker is done
	 * 
	 * @return true if the contest preparations is done
	 */
	protected abstract boolean isContestPreparationWorkerDone();

}
