/**
 * FormatValidator.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client.ui.controller;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ui.CodiFormatterDetailPanel;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Verifies the format of given files with certain pattern.
 */
public class FormatValidator implements Runnable {

	/**
	 * Regular expression bundle. It contains all format rules to be applied to
	 * CODI files
	 */
	static ResourceBundle regularExp = ResourceBundle
			.getBundle("com/smartmatic/waffle/ecm/client/resources/codiFormatValidations");

	/**
	 * Regular expression bundle. It contains all format rules to be applied to
	 * csv format CODI files
	 */
	static ResourceBundle csvRegularExp = ResourceBundle
			.getBundle("com/smartmatic/waffle/ecm/client/resources/csvFormatValidations");

	/**
	 * Language bundle. Resource bundle for languages
	 */
	static ResourceBundle languageBundle = ResourceBundle
			.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp");

	/** For log messages purpose. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(FormatValidator.class);

	/**
	 * Maps the a current set of files with a list of errors included in that
	 * file.
	 */
	private static Map<File, List<String>> currentErrorMap = new HashMap<File, List<String>>();

	/**
	 * Define the maximum number of error lines to be shown in the tree per
	 * file.
	 */
	private static final int MAX_ERROR_LINES_ALLOWED = 12;

	/** Election type. */
	private static String electionType;

	/**
	 * Structure which holds a validation object.
	 */
	class Validation {

		/** The regular expression. */
		String regularExpression;

		/** The is mandatory. */
		boolean isMandatory;

		/** The is mandatory for. */
		String isMandatoryFor;

		/**
		 * Instantiates a new validation.
		 * 
		 * @param reg
		 *            the reg
		 * @param mand
		 *            the mand
		 * @param mandFor
		 *            the mand for
		 */
		public Validation(String reg, boolean mand, String mandFor) {
			this.regularExpression = reg;
			this.isMandatory = mand;
			this.isMandatoryFor = mandFor;
		}
	}

	/** HashMap which contains the validations associated with each register. */
	static HashMap<String, Validation[]> validations = new HashMap<String, Validation[]>();

	/**
	 * HashMap which contains the csv files validations associated with each
	 * register.
	 */
	static HashMap<String, Validation[]> csvValidations = new HashMap<String, Validation[]>();

	/** File array which contains the given CODI files to be processed. */
	private List<File> codiFilesSelected;

	/** File array which contains the given Sites CODI files to be processed. */
	private List<File> siteCodiFilesSelected;

	/** List of invalid files. */
	private List<File> invalidCodiFiles = new ArrayList<File>();

	/** The panel which will be painted after the process. */
	private CodiFormatterDetailPanel codiFormatterDetailPanel;

	/**
	 * Default constructor of the class.
	 */
	public FormatValidator() {
		super();
	}

	/**
	 * Constructor which initializes the validations Map.
	 * 
	 * @param _validations
	 *            , the validations Map
	 * @param _codiFiles
	 *            , The CODI files to be processed
	 * @param _codiFormatterDetailPanel
	 *            the _codi formatter detail panel
	 */
	public FormatValidator(HashMap<String, Validation[]> _validations,
			List<File> _codiFiles,
			CodiFormatterDetailPanel _codiFormatterDetailPanel) {
		validations = _validations;
		this.codiFilesSelected = null;
		this.codiFilesSelected = _codiFiles;

		this.codiFormatterDetailPanel = _codiFormatterDetailPanel;
	}

	/**
	 * Constructor which initializes the validations Map.
	 * 
	 * @param _validations
	 *            The validations Map
	 * @param _csvValidations
	 *            The csv format validations map
	 * @param _codiFiles
	 *            The CODI files to be processed
	 * @param _siteCodiFiles
	 *            the _site codi files
	 * @param _codiFormatterDetailPanel
	 *            The panel to load the result
	 */
	public FormatValidator(HashMap<String, Validation[]> _validations,
			HashMap<String, Validation[]> _csvValidations,
			List<File> _codiFiles, List<File> _siteCodiFiles,
			CodiFormatterDetailPanel _codiFormatterDetailPanel) {
		validations = _validations;
		csvValidations = _csvValidations;
		this.codiFilesSelected = null;
		this.codiFilesSelected = _codiFiles;

		this.siteCodiFilesSelected = null;
		this.siteCodiFilesSelected = _siteCodiFiles;
		this.codiFormatterDetailPanel = _codiFormatterDetailPanel;
	}

	/**
	 * Getter for the currentErrorMap.
	 * 
	 * @return the current error map
	 */
	public Map<File, List<String>> getCurrentErrorMap() {
		return FormatValidator.currentErrorMap;
	}

	/**
	 * Getter for the validations Map.
	 * 
	 * @return the current validations map
	 */
	public HashMap<String, Validation[]> getCurrentValidationsMap() {
		return FormatValidator.validations;
	}

	/**
	 * Gets the current csv validations map.
	 * 
	 * @return the current csv validations map
	 */
	public HashMap<String, Validation[]> getCurrentCsvValidationsMap() {
		return FormatValidator.csvValidations;
	}

	/**
	 * Getter for the invalid files found.
	 * 
	 * @return the current invalid files found
	 */
	public List<File> getInvalidFilesFound() {
		return this.invalidCodiFiles;
	}

	/**
	 * Removes all elements of the current error map.
	 */
	public void resetCurrentErrorMap() {
		currentErrorMap.clear();
	}

	/**
	 * Indicates whether the currentErrorMap is empty or not.
	 * 
	 * @return true if the currentErrorMap is empty, false if not
	 */
	public boolean currentErrorMapIsEmpty() {
		return currentErrorMap.isEmpty();
	}

	/**
	 * Adds a generic validation, reading from the regExp file, given the
	 * register associated.
	 * 
	 * @param parent
	 *            , the register associated with the validations
	 */
	private static void addHashEntry(String parent) {

		FormatValidator fv = new FormatValidator();
		int quantityOfElements = Integer.parseInt(regularExp
				.getString("numberOf" + parent));
		;
		Validation[] detailedValidations = new Validation[quantityOfElements];

		/* Validations for <parent> */

		for (int i = 0; i < quantityOfElements; ++i) {
			String line = regularExp.getString(parent + (i + 1));
			String[] expression = line.split(";");

			String validator = expression[0];

			boolean mandatory = Boolean.parseBoolean(expression[1]);

			String madatoryFor = null;

			if (expression.length > 2) {
				madatoryFor = expression[2];
			}

			detailedValidations[i] = fv.new Validation(validator, mandatory,
					madatoryFor);
		}

		validations.put(parent, detailedValidations);
	}

	/**
	 * Adds a generic validation, reading from the csvRegularExp file, given the
	 * register associated.
	 * 
	 * @param parent
	 *            The register associated with the validations
	 */
	private static void addCSVHashEntry(String parent) {
		FormatValidator fv = new FormatValidator();
		int quantityOfElements = Integer.parseInt(csvRegularExp
				.getString("numberOf" + parent));

		Validation[] detailedValidations = new Validation[quantityOfElements];

		/* Validations for <parent> */

		for (int i = 0; i < quantityOfElements; ++i) {
			String line = csvRegularExp.getString(parent + (i + 1));
			String[] expression = line.split(";");

			String validator = expression[0];

			boolean mandatory = Boolean.parseBoolean(expression[1]);

			detailedValidations[i] = fv.new Validation(validator, mandatory,
					null);
		}

		csvValidations.put(parent, detailedValidations);
	}

	/**
	 * Loads the validations into the global HashMap.
	 */
	public static void loadValidations() {
		addHashEntry("G"); // Validations for G
		addHashEntry("S"); // Validations for S
		addHashEntry("L"); // Validations for L
		addHashEntry("C"); // Validations for C
		addHashEntry("H"); // Validations for H
		addHashEntry("T"); // Validations for T
		addHashEntry("B"); // Validations for B
	}

	/**
	 * Load csv format validations into HashMap.
	 */
	public static void loadCsvValidations() {
		addCSVHashEntry("G");
	}

	/**
	 * Process a single CODI line, identified by the register at the beginning.
	 * 
	 * @param line
	 *            , the line from the CODI file splitted into an array
	 * @param completeForLogging
	 *            , the complete line for logging
	 * @return true if the line passes the validation, false if not
	 */
	private static List<Integer> processCodiLine(String[] line,
			String completeForLogging) {

		/* Columns in error */
		List<Integer> columnsInError = new ArrayList<Integer>();

		int len = line.length;
		Validation[] lineValidators = validations
				.get(line[0].replace("\"", ""));
		if (lineValidators == null) {
			logger.error("Failed in line: '"
					+ completeForLogging
					+ "'. It doesn't fix into any category (Possibly Bad column #1");

			/* Adding column 1 into columns in error */
			columnsInError.add(1);
			// return false;
			return columnsInError;
		}

		// boolean flag = true;
		for (int i = 1; i < len && i < lineValidators.length; ++i) {
			line[i] = line[i].replace("\"", "");

			if (lineValidators[i].isMandatoryFor != null
					&& electionType != null) {

				String mandatoryFor = lineValidators[i].isMandatoryFor;

				boolean isMandatory = lineValidators[i].isMandatory;

				if (isMandatory
						&& StringUtils.contains(mandatoryFor, electionType)
						&& line[i].equals("")) {
					logger.error("Failed in column " + (i + 1) + " of line: '"
							+ completeForLogging
							+ "'.\nThe value shold be mandatory for: "
							+ lineValidators[i].isMandatoryFor);
					/* Adding column 'i' into columns in error */
					columnsInError.add(i + 1);
				} else if (!isMandatory
						&& !StringUtils.contains(mandatoryFor, electionType)
						&& !line[i].equals("")) {
					logger.error("Failed in column " + (i + 1) + " of line: '"
							+ completeForLogging
							+ "'.\nThe value shold be in: "
							+ lineValidators[i].isMandatoryFor);
					/* Adding column 'i' into columns in error */
					columnsInError.add(i + 1);
				}
			} else {
				if (!line[i].matches(lineValidators[i].regularExpression)) {
					if (lineValidators[i].isMandatory) {
						logger.error("Failed in column " + (i + 1)
								+ " of line: '" + completeForLogging
								+ "'.\nShould follow this rule: "
								+ lineValidators[i].regularExpression);
						/* Adding column 'i' into columns in error */
						columnsInError.add(i + 1);
						// flag = false;
					} else {
						if (!line[i].equals("")) {
							logger.warn("Failed in line: '"
									+ completeForLogging
									+ "'. This line is not mandatory but is included and doesn't follow the rule: "
									+ lineValidators[i].regularExpression
									+ " at column " + (i + 1));
							/* Adding column 'i' into columns in error */
							columnsInError.add(i + 1);
							// flag = false;
						}
					}
				}
			}
		}
		return columnsInError;
		// return flag;
	}

	/**
	 * Process a single csv CODI line, identified by the register at the
	 * beginning.
	 * 
	 * @param line
	 *            the line from the CODI file splitted into an array
	 * @param completeForLogging
	 *            the complete line for logging
	 * @return A list of columns number with errors.
	 */
	private static List<Integer> processSitesCodiLine(String[] line,
			String completeForLogging) {

		/* Columns in error */
		List<Integer> columnsInError = new ArrayList<Integer>();

		int len = line.length;
		Validation[] lineValidators = csvValidations.get(line[0].replace("\"",
				""));
		if (lineValidators == null) {
			logger.error("Failed in line: '"
					+ completeForLogging
					+ "'. It doesn't fix into any category (Possibly Bad column #1");

			/* Adding column 1 into columns in error */
			columnsInError.add(1);
			// return false;
			return columnsInError;
		}

		for (int i = 1; i < len && i < lineValidators.length; ++i) {

			line[i] = line[i].replace("\"", "");

			if (!line[i].matches(lineValidators[i].regularExpression)) {
				if (lineValidators[i].isMandatory) {
					logger.error("Failed in column " + (i + 1) + " of line: '"
							+ completeForLogging
							+ "'.\nShould follow this rule: "
							+ lineValidators[i].regularExpression);
					/* Adding column 'i' into columns in error */
					columnsInError.add(i + 1);
					// flag = false;
				} else {
					if (!line[i].equals("")) {
						logger.warn("Failed in line: '"
								+ completeForLogging
								+ "'. This line is not mandatory but is included and doesn't follow the rule: "
								+ lineValidators[i].regularExpression
								+ " at column " + (i + 1));
						/* Adding column 'i' into columns in error */
						columnsInError.add(i + 1);
						// flag = false;
					}
				}
			}
		}

		return columnsInError;
	}

	/**
	 * Validates the format of a CODI files using the loaded validations in
	 * <validations> structure.
	 * 
	 * @param codiFile
	 *            , the file to be validated
	 * @return true if the file passes the validations, false if not
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static boolean validateFormat(File codiFile) throws IOException {
		List<String> codiLines = FileUtils.readLines(codiFile, "IBM850");
		Iterator<String> it = codiLines.iterator();

		/* Lines in error */
		List<String> linesInError = new ArrayList<String>();

		/* Iterate over all lines */
		boolean errorsFound = false;
		int errorLineCount = 1;
		int lineCount = 2;

		while (it.hasNext()) {
			String completeForLogging = it.next();
			String[] line = completeForLogging.split("\",\"");

			/* If the list is empty, return false */
			List<Integer> columnsInError = processCodiLine(line,
					completeForLogging);

			if (lineCount == 2 && columnsInError.isEmpty()) {
				electionType = line[1];
			}

			if (!columnsInError.isEmpty()) {
				/* Add line into the array of errors per line */
				String lineString = languageBundle
						.getString("CodiFormatterDetailPanel.line");
				String columnString = languageBundle
						.getString("CodiFormatterDetailPanel.columns");
				String message = lineString + " " + lineCount + ". "
						+ columnString + " " + columnsInError + ". < "
						+ completeForLogging + " >";
				linesInError.add(message);
				errorsFound = true;
				// return false;

				/* If the errors found exceed the maximum allowed, exit */
				if ((errorLineCount++) == MAX_ERROR_LINES_ALLOWED) {
					String errorMessage = languageBundle
							.getString(
									"CodiFormatterDetailPanel.max_text_allowed_excedeed")
							.replace("{0}",
									String.valueOf(MAX_ERROR_LINES_ALLOWED));
					linesInError.add(errorMessage);
					break;
				}
			}
			lineCount++;
		}
		if (errorsFound) {
			currentErrorMap.put(codiFile, linesInError);
		}

		return !errorsFound;
		// return true;
	}

	/**
	 * Validates the format of a csv CODI files using the loaded validations in
	 * <validations> structure.
	 * 
	 * @param siteCodiFile
	 *            the site codi file
	 * @return true if the file passes the validations, false if not
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static boolean validateCSVFormat(File siteCodiFile)
			throws IOException {
		List<String> codiLines = FileUtils.readLines(siteCodiFile, "IBM850");
		Iterator<String> it = codiLines.iterator();

		/* Lines in error */
		List<String> linesInError = new ArrayList<String>();

		/* Iterate over all lines */
		boolean errorsFound = false;
		int errorLineCount = 1;
		int lineCount = 2;

		while (it.hasNext()) {
			String completeForLogging = it.next();
			String[] line = completeForLogging.split(";");

			/* If the list is empty, return false */
			List<Integer> columnsInError = processSitesCodiLine(line,
					completeForLogging);

			if (!columnsInError.isEmpty()) {
				/* Add line into the array of errors per line */
				String lineString = languageBundle
						.getString("CodiFormatterDetailPanel.line");
				String columnString = languageBundle
						.getString("CodiFormatterDetailPanel.columns");
				String message = lineString + " " + lineCount + ". "
						+ columnString + " " + columnsInError + ". < "
						+ completeForLogging + " >";
				linesInError.add(message);
				errorsFound = true;
				// return false;

				/* If the errors found exceed the maximum allowed, exit */
				if ((errorLineCount++) == MAX_ERROR_LINES_ALLOWED) {
					String errorMessage = languageBundle
							.getString(
									"CodiFormatterDetailPanel.max_text_allowed_excedeed")
							.replace("{0}",
									String.valueOf(MAX_ERROR_LINES_ALLOWED));
					linesInError.add(errorMessage);
					break;
				}
			}
			lineCount++;
		}
		if (errorsFound) {
			currentErrorMap.put(siteCodiFile, linesInError);
		}

		return !errorsFound;
		// return true;
	}

	/**
	 * Starts to process the list of selected CODI files.
	 */
	public void processGivenCodiFiles() {

		int numberOfFiles = this.codiFilesSelected.size();
		if (this.siteCodiFilesSelected != null) {
			numberOfFiles += this.siteCodiFilesSelected.size();
		}

		List<File> allCodiFiles = new ArrayList<File>();
		double barProgress = 0;
		boolean isSitesCodi = false;
		allCodiFiles.addAll(this.codiFilesSelected);
		if (this.siteCodiFilesSelected != null) {
			allCodiFiles.addAll(this.siteCodiFilesSelected);
		}
		for (File actualCodi : allCodiFiles) {
			/* Validation of .zip or .tar.gz files is not supported */
			/* Valid parameters for validation are not defined */
			if (actualCodi.getName().endsWith(".zip")
					|| actualCodi.getName().endsWith(".tar.gz")) {
				continue;
			}

			if (actualCodi.getName().endsWith(".csv")) {
				isSitesCodi = true;
			}

			boolean isCodiOk = false;
			try {
				if (isSitesCodi) {
					isCodiOk = validateCSVFormat(actualCodi);
				} else {
					isCodiOk = validateFormat(actualCodi);
				}

			} catch (IOException e) {
				logger.error("An exception occurred validating CODI formats", e);
			}
			if (!isCodiOk) {
				this.invalidCodiFiles.add(actualCodi);
			}

			/* Set the progress bar actual value */
			barProgress += (100.00 / numberOfFiles);
			this.codiFormatterDetailPanel
					.setProgressBarValue((int) barProgress);
		}
		this.codiFormatterDetailPanel.setProgressBarValue(100);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run() {
		processGivenCodiFiles();
		if (currentErrorMapIsEmpty()) {
			this.codiFormatterDetailPanel.paintNoErrorsInTheTree();
		} else {
			this.codiFormatterDetailPanel.paintErrorsTree(getCurrentErrorMap());
		}
	}

}
