/**
 * ImageUtilities.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client.util;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.imageio.ImageIO;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class to execute utility functionalities to Images.
 */
public class ImageUtilities {

	/** For log messages purpose. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ImageUtilities.class);

	/**
	 * Transforms an ImageWrapper into an Image.
	 * 
	 * @param imageWrapper
	 *            Image Wrapper from model.
	 * @return java.awt.Image.
	 */
	public static Image imageWrapperToImage(ImageWrapper imageWrapper) {

		Image image = null;
		if (imageWrapper != null) {
			try {
				InputStream in = new ByteArrayInputStream(
						imageWrapper.getBinaryData());

				BufferedImage bufferedimage = ImageIO.read(in);
				in.close();

				image = bufferedimage;
			} catch (IOException e) {
				logger.warn(e, StringUtils.defaultString(e.getMessage()));
			}
		}
		return image;
	}

	/**
	 * Gets an ImageIcon scaled to a bounded size keeping aspect ratio.
	 * 
	 * @param image
	 *            The Image to scale
	 * @param boundDimension
	 *            The bound dimension to be scale
	 * @return Image scaled
	 */
	public static Image getScaledImageWithBounds(Image image,
			Dimension boundDimension) {

		Image scaledImage = null;

		if (boundDimension == null) {
			return image;
		}

		if (image != null) {
			Dimension scaledD = getScaledDimension(image, boundDimension);
			scaledImage = getScaledImage(image, scaledD);

		}
		return scaledImage;
	}

	/**
	 * Gets the dimension necessary to maintain aspect ratio given an Image and
	 * a bound Dimension.
	 * 
	 * @param image
	 *            Image to be scaled
	 * @param boundDimension
	 *            Bound dimension
	 * @return The dimension to maintain aspect ratio
	 */
	private static Dimension getScaledDimension(Image image,
			Dimension boundDimension) {

		int original_width = image.getWidth(null);
		int original_height = image.getHeight(null);
		int bound_width = boundDimension.width;
		int bound_height = boundDimension.height;
		int new_width = original_width;
		int new_height = original_height;

		if (original_width > bound_width) {
			new_width = bound_width;
			new_height = (new_width * original_height) / original_width;
		}

		if (new_height > bound_height) {
			new_height = bound_height;
			new_width = (new_height * original_width) / original_height;
		}

		return new Dimension(new_width, new_height);
	}

	/**
	 * Gets a scaled image given an specific dimension
	 * 
	 * @param srcImg
	 *            The image to be scaled
	 * @param newDimension
	 *            The new dimension to scale
	 * @return Scaled Image
	 */
	private static Image getScaledImage(Image srcImg, Dimension newDimension) {

		int w = newDimension.width;
		int h = newDimension.height;

		BufferedImage resizedImg = new BufferedImage(w, h,
				BufferedImage.TYPE_INT_ARGB);
		Graphics2D g2 = resizedImg.createGraphics();

		g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
				RenderingHints.VALUE_INTERPOLATION_BILINEAR);
		g2.drawImage(srcImg, 0, 0, w, h, null);
		g2.dispose();

		return resizedImg;
	}

}
