/**
 * PDFSigner.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Security;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;

import org.bouncycastle.jce.provider.BouncyCastleProvider;

import com.itextpdf.text.DocumentException;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfSignatureAppearance;
import com.itextpdf.text.pdf.PdfStamper;
import com.itextpdf.text.pdf.security.BouncyCastleDigest;
import com.itextpdf.text.pdf.security.DigestAlgorithms;
import com.itextpdf.text.pdf.security.ExternalDigest;
import com.itextpdf.text.pdf.security.ExternalSignature;
import com.itextpdf.text.pdf.security.MakeSignature;
import com.itextpdf.text.pdf.security.MakeSignature.CryptoStandard;
import com.itextpdf.text.pdf.security.PrivateKeySignature;

/**
 * Sign a PDF document making the signature visible to the naked eye.
 * 
 * @author Daniel Bastidas
 */
public class PDFSigner {

	/**
	 * Sign the PDF document represented by the path pdfPath making the
	 * signature visible to the naked eye.
	 * 
	 * @param pdfPath
	 *            the file system path of the PDF document
	 * @param keyStore
	 *            the keystore that contains the key used to sign
	 * @param alias
	 *            the alias to retrieve the key used to sign
	 * @param password
	 *            the password's keystore
	 * @return the byte[]
	 * @throws KeyStoreException
	 *             to report a generic KeyStore exception
	 * @throws NoSuchAlgorithmException
	 *             to report that a particular cryptographic algorithm is
	 *             requested but is not available in the environment
	 * @throws CertificateException
	 *             to report a variety of certificate problems
	 * @throws IOException
	 *             to report input/output error
	 * @throws UnrecoverableKeyException
	 *             to report that a key in the keystore cannot be retrieved
	 * @throws DocumentException
	 *             to report a problem signing the document
	 * @throws GeneralSecurityException
	 *             to report a problem during the signing process
	 */
	public static byte[] sign(String pdfPath, InputStream keyStore,
			String alias, String password) throws KeyStoreException,
			NoSuchAlgorithmException, CertificateException, IOException,
			UnrecoverableKeyException, DocumentException,
			GeneralSecurityException {

		PdfStamper stp = null;
		ByteArrayOutputStream bout = null;
		try {

			Security.addProvider(new BouncyCastleProvider());

			KeyStore ks = KeyStore.getInstance("PKCS12");
			ks.load(keyStore, password.toCharArray());
			PrivateKey key = (PrivateKey) ks.getKey(alias,
					password.toCharArray());
			Certificate[] chain = ks.getCertificateChain(alias);

			PdfReader reader = new PdfReader(pdfPath);
			bout = new ByteArrayOutputStream();
			stp = PdfStamper.createSignature(reader, bout, '\0');

			PdfSignatureAppearance sap = stp.getSignatureAppearance();
			sap.setReason("SCREEN FILE PDF SIGNATURE");
			sap.setLocation("BELGIUM");

			// comment next line to have an invisible signature
			// sap.setVisibleSignature(new Rectangle(100, 100, 200, 200), 1,
			// null);

			ExternalSignature signature = new PrivateKeySignature(key,
					DigestAlgorithms.SHA256, "BC");

			ExternalDigest digest = new BouncyCastleDigest();
			MakeSignature.signDetached(sap, digest, signature, chain, null,
					null, null, 0, CryptoStandard.CMS);

		} finally {
			bout.close();
		}
		return bout.toByteArray();

	}
}
