package com.smartmatic.waffle.ecm.client.worker;

import java.util.List;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.service.lang.NominationException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * This class is used to search parties and contests objects sending the request
 * to the server and updating the status and the data in the client.
 */
public class EligibleOptionAdvancedSearchWorker extends AsynchronousOperation {

	private static SMTTLogger logger = SMTTLogger
			.getLogger(DecryptWorker.class);

	private List<RegionData> regions;
	private List<PartyData> parties;
	private List<ContestData> contests;
	private LanguageData language;
	private boolean searchParties = false;

	private List<PartyData> partiesList;
	private List<ContestData> contestsList;

	/**
	 * Creates a new instance of EligibleOptionAdvancedSearchWorker
	 * 
	 * @param regions
	 *            the regions list filter
	 * @param parties
	 *            the parties list filter
	 * @param language
	 *            the current client language
	 */
	public EligibleOptionAdvancedSearchWorker(List<RegionData> regions,
			List<PartyData> parties, LanguageData language) {
		this.regions = regions;
		this.parties = parties;
		this.language = language;
	}

	/**
	 * Creates a new instance of EligibleOptionAdvancedSearchWorker
	 * 
	 * @param regions
	 *            the regions list filter
	 * @param contests
	 *            the contests list filter
	 * @param searchParties
	 *            true if the worker must perform a search of parties, otherwise
	 *            will perform a search of contests.
	 */
	public EligibleOptionAdvancedSearchWorker(List<RegionData> regions,
			List<ContestData> contests, boolean searchParties) {
		this.regions = regions;
		this.contests = contests;
		this.searchParties = searchParties;
	}

	/**
	 * Gets the parties list from the current search.
	 * 
	 * @return the parties list
	 */
	public List<PartyData> getPartiesList() {
		return this.partiesList;
	}

	/**
	 * Gets the contests lists from the current search.
	 * 
	 * @return the contests list
	 */
	public List<ContestData> getContestsList() {
		return this.contestsList;
	}

	/**
	 * Method that performs the search server side search of the parties and the
	 * result is stored in the partiesList.
	 * 
	 * @throws NominationException
	 *             a signal that an error occurred retrieving the parties list
	 * @throws ConnectionException
	 *             a signal that an error occurred with the server connection
	 */
	private void searchParties() throws ConnectionException,
			NominationException {
		this.partiesList = ECMServiceProvider.getParties(this.regions,
				this.contests);
	}

	/**
	 * Method that performs the search server side search of the contests and
	 * the result is stored in the contestsList.
	 * 
	 * @throws NominationException
	 *             a signal that an error occurred retrieving the contests list
	 * @throws ConnectionException
	 *             a signal that an error occurred with the server connection
	 */
	private void searchContests() throws NominationException,
			ConnectionException {
		this.contestsList = ECMServiceProvider.getContests(this.regions,
				this.parties, this.language);
	}

	@Override
	protected void performOperation() {
		fireAwtEvent(AwtOperationType.DISABLE_COMPONENTS);
		try {
			if (this.searchParties) {
				searchParties();
			} else {
				searchContests();
			}
			fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		} finally {
			fireAwtEvent(AwtOperationType.ENABLE_COMPONENTS);
		}
	}

	@Override
	protected void onCompleted() {
		fireAwtEvent(AwtOperationType.ENABLE_COMPONENTS);
	}

}
