package com.smartmatic.waffle.ecm.client.worker;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.ProgrammingUnitData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.core.util.TimestampUtil;
import com.smartmatic.waffle.ecm.data.transfer.comparators.ContestPrecedenceComparator;
import com.smartmatic.waffle.ecm.data.transfer.objects.ActiveContestForElectionDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionObjectDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionXmlComponentsDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ProgrammingUnitElectionObjectDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ProgrammingUnitLastLevelRegionDTO;
import com.smartmatic.waffle.ecm.service.lang.DemographyException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.ecm.service.lang.MasterUSBGenerationException;
import com.smartmatic.waffle.ecm.service.lang.NominationException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Worker class that performs the operation and calls the server side methods to
 * prepare the contests data for the selected regions from the active election.
 *
 */
public class MasterUSBPrepareContestsWorker extends AsynchronousOperation {

	/** Static SMTT logger instance. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(MasterUSBPrepareContestsWorker.class);

	private List<Object> selectedObjects;
	private LanguageData language;
	private boolean generateUsingProgrammingUnits;

	private int contestsToProcess;

	private ProgrammingUnitElectionObjectDTO programmingUnitElectionObject;

	public MasterUSBPrepareContestsWorker(List<Object> selectedObjects,
			LanguageData language, boolean generateUsingProgrammingUnits) {
		this.selectedObjects = selectedObjects;
		this.language = language;
		this.generateUsingProgrammingUnits = generateUsingProgrammingUnits;
		this.contestsToProcess = 0;
	}

	/**
	 * Returns the programming unit election object that was set by the current
	 * generation
	 * 
	 * @return the programming unit election object.
	 */
	public ProgrammingUnitElectionObjectDTO getProgrammingUnitElectionObject() {
		return this.programmingUnitElectionObject;
	}

	/**
	 * Returns the number of contests to process.
	 * 
	 * @return the number of contests to process
	 */
	public int getContestsToProcess() {
		return this.contestsToProcess;
	}

	/**
	 * Starts processing the selected objects, this process will generate and
	 * set the objects required for the further file generation worker.
	 * 
	 * @throws ConnectionException
	 *             a signal that an error has occurred with the server
	 *             connection
	 * @throws ElectoralOrganizationOperationException
	 *             a signal that an error has occurred retrieving the election
	 *             data
	 * @throws DemographyException
	 *             a signal that an error has occurred getting the programming
	 *             units by regions
	 * @throws MasterUSBGenerationException
	 *             a signal that an error has occurred generating the election
	 *             object
	 * @throws NominationException
	 *             a signal that an error has occurred getting the eligible
	 *             options
	 */
	protected void processSelectedObjects() throws ConnectionException,
			ElectoralOrganizationOperationException, DemographyException,
			MasterUSBGenerationException, NominationException {

		Map<Long, ElectionObjectDTO> electionMap = new HashMap<Long, ElectionObjectDTO>();

		this.programmingUnitElectionObject = new ProgrammingUnitElectionObjectDTO();

		Set<ProgrammingUnitData> programmingUnitSet = new HashSet<ProgrammingUnitData>();

		int generationCount = 0;
		int totalRegions = 0;

		if (this.generateUsingProgrammingUnits) {

			totalRegions = processUsingProgramingUnits(electionMap,
					programmingUnitSet, generationCount);
		} else {
			totalRegions = processSingleElection(electionMap,
					programmingUnitSet, generationCount);
		}

		this.programmingUnitElectionObject.setTotalRegions(totalRegions);
		this.programmingUnitElectionObject.setElectionObjectMap(electionMap);
		this.programmingUnitElectionObject
				.setProgrammingUnitSet(programmingUnitSet);

	}

	/**
	 * This method will process a single election object when no programming
	 * units included in data, and return the total regions processed for the
	 * current generation.
	 * 
	 * @param electionMap
	 *            election map reference which contains the elections objects
	 *            generated
	 * @param programmingUnitSet
	 *            programming unit set reference to include in the returned
	 *            object from this worker
	 * @param generationCount
	 *            when multiple programming units in data, a counter parameter
	 *            for generations must be included.
	 * @return the total of regions generated.
	 * @throws DemographyException
	 *             a signal that an error has occurred getting the programming
	 *             units by regions
	 * @throws ConnectionException
	 *             a signal that an error has occurred with the server
	 *             connection
	 * @throws NominationException
	 *             a signal that an error has occurred getting the eligible
	 *             options
	 * @throws MasterUSBGenerationException
	 *             a signal that an error has occurred generating the election
	 *             object
	 * @throws ElectoralOrganizationOperationException
	 *             a signal that an error has occurred retrieving the election
	 *             data
	 */
	private int processSingleElection(Map<Long, ElectionObjectDTO> electionMap,
			Set<ProgrammingUnitData> programmingUnitSet, int generationCount)
			throws DemographyException, ConnectionException,
			NominationException, MasterUSBGenerationException,
			ElectoralOrganizationOperationException {

		Map<Long, ContestData> contestsMap = new HashMap<Long, ContestData>();

		ElectionData election = ECMServiceProvider.retrieveActiveElection();

		List<RegionData> lastLevelRegions = ECMServiceProvider
				.getLastLevelRegionsLight(this.getSelectedRegions());

		electionMap.put(
				0l,
				getElection(lastLevelRegions, contestsMap, election,
						generationCount++));

		ProgrammingUnitData programmingUnit = new ProgrammingUnitData();

		programmingUnit.setCode(0);
		programmingUnit.setCustomCode(election.getCustomCode());
		programmingUnit.setProgrammingUnitName(election.getCustomCode());

		programmingUnitSet.add(programmingUnit);

		return lastLevelRegions.size();
	}

	/**
	 * This method will process multiple election objects from the set of
	 * programming units included in data, and return the total regions
	 * processed for the current generation.
	 * 
	 * @param electionMap
	 *            election map reference which contains the elections objects
	 *            generated
	 * @param programmingUnitSet
	 *            programming unit set reference to include in the returned
	 *            object from this worker
	 * @param generationCount
	 *            when multiple programming units in data, a counter parameter
	 *            for generations must be included.
	 * @return the total of regions generated from the all programming units.
	 * @throws DemographyException
	 *             a signal that an error has occurred getting the programming
	 *             units by regions
	 * @throws ConnectionException
	 *             a signal that an error has occurred with the server
	 *             connection
	 * @throws NominationException
	 *             a signal that an error has occurred getting the eligible
	 *             options
	 * @throws MasterUSBGenerationException
	 *             a signal that an error has occurred generating the election
	 *             object
	 * @throws ElectoralOrganizationOperationException
	 *             a signal that an error has occurred retrieving the election
	 *             data
	 */
	private int processUsingProgramingUnits(
			Map<Long, ElectionObjectDTO> electionMap,
			Set<ProgrammingUnitData> programmingUnitSet, int generationCount)
			throws DemographyException, ConnectionException,
			NominationException, MasterUSBGenerationException,
			ElectoralOrganizationOperationException {

		Map<Long, ContestData> contestsMap = new HashMap<Long, ContestData>();

		ElectionData election = ECMServiceProvider.retrieveActiveElection();

		Set<ProgrammingUnitLastLevelRegionDTO> programmingUnitLastLevelRegionSet = ECMServiceProvider
				.getProgrammingUnitsByRegions(this.getSelectedRegions());

		int totalRegions = 0;

		for (ProgrammingUnitLastLevelRegionDTO programminUnitLastLevelRegion : programmingUnitLastLevelRegionSet) {

			ProgrammingUnitData programmingUnit = programminUnitLastLevelRegion
					.getProgrammingUnit();

			long programminUnitGenerationTime = System.currentTimeMillis();

			logger.info("Processing programming unit: "
					+ programmingUnit.getProgrammingUnitName());

			List<RegionData> lastLevelRegions = programminUnitLastLevelRegion
					.getLastLevelRegionForProgrammingUnit();

			electionMap.put(
					programmingUnit.getCode(),
					getElection(lastLevelRegions, contestsMap, election,
							generationCount++));

			programmingUnitSet.add(programmingUnit);

			totalRegions += lastLevelRegions.size();

			logger.info("Time processing programming unit: "
					+ programmingUnit.getProgrammingUnitName()
					+ " "
					+ TimestampUtil.prettyTimeDifference(
							programminUnitGenerationTime,
							System.currentTimeMillis()));
		}
		return totalRegions;
	}

	/**
	 * Returns the election object filled with the data needed for the files
	 * generation, this data includes eligible options, languages for regions
	 * and contests groups.
	 * 
	 * @param lastLevelRegions
	 *            a list of last level region to process by the current
	 *            generation
	 * @param contestsMap
	 *            a contest map used as a reference
	 * @param election
	 *            the active election
	 * @param generationCount
	 *            when multiple programming units in data, a counter parameter
	 *            for generations must be included.
	 * @return the election object with the data
	 * @throws ConnectionException
	 *             a signal that an error has occurred with the server
	 *             connection
	 * @throws NominationException
	 *             a signal that an error has occurred with getting the eligible
	 *             options data
	 * @throws MasterUSBGenerationException
	 *             a signal that an error has occurred with the generating the
	 *             election request
	 */
	protected ElectionObjectDTO getElection(List<RegionData> lastLevelRegions,
			Map<Long, ContestData> contestsMap, ElectionData election,
			int generationCount) throws ConnectionException,
			NominationException, MasterUSBGenerationException {

		ElectionObjectDTO electionObjectDto = new ElectionObjectDTO();

		ActiveContestForElectionDTO activeContest = ECMServiceProvider
				.getActiveContestForElection(lastLevelRegions, this.language);

		List<ContestGroupData> contestGroups = new ArrayList<ContestGroupData>();
		if (activeContest.getRegionContestGroup() != null) {
			for (List<ContestGroupData> regionContestGroups : activeContest
					.getRegionContestGroup().values()) {
				contestGroups.addAll(regionContestGroups);
			}
		}

		List<ContestData> contestList = getContestsForElection(
				activeContest.getContestList(), contestsMap);

		Collections.sort(contestList, new ContestPrecedenceComparator());

		Map<String, Map<String, String>> resourcesLanguages = activeContest
				.getResourcesLanguages();

		ElectionXmlComponentsDTO electionDto = ECMServiceProvider
				.generateElectionForContest(contestList, generationCount,
						election, resourcesLanguages, contestGroups);

		resourcesLanguages = electionDto.getResourcesLanguages();

		// TODO: This should be checked if needed
		// Map<Long, Set<LanguageData>> regionLanguageMap =
		// replaceRegionLanguages(
		// activeContest.getRegionLanguageMap(),
		// electionDto.getElectionLanguages());
		// electionObjectDto.setRegionLanguageMap(regionLanguageMap);

		electionObjectDto.setRegionLanguageMap(activeContest
				.getRegionLanguageMap());

		electionObjectDto.setElectionDto(electionDto);

		electionObjectDto.setLastLevelRegionToProcess(lastLevelRegions);

		electionObjectDto.setRegionContestGroup(activeContest
				.getRegionContestGroup());

		electionObjectDto.setResourcesLanguages(resourcesLanguages);

		electionObjectDto.setDeviceParameters(
			ECMServiceProvider.retrieveDeviceParameters()
		);
		return electionObjectDto;

	}

	/**
	 * Return the contest list which should be used for current election
	 * generation
	 * 
	 * @param contestList
	 *            the contest list to set
	 * @param contestsMap
	 *            the contest map reference for the whole generation
	 * @return the contest list
	 */
	protected List<ContestData> getContestsForElection(
			List<ContestData> contestList, Map<Long, ContestData> contestsMap) {

		List<ContestData> contestsRequested = new ArrayList<ContestData>();

		this.contestsToProcess = contestList.size();

		for (ContestData contest : contestList) {
			if (!contestsMap.containsKey(contest.getCode())) {
				contestsRequested.add(contest);
				contestsMap.put(contest.getCode(), contest);
			} else {
				contestsRequested.add(contestsMap.get(contest.getCode()));
			}
		}

		return contestsRequested;
	}

	private Map<Long, Set<LanguageData>> replaceRegionLanguages(
			Map<Long, Set<LanguageData>> regionLanguageMap,
			Set<LanguageData> languagesSetToReplace) {

		Map<Long, Set<LanguageData>> newRegionLanguageMap = new HashMap<Long, Set<LanguageData>>();

		for (Long regionCode : regionLanguageMap.keySet()) {
			Set<LanguageData> newLanguageSet = new HashSet<LanguageData>();

			for (LanguageData languageDataToBeReplaced : regionLanguageMap
					.get(regionCode)) {
				for (LanguageData languageDataToReplace : languagesSetToReplace) {
					if (languageDataToBeReplaced.getCustomCode().equals(
							languageDataToReplace.getCustomCode())) {

						newLanguageSet.add(languageDataToReplace);
						break;
					}
				}
			}

			newRegionLanguageMap.put(regionCode, newLanguageSet);
		}
		return newRegionLanguageMap;
	}

	protected List<RegionData> getSelectedRegions() {

		List<RegionData> selectedRegions = new ArrayList<RegionData>();
		for (Object selectedObject : this.selectedObjects) {
			selectedRegions.add((RegionData) selectedObject);
		}

		return selectedRegions;
	}

	@Override
	protected void performOperation() {
		try {
			fireAwtEvent(AwtOperationType.REFRESH_LABEL);
			processSelectedObjects();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	@Override
	protected void onCompleted() {
		logger.info("Master USB contest preparation process completed.");
		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
	}

}
