package com.smartmatic.waffle.ecm.client.worker;

import java.io.File;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.core.lang.PendriveException;
import com.smartmatic.waffle.ecm.core.lang.PendriveException.PendriveExceptionInfo;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * 
 * Worker class that performs server request to get the number of processed
 * contest from the prepare contests worker once invoked from the client.
 *
 */
public class ProcessedContestsPollingWorker extends AsynchronousOperation {

	/** Logger. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ProcessedContestsPollingWorker.class);

	private boolean stopPolling;
	private int processedContests;

	private String mountedPath;

	public ProcessedContestsPollingWorker(String mountedPath) {
		this.mountedPath = mountedPath;
		this.stopPolling = false;
		resetCounter();
	}

	/**
	 * Stops the current polling requests.
	 */
	public void stopPolling() {
		this.stopPolling = true;
		resetCounter();
	}

	/**
	 * Return the number of processed contests.
	 * 
	 * @return the number of processed contests.
	 */
	public int getProcessedContests() {
		return this.processedContests;
	}

	/**
	 * Starts the polling process until the stop signal is sent from the client.
	 * 
	 * @throws InterruptedException
	 *             signal that an error occurred while sleeping the thread.
	 * @throws ConnectionException
	 *             signal that an error occurred getting the processed contests.
	 * @throws PendriveException
	 *             signal that the USB Device has been disconnected.
	 */
	protected void startPolling() throws InterruptedException,
			ConnectionException, PendriveException {

		int lastProcessed = -1;
		while (!this.stopPolling) {
			if (!isDrivePresent()) {
				throw new PendriveException(
						PendriveExceptionInfo.PENDRIVE_NOT_FOUND);
			}

			this.processedContests = ECMServiceProvider.getProcessedContests();

			if (this.processedContests > lastProcessed) {
				lastProcessed = this.processedContests;
				logger.info("Processed contests: " + this.processedContests);
				fireAwtEvent(AwtOperationType.REFRESH_LABEL);
			}

			Thread.sleep(200);
		}
	}

	/**
	 * Reset the processed contest counter.
	 */
	protected void resetCounter() {
		try {
			this.processedContests = 0;
			ECMServiceProvider.resetProcessedContests();
		} catch (ConnectionException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	protected void performOperation() {

		try {
			startPolling();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			stopPolling();
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);

		}
	}

	@Override
	protected void onCompleted() {

	}

	/**
	 * Check if exist the mounted path.
	 * 
	 * @return true if the mounted path exist.
	 */
	private boolean isDrivePresent() {
		File mountedPathFile = new File(this.mountedPath);
		return mountedPathFile.exists();
	}

}
