package com.smartmatic.waffle.ecm.client.worker;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.contest.ContestClassData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.PollingPlaceData;
import com.smartmatic.waffle.domain.demographics.PollingStationData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.demographics.VotingBoothData;
import com.smartmatic.waffle.domain.device.DeviceParameterData;
import com.smartmatic.waffle.domain.device.VotingCentersData.VotingCenter;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.device.VotingDevicesData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.domain.i18n.ContestLanguagesPairData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.i18n.LanguageForVotingDeviceData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.core.lang.PendriveException;
import com.smartmatic.waffle.ecm.core.lang.PendriveException.PendriveExceptionInfo;
import com.smartmatic.waffle.ecm.core.util.TimestampUtil;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionXmlComponentsDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.VotingDeviceDTO;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.ecm.service.lang.MasterUSBGenerationException;
import com.smartmatic.waffle.ecm.service.lang.NominationException;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Worker class that performs the operation and calls the server side methods to
 * prepare the contests and voting devices data for the selected contests from
 * the active election.
 *
 */
public class ScreenshotsPrepareContestsWorker extends AsynchronousOperation {

	/** Static SMTT logger instance. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ScreenshotsPrepareContestsWorker.class);

	protected String mountedPath;

	private int contestsToProcess;

    /**
     * Persisted Device Parameters.
     */
    private List<DeviceParameterData> devParams;

	private LanguageData language;

	private int processedContests;

	private List<Object> selectedObjects;

	private Map<VotingCenter, ElectionData> votingCenterElectionMap;


    public ScreenshotsPrepareContestsWorker(
        List<Object> selectedObjects, LanguageData language, String mountedPath
    ) {
        this.devParams   = new ArrayList<DeviceParameterData>();
        this.language    = language;
        this.mountedPath = mountedPath;
        this.selectedObjects = selectedObjects;
        this.votingCenterElectionMap = new HashMap<VotingCenter, ElectionData>();
    }


	/**
	 * Returns the number of contests to process.
	 * 
	 * @return the number of contests to process
	 */
	public int getContestsToProcess() {
		return this.contestsToProcess;
	}


    /**
     * Returns those {@link DeviceParameterData DeviceParameters} persisted
     * in the ECM data repository.
     *
     * @return
     *     {@code List<DeviceParameterData>} (never {@code null})
     */
    public List<DeviceParameterData> getDeviceParameters() {
        return this.devParams;
    }


	/**
	 * Returns the number of processed contests
	 * 
	 * @return the number of processed contests
	 */
	public int getProcessedContests() {
		return this.processedContests;
	}


	/**
	 * Return a map with a voting center as a key and its corresponding election
	 * used to generate that voting center.
	 * 
	 * @return the voting center election map
	 */
	public Map<VotingCenter, ElectionData> getVotingCenterElectionMap() {

		return this.votingCenterElectionMap;
	}

	/**
	 * Process the selected objects from the client, objects could be contest
	 * classes or contest and will generate the objects required for further
	 * file generation process.
	 * 
	 * @throws NominationException
	 *             signal that an error occurred processing contest or eligible
	 *             option data
	 * @throws ConnectionException
	 *             a signal that an error has occurred with the server
	 *             connection
	 * @throws UtilitiesException
	 *             signal that an error has occurred getting language data
	 * @throws MasterUSBGenerationException
	 *             signal that an error occurred generating contest object
	 * @throws ElectoralOrganizationOperationException
	 *             signal that an error occurred getting the active election
	 *             data
	 */
	public void processSelectedObjects() throws NominationException,
			ConnectionException, UtilitiesException,
			MasterUSBGenerationException,
			ElectoralOrganizationOperationException, PendriveException {

		Set<ContestData> selectedContests = this.getSelectedContests();

		this.contestsToProcess = selectedContests.size();

		int objectCodePrefix = 1;

		int pageSize = 1;
		int totalPages = ClientCommon.getNumberOfPages(selectedContests.size(),
				pageSize);

		List<Long> contestCodes = new ArrayList<Long>();

		for (ContestData contest : selectedContests) {
			contestCodes.add(contest.getCode());
		}

		this.devParams = ECMServiceProvider.retrieveDeviceParameters();
		this.processedContests = 0;

		for (int i = 0; i < totalPages; i++) {

			List<ContestData> contests = ECMServiceProvider.getContests(
					contestCodes, i, pageSize);
			for (ContestData contest : contests) {

				if (!pendriveDisconected()) {
					throw new PendriveException(
							PendriveExceptionInfo.PENDRIVE_NOT_FOUND);
				}

				long contestGenerationTime = System.currentTimeMillis();
				logger.info("Processing contest: " + contest.getName()
						+ " code: " + contest.getCustomCode());

				List<EligibleOptionData> eligibleOptions = ECMServiceProvider
						.getEligibleOptionsByContest(contest);

				List<EligibleOptionData> eligibleOptionsWithEligibleOptionRole = ECMServiceProvider
						.getEligibleOptionRoleByEligibleOption(eligibleOptions);

				List<EligibleOptionData> eligibleOptionsWithBallotOptions = ECMServiceProvider
						.getBallotOptionByEligibleOption(eligibleOptionsWithEligibleOptionRole);

				contest.setEligibleOptions(eligibleOptionsWithBallotOptions);

				Set<LanguageData> electionLanguages = getElectionLanguages(contest);

				VotingDeviceData votingDevice = getVotingDevice(contest);

				if (electionLanguages.size() > 2) {
					generateForMultipleLanguages(contest, votingDevice,
							objectCodePrefix);
				} else {
					generateVotingCenter(contest, votingDevice,
							electionLanguages, objectCodePrefix,
							votingDevice.getCustomCode());
				}

				this.processedContests++;

				fireAwtEvent(AwtOperationType.REFRESH_LABEL);

				logger.info("Time processing contest: "
						+ contest.getCustomCode()
						+ " "
						+ TimestampUtil.prettyTimeDifference(
								contestGenerationTime,
								System.currentTimeMillis()));
			}
		}

	}

	/**
	 * Check if exist the mounted path.
	 * 
	 * @return true if the mounted path exist.
	 */
	private boolean pendriveDisconected() {
		File mountedPathFile = new File(this.mountedPath);
		return mountedPathFile.exists();
	}

	/**
	 * This method will process contest languages combinations and generate a
	 * folder with each of its possible combination.
	 * 
	 * @param contest
	 *            the contest to generate
	 * @param votingDevice
	 *            the voting device for generation
	 * @param objectCodePrefix
	 *            the object code prefix reference
	 * @throws ConnectionException
	 *             a signal that an error has occurred with the server
	 *             connection
	 * @throws UtilitiesException
	 *             signal that an error has occurred getting language data
	 * @throws MasterUSBGenerationException
	 *             signal that an error occurred generating contest object
	 * @throws ElectoralOrganizationOperationException
	 *             signal that an error occurred getting the active election
	 *             data
	 */
	protected void generateForMultipleLanguages(ContestData contest,
			VotingDeviceData votingDevice, int objectCodePrefix)
			throws ConnectionException, UtilitiesException,
			MasterUSBGenerationException,
			ElectoralOrganizationOperationException {

		List<ContestLanguagesPairData> languageCombinations = ECMServiceProvider
				.getLanguageCombinationsByContest(contest);

		String languagesSeparator = getLanguagesSeparator();

		Set<LanguageData> contestLanguages = new HashSet<LanguageData>();

		for (ContestLanguagesPairData contestLanguagesPair : languageCombinations) {
			String folderName = votingDevice.getCustomCode()
					+ languagesSeparator;

			if (contestLanguagesPair.getLanguage1() != null) {
				LanguageData language1 = contestLanguagesPair.getLanguage1();
				folderName = folderName + language1.getLanguageRFC();
				contestLanguages.add(language1);
			}

			if (contestLanguagesPair.getLanguage2() != null) {
				LanguageData language2 = contestLanguagesPair.getLanguage2();
				if (folderName.equals(votingDevice.getCustomCode()
						+ languagesSeparator)) {
					folderName = folderName + language2.getLanguageRFC();
				} else {
					folderName = folderName + languagesSeparator
							+ language2.getLanguageRFC();
				}
				contestLanguages.add(language2);
			}

			generateVotingCenter(contest, votingDevice, contestLanguages,
					objectCodePrefix, folderName);

			contestLanguages.clear();
		}
	}

	/**
	 * Method that generates the data required for the further file generation
	 * process to run an election for the given contest.
	 * 
	 * @param contest
	 *            the contest to generate
	 * @param votingDevice
	 *            the voting device reference
	 * @param languages
	 *            the language set used for the current contest
	 * @param objectCodePrefix
	 * @param folderName
	 *            the folder name to save the election data
	 * @throws MasterUSBGenerationException
	 *             a signal that an error has occurred with the server
	 *             connection
	 * @throws ConnectionException
	 *             a signal that an error has occurred with the server
	 *             connection
	 * @throws ElectoralOrganizationOperationException
	 *             signal that an error occurred getting the active election
	 *             data
	 */
	protected void generateVotingCenter(ContestData contest,
			VotingDeviceData votingDevice, Set<LanguageData> languages,
			int objectCodePrefix, String folderName)
			throws MasterUSBGenerationException, ConnectionException,
			ElectoralOrganizationOperationException {

		Map<String, Map<String, String>> resourcesLanguages = new HashMap<String, Map<String, String>>();

		List<ContestData> contestList = new ArrayList<ContestData>();
		contestList.add(contest);

		ElectionData election = ECMServiceProvider.retrieveActiveElection();

		ElectionXmlComponentsDTO electionDTO = ECMServiceProvider
				.generateElectionForContest(contestList, objectCodePrefix,
						election, resourcesLanguages, null);

		election = electionDTO.getElection();

		List<ContestGroupData> contestGroupList = electionDTO.getContestGroup();

		RegionData regionDummy = new RegionData();

		regionDummy.setCode(contest.getCode());
		regionDummy.setCustomCode(contest.getCustomCode());
		regionDummy.setRegionName(contest.getName());

		votingDevice.setLanguages(languages);

		VotingDeviceDTO votingDeviceDto = ECMServiceProvider
				.generateVotingDeviceForRegion(votingDevice, regionDummy,
						contestGroupList, electionDTO.getResourcesLanguages(),
						true, election, false);

		VotingDevicesData votingDevices = votingDeviceDto
				.getVotingDevicesData();

		votingDevices
				.setXmlnsEl("http://www.smartmatic.com/Saes/Voting/Election");

		List<LanguageData> languageList = new ArrayList<LanguageData>();
		languageList.addAll(languages);

		votingDevices.setLanguages(getVotingDevicesLanguages(languageList));

		VotingCenter votingCenter = new VotingCenter(votingDevice.getName(),
				"", folderName, folderName);

		votingCenter.setVotingDevices(votingDevices);

		this.votingCenterElectionMap.put(votingCenter, election);

	}

	/**
	 * Return the languages for a given contest
	 * 
	 * @param contest
	 *            the contest to search the languages
	 * @return the set of languages that applies to the given contest
	 * @throws ConnectionException
	 *             a signal that an error has occurred with the server
	 *             connection
	 * @throws UtilitiesException
	 *             signal that an error has occurred getting language data
	 */
	protected Set<LanguageData> getElectionLanguages(ContestData contest)
			throws ConnectionException, UtilitiesException {

		Set<LanguageData> languages = new HashSet<LanguageData>();
		List<String> languagesNames = new ArrayList<String>();

		List<LanguageData> parentLanguages = ECMServiceProvider
				.getParentLanguages(contest);

		if (parentLanguages != null) {
			for (LanguageData language : parentLanguages) {
				if (!languagesNames.contains(language.getLanguageRFC())) {
					languagesNames.add(language.getLanguageRFC());
					languages.add(language);
				}
			}
		}
		List<LanguageData> childrenLanguages = ECMServiceProvider
				.getChildrenLanguages(contest);

		if (childrenLanguages != null) {
			for (LanguageData language : childrenLanguages) {
				if (!languagesNames.contains(language.getLanguageRFC())) {
					languagesNames.add(language.getLanguageRFC());
					languages.add(language);
				}
			}
		}

		return languages;
	}

	/**
	 * Returns a voting device object filled with contest data, required to
	 * generate the election object.
	 * 
	 * @param contest
	 *            the contest object reference
	 * @return the voting device object
	 */
	protected VotingDeviceData getVotingDevice(ContestData contest) {

		VotingDeviceData votingDevice = new VotingDeviceData();
		votingDevice.setCustomCode(contest.getCustomCode());
		votingDevice.setName(contest.getName());

		PollingPlaceData pollingPlace = new PollingPlaceData();
		pollingPlace.setCode(contest.getCode());
		pollingPlace.setCustomCode(contest.getCustomCode());
		pollingPlace.setName(contest.getName());
		pollingPlace.setAddress("N/A");

		PollingStationData pollingStation = new PollingStationData();
		pollingStation.setCode(contest.getCode());
		pollingStation.setCustomCode(contest.getCustomCode());
		pollingStation.setPollingPlace(pollingPlace);
		pollingStation.setSplitCriteria("N/A");

		VotingBoothData votingBooth = new VotingBoothData();
		votingBooth.setCode(contest.getCode());
		votingBooth.setCustomCode(contest.getCustomCode());

		votingBooth.setPollingStation(pollingStation);

		Set<VotingBoothData> votingBoothSet = new HashSet<VotingBoothData>();
		votingBoothSet.add(votingBooth);

		votingDevice.setVotingBooths(votingBoothSet);
		votingDevice.setVotingBooth(votingBooth);

		return votingDevice;
	}

	/**
	 * Returns a list with the contests selected from the client.
	 * 
	 * @return a list of contests selected.
	 * @throws NominationException
	 *             signal that an error occurred processing contest or eligible
	 *             option data
	 * @throws ConnectionException
	 *             a signal that an error has occurred with the server
	 *             connection
	 */
	protected Set<ContestData> getSelectedContests()
			throws NominationException, ConnectionException {

		Set<ContestData> selectedContests = new HashSet<ContestData>();
		for (Object selectedObject : this.selectedObjects) {
			if (selectedObject instanceof ContestClassData) {
				ContestClassData contestClass = (ContestClassData) selectedObject;
				selectedContests.addAll(ECMServiceProvider
						.getContestByContestClass(contestClass.getCustomCode(),
								this.language));
			} else if (selectedObject instanceof ContestData) {
				selectedContests.add((ContestData) selectedObject);
			}
		}

		return selectedContests;
	}

	/**
	 * Returns the language separator character used when generating the folder
	 * names.
	 * 
	 * @return the language separator
	 */
	protected String getLanguagesSeparator() {
		return ECMClientSideConstants.SCREENSHOT_REPORT_LANGUAGES_SEPARATOR;
	}

	@Override
	protected void performOperation() {

		try {
			fireAwtEvent(AwtOperationType.REFRESH_LABEL);
			processSelectedObjects();

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	@Override
	protected void onCompleted() {
		logger.info("Screenshots contest preparation process completed.");
		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
	}

	/**
	 * Returns a list of language for voting device object required when
	 * generating the files for voting machines given a list of languages
	 * 
	 * @param languages
	 *            a list of languages
	 * @return the list of voting devices languages.
	 */
	private List<LanguageForVotingDeviceData> getVotingDevicesLanguages(
			List<LanguageData> languages) {

		List<LanguageForVotingDeviceData> languageForVotingDeviceList = new ArrayList<LanguageForVotingDeviceData>();

		for (LanguageData language : languages) {
			LanguageForVotingDeviceData languageForVotingDevice = new LanguageForVotingDeviceData();

			languageForVotingDevice.setCode(language.getCode());
			languageForVotingDevice.setCustomCode(language.getCustomCode());
			languageForVotingDevice.setOfficialLanguage(language
					.getOfficialLanguage());
			languageForVotingDevice.setAudio(language.isAudio());
			languageForVotingDevice.setLanguageName(language.getLanguageName());
			languageForVotingDevice.setLanguageRFC(language.getLanguageRFC());
			languageForVotingDevice.setCountry(language.getCountry());
			languageForVotingDevice.setKey(' ');
			languageForVotingDevice.setMessage(language.getMessage());
			languageForVotingDevice.setPrecedence(language.getPrecedence());

			languageForVotingDeviceList.add(languageForVotingDevice);
		}

		return languageForVotingDeviceList;
	}

}
