package com.smartmatic.waffle.ecm.client.worker;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.general.KeystoreData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.util.PDFSigner;
import com.smartmatic.waffle.ecm.core.util.SAESCommandUtil;
import com.smartmatic.waffle.ecm.core.util.SAESZipUtil;
import com.smartmatic.waffle.ecm.data.transfer.comparators.ScreenShotImageFileComparator;
import com.smartmatic.waffle.ecm.data.transfer.objects.ContestLanguagesDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ImageFileDTO;
import com.smartmatic.waffle.ecm.service.lang.ElectoralDemographyException;
import com.smartmatic.waffle.ecm.service.lang.MasterUSBGenerationException;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Worker class that process from a mounted drive the screenshots previously
 * generated by the voting machine and will create a compressed zip file with
 * the PDF files generated.
 *
 */
public class ScreenshotsProcessingWorker extends AsynchronousOperation {

	/** Logger. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ScreenshotsProcessingWorker.class);

	private String mountedPath;
	private List<ContestData> contestList;

	private Set<LanguageData> electionLanguages;

	private File tmpScreenshotsFolder;

	private File tmpScreenshotsImgsFolder;

	private File tmpScreenshotsFile;

	private String contestName;

	private int processingProgress;

	private boolean processingDone;

	public ScreenshotsProcessingWorker(String mountedPath,
			List<ContestData> contestList) {

		this.mountedPath = mountedPath;
		this.contestList = contestList;
	}

	/**
	 * Returns the contest name being processed.
	 * 
	 * @return the contest name
	 */
	public String getContestName() {
		return this.contestName;
	}

	/**
	 * Returns the progress of the current processing operation
	 * 
	 * @return the progress value
	 */
	public int getProcessingProgress() {
		return this.processingProgress;
	}

	/**
	 * Return the status of the current processing
	 * 
	 * @return true if done, otherwise false.
	 */
	public boolean isProcessingDone() {
		return this.processingDone;
	}

	@Override
	protected void performOperation() {

		logger.info("Starting screenshots processing");
		try {
			fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
			startProcessing();
			fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	@Override
	protected void onCompleted() {
		logger.info("Screenshots processing done.");
	}

	/**
	 * Start processing the screenshots PNG files
	 */
	protected void startProcessing() throws Exception {

		this.processingDone = false;

		prepareFolders();

		loadElectionLanguages();

		Map<String, ContestLanguagesDTO> contestLanguageMap = getContestLanguageMap(this.contestList);

		processContests(contestLanguageMap);

		this.processingDone = true;
	}

	/**
	 * Process the contests folder that were found in the drive for each
	 * language that applies to the contest.
	 * 
	 * @param contestLanguageMap
	 *            the map with the contest custom code as a key and a
	 *            <code>ContestLanguagesDTO</cod> object as value
	 * @throws Exception
	 *             if an error occurred processing the contest
	 */
	protected void processContests(
			Map<String, ContestLanguagesDTO> contestLanguageMap)
			throws Exception {

		File screenshotsFolder = new File(this.mountedPath + File.separator
				+ getPathInDriveForScreenshots());

		String screenshotReportLanguagesSeparator = getScreenshotsReportLanguagesSeparator();

		File[] imagesFolderList = getImageFolders(screenshotsFolder);

		int totalFoldersToProcess = imagesFolderList.length;

		int imageFoldersProcessed = 0;

		for (File screenshotsImagesFolder : imagesFolderList) {

			String contestCustomCode = StringUtils.split(
					screenshotsImagesFolder.getName(),
					screenshotReportLanguagesSeparator)[0];

			logger.info("Processing contest: " + contestCustomCode);

			ContestLanguagesDTO contestLanguage = contestLanguageMap
					.get(contestCustomCode);

			if (contestLanguage != null) {

				this.contestName = getContestNameTranslated(contestLanguage
						.getContestData());

				fireAwtEvent(AwtOperationType.REFRESH_LABEL);

				processContestFolder(screenshotsImagesFolder, contestLanguage);

			}

			this.processingProgress = (++imageFoldersProcessed * 100)
					/ totalFoldersToProcess;

			fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
		}

		generateZip();
	}

	/**
	 * Process the files contained in the contest folder, if the folder contains
	 * selection images (generated by the voting machine) this method will
	 * create a different PDF with those files.
	 * 
	 * @param screenshotsImagesFolder
	 *            the screenshots folder to process
	 * @param contestLanguage
	 *            the contest language object which contains contest and
	 *            language data
	 * @throws ConnectionException
	 *             if an error occurred during the client server connection
	 * @throws UtilitiesException
	 *             if an error occurred translating data
	 * @throws Exception
	 *             if an error occurred processing files
	 */
	protected void processContestFolder(File screenshotsImagesFolder,
			ContestLanguagesDTO contestLanguage) throws ConnectionException,
			UtilitiesException, Exception {

		String screenshotReportLanguagesSeparator = getScreenshotsReportLanguagesSeparator();

		List<LanguageData> languagesHeaderList = contestLanguage
				.getLanguageHeaderList();

		List<LanguageData> languagesFooterList = contestLanguage
				.getLanguageFooterList();

		ContestData contest = contestLanguage.getContestData();

		for (LanguageData languageFooter : languagesFooterList) {

			String footerText = getFooterText(contest, languageFooter);

			for (LanguageData languageHeader : languagesHeaderList) {

				if (languagesFooterList.size() > 1
						&& !languageFooter.getLanguageRFC().equals(
								languageHeader.getLanguageRFC())) {
					continue;
				}

				String languageCode = languageHeader.getLanguageRFC()
						.toLowerCase()
						+ "_"
						+ languageHeader.getCountry().toLowerCase();

				String pdfFilename = contest.getCustomCode()
						.concat(screenshotReportLanguagesSeparator)
						.concat(languageHeader.getLanguageRFC());

				File[] pngFiles = getPNGFiles(screenshotsImagesFolder,
						languageCode);

				if (pngFiles != null && pngFiles.length != 0) {

					processContestFiles(pngFiles, pdfFilename + ".pdf",
							footerText);
				}

				File[] selectionPngFiles = getSelectionPNGFiles(
						screenshotsImagesFolder, languageCode);

				if (selectionPngFiles != null && selectionPngFiles.length != 0) {

					processContestFiles(selectionPngFiles, pdfFilename
							+ ".selection.pdf", footerText);

				}
			}
		}
	}

	/**
	 * Process a list of list found in the folder an executes a conversion from
	 * PNG to PDF format adding footer text and number of page in each file.
	 * 
	 * @param files
	 *            the list of files to process
	 * @param pdfFilename
	 *            the filename for the result PDF file.
	 * @param footerText
	 *            the footer to add to each image
	 * @throws Exception
	 *             if an error occurred during conversion process
	 */
	protected void processContestFiles(File[] files, String pdfFilename,
			String footerText) throws Exception {

		List<ImageFileDTO> imageFileDTOList = getImageFileDtoList(files);

		List<File> pngToPdfList = new ArrayList<File>();

		int pageNumber = 1;

		for (ImageFileDTO imageFileDTO : imageFileDTOList) {

			logger.info("Processing screenshot file: "
					+ imageFileDTO.getFileReference().getName());

			File modifiedImageFile = new File(this.tmpScreenshotsImgsFolder,
					imageFileDTO.getFileReference().getName());

			Process process = executeConvert(imageFileDTO.getFileReference()
					.getAbsolutePath(), modifiedImageFile.getAbsolutePath(),
					footerText, pageNumber, files.length);

			String convertProcessError = IOUtils.toString(process
					.getErrorStream());

			if (!convertProcessError.isEmpty()) {
				logger.error(convertProcessError);
				throw new IOException(convertProcessError);
			}

			pngToPdfList.add(modifiedImageFile);

			pageNumber++;
		}

		File pdfFile = buildPdf(pngToPdfList, pdfFilename);

		if (signScreenshotsPdf()) {
			signPDF(pdfFile);
		}
		logger.info("Successfuly generated: " + pdfFilename);
	}

	/**
	 * Generates and sort a list of ImageFileDTO from a list o files
	 * 
	 * @param files
	 *            the list of files to generate
	 * @return the list of ImageFileDTO objects
	 */
	private List<ImageFileDTO> getImageFileDtoList(File[] files) {
		List<ImageFileDTO> imageFileDTOList = new ArrayList<ImageFileDTO>();

		for (File pngFile : files) {

			ImageFileDTO imageFileDto = new ImageFileDTO();
			imageFileDto.setFileName(StringUtils.remove(pngFile.getName(),
					".png"));
			imageFileDto.setFileReference(pngFile);
			imageFileDTOList.add(imageFileDto);

		}

		Collections.sort(imageFileDTOList, new ScreenShotImageFileComparator());

		return imageFileDTOList;
	}

	/**
	 * Sign a PDF file with the election keystore used for the generation.
	 * 
	 * @param pdfFile
	 *            the PDF file to sign
	 * @throws Exception
	 *             an error occurred signing the file
	 */
	private void signPDF(File pdfFile) throws Exception {

		ElectionData election = ECMServiceProvider.retrieveActiveElection();

		KeystoreData keystoreData = election.getKeystore();

		String password = keystoreData.getPassword();
		String alias = keystoreData.getAlias();
		byte[] certificate = keystoreData.getPkcs12().getBinaryData();

		ByteArrayInputStream keystoreStream = new ByteArrayInputStream(
				certificate);

		try {

			byte[] pdfFileSigned = PDFSigner.sign(pdfFile.getAbsolutePath(),
					keystoreStream, alias, password);

			pdfFile.delete();

			FileUtils.writeByteArrayToFile(new File(getTmpScreenshotsFolder(),
					pdfFile.getName()), pdfFileSigned);
		} catch (Exception e) {
			logger.error("An error occurred during the PDF signing process for the file: "
					+ pdfFile.getName());
			throw e;
		} finally {
			if (keystoreStream != null) {
				keystoreStream.close();
			}
		}

	}

	/**
	 * Generates a compressed zip file of the folder which contains the
	 * generated PDFs.
	 * 
	 * @throws IOException
	 *             if an error occurred during the compression process
	 */
	private void generateZip() throws IOException {

		SAESZipUtil.zipFolder(getTmpScreenshotsFolder().getAbsolutePath(),
				this.tmpScreenshotsFile.getAbsolutePath());

		if (getTmpScreenshotsFolder().exists()) {
			FileUtils.forceDelete(getTmpScreenshotsFolder());
		}

	}

	/**
	 * Builds a single PDF file from a list of PDFs.
	 * 
	 * @param imagesList
	 *            the images PDF files
	 * @param pdfFilename
	 *            the filename for the result PDF
	 * @return the PDF file built
	 * @throws Exception
	 *             if an error occurred building the PDF
	 */
	private File buildPdf(List<File> imagesList, String pdfFilename)
			throws Exception {

		logger.info("Building " + pdfFilename);

		File pdfFile = new File(getTmpScreenshotsFolder(), pdfFilename);

		List<String> imagesParameterList = new ArrayList<String>();

		StringBuilder parameters = new StringBuilder();

		for (File modifiedImage : imagesList) {

			parameters.append(" %s");
			imagesParameterList.add(modifiedImage.getAbsolutePath());

		}

		Object[] parameter = imagesParameterList.toArray(new String[0]);

		String formatted = String.format(parameters.toString(), parameter);

		Process buildPngProcess = SAESCommandUtil.execute("convert", formatted,
				pdfFile.getAbsolutePath());

		String buildProcessError = IOUtils.toString(buildPngProcess
				.getErrorStream());

		if (!buildProcessError.isEmpty()) {
			logger.error(buildProcessError);
			throw new Exception(buildProcessError);
		}

		return pdfFile;
	}

	/**
	 * Executes the conversion from PNG to PDF and also adds a footer which
	 * includes the text, page number and a total of pages
	 * 
	 * @param fileReferencePath
	 *            the file path that will be converted
	 * @param convertedFilePath
	 *            the converted file path
	 * @param footerText
	 *            the text to include
	 * @param pageNumber
	 *            the page number
	 * @param totalPages
	 *            the total of pages
	 * @return the process that executes the conversion
	 */
	private Process executeConvert(String fileReferencePath,
			String convertedFilePath, String footerText, int pageNumber,
			int totalPages) {

		Process process = SAESCommandUtil
				.execute("convert", fileReferencePath, "-extent 1024x1300",
						"-pointsize 20", "-draw \"text 4,1294 '" + footerText
								+ "'\"", "-fill black",
						"-annotate +970+1294 \"" + pageNumber + "/"
								+ totalPages + "\"", convertedFilePath);
		return process;

	}

	/**
	 * Returns the footer text translated in the language given
	 * 
	 * @param contestData
	 *            the contest data
	 * @param language
	 *            the language to be translated
	 * @return the footer text translated
	 * @throws ConnectionException
	 *             if an error occurred during the server connection
	 * @throws UtilitiesException
	 *             if an error occurred getting the district name
	 */
	private String getFooterText(ContestData contestData, LanguageData language)
			throws ConnectionException, UtilitiesException {

		DistrictData district = contestData.getDistrict();

		district = ECMServiceProvider.getDistrictNameTranslated(district,
				language);

		return district.getDistrictNameTraslated();
	}

	/**
	 * Returns the number of directories from a parent directory
	 * 
	 * @param parentFolder
	 *            the parent folder
	 * @return an array of folders
	 */
	private File[] getImageFolders(File parentFolder) {

		File[] imagesFolder = parentFolder.listFiles(new FileFilter() {

			@Override
			public boolean accept(File file) {
				return file.isDirectory();
			}
		});

		return imagesFolder;
	}

	/**
	 * Returns an array of screenshots given the language code
	 * 
	 * @param screenshotsImagesFolder
	 *            the folder to find the screenshots files
	 * @param languageCode
	 *            the language code
	 * @return the array of screenshots files
	 */
	private File[] getPNGFiles(File screenshotsImagesFolder,
			final String languageCode) {

		FilenameFilter filter = new FilenameFilter() {

			@Override
			public boolean accept(File dir, String name) {
				String lowercaseName = name.toLowerCase();
				if (!StringUtils.contains(lowercaseName, "zz")
						&& StringUtils.contains(lowercaseName, languageCode)
						&& lowercaseName.endsWith(".png")) {
					return true;
				} else {
					return false;
				}
			}
		};

		File[] files = screenshotsImagesFolder.listFiles(filter);

		return files;
	}

	/**
	 * Returns an array of selection screenshots given the language code
	 * 
	 * @param screenshotsImagesFolder
	 *            the folder to find the screenshots files
	 * @param languageCode
	 *            the language code
	 * @return the array of screenshots files
	 */
	private File[] getSelectionPNGFiles(File screenshotsImagesFolder,
			final String languageCode) {

		FilenameFilter filter = new FilenameFilter() {

			@Override
			public boolean accept(File dir, String name) {
				String lowercaseName = name.toLowerCase();
				String regex = ECMClientSideConstants.REGEX_FOR_SCREENSHOT_SELECTION_IMAGES;
				if (lowercaseName.matches(regex)
						&& StringUtils.contains(lowercaseName, languageCode)) {
					return true;
				} else {
					return false;
				}
			}
		};

		File[] files = screenshotsImagesFolder.listFiles(filter);

		return files;
	}

	/**
	 * Return a map of contest customs codes as keys and a ContestLanguagesDTO
	 * object as a value
	 * 
	 * @param contests
	 *            the list of contests for the current election
	 * @return a map of contest and ContestLanguagesDTO objects
	 * @throws ElectoralDemographyException
	 *             if an error occurred getting the district languages
	 * @throws ConnectionException
	 *             if an error occurred during the server connection
	 */
	private Map<String, ContestLanguagesDTO> getContestLanguageMap(
			List<ContestData> contests) throws ElectoralDemographyException,
			ConnectionException {

		HashMap<String, ContestLanguagesDTO> contestLanguageMap = new HashMap<String, ContestLanguagesDTO>();

		for (ContestData contestData : this.contestList) {

			ContestLanguagesDTO contestLanguage = new ContestLanguagesDTO();
			contestLanguage.setContestData(contestData);
			contestLanguage.setLanguageHeaderList(ECMServiceProvider
					.getDistrictLanguages(contestData.getDistrict()
							.getCustomCode()));

			contestLanguage
					.setLanguageFooterList(getFooterLanguageList(contestData));

			contestLanguageMap
					.put(contestData.getCustomCode(), contestLanguage);
		}
		return contestLanguageMap;
	}

	/**
	 * Return a list of the contest footer languages for the screenshots
	 * 
	 * @param contest
	 *            the contest data object to find its footer languages
	 * @return the list of languages
	 */
	private List<LanguageData> getFooterLanguageList(ContestData contest) {

		String footerLanguages = contest.getCollegeLanguageRFC();

		String[] footerLanguageCustomCodes = StringUtils.split(footerLanguages,
				ECMClientSideConstants.SCREENSHOT_FOOTER_LANGUAGES_SEPARATOR);

		List<LanguageData> footerLanguageList = new ArrayList<LanguageData>();

		for (String languageCustomCode : footerLanguageCustomCodes) {
			LanguageData languageData = getLanguageByCustomCode(languageCustomCode);

			if (languageData != null) {
				footerLanguageList.add(languageData);
			}
		}

		return footerLanguageList;
	}

	/**
	 * Returns a contest name translated to the selected language
	 * 
	 * @param contest
	 *            the contest to translate its name
	 * @return the contest name translated
	 * @throws UtilitiesException
	 *             if an error occurred translating the contest name
	 * @throws ConnectionException
	 *             if an error occurred during the server connection
	 */
	private String getContestNameTranslated(ContestData contest)
			throws UtilitiesException, ConnectionException {

		LanguageData language = LanguageController.getInstance()
				.getSelectedLanguage();

		ContestData contestTranslated = ECMServiceProvider
				.getContestNameTranslated(contest, language);

		return contestTranslated.getContestNameTranslate();
	}

	/**
	 * Return a language data object given its custom code
	 * 
	 * @param customCode
	 *            the custom code to find
	 * @return the language data object matched
	 */
	private LanguageData getLanguageByCustomCode(String customCode) {

		for (LanguageData language : this.electionLanguages) {
			if (language.getCustomCode().equals(customCode)) {
				return language;
			}
		}
		return null;
	}

	/**
	 * Load all the election languages into class object to further use
	 * 
	 * @throws MasterUSBGenerationException
	 *             if an error occurred
	 * @throws ConnectionException
	 *             if an error occurred during the server connection
	 */
	private void loadElectionLanguages() throws MasterUSBGenerationException,
			ConnectionException {

		this.electionLanguages = ECMServiceProvider.getLanguages();
	}

	/**
	 * Removes old data and creates the folders that the processing will use.
	 * 
	 * @throws IOException
	 *             if an error occurred during the preparation
	 */
	private void prepareFolders() throws IOException {

		this.tmpScreenshotsFolder = new File(
				ECMClientSideConstants.TEMP_PATH_IN_FILE_SYSTEM_FOR_SCREEN_SHOT);

		this.tmpScreenshotsImgsFolder = new File(
				ECMClientSideConstants.TEMP_PATH_IN_FILE_SYSTEM_FOR_SCREEN_SHOT_IMG);

		this.tmpScreenshotsFile = new File(
				this.tmpScreenshotsFolder.getParent(),
				ECMClientSideConstants.SCREEN_SHOT_FILE);

		if (this.tmpScreenshotsFolder.exists()) {
			FileUtils.forceDelete(this.tmpScreenshotsFolder);
		}
		this.tmpScreenshotsFolder.mkdirs();

		if (this.tmpScreenshotsImgsFolder.exists()) {
			FileUtils.forceDelete(this.tmpScreenshotsImgsFolder);
		}
		this.tmpScreenshotsImgsFolder.mkdirs();

		if (this.tmpScreenshotsFile.exists()) {
			FileUtils.forceDelete(this.tmpScreenshotsFile);
		}

	}

	/**
	 * Returns the screenshots path to locate the files
	 * 
	 * @return the screenshots path
	 */
	private String getPathInDriveForScreenshots() {
		return ECMClientSideConstants.PATH_IN_DRIVE_FOR_SCREEN_SHOT;
	}

	/**
	 * Returns the language separator used by the screenshots generation.
	 * 
	 * @return the language separator
	 */
	private String getScreenshotsReportLanguagesSeparator() {
		return ECMClientSideConstants.SCREENSHOT_REPORT_LANGUAGES_SEPARATOR;
	}

	/**
	 * True if the result PDF file must be signed, otherwise false.
	 * 
	 * @return true if must be signed, otherwise false.
	 */
	private boolean signScreenshotsPdf() {
		return ECMClientSideConstants.SIGN_SCREENSHOT_REPORT_PDF;
	}

	/**
	 * Return the temporary folder path for screenshots PDF generation
	 * 
	 * @return the folder path
	 */
	private File getTmpScreenshotsFolder() {
		return this.tmpScreenshotsFolder;
	}
}
