package com.smartmatic.waffle.ecm.client.worker;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionRoleData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionRoleData.Role;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.data.transfer.objects.EligibleOptionSearchDTO;
import com.smartmatic.waffle.ecm.service.lang.NominationException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * This class is used to search candidate objects sending the request to the
 * server and updating the status and the data in the client.
 * 
 */
public class SearchCandidateWorker extends AsynchronousOperation {

	/**
	 * Static reference to log any info or error which occurred executing
	 * methods from this class.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(SearchCandidateWorker.class);

	private List<EligibleOptionSearchDTO> candidatesList;

	private String candidateSearchText;
	private List<RegionData> regions;
	private List<ContestData> contests;
	private List<PartyData> parties;
	private LanguageData language;
	private int pageNumber;
	private int totalNumberOfPages;
	private String candidateTypeMain;
	private String candidateTypeAlternate;

	/**
	 * Creates a new instance of the SearchCandidateWorker
	 * 
	 * @param candidateSearchText
	 *            the candidate search text
	 * @param regions
	 * 
	 * @param contests
	 *            the contest list filter
	 * @param parties
	 *            the parties list filter
	 * @param language
	 *            the current client language
	 * @param pageNumber
	 *            the page number to fetch data
	 */
	public SearchCandidateWorker(String candidateSearchText,
			List<RegionData> regions, List<ContestData> contests,
			List<PartyData> parties, LanguageData language, int pageNumber) {
		this.candidateSearchText = candidateSearchText;
		this.regions = regions;
		this.contests = contests;
		this.parties = parties;
		this.language = language;
		this.pageNumber = pageNumber;
	}

	/**
	 * Gets the total number of pages from the current search
	 * 
	 * @return the total number of pages
	 */
	public int getTotalNumberOfPages() {
		return this.totalNumberOfPages;
	}

	/**
	 * Gets the candidates list from the current search
	 * 
	 * @return the candidates list
	 */
	public List<EligibleOptionSearchDTO> getCandidatesList() {
		return this.candidatesList;
	}

	/**
	 * Sets the main candidate's name to show in the results table.
	 * 
	 * @param candidateTypeMain
	 *            the main candidate's name to show.
	 */
	public void setCandidateTypeMain(String candidateTypeMain) {
		this.candidateTypeMain = candidateTypeMain;
	}

	/**
	 * Sets the alternate candidate's name to show in the results table.
	 * 
	 * @param candidateTypeAlternate
	 *            the alternate candidate's name to show.
	 */
	public void setCandidateTypeAlternate(String candidateTypeAlternate) {
		this.candidateTypeAlternate = candidateTypeAlternate;
	}

	/**
	 * Sets the total number of pages from the current search
	 * 
	 * @param pageSize
	 *            the page size
	 * @throws NominationException
	 *             a signal that an error occurred retrieving the total number
	 *             of candidates
	 * @throws ConnectionException
	 *             a signal that an error occurred with the server connection
	 */
	public void setTotalNumberOfPages(int pageSize) throws NominationException,
			ConnectionException {

		int totalNumberOfElements = ECMServiceProvider
				.retrieveNumberOfCandidates(this.candidateSearchText,
						this.regions, this.contests, this.parties);

		this.totalNumberOfPages = ClientCommon.getNumberOfPages(
				totalNumberOfElements, pageSize);

		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
	}

	/**
	 * Gets the list of candidates from the current search
	 * 
	 * @param pageSize
	 *            the current page size
	 * @return the list of candidates
	 * @throws NominationException
	 *             a signal that an error occurred retrieving the list of
	 *             candidates
	 * @throws ConnectionException
	 *             a signal that an error occurred with the server connection
	 */
	public List<EligibleOptionSearchDTO> getCandidates(int pageSize)
			throws NominationException, ConnectionException {

		List<EligibleOptionData> candidates = ECMServiceProvider.getCandidates(
				this.pageNumber, pageSize, this.candidateSearchText,
				this.regions, this.contests, this.parties, this.language);

		List<EligibleOptionSearchDTO> candidatesList = new ArrayList<EligibleOptionSearchDTO>();
		String[] values;
		for (EligibleOptionData indexResult : candidates) {

			EligibleOptionRoleData eligibleOptionRol = indexResult
					.getEligibleOptionRoleData().first();

			values = new String[6];

			values[0] = indexResult.getBallotName().trim().replace('#', ' ');
			values[1] = eligibleOptionRol.getCandidate().getIdentification();
			values[2] = (eligibleOptionRol.getRole() == Role.CHIEF) ? this.candidateTypeMain
					: this.candidateTypeAlternate;
			values[3] = indexResult.getContest().getContestNameTranslate();
			values[4] = indexResult.getEligibleOptionParent().getBallotName();
			values[5] = null;

			EligibleOptionSearchDTO eligibleOptionDTO = new EligibleOptionSearchDTO();
			eligibleOptionDTO.setEligibleOption(indexResult);
			eligibleOptionDTO.setValues(values);
			candidatesList.add(eligibleOptionDTO);
		}
		return candidatesList;
	}

	@Override
	protected void performOperation() {
		try {

			fireAwtEvent(AwtOperationType.DISABLE_COMPONENTS);

			int pageSize = Integer
					.valueOf(ECMClientSideConstants.NUMBER_OF_PAGES_FOR_SEARCH_ELIGIBLE_OPTION);

			setTotalNumberOfPages(pageSize);
			this.candidatesList = getCandidates(pageSize);

			fireAwtEvent(AwtOperationType.SHOW_RESULT_MESSAGE);

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} finally {
			fireAwtEvent(AwtOperationType.ENABLE_COMPONENTS);
		}
	}

	@Override
	protected void onCompleted() {
		fireAwtEvent(AwtOperationType.ENABLE_COMPONENTS);
	}
}
