/**
 * KeyGenerationWorker.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.service.lang.DemographyException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralPhysicalLocationsRegistryException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * This class is used to perform the operation of security components generation
 * sending the request to the server application and updating the status to the
 * client.
 */
public class SecurityComponentsGenerationWorker extends AsynchronousOperation {

	/** Static SMTT logger instance. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(SecurityComponentsGenerationWorker.class);

	/** progressBar value. */
	private int progressBarValue;

	/** The error bundle key to show to the client. */
	private String errorBundleKey;

	/** The maximum value of the progress bar. */
	private int maxProgressBar;

	/** The start time of generation process. */
	private long startTime;

	/** The stop time of the generation process. */
	private long stopTime;

	/**
	 * Gets the start time of generation process.
	 * 
	 * @return the start time of generation process
	 */
	public long getStartTime() {
		return this.startTime;
	}

	/**
	 * Sets the start time of generation process.
	 * 
	 * @param startTime
	 *            the new start time of generation process
	 */
	public void setStartTime(long startTime) {
		this.startTime = startTime;
	}

	/**
	 * Gets the stop time of the generation process.
	 * 
	 * @return the stop time of the generation process
	 */
	public long getStopTime() {
		return this.stopTime;
	}

	/**
	 * Sets the stop time of the generation process.
	 * 
	 * @param stopTime
	 *            the new stop time of the generation process
	 */
	public void setStopTime(long stopTime) {
		this.stopTime = stopTime;
	}

	/**
	 * Gets the maximum value of the progress bar.
	 * 
	 * @return the maximum value of the progress bar
	 */
	public int getMaxProgressBar() {
		return this.maxProgressBar;
	}

	/**
	 * Sets the maximum value of the progress bar.
	 * 
	 * @param maxProgressBar
	 *            the new maximum value of the progress bar
	 */
	public void setMaxProgressBar(int maxProgressBar) {
		this.maxProgressBar = maxProgressBar;
	}

	/**
	 * Gets the progress bar value.
	 * 
	 * @return the progress bar value
	 */
	public int getProgressBarValue() {
		return this.progressBarValue;
	}

	/**
	 * Sets the progress bar value.
	 * 
	 * @param progressBarValue
	 *            the new progress bar value
	 */
	public void setProgressBarValue(int progressBarValue) {
		this.progressBarValue = progressBarValue;
	}

	/**
	 * Gets the error message to show to the client.
	 * 
	 * @return the error message to show to the client
	 */
	public String getErrorBundleKey() {
		return this.errorBundleKey;
	}

	/**
	 * Sets the error message to show to the client.
	 * 
	 * @param errorMessage
	 *            the new error message to show to the client
	 */
	public void setErrorBundleKey(String errorMessage) {
		this.errorBundleKey = errorMessage;
	}

	/**
	 * Show error message.
	 * 
	 * @param errorClass
	 *            the error class
	 * @param errorMessage
	 *            the error message
	 */
	private void showErrorMessage(String errorBundleKey) {
		setErrorBundleKey(errorBundleKey);
		fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		fireAwtEvent(AwtOperationType.ENABLE_COMPONENTS);
	}

	/**
	 * Starts security components generation process.
	 * 
	 * @throws ConnectionException
	 *             Signal that an error occurred with the connection to the
	 *             server
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             Signal that an error occurred performing an operation with
	 *             the electoral locations registry service
	 * @throws DemographyException
	 *             Signal that an error occurred performing an operation with
	 *             the demography service.
	 * @throws ElectoralOrganizationOperationException
	 *             Signal that an error occurred performing an operation with
	 *             the electoral organization operation service
	 */
	public void startSecurityComponentsGeneration() throws ConnectionException,
			ElectoralPhysicalLocationsRegistryException, DemographyException,
			ElectoralOrganizationOperationException {

		setProgressBarValue(0);
		fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);

		boolean useProgrammingUnit = ECMClientSideConstants.GENERATE_FILES_USING_PROGRAMMING_UNIT;
		boolean generateElectoralAuthorities = ECMClientSideConstants.GENERATE_TOTALIZATION_DEVICE;

		setStartTime(System.currentTimeMillis());

		ECMServiceProvider.deleteVotingDeviceSecurityComponents();
		ECMServiceProvider.deleteProgrammingUnitSecurityComponents();
		ECMServiceProvider.deleteElectoralAuthoritiesSecurityComponents();

		setSecurityComponents(useProgrammingUnit, generateElectoralAuthorities);

		setStopTime(System.currentTimeMillis());
		fireAwtEvent(AwtOperationType.SHOW_RESULT_MESSAGE);

		setProgressBarValue(100);
		fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);

	}

	/**
	 * Sets the security components to programming units and voting devices.
	 * 
	 * @param useProgrammingUnit
	 *            To indicate if programming units security components should be
	 *            generated.
	 * @throws ConnectionException
	 *             Signal that an error occurred with the connection to the
	 *             server
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             Signal that an error occurred performing an operation with
	 *             the electoral locations registry service
	 * @throws DemographyException
	 *             Signal that an error occurred performing an operation with
	 *             the demography service.
	 * @throws ElectoralOrganizationOperationException
	 *             Signal that an error occurred performing an operation with
	 *             the electoral organization operation service
	 */
	public void setSecurityComponents(boolean useProgrammingUnit,
			boolean generateElectoralAuthorities) throws ConnectionException,
			ElectoralPhysicalLocationsRegistryException, DemographyException,
			ElectoralOrganizationOperationException {

		int pageSize = 100;

		int votingDevicesAmount = getVotingDevicesAmount();

		int programmingUnitsAmount = getProgrammingUnitsAmount();

		int electoralAuthoritiesAmount = getElectoralAuthoritiesAmount();

		int numberOfVotingDevicePages = ClientCommon.getNumberOfPages(
				votingDevicesAmount, pageSize);

		int numberOfElectoralAuthoritiesPages = ClientCommon.getNumberOfPages(
				electoralAuthoritiesAmount, pageSize);

		int numberOfProgrammingUnitPages = ClientCommon.getNumberOfPages(
				programmingUnitsAmount, pageSize);

		int elementsProcessed = 0;

		int progressBarSize = numberOfVotingDevicePages;
		if (useProgrammingUnit) {
			progressBarSize += numberOfProgrammingUnitPages;
		}
		if (generateElectoralAuthorities) {
			progressBarSize += numberOfElectoralAuthoritiesPages;
		}
		setMaxProgressBar(progressBarSize);

		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);

		if (useProgrammingUnit) {

			fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);

			for (int i = 0; i < numberOfProgrammingUnitPages; i++) {

				setProtectionKeyToProgrammingUnit(i, pageSize);
				elementsProcessed++;

				setProgressBarValue(elementsProcessed);
				fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
			}
		}

		if (generateElectoralAuthorities) {

			fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);

			for (int i = 0; i < numberOfElectoralAuthoritiesPages; i++) {

				setSecurityComponentsToElectoralAuthorities(i, pageSize);
				elementsProcessed++;

				setProgressBarValue(elementsProcessed);
				fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
			}
		}

		for (int i = 0; i < numberOfVotingDevicePages; i++) {

			setSecurityComponentsToVotingDevices(i, pageSize);
			elementsProcessed++;

			setProgressBarValue(elementsProcessed);
			fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
		}

	}

	/**
	 * Sets the security components to voting devices.
	 * 
	 * @param pageSize
	 *            The page size total
	 * @param pageNumber
	 *            The page number to process.
	 * @throws ConnectionException
	 *             Signal that an error occurred with the connection to the
	 *             server
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             Signal that an error occurred performing an operation with
	 *             the electoral locations registry service
	 * @throws ElectoralOrganizationOperationException
	 *             Signal that an error occurred performing an operation with
	 *             the electoral organization operation service
	 */
	private void setSecurityComponentsToVotingDevices(int pageNumber,
			int pageSize) throws ConnectionException,
			ElectoralPhysicalLocationsRegistryException,
			ElectoralOrganizationOperationException {
		ECMServiceProvider.setSecurityComponentsToVotingDevices(pageNumber,
				pageSize);
	}

	/**
	 * Sets the protection keys to programming unit.
	 * 
	 * @param pageSize
	 *            The page size total
	 * @param pageNumber
	 *            The page number to process.
	 * @throws ConnectionException
	 *             Signal that an error occurred with the connection to the
	 *             server
	 * @throws DemographyException
	 *             Signal that an error occurred performing an operation with
	 *             the demography service.
	 */
	private void setProtectionKeyToProgrammingUnit(int pageNumber, int pageSize)
			throws ConnectionException, DemographyException {
		ECMServiceProvider.setProtectionKeyToProgrammingUnit(pageNumber,
				pageSize);
	}

	/**
	 * Set the security components to electoral authorities.
	 * 
	 * @param pageNumber
	 *            page to process.
	 * @param pageSize
	 *            total page size.
	 * @throws ConnectionException
	 *             Signal that an error occurred with the connection to the
	 *             server
	 * @throws ElectoralOrganizationOperationException
	 *             Signal that an error occurred performing an operation with
	 *             the electoral organization service.
	 */
	private void setSecurityComponentsToElectoralAuthorities(int pageNumber,
			int pageSize) throws ConnectionException,
			ElectoralOrganizationOperationException {
		ECMServiceProvider.setSecurityComponentsToElectoralAuthorities(
				pageNumber, pageSize);
	}

	/**
	 * Gets the program units amount.
	 * 
	 * @return the program units amount
	 * @throws ConnectionException
	 *             Signal that an error occurred with the connection to the
	 *             server
	 * @throws DemographyException
	 *             Signal that an error occurred with the connection to the
	 *             server
	 */
	private int getProgrammingUnitsAmount() throws ConnectionException,
			DemographyException {
		return ECMServiceProvider.retrieveProgrammingUnitAmount();
	}

	/**
	 * Gets the voting devices amount.
	 * 
	 * @return the voting devices amount
	 * @throws ConnectionException
	 *             Signal that an error occurred with the connection to the
	 *             server
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             Signal that an error occurred performing an operation with
	 *             the electoral locations registry service
	 */
	private int getVotingDevicesAmount() throws ConnectionException,
			ElectoralPhysicalLocationsRegistryException {
		return ECMServiceProvider.retrieveVotingDevicesAmount();
	}

	/**
	 * Gets the electoral authorities amount
	 * 
	 * @return the electoral authorities amount
	 * @throws ConnectionException
	 *             Signal that an error occurred with the connection to the
	 *             server
	 * @throws ElectoralOrganizationOperationException
	 *             Signal that an error occurred performing an operation with
	 *             the electoral organization operation service
	 */
	private int getElectoralAuthoritiesAmount() throws ConnectionException,
			ElectoralOrganizationOperationException {
		return ECMServiceProvider.retrieveElectoralAuthoritiesAmount();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#
	 * performOperation()
	 */
	@Override
	protected void performOperation() {

		fireAwtEvent(AwtOperationType.DISABLE_COMPONENTS);

		try {
			startSecurityComponentsGeneration();
		} catch (ConnectionException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			showErrorMessage(e.getBundleKey());
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} catch (ElectoralPhysicalLocationsRegistryException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			showErrorMessage(e.getBundleKey());
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} catch (DemographyException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			showErrorMessage(e.getBundleKey());
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} catch (ElectoralOrganizationOperationException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			showErrorMessage(e.getBundleKey());
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			setErrorBundleKey("ecm.generic.error");
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} finally {
			setProgressBarValue(0);
			fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
			fireAwtEvent(AwtOperationType.ENABLE_COMPONENTS);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#onCompleted
	 * ()
	 */
	@Override
	protected void onCompleted() {

	}

}
