/**
 * DecryptWorkerTest.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.IOException;
import java.util.Scanner;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PowerMockIgnore;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.DecryptException;
import com.smartmatic.waffle.ecm.client.test.stub.DecryptWorkerStub;
import com.smartmatic.waffle.ecm.client.worker.DecryptWorker.PBEType;
import com.smartmatic.waffle.ecm.core.pendrive.util.PendriveUtil;

/**
 * Class to test DecryptWorker.
 */
@RunWith(PowerMockRunner.class)
@PowerMockIgnore({ "javax.crypto.*" })
@PrepareForTest({ ECMServiceProvider.class, ECMClientSideConstants.class,
		PendriveUtil.class })
public class DecryptWorkerTest {

	private static final String INVALID_SECRET_KEY = "invalidsecretkey";

	private static final String INVALID_UP_KEY = "invalidupkey";

	private static final String EXPECTED_UP_SECRET_KEY = "HTTyAB+zFbbn1g/SSLNhYJTYcaxbEhTGHFo8jhdMZ6o=";

	private static final String EXPECTED_SYMMETRIC_KEY = "3J6wCGqJT6nINLBVcwk/9v6WNFwUcyRQSvljTu215bU=";

	private static final String VOTING_DEVICE_PASSWORD = "swdammjfqvjxbndr";

	private static final String APP_KEY = "+DrhkiXyXx4v5jY3sUhZv9GSBr5xDdU7ZlEVrn/qPEI=";

	/**
	 * Prepares the test.
	 * 
	 * @throws Exception
	 *             signals that an exception has occurred preparing the test.
	 *             This should not throw any exception.
	 */
	@Before
	public void setUp() throws Exception {
		this.resourcesFolder = new File("test", "resources");

		this.programminUnitFolder = new File(this.resourcesFolder,
				"PROGRAMMING_UNIT_NAME");

		this.votingDeviceFolder = new File(this.programminUnitFolder,
				"Saint-Gilles_001_21013");

		prepareFilesToTest();
	}

	/**
	 * Executes removal folders and files created to run the tests.
	 * 
	 * @throws Exception
	 *             signals that an exception has occurred removing file and
	 *             folders. This should not throw any exception.
	 */
	@After
	public void tearDown() throws Exception {
		FileUtils.forceDelete(this.programminUnitFolder);
	}

	/**
	 * Tests validateSelectedFolder method. Case: Selected valid folder.
	 * 
	 * @throws Exception
	 *             This should not throw any exception.
	 */
	@Test
	public void testValidateSelectedValidFolder() throws Exception {
		// arrange
		DecryptWorker decryptWorker = new DecryptWorkerStub(
				this.votingDeviceFolder, null, APP_KEY);

		// act
		boolean isValidFolder = decryptWorker.validateSelectedFolder();

		// assert
		assertTrue(isValidFolder);
	}

	/**
	 * Tests validateSelectedFolder method. Case: Selected invalid folder.
	 * 
	 * @throws Exception
	 *             This test only throws DecryptException.
	 */
	@Test(expected = DecryptException.class)
	public void testValidateSelectedInvalidFolder() throws Exception {

		// arrange
		DecryptWorker decryptWorker = new DecryptWorkerStub(
				this.programminUnitFolder, null, APP_KEY);

		try {
			// act
			decryptWorker.validateSelectedFolder();
		} catch (DecryptException e) {
			// assert
			assertEquals("DecryptException.voting.device.file.not.found",
					e.getBundleKey());
			throw e;
		}
	}

	/**
	 * Tests getDataFromCipheredFile method.
	 * 
	 * @throws Exception
	 *             This should not throw any exception.
	 */
	@Test
	public void testGetDataFromCipheredFile() throws Exception {
		// arrange
		File pbeFile = new File(this.resourcesFolder, "secret-key");
		DecryptWorker decryptWorker = new DecryptWorkerStub(
				this.votingDeviceFolder, VOTING_DEVICE_PASSWORD, APP_KEY);

		PowerMockito.mockStatic(ECMServiceProvider.class);

		PowerMockito.when(
				ECMServiceProvider.getServerPropertyAsString(Mockito
						.anyString())).thenReturn("16");

		// act
		String symmetricKey = decryptWorker.getDataFromCipheredFile(pbeFile);

		// assert
		assertEquals(EXPECTED_SYMMETRIC_KEY, symmetricKey.trim());
	}

	/**
	 * Tests getSpecificSymmetricKey method. Case: Opening the up-key with a
	 * valid passphrase.
	 * 
	 * @throws Exception
	 *             This should not throw any exception.
	 */
	@Test
	public void testGetSpecificSymmetricKeyValidUpKey() throws Exception {
		// arrange
		DecryptWorker decryptWorker = new DecryptWorkerStub(
				this.votingDeviceFolder, VOTING_DEVICE_PASSWORD, APP_KEY);

		// act
		String symmetricKey = decryptWorker.getSpecificSymmetricKey(PBEType.UP);

		// assert
		assertEquals(EXPECTED_UP_SECRET_KEY, symmetricKey.trim());
	}

	/**
	 * Tests getSpecificSymmetricKey method. Case: Opening the up-recount-key
	 * with a valid passphrase.
	 * 
	 * @throws Exception
	 *             This should not throw any exception.
	 */
	@Test
	public void testGetSpecificSymmetricKeyValidUpRecountKey() throws Exception {
		// arrange
		DecryptWorker decryptWorker = new DecryptWorkerStub(
				this.votingDeviceFolder, VOTING_DEVICE_PASSWORD, APP_KEY);

		// act
		String symmetricKey = decryptWorker.getSpecificSymmetricKey(PBEType.UP);

		// assert
		assertEquals(EXPECTED_UP_SECRET_KEY, symmetricKey.trim());
	}

	/**
	 * Tests getSpecificSymmetricKey method. Case: invalid passphrase.
	 * 
	 * @throws Exception
	 *             This should not throw any exception.
	 */
	@Test(expected = DecryptException.class)
	public void testGetSpecificSymmetricKeyInvalidUpKey()
			throws DecryptException {
		// arrange
		DecryptWorker decryptWorker = new DecryptWorkerStub(
				this.votingDeviceFolder, INVALID_UP_KEY, APP_KEY);

		// act
		try {
			decryptWorker.getSpecificSymmetricKey(PBEType.UP);
		} catch (DecryptException e) {
			// assert
			assertEquals("DecryptException.invalid.passphrase",
					e.getBundleKey());
			throw e;
		}

	}

	/**
	 * Tests getSpecificSymmetricKey method. Case: Opening the secret-key with a
	 * valid passphrase.
	 * 
	 * @throws Exception
	 *             This should not throw any exception.
	 */
	@Test
	public void testGetSpecificSymmetricKeyValidSecretKey() throws Exception {
		// arrange
		DecryptWorker decryptWorker = new DecryptWorkerStub(
				this.votingDeviceFolder, VOTING_DEVICE_PASSWORD, APP_KEY);

		// act
		String symmetricKey = decryptWorker
				.getSpecificSymmetricKey(PBEType.SECRET);

		// assert
		assertEquals(EXPECTED_SYMMETRIC_KEY, symmetricKey.trim());
	}

	/**
	 * Tests getSpecificSymmetricKey method. Case: Opening the recount-key with
	 * a valid passphrase.
	 * 
	 * @throws Exception
	 *             This should not throw any exception.
	 */
	@Test
	public void testGetSpecificSymmetricKeyValidRecountKey() throws Exception {
		// arrange
		DecryptWorker decryptWorker = new DecryptWorkerStub(
				this.votingDeviceFolder, VOTING_DEVICE_PASSWORD, APP_KEY);

		// act
		String symmetricKey = decryptWorker
				.getSpecificSymmetricKey(PBEType.SECRET);

		// assert
		assertEquals(EXPECTED_SYMMETRIC_KEY, symmetricKey.trim());
	}

	/**
	 * Tests getSpecificSymmetricKey method. Case: invalid passphrase.
	 * 
	 * @throws Exception
	 *             This test only throws DecryptException.
	 */
	@Test(expected = DecryptException.class)
	public void testGetSpecificSymmetricKeyInvalidSecretKey()
			throws DecryptException {
		// arrange
		DecryptWorker decryptWorker = new DecryptWorkerStub(
				this.votingDeviceFolder, INVALID_SECRET_KEY, APP_KEY);

		// act
		try {
			decryptWorker.getSpecificSymmetricKey(PBEType.SECRET);
		} catch (DecryptException e) {
			// assert
			assertEquals("DecryptException.invalid.passphrase",
					e.getBundleKey());
			throw e;
		}

	}

	/**
	 * Tests decipherFile method.
	 * 
	 * @throws Exception
	 *             This should not throw any exception.
	 */
	@Test
	public void decipherFileTest() throws Exception {
		// arrange
		DecryptWorker decryptWorker = new DecryptWorkerStub(
				this.votingDeviceFolder, VOTING_DEVICE_PASSWORD, APP_KEY);

		File electionFile = new File(this.programminUnitFolder, "election.xml");

		String symmetricKey = decryptWorker.getSpecificSymmetricKey(PBEType.UP);

		// act
		decryptWorker.decipherFile(electionFile, symmetricKey, 0);

		// assert
		assertTrue(new File(this.programminUnitFolder, "election-plain.xml")
				.exists());
		assertTrue(checkValidXMLFile(new File(this.programminUnitFolder,
				"election-plain.xml")));
	}

	/**
	 * Tests decipherFile method.
	 * 
	 * @throws Exception
	 *             This should not throw any exception.
	 */
	@Test
	public void testDecryptFiles() throws Exception {
		PowerMockito.mockStatic(PendriveUtil.class);

		PowerMockito.doNothing().when(PendriveUtil.class, "sync");

		// arrange
		DecryptWorker decryptWorker = new DecryptWorkerStub(
				this.votingDeviceFolder, VOTING_DEVICE_PASSWORD, APP_KEY);

		// act
		decryptWorker.decryptFiles();

		// assert
		assertTrue(new File(this.programminUnitFolder, "election-plain.xml")
				.exists());
		assertTrue(checkValidXMLFile(new File(this.programminUnitFolder,
				"election-plain.xml")));
		assertTrue(new File(this.votingDeviceSaesFolder,
				"voting-devices-plain.xml").exists());
		assertTrue(checkValidXMLFile(new File(this.votingDeviceSaesFolder,
				"voting-devices-plain.xml")));
		assertTrue(new File(this.stringResourcesFolder,
				"election-resources-plain.xml").exists());
		assertTrue(checkValidXMLFile(new File(this.stringResourcesFolder,
				"election-resources-plain.xml")));
	}

	/**
	 * Prepare a set of files and folder to simulate an voting device folder to
	 * decrypt.
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private void prepareFilesToTest() throws IOException {

		this.votingDeviceSaesFolder = new File(this.votingDeviceFolder, "SAES");

		this.stringResourcesFolder = new File(this.votingDeviceSaesFolder,
				"StringResources");

		File electionFile = new File(this.resourcesFolder, "election.xml");

		File votingDeviceFile = new File(this.resourcesFolder,
				"voting-devices.xml");

		File electionResources = new File(this.resourcesFolder,
				"election-resources.xml");

		File recountKeyFile = new File(this.resourcesFolder, "recount-key");

		File secretKeyFile = new File(this.resourcesFolder, "secret-key");

		File upKeyFile = new File(this.resourcesFolder, "up-key");

		File upRecountKeyFile = new File(this.resourcesFolder, "up-recount-key");

		this.stringResourcesFolder.mkdirs();

		FileUtils.copyFile(electionFile, new File(this.programminUnitFolder,
				"election.xml"));

		FileUtils.copyFile(votingDeviceFile, new File(
				this.votingDeviceSaesFolder, "voting-devices.xml"));

		FileUtils.copyFile(electionResources, new File(
				this.stringResourcesFolder, "election-resources.xml"));

		FileUtils.copyFile(recountKeyFile, new File(this.votingDeviceFolder,
				"recount-key"));

		FileUtils.copyFile(secretKeyFile, new File(this.votingDeviceFolder,
				"secret-key"));

		FileUtils.copyFile(upKeyFile, new File(this.votingDeviceFolder,
				"up-key"));

		FileUtils.copyFile(upRecountKeyFile, new File(this.votingDeviceFolder,
				"up-recount-key"));
	}

	/**
	 * Checks if the file contains a valid XML.
	 * 
	 * @param fileToCheck
	 *            the file to check
	 * 
	 * @return true, if is a valid xml
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private boolean checkValidXMLFile(File fileToCheck) throws IOException {

		String fistLine = StringUtils.EMPTY;

		Scanner scanner = new Scanner(fileToCheck);

		try {
			fistLine = scanner.nextLine();
		} finally {
			scanner.close();
		}

		if (StringUtils.contains(fistLine,
				"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>")) {
			return true;
		} else {
			return false;
		}
	}

	/** The resources folder. */
	private File resourcesFolder;

	/** The mock ProgramminUnit folder. */
	private File programminUnitFolder;

	/** The mock voting device folder. */
	private File votingDeviceFolder;

	/** The mock voting device saes folder. */
	private File votingDeviceSaesFolder;

	/** The mock string resources folder. */
	private File stringResourcesFolder;
}
