package com.smartmatic.waffle.ecm.client.worker;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.data.transfer.objects.ContestLanguagesDTO;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;

@RunWith(PowerMockRunner.class)
@PrepareForTest({ ECMServiceProvider.class, ECMClientSideConstants.class,
		ScreenshotsProcessingWorker.class })
public class ScreenshotsProcessingWorkerTest {

	File tmpScreenshotsFolder;

	ScreenshotsProcessingWorker screenshotsProcessingWorker;

	@Before
	public void setUp() {

		this.tmpScreenshotsFolder = new File("test/resources/tmp");

		this.tmpScreenshotsFolder.mkdir();

		String mountedPath = "test/resources";

		List<ContestData> contestList = new ArrayList<ContestData>();

		ContestData contest1 = new ContestData();
		contest1.setCustomCode("EU12025NN");

		ContestData contest2 = new ContestData();
		contest2.setCustomCode("VL31005NN");

		this.screenshotsProcessingWorker = PowerMockito
				.spy(new ScreenshotsProcessingWorker(mountedPath, contestList));

	}

	@Test
	public void testStartProcessing() throws Exception {

		PowerMockito.doNothing().when(this.screenshotsProcessingWorker,
				"prepareFolders");

		PowerMockito.doNothing().when(this.screenshotsProcessingWorker,
				"loadElectionLanguages");

		PowerMockito.doNothing().when(this.screenshotsProcessingWorker,
				"processContests",
				Mockito.anyMapOf(String.class, ContestLanguagesDTO.class));

		this.screenshotsProcessingWorker.startProcessing();

		assertTrue(this.screenshotsProcessingWorker.isProcessingDone());

		PowerMockito.verifyPrivate(this.screenshotsProcessingWorker).invoke(
				"prepareFolders");

		PowerMockito.verifyPrivate(this.screenshotsProcessingWorker).invoke(
				"loadElectionLanguages");

		PowerMockito.verifyPrivate(this.screenshotsProcessingWorker).invoke(
				"processContests",
				Mockito.anyMapOf(String.class, ContestLanguagesDTO.class));
	}

	@Test
	public void testProcessContests() throws Exception {

		Map<String, ContestLanguagesDTO> contestLanguageMap = new HashMap<String, ContestLanguagesDTO>();

		List<LanguageData> languages = new ArrayList<LanguageData>();

		LanguageData language1 = new LanguageData();
		language1.setLanguageRFC("NL");

		LanguageData language2 = new LanguageData();
		language1.setLanguageRFC("DE");

		languages.add(language1);
		languages.add(language2);

		ContestData contest1 = new ContestData();
		contest1.setCustomCode("EU12025NN");

		ContestData contest2 = new ContestData();
		contest2.setCustomCode("VL31005NN");

		ContestLanguagesDTO contestLanguage1 = new ContestLanguagesDTO();

		contestLanguage1.setContestData(contest1);
		contestLanguage1.setLanguageHeaderList(languages);
		contestLanguage1.setLanguageFooterList(languages);

		ContestLanguagesDTO contestLanguage2 = new ContestLanguagesDTO();

		contestLanguage2.setContestData(contest2);
		contestLanguage2.setLanguageHeaderList(languages);
		contestLanguage2.setLanguageFooterList(languages);

		contestLanguageMap.put("EU12025NN", contestLanguage1);
		contestLanguageMap.put("VL31005NN", contestLanguage2);

		PowerMockito.doReturn("screenshots").when(
				this.screenshotsProcessingWorker,
				"getPathInDriveForScreenshots");

		PowerMockito.doReturn("-").when(this.screenshotsProcessingWorker,
				"getScreenshotsReportLanguagesSeparator");

		PowerMockito.doReturn("contest").when(this.screenshotsProcessingWorker,
				"getContestNameTranslated", Mockito.any(ContestData.class));

		PowerMockito.doNothing().when(this.screenshotsProcessingWorker,
				"processContestFolder", Mockito.any(File.class),
				Mockito.any(ContestLanguagesDTO.class));

		PowerMockito.doNothing().when(this.screenshotsProcessingWorker,
				"generateZip");

		this.screenshotsProcessingWorker.processContests(contestLanguageMap);

		// assert
		assertEquals(100,
				this.screenshotsProcessingWorker.getProcessingProgress());

		PowerMockito.verifyPrivate(this.screenshotsProcessingWorker,
				Mockito.times(2))
				.invoke("processContestFolder", Mockito.any(File.class),
						Mockito.any(ContestLanguagesDTO.class));
	}

	@Test
	public void testProcessContestFolder() throws ConnectionException,
			UtilitiesException, Exception {

		File screenshotsImagesFolder = new File("test/resources/screenshots",
				"VL31005NN");

		List<LanguageData> languages = new ArrayList<LanguageData>();

		LanguageData language1 = new LanguageData();
		language1.setLanguageRFC("NL");
		language1.setCountry("BE");

		languages.add(language1);

		ContestData contest = new ContestData();
		contest.setCustomCode("VL31005NN");

		ContestLanguagesDTO contestLanguages = new ContestLanguagesDTO();

		contestLanguages.setContestData(contest);
		contestLanguages.setLanguageFooterList(languages);
		contestLanguages.setLanguageHeaderList(languages);

		PowerMockito.doReturn("-").when(this.screenshotsProcessingWorker,
				"getScreenshotsReportLanguagesSeparator");

		PowerMockito.doReturn("footer text").when(
				this.screenshotsProcessingWorker, "getFooterText",
				Mockito.any(DistrictData.class),
				Mockito.any(LanguageData.class));

		PowerMockito.doNothing().when(this.screenshotsProcessingWorker,
				"processContestFiles", Mockito.any(File[].class),
				Mockito.anyString(), Mockito.anyString());

		File[] files = new File[0];

		PowerMockito.doReturn(files).when(this.screenshotsProcessingWorker,
				"getSelectionPNGFiles", Mockito.any(File.class),
				Mockito.anyString());

		this.screenshotsProcessingWorker.processContestFolder(
				screenshotsImagesFolder, contestLanguages);

		PowerMockito.verifyPrivate(this.screenshotsProcessingWorker).invoke(
				"processContestFiles", Mockito.any(File[].class),
				Mockito.anyString(), Mockito.anyString());
	}

	@Ignore
	@Test
	public void testProcessContestFiles() throws Exception {

		File contestFolder = new File("test/resources/screenshots/VL31005NN");

		File[] contestFiles = contestFolder.listFiles();

		PowerMockito.doReturn(this.tmpScreenshotsFolder).when(
				this.screenshotsProcessingWorker, "getTmpScreenshotsFolder");

		PowerMockito.doReturn(false).when(this.screenshotsProcessingWorker,
				"signScreenshotsPdf");

		this.screenshotsProcessingWorker.processContestFiles(contestFiles,
				"VL31005NN.pdf", "footer text");

		File generatedPdf = new File(this.tmpScreenshotsFolder, "VL31005NN.pdf");

		assertTrue(generatedPdf.exists());

		PowerMockito.verifyPrivate(this.screenshotsProcessingWorker).invoke(
				"buildPdf", Mockito.anyListOf(File.class), Mockito.anyString());
	}

	@After
	public void clean() {

		FileUtils.deleteQuietly(this.tmpScreenshotsFolder);
	}
}
