/**
 * ConnectivityManager.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004, 2013 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.monitoring;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.nio.channels.SocketChannel;
import java.util.ArrayList;

import com.smartmatic.waffle.ecm.core.monitoring.spec.IConnectionCheckListener;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to perform a check of the connection to the server.
 * 
 * @version 1.0
 * @author Hector Rodriguez <hector.rodriguez@smartmatic.com>
 */
public class ConnectivityManager {

	/**
	 * Dleep time in thread execute (1 seg).
	 */
	private static final Integer DELAY_1000 = 1000;

	/**
	 * Field used for logging purpose.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ConnectivityManager.class);

	/** The server name / IP. */
	private String server;

	/**
	 * The port number.
	 */
	private Integer port;

	/**
	 * The singleton instance.
	 */
	private static ConnectivityManager instance;

	/**
	 * Enum to indicate the conection status. author Hector Rodriguez
	 * <hector.rodriguez@smartmatic.com>
	 * 
	 */
	public enum ConnectionState {

		/** The init. */
		INIT,
		/** The connected. */
		CONNECTED,
		/** The disconnected. */
		DISCONNECTED,
		/** The stopped. */
		STOPPED;
	}

	/**
	 * The current state of the connection.
	 */
	private ConnectionState state = ConnectionState.STOPPED;

	/** The event thread. */
	private Thread eventThread;

	/** The listener list to notify about status change. */
	private ArrayList<IConnectionCheckListener> stateListeners = new ArrayList<IConnectionCheckListener>();

	/** The ip adress. */
	private InetAddress ipAdress;

	/** The socket address. */
	private InetSocketAddress socketAddress;

	/**
	 * Class constructor.
	 * 
	 * @param host
	 *            the server name / IP
	 * @param p
	 *            the port number.
	 */
	private ConnectivityManager(String host, Integer p) {
		this.server = host;
		this.port = p;
	}

	/**
	 * Method used to create the singleton instance.
	 * 
	 * @param server
	 *            the server name / IP
	 * @param port
	 *            the port number
	 */
	public static void init(String server, Integer port) {
		instance = new ConnectivityManager(server, port);
	}

	/**
	 * The Class ConnectionTask.
	 */
	class ConnectionTask implements Runnable {

		/***
		 * {@inheritDoc}<br>
		 * Method used to perform a check of the connection to the server.
		 */
		@Override
		public void run() {
			SocketChannel socketChannel = null;

			while (true) {
				try {
					try {
						socketChannel = SocketChannel.open();
						socketChannel.configureBlocking(false);
						socketChannel.connect(instance.socketAddress);
						Thread.sleep(1000);
						socketChannel.finishConnect();
						Thread.sleep(1000);
						if (socketChannel.isConnected()) {
							if (instance.state != ConnectionState.CONNECTED) {
								boolean notifyInitialHostConnectedEvent = (instance.state != ConnectionState.STOPPED);
								instance.state = ConnectionState.CONNECTED;
								if (notifyInitialHostConnectedEvent) {
									instance.fireConnectedHostEvent();
								}
							}
						} else {
							if (instance.state != ConnectionState.DISCONNECTED) {
								instance.state = ConnectionState.DISCONNECTED;
								instance.fireDisconnectedHostEvent();
							}
						}
					} catch (Exception e) {
						if (instance.state != ConnectionState.DISCONNECTED) {
							instance.state = ConnectionState.DISCONNECTED;
							instance.fireDisconnectedHostEvent();
						}
					} finally {
						if (socketChannel != null) {
							try {
								socketChannel.close();
							} catch (IOException e) {
								logger.info(e,
										"There is a problem closing the socket");
							}
							socketChannel = null;
						}
					}
					Thread.sleep(DELAY_1000);
				} catch (InterruptedException e) {
					logger.error(
							"There was an error trying to sleep connectivity manager thread ",
							e);
				}
			}
		}
	}

	/**
	 * Getter of {@link #state} field.
	 * 
	 * @return {@link #state} value.
	 */
	public ConnectionState getState() {
		return this.state;
	}

	/**
	 * Method used to get the singleton instance.
	 * 
	 * @return the singleton instance.
	 */
	public static ConnectivityManager getInstance() {
		if (instance != null) {
			return instance;
		}
		throw new IllegalStateException(
				"ConnectivityManager cannot been initialized");
	}

	/**
	 * Method used to start the monitoring connectivity.
	 */
	public void start() {
		logger.info("Checking connection with server [" + this.server
				+ "] on port [" + this.port + "]");

		try {
			instance.ipAdress = InetAddress.getByName(instance.server);
			instance.socketAddress = new InetSocketAddress(instance.ipAdress,
					instance.port);
		} catch (UnknownHostException e) {
			logger.error(e, "There was an error init connectivity test");
			return;
		}

		this.eventThread = new Thread(new ConnectionTask(),
				"CONNECTIVITY_MANAGER_THREAD");
		this.eventThread.start();
	}

	/**
	 * Method used to notify to all listeners about the event host connected.
	 */
	private void fireConnectedHostEvent() {
		synchronized (this.stateListeners) {
			for (IConnectionCheckListener listener : this.stateListeners) {
				try {
					logger.info("Host " + this.server + ":" + this.port
							+ " connected");
					listener.hostConnected();
				} catch (Exception e) {
					logger.error(
							"There was an error when throw the host is connected [cause]: "
									+ e.getMessage(), e);
				}
			}
		}
	}

	/**
	 * Method used to notify to all listeners about the event host disconnected.
	 */
	private void fireDisconnectedHostEvent() {
		synchronized (this.stateListeners) {
			for (IConnectionCheckListener listener : this.stateListeners) {
				try {
					logger.info("Host " + this.server + ":" + this.port
							+ " disconnected");
					listener.hostDisconnected();
				} catch (Exception e) {
					logger.error(
							"There was an error when throw the host is disconnected [cause]: "
									+ e.getMessage(), e);
				}
			}
		}
	}

	/**
	 * Method used to add listeners to manager of events, to further
	 * notification.
	 * 
	 * @param e
	 *            listener to add
	 */
	public void addConnectionListener(IConnectionCheckListener e) {
		synchronized (this.stateListeners) {
			if (!this.containsConnectionListener(e)) {
				this.stateListeners.add(e);
			}
		}
	}

	/**
	 * Method used to remove listeners to manager of events, to further
	 * notification.
	 * 
	 * @param client
	 *            listener to remove.
	 */
	public void removeConnectionListener(IConnectionCheckListener client) {
		synchronized (this.stateListeners) {
			if (this.containsConnectionListener(client)) {
				this.stateListeners.remove(client);
			}
		}
	}

	/**
	 * Contains connection listener.
	 * 
	 * @param client
	 *            the client
	 * @return true, if successful
	 */
	public boolean containsConnectionListener(IConnectionCheckListener client) {
		synchronized (this.stateListeners) {
			return this.stateListeners.contains(client);
		}
	}
}
