/**
 * PrinterAssistant.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.printing;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.print.PrintException;
import javax.print.attribute.Attribute;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRPrintPage;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.util.JRLoader;
import net.sf.jasperreports.engine.util.JRSaver;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.io.IOUtils;
import org.cups4j.CupsClient;
import org.cups4j.CupsPrinter;
import org.cups4j.PrintJob;

import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Utility class used to excute multiple tast repated with printing.
 */
public class PrinterAssistant {

	/** For logging purposes. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(PrinterAssistant.class);

	/** The Constant compiledReports. */
	private static Map<String, JasperReport> compiledReports = new HashMap<String, JasperReport>();

	/**
	 * Merge jaspers.
	 * 
	 * @param jasperPrints
	 *            the jasper prints
	 * @return the jasper print
	 */
	public static JasperPrint mergeJaspers(Collection<JasperPrint> jasperPrints) {
		if (jasperPrints == null || jasperPrints.isEmpty()) {
			return null;
		}
		Iterator<JasperPrint> iter = jasperPrints.iterator();

		JasperPrint firstJasperPrint = iter.next();
		JasperPrint otherJasper;
		while (iter.hasNext()) {
			otherJasper = iter.next();
			List<JRPrintPage> pages = otherJasper.getPages();
			for (JRPrintPage page : pages) {
				firstJasperPrint.addPage(page);
			}
		}
		return firstJasperPrint;

	}

	/**
	 * Checks the integrity of the compiled file.
	 * 
	 * @param theFolderName
	 *            the container folder
	 * @param fileName
	 *            the file to check
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws JRException
	 *             the jR exception
	 */
	public static void checkCompiledIntegrity(File theFolderName,
			String fileName) throws IOException, JRException {
		File xml = new File(theFolderName, fileName + ".jrxml");

		File jasper = new File(theFolderName, fileName + ".jasper");
		if (!jasper.exists() || xml.lastModified() > jasper.lastModified()) {
			compileAll(theFolderName);
		}
	}

	/**
	 * Load and compile template.
	 * 
	 * @param folderName
	 *            the folder name
	 * @param templateName
	 *            the template name
	 * @return the jasper report
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws JRException
	 *             the jR exception
	 */
	public static JasperReport loadAndCompileTemplate(File folderName,
			String templateName) throws IOException, JRException {

		validateCompiledReport(folderName, templateName);

		JasperReport compiledTemplate = compiledReports.get(templateName);

		if (compiledTemplate == null) {

			checkCompiledIntegrity(folderName, templateName);

			File jasper = new File(folderName, templateName + ".jasper");

			compiledTemplate = (JasperReport) JRLoader.loadObject(jasper);

			compiledReports.put(templateName, compiledTemplate);
		}

		return compiledTemplate;
	}

	/**
	 * Check if the respective jasper file exist in the file system.
	 * 
	 * @param folderName
	 * @param templateName
	 */
	private static void validateCompiledReport(File folderName,
			String templateName) {
		File compiledFile = new File(folderName, templateName + ".jasper");

		if (!compiledFile.exists()) {
			compiledReports.clear();
		}
	}

	/**
	 * Fill template.
	 * 
	 * @param folderName
	 *            the folder name
	 * @param templateName
	 *            the template name
	 * @param parameters
	 *            the parameters
	 * @param attributesParameters
	 *            the attributes parameters
	 * @param reportParameters
	 *            the report parameters
	 * @return the jasper print
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws JRException
	 *             the jR exception
	 */
	public static JasperPrint fillTemplate(File folderName,
			String templateName, Collection parameters,
			List<? extends Attribute> attributesParameters,
			Map<String, Object> reportParameters) throws IOException,
			JRException {

		JasperReport compiledTemplate = loadAndCompileTemplate(folderName,
				templateName);

		return JasperFillManager.fillReport(compiledTemplate, reportParameters,
				new JRBeanCollectionDataSource(parameters));
	}

	/**
	 * Generate preview.
	 * 
	 * @param folderName
	 *            the folder name
	 * @param templateName
	 *            the template name
	 * @param parameters
	 *            the parameters
	 * @param reportParameters
	 *            the report parameters
	 * @return the jasper print
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws JRException
	 *             the jR exception
	 * @throws PrintException
	 *             the print exception
	 */
	public static JasperPrint generatePreview(File folderName,
			String templateName, Collection parameters,
			Map<String, Object> reportParameters) throws IOException,
			JRException, PrintException {

		return fillTemplate(folderName, templateName, parameters, null,
				reportParameters);
	}

	/**
	 * Compile all.
	 * 
	 * @param folderName
	 *            the folder name
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws JRException
	 *             the jR exception
	 */
	public static void compileAll(File folderName) throws IOException,
			JRException {

		// getting all xml files
		String[] fileNames = folderName.list(new FilenameFilter() {

			@Override
			public boolean accept(File theDir, String theName) {
				if (theName.endsWith(".jrxml")) {
					return true;
				}
				return false;
			}
		});
		for (String fileName : fileNames) {
			File xml = new File(folderName, fileName);
			String[] name = fileName.split("\\.");
			InputStream is = new FileInputStream(xml);

			try {

				JasperDesign template = JRXmlLoader.load(is);

				// Generating the jasperReport
				JasperReport compiledTemplate = JasperCompileManager
						.compileReport(template);
				JRSaver.saveObject(compiledTemplate, folderName + "/" + name[0]
						+ ".jasper");
			} finally {
				IOUtils.closeQuietly(is);

			}
		}
	}

	/**
	 * Prints the text.
	 * 
	 * @param document
	 *            the text
	 * @param priority
	 *            the priority
	 * @throws Exception
	 * @throws PrintException
	 *             the print exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static void sendDocumentToPrint(byte[] document) throws Exception {
		CupsClient cc = new CupsClient();
		CupsPrinter cp = cc.getPrinters().iterator().next();
		org.cups4j.PrintJob pj = new PrintJob.Builder(document).build();
		cp.print(pj);
	}

	/**
	 * Cancel all the print jobs in the queue.
	 * 
	 * @throws IOException
	 * @throws InterruptedException
	 */
	public static void cancelPrintDocuments() {
		ProcessBuilder builder = new ProcessBuilder();
		Process process = null;

		builder.command("sh", "-c", "cancel -a");

		try {
			process = builder.start();
			process.waitFor();
		} catch (IOException e) {
			logger.info(String
					.format("An error occurred executing the command to cancel all print jobs"));
		} catch (InterruptedException e) {
			logger.info(String
					.format("An error occurred cancelling all print jobs"));
		} finally {
			if (process != null) {
				process.destroy();
			}
		}
	}
}
