/**
 * ECMDemographyServiceImpl.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.demographics.PathData;
import com.smartmatic.waffle.domain.demographics.ProgrammingUnitData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.demographics.RegionViewData;
import com.smartmatic.waffle.ecm.data.transfer.objects.ProgrammingUnitLastLevelRegionDTO;
import com.smartmatic.waffle.ecm.service.lang.DemographyException;
import com.smartmatic.waffle.ecm.service.lang.DemographyException.DemographyExceptionInfo;
import com.smartmatic.waffle.ecm.service.lang.XmlGenerationException;
import com.smartmatic.waffle.ecm.service.lang.XmlGenerationException.ExceptionInfo;
import com.smartmatic.waffle.ecm.service.spec.IECMDemographyService;
import com.smartmatic.waffle.ecm.service.spec.IECMDemographyServiceLocal;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.search.criteria.CriteriaFactory;
import com.smartmatic.waffle.persistence.search.criteria.OrderCriteria;
import com.smartmatic.waffle.persistence.search.transformation.TransformationFactory;
import com.smartmatic.waffle.persistence.search.transformation.TransformationsList;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;

/**
 * Session beans that handles all server side operations related with
 * Demography. This class implements all the server services defined in
 * <code>IECMDemographyService</code>
 */
@Stateless(name = IECMDemographyService.SERVICE_NAME)
@Remote(IECMDemographyService.class)
@Local(IECMDemographyServiceLocal.class)
public class ECMDemographyServiceImpl implements IECMDemographyServiceLocal {

	/** Object to generate logs during the application execution. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ECMDemographyServiceImpl.class);

	/**
	 * PersistenceServiceLocal Instance to do any database persistence
	 * operation.
	 */
	@EJB
	private IPersistenceServiceLocal persistenceService;

	/**
	 * Sets the persistenceServiceLocal Instance to do any database persistence
	 * operation.
	 * 
	 * @param persistenceService
	 *            the new persistenceServiceLocal Instance to do any database
	 *            persistence operation
	 */
	public void setPersistenceService(
			IPersistenceServiceLocal persistenceService) {
		this.persistenceService = persistenceService;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public int retrieveProgrammingUnitsAmount() throws DemographyException {

		try {
			SearchExpression search = new SearchExpression(
					ProgrammingUnitData.class);

			return this.persistenceService.countObjects(search, null);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new DemographyException(
					e,
					DemographyExceptionInfo.RETRIEVING_PROGRAMMING_UNIT_AMOUNT_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("unchecked")
	public void setProtectionKeyToProgrammingUnit(int pageNumber, int pageSize)
			throws DemographyException {

		try {
			SearchExpression search = new SearchExpression(
					ProgrammingUnitData.class);

			search.addOrder(OrderCriteria.asc("customCode"));

			Collection<ProgrammingUnitData> programmingUnitDataList;

			programmingUnitDataList = this.persistenceService
					.searchPagedObjects(search, null, pageNumber, pageSize);

			for (Iterator<ProgrammingUnitData> iterator = programmingUnitDataList
					.iterator(); iterator.hasNext();) {
				ProgrammingUnitData programmingUnitData = iterator.next();

				programmingUnitData.setProtectionKey(ECMServerCommonImpl
						.generateSymmetricKey());

				this.persistenceService.update(programmingUnitData, null);
			}
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new DemographyException(
					e,
					DemographyExceptionInfo.SETTING_PROTECTION_KEY_TO_PROGRAMMING_UNIT_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public int deleteProgrammingUnitSecurityComponents()
			throws DemographyException {
		try {
			return this.persistenceService
					.executeUpdateNativeQuery("update programming_unit set protection_key = null;");
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new DemographyException(
					e,
					DemographyExceptionInfo.DELETING_PROGRAMMING_UNIT_SECURITY_COMPONENTS_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("unchecked")
	public List<RegionData> getRegionsWithParentsByRegionCodes(
			List<Long> regionCodes) throws DemographyException {

		try {
			SearchExpression searchRegion = new SearchExpression(
					RegionData.class);

			searchRegion.createAlias("allParents", "allParents");

			searchRegion.addSearchCriteria(CriteriaFactory.in("code",
					regionCodes));

			String[] path = { "allParents.districts",
					"allParents.electionsLanguage" };

			List<RegionData> regionDataList = (List<RegionData>) this.persistenceService
					.searchObjects(searchRegion, path, null);

			return regionDataList;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new DemographyException(
					e,
					DemographyExceptionInfo.PROBLEM_RETRIEVING_LAST_LEVEL_REGION_BY_REGION_CODES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<RegionData> getLastLevelRegions(
			List<RegionData> regionsFilterList) throws DemographyException {

		try {

			SearchExpression search = new SearchExpression(RegionData.class);

			search.createAlias("category", "category");

			search.createAlias("allParents", "allParents");

			List<Long> regionCodes = new ArrayList<Long>();

			if (regionsFilterList != null) {
				for (RegionData indexRegion : regionsFilterList) {
					regionCodes.add(indexRegion.getCode());
				}

				if (regionCodes.size() > 0) {
					search.addSearchCriteria(CriteriaFactory.in(
							"allParents.code", regionCodes));
				}
			}

			search.addSearchCriteria(CriteriaFactory.eq(
					"category.lastLevelCategory", true));

			List<RegionData> regionDataList = (List<RegionData>) this.persistenceService
					.searchObjects(search, null);

			return regionDataList;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new DemographyException(
					e,
					DemographyExceptionInfo.PROBLEM_RETRIEVING_LAST_LEVEL_REGION_BY_REGION_CODES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<RegionData> getLastLevelRegionsLight(
			List<RegionData> regionsFilterList) throws DemographyException {

		try {

			List<RegionData> lastLevelRegionsList = getLastLevelRegions(regionsFilterList);

			if (lastLevelRegionsList == null) {
				throw new DemographyException(
						DemographyExceptionInfo.NO_REGIONS_WERE_FOUND_FAILURE);
			}

			SearchExpression search = new SearchExpression(RegionData.class);

			search.createAlias("category", "category");

			search.createAlias("allParents", "allParents");

			search.createAlias("parentDistricts", "allParents.districts");

			search.createAlias("parentContests", "parentDistricts.contests");

			search.createAlias("pollingPlaces", "pollingPlaces");

			search.createAlias("pollingStations",
					"pollingPlaces.pollingStations");

			search.createAlias("votingBooths", "pollingStations.votingBooths");

			search.addSearchCriteria(CriteriaFactory
					.isNotNull("votingBooths.votingDevice"));

			search.addSearchCriteria(CriteriaFactory.eq(
					"category.lastLevelCategory", true));

			search.addSearchCriteria(CriteriaFactory.collectionSizeNotEq(
					"parentContests.contestGroupList", 0));

			search.addSearchCriteria(CriteriaFactory.collectionSizeNotEq(
					"parentContests.eligibleOptions", 0));

			List<Long> regionCodes = new ArrayList<Long>();

			for (RegionData region : lastLevelRegionsList) {
				regionCodes.add(region.getCode());
			}

			if (regionCodes.size() > 0) {
				search.addSearchCriteria(CriteriaFactory
						.in("code", regionCodes));
			}

			List<RegionData> regionDataList = new ArrayList<RegionData>();

			TransformationsList lightTransformation = TransformationFactory
					.transformationList();

			lightTransformation.add(TransformationFactory
					.distinctTransformation(TransformationFactory
							.propertyTransformation("code")));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("regionName"));

			search.setTransformation(lightTransformation);

			Collection<Object[]> regionObjects = this.persistenceService
					.searchObjects(search, null);

			for (Object[] object : regionObjects) {
				long code = (Long) object[0];
				String regionName = (String) object[1];

				RegionData region = new RegionData();
				region.setCode(code);
				region.setRegionName(regionName);

				regionDataList.add(region);
			}

			if (regionDataList.size() == 0) {
				throw new DemographyException(
						DemographyExceptionInfo.NO_REGIONS_WERE_FOUND_FAILURE);
			}

			return regionDataList;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new DemographyException(
					e,
					DemographyExceptionInfo.PROBLEM_RETRIEVING_LAST_LEVEL_REGION_BY_REGION_CODES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public Set<ProgrammingUnitLastLevelRegionDTO> getProgrammingUnitsByRegions(
			List<RegionData> regionList) throws DemographyException {

		try {
			SearchExpression searchRegion = new SearchExpression(
					ProgrammingUnitData.class);

			searchRegion.createAlias("regions", "regions");

			searchRegion.createAlias("allParents", "regions.allParents");

			searchRegion.createAlias("allChildren", "regions.allChildren");

			List<Long> regionCodes = new ArrayList<Long>();
			for (RegionData region : regionList) {
				regionCodes.add(region.getCode());
			}

			searchRegion.addSearchCriteria(CriteriaFactory.or(
					CriteriaFactory.in("allParents.code", regionCodes),
					CriteriaFactory.in("allChildren.code", regionCodes)));

			Set<ProgrammingUnitLastLevelRegionDTO> programmingUnitsSet = new HashSet<ProgrammingUnitLastLevelRegionDTO>();

			List<ProgrammingUnitData> programmingUnitList = (List<ProgrammingUnitData>) this.persistenceService
					.searchObjects(searchRegion, null);

			for (ProgrammingUnitData programmingUnitData : programmingUnitList) {

				List<RegionData> lastLevelRegions = null;

				try {
					lastLevelRegions = getLastLevelRegionsLight(regionList);
				} catch (DemographyException e) {
					continue;
				}

				ProgrammingUnitLastLevelRegionDTO programminUnitLastLevelRegionDTO = new ProgrammingUnitLastLevelRegionDTO();

				programminUnitLastLevelRegionDTO
						.setProgrammingUnit(programmingUnitData);

				programminUnitLastLevelRegionDTO
						.setLastLevelRegionForProgrammingUnit(lastLevelRegions);

				programmingUnitsSet.add(programminUnitLastLevelRegionDTO);
			}

			return programmingUnitsSet;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new DemographyException(
					e,
					DemographyExceptionInfo.PROBLEM_GETTING_PROGRAMMING_UNITS_BY_REGIONS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	public RegionData getAllParentsRegions(RegionData region)
			throws DemographyException {

		SearchExpression searchRegion = new SearchExpression(RegionData.class);

		searchRegion.createAlias("allParents", "allParents");

		searchRegion.createAlias("districts", "allParents.districts");

		searchRegion.createAlias("category", "category");

		searchRegion.addSearchCriteria(CriteriaFactory.eq("code",
				region.getCode()));

		try {
			String[] path = { "allParents.districts",
					"allParents.electionsLanguage", "category.paths" };

			RegionData regionWithParents = (RegionData) this.persistenceService
					.searchObject(searchRegion, path, null);

			return regionWithParents;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new DemographyException(
					e,
					DemographyExceptionInfo.PROBLEM_GETTING_ALL_PARENTS_REGIONS,
					StringUtils.defaultString(e.getMessage()));
		}

	}

	@Override
	public RegionData checkRegionParentPath(RegionData region, PathData path)
			throws DemographyException {

		SearchExpression search = new SearchExpression(RegionData.class);

		search.createAlias("children", "children").addSearchCriteria(
				CriteriaFactory.eq("code", region.getCode()));

		search.createAlias("category", "category")
				.createAlias("paths", "paths")
				.addSearchCriteria(CriteriaFactory.eq("code", path.getCode()));

		try {
			String paths[] = { "category.paths" };

			return (RegionData) this.persistenceService.searchObject(search,
					paths, null);

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new DemographyException(
					e,
					DemographyExceptionInfo.PROBLEM_CHECKING_REGION_PARENT_PATH,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<RegionData> getAvailableRegionsForWidget()
			throws DemographyException {

		try {

			SearchExpression search = new SearchExpression(RegionViewData.class);

			List<RegionViewData> regionsList = (List<RegionViewData>) this.persistenceService
					.searchObjects(search, null);

			Set<RegionData> regionsSet = new HashSet<RegionData>();

			for (RegionViewData current : regionsList) {

				RegionData regionData = new RegionData();
				regionData.setCode(current.getCode());
				regionsSet.add(regionData);
			}

			return new ArrayList<RegionData>(regionsSet);

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new DemographyException(
					e,
					DemographyExceptionInfo.PROBLEM_GETTING_AVAILABLE_REGIONS_FOR_WIDGET,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	public RegionData getAllRegionParentsWithDistricts(RegionData region)
			throws XmlGenerationException {

		SearchExpression searchRegion = new SearchExpression(RegionData.class);

		searchRegion.createAlias("allParents", "allParents");

		searchRegion.createAlias("districts", "allParents.districts");

		searchRegion.addSearchCriteria(CriteriaFactory.eq("code",
				region.getCode()));

		try {
			String[] path = { "allParents.districts" };

			RegionData regionWithParent = (RegionData) this.persistenceService
					.searchObject(searchRegion, path, null);

			return regionWithParent;

		} catch (BaseException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_GETTING_REGION_ALL_PARENT, e);
		} catch (RuntimeException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_GETTING_REGION_ALL_PARENT, e);
		}
	}
}
