/**
 * ECMServiceImpl.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004, 2014 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.impl;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.sql.DataSource;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.contest.ContestClassData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.CategoryData;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.PollingPlaceData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.DeviceParameterData;
import com.smartmatic.waffle.domain.device.LogicalVotingGroupData;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.general.ParameterData;
import com.smartmatic.waffle.domain.general.ReportTemplateData;
import com.smartmatic.waffle.domain.i18n.ContestLanguagesPairData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.multiLanguage.TranslationKeyData;
import com.smartmatic.waffle.ecm.bulk.service.spec.IBulkServiceLocal;
import com.smartmatic.waffle.ecm.core.util.BackupDatabaseUtil;
import com.smartmatic.waffle.ecm.core.util.InetUtility;
import com.smartmatic.waffle.ecm.core.util.SAESCommandUtil;
import com.smartmatic.waffle.ecm.service.lang.BackupDatabaseGenerationExeption;
import com.smartmatic.waffle.ecm.service.lang.ElectoralDemographyException;
import com.smartmatic.waffle.ecm.service.lang.ReportGenerationException.ReportGenerationExceptionInfo;
import com.smartmatic.waffle.ecm.service.lang.XmlGenerationException;
import com.smartmatic.waffle.ecm.service.lang.XmlGenerationException.ExceptionInfo;
import com.smartmatic.waffle.ecm.service.spec.IECMDemographyServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralDemographyServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralOrganizationServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralPhysicalLocationsRegistryService;
import com.smartmatic.waffle.ecm.service.spec.IECMMasterUSBGenerationServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMService;
import com.smartmatic.waffle.ecm.service.spec.IECMServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMUtilitiesServiceLocal;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.search.criteria.CriteriaFactory;
import com.smartmatic.waffle.persistence.search.criteria.MatchCriteriaMode;
import com.smartmatic.waffle.persistence.search.criteria.OrderCriteria;
import com.smartmatic.waffle.persistence.search.transformation.TransformationFactory;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;
import com.smartmatic.waffle.properties.ApplicationProperties;

/**
 * Session beans that handles all server side operations that ECM execute. Class
 * that implements all the server services defined in <code>IECMService</code>
 */
@Stateless(name = IECMService.SERVICE_NAME)
@Remote(IECMService.class)
@Local(IECMServiceLocal.class)
public class ECMServiceImpl implements IECMServiceLocal {

	/**
	 * Object to generate logs during the application execution
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ECMServiceImpl.class);

	/**
	 * ECM Data source
	 */
	@Resource(mappedName = "java:ECM_DS")
	private DataSource ds;

	/**
	 * Instance of IPersistenceServiceLocal
	 */
	@EJB
	private IPersistenceServiceLocal persistenceService;

	/**
	 * Instance of IBulkServiceLocal
	 */
	@EJB
	private IBulkServiceLocal bulkService;

	/**
	 * Instance of IECMMasterUSBGenerationServiceLocal
	 */
	@EJB
	private IECMMasterUSBGenerationServiceLocal masterUSBGenerationService;

	/**
	 * Instance of IECMMasterUSBGenerationServiceLocal
	 */
	@EJB
	private IECMElectoralOrganizationServiceLocal electoralOrganizationService;

	/**
	 * Instance of IECMElectoralDemographyServiceLocal
	 */
	@EJB
	private IECMElectoralDemographyServiceLocal electoralDemographyService;

	/**
	 * Instance of IECMElectoralPhysicalLocationsRegistryService
	 */
	@EJB
	private IECMElectoralPhysicalLocationsRegistryService electoralPhysicalLocationsRegistryService;
	/**
	 * Instance of IECMDemographyServiceLocal
	 */
	@EJB
	private IECMDemographyServiceLocal demographyService;

	/** Instance of IECMUtilitiesServiceLocal. */
	@EJB
	private IECMUtilitiesServiceLocal utilitiesService;

	/**
	 * Default constructor.
	 */
	public ECMServiceImpl() {

	}

	/**
	 * Method to get an official languages list
	 * 
	 * @return List of official languages
	 */
	private Map<String, LanguageData> getOfficalLanguages()
			throws BaseException {

		SearchExpression search = new SearchExpression(LanguageData.class);

		search.addSearchCriteria(CriteriaFactory.like("message", "Official",
				MatchCriteriaMode.START));

		List<LanguageData> languageByRegion = (List<LanguageData>) this.persistenceService
				.searchObjects(search, null);

		Map<String, LanguageData> officalLanguagesMap = new HashMap<String, LanguageData>();

		for (Iterator<LanguageData> iterator = languageByRegion.iterator(); iterator
				.hasNext();) {
			LanguageData languageData = iterator.next();

			officalLanguagesMap
					.put(languageData.getLanguageRFC(), languageData);
		}

		return officalLanguagesMap;
	}

	private List<RegionData> getChildrenRegionsByParent(RegionData parent,
			List<Long> codesOfRegionsAvailables) throws XmlGenerationException {

		if (parent == null) {
			return null;
		}
		SearchExpression search = new SearchExpression(RegionData.class);

		search.createAlias("parent", "parent");

		search.addSearchCriteria(CriteriaFactory.eq("parent.code",
				parent.getCode()));

		if ((codesOfRegionsAvailables != null)
				&& (!codesOfRegionsAvailables.isEmpty())) {
			search.addSearchCriteria(CriteriaFactory.in("code",
					codesOfRegionsAvailables));
		}

		try {
			List<RegionData> regionDataList = (List<RegionData>) this.persistenceService
					.searchObjects(search, null);

			return regionDataList;

		} catch (BaseException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_GETTING_REGION_ALL_PARENT, e);
		} catch (RuntimeException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_GETTING_REGION_ALL_PARENT, e);
		}
	}

	/***
	 * 
	 * @param parent
	 * @param customCodesOfRegionsAvailables
	 * @return
	 * @throws XmlGenerationException
	 */
	@Override
	@TransactionAttribute(TransactionAttributeType.NOT_SUPPORTED)
	public List<RegionData> getChildrenRegions(RegionData parent,
			List<Long> codesOfRegionsAvailables, LanguageData language)
			throws XmlGenerationException {

		List<RegionData> regionsByLevel = getChildrenRegionsByParent(parent,
				codesOfRegionsAvailables);

		List<RegionData> regionListTranslated = new ArrayList<RegionData>();

		for (RegionData indexRegion : regionsByLevel) {
			RegionData regionTranslated = translateRegionsName(indexRegion,
					language);

			regionListTranslated.add(regionTranslated);

		}
		return regionListTranslated;
	}

	@Override
	@TransactionAttribute(TransactionAttributeType.NOT_SUPPORTED)
	public List<RegionData> getRegionsByCategoryLevel(
			CategoryData.Level categoryLevel,
			List<Long> codesOfRegionsAvailables, LanguageData language)
			throws XmlGenerationException {

		List<RegionData> regionsByLevel = getRegionsByLevel(categoryLevel,
				codesOfRegionsAvailables);
		List<RegionData> regionListTranslated = new ArrayList<RegionData>();

		for (RegionData indexRegion : regionsByLevel) {
			RegionData regionTranslated = translateRegionsName(indexRegion,
					language);

			regionListTranslated.add(regionTranslated);

		}
		return regionListTranslated;
	}

	@Override
	public void setServerDateTime(Date datePicked, String scriptPath,
			String machineDate) throws BaseException {
		Process process = null;
		try {
			process = SAESCommandUtil.execute("sudo",
					new String[] { scriptPath + " " + machineDate + " \""
							+ datePicked.toString() + "\"" });

			String aux = IOUtils.toString(process.getInputStream());

			if (aux != null && !aux.isEmpty()) {
				logger.info(aux.trim());
			}

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public String getServerDateTime(String scriptPath, String machineDateTime)
			throws BaseException {
		Process process = null;
		String aux = "";
		try {
			process = SAESCommandUtil.execute("sudo", new String[] { scriptPath
					+ " " + machineDateTime });

			aux = IOUtils.toString(process.getInputStream());

			if (aux != null && !aux.isEmpty()) {
				logger.info(aux.trim());
			}

		} catch (IOException e) {
			e.printStackTrace();
		}
		return aux;

	}

	private List<RegionData> getRegionsByLevel(
			CategoryData.Level categoryLevel,
			List<Long> codesOfRegionsAvailables) throws XmlGenerationException {

		try {

			SearchExpression search = new SearchExpression(RegionData.class);
			search.createAlias("category", "category");

			search.addSearchCriteria(CriteriaFactory.eq(
					"category.categoryLevel", categoryLevel));

			if ((codesOfRegionsAvailables != null)
					&& (!codesOfRegionsAvailables.isEmpty())) {
				search.addSearchCriteria(CriteriaFactory.in("code",

				codesOfRegionsAvailables));
			}

			List<RegionData> regionDataList = (List<RegionData>) this.persistenceService
					.searchObjects(search, null);

			return regionDataList;

		} catch (BaseException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_GETTING_REGION_ALL_PARENT, e);
		} catch (RuntimeException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_GETTING_REGION_ALL_PARENT, e);
		}
	}

	/**
	 * Translated an specific region to a given language
	 * 
	 * @param regionToTranslate
	 *            Region to be translated
	 * @param language
	 *            Language to translate the region into
	 * @throws XmlGenerationException
	 * @return List of regions corresponding to the siblings of the specified
	 *         region's parent
	 * @author Mario Bislick (mario.bislick@smartmatic.com)
	 */
	private RegionData translateRegionsName(RegionData regionToTranslate,
			LanguageData language) {

		try {

			TranslationKeyData traducctionRegion = getTraslation(
					"REGIONDATA__REGIONNAME_",
					regionToTranslate.getCustomCode(), language.getCustomCode());

			if (traducctionRegion != null) {
				regionToTranslate.setName(traducctionRegion
						.getTranslationText());
			} else {
				regionToTranslate.setName(regionToTranslate.getRegionName());
			}

			CategoryData category = regionToTranslate.getCategory();

			TranslationKeyData traducctionCategory = getTraslation(
					"CATEGORYDATA__CATEGORYNAME_", category.getCustomCode(),
					language.getCustomCode());

			if (traducctionCategory != null) {
				regionToTranslate.getCategory().setCategoryName(
						traducctionCategory.getTranslationText());
			} else {
				regionToTranslate.getCategory().setCategoryName(
						category.getCategoryName());
			}

		} catch (BaseException e) {
			logger.error(e, e.getMessage());
		}

		return regionToTranslate;
	}

	/**
	 * Method to get the translation from any property.
	 * 
	 * @param propertieToTraslate
	 *            Property to traslate.
	 * @param customCode
	 *            Property custom code.
	 * @param languageCustomCode
	 *            Language to traslate custom code.
	 * @return TranslationKeyData object with the property traslated.
	 * 
	 * @throws BaseException
	 *             Abstract base class to notify the client that a problem
	 *             occurred
	 */
	private TranslationKeyData getTraslation(String propertieToTraslate,
			String customCode, String languageCustomCode) throws BaseException {

		SearchExpression expression = new SearchExpression(
				TranslationKeyData.class);

		expression.createAlias("multilingualKey", "multilingualKey");
		expression.createAlias("electionLanguage", "electionLanguage");

		expression
				.addSearchCriteria(CriteriaFactory.eq(
						"multilingualKey.customCode", propertieToTraslate
								+ customCode));

		expression.addSearchCriteria(CriteriaFactory.eq(
				"electionLanguage.customCode", languageCustomCode));

		TranslationKeyData traducction;

		traducction = (TranslationKeyData) this.persistenceService
				.searchObject(expression, null);

		return traducction;
	}

	/**
	 * Method that generate a database backup whit out voting device's
	 * asymmetric_key.
	 * 
	 * @param backupDirTmp
	 *            File to make the backup.
	 * @param databaseServerHostname
	 *            database Server Host Name
	 * @return Byte object with the generated dump.
	 * @throws BaseException
	 *             Abstract base class to notify the client that a problem
	 *             occurred
	 */
	private byte[] generateDumRemote(File backupDirTmp,
			String databaseServerHostname) throws BaseException {
		byte[] backup;

		logger.info("Creating temporary database");
		executeScriptToDb("\"create database ecm_db_tmp\"",
				databaseServerHostname);
		logger.info("Exporting temporary backup");
		BackupDatabaseUtil.createBackupTmp(backupDirTmp, "ecm_db",
				databaseServerHostname);
		logger.info("Importing temporary backup");
		BackupDatabaseUtil.executePgRestore(backupDirTmp, "ecm_db_tmp",
				databaseServerHostname);
		logger.info("Removing certificates");
		executeScriptToDb("\"update voting_device set asymmetric_key=null\"",
				"ecm_db_tmp", databaseServerHostname);
		backup = BackupDatabaseUtil.createBackup("ecm_db_tmp", backupDirTmp,
				databaseServerHostname);
		logger.info("Deleting temporary database");
		executeScriptToDb("\"drop database ecm_db_tmp\"", null,
				databaseServerHostname);
		return backup;
	}

	/**
	 * Method to get a List of Region codes giving a List of region data.
	 * 
	 * @param regionDataList
	 *            Region Data List to get their codes.
	 * 
	 * @return List of Region codes.
	 */
	private List<Long> extractRegionCodes(List<RegionData> regionDataList) {
		List<Long> regionCodes = new ArrayList<Long>();

		for (Iterator<RegionData> iterator = regionDataList.iterator(); iterator
				.hasNext();) {
			RegionData regionData = iterator.next();
			regionCodes.add(regionData.getCode());
		}

		return regionCodes;
	}

	/**
	 * Persist the contest language
	 * 
	 * @param languagesCombinationsByContest
	 *            the languages set associated with the contest
	 * 
	 * @throws BaseException
	 *             The exception is thrown when there is an error executing the
	 *             post load process.
	 */
	private void persistContestLanguages(
			Map<Long, Set<Set<LanguageData>>> languagesCombinationsByContest)
			throws BaseException {

		Set<Long> keySet = languagesCombinationsByContest.keySet();

		try {
			for (Iterator<Long> iterator = keySet.iterator(); iterator
					.hasNext();) {
				long contestCode = iterator.next();

				Set<Set<LanguageData>> setCombinations = languagesCombinationsByContest
						.get(contestCode);

				ContestData contestData = new ContestData();
				contestData.setCode(contestCode);

				for (Iterator<Set<LanguageData>> iterator2 = setCombinations
						.iterator(); iterator2.hasNext();) {

					Set<LanguageData> set = iterator2.next();

					ContestLanguagesPairData contestLanguagesPair = new ContestLanguagesPairData();

					for (Iterator<LanguageData> iterator3 = set.iterator(); iterator3
							.hasNext();) {

						LanguageData languageData = iterator3.next();

						contestLanguagesPair.setLanguage1(languageData);

						if (iterator3.hasNext()) {
							contestLanguagesPair.setLanguage2(iterator3.next());
						}
						contestLanguagesPair.setContest(contestData);
						this.persistenceService.persist(contestLanguagesPair,
								null);
					}
				}
			}
		} catch (BaseException e) {
			logger.error(e, e.getMessage());
			throw e;
		}
	}

	/**
	 * Method used to get a server property in boolean.
	 * 
	 * @param key
	 *            The key of the property to search.
	 */
	private boolean getServerPropertyBoolean(String key) {
		return ApplicationProperties.getBooleanValue(key);
	}

	/**
	 * Method used to get a server property in boolean.
	 * 
	 * @param key
	 *            The key of the property to search.
	 */
	private String getServerPropertyString(String key) {
		return ApplicationProperties.getValue(key);
	}

	private void executeScriptToDb(String command, String dataBaseName) {
		executeScriptToDb(command, dataBaseName, null);
	}

	private void executeScriptToDb(String command, String dataBaseName,
			String databaseServerHostname) {
		try {

			ArrayList<Object> params = new ArrayList<Object>();
			params.add("-U");
			params.add("postgres");
			if (dataBaseName != null && !dataBaseName.isEmpty()) {
				params.add(dataBaseName);
			} else {
				params.add("postgres");
			}
			if (databaseServerHostname != null
					&& !databaseServerHostname.isEmpty()) {
				params.add("-h");
				params.add(databaseServerHostname);
			}
			params.add("-c");
			params.add(command);

			Process execute = SAESCommandUtil.execute("psql",
					params.toArray(new String[0]));
			String aux = IOUtils.toString(execute.getInputStream());

			if (aux != null && !aux.isEmpty()) {
				logger.info(aux.trim());
			}
		} catch (Exception e) {
			logger.error(e, e.getMessage());
		}
	}

	@Override
	public int getIntParameter(String key) throws IllegalStateException,
			BaseException {
		String value = this.getStringParameter(key);
		if (value == null) {
			throw new IllegalStateException(
					"Can't get the parameter called : [" + key + "] from DB");
		}
		return Integer.parseInt(value);

	}

	@Override
	public String getStringParameter(String key) throws BaseException {
		SearchExpression searchExpression = new SearchExpression(
				ParameterData.class);

		searchExpression.addSearchCriteria(CriteriaFactory.eq("parameterKey",
				key));

		ParameterData parameter = (ParameterData) this.persistenceService
				.searchObject(searchExpression, null);

		if (parameter == null) {
			return null;
		}

		return parameter.getParameterValue();
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void updateParameter(ParameterData parameter)
			throws XmlGenerationException {

		try {

			this.persistenceService.update(parameter, null);

		} catch (BaseException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_UPDATING_PARAMETERS, e);
		} catch (RuntimeException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_UPDATING_PARAMETERS, e);
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public void setParameter(String key, String value) throws BaseException {

		SearchExpression searchExpression = new SearchExpression(
				ParameterData.class);

		List<ParameterData> parameterList = (List<ParameterData>) this.persistenceService
				.searchObjects(searchExpression, null);

		for (Iterator<ParameterData> iterator = parameterList.iterator(); iterator
				.hasNext();) {
			ParameterData parameterData = iterator.next();

			if (parameterData.getParameterKey().equals(key)) {
				parameterData.setParameterValue(value);
				this.persistenceService.update(parameterData, null);
				return;
			}
		}

		// otherwise
		ParameterData parameter = new ParameterData();
		parameter.setParameterKey(key);
		parameter.setParameterValue(value);
		this.persistenceService.persist(parameter, null);
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public List<ParameterData> getParameters() throws XmlGenerationException {

		SearchExpression search = new SearchExpression(ParameterData.class);

		try {

			List<ParameterData> parameters = (List<ParameterData>) this.persistenceService
					.searchObjects(search, null);

			return parameters;

		} catch (BaseException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_GETTING_PARAMETERS, e);
		} catch (RuntimeException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_GETTING_PARAMETERS, e);
		}
	}

	/**
	 * @throws ElectoralDemographyException
	 * @inheritDoc
	 */
	@Override
	public Map<String, LogicalVotingGroupData> postLoadProcess(
			RegionData regionIndex) throws Exception {

		Map<Long, ContestGroupData> contestsForLastLevelRegion;

		RegionData region = null;

		region = this.demographyService
				.getAllRegionParentsWithDistricts(regionIndex);

		List<VotingDeviceData> votingDeviceList = this.electoralPhysicalLocationsRegistryService
				.getVotingDevicesByRegion(regionIndex);

		Map<String, LogicalVotingGroupData> logicalVotingGroupsData = new HashMap<String, LogicalVotingGroupData>();

		if (region == null || region.getAllParents() == null) {

			logger.warn("The region " + regionIndex.getCode() + " "
					+ regionIndex.getRegionName() + " has no parent associated");

		} else {
			contestsForLastLevelRegion = this.electoralDemographyService
					.getContestsForLastLevelRegion(regionIndex.getCode());

			this.electoralDemographyService.buildSpecificContestGroup(
					contestsForLastLevelRegion, regionIndex.getCode());

		}

		return logicalVotingGroupsData;
	}

	/**
	 * @throws ElectoralDemographyException
	 * @inheritDoc
	 */
	@Override
	public void persistLogicalVotingGroup(
			Map<String, LogicalVotingGroupData> logicalVotingGroupsData)
			throws XmlGenerationException, ElectoralDemographyException {

		try {

			Map<String, ContestGroupData> contestGroupMap = new HashMap<String, ContestGroupData>();

			List<ContestGroupData> contestGroupList = this.electoralDemographyService
					.getContestGroupsLoadedInDatabase();

			for (Iterator<ContestGroupData> iterator = contestGroupList
					.iterator(); iterator.hasNext();) {
				ContestGroupData contestGroupData = iterator.next();
				contestGroupMap.put(contestGroupData.getCustomCode(),
						contestGroupData);
			}

			int i = 1;
			Collection<LogicalVotingGroupData> logicalVotingGroupCollection = logicalVotingGroupsData
					.values();

			for (Iterator<LogicalVotingGroupData> iterator = logicalVotingGroupCollection
					.iterator(); iterator.hasNext();) {
				LogicalVotingGroupData logicalVotingGroup = iterator.next();
				logicalVotingGroup.setName("Logical Voting Group " + i++);

				logicalVotingGroup.setContestGroup(contestGroupMap
						.get(logicalVotingGroup.getContestGroup()
								.getCustomCode().split("\\-")[0]));

				this.persistenceService.persist(logicalVotingGroup, null);
			}
		} catch (BaseException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_PERSISTING_LOGICAL_VOTING_GROUP, e);
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public byte[] dumpDatabase(String databaseIp)
			throws BackupDatabaseGenerationExeption {

		int databasePort = Integer
				.valueOf(getServerPropertyString("master.db.port"));

		boolean isServerReachable = InetUtility.isServerReachable(databaseIp,
				databasePort);
		if (isServerReachable) {
			try {

				byte[] backup;

				String nameBackupDirTmp = System
						.getProperty("jboss.server.home.dir")
						+ "/db_backup_tmp/";

				File backupDirTmp = new File(nameBackupDirTmp);
				if (!backupDirTmp.exists()) {
					backupDirTmp.mkdirs();
				}

				backup = this.generateDumRemote(backupDirTmp, databaseIp);

				FileUtils.deleteQuietly(backupDirTmp);
				return backup;

			} catch (Exception e) {
				logger.error(e, "Error creating backup of the database");
				throw new BackupDatabaseGenerationExeption(
						BackupDatabaseGenerationExeption.ExceptionInfo.PROBLEM_GENERATING_DATABASE_BACKUP,
						e);
			}
		} else {
			throw new BackupDatabaseGenerationExeption(
					BackupDatabaseGenerationExeption.ExceptionInfo.PROBLEM_REACHING_DATA_BASE);
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public List<PollingPlaceData> getPollingPlacesPerRegion(RegionData region)
			throws XmlGenerationException {
		List<PollingPlaceData> result = null;
		SearchExpression expression = new SearchExpression(
				PollingPlaceData.class);
		SearchExpression subQuery = new SearchExpression(PollingPlaceData.class);
		subQuery.createAlias("region", "region");
		subQuery.addSearchCriteria(CriteriaFactory.eq("region.code",
				region.getCode()));

		subQuery.setTransformation(TransformationFactory
				.distinctTransformation(TransformationFactory
						.propertyTransformation("code")));

		expression.addSearchCriteria(CriteriaFactory.propertyIn("code",
				subQuery));

		expression.addOrder(OrderCriteria.asc("customCode"));

		try {
			result = (List<PollingPlaceData>) this.persistenceService
					.searchObjects(expression, null);
		} catch (BaseException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_CREATING_XML_FILE, e);
		} catch (RuntimeException e) {
			logger.error(e, e.getMessage());
			throw new XmlGenerationException(
					ExceptionInfo.PROBLEM_CREATING_XML_FILE, e);
		}
		return result;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public String getServerIP() {

		return InetUtility.obtainServerIp();
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public boolean integrityValidationContesContestClassContestGroup()
			throws BaseException {

		SearchExpression expression = new SearchExpression(
				ContestClassData.class);

		expression.createAlias("contests", "contests");

		expression.createAlias("contestGroupList", "contests.contestGroupList");

		if (this.persistenceService.countObjects(expression, null) > 0) {
			return true;
		}

		return false;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void fillLanguageCombinations() throws Exception {

		List<Long> lastLevelRegionCodes = extractRegionCodes(this.demographyService
				.getLastLevelRegionsLight(null));

		List<RegionData> regionDataList = this.demographyService
				.getRegionsWithParentsByRegionCodes(lastLevelRegionCodes);

		Map<String, LanguageData> officalLanguagesMap = getOfficalLanguages();

		Map<Long, Set<Set<LanguageData>>> languagesCombinationsByContest = new HashMap<Long, Set<Set<LanguageData>>>();

		for (Iterator<RegionData> iterator = regionDataList.iterator(); iterator
				.hasNext();) {
			RegionData regionData = iterator.next();

			Set<DistrictData> regionDistrictSet = new HashSet<DistrictData>();
			Set<LanguageData> regionLanguageSet = new HashSet<LanguageData>();

			for (Iterator<RegionData> iteratorForRegionParents = regionData
					.getAllParents().iterator(); iteratorForRegionParents
					.hasNext();) {
				RegionData regionParentData = iteratorForRegionParents.next();
				regionDistrictSet.addAll(regionParentData.getDistricts());
				regionLanguageSet.addAll(regionParentData
						.getElectionsLanguage());
			}

			List<ContestData> contestRegionList = this.electoralDemographyService
					.getContestByDistrict(regionDistrictSet);

			Set<LanguageData> regionLanguages = new HashSet<LanguageData>();

			for (Iterator<LanguageData> iterator2 = regionLanguageSet
					.iterator(); iterator2.hasNext();) {
				LanguageData languageData = iterator2.next();

				regionLanguages.add(officalLanguagesMap.get(languageData
						.getLanguageRFC()));
			}

			for (Iterator<ContestData> iterator2 = contestRegionList.iterator(); iterator2
					.hasNext();) {
				ContestData contestData = iterator2.next();

				if (languagesCombinationsByContest.containsKey(contestData
						.getCode())) {

					Set<Set<LanguageData>> storedCombinations = languagesCombinationsByContest
							.get(contestData.getCode());

					if (!storedCombinations.contains(regionLanguages)) {
						storedCombinations.add(regionLanguages);
					}
				} else {
					Set<Set<LanguageData>> newCombinations = new HashSet<Set<LanguageData>>();

					newCombinations.add(regionLanguages);

					languagesCombinationsByContest.put(contestData.getCode(),
							newCombinations);
				}
			}
		}
		this.persistContestLanguages(languagesCombinationsByContest);
	}

	/**
	 * @throws BaseException
	 * @inheritDoc
	 */
	@Override
	public void downloadECMReports() throws BaseException {

		SearchExpression searchReports = new SearchExpression(
				ReportTemplateData.class);
		searchReports.addSearchCriteria(CriteriaFactory.or(CriteriaFactory.or(
				CriteriaFactory.eq("type",
						ReportTemplateData.Type.PM_PSWD_REPORT),
				CriteriaFactory.eq("type",
						ReportTemplateData.Type.RC_PSWD_REPORT)),
				CriteriaFactory.eq("type",
						ReportTemplateData.Type.RTS_PSWD_REPORT)));
		searchReports.addSearchCriteria(CriteriaFactory.eq("app",
				ReportTemplateData.Application.ECM));

		List<ReportTemplateData> reports = (List<ReportTemplateData>) this.persistenceService
				.searchObjects(searchReports, null);

		String reportFilesPath = getServerPropertyString("reports.files.folder.name");
		File reportFilesDir = new File(
				System.getProperty("jboss.server.home.dir"), reportFilesPath);
		// If the Directory doesn't exist in the server, nothing happens
		if (reportFilesDir.exists()) {
			for (Iterator<ReportTemplateData> iterator = reports.iterator(); iterator
					.hasNext();) {
				ReportTemplateData reportTemplateData = iterator.next();

				if (reportTemplateData.getJrxml().getBinaryData() != null) {
					File template = new File(reportFilesDir,
							reportTemplateData.getExportedName());
					try {
						FileUtils.writeByteArrayToFile(template,
								reportTemplateData.getJrxml().getBinaryData());
					} catch (IOException e) {
						logger.error(e,
								StringUtils.defaultString(e.getMessage()));
						throw new BaseException(
								ReportGenerationExceptionInfo.GENERATION_REPORT_PROBLEM);
					}
				}
			}
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public Map<String, byte[]> getLogFiles() throws IOException {
		String logFilesFolder = getServerPropertyString("server.side.log.folder");
		File logFilesDir = new File(
				System.getProperty("jboss.server.home.dir"), logFilesFolder);
		HashMap<String, byte[]> logFilesMap = new HashMap<String, byte[]>();
		// If the Directory doesn't exist in the server, nothing happens
		if (logFilesDir.exists()) {
			// The list of files retrieved as File objects
			File[] logFiles = logFilesDir.listFiles();
			if (logFiles.length == 0) {
				// If the folder exists and does not have elements, there's an
				// error
				throw new IOException("None log file was found.");
			}
			// This filter only returns files
			FileFilter fileFilter = new FileFilter() {
				@Override
				public boolean accept(File file) {
					return file.isFile()
							&& (StringUtils.contains(file.getName(),
									"server.log") || StringUtils.contains(
									file.getName(), ".log.gz"));
				}
			};
			// Retrieve all files from that directory
			logFiles = logFilesDir.listFiles(fileFilter);

			long maxSizeAllowed = Long
					.parseLong(getServerPropertyString("max.size.to.export.log"));

			for (int i = 0; i < logFiles.length; ++i) {
				File logFile = logFiles[i];

				long fileLength = (logFile.length() / 1024) / 1024;

				if (fileLength > maxSizeAllowed) {

					logger.error(logFile.getName()
							+ " exceeds the maximum size (" + maxSizeAllowed
							+ " MB) allowed");

					return null;
				}

				byte[] logFileByte = FileUtils.readFileToByteArray(logFile);

				logFilesMap.put(logFile.getName(), logFileByte);

			}
		}
		return logFilesMap;
	}


    /**
     * {@inheritDoc}
     */
    public List<DeviceParameterData> retrieveDeviceParameters() {
        return this.retrieveDeviceParameters(true);
    }


    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public List<DeviceParameterData>
    retrieveDeviceParameters(boolean includeDefs) {
        List<DeviceParameterData> result;       // Method's result
        SearchExpression search;                // SearchExpression
        Map<String, DeviceParameterData> merge; // Mix of Device parameters

        try {
            search = new SearchExpression(DeviceParameterData.class);
            result = (List<DeviceParameterData>)
                this.persistenceService.searchObjects(search, null);

            if (includeDefs) {
                merge = DeviceParameterData.defaultInstances();

                for (DeviceParameterData dp : result) {
                    merge.put(dp.getName(), dp);
                }
                result = new ArrayList<DeviceParameterData>(merge.values());
            }
            Collections.sort(result, DeviceParameterData.Comparators.NAME_ASC);
            return result;

        } catch (BaseException ex) {
            throw new RuntimeException(ex);
        }
    }
}
