/**
 * ECMWorkflowServiceImpl.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.impl;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Init;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Remove;
import javax.ejb.Stateful;

import org.apache.commons.io.IOUtils;

import com.smartmatic.waffle.client.ClientJNDIWrapper;
import com.smartmatic.waffle.client.JNDICommunicationException;
import com.smartmatic.waffle.domain.security.FunctionalityData;
import com.smartmatic.waffle.domain.user.UserData;
import com.smartmatic.waffle.ecm.service.spec.IECMWorkflowService;
import com.smartmatic.waffle.ecm.service.spec.IECMWorkflowServiceLocal;
import com.smartmatic.waffle.ecm.service.workflow.ECMWorkflowServiceInvoker;
import com.smartmatic.waffle.ecm.service.workflow.ECMWorkflowStatus;
import com.smartmatic.waffle.ecm.workflow.service.spec.IWorkflowService;
import com.smartmatic.waffle.ecm.workflow.service.spec.IWorkflowServiceLocal;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.search.criteria.CriteriaFactory;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;

/**
 * Session beans that handles all ECM workflow operations. Class that implements
 * all the server services defined in <code>IECMWorkflowService</code>
 */
@Stateful(name = IECMWorkflowService.SERVICE_NAME)
@Remote(IECMWorkflowService.class)
@Local(IECMWorkflowServiceLocal.class)
@org.jboss.ejb3.annotation.CacheConfig(idleTimeoutSeconds = 3600)
public class ECMWorkflowServiceImpl implements IECMWorkflowService,
		IECMWorkflowServiceLocal {

	/** For log messages purpose. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ECMWorkflowServiceImpl.class);

	/**
	 * Instance of WorkflowServiceBean to get reference to service layer.
	 */
	@EJB
	private IWorkflowServiceLocal workflowService;

	/**
	 * Instance of PersistenceServiceBean to get reference to service layer.
	 */
	@EJB
	private IPersistenceServiceLocal persistenceService;

	/***
	 * Original list of available functionalities to current user.
	 */
	private Set<FunctionalityData> originalFunctionalities;

	/***
	 * the current user logged into app.
	 */
	private UserData userData;

	/**
	 * Default constructor.
	 */
	public ECMWorkflowServiceImpl() {
	}

	/**
	 * Inits the.
	 */
	@Init
	public void init() {
		byte[] processData;
		InputStream is = ECMServiceImpl.class
				.getResourceAsStream("/resources/ecm.bpmn");
		try {
			processData = IOUtils.toByteArray(is);
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(is);
		}
		this.workflowService.init(processData);

		try {
			startWorkflow();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Removes the.
	 */
	@Remove
	public void remove() {
		this.workflowService.remove();
		this.workflowService = null;
	}

	/**
	 * Passivate.
	 */
	@javax.ejb.PrePassivate
	public void passivate() {
	}

	/**
	 * Activate.
	 */
	@javax.ejb.PostActivate
	public void activate() {
		try {

			this.workflowService = (IWorkflowServiceLocal) ClientJNDIWrapper
					.getInstance().getReference(IWorkflowService.class);

			init();
		} catch (ClassCastException e) {
			logger.error(e,
					"There is an error getting instance of workflow service");
		} catch (JNDICommunicationException e) {
			logger.error(e,
					"There is an error getting instance of workflow service");
		}
	}

	/**
	 * Method to start the workflow.
	 * 
	 * @throws BaseException
	 *             - When there is an error
	 */
	private void startWorkflow() throws BaseException {
		Long[] workflowIds = this.workflowService.getWorkflowIds();

		Map<String, Object> parameters = new HashMap<String, Object>();
		parameters.put("valid", Boolean.TRUE);
		parameters.put("showWarning", Boolean.FALSE);

		if (workflowIds.length != 0) {
			this.workflowService.loadProcess(workflowIds[0], null);
		} else {
			this.workflowService.startProcess("com.smartmatic.waffle.ecm",
					parameters, new ECMWorkflowServiceInvoker());
		}
	}

	/**
	 * Method to filter the list of features available to user at the current
	 * node in the workflow.
	 * 
	 * @param userData
	 *            - the current user.
	 */
	private void filterByWorkflow(UserData userData) {

		String functionalitiesAllowed = (String) this.workflowService
				.getVariable("Content");

		String[] functionalities = functionalitiesAllowed.split(",");

		this.originalFunctionalities = userData.getProfile()
				.getFunctionalities();

		Set<FunctionalityData> filteredFunctionalities = new HashSet<FunctionalityData>();

		for (FunctionalityData functionality : userData.getProfile()
				.getFunctionalities()) {

			for (String functionalityName : functionalities) {

				if (functionality.getFunctionalityName().equalsIgnoreCase(
						functionalityName.trim())) {

					filteredFunctionalities.add(functionality);
				}

			}

		}

		userData.getProfile().setFunctionalities(filteredFunctionalities);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public UserData getActualUser() {

		this.userData.getProfile().setFunctionalities(
				this.originalFunctionalities);

		filterByWorkflow(this.userData);
		return this.userData;

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	@RolesAllowed({ "ADMIN", "FILE_CONVERTION", "BULK_UPLOAD",
			"KEY_GENERATION", "IMAGES_UPLOAD", "REPORT_GENERATION",
			"FILE_GENERATION", "PARAMETER_CONFIGURATION" })
	public UserData getUserById(String user) throws BaseException {

		if (this.userData == null || !this.userData.getName().equals(user)) {

			String[] pathsToInit = { "profile.functionalities" };
			SearchExpression searchExpression = new SearchExpression(
					UserData.class);
			searchExpression.addSearchCriteria(CriteriaFactory
					.eq("login", user));
			Object userObject = this.persistenceService.searchObject(
					searchExpression, pathsToInit, null);

			if (userObject != null && userObject instanceof UserData) {
				this.userData = (UserData) userObject;
				this.persistenceService.detach(this.userData);
				this.persistenceService.detach(this.userData.getProfile());
				filterByWorkflow(this.userData);
			}
		} else {
			this.userData.getProfile().setFunctionalities(
					this.originalFunctionalities);
			filterByWorkflow(this.userData);
		}

		return this.userData;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws BaseException
	 */
	@Override
	public boolean isAdminUserConfigured() throws BaseException {

		String[] pathsToInit = { "profile.functionalities" };
		SearchExpression searchExpression = new SearchExpression(UserData.class);
		searchExpression
				.addSearchCriteria(CriteriaFactory.eq("login", "admin"));
		Object userObject = this.persistenceService.searchObject(
				searchExpression, pathsToInit, null);

		if (userObject != null && userObject instanceof UserData) {
			UserData admin = (UserData) userObject;
			if (admin.getPassword() != null && !admin.getPassword().isEmpty()) {
				return true;
			}
		}
		return false;

	}

	@Override
	public UserData updateUser(String user, String password)
			throws BaseException {

		UserData userData = getUserById(user);
		userData.setPassword(password);
		this.persistenceService.update(userData, null);
		return userData;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ECMWorkflowStatus gotoNext() {

		this.workflowService.gotoNext();

		ECMWorkflowStatus status = (ECMWorkflowStatus) this.workflowService
				.getVariable("status");

		return status;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void goBack() {

		this.workflowService.goBack();

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void moveForward() {
		this.workflowService.moveForward();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void moveBackward() {
		this.workflowService.moveBackward();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getActualNodeName() {
		return this.workflowService.getActualNodeName();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Object getVariable(String name) {
		return this.workflowService.getVariable(name);
	}

}
