/**
 * IECMElectoralOrganizationService.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.spec;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.ejb.Remote;

import com.smartmatic.waffle.domain.authority.ElectoralAuthorityData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionKeyStoreComponentsDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectoralAuthoritySecurityComponentsDTO;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;

/**
 * Remote interface which establishes methods contract that must be implemented
 * by <code>ECMElectoralOrganizationServiceImpl</code> class
 */
@Remote
public interface IECMElectoralOrganizationService {

	/** Service name Constant. */
	public static final String SERVICE_NAME = "com.smartmatic.waffle.ecm.service.spec.IECMElectoralOrganizationService";

	/**
	 * Method to retrieve the active election in a <code>ElectionData</code>
	 * object.
	 * 
	 * @return <code>ElectionData</code> object with the active election
	 * 
	 * @throws ElectoralOrganizationOperationException
	 *             Signal that an exception has occurred retrieving the active
	 *             election
	 */
	public ElectionData retrieveActiveElection()
			throws ElectoralOrganizationOperationException;

	/**
	 * Method to retrieve the key store components from the active election.
	 * 
	 * @return DTO with key store components from the active election
	 * 
	 * @throws ElectoralOrganizationOperationException
	 *             Signal that an exception has occurred retrieving key store
	 *             components
	 */
	public ElectionKeyStoreComponentsDTO getElectionKeyStoreComponents()
			throws ElectoralOrganizationOperationException;

	/**
	 * Method to retrieve the electoral authorities amount stored in database.
	 * 
	 * @return the amount of electoral authorities
	 * @throws ElectoralOrganizationOperationException
	 */
	public int retrieveElectoralAuthoritiesAmount()
			throws ElectoralOrganizationOperationException;

	/**
	 * Sets the security components of a number of electoral authorities stored
	 * in database.
	 * 
	 * These security components are: protection key, alias, asymmetric key,
	 * user name and symmetric key.
	 * 
	 * The electoral authorities will be retrieved from the database by chunks,
	 * or pages, of <code>pageSize</code>, and the method will set the
	 * components for page number <code>pageNumber</code>.
	 * 
	 * @param pageNumber
	 *            page number to process
	 * @param pageSize
	 *            the total page size
	 * @throws ElectoralOrganizationOperationException
	 *             signal that an error occurred setting the security components
	 */
	public void setSecurityComponentsToElectoralAuthorities(int pageNumber,
			int pageSize) throws ElectoralOrganizationOperationException;

	/**
	 * Deletes the security components stored for the electoral authorities.
	 * 
	 * @return number of removed records
	 * 
	 * @throws ElectoralOrganizationOperationException
	 *             signal that an error occurred deleting the security
	 *             components
	 */
	public int deleteElectoralAuthoritiesSecurityComponents()
			throws ElectoralOrganizationOperationException;

	/**
	 * Retrieves the security components stored for the electoral authorities
	 * 
	 * The electoral authorities will be retrieved from the database by chunks,
	 * or pages, of <code>pageSize</code>, and the method will set the
	 * components for page number <code>pageNumber</code>.
	 * 
	 * @param pageNumber
	 *            page number to process
	 * @param pageSize
	 *            the total page size
	 * @return a list of the electoral authorities components objects
	 * @throws ElectoralOrganizationOperationException
	 *             signal that an error occurred retrieving the electoral
	 *             authorities security components
	 */
	public List<ElectoralAuthoritySecurityComponentsDTO> retrieveElectoralAuthoritiesSecurityComponents(
			int pageNumber, int pageSize)
			throws ElectoralOrganizationOperationException;

	/**
	 * This method is used to get a map which the key is the lower electoral
	 * authority defined by the region category level and the value a collection
	 * of electoral authorities which belong to the last level region provided.
	 * 
	 * @param lastLevelRegion
	 *            the last level region the get the electoral authorities
	 * @return a map of the lower electoral authority as key and a collection as
	 *         value
	 * @throws ElectoralOrganizationOperationException
	 *             signal than an error has occurred getting the electoral
	 *             authorities.
	 */
	public Map<ElectoralAuthorityData, Collection<ElectoralAuthorityData>> getElectoralAuthoritiesByLastLevelRegion(
			RegionData lastLevelRegion)
			throws ElectoralOrganizationOperationException;

	/**
	 * Method used to return a list of electoral authorities data objects given
	 * a list of regions.
	 * 
	 * @param selectedRegions
	 *            the list of regions to get the electoral authorities
	 * @return the list of electoral authorities that match the criteria
	 * @throws ElectoralOrganizationOperationException
	 *             Signal that an error has occurred getting the electoral
	 *             authorities
	 */
	public List<ElectoralAuthorityData> getElectoralAuthoritiesByRegion(
			List<RegionData> selectedRegions)
			throws ElectoralOrganizationOperationException;
}
