package com.smartmatic.waffle.ecm.service.spec;

import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.Remote;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.data.transfer.objects.ActiveContestForElectionDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionXmlComponentsDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.VotingDeviceDTO;
import com.smartmatic.waffle.ecm.service.lang.MasterUSBGenerationException;

/**
 * Remote interface which establishes methods contract that must be implemented
 * by <code>ECMMasterUSBGenerationServiceImpl</code> class
 */
@Remote
public interface IECMMasterUSBGenerationService {

	/**
	 * Holds the name of the bean that implements this interface.
	 * 
	 */
	public static final String SERVICE_NAME = "com.smartmatic.waffle.ecm.service.spec.IECMMasterUSBGenerationService";

	/**
	 * Retrieves a DTO with the contests, languages, and languages resources
	 * (needed to generate the election.xml)
	 * 
	 * @param lastLevelRegions
	 *            last level region list selected by user to generate Master
	 *            USB.
	 * @param appLanguage
	 *            current language with which the application is running.
	 * 
	 * @return DTO object with components needed to build the election.xml.
	 * 
	 * @throws MasterUSBGenerationException
	 *             signals that something happened generating election.xml
	 *             components
	 */
	public ActiveContestForElectionDTO getActiveContestForElection(
			List<RegionData> lastLevelRegions, LanguageData appLanguage)
			throws MasterUSBGenerationException;

	/**
	 * Method that generate an election object with all electoral information to
	 * create the election.xml for a list of contests
	 * 
	 * @param contestList
	 *            the contest list to generate the election.
	 * @param objectCodePrefix
	 *            the prefix used for object's codes when generating election
	 *            objects.
	 * @param election
	 *            the election object reference
	 * @param resourcesLanguages
	 *            the resources languages translations object
	 * @param contestGroups
	 *            the constest groups.
	 * @return the election xml components object
	 * @throws MasterUSBGenerationException
	 *             signals that something happened generating election
	 *             components
	 */
	public ElectionXmlComponentsDTO generateElectionForContest(
			List<ContestData> contestList, int objectCodePrefix,
			ElectionData election,
			Map<String, Map<String, String>> resourcesLanguages,
			List<ContestGroupData> contestGroups)
			throws MasterUSBGenerationException;

	/**
	 * Method that generates an voting device object with all the voting device
	 * information to create the voting-device.xml for a region. This file is
	 * loaded by the voting machine to display the election data.
	 * 
	 * @param votingDevice
	 *            the voting device object
	 * @param region
	 *            the region to create to generate the voting device DTO
	 * @param contestGroup
	 *            the contest group for which the voting device must be
	 *            generated
	 * @param resourcesLanguages
	 *            the resources languages object reference
	 * @param generateDummyVotingDevice
	 *            true if the voting device must be created with dummy data,
	 *            this is required when generating a master USB for the
	 *            screenshots functionality.
	 * @param election
	 *            the current active election
	 * @param audioVotingEnabled
	 *            The audio voting flag
	 * @return the voting device DTO object
	 * @throws MasterUSBGenerationException
	 *             A signal that an error has occurred generating the voting
	 *             device object.
	 */
	public VotingDeviceDTO generateVotingDeviceForRegion(
			VotingDeviceData votingDevice, RegionData region,
			List<ContestGroupData> contestGroup,
			Map<String, Map<String, String>> resourcesLanguages,
			boolean generateDummyVotingDevice, ElectionData election,
			boolean audioVotingEnabled) throws MasterUSBGenerationException;

	/**
	 * Method that returns a set of language data objects stored in the database
	 * and used for the current election.
	 * 
	 * @return a set of language data objects
	 * @throws MasterUSBGenerationException
	 *             A signal that an error has occurred getting the language data
	 *             set.
	 */
	public Set<LanguageData> getLanguages() throws MasterUSBGenerationException;

	/**
	 * Return the amount of processed contests while generating the election
	 * data.
	 * 
	 * @return the amount of processed contests.
	 */
	public int getProcessedContests();

	/**
	 * Reset the processed contest counter.
	 */
	public void resetProcessedContests();

}