package com.smartmatic.waffle.ecm.service.spec;

import java.util.List;

import javax.ejb.Remote;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.service.lang.NominationException;

@Remote
public interface IECMNominationService {
	/** SERVICE_NAME Constant. */
	public static final String SERVICE_NAME = "com.smartmatic.waffle.ecm.service.spec.IECMNominationService";

	/**
	 * Method to return the amount of candidates fetched given the filter
	 * parameters.
	 * 
	 * @param candidateName
	 *            Candidates search text
	 * @param regions
	 *            The regions list to filter the search
	 * @param contests
	 *            The contests list to filter the search
	 * @param parties
	 *            The parties list to filter the search.
	 * @return The amount of candidates matching the search parameters.
	 * @throws NominationException
	 *             A signal that an error occurred retrieving candidates amount.
	 */
	public int retrieveNumberOfCandidates(String candidateName,
			List<RegionData> regions, List<ContestData> contests,
			List<PartyData> parties) throws NominationException;

	/**
	 * Method to get the list of candidates matching the search parameters.
	 * 
	 * @param pageNumber
	 *            The page number to process.
	 * @param numberOfPages
	 *            The total number of pages
	 * @param candidateName
	 *            Candidates search text
	 * @param regions
	 *            The regions list to filter the search
	 * @param contests
	 *            The contests list to filter the search
	 * @param parties
	 *            The parties list to filter the search.
	 * @param language
	 *            The selected language
	 * @return a list of candidates matching the search parameters.
	 * @throws NominationException
	 *             A signal that an error occurred retrieving candidates.
	 */
	public List<EligibleOptionData> getCandidates(int pageNumber,
			int numberOfPages, String candidateName, List<RegionData> regions,
			List<ContestData> contests, List<PartyData> parties,
			LanguageData language) throws NominationException;

	/**
	 * Method to retrieve the amount of parties matching the filter parameters.
	 * 
	 * @param partyName
	 *            parties search text
	 * @param regions
	 *            The regions list to filter the search
	 * @param contests
	 *            The contest list to filter the search
	 * @return the total of matching parties.
	 * @throws NominationException
	 *             A signal that an error occurred retrieving parties amount.
	 */
	public int retrieveNumberOfParties(String partyName,
			List<RegionData> regions, List<ContestData> contests)
			throws NominationException;

	/**
	 * Method to retrieve the parties matching the filter parameters.
	 * 
	 * @param pageNumber
	 *            page number to process.
	 * @param numberOfPages
	 *            the total of pages to retrieve.
	 * @param partyName
	 *            parties search text
	 * @param regions
	 *            The regions list to filter the search
	 * @param contests
	 *            The contest list to filter the search
	 * @param language
	 *            The selected language
	 * @return A list of parties matching the search parameters.
	 * @throws NominationException
	 *             A signal that an error occurred retrieving parties.
	 */
	public List<EligibleOptionData> getParties(int pageNumber,
			int numberOfPages, String partyName, List<RegionData> regions,
			List<ContestData> contests, LanguageData language)
			throws NominationException;

	/**
	 * Method to retrieve a list of parties given the filter parameters.
	 * 
	 * @param regions
	 *            The regions list to filter the search
	 * @param contests
	 *            The contests list to filter the search
	 * @return A list of parties matching the search parameters.
	 * @throws NominationException
	 *             A signal that an error occurred retrieving parties.
	 */
	public List<PartyData> getParties(List<RegionData> regions,
			List<ContestData> contests) throws NominationException;

	/**
	 * Method to retrieve a list of contests given the filter parameters.
	 * 
	 * @param regions
	 *            The regions list to filter the search
	 * @param parties
	 *            The parties list to filter the search
	 * @param language
	 *            The selected language
	 * @return A list of contests matching the search parameters.
	 * @throws NominationException
	 *             A signal that an error occurred retrieving contests.
	 */
	public List<ContestData> getContests(List<RegionData> regions,
			List<PartyData> parties, LanguageData language)
			throws NominationException;

	/**
	 * Method that updates an eligible option data.
	 * 
	 * @param eligibleOption
	 *            The eligible option to update
	 * @return the updated object
	 * @throws NominationException
	 *             A signal that an error occurred updating the data.
	 */
	public EligibleOptionData updateEligibleOption(
			EligibleOptionData eligibleOption) throws NominationException;

	/**
	 * Method that updates an eligible option and party name.
	 * 
	 * @param eligibleOption
	 *            The eligible option to update
	 * @return the updated object
	 * @throws NominationException
	 *             A signal that an error occurred updating the data.
	 */
	public PartyData updateEligibleOptionAndParty(
			EligibleOptionData eligibleOption) throws NominationException;

	/**
	 * Method that returns a list of eligible options data objects with the
	 * information of its corresponding ballot option data.
	 * 
	 * @param eligibleOptionsList
	 *            the eligible option list to fill the ballot option data.
	 * @return a list of eligible options with its ballot option data set.
	 * @throws NominationException
	 *             A signal that an error has occurred getting the ballot
	 *             options data list.
	 */
	public List<EligibleOptionData> getBallotOptionByEligibleOptions(
			List<EligibleOptionData> eligibleOptionsList)
			throws NominationException;

	/**
	 * Method that returns a list of eligible options data objects with the
	 * information of its corresponding role data.
	 * 
	 * @param eligibleOptionsList
	 *            the eligible option list to fill the role data.
	 * @return a list of eligible options with its role data set.
	 * @throws NominationException
	 *             A signal that an error has occurred getting the eligible
	 *             option roles data list.
	 */
	public List<EligibleOptionData> getEligibleOptionRoleByEligibleOption(
			List<EligibleOptionData> eligibleOptionsList)
			throws NominationException;

	/**
	 * Method that returns a list of eligible option data objects filtered by
	 * contest.
	 * 
	 * @param contestData
	 *            the contest data object to filter the search
	 * @return a list of eligible options that matched the search
	 * @throws NominationException
	 *             a signal that an error has occurred getting the eligible
	 *             options by contest.
	 */
	public List<EligibleOptionData> getEligibleOptionsByContest(
			ContestData contestData) throws NominationException;

	/**
	 * Method that returns a list of contest data objects given a list of
	 * contest codes to search.
	 * 
	 * @param constestsCodesList
	 *            a contest codes list to search
	 * @param pageNumber
	 *            the page to process
	 * @param pageSize
	 *            the total size of the resultset
	 * @return the contest data objects list that matched the search.
	 * @throws NominationException
	 *             A signal that an error has occurred getting the contests
	 *             list.
	 */
	public List<ContestData> getContests(List<Long> constestsCodesList,
			int pageNumber, int pageSize) throws NominationException;

	/**
	 * Method that returns a list of contest given a contest class
	 * 
	 * @param contestClass
	 *            the contest class to fetch all of its contests
	 * @param language
	 *            the language to translate contest classes and its contests
	 * @return the list of contest
	 * @throws NominationException
	 *             a signal that an error occurred getting the contests
	 */
	public List<ContestData> getContestsByContestClass(String contestClass,
			LanguageData language) throws NominationException;
}
