/**
 * DataTransformation.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.transformData;

import static com.smartmatic.waffle.ecm.service.lang.TransformationException.TransformationExceptionInfo.DEV_PARAM_INVALID_VALUE;
import static com.smartmatic.waffle.ecm.service.lang.TransformationException.TransformationExceptionInfo.MISSING_REQUIRED_FIELDS_ERROR;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.pentaho.di.core.KettleEnvironment;
import org.pentaho.di.core.logging.CentralLogStore;
import org.pentaho.di.core.logging.Log4jBufferAppender;
import org.pentaho.di.core.logging.LogLevel;
import org.pentaho.di.core.logging.LoggingRegistry;
import org.pentaho.di.trans.Trans;
import org.pentaho.di.trans.TransMeta;

import com.smartmatic.waffle.ecm.data.transfer.objects.EMLTransformationResultDTO;
import com.smartmatic.waffle.ecm.service.lang.TransformationException;
import com.smartmatic.waffle.ecm.service.lang.TransformationException.TransformationExceptionInfo;

/**
 * This class transforms the format files to be load in bulk load process.
 */
public class DataTransformation {

	/**
	 * Folder to execute the transformation.
	 */
	private File folderToExecuteTransformation;

	/**
	 * Instantiates a new DataTransformation.
	 */
	public DataTransformation() {
		this.folderToExecuteTransformation = new File(
				System.getProperty("jboss.server.home.dir"), "template");
	}

	/**
	 * Instantiates a new data transformation.
	 * 
	 * @param folderToExecuteTransformation
	 *            folder to execute the transformation
	 */
	public DataTransformation(String folderToExecuteTransformation) {
		this.folderToExecuteTransformation = new File(
				folderToExecuteTransformation);
	}

	/**
	 * Method that transforms smartmatic files as a plain text file to xml file
	 * with specific format to be bulk uploaded to database.
	 * 
	 * @param kettleTransformationFileName
	 *            name to the kettle transformation file (.ktr)
	 * 
	 * @param fileToTransformContent
	 *            file to transform
	 * 
	 * @return a byte array with the content of transformed file
	 * 
	 * @throws TransformationException
	 *             signal that an exception has occurred executing the file
	 *             transformation
	 */
	public byte[] executeSmttFilesTransformation(
			String kettleTransformationFileName, byte[] fileToTransformContent)
			throws TransformationException {

		File fileToTransform = null;
		File transformedFile = null;

		TransMeta transformationMetaDataForSmttFiles = null;
		Trans smttFileTransformer = null;

		try {

			fileToTransform = new File(this.folderToExecuteTransformation,
					"smttFile.txt");
			transformedFile = new File(this.folderToExecuteTransformation,
					"smttFile.xml");

			FileUtils.writeByteArrayToFile(fileToTransform,
					fileToTransformContent);

			File kettleTransformationFile = new File(
					this.folderToExecuteTransformation,
					kettleTransformationFileName);

			if (!KettleEnvironment.isInitialized()) {
				KettleEnvironment.init(false);
			}

			transformationMetaDataForSmttFiles = new TransMeta(
					kettleTransformationFile.getAbsolutePath());

			smttFileTransformer = new Trans(transformationMetaDataForSmttFiles);

			smttFileTransformer.setLogLevel(LogLevel.ERROR);

			smttFileTransformer.setParameterValue("pathInputFile",
					fileToTransform.getAbsolutePath());

			smttFileTransformer.setParameterValue(
					"pathOuputFile",
					StringUtils.substringBeforeLast(
							transformedFile.getAbsolutePath(), "."));

			smttFileTransformer.execute(null);

			smttFileTransformer.waitUntilFinished();
			this.detectTransformationErrors(smttFileTransformer);

			return FileUtils.readFileToByteArray(transformedFile);

        } catch (TransformationException ex) {
            throw ex;

		} catch (Exception e) {
			throw new TransformationException(
					e,
					TransformationExceptionInfo.UNEXPECTED_TRANSFORMATION_ERROR,
					StringUtils.defaultString(e.getMessage()));
		} finally {
			if (fileToTransform != null) {
				FileUtils.deleteQuietly(fileToTransform);
			}

			if (transformedFile != null) {
				FileUtils.deleteQuietly(transformedFile);
			}
			if (transformationMetaDataForSmttFiles != null) {
				transformationMetaDataForSmttFiles.clear();
			}

			if (smttFileTransformer != null) {
				smttFileTransformer.cleanup();
				smttFileTransformer.stopAll();
			}

			// LoggingRegistry is never cleaned during the lifecycle of every
			// kettle
			// transformation and it is keeping reference to Trans object
			LoggingRegistry.getInstance().getMap().clear();
		}
	}

	/**
	 * Method that transforms EML files as a plain text file to xml file with
	 * specific format to be bulk uploaded to database.
	 * 
	 * @param kettleTransformationFileName
	 *            name to the kettle transformation file (.ktr)
	 * 
	 * @param fileToTransformContent
	 *            file to transform content
	 * 
	 * @return a list of byte array with the content of transformed files
	 * 
	 * @throws TransformationException
	 *             the transformation exception
	 */
	public EMLTransformationResultDTO executeEMLFilesTransformation(
			String kettleTransformationFileName, byte[] fileToTransformContent)
			throws TransformationException {

		File fileToTransform = new File(this.folderToExecuteTransformation,
				"emlFile.txt");

		File partyTransformedFile = new File(
				this.folderToExecuteTransformation, "party.xml");

		File candidateTransformedFile = new File(
				this.folderToExecuteTransformation, "candidate.xml");

		File eligibleOptionTransformedFile = new File(
				this.folderToExecuteTransformation, "eligibleOption.xml");

		File eligibleOptionRolTransformedFile = new File(
				this.folderToExecuteTransformation, "eligibleOptionRol.xml");

		File ballotOptionTransformedFile = new File(
				this.folderToExecuteTransformation, "ballotOption.xml");

		File kettleTransformationFile = new File(
				this.folderToExecuteTransformation,
				kettleTransformationFileName);

		try {

			FileUtils.writeByteArrayToFile(fileToTransform,
					fileToTransformContent);

			if (!KettleEnvironment.isInitialized()) {
				KettleEnvironment.init(false);
			}

			TransMeta transformationMetaDataForEMLFiles = new TransMeta(
					kettleTransformationFile.getAbsolutePath());

			Trans emlFileTransformer = new Trans(
					transformationMetaDataForEMLFiles);

			emlFileTransformer.setLogLevel(LogLevel.ERROR);

			emlFileTransformer.setParameterValue("pathInputFile",
					fileToTransform.getAbsolutePath());

			emlFileTransformer.setParameterValue(
					"OUTPUT_FILE_PARTY",
					StringUtils.substringBeforeLast(
							partyTransformedFile.getName(), "."));

			emlFileTransformer.setParameterValue(
					"OUTPUT_FILE_CANDIDATE",
					StringUtils.substringBeforeLast(
							candidateTransformedFile.getName(), "."));

			emlFileTransformer.setParameterValue("OUTPUT_FILE_ELIGIBLE_OPTION",
					StringUtils.substringBeforeLast(
							eligibleOptionTransformedFile.getName(), "."));

			emlFileTransformer.setParameterValue(
					"OUTPUT_FILE_ELIGIBLE_OPTION_ROL", StringUtils
							.substringBeforeLast(
									eligibleOptionRolTransformedFile.getName(),
									"."));

			emlFileTransformer.setParameterValue(
					"OUTPUT_FILE_BALLOT_OPTION",
					StringUtils.substringBeforeLast(
							ballotOptionTransformedFile.getName(), "."));

			emlFileTransformer.prepareExecution(null);
			emlFileTransformer.startThreads();
			emlFileTransformer.waitUntilFinished();

			String contestCustomCode = emlFileTransformer.getResult().getRows()
					.get(0).getData()[5].toString();
			emlFileTransformer.stopAll();
			emlFileTransformer.killAll();

			List<byte[]> resultFilesContent = new ArrayList<byte[]>();

			resultFilesContent.add(FileUtils
					.readFileToByteArray(partyTransformedFile));

			resultFilesContent.add(FileUtils
					.readFileToByteArray(candidateTransformedFile));

			resultFilesContent.add(FileUtils
					.readFileToByteArray(eligibleOptionTransformedFile));

			resultFilesContent.add(FileUtils
					.readFileToByteArray(eligibleOptionRolTransformedFile));

			resultFilesContent.add(FileUtils
					.readFileToByteArray(ballotOptionTransformedFile));

			EMLTransformationResultDTO codiTransformationResultDTO = new EMLTransformationResultDTO();

			codiTransformationResultDTO.setContestCustomCode(contestCustomCode);

			codiTransformationResultDTO
					.setResultFilesContent(resultFilesContent);

			return codiTransformationResultDTO;

		} catch (Exception e) {
			throw new TransformationException(
					e,
					TransformationExceptionInfo.UNEXPECTED_TRANSFORMATION_ERROR,
					"Error executing EML transformation");
		} finally {

			FileUtils.deleteQuietly(fileToTransform);
			FileUtils.deleteQuietly(partyTransformedFile);
			FileUtils.deleteQuietly(candidateTransformedFile);
			FileUtils.deleteQuietly(eligibleOptionTransformedFile);
			FileUtils.deleteQuietly(eligibleOptionRolTransformedFile);
			FileUtils.deleteQuietly(ballotOptionTransformedFile);

			// LoggingRegistry is never cleaned during the lifecycle of every
			// kettle transformation and it is keeping reference to Trans object
			LoggingRegistry.getInstance().getMap().clear();
		}
	}

	/**
	 * Method that transforms SITE files as a plain text file to xml file with
	 * specific format to be bulk uploaded to database.
	 * 
	 * @param kettleTransformationFileName
	 *            name to the kettle transformation file (.ktr)
	 * 
	 * @param fileToTransformContent
	 *            file to transform content
	 * 
	 * @return a map of name and byte array with the content of transformed
	 *         files
	 * 
	 * @throws TransformationException
	 *             signal that an exception has occurred executing the file
	 *             transformation
	 */
	public List<byte[]> executeSitesFilesTransformation(
			String kettleTransformationFileName, byte[] fileToTransformContent,
			boolean useRegionNisAsPollingPlaceCode)
			throws TransformationException {

		File fileToTransform = new File(this.folderToExecuteTransformation,
				"siteFile.txt");
		File pollingPlaceTransformedFile = new File(
				this.folderToExecuteTransformation, "pollingPlace.xml");
		File pollingStationTransformedFile = new File(
				this.folderToExecuteTransformation, "pollingStation.xml");
		File votingDeviceTransformedFile = new File(
				this.folderToExecuteTransformation, "votingDevice.xml");
		File votingBoothTransformedFile = new File(
				this.folderToExecuteTransformation, "votingBooth.xml");

		try {

			FileUtils.writeByteArrayToFile(fileToTransform,
					fileToTransformContent);

			if (!KettleEnvironment.isInitialized()) {
				KettleEnvironment.init(false);
			}

			File kettleTransformationFile = new File(
					this.folderToExecuteTransformation,
					kettleTransformationFileName);

			TransMeta transformationMetaDataForSiteFiles = new TransMeta(
					kettleTransformationFile.getAbsolutePath());

			Trans siteFileTransformer = new Trans(
					transformationMetaDataForSiteFiles);

			siteFileTransformer.setLogLevel(LogLevel.ERROR);

			siteFileTransformer
					.setParameterValue("OuputFilePollingPlace", StringUtils
							.substringBeforeLast(pollingPlaceTransformedFile
									.getAbsolutePath(), "."));
			siteFileTransformer.setParameterValue("OuputFilePollingStation",
					StringUtils.substringBeforeLast(
							pollingStationTransformedFile.getAbsolutePath(),
							"."));
			siteFileTransformer
					.setParameterValue("OuputFileVotingDevice", StringUtils
							.substringBeforeLast(votingDeviceTransformedFile
									.getAbsolutePath(), "."));
			siteFileTransformer.setParameterValue(
					"OuputFileVotingBooth",
					StringUtils.substringBeforeLast(
							votingBoothTransformedFile.getAbsolutePath(), "."));

			siteFileTransformer.setParameterValue("pathInputFile",
					fileToTransform.getAbsolutePath());

			siteFileTransformer.setParameterValue(
					"useRegionNisAsPollingPlaceCode",
					useRegionNisAsPollingPlaceCode ? "TRUE" : "FALSE");

			siteFileTransformer.prepareExecution(null);
			siteFileTransformer.startThreads();
			siteFileTransformer.waitUntilFinished();

			siteFileTransformer.stopAll();
			siteFileTransformer.killAll();
			siteFileTransformer.clearParameters();

			List<byte[]> resultFilesContent = new ArrayList<byte[]>();

			resultFilesContent.add(FileUtils
					.readFileToByteArray(pollingPlaceTransformedFile));

			resultFilesContent.add(FileUtils
					.readFileToByteArray(pollingStationTransformedFile));

			resultFilesContent.add(FileUtils
					.readFileToByteArray(votingDeviceTransformedFile));

			resultFilesContent.add(FileUtils
					.readFileToByteArray(votingBoothTransformedFile));

			return resultFilesContent;
		} catch (Exception e) {
			throw new TransformationException(
					e,
					TransformationExceptionInfo.UNEXPECTED_TRANSFORMATION_ERROR,
					StringUtils.defaultString(e.getMessage()));
		} finally {

			FileUtils.deleteQuietly(fileToTransform);
			FileUtils.deleteQuietly(pollingPlaceTransformedFile);
			FileUtils.deleteQuietly(pollingStationTransformedFile);
			FileUtils.deleteQuietly(votingDeviceTransformedFile);
			FileUtils.deleteQuietly(votingBoothTransformedFile);

			// LoggingRegistry is never cleaned during the lifecycle of every
			// kettle transformation and it is keeping reference to Trans object
			LoggingRegistry.getInstance().getMap().clear();
		}
	}


    /**
     * Detects any transformation error generated during Kettle Execution.
     *
     * @param error
     *      An error message decorated by Kettle
     *
     * @throws TransformationException
     *      if any handled error is detected
     */
    private void detectTransformationErrors(String error)
    throws TransformationException {
        final String jsErrorText      = "$JS_ERROR";
        final String missingFieldText = "Missing required fields";

        if (error.contains(jsErrorText)) {
            this.parseJavascriptError(error);

        } else if (error.contains(missingFieldText)) {
            throw new TransformationException(MISSING_REQUIRED_FIELDS_ERROR);
        }
    }


    /**
     * Detects any transformation error generated during Kettle Execution.
     *
     * @param trans
     *      {@code Trans} instance
     *
     * @throws IOException
     *      if any {@code IOException} happens
     *
     * @throws TransformationException
     *      if any transformation error is detected
     */
    private void detectTransformationErrors(Trans trans)
    throws IOException, TransformationException {
        BufferedReader reader;          // Errors Reader
        Log4jBufferAppender appender;   // Log4jBufferAppender
        String channelId;               // Log Channel ID
        String errorMsgs;               // Error messages
        String line;                    // Error message as a line

        channelId = trans.getLogChannelId();
        appender  = CentralLogStore.getAppender();

        try {
            if (trans.getErrors() > 0) {
                errorMsgs = appender
                                .getBuffer(channelId, false)
                                .toString();

                reader = new BufferedReader( new StringReader(errorMsgs) );
                line   = reader.readLine();

                do {
                    this.detectTransformationErrors(line);
                    line = reader.readLine();

                } while (line != null);
            }

        } finally {
            appender.removeChannelFromBuffer(channelId);
        }
    }


    /**
     * Parses a Kettle Error generated from a Javascript Step using
     * a specific communication syntax.
     *
     * @param error
     *      An error message decorated by Kettle
     *
     * @throws TransformationException
     *      To notify the kind of error detected
     */
    private void parseJavascriptError(String error)
    throws TransformationException {
        final String tag = "$JS_ERROR";     // Javascript Error Tag

        int index;                  // tag index in error String
        Map<String,String> ctxt;    // Contains Context information
        String[] subItems;          // Context components
        String[] items;             // Exception's details
        String msg;                 // Error message itself (no decorations!)
        TransformationExceptionInfo exInfo;         // Type of Failure

        index  = error.indexOf(tag);
        msg    = error.substring(index).trim();
        items  = msg.split("\\|");
        exInfo = TransformationExceptionInfo.valueOf(items[1]);
        ctxt   = new HashMap<String, String>();

        // NOTE: the last item is just noise; that's why we're skipping it! ...
        for (int i = 2; i < items.length - 1; i++) {
            subItems = items[i].split("=");
            ctxt.put(subItems[0], subItems[1]);
        }
        throw new TransformationException(exInfo, ctxt);
    }
}
