/**
 * ECMWorkflowServiceInvoker.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.workflow;

import java.io.Serializable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.service.lang.ElectoralPhysicalLocationsRegistryException;
import com.smartmatic.waffle.ecm.service.spec.IECMDemographyService;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralOrganizationService;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralPhysicalLocationsRegistryService;
import com.smartmatic.waffle.ecm.service.spec.IECMService;
import com.smartmatic.waffle.ecm.service.spec.IECMUtilitiesService;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to execute all conditions needed between each wokflow step.
 */
public class ECMWorkflowServiceInvoker implements Serializable {

	/**
	 * Checks that all the tables needed to generate the Master USB are not
	 * empty.
	 * 
	 * @return the status of validation.
	 */
	public ECMWorkflowStatus checkIfDataIsLoaded() {

		updateECMUtilitiesService();

		try {
			if (this.ecmUtilitiesService.checkIfDataIsLoaded()) {
				return new ECMWorkflowStatus();
			} else {
				return new ECMWorkflowStatus()
						.setValidationsFailed(new ECMWorkflowStatus.Validation[] { ECMWorkflowStatus.Validation.DATA_LOADED });
			}
		} catch (BaseException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			return null;
		}
	}

	/***
	 * Method that check if contest, contest class and contest group exist in
	 * database.
	 * 
	 * @return the status of validation.
	 */
	public ECMWorkflowStatus integrityValidationContesContestClassContestGroup() {

		updateECMServiceReference();

		try {
			if (this.ecmService
					.integrityValidationContesContestClassContestGroup()) {
				return new ECMWorkflowStatus();
			} else {
				return new ECMWorkflowStatus()
						.setValidationsFailed(new ECMWorkflowStatus.Validation[] { ECMWorkflowStatus.Validation.INTEGRITY_VALIDATION_CONTEST_CONTESTCLASS_CONTESTGROUP });
			}
		} catch (BaseException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			return null;
		}
	}

	/**
	 * Method to check if all certificates were generated.
	 * 
	 * @return the status of validation.
	 */
	public ECMWorkflowStatus checkIfCertificatesIsGenerated() {

		updateECMUtilitiesService();

		try {
			if (this.ecmUtilitiesService
					.checkIfSecurityComponentsAreGenerated()) {
				return new ECMWorkflowStatus();
			} else {
				return new ECMWorkflowStatus()
						.setValidationsFailed(new ECMWorkflowStatus.Validation[] { ECMWorkflowStatus.Validation.CERTIFICATES_GENERATED });
			}
		} catch (ElectoralPhysicalLocationsRegistryException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			return null;
		}
	}

	/**
	 * Method that deletes programming unit, voting device and electoral
	 * authority keys.
	 */
	public void deleteKeys() {

		updateECMDemographyServiceReference();
		updateECMElectoralOrganizationService();
		updateECMElectoralPhysicalLocationsRegistryServiceReference();

		try {
			this.ecmElectoralPhysicalLocationsRegistryService
					.deleteVotingDeviceSecurityComponents();
			this.ecmDemographyService.deleteProgrammingUnitSecurityComponents();
			this.ecmElectoralOrganizationService
					.deleteElectoralAuthoritiesSecurityComponents();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Method used to delete the elements generated in post upload process.
	 */
	public void deletePostGeneratedElements() {
		updateECMUtilitiesService();
		try {
			this.ecmUtilitiesService.truncateLogicalVotingGroup();
			this.ecmUtilitiesService.truncateContestLanguagesPair();
			this.ecmUtilitiesService.deleteReportFiles();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Method that updates the reference to IECMUtilitiesService
	 */
	private void updateECMServiceReference() {

		if (this.ecmService == null) {
			try {

				Context ctx = new InitialContext();

				Object reference = ctx.lookup("com.smartmatic.waffle.ecm/"
						+ IECMService.SERVICE_NAME + "/local");
				this.ecmService = (IECMService) PortableRemoteObject.narrow(
						reference, IECMService.class);
			} catch (NamingException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
			}
		}
	}

	/**
	 * Method that updates the reference to
	 * IECMElectoralPhysicalLocationsRegistryService
	 */
	private void updateECMElectoralPhysicalLocationsRegistryServiceReference() {
		if (this.ecmElectoralPhysicalLocationsRegistryService == null) {
			try {

				Context ctx = new InitialContext();

				Object reference = ctx
						.lookup("com.smartmatic.waffle.ecm/"
								+ IECMElectoralPhysicalLocationsRegistryService.SERVICE_NAME
								+ "/local");
				this.ecmElectoralPhysicalLocationsRegistryService = (IECMElectoralPhysicalLocationsRegistryService) PortableRemoteObject
						.narrow(reference,
								IECMElectoralPhysicalLocationsRegistryService.class);
			} catch (NamingException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
			}
		}
	}

	/**
	 * Method that updates the reference to IECMUtilitiesService
	 */
	private void updateECMUtilitiesService() {
		if (this.ecmUtilitiesService == null) {
			try {

				Context ctx = new InitialContext();

				Object reference = ctx.lookup("com.smartmatic.waffle.ecm/"
						+ IECMUtilitiesService.SERVICE_NAME + "/local");
				this.ecmUtilitiesService = (IECMUtilitiesService) PortableRemoteObject
						.narrow(reference, IECMUtilitiesService.class);
			} catch (NamingException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
			}
		}
	}

	/**
	 * Method that updates the reference to IECMDemographyService
	 */
	private void updateECMDemographyServiceReference() {
		if (this.ecmDemographyService == null) {
			try {

				Context ctx = new InitialContext();

				Object reference = ctx.lookup("com.smartmatic.waffle.ecm/"
						+ IECMDemographyService.SERVICE_NAME + "/local");
				this.ecmDemographyService = (IECMDemographyService) PortableRemoteObject
						.narrow(reference, IECMDemographyService.class);
			} catch (NamingException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
			}
		}
	}

	/**
	 * Method that updates the reference to IECMElectoralOrganizationService
	 */
	private void updateECMElectoralOrganizationService() {
		if (this.ecmElectoralOrganizationService == null) {
			try {

				Context ctx = new InitialContext();

				Object reference = ctx.lookup("com.smartmatic.waffle.ecm/"
						+ IECMElectoralOrganizationService.SERVICE_NAME
						+ "/local");
				this.ecmElectoralOrganizationService = (IECMElectoralOrganizationService) PortableRemoteObject
						.narrow(reference,
								IECMElectoralOrganizationService.class);
			} catch (NamingException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
			}
		}
	}

	/**
	 * For logging purposes.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ECMWorkflowServiceInvoker.class);

	/**
	 * Instance of IECMService
	 */
	private transient IECMService ecmService;

	/**
	 * Instance of IECMService
	 */
	private transient IECMDemographyService ecmDemographyService;

	/**
	 * Instance of IECMService
	 */
	private transient IECMElectoralPhysicalLocationsRegistryService ecmElectoralPhysicalLocationsRegistryService;

	/**
	 * Instance of IECMService
	 */
	private transient IECMUtilitiesService ecmUtilitiesService;

	/**
	 * Instance of IECMService
	 */
	private transient IECMElectoralOrganizationService ecmElectoralOrganizationService;

	/**
	 * Serialization code
	 */
	private static final long serialVersionUID = 5855198483804011381L;
}