package com.smartmatic.waffle.ecm.service.impl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.sql.DataSource;

import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.mockito.Mockito;
import org.powermock.api.mockito.PowerMockito;

import com.smartmatic.waffle.domain.contest.ContestClassData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.LogicalVotingGroupData;
import com.smartmatic.waffle.domain.user.IUser;
import com.smartmatic.waffle.ecm.service.spec.IECMDemographyServiceLocal;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

public class ECMElectoralDemographyServiceImplTest {

	/** Instance of ECMDemographyServiceImpl. */
	ECMElectoralDemographyServiceImpl ecmElectoralDemographyServiceImpl;

	/**
	 * SetUp method to instantiate ECMDemographyServiceImpl.
	 * 
	 * @throws Exception
	 *             signal that an exception occurs setting up the test
	 */
	@Before
	public void setUp() throws Exception {
		this.ecmElectoralDemographyServiceImpl = new ECMElectoralDemographyServiceImpl();
	}

	@Test
	public void testGetContestByDistrict() throws Exception {
		// arrange
		IPersistenceServiceLocal persistenceServiceLocal = mockPersistenceServiceLocalToGetListOfContestAndContestGroups();

		this.ecmElectoralDemographyServiceImpl
				.setPersistenceService(persistenceServiceLocal);

		Set<DistrictData> districtDataSet = new HashSet<DistrictData>();

		// act
		List<ContestData> contestListOut = this.ecmElectoralDemographyServiceImpl
				.getContestByDistrict(districtDataSet);

		// assert
		assertEquals(3, contestListOut.size());
		PowerMockito.verifyPrivate(persistenceServiceLocal).invoke(
				"searchObjects", Mockito.any(SearchExpression.class),
				Mockito.any(String[].class), Mockito.any(IUser.class));
	}

	@Test
	public void testGetContestsByLastLevelRegions() throws Exception {
		// arrange
		IPersistenceServiceLocal persistenceServiceLocal = mockPersistenceServiceLocalToGetListOfContestAndContestGroups();

		IECMDemographyServiceLocal demographyService = mockDemographyService();

		this.ecmElectoralDemographyServiceImpl
				.setPersistenceService(persistenceServiceLocal);

		this.ecmElectoralDemographyServiceImpl
				.setDemographyService(demographyService);

		List<Long> regionCodes = new ArrayList<Long>();

		// act
		List<ContestData> contestSetOut = this.ecmElectoralDemographyServiceImpl
				.getContestsByLastLevelRegions(regionCodes);

		// assert
		assertEquals(3, contestSetOut.size());
		PowerMockito.verifyPrivate(persistenceServiceLocal).invoke(
				"searchObjects", Mockito.any(SearchExpression.class),
				Mockito.any(String[].class), Mockito.any(IUser.class));
		PowerMockito.verifyPrivate(demographyService).invoke(
				"getRegionsWithParentsByRegionCodes", Mockito.any(List.class));
	}

	@Test
	public void testGetContestGroupByLastLevelRegion() throws Exception {
		// arrange
		IPersistenceServiceLocal persistenceServiceLocal = mockPersistenceServiceLocalToGetListOfContestGroupByLastLevelRegion();

		this.ecmElectoralDemographyServiceImpl
				.setPersistenceService(persistenceServiceLocal);

		RegionData lastLevelRegion = new RegionData();

		lastLevelRegion.setAllParents(new HashSet<RegionData>());

		// act
		List<ContestGroupData> contestGroupListOut = this.ecmElectoralDemographyServiceImpl
				.getContestGroupByLastLevelRegion(lastLevelRegion);
		// assert
		assertEquals(3, contestGroupListOut.size());
		PowerMockito.verifyPrivate(persistenceServiceLocal).invoke(
				"searchObjects", Mockito.any(SearchExpression.class),
				Mockito.any(String[].class), Mockito.any(IUser.class));
	}

	@Test
	public void testGetContestGroupLoadedInDatabase() throws Exception {
		// arrange
		IPersistenceServiceLocal persistenceServiceLocal = mockPersistenceServiceLocalToGetListOfContestGroupByLastLevelRegion();

		this.ecmElectoralDemographyServiceImpl
				.setPersistenceService(persistenceServiceLocal);

		// act
		List<ContestGroupData> contestGroupListOut = this.ecmElectoralDemographyServiceImpl
				.getContestGroupsLoadedInDatabase();

		// assert
		assertEquals(3, contestGroupListOut.size());
		PowerMockito.verifyPrivate(persistenceServiceLocal).invoke(
				"searchObjects", Mockito.any(SearchExpression.class),
				Mockito.any(String[].class), Mockito.any(IUser.class));
	}

	@Test
	public void testMakeContestsSpecificContestGroups() throws Exception {

		// arrange
		IPersistenceServiceLocal persistenceServiceLocal = mockPersistenceServiceLocalToGetListOfContestGroupByLastLevelRegion();

		this.ecmElectoralDemographyServiceImpl
				.setPersistenceService(persistenceServiceLocal);

		List<ContestData> contestList = CommonTestMethods
				.getContestLoadedInDatabase();

		RegionData lastLevelRegion = new RegionData();

		lastLevelRegion.setAllParents(new HashSet<RegionData>());

		// act
		List<ContestGroupData> contestGroupResult = this.ecmElectoralDemographyServiceImpl
				.makeRegionSpecificContestGroups(lastLevelRegion, contestList,
						1);

		Set<ContestGroupData> contestGroupResultSet = new HashSet<ContestGroupData>(
				contestGroupResult);

		List<Long> contestGroupCodesList = new ArrayList<Long>();

		List<ContestData> contestsListFromResult = new ArrayList<ContestData>();

		for (ContestGroupData contestGroup : contestGroupResultSet) {
			contestGroupCodesList.add(contestGroup.getCode());
			contestsListFromResult.addAll(contestGroup.getContests());
		}

		Set<Long> contestGroupCodesSet = new HashSet<Long>(
				contestGroupCodesList);

		// assert

		// All the contest were processed and included in one contest group.
		for (ContestData contest : contestList) {
			assertTrue(contestsListFromResult.contains(contest));
		}
		// if no duplicates, size for both should be equals.
		assertEquals(contestGroupCodesList.size(), contestGroupCodesSet.size());

	}

	@Ignore
	@Test
	public void testMakeRegionSpecificContestGroups() throws Exception {

		// arrange
		IPersistenceServiceLocal persistenceServiceLocal = mockPersistenceServiceLocalToGetListOfContestGroupByLastLevelRegion();

		this.ecmElectoralDemographyServiceImpl
				.setPersistenceService(persistenceServiceLocal);

		List<ContestData> contestList = CommonTestMethods
				.getContestLoadedInDatabase();

		RegionData lastLevelRegion = new RegionData();

		lastLevelRegion.setAllParents(new HashSet<RegionData>());

		// act
		List<ContestGroupData> contestGroupResult = this.ecmElectoralDemographyServiceImpl
				.makeRegionSpecificContestGroups(lastLevelRegion, contestList,
						1);

		Set<ContestGroupData> contestGroupResultSet = new HashSet<ContestGroupData>(
				contestGroupResult);

		List<Long> contestGroupCodesList = new ArrayList<Long>();

		List<ContestData> contestsListFromResult = new ArrayList<ContestData>();

		for (ContestGroupData contestGroup : contestGroupResultSet) {
			contestGroupCodesList.add(contestGroup.getCode());
			contestsListFromResult.addAll(contestGroup.getContests());
		}

		Set<Long> contestGroupCodesSet = new HashSet<Long>(
				contestGroupCodesList);

		// assert

		// All the contest were processed and included in one contest group.
		for (ContestData contest : contestList) {
			assertTrue(contestsListFromResult.contains(contest));
		}
		// if no duplicates, size for both should be equals.
		assertEquals(contestGroupCodesList.size(), contestGroupCodesSet.size());

	}

	@Test
	public void testGetContestByCode() throws Exception {
		// arrange
		IPersistenceServiceLocal persistenceServiceLocal = PowerMockito
				.mock(IPersistenceServiceLocal.class);

		ContestData contest = new ContestData();

		PowerMockito.doReturn(contest).when(persistenceServiceLocal,
				"searchObject", Mockito.any(SearchExpression.class),
				Mockito.any(IUser.class));

		this.ecmElectoralDemographyServiceImpl
				.setPersistenceService(persistenceServiceLocal);

		// act
		ContestData contestResult = this.ecmElectoralDemographyServiceImpl
				.getContestByCode(0);

		// assert
		assertEquals(contest, contestResult);
	}

	@Test
	public void testGetLogicalVotingGroup() throws Exception {
		// arrange
		IPersistenceServiceLocal persistenceServiceLocal = PowerMockito
				.mock(IPersistenceServiceLocal.class);

		LogicalVotingGroupData logicalVotingGroup = new LogicalVotingGroupData();

		PowerMockito.doReturn(logicalVotingGroup).when(persistenceServiceLocal,
				"searchObject", Mockito.any(SearchExpression.class),
				Mockito.any(IUser.class));

		this.ecmElectoralDemographyServiceImpl
				.setPersistenceService(persistenceServiceLocal);

		// act
		LogicalVotingGroupData logicalVotingGroupResult = this.ecmElectoralDemographyServiceImpl
				.getLogicalVotingGroup("votingGroup");

		// assert
		assertEquals(logicalVotingGroup, logicalVotingGroupResult);

	}

	@Test
	public void testGetLogicalVotingGroupByContestGroup() throws Exception {
		// arrange
		IPersistenceServiceLocal persistenceServiceLocal = PowerMockito
				.mock(IPersistenceServiceLocal.class);

		List<LogicalVotingGroupData> logicalVotingGroupList = new ArrayList<LogicalVotingGroupData>();

		PowerMockito.doReturn(logicalVotingGroupList).when(
				persistenceServiceLocal, "searchObject",
				Mockito.any(SearchExpression.class), Mockito.any(IUser.class));

		this.ecmElectoralDemographyServiceImpl
				.setPersistenceService(persistenceServiceLocal);

		// act
		List<LogicalVotingGroupData> logicalVotingGroupListResult = this.ecmElectoralDemographyServiceImpl
				.getLogicalVotingGroupByContestGroup("code-01", "contestGroup");

		// assert
		assertEquals(logicalVotingGroupList, logicalVotingGroupListResult);

	}

	private List<ContestGroupData> getRegionContestGroup() {

		List<ContestGroupData> contestGroupList = new ArrayList<ContestGroupData>();

		File contestGroupMainFolder = new File(
				"test/resources/region.contest.groups");

		XStream xstream = new XStream(new DomDriver("UTF-8"));

		for (int i = 1; i <= 3; i++) {
			File contestGroupFolder = new File(contestGroupMainFolder,
					String.valueOf(i));

			String folderIndex = contestGroupFolder.getName();

			File contestGroupFile = new File(contestGroupFolder,
					"ContestGroup_" + folderIndex + ".xml");

			xstream.processAnnotations(ContestGroupData.class);

			ContestGroupData contestGroup = (ContestGroupData) xstream
					.fromXML(contestGroupFile);

			File contestFolder = new File(contestGroupFolder, "contests");

			File[] contestList = contestFolder.listFiles();

			Set<ContestData> contestSet = new HashSet<ContestData>();

			for (File contestFile : contestList) {
				xstream.processAnnotations(ContestData.class);

				ContestData contest = (ContestData) xstream
						.fromXML(contestFile);

				contestSet.add(contest);
			}

			contestGroup.setContests(contestSet);

			contestGroupList.add(contestGroup);
		}

		return contestGroupList;
	}

	private IECMDemographyServiceLocal mockDemographyService() throws Exception {
		IECMDemographyServiceLocal demographyService = PowerMockito
				.mock(IECMDemographyServiceLocal.class);

		List<RegionData> lastLevelRegions = new ArrayList<RegionData>();

		RegionData region1 = new RegionData();

		Set<RegionData> regionParentsSet = new HashSet<RegionData>();

		Set<DistrictData> districts = new HashSet<DistrictData>();

		DistrictData district1 = new DistrictData();

		districts.add(district1);

		DistrictData district2 = new DistrictData();

		districts.add(district2);

		RegionData regionParent1 = new RegionData();
		regionParent1.setDistricts(districts);

		regionParentsSet.add(regionParent1);
		RegionData regionParent2 = new RegionData();
		regionParentsSet.add(regionParent2);

		region1.setAllParents(regionParentsSet);

		lastLevelRegions.add(region1);

		PowerMockito.doReturn(lastLevelRegions).when(demographyService,
				"getRegionsWithParentsByRegionCodes", Mockito.any(List.class));

		return demographyService;
	}

	@Test
	public void testGetContestForLastLevelRegion() throws Exception {
		// arrange
		DataSource dataSource = PowerMockito.mock(DataSource.class);
		Connection connection = PowerMockito.mock(Connection.class);
		PreparedStatement prepareStatement = PowerMockito
				.mock(PreparedStatement.class);
		ResultSet resultSet = PowerMockito.mock(ResultSet.class);

		this.ecmElectoralDemographyServiceImpl.setDataSource(dataSource);

		PowerMockito.doReturn(connection).when(dataSource, "getConnection");
		PowerMockito.doReturn(prepareStatement).when(connection,
				"prepareStatement", Mockito.any(String.class));
		PowerMockito.doReturn(resultSet).when(prepareStatement, "executeQuery");

		Mockito.when(resultSet.next()).thenReturn(true).thenReturn(false);

		Mockito.when(resultSet.getLong(Mockito.anyString())).thenReturn(1L)
				.thenReturn(2L).thenReturn(3L).thenReturn(4L);
		Mockito.when(resultSet.getString(Mockito.anyString()))
				.thenReturn("ABC").thenReturn("DEF").thenReturn("GHI");

		// act
		Map<Long, ContestGroupData> contestGroupMap = this.ecmElectoralDemographyServiceImpl
				.getContestsForLastLevelRegion(1L);

		// assert
		assertEquals(false, contestGroupMap.isEmpty());
		assertEquals(true, contestGroupMap.containsKey(1L));
		assertEquals(1L, contestGroupMap.get(1L).getCode());
	}

	// @Test(expected = ElectoralDemographyException.class)
	public void testGetContestForLastLevelRegionException() throws Exception {
		// arrange
		DataSource dataSource = PowerMockito.mock(DataSource.class);
		Connection connection = PowerMockito.mock(Connection.class);
		PreparedStatement prepareStatement = PowerMockito
				.mock(PreparedStatement.class);

		this.ecmElectoralDemographyServiceImpl.setDataSource(dataSource);

		PowerMockito.doReturn(connection).when(dataSource, "getConnection");
		PowerMockito.doReturn(prepareStatement).when(connection,
				"prepareStatement", Mockito.any(String.class));

		Mockito.when(prepareStatement.executeQuery()).thenThrow(
				new SQLException());

		// act
		this.ecmElectoralDemographyServiceImpl
				.getContestsForLastLevelRegion(1L);
	}

	@Test
	public void testGetContestForLastLevelRegionResultSetNull()
			throws Exception {
		// arrange
		DataSource dataSource = PowerMockito.mock(DataSource.class);
		Connection connection = PowerMockito.mock(Connection.class);
		PreparedStatement prepareStatement = PowerMockito
				.mock(PreparedStatement.class);

		this.ecmElectoralDemographyServiceImpl.setDataSource(dataSource);

		PowerMockito.doReturn(connection).when(dataSource, "getConnection");
		PowerMockito.doReturn(prepareStatement).when(connection,
				"prepareStatement", Mockito.any(String.class));

		// act
		Map<Long, ContestGroupData> contestGroupMap = this.ecmElectoralDemographyServiceImpl
				.getContestsForLastLevelRegion(1L);

		// assert
		assertEquals(true, contestGroupMap.isEmpty());
	}

	@Test
	public void testBuildSpecificContestGroupNullContestGroupMap()
			throws Exception {

		// act
		this.ecmElectoralDemographyServiceImpl.buildSpecificContestGroup(null,
				1L);
	}

	@Test
	public void testBuildSpecificContestGroupEmptyContestGroupMap()
			throws Exception {

		// act
		this.ecmElectoralDemographyServiceImpl.buildSpecificContestGroup(
				new HashMap<Long, ContestGroupData>(), 1L);
	}

	@Test
	public void testBuildSpecificContestGroupContestGroupMap() throws Exception {
		// act
		this.ecmElectoralDemographyServiceImpl.buildSpecificContestGroup(
				generateContestGroupMap(), 1L);
	}

	private Map<Long, ContestGroupData> generateContestGroupMap() {
		Map<Long, ContestGroupData> contestGroupMap = new HashMap<Long, ContestGroupData>();

		ContestGroupData contestGroupPivot = new ContestGroupData();
		contestGroupPivot.setCode(1L);
		contestGroupPivot.setCustomCode("ABC");

		ContestGroupData contestGroup = contestGroupMap.get(contestGroupPivot
				.getCode());

		if (contestGroup == null) {
			contestGroupPivot.setContests(new HashSet<ContestData>());
		} else {
			contestGroupPivot.setContests(contestGroup.getContests());
		}

		ContestData contest = new ContestData();
		contest.setCode(2L);
		contest.setCustomCode("DEF");
		contest.setContestClassCode(3L);

		long contestLinkCode = 4L;

		if (contestLinkCode > 0) {
			ContestData contestLink = new ContestData();
			contestLink.setCode(contestLinkCode);
			contestLink.setCustomCode("GHI");
			contest.setContestLink(contestLink);
		}

		ContestClassData contestClass = new ContestClassData();
		contestClass.setCode(5L);
		contest.setContestClass(contestClass);

		contestGroupPivot.getContests().add(contest);
		contestGroupMap.put(contestGroupPivot.getCode(), contestGroupPivot);

		return contestGroupMap;
	}

	private IPersistenceServiceLocal mockPersistenceServiceLocalToGetListOfContest()
			throws Exception {
		IPersistenceServiceLocal persistenceServiceLocal = PowerMockito
				.mock(IPersistenceServiceLocal.class);

		List<ContestData> contestList = new ArrayList<ContestData>();

		ContestData contest1 = new ContestData();

		contest1.setCode(1);
		contest1.setName("contest1");
		contest1.setCustomCode("contest1");

		ContestData contest2 = new ContestData();

		contest2.setCode(2);
		contest2.setName("contest2");
		contest2.setCustomCode("contest2");

		ContestData contest3 = new ContestData();

		contest3.setCode(3);
		contest3.setName("contest3");
		contest3.setCustomCode("contest3");

		contestList.add(contest1);
		contestList.add(contest2);
		contestList.add(contest3);

		PowerMockito.doReturn(contestList).when(persistenceServiceLocal,
				"searchObjects", Mockito.any(SearchExpression.class),
				Mockito.any(IUser.class));

		return persistenceServiceLocal;
	}

	private IPersistenceServiceLocal mockPersistenceServiceLocalToGetListOfContestAndContestGroups()
			throws Exception {
		IPersistenceServiceLocal persistenceServiceLocal = PowerMockito
				.mock(IPersistenceServiceLocal.class);

		List<ContestData> contestList = new ArrayList<ContestData>();

		ContestData contest1 = new ContestData();

		contest1.setCode(1);
		contest1.setName("contest1");
		contest1.setCustomCode("contest1");

		ContestData contest2 = new ContestData();

		contest2.setCode(2);
		contest2.setName("contest2");
		contest2.setCustomCode("contest2");

		ContestData contest3 = new ContestData();

		contest3.setCode(3);
		contest3.setName("contest3");
		contest3.setCustomCode("contest3");

		contestList.add(contest1);
		contestList.add(contest2);
		contestList.add(contest3);

		PowerMockito.doReturn(contestList).when(persistenceServiceLocal,
				"searchObjects", Mockito.any(SearchExpression.class),
				Mockito.any(String[].class), Mockito.any(IUser.class));

		return persistenceServiceLocal;
	}

	private IPersistenceServiceLocal mockPersistenceServiceLocalToGetListOfContestGroupByLastLevelRegion()
			throws Exception {
		IPersistenceServiceLocal persistenceServiceLocal = PowerMockito
				.mock(IPersistenceServiceLocal.class);

		PowerMockito.doReturn(getRegionContestGroup()).when(
				persistenceServiceLocal, "searchObjects",
				Mockito.any(SearchExpression.class),
				Mockito.any(String[].class), Mockito.any(IUser.class));

		return persistenceServiceLocal;
	}
}
