/**
 * DotDashBorder.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.ui.util;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.border.AbstractBorder;

/**
 * DotDashBorder is a <CODE>java.swing.border.Border</CODE> implementation that
 * draws a dotted and/or dashed line border. The dot/dash pattern is defined as
 * an array of ints, with every two indices defining the number of pixels for
 * each dot/dash and space. E.g.: <BR>
 * { 1, 1 } == 1-pixel-dot, 1-pixel-space, 1-pixel-dot, 1-pixel-space, etc.<BR>
 * { 4, 2 } == 4-pixel-dash, 2-pixel-space, 4-pixel-dash, 2-pixel-space, etc.<BR>
 */
public class DotDashBorder extends AbstractBorder {

	public enum PatternType {
		/**
		 * Defines a dotted line pattern:
		 * 
		 * <PRE>
		 * . . . .
		 * </PRE>
		 */
		DOT(new int[] { 1, 1 }),
		/**
		 * Defines a dashed line pattern:
		 * 
		 * <PRE>
		 * - - - -
		 * </PRE>
		 */
		DASH_SHORT(new int[] { 4, 4 }),
		/**
		 * Defines a long dashed line pattern:
		 * 
		 * <PRE>
		 * ---   ---   ---
		 * </PRE>
		 */
		DASH_LONG(new int[] { 10, 10 }),
		/**
		 * Defines a long/short dashed line pattern:
		 * 
		 * <PRE>
		 * -- - -- - -- -
		 * </PRE>
		 */
		DASH_LONG_SHORT(new int[] { 6, 3, 3, 3 }),
		/**
		 * Defines a long/short/short dashed line pattern:
		 * 
		 * <PRE>
		 * -- - - -- - - -- - -
		 * </PRE>
		 */
		DASH_LONG_SHORT_SHORT(new int[] { 6, 3, 3, 3, 3, 3 });

		private final int[] pattern;

		private PatternType(int[] pattern) {
			this.pattern = pattern;
		}

		public int[] getPattern() {
			return this.pattern;
		}

	}

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -6130602580428124308L;

	/**
	 * The dot/dash pattern.
	 */
	private int[] pattern = PatternType.DOT.getPattern();

	/** The border thickness. */
	private int thickness = 1;

	/**
	 * The foreground color. If not specified, the component's foreground color
	 * is used.
	 */
	private Color colorFG = null;

	/**
	 * The background color. If not specified, the component's background color
	 * is used.
	 */
	private Color colorBG = null;

	/**
	 * Thicken border pattern relative to thickness flag.
	 */
	private boolean thicken = false;

	/**
	 * Create a new 1-pixel thick DotDashBorder using the component's background
	 * color.
	 * 
	 * @param pattern
	 *            int[]: the dot/dash-space pattern
	 * @param fg
	 *            Color: the foreground (dot/dash) color
	 */
	public DotDashBorder(int[] pattern, Color fg) {
		this(pattern, 1, fg, null);
	}

	/**
	 * Create a new DotDashBorder using the component's foreground and
	 * background colors.
	 * 
	 * @param pattern
	 *            int[]: the dot/dash-space pattern
	 * @param thickness
	 *            int: the border thickness
	 */
	public DotDashBorder(int[] pattern, int thickness) {
		this(pattern, thickness, null, null);
	}

	/**
	 * Create a new DotDashBorder using the component's background color.
	 * 
	 * @param pattern
	 *            int[]: the dot/dash-space pattern
	 * @param thickness
	 *            int: the border thickness
	 * @param fg
	 *            Color: the foreground (dot/dash) color
	 */
	public DotDashBorder(int[] pattern, int thickness, Color fg) {
		this(pattern, thickness, fg, null);
	}

	/**
	 * Create a new DotDashBorder.
	 * 
	 * @param parameter
	 *            int[]: the dot/dash-space pattern
	 * @param thickness
	 *            int: the border thickness
	 * @param fg
	 *            Color: the foreground (dot/dash) color
	 * @param bg
	 *            Color: the background (space) color
	 */
	public DotDashBorder(int[] parameter, int thickness, Color fg, Color bg) {
		for (int i = 0; i < parameter.length; i++) {
			if (parameter[i] <= 0) {
				throw new IllegalArgumentException(
						"Pattern cannot have values <= 0.");
			}
		}
		this.pattern = (parameter != null) ? parameter.clone() : null;
		if (thickness <= 0) {
			throw new IllegalArgumentException("Thickness cannot be <= 0.");
		}
		this.thickness = thickness;
		this.colorFG = fg;
		this.colorBG = bg;
	}

	/**
	 * Get the insets for the border.
	 * 
	 * @param c
	 *            the c
	 * @return Insets: the insets for the border
	 * @see #getBorderInsets(Component, Insets)
	 */
	@Override
	public Insets getBorderInsets(Component c) {
		return new Insets(this.thickness, this.thickness, this.thickness,
				this.thickness);
	}

	/**
	 * Get the insets for the border.
	 * 
	 * @param c
	 *            the c
	 * @param insets
	 *            Insets: the insets
	 * @return Insets: the insets for the border
	 * @see #getBorderInsets(Component)
	 */
	@Override
	public Insets getBorderInsets(Component c, Insets insets) {
		return new Insets(this.thickness, this.thickness, this.thickness,
				this.thickness);
	}

	/**
	 * Check if the border is opaque.
	 * 
	 * @return boolean: true if the border is opaque
	 */
	@Override
	public boolean isBorderOpaque() {
		return true;
	}

	/**
	 * Check if pattern will be thickened by thickness.
	 * 
	 * @return boolean: true if pattern will be thickened by thickness
	 * @see #setThickenPattern(boolean)
	 */
	public boolean isThickenPattern() {
		return this.thicken;
	}

	/**
	 * Set if the pattern should be thickened by thickness. If true, a pattern
	 * of { 1, 1 } and thickness of 5, the pattern would be expanded to { 5, 5
	 * }. This allows setting a pattern based on small pixel measurements that
	 * grows proportionally with the thickness.
	 * 
	 * @param t
	 *            the new thicken pattern
	 * @see #isThickenPattern()
	 */
	public void setThickenPattern(boolean t) {
		this.thicken = t;
	}

	/**
	 * Paint the border.
	 * 
	 * @param c
	 *            the c
	 * @param g
	 *            the g
	 * @param x
	 *            int: the border y position
	 * @param y
	 *            int: the border x position
	 * @param width
	 *            int: the border width
	 * @param height
	 *            int: the border height
	 */
	@Override
	public void paintBorder(Component c, Graphics g, int x, int y, int width,
			int height) {
		Color colorFGX = c.getBackground();
		if (this.colorFG != null) {
			colorFGX = this.colorFG;
		}
		Color colorBGX = c.getBackground();
		if (this.colorBG != null) {
			colorBGX = this.colorBG;
		}
		g.setColor(colorFGX);
		g.fillRect(x, x, width, this.thickness); // top
		g.fillRect(x, y + height - this.thickness, width, this.thickness); // bottom
		g.fillRect(x, y, this.thickness, height); // left
		g.fillRect(x + width - this.thickness, y, this.thickness, height); // right
		g.setColor(colorBGX);
		// top/bottom
		int cx = 0;
		// get real pattern
		int[] realPattern = new int[this.pattern.length];
		for (int i = 0; i < this.pattern.length; i++) {
			if (this.thicken) {
				realPattern[i] = this.pattern[i] * this.thickness;
			} else {
				realPattern[i] = this.pattern[i];
			}
		}
		for (int i = 0, j = 0; i < width; i++, j += 2) {
			if (j >= realPattern.length) {
				j = 0;
			}
			cx += realPattern[j];
			g.fillRect(cx, y, realPattern[j + 1], this.thickness); // top
			g.fillRect(cx, y + height - this.thickness, realPattern[j + 1],
					this.thickness); // bottom
			cx += realPattern[j + 1];
		}
		// left/right
		int cy = 0;
		for (int i = 0, j = 0; i < height; i++, j += 2) {
			if (j >= realPattern.length) {
				j = 0;
			}
			cy += realPattern[j];
			g.fillRect(x, cy, this.thickness, realPattern[j + 1]); // left
			g.fillRect(x + width - this.thickness, cy, this.thickness,
					realPattern[j + 1]); // right
			cy += realPattern[j + 1];
		}
	}
}
