package com.smartmatic.waffle.ecm.ui.util;

import java.awt.Color;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import javax.swing.JTextField;

/**
 * The class is used as an extension of JTextField but with support of unicode
 * typing characters.
 */
public class UnicodeTextField extends JTextField {

	private static final long serialVersionUID = -7734616673936666856L;

	/**
	 * Used to define which character must the field expect to start recording
	 * the unicode sequence.
	 */
	private static final char UNICODE_ESCAPE_CHARACTER = '#';

	/** Set of the allowed combining unicodes */
	private static Set<String> combiningChars = new HashSet<String>(
			Arrays.asList(new String[] { "300", "301", "302", "303", "304",
					"305", "306", "307", "308", "309", "30A", "30B", "30C",
					"30D", "30E", "30F", "310", "311", "312", "313", "314",
					"315", "316", "317", "318", "319", "31A", "31B", "31C",
					"31D", "31E", "31F", "320", "321", "322", "323", "324",
					"325", "326", "327", "328", "329", "32A", "32B", "32C",
					"32D", "32E", "32F", "330", "331", "332", "333", "334",
					"335", "336", "337", "338", "339", "33A", "33B", "33C",
					"33D", "33E", "33F", "340", "341", "342", "343", "344",
					"345", "346", "347", "348", "349", "34A", "34B", "34C",
					"34D", "34E", "34F", "350", "351", "352", "353", "354",
					"355", "356", "357", "358", "359", "35A", "35B", "35C",
					"35D", "35E", "35F", "360", "361", "362", "363", "364",
					"365", "366", "367", "368", "369", "36A", "36B", "36C",
					"36D", "36E", "36F" }));

	/** The maximum number of characters the field is allowed to accept */
	private int maxNumberOfCharacters;

	/** The number of keys pressed in the currently recording sequence. */
	private int keysPressed;

	/** To indicate if unicode sequence should be recorded */
	private boolean isRecordingUnicodeSequence;

	/** To indicate if is text field is ready to save */
	private boolean readyToSave;

	/**
	 * Contains the series of (up to) 4 digits forming the Unicode hex code
	 * point which will be converted. Typically, contains something like "A3FE".
	 */
	private StringBuilder unicode;

	/**
	 * Instantiates a new unicode text field.
	 */
	public UnicodeTextField(int maxNumberOfCharacters) {
		super();
		this.maxNumberOfCharacters = maxNumberOfCharacters;
		this.keysPressed = 0;
		this.isRecordingUnicodeSequence = false;
		this.unicode = new StringBuilder();

		this.addKeyListener(new KeyListener() {

			@Override
			public void keyTyped(KeyEvent e) {
				unicodeKeyTypedActionPerformed(e);
			}

			@Override
			public void keyPressed(KeyEvent e) {

			}

			@Override
			public void keyReleased(KeyEvent e) {
				unicodeKeyReleasedActionPerformed(e);
			}
		});
	}

	/**
	 * Returns true if the field is ready to save the actual value, otherwise
	 * false.
	 * 
	 * @return true if is ready to save, otherwise false.
	 */
	public boolean isReadyToSave() {
		return this.readyToSave;
	}

	/**
	 * Performs the unicode replacement operations.
	 * 
	 * @param keyEvent
	 *            The key event that triggered this.
	 */
	private void unicodeKeyTypedActionPerformed(KeyEvent keyEvent) {

		// If the key pressed is the escape character to start recording the
		// sequence.
		if (keyEvent.getKeyChar() == UNICODE_ESCAPE_CHARACTER) {
			this.startTypingSequence();
			this.cleanupTextField(Color.RED);
			this.readyToSave = false;
		}
		// If the escape character was already pressed and is recording the
		// sequence.
		else if (this.isRecordingUnicodeSequence) {
			char keyPressed = keyEvent.getKeyChar();
			// If the entered character is a hex digit.
			if (Character.digit(keyPressed, 16) >= 0) {

				this.unicode.append(keyPressed);
				this.keysPressed++;

			} else {
				// If we are not backspacing, cancel the unicode input by
				// erasing what has been entered since the escape character.
				if (keyPressed != '\b') {
					this.setText(this.getText().replaceAll(
							String.valueOf(UNICODE_ESCAPE_CHARACTER)
									+ this.unicode, ""));

					keyEvent.consume();
					this.resetTypingSequence();
					this.cleanupTextField(Color.BLACK);
					this.readyToSave = true;
				}
			}
			if (this.keysPressed == 4) {
				String[] split = this.getText().trim()
						.split(String.valueOf(UNICODE_ESCAPE_CHARACTER));
				String complementaryText = (split.length > 1) ? split[1]
						.substring(3) : "";

				this.setText(split[0]
						+ codePointToChar(this.unicode.toString())
						+ complementaryText);

				keyEvent.consume();
				this.resetTypingSequence();
				this.cleanupTextField(Color.BLACK);
			}
		}
	}

	/**
	 * Performs the unicode key releases operations
	 * 
	 * @param keyEvent
	 *            the key event that triggered this method.
	 */
	private void unicodeKeyReleasedActionPerformed(KeyEvent keyEvent) {

		// manage backspace during unicode input.
		if (this.isRecordingUnicodeSequence
				&& keyEvent.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
			if (this.keysPressed > 0) {
				this.keysPressed--;
				this.unicode.deleteCharAt(this.keysPressed);
			}

			// manage the backspace removing the UNICODE_ESCAPE_CHARACTER
			if (!this.getText().contains(
					String.valueOf(UNICODE_ESCAPE_CHARACTER))) {
				this.resetTypingSequence();
				this.cleanupTextField(Color.BLACK);
			}
		}

		if (this.getText().contains(String.valueOf(UNICODE_ESCAPE_CHARACTER))) {
			// As long as the escape char is present, we cannot save
			this.readyToSave = false;
		} else {
			// check for max length
			String txt = this.getText();
			this.readyToSave = (this.getUnicodeLength(txt) <= this.maxNumberOfCharacters);
		}
	}

	/**
	 * Sets the required fields with the initial values when the user starts
	 * typing the unicode sequence.
	 */
	private void startTypingSequence() {
		this.keysPressed = 0;
		this.isRecordingUnicodeSequence = true;
	}

	/**
	 * Reset the fields to the initial values when the user stops typing the
	 * unicode sequence.
	 */
	private void resetTypingSequence() {
		this.isRecordingUnicodeSequence = false;
		this.unicode = new StringBuilder();
	}

	/**
	 * Updates the text field with the current text replacing all the
	 * UNICODE_ESCAPE_CHARACTER occurrences and sets the caret color.
	 * 
	 * @param caretColor
	 *            the caret color
	 */
	private void cleanupTextField(Color caretColor) {
		String currentText = this.getText();
		this.setText(currentText.replaceAll(
				String.valueOf(UNICODE_ESCAPE_CHARACTER), ""));
		this.setCaretColor(caretColor);
	}

	/**
	 * Method used to calculate and return the length of the text, if combining
	 * unicodes are present they count only as one per repetition.
	 * 
	 * @param text
	 *            the text to calculate the length
	 * @return the real length of the text
	 */
	public int getUnicodeLength(String text) {

		int length = 0;
		for (int i = 0; i < text.length(); i++) {
			String character = Integer.toString(text.codePointAt(i), 16);
			if (!combiningChars.contains(character)) {
				length++;
			}
		}
		return length;
	}

	/**
	 * Parses a unicode sequence and converts it to the corresponding character.
	 * 
	 * @param stringToParse
	 *            the string to parse
	 * @return the string with the corresponding character
	 */
	public String codePointToChar(String stringToParse) {

		int parsedInt = Integer.parseInt(stringToParse, 16);

		return Character.toString((char) parsedInt);
	}
}
