package com.smartmatic.waffle.localpersistence;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.crypto.SecretKey;

import org.apache.commons.lang.SystemUtils;
import org.dom4j.DocumentException;

import com.smartmatic.waffle.localpersistence.exception.HistoricFileException;
import com.smartmatic.waffle.localpersistence.exception.UnreachableFileException;
import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamOmitField;

/**
 * Abstract representation of files and directory used for the application.
 */
@XStreamAlias(value = "common-destinations")
public class CommonDestinations {

	protected static CommonDestinations instance;

	@XStreamAlias(value = "base-dir")
	@CommonDestinationInfo(existence = Existence.MANDATORY, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File baseDir;

	@XStreamAlias(value = "data-dir")
	@CommonDestinationInfo(existence = Existence.MANDATORY, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File dataDir;

	@XStreamAlias(value = "conf-dir")
	@CommonDestinationInfo(existence = Existence.MANDATORY, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File confDir;

	@XStreamAlias(value = "appstate-dir")
	@CommonDestinationInfo(existence = Existence.OPTIONAL_AUTOCREATE, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File appStateDir;

	@XStreamAlias(value = "logs-dir")
	@CommonDestinationInfo(existence = Existence.OPTIONAL_AUTOCREATE, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File logsDir;

	@XStreamAlias(value = "images-dir")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File imagesDir;

	@XStreamAlias(value = "reports-dir")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File reportsDir;

	@XStreamAlias(value = "logs-config-data")
	@CommonDestinationInfo(existence = Existence.MANDATORY, existenceLevel = 0, type = DestinationType.FILE_PLAIN)
	public final File logsConfigData;

	@XStreamAlias(value = "configuration-entity")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_XML)
	public final File configurationEntityFile;

	/**
	 * Holds the reference to the file encrypted using PBE (Passphrase based
	 * encryption) used to check that the user holds the passphrase to use the
	 * software
	 */
	@XStreamAlias(value = "pbe-file")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE)
	public final File pbeFile;

	@XStreamAlias(value = "scripts-dir")
	@CommonDestinationInfo(existence = Existence.MANDATORY, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File scriptsDir;

	@XStreamAlias(value = "xsd-external-dir")
	@CommonDestinationInfo(existence = Existence.MANDATORY, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File xsdExternalDir;

	@XStreamAlias(value = "xsd-datatypes-file")
	@CommonDestinationInfo(existence = Existence.MANDATORY, existenceLevel = 0, type = DestinationType.FILE)
	public final File xsdDataTypes;

	@XStreamAlias(value = "xsd-xmlschema-file")
	@CommonDestinationInfo(existence = Existence.MANDATORY, existenceLevel = 0, type = DestinationType.FILE)
	public final File xsdXmlSchema;

	/**
	 * Additional info for common destination fields. Made it protected so
	 * subclasses can customize it. Be carefully in synchronizing the access to
	 * this map to ensure thread safely
	 */
	@XStreamOmitField
	protected Map<File, CommonDestinationInfo> destinationInfos;

	/**
	 * Creates a new CommonDestinations instance
	 * 
	 * @param theBaseDir
	 *            base directory
	 * @throws IllegalArgumentException
	 *             Indicate that a method has been passed an illegal or
	 *             inappropriate argument.
	 * @throws IllegalAccessException
	 */
	public CommonDestinations(File theBaseDir) throws IllegalArgumentException,
			IllegalAccessException {
		if (theBaseDir == null) {

			this.baseDir = SystemUtils.getUserDir();
		} else {

			this.baseDir = theBaseDir;
		}

		this.dataDir = new File(this.baseDir, "data");
		this.confDir = new File(this.dataDir, "conf");
		this.appStateDir = new File(this.dataDir, "state");
		this.logsDir = new File(this.baseDir, "log");
		this.imagesDir = new File(this.confDir, "images");
		this.reportsDir = new File(this.confDir, "reports");
		this.logsConfigData = new File(this.confDir, "app-client-log4j.xml");
		this.configurationEntityFile = new File(this.confDir,
				"plain-configuration-entity.xml");

		this.pbeFile = new File(this.confDir, "pbe-lock");
		this.scriptsDir = new File(this.baseDir, "scripts");

		this.xsdExternalDir = new File(this.confDir, "external");
		this.xsdDataTypes = new File(this.xsdExternalDir, "datatypes.dtd");
		this.xsdXmlSchema = new File(this.xsdExternalDir, "XMLSchema.dtd");

		this.processAnnotations();
	}

	protected void processAnnotations() throws IllegalArgumentException,
			IllegalAccessException {

		if (this.destinationInfos == null) {
			this.destinationInfos = Collections
					.synchronizedMap(new HashMap<File, CommonDestinationInfo>());
		} else {
			this.destinationInfos.clear();
		}

		Field[] fields = this.getClass().getFields();
		for (Field theField : fields) {
			if (File.class.isAssignableFrom(theField.getType())) {

				CommonDestinationInfo annotation = theField
						.getAnnotation(CommonDestinationInfo.class);
				if (annotation == null) {

					/* Initialize annotation with default values */
					annotation = new CommonDestinationInfo() {

						@Override
						public Existence existence() {
							return Existence.OPTIONAL;

						}

						@Override
						public int existenceLevel() {
							return 0;

						}

						@Override
						public DestinationType type() {
							return DestinationType.FOLDER;

						}

						@Override
						public Class<? extends Annotation> annotationType() {
							return CommonDestinationInfo.class;

						}

						@Override
						public Encryption encryption() {
							return Encryption.PLAIN;

						}

					};
				}

				this.destinationInfos
						.put((File) theField.get(this), annotation);

			}
		}
		return;
	}

	/**
	 * Creates a new CommonDestinations instance from a configuration file
	 * 
	 * @param theBaseDir
	 *            base directory
	 * @param confFile
	 *            the configuration file
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 */
	public CommonDestinations(File theBaseDir, InputStream confFile)
			throws IllegalArgumentException, IllegalAccessException {
		if (theBaseDir == null) {
			this.baseDir = SystemUtils.getUserDir();
		} else {

			this.baseDir = theBaseDir;
		}

		LocalPersistenceManager.getInstance().processAnnotations(
				CommonDestinations.class);

		CommonDestinations instance = (CommonDestinations) LocalPersistenceManager
				.getInstance().loadFromXML(confFile);

		CommonDestinations defaultInstance = new CommonDestinations(theBaseDir);

		this.dataDir = this.insertBaseDir(instance.dataDir,
				defaultInstance.dataDir);
		this.confDir = this.insertBaseDir(instance.confDir,
				defaultInstance.confDir);
		this.appStateDir = this.insertBaseDir(instance.appStateDir,
				defaultInstance.appStateDir);
		this.logsDir = this.insertBaseDir(instance.logsDir,
				defaultInstance.logsDir);
		this.imagesDir = this.composePath(this.confDir, instance.imagesDir,
				defaultInstance.imagesDir);
		this.reportsDir = this.composePath(this.confDir, instance.reportsDir,
				defaultInstance.reportsDir);
		this.logsConfigData = this.composePath(this.confDir,
				instance.logsConfigData, defaultInstance.logsConfigData);
		this.configurationEntityFile = this.composePath(this.confDir,
				instance.configurationEntityFile,
				defaultInstance.configurationEntityFile);

		this.pbeFile = this.insertBaseDir(instance.pbeFile,
				defaultInstance.pbeFile);

		this.scriptsDir = this.composePath(this.baseDir, instance.scriptsDir,
				defaultInstance.scriptsDir);

		this.xsdExternalDir = this.composePath(this.confDir,
				instance.xsdExternalDir, defaultInstance.xsdExternalDir);
		this.xsdDataTypes = this.composePath(this.xsdExternalDir,
				instance.xsdDataTypes, defaultInstance.xsdDataTypes);
		this.xsdXmlSchema = this.composePath(this.xsdExternalDir,
				instance.xsdXmlSchema, defaultInstance.xsdXmlSchema);

		this.processAnnotations();
	}

	/**
	 * @param theDefaultInstance
	 * @param instance
	 * @return
	 */
	protected File composePath(File theBaseDir, File pathToCompose,
			File defaultPath) {
		if (pathToCompose != null) {

			return new File(theBaseDir, pathToCompose.getPath());
		} else {

			return defaultPath;
		}
	}

	protected File insertBaseDir(File pathToCompose, File defaultPath) {
		return this.composePath(this.baseDir, pathToCompose, defaultPath);
	}

	/**
	 * Getter of <code>instance</code> property.
	 * 
	 * @return <code>instance</code> property value.
	 * @see #instance
	 */

	public static CommonDestinations getInstance() {
		if (instance == null) {
			throw new IllegalStateException(
					"CommonDestinations must be initialized first using setInstance's method");
		}

		return instance;
	}

	/**
	 * Setter of <code>instance</code> property.
	 * 
	 * @param theInstance
	 *            The <code>instance</code> to set.
	 * @see #instance
	 */
	public static void setInstance(CommonDestinations theInstance) {
		if (instance != null) {
			throw new IllegalStateException(
					"CommonDestinations is already initialized");
		}

		instance = theInstance;

		return;
	}

	/**
	 * Returns the information of a destination file
	 * 
	 * @param destination
	 *            the file destination to get information
	 * @return the destination information
	 */
	public CommonDestinationInfo getInfo(File destination) {
		return this.destinationInfos.get(destination);
	}

	/**
	 * Returns the type of a destination file
	 * 
	 * @param destination
	 *            destination file
	 * @return type of the destination file
	 */
	public DestinationType getType(File destination) {
		CommonDestinationInfo destinationInfo = this.getInfo(destination);

		if (destinationInfo != null) {
			return destinationInfo.type();

		} else {
			return null;
		}
	}

	/**
	 * Returns the existence property of the destination file
	 * 
	 * @param destination
	 *            destination file
	 * @return existence of the destination file
	 */
	public Existence getExistence(File destination) {
		CommonDestinationInfo destinationInfo = this.getInfo(destination);

		if (destinationInfo != null) {
			return destinationInfo.existence();

		} else {
			return null;
		}
	}

	/**
	 * Returns the existence level property of the destination file
	 * 
	 * @param destination
	 *            destination file
	 * @return existence level of the destination file
	 */
	public int getExistenceLevel(File destination) {
		CommonDestinationInfo destinationInfo = this.getInfo(destination);

		if (destinationInfo != null) {
			return destinationInfo.existenceLevel();

		} else {
			return 0;
		}
	}

	/**
	 * Returns the encryption property of the destination file
	 * 
	 * @param destination
	 *            destination file
	 * @return encryption of the destination file
	 */
	public Encryption getEncryption(File destination) {
		CommonDestinationInfo destinationInfo = this.getInfo(destination);

		if (destinationInfo != null) {
			return destinationInfo.encryption();

		} else {
			return null;
		}
	}

	/**
	 * Getter of <code>destinationInfos</code> property.
	 * 
	 * @return <code>destinationInfos</code> property value.
	 * @see #destinationInfos
	 */

	public Map<File, CommonDestinationInfo> getDestinationInfos() {
		return new HashMap<File, CommonDestinationInfo>(this.destinationInfos);
	}

	/**
	 * Returns the last file
	 * 
	 * @param dir
	 * @param password
	 * @return
	 * @throws UnreachableFileException
	 * @throws HistoricFileException
	 */
	public File getLastFile(File dir, SecretKey password)
			throws UnreachableFileException, HistoricFileException {
		String[] children = dir.list();
		if (children.length == 0) {
			throw new UnreachableFileException(
					UnreachableFileException.ExceptionInfo.FILE_CORRUPTED_OR_UNREACHABLE);
		}
		List<Integer> fileNames = new ArrayList<Integer>();
		int maximum = 0;
		try {
			for (String fileName : children) {
				String[] innerName = fileName.split("\\.");
				fileNames.add(Integer.valueOf(innerName[0]));
			}

			maximum = Collections.max(fileNames);
		} catch (Exception e) {
			throw new UnreachableFileException(
					UnreachableFileException.ExceptionInfo.FILE_CORRUPTED_OR_UNREACHABLE);
		}
		File newXml = new File(dir, (maximum + ".xml"));
		if (LocalPersistenceManager.getInstance().checkXmlReadable(newXml,
				password, this.xsdExternalDir) == DestinationStatus.VALID) {
			return newXml;
		} else {
			if (maximum <= 1) {
				throw new UnreachableFileException(
						UnreachableFileException.ExceptionInfo.FILE_CORRUPTED_OR_UNREACHABLE);
			} else {
				File secondXml = new File(dir, (maximum - 1 + ".xml"));
				if (LocalPersistenceManager.getInstance().checkXmlReadable(
						secondXml, password, this.xsdExternalDir) == DestinationStatus.VALID) {
					return secondXml;
				} else {
					throw new HistoricFileException(
							HistoricFileException.ExceptionInfo.UNRECOVERY_HISTORIC_FILE_EXCEPTION);
				}
			}
		}
	}

	/**
	 * Returns an object loaded from XML file
	 * 
	 * @param objectToLoadFile
	 * @return
	 * @throws UnreachableFileException
	 * @throws HistoricFileException
	 * @throws CryptographicStaticException
	 * @throws CryptographicRuntimeException
	 * @throws IOException
	 */
	public Object loadFromXML(File objectToLoadFile)
			throws UnreachableFileException, HistoricFileException,
			CryptographicStaticException, CryptographicRuntimeException,
			IOException {

		return this.loadFromXML(objectToLoadFile, null);

	}

	/**
	 * Returns an object loaded from XML file
	 * 
	 * @param objectToLoadFile
	 * @param password
	 * @return the object loaded
	 * @throws IOException
	 * @throws UnreachableFileException
	 * @throws HistoricFileException
	 * @throws CryptographicStaticException
	 * @throws CryptographicRuntimeException
	 */
	public Object loadFromXML(File objectToLoadFile, SecretKey password)
			throws IOException, UnreachableFileException,
			HistoricFileException, CryptographicStaticException,
			CryptographicRuntimeException {

		if (this.getType(objectToLoadFile) == DestinationType.HISTORIC_XML_FILE) {

			return this.loadHistoricFromXML(objectToLoadFile, password);
		} else {

			return LocalPersistenceManager.getInstance().loadFromXML(
					objectToLoadFile, password);
		}

	}

	/**
	 * Returns an object loaded from XML file
	 * 
	 * @param objectToLoadFile
	 * @return the object loaded
	 * @throws IOException
	 * @throws UnreachableFileException
	 * @throws HistoricFileException
	 */
	public Object loadHistoricFromXML(File objectToLoadFile)
			throws IOException, UnreachableFileException, HistoricFileException {
		String[] parts = objectToLoadFile.getName().split("\\.");
		String name = parts[0];
		File newFile = new File(name);

		Object xmlStream = LocalPersistenceManager.getInstance().loadFromXML(
				this.getLastFile(newFile, null));

		return xmlStream;

	}

	/**
	 * Returns an object loaded from XML file
	 * 
	 * @param objectToLoadFile
	 * @param password
	 * @return the object loaded
	 * @throws IOException
	 * @throws UnreachableFileException
	 * @throws HistoricFileException
	 * @throws CryptographicStaticException
	 * @throws CryptographicRuntimeException
	 */
	public Object loadHistoricFromXML(File objectToLoadFile, SecretKey password)
			throws IOException, UnreachableFileException,
			HistoricFileException, CryptographicStaticException,
			CryptographicRuntimeException {
		String[] parts = objectToLoadFile.getName().split("\\.");
		String name = parts[0];
		File newFile = new File(objectToLoadFile.getParent(), name);

		InputStream stream = new BufferedInputStream(new FileInputStream(
				this.getLastFile(newFile, password)));
		try {

			return LocalPersistenceManager.getInstance().loadFromXML(stream,
					password);

		} finally {
			stream.close();
		}
	}

	/**
	 * Persists an historic object to XML
	 * 
	 * @param objectToPersist
	 * @param file
	 * @param password
	 * @throws UnreachableFileException
	 * @throws HistoricFileException
	 * @throws CryptographicStaticException
	 * @throws CryptographicRuntimeException
	 * @throws DocumentException
	 * @throws IOException
	 */
	public void persistHistoricToXml(Object objectToPersist, File file,
			SecretKey password) throws UnreachableFileException,
			HistoricFileException, CryptographicStaticException,
			CryptographicRuntimeException, DocumentException, IOException {
		File targetFile = file;
		switch (this.getType(file)) {

		case HISTORIC_XML_FOLDER:
			File lastOne = this.getLastFile(file, password);
			String[] parts = lastOne.getName().split("\\.");
			String name = parts[0];
			targetFile = new File(file, (Integer.parseInt(name) + 1 + ".xml"));
			break;

		case HISTORIC_XML_FILE:
			String historicFileName = file.getName();
			String[] part = historicFileName.split("\\.");

			File dir = new File(file.getParentFile(), part[0]);
			if (!dir.isDirectory()) {
				dir.mkdirs();
			}

			int maximum;
			File[] children = dir.listFiles();

			List<Integer> fileNames = new ArrayList<Integer>();
			File child = null;
			String[] innerName;
			for (int aux = 0; aux < children.length; aux++) {
				child = children[aux];
				if (child.isFile() && child.getName().endsWith(".xml")) {
					innerName = children[aux].getName().split("\\.");
					try {
						fileNames.add(Integer.valueOf(innerName[0]));
					} catch (java.lang.NumberFormatException e) {
						continue;
					}
				}
			}

			if (fileNames.size() > 0) {

				maximum = Collections.max(fileNames);
			} else {
				maximum = 0;
			}
			targetFile = new File(dir, (maximum + 1 + ".xml"));

			break;

		default:
			break;
		}

		LocalPersistenceManager.getInstance().persistToXML(objectToPersist,
				targetFile, password);

	}

	/**
	 * Persist an object to XML
	 * 
	 * @param objectToPersist
	 * @param file
	 * @param password
	 * @throws UnreachableFileException
	 * @throws HistoricFileException
	 * @throws CryptographicStaticException
	 * @throws CryptographicRuntimeException
	 * @throws DocumentException
	 * @throws IOException
	 */
	public void persistToXml(Object objectToPersist, File file,
			SecretKey password) throws UnreachableFileException,
			HistoricFileException, CryptographicStaticException,
			CryptographicRuntimeException, DocumentException, IOException {

		if (this.getType(file) == DestinationType.HISTORIC_XML_FILE) {

			this.persistHistoricToXml(objectToPersist, file, password);
		} else {

			LocalPersistenceManager.getInstance().persistToXML(objectToPersist,
					file, password);
		}
	}

	/**
	 * 
	 * @param level
	 * @param password
	 * @return
	 * @throws HistoricFileException
	 */
	public Map<File, CommonResultInfo> checkFiles(int level, SecretKey password)
			throws HistoricFileException {
		Set<Entry<File, CommonDestinationInfo>> commons = this
				.getDestinationInfos().entrySet();
		Map<File, CommonResultInfo> results = new HashMap<File, CommonResultInfo>();

		for (Entry<File, CommonDestinationInfo> commonEntry : commons) {
			CommonResultInfo cri = new CommonResultInfo();
			File file = commonEntry.getKey();
			CommonDestinationInfo commonDestination = commonEntry.getValue();
			DestinationType dest = commonDestination.type();

			Existence existence = commonDestination.existence();
			cri.setExistence(existence);
			cri.setDestinationType(dest);

			DestinationStatus status;

			int existenceLevel = commonDestination.existenceLevel();

			status = this.checkFile(level, password, file, dest, existence,
					existenceLevel);

			cri.setStatus(status);
			results.put(file, cri);

		}
		return results;

	}

	/**
	 * 
	 * @param level
	 * @param password
	 * @param file
	 * @param dest
	 * @param existence
	 * @param existenceLevel
	 * @return
	 * @throws HistoricFileException
	 */
	private DestinationStatus checkFile(int level, SecretKey password,
			File file, DestinationType dest, Existence existence,
			int existenceLevel) throws HistoricFileException {

		DestinationStatus status;

		if (existenceLevel <= level) {

			switch (existence) {
			case MANDATORY:
			case MANDATORY_AUTOCREATE:
				status = this.checkMandatoryFile(password, file, dest);
				break;

			case OPTIONAL:
				status = this.checkOptionalFile(password, file, dest);
				break;

			case MANDATORY_VALID:
				status = this.checkMandatoryValidFile(password, file, dest);
				break;

			case OPTIONAL_AUTOCREATE:
				status = this.checkOptionalAutocreateFile(file, dest);
				break;

			case OPTIONAL_VALID:
				status = this.checkOptionalFile(password, file, dest);

				if (status == DestinationStatus.FOUND) {
					status = this.checkMandatoryValidFile(password, file, dest);
				}

				break;

			default:
				throw new IllegalArgumentException();
			}
		} else {

			switch (existence) {
			case MANDATORY_AUTOCREATE:
			case OPTIONAL_AUTOCREATE:
				status = this.checkOptionalAutocreateFile(file, dest);
				break;

			case MANDATORY_VALID:
			case OPTIONAL_VALID:
				status = this.checkOptionalFile(password, file, dest);

				if (status == DestinationStatus.FOUND) {
					status = this.checkMandatoryValidFile(password, file, dest);
				}
				break;
			default:
				status = null;
				break;
			}

			if (status == DestinationStatus.NOT_CREATED
					|| status == DestinationStatus.NOT_FOUND
					|| status == DestinationStatus.NOT_FOUND_OPTIONAL
					|| status == null) {

				status = DestinationStatus.NOT_REQUIRED_LEVEL;
			}
		}
		return status;
	}

	/**
	 * Check if the file <code>file</code> exist and is valid. The file MUST be
	 * defined as CommonDestinations' field
	 */
	public DestinationStatus checkFile(int level, SecretKey password, File file)
			throws HistoricFileException {

		CommonDestinationInfo commonDestination = this.getDestinationInfos()
				.get(file);
		DestinationType dest = commonDestination.type();

		Existence existence = commonDestination.existence();

		int existenceLevel = commonDestination.existenceLevel();

		return this.checkFile(level, password, file, dest, existence,
				existenceLevel);
	}

	/**
	 * Check if the file <code>file</code> exist and is valid. The file may not
	 * be defined as CommonDestinations' field
	 */
	public DestinationStatus checkFile(SecretKey password, File file,
			DestinationType dest) throws HistoricFileException {

		return this.checkMandatoryValidFile(password, file, dest);
	}

	/**
	 * Returns the status of the optional auto create file or folder
	 * 
	 * @param file
	 *            file to check
	 * @param dest
	 *            type of the file
	 * @return the status of the file
	 */
	private DestinationStatus checkOptionalAutocreateFile(File file,
			DestinationType dest) {
		DestinationStatus statusToReturn;
		switch (dest) {
		case FOLDER:
			if (!file.exists()) {
				boolean created = file.mkdirs();

				if (created) {
					statusToReturn = DestinationStatus.CREATED;
				} else {
					statusToReturn = DestinationStatus.NOT_CREATED;
				}
			} else {
				statusToReturn = DestinationStatus.FOUND;
			}
			break;
		case HISTORIC_XML_FOLDER:
			String name = file.getName();
			String[] parts = name.split("\\.");
			File dirFile = new File(parts[0]);
			if (!dirFile.exists()) {
				boolean created = dirFile.mkdirs();

				if (created) {
					statusToReturn = DestinationStatus.CREATED;
				} else {
					statusToReturn = DestinationStatus.NOT_CREATED;
				}
			} else {
				statusToReturn = DestinationStatus.FOUND;
			}
			break;
		default:
			throw new IllegalArgumentException(
					dest.toString()
							+ " is not a valid destination type for an optional autocreate check");

		}
		return statusToReturn;
	}

	/**
	 * Returns the status of the given file or folder
	 * 
	 * @param password
	 *            secret key
	 * @param file
	 *            the file to check
	 * @param dest
	 *            destination type
	 * @return the status of the file
	 * @throws HistoricFileException
	 */
	private DestinationStatus checkMandatoryValidFile(SecretKey password,
			File file, DestinationType dest) throws HistoricFileException {
		switch (dest) {
		case FILE:
			if (file.exists() == false) {
				return DestinationStatus.NOT_FOUND;

			} else if (this.checkDestinationFile(file, password)) {
				return DestinationStatus.VALID;

			} else {
				return DestinationStatus.INVALID;
			}

		case FILE_XML:
			return this.checkDestinationFileXML(file, password);

		case FILE_PLAIN:

			if (file.exists() == false) {
				return DestinationStatus.NOT_FOUND;
			} else if (this.checkDestinationFile(file, null)) {
				return DestinationStatus.VALID;
			} else {
				return DestinationStatus.INVALID;
			}

		case FILE_XML_PLAIN:
			return this.checkDestinationFileXML(file, null);

		case FOLDER:
			if (file.exists() && file.isDirectory()) {
				return DestinationStatus.FOUND;
			} else {
				return DestinationStatus.NOT_FOUND;
			}

		case HISTORIC_XML_FILE:
			return this.checkDestinationHistoricXMLFile(file, password);

		case HISTORIC_XML_FOLDER:
			return this.checkDestinationHistoricXMLFolder(file, password);

		default:
			throw new IllegalArgumentException(
					dest.toString()
							+ " is not a valid destination type for a mandatory valid check");
		}
	}

	/**
	 * Returns the status of the given file or folder
	 * 
	 * @param password
	 *            secret key
	 * @param file
	 *            the file to check
	 * @param dest
	 *            destination type
	 * @return the status of the file
	 * @throws HistoricFileException
	 */
	private DestinationStatus checkOptionalFile(SecretKey password, File file,
			DestinationType dest) throws HistoricFileException {
		switch (dest) {
		case FILE:
		case FILE_PLAIN:
		case FILE_XML:
		case FILE_XML_PLAIN:
			if (file.exists() && file.isFile()) {
				return DestinationStatus.FOUND;
			} else {
				return DestinationStatus.NOT_FOUND_OPTIONAL;
			}

		case FOLDER:
			if (file.exists() && file.isDirectory()) {
				return DestinationStatus.FOUND;
			} else {
				return DestinationStatus.NOT_FOUND_OPTIONAL;
			}

		case HISTORIC_XML_FILE:
			String name = file.getName();
			String[] parts = name.split("\\.");
			File dirFile = new File(file.getParentFile(), parts[0]);
			if (dirFile.exists() && dirFile.isDirectory()) {
				try {
					this.getLastFile(dirFile, password);
					return DestinationStatus.FOUND;
				} catch (UnreachableFileException e) {
					return DestinationStatus.NOT_FOUND_OPTIONAL;
				}
			} else {
				return DestinationStatus.NOT_FOUND_OPTIONAL;
			}

		case HISTORIC_XML_FOLDER:
			String fileName = file.getName();
			String[] part = fileName.split("\\.");
			File dirFileFolder = new File(file.getParent(), part[0]);
			if (dirFileFolder.exists() && dirFileFolder.isDirectory()) {
				return DestinationStatus.FOUND;
			} else {
				return DestinationStatus.NOT_FOUND_OPTIONAL;
			}

		default:
			throw new IllegalArgumentException(dest.toString()
					+ " is not a valid destination type for an optional check");

		}
	}

	/**
	 * Returns the status of the given file or folder
	 * 
	 * @param password
	 *            secret key
	 * @param file
	 *            the file to check
	 * @param dest
	 *            destination type
	 * @return the status of the file
	 * @throws HistoricFileException
	 */
	private DestinationStatus checkMandatoryFile(SecretKey password, File file,
			DestinationType dest) throws HistoricFileException {
		switch (dest) {
		case FILE:
		case FILE_PLAIN:
		case FILE_XML:
		case FILE_XML_PLAIN:
			if (file.exists() && file.isFile()) {
				return DestinationStatus.FOUND;
			} else {
				return DestinationStatus.NOT_FOUND;
			}

		case FOLDER:
			if (file.exists() && file.isDirectory()) {
				return DestinationStatus.FOUND;
			} else {
				return DestinationStatus.NOT_FOUND;
			}

		case HISTORIC_XML_FILE:
			String name = file.getName();
			String[] parts = name.split("\\.");
			File dirFile = new File(file.getParentFile(), parts[0]);
			if (dirFile.exists() && dirFile.isDirectory()) {
				try {
					this.getLastFile(dirFile, password);
					return DestinationStatus.FOUND;
				} catch (UnreachableFileException e) {
					return DestinationStatus.NOT_FOUND;
				}
			} else {
				return DestinationStatus.NOT_FOUND;
			}
		case HISTORIC_XML_FOLDER:
			String fileName = file.getName();
			String[] part = fileName.split("\\.");
			File dirFileFolder = new File(file.getParent(), part[0]);
			if (dirFileFolder.exists() && dirFileFolder.isDirectory()) {
				return DestinationStatus.FOUND;
			} else {
				return DestinationStatus.NOT_FOUND;
			}

		default:
			throw new IllegalArgumentException(dest.toString()
					+ " is not a valid destination type for a mandatory check");
		}
	}

	/**
	 * Returns true if the given file is readable, otherwise false
	 * 
	 * @param file
	 *            the file to check
	 * @param password
	 *            secret key
	 * @return true if readable, otherwise false
	 */
	private boolean checkDestinationFile(File file, SecretKey password) {
		return LocalPersistenceManager.getInstance().checkFileReadable(file,
				password);
	}

	/**
	 * Returns true if the given file is readable, otherwise false
	 * 
	 * @param file
	 *            the file to check
	 * @param password
	 *            secret key
	 * @return if readable, otherwise false
	 */
	public DestinationStatus checkDestinationFileXML(File file,
			SecretKey password) {
		return LocalPersistenceManager.getInstance().checkXmlReadable(file,
				password, this.xsdExternalDir);

	}

	/**
	 * Returns the status of the historic destination file
	 * 
	 * @param file
	 *            the file to check
	 * @param password
	 *            the secret key
	 * @return the status of the file
	 * @throws HistoricFileException
	 */
	public DestinationStatus checkDestinationHistoricXMLFile(File file,
			SecretKey password) throws HistoricFileException {
		try {
			String name = file.getName();
			String[] parts = name.split("\\.");
			File dir = new File(file.getParent(), parts[0]);
			File lastFile = this.getLastFile(dir, password);
			return LocalPersistenceManager.getInstance().checkXmlReadable(
					lastFile, password, this.xsdExternalDir);
		} catch (UnreachableFileException e) {
			return DestinationStatus.INVALID;
		}

	}

	/**
	 * Returns the status of the historic destination folder
	 * 
	 * @param file
	 *            the file to check
	 * @param password
	 *            the secret key
	 * @return the status of the folder
	 * @throws HistoricFileException
	 */
	public DestinationStatus checkDestinationHistoricXMLFolder(File file,
			SecretKey password) throws HistoricFileException {
		try {
			File lastFile = this.getLastFile(file, password);
			return LocalPersistenceManager.getInstance().checkXmlReadable(
					lastFile, password, this.xsdExternalDir);
		} catch (UnreachableFileException e) {
			return DestinationStatus.INVALID;
		}

	}

}
