/**
 * SAESLocalPersistenceManager.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.localpersistence;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import javax.crypto.SecretKey;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.dom4j.DocumentException;

import com.smartmatic.waffle.localpersistence.environment.FileSystemStatus;
import com.smartmatic.waffle.localpersistence.environment.ResourceSubjectTOBackup;
import com.smartmatic.waffle.localpersistence.exception.HistoricFileException;
import com.smartmatic.waffle.localpersistence.exception.SAESLocalPersistenceException;
import com.smartmatic.waffle.localpersistence.exception.UnreachableFileException;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.security.asymmetric.EncryptDecryptUtil;
import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;
import com.smartmatic.waffle.security.symmetric.SymmetricKeyManager;

/**
 * Class in charged of management of files persistence in the application. In
 * addition, handles events of detection of store medias and mounted drives.
 * 
 * @version 1.0 (Jun 10, 2011)
 * @author Carlos Ramirez (cramirez@smartmatic.com)
 */
public class SAESLocalPersistenceManager extends LocalPersistenceManager {

	/**
	 * For logging purposes.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(SAESLocalPersistenceManager.class);

	/**
	 * Field used to perform encryption/decryption operations on files.
	 */
	private SecretKey key;

	/**
	 * List of mounted devices represented as common media destinations.
	 * 
	 * @see com.smartmatic.waffle.localpersistence.SAESCommonDestinations
	 */
	private final List<SAESCommonDestinations> storeMedias;

	/**
	 * List of rejected devices represented as common media destinations.
	 * 
	 * @see com.smartmatic.waffle.localpersistence.SAESCommonDestinations
	 */
	private final List<SAESCommonDestinations> rejectedMedias;

	/** Timer used to perform the action of checking of mounted devices. */
	private Timer timer;

	/**
	 * Field that contains the value of a system property.
	 */
	private static final boolean DESTINATIONS_FROM_FILE;

	/**
	 * List of listeners.
	 * 
	 * @see com.smartmatic.waffle.localpersistence.SAESCommonDestinationListener
	 */
	private final List<SAESCommonDestinationListener> listeners = new Vector<SAESCommonDestinationListener>();

	/**
	 * Instance of listener that create the representation of common
	 * destinations.
	 * 
	 */
	private SAESLocalPersistenceListener localPersistenceListener;

	/**
	 * Static initialization code
	 */
	static {
		DESTINATIONS_FROM_FILE = Boolean.getBoolean("destinations-from-file");
		final SAESLocalPersistenceManager manager = new SAESLocalPersistenceManager();
		instance = manager;
		try {
			SAESCommonDestinations commonDestinations = manager
					.readFromFile(null);

			synchronized (manager.storeMedias) {
				manager.storeMedias.add(commonDestinations);

			}
		} catch (IOException exception) {
			logger.error(exception, exception.getMessage());
		} catch (IllegalArgumentException exception) {
			logger.error(exception, exception.getMessage());
		} catch (IllegalAccessException exception) {
			logger.error(exception, exception.getMessage());
		}
	}

	/**
	 * Default private constructor.
	 */
	private SAESLocalPersistenceManager() {

		this.reset();

		this.key = SymmetricKeyManager
				.getSymmetricKey("0JH3TIw9p0d0Q/IP2khCLA==");

		this.storeMedias = Collections
				.synchronizedList(new ArrayList<SAESCommonDestinations>());
		this.rejectedMedias = Collections
				.synchronizedList(new ArrayList<SAESCommonDestinations>());

		this.timer = new Timer();

		return;

	}

	/**
	 * Method used to clear the references to the list of store and rejected
	 * medias at stop time.
	 */
	private void purgeStoreMedia() {

		synchronized (this.storeMedias) {
			Iterator<SAESCommonDestinations> commons = this.storeMedias
					.iterator();

			SAESCommonDestinations defaultCommonDestination = commons.next();

			this.storeMedias.clear();
			this.rejectedMedias.clear();
			this.storeMedias.add(defaultCommonDestination);

		}
	}

	/**
	 * Returns a copy of the given file, replicated on all store media detected.
	 * 
	 * @param theFile
	 *            the file
	 * @param storeMedias
	 *            list of devices to be replicated
	 * @return list of file replicated
	 */
	private File[] calculatePathAllMedias(File theFile,
			List<SAESCommonDestinations> storeMedias) {
		synchronized (storeMedias) {

			File[] files = new File[storeMedias.size()];

			String theFilePath = theFile.getAbsolutePath();
			String defaultBaseDirPath = this.getDefaultDestinations().baseDir
					.getAbsolutePath();
			String theFilePathWithoutBase = StringUtils.difference(
					defaultBaseDirPath, theFilePath);

			files[0] = theFile;

			int i = 0;
			for (SAESCommonDestinations destinations : storeMedias) {
				if (i != 0) {
					File baseDir = destinations.baseDir;

					files[i] = new File(baseDir, theFilePathWithoutBase);

				}
				i++;
			}

			return files;
		}

	}

	/**
	 * Method used to get a representation of a Common Destination from a
	 * configuration file. If the system property "destinations-from-file" is
	 * set then this method read from file system and return the common
	 * destination reference.
	 * 
	 * @param theNewDrive
	 *            the the new drive
	 * @return an instance of common destination class
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @throws IllegalArgumentException
	 *             If it has been passed an illegal or inappropriate argument.
	 * @throws IllegalAccessException
	 *             If an error of concurrency occurs.
	 */
	private SAESCommonDestinations readFromFile(MountedDrive theNewDrive)
			throws IOException, IllegalArgumentException,
			IllegalAccessException {

		assert instance != null : "instance static variable can't be null";

		if (DESTINATIONS_FROM_FILE) {
			InputStream stream = new BufferedInputStream(
					SAESCommonDestinations.class
							.getResourceAsStream("/conf/common-destinations.xml"));

			try {
				this.processAnnotations(SAESCommonDestinations.class);

				stream.mark(Integer.MAX_VALUE);

				SAESCommonDestinations theCommonDestinations = null;

				if (this.localPersistenceListener != null) {
					theCommonDestinations = this.localPersistenceListener
							.createCommonDestinations(theNewDrive, null, stream);
				}

				if (theCommonDestinations == null) {
					theCommonDestinations = new SAESCommonDestinations(

					theNewDrive.mountedPath, null, stream, theNewDrive);
				}

				return theCommonDestinations;
			} finally {
				stream.close();
			}

		} else {
			SAESCommonDestinations theCommonDestinations = null;

			if (this.localPersistenceListener != null) {
				theCommonDestinations = this.localPersistenceListener
						.createCommonDestinations(theNewDrive, null, null);
			}

			if (theCommonDestinations == null) {

				if (theNewDrive == null) {

					theCommonDestinations = new SAESCommonDestinations(null,
							null, theNewDrive);
				} else {
					theCommonDestinations = new SAESCommonDestinations(
							theNewDrive.mountedPath, null, theNewDrive);
				}
			}

			return theCommonDestinations;
		}
	}

	/**
	 * Gets a list of mounted drives on the system. This method checks the OS
	 * where the application is running to determine the regex to use on query
	 * 
	 * @return list of mounted drives on the system.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @see com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager.MountedDrive
	 */
	private static MountedDrive[] listMountedDrives() throws IOException {
		String os = System.getProperty("os.name").toLowerCase();

		if (os.indexOf("windows") != -1) {
			return listWindowsMountedDrives();

		} else if (os.indexOf("linux") != -1) {

			return listLinuxMountedDrives();

		} else {

			return listWindowsMountedDrives();

		}

	}

	/**
	 * Gets a list of mounted drives on the system. This method is invoked if
	 * the application is running on platform windows.
	 * 
	 * @return list of mounted drives on the system.
	 */
	private static MountedDrive[] listWindowsMountedDrives() {

		File[] roots = File.listRoots();
		MountedDrive[] drives = new MountedDrive[roots.length];
		int i = 0;
		for (File theFile : roots) {

			String partitionName = theFile.getPath();
			String deviceName = theFile.getPath();
			File mountedPath = theFile;

			drives[i] = ((SAESLocalPersistenceManager) instance).new MountedDrive(
					deviceName, partitionName, mountedPath);

			i++;
		}

		return drives;
	}

	/**
	 * Gets a list of mounted drives on the system. This method is invoked if
	 * the application is running on platform windows.
	 * 
	 * @return list of mounted drives on the system.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 */
	private static MountedDrive[] listLinuxMountedDrives() throws IOException {
		ProcessBuilder processBuilder = new ProcessBuilder();
		processBuilder
				.command("sh", "-c",
						"mount | egrep 'ext|fat|ntfs|fuseblk' | awk '{print $1, $3, $5}'");

		processBuilder.redirectErrorStream(true);
		Process process = processBuilder.start();
		try {
			process.waitFor();

		} catch (InterruptedException exception) {
			logger.error(exception, exception.getMessage());
		}

		InputStream programOutput = process.getInputStream();
		try {
			List<String> lines = IOUtils.readLines(programOutput);

			MountedDrive[] drives = new MountedDrive[lines.size()];
			int i = 0;
			for (String theLine : lines) {
				if (StringUtils.contains(theLine, "dev")) {
					String[] devicePath = theLine.split(" ");

					String partitionName = devicePath[0];
					String deviceName = StringUtils.stripEnd(partitionName,
							"0123456789");
					File mountedPath = new File(devicePath[1]);
					String fileSystem = devicePath[2];

					drives[i] = ((SAESLocalPersistenceManager) instance).new MountedDrive(
							deviceName, partitionName, mountedPath, fileSystem);

					i++;
				}
			}
			logger.debug("There is [" + drives.length + "] detected in machine");
			return drives;
		} finally {
			programOutput.close();
			if (process != null) {
				process.destroy();
			}
		}

	}

	/**
	 * Class used to represent the storage devices mounted on the current
	 * system.
	 * 
	 * @version 1.0 (Jun 10, 2011)
	 * @author Carlos Ramirez (cramirez@smartmatic.com)
	 */
	public class MountedDrive implements Comparator<MountedDrive> {
		/**
		 * Device name.
		 */
		public final String deviceName;
		/**
		 * Partition name.
		 */
		public final String partitionName;
		/**
		 * Mounted path.
		 */
		public final File mountedPath;

		private String fileSystem;

		/**
		 * Default constructor.
		 * 
		 * @param theDeviceName
		 *            the device name
		 * @param thePartitionName
		 *            the partition name
		 * @param theMountedPath
		 *            the mounted path
		 */
		private MountedDrive(String theDeviceName, String thePartitionName,
				File theMountedPath) {
			super();
			this.deviceName = theDeviceName;
			this.partitionName = thePartitionName;
			this.mountedPath = theMountedPath;
		}

		private MountedDrive(String theDeviceName, String thePartitionName,
				File theMountedPath, String fileSystem) {
			super();
			this.deviceName = theDeviceName;
			this.partitionName = thePartitionName;
			this.mountedPath = theMountedPath;
			this.fileSystem = fileSystem;
		}

		/**
		 * Returns the mountedPath
		 * 
		 * @return the mountedPath
		 */
		public File getMountedPath() {
			return this.mountedPath;
		}

		/**
		 * Returns the device name
		 * 
		 * @return the device name
		 */
		public String getDeviceName() {
			return this.deviceName;
		}

		/**
		 * Return the partition name
		 * 
		 * @return the partition name
		 */
		public String getPartitionName() {
			return this.partitionName;
		}

		public String getFileSystem() {
			return this.fileSystem;
		}

		/**
		 * Compares its two arguments for order.
		 * 
		 * @param o1
		 *            the o1
		 * @param o2
		 *            the o2
		 * @return the int
		 * @inheritDoc
		 */
		@Override
		public int compare(MountedDrive o1, MountedDrive o2) {
			return (o1.deviceName + "." + o1.mountedPath.getAbsolutePath()
					+ "." + o1.partitionName)
					.compareTo(o2.deviceName + "."
							+ o2.mountedPath.getAbsolutePath() + "."
							+ o2.partitionName);
		}

		/**
		 * Hash code.
		 * 
		 * @return the int
		 * @inheritDoc
		 */
		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime
					* result
					+ ((this.deviceName == null) ? 0 : this.deviceName
							.hashCode());
			result = prime
					* result
					+ ((this.mountedPath == null) ? 0 : this.mountedPath
							.getAbsolutePath().hashCode());
			result = prime
					* result
					+ ((this.partitionName == null) ? 0 : this.partitionName
							.hashCode());
			return result;
		}

		/**
		 * Equals.
		 * 
		 * @param obj
		 *            the obj
		 * @return true, if successful
		 * @inheritDoc
		 */
		@Override
		public boolean equals(Object obj) {
			if (this == obj) {
				return true;
			}
			if (obj == null) {
				return false;
			}
			if (getClass() != obj.getClass()) {
				return false;
			}
			MountedDrive other = (MountedDrive) obj;

			if (this.deviceName == null) {
				if (other.deviceName != null) {
					return false;
				}
			} else if (!this.deviceName.equals(other.deviceName)) {
				return false;
			}
			if (this.mountedPath == null) {
				if (other.mountedPath != null) {
					return false;
				}
			} else if (!this.mountedPath.getAbsolutePath().equals(
					other.mountedPath.getAbsolutePath())) {
				return false;
			}
			if (this.partitionName == null) {
				if (other.partitionName != null) {
					return false;
				}
			} else if (!this.partitionName.equals(other.partitionName)) {
				return false;
			}
			return true;
		}

	}

	/**
	 * Method used to schedule the check task of mounted devices on system. This
	 * task will perform every 5 secs.
	 */
	public void scheduleCheckMountedDevices() {
		if (this.timer == null) {
			this.timer = new Timer();
		}
		this.timer.schedule(new TimerTask() {

			@Override
			public void run() {
				try {
					SAESLocalPersistenceManager.this.checkMountedDevices();
				} catch (IOException exception) {
					logger.error(exception, exception.getMessage());
				} catch (IllegalArgumentException exception) {
					logger.error(exception, exception.getMessage());
				} catch (IllegalAccessException exception) {
					logger.error(exception, exception.getMessage());
				}

				if (SAESLocalPersistenceManager.this.timer != null) {
					SAESLocalPersistenceManager.this
							.scheduleCheckMountedDevices();
				}

				return;
			}

		}, 5000);
	}

	/**
	 * Method used to stop daemon scheduled to check mounted devices.
	 */
	public void stop() {
		if (this.timer != null) {
			this.timer.cancel();
			this.purgeStoreMedia();
		}
		this.timer = null;
	}

	/**
	 * Perform the check process of mounted drives.
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @throws IllegalAccessException
	 *             If an error of concurrency occurs.
	 * @throws IllegalArgumentException
	 *             If it has been passed an illegal or inappropriate argument.
	 */
	public void checkMountedDevices() throws IOException,
			IllegalAccessException, IllegalArgumentException {

		logger.debug("Checking mounted devices...");
		MountedDrive[] mountedDrives = listMountedDrives();
		List<MountedDrive> newDrives = new ArrayList<MountedDrive>();
		List<SAESCommonDestinations> aliveDestinations = new ArrayList<SAESCommonDestinations>();

		List<SAESCommonDestinations> addedDestinations = new ArrayList<SAESCommonDestinations>();
		List<SAESCommonDestinations> removedDestinations = new ArrayList<SAESCommonDestinations>();

		/* Look for only new drives mounted */
		for (MountedDrive mountedDrive : mountedDrives) {

			boolean found = false;
			synchronized (SAESLocalPersistenceManager.this.storeMedias) {

				for (SAESCommonDestinations actualMedia : SAESLocalPersistenceManager.this.storeMedias) {

					if (mountedDrive != null
							&& actualMedia.baseDir.getAbsolutePath()
									.startsWith(
											mountedDrive.mountedPath
													.getAbsolutePath())) {
						if (actualMedia.getDeviceInfo() == null) {
							actualMedia.setDeviceInfo(mountedDrive);
						}
					}
					if (mountedDrive != null
							&& actualMedia.getDeviceInfo().mountedPath
									.getAbsolutePath().startsWith(
											mountedDrive.mountedPath
													.getAbsolutePath())) {
						found = true;

						/**
						 * Sometimes the ECM will format a USB drive to write
						 * data, and the file system can change to a different
						 * type. Here, we verify and update this info.
						 */
						if (actualMedia.getDeviceInfo().fileSystem != null
								&& !actualMedia.getDeviceInfo().fileSystem
										.equals(mountedDrive.fileSystem)) {
							String previousType = actualMedia.getDeviceInfo().fileSystem;
							String currentType = mountedDrive.fileSystem;
							String mountedDrivePath = mountedDrive
									.getMountedPath().getAbsolutePath();
							actualMedia.getDeviceInfo().fileSystem = mountedDrive
									.getFileSystem();
							logger.info("Updated the '" + mountedDrivePath
									+ "' file system from " + previousType
									+ " to " + currentType);
						}
						aliveDestinations.add(actualMedia);
						break;
					}
				}
			}

			if (!found) {
				synchronized (SAESLocalPersistenceManager.this.rejectedMedias) {

					/* Look for previously rejected medias */
					for (SAESCommonDestinations rejectedMedia : SAESLocalPersistenceManager.this.rejectedMedias) {

						if (rejectedMedia.getDeviceInfo().partitionName
								.startsWith(mountedDrive.partitionName)) {

							found = true;

							break;
						}

					}
				}
			}

			if (!found) {
				newDrives.add(mountedDrive);
			}
		}

		synchronized (SAESLocalPersistenceManager.this.storeMedias) {
			/* Look for drives no longer mounted */
			for (Iterator<SAESCommonDestinations> storeMediasIt = SAESLocalPersistenceManager.this.storeMedias
					.iterator(); storeMediasIt.hasNext();) {

				final SAESCommonDestinations theStoreMedia = storeMediasIt
						.next();
				if (aliveDestinations.contains(theStoreMedia) == false) {

					removedDestinations.add(theStoreMedia);
					storeMediasIt.remove();
				}
			}

			/* Create destinations for new mounted drives */
			for (MountedDrive theNewDrive : newDrives) {
				final SAESCommonDestinations theStoreMedia = SAESLocalPersistenceManager.this
						.readFromFile(theNewDrive);

				addedDestinations.add(theStoreMedia);
				SAESLocalPersistenceManager.this.storeMedias.add(theStoreMedia);
			}
		}
		synchronized (SAESLocalPersistenceManager.this.listeners) {
			Iterable<SAESCommonDestinationListener> theListeners = getInstance().listeners;
			for (SAESCommonDestinationListener theListener : theListeners) {

				if (addedDestinations.isEmpty() == false) {

					final SAESCommonDestinations[] addedDestinationsArray = addedDestinations
							.toArray(new SAESCommonDestinations[addedDestinations
									.size()]);

					boolean[] allowed = theListener
							.storeMediaAdded(addedDestinationsArray);

					int i = 0;
					// used only to notify that has been added a valid media in
					// consolidation process
					boolean allowedValidMedia = false;
					for (boolean mediaAllowed : allowed) {
						if (!mediaAllowed) {
							SAESCommonDestinations destinationAdded = addedDestinationsArray[i];

							SAESLocalPersistenceManager.this.storeMedias
									.remove(destinationAdded);
							logger.info("Removing usb from valid list ["
									+ destinationAdded.getDeviceInfo().mountedPath
											.getName() + "]");
							SAESLocalPersistenceManager.this.rejectedMedias
									.add(destinationAdded);
						} else {
							allowedValidMedia = true;
						}
						i++;
					}
					if (allowedValidMedia) {
						theListener.storeMediaChanged(this.storeMedias
								.toArray(new SAESCommonDestinations[0]));
						allowedValidMedia = false;
					}
				}

				if (removedDestinations.isEmpty() == false) {

					theListener
							.storeMediaRemoved(removedDestinations
									.toArray(new SAESCommonDestinations[removedDestinations
											.size()]));
				}

			}

		}
	}

	/**
	 * Gets the drive by name.
	 * 
	 * @param driveName
	 *            the drive name
	 * @return the drive by name
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */

	public MountedDrive getDriveByName(String driveName) throws IOException {

		MountedDrive[] mountedDrives = listMountedDrives();

		for (MountedDrive mountedDrive : mountedDrives) {
			if (mountedDrive.deviceName.equals(driveName)) {
				return mountedDrive;
			}
		}

		return null;
	}

	/**
	 * Getter of <code>instance</code> property.
	 * 
	 * @return <code>instance</code> property value.
	 * @see #INSTANCE
	 */

	public static SAESLocalPersistenceManager getInstance() {
		return (SAESLocalPersistenceManager) instance;
	}

	/**
	 * Gets an object instance from a xml encrypted file, given its secret key
	 * to decrypt the file.
	 * 
	 * @param objectToLoadFile
	 *            the file
	 * @return an instance of object represented by the given string
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 */
	public Object loadEncryptedObject(File objectToLoadFile)
			throws IOException, CryptographicStaticException,
			CryptographicRuntimeException {

		return this.loadFromXML(objectToLoadFile, this.key);

	}

	/**
	 * Gets an object instance from a xml file, using the class as references to
	 * read its annotations.
	 * 
	 * @param objectToLoadFile
	 *            the file
	 * @param theClass
	 *            the class of object to be loaded.
	 * @return an instance of object represented by the given file
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 */
	public Object loadEncryptedObject(File objectToLoadFile,
			@SuppressWarnings("rawtypes") Class... theClass)
			throws IOException, CryptographicStaticException,
			CryptographicRuntimeException {
		this.processAnnotations(theClass);
		return this.loadFromXML(objectToLoadFile, this.key);

	}

	/**
	 * It Serialize an object to persist in a xml format, written it encrypted
	 * in the given file.
	 * 
	 * @param objectToPersist
	 *            the object to be persisted
	 * @param file
	 *            the file which will be written
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 * @throws DocumentException
	 *             If an exception occurs during the processing of a DOM4J
	 *             document.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 */
	public void persistEncryptedObject(Object objectToPersist, File file)
			throws CryptographicStaticException, CryptographicRuntimeException,
			DocumentException, IOException {

		this.persistToXML(objectToPersist, file, this.key);
	}

	/**
	 * It Serialize an object to persist in a xml format, leaving an historic
	 * trace of this file.
	 * 
	 * @param objectToPersist
	 *            the object to be persisted
	 * @param file
	 *            the file which will be written
	 * @throws UnreachableFileException
	 *             If an exception occurs during reading system files.
	 * @throws HistoricFileException
	 *             If an exception occurs during reading historic files.
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 * @throws DocumentException
	 *             If an exception occurs during the processing of a DOM4J
	 *             document.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 */
	public void persistEncryptedObjectHistoricXML(Object objectToPersist,
			File file) throws UnreachableFileException, HistoricFileException,
			CryptographicStaticException, CryptographicRuntimeException,
			DocumentException, IOException {
		SAESCommonDestinations.getInstance().persistHistoricToXml(
				objectToPersist, file, this.key);
	}

	/**
	 * It Serialize a representation in string format of an object to persist
	 * it, written it encrypted in the given file.
	 * 
	 * @param objectToPersist
	 *            the object to be persisted
	 * @param file
	 *            the file which will be written
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 * @throws DocumentException
	 *             If an exception occurs during the processing of a DOM4J
	 *             document.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 */
	public void persistEncryptedObject(String objectToPersist, File file)
			throws CryptographicStaticException, CryptographicRuntimeException,
			DocumentException, IOException {

		this.persistToXML(objectToPersist, file, this.key);
	}

	/**
	 * It Serialize an object to persist a xml format in all mounted devices.
	 * 
	 * @param objectToPersist
	 *            the object to be persisted
	 * @param file
	 *            the file which will be written
	 * @throws SAESLocalPersistenceException
	 *             if an error occurred during persistence operations on local
	 *             system files
	 * 
	 */
	public void persistEncryptedObjectAllMedias(Object objectToPersist,
			File file) throws SAESLocalPersistenceException {

		synchronized (this.storeMedias) {
			File[] fileInMedias = this.calculatePathAllMedias(file);

			Iterator<SAESCommonDestinations> it = this.storeMedias.iterator();
			for (File theFile : fileInMedias) {

				SAESCommonDestinations theDestinations = it.next();
				try {
					theDestinations.persistToXml(objectToPersist, theFile,
							this.key);
				} catch (Exception exception) {
					logger.error(exception, exception.getMessage());

					throw new SAESLocalPersistenceException(
							SAESLocalPersistenceException.ExceptionInfo.ERROR_WRITTING_IN_MEDIA,
							theDestinations, theFile, exception);
				}
			}
		}

	}

	/**
	 * Calculate path all medias.
	 * 
	 * @param theFile
	 *            the the file
	 * @return the file[]
	 */
	public File[] calculatePathAllMedias(File theFile) {
		synchronized (this.storeMedias) {

			File[] files = new File[this.storeMedias.size()];

			String theFilePath = theFile.getAbsolutePath();
			String defaultBaseDirPath = this.getDefaultDestinations().baseDir
					.getAbsolutePath();
			String theFilePathWithoutBase = StringUtils.difference(
					defaultBaseDirPath, theFilePath);

			files[0] = theFile;

			int i = 0;
			for (SAESCommonDestinations destinations : this.storeMedias) {
				if (i != 0) {
					File baseDir = destinations.baseDir;

					files[i] = new File(baseDir, theFilePathWithoutBase);

				}
				i++;
			}

			return files;
		}

	}

	/**
	 * Adds the listener.
	 * 
	 * @param listener
	 *            the listener
	 */
	public void addListener(SAESCommonDestinationListener listener) {
		synchronized (this.listeners) {
			this.listeners.add(listener);
		}
	}

	/**
	 * Removes the listener.
	 * 
	 * @param listener
	 *            the listener
	 */
	public void removeListener(SAESCommonDestinationListener listener) {
		synchronized (this.listeners) {
			this.listeners.remove(listener);
		}
	}

	/**
	 * Contains listener.
	 * 
	 * @param listener
	 *            the listener
	 * @return true, if successful
	 */
	public boolean containsListener(SAESCommonDestinationListener listener) {
		synchronized (this.listeners) {
			return this.listeners.contains(listener);
		}
	}

	/**
	 * Getter of <code>localPersistenceListener</code> property.
	 * 
	 * @return <code>localPersistenceListener</code> property value.
	 * @see #localPersistenceListener
	 */

	public SAESLocalPersistenceListener getLocalPersistenceListener() {
		return this.localPersistenceListener;
	}

	/**
	 * Setter of <code>localPersistenceListener</code> property.
	 * 
	 * @param theLocalPersistenceListener
	 *            The <code>localPersistenceListener</code> to set.
	 * @see #localPersistenceListener
	 */
	public void setLocalPersistenceListener(
			SAESLocalPersistenceListener theLocalPersistenceListener) {

		synchronized (this.storeMedias) {

			if (this.storeMedias.size() > 1) {
				throw new IllegalStateException(
						"The listener must be assigned before the store medias are detected.");
			}

			assert this.storeMedias.size() == 1;

			this.localPersistenceListener = theLocalPersistenceListener;

			this.storeMedias.clear();
			try {
				this.storeMedias.add(this.readFromFile(null));
			} catch (IllegalArgumentException exception) {
				logger.error(exception.getMessage(), exception);
			} catch (IOException exception) {
				logger.error(exception.getMessage(), exception);
			} catch (IllegalAccessException exception) {
				logger.error(exception.getMessage(), exception);
			}
			return;
		}
	}

	/**
	 * Replace local persistence listener.
	 * 
	 * @param theLocalPersistenceListener
	 *            the the local persistence listener
	 */
	public void replaceLocalPersistenceListener(
			SAESLocalPersistenceListener theLocalPersistenceListener) {
		this.localPersistenceListener = theLocalPersistenceListener;
	}

	/**
	 * Getter of <code>storeMedias</code> property.
	 * 
	 * @return <code>storeMedias</code> property value.
	 * @see #storeMedias
	 */

	public List<SAESCommonDestinations> getStoreMedias() {
		synchronized (this.storeMedias) {
			return new ArrayList<SAESCommonDestinations>(this.storeMedias);
		}
	}

	/**
	 * Getter of <code>rejectedMedias</code> property.
	 * 
	 * @return <code>rejectedMedias</code> property value.
	 * @see #rejectedMedias
	 */

	public List<SAESCommonDestinations> getRejectedMedias() {

		synchronized (this.rejectedMedias) {

			return new ArrayList<SAESCommonDestinations>(this.rejectedMedias);
		}
	}

	/**
	 * Getter of <code>key</code> property.
	 * 
	 * @return <code>key</code> property value.
	 * @see #key
	 */

	public SecretKey getKey() {
		return this.key;
	}

	/**
	 * Sets the field used to perform encryption/decryption operations on files.
	 * 
	 * @param key
	 *            the new field used to perform encryption/decryption operations
	 *            on files
	 */
	public void setKey(SecretKey key) {
		this.key = key;
	}

	/**
	 * Returns the common destination where the application is running.
	 * 
	 * @return the default destinations
	 */
	public SAESCommonDestinations getDefaultDestinations() {
		synchronized (this.storeMedias) {
			return this.storeMedias.iterator().next();
		}
	}

	/**
	 * Method used to encrypt data to be recorded in the memory card.
	 * 
	 * @param data
	 *            the data
	 * @return the byte[]
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 */
	public byte[] encryptData(byte[] data)
			throws CryptographicRuntimeException, CryptographicStaticException {
		return EncryptDecryptUtil.encryptData(data, this.key);
	}

	/**
	 * Method used to decrypt data comes from the card.
	 * 
	 * @param data
	 *            the data
	 * @return the byte[]
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */
	public byte[] decryptData(byte[] data) throws CryptographicStaticException,
			CryptographicRuntimeException {
		return EncryptDecryptUtil.decryptData(data, this.key);
	}

	/**
	 * Method used to persist a local file that indicates the state on file
	 * system of the machine.
	 * 
	 * @param status
	 *            the status
	 * @param resources
	 *            the resources
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @see com.smartmatic.waffle.localpersistence.environment.FileSystemStatus
	 */
	public void saveFileSystemStatus(FileSystemStatus status,
			ResourceSubjectTOBackup... resources) throws IOException {
		File unsaved = SAESCommonDestinations.getInstance().unsaved;
		switch (status) {
		case SAVED:
			if (unsaved.exists()) {
				FileUtils.forceDelete(unsaved);
			}
			break;
		case UNSAVED:
			String data = null;
			StringBuilder builder = null;
			if (!unsaved.exists()) {
				builder = new StringBuilder();
				int k = 0;
				for (ResourceSubjectTOBackup resourceSubjectTOBackup : resources) {
					builder.append(resourceSubjectTOBackup.name());
					k++;
					if (k < resources.length) {
						builder.append(";");
					}
				}
				data = builder.toString();
			} else {
				Set<ResourceSubjectTOBackup> set = new HashSet<ResourceSubjectTOBackup>(
						Arrays.asList(this.getUnsavedResources()));
				String content = FileUtils.readLines(unsaved).iterator().next();
				builder = new StringBuilder(content);
				for (ResourceSubjectTOBackup resourceSubjectTOBackup : resources) {
					if (!set.contains(resourceSubjectTOBackup)) {
						builder.append(";");
						builder.append(resourceSubjectTOBackup.name());
					}
				}
				data = builder.toString();
			}
			FileUtils.writeStringToFile(unsaved, data);
			break;
		default:
			break;
		}
	}

	/**
	 * Method used to get which resources presents an unsaved status.
	 * 
	 * @return the unsaved resources
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @see com.smartmatic.waffle.localpersistence.environment.FileSystemStatus
	 */
	public ResourceSubjectTOBackup[] getUnsavedResources() throws IOException {
		List<String> list = FileUtils.readLines(SAESCommonDestinations
				.getInstance().unsaved);
		List<ResourceSubjectTOBackup> resources = new ArrayList<ResourceSubjectTOBackup>();
		for (String string : list) {
			String[] splitted = string.split(";");
			for (String string2 : splitted) {
				ResourceSubjectTOBackup resource = Enum.valueOf(
						ResourceSubjectTOBackup.class, string2);
				resources.add(resource);
			}
		}
		return resources.toArray(new ResourceSubjectTOBackup[0]);
	}
}
