/**
 * SMTTLogger.java
 *
 * $Revision: 1.18 $ ($Date: 2011-04-06 04:43:02 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.log;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Map;
import java.util.ResourceBundle;

import javax.crypto.SecretKey;

import org.apache.log4j.Level;
import org.apache.log4j.MDC;
import org.apache.log4j.xml.DOMConfigurator;

/**
 * Log4j Logger wrapper providing extended logging services.
 * <p>
 * All logging methods in this class use a resource bundle to display messages.
 * <p>
 * All logging methods in this class set the following <code>MDC</code> mappings
 * before logging any message:
 * <ul>
 * <li>SMTTaddress - IP address of the local host</li>
 * <li>SMTTmessageCode - the provided log message resource bundle key</li>
 * </ul>
 * <p>
 * All logging methods in this class set the following <code>MDC</code> mappings
 * when the specified condition applies:
 * <ul>
 * <li>SMTTsessionID - current session ID, set when a session is being tracked
 * by the current thread</li>
 * <li>SMTTwildcard1 - wildcard1, set when provided</li>
 * <li>SMTTwildcard2 - wildcard2, set when provided</li>
 * <li>SMTTwildcard3 - wildcard3, set when provided</li>
 * <li>SMTTclientCode - clientCode, set when provided</li>
 * </ul>
 * <p>
 * In addition, if a ContextProvider is given at the time of logger creation,
 * its <code>getContext</code> method will be called with appropriate parameters
 * and the returned context will be stored as mappings in the current thread's
 * <code>MDC</code>. If the returned context contains a key equal to one of the
 * previously mentioned mappings (e.g. SMTTaddress), that mapping will be
 * overwritten.
 * 
 * @see org.apache.log4j.Logger
 * @see org.apache.log4j.MDC
 * 
 * @author Daniel Izquierdo
 */
public class SMTTLogger {

	/**
	 * The resource bundle where all the localized messages are obtained.
	 */

	private static ResourceBundle bundle = null;

	/**
	 * The station IP Address.
	 */

	private static String address;

	/**
	 * The inner logger responsible of performing the actual logging operations.
	 */

	private final org.apache.log4j.Logger logger;

	/**
	 * The object responsible of provide the additional parameters required to
	 * add in every logging statement.
	 */

	private final ContextProvider contextProvider;

	/**
	 * SecretKey to encrypt the logs
	 * */

	private static SecretKey key;

	/**
	 * Getter of <code>key</code> property.
	 * 
	 * @return <code>key</code> property value.
	 * @see #key
	 */
	public static SecretKey getKey() {
		return key;
	}

	/**
	 * Setter of <code>key</code> property.
	 * 
	 * @param key
	 *            The <code>key</code> to set.
	 * @see #key
	 */
	public static void setKey(SecretKey key) {
		SMTTLogger.key = key;
	}

	/**
	 * Create a Logger instance using the Log4j logger with the given name.
	 * 
	 * @param name
	 *            name of the Log4j logger to use
	 * @param contextProvider
	 *            The context provider to use
	 */

	private SMTTLogger(String name, ContextProvider contextProvider) {
		// Creates the inner logger
		this.logger = org.apache.log4j.Logger.getLogger(name);
		this.contextProvider = contextProvider;

		this.logger.setResourceBundle(SMTTLogger.bundle);

		// Initialize the address field the first time a logger is created
		if (SMTTLogger.address == null) {
			try {
				SMTTLogger.address = InetAddress.getLocalHost()
						.getHostAddress();
			} catch (UnknownHostException e) {
				SMTTLogger.address = "127.0.0.1";
			}
		}
	}

	/**
	 * Get a new Logger instance using the Log4j logger with the given name.
	 * 
	 * @param name
	 *            name of the Log4j logger to use
	 * @return new logger instance
	 */

	public static SMTTLogger getLogger(String name) {
		return SMTTLogger.getLogger(name, null);
	}

	/**
	 * Get a new Logger instance using the Log4j logger with the given name.
	 * 
	 * @param name
	 *            name of the Log4j logger to use
	 * @param contextProvider
	 *            context provider to be used by this logger
	 * @return new logger instance
	 */

	public static SMTTLogger getLogger(String name,
			ContextProvider contextProvider) {
		return new SMTTLogger(name, contextProvider);
	}

	/**
	 * Get a new Logger instance using the Log4j logger with the name of the
	 * given class.
	 * 
	 * @param clazz
	 *            class whose name will be
	 * 
	 * @return new logger instance
	 */

	public static <T> SMTTLogger getLogger(Class<T> clazz) {
		return SMTTLogger.getLogger(clazz, null);
	}

	/**
	 * Get a new Logger instance using the Log4j logger with the given name.
	 * 
	 * @param name
	 *            name of the Log4j logger to use
	 * @param contextProvider
	 *            context provider to be used by this logger
	 * @return new logger instance
	 */

	public static <T> SMTTLogger getLogger(Class<T> clazz,
			ContextProvider contextProvider) {
		return SMTTLogger.getLogger(clazz.getName(), contextProvider);
	}

	/**
	 * Set the resource bundle to be used for new loggers. All loggers returned
	 * by {@link #getLogger(Class)} or {@link #getLogger(String)} after this
	 * method is called will use the provided resource bundle.
	 * 
	 * @param resourceBundle
	 *            the resource bundle to be used by all new loggers
	 */

	public static void setResourceBundle(ResourceBundle resourceBundle) {
		SMTTLogger.bundle = resourceBundle;
	}

	/**
	 * Begin a new logger session. A session ID is generated and assigned to the
	 * current thread. The session ID is thread-specific: multiple threads may
	 * use the same logger instance to track their sessions concurrently.
	 * <p>
	 * As a result of calling this method, this thread's <i>Mapped Diagnostic
	 * Context</i> will contain a mapping with key 'sessionId' and the generated
	 * session ID as its value.
	 * 
	 * @return the generated session ID
	 * @see org.apache.log4j.MDC
	 */
	public String beginSession() {
		String sessionId = generateSessionID();
		this.resumeSession(sessionId);
		return sessionId;
	}

	/**
	 * Resume a logger session. The given session ID is assigned to the current
	 * thread.
	 * <p>
	 * As a result of calling this method, this thread's <i>Mapped Diagnostic
	 * Context</i> will contain a mapping with key 'sessionId' and the provided
	 * session ID as its value.
	 * 
	 * @param sessionId
	 *            the ID of the session the current thread is tracking
	 * @see org.apache.log4j.MDC
	 */

	public void resumeSession(String sessionId) {
		MDC.put("SMTTsessionId", sessionId);
	}

	/**
	 * End the currently tracked logger session.
	 * <p>
	 * As a result of calling this method, this thread's <i>Mapped Diagnostic
	 * Context</i> will contain no mapping with the key 'sessionId'.
	 * 
	 * @see org.apache.log4j.MDC
	 */
	public void endSession() {
		MDC.remove("SMTTsessionId");
	}

	/**
	 * Generate a new session ID.
	 * 
	 * @return the generated session ID
	 */
	private static synchronized String generateSessionID() {
		return (new java.rmi.server.UID()) + "@" + SMTTLogger.address;
	}

	/**
	 * Log a debug level message
	 * 
	 * @param messageCode
	 *            The message code to log The message code to log
	 * 
	 */
	public void debug(String messageCode) {
		this.doLog(Level.DEBUG, null, null, null, null, messageCode, null, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log The message code to log
	 * 
	 */
	public void debug(Object wildcard1, String messageCode) {
		this.doLog(Level.DEBUG, wildcard1, null, null, null, messageCode, null,
				null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log The message code to log
	 * 
	 */
	public void debug(Object wildcard1, Object wildcard2, String messageCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, null, null, messageCode,
				null, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log The message code to log
	 * 
	 */
	public void debug(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, wildcard3, null,
				messageCode, null, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param params
	 *            log message's parameters
	 * 
	 * 
	 */
	public void debug(String messageCode, Object[] params) {
		this.doLog(Level.DEBUG, null, null, null, null, messageCode, params,
				null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void debug(Object wildcard1, String messageCode, Object[] params) {
		this.doLog(Level.DEBUG, wildcard1, null, null, null, messageCode,
				params, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2 Optional log information
	 *            object #2
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void debug(Object wildcard1, Object wildcard2, String messageCode,
			Object[] params) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, null, null, messageCode,
				params, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void debug(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object[] params) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, wildcard3, null,
				messageCode, params, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void debug(Throwable throwable, String messageCode) {
		this.doLog(Level.DEBUG, null, null, null, null, messageCode, null,
				throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void debug(Object wildcard1, Throwable throwable, String messageCode) {
		this.doLog(Level.DEBUG, wildcard1, null, null, null, messageCode, null,
				throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void debug(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, null, null, messageCode,
				null, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */

	public void debug(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, wildcard3, null,
				messageCode, null, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void debug(Throwable throwable, String messageCode, Object[] params) {
		this.doLog(Level.DEBUG, null, null, null, null, messageCode, params,
				throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void debug(Object wildcard1, Throwable throwable,
			String messageCode, Object[] params) {
		this.doLog(Level.DEBUG, wildcard1, null, null, null, messageCode,
				params, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void debug(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object[] params) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, null, null, messageCode,
				params, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void debug(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object[] params) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, wildcard3, null,
				messageCode, params, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object Client code Object
	 */

	public void debug(String messageCode, Object clientCode) {
		this.doLog(Level.DEBUG, null, null, null, clientCode, messageCode,
				null, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, String messageCode, Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, null, null, clientCode, messageCode,
				null, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, Object wildcard2, String messageCode,
			Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, null, clientCode,
				messageCode, null, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, null, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.DEBUG, null, null, null, clientCode, messageCode,
				params, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, null, null, clientCode, messageCode,
				params, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, Object wildcard2, String messageCode,
			Object[] params, Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, null, clientCode,
				messageCode, params, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, params, null);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Throwable throwable, String messageCode, Object clientCode) {
		this.doLog(Level.DEBUG, null, null, null, clientCode, messageCode,
				null, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, Throwable throwable,
			String messageCode, Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, null, null, clientCode, messageCode,
				null, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, null, clientCode,
				messageCode, null, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, null, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Throwable throwable, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.DEBUG, null, null, null, clientCode, messageCode,
				params, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, Throwable throwable,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, null, null, clientCode, messageCode,
				params, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, null, clientCode,
				messageCode, params, throwable);
	}

	/**
	 * Log a debug level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void debug(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.DEBUG, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, params, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 */

	public void info(String messageCode) {
		this.doLog(Level.INFO, null, null, null, null, messageCode, null, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 */

	public void info(Object wildcard1, String messageCode) {
		this.doLog(Level.INFO, wildcard1, null, null, null, messageCode, null,
				null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 */

	public void info(Object wildcard1, Object wildcard2, String messageCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, null, null, messageCode,
				null, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 */

	public void info(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, wildcard3, null,
				messageCode, null, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void info(String messageCode, Object[] params) {
		this.doLog(Level.INFO, null, null, null, null, messageCode, params,
				null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void info(Object wildcard1, String messageCode, Object[] params) {
		this.doLog(Level.INFO, wildcard1, null, null, null, messageCode,
				params, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void info(Object wildcard1, Object wildcard2, String messageCode,
			Object[] params) {
		this.doLog(Level.INFO, wildcard1, wildcard2, null, null, messageCode,
				params, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void info(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object[] params) {
		this.doLog(Level.INFO, wildcard1, wildcard2, wildcard3, null,
				messageCode, params, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */

	public void info(Throwable throwable, String messageCode) {
		this.doLog(Level.INFO, null, null, null, null, messageCode, null,
				throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */

	public void info(Object wildcard1, Throwable throwable, String messageCode) {
		this.doLog(Level.INFO, wildcard1, null, null, null, messageCode, null,
				throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */

	public void info(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, null, null, messageCode,
				null, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */

	public void info(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, wildcard3, null,
				messageCode, null, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void info(Throwable throwable, String messageCode, Object[] params) {
		this.doLog(Level.INFO, null, null, null, null, messageCode, params,
				throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void info(Object wildcard1, Throwable throwable, String messageCode,
			Object[] params) {
		this.doLog(Level.INFO, wildcard1, null, null, null, messageCode,
				params, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void info(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object[] params) {
		this.doLog(Level.INFO, wildcard1, wildcard2, null, null, messageCode,
				params, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */

	public void info(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object[] params) {
		this.doLog(Level.INFO, wildcard1, wildcard2, wildcard3, null,
				messageCode, params, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void info(String messageCode, Object clientCode) {
		this.doLog(Level.INFO, null, null, null, clientCode, messageCode, null,
				null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void info(Object wildcard1, String messageCode, Object clientCode) {
		this.doLog(Level.INFO, wildcard1, null, null, clientCode, messageCode,
				null, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void info(Object wildcard1, Object wildcard2, String messageCode,
			Object clientCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, null, clientCode,
				messageCode, null, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void info(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object clientCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, null, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void info(String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.INFO, null, null, null, clientCode, messageCode,
				params, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void info(Object wildcard1, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.INFO, wildcard1, null, null, clientCode, messageCode,
				params, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void info(Object wildcard1, Object wildcard2, String messageCode,
			Object[] params, Object clientCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, null, clientCode,
				messageCode, params, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void info(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, params, null);
	}

	/**
	 * Log an info level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void info(Throwable throwable, String messageCode, Object clientCode) {
		this.doLog(Level.INFO, null, null, null, clientCode, messageCode, null,
				throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void info(Object wildcard1, Throwable throwable, String messageCode,
			Object clientCode) {
		this.doLog(Level.INFO, wildcard1, null, null, clientCode, messageCode,
				null, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void info(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object clientCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, null, clientCode,
				messageCode, null, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void info(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object clientCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, null, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void info(Throwable throwable, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.INFO, null, null, null, clientCode, messageCode,
				params, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void info(Object wildcard1, Throwable throwable, String messageCode,
			Object[] params, Object clientCode) {
		this.doLog(Level.INFO, wildcard1, null, null, clientCode, messageCode,
				params, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void info(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, null, clientCode,
				messageCode, params, throwable);
	}

	/**
	 * Log an info level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void info(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.INFO, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, params, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 */
	public void warn(String messageCode) {
		this.doLog(Level.WARN, null, null, null, null, messageCode, null, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 */
	public void warn(Object wildcard1, String messageCode) {
		this.doLog(Level.WARN, wildcard1, null, null, null, messageCode, null,
				null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 */
	public void warn(Object wildcard1, Object wildcard2, String messageCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, null, null, messageCode,
				null, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 */
	public void warn(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, wildcard3, null,
				messageCode, null, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void warn(String messageCode, Object[] params) {
		this.doLog(Level.WARN, null, null, null, null, messageCode, params,
				null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void warn(Object wildcard1, String messageCode, Object[] params) {
		this.doLog(Level.WARN, wildcard1, null, null, null, messageCode,
				params, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void warn(Object wildcard1, Object wildcard2, String messageCode,
			Object[] params) {
		this.doLog(Level.WARN, wildcard1, wildcard2, null, null, messageCode,
				params, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void warn(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object[] params) {
		this.doLog(Level.WARN, wildcard1, wildcard2, wildcard3, null,
				messageCode, params, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void warn(Throwable throwable, String messageCode) {
		this.doLog(Level.WARN, null, null, null, null, messageCode, null,
				throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void warn(Object wildcard1, Throwable throwable, String messageCode) {
		this.doLog(Level.WARN, wildcard1, null, null, null, messageCode, null,
				throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void warn(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, null, null, messageCode,
				null, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void warn(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, wildcard3, null,
				messageCode, null, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void warn(Throwable throwable, String messageCode, Object[] params) {
		this.doLog(Level.WARN, null, null, null, null, messageCode, params,
				throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void warn(Object wildcard1, Throwable throwable, String messageCode,
			Object[] params) {
		this.doLog(Level.WARN, wildcard1, null, null, null, messageCode,
				params, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void warn(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object[] params) {
		this.doLog(Level.WARN, wildcard1, wildcard2, null, null, messageCode,
				params, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void warn(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object[] params) {
		this.doLog(Level.WARN, wildcard1, wildcard2, wildcard3, null,
				messageCode, params, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(String messageCode, Object clientCode) {
		this.doLog(Level.WARN, null, null, null, clientCode, messageCode, null,
				null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Object wildcard1, String messageCode, Object clientCode) {
		this.doLog(Level.WARN, wildcard1, null, null, clientCode, messageCode,
				null, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Object wildcard1, Object wildcard2, String messageCode,
			Object clientCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, null, clientCode,
				messageCode, null, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object clientCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, null, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.WARN, null, null, null, clientCode, messageCode,
				params, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Object wildcard1, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.WARN, wildcard1, null, null, clientCode, messageCode,
				params, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */

	public void warn(Object wildcard1, Object wildcard2, String messageCode,
			Object[] params, Object clientCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, null, clientCode,
				messageCode, params, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, params, null);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Throwable throwable, String messageCode, Object clientCode) {
		this.doLog(Level.WARN, null, null, null, clientCode, messageCode, null,
				throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Object wildcard1, Throwable throwable, String messageCode,
			Object clientCode) {
		this.doLog(Level.WARN, wildcard1, null, null, clientCode, messageCode,
				null, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object clientCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, null, clientCode,
				messageCode, null, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */

	public void warn(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object clientCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, null, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Throwable throwable, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.WARN, null, null, null, clientCode, messageCode,
				params, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Object wildcard1, Throwable throwable, String messageCode,
			Object[] params, Object clientCode) {
		this.doLog(Level.WARN, wildcard1, null, null, clientCode, messageCode,
				params, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, null, clientCode,
				messageCode, params, throwable);
	}

	/**
	 * Log a warn level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void warn(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.WARN, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, params, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 */
	public void error(String messageCode) {
		this.doLog(Level.ERROR, null, null, null, null, messageCode, null, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 */
	public void error(Object wildcard1, String messageCode) {
		this.doLog(Level.ERROR, wildcard1, null, null, null, messageCode, null,
				null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 */
	public void error(Object wildcard1, Object wildcard2, String messageCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, null, null, messageCode,
				null, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 */

	public void error(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, wildcard3, null,
				messageCode, null, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void error(String messageCode, Object[] params) {
		this.doLog(Level.ERROR, null, null, null, null, messageCode, params,
				null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void error(Object wildcard1, String messageCode, Object[] params) {
		this.doLog(Level.ERROR, wildcard1, null, null, null, messageCode,
				params, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void error(Object wildcard1, Object wildcard2, String messageCode,
			Object[] params) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, null, null, messageCode,
				params, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void error(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object[] params) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, wildcard3, null,
				messageCode, params, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void error(Throwable throwable, String messageCode) {
		this.doLog(Level.ERROR, null, null, null, null, messageCode, null,
				throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void error(Object wildcard1, Throwable throwable, String messageCode) {
		this.doLog(Level.ERROR, wildcard1, null, null, null, messageCode, null,
				throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void error(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, null, null, messageCode,
				null, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void error(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, wildcard3, null,
				messageCode, null, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void error(Throwable throwable, String messageCode, Object[] params) {
		this.doLog(Level.ERROR, null, null, null, null, messageCode, params,
				throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void error(Object wildcard1, Throwable throwable,
			String messageCode, Object[] params) {
		this.doLog(Level.ERROR, wildcard1, null, null, null, messageCode,
				params, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void error(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object[] params) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, null, null, messageCode,
				params, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void error(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object[] params) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, wildcard3, null,
				messageCode, params, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(String messageCode, Object clientCode) {
		this.doLog(Level.ERROR, null, null, null, clientCode, messageCode,
				null, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, String messageCode, Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, null, null, clientCode, messageCode,
				null, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, Object wildcard2, String messageCode,
			Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, null, clientCode,
				messageCode, null, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, null, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.ERROR, null, null, null, clientCode, messageCode,
				params, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, null, null, clientCode, messageCode,
				params, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, Object wildcard2, String messageCode,
			Object[] params, Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, null, clientCode,
				messageCode, params, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, params, null);
	}

	/**
	 * Log an error level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Throwable throwable, String messageCode, Object clientCode) {
		this.doLog(Level.ERROR, null, null, null, clientCode, messageCode,
				null, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, Throwable throwable,
			String messageCode, Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, null, null, clientCode, messageCode,
				null, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, null, clientCode,
				messageCode, null, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, null, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Throwable throwable, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.ERROR, null, null, null, clientCode, messageCode,
				params, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, Throwable throwable,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, null, null, clientCode, messageCode,
				params, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, null, clientCode,
				messageCode, params, throwable);
	}

	/**
	 * Log an error level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void error(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.ERROR, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, params, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 */
	public void fatal(String messageCode) {
		this.doLog(Level.FATAL, null, null, null, null, messageCode, null, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 */
	public void fatal(Object wildcard1, String messageCode) {
		this.doLog(Level.FATAL, wildcard1, null, null, null, messageCode, null,
				null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 */
	public void fatal(Object wildcard1, Object wildcard2, String messageCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, null, null, messageCode,
				null, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 */
	public void fatal(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, wildcard3, null,
				messageCode, null, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void fatal(String messageCode, Object[] params) {
		this.doLog(Level.FATAL, null, null, null, null, messageCode, params,
				null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void fatal(Object wildcard1, String messageCode, Object[] params) {
		this.doLog(Level.FATAL, wildcard1, null, null, null, messageCode,
				params, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void fatal(Object wildcard1, Object wildcard2, String messageCode,
			Object[] params) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, null, null, messageCode,
				params, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void fatal(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object[] params) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, wildcard3, null,
				messageCode, params, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void fatal(Throwable throwable, String messageCode) {
		this.doLog(Level.FATAL, null, null, null, null, messageCode, null,
				throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void fatal(Object wildcard1, Throwable throwable, String messageCode) {
		this.doLog(Level.FATAL, wildcard1, null, null, null, messageCode, null,
				throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void fatal(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, null, null, messageCode,
				null, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 */
	public void fatal(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, wildcard3, null,
				messageCode, null, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void fatal(Throwable throwable, String messageCode, Object[] params) {
		this.doLog(Level.FATAL, null, null, null, null, messageCode, params,
				throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void fatal(Object wildcard1, Throwable throwable,
			String messageCode, Object[] params) {
		this.doLog(Level.FATAL, wildcard1, null, null, null, messageCode,
				params, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void fatal(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object[] params) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, null, null, messageCode,
				params, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 */
	public void fatal(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object[] params) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, wildcard3, null,
				messageCode, params, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(String messageCode, Object clientCode) {
		this.doLog(Level.FATAL, null, null, null, clientCode, messageCode,
				null, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, String messageCode, Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, null, null, clientCode, messageCode,
				null, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, Object wildcard2, String messageCode,
			Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, null, clientCode,
				messageCode, null, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, null, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.FATAL, null, null, null, clientCode, messageCode,
				params, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, null, null, clientCode, messageCode,
				params, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, Object wildcard2, String messageCode,
			Object[] params, Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, null, clientCode,
				messageCode, params, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, Object wildcard2, Object wildcard3,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, params, null);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Throwable throwable, String messageCode, Object clientCode) {
		this.doLog(Level.FATAL, null, null, null, clientCode, messageCode,
				null, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, Throwable throwable,
			String messageCode, Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, null, null, clientCode, messageCode,
				null, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, null, clientCode,
				messageCode, null, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, null, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Throwable throwable, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.FATAL, null, null, null, clientCode, messageCode,
				params, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, Throwable throwable,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, null, null, clientCode, messageCode,
				params, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, Object wildcard2, Throwable throwable,
			String messageCode, Object[] params, Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, null, clientCode,
				messageCode, params, throwable);
	}

	/**
	 * Log a very severe level message
	 * 
	 * @param wildcard1
	 *            Optional log information object #1
	 * @param wildcard2
	 *            Optional log information object #2
	 * @param wildcard3
	 *            Optional log information object #3
	 * @param throwable
	 *            Throwable to log
	 * @param messageCode
	 *            The message code to log
	 * @param params
	 *            log message's parameters
	 * @param clientCode
	 *            Client code Object
	 */
	public void fatal(Object wildcard1, Object wildcard2, Object wildcard3,
			Throwable throwable, String messageCode, Object[] params,
			Object clientCode) {
		this.doLog(Level.FATAL, wildcard1, wildcard2, wildcard3, clientCode,
				messageCode, params, throwable);
	}

	/**
	 * Perform the actual logging.
	 * 
	 * @param level
	 *            log message's level
	 * @param wildcard1
	 *            optional log information object #1
	 * @param wildcard2
	 *            optional log information object #2
	 * @param wildcard3
	 *            optional log information object #3
	 * @param key
	 *            log message's resource bundle key
	 * @param params
	 *            log message's parameters
	 * @param throwable
	 *            Throwable to log
	 * 
	 * @see org.apache.log4j.MDC
	 */
	private void doLog(Level level, Object wildcard1, Object wildcard2,
			Object wildcard3, Object clientCode, String key, Object[] params,
			Throwable throwable) {
		/*
		 * avoid setting MDC mappings and making a log call when not necessary
		 * 
		 * l7dlog() will make the exact same check, but we can't avoid it unless
		 * we make a forcedLog() call instead and implement everything else
		 * ourselves
		 */

		if (!this.logger.isEnabledFor(level)) {
			return;
		}

		/* context setup */

		Map<String, Object> context = null;

		if (this.contextProvider != null) {
			context = this.contextProvider.getContext(level, key);
		}

		/* MDC setup */

		MDC.put("SMTTaddress", SMTTLogger.address);
		MDC.put("SMTTmessageCode", key);

		if (wildcard1 != null) {
			MDC.put("SMTTwildcard1", wildcard1);
		}
		if (wildcard2 != null) {
			MDC.put("SMTTwildcard2", wildcard2);
		}
		if (wildcard3 != null) {
			MDC.put("SMTTwildcard3", wildcard3);
		}
		if (clientCode != null) {
			MDC.put("SMTTclientCode", clientCode);
		}

		if (context != null) {
			for (String ckey : context.keySet()) {
				MDC.put(ckey, context.get(ckey));
			}
		}

		/* log call */

		this.logger.l7dlog(level, key, params, throwable);

		/* MDC cleanup */

		MDC.remove("SMTTaddress");
		MDC.remove("SMTTmessageCode");

		if (wildcard1 != null) {
			MDC.remove("SMTTwildcard1");
		}
		if (wildcard2 != null) {
			MDC.remove("SMTTwildcard2");
		}
		if (wildcard3 != null) {
			MDC.remove("SMTTwildcard3");
		}
		if (clientCode != null) {
			MDC.remove("SMTTclientCode");
		}

		if (context != null) {
			for (String ckey : context.keySet()) {
				MDC.remove(ckey);
			}
		}
	}

	/**
	 * Read the configuration file <code>pathFile</code> if it exists. Moreover,
	 * a thread will be created that will periodically check if
	 * <code>configFilename</code> has been created or modified. The period is
	 * determined by the <code>delay</code> argument. If a change or file
	 * creation is detected, then <code>configFilename</code> is read to
	 * configure log4j. It uses DEFAULT_DELAY
	 * 
	 * @param configFilename
	 *            A log4j configuration file in XML format.
	 */
	public static void configureLoggingEnvironment(String pathFile) {
		DOMConfigurator.configureAndWatch(pathFile);
	}

	public void add(String key, Object parameter) {
		MDC.put(key, parameter);
	}

	public void remove(String key) {
		MDC.remove(key);
	}
}