/**
 * FilePathConfigurableAppender.java
 *
 * $Revision: 1.4 $ ($Date: 2011-04-06 04:43:03 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.log.custom.appender;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.apache.log4j.AppenderSkeleton;
import org.apache.log4j.spi.LoggingEvent;

/**
 * File path configurable log appender
 * 
 * @author Carlos Lameda (carlos.lameda@smartmatic.com)
 * @version 1.0 (Nov 11, 2009)
 */
public class FilePathConfigurableAppender extends AppenderSkeleton {

	/**
	 * Pre create logging directory if it does not exist
	 */
	private boolean preCreateLoggingDirectory = false;

	/**
	 * File full path
	 */
	private String fullPath;

	/**
	 * Log file maximum size
	 */
	private int maxFileSize = 3072000;

	/**
	 * Provider use to log messages inside a file
	 */
	private LoggingPathProvider pathProvider;

	/**
	 * Getter of <code>preCreateLoggingDirectory</code> property.
	 * 
	 * @return <code>preCreateLoggingDirectory</code> property value.
	 * @see #preCreateLoggingDirectory
	 */
	public boolean isPreCreateLoggingDirectory() {
		return this.preCreateLoggingDirectory;
	}

	/**
	 * Setter of <code>preCreateLoggingDirectory</code> property.
	 * 
	 * @param preCreateLoggingDirectory
	 *            The <code>preCreateLoggingDirectory</code> to set.
	 * @see #preCreateLoggingDirectory
	 */
	public void setPreCreateLoggingDirectory(boolean preCreateLoggingDirectory) {
		this.preCreateLoggingDirectory = preCreateLoggingDirectory;
	}

	/**
	 * Getter of <code>fullPath</code> property.
	 * 
	 * @return <code>fullPath</code> property value.
	 * @see #fullPath
	 */
	public String getFullPath() {
		return this.fullPath;
	}

	/**
	 * Setter of <code>fullPath</code> property.
	 * 
	 * @param fullPath
	 *            The <code>fullPath</code> to set.
	 * @see #fullPath
	 */
	public void setFullPath(String fullPath) {
		this.fullPath = fullPath;
	}

	/**
	 * Getter of <code>maxFileSize</code> property.
	 * 
	 * @return <code>maxFileSize</code> property value.
	 * @see #maxFileSize
	 */
	public int getMaxFileSize() {
		return this.maxFileSize;
	}

	/**
	 * Setter of <code>maxFileSize</code> property.
	 * 
	 * @param maxFileSize
	 *            The <code>maxFileSize</code> to set.
	 * @see #maxFileSize
	 */
	public void setMaxFileSize(int maxFileSize) {
		this.maxFileSize = maxFileSize;
	}

	/**
	 * Getter of <code>pathProvider</code> property.
	 * 
	 * @return <code>pathProvider</code> property value.
	 * @see #pathProvider
	 */
	public LoggingPathProvider getPathProvider() {
		return this.pathProvider;
	}

	/**
	 * Setter of <code>pathProvider</code> property.
	 * 
	 * @param pathProvider
	 *            The <code>pathProvider</code> to set.
	 * @see #pathProvider
	 */
	public void setPathProvider(LoggingPathProvider pathProvider) {
		this.pathProvider = pathProvider;
	}

	/**
	 * Perform actual logging. See also {@link #doAppend
	 * AppenderSkeleton.doAppend} method.
	 * 
	 * @param event
	 *            Logging event
	 */
	@Override
	protected void append(LoggingEvent event) {
		DataOutputStream output;
		String eventLine;
		byte[] eventData;

		try {
			if (checkBeforeAppend()) {
				output = new DataOutputStream(new FileOutputStream(
						this.fullPath));
				eventLine = event.getRenderedMessage();
				eventData = eventLine.getBytes("UTF-8");
				output.write(eventData, 0, eventData.length);
				output.close();
			}
		} catch (IOException exc) {
			// TODO Warn user of in/out exception. Probably inexistent file.
		}
	}

	/**
	 * Check the existence and the file mode to append data
	 * 
	 * @return If the file is ready to be appended
	 * @throws IOException
	 *             Error with the log file
	 */
	private boolean checkBeforeAppend() throws IOException {
		boolean ready = false;
		String fullPathDirectory = convertToFullPath(this.pathProvider
				.getTargetDirectory()), rootDrive, rootCandidate, currentFilePath, fileExtension, fileWithoutExtension;
		File fullPathFile, dirFullNameFile, tempFile;
		int i, lastIndex;
		rootDrive = "";

		/* Finds the root for the path provider's directory */

		for (File f : File.listRoots()) {
			// Beware of this code...
			rootCandidate = f.getCanonicalPath();
			if (fullPathDirectory.startsWith(rootCandidate)) {
				rootDrive = new String(rootCandidate);
				break;
			}
		}

		if (rootDrive != null && rootDrive.length() > 0
				&& (new File(rootDrive)).exists()) {
			currentFilePath = convertToFullPath(
					this.pathProvider.getTargetDirectory(),
					this.pathProvider.getTargetFilename());
			if (!currentFilePath.equals(this.fullPath)) {
				this.fullPath = new String(currentFilePath);
			}
			fullPathFile = new File(this.fullPath);
			dirFullNameFile = fullPathFile.getParentFile();

			if (dirFullNameFile.exists()) {
				/* If directory exists, check whether file must be rolled */
				if (fullPathFile.exists() && this.maxFileSize > 0
						&& fullPathFile.length() > this.maxFileSize) {
					lastIndex = this.fullPath.lastIndexOf('.');
					if (lastIndex == -1) {
						fileExtension = "";
						fileWithoutExtension = new String(this.fullPath);
					} else {
						fileExtension = this.fullPath.substring(lastIndex + 1);
						fileWithoutExtension = this.fullPath.substring(0,
								lastIndex);
					}
					/* Loop until an inexistent file is encountered */
					for (i = 0; true; i++) {
						tempFile = new File(fileWithoutExtension + i
								+ fileExtension);
						if (tempFile.exists()) {
							break;
						}
					}
					fullPathFile.renameTo(tempFile);
				}
				ready = true;
			} else if (this.preCreateLoggingDirectory) {
				/*
				 * If directory doesn't exist, create it if it is allowed to do
				 * so
				 */
				dirFullNameFile.mkdir();
				ready = true;
			}
		}

		return ready;
	}

	/**
	 * Utility method to convert path string to full path
	 * 
	 * @param path
	 *            The filename
	 * @return The full path
	 * @throws IOException
	 *             If path is null
	 */
	public static String convertToFullPath(String path) throws IOException {
		return (new File(path)).getCanonicalPath();
	}

	/**
	 * Utility method to convert parent string and child name string to full
	 * path
	 * 
	 * @param parent
	 *            The parent filename
	 * @param childThe
	 *            child filename
	 * @return The full path
	 * @throws IOException
	 *             If child is null
	 */
	public static String convertToFullPath(String parent, String child)
			throws IOException {
		return (new File(parent, child)).getCanonicalPath();
	}

	/**
	 * Release any resources allocated within the appender such as file handles,
	 * network connections, etc.
	 * <p>
	 * It is a programming error to append to a closed appender.
	 */
	@Override
	public void close() {
		return;
	}

	/**
	 * Configurators call this method to meaning that layout is required, then
	 * the configurator will configure a layout using the configuration
	 * information at its disposal.
	 * 
	 * @return If the class requires layout
	 */
	@Override
	public boolean requiresLayout() {
		return true;
	}

}
