/**
 * FilePathConfigurableRollingFileAppender.java
 *
 * $Revision: 1.3 $ ($Date: 2011-04-06 04:43:03 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.log.custom.appender;

import java.io.File;
import java.io.IOException;

import org.apache.log4j.RollingFileAppender;
import org.apache.log4j.spi.LoggingEvent;

/**
 * FilePathConfigurable extends RollingFileAppender to backup the log files when
 * pathProvider is used
 * 
 * @author Carlos Lameda (carlos.lameda@smartmatic.com)
 * @version 1.0 (Nov 11, 2009)
 */
public class FilePathConfigurableRollingFileAppender extends
		RollingFileAppender {

	/**
	 * Pre-create logging directory if it does not exist
	 */
	private boolean preCreateLoggingDirectory = false;

	/**
	 * If the options have been activated inside the parent class
	 */
	private boolean optionsHaveBeenActivated = false;

	/**
	 * Log file full path
	 */
	private String fullPath;

	/**
	 * Provider use to log messages inside a file
	 */
	private LoggingPathProvider pathProvider;

	/**
	 * Getter of <code>preCreateLoggingDirectory</code> property.
	 * 
	 * @return <code>preCreateLoggingDirectory</code> property value.
	 * @see #preCreateLoggingDirectory
	 */
	public boolean isPreCreateLoggingDirectory() {
		return this.preCreateLoggingDirectory;
	}

	/**
	 * Setter of <code>preCreateLoggingDirectory</code> property.
	 * 
	 * @param preCreateLoggingDirectory
	 *            The <code>preCreateLoggingDirectory</code> to set.
	 * @see #preCreateLoggingDirectory
	 */
	public void setPreCreateLoggingDirectory(boolean preCreateLoggingDirectory) {
		this.preCreateLoggingDirectory = preCreateLoggingDirectory;
	}

	/**
	 * Getter of <code>optionsHaveBeenActivated</code> property.
	 * 
	 * @return <code>optionsHaveBeenActivated</code> property value.
	 * @see #optionsHaveBeenActivated
	 */
	public boolean isOptionsHaveBeenActivated() {
		return this.optionsHaveBeenActivated;
	}

	/**
	 * Setter of <code>optionsHaveBeenActivated</code> property.
	 * 
	 * @param optionsHaveBeenActivated
	 *            The <code>optionsHaveBeenActivated</code> to set.
	 * @see #optionsHaveBeenActivated
	 */
	public void setOptionsHaveBeenActivated(boolean optionsHaveBeenActivated) {
		this.optionsHaveBeenActivated = optionsHaveBeenActivated;
	}

	/**
	 * Getter of <code>fullPath</code> property.
	 * 
	 * @return <code>fullPath</code> property value.
	 * @see #fullPath
	 */
	public String getFullPath() {
		return this.fullPath;
	}

	/**
	 * Setter of <code>fullPath</code> property.
	 * 
	 * @param fullPath
	 *            The <code>fullPath</code> to set.
	 * @see #fullPath
	 */
	public void setFullPath(String fullPath) {
		this.fullPath = fullPath;
	}

	/**
	 * Getter of <code>pathProvider</code> property.
	 * 
	 * @return <code>pathProvider</code> property value.
	 * @see #pathProvider
	 */
	public LoggingPathProvider getPathProvider() {
		return this.pathProvider;
	}

	/**
	 * Setter of <code>pathProvider</code> property.
	 * 
	 * @param pathProvider
	 *            The <code>pathProvider</code> to set.
	 * @see #pathProvider
	 */
	public void setPathProvider(LoggingPathProvider pathProvider) {
		this.pathProvider = pathProvider;
	}

	/**
	 * If the value of <b>File</b> is not <code>null</code>, then
	 * {@link #setFile} is called with the values of <b>File</b> and
	 * <b>Append</b> properties.
	 */
	@Override
	public void activateOptions() {
		try {
			if (this.pathProvider != null) {
				this.fullPath = FilePathConfigurableAppender.convertToFullPath(
						this.pathProvider.getTargetDirectory(),
						this.pathProvider.getTargetFilename());
				super.setFile(this.fullPath);
			}
			if (this.preCreateLoggingDirectory
					|| (new File(this.fullPath)).getParentFile().exists()) {
				super.activateOptions();
				this.optionsHaveBeenActivated = true;
			}
		} catch (IOException exc) {
			// TODO Warn user of in/out exception. Probably inexistent file.
		}
	}

	/**
	 * If the output stream exists and is writable then write a log statement to
	 * the output stream. Otherwise, write a single warning message to
	 * <code>System.err</code>.
	 * 
	 * <p>
	 * The format of the output will depend on this appender's layout.
	 * 
	 * @param event
	 *            Logging event
	 */
	@Override
	public void append(LoggingEvent event) {
		if (this.optionsHaveBeenActivated) {
			super.append(event);
		}
	}

	/**
	 * This method differentiates RollingFileAppender from its super class.
	 * 
	 * @param event
	 *            Logging event
	 */
	@Override
	public void subAppend(LoggingEvent event) {
		if (this.optionsHaveBeenActivated) {
			super.subAppend(event);
		}
	}

	/**
	 * This method determines if there is a sense in attempting to append.
	 * 
	 * @return if there is a sense in attempting to append
	 */
	@Override
	public boolean checkEntryConditions() {
		// Set the general log append variables
		try {
			String currentFilePath;
			if (this.pathProvider != null) {
				currentFilePath = FilePathConfigurableAppender
						.convertToFullPath(
								this.pathProvider.getTargetDirectory(),
								this.pathProvider.getTargetFilename());
				if (!this.fullPath.equals(currentFilePath)) {
					this.fullPath = new String(currentFilePath);
					this.optionsHaveBeenActivated = false;
				}
				if (!this.optionsHaveBeenActivated
						&& (this.preCreateLoggingDirectory || (new File(
								this.fullPath)).getParentFile().exists())) {
					super.activateOptions();
					this.optionsHaveBeenActivated = true;
				}
			}
			return super.checkEntryConditions();
		} catch (IOException e) {
			// TODO Warn user of in/out exception. Probably inexistent file.
			return false;
		}
	}

}
