/**
 * CryptoLayout.java
 *
 * $Revision: 1.6 $ ($Date: 2011-04-06 04:43:01 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.log.custom.layout;

import org.apache.log4j.Layout;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.spi.LoggingEvent;

import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

/**
 * Class transformer used to encrypt / decrypt the the data to the Logger.
 * 
 * @author Daniel Mora daniel.mora@smartmatic.com
 * @version 1.0
 * @since jdk.1.6.0_11
 */
public class CryptoLayout extends Layout {

	/**
	 * Helper for encrypting / decrypting strings.
	 */
	public CryptoLayoutTransformer transformer = null;

	/**
	 * Layout used after the encrypting to format the data.
	 */
	public Layout innerLayout = null;

	/**
	 * Default constructor that assign the defaults transformer and the
	 * innerLayout attributes.
	 */
	public CryptoLayout() {
		try {
			this.transformer = new SmartCryptoLayoutTransformer();
		} catch (CryptographicStaticException e) {
			e.printStackTrace();
		} catch (CryptographicRuntimeException e) {
			e.printStackTrace();
		}
		this.innerLayout = new PatternLayout("%-5p [%t]: %m%n");
	}

	/**
	 * Full constructor that receives the values to be assigned to the
	 * transformer and the innerLayout attributes.
	 * 
	 * @param innerLayout
	 *            The <code>innerLayout</code> to set.
	 * @param transformer
	 *            The <code>transformer</code> to set.
	 */
	public CryptoLayout(Layout innerLayout, CryptoLayoutTransformer transformer) {
		this.innerLayout = innerLayout;
		this.transformer = transformer;
	}

	/**
	 * Layout format.
	 * 
	 * @param The
	 *            logging event to format
	 * @returns "text/plain"
	 * 
	 * @see org.apache.log4j.Layout#format(org.apache.log4j.spi.LoggingEvent)
	 */
	@Override
	public String format(LoggingEvent event) {
		return this.transformer.encryptString(this.innerLayout.format(event));
	}

	/**
	 * If the layout handles the throwable object contained within
	 * {@link LoggingEvent}, then the layout should return <code>false</code>.
	 * Otherwise, if the layout ignores throwable object, then the layout should
	 * return <code>true</code>.
	 * 
	 * @see org.apache.log4j.Layout#ignoresThrowable()
	 */
	@Override
	public boolean ignoresThrowable() {
		return true;
	}

	/**
	 * Getter of <code>transformer</code> property.
	 * 
	 * @return <code>transformer</code> property value.
	 * @see #transformer
	 */
	public CryptoLayoutTransformer getTransformer() {
		return this.transformer;
	}

	/**
	 * Setter of <code>transformer</code> property.
	 * 
	 * @param transformer
	 *            The <code>transformer</code> to set.
	 * @see #transformer
	 */
	public void setTransformer(CryptoLayoutTransformer transformer) {
		this.transformer = transformer;
	}

	/**
	 * Getter of <code>innerLayout</code> property.
	 * 
	 * @return <code>innerLayout</code> property value.
	 * @see #innerLayout
	 */
	public Layout getInnerLayout() {
		return this.innerLayout;
	}

	/**
	 * Setter of <code>innerLayout</code> property.
	 * 
	 * @param innerLayout
	 *            The <code>innerLayout</code> to set.
	 * @see #innerLayout
	 */
	public void setInnerLayout(Layout innerLayout) {
		this.innerLayout = innerLayout;
	}

	/**
	 * Activate the options that were previously set with calls to option
	 * setters.
	 * 
	 * @see org.apache.log4j.Layout#activateOptions()
	 */
	@Override
	public void activateOptions() {
		if (this.innerLayout == null) {
			throw new IllegalStateException("The inner layout must be set.");
		}
		if (this.transformer == null) {
			throw new IllegalStateException("The transformer must be set.");
		}
	}

}
