/**
 * CriteriaFactory.java
 * 
 * $Revision: 1.8 $ ($Date: 2011-04-06 04:44:55 $)
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.search.criteria;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.search.criteria.SizeCollectionCriteria.RestrictMode;

/**
 * The <tt>expression</tt> package may be used by applications as a framework
 * for building new kinds of <tt>Criterion</tt>. However, it is intended that
 * most applications will simply use the built-in criterion types via the static
 * factory methods of this class.
 * 
 * @author Ithamar Torres
 */
public final class CriteriaFactory {

	/**
	 * Default constructor. It is private because this is a class used to create
	 * a factory pattern. And this class doesn't have any semantics but the
	 * creation of another classes
	 */

	private CriteriaFactory() {
	}

	/**
	 * Apply an "equal" constraint to the key of the object.
	 * 
	 * @param value
	 *            The value to search
	 * @return The equals identifier criteria
	 */
	public static ISearchCriteria idEq(Object value) {
		return new IdentifierEqCriteria(value);
	}

	/**
	 * Apply an "equal" constraint to the named property
	 * 
	 * @param propertyName
	 *            Property name to search
	 * @param value
	 *            The value to search
	 * @return The equals criteria
	 */
	public static SimpleCriteria eq(String propertyName, Object value) {
		return new EqCriteria(propertyName, value);
	}

	/**
	 * Apply a "like" constraint to the named property
	 * 
	 * @param propertyName
	 *            The property name to search
	 * @param value
	 *            The value to search
	 * @return The like criteria
	 */
	public static SimpleCriteria like(String propertyName, Object value) {
		return new LikeCriteria(propertyName, value);
	}

	/**
	 * Apply a "like" constraint to the named property
	 * 
	 * @param propertyName
	 *            The property name to search
	 * @param value
	 *            The value to search
	 * @param matchMode
	 *            The mode to find the string
	 * @return The like criteria
	 */
	public static SimpleCriteria like(String propertyName, String value,
			MatchCriteriaMode matchMode) {
		return new LikeCriteria(propertyName, value, matchMode);
	}

	/**
	 * A case-insensitive "like", similar to Postgres <tt>ilike</tt> operator
	 * 
	 * @param propertyName
	 *            The property name to search
	 * @param value
	 *            The value to search
	 * @param matchMode
	 *            The mode to find the string
	 * @return The ilike criteria
	 */
	public static ISearchCriteria ilike(String propertyName, String value,
			MatchCriteriaMode matchMode) {
		return new IlikeCriteria(propertyName, value, matchMode);
	}

	/**
	 * A case-insensitive "like", similar to Postgres <tt>ilike</tt> operator
	 * 
	 * @param propertyName
	 *            The property name to search
	 * @param value
	 *            The value to search
	 * @return The ilike criteria
	 */
	public static ISearchCriteria ilike(String propertyName, Object value) {
		return new IlikeCriteria(propertyName, value);
	}

	/**
	 * Apply a "greater than" constraint to the named property
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param value
	 *            The value to compare
	 * @return The greater than criteria
	 */
	public static SimpleCriteria gt(String propertyName, Object value) {
		return new GtCriteria(propertyName, value);
	}

	/**
	 * Apply a "less than" constraint to the named property
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param value
	 *            The value to compare
	 * @return The less than criteria
	 */
	public static SimpleCriteria lt(String propertyName, Object value) {
		return new LtCriteria(propertyName, value);
	}

	/**
	 * Apply a "less than or equal" constraint to the named property
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param value
	 *            The value to compare
	 * @return The less than or equal criteria
	 */
	public static SimpleCriteria le(String propertyName, Object value) {
		return new LeCriteria(propertyName, value);
	}

	/**
	 * Apply a "greater than or equal" constraint to the named property
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param value
	 *            The value to compare
	 * @return The greater than or equal criteria
	 */
	public static SimpleCriteria ge(String propertyName, Object value) {
		return new GeCriteria(propertyName, value);
	}

	/**
	 * Apply a "between" constraint to the named property (inclusive)
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param lo
	 *            The minimum value (inclusive)
	 * @param hi
	 *            The maximum value (inclusive)
	 * @return The between criteria
	 */
	public static ISearchCriteria between(String propertyName, Object lo,
			Object hi) {
		return new BetweenCriteria(propertyName, lo, hi);
	}

	/**
	 * Apply an "in" constraint to the named property
	 * 
	 * @param propertyName
	 *            The property name to search
	 * @param values
	 *            The array of values to find for
	 * @return The in criteria
	 */
	public static ISearchCriteria in(String propertyName, Object[] values) {
		if (values.length == 1) {
			return CriteriaFactory.eq(propertyName, values[0]);
		} else {
			return new InCriteria(propertyName, values);
		}
	}

	/**
	 * Apply an "in" constraint to the named property
	 * 
	 * @param propertyName
	 *            The property name to search
	 * @param values
	 *            The collection of values to find for
	 * @return The in criteria
	 */
	public static ISearchCriteria in(String propertyName, Collection values) {
		if (values.size() == 1) {
			return CriteriaFactory.eq(propertyName, values.iterator().next());
		} else {
			return new InCriteria(propertyName, values.toArray());
		}
	}

	/**
	 * Apply an SQL restriction
	 * 
	 * @param sql
	 *            the query
	 * @return Criterion The criteria with restricted SQL
	 */
	public static ISearchCriteria sql(String sql) {
		return new SqlCriteria(sql);
	}

	/**
	 * Apply an "is null" constraint to the named property
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @return The is null criteria
	 */
	public static ISearchCriteria isNull(String propertyName) {
		return new NullCriteria(propertyName);
	}

	/**
	 * Apply an "equal" constraint to two properties
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param otherPropertyName
	 *            The other property name to compare
	 * @return The equal property criteria
	 */
	public static ISearchCriteria eqProperty(String propertyName,
			String otherPropertyName) {
		return new EqPropertyCriteria(propertyName, otherPropertyName);
	}

	/**
	 * Apply a "less than" constraint to two properties
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param otherPropertyName
	 *            The other property name to compare
	 * @return The less than property criteria
	 */
	public static ISearchCriteria ltProperty(String propertyName,
			String otherPropertyName) {
		return new LtPropertyCriteria(propertyName, otherPropertyName);
	}

	/**
	 * Apply a "less than or equal" constraint to two properties
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param otherPropertyName
	 *            The other property name to compare
	 * @return The less than or equal property criteria
	 */
	public static ISearchCriteria leProperty(String propertyName,
			String otherPropertyName) {
		return new LePropertyCriteria(propertyName, otherPropertyName);
	}

	/**
	 * Apply an "is not null" constraint to the named property
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @return The is not null criteria
	 */
	public static ISearchCriteria isNotNull(String propertyName) {
		return new NotNullCriteria(propertyName);
	}

	/**
	 * Return the conjunction of two expressions
	 * 
	 * @param lhs
	 *            The first expression
	 * @param rhs
	 *            The second expression
	 * @return The conjunction criteria
	 */
	public static ISearchCriteria and(ISearchCriteria lhs, ISearchCriteria rhs) {
		return new AndCriteria(lhs, rhs);
	}

	/**
	 * Return the disjunction of two expressions
	 * 
	 * @param lhs
	 *            The first expression
	 * @param rhs
	 *            The second expression
	 * @return The disjunction criteria
	 */
	public static ISearchCriteria or(ISearchCriteria lhs, ISearchCriteria rhs) {
		return new OrCriteria(lhs, rhs);
	}

	/**
	 * Return the negation of an expression
	 * 
	 * @param expression
	 *            The expression
	 * @return The negated expression criteria
	 */
	public static ISearchCriteria not(ISearchCriteria expression) {
		return new NotCriteria(expression);
	}

	/**
	 * Group expressions together in a single conjunction (A and B and C...)
	 * 
	 * @return The conjunction criteria
	 */
	public static ConjunctionCriteria conjunction() {
		return new ConjunctionCriteria();
	}

	/**
	 * Group expressions together in a single disjunction (A or B or C...)
	 * 
	 * @return The disjunction criteria
	 */
	public static DisjunctionCriteria disjunction() {
		return new DisjunctionCriteria();
	}

	/**
	 * Apply an "equals" constraint to each property in the key set of a
	 * <tt>Map</tt>
	 * 
	 * @param propertyNameValues
	 *            A map from property names to values
	 * @return The all properties equals criteria
	 */
	public static ISearchCriteria allEq(Map propertyNameValues) {
		ConjunctionCriteria conj = CriteriaFactory.conjunction();
		Iterator iter = propertyNameValues.entrySet().iterator();
		while (iter.hasNext()) {
			Map.Entry me = (Map.Entry) iter.next();
			conj.add(CriteriaFactory.eq((String) me.getKey(), me.getValue()));
		}
		return conj;
	}

	/**
	 * Apply an "equal" constraint to a collection valued property by size.
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param size
	 *            The collection size
	 * @return The equal size criteria
	 */
	public static ISearchCriteria collectionSizeEq(String propertyName, int size) {
		return new SizeCollectionCriteria(RestrictMode.EQUAL, propertyName,
				size);
	}

	/**
	 * Apply an "Not equal" constraint to a collection valued property by size.
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param size
	 *            The collection size
	 * @return The equal size criteria
	 */
	public static ISearchCriteria collectionSizeNotEq(String propertyName,
			int size) {
		return new SizeCollectionCriteria(RestrictMode.NOTEQUAL, propertyName,
				size);
	}

	/**
	 * Apply an "in" constraint to the named property based on a provided
	 * sub-query in a SearchExpression form
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param subQuery
	 *            Sub-query to be executed and whose value will be used in a
	 *            WHERE comparison
	 * @return The Sub-query filtered criteria
	 */
	public static ISearchCriteria propertyIn(String propertyName,
			SearchExpression subQuery) {

		return new InSubqueryPropertyCriteria(subQuery, propertyName);

	}

	/**
	 * Apply an "equal" constraint to a collection valued property by size.
	 * 
	 * @param propertyName
	 *            The property name to compare
	 * @param size
	 *            The collection size
	 * @return The equal size criteria
	 */
	public static SizeEqCriteria sizeEq(String propertyName, int size) {
		return new SizeEqCriteria(propertyName, size);
	}
}
