/**
 * RegExprFieldMatcher.java
 *
 * $Revision: 1.6 $ ($Date: 2011-04-06 04:45:13 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.search.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Generates a field-value map for the given class and data provided in an only
 * string parameter. We will call it the <b>search data</b>.
 * 
 * It must be configured using two parameters:
 * <ol>
 * <li>A list of regular expressions that uniquely will match against the search
 * data and will identify the fields contained in</li>
 * <li>A field - regex map that allows identify the data that corresponds to
 * each field</li>
 * </ol>
 * 
 * After be configured a field-value map for looking the class can be generated
 * providing the search data.
 * 
 * @version 1.0 (15/10/2008)
 * @author Carlos Ramirez (cramirez@smartmatic.com)
 */
public class RegExprFieldMatcher {

	/**
	 * A field - regex map that allows identify the data that corresponds to
	 * each field
	 * 
	 * <p>
	 * Example:
	 * </p>
	 * 
	 * <pre>
	 * Map&lt;String, String&gt; m = &lt;b&gt;new&lt;/b&gt; HashMap&lt;String, String&gt;();
	 * m.put(&quot;eventType.code&quot;, &quot;[A-Z]+&quot;);
	 * m.put(&quot;caller.customCode&quot;, &quot;\\d[\\d,]*&quot;);
	 * </pre>
	 */
	private Map<String, String> fieldsRegExpr;

	/**
	 * A list of regular expressions that uniquely will match against the search
	 * data and will identify the fields contained in.
	 * 
	 * <p>
	 * Example:
	 * </p>
	 * 
	 * <pre>
	 * &lt;b&gt;new&lt;/b&gt; String[] {&quot;${eventType.code} ${caller.customCode}&quot;}
	 * </pre>
	 */
	private String[] templates;

	/**
	 * The compiled version for <code>templatesComp</code> array.
	 */
	private Pattern[] templatesComp;

	/**
	 * Stores the fields found in each template
	 */
	private String[][] templatesGroups;

	/**
	 * Regular expression used to identify fields inside templates
	 */
	private Pattern fieldRegEx;

	/**
	 * Regular expression used to identify identifier names inside the ${ ... }
	 */
	private Pattern identifierRegEx;

	/**
	 * Constructs a RegExprSearchExpressionBuilder with the supplied field-regex
	 * and templates
	 * 
	 * @param theFieldsRegExpr
	 *            A field - regex map that allows identify the data that
	 *            corresponds to each field
	 * @param theTemplates
	 *            A list of regular expressions that uniquely will match against
	 *            the search data and will identify the fields contained in.
	 */
	public RegExprFieldMatcher(Map<String, String> theFieldsRegExpr,
			String... theTemplates) {

		this.fieldsRegExpr = theFieldsRegExpr;
		this.templates = theTemplates;
		this.fieldRegEx = Pattern.compile("\\$\\{[a-zA-Z][\\w\\.]*\\}");
		this.identifierRegEx = Pattern.compile("[a-zA-Z][\\w\\.]*");

		this.compileRegularExpressions();

	}

	/**
	 * Creates an efficient representation for regular expressions in order to
	 * be used faster.
	 */
	protected void compileRegularExpressions() {

		// Compile the template patterns
		final int theTemplatesLength = this.templates.length;
		Pattern[] theTemplatesComp = new Pattern[theTemplatesLength];
		this.templatesGroups = new String[theTemplatesLength][];

		for (int i = 0; i < theTemplatesLength; i++) {
			String theTemplate = this.replaceFields(i);
			theTemplatesComp[i] = Pattern.compile(theTemplate);
		}

		this.templatesComp = theTemplatesComp;
		return;

	}

	/**
	 * Replace the fields in template and stores them in an array for group
	 * identification.
	 * 
	 * @param theTemplateIndex
	 *            #templates array index
	 * @return The replaced field or null if theTemplateIndex is out of the
	 *         bounds of templates and templatesGroups
	 * @see #templates
	 * @see #templatesGroups
	 */
	protected String replaceFields(int theTemplateIndex) {
		List<String> fields = new ArrayList<String>();

		if (theTemplateIndex < 0 || theTemplateIndex >= this.templates.length
				|| theTemplateIndex >= this.templatesGroups.length) {
			return null;
		}

		String theOriginalTemplate = this.templates[theTemplateIndex];
		StringBuffer theRegExTemplate = new StringBuffer();

		Matcher matcher = this.fieldRegEx.matcher(theOriginalTemplate);

		boolean result = matcher.find();
		if (result) {
			do {
				final String variable = matcher.group();
				final Matcher variableMatcher = this.identifierRegEx
						.matcher(variable);
				boolean found = variableMatcher.find();
				assert found;

				final String variableName = variableMatcher.group();

				String fieldRegex = this.fieldsRegExpr.get(variableName);
				if (fieldRegex != null) {
					fields.add(variableName);
					matcher.appendReplacement(theRegExTemplate,
							Matcher.quoteReplacement("(" + fieldRegex + ")"));
				}

				result = matcher.find();
			} while (result);
			matcher.appendTail(theRegExTemplate);

		} else {
			return theOriginalTemplate;
		}

		this.templatesGroups[theTemplateIndex] = fields
				.toArray(new String[fields.size()]);

		return theRegExTemplate.toString();

	}

	/**
	 * Try to match the supplied text against the template list.
	 * <ul>
	 * <li>If there are no matches it will return -1.</li>
	 * <li>If there is one matches it returns the template index as received in
	 * the templates initialization array.</li>
	 * <li>If there are more than one matches it will return the first template
	 * that matches.</li>
	 * </ul>
	 * The templates must be designed to ensure that only one matches.
	 * 
	 */
	public int matchTemplate(String textToMatch,
			Map<String, Object> fieldValueFound) {

		// assert the text to match against all the pre-compiled templates
		int i = 0;
		for (Pattern template : this.templatesComp) {

			Matcher matcher = template.matcher(textToMatch);

			if (matcher.matches()) {

				int count = matcher.groupCount();

				String[] fields = this.templatesGroups[i];
				assert fields.length == count;

				for (int j = 0; j < count; j++) {
					final String value = matcher.group(j + 1);

					fieldValueFound.put(fields[j], value);
				}

				return i;
			}

			i++;
		}

		return -1;
	}

	/**
	 * Getter of <code>fieldsRegExpr</code> property.
	 * 
	 * @return <code>fieldsRegExpr</code> property value.
	 * @see #fieldsRegExpr
	 */

	public Map<String, String> getFieldsRegExpr() {
		return this.fieldsRegExpr;
	}

	/**
	 * Getter of <code>templates</code> property.
	 * 
	 * @return <code>templates</code> property value.
	 * @see #templates
	 */

	public String[] getTemplates() {
		return this.templates;
	}

	/**
	 * Getter of <code>fieldRegEx</code> property.
	 * 
	 * @return <code>fieldRegEx</code> property value.
	 * @see #fieldRegEx
	 */

	public Pattern getFieldRegEx() {
		return this.fieldRegEx;
	}

	/**
	 * Getter of <code>identifierRegEx</code> property.
	 * 
	 * @return <code>identifierRegEx</code> property value.
	 * @see #identifierRegEx
	 */

	public Pattern getIdentifierRegEx() {
		return this.identifierRegEx;
	}

	/**
	 * Setter of <code>fieldsRegExpr</code> property.
	 * 
	 * @param theFieldsRegExpr
	 *            The <code>fieldsRegExpr</code> to set.
	 * @see #fieldsRegExpr
	 */
	public void setFieldsRegExpr(Map<String, String> theFieldsRegExpr) {
		this.fieldsRegExpr = theFieldsRegExpr;
		this.compileRegularExpressions();
	}

	/**
	 * Setter of <code>templates</code> property.
	 * 
	 * @param theTemplates
	 *            The <code>templates</code> to set.
	 * @see #templates
	 */
	public void setTemplates(String[] theTemplates) {
		this.templates = (theTemplates != null) ? theTemplates.clone() : null;
		this.compileRegularExpressions();
	}

	/**
	 * Setter of <code>fieldRegEx</code> property.
	 * 
	 * @param theFieldRegEx
	 *            The <code>fieldRegEx</code> to set.
	 * @see #fieldRegEx
	 */
	public void setFieldRegEx(Pattern theFieldRegEx) {
		this.fieldRegEx = theFieldRegEx;
		this.compileRegularExpressions();
	}

	/**
	 * Setter of <code>identifierRegEx</code> property.
	 * 
	 * @param theIdentifierRegEx
	 *            The <code>identifierRegEx</code> to set.
	 * @see #identifierRegEx
	 */
	public void setIdentifierRegEx(Pattern theIdentifierRegEx) {
		this.identifierRegEx = theIdentifierRegEx;
		this.compileRegularExpressions();
	}
}
