/**
 * HBProjectionTransformer.java
 *
 * $Revision: 1.4 $ ($Date: 2011-04-06 04:45:10 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.service.impl;

import java.util.Map;

import org.hibernate.criterion.Projection;
import org.hibernate.criterion.ProjectionList;
import org.hibernate.criterion.Projections;

import com.smartmatic.waffle.persistence.search.transformation.AliasTransformationsMap;
import com.smartmatic.waffle.persistence.search.transformation.CountTransformation;
import com.smartmatic.waffle.persistence.search.transformation.GroupTransformation;
import com.smartmatic.waffle.persistence.search.transformation.ITransformation;
import com.smartmatic.waffle.persistence.search.transformation.IdentifierTransformation;
import com.smartmatic.waffle.persistence.search.transformation.MaxTransformation;
import com.smartmatic.waffle.persistence.search.transformation.MinTransformation;
import com.smartmatic.waffle.persistence.search.transformation.PropertyTransformation;
import com.smartmatic.waffle.persistence.search.transformation.RowCountTransformation;
import com.smartmatic.waffle.persistence.search.transformation.SumTransformation;
import com.smartmatic.waffle.persistence.search.transformation.TransformationsList;

/**
 * Class used to transform a proper projection into a projection in the
 * hibernate language.
 * 
 * @version 1.0 (Aug 29, 2006)
 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
 */

public class HBProjectionTransformer {
	/**
	 * Method used to transform a PropertyTransformation into a projection in
	 * the hibernate language.
	 * 
	 * @param propertyTransformation
	 *            The projection that needs to be transformed
	 * @return The projection transformed in Hibernate.
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public static Projection transformProjection(
			PropertyTransformation propertyTransformation) {
		return Projections.property(propertyTransformation.getPropertyName());
	}

	/**
	 * A projected identifier value
	 * 
	 * @param idTransformation
	 *            Identifier to project
	 * @return The projection transformed in Hibernate.
	 */
	public static Projection transformProjection(
			IdentifierTransformation idTransformation) {
		return Projections.id();
	}

	/**
	 * Method used to transform a MinTransformation into a projection in the
	 * hibernate language.
	 * 
	 * @param minTransformation
	 *            The projection that needs to be transformed
	 * @return The projection transformed in Hibernate.
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public static Projection transformProjection(
			MinTransformation minTransformation) {
		return Projections.min(minTransformation.getProperty());
	}

	/**
	 * Method used to transform a MaxTransformation into a projection in the
	 * hibernate language.
	 * 
	 * @param maxTransformation
	 *            The projection that needs to be transformed
	 * @return The projection transformed in Hibernate.
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public static Projection transformProjection(
			MaxTransformation maxTransformation) {
		return Projections.max(maxTransformation.getProperty());
	}

	/**
	 * Method used to transform a SumTransformation into a projection in the
	 * hibernate language.
	 * 
	 * @param sumTransformation
	 *            The projection that needs to be transformed
	 * @return The projection transformed in Hibernate.
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public static Projection transformProjection(
			SumTransformation sumTransformation) {
		return Projections.sum(sumTransformation.getProperty());
	}

	/**
	 * Method used to transform a GroupTransformation into a projection in the
	 * hibernate language.
	 * 
	 * @param groupTransformation
	 *            The projection that needs to be transformed
	 * @return The projection transformed in Hibernate.
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public static Projection transformProjection(
			GroupTransformation groupTransformation) {
		return Projections.groupProperty(groupTransformation.getProperty());
	}

	/**
	 * Method used to transform a TransformationsList into a projection in the
	 * hibernate language.
	 * 
	 * @param transformationsList
	 *            The projection that needs to be transformed
	 * @return The projection transformed in Hibernate.
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public static Projection transformProjection(
			TransformationsList transformationsList) {

		ProjectionList projectionList = Projections.projectionList();
		ITransformation[] listTransformation = transformationsList
				.getTransformations();
		for (ITransformation element : listTransformation) {
			projectionList.add(HBPersistenceServiceImpl
					.transformProjection(element));
		}

		return projectionList;
	}

	/**
	 * Method used to transform a AliasTransformationsMap into a projection in
	 * the hibernate language.
	 * 
	 * @param aliasTransformationsMap
	 *            The projection that needs to be transformed
	 * @return The projection transformed in Hibernate.
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public static Projection transformProjection(
			AliasTransformationsMap aliasTransformationsMap) {

		ProjectionList projectionList = Projections.projectionList();
		Map<String, ITransformation> aliasTransformationMap = aliasTransformationsMap
				.getTransformations();

		for (Map.Entry<String, ITransformation> theEntry : aliasTransformationMap
				.entrySet()) {
			projectionList.add(HBPersistenceServiceImpl
					.transformProjection(theEntry.getValue()), theEntry
					.getKey());
		}

		return projectionList;
	}

	/**
	 * Method used to transform a RowCountTransformation into a projection in
	 * the hibernate language.
	 * 
	 * @param rowCountTransformation
	 *            The projection that needs to be transformed
	 * @return The projection transformed in Hibernate.
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public static Projection transformProjection(
			RowCountTransformation rowCountTransformation) {
		return Projections.rowCount();
	}

	/**
	 * Method used to transform a DistinctTransformation into a projection in
	 * the hibernate language
	 * 
	 * @param transformProjection
	 *            Projection to be transformed
	 * @return The projection transformed in Hibernate
	 */
	public static Projection transformProjection(Projection transformProjection) {

		return Projections.distinct(transformProjection);
	}

	/**
	 * Method used to transform a CountTransformation into a projection in the
	 * hibernate language.
	 * 
	 * @param countTransformation
	 *            The projection that needs to be transformed
	 * @return The projection transformed in Hibernate.
	 * 
	 * @author Carlos Rafael Ramirez
	 */
	public static Projection transformProjection(
			CountTransformation countTransformation) {

		if (countTransformation.getDistinct() == true) {

			return Projections.countDistinct(countTransformation
					.getPropertyName());
		} else {

			return Projections.count(countTransformation.getPropertyName());
		}
	}

}
