/**
 * PersistenceServiceBean.java
 * 
 * $Revision: 1.32 $ ($Date: 2011-06-11 00:16:55 $)
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.service.impl;

import java.lang.reflect.Method;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.log4j.Logger;

import com.smartmatic.waffle.domain.IBaseDataObject;
import com.smartmatic.waffle.domain.filter.FilterDomainAnnotation;
import com.smartmatic.waffle.domain.user.IUser;
import com.smartmatic.waffle.generator.spec.GeneratorException;
import com.smartmatic.waffle.generator.spec.IGenerator;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.lang.annotations.basic.GeneratedPropertyAnnotation;
import com.smartmatic.waffle.lang.util.ExceptionHandler;
import com.smartmatic.waffle.lang.util.ReflectHelper;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.search.criteria.ConjunctionCriteria;
import com.smartmatic.waffle.persistence.search.criteria.CriteriaFactory;
import com.smartmatic.waffle.persistence.search.criteria.ISearchCriteria;
import com.smartmatic.waffle.persistence.search.criteria.IdentifierEqCriteria;
import com.smartmatic.waffle.persistence.service.impl.ValidatorEvaluator.ValidatorMode;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceService;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;

/**
 * Session beans that handles all the persistence operations. This beans its
 * responsible of all the CRUD operations in the system.
 * 
 * @version 1.0 (23/06/2006)
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

@Stateless(name = IPersistenceService.SERVICE_NAME)
@Remote(IPersistenceService.class)
@Local(IPersistenceServiceLocal.class)
public class PersistenceServiceBean implements IPersistenceService {

	/**
	 * The injected entity manager
	 */
	@PersistenceContext(unitName = "smtt")
	protected EntityManager entityManager;

	/**
	 * Local creation mode to stand-alone application
	 */
	protected static boolean localCreationMode = false;

	/**
	 * Retrieve actual entity manager
	 */
	protected static ThreadLocal<EntityManager> actualEntityManager = new ThreadLocal<EntityManager>() {
		/**
		 * Initialize the actual entity manager using the factory
		 */
		@Override
		protected synchronized EntityManager initialValue() {
			return PersistenceServiceBean.entityManagerFactory
					.createEntityManager();
		}
	};

	/**
	 * Bean's entity manager factory
	 */
	protected static EntityManagerFactory entityManagerFactory;

	/**
	 * Holds the object used to log all the relevant information that this class
	 * needs to communicate.
	 */

	private static Logger logger = Logger
			.getLogger(PersistenceServiceBean.class);

	/**
	 * Basic constructor
	 */
	public PersistenceServiceBean() {
		// Check if the entity manager is null to create an instance or retrieve
		// it
		if (this.entityManager == null
				&& PersistenceServiceBean.localCreationMode == true) {
			if (PersistenceServiceBean.entityManagerFactory == null) {
				PersistenceServiceBean.entityManagerFactory = Persistence
						.createEntityManagerFactory("smtt");
			}
			this.entityManager = PersistenceServiceBean.actualEntityManager
					.get();
		}
	}

	/**
	 * Method used to generate any auto-generated attribute from the object
	 * 
	 * @param object
	 *            object that contains the property to be generated
	 * @throws GeneratorException
	 *             Thrown when an error occurred during the object attributes
	 *             auto-generation
	 */
	@SuppressWarnings("unchecked")
	private void invokeGenerators(Object object) throws GeneratorException {
		try {
			Method[] methods = object.getClass().getMethods();
			// Process every method
			for (Method method : methods) {

				Class annotationClass = GeneratedPropertyAnnotation.class;
				GeneratedPropertyAnnotation annotation = (GeneratedPropertyAnnotation) method
						.getAnnotation(annotationClass);

				if (annotation != null && method.invoke(object) == null) {
					IGenerator generator = annotation.generatorClass()
							.newInstance();
					String propertyName = annotation.property().length() != 0 ? annotation
							.property() : ReflectHelper
							.getPropertyNameFromGetter(method);
					Object value = generator.generate();
					PropertyUtils.setProperty(object, propertyName, value);
				}
			}
		} catch (Exception exception) {
			throw new GeneratorException(
					GeneratorException.ExceptionInfo.VALUE_GENERATION,
					exception);
		}
	}

	/**
	 * Make and entity instance managed and persistent.
	 * 
	 * @param component
	 *            The entity to persist
	 * @param user
	 *            The user who is performing the operation
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	@Override
	@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
	public <T extends IBaseDataObject> T persist(T bcomponent, IUser user)
			throws BaseException, RuntimeException {
		try {
			if (user != null) {
				ThreadLocalWrapper.set(user.getLogin());
			} else {
				ThreadLocalWrapper.set("");
			}
			this.invokeGenerators(bcomponent);
			ValidatorEvaluator.invokeMethod(ValidatorMode.PREPERSIST,
					bcomponent);
			this.entityManager.persist(bcomponent);
			this.entityManager.flush();
			ValidatorEvaluator.invokeMethod(ValidatorMode.POSTPERSIST,
					bcomponent);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
		return bcomponent;
	}

	/**
	 * Make and entity instance managed and persistent suitable for batch
	 * processing.
	 * 
	 * The use of this method requires a call to flush method every
	 * hibernate.jdbc.batch_size or multiples of it.
	 * 
	 * @param component
	 *            The entity to persist
	 * @param user
	 *            The user who is performing the operation
	 * @throws BaseException
	 *             If any problem occurred
	 */
	@Override
	@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
	public <T extends IBaseDataObject> T batchPersist(T bcomponent, IUser user)
			throws BaseException, RuntimeException {
		try {
			if (user != null) {
				ThreadLocalWrapper.set(user.getLogin());
			} else {
				ThreadLocalWrapper.set("");
			}
			this.invokeGenerators(bcomponent);
			ValidatorEvaluator.invokeMethod(ValidatorMode.PREPERSIST,
					bcomponent);
			this.entityManager.persist(bcomponent);
			ValidatorEvaluator.invokeMethod(ValidatorMode.POSTPERSIST,
					bcomponent);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
		return bcomponent;
	}

	/**
	 * Updates a instance into the database.
	 * 
	 * @param user
	 *            The user who is performing the operation
	 * @param bcomponent
	 *            The bcomponent to update
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	@Override
	@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
	public <T extends IBaseDataObject> T update(T bcomponent, IUser user)
			throws BaseException, RuntimeException {
		try {

			if (user != null) {
				ThreadLocalWrapper.set(user.getLogin());
			} else {
				ThreadLocalWrapper.set("");
			}
			this.invokeGenerators(bcomponent);
			ValidatorEvaluator
					.invokeMethod(ValidatorMode.PREUPDATE, bcomponent);
			T result = this.entityManager.merge(bcomponent);
			this.entityManager.flush();
			ValidatorEvaluator.invokeMethod(ValidatorMode.POSTUPDATE,
					bcomponent);
			return result;
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Remove the entity instance
	 * 
	 * @param bcomponent
	 *            The bcomponent to remove
	 * @param user
	 *            The user that commands the action
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	@Override
	@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
	public void remove(IBaseDataObject bcomponent, IUser user)
			throws BaseException, RuntimeException {
		try {
			if (user != null) {
				ThreadLocalWrapper.set(user.getLogin());
			} else {
				ThreadLocalWrapper.set("");
			}
			ValidatorEvaluator
					.invokeMethod(ValidatorMode.PREREMOVE, bcomponent);
			this.entityManager.remove(bcomponent);
			this.entityManager.flush();
			ValidatorEvaluator.invokeMethod(ValidatorMode.POSTREMOVE,
					bcomponent);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a search expression, returning a collection of
	 * objects
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param user
	 *            The user that commands the action
	 * @return The collection of objects
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */

	@Override
	@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
	public Collection searchObjects(SearchExpression searchExpression,
			IUser user) throws BaseException, RuntimeException {
		try {
			return this.searchObjects(searchExpression, null, user);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a search expression with paginate option
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param user
	 *            The user who is performing the operation
	 * @param pageNumber
	 * @param pageSize
	 * @return
	 * @throws BaseException
	 *             If any problem occurred
	 */
	@Override
	@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
	public Collection searchPagedObjects(SearchExpression searchExpression,
			IUser user, int pageNumber, int pageSize) throws BaseException,
			RuntimeException {
		try {
			return this.searchObjects(searchExpression, null, user, pageNumber,
					pageSize);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a search expression with paginate option
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param path
	 *            The paths to initialize
	 * @param user
	 *            The user who is performing the operation
	 * @param pageNumber
	 *            The page number
	 * @param pageSize
	 *            The item number per page
	 * @return The requested Objects
	 * @throws BaseException
	 *             If any problem occurred
	 */
	@Override
	@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
	public Collection searchPagedObjects(SearchExpression searchExpression,
			String[] path, IUser user, int pageNumber, int pageSize)
			throws BaseException, RuntimeException {
		try {
			return this.searchObjects(searchExpression, path, user, pageNumber,
					pageSize);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a search expression, returning with paginate option
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param path
	 *            The paths to initialize
	 * @param user
	 *            The user who is performing the operation
	 * @param pageNumber
	 *            The page number
	 * @param pageSize
	 *            The item number per page
	 * @return The requested Objects
	 * @throws BaseException
	 *             If any problem occurred
	 */
	@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
	public Collection searchObjects(SearchExpression searchExpression,
			String[] path, IUser user, int pageNumber, int pageSize)
			throws BaseException, RuntimeException {
		try {
			return this.searchObjects(searchExpression, path, false, user,
					pageNumber, pageSize);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a search expression, returning a collection of
	 * objects. It also ensures that the required properties has to be
	 * initialized in the returned collection of objects.
	 * 
	 * @param searchExpression
	 *            The search expression that defines a query expression.
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param recursive
	 *            The boolean that indicate if the path is initialized in a
	 *            recursive way
	 * @param user
	 *            The user who is performing the operation
	 * @param pageNumber
	 *            The page number
	 * @param pageSize
	 *            The item number per page
	 * @return The collection of objects that match the specified expression
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */

	@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
	public Collection searchObjects(SearchExpression searchExpression,
			String[] path, boolean recursive, IUser user, int pageNumber,
			int pageSize) throws BaseException, RuntimeException {
		try {
			if (user != null) {
				this.filterByUser(searchExpression, user);
			}
			Collection result = HBPersistenceServiceImpl.searchObjects(
					searchExpression, this.entityManager, path, recursive,
					pageNumber, pageSize);
			return result;
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a search expression, returning a collection of
	 * objects. It also ensures that the required properties has to be
	 * initialized in the returned collection of objects.
	 * 
	 * @param searchExpression
	 *            The search expression that defines a query expression.
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param user
	 *            The user who is performing the operation
	 * @return The collection of objects that match the specified expression
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */

	@Override
	@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
	public Collection searchObjects(SearchExpression searchExpression,
			String[] path, IUser user) throws BaseException, RuntimeException {
		try {
			return this.searchObjects(searchExpression, path, false, user);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a search expression with a boolean option to search
	 * the path recursively
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param recursive
	 *            The boolean that indicate if the path is initialized in a
	 *            recursive way
	 * @param user
	 *            The user who is performing the operation
	 * @return The Object that match the defined search expression or null
	 *         otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 * @throws RuntimeException
	 *             If any problem occurred
	 */
	@Override
	@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
	public Collection searchObjects(SearchExpression searchExpression,
			String[] path, boolean recursive, IUser user) throws BaseException,
			RuntimeException {
		try {
			if (user != null) {
				this.filterByUser(searchExpression, user);
			}
			Collection result = HBPersistenceServiceImpl.searchObjects(
					searchExpression, this.entityManager, path, recursive);
			return result;
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a search expression with a boolean option to search
	 * the path recursively
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param recursive
	 *            The boolean that indicate if the path is initialized in a
	 *            recursive way
	 * @param user
	 *            The user who is performing the operation
	 * @return The Object that match the defined search expression or null
	 *         otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 * @throws RuntimeException
	 *             If any problem occurred
	 */

	@Override
	@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
	public int countObjects(SearchExpression searchExpression, IUser user)
			throws BaseException, RuntimeException {
		try {
			return HBPersistenceServiceImpl.countObjects(searchExpression,
					this.entityManager);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a search expression, returning a unique object
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param user
	 *            The user that commands the action
	 * @return The Object
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */

	@Override
	@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
	public Object searchObject(SearchExpression searchExpression, IUser user)
			throws BaseException, RuntimeException {
		try {
			return this.searchObject(searchExpression, null, user);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a search expression, returning a unique object
	 * result.
	 * 
	 * @param searchExpression
	 *            The search expression that defines a query expression
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param user
	 *            The user who is performing the operation
	 * @return The Object that match the defined search expression or null
	 *         otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */

	@Override
	@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
	public Object searchObject(SearchExpression searchExpression,
			String[] path, IUser user) throws BaseException, RuntimeException {
		try {
			return this.searchObject(searchExpression, path, false, user);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a search expression, returning a unique object, with
	 * a boolean option to search the path recursively
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param recursive
	 *            The boolean that indicate if the path is initialized in a
	 *            recursive way
	 * @param user
	 *            The user who is performing the operation
	 * @return The Object that match the defined search expression or null
	 *         otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 * @throws RuntimeException
	 *             If any problem occurred
	 */
	@Override
	@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
	public Object searchObject(SearchExpression searchExpression,
			String[] path, boolean recursive, IUser user) throws BaseException,
			RuntimeException {
		try {

			/*
			 * look for an object by its id can be faster than its search
			 * expression counterpart due to JPA implementation can use second
			 * Level cache
			 */
			final Object pk = this
					.checkForSearchingAnObjectById(searchExpression);
			if (pk != null && user == null) {

				return this.find(searchExpression.getEntityClass(), pk, path,
						recursive);

			} else {
				if (user != null) {
					this.filterByUser(searchExpression, user);
				}
				return HBPersistenceServiceImpl.searchObject(searchExpression,
						this.entityManager, path, recursive);
			}
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Retreives an object by its primary key. Also initializes the properties
	 * specified in path parameter
	 * 
	 * @param entityClass
	 *            The class of the object to search for
	 * @param pk
	 *            The primary key to search for
	 * @param path
	 *            The path variables to initialize
	 * @param recursive
	 *            if recursively process the entity fields
	 * @return the found object
	 * @throws BaseException
	 *             If any problem occurred
	 */
	@Override
	@SuppressWarnings("unchecked")
	public <T extends IBaseDataObject> T find(Class<T> entityClass, Object pk,
			String[] path, boolean recursive) throws BaseException {

		try {
			return (T) HBPersistenceServiceImpl.find(entityClass, pk,
					this.entityManager, path, recursive);
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}

	}

	/**
	 * If this searchExpression represents only an idCriteria, this method
	 * returns the value inside the idCriteria
	 * 
	 * @return the value inside the idCriteria inside the searchExpression if
	 *         any.
	 */
	private Object getObjectPk(SearchExpression searchExpression) {

		ISearchCriteria searchCriteria = searchExpression.getSearchCriteria();
		if (searchCriteria instanceof ConjunctionCriteria) {
			ConjunctionCriteria conjunctionCriteria = (ConjunctionCriteria) searchCriteria;

			final List<ISearchCriteria> criterias = conjunctionCriteria
					.getCriteria();
			if (criterias.size() == 1) {

				ISearchCriteria idCriteria = criterias.iterator().next();

				if (idCriteria instanceof IdentifierEqCriteria) {
					return ((IdentifierEqCriteria) idCriteria).getValue();
				}

			}
		}

		return null;
	}

	/**
	 * Checks for a search an object by its id search expression and returns the
	 * id value that its represents. Otherwise it returns null
	 * 
	 * @param searchExpression
	 * @return
	 */
	private Object checkForSearchingAnObjectById(
			SearchExpression searchExpression) {

		Object pk = this.getObjectPk(searchExpression);

		if (pk == null) {
			return null;
		}

		if (searchExpression.getSubExpressions().hasNext()) {
			return null;
		}

		if (searchExpression.getTransformation() != null) {
			return null;
		}

		return pk;
	}

	/**
	 * Method that filters queries for each user using an annotation located in
	 * the bean.
	 * 
	 * @param search
	 *            Search expression to filter
	 * @param user
	 *            The user that commands the action
	 */
	@SuppressWarnings("unchecked")
	private void filterByUser(SearchExpression search, IUser user) {
		Class bComponentClass = search.getEntityClass();
		Class filterDomainAnnotationClass = FilterDomainAnnotation.class;
		FilterDomainAnnotation annotation = (FilterDomainAnnotation) bComponentClass
				.getAnnotation(filterDomainAnnotationClass);
		if (annotation != null) {
			String pathUserFilter = annotation.pathUserFilter();
			if (pathUserFilter.length() != 0 && !user.isAdmin()) {
				this.restrictSearchExpression(pathUserFilter, search, user);
			}
		}
	}

	/**
	 * Helper method to modify the search expression given by filterByUse using
	 * a path.
	 * 
	 * @param path
	 *            Path to the user
	 * @param search
	 *            Search expression to filter
	 * @param user
	 *            The user that commands the action
	 */

	private void restrictSearchExpression(String path, SearchExpression search,
			IUser user) {
		// restrict the search expression until the path is users
		if (path.indexOf(".") < 0) {
			if (path.equals("users")) {
				SearchExpression searchAlias = search.createAlias(path, path);
				searchAlias.addSearchCriteria(CriteriaFactory.eq("code",
						user.getCode()));
			} else {
				search.addSearchCriteria(CriteriaFactory.eq(path, user));
			}
		} else {
			String alias = path.substring(0, path.indexOf("."));
			SearchExpression searchAlias = search.createAlias(alias, alias);
			String restOfPath = path.substring(path.indexOf(".") + 1);
			if (restOfPath.indexOf(".") < 0) {
				if (restOfPath.equals("users")) {
					SearchExpression searchAlias2 = searchAlias.createAlias(
							restOfPath, restOfPath);
					searchAlias2.addSearchCriteria(CriteriaFactory.eq("code",
							user.getCode()));
				} else {
					searchAlias.addSearchCriteria(CriteriaFactory.eq(
							restOfPath, user));
				}
			} else {
				this.restrictSearchExpression(restOfPath, searchAlias, user);
			}
		}
	}

	/**
	 * Method that executes an update native query in database, returning the
	 * updated fields
	 * 
	 * @param query
	 *            The native query
	 * @return The number of updated fields
	 * @throws BaseException
	 *             If any problem occurred
	 */
	@Override
	@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
	public int executeUpdateNativeQuery(String query) throws BaseException,
			RuntimeException {
		try {
			Query dbQuery = this.entityManager.createNativeQuery(query);
			int result = dbQuery.executeUpdate();
			return result;
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes an update query in database, returning the updated
	 * fields
	 * 
	 * @param query
	 *            The update query
	 * @param parameters
	 *            Query's parameters
	 * @return The number of updated fields
	 * @throws BaseException
	 *             If any problem occurred
	 */
	@Override
	@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
	public int executeUpdateQuery(String query, Object... parameters)
			throws BaseException, RuntimeException {
		try {
			Query dbQuery = this.entityManager.createQuery(query);
			int i = 1;
			for (Object o : parameters) {
				dbQuery.setParameter(i++, o);
			}

			int result = dbQuery.executeUpdate();
			return result;
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a Native Query in database, returning List object
	 * 
	 * @param query
	 *            query
	 * @return List<Object> The result of the query after run or null otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 * @author Vanessa Cobis(vanessa.cobis@smartmatic.com)
	 */

	@Override
	@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
	public List<Object> executeNativeQuery(String query) throws BaseException,
			RuntimeException {
		try {
			Query dbQuery = this.entityManager.createNativeQuery(query);
			List<Object> result = dbQuery.getResultList();
			return result;
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a Native Query in database and using the requested
	 * parameters, returning List object
	 * 
	 * @param query
	 *            Query to execute
	 * @param parameters
	 *            Query's parameters
	 * @return List<Object> The result of the query after run or null otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 */
	@Override
	@SuppressWarnings("unchecked")
	@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
	public List<Object> executeNativeQuery(String query, Object... parameters)
			throws BaseException, RuntimeException {
		try {
			Query dbQuery = this.entityManager.createNativeQuery(query);
			int i = 1;
			for (Object o : parameters) {
				dbQuery.setParameter(i++, o);
			}
			List<Object> result = dbQuery.getResultList();
			return result;
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Method that executes a Native Query in database and using the requested
	 * parameters, returning List object
	 * 
	 * @param query
	 *            Query to execute
	 * @param parameters
	 *            Query's parameters
	 * @return List<Object> The result of the query after run or null otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 */
	@Override
	@SuppressWarnings("unchecked")
	@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
	public List<Object> executeNativeQueryNamedParameters(String query,
			Map<String, ?> parameters) throws BaseException, RuntimeException {
		try {
			Query dbQuery = this.entityManager.createNativeQuery(query);
			int i = 1;
			for (Map.Entry<String, ?> o : parameters.entrySet()) {
				dbQuery.setParameter(o.getKey(), o.getValue());
			}
			List<Object> result = dbQuery.getResultList();
			return result;
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

	/**
	 * Begin the current transaction
	 */
	public void beginTransaction() {
		this.entityManager.getTransaction().begin();
	}

	/**
	 * Commit the current transaction
	 */
	public void commitTransaction() {
		this.entityManager.getTransaction().commit();
	}

	/**
	 * Roll back the current transaction
	 */
	public void rollbackTransaction() {
		this.entityManager.getTransaction().rollback();
	}

	/**
	 * Getter of <code>entityManager</code> property.
	 * 
	 * @return <code>entityManager</code> property value.
	 * @see #entityManager
	 */
	public EntityManager getEntityManager() {
		return this.entityManager;
	}

	/**
	 * Set local creation local mode true, to stand-alone applications
	 */
	public static void setLocalCreationMode() {
		PersistenceServiceBean.localCreationMode = true;
	}

	/**
	 * Method return a unique object using the supplied primary key as search
	 * criteria and the user.
	 * 
	 * @author Carlos Lameda(carlos.lameda@smartmatic.com)
	 * @param theClass
	 *            type of the object to find
	 * @param pk
	 *            the supplied primary key
	 * @param user
	 *            The user who is performing the operation
	 * @return the found Entity or null if it was not found
	 * @throws BaseException
	 *             if a database exception occurs
	 */
	@Override
	public <T extends IBaseDataObject> T searchByKey(Class<T> theClass,
			Object pk, IUser user) throws BaseException {
		return this.searchByKey(theClass, pk, null, user);
	}

	/**
	 * Method return a unique object,with an array of properties that has to be
	 * initialized using the supplied primary key as search criteria and the
	 * user.
	 * 
	 * @author Carlos Lameda(carlos.lameda@smartmatic.com)
	 * @param theClass
	 *            type of the object to find
	 * @param pk
	 *            the supplied primary key
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param user
	 *            The user who is performing the operation
	 * @return the found Entity or null if it was not found
	 * @throws BaseException
	 *             if a database exception occurs
	 */
	@Override
	public <T extends IBaseDataObject> T searchByKey(Class<T> theClass,
			Object pk, String[] pathsToInit, IUser user) throws BaseException {
		return this.searchByKey(theClass, pk, pathsToInit, false, user);
	}

	/**
	 * Method return a unique object,with an array of properties that has to be
	 * initialized and a boolean option to search the path recursively using the
	 * supplied primary key as search criteria and the user.
	 * 
	 * @author Carlos Lameda(carlos.lameda@smartmatic.com)
	 * @param theClass
	 *            type of the object to find
	 * @param pk
	 *            the supplied primary key
	 * @param pathsToInit
	 *            The array of properties that has to be initialized
	 * @param recursive
	 *            The boolean that indicate if the path is initialized in a
	 *            recursive way
	 * @param user
	 *            The user who is performing the operation
	 * @return the found Entity or null if it was not found
	 * @throws BaseException
	 *             if a database exception occurs
	 */
	@Override
	public <T extends IBaseDataObject> T searchByKey(Class<T> theClass,
			Object pk, String[] pathsToInit, boolean recursive, IUser user)
			throws BaseException {

		SearchExpression searchExpression = new SearchExpression(theClass);
		searchExpression.addSearchCriteria(CriteriaFactory.idEq(pk));

		if (user != null) {
			this.filterByUser(searchExpression, user);
		}

		return (T) HBPersistenceServiceImpl.searchObject(searchExpression,
				this.entityManager, pathsToInit, recursive);
	}

	/**
	 * Method used to clean all the second level cache. This method occurs
	 * outside of any transaction; it performs an immediate "hard" remove in all
	 * the entities of the second-level cache, so does not respect any
	 * transaction isolation semantics of the usage strategy. Use with care.
	 * 
	 */
	@Override
	public void cleanSecondLevelCache() {
		try {
			HBPersistenceServiceImpl.cleanSecondLevelCache(this.entityManager);
		} catch (Exception e) {

			e.printStackTrace();
		}
	}

	/**
	 * Method used to clean all collections contained in the second level cache.
	 * This method occurs outside of any transaction; it performs an immediate
	 * "hard" remove in all the collections of the second-level cache, so does
	 * not respect any transaction isolation semantics of the usage strategy.
	 * Use with care.
	 */
	@Override
	public void cleanCollectionSecondLevelCache() {
		try {
			HBPersistenceServiceImpl
					.cleanCollectionSecondLevelCache(this.entityManager);
		} catch (Exception e) {

			e.printStackTrace();
		}
	}

	/**
	 * Method for clean and free entity manager's batch allocation buffer
	 */
	@Override
	public void flush() {
		this.entityManager.flush();
		this.entityManager.clear();
	}

	/**
	 * Remove the given entity from the persistence context, causing a managed
	 * entity to become detached. Unflushed changes made to the entity if any
	 * (including removal of the entity), will not be synchronized to the
	 * database. Entities which previously referenced the detached entity will
	 * continue to reference it.
	 * 
	 * @param entity
	 *            entity instance
	 * 
	 */

	@Override
	public void detach(Object entity) {

		HBPersistenceServiceImpl.detach(entity, this.entityManager);
	}

	/**
	 * Method to delete an elements of a given entity class.
	 * 
	 * @param entityClass
	 *            the entity class that the elements belongs
	 * @param pk
	 *            the primary key of the element to be deleted.
	 * @throws BaseException
	 *             If any error occurs deleting an element
	 */
	@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
	@Override
	public void deleteById(Class<? extends IBaseDataObject> entityClass,
			Number pk) throws BaseException {

		IBaseDataObject objectToDelete = this.entityManager.getReference(
				entityClass, pk);
		this.remove(objectToDelete, null);
	}

	/**
	 * Method that returns a reference of a given class and a primary key. Is a
	 * wrapper of the {@link EntityManager#getReference(Class, Object)} Method.
	 * This method is mainly used to get an attached reference of an persistence
	 * object, an avoid detached object exceptions.
	 * 
	 * @param <T>
	 *            The <code>IBaseDataObject</code> class.
	 * @param entityClass
	 *            the entity class to be loaded he reference.
	 * @param pk
	 *            the primary key of the element to be obtained.
	 * @return a
	 */
	@Override
	public <T extends IBaseDataObject> T getReference(Class<T> entityClass,
			Object pk) {

		return this.entityManager.getReference(entityClass, pk);
	}

	/**
	 * Method that executes an update native query in database, returning the
	 * updated fields and set the given parameters to the query
	 * 
	 * @param query
	 *            The native query
	 * @param parameters
	 *            The array of parameters to be set to the query
	 * @return The number of updated fields
	 * @throws BaseException
	 *             If any problem occurred
	 */
	@Override
	@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
	public int executeUpdateNativeQuery(String query, Object... parameters)
			throws BaseException, RuntimeException {

		try {
			Query dbQuery = this.entityManager.createNativeQuery(query);
			int i = 1;
			for (Object o : parameters) {
				dbQuery.setParameter(i++, o);
			}
			int result = dbQuery.executeUpdate();
			return result;
		} catch (RuntimeException ex) {
			if (ExceptionHandler.isWrapOfBaseException(ex)) {
				throw ExceptionHandler.wrapException(ex);
			}
			throw ex;
		}
	}

}