/**
 * IPersistenceService.java
 * 
 * $Revision: 1.21 $ ($Date: 2011-06-11 00:16:53 $)
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.service.spec;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;

import com.smartmatic.waffle.domain.IBaseDataObject;
import com.smartmatic.waffle.domain.user.IUser;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.persistence.search.SearchExpression;

/**
 * Remote interface that handles all the persistence operations.
 * 
 * @version 1.0 (23/06/2006)
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

public interface IPersistenceService {

	/**
	 * Holds the name of the bean that implements this interface.
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */
	public static final String SERVICE_NAME = "com.smartmatic.waffle.persistence.service.spec.IPersistenceService";

	/**
	 * Make and entity instance managed and persistent.
	 * 
	 * @param component
	 *            The entity to persist
	 * @param user
	 *            The user who is performing the operation
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */
	public <T extends IBaseDataObject> T persist(T component, IUser user)
			throws BaseException;

	/**
	 * Updates a instance into the database.
	 * 
	 * @param user
	 *            The user who is performing the operation
	 * @param bcomponent
	 *            The bcomponent to update
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */
	public <T extends IBaseDataObject> T update(T component, IUser user)
			throws BaseException;

	/**
	 * Remove the entity instance
	 * 
	 * @param user
	 *            The user who is performing the operation
	 * @param bcomponent
	 *            The bcomponent to remove
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */
	public void remove(IBaseDataObject component, IUser user)
			throws BaseException;

	/**
	 * Method that executes a search expression, returning a collection of
	 * objects that match the search expression received.
	 * 
	 * @param searchExpression
	 *            The search expression that defines a query expression.
	 * @param user
	 *            The user who is performing the operation
	 * @return The collection of objects that match the specified expression
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public Collection searchObjects(SearchExpression searchExpression,
			IUser user) throws BaseException;

	/**
	 * Method that executes a search expression, returning a collection of
	 * objects. It also ensures that the required properties has to be
	 * initialized in the returned collection of objects.
	 * 
	 * @param searchExpression
	 *            The search expression that defines a query expression.
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param user
	 *            The user who is performing the operation
	 * @return The collection of objects that match the specified expression
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public Collection searchObjects(SearchExpression searchExpression,
			String[] path, IUser user) throws BaseException;

	/**
	 * Method that determines the amount of objects that a given search
	 * expression will return.
	 * 
	 * @param searchExpression
	 *            The search expression that defines a query expression
	 * @param user
	 *            The user who is performing the operation
	 * @return The number of objects that should be returned if the search
	 *         expression is executed.
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public int countObjects(SearchExpression searchExpression, IUser user)
			throws BaseException;

	/**
	 * Method that executes a search expression, returning a unique object
	 * result.
	 * 
	 * @param searchExpression
	 *            The search expression that defines a query expression
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param user
	 *            The user who is performing the operation
	 * @return The Object that match the defined search expression or null
	 *         otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public Object searchObject(SearchExpression searchExpression,
			String[] path, IUser user) throws BaseException;

	/**
	 * Method that executes a search expression, returning a unique object
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param user
	 *            The user who is performing the operation
	 * @return The Object that match the defined search expression or null
	 *         otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */
	public Object searchObject(SearchExpression searchExpression, IUser user)
			throws BaseException;

	/**
	 * Method that executes a search expression, returning a unique object, with
	 * a boolean option to search the path recursively
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param recursive
	 *            The boolean that indicate if the path is initialized in a
	 *            recursive way
	 * @param user
	 *            The user who is performing the operation
	 * @return The Object that match the defined search expression or null
	 *         otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 * @throws RuntimeException
	 *             If any problem occurred
	 */
	public Object searchObject(SearchExpression searchExpression,
			String[] path, boolean recursive, IUser user) throws BaseException,
			RuntimeException;

	/**
	 * Method that executes a search expression with a boolean option to search
	 * the path recursively
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param recursive
	 *            The boolean that indicate if the path is initialized in a
	 *            recursive way
	 * @param user
	 *            The user who is performing the operation
	 * @return The Object that match the defined search expression or null
	 *         otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 * @throws RuntimeException
	 *             If any problem occurred
	 */
	public Collection searchObjects(SearchExpression searchExpression,
			String[] path, boolean recursive, IUser user) throws BaseException,
			RuntimeException;

	/**
	 * Method that executes a search expression with paginate option
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param user
	 *            The user who is performing the operation
	 * @param pageNumber
	 *            The page number
	 * @param pageSize
	 *            The item number per page
	 * @return The objects paged
	 * @throws BaseException
	 *             If any problem occurred
	 */
	public Collection searchPagedObjects(SearchExpression searchExpression,
			IUser user, int pageNumber, int pageSize) throws BaseException;

	/**
	 * Method that executes a search expression, with paginate option
	 * 
	 * @param searchExpression
	 *            The search expression
	 * @param path
	 *            The paths to initialize
	 * @param user
	 *            The user who is performing the operation
	 * @param pageNumber
	 *            The page number
	 * @param pageSize
	 *            The item number per page
	 * @return The requested Objects
	 * @throws BaseException
	 *             If any problem occurred
	 */
	public Collection searchPagedObjects(SearchExpression searchExpression,
			String[] path, IUser user, int pageNumber, int pageSize)
			throws BaseException;

	/**
	 * Method that executes an update native query in database, returning the
	 * updated fields
	 * 
	 * @param query
	 *            The native query
	 * @return The number of updated fields
	 * @throws BaseException
	 *             If any problem occurred
	 */
	public int executeUpdateNativeQuery(String query) throws BaseException;

	/**
	 * Method that executes an update native query in database, returning the
	 * updated fields and set the given parameters to the query
	 * 
	 * @param query
	 *            The native query
	 * @param parameters
	 *            The array of parameters to be set to the query
	 * @return The number of updated fields
	 * @throws BaseException
	 *             If any problem occurred
	 */
	public int executeUpdateNativeQuery(String query, Object... parameters)
			throws BaseException;

	/**
	 * Method that executes an update query in database, returning the updated
	 * fields
	 * 
	 * @param query
	 *            The update query
	 * @param parameters
	 *            Query's parameters
	 * @return The number of updated fields
	 * @throws BaseException
	 *             If any problem occurred
	 */
	public int executeUpdateQuery(String query, Object... parameters)
			throws BaseException;

	/**
	 * Method that executes a Native Query in database, returning List object
	 * 
	 * @param query
	 *            Query to execute
	 * @return List<Object> The result of the query after run or null otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 * @author Vanessa Cobis(vanessa.cobis@smartmatic.com)
	 */
	public List<Object> executeNativeQuery(String query) throws BaseException;

	/**
	 * Method that executes a Native Query in database and using the requested
	 * parameters, returning List object
	 * 
	 * @param query
	 *            Query to execute
	 * @param parameters
	 *            Query's parameters
	 * @return List<Object> The result of the query after run or null otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 */
	public List<Object> executeNativeQuery(String query, Object... parameters)
			throws BaseException;

	/**
	 * Method that executes a Native Query in database and using the requested
	 * parameters, returning List object
	 * 
	 * @param query
	 *            Query to execute
	 * @param parameters
	 *            Query's parameters
	 * @return List<Object> The result of the query after run or null otherwise
	 * @throws BaseException
	 *             If any problem occurred
	 */
	public List<Object> executeNativeQueryNamedParameters(String query,
			Map<String, ?> parameters) throws BaseException, RuntimeException;

	/**
	 * Method return a unique object using the supplied primary key as search
	 * criteria and the user.
	 * 
	 * @author Carlos Lameda(carlos.lameda@smartmatic.com)
	 * @param theClass
	 *            type of the object to find
	 * @param pk
	 *            the supplied primary key
	 * @param user
	 *            The user who is performing the operation
	 * @return the found Entity or null if it was not found
	 * @throws BaseException
	 *             if a database exception occurs
	 */
	public <T extends IBaseDataObject> T searchByKey(Class<T> theClass,
			Object pk, IUser user) throws BaseException;

	/**
	 * Method return a unique object,with an array of properties that has to be
	 * initialized using the supplied primary key as search criteria and the
	 * user.
	 * 
	 * @author Carlos Lameda(carlos.lameda@smartmatic.com)
	 * @param theClass
	 *            type of the object to find
	 * @param pk
	 *            the supplied primary key
	 * @param path
	 *            The array of properties that has to be initialized
	 * @param user
	 *            The user who is performing the operation
	 * @return the found Entity or null if it was not found
	 * @throws BaseException
	 *             if a database exception occurs
	 */
	public <T extends IBaseDataObject> T searchByKey(Class<T> theClass,
			Object pk, String[] pathsToInit, IUser user) throws BaseException;

	/**
	 * Method return a unique object,with an array of properties that has to be
	 * initialized and a boolean option to search the path recursively using the
	 * supplied primary key as search criteria and the user.
	 * 
	 * @author Carlos Lameda(carlos.lameda@smartmatic.com)
	 * @param theClass
	 *            type of the object to find
	 * @param pk
	 *            the supplied primary key
	 * @param pathsToInit
	 *            The array of properties that has to be initialized
	 * @param recursive
	 *            The boolean that indicate if the path is initialized in a
	 *            recursive way
	 * @param user
	 *            The user who is performing the operation
	 * @return the found Entity or null if it was not found
	 * @throws BaseException
	 *             if a database exception occurs
	 */
	public <T extends IBaseDataObject> T searchByKey(Class<T> theClass,
			Object pk, String[] pathsToInit, boolean recursive, IUser user)
			throws BaseException;

	/**
	 * Retreives an object by its primary key. Also initializes the properties
	 * specified in path parameter
	 * 
	 * @param entityClass
	 *            The class of the object to search for
	 * @param pk
	 *            The primary key to search for
	 * @param path
	 *            The path variables to initialize
	 * @param recursive
	 *            if recursively process the entity fields
	 * @return the found object
	 * @throws BaseException
	 *             If any problem occurred
	 */
	public <T extends IBaseDataObject> T find(Class<T> entityClass, Object pk,
			String[] path, boolean recursive) throws BaseException;

	/**
	 * Method used to clean all the second level cache. This method occurs
	 * outside of any transaction; it performs an immediate "hard" remove in all
	 * the entities of the second-level cache, so does not respect any
	 * transaction isolation semantics of the usage strategy. Use with care.
	 */
	public void cleanSecondLevelCache();

	/**
	 * Method used to clean all collections contained in the second level cache.
	 * This method occurs outside of any transaction; it performs an immediate
	 * "hard" remove in all the collections of the second-level cache, so does
	 * not respect any transaction isolation semantics of the usage strategy.
	 * Use with care.
	 */
	public void cleanCollectionSecondLevelCache();

	/**
	 * Make and entity instance managed and persistent suitable for batch
	 * processing.
	 * 
	 * The use of this method requires a call to flush method every
	 * hibernate.jdbc.batch_size or multiples of it.
	 * 
	 * @param component
	 *            The entity to persist
	 * @param user
	 *            The user who is performing the operation
	 * @throws BaseException
	 *             If any problem occurred
	 * 
	 */
	public <T extends IBaseDataObject> T batchPersist(T bcomponent, IUser user)
			throws BaseException, RuntimeException;

	/**
	 * Method for clean and free entity manager's batch allocation buffer
	 */
	public void flush();

	/**
	 * Remove the given entity from the persistence context, causing a managed
	 * entity to become detached. Unflushed changes made to the entity if any
	 * (including removal of the entity), will not be synchronized to the
	 * database. Entities which previously referenced the detached entity will
	 * continue to reference it.
	 * 
	 * @param entity
	 *            entity instance
	 * 
	 */
	public void detach(Object entity);

	/**
	 * Method to delete an elements of a given entity class.
	 * 
	 * @param entityClass
	 *            the entity class that the elements belongs
	 * @param pk
	 *            the primary key of the element to be deleted.
	 * @throws BaseException
	 *             If any error occurs deleting an element
	 */
	public void deleteById(Class<? extends IBaseDataObject> entityClass,
			Number pk) throws BaseException;

	/**
	 * Method that returns a reference of a given class and a primary key. Is a
	 * wrapper of the {@link EntityManager#getReference(Class, Object)} Method.
	 * This method is mainly used to get an attached reference of an persistence
	 * object, an avoid detached object exceptions.
	 * 
	 * @param <T>
	 *            The <code>IBaseDataObject</code> class.
	 * @param entityClass
	 *            the entity class to be loaded he reference.
	 * @param pk
	 *            the primary key of the element to be obtained.
	 * @return a
	 */
	public <T extends IBaseDataObject> T getReference(Class<T> entityClass,
			Object pk);

}
