/**
 * SignDirUtil.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2018 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.signingtool;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.Provider;
import java.security.Security;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.FileUtils;

import com.smartmatic.waffle.security.asymmetric.AsymmetricKeyManager;

/**
 * Utiliy class used to sign full directories.
 */
public class SignDirUtil {

	public static void checkDependencies(){
		
		String currentVersion = System.getProperty("java.version");
		
		boolean sendWarning = false;
		try{
			sendWarning = Float.valueOf(currentVersion.substring(0, 3)) < 1.7F;
		}catch(Exception ex){
			ex.printStackTrace();
		}
		StringBuilder sb = new StringBuilder();
		sb.append("Java Version is: ");
		sb.append(currentVersion).append("\n");
		
		if(sendWarning){
			boolean bcConfigured = false;
			for (Provider provider: Security.getProviders()){
			  if(provider.getName().equals("BC")){
				  bcConfigured = true;
				  break;
			  }
			}
			
			
			if(!bcConfigured){
				sb.append("This application is full compatible with Java 1.8 and 1.7\n");
				sb.append("If you are using Java 1.6, you will need to configure Bouncy Castle provider \n");
				sb.append("Check the documentation for details.");
				System.out.println(sb.toString());
				System.exit(1);
			}
		}
		System.out.println(sb.toString());
	}
	
	
	/**
	 * Sings a directory, reads all the files from a directory and write it in a
	 * XML file then signs the XML with a certificate.
	 * 
	 * @param directory
	 *            directory to sign
	 * @param certificatePath
	 *            certificate file path
	 * @param alias
	 *            certificate alias
	 * @param password
	 *            certificate password
	 * @throws Exception
	 */
	public static void signDir(File directory, String certificatePath,
			String alias, String password) throws SignDirException {
		try {
			
			byte[] pkcs12Bytes = FileUtils.readFileToByteArray(new File(certificatePath));
			DetachedSignatureCreator detachedSignatureCreator = new DetachedSignatureCreator(pkcs12Bytes, alias, password);
			detachedSignatureCreator.sign(directory);
		} catch (Exception e) {
			throw new SignDirException("Error signing Directory", e);
		}
	}
	
	/**
	 * Writes the security certificate file into an specific location
	 * 
	 * @param pathFile
	 *            Path to where the certificate file will be write into
	 * @param certificatePath
	 *            Original certificate file
	 * @param aliasCert
	 *            Certificate Alias
	 * @param passwordCert
	 *            Certificate Password
	 * @throws SignDirException
	 */
	public static void writeCertificateFile(File pathFile,
			String certificatePath, String aliasCert, String passwordCert)
			throws SignDirException {
		try {
			final Certificate[] certificateChain = getCertificate(
					certificatePath, aliasCert, passwordCert);

			if (pathFile.exists()) {
				pathFile.delete();
			}

			final PrintWriter os = new PrintWriter(new FileWriter(pathFile));
			try {
				for (Certificate certificate : certificateChain) {
					byte[] buf = certificate.getEncoded();

					os.println("-----BEGIN CERTIFICATE-----");
					os.print(new String(Base64.encodeBase64(buf, true)));
					os.println("-----END CERTIFICATE-----");
				}
			} finally {
				os.close();
			}
		} catch (Exception e) {
			throw new SignDirException("Error writting certificate file", e);
		}
	}

	/**
	 * Gets certificate Chain
	 * 
	 * @param certificatePath
	 *            Original certificate file
	 * @param aliasCert
	 *            Certificate Alias
	 * @param passwordCert
	 *            Certificate Password
	 * @return Certificate Chain
	 * @throws Exception
	 */
	private static Certificate[] getCertificate(String certificatePath,
			String aliasCert, String passwordCert) throws IOException,
			KeyStoreException, NoSuchAlgorithmException, CertificateException {

		KeyStore keyStore = getKeyStore(certificatePath, passwordCert);

		final Certificate[] certificateChain = keyStore
				.getCertificateChain(aliasCert);

		Certificate[] rootCACertificate = new Certificate[1];
		rootCACertificate[0] = certificateChain[0];

		return rootCACertificate;
	}

	/**
	 * Loads KeyStore using certificate file path and password.
	 * 
	 * @param certificatePath
	 *            Original certificate file
	 * @param password
	 *            Certificate Password
	 * @return KeyStore
	 * @throws Exception
	 */
	private static KeyStore getKeyStore(String certificatePath, String password)
			throws IOException, KeyStoreException, NoSuchAlgorithmException,
			CertificateException {
		ByteArrayInputStream bytes = null;
		KeyStore keyStore = null;
		try {
			byte[] keystoreData = FileUtils.readFileToByteArray(new File(
					certificatePath));
			bytes = new ByteArrayInputStream(keystoreData);
			keyStore = KeyStore
				.getInstance(AsymmetricKeyManager.KEYSTORE_PKCS12);
		
			
			keyStore.load(bytes, password.toCharArray());
		} finally {
			if(bytes != null){
				bytes.close();
			}
		}
		

		return keyStore;
	}
	
	/**
	 * Validate certificate, password and alias.
	 * 
	 * @param certificatePath
	 * @param aliasCert
	 *            Certificate Alias
	 * @param passwordCert
	 *            Certificate Password
	 * @return True if valid Certificate and Credentials.
	 * @throws IOException
	 * @throws KeyStoreException
	 * @throws NoSuchAlgorithmException
	 * @throws CertificateException
	 * @throws SignDirException invalid Alias 
	 */
	public static boolean validateCertificate(String certificatePath,
			String alias, String password) throws IOException,
			KeyStoreException, NoSuchAlgorithmException, CertificateException,
			SignDirException {

		KeyStore keyStore = getKeyStore(certificatePath, password);
		X509Certificate x509cert = (X509Certificate) keyStore
				.getCertificate(alias);

		if (x509cert == null) {
			throw new SignDirException("Alias not found," + alias);
		}
		return true;
	}
}
