/**
 * KeystoreDataValidation.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.validator.demographics;

import java.io.ByteArrayInputStream;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.util.Calendar;
import java.util.Date;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.general.KeystoreData;
import com.smartmatic.waffle.security.asymmetric.AsymmetricKeyManager;
import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.validator.ObjectDataValidator;
import com.smartmatic.waffle.validator.ObjectDataValidatorException;
import com.smartmatic.waffle.validator.ObjectDataValidatorException.ExceptionInfo;

/**
 * Class used to validate KeystoreData entity.
 */
public class KeystoreDataValidation extends ObjectDataValidator<KeystoreData> {

	/**
	 * Validates all keystore components: alias, password, certificates and
	 * validity time.
	 * 
	 * @see com.smartmatic.waffle.validator.ObjectDataValidator#prePersist(java.lang
	 *      .Object)
	 */
	@Override
	public void prePersist(KeystoreData component)
			throws ObjectDataValidatorException {

		try {

			if (component.getPkcs12() != null) {

				ByteArrayInputStream pkcs12InputStream = new ByteArrayInputStream(
						component.getPkcs12().getBinaryData());
				try {

					Certificate[] certificates = AsymmetricKeyManager
							.getCertificateChain(pkcs12InputStream,
									component.getPassword(),
									component.getAlias());

					// Check the certificates in the keystore

					if (certificates == null) {
						throw new ObjectDataValidatorException(
								ExceptionInfo.INVALID_KEYSTORE_ALIAS);
					}

					X509Certificate ecmCert = null;

					X509Certificate rootCert = null;

					// Checks that the keystore contains exactly 2 certificates,
					// and that one is self signed (the root) and the other not
					// (the ecm)

					if (certificates.length != 2) {
						throw new ObjectDataValidatorException(
								ExceptionInfo.INVALID_KEYSTORE);
					}

					for (int i = 0; i < certificates.length; i++) {
						X509Certificate certificate = (X509Certificate) certificates[i];

						if (certificate.getIssuerDN().getName()
								.equals(certificate.getSubjectDN().getName())) {
							rootCert = certificate;
						} else if (!certificate.getIssuerDN().getName()
								.equals(certificate.getSubjectDN().getName())) {
							ecmCert = certificate;
						}
					}

					if (ecmCert == null || rootCert == null) {
						throw new ObjectDataValidatorException(
								ExceptionInfo.INVALID_KEYSTORE);
					}

					// check the root certificate authenticates the ecm
					// certificate.

					try {
						ecmCert.verify(rootCert.getPublicKey());
					} catch (Exception e) {
						throw new ObjectDataValidatorException(
								ExceptionInfo.CERTIFICATE_FAIL_VERIFIED,e);
					}

					// check ecm certificate validity dates cover the current
					// date.

					Date now = Calendar.getInstance().getTime();
					Date ecmEndDate = ecmCert.getNotAfter();
					Date ecmOpenDate = ecmCert.getNotBefore();
					Date rootEndDate = ecmCert.getNotAfter();
					Date rootOpenDate = ecmCert.getNotBefore();

					if (ecmEndDate.compareTo(now) < 0
							|| rootEndDate.compareTo(now) < 0) {
						throw new ObjectDataValidatorException(
								ExceptionInfo.KEYSTORE_EXPIRED);
					}

					if (ecmOpenDate.compareTo(now) > 0
							|| rootOpenDate.compareTo(now) > 0) {
						throw new ObjectDataValidatorException(
								ExceptionInfo.KEYSTORE_VALIDATION_BEFORE);
					}
				} finally {
					pkcs12InputStream.close();
				}
			} else {
				throw new ObjectDataValidatorException(
						ExceptionInfo.KEYSTORE_NOT_FOUND);
			}
		} catch (ObjectDataValidatorException e) {
			throw e;
		} catch (CryptographicRuntimeException e) {
			String[] cause = StringUtils.split(e.getCause().toString(), ':');
			if (StringUtils.contains(cause[cause.length - 1].trim(),
					"stream does not represent a PKCS12 key store")) {
				throw new ObjectDataValidatorException(
						ExceptionInfo.INVALID_KEYSTORE, e);
			} else {
				throw new ObjectDataValidatorException(
						ExceptionInfo.INVALID_KEYSTORE_PASSWORD, e);
			}
		} catch (Exception e) {
			throw new ObjectDataValidatorException(
					ExceptionInfo.KEYSTORE_UNKNOWN_ERROR, e);
		}
	}
}
