/**
 * ReflectHelper.java
 *
 * $Revision: 1.4 $ ($Date: 2011-04-06 05:08:29 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.lang.util;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.commons.beanutils.MethodUtils;

/**
 * Utility class to help with common reflection tasks
 * 
 * @author Ithamar Torres (itorres@smartmatic.com)
 * @version 1.0 (01/02/2005)
 */
public class ReflectHelper {

	/**
	 * Method that allows to obtain a field from a class or any of its parents
	 * 
	 * @param theClass
	 *            Class to obtain fields
	 * @param fieldName
	 *            The field name
	 * @return The requested field
	 * @throws NoSuchFieldException
	 *             When the field have not been found
	 */
	public static Field getFieldFromClass(Class theClass, String fieldName)
			throws NoSuchFieldException {
		try {
			Field field = theClass.getDeclaredField(fieldName);
			return field;
		} catch (NoSuchFieldException e) {
			Class superClass = theClass.getSuperclass();
			if (superClass != null) {
				return getFieldFromClass(superClass, fieldName);
			} else {
				throw e;
			}
		}
	}

	/**
	 * Method used to obtain a native type from his string value and the name of
	 * the class that represents the type (ie: java.lang.Integer).
	 * 
	 * @param type
	 *            The name of the class that parse objects of the type required.
	 * @param parseMethod
	 *            The name of the method that parse objects of the type
	 *            required.
	 * @param value
	 *            The string value of the object
	 * @return The requested field
	 * @throws ClassNotFoundException
	 * @throws NoSuchMethodException
	 *             Thrown when a method cannot be found
	 * @throws SecurityException
	 *             Thrown by the security manager to indicate a security
	 *             violation
	 * @throws InvocationTargetException
	 *             Is a checked exception that wraps an exception thrown by
	 *             method or constructor invocation
	 * @throws IllegalAccessException
	 *             An IllegalAccessException is thrown when an application tries
	 *             to reflectively create an instance (other than an array), set
	 *             or get a field, or invoke a method, but the currently
	 *             executing method does not have access to the definition of
	 *             the specified class, field, method or constructor.
	 * @throws IllegalArgumentException
	 *             Thrown indicating that a method has been passed an invalid
	 *             argument
	 */
	public static Object parseNativeType(String type, String parseMethod,
			String value) throws ClassNotFoundException, SecurityException,
			NoSuchMethodException, IllegalArgumentException,
			IllegalAccessException, InvocationTargetException {
		Class theClass = Class.forName(type);
		Class[] typeReceived = { String.class };
		Method method = theClass.getMethod(parseMethod, typeReceived);
		return method.invoke(null, value);
	}

	/**
	 * Return the property name using from the getter method using naming
	 * conventions: isPROPERTY_NAME(...) or getPROPERTY_NAME(...)
	 * 
	 * @param method
	 *            the getter method
	 * @return The string property name
	 */
	public static String getPropertyNameFromGetter(Method method) {
		/* getter naming conventions: isXxx, getYyy */
		String getter = method.getName();
		String subStr = getter.substring(getter.startsWith("is") ? 2 : 3);
		char ch = subStr.charAt(0);
		if (Character.isUpperCase(ch)) {
			ch = Character.toLowerCase(ch);
		} else {
			ch = Character.toUpperCase(ch);
		}
		String end = subStr.substring(1);
		return ch + end;
	}

	/**
	 * Method used to help to get the return type of a getter using reflection.
	 * 
	 * @param theClass
	 *            The class to retrieve
	 * @param fieldName
	 *            The field name to retrieve
	 * @return The return type
	 * @throws SecurityException
	 *             Thrown by the security manager to indicate a security
	 *             violation
	 * @throws NoSuchMethodException
	 *             Thrown when a method cannot be found
	 * @throws IllegalArgumentException
	 *             Thrown indicating that a method has been passed an invalid
	 *             argument
	 */
	@SuppressWarnings("unchecked")
	public static Class getGetterReturnType(Class theClass, String fieldName)
			throws SecurityException, NoSuchMethodException,
			IllegalArgumentException {
		String upperFirstLetter = Character.toUpperCase(fieldName.charAt(0))
				+ fieldName.substring(1);

		// Obtains the return type of corresponding getter method
		String methodName = "get" + upperFirstLetter;
		Method method = theClass.getMethod(methodName);
		return method.getReturnType();
	}

	/**
	 * Invokes the method <code>methodName</code> on the object <code>obj</code>
	 * with the parameter <code>parameters</code>.
	 * 
	 * @param obj
	 *            The object that contains the method
	 * @param methodName
	 *            The name of the methods to invoke
	 * @param parameters
	 *            The parameters to supply to the method
	 * @return The method result
	 * @throws NoSuchMethodException
	 *             Thrown when a method cannot be found
	 * @throws IllegalAccessException
	 *             An IllegalAccessException is thrown when an application tries
	 *             to reflectively create an instance (other than an array), set
	 *             or get a field, or invoke a method, but the currently
	 *             executing method does not have access to the definition of
	 *             the specified class, field, method or constructor.
	 * @throws InvocationTargetException
	 *             Is a checked exception that wraps an exception thrown by
	 *             method or constructor invocation
	 */
	public static Object invokeMethod(Object obj, String methodName,
			Object... parameters) throws NoSuchMethodException,
			IllegalAccessException, InvocationTargetException {
		return MethodUtils.invokeMethod(obj, methodName, parameters);
	}

	/**
	 * Method used to help to invoke a getter using reflection.
	 * 
	 * @param bcomponent
	 *            Object where the getter needs to be invoket
	 * @param fieldName
	 *            Field name of the field asociated whith the getter
	 * @return The value of the field
	 * @throws SecurityException
	 * @throws NoSuchMethodException
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 */
	public static Object invokeGetter(Object bcomponent, String fieldName)
			throws SecurityException, NoSuchMethodException,
			IllegalArgumentException, IllegalAccessException,
			InvocationTargetException {
		Class theClass = bcomponent.getClass();
		String methodName = "get" + Character.toUpperCase(fieldName.charAt(0))
				+ fieldName.substring(1);
		Method method = theClass.getMethod(methodName, null);
		return method.invoke(bcomponent, null);
	}

	/**
	 * Method used to help to invoke a seter using reflection.
	 * 
	 * @param bcomponent
	 *            Object where the setter needs to be invoket
	 * @param fieldName
	 *            Field name of the field asociated whith the setter
	 * @param value
	 *            The value of the field
	 * @throws SecurityException
	 * @throws NoSuchMethodException
	 * @throws NoSuchFieldException
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 */
	public static void invokeSetter(Object bcomponent, String fieldName,
			Object value) throws SecurityException, NoSuchMethodException,
			NoSuchFieldException, IllegalArgumentException,
			IllegalAccessException, InvocationTargetException {
		Class theClass = bcomponent.getClass();
		String upperFirstLetter = Character.toUpperCase(fieldName.charAt(0))
				+ fieldName.substring(1);

		// Obtains the return type of corresponding getter method
		String methodName = "get" + upperFirstLetter;
		Method method = theClass.getMethod(methodName, null);
		Class returnType = method.getReturnType();

		// Search and invokes the setter method
		methodName = "set" + upperFirstLetter;
		method = theClass.getMethod(methodName, returnType);
		method.invoke(bcomponent, value);
	}
}
