/**
 * @file	InstallationPrintDocument.h
 * @brief	Declaration of entity that prints the installation report.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#ifndef _INSTALLATIONPRINTDOCUMENT_H_
#define _INSTALLATIONPRINTDOCUMENT_H_

#include "Printing/BasePrintDocument.h"
#include "Configuration/s-a-e-s-conf-manager.h"
#include "Graphics/Encoding2D/QREncoding.h"
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include "Voting/Status/voting-machine-status-schema.hxx"
#include "Printing/DocumentOptions.h"
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            class ElectoralConfiguration;
            class OperationStatus;
        }
        namespace Printing
        {
            /**
             * @class	InstallationPrintDocument
             * @brief	 Class which contains the printing operation for the installation report.
             * @author	Eduardo.campione
             * @date	28/04/2011
             */

            class InstallationPrintDocument : public BasePrintDocument
            {
                public:

                    /**
                     * @fn	InstallationPrintDocument::InstallationPrintDocument (Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                     * 			Smartmatic::SAES::Voting::OperationStatus * operationStatus, bool InstallationReport, bool createBarcodeOnConstructor);
                     * @brief	 Constructor initializes an instance of the InstallationPrintDocument class.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	electoralConfiguration The pointer reference to electoral configuration.
                     * @param	InstallationReport					true for install the report.
                     * @param	createBarcodeOnConstructor	true to create barcode on constructor.
                     */

                    InstallationPrintDocument ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                                                Smartmatic::SAES::Voting::OperationStatus * operationStatus,
                                                bool InstallationReport,
                                                bool createBarcodeOnConstructor );

                    /**
                     * @fn	void InstallationPrintDocument::PrintDoc();
                     * @brief	Print document.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    void PrintDoc();

                    /**
                     * @fn	void InstallationPrintDocument::BeginPrint();
                     * @brief	Begins a print.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    void BeginPrint();

                    /**
                     * @fn	void InstallationPrintDocument::EndPrint();
                     * @brief	Ends a print.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    void EndPrint();

                    /**
                     * @fn	void InstallationPrintDocument::setIsPasswordRetry(bool isPasswordRetryA);
                     * @brief	Set is password retry.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	isPasswordRetryA	true if this object is password retry a.
                     */
                    void setIsPasswordRetry ( bool isPasswordRetryA );

                    /**
                     * @fn	void InstallationPrintDocument::setPixbufBarCode(Glib::RefPtr<Gdk::Pixbuf> pixbufBarCode);
                     * @brief	Sets pixbuf bar code.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	pixbufBarCode	The pixbuf bar code.
                     */
                    void setPixbufBarCode ( Glib::RefPtr<Gdk::Pixbuf> pixbufBarCode );

                    /**
                     * @fn	Glib::RefPtr<Gdk::Pixbuf> InstallationPrintDocument::GetQRCodeVotingMachineStatus();
                     * @brief	Gets qr code voting machine status.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @return	The qr code voting machine status.
                     */
                    Glib::RefPtr<Gdk::Pixbuf> GetQRCodeVotingMachineStatus();

                private:

                    Smartmatic::SAES::Voting::ElectoralConfiguration * _electoralConfiguration;	///< The electoral configuration
                    Smartmatic::SAES::Voting::OperationStatus * operationStatus; ///< The operation status
                    Smartmatic::SAES::Configuration::ConfigurationInstallation configurationInstallation; ///< The configuration installation

                    /**
                     * @fn	void InstallationPrintDocument::AddHeader(Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel1,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel2,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel3,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationInfo);
                     * @brief	Add the header to the report.
                     * @author	Freddy.Gomez
                     * @date	23/03/2011
                     * @param [in,out]	textInstallationTitleLevel1	The first text installation title level.
                     * @param [in,out]	textInstallationTitleLevel2	The second text installation title level.
                     * @param [in,out]	textInstallationTitleLevel3	The third text installation title level.
                     * @param [in,out]	textInstallationInfo			 	Information describing the text installation.
                     */
                    void AddHeader ( Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel1,
                                     Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel2,
                                     Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel3,
                                     Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationInfo );

                    /**
                     * @fn	void InstallationPrintDocument::AddRegions(Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel1,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel2,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel3,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationInfo);
                     * @brief	Add the region info to the report.
                     * @author	Freddy.Gomez
                     * @date	23/03/2011
                     * @param [in,out]	textInstallationTitleLevel1	The first text installation title level.
                     * @param [in,out]	textInstallationTitleLevel2	The second text installation title level.
                     * @param [in,out]	textInstallationTitleLevel3	The third text installation title level.
                     * @param [in,out]	textInstallationInfo			 	Information describing the text installation.
                     */
                    void AddRegions ( Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel1,
                                      Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel2,
                                      Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel3,
                                      Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationInfo );

                    /**
                     * @fn	void InstallationPrintDocument::AddDeviceStatus(Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel1,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel2,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel3,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationInfo);
                     * @brief	Add the device status to the report.
                     * @author	Freddy.Gomez
                     * @date	23/03/2011
                     * @param [in,out]	textInstallationTitleLevel1	The first text installation title level.
                     * @param [in,out]	textInstallationTitleLevel2	The second text installation title level.
                     * @param [in,out]	textInstallationTitleLevel3	The third text installation title level.
                     * @param [in,out]	textInstallationInfo			 	Information describing the text installation.
                     */
                    void AddDeviceStatus ( Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel1,
                                           Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel2,
                                           Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel3,
                                           Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationInfo );

                    /**
                     * @fn	void InstallationPrintDocument::AddKeys(Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel1,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel2,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationTitleLevel3,
                     * 			Smartmatic::SAES::Printing::DocumentTextOptions& textInstallationInfo);
                     * @brief	Add the activation key to the report.
                     * @author	Freddy.Gomez
                     * @date	23/03/2011
                     * @param [in,out]	textInstallationTitleLevel1	The first text installation title level.
                     * @param [in,out]	textInstallationTitleLevel2	The second text installation title level.
                     * @param [in,out]	textInstallationTitleLevel3	The third text installation title level.
                     * @param [in,out]	textInstallationInfo			 	Information describing the text installation.
                     */
                    void AddKeys ( Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel1,
                                   Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel2,
                                   Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel3,
                                   Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationInfo );

                    /**
                     * @fn	std::string InstallationPrintDocument::getType(Smartmatic::SAES::Common::Configuration::DeviceType & type);
                     * @brief	method get string type.
                     * @author	Yadickson Soto
                     * @date	19/09/2012
                     * @param	type	type.
                     * @return	string type.
                     */
                    std::string getType ( Smartmatic::SAES::Common::Configuration::DeviceType & type );
                    /**
                     * @fn	std::string InstallationPrintDocument::getStatus(Smartmatic::SAES::Voting::Status::VMStatus & status);
                     * @brief	method get string status.
                     * @author	Yadickson Soto
                     * @date	19/09/2012
                     * @param	status	status.
                     * @return	string status.
                     */
                    std::string getStatus ( Smartmatic::SAES::Voting::Status::VMStatus & status );
                    /**
                     * @fn	std::vector<Glib::ustring> InstallationPrintDocument::GetRegionsStrings(const Smartmatic::SAES::Voting::VotingDevice::Voting_booth & votingBooth, std::list<Glib::ustring> & list );
                     * @brief	Gets the regions strings.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	votingBooth	The voting booth.
                     * @param	list The regions strings result.
                     */

                    void GetRegionsStrings ( const Smartmatic::SAES::Voting::VotingDevice::Voting_booth & votingBooth, std::list<Glib::ustring> & list );

                    bool installationReport;	///< true to installation report
                    Glib::RefPtr<Gdk::Pixbuf> pixbufBarCode;	///< The pixbuf bar code
                    bool isPasswordRetry; ///< true if this object is password retry

                    static Smartmatic::Log::ISMTTLog * logger; ///< logger
            };
        }
    }
}

#endif // _INSTALLATIONPRINTDOCUMENT_H_
