/**
 * @file	ReportPrintingController.h
 * @brief	Declaration of entity that prints the reports collection obtained  from <tt>SAESCommon</tt> settings.
 * @author	Youssef.tourkmani
 * @author      manuel.arguelles
 * @date	31/10/2012
 */

#ifndef REPORTPRINTINGCONTROLLER_H_
#define REPORTPRINTINGCONTROLLER_H_

#include <Configuration/SaesConfiguration-schema.hxx>
#include <Printing/IBasePrintDocument.hxx>
#include <glibmm.h>
#include <list>
#include <vector>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Printing
        {
            class PrintDocumentClient;

            /**
             * @class	ReportPrintingController
             * @brief	Controller which contains the printing operation for the reports collection obtained  from <tt>SAESCommon</tt> settings.
             * @author	Eduardo.campione
             * @date	28/04/2011
             */
            class ReportPrintingController : public IBasePrintDocument
            {

                public:

                    /**
                     * @fn	ReportPrintingController::ReportPrintingController();
                     * @brief	Default constructor initializes an instance of the ReportPrintingController class.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */
                    ReportPrintingController();

                    /**
                     * @fn	virtual ReportPrintingController::~ReportPrintingController();
                     * @brief	Finalizes the instance of the ReportPrintingController class.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */
                    virtual ~ReportPrintingController();

                    /**
                     * @fn	Glib::Dispatcher& ReportPrintingController::signalReportChanged ()
                     * @brief	Raises the report changed event.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return The reference to slot for the changed event.
                     */
                    Glib::Dispatcher& signalReportChanged();

                    /**
                     * @fn	Glib::Dispatcher& ReportPrintingController::signalReportPrintFinished ()
                     * @brief	Notifies when the print operation is finished.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return The reference to slot for the print finished event.
                     */
                    Glib::Dispatcher& signalReportPrintFinished();

                    /**
                     * @fn	Glib::Dispatcher& ReportPrintingController::signalReportPrintErrorDeteced ()
                     * @brief	Notifies when occurs an error while trying print the report.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return The reference to slot for the error event.
                     */
                    Glib::Dispatcher& signalReportPrintErrorDeteced();

                    /**
                     * @fn	void ReportPrintingController::loadReports();
                     * @brief	Loads the reports collection specified in the <tt>SAESCommon</tt> settings.
                     * @author	Yadickson Soto
                     * @date	05/04/2012
                     */
                    void loadReports();

                    /**
                     * @fn	void ReportPrintingController::Print();
                     * @brief	Prints the report list.
                     * This method triggers the callback client which describes the printer operation.
                     * @author	Yadickson Soto
                     * @date	29/03/2012
                     */
                    void Print();

                    /**
                     * @fn	void ReportPrintingController::setClient (PrintDocumentClient * client);
                     * @brief	Sets the reference to the client used to retrieve the printer result.
                     * @author	Yadickson Soto
                     * @date	30/11/2012
                     * @param	client	The pointer reference to client instance.
                     */
                    void setClient(PrintDocumentClient* client);

                    /**
                     * @fn	int ReportPrintingController::GetNumOfCopies ();
                     * @brief	Gets the number of copies.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return	The number of copies.
                     */
                    int GetNumOfCopies();

                    /**
                     * @fn	int ReportPrintingController::GetNumOfCppiesPrinted ();
                     * @brief	Gets the number of copies printed.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return	The number of copies printed.
                     */
                    int GetNumOfCopiesPrinted();

                    /**
                     * @fn	const std::vector< ::Smartmatic::SAES::Configuration::ConfigurationReportFile>& ReportPrintingController::GetReportList ();
                     * @brief	Gets a reference to report list.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return	The report list.
                     */
                    const std::vector<SAES::Configuration::ConfigurationReportFile>& GetReportList();

                    /**
                     * @fn	void ReportPrintingController::setReportsList();
                     * @brief	Sets the reports list.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     */
                    void setReportsList();

                    /**
                     * @fn bool verifySignature();
                     * @brief Verifies the reports signatures.
                     * @author Eduardo Fernández
                     * @date   11/10/2018
                     * @return true if the signatures are verified successfully.
                     */
                    bool verifySignature();

                    /**
                     * @fn	int ReportPrintingController::getTotalDocumentsToPrint();
                     * @brief	Gets total documents to print.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @return	The total documents to print.
                     */
                    int getTotalDocumentsToPrint();

                private:
                    PrintDocumentClient* m_client;
                    std::vector<SAES::Configuration::ConfigurationReportFile> m_reportList;
                    Glib::Dispatcher m_signalPrintFinished;
                    Glib::Dispatcher m_signalErrorDetected;
                    Glib::Dispatcher m_signalPrintFile;
                    Glib::ustring m_reportName;
                    Glib::ustring m_reportDescription;
                    int m_numOfCopies;
                    int m_numPrinted;
                    std::list<std::string> m_languagesFullName;
                    std::list<std::string> m_fileNameReportList;
                    bool m_allLanguages;
            };

        }
    }
}

#endif /* REPORTPRINTINGCONTROLLER_H_ */
