/**
 * @file	card-data-wrapper.cc
 * @brief	card data wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include <Security/Encryption.hxx>
#include "System/Compression/compression-class.h"
#include "System/Encoding/Base64Encoder.h"
#include "Voting/CardData/card-data-wrapper.h"
#include <Voting/CardData/CardDataFactory.hxx>
#include <Voting/VotingDevice/LogicalVoterGroupFactory.hxx>
#include "Configuration/s-a-e-s-conf-manager.h"
#include <Log/SMTTLogManager.h>
#include <glibmm/timeval.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::System;
using namespace Smartmatic::System::Compression;
using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::CardData;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Voting::VotingDevice;

ISMTTLog * CardDataWrapper::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.VotingDevice.CardDataWrapper", GETTEXT_PACKAGE );

CardDataWrapper::CardDataWrapper()
{
}

CardDataWrapper::~CardDataWrapper()
{
}

bool CardDataWrapper::checkIfCardTimeIsValid ( std::string toCheck )
{
    if ( Smartmatic::SAES::Configuration::SAESConfigurationManager::getInstance()->getCardValidity() == 0 )
    {
        return true;
    }
    else
    {
        Glib::TimeVal timeValCreation;
        timeValCreation.assign_from_iso8601 ( toCheck );
        Glib::TimeVal timeValNow;
        timeValNow.assign_current_time ();
        timeValCreation.add_seconds ( Smartmatic::SAES::Configuration::SAESConfigurationManager::getInstance()->getCardValidity() );

        if ( timeValCreation < timeValNow )
        {
            return false;
        }
        else
        {
            return true;
        }
    }
}

void CardDataWrapper::CompressCardData ( Card_data & cardData, SafeBuffer & dataOut )
{
    dataOut.Clear();

    SafeBuffer bufferCardData;

    if ( CardDataFactory::cardDataToBuffer ( cardData, bufferCardData ) )
    {
        CompressionClass::getInstance()->Deflate ( bufferCardData, dataOut );
    }
    else
    {
        logger->Debug ( "FATAL - Error CompressCardData" );
    }
}

void CardDataWrapper::DecompressCardData ( SafeBuffer & data, Card_data & cardDataOut )
{
    SafeBuffer bufferInflate;
    CompressionClass::getInstance()->Inflate ( data, bufferInflate );

    if ( !CardDataFactory::bufferToCardData ( bufferInflate, cardDataOut ) )
    {
        logger->Debug ( "FATAL - Error DecompressCardData" );
    }
}

std::string CardDataWrapper::GetTimeNow()
{
    Glib::TimeVal timeValNow;
    timeValNow.assign_current_time ();
    return timeValNow.as_iso8601();
}

std::string CardDataWrapper::GenerateCypheredActivationCode()
throw ( Smartmatic::System::Exception::CryptoException )
{
    std::string uncypheredDate = GetTimeNow();
    std::string cypheredDate =
        Smartmatic::SAES::Security::Encryption::getInstance()->cryptoStringToB64 (
            uncypheredDate,
            Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );
    return cypheredDate;
}

std::string CardDataWrapper::GetUncypheredActivationCode ( std::string cypheredDate )
throw ( Smartmatic::System::Exception::CryptoException )
{
    std::string uncypheredDate =
        Smartmatic::SAES::Security::Encryption::getInstance()->decryptoB64ToString (
            cypheredDate,
            Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );
    return uncypheredDate;
}

void CardDataWrapper::GenerateCardData ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
        Smartmatic::SAES::Voting::OperationStatus * operationStatus,
        std::vector<Smartmatic::SAES::Voting::VotingDevice::Custom_attribute> attributes, Card_data & cardData,
        bool cryptoCode, Smartmatic::SAES::Voting::CardData::Card_type type, bool isAudio )
throw ( Smartmatic::System::Exception::CryptoException )
{
    Logical_voter_group logicalVoterGroup = LogicalVoterGroupFactory::createLogicalVoterGroup ( attributes );

    Voting_device vd = electoralConfiguration->getVotingDevice()->getFirstVotingDevice();
    std::list <Region_category> & listCategoryRegion ( electoralConfiguration->getRegionCategoryList() );

    if ( !listCategoryRegion.empty() )
    {
        Region_category category = ( * ( listCategoryRegion.rbegin() ) );
        logicalVoterGroup.setRegion_code ( category.getRegion().getCode () );
    }

    logicalVoterGroup.setOperation_mode_code ( vd.getOperation_modes ().getOperation_mode () [0].getCode() );
    logicalVoterGroup.setVoting_booth_code ( vd.getVoting_booths ().getVoting_booth () [0].getCode() );

    cardData = CardDataFactory::createCardData ( logicalVoterGroup, operationStatus );

    if ( cryptoCode )
    {
        cardData.setActivation_code ( GenerateCypheredActivationCode() );
    }
    else
    {
        cardData.setActivation_code ( "" );
    }

    cardData.setCard_type ( type );
    cardData.setCard_status ( Smartmatic::SAES::Voting::CardData::Card_status::CARD_DATA_NOT_USED );
    cardData.setIsAudio ( isAudio );
}

bool CardDataWrapper::isUsed ( Card_data & cardData )
{
    return cardData.getCard_status() == CardData::Card_status::CARD_DATA_USED;
}

void CardDataWrapper::markUsed ( Card_data & cardData )
{
    cardData.setCard_status ( CardData::Card_status::CARD_DATA_USED );
    cardData.setBallotID ( "" );
    cardData.setSessionID ( "" );
}
