/**
 * @file	card-data-wrapper.h
 * @brief	Declares the card data wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#ifndef _CARD_DATA_WRAPPER_H_
#define _CARD_DATA_WRAPPER_H_

#include <time.h>
#include <Voting/VotingDevice/logical-voter-group-schema.hxx>
#include <Voting/CardData/card-data-schema.hxx>
#include <System/Exception/CryptoException.hxx>
#include <System/Exception/SignedException.hxx>
#include <System/SafeBuffer.h>
#include <Voting/ElectoralConfiguration.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            class OperationStatus;

            namespace CardData
            {
                /**
                 * @class	CardDataWrapper
                 * @brief	Card data wrapper.
                 * @author	Eduardo.campione
                 * @date	28/04/2011
                 */

                class CardDataWrapper
                {
                    public:

                        /**
                         * @fn	CardDataWrapper::CardDataWrapper();
                         * @brief	Default constructor.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        CardDataWrapper();

                        /**
                         * @fn	virtual CardDataWrapper::~CardDataWrapper();
                         * @brief	Finaliser.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        virtual ~CardDataWrapper();

                        /**
                         * @fn	static void CardDataWrapper::GenerateCardData(Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                         *                              Smartmatic::SAES::Voting::OperationStatus * operationStatus,
                         *                              std::vector<Smartmatic::SAES::Voting::VotingDevice::Custom_attribute> attributes,
                         *                              Card_data & cardDataOut,
                         *                              bool cryptoCode,
                         *                              Smartmatic::SAES::Voting::CardData::Card_type type,
                         *                              bool isAudio = false ) throw (Smartmatic::System::Exception::CryptoException);
                         * @brief	Generates a card data.
                         * @author	Yadickson Soto
                         * @date	28/04/2014
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                         * 																														occurs.
                         * @param	electoralConfiguration	The electoral configuration.
                         * @param	operationStatus The operation status.
                         * @param	attributes							If non-null, the attributes.
                         * @param	cardDataOut							result card data.
                         * @param	cryptoCode							crypto code.
                         * @param	type										The type.
                         * @param	isAudio									is card audio.
                         */

                        static void GenerateCardData ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                                                       Smartmatic::SAES::Voting::OperationStatus * operationStatus,
                                                       std::vector<Smartmatic::SAES::Voting::VotingDevice::Custom_attribute> attributes,
                                                       Card_data & cardDataOut,
                                                       bool cryptoCode,
                                                       Smartmatic::SAES::Voting::CardData::Card_type type,
                                                       bool isAudio = false )
                        throw ( Smartmatic::System::Exception::CryptoException );

                        /**
                         * @fn	static void CardDataWrapper::CompressCardData(Card_data & cardData,
                         * 			Smartmatic::System::SafeBuffer & dataout);
                         * @brief	Compress card data.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	cardData	If non-null, information describing the card.
                         * @param	dataout 	The dataout.
                         */

                        static void CompressCardData ( Card_data & cardData, Smartmatic::System::SafeBuffer & dataout );

                        /**
                         * @fn	static void CardDataWrapper::DecompressCardData(Smartmatic::System::SafeBuffer & data,
                         * 			Card_data & cardDataOut);
                         * @brief	Decompress card data.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	data			 	If non-null, information describing the card.
                         * @param	cardDataOut	The card data out.
                         */

                        static void DecompressCardData ( Smartmatic::System::SafeBuffer & data, Card_data & cardDataOut );

                        /**
                         * @fn	static bool CardDataWrapper::checkIfCardTimeIsValid(std::string toCheck);
                         * @brief	Check if card time is valid.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	toCheck	to check.
                         * @return	true if it succeeds, false if it fails.
                         */
                        static bool checkIfCardTimeIsValid ( std::string toCheck );

                        /**
                         * @fn	static std::string CardDataWrapper::GenerateCypheredActivationCode() throw (Smartmatic::System::Exception::CryptoException);
                         * @brief	Generates a cyphered activation code.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                         * 																														occurs.
                         * @return	The cyphered activation code.
                         */

                        static std::string GenerateCypheredActivationCode() throw ( Smartmatic::System::Exception::CryptoException );

                        /**
                         * @fn	static std::string CardDataWrapper::GetUncypheredActivationCode(std::string cypheredDate) throw (Smartmatic::System::Exception::CryptoException);
                         * @brief	Gets a uncyphered activation code.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                         * 																														occurs.
                         * @param	cypheredDate	Date of the cyphered.
                         * @return	The uncyphered activation code.
                         */

                        static std::string GetUncypheredActivationCode ( std::string cypheredDate ) throw ( Smartmatic::System::Exception::CryptoException );

                        /**
                         * @fn	static std::string CardDataWrapper::GetTimeNow();
                         * @brief	Gets the time now.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	The time now.
                         */

                        static std::string GetTimeNow();

                        /**
                         * @fn static bool isUsed ( Card_data & cardData );
                         * @brief Get if card data is used
                         * @date 13/05/2013
                         * @author Yadickson Soto
                         * @param cardData card data to validate
                         * @return true is used
                         */
                        static bool isUsed ( Card_data & cardData );

                        /**
                         * @fn static void markUsed ( Card_data & cardData );
                         * @brief Set card data used, clean ballot id and session id
                         * @date 13/05/2013
                         * @author Yadickson Soto
                         * @param cardData card data to mark
                         */
                        static void markUsed ( Card_data & cardData );

                    private:

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif // _CARD_DATA_WRAPPER_H_
