/**
 * @file	ElectionInstalationLanguages.h
 * @brief	Declares the election instalation languages class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef ELECTIONINSTALATIONLANGUAGES_H_
#define ELECTIONINSTALATIONLANGUAGES_H_

#include "Voting/Election/election-language-schema.hxx"
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include <System/Exception/CryptoException.hxx>
#include <System/Exception/SignedException.hxx>
#include <System/Exception/XmlException.hxx>
#include <list>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Voting
        {
            namespace Election
            {
                /**
                 * @class	ElectionInstalationLanguages
                 * @brief	Election instalation languages.
                 * @author	Eduardo.campione
                 * @date	28/04/2011
                 */

                class ElectionInstalationLanguages
                {
                    private:

                        /**
                         * @fn	ElectionInstalationLanguages::ElectionInstalationLanguages();
                         * @brief	Default constructor.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        ElectionInstalationLanguages();
                        static ElectionInstalationLanguages * instance; ///< The instance

                    public:

                        /**
                         * @fn	std::list<Smartmatic::SAES::Voting::Election::Language> ElectionInstalationLanguages::getLanguages();
                         * @brief	Gets the languages.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	The languages.
                         */

                        std::list<Smartmatic::SAES::Voting::Election::Language> getLanguages();

                        /**
                         * @fn	void ElectionInstalationLanguages::setInstallationLanguages(Smartmatic::SAES::Voting::Election::Language& lang);
                         * @brief	Sets an installation languages.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	lang	The language.
                         */

                        void setInstallationLanguages ( Smartmatic::SAES::Voting::Election::Language & lang );

                        /**
                         * @fn	void ElectionInstalationLanguages::addInstallationLanguages(Smartmatic::SAES::Voting::Election::Language& lang);
                         * @brief	Adds an installation languages.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	lang	The language.
                         */

                        void addInstallationLanguages ( Smartmatic::SAES::Voting::Election::Language & lang );

                        /**
                         * @fn	virtual ElectionInstalationLanguages::~ElectionInstalationLanguages();
                         * @brief	Finaliser.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        virtual ~ElectionInstalationLanguages();

                        /**
                         * @fn	void ElectionInstalationLanguages::persistLanguage() throw (Smartmatic::System::Exception::CryptoException,
                         * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                         * @brief	Persist language.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                         * 																														occurs.
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                         * 																														occurs.
                         * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                         * 																														occurs.
                         */

                        void persistLanguage()
                        throw ( Smartmatic::System::Exception::CryptoException,
                                Smartmatic::System::Exception::SignedException,
                                Smartmatic::System::Exception::XmlException );

                        /**
                         * @fn	std::list <std::string> ElectionInstalationLanguages::getStringLanguages();
                         * @brief	Method getter all string languages.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	11/05/2011
                         * @return	list string languages in format ll_CC.
                         */

                        std::list <std::string> getStringLanguages();

                        /**
                         * @fn	void ElectionInstalationLanguages::clearLanguage();
                         * @brief	Clears the language.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void clearLanguage();

                        /**
                         * @fn	void ElectionInstalationLanguages::ReloadLanguages();
                         * @brief	Reload languages.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void ReloadLanguages();

                        /**
                         * @fn	static ElectionInstalationLanguages * ElectionInstalationLanguages::getInstance();
                         * @brief	Gets the instance.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	null if it fails, else the instance.
                         */

                        static ElectionInstalationLanguages * getInstance();

                        /**
                         * @fn	static std::list<Smartmatic::SAES::Voting::Election::Language> ElectionInstalationLanguages::GetOperatingLanguages(std::list<Smartmatic::SAES::Voting::Election::Language> allLang);
                         * @brief	Gets operating languages.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	allLang	all language.
                         * @return	The operating languages.
                         */
                        static std::list<Smartmatic::SAES::Voting::Election::Language> GetOperatingLanguages ( std::list<Smartmatic::SAES::Voting::Election::Language> allLang );
                    private:
                        std::list<Smartmatic::SAES::Voting::Election::Language> languages;	///< The languages

                        /**
                         * @fn	bool ElectionInstalationLanguages::LoadFile(std::string file);
                         * @brief	Loads a file.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	file	The file.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool LoadFile ( std::string file );
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}

#endif /* ELECTIONINSTALATIONLANGUAGES_H_ */
