/**
 * @file	ElectionWrapper.hxx
 * @brief	election wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef __ELECTION_WRAPPER_H__
#define __ELECTION_WRAPPER_H__

#include <list>
#include <vector>
#include <Voting/Election/election-schema.hxx>
#include <System/Exception/ElectionException.hxx>
#include <Runtime/Environment/saes-directories.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            namespace Election
            {
                /**
                 * @class	ElectionWrapper
                 * @brief	Election wrapper.
                 * @author	Yadickson Soto
                 * @date	28/04/2011
                 */

                class ElectionWrapper
                {
                    public:

                        /**
                         * @fn	ElectionWrapper::ElectionWrapper(Election & election);
                         * @brief	Constructor.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	election	The election.
                         */

                        explicit ElectionWrapper ( Election & election );

                        /**
                         * @fn	virtual ElectionWrapper::~ElectionWrapper();
                         * @brief	Finaliser.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        virtual ~ElectionWrapper();

                        /**
                         * @fn	Election & ElectionWrapper::getElection ();
                         * @brief	Gets the election.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	The election.
                         */

                        Election & getElection ();

                        /**
                         * @fn Checksums getChecksums();
                         * @brief  Gets the files checksums.
                         * @author eduardo.fernandez
                         * @date   07/01/2019
                         * @return The files checksums.
                         */
                        Election::ChecksumsOptional getChecksums();

                        /**
                         * @fn	Ballots & ElectionWrapper::getBallots ();
                         * @brief	Gets the ballots.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	The ballots.
                         */

                        Ballots & getBallots ();

                        /**
                         * @fn	Ballot & ElectionWrapper::getBallot ( std::string codeBallot ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a ballot.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	codeBallot	The code ballot.
                         * @return	The ballot.
                         */

                        Ballot & getBallot ( std::string codeBallot )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Ballot_cards & ElectionWrapper::getBallotCards ( std::string codeBallot ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a ballot cards.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	codeBallot	The code ballot.
                         * @return	The ballot cards.
                         */

                        Ballot_cards & getBallotCards ( const std::string& codeBallot )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Ballot_cards & ElectionWrapper::getBallotCards ( Ballot & ballot );
                         * @brief	Gets a ballot cards.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	ballot	The ballot.
                         * @return	The ballot cards.
                         */

                        Ballot_cards & getBallotCards ( Ballot & ballot );

                        /**
                         * @fn	Ballot_card ElectionWrapper::getBallotCard ( std::string codeBallotCard,
                         * 			Ballot & ballot ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a ballot card.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	codeBallotCard	The code ballot card.
                         * @param	ballot					The ballot.
                         * @return	The ballot card.
                         */

                        Ballot_card getBallotCard ( std::string codeBallotCard, Ballot & ballot )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Contests & ElectionWrapper::getContests ( Ballot_card & ballotCard );
                         * @brief	Gets the contests.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	ballotCard	The ballot card.
                         * @return	The contests.
                         */

                        Contests & getContests ( Ballot_card & ballotCard );

                        /**
                         * @fn	Contest & ElectionWrapper::getContest ( std::string codeContest,
                         * 			Ballot_card & ballotCard ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a contest.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	codeContest	The code contest.
                         * @param	ballotCard 	The ballot card.
                         * @return	The contest.
                         */

                        Contest & getContest ( std::string codeContest, Ballot_card & ballotCard )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Eligibles_options & ElectionWrapper::getEligiblesOptions ( Contest & contest );
                         * @brief	method to obtain the eligibles options.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	contest	root contest of eligibles options.
                         * @return	a reference of the eligibles options.
                         */

                        Eligibles_options & getEligiblesOptions ( Contest & contest ) ;

                        /**
                         * @fn	Eligible_option & ElectionWrapper::getEligibleOption ( std::string codeEligibleOption,
                         * 			Contest & contest ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets an eligible option.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	codeEligibleOption	The code eligible option.
                         * @param	contest							The contest.
                         * @return	The eligible option.
                         */

                        Eligible_option & getEligibleOption ( std::string codeEligibleOption, Contest & contest )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Ballot_options & ElectionWrapper::getBallotOptions ( Eligible_option & eligibleOption );
                         * @brief	method to obtain the ballot options.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	eligibleOption	root eligible option of ballot options.
                         * @return	a reference of the ballot options.
                         */

                        Ballot_options & getBallotOptions ( Eligible_option & eligibleOption );

                        /**
                         * @fn	Ballot_option & ElectionWrapper::getBallotOptionByEligibleOption ( Eligible_option & eligibleOption ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a ballot option by eligible option.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	eligibleOption	The eligible option.
                         * @return	The ballot option by eligible option.
                         */

                        Ballot_option & getBallotOptionByEligibleOption ( Eligible_option & eligibleOption )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Ballot_option & ElectionWrapper::getBallotOption ( std::string codeBallotOption,
                         * 			Eligible_option & eligibleOption ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Constructor.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	codeBallotOption					of the exception.
                         * @param [in,out]	eligibleOption	The eligible option.
                         * @return	The ballot option.
                         */
                        Ballot_option & getBallotOption ( std::string codeBallotOption, Eligible_option & eligibleOption )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Pad_position_group & ElectionWrapper::getPadPositionGroup ( Ballot_option & ballotOption );
                         * @brief	method to obtain the pad position group.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	ballotOption	root ballot option of pad position group.
                         * @return	a reference of the pad position group.
                         */

                        Pad_position_group & getPadPositionGroup ( Ballot_option & ballotOption );

                        /**
                         * @fn	Languages & ElectionWrapper::getLanguages ();
                         * @brief	method to obtain the languages.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	a reference of the languages.
                         */

                        Languages & getLanguages ();

                        /**
                         * @fn	bool ElectionWrapper::IsLanguagesPresent ();
                         * @brief	Query if this object is languages present.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	true if languages present, false if not.
                         */
                        bool IsLanguagesPresent ();

                        /**
                         * @fn	Contest_classes & ElectionWrapper::getContestClasses ();
                         * @brief	method to obtain the contest classes.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	a reference of the contest classes.
                         */

                        Contest_classes & getContestClasses ();

                        /**
                         * @fn	Contest_class & ElectionWrapper::getContestClass ( std::string codeContestClass ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a contest class.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	codeContestClass	The code contest class.
                         * @return	The contest class.
                         */

                        Contest_class & getContestClass ( std::string codeContestClass )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Contest_class & ElectionWrapper::getContestClassByCustomCode ( std::string customCoodeContestClass ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a contest class by custom code.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	customCoodeContestClass	The custom coode contest class.
                         * @return	The contest class by custom code.
                         */

                        Contest_class & getContestClassByCustomCode ( std::string customCoodeContestClass )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Contest_groups & ElectionWrapper::getContestGroups ();
                         * @brief	method to obtain the contest groups.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	a reference of the contest groups.
                         */

                        Contest_groups & getContestGroups ();

                        /**
                         * @fn	Contest_group & ElectionWrapper::getContestGroup ( std::string codeContestGroup )
                         *       throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets contest group.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	codeContestGroup	Group the code contest belongs to.
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * @return	The contest group.
                         */
                        Contest_group & getContestGroup ( std::string codeContestGroup )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::getContestGroupByList ( std::list < std::string > & codeContestGroups,
                         * 			std::list < Contest_group > & list) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a contest group by list.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	codeContestGroups	Groups the code contest belongs to.
                         * @param	list						 	result the contest group by list.
                         */

                        void getContestGroupByList ( std::list < std::string > & codeContestGroups, std::list < Contest_group * > & list )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Contest & ElectionWrapper::getContest ( std::string codeContest ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a contest.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	codeContest	The code contest.
                         * @return	The contest.
                         */

                        Contest & getContest ( std::string codeContest )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::getContestByList ( std::list < std::string > & codeContests,
                         * 			std::list < Contest * > & list ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a contest by list.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	codeContests	The code contests.
                         * @param	list					result the contest by list.
                         */

                        void getContestByList ( std::list < std::string > & codeContests, std::list < Contest * > & list )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::getContestByContestClass ( std::list < std::string > & codeContests,
                         * 			std::list < Contest * > & list ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a contest by contest class.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	codeContests	The code contests.
                         * @param	list					result the contest by contest class.
                         */

                        void getContestByContestClass ( std::list < std::string > & codeContests, std::list < Contest * > & list )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::getPartys ( Contest & contest, std::list < Party * > & list );
                         * @brief	Gets the partys.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @param	contest	The contest.
                         * @param	list	 	result the party list.
                         */

                        void getPartys ( Contest & contest, std::list < Party * > & list );

                        /**
                         * @fn	void ElectionWrapper::getPartysByContestCode( std::string contestCode,
                         * 			std::list < Party * > & list ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets the partys by contest code.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	contestCode	The contest.
                         * @param	list			 	result the party list.
                         */
                        void getPartysByContestCode ( const std::string& contestCode, std::list < Party * > & list )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::getBallotOptionPartyByContest ( Contest & contest,
                         * 			std::list < Ballot_option * > & list, Category category = Category::ELECTION_PRINCIPAL );
                         * @brief	Gets a ballot option party by contest.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @param	contest 	The contest.
                         * @param	list			result the ballot options party by contest.
                         * @param	category	The category.
                         */
                        void getBallotOptionPartyByContest ( Contest & contest,
                                                             std::list < Ballot_option * > & list,
                                                             Category category = Category::ELECTION_PRINCIPAL );

                        /**
                         * @fn	Party & ElectionWrapper::getPartyByContest ( Contest & contest, std::string partyCode,
                         * 			Category category = Category::ELECTION_PRINCIPAL ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a party by contest.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	contest	 	The contest.
                         * @param	partyCode	The party code.
                         * @param	category 	The category.
                         * @return	The party by contest.
                         */

                        Party & getPartyByContest ( Contest & contest, std::string partyCode, Category category = Category::ELECTION_PRINCIPAL )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Party & ElectionWrapper::getPartyAbbByContest ( Contest & contest, std::string partyAbb,
                         * 			Category category = Category::ELECTION_PRINCIPAL ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a party abb by contest.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	contest 	The contest.
                         * @param	partyAbb	The party abb.
                         * @param	category	The category.
                         * @return	The party abb by contest.
                         */

                        Party & getPartyAbbByContest ( Contest & contest, std::string partyAbb, Category category = Category::ELECTION_PRINCIPAL )
                        throw ( Smartmatic::System::Exception::ElectionException );


                        /**
                         * @fn	bool ElectionWrapper::getIfCanPartyAbbByContest ( Contest & contest, std::string partyAbb,
                         * 			Party & returnParty, Category category = Category::ELECTION_PRINCIPAL);
                         * @brief	Gets if can obtain a party abb by contest.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	contest							 	The contest.
                         * @param	partyAbb						 	The party abb.
                         * @param [in,out]	returnParty	The return party.
                         * @param	category						 	The category.
                         * @return	true if can get party abb by contest.
                         */

                        bool getIfCanPartyAbbByContest ( Contest & contest, std::string partyAbb, Party & returnParty, Category category = Category::ELECTION_PRINCIPAL );

                        /**
                         * @fn	void ElectionWrapper::getBallotOptionsByContest ( Contest & contest,
                         * 			std::list < Ballot_option * > & ballotOptionList,
                         * 			Category category = Category::ELECTION_PRINCIPAL, bool sort = true );
                         * @brief	Gets a ballot options by contest.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	contest											The contest.
                         * @param [in,out]	ballotOptionList	List of ballot options.
                         * @param	category										The category.
                         * @param	sort												(optional) the sort.
                         * ### return	The ballot options by contest.
                         */

                        void getBallotOptionsByContest ( Contest & contest, std::list < Ballot_option * > & ballotOptionList, Category category = Category::ELECTION_PRINCIPAL, bool sort = true );

                        /**
                         * @fn	void ElectionWrapper::getMapBallotOptionsByContest ( Contest & contest,
                         * 			std::map < std::string , Ballot_option * > & ballotOptionMap,
                         * 			Category category = Category::ELECTION_PRINCIPAL );
                         * @brief	Gets map ballot options by contest.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @param	contest				 	The contest.
                         * @param	ballotOptionMap	result map of ballot option.
                         * @param	category			 	The category.
                         * ### return	The ballot options by contest.
                         */
                        void getMapBallotOptionsByContest ( Contest & contest,
                                                            std::map < std::string , Ballot_option * > & ballotOptionMap,
                                                            Category category = Category::ELECTION_PRINCIPAL );

                        /**
                         * @fn	void ElectionWrapper::getBallotOptionsListByContest ( Contest & contest,
                         * 			std::list < Ballot_option * > & ballotOptionList, bool sort = true );
                         * @brief	Gets a ballot options list by contest.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @param	contest						The contest.
                         * @param	ballotOptionList	result ballot option list.
                         * @param	sort							flag sort ballot option list.
                         */

                        void getBallotOptionsListByContest ( Contest & contest, std::list < Ballot_option * > & ballotOptionList, bool sort = true );

                        /**
                         * @fn	void ElectionWrapper::getMapBallotOptionsListByContest ( Contest & contest,
                         * 			std::map < std::string , Ballot_option * > & ballotOptionMap );
                         * @brief	Gets map ballot options list by contest.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @param	contest				 	The contest.
                         * @param	ballotOptionMap	result map of ballot option list.
                         * ### param	category	 	The category.
                         * ### return	The ballot options by contest.
                         */
                        void getMapBallotOptionsListByContest ( Contest & contest,
                                                                std::map < std::string , Ballot_option * > & ballotOptionMap );

                        /**
                         * @fn	void ElectionWrapper::getBallotOptionsListByContest ( Contest & contest,
                         * 			std::list <std::string> & codes, std::list < Ballot_option * > & list );
                         * @brief	Gets a ballot options list by contest codes.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @param	contest	The contest.
                         * @param	codes	 	list codes.
                         * @param	list	 	result ballot option list.
                         */

                        void getBallotOptionsListByContest ( Contest & contest, std::list <std::string> & codes, std::list < Ballot_option * > & list );

                        /**
                         * @fn	void ElectionWrapper::getBallotOptionsByContest ( Contest & contest,
                         * 			std::list <std::string> & codes, std::list < Ballot_option * > & list,
                         * 			Category category = Category::ELECTION_PRINCIPAL );
                         * @brief	Gets a ballot options by contest.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @param	contest 	The contest.
                         * @param	codes			list codes.
                         * @param	list			result ballot option list.
                         * @param	category	The category.
                         */

                        void getBallotOptionsByContest ( Contest & contest, std::list <std::string> & codes, std::list < Ballot_option * > & list, Category category = Category::ELECTION_PRINCIPAL );

                        /**
                         * @fn	void ElectionWrapper::getContestByContestGroup ( std::string codeContestGroup,
                         * 			std::list < Contest * > & list ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a contest by contest group.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	codeContestGroup	Group the code contest belongs to.
                         * @param	list							result ballot option by contest group.
                         */

                        void getContestByContestGroup ( const std::string& codeContestGroup, std::list < Contest * > & list )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::getAllContest (std::list < Contest * > & contestList);
                         * @brief	Gets all contest.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @param	contestList	all contest.
                         */

                        void getAllContest ( std::list < Contest * > & contestList );

                        /**
                         * @fn	void ElectionWrapper::getAllContestCode (std::list < std::string > & contestList);
                         * @brief	Gets all contest code.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @param	contestList	all contest code.
                         */

                        void getAllContestCode ( std::list < std::string > & contestList );

                        /**
                         * @fn	int ElectionWrapper::getMaxPartysAllContests();
                         * @brief	Gets the maximum partys all contests.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	The maximum partys all contests.
                         */

                        int getMaxPartysAllContests();

                        /**
                         * @fn	int ElectionWrapper::getMaxBallotOptionsByContest(Contest & contest, Party & party);
                         * @brief	Gets a maximum ballot options by contest.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	contest				 	The contest.
                         * @param [in,out]	party	The party.
                         * @return	The maximum ballot options by contest.
                         */

                        int getMaxBallotOptionsByContest ( Contest & contest, Party & party );

                        /**
                         * @fn	void ElectionWrapper::getBallotOptionsByContestParty ( Contest & contest, Party & party,
                         * 			std::list < Ballot_option * > & list,
                         * 			Category category) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a ballot options by contest party.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	contest					The contest.
                         * @param	party						The party.
                         * @param [in,out]	list	The list.
                         * @param	category				The category.
                         * ### param	getList			true to get list.
                         * ### return	The ballot options by contest party.
                         */

                        void getBallotOptionsByContestParty ( Contest & contest,
                                                              Party & party,
                                                              std::list < Ballot_option * > & list,
                                                              Category category )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::getBallotOptionsListByContestParty ( Contest & contest, Party & party,
                         * 			std::list < Ballot_option * > & list) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets ballot options list by contest party.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error ondition occurs.
                         * @param [in,out]	contest	The contest.
                         * @param [in,out]	party	 	The party.
                         * @param [in,out]	list	 	The list.
                         */
                        void getBallotOptionsListByContestParty ( Contest & contest,
                                Party & party,
                                std::list < Ballot_option * > & list )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::getAllBallotOptionsListCodeByContestParty ( Contest & contest,
                         * 			Party & party,
                         * 			std::list < std::string > & list ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets all ballot options list code by contest party.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param [in,out]	contest	The contest.
                         * @param [in,out]	party	 	The party.
                         * @param [in,out]	list	 	The list.
                         */
                        void getAllBallotOptionsListCodeByContestParty ( Contest & contest, Party & party, std::list < std::string > & list )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::getBallotOptionsByContestPartyAbb ( Contest & contest,
                         * 			std::string partyAbb, std::list < Ballot_option > & list, Category category,
                         * 			bool getList = true) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a ballot options by contest party abb.
                         * @author	Yadickson Soto
                         * @date	22/02/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	contest 	The contest.
                         * @param	partyAbb	The party abb.
                         * @param	list			result of ballot options.
                         * @param	category	The category.
                         * @param	getList 	true to get list.
                         * ### return	The ballot options by contest party abb.
                         */

                        void getBallotOptionsByContestPartyAbb ( Contest & contest,
                                std::string partyAbb,
                                std::list < Ballot_option * > & list,
                                Category category,
                                bool getList = true )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::fillAllEligibleOptionsByContestParty ( Contest & contest,
                         * 			Party & party, std::list < Eligible_option * > & eligibleOptionList,
                         * 			std::list < Eligible_option * > & eligibleOptionPrincipal,
                         * 			std::list < Eligible_option * > & eligibleOptionStandIn) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Fill all eligible options by contest party.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	contest													 	The contest.
                         * @param	party														 	The party.
                         * @param [in,out]	eligibleOptionList		 	List of eligible option principals.
                         * @param [in,out]	eligibleOptionPrincipal	List of eligible option stand ins.
                         * @param [in,out]	eligibleOptionStandIn	 	true to get principal list.
                         * ### param	getStandInList							 	true to get stand in list.
                         */

                        void fillAllEligibleOptionsByContestParty ( Contest & contest,
                                Party & party,
                                std::list < Eligible_option * > & eligibleOptionList,
                                std::list < Eligible_option * > & eligibleOptionPrincipal,
                                std::list < Eligible_option * > & eligibleOptionStandIn )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::getCountBallotOptionsByContestParty ( Contest & contest,
                         * 			Party & party, int & countList, int & countPrincipal,
                         * 			int & countStandIn) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets count ballot options by contest party.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param [in,out]	contest					The contest.
                         * @param [in,out]	party						The party.
                         * @param [in,out]	countList				List of counts.
                         * @param [in,out]	countPrincipal	The count principal.
                         * @param [in,out]	countStandIn		The count stand in.
                         */
                        void getCountBallotOptionsByContestParty ( Contest & contest,
                                Party & party,
                                int & countList,
                                int & countPrincipal,
                                int & countStandIn )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::fillAllEligibleOptionsByContest ( Contest & contest,
                         * 			std::list < Eligible_option * > & eligibleOptionList) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Fill all eligible options by contest.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	13/06/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	contest							The contest.
                         * @param	eligibleOptionList	List of eligible option ins.
                         */

                        void fillAllEligibleOptionsByContest ( Contest & contest,
                                                               std::list < Eligible_option * > & eligibleOptionList )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void ElectionWrapper::getBallotOptionsByContestParty ( Contest & contest, Party & party,
                         * 			std::list< std::string > & codes,
                         * 			std::list < Ballot_option * > & list) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets a ballot options by contest party.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	contest					The contest.
                         * @param	party						The party.
                         * @param	codes						The codes.
                         * @param [in,out]	list	The list.
                         * ### return	The ballot options by contest party.
                         */

                        void getBallotOptionsByContestParty ( Contest & contest,
                                                              Party & party,
                                                              std::list< std::string > & codes,
                                                              std::list < Ballot_option * > & list )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	Eligible_option & ElectionWrapper::getEligibleOptionByBallotOption ( Contest & contest,
                         * 			Ballot_option & ballotOption ) throw (Smartmatic::System::Exception::ElectionException);
                         * @brief	Gets an eligible option by ballot option.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error condition occurs.
                         * @param	contest				The contest.
                         * @param	ballotOption	The ballot option.
                         * @return	The eligible option by ballot option.
                         */

                        Eligible_option & getEligibleOptionByBallotOption ( Contest & contest, Ballot_option & ballotOption )
                        throw ( Smartmatic::System::Exception::ElectionException );


                        /**
                         * @fn	std::string ElectionWrapper::getEligibleOptionCodeByBallotOptionCode ( const std::string & contestCode ,
                         * 			const std::string & parentBallotCode );
                         * @brief	Gets eligible option code by ballot option code.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	contestCode				The contest code.
                         * @param	parentBallotCode	The parent ballot code.
                         * @return	The eligible option code by ballot option code.
                         */
                        std::string getEligibleOptionCodeByBallotOptionCode (
                            const std::string & contestCode
                            , const std::string & parentBallotCode );


                        /**
                         * @fn	bool ElectionWrapper::getIfCanEligibleOptionByBallotOption ( Contest & contest,
                         * 			Ballot_option & ballotOption, Eligible_option & returnEligibleOption );
                         * @brief	Gets if can obtain an eligible option by ballot option.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	contest													The contest.
                         * @param	ballotOption										The ballot option.
                         * @param [in,out]	returnEligibleOption	The return eligible option.
                         * @return	true if can get an eligible option.
                         */

                        bool getIfCanEligibleOptionByBallotOption ( Contest & contest, Ballot_option & ballotOption, Eligible_option & returnEligibleOption );

                        /**
                         * @fn	void ElectionWrapper::getAllContestCodeByContestGroup ( std::list < std::string > & contestGroupCodes,
                         * 			std::list < std::string > & list);
                         * @brief	Gets all contest code by contest group.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	contestGroupCodes	The contest group codes.
                         * @param [in,out]	list	 	The list.
                         * ### return	all contest code by contest group.
                         */

                        void getAllContestCodeByContestGroup ( std::list < std::string > & contestGroupCodes,
                                                               std::list < std::string > & list );

                        /**
                         * @fn	void ElectionWrapper::sortBallotOptionsListByTallyPosition ( std::list < Ballot_option * > & ballotOptionList );
                         * @brief	Sort ballot options list by tally position.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	ballotOptionList	List of ballot options.
                         */

                        void sortBallotOptionsListByTallyPosition ( std::list < Ballot_option * > & ballotOptionList );

                        /**
                         * @fn	void ElectionWrapper::sortEligibleOptionsListByTallyPosition ( std::list < Eligible_option * > & eligibleOptionList );
                         * @brief	Sort eligible options list by tally position.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	eligibleOptionList	List of eligible options.
                         */

                        void sortEligibleOptionsListByTallyPosition ( std::list < Eligible_option * > & eligibleOptionList );

                        /**
                         * @fn	void ElectionWrapper::sortBallotOptionsVectorByTallyPosition ( std::vector < Ballot_option * > & ballotOptionVector );
                         * @brief	Sort ballot options vector by tally position.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	ballotOptionVector	The ballot option vector.
                         */

                        void sortBallotOptionsVectorByTallyPosition ( std::vector < Ballot_option * > & ballotOptionVector );

                        /**
                         * @fn	void ElectionWrapper::sortBallotOptionsPartyVectorByTallyPosition ( std::vector < Ballot_option * > & ballotOptionVector );
                         * @brief	Sort ballot options party vector by tally position.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	ballotOptionVector	The ballot option vector.
                         */

                        void sortBallotOptionsPartyVectorByTallyPosition ( std::vector < Ballot_option * > & ballotOptionVector );

                        /**
                         * @fn	void ElectionWrapper::getAllContestVector (std::vector < Contest * > & vector);
                         * @brief	Gets all contest vector.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param [in,out]	vector	The vector.
                         * ### return	all contest vector.
                         */

                        void getAllContestVector ( std::vector < Contest * > & vector );

                        /**
                         * @fn	void ElectionWrapper::getAllPartyVector (std::vector < Ballot_option * > & vector,
                         * 			Category category = Category::ELECTION_PRINCIPAL);
                         * @brief	Gets all party vector.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param [in,out]	vector	The vector.
                         * @param	category					The category.
                         * ### return	all party vector.
                         */

                        void getAllPartyVector ( std::vector < Ballot_option * > & vector, Category category = Category::ELECTION_PRINCIPAL );

                        /**
                         * @fn	void ElectionWrapper::getContestsByContestClassCode(std::string code,
                         * 			std::list <std::string> & list);
                         * @brief	Gets contests by contest class code.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	code						The code.
                         * @param [in,out]	list	The list.
                         */
                        void getContestsByContestClassCode ( const std::string& code, std::list <std::string> & list );

                        /**
                         * @fn	void ElectionWrapper::getContestCodeOrderByContestClass(std::list <std::string> & list);
                         * @brief	Gets contest code order by contest class.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param [in,out]	list	The list.
                         */
                        void getContestCodeOrderByContestClass ( std::list <std::string> & list );

                        /**
                         * @fn	void ElectionWrapper::getContestOrderByContestClassByContestGroup ( std::string codeContestGroup,
                         * 			std::list < Contest * > & list );
                         * @brief	Gets contest order by contest class by contest group.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	codeContestGroup	Group the code contest belongs to.
                         * @param [in,out]	list		The list.
                         */
                        void getContestOrderByContestClassByContestGroup ( const std::string& codeContestGroup, std::list < Contest * > & list );

                        /**
                         * @fn	void ElectionWrapper::getBallotOptionsByParentList (const std::string & contestCode,
                         * 			const std::string & parentBallotCode, std::list<Ballot_option * > & children);
                         * @brief	Gets ballot options by parent list.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	contestCode					The contest code.
                         * @param	parentBallotCode		The parent ballot code.
                         * @param [in,out]	children	The children.
                         */
                        void getBallotOptionsByParentList ( const std::string & contestCode, const std::string & parentBallotCode, std::list<Ballot_option * > & children );

                        /**
                         * @fn	void ElectionWrapper::FillIndexes ();
                         * @brief	Fill indexes.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void FillIndexes ();

                        /**
                         * @fn	void ElectionWrapper::persistence(Smartmatic::SAES::Runtime::Environment::SaesDirectories & directories);
                         * @brief	Persistences the given directories.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param [in,out]	directories	The directories.
                         */
                        void persistence ( Smartmatic::SAES::Runtime::Environment::SaesDirectories & directories );

                    private:
                        Election election;///< The election
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger

                };
            }
        }
    }
}

#endif /*__ELECTION_WRAPPER_H__*/
