/**
 * @file HashValidator.cxx
 * @brief Body class for election files hashes validation.
 * @author  Eduardo Fernández
 * @date    07/01/2019
 */

#include "HashValidator.hxx"

#include <fstream>

#include <archive.h>
#include <archive_entry.h>

#include <Configuration/SaesCommonConfiguration-schema.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Runtime/Environment/MemoryPathController.h>
#include <System/IO/Directory.h>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/SafeBuffer.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::System;
using namespace Smartmatic::System::IO;

static const std::string DOM_TEMPORARY_DIRECTORY   = "/tmp/dom";
static const std::string FLASH_TEMPORARY_DIRECTORY = "/tmp/flash";
static const std::string HEXADECIMAL_PREFIX        = "0x";
static const int         CHUNK_SIZE                = 1048576;

ISMTTLog *HashValidator::m_logger =
    SMTTLogManager::GetLogger("Smartmatic.SAES.Voting.Election.HashValidator", GETTEXT_PACKAGE);

/**
 * @brief Creates the temporary directories where compressed files are extracted.
 * @author eduardo.fernandez
 * @date   09/01/2019
 * @param  logger The logger.
 * @return Whether the creation of temporary directories are successful.
 */
static bool
createTemporaryDirectories(ISMTTLog *logger)
{
    logger->Debug("Creating temporary directory '" + DOM_TEMPORARY_DIRECTORY + "'.");
    bool result = Directory::getInstance()->Mkdir(DOM_TEMPORARY_DIRECTORY);

    if (result)
    {
        logger->Debug("Creating temporary directory '" + FLASH_TEMPORARY_DIRECTORY + "'.");
        result = Directory::getInstance()->Mkdir(FLASH_TEMPORARY_DIRECTORY);
    }

    if (result == false)
    {
        logger->Error("An error occurred while trying to create temporary directories.");
    }

    return result;
}

/**
 * @brief Deletes the temporary directories where compressed files are extracted.
 * @author eduardo.fernandez
 * @date   09/01/2019
 * @param  logger The logger.
 * @return Whether the deletion of temporary directories are successful.
 */
static void
deleteTemporaryDirectories(ISMTTLog *logger)
{
    logger->Debug("Deleting temporary directory '" + DOM_TEMPORARY_DIRECTORY + "'.");
    bool result = Directory::getInstance()->Delete(DOM_TEMPORARY_DIRECTORY);

    logger->Debug("Deleting temporary directory '" + FLASH_TEMPORARY_DIRECTORY + "'.");
    result &= Directory::getInstance()->Delete(FLASH_TEMPORARY_DIRECTORY);

    if (result == false)
    {
        logger->Error("An error occurred while trying to delete temporary directories.");
    }
}

/**
 * @brief Converts given hexadecimal string to its decimal representation.
 * @author eduardo.fernandez
 * @date   09/01/2019
 * @param  hexadecimalString String to convert.
 * @return Decimal representation of the given string.
 */
static int
convertHexadecimalToDecimal(const std::string& hexadecimalString)
{
    unsigned int decimal;
    std::stringstream ss;
    ss << std::hex << hexadecimalString;
    ss >> decimal;

    return decimal;
}

/**
 * @brief Formats the hexadecimal digest contained in the election file, so appropriate base64 hash could be obtained.
 * @author eduardo.fernandez
 * @date   09/01/2019
 * @param  hexadecimalDigest Raw hexadecimal digest.
 * @return Formatted hexadecimal digest.
 */
static std::string
formatHexadecimalDigest(const std::string& hexadecimalDigest)
{
    std::string formattedDigest;

    for (size_t i = 0; i < hexadecimalDigest.size(); i += 2)
    {
        const std::string hexadecimalString = HEXADECIMAL_PREFIX + hexadecimalDigest[i] + hexadecimalDigest[i + 1];

        const int decimal = convertHexadecimalToDecimal(hexadecimalString);

        formattedDigest.push_back(decimal);
    }

    return formattedDigest;
}

/**
 * @brief Sets up the libarchive library in order to make the file extraction.
 * @author eduardo.fernandez
 * @date   09/01/2019
 * @param  a The handler of the file to be extracted.
 * @param  baseFile Compressed file.
 * @return Whether the library setup was successful.
 */
static bool
openBaseFileForReading(struct archive *a, const std::string& baseFile)
{
    bool result = false;

    archive_read_support_filter_all(a);
    archive_read_support_format_all(a);

    if (archive_read_open_filename(a, baseFile.c_str(), 10240) == ARCHIVE_OK)
    {
        result = true;
    }

    return result;
}

/**
 * @brief Extracts given file to the specified destination directory.
 * @author eduardo.fernandez
 * @date   09/01/2019
 * @param  baseFile Compressed file.
 * @param  fileToExtract Relative path of the file to be extracted.
 * @param  destinationDirectory Extracted file destination directory.
 * @return Whether the file extraction was successful.
 */
static bool
extractFile(const std::string& baseFile, const std::string& fileToExtract, const std::string& destinationDirectory)
{
    struct archive       *a = archive_read_new();
    struct archive_entry *entry;

    bool result = openBaseFileForReading(a, baseFile);

    if (result)
    {
        int flags = ARCHIVE_EXTRACT_TIME | ARCHIVE_EXTRACT_PERM | ARCHIVE_EXTRACT_ACL | ARCHIVE_EXTRACT_FFLAGS;

        while (archive_read_next_header(a, &entry) == ARCHIVE_OK)
        {
            const char* currentFile = archive_entry_pathname(entry);

            if (currentFile == fileToExtract)
            {
              const std::string fullPath = Path::Combine(destinationDirectory, currentFile);
              archive_entry_set_pathname(entry, fullPath.c_str());
              result = (archive_read_extract(a, entry, flags) == ARCHIVE_OK) ? true : false;
              break;
            }

            archive_read_data_skip(a);
        }

        if (archive_read_free(a) == ARCHIVE_OK)
        {
            result &= true;
        }
    }

    return result;
}

static std::ifstream::pos_type getFileSize(const char* file)
{
    std::ifstream in(file, std::ifstream::ate | std::ifstream::binary);
    return in.tellg();
}

/**
 * @brief Checks given DOM and flash files hashes.
 * @author eduardo.fernandez
 * @date   08/01/2019
 * @param  base64Hash Expected files hash.
 * @param  domFile Path to the DOM file.
 * @param  flashFile Path to the flash file.
 * @param  runningOnPM Whether the verification is being run on a PM.
 * @return Whether the check is successful.
 */
static bool
checkHashes(
    const std::string& base64Hash,
    const std::string& domFile,
    const std::string& flashFile,
    const bool runningOnPM)
{
    SafeBuffer fileHashBuffer;

    int fileSize = getFileSize(domFile.c_str());

    if (fileSize > CHUNK_SIZE)
    {
        File::getInstance()->getHashFileByChunks(domFile, CHUNK_SIZE, fileHashBuffer);
    }
    else
    {
        File::getInstance()->getHashFile(domFile, fileHashBuffer);
    }

    bool result = base64Hash == fileHashBuffer.GetBase64Data();

    if (result)
    {
        if (runningOnPM)
        {
            fileSize = getFileSize(flashFile.c_str());

            if (fileSize > CHUNK_SIZE)
            {
                File::getInstance()->getHashFileByChunks(flashFile, CHUNK_SIZE, fileHashBuffer);
            }
            else
            {
                File::getInstance()->getHashFile(flashFile, fileHashBuffer);
            }

            if (base64Hash != fileHashBuffer.GetBase64Data())
            {
                result = false;
            }
        }
    }

    return result;
}

/**
 * @brief Verifies given file's hash.
 * @author eduardo.fernandez
 * @date   09/01/2019
 * @param  logger The logger.
 * @param  file Relative path of the file to be verified.
 * @param  base64Hash File's hash in base 64 format.
 * @param  domFile File path in the DOM directory.
 * @param  flashFile File path in the flash directory.
 * @param  runningOnPM Whether the verification is being run on a PM.
 * @param  errorMessage Error message to be set in case of error.
 * @return Whether the verification is successful.
 */
static bool
verifyFile(
    ISMTTLog *logger,
    const std::string& file,
    const std::string& base64Hash,
    const std::string& domFile,
    const std::string& flashFile,
    const bool runningOnPM,
    std::string& errorMessage)
{
    bool result = true;

    if (checkHashes(base64Hash, domFile, flashFile, runningOnPM))
    {
        logger->Debug("Verification was sucessful.");
    }
    else
    {
        errorMessage = "Verification of the file '" + file + "' failed.";
        logger->Error("Verification failed.");
        result = false;
    }

    return result;
}

/**
 * @brief Sets the file's absolute paths after extraction.
 * @author eduardo.fernandez
 * @date   09/01/2019
 * @param  it Current file iterator.
 * @param  domDirectory DOM directory root path.
 * @param  flashDirectory Flash directory root path.
 * @param  file Relative path of the file to be verified.
 * @param  runningOnPM Whether the verification is being run on a PM.
 * @param  domFile File path in the DOM directory.
 * @param  flashFile File path in the flash directory.
 * @param  errorMessage Error message to be set in case of error.
 * @return Whether the setup is successful.
 */
static bool
setExtractedFileAbsolutePaths(
    Checksums::ChecksumSequence::const_iterator it,
    const std::string& domDirectory,
    const std::string& flashDirectory,
    const std::string& file,
    const bool runningOnPM,
    std::string& domFile,
    std::string& flashFile,
    std::string& errorMessage)
{
    const std::string base = it->getBase().get();

    const std::string domBaseFile   = Path::Combine(domDirectory, base);
    const std::string flashBaseFile = Path::Combine(flashDirectory, base);

    bool result = extractFile(domBaseFile, file, DOM_TEMPORARY_DIRECTORY);

    if (runningOnPM)
    {
        result = result && extractFile(flashBaseFile, file, FLASH_TEMPORARY_DIRECTORY);
    }

    if (result)
    {
        domFile   = Path::Combine(DOM_TEMPORARY_DIRECTORY, file);
        flashFile = Path::Combine(FLASH_TEMPORARY_DIRECTORY, file);
    }
    else
    {
        errorMessage = "An error occurred when trying to extract the file '" + file + "'.";
    }

    return result;
}

HashValidator::HashValidator(SaesDirectories * const saesDirectories, const bool runningOnPM)
: m_saesDirectories(saesDirectories)
, m_runningOnPM(runningOnPM)
{
}

HashValidator::~HashValidator()
{
}

bool
HashValidator::validateChecksums(const Election::ChecksumsOptional& checksums, std::string& errorMessage)
{
    const std::string domDirectory   =
        m_runningOnPM ?
        m_saesDirectories->getDevicePath(BasePathType::CONFIGURATION_DOM) :
        MemoryPathController::getInstance()->getBackupMemoryBasePath();
    const std::string flashDirectory = m_saesDirectories->getDevicePath(BasePathType::CONFIGURATION_FLASH);

    Checksums::ChecksumSequence::const_iterator       it  = checksums.get().getChecksum().begin();
    const Checksums::ChecksumSequence::const_iterator end = checksums.get().getChecksum().end();

    bool result = createTemporaryDirectories(m_logger);

    for (; it != end && result; ++it)
    {
        const std::string file = it->getFile();
        const std::string hash = formatHexadecimalDigest(it->getHash());

        const SafeBuffer  hashBuffer(hash);
        const std::string base64Hash = hashBuffer.GetBase64Data();

        std::string domFile;
        std::string flashFile;

        if (it->getBase().present() == false)
        {
            domFile   = Path::Combine(domDirectory, file);
            flashFile = Path::Combine(flashDirectory, file);
        }
        else
        {
            result = setExtractedFileAbsolutePaths(it, domDirectory, flashDirectory, file, m_runningOnPM, domFile, flashFile, errorMessage);
        }

        if (result)
        {
            m_logger->Debug("Checking file '" + file + "''s hash (configured hash: '" + it->getHash() + "').");
            result = verifyFile(m_logger, file, base64Hash, domFile, flashFile, m_runningOnPM, errorMessage);
        }
    }

    deleteTemporaryDirectories(m_logger);

    return result;
}
