/**
 * @file	ElectoralConfiguration.cxx
 * @brief	electoral configuration class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include "ElectoralConfiguration.hxx"

#include <Configuration/MachineModelEnum.hxx>
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>
#include <Log/SMTTLogManager.h>
#include <Environment/Configurations/base-language-controller.h>
#include <Exceptions/Voting/ElectoralConfiguration/CandidateNotFoundException.hxx>
#include <Exceptions/Voting/ElectoralConfiguration/PartyNotFoundException.hxx>
#include <Log/ISMTTLog.h>
#include <Resources/election-string-resources-manager.h>
#include <Resources/ElectionAudioResourcesManager.hxx>
#include <Resources/GeneralAudioResourcesManager.hxx>
#include <Security/Encryption.hxx>
#include <System/Compression/compression-class.h>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <Voting/DeviceParametersManager.hxx>
#include <System/Exception/HashException.hxx>
#include <System/Exception/VoteRecoveryException.hxx>
#include <Voting/Election/ElectionInstalationLanguages.h>
#include <Voting/Election/HashValidator.hxx>
#include <Voting/Eml/EmlFileInfo.hxx>
#include <Voting/PBReports/PVContestWrapper.hxx>
#include <Voting/set-language-functionality.h>
#include <Voting/Status/VMProcessStatusWrapper.hxx>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Exceptions::Voting;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Voting::Eml;
using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Compression;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

ISMTTLog * ElectoralConfiguration::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.ElectoralConfiguration", GETTEXT_PACKAGE );

ElectoralConfiguration::ElectoralConfiguration ()
{
    this->directories = NULL;
    this->election = NULL;
    this->voteRecovery = NULL;
    this->voteMaxCalculate = NULL;
    this->votingDevice = NULL;
    this->maxBallotOptionByVote = 0;
    this->maxContestByBlankVote = 0;
    this->maxContestByVote = 0;
    this->maxCustomAttributeByVote = 0;
    this->maxPartiesByAllContest = 0;
    this->reload = true;
    this->loadData = false;
}

ElectoralConfiguration::~ElectoralConfiguration ()
{
    dispose();
}

void ElectoralConfiguration::init ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories, BasePathType type )
{
    if ( !reload )
    {
        return;
    }

    this->directories = directories;
    dispose();
    logger->Debug ( "Init class" );

    std::string strVotingDevice;
    std::string fileVotingDevice = directories->getVotingDeviceFileFullPath ( type );
    Smartmatic::System::SafeBuffer bufferVotingDevice;
    logger->Debug ( "Open xml file " + fileVotingDevice );

    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.OpenVotingDevice" ) );

    Smartmatic::SAES::Security::Encryption::getInstance()->decryptoFileToBuffer ( fileVotingDevice,
            bufferVotingDevice,
            Smartmatic::SAES::Security::Encryption::EXTERNAL_FILE );

    if ( SAESCommonConfigurationManager::getInstance()->isCompressVotingDevices() )
    {
        std::stringstream ssc;
        ssc << "Decompress xml data of file " << fileVotingDevice;
        logger->Debug ( ssc.str() );
        Smartmatic::System::SafeBuffer inflate;
        CompressionClass::getInstance()->Inflate ( bufferVotingDevice, inflate );

        if ( inflate.GetSize() > 0 )
        {
            strVotingDevice.assign ( ( char * ) inflate.GetData(), inflate.GetSize() );
            strVotingDevice.resize ( inflate.GetSize() );
        }
        else
        {
            logger->Debug ( "File " + fileVotingDevice + " not compress." );
        }
    }
    else if ( bufferVotingDevice.GetSize() > 0 )
    {
        strVotingDevice.assign ( ( char * ) bufferVotingDevice.GetData(), bufferVotingDevice.GetSize() );
        strVotingDevice.resize ( bufferVotingDevice.GetSize() );
    }
    else
    {
        logger->Debug ( "Error open xml file " + fileVotingDevice );
    }

    std::istringstream istrVotingDevice ( strVotingDevice );
    this->votingDevice = ElectoralFactory::getVotingDevice ( istrVotingDevice );
    std::string protectionKey = votingDevice->getFirstVotingDevice().getKeys().getProtection_key();
    std::string cbcMacKey = votingDevice->getFirstVotingDevice().getKeys().getTransmission_key();

    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.OpenElection" ) );

    std::string fileElectoral = directories->getElectionFileFullPath ( type );
    std::string fileElectoralRed = directories->getElectionReduceFileFullPath ( type );

    std::string electoralData;

    if ( File::getInstance()->canOpenFile ( fileElectoralRed ) )
    {
        logger->Debug ( "Load Electoral File " + fileElectoralRed );

        electoralData = Smartmatic::SAES::Security::Encryption::getInstance()->openXmlFile ( fileElectoralRed,
                        Smartmatic::SAES::Security::Encryption::EXTERNAL_FILE,
                        Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_UP_KEY,
                        false, true, false );
    }
    else
    {
        logger->Debug ( "Load Electoral File " + fileElectoral );

        electoralData = Smartmatic::SAES::Security::Encryption::getInstance()->openXmlFile ( fileElectoral,
                        Smartmatic::SAES::Security::Encryption::EXTERNAL_FILE,
                        Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_UP_KEY,
                        SAESCommonConfigurationManager::getInstance()->isCompressElection(),
                        true, false );
    }

    std::istringstream istrElectoral ( electoralData );
    this->election = ElectoralFactory::getElection ( istrElectoral );

    setMachineTime();

    Encryption::getInstance()->initSigned ( directories, protectionKey, cbcMacKey );

    const std::string configuredRootFingerprint = election->getElection().getRoot_fp();
    Encryption::getInstance()->validateIssuerCertificates(configuredRootFingerprint);

    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.ValidateVotingDevice" ) );

    Smartmatic::SAES::Security::Encryption::getInstance()->validateSignedString ( strVotingDevice,
            Smartmatic::SAES::Security::Encryption::EXTERNAL_FILE );

    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.LoadStringResources" ) );

    this->voteRecovery = new Smartmatic::SAES::Voting::Vote::VoteRecovery ( this );
    this->voteMaxCalculate = new Smartmatic::SAES::Voting::Vote::VoteMaxCalculate ( this );

    ElectionStringResourcesManager::getInstance()->init ( directories, type );
    ElectionAudioResourcesManager::getInstance()->init ( directories, type );
    GeneralAudioResourcesManager::getInstance()->init ( directories, type );
    DeviceParametersManager::getInstance()->init ( directories, type );
}

void ElectoralConfiguration::dispose ()
{
    logger->Debug ( "Dispose class" );

    mapBallotOptionsListByContestParty.clear();
    mapBallotOptionsPartyByContest.clear();
    mapBallotOptionsPrincipalByContestParty.clear();
    mapBallotOptionsStandInByContestParty.clear();
    mapContestCode.clear();
    mapContestClass.clear();
    mapContestGroupByCode.clear();
    mapContestParty.clear();
    mapEligibleOptionsList.clear();
    mapEligibleOptionsListByContest.clear();
    mapEligibleOptionsListByContestParty.clear();
    mapEligibleOptionsListByParty.clear();
    mapEligibleOptionsPrincipal.clear();
    mapEligibleOptionsPrincipalByContest.clear();
    mapEligibleOptionsPrincipalByContestParty.clear();
    mapEligibleOptionsPrincipalByParty.clear();
    mapEligibleOptionsStandIn.clear();
    mapEligibleOptionsStandInByContest.clear();
    mapEligibleOptionsStandInByContestParty.clear();
    mapEligibleOptionsStandInByParty.clear();
    mapListContestByContestGroup.clear();
    allCodeBallot.clear();
    partyCodeByBallotOptionCode.clear();
    ballotList.clear();
    contestGroupList.clear();
    contestParty.clear();
    eligibleOptionByBallotOptionCodeMap.clear();
    eligibleOptionByEligibleOptionCodeMap.clear();
    listContest.clear();
    listContestCode.clear();
    listLVG.clear();
    maxBallotOptionByContest.clear();
    maxListBallotOptionByContestParty.clear();
    maxPrincipalBallotOptionByContestParty.clear();
    maxStandInBallotOptionByContestParty.clear();
    mapPartyByCode.clear();
    regionCategoryList.clear();
    mapPartyByCustomCode.clear();
    mapEligibleOptionByCustomCode.clear();

    reload = true;
    loadData = false;

    if ( election != NULL )
    {
        delete election;
        election = NULL;
    }

    if ( votingDevice != NULL )
    {
        delete votingDevice;
        votingDevice = NULL;
    }

    if ( voteRecovery != NULL )
    {
        delete voteRecovery;
        voteRecovery = NULL;
    }

    if ( voteMaxCalculate != NULL )
    {
        delete voteMaxCalculate;
        voteMaxCalculate = NULL;
    }
}

Smartmatic::SAES::Voting::Election::ElectionWrapper * ElectoralConfiguration::getElection ()
{
    if ( election == NULL )
    {
        throw ElectionException ( _ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.ObjectElectionNull" ),
                                  N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.ObjectElectionNull" ), logger );
    }

    return election;
}

VotingDevice::VotingDeviceWrapper * ElectoralConfiguration::getVotingDevice ()
{
    if ( votingDevice == NULL )
    {
        throw VotingDeviceException ( _ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.ObjectVotingDeviceNull" ),
                                      N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.ObjectVotingDeviceNull" ), logger );
    }

    return votingDevice;
}

std::list <std::string> ElectoralConfiguration::getLanguageListFullName ( bool onlyDefaultTrue, bool onlyLanguage )
{
    std::list <std::string> list;
    std::list <Smartmatic::SAES::Voting::Election::Language> languages = getLanguageList ( onlyDefaultTrue );
    std::list<Smartmatic::SAES::Voting::Election::Language>::iterator it;

    for ( it = languages.begin();
            it != languages.end();
            ++it )
    {
        std::string lang = ( *it ).getLanguage();
        std::string country = ( *it ).getCountry();

        if ( country.empty() )
        {
            country = lang;
        }

        if ( !onlyLanguage )
        {
            lang = StringUtils::toLower ( lang );
        }
        else
        {
            lang = StringUtils::toUpper ( lang );
        }

        country = StringUtils::toUpper ( country );

        if ( country.empty() )
        {
            list.push_back ( lang );
        }
        else if ( !onlyLanguage )
        {
            list.push_back ( lang + "_" + country );
        }
        else
        {
            list.push_back ( "_" + lang );
        }
    }

    return list;
}

std::list <Smartmatic::SAES::Voting::Election::Language> ElectoralConfiguration::getLanguageList ( bool onlyDefaultTrue )
{
    std::list <Smartmatic::SAES::Voting::Election::Language> languages;
    Smartmatic::SAES::Voting::Election::Languages::LanguageSequence & sequence ( getLanguages().getLanguage() );
    Smartmatic::SAES::Voting::Election::Languages::LanguageIterator it;

    for ( it = sequence.begin();
            it != sequence.end();
            ++it )
    {
        if ( ( *it ).getIs_default() || !onlyDefaultTrue )
        {
            languages.push_back ( *it );
        }
    }

    return languages;
}

Smartmatic::SAES::Voting::Election::Languages & ElectoralConfiguration::getLanguages ()
{
    if ( votingDevice->IsLanguagesPresent () )
    {
        return votingDevice->getLanguages ();
    }

    return election->getLanguages ();
}

Smartmatic::SAES::Voting::Vote::VoteRecovery * ElectoralConfiguration::getVoteRecovery()
{
    if ( voteRecovery == NULL )
    {
        throw VoteRecoveryException ( _ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.ObjectVoteRecoveryNull" ),
                                      N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.ObjectVoteRecoveryNull" ), logger );
    }

    return voteRecovery;
}

Smartmatic::SAES::Voting::Vote::VoteMaxCalculate * ElectoralConfiguration::getVoteMaxCalculate()
{
    return voteMaxCalculate;
}

void ElectoralConfiguration::createMap()
{
    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.GetContest" ) );

    logger->Debug ( "Start create map" );
    Smartmatic::SAES::Voting::Election::Ballots & ballots ( getElection()->getBallots() );
    Smartmatic::SAES::Voting::Election::Ballots::BallotIterator ballotIterator;

    for ( ballotIterator = ballots.getBallot().begin(); ballotIterator != ballots.getBallot().end(); ballotIterator++ )
    {
        Smartmatic::SAES::Voting::Election::Ballot_cards::Ballot_cardIterator ballotCardIterator;

        for ( ballotCardIterator = ballotIterator->getBallot_cards().getBallot_card().begin();
                ballotCardIterator != ballotIterator->getBallot_cards().getBallot_card().end();
                ++ballotCardIterator )
        {
            Smartmatic::SAES::Voting::Election::Contests::ContestIterator contestIterator;

            for ( contestIterator = ballotCardIterator->getContests().getContest().begin();
                    contestIterator != ballotCardIterator->getContests().getContest().end();
                    ++contestIterator )
            {
                Smartmatic::SAES::Voting::Election::Contest & contest ( *contestIterator );

                if ( !isContestValidate ( contest ) )
                {
                    continue;
                }

                listContest.push_back ( &contest );
                listContestCode.push_back ( contest.getCode() );
                ballotList.push_back ( & ( *ballotIterator ) );

                Smartmatic::SAES::Voting::Election::Eligibles_options::Eligible_optionIterator eligibleOptionIterator;

                for ( eligibleOptionIterator = contestIterator->getEligibles_options().getEligible_option().begin();
                        eligibleOptionIterator != contestIterator->getEligibles_options().getEligible_option().end();
                        ++eligibleOptionIterator )
                {
                    Smartmatic::SAES::Voting::Election::Eligible_option * eligibleOption = & ( *eligibleOptionIterator );

                    eligibleOptionByEligibleOptionCodeMap[contest.getCode()][eligibleOption->getCode()] = eligibleOption;

                    Smartmatic::SAES::Voting::Election::Ballot_options::Ballot_optionIterator ballotOptionIterator;

                    for ( ballotOptionIterator = eligibleOption->getBallot_options().getBallot_option().begin();
                            ballotOptionIterator != eligibleOption->getBallot_options().getBallot_option().end();
                            ++ballotOptionIterator )
                    {
                        eligibleOptionByBallotOptionCodeMap[contest.getCode()][ballotOptionIterator->getCode()] = eligibleOption;
                    }
                }
            }
        }
    }

    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.GetEligibleOptions" ) );

    reduceElectionContests();
    std::list < Smartmatic::SAES::Voting::Election::Contest * >::iterator itcontest;

    for ( itcontest = listContest.begin();
            itcontest != listContest.end();
            ++itcontest )
    {
        std::string contestCode = ( *itcontest )->getCode();
        std::string contestClassCode = ( *itcontest )->getContest_class_code();

        mapContestCode[contestCode] = ( *itcontest );
        mapContestClass [ contestClassCode ] = & ( getElection()->getContestClass ( contestClassCode ) );

        getElection()->getBallotOptionPartyByContest ( * ( *itcontest ), mapBallotOptionsPartyByContest[contestCode], Smartmatic::SAES::Voting::Election::Category::ELECTION_PRINCIPAL );
        getElection()->getBallotOptionsListByContest ( * ( *itcontest ), mapEligibleOptionsListByContest[contestCode], true );
        getElection()->getBallotOptionsByContest ( * ( *itcontest ), mapEligibleOptionsPrincipalByContest[contestCode], Smartmatic::SAES::Voting::Election::Category::ELECTION_PRINCIPAL, true );
        getElection()->getBallotOptionsByContest ( * ( *itcontest ), mapEligibleOptionsStandInByContest[contestCode], Smartmatic::SAES::Voting::Election::Category::ELECTION_STAND_IN, true );
        getElection()->getMapBallotOptionsListByContest ( * ( *itcontest ), mapEligibleOptionsList[contestCode] );
        getElection()->getMapBallotOptionsByContest ( * ( *itcontest ), mapEligibleOptionsPrincipal[contestCode], Smartmatic::SAES::Voting::Election::Category::ELECTION_PRINCIPAL );
        getElection()->getMapBallotOptionsByContest ( * ( *itcontest ), mapEligibleOptionsStandIn[contestCode], Smartmatic::SAES::Voting::Election::Category::ELECTION_STAND_IN );

        PVContestWrapper::getInstance()->setContest( *itcontest );
    }

    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.GetParties" ) );

    std::list < std::string >::iterator it;
    maxPartiesByAllContest = 0;

    for ( it = listContestCode.begin();
            it != listContestCode.end();
            ++it )
    {
        std::string contestCode = ( *it );
        std::list < Smartmatic::SAES::Voting::Election::Party * > listParty;
        getElection()->getPartysByContestCode ( contestCode, listParty );
        mapContestParty[contestCode] = listParty;

        if ( maxPartiesByAllContest < listParty.size() )
        {
            maxPartiesByAllContest = listParty.size();
        }

        std::list < Smartmatic::SAES::Voting::Election::Party * >::iterator itp;

        for ( itp = listParty.begin();
                itp != listParty.end();
                ++itp )
        {
            mapPartyByCode [ ( *itp )->getCode() ] = ( *itp );
            mapPartyByCustomCode [ (*itp)->getCustom_code() ] = ( *itp );
        }
    }

    for ( itcontest = listContest.begin();
            itcontest != listContest.end();
            ++itcontest )
    {
        std::string contestCode = ( *itcontest )->getCode();
        std::list < Smartmatic::SAES::Voting::Election::Party * > & listParty ( getMapContestParty() [contestCode] );
        std::list < Smartmatic::SAES::Voting::Election::Party * >::iterator itp;

        for ( itp = listParty.begin(); itp != listParty.end(); ++itp )
        {
            std::string partyCode = ( *itp )->getCode();

            getElection()->getBallotOptionsListByContestParty ( * ( *itcontest ), * ( *itp ), mapBallotOptionsListByContestParty[contestCode][partyCode] );
            getElection()->getBallotOptionsByContestParty ( * ( *itcontest ), * ( *itp ), mapBallotOptionsPrincipalByContestParty[contestCode][partyCode], Smartmatic::SAES::Voting::Election::Category::ELECTION_PRINCIPAL );
            getElection()->getBallotOptionsByContestParty ( * ( *itcontest ), * ( *itp ), mapBallotOptionsStandInByContestParty[contestCode][partyCode], Smartmatic::SAES::Voting::Election::Category::ELECTION_STAND_IN );

            std::list < std::string >::iterator itCode;
            getElection()->getAllBallotOptionsListCodeByContestParty ( * ( *itcontest ), * ( *itp ), allCodeBallot[contestCode][partyCode] );

            std::list < std::string > & listCode ( allCodeBallot[contestCode][partyCode] );

            for ( itCode = listCode.begin();
                    itCode != listCode.end();
                    ++itCode )
            {
                partyCodeByBallotOptionCode[*itCode] = partyCode;
            }

            getElection()->fillAllEligibleOptionsByContestParty ( * ( *itcontest ), * ( *itp ),
                    mapEligibleOptionsListByContestParty[contestCode][partyCode],
                    mapEligibleOptionsPrincipalByContestParty[contestCode][partyCode],
                    mapEligibleOptionsStandInByContestParty[contestCode][partyCode] );

            std::list < Smartmatic::SAES::Voting::Election::Eligible_option *>::const_iterator eoIt;

            for (eoIt = mapEligibleOptionsPrincipalByContestParty[contestCode][partyCode].begin();
                 eoIt != mapEligibleOptionsPrincipalByContestParty[contestCode][partyCode].end();
                 ++eoIt)
            {
                mapEligibleOptionByCustomCode[(*eoIt)->getCustom_code()] = *eoIt;
            }

            for (eoIt = mapEligibleOptionsStandInByContestParty[contestCode][partyCode].begin();
                 eoIt != mapEligibleOptionsStandInByContestParty[contestCode][partyCode].end();
                 ++eoIt)
            {
                mapEligibleOptionByCustomCode[(*eoIt)->getCustom_code()] = *eoIt;
            }
        }
    }

    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.GetLogicalVoterGroup" ) );

    std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * >::iterator itlvg;
    Smartmatic::SAES::Voting::VotingDevice::Voting_device & vd ( getVotingDevice()->getFirstVotingDevice() );
    getVotingDevice()->getLogicalVoterGroupList ( vd, listLVG );

    for ( itlvg = listLVG.begin(); itlvg != listLVG.end(); ++itlvg )
    {
        std::string contestGroupCode = getVotingDevice()->getLogicalVoterGroup ( vd, ( *itlvg )->getCode() ).getContest_group_code ();
        getElection()->getContestByContestGroup ( contestGroupCode, mapListContestByContestGroup[contestGroupCode] );

        Smartmatic::SAES::Voting::Election::Contest_group & contestGroup ( getElection()->getContestGroup ( contestGroupCode ) );
        contestGroupList.push_back ( &contestGroup );
        mapContestGroupByCode[contestGroupCode] = &contestGroup;
    }

    reduceElectionContestGroups();
    verifyIntegrityData();
    logger->Debug ( "End create map" );
}

void ElectoralConfiguration::calculateMax ()
{
    logger->Debug ( "Start calculate max" );
    maxContestByVote = 0;
    maxBallotOptionByVote = 0;
    std::list < Smartmatic::SAES::Voting::Election::Contest * >::iterator it;
    maxContestByBlankVote = getContestList().size();

    for ( it = getContestList().begin();
            it != getContestList().end();
            ++it )
    {
        std::string contestCode = ( *it )->getCode();
        Smartmatic::SAES::Voting::Election::Party party;
        int maxCount = getElection ()->getMaxBallotOptionsByContest ( * ( *it ), party );
        maxBallotOptionByContest[ contestCode ] = maxCount;
        contestParty[ contestCode ] = ( party.getName() != NULL ? party.getName().get() : "none" );
        std::list < Smartmatic::SAES::Voting::Election::Party * > & partys ( getMapContestParty() [contestCode] );
        std::list < Smartmatic::SAES::Voting::Election::Party * >::iterator itp;

        for ( itp = partys.begin();
                itp != partys.end();
                ++itp )
        {
            std::string partyCode = ( *itp )->getCode();
            int countList, countPrincipal, countStandIn;
            getElection ()->getCountBallotOptionsByContestParty ( * ( *it ), * ( *itp ), countList, countPrincipal, countStandIn );
            maxListBallotOptionByContestParty[ contestCode ][ partyCode ] = countList;
            maxPrincipalBallotOptionByContestParty[ contestCode ][ partyCode ] = countPrincipal;
            maxStandInBallotOptionByContestParty[ contestCode ][ partyCode ] = countStandIn;
        }
    }

    std::string maxContextGroup = "none";
    std::string maxContextGroupBallotOptions = "none";
    std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * >::iterator itlvg;

    for ( itlvg = listLVG.begin();
            itlvg != listLVG.end();
            ++itlvg )
    {
        Smartmatic::SAES::Voting::Election::Contest_group & contestGroup ( getElection()->getContestGroup ( ( *itlvg )->getContest_group_code() ) );
        Smartmatic::SAES::Voting::Election::Contest_group::Contest_codeSequence & seq ( contestGroup.getContest_code() );
        unsigned int maxContest = seq.size();

        if ( maxContest > maxContestByVote )
        {
            maxContestByVote = maxContest;
            maxContextGroup = contestGroup.getCode();
        }

        unsigned int maxBallots = 0;
        Smartmatic::SAES::Voting::Election::Contest_group::Contest_codeIterator itseq;

        for ( itseq = seq.begin();
                itseq != seq.end();
                ++itseq )
        {
            unsigned int value = maxBallotOptionByContest[ ( *itseq )];
            maxBallots += value;
        }

        if ( maxBallots > maxBallotOptionByVote )
        {
            maxBallotOptionByVote = maxBallots;
            maxContextGroupBallotOptions = contestGroup.getCode();
        }
    }

    Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group & lvg ( getVotingDevice()->getLogicalVoterGroupByContestGroupCode ( maxContextGroup ) );
    maxCustomAttributeByVote = 0;

    if ( lvg.getCustom_attributes() != NULL )
    {
        Smartmatic::SAES::Voting::VotingDevice::Custom_attributes & ca ( lvg.getCustom_attributes().get() );
        Smartmatic::SAES::Voting::VotingDevice::Custom_attributes::Custom_attributeSequence & seq ( ca.getCustom_attribute() );
        Smartmatic::SAES::Voting::VotingDevice::Custom_attributes::Custom_attributeIterator itCA;

        for ( itCA = seq.begin();
                itCA != seq.end();
                ++itCA )
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition caDefs;
            bool caOk = getVotingDevice()->GetCustomAttributeDefinition ( ( *itCA ).getCode(), caDefs );

            if ( caOk && caDefs.getMachine_type() == Smartmatic::SAES::Voting::VotingDevice::Machine_type::VOTING_DEVICES_VM )
            {
                std::string value = ElectionStringResourcesManager::getInstance()->getResource ( ( *itCA ).getValue() );

                if ( !value.empty() )
                {
                    maxCustomAttributeByVote++;
                }
            }
        }
    }

    logger->Debug ( "--------------------------------------------------------------------------------------" );
    std::stringstream cc;
    cc << "MaxContestByVote : " << maxContestByVote << " in ContestGroup : " << maxContextGroup;
    logger->Debug ( cc.str() );
    std::stringstream ss;
    ss << "MaxBallotOptionByVote : " << maxBallotOptionByVote << " in ContestGroup : " << maxContextGroupBallotOptions;
    logger->Debug ( ss.str() );
    std::stringstream ca;
    ca << "MaxCustomAttributeByVote : " << maxCustomAttributeByVote;
    logger->Debug ( ca.str() );
    logger->Debug ( "--------------------------------------------------------------------------------------" );
    viewInfo ( maxContextGroup );
    viewInfo ( maxContextGroupBallotOptions );
    logger->Debug ( "End calculate max" );
}

void ElectoralConfiguration::reduceElectionContests ()
{
    Smartmatic::SAES::Voting::Election::Ballots & ballots ( getElection()->getBallots() );
    std::list < Smartmatic::SAES::Voting::Election::Contest * > internalList;
    internalList.insert ( internalList.begin(), listContest.begin(), listContest.end() );

    for ( Smartmatic::SAES::Voting::Election::Ballots::BallotIterator
            b ( ballots.getBallot ().begin () );
            b != ballots.getBallot ().end ();
            ++b )
    {
        Smartmatic::SAES::Voting::Election::Ballot_cards & ballotCards ( getElection()->getBallotCards ( *b ) );

        for ( Smartmatic::SAES::Voting::Election::Ballot_cards::Ballot_cardIterator
                bc ( ballotCards.getBallot_card ().begin () );
                bc != ballotCards.getBallot_card ().end ();
                ++bc )
        {
            Smartmatic::SAES::Voting::Election::Contests & contests ( getElection()->getContests ( *bc ) );
            Smartmatic::SAES::Voting::Election::Contests::ContestSequence & seq ( contests.getContest() );
            Smartmatic::SAES::Voting::Election::Contests::ContestIterator cit;
            Smartmatic::SAES::Voting::Election::Contests::ContestIterator citAux;
            std::list < Smartmatic::SAES::Voting::Election::Contest * >::iterator it;
            cit = seq.begin();
            citAux = cit;

            while ( cit != seq.end() )
            {
                bool found = false;

                for ( it = internalList.begin();
                        it != internalList.end();
                        ++it )
                {
                    if ( ( *cit ).getCode() == ( *it )->getCode() )
                    {
                        found = true;
                        internalList.erase ( it );
                        break;
                    }
                }

                if ( found )
                {
                    cit++;
                    citAux++;
                }
                else
                {
                    seq.erase ( cit );
                    cit = citAux;
                }
            }
        }
    }
}

void ElectoralConfiguration::reduceElectionContestGroups ()
{
    Smartmatic::SAES::Voting::Election::Contest_groups & contestGroups ( getElection()->getContestGroups() );
    Smartmatic::SAES::Voting::Election::Contest_groups::Contest_groupSequence & seq ( contestGroups.getContest_group() );
    Smartmatic::SAES::Voting::Election::Contest_groups::Contest_groupIterator cit;
    Smartmatic::SAES::Voting::Election::Contest_groups::Contest_groupIterator citAux;
    std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * > internalList;
    internalList.insert ( internalList.begin(), listLVG.begin(), listLVG.end() );
    std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * >::iterator it;
    cit = seq.begin();
    citAux = cit;

    while ( cit != seq.end() )
    {
        bool found = false;

        for ( it = internalList.begin();
                it != internalList.end();
                ++it )
        {
            if ( ( *cit ).getCode() == ( *it )->getContest_group_code() )
            {
                found = true;
                internalList.erase ( it );
                break;
            }
        }

        if ( found )
        {
            cit++;
            citAux++;
        }
        else
        {
            seq.erase ( cit );
            cit = citAux;
        }
    }
}

void ElectoralConfiguration::verifyIntegrityData ()
{
    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.VerifyIntegrityData" ) );

    getVotingDevice()->getRegionCategoryList ( regionCategoryList );

    if ( regionCategoryList.empty() )
    {
        logger->Debug ( "Default Region Category not found." );
        throw VotingDeviceException ( _ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.DefaultRegionCategoryNotFound" ),
                                      N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.DefaultRegionCategoryNotFound" ), logger );
    }

    // Verify contest in contest groups
    Smartmatic::SAES::Voting::Election::Contest_groups & contestGroups ( getElection()->getContestGroups() );
    Smartmatic::SAES::Voting::Election::Contest_groups::Contest_groupSequence & seq ( contestGroups.getContest_group() );
    Smartmatic::SAES::Voting::Election::Contest_groups::Contest_groupIterator it;

    for ( it = seq.begin();
            it != seq.end();
            ++it )
    {
        Smartmatic::SAES::Voting::Election::Contest_group::Contest_codeSequence & cseq ( ( *it ).getContest_code() );
        Smartmatic::SAES::Voting::Election::Contest_group::Contest_codeIterator cit;

        for ( cit = cseq.begin();
                cit != cseq.end();
                ++cit )
        {
            election->getContest ( *cit );
        }
    }

    // Verify exists one voting booth

    if ( getVotingDevice()->getFirstVotingDevice().getVoting_booths().getVoting_booth().size() == 0 )
    {
        logger->Debug ( "Voting booth not found." );
        throw VotingDeviceException ( _ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.VotingBoothNotFound" ),
                                      N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.VotingBoothNotFound" ), logger );
    }

    // Verify exists one region hierarchies

    if ( getVotingDevice()->getFirstVotingDevice().getVoting_booths().getVoting_booth() [0].getRegion_hierarchies().getRegion_hierarchy().size() == 0 )
    {
        logger->Debug ( "Region hierarchies not found." );
        throw VotingDeviceException ( _ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.RegionHierarchiesNotFound" ),
                                      N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.RegionHierarchiesNotFound" ), logger );
    }

    // Verify exists one operation mode

    if ( getVotingDevice()->getFirstVotingDevice().getOperation_modes().getOperation_mode().size() == 0 )
    {
        logger->Debug ( "Operation mode not found." );
        throw VotingDeviceException ( _ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.OperationModeNotFound" ),
                                      N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.OperationModeNotFound" ), logger );
    }

    // Verify exists one role

    if ( getVotingDevice()->getFirstVotingDevice().getOperation_modes().getOperation_mode() [0].getRoles().getRole().size() == 0 )
    {
        logger->Debug ( "Role not found." );
        throw VotingDeviceException ( _ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.RoleNotFound" ),
                                      N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.RoleNotFound" ), logger );
    }

    // Verify EML files
    if (SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getEnableEml())
    {
        VMProcessStatusWrapper::getInstance()->addSuccess(
            GETTEXT_PACKAGE,
            N_("Smartmatic.SAES.Voting.ElectoralConfiguration.VerifyEmlFiles"));

        std::list<Smartmatic::SAES::Voting::Election::Contest*>::iterator cit;

        for (cit = listContest.begin(); cit != listContest.end(); ++cit)
        {
            logger->Debug( "Search eml file by custom code (" + (*cit)->getCustom_code() + ").");

            std::string file = EmlFileInfo::getInputContestFileName((*cit)->getCustom_code());
            std::string filePath = Path::Combine(directories->getEmlFilesFolderFullPath(), file);

            if (!File::getInstance()->canOpenFile(filePath))
            {
                logger->Debug("EML file " + filePath + " not found.");
                throw ElectionException(
                    _("Smartmatic.SAES.Voting.ElectoralConfiguration.EmlFileNotFound"),
                    N_("Smartmatic.SAES.Voting.ElectoralConfiguration.EmlFileNotFound"),
                    logger);
            }
        }
    }

    if ( ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() != EnumToString( VM_SAES3370 ) ) &&
        ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() != EnumToString( VM_A4517 ) ) )
    {
        bool success = true;
        std::string errorMessage;

        // Verify election files checksums
        if (SAESConfigurationManager::getInstance()->getFileHashConfiguration().getEnableHashVerification())
        {
            Smartmatic::SAES::Voting::Election::Election::ChecksumsOptional checksums = election->getChecksums();

            if (checksums.present())
            {
                Smartmatic::SAES::Voting::Election::HashValidator hashValidator(directories);

                try
                {
                    success = hashValidator.validateChecksums(checksums, errorMessage);
                }
                catch (...)
                {
                    success = false;
                }
            }
            else
            {
                success = false;
                errorMessage = "Hashes verification setting is enabled, but no checksums node was found in the election.xml file.";
            }
        }

        if (success == false)
        {
            logger->Debug(errorMessage);
            throw HashException(
                _("Smartmatic.SAES.Voting.ElectoralConfiguration.HashValidationFailed"),
                N_("Smartmatic.SAES.Voting.ElectoralConfiguration.HashValidationFailed"),
                logger);
        }
    }
}

void ElectoralConfiguration::viewInfo ( std::string contextGroup )
{
    try
    {
        Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group & lvg ( getVotingDevice()->getLogicalVoterGroupByContestGroupCode ( contextGroup ) );
        std::stringstream ss;
        ss << "ContextGroup : " << contextGroup << " - LVG : " << lvg.getCode();
        logger->Debug ( ss.str() );

        if ( lvg.getCustom_attributes() != NULL )
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attributes & ca ( lvg.getCustom_attributes().get() );
            Smartmatic::SAES::Voting::VotingDevice::Custom_attributes::Custom_attributeSequence & seq ( ca.getCustom_attribute() );
            Smartmatic::SAES::Voting::VotingDevice::Custom_attributes::Custom_attributeIterator itCA;
            logger->Debug ( "######################################################################################" );

            for ( itCA = seq.begin();
                    itCA != seq.end();
                    ++itCA )
            {
                std::stringstream cc;
                cc << "CustomAttribute : " << ( *itCA ).getCode() << " - Value : " << ElectionStringResourcesManager::getInstance()->getResource ( ( *itCA ).getValue() );
                logger->Debug ( cc.str() );
            }

            logger->Debug ( "######################################################################################" );
        }

        std::list <Smartmatic::SAES::Voting::Election::Contest * > listC;
        std::list <Smartmatic::SAES::Voting::Election::Contest * >::iterator itC;
        getElection ()->getContestOrderByContestClassByContestGroup ( contextGroup, listC );

        for ( itC = listC.begin();
                itC != listC.end();
                ++itC )
        {
            std::stringstream cc;
            Smartmatic::SAES::Voting::Election::Contest_class * contestClass = getContestClassByCode ( ( *itC )->getContest_class_code() );
            cc << "ContestCode : " << ( *itC )->getCode() << " - ContestName : " << contestClass->getCustom_code() << " - Party : " << getPartyByMaxBallotOptionContestCode ( ( *itC )->getCode() ) << " - BallotOptions : " << getMaxBallotOptionByContestCode ( ( *itC )->getCode() );
            logger->Debug ( cc.str() );
        }
    }
    catch ( ... )
    {
    }

    logger->Debug ( "--------------------------------------------------------------------------------------" );
}

Smartmatic::SAES::Voting::Election::Ballot_option * ElectoralConfiguration::getBallotOptionListFromBallotOptionCode ( std::string contestCode, std::string ballotOptionCode )
{
    Smartmatic::SAES::Voting::Election::Eligible_option * eligibleOption = ( eligibleOptionByBallotOptionCodeMap[contestCode] ) [ballotOptionCode];

    if ( eligibleOption->getParent_code() != "" )
    {
        Smartmatic::SAES::Voting::Election::Eligible_option * eligibleOptionList = ( eligibleOptionByEligibleOptionCodeMap[contestCode] ) [eligibleOption->getParent_code()];
        Smartmatic::SAES::Voting::Election::Ballot_options & ballotOptions ( eligibleOptionList->getBallot_options() );
        Smartmatic::SAES::Voting::Election::Ballot_options::Ballot_optionSequence& sequence = ballotOptions.getBallot_option();

        if (!sequence.empty())
        {
            ballotOptionCode = sequence.front().getCode();
        }
    }

    return ( mapEligibleOptionsList[contestCode] ) [ballotOptionCode];
}

void ElectoralConfiguration::notReload ()
{
    reload = false;
}

bool ElectoralConfiguration::isContestValidate ( Smartmatic::SAES::Voting::Election::Contest & contest )
{
    bool validate = false;
    Smartmatic::SAES::Voting::VotingDevice::Logical_voter_groups & lvgs ( getVotingDevice()->getLogicalVoterGroups() );
    Smartmatic::SAES::Voting::VotingDevice::Logical_voter_groups::Logical_voter_groupIterator it;

    for ( it = lvgs.getLogical_voter_group().begin();
            it != lvgs.getLogical_voter_group().end() && !validate;
            ++it )
    {
        std::string contestGroupCode = ( *it ).getContest_group_code();
        Smartmatic::SAES::Voting::Election::Contest_group::Contest_codeSequence cgs = ( getElection()->getContestGroup ( contestGroupCode ).getContest_code() );
        Smartmatic::SAES::Voting::Election::Contest_group::Contest_codeIterator itcg;

        for ( itcg = cgs.begin();
                itcg != cgs.end() && !validate;
                ++itcg )
        {
            if ( ( *itcg ) == contest.getCode() )
            {
                validate = true;
            }
        }
    }

    return validate;
}

void ElectoralConfiguration::loadAllData()
{
    if ( !loadData )
    {
        VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Voting.ElectoralConfiguration.LoadElectoralConfiguration" ) );

        logger->Debug ( "Load all data" );
        loadData = true;

        PVContestWrapper::getInstance()->init();

        createMap();
        calculateMax();

        voteMaxCalculate->calculate();

        if ( SAESCommonConfigurationManager::getInstance()->isForceReduceElection() )
        {
            Smartmatic::SAES::Runtime::Environment::SaesDirectories directories;
            getElection()->persistence ( directories );
        }
    }
}

void ElectoralConfiguration::setMachineTime()
{
    std::string openingDateTime = getVotingDevice()->getFirstVotingDevice().getOperation_modes().getOperation_mode() [0].getOpening_date_time();

    Glib::TimeVal openingTime;
    openingTime.assign_from_iso8601 ( openingDateTime );
    double openingD = openingTime.as_double();

    Encryption::getInstance()->setMachineTime ( openingD );
}

unsigned int ElectoralConfiguration::getMaxBallotOptionsByVote ()
{
    return maxBallotOptionByVote;
}

unsigned int ElectoralConfiguration::getMaxContestByVote ()
{
    return maxContestByVote;
}

unsigned int ElectoralConfiguration::getMaxContestByBlankVote ()
{
    return maxContestByBlankVote;
}

unsigned int ElectoralConfiguration::getMaxCustomAttributeByVote ()
{
    return maxCustomAttributeByVote;
}

std::map < std::string, int > & ElectoralConfiguration::getMaxBallotOptionByContestCode()
{
    return maxBallotOptionByContest;
}

unsigned int ElectoralConfiguration::getMaxBallotOptionByContestCode ( std::string contestCode )
{
    return getMaxBallotOptionByContestCode() [ contestCode ];
}

std::string ElectoralConfiguration::getPartyByMaxBallotOptionContestCode ( std::string contestCode )
{
    return contestParty[ contestCode ];
}

std::map < std::string, std::map <std::string, unsigned int > > & ElectoralConfiguration::getMapListBallotOptionByContestParty ( )
{
    return maxListBallotOptionByContestParty;
}

std::map < std::string, std::map <std::string, unsigned int > > & ElectoralConfiguration::getMapPrincipalBallotOptionByContestParty ( )
{
    return maxPrincipalBallotOptionByContestParty;
}

std::map < std::string, std::map <std::string, unsigned int > > & ElectoralConfiguration::getMapStandInBallotOptionByContestParty ( )
{
    return maxStandInBallotOptionByContestParty;
}

unsigned int ElectoralConfiguration::getCountListBallotOptionByContestParty ( std::string contestCode, std::string partyCode )
{
    return getMapListBallotOptionByContestParty() [contestCode][partyCode];
}

unsigned int ElectoralConfiguration::getCountPrincipalBallotOptionByContestParty ( std::string contestCode, std::string partyCode )
{
    return getMapPrincipalBallotOptionByContestParty() [contestCode][partyCode];
}

unsigned int ElectoralConfiguration::getCountStandInBallotOptionByContestParty ( std::string contestCode, std::string partyCode )
{
    return getMapStandInBallotOptionByContestParty() [contestCode][partyCode];
}

unsigned int ElectoralConfiguration::getCountBallotOptionByContestParty ( std::string contestCode, std::string partyCode )
{
    return getAllCodeBallotOptionByContestParty ( contestCode, partyCode ).size();
}

void ElectoralConfiguration::getCountBallotOptionByContestParty ( std::string contestCode, std::string partyCode, int & countList, int & countPrincipal, int & countStandIn )
{
    countList = getCountListBallotOptionByContestParty ( contestCode, partyCode );
    countPrincipal = getCountPrincipalBallotOptionByContestParty ( contestCode, partyCode );
    countStandIn = getCountStandInBallotOptionByContestParty ( contestCode, partyCode );
}

std::list < std::string > & ElectoralConfiguration::getAllCodeBallotOptionByContestParty ( std::string contestCode, std::string partyCode )
{
    return getAllCodeBallotOptionByContestParty() [contestCode][partyCode];
}

std::map <std::string, std::map < std::string, std::list <std::string> > > & ElectoralConfiguration::getAllCodeBallotOptionByContestParty()
{
    return allCodeBallot;
}

std::string ElectoralConfiguration::getPartyCodeByBallotOptionCode ( std::string ballotCode )
{
    return partyCodeByBallotOptionCode[ballotCode];
}

Smartmatic::SAES::Voting::Election::Eligible_option * ElectoralConfiguration::getEligibleOptionFromEligibleOptionCode ( std::string contestCode, std::string eligibleOptionCode )
{
    return ( eligibleOptionByEligibleOptionCodeMap[contestCode] ) [eligibleOptionCode];
}

Smartmatic::SAES::Voting::Election::Eligible_option * ElectoralConfiguration::getEligibleOptionFromBallotOptionCode ( const std::string & contestCode, const std::string & ballotOptionCode )
{
    return ( eligibleOptionByBallotOptionCodeMap[contestCode] ) [ballotOptionCode];
}

std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getEligibleOptionsListByContestCode ( std::string contestCode )
{
    return mapEligibleOptionsListByContest[contestCode];
}

std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getEligibleOptionsPrincipalByContestCode ( std::string contestCode )
{
    return mapEligibleOptionsPrincipalByContest[contestCode];
}

std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getEligibleOptionsStandInByContestCode ( std::string contestCode )
{
    return mapEligibleOptionsStandInByContest[contestCode];
}

std::map < std::string, Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getMapEligibleOptionsListByContestCode ( std::string contestCode )
{
    return mapEligibleOptionsList[contestCode];
}

std::map < std::string, Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getMapEligibleOptionsPrincipalByContestCode ( std::string contestCode )
{
    return mapEligibleOptionsPrincipal[contestCode];
}

std::map < std::string, Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getMapEligibleOptionsStandInByContestCode ( std::string contestCode )
{
    return mapEligibleOptionsStandIn[contestCode];
}

Smartmatic::SAES::Voting::Election::Contest * ElectoralConfiguration::getContestByCode ( std::string code )
{
    return mapContestCode[code];
}

std::list < Smartmatic::SAES::Voting::Election::Party * > & ElectoralConfiguration::getListPartyByContestCode ( std::string code )
{
    return getMapContestParty() [code];
}

std::list < Smartmatic::SAES::Voting::Election::Contest * > & ElectoralConfiguration::getListContestByContestGroupCode ( std::string code )
{
    return mapListContestByContestGroup[code];
}

std::list < Smartmatic::SAES::Voting::Election::Contest * > & ElectoralConfiguration::getContestList()
{
    return listContest;
}

std::list < std::string > & ElectoralConfiguration::getContestCodeList()
{
    return listContestCode;
}

std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * > & ElectoralConfiguration::getLogicalVoterGroupList()
{
    return listLVG;
}

std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getBallotOptionsListByContestParty ( std::string contestCode, std::string partyCode )
{
    return mapBallotOptionsListByContestParty[contestCode][partyCode];
}

std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getBallotOptionsPrincipalByContestParty ( std::string contestCode, std::string partyCode )
{
    return mapBallotOptionsPrincipalByContestParty[contestCode][partyCode];
}

std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getBallotOptionsStandInByContestParty ( std::string contestCode, std::string partyCode )
{
    return mapBallotOptionsStandInByContestParty[contestCode][partyCode];
}

std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getBallotOptionPartyByContest ( std::string contestCode )
{
    return mapBallotOptionsPartyByContest[contestCode];
}

unsigned int ElectoralConfiguration::getMaxPartiesByAllContest()
{
    return maxPartiesByAllContest;
}

std::list < Smartmatic::SAES::Voting::Election::Eligible_option * > & ElectoralConfiguration::getEligibleOptionsListByContestParty ( std::string contestCode, std::string partyCode )
{
    return mapEligibleOptionsListByContestParty[contestCode][partyCode];
}

std::list < Smartmatic::SAES::Voting::Election::Eligible_option * > & ElectoralConfiguration::getEligibleOptionsPrincipalByContestParty ( std::string contestCode, std::string partyCode )
{
    return mapEligibleOptionsPrincipalByContestParty[contestCode][partyCode];
}

std::list < Smartmatic::SAES::Voting::Election::Eligible_option * > & ElectoralConfiguration::getEligibleOptionsStandInByContestParty ( std::string contestCode, std::string partyCode )
{
    return mapEligibleOptionsStandInByContestParty[contestCode][partyCode];
}

std::list <Smartmatic::SAES::Voting::Election::Contest_group * > & ElectoralConfiguration::getContestGroups()
{
    return contestGroupList;
}

Smartmatic::SAES::Voting::Election::Contest_group * ElectoralConfiguration::getContestGroup ( std::string contestGroupCode )
{
    return mapContestGroupByCode[contestGroupCode];
}

std::list < Smartmatic::SAES::Voting::Election::Ballot * > & ElectoralConfiguration::getBallots()
{
    return ballotList;
}

std::list <Smartmatic::SAES::Voting::VotingDevice::Region_category > & ElectoralConfiguration::getRegionCategoryList()
{
    return regionCategoryList;
}

void ElectoralConfiguration::getFileNamesByLanguages ( std::string file, std::list <std::string> & fileNames, bool allLanguages, unsigned int delta, bool onlyLanguage )
{
    fileNames.clear();
    std::list <std::string>::iterator itl;
    std::list <std::string> languages = getLanguageListFullName ( true, onlyLanguage );

    if ( getLanguageListFullName().size() > delta )
    {
        for ( itl = languages.begin();
                itl != languages.end();
                ++itl )
        {
            std::string tmpFile = Path::GetFileNameWithOutExtension ( file ) + ( *itl ) + Path::GetFileExtension ( file );
            fileNames.push_back ( tmpFile );
        }
    }
    else if ( allLanguages )
    {
        std::string currentLanguage = BaseLanguageController::getActualLanguage();
        std::string tmpFile = Path::GetFileNameWithOutExtension ( file ) + currentLanguage + Path::GetFileExtension ( file );
        fileNames.push_back ( tmpFile );
    }
    else
    {
        fileNames.push_back ( file );
    }
}

Smartmatic::SAES::Voting::Election::Party * ElectoralConfiguration::getPartyByCode ( std::string code )
{
    return mapPartyByCode[ code ];
}

void ElectoralConfiguration::loadDefaultLanguages()
{
    std::list < Smartmatic::SAES::Voting::Election::Language> listLang;
    std::list<Smartmatic::SAES::Voting::Election::Language>::iterator it;

    Smartmatic::SAES::Voting::Election::ElectionInstalationLanguages::getInstance()->ReloadLanguages();
    listLang = Smartmatic::SAES::Voting::Election::ElectionInstalationLanguages::getInstance()->getLanguages();

    for ( it = listLang.begin(); it != listLang.end(); ++it )
    {
        if ( it->getIs_default() || listLang.size() == 1 )
        {
            logger->Debug ( "Found language " + it->getLanguage() + " " +  it->getCountry() );
            Smartmatic::SAES::Voting::SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( it->getLanguage(), it->getCountry() );
            return;
        }
    }
}

std::map < std::string, std::list < ElectoralConfiguration::typeParty * > > & ElectoralConfiguration::getMapContestParty()
{
    return mapContestParty;
}

Smartmatic::SAES::Voting::Election::Contest_class * ElectoralConfiguration::getContestClassByCode ( const std::string & code )
{
    return mapContestClass[ code ];
}

void ElectoralConfiguration::getBallotOptionNames ( const Election::Eligible_option * eligible, std::string & firstName, std::string & lastName, std::string & initialFirstName ) const
{
    std::string separator = SAESConfigurationManager::getInstance()->getPrintVoteConfiguration().getBallotOptionNameSeparator();
    std::string sname = eligible->getBallot_name_key();
    sname = ElectionStringResourcesManager::getInstance()->getResource ( sname );

    std::vector <std::string> parts;
    StringUtils::splitToVector ( sname, separator, parts );

    firstName.clear();
    lastName.clear();
    initialFirstName.clear();

    if ( parts.size() >= 1 )
    {
        lastName = parts[0];
    }

    if ( parts.size() >= 2 )
    {
        firstName = parts[1];
    }

    if ( parts.size() >= 3 )
    {
        initialFirstName = parts[2];
    }
    else
    {
        initialFirstName = StringUtils::trim ( firstName ).substr ( 0, 1 );
    }
}

std::string ElectoralConfiguration::getPartyNameByExpectedConfirmationReference(const std::string& reference) const
{
    std::map<std::string, typeParty*>::const_iterator it = mapPartyByCustomCode.find(reference);

    if (it != mapPartyByCustomCode.end())
    {
        std::stringstream name;

        name << (it->second)->getName();

        return name.str();
    }

    throw PartyNotFoundException("Party with expected confirmation reference " + reference + " not found",
                                 "Party not found");
}

std::string ElectoralConfiguration::getCandidateNameByExpectedConfirmationReference(const std::string& reference) const
{
    std::map<std::string, typeEligibleOption*>::const_iterator it = mapEligibleOptionByCustomCode.find(reference);

    if (it != mapEligibleOptionByCustomCode.end())
    {
        std::string firstName;
        std::string lastName;
        std::string initials;

        getBallotOptionNames ( it->second, firstName, lastName, initials );

        return firstName + " " + lastName;
    }

    throw CandidateNotFoundException("Candidate with expected confirmation reference " + reference + " not found",
                                     "Candidate not found");
}
