/**
 * @file	ElectoralConfiguration.hxx
 * @brief	electoral configuration class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef __ELECTORAL_CONFIGURATION_H__
#define __ELECTORAL_CONFIGURATION_H__

#include <Voting/Election/ElectionWrapper.hxx>
#include <Voting/VotingDevice/VotingDeviceWrapper.hxx>
#include <Voting/Vote/VoteRecovery.h>
#include <Voting/Vote/VoteMaxCalculate.hxx>
#include "ElectoralFactory.hxx"
#include <Runtime/Environment/saes-directories.h>
#include <string>
#include <map>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            /**
             * @class	ElectoralConfiguration
             * @brief	Electoral configuration.
             * @author	Eduardo.campione
             * @date	28/04/2011
             */

            class ElectoralConfiguration
            {
                public:

                    /**
                     * @fn	ElectoralConfiguration::ElectoralConfiguration();
                     * @brief	Class constructor.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    ElectoralConfiguration();

                    /**
                     * @fn	virtual ElectoralConfiguration::~ElectoralConfiguration()
                     * @brief	Class destructor.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    virtual ~ElectoralConfiguration();

                    /**
                     * @brief	Initialises this object.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @throws  ElectionException     Thrown when an election error condition occurs.
                     * @throws  VotingDeviceException Thrown when a voting device error condition occurs.
                     * @throws  CryptoException       Thrown when a crypto error condition occurs.
                     * @throws  SignedException       Thrown when a signed error condition occurs.
                     * @throws  FileSystemException   Thrown when a file system error condition occurs.
                     * @param	directories           The SAES directories.
                     * @param	type                  Type of load default.
                     */
                    void init ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories,
                                Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                    /**
                     * @fn	void ElectoralConfiguration::dispose();
                     * @brief	method dispose.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    void dispose();

                    /**
                     * @brief	Gets the election.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @throws	ElectionException Thrown when an election error condition occurs.
                     * @return	null if it fails, else the election.
                     */
                    Smartmatic::SAES::Voting::Election::ElectionWrapper* getElection ();

                    /**
                     * @brief	Gets the voting device.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @throws	VotingDeviceException Thrown when a voting device error condition occurs.
                     * @return	null if it fails, else the voting device.
                     */
                    Smartmatic::SAES::Voting::VotingDevice::VotingDeviceWrapper* getVotingDevice ();

                    /**
                     * @brief	Gets the vote recovery.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @throws  VoteRecoveryException Thrown when a vote recovery error condition occurs.
                     * @return	null if it fails, else the vote recovery.
                     */
                    Smartmatic::SAES::Voting::Vote::VoteRecovery* getVoteRecovery();


                    virtual Smartmatic::SAES::Voting::Vote::VoteMaxCalculate * getVoteMaxCalculate();

                    /**
                     * @brief	method for create data map and calculate max.
                     * @author	Yadickson Soto
                     * @date	13/03/2012
                     * @throws  ElectionException     Thrown when an election error condition occurs.
                     * @throws  VotingDeviceException Thrown when a voting device error condition occurs.
                     */
                    void loadAllData();

                    /**
                     * @fn	virtual unsigned int ElectoralConfiguration::getMaxBallotOptionsByVote ();
                     * @brief	method getter max ballot options of electoral configuration.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return	max ballot options.
                     */

                    virtual unsigned int getMaxBallotOptionsByVote ();

                    /**
                     * @fn	unsigned int ElectoralConfiguration::getMaxContestByVote ();
                     * @brief	method getter max contest of electoral configuration.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return	max contest.
                     */

                    unsigned int getMaxContestByVote ();

                    /**
                     * @fn	unsigned int ElectoralConfiguration::getMaxContestByBlankVote ();
                     * @brief	method getter max contest of electoral configuration.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	17/06/2011
                     * @return	max contest by blank vote.
                     */

                    unsigned int getMaxContestByBlankVote ();

                    /**
                     * @fn	unsigned int ElectoralConfiguration::getMaxCustomAttributeByVote ();
                     * @brief	method getter max custom attribute of electoral configuration.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @return	custom attribute vote.
                     */
                    unsigned int getMaxCustomAttributeByVote ();

                    /**
                     * @fn	virtual unsigned int ElectoralConfiguration::getMaxBallotOptionByContestCode ( std::string contestCode );
                     * @brief	Gets a maximum ballot option by contest code.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	contestCode	The contest code.
                     * @return	The maximum ballot option by contest code.
                     */

                    virtual unsigned int getMaxBallotOptionByContestCode ( std::string contestCode );

                    /**
                     * @fn	virtual std::map < std::string, int > & ElectoralConfiguration::getMaxBallotOptionByContestCode();
                     * @brief	method getter map of max ballot option by contest code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @return	map of max ballot option by contest code.
                     */
                    virtual std::map < std::string, int > & getMaxBallotOptionByContestCode();

                    /**
                     * @fn	std::string ElectoralConfiguration::getPartyByMaxBallotOptionContestCode ( std::string contestCode );
                     * @brief	method getter party code by max ballot option by contest code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode	contest code.
                     * @return	party code.
                     */
                    std::string getPartyByMaxBallotOptionContestCode ( std::string contestCode );

                    /**
                     * @fn	unsigned int ElectoralConfiguration::getCountListBallotOptionByContestParty ( std::string contestCode,
                     * 			std::string partyCode );
                     * @brief	method getter count list of ballot option by contest code and party code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode	contest code.
                     * @param	partyCode	 	party code.
                     * @return	count list.
                     */
                    unsigned int getCountListBallotOptionByContestParty ( std::string contestCode, std::string partyCode );

                    /**
                     * @fn	unsigned int ElectoralConfiguration::getCountPrincipalBallotOptionByContestParty ( std::string contestCode,
                     * 			std::string partyCode );
                     * @brief	method getter count principal of ballot option by contest code and party code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode	contest code.
                     * @param	partyCode	 	party code.
                     * @return	count principal.
                     */
                    unsigned int getCountPrincipalBallotOptionByContestParty ( std::string contestCode, std::string partyCode );

                    /**
                     * @fn	virtual unsigned int ElectoralConfiguration::getCountStandInBallotOptionByContestParty ( std::string contestCode,
                     * 			std::string partyCode );
                     * @brief	method getter count stand in of ballot option by contest code and party code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode	contest code.
                     * @param	partyCode	 	party code.
                     * @return	count stand in.
                     */
                    virtual unsigned int getCountStandInBallotOptionByContestParty ( std::string contestCode, std::string partyCode );

                    /**
                     * @fn	virtual unsigned int ElectoralConfiguration::getCountBallotOptionByContestParty ( std::string contestCode,
                     * 			std::string partyCode );
                     * @brief	method getter all count of ballot option by contest code and party code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode	contest code.
                     * @param	partyCode	 	party code.
                     * @return	count.
                     */
                    virtual unsigned int getCountBallotOptionByContestParty ( std::string contestCode, std::string partyCode );

                    /**
                     * @fn	virtual void ElectoralConfiguration::getCountBallotOptionByContestParty ( std::string contestCode,
                     * 			std::string partyCode, int & countList, int & countPrincipal, int & countStandIn );
                     * @brief	method getter all count of ballot option by contest code and party code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode			contest code.
                     * @param	partyCode				party code.
                     * @param	countList				in, out count list.
                     * @param	countPrincipal	in, out count principal.
                     * @param	countStandIn		in, out count stand in.
                     */
                    virtual void getCountBallotOptionByContestParty ( std::string contestCode, std::string partyCode, int & countList, int & countPrincipal, int & countStandIn );

                    /**
                     * @fn	virtual std::list < std::string > & ElectoralConfiguration::getAllCodeBallotOptionByContestParty(std::string contestCode,
                     * 			std::string partyCode );
                     * @brief	Gets all code ballot option by contest party.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	contestCode	The contest code.
                     * @param	partyCode	 	The party code.
                     * @return	all code ballot option by contest party.
                     */
                    virtual std::list < std::string > & getAllCodeBallotOptionByContestParty ( std::string contestCode, std::string partyCode );

                    /**
                     * @fn	virtual std::map <std::string, std::map < std::string, std::list <std::string> > > & ElectoralConfiguration::getAllCodeBallotOptionByContestParty();
                     * @brief	Gets all code ballot option by contest party.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @return	all code ballot option by contest party.
                     */
                    virtual std::map <std::string, std::map < std::string, std::list <std::string> > > & getAllCodeBallotOptionByContestParty();

                    /**
                     * @fn	virtual std::string ElectoralConfiguration::getPartyCodeByBallotOptionCode(std::string ballotCode);
                     * @brief	Gets party code by ballot option code.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	ballotCode	The ballot code.
                     * @return	The party code by ballot option code.
                     */
                    virtual std::string getPartyCodeByBallotOptionCode ( std::string ballotCode );

                    /**
                     * @fn	void ElectoralConfiguration::viewInfo(std::string contextGroup);
                     * @brief	method view info debug context group.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contextGroup	contest group code.
                     */
                    void viewInfo ( std::string contextGroup );

                    /**
                     * @fn	void ElectoralConfiguration::notReload();
                     * @brief	method not reload config files.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     */
                    void notReload();

                    /**
                     * @fn	Smartmatic::SAES::Voting::Election::Ballot_option * ElectoralConfiguration::getBallotOptionListFromBallotOptionCode(std::string contestCode,
                     * 			std::string ballotOptionCode);
                     * @brief	method getter ballot option by contest code and ballot code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode				contest code.
                     * @param	ballotOptionCode	ballot option code.
                     * @return	ballot option.
                     */
                    Smartmatic::SAES::Voting::Election::Ballot_option * getBallotOptionListFromBallotOptionCode ( std::string contestCode, std::string ballotOptionCode );

                    /**
                     * @fn	virtual Smartmatic::SAES::Voting::Election::Eligible_option * ElectoralConfiguration::getEligibleOptionFromBallotOptionCode( const std::string & contestCode,
                     * 			const std::string & ballotOptionCode);
                     * @brief	method getter eligible option by contest code and ballot code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode				contest code.
                     * @param	ballotOptionCode	ballot option code.
                     * @return	eligible option.
                     */
                    virtual Smartmatic::SAES::Voting::Election::Eligible_option * getEligibleOptionFromBallotOptionCode ( const std::string & contestCode, const std::string & ballotOptionCode );

                    /**
                     * @fn	Smartmatic::SAES::Voting::Election::Eligible_option * ElectoralConfiguration::getEligibleOptionFromEligibleOptionCode(std::string contestCode,
                     * 			std::string eligibleOptionCode);
                     * @brief	method getter eligible option by contest code and eligible option code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode					contest code.
                     * @param	eligibleOptionCode	eligible option code.
                     * @return	eligible option.
                     */
                    Smartmatic::SAES::Voting::Election::Eligible_option * getEligibleOptionFromEligibleOptionCode ( std::string contestCode, std::string eligibleOptionCode );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Ballot_option > & ElectoralConfiguration::getEligibleOptionsListByContestCode (std::string contestCode);
                     * @brief	method getter ballot option list by contest code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode	contest code.
                     * @return	ballot option list.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & getEligibleOptionsListByContestCode ( std::string contestCode );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Ballot_option > & ElectoralConfiguration::getEligibleOptionsPrincipalByContestCode (std::string contestCode);
                     * @brief	method getter ballot option principal by contest code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode	contest code.
                     * @return	ballot option principal.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & getEligibleOptionsPrincipalByContestCode ( std::string contestCode );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Ballot_option > & ElectoralConfiguration::getEligibleOptionsStandInByContestCode (std::string contestCode);
                     * @brief	method getter ballot option stand in by contest code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode	contest code.
                     * @return	ballot option stand in.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & getEligibleOptionsStandInByContestCode ( std::string contestCode );

                    /**
                     * @fn	std::map < std::string, Smartmatic::SAES::Voting::Election::Ballot_option > & ElectoralConfiguration::getMapEligibleOptionsListByContestCode (std::string contestCode);
                     * @brief	method getter eligible option map list by contest code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode	contest code.
                     * @return	eligible option map list.
                     */
                    std::map < std::string, Smartmatic::SAES::Voting::Election::Ballot_option * > & getMapEligibleOptionsListByContestCode ( std::string contestCode );

                    /**
                     * @fn	std::map < std::string, Smartmatic::SAES::Voting::Election::Ballot_option > & ElectoralConfiguration::getMapEligibleOptionsPrincipalByContestCode (std::string contestCode);
                     * @brief	method getter eligible option map principal by contest code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode	contest code.
                     * @return	eligible option map principal.
                     */
                    std::map < std::string, Smartmatic::SAES::Voting::Election::Ballot_option * > & getMapEligibleOptionsPrincipalByContestCode ( std::string contestCode );

                    /**
                     * @fn	std::map < std::string, Smartmatic::SAES::Voting::Election::Ballot_option > & ElectoralConfiguration::getMapEligibleOptionsStandInByContestCode (std::string contestCode);
                     * @brief	method getter eligible option map stand in by contest code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	contestCode	contest code.
                     * @return	eligible option map stand in.
                     */
                    std::map < std::string, Smartmatic::SAES::Voting::Election::Ballot_option * > & getMapEligibleOptionsStandInByContestCode ( std::string contestCode );

                    /**
                     * @fn	std::list <std::string> ElectoralConfiguration::getLanguageListFullName(bool onlyDefaultTrue = true,
                     * 			bool onlyLanguage = false);
                     * @brief	method getter language list full name.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	onlyDefaultTrue	only default true parameter.
                     * @param	onlyLanguage	 	(optional) the only language.
                     * @return	language list full name.
                     */
                    std::list <std::string> getLanguageListFullName ( bool onlyDefaultTrue = true, bool onlyLanguage = false );

                    /**
                     * @fn	std::list <Smartmatic::SAES::Voting::Election::Language> ElectoralConfiguration::getLanguageList(bool onlyDefaultTrue = true);
                     * @brief	method getter language list.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @param	onlyDefaultTrue	only default true parameter.
                     * @return	language list.
                     */
                    std::list <Smartmatic::SAES::Voting::Election::Language> getLanguageList ( bool onlyDefaultTrue = true );

                    /**
                     * @fn	Smartmatic::SAES::Voting::Election::Languages & ElectoralConfiguration::getLanguages ();
                     * @brief	method getter languages.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/02/2012
                     * @return	languages.
                     */
                    Smartmatic::SAES::Voting::Election::Languages & getLanguages ();

                    /**
                     * @fn	void loadDefaultLanguages();
                     * @brief	method load default languages.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	04/06/2013
                     */
                    void loadDefaultLanguages();

                    /**
                     * @fn	virtual Smartmatic::SAES::Voting::Election::Contest & ElectoralConfiguration::getContestByCode (std::string code);
                     * @brief	method getter contest by code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	23/02/2012
                     * @param	code	contest code.
                     * @return	contest by code.
                     */
                    virtual Smartmatic::SAES::Voting::Election::Contest * getContestByCode ( std::string code );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Party * > & ElectoralConfiguration::getListPartyByContestCode (std::string code);
                     * @brief	method getter list parties by contest code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	22/02/2012
                     * @param	code	contest code.
                     * @return	list parties by contest code.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Party * > & getListPartyByContestCode ( std::string code );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Contest > & ElectoralConfiguration::getListContestByContestGroupCode (std::string code);
                     * @brief	method getter list contest by contest group code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	22/02/2012
                     * @param	code	contest group code.
                     * @return	list contest by contest group code.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Contest * > & getListContestByContestGroupCode ( std::string code );

                    /**
                     * @fn	virtual std::list < Smartmatic::SAES::Voting::Election::Contest > & ElectoralConfiguration::getContestList();
                     * @brief	method getter list all contest.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	23/02/2012
                     * @return	list all contest.
                     */
                    virtual std::list < Smartmatic::SAES::Voting::Election::Contest * > & getContestList();

                    /**
                     * @fn	virtual std::list < std::string > & ElectoralConfiguration::getContestCodeList();
                     * @brief	method getter list all contest code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	31/10/2012
                     * @return	list all contest code.
                     */
                    virtual std::list < std::string > & getContestCodeList();

                    /**
                     * @fn	virtual Smartmatic::SAES::Voting::Election::Contest_class * getContestClassByCode( const std::string & code );
                     * @brief	Gets contest class by code.
                     * @author	Yadickson Soto
                     * @date	26/03/2014
                     * @param	code The contest class code
                     * @return	The contest class.
                     */
                    virtual Smartmatic::SAES::Voting::Election::Contest_class * getContestClassByCode ( const std::string & code );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * > & ElectoralConfiguration::getLogicalVoterGroupList();
                     * @brief	method getter list all logical voter group.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	23/02/2012
                     * @return	list all logical voter group.
                     */
                    std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * > & getLogicalVoterGroupList();

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Eligible_option * > & ElectoralConfiguration::getEligibleOptionsListByContestParty(std::string contestCode,
                     * 			std::string partyCode);
                     * @brief	method getter eligible option list by contest party.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13/03/2012
                     * @param	contestCode	contest code.
                     * @param	partyCode	 	party code.
                     * @return	eligible option list by contest party.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Eligible_option * > & getEligibleOptionsListByContestParty ( std::string contestCode, std::string partyCode );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Eligible_option * > & ElectoralConfiguration::getEligibleOptionsPrincipalByContestParty(std::string contestCode,
                     * 			std::string partyCode);
                     * @brief	method getter eligible option principal by contest party.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13/03/2012
                     * @param	contestCode	contest code.
                     * @param	partyCode	 	party code.
                     * @return	ballot eligible principal by contest party.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Eligible_option * > & getEligibleOptionsPrincipalByContestParty ( std::string contestCode, std::string partyCode );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Eligible_option * > & ElectoralConfiguration::getEligibleOptionsStandInByContestParty(std::string contestCode,
                     * 			std::string partyCode);
                     * @brief	method getter eligible option stand in by contest party.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13/03/2012
                     * @param	contestCode	contest code.
                     * @param	partyCode	 	party code.
                     * @return	eligible option stand in by contest party.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Eligible_option * > & getEligibleOptionsStandInByContestParty ( std::string contestCode, std::string partyCode );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getBallotOptionPartyByContest(std::string contestCode);
                     * @brief	method getter ballot option party by contest code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13/03/2012
                     * @param	contestCode	contest code.
                     * @return	ballot option party by contest.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & getBallotOptionPartyByContest ( std::string contestCode );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getBallotOptionsListByContestParty(std::string contestCode,
                     * 			std::string partyCode);
                     * @brief	method getter ballot option list by contest party.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	23/02/2012
                     * @param	contestCode	contest code.
                     * @param	partyCode	 	party code.
                     * @return	ballot option list by contest party.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & getBallotOptionsListByContestParty ( std::string contestCode, std::string partyCode );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getBallotOptionsPrincipalByContestParty(std::string contestCode,
                     * 			std::string partyCode);
                     * @brief	method getter ballot option principal by contest party.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	23/02/2012
                     * @param	contestCode	contest code.
                     * @param	partyCode	 	party code.
                     * @return	ballot option principal by contest party.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & getBallotOptionsPrincipalByContestParty ( std::string contestCode, std::string partyCode );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & ElectoralConfiguration::getBallotOptionsStandInByContestParty(std::string contestCode,
                     * 			std::string partyCode);
                     * @brief	method getter ballot option stand in by contest party.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	23/02/2012
                     * @param	contestCode	contest code.
                     * @param	partyCode	 	party code.
                     * @return	ballot option stand in by contest party.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & getBallotOptionsStandInByContestParty ( std::string contestCode, std::string partyCode );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Contest_group * > & ElectoralConfiguration::getContestGroups();
                     * @brief	method getter contest group.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13/03/2012
                     * @return	contest group list.
                     */
                    virtual std::list < Smartmatic::SAES::Voting::Election::Contest_group * > & getContestGroups();

                    /**
                     * @fn	Smartmatic::SAES::Voting::Election::Contest_group * ElectoralConfiguration::getContestGroup(std::string contestGroupCode);
                     * @brief	method getter contest group by code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13/03/2012
                     * @param	contestGroupCode	contest group code.
                     * @return	contest group.
                     */
                    Smartmatic::SAES::Voting::Election::Contest_group * getContestGroup ( std::string contestGroupCode );

                    /**
                     * @fn	std::list < Smartmatic::SAES::Voting::Election::Ballot * > & ElectoralConfiguration::getBallots();
                     * @brief	method getter ballots.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13/03/2012
                     * @return	ballots.
                     */
                    std::list < Smartmatic::SAES::Voting::Election::Ballot * > & getBallots();

                    /**
                     * @fn	Smartmatic::SAES::Voting::Election::Party * getPartyByCode( std::string code );
                     * @brief	method getter party by party code.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	14/11/2012
                     * @param code code by party
                     * @return	party by party code.
                     */
                    Smartmatic::SAES::Voting::Election::Party * getPartyByCode ( std::string code );

                    /**
                     * @fn	bool ElectoralConfiguration::isContestValidate ( Smartmatic::SAES::Voting::Election::Contest & contest );
                     * @brief	Method validate contest in voting device.
                     * @author	Yadickson Soto
                     * @date	13/03/2012
                     * @param	contest	contest.
                     * @return	true is validate.
                     */
                    bool isContestValidate ( Smartmatic::SAES::Voting::Election::Contest & contest );
                    /**
                     * @fn	unsigned int ElectoralConfiguration::getMaxPartiesByAllContest();
                     * @brief	Method getter max parties by all contest.
                     * @author	Yadickson Soto
                     * @date	13/03/2012
                     * @return	max parties by all contest.
                     */
                    unsigned int getMaxPartiesByAllContest();
                    /**
                     * @fn	std::list <Smartmatic::SAES::Voting::VotingDevice::Region_category > & ElectoralConfiguration::getRegionCategoryList();
                     * @brief	method getter all region category.
                     * @author	Yadickson Soto
                     * @date	04/04/2012
                     * @return	all region category.
                     */
                    std::list <Smartmatic::SAES::Voting::VotingDevice::Region_category > & getRegionCategoryList();
                    /**
                     * @fn	void ElectoralConfiguration::getFileNamesByLanguages(std::string file,
                     * 			std::list <std::string> & fileNames, bool allLanguages, unsigned int delta = 0,
                     * 			bool onlyLanguage = false);
                     * @brief	Method getter file names by languages.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10/05/2012
                     * @param	file		 	file name
                     * @param [in,out]	fileNames	List of names of the files.
                     * @param	allLanguages	 	all languages.
                     * @param	delta		 	delta compare language.
                     * @param	onlyLanguage	 	only language.
                     */
                    void getFileNamesByLanguages ( std::string file, std::list <std::string> & fileNames, bool allLanguages, unsigned int delta = 0, bool onlyLanguage = false );

                    /**
                     * @fn	virtual void getBallotOptionNames ( const Election::Eligible_option * eligible, std::string & firstName, std::string & lastName, std::string & initialFirstName );
                     * @brief	Gest firstname, lastname and initial firstname by ballot option.
                     * @author	Yadickson Soto
                     * @date	10/0/2014
                     * @param	eligible		The input eligible option.
                     * @param	firstName	The first name result.
                     * @param	lastName	The last name result.
                     * @param	initialFirstName	The initial first name.
                     */
                    virtual void getBallotOptionNames ( const Election::Eligible_option * eligible, std::string & firstName, std::string & lastName, std::string & initialFirstName ) const;

                    /**
                     * @brief  Gets the party name of a given expected confirmation reference code.
                     * @author manuel.arguelles
                     * @date   Mar 26, 2018
                     * @throws PartyNotFoundException If the party is not found.
                     * @param  reference The expected confirmation reference.
                     * @return The party name.
                     */
                    virtual std::string getPartyNameByExpectedConfirmationReference(const std::string& reference) const;

                    /**
                     * @brief  Gets the candidate name of a given expected confirmation reference code.
                     * @author manuel.arguelles
                     * @date   Mar 26, 2018
                     * @throws CandidateNotFoundException If the candidate is not found.
                     * @param  reference The expected confirmation reference.
                     * @return The candidate name.
                     */
                    virtual std::string getCandidateNameByExpectedConfirmationReference(const std::string& reference) const;

                private:
                    /**
                     * @brief	method for create eligible option map.
                     * @author	Juan Delgado
                     * @date	10/02/2012
                     * @throws  ElectionException     Thrown when an election error condition occurs.
                     * @throws  VotingDeviceException Thrown when a voting device error condition occurs.
                     */
                    void createMap();

                    /**
                     * @brief	calculate max.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @throws  ElectionException     Thrown when an election error condition occurs.
                     * @throws  VotingDeviceException Thrown when a voting device error condition occurs.
                     */
                    void calculateMax ();

                    /**
                     * @brief	Reduce election contests.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @throws  ElectionException     Thrown when an election error condition occurs.
                     * @throws  VotingDeviceException Thrown when a voting device error condition occurs.
                     */
                    void reduceElectionContests();

                    /**
                     * @brief	Reduce election contest groups.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @throws  ElectionException     Thrown when an election error condition occurs.
                     * @throws  VotingDeviceException Thrown when a voting device error condition occurs.
                     */
                    void reduceElectionContestGroups();

                    /**
                     * @brief	Verify integrity data.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @throws  ElectionException     Thrown when an election error condition occurs.
                     * @throws  VotingDeviceException Thrown when a voting device error condition occurs.
                     */
                    void verifyIntegrityData ();

                    /**
                     * @brief Sets the machine time from the information contained in the voting devices file.
                     * @author eduardo.fernandez
                     * @date   19/11/2018
                     */
                    void setMachineTime();

                    Smartmatic::SAES::Voting::Election::ElectionWrapper * election; ///< instance of class ElectionWrapper
                    Smartmatic::SAES::Voting::VotingDevice::VotingDeviceWrapper * votingDevice; ///< instance of class VotingDeviceWrapper
                    Smartmatic::SAES::Voting::Vote::VoteRecovery * voteRecovery; ///< instance of vote recovery object
                    Smartmatic::SAES::Voting::Vote::VoteMaxCalculate * voteMaxCalculate; ///< max calculate contest layout to print vote
                    Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories; ///< directories
                    static Smartmatic::Log::ISMTTLog * logger; ///<logger

                protected:

                    typedef Smartmatic::SAES::Voting::Election::Eligible_option typeEligibleOption; ///< type eligible option
                    typedef Smartmatic::SAES::Voting::Election::Ballot_option typeBallotOption; ///< type ballot option
                    typedef Smartmatic::SAES::Voting::Election::Contest_group typeContestGroup; ///< type contest group
                    typedef Smartmatic::SAES::Voting::Election::Contest typeContest; ///< type contest
                    typedef Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group typeLogicalVoterGroup; ///< type logical voter group
                    typedef Smartmatic::SAES::Voting::Election::Ballot typeBallot; ///< type ballot
                    typedef Smartmatic::SAES::Voting::Election::Party typeParty; ///< type party
                    typedef Smartmatic::SAES::Voting::Election::Contest_class typeContestClass; ///< type contest class

                    virtual std::map < std::string, std::map <std::string, unsigned int > > & getMapListBallotOptionByContestParty ( );

                    virtual std::map < std::string, std::map <std::string, unsigned int > > & getMapPrincipalBallotOptionByContestParty ( );

                    virtual std::map < std::string, std::map <std::string, unsigned int > > & getMapStandInBallotOptionByContestParty ( );

                    unsigned int maxPartiesByAllContest; ///< max parties by all contest
                    unsigned int maxContestByVote; ///<max contest of electoral configuration
                    unsigned int maxContestByBlankVote; ///<max contest of electoral configuration for blank vote
                    unsigned int maxBallotOptionByVote; ///<max ballot option of electoral configuration
                    unsigned int maxCustomAttributeByVote; ///< max custom attribute by vote
                    bool reload; ///< flag for not reload config files
                    bool loadData; ///< flag load data

                    std::map <std::string, std::map<std::string, typeEligibleOption * > > eligibleOptionByBallotOptionCodeMap;  ///< instance of eligible option by ballot option code map
                    std::map <std::string, std::map<std::string, typeEligibleOption * > > eligibleOptionByEligibleOptionCodeMap;  ///< instance of eligible option by eligible option code map

                    std::map < std::string, int > maxBallotOptionByContest; ///<max ballot option by contest
                    std::map < std::string, std::string > contestParty; ///< map contest party
                    std::list < std::string > listContestCode; ///< list contest code
                    std::list < typeContest * > listContest; ///< list contest
                    std::list < typeLogicalVoterGroup * > listLVG; ///< list logical voter group
                    std::list < typeContestGroup * > contestGroupList; ///< list contest group
                    std::list < typeBallot * >  ballotList; ///< list ballot
                    std::list < Smartmatic::SAES::Voting::VotingDevice::Region_category > regionCategoryList; ///< region category list

                    std::map < std::string, typeContestGroup * > mapContestGroupByCode; ///< map contest group by code
                    std::map < std::string, std::list < typeContest * > > mapListContestByContestGroup; ///< map list contest by contest group
                    std::map < std::string, typeContest * > mapContestCode; ///< map contest code
                    std::map < std::string, std::list < typeParty * > > mapContestParty; ///< map contest party
                    std::map < std::string, typeContestClass * > mapContestClass; ///< map contest class

                    std::map <std::string, std::list < typeBallotOption * > > mapBallotOptionsPartyByContest; ///< map ballot option party by contest

                    std::map < std::string, std::map <std::string, unsigned int > > maxListBallotOptionByContestParty; ///<max list ballot option by contest party
                    std::map < std::string, std::map <std::string, unsigned int > > maxPrincipalBallotOptionByContestParty; ///<max principal ballot option by contest party
                    std::map < std::string, std::map <std::string, unsigned int > > maxStandInBallotOptionByContestParty; ///<max stand in ballot option by contest party

                    std::map <std::string, std::map < std::string, std::list < typeBallotOption * > > > mapBallotOptionsListByContestParty; ///<map ballot option list by contest party
                    std::map <std::string, std::map < std::string, std::list < typeBallotOption * > > > mapBallotOptionsPrincipalByContestParty; ///<map ballot option principal by contest party
                    std::map <std::string, std::map < std::string, std::list < typeBallotOption * > > > mapBallotOptionsStandInByContestParty; ///<map ballot option stand in by contest party

                    std::map <std::string, std::map < std::string, std::list < typeEligibleOption * > > > mapEligibleOptionsListByContestParty; ///<map eligible option list by contest party
                    std::map <std::string, std::map < std::string, std::list < typeEligibleOption * > > > mapEligibleOptionsPrincipalByContestParty; ///<map eligible option principal by contest party
                    std::map <std::string, std::map < std::string, std::list < typeEligibleOption * > > > mapEligibleOptionsStandInByContestParty; ///<map eligible option stand in by contest party

                    std::map <std::string, std::list < typeBallotOption * > > mapEligibleOptionsListByContest; ///<max eligible option list by contest
                    std::map <std::string, std::list < typeBallotOption * > > mapEligibleOptionsPrincipalByContest; ///<max eligible option principal by contest
                    std::map <std::string, std::list < typeBallotOption * > > mapEligibleOptionsStandInByContest; ///<max eligible option stand in by contest

                    std::map <std::string, std::map < std::string, typeBallotOption * > > mapEligibleOptionsList; ///<max eligible option list
                    std::map <std::string, std::map < std::string, typeBallotOption * > > mapEligibleOptionsPrincipal; ///<max eligible option principal
                    std::map <std::string, std::map < std::string, typeBallotOption * > > mapEligibleOptionsStandIn; ///<max eligible option stand in

                    std::map <std::string, std::map < std::string, typeBallotOption * > > mapEligibleOptionsListByParty; ///<max eligible option list by party
                    std::map <std::string, std::map < std::string, typeBallotOption * > > mapEligibleOptionsPrincipalByParty; ///<max eligible option principal by party
                    std::map <std::string, std::map < std::string, typeBallotOption * > > mapEligibleOptionsStandInByParty; ///<max eligible option stand in by party

                    std::map <std::string, std::map < std::string, std::list <std::string> > > allCodeBallot; ///< all code ballot
                    std::map <std::string, std::string> partyCodeByBallotOptionCode; ///< parent code by ballot option code

                    std::map <std::string, typeParty * > mapPartyByCode; ///< map party by code

                    std::map <std::string, typeParty * >          mapPartyByCustomCode;
                    std::map <std::string, typeEligibleOption * > mapEligibleOptionByCustomCode;

                public:

                    virtual std::map < std::string, std::list < typeParty * > > & getMapContestParty();

            };
        }
    }
}

#endif /*__ELECTORAL_CONFIGURATION_H__*/
