/**
 * @file   EmlFileInfo.cxx
 * @brief  Implementation of EmlFileInfo functions.
 * @author manuel.arguelles
 * @date   Jan  5, 2018
 */

#include "EmlFileInfo.hxx"
#include "EmlInfo.hxx"
#include "EmlCodeType.hxx"
#include "EmlFileManager.hxx"
#include "EmlTransformation.hxx"
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Runtime/Environment/saes-directories.h>
#include <System/Exception/FileSystemException.hxx>
#include <System/XML/Xslt.hxx>
#include <memory>
#include <sstream>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::Eml;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::XML;

static ISMTTLog* logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Voting.Eml.EmlFileInfo",
    GETTEXT_PACKAGE);

/**
 * @brief  Get the type of file.
 * @author manuel.arguelles
 * @date   Jan  5, 2018
 * @param  electionType The type of the election.
 * @return The file type.
 */
static std::string getTypeFile(const std::string& electionType)
{
    std::string typeFile;
    std::map<std::string, std::string> emlToFileType;
    std::map<std::string, std::string>::const_iterator it;

    emlToFileType["CK"] = "DC";
    emlToFileType["SE"] = "DL";
    emlToFileType["EU"] = "DL";
    emlToFileType["VL"] = "DC";
    emlToFileType["WL"] = "DC";
    emlToFileType["BR"] = "DR";
    emlToFileType["DE"] = "DG";
    emlToFileType["PR"] = "DD";
    emlToFileType["CG"] = "DM";
    emlToFileType["DI"] = "DI";
    emlToFileType["CS"] = "DM";

    it = emlToFileType.find(electionType);

    if (it != emlToFileType.end())
    {
        typeFile = it->second;
    }
    else
    {
        logger->Debug("Error - Unknown election type: " + electionType);
    }

    return typeFile;
}

std::string EmlFileInfo::getInputContestFileName(const std::string& customCode)
{
    std::string fileName;
    std::string typeFile;
    std::string electionType = EmlInfo::getElectionType(customCode);
    std::string nisCode      = EmlInfo::getInputNisCode(customCode);

    typeFile = getTypeFile(electionType);
    fileName = typeFile + nisCode + "_" + electionType + ".EML";

    return fileName;
}

std::string EmlFileInfo::getOutputContestFileName(
    const std::string& customCode,
    const std::string& machine,
    const std::string& votingType,
    const std::string& customNisCode)
{
    std::string outFilename;

    try
    {
        std::string electionType = EmlInfo::getElectionType(customCode);
        std::string nisCode = EmlInfo::getOutputNisCode(customCode, customNisCode);
        std::string file = SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getFirstStringFileName();

        outFilename = file + machine + nisCode + "_" + electionType + ".EML";
    }
    catch (...)
    {
        logger->Debug("Fatal - Error in getOutputContestFileName");
    }

    return outFilename;
}

EML EmlFileInfo::loadEmlFromFile(const std::string& filename)
{
    EML eml;

    try
    {
        std::istringstream emlStr(Xslt::transformFile(filename, EML_TRANSFORMATION));
        std::auto_ptr<EML> eml510(parseEML(emlStr, ::xml_schema::Flags::dont_validate));
        EML final(*eml510.get());

        eml = final;
    }
    catch (xsd::cxx::tree::parsing<char>& exception)
    {
        logger->Debug("Failed to parse EML file: " + filename + ": " + exception.what());
        throw FileSystemException("Error opening EML file", "Error opening EML file", logger);
    }
    catch (...)
    {
        logger->Debug("Failed to parse EML file: " + filename);
        throw FileSystemException("Error opening EML file", "Error opening EML file", logger);
    }

    return eml;
}
