/**
 * @file   EmlInfo.cxx
 * @brief  Implementation of EML information functions.
 * @author manuel.arguelles
 * @date   Jan  5, 2018
 */

#include "EmlInfo.hxx"
#include <System/Utils/StringUtils.hxx>
#include <map>

static const std::string::size_type CUSTOM_CODE_SIZE    = 8;
static const std::string::size_type ELECTION_TYPE_INDEX = 0;
static const std::string::size_type ELECTION_TYPE_SIZE  = 2;
static const std::string::size_type LANGUAGE_INDEX      = 7;
static const std::string::size_type LANGUAGE_SIZE       = 1;
static const std::string::size_type NIS_CODE_INDEX      = 2;
static const std::string::size_type NIS_CODE_SIZE       = 5;

using namespace Smartmatic::SAES::Voting::Eml;
using namespace Smartmatic::System::Utils;

/**
 * @brief  Gets a map of the election type, code and description in the main language.
 * @author manuel.arguelles
 * @date   Feb 28, 2018
 * @return A map of the election type, id and description.
 */
static std::map<std::string, std::pair<std::string, std::string> > getMainLangElectionTypeMap()
{
    std::map<std::string, std::pair<std::string, std::string> > electionTypeMap;

    electionTypeMap["EU"] = std::pair<std::string, std::string>("1", "Parlement européen");
    electionTypeMap["CK"] = std::pair<std::string, std::string>("2", "Chambre");
    electionTypeMap["SE"] = std::pair<std::string, std::string>("3", "Sénat");
    electionTypeMap["WL"] = std::pair<std::string, std::string>("4", "Parlement wallon");
    electionTypeMap["PR"] = std::pair<std::string, std::string>("5", "Conseils provinciaux");
    electionTypeMap["CG"] = std::pair<std::string, std::string>("6", "Conseils communaux");
    electionTypeMap["VL"] = std::pair<std::string, std::string>("7", "Parlement flamand");
    electionTypeMap["DI"] = std::pair<std::string, std::string>("8", "Conseil de district");
    electionTypeMap["CS"] = std::pair<std::string, std::string>("9", "Conseil CPAS");
    electionTypeMap["BR"] = std::pair<std::string, std::string>("10", "Parlement de la Région de Bruxelles-Capitale");
    electionTypeMap["DE"] = std::pair<std::string, std::string>("12", "Parlement de la Communauté germanophone");

    return electionTypeMap;
}

/**
 * @brief  Gets a map of the election type, code and description in the secondary language.
 * @author manuel.arguelles
 * @date   Feb 28, 2018
 * @return A map of the election type, id and description.
 */
static std::map<std::string, std::pair<std::string, std::string> > getSecondLangElectionTypeMap()
{
    std::map<std::string, std::pair<std::string, std::string> > electionTypeMap;

    electionTypeMap["EU"] = std::pair<std::string, std::string>("1", "Europees Parlement");
    electionTypeMap["CK"] = std::pair<std::string, std::string>("2", "Kamer");
    electionTypeMap["SE"] = std::pair<std::string, std::string>("3", "Senaat");
    electionTypeMap["WL"] = std::pair<std::string, std::string>("4", "Waals Parlement");
    electionTypeMap["PR"] = std::pair<std::string, std::string>("5", "Provincies");
    electionTypeMap["CG"] = std::pair<std::string, std::string>("6", "Gemeenten");
    electionTypeMap["VL"] = std::pair<std::string, std::string>("7", "Vlaams Parlement");
    electionTypeMap["DI"] = std::pair<std::string, std::string>("8", "Districtsraad");
    electionTypeMap["CS"] = std::pair<std::string, std::string>("9", "OCMW");
    electionTypeMap["BR"] = std::pair<std::string, std::string>("10", "Brussels Hoofdstedelijk Parlement");
    electionTypeMap["DE"] = std::pair<std::string, std::string>("12", "Parlement van de Duitstalige Gemeenschap");

    return electionTypeMap;
}


std::string EmlInfo::getElectionType(const std::string& customCode)
{
    std::string electionType;

    if (customCode.size() == CUSTOM_CODE_SIZE)
    {
        electionType = customCode.substr(ELECTION_TYPE_INDEX, ELECTION_TYPE_SIZE);
    }

    return electionType;
}

std::string EmlInfo::getInputNisCode(const std::string& customCode)
{
    std::string nisCode;

    if (customCode.size() == CUSTOM_CODE_SIZE)
    {
        nisCode = customCode.substr(NIS_CODE_INDEX, NIS_CODE_SIZE);
    }

    return nisCode;
}

std::string EmlInfo::getOutputNisCode(const std::string& customCode,
                                      const std::string& customNisCode)
{
    std::string nisCode = customNisCode;

    if (nisCode.empty() && customCode.size() == CUSTOM_CODE_SIZE)
    {
        nisCode = customCode.substr(NIS_CODE_INDEX, NIS_CODE_SIZE);
    }

    return nisCode;
}

std::string EmlInfo::getLanguage(const std::string& customCode)
{
    std::string language;

    if (customCode.size() == CUSTOM_CODE_SIZE)
    {
        language = customCode.substr(LANGUAGE_INDEX, LANGUAGE_SIZE);
    }

    return language;
}

std::string EmlInfo::getTotalVotesSelectionCategory(const std::string& customCode)
{
    std::string selectionCategory;
    std::string nisCode = getInputNisCode(customCode);
    std::string languageCode = getLanguage(customCode);

    selectionCategory = nisCode + languageCode;

    return selectionCategory;
}

int EmlInfo::getLinguisticRegime(const std::string& customCode)
{
    std::string language = getLanguage(customCode);

    return StringUtils::stringToInt(language);
}

std::string EmlInfo::getEventQualifier(const std::string& emlFilename)
{
    std::string::size_type underscorePosition = emlFilename.find("_");
    std::string eventQualifier;

    if (underscorePosition != std::string::npos)
    {
        eventQualifier = emlFilename.substr(0, underscorePosition);
    }

    return eventQualifier;
}

std::string EmlInfo::getElectionIdentifierId(const std::string& customCode)
{
    std::string id;
    std::string type = getElectionType(customCode);

    std::map<std::string, std::pair<std::string, std::string> > electionType;

    if (getLinguisticRegime(customCode) == 2)
    {
        electionType = getSecondLangElectionTypeMap();
    }
    else
    {
        electionType = getMainLangElectionTypeMap();
    }

    id = electionType[type].first;

    return id;
}

std::string EmlInfo::getElectionIdentifierName(const std::string &customCode)
{
    std::string name;
    std::string type = getElectionType(customCode);

    std::map<std::string, std::pair<std::string, std::string> > electionType;

    if (getLinguisticRegime(customCode) == 2)
    {
        electionType = getSecondLangElectionTypeMap();
    }
    else
    {
        electionType = getMainLangElectionTypeMap();
    }

    name = electionType[type].second;

    return name;

}

std::string EmlInfo::getOutputNisCode(const std::string& outputFilename)
{
    std::string nisCode;
    std::string::size_type underscorePosition = outputFilename.find("_");

    if (underscorePosition != std::string::npos && underscorePosition > NIS_CODE_SIZE)
    {
        nisCode = outputFilename.substr(underscorePosition - NIS_CODE_SIZE, NIS_CODE_SIZE);
    }

    return nisCode;
}
