/**
 * @file   EmlWrapper.cxx
 * @brief  Implementation of EmlWrapper class.
 * @author manuel.arguelles
 * @date   Jan 11, 2018
 */

#include "EmlWrapper.hxx"
#include "EmlInfo.hxx"
#include <Exceptions/Voting/Eml/EmlChecksumException.hxx>
#include <Exceptions/Voting/Eml/EmlElementNotFoundException.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <fstream>
#include <xercesc/dom/DOM.hpp>
#include <vector>
#include <glibmm.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

static const std::size_t REFERENCE_CANDIDATE_ID_POSITION   = 14;
static const std::size_t REFERENCE_CANDIDATE_ID_SIZE       = 2;
static const std::size_t REFERENCE_CANDIDATE_TYPE_POSITION = 13;
static const std::size_t REFERENCE_CANDIDATE_TYPE_SIZE     = 1;
static const std::size_t REFERENCE_PARTY_ID_POSITION       = 11;
static const std::size_t REFERENCE_PARTY_ID_SIZE           = 2;
static const std::size_t REFERENCE_PARTY_NIS_POSITION      = 6;
static const std::size_t REFERENCE_PARTY_NIS_SIZE          = 5;

static const std::string DATE_FORMAT = "%Y-%m-%d";

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Exceptions::Voting::Eml;
using namespace Smartmatic::SAES::Voting::Eml;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Utils;

static ISMTTLog* logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Voting.Eml.EmlWrapper",
    GETTEXT_PACKAGE);

static long getPartyNisFromExpectedConfirmationReference(const std::string& reference)
{
    std::string nis = reference.substr(REFERENCE_PARTY_NIS_POSITION, REFERENCE_PARTY_NIS_SIZE);

    return StringUtils::stringToLong(nis);
}

/**
 * @brief  Gets the party id from the expected confirmation reference.
 * @author manuel.arguelles
 * @date   Mar 19, 2018
 * @param  reference The expected confirmation reference.
 * @return The id of the party.
 */
static long getPartyIdFromExpectedConfirmationReference(const std::string& reference)
{
    std::string id = reference.substr(REFERENCE_PARTY_ID_POSITION, REFERENCE_PARTY_ID_SIZE);

    return StringUtils::stringToLong(id);
}

/**
 * @brief  Gets the candidate id from the expected confirmation reference.
 * @author manuel.arguelles
 * @date   Mar 19, 2018
 * @param  reference The expected confirmation reference.
 * @return The id of the candidate.
 */
static long getCandidateIdFromExpectedConfirmationReference(const std::string& reference)
{
    std::string id = reference.substr(REFERENCE_CANDIDATE_ID_POSITION, REFERENCE_CANDIDATE_ID_SIZE);

    return StringUtils::stringToLong(id);;
}

/**
 * @brief  Gets the candidate type from the expected confirmation reference.
 * @author manuel.arguelles
 * @date   Mar 19, 2018
 * @param  reference The expected confirmation reference.
 * @return The type of the candidate (E or S).
 */
static std::string getCandidateTypeFromExpectedConfirmationReference(const std::string& reference)
{
    return reference.substr(REFERENCE_CANDIDATE_TYPE_POSITION, REFERENCE_CANDIDATE_TYPE_SIZE);
}

/**
 * @brief  Gets the Selection category of TotalVotes
 * @author manuel.arguelles
 * @date   Jan 12, 2018
 * @param  contest  The contest.
 * @param  category The category of the selection.
 * @return The Selection of the given category.
 * @throws EmlElementNotFoundException if the selection of the given category is not present.
 */
static Selection& getTotalVotesSelection(Contest& contest, const std::string& category)
{
    if (contest.getTotalVotes().present())
    {
        TotalVotes::SelectionSequence& selections = contest.getTotalVotes().get().getSelection();
        TotalVotes::SelectionIterator selectionIt;

        for (selectionIt = selections.begin(); selectionIt != selections.end(); ++selectionIt)
        {
            if (selectionIt->getCategory().present() &&
                selectionIt->getCategory().get() == category)
            {
                return *selectionIt;
            }
        }
    }

    throw EmlElementNotFoundException("TotalVotes selection not found for category " + category,
                                      "TotalVotes selection not found");
}

/**
 * @brief  Gets the Selection for the ReportingUnitVotes element of a party.
 * @author manuel.arguelles
 * @date   Jan 15, 2018
 * @param  contest               The contest of the party.
 * @param  confirmationReference The id code of the party.
 * @return A reference of the Selection object of the given party.
 * @throws EmlElementNotFoundException if the selection of the given party is
 *  not present.
 */
static Selection& getPartyReportingUnitVotesSelection(Contest& contest,
                                                      const std::string& confirmationReference)
{
    Contest::ReportingUnitVotesSequence& reportingUnitVotes = contest.getReportingUnitVotes();
    Contest::ReportingUnitVotesIterator it;

    for (it = reportingUnitVotes.begin(); it != reportingUnitVotes.end(); ++it)
    {
        ReportingUnitVotes::SelectionSequence& selections = it->getSelection();
        ReportingUnitVotes::SelectionIterator sIt;

        for (sIt = selections.begin(); sIt != selections.end(); ++sIt)
        {
            if (sIt->getAffiliationIdentifier().present() &&
                sIt->getAffiliationIdentifier().get().getExpectedConfirmationReference().present() &&
                sIt->getAffiliationIdentifier().get().getExpectedConfirmationReference().get() == confirmationReference)
            {
                return *sIt;
            }
        }
    }

   throw EmlElementNotFoundException("ReportingUnitVotes selection not found for " + confirmationReference,
                                     "ReportingUnitVotes selection not found");
}

/**
 * @brief  Gets the AffiliationIdentifier of a ReportingUnitVotes node.
 * @author manuel.arguelles
 * @date   Feb 11, 2018
 * @param  reportingUnitVotes The reporting unit votes node.
 * @return The affiliation Identifier of the node.
 * @throws EmlElementNotFoundException If the EML file is malformed.
 */
static const AffiliationIdentifierStructure& getAffiliationIdentifierOfReportingUnitVotes(
    const Contest::ReportingUnitVotesType& reportingUnitVotes)
{
    const ReportingUnitVotes::SelectionSequence& selections = reportingUnitVotes.getSelection();
    ReportingUnitVotes::SelectionConstIterator it;

    for (it = selections.begin(); it != selections.end(); ++it)
    {
        if (it->getAffiliationIdentifier().present())
        {
            return it->getAffiliationIdentifier().get();
        }
    }

    throw EmlElementNotFoundException("Affiliation Identifier selection not found",
                                      "Affiliation Identifier not found in ReportingUnitVotes");
}

/**
 * @brief  Gets the Selection having an AffiliationIdentifier element from a ReportingUnitVotes.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  reportingUnitVotes The reporting unit votes element.
 * @return The Selection element having an AffiliationIdentifier element.
 */
static Selection& getPartyReportingUnitVotesSelection(ReportingUnitVotes& reportingUnitVotes)
{
    ReportingUnitVotes::SelectionSequence& selections = reportingUnitVotes.getSelection();
    ReportingUnitVotes::SelectionIterator it;

    for (it = selections.begin(); it != selections.end(); ++it)
    {
        if (it->getAffiliationIdentifier().present())
        {
            return *it;
        }
    }

    throw EmlElementNotFoundException("Affiliation Identifier selection not found",
                                      "Affiliation Identifier selection not found");
}

/**
 * @brief  Increments the count metric of the reporting unit votes by type.
 * @author manuel.arguelles
 * @date   Jan 15, 2018
 * @param  type The type of the CountMetric attribute.
 * @return True if the type was found and increased.
 */
static bool increaseReportingUnitVotesMetricByType(Selection& selection, const std::string& type)
{
    ReportingUnitVotes::CountMetricSequence& metrics = selection.getCountMetric();
    ReportingUnitVotes::CountMetricIterator it;
    bool found = false;

    for (it = metrics.begin(); it != metrics.end(); ++it)
    {
        if (it->getType() == type)
        {
            ++(*it);
            found = true;
        }
    }

    return found;
}

/**
 * @brief  Gets the Selection of the ReportingUnitVotes element for a candidate.
 * @author manuel.arguelles
 * @date   Jan 15, 2018
 * @param  contest  The contest of the candidate.
 * @param  idNumber The id code of the candidate.
 * @return A reference of the Selection object of the given candidate.
 * @throws EmlElementNotFoundException if the candidate is not present in the
 * given contest.
 */
static Selection& getCandidateReportingUnitVotesSelection(Contest& contest,
                                                          const std::string& confirmationReference)
{
    Contest::ReportingUnitVotesSequence& reportingUnitVotes = contest.getReportingUnitVotes();
    Contest::ReportingUnitVotesIterator it;

    for (it = reportingUnitVotes.begin(); it != reportingUnitVotes.end(); ++it)
    {
        ReportingUnitVotes::SelectionSequence& selections = it->getSelection();
        ReportingUnitVotes::SelectionIterator sIt;

        for (sIt = selections.begin(); sIt != selections.end(); ++sIt)
        {
            if (sIt->getCandidate().present() &&
                sIt->getCandidate().get().getCandidateIdentifier().present() &&
                sIt->getCandidate().get().getCandidateIdentifier().get().getExpectedConfirmationReference().present() &&
                sIt->getCandidate().get().getCandidateIdentifier().get().getExpectedConfirmationReference().get() == confirmationReference)
            {
                return *sIt;
            }
        }
    }

    throw EmlElementNotFoundException("ReportingUnitVotes selection not found for " + confirmationReference,
                                      "ReportingUnitVotes selection not found");
}

/**
 * @brief  Increase the total votes deposit for a contest.
 * @author manuel.arguelles
 * @date   Jan 15, 2018
 * @param  contest  The contest of the vote.
 * @param  category The category code of the contest's selection (should be the
 * nis code).
 * @return True if a VotesDeposited entry was found for the given contest and
 * category.
 */
static bool increaseTotalVotesDeposit(Contest& contest, const std::string& category)
{
    Selection& selection = getTotalVotesSelection(contest, category);
    bool found = false;
    TotalVotes::CountMetricSequence& metrics = selection.getCountMetric();
    TotalVotes::CountMetricIterator  metricIt;

    for (metricIt = metrics.begin(); metricIt != metrics.end(); ++metricIt)
    {
        if (metricIt->getType() == "VotesDeposited")
        {
            ++(*metricIt);
            found = true;
        }
    }

    return found;
}

/**
 * @brief  Increase the total valid votes of a category contest.
 * @author manuel.arguelles
 * @date   Jan 18, 2018
 * @param  contest  The contest of the vote.
 * @param  category The category code of the contest to increase.
 */
static void increaseTotalValidVotes(Contest& contest, const std::string& category)
{
    Selection& selection = getTotalVotesSelection(contest, category);
    Selection& globalSelection = getTotalVotesSelection(contest, "GlobalSituation");

    ++selection.getValidVotes();
    ++globalSelection.getValidVotes();
}

/**
 * @brief  Sum all the values of a CountMetric of the given type, excluding the
 * GlobalSituation type.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  selection The TotalVotes::Selection sequence.
 * @param  type      The type of the CountMetric to sum.
 * @return The sum of all the given CountMetric's type.
 */
static unsigned long sumTotalVotesSequenceByType(const TotalVotes::SelectionSequence& selection,
                                                 const std::string& type)
{
    unsigned long total = 0;
    TotalVotes::SelectionConstIterator it;

    for (it = selection.begin(); it != selection.end(); ++it)
    {
        if (it->getCategory().present() && it->getCategory().get() != "GlobalSituation")
        {
            TotalVotes::CountMetricConstIterator mIt = it->getCountMetric().begin();

            while (mIt != it->getCountMetric().end())
            {
                if (mIt->getType() == type)
                {
                    total += *mIt;
                }

                ++mIt;
            }
        }
    }

    return total;
}

/**
 * @brief  Get the total votes deposited value.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  selection The TotalVotes' selection sequence to sum.
 * @return The amount of votes deposited within the selection.
 */
static unsigned long getTotalVotesDeposited(const TotalVotes::SelectionSequence& selection)
{
    return sumTotalVotesSequenceByType(selection, "VotesDeposited");
}

/**
 * @brief  Get the total blank or invalid votes value.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  selection The TotalVotes' selection sequence to sum.
 * @return The amount of blank or invalid votes within the selection.
 */
static unsigned long getTotalVotesBlankAndInvalid(const TotalVotes::SelectionSequence& selection)
{
    return sumTotalVotesSequenceByType(selection, "BlankAndInvalidVotes");
}

/**
 * @brief  Updates the GlobalSituation valid votes.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  selections The TotalVotes::Selection sequence to update.
 */
static void updateValidVotesGlobalSituation(TotalVotes::SelectionSequence& selections)
{
    long totalValidVotes = 0;
    TotalVotes::SelectionIterator globalSituationEntry = selections.end();
    TotalVotes::SelectionIterator it;

    for (it = selections.begin(); it != selections.end(); ++it)
    {
        if (it->getCategory().present() && it->getCategory().get() == "GlobalSituation")
        {
            globalSituationEntry = it;
        }
        else
        {
            totalValidVotes += it->getValidVotes();
        }
    }

    if (globalSituationEntry != selections.end())
    {
        globalSituationEntry->setValidVotes(totalValidVotes);
    }
}

/**
 * @brief  Updates all the CountMetrics of a given type, except the GlobalSituation.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  selections The sequence of TotalVotes::Selection to update.
 * @param  metricType The type of the metric to update.
 * @param  value      The new value of the metric.
 */
static void updateAllTotalVotesMetrics(TotalVotes::SelectionSequence& selections,
                                       const std::string& metricType,
                                       const long value)
{
    TotalVotes::SelectionIterator it;

    for (it = selections.begin(); it != selections.end(); ++it)
    {
        if (it->getCategory().present() && it->getCategory().get() != "GlobalSituation")
        {
            TotalVotes::CountMetricIterator mIt = it->getCountMetric().begin();

            while (mIt != it->getCountMetric().end())
            {
                if (mIt->getType() == metricType)
                {
                    *mIt += (value - *mIt);
                }

                ++mIt;
            }
        }
    }
}

/**
 * @brief  Merge a TotalVotes's selection sequence into another.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  target The sequence to merge with.
 * @param  extra  The sequence to add to target sequence.
 */
static void mergeTotalVotesSelections(TotalVotes::SelectionSequence& target,
                                      const TotalVotes::SelectionSequence& extra)
{
    TotalVotes::SelectionConstIterator it;
    unsigned long totalInvalid   = 0;
    unsigned long totalDeposited = 0;

    totalInvalid = getTotalVotesBlankAndInvalid(target) + getTotalVotesBlankAndInvalid(extra);
    totalDeposited = getTotalVotesDeposited(target) + getTotalVotesDeposited(extra);

    for (it = extra.begin(); it != extra.end(); ++it)
    {
        if (it->getCategory().present() && it->getCategory().get() != "GlobalSituation")
        {
            target.push_back(*it);
        }
    }

    updateAllTotalVotesMetrics(target, "VotesDeposited", totalDeposited);
    updateAllTotalVotesMetrics(target, "BlankAndInvalidVotes", totalInvalid);
}

/**
 * @brief  Gets the CountMetric of a given type.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  selection The selection of ReportingUnitVotes.
 * @param  type      The type of the CountMetric to search.
 * @return The CountMetric of the given type.
 * @throws EmlElementNotFoundException if a CountMetric with the given type is
 * not found.
 */
static ReportingUnitVotes::CountMetricType& getCountMetricByType(Selection& selection, const std::string& type)
{
    ReportingUnitVotes::CountMetricSequence& metrics = selection.getCountMetric();
    ReportingUnitVotes::CountMetricIterator it;

    for (it = metrics.begin(); it != metrics.end(); ++it)
    {
        if (it->getType() == type)
        {
            return *it;
        }
    }

    throw EmlElementNotFoundException("CountMetric selection " + type + " not found",
                                      "CountMetric selection not found");
}

/**
 * @brief  Gets the CountMetric of a given type.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  reportingUnitVotes The ReportingUnitVotes with the CountMetrics to find.
 * @param  type               The type of the CountMetric to search.
 * @return The CountMetric of the given type.
 * @throws EmlElementNotFoundException if a CountMetric with the given type is
 * not found.
 */
static ReportingUnitVotes::CountMetricType& getCountMetricByType(ReportingUnitVotes& reportingUnitVotes,
                                                                 const std::string& type)
{
    ReportingUnitVotes::CountMetricSequence& metrics = reportingUnitVotes.getCountMetric();
    ReportingUnitVotes::CountMetricIterator it;

    for (it = metrics.begin(); it != metrics.end(); ++it)
    {
        if (it->getType() == type)
        {
            return *it;
        }
    }

    throw EmlElementNotFoundException("CountMetric selection " + type + " not found",
                                      "CountMetric selection not found");
}

/**
 * @brief  Gets the ElectionParam element for a given linguistic regime and type.
 * @author manuel.arguelles
 * @date   Feb 21, 2018
 * @param  selection        The Selection with the ElectionParam to find.
 * @param  linguisticRegime The linguistic regime of the required ElectionParam.
 * @param  type             The type of the ElectionParam.
 * @return The ElectionParam found.
 * @throws EmlElementNotFoundException if the ElectionParam element was not found.
 */
static Selection::ElectionParamType& getElectionParamByType(Selection& selection,
                                                            const long linguisticRegime,
                                                            const std::string& type)
{
    Selection::ElectionParamSequence& electionParams = selection.getElectionParam();
    Selection::ElectionParamIterator it;
    std::string regime = StringUtils::longToString(linguisticRegime);

    for (it = electionParams.begin(); it != electionParams.end(); ++it)
    {
        if (it->getGrp_linguistic().present() && it->getGrp_linguistic().get() == regime &&
            it->getType().present() && it->getType().get() == type)
        {
            return *it;
        }
    }

    throw EmlElementNotFoundException("ElectionParam " + type + " not found",
                                      "ElectionParam not found");
}

/**
 * @brief  Removes all the ElectionParam elements of a given selection.
 * @author manuel.arguelles
 * @date   Feb 21, 2018
 * @param  selection The selection which ElectionParams will be removed.
 */
static void removeAllElectionParam(Selection& selection)
{
    Selection::ElectionParamSequence& electionParams = selection.getElectionParam();

    electionParams.clear();
}

static void addSeatsToBeFilledFromElectionParam(Selection& selection,
                                                const long linguisticRegime)
{
    try
    {
        Selection::ElectionParamType& param = getElectionParamByType(selection,
                                                                     linguisticRegime,
                                                                     "SeatToBeFilled");
        ReportingUnitVotes::CountMetricType countMetric(
            StringUtils::stringToLong(param),
            "SeatsToBeFilled");

        countMetric.setId("S4");

        selection.getCountMetric().push_back(countMetric);
    }
    catch (const EmlElementNotFoundException& exception)
    {
        logger->Debug("SeatToBeFilled not found");
    }
}

static void addSubstitutesMaxFromElectionParam(Selection& selection,
                                               const long linguisticRegime)
{
    try
    {
        Selection::ElectionParamType& param = getElectionParamByType(selection,
                                                                     linguisticRegime,
                                                                     "SubstitutesMax");
        ReportingUnitVotes::CountMetricType countMetric(
            StringUtils::stringToLong(param),
            "SubstitutesMax");

        countMetric.setId("S5");

        selection.getCountMetric().push_back(countMetric);
    }
    catch (const EmlElementNotFoundException& exception)
    {
        logger->Debug("SubstitutesMax not found");
    }
}

static void addLinguisticRegimeFromElectionParam(Selection& selection,
                                                 const long linguisticRegime)
{
    try
    {
        Selection::ElectionParamType& param = getElectionParamByType(selection,
                                                                     linguisticRegime,
                                                                     "LinguisticRegime");
        ReportingUnitVotes::CountMetricType countMetric(
            StringUtils::stringToLong(param),
            "LinguisticRegime");

        countMetric.setId("S9");

        selection.getCountMetric().push_back(countMetric);

    }
    catch (const EmlElementNotFoundException& exception)
    {
        logger->Debug("LinguisticRegime not found");
    }
}

/**
 * @brief  Adds the LevelDepositList value to the category name.
 * @author manuel.arguelles
 * @date   Feb 12, 2018
 * @param  selections The TotalVotes::SelectionSequence having the categories to update.
 */
static void addLevelDepositListToCategory(TotalVotes::SelectionSequence& selections)
{
    TotalVotes::SelectionIterator it;

    for (it = selections.begin(); it != selections.end(); ++it)
    {
        if (it->getCategory().present() && it->getCategory().get() != "GlobalSituation")
        {
            try
            {
                TotalVotes::CountMetricType& metric = getCountMetricByType(*it, "LevelDepositList");

                if (metric.getId().present())
                {
                    std::string letter = metric.getId().get();
                    it->getCategory().get() = letter + it->getCategory().get();
                }
            }
            catch (const EmlElementNotFoundException& exception)
            {
                logger->Debug("ERROR: Total votes selection without LevelDepositList element");
            }
        }
    }
}

/**
 * @brief  Adds the polling station id to the global situation node.
 * @author manuel.arguelles
 * @date   Apr 27, 2018
 * @param  selections       The TotalVotes selection sequence.
 * @param  pollingStationId The value of the polling station id.
 */
static void addPollingStationIdToGlobalSituation(TotalVotes::SelectionSequence& selections, const long pollingStationId)
{
    TotalVotes::SelectionIterator it;

    for (it = selections.begin(); it != selections.end(); ++it)
    {
        if (it->getCategory().present() && it->getCategory().get() == "GlobalSituation")
        {
            try
            {
                TotalVotes::CountMetricType& metric = getCountMetricByType(*it, "PollingStationId");

                metric += (pollingStationId - metric);
            }
            catch (const EmlElementNotFoundException& exception)
            {
                logger->Debug("ERROR: PollingStationId not found");
            }
        }
    }
}

/**
 * @brief  Calculates the checksum
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  input The vector with input values.
 * @return The checksum.
 */
static long calculateChecksum(const std::vector<long>& input)
{
    long checksum = 0;
    std::vector<long>::const_iterator it;

    for (it = input.begin(); it != input.end(); ++it)
    {
        if (it == input.begin())
        {
            checksum = *it;
        }
        else
        {
            if (*it == 0)
            {
                checksum += 9L;
            }
            else
            {
                checksum = ((checksum * *it) + 1) % 99991L;
            }
        }
    }

    return checksum;
}

/**
 * @brief  Calculates the party checksum.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  unitVotes The ReportingUnitVotes with the parties to calculate.
 */
static void calculatePartyChecksum(ReportingUnitVotes& unitVotes)
{
    std::vector<long> inputData;
    Selection& party = getPartyReportingUnitVotesSelection(unitVotes);
    ReportingUnitVotes::SelectionSequence& candidates = unitVotes.getSelection();
    ReportingUnitVotes::SelectionIterator it;

    inputData.push_back(getCountMetricByType(party, "NumberOfHeaderVotes"));
    inputData.push_back(getCountMetricByType(party, "NumberOfBallotsWithOnlyTitularVotes"));
    inputData.push_back(getCountMetricByType(party, "NumberOfBallotsWithOnlySubstitutesVotes"));
    inputData.push_back(getCountMetricByType(party, "NumberOfBallotsWithTitularAndSubstitutesVotes"));

    for (it = candidates.begin(); it != candidates.end(); ++it)
    {
        if (it->getCandidate().present())
        {
            inputData.push_back(it->getValidVotes());
        }
    }

    getCountMetricByType(party, "CheckSum") -= getCountMetricByType(party, "CheckSum");
    getCountMetricByType(party, "CheckSum") += calculateChecksum(inputData);
}

/**
 * @brief  Calculates the global checksum of a contest.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  contest The contest to calculate the checksum.
 */
static long calculateGlobalChecksum(Contest& contest)
{
    std::vector<long> inputData;
    std::vector<long> candidatesInputData;
    std::vector<long> partyInputData;
    long votesValid        = 0;
    long votesDeposited    = 0;
    long votesBlankInvalid = 0;
    long checksum          = 0;
    TotalVotes::SelectionIterator globalSituationIt;

    if (contest.getTotalVotes().present())
    {
        TotalVotes::SelectionSequence& sequence = contest.getTotalVotes().get().getSelection();
        TotalVotes::SelectionIterator it;

        for (it = sequence.begin(); it != sequence.end(); ++it)
        {
            if (it->getCategory().present() && it->getCategory().get() == "GlobalSituation")
            {
                votesValid = it->getValidVotes();
                globalSituationIt = it;
            }
            else
            {
                TotalVotes::CountMetricSequence& metrics = it->getCountMetric();
                TotalVotes::CountMetricIterator mIt;

                for (mIt = metrics.begin(); mIt != metrics.end(); ++mIt)
                {
                    if (mIt->getType() == "VotesDeposited")
                    {
                        votesDeposited = *mIt;
                    }

                    if (mIt->getType() == "BlankAndInvalidVotes")
                    {
                        votesBlankInvalid += *mIt;
                    }
                }
            }
        }
    }

    inputData.push_back(votesValid);
    inputData.push_back(votesDeposited);
    inputData.push_back(votesBlankInvalid);

    Contest::ReportingUnitVotesSequence& unitVotes = contest.getReportingUnitVotes();
    Contest::ReportingUnitVotesIterator it;

    for (it = unitVotes.begin(); it != unitVotes.end(); ++it)
    {
        ReportingUnitVotes::SelectionSequence& selections = it->getSelection();

        if (selections.size() > 1)
        {
            if (selections[0].getAffiliationIdentifier().present())
            {
                partyInputData.push_back(getCountMetricByType(selections[0], "NumberOfHeaderVotes"));
                partyInputData.push_back(getCountMetricByType(selections[0], "NumberOfBallotsWithOnlyTitularVotes"));
                partyInputData.push_back(getCountMetricByType(selections[0], "NumberOfBallotsWithOnlySubstitutesVotes"));
                partyInputData.push_back(getCountMetricByType(selections[0], "NumberOfBallotsWithTitularAndSubstitutesVotes"));
            }
            else
            {
                logger->Debug("ERROR: EML is not ordered");
                throw EmlChecksumException("Error in Reporting Unit Votes Selection order",
                                           "Checksum error");
            }

            for (size_t i = 1; i < selections.size(); ++i)
            {
                if (selections[i].getCandidate().present())
                {
                    candidatesInputData.push_back(selections[i].getValidVotes());
                }
                else
                {
                    logger->Debug("ERROR: EML is not ordered");
                    throw EmlChecksumException("Error in Candidate Selection order",
                                               "Checksum error");
                }
            }
        }
        else
        {
            logger->Debug("ERROR: EML file is empty");
            throw EmlChecksumException("Error in amount of Reporting Unit Votes Selection",
                                       "Checksum error");
        }
    }

    inputData.insert(inputData.end(), partyInputData.begin(), partyInputData.end());
    inputData.insert(inputData.end(), candidatesInputData.begin(), candidatesInputData.end());

    checksum = calculateChecksum(inputData);

    TotalVotes::CountMetricSequence& metrics = globalSituationIt->getCountMetric();
    TotalVotes::CountMetricIterator mIt;

    for (mIt = metrics.begin(); mIt != metrics.end(); ++mIt)
    {
        if (mIt->getType() == "CheckSum")
        {
            long currentValue = *mIt;

            *mIt -= currentValue;
            *mIt += checksum;
        }
    }

    return checksum;
}

/**
 * @brief  Compare two ReportingUnitVotes selections.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  a The first selection.
 * @param  b The second selection.
 * @return True if a is less than b, false otherwise.
 */
static bool compareSelections(const ReportingUnitVotes::SelectionType& a, const ReportingUnitVotes::SelectionType& b)
{
    if (a.getAffiliationIdentifier().present())
    {
        return true;
    }
    else if (b.getAffiliationIdentifier().present())
    {
        return false;
    }
    else if (a.getCandidate().present() && b.getCandidate().present() &&
             a.getCandidate().get().getCandidateIdentifier().present() &&
             b.getCandidate().get().getCandidateIdentifier().present())
    {
        if (a.getCandidate().get().getCandidateIdentifier().get().getExpectedConfirmationReference().present() &&
            b.getCandidate().get().getCandidateIdentifier().get().getExpectedConfirmationReference().present())
        {
            std::string referenceA = \
                a.getCandidate().get().getCandidateIdentifier().get().getExpectedConfirmationReference().get();
            std::string referenceB = \
                b.getCandidate().get().getCandidateIdentifier().get().getExpectedConfirmationReference().get();

            long        aId   = getCandidateIdFromExpectedConfirmationReference(referenceA);
            long        bId   = getCandidateIdFromExpectedConfirmationReference(referenceB);
            std::string aType = getCandidateTypeFromExpectedConfirmationReference(referenceA);
            std::string bType = getCandidateTypeFromExpectedConfirmationReference(referenceB);

            if (aType == "E" && bType == "S")
            {
                return true;
            }
            else if (aType == "S" && bType == "E")
            {
                return false;
            }

            if (aId < bId)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }

    return false;
}

/**
 * @brief  Compares two ReportingUnitVotes.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  a The first ReportingUnitVotes.
 * @param  b The second ReportingUnitVotes.
 * @return True if a is less than b, false otherwise.
 */
static bool compareReportingUnitVotes(const Contest::ReportingUnitVotesType& a, const Contest::ReportingUnitVotesType& b)
{
    if (getAffiliationIdentifierOfReportingUnitVotes(a).getExpectedConfirmationReference().present() &&
        getAffiliationIdentifierOfReportingUnitVotes(b).getExpectedConfirmationReference().present())
    {
        std::string referenceA = getAffiliationIdentifierOfReportingUnitVotes(a).getExpectedConfirmationReference().get();
        std::string referenceB = getAffiliationIdentifierOfReportingUnitVotes(b).getExpectedConfirmationReference().get();

        long aNisOrder = getPartyNisFromExpectedConfirmationReference(referenceA);
        long bNisOrder = getPartyNisFromExpectedConfirmationReference(referenceB);

        long aIdOrder = getPartyIdFromExpectedConfirmationReference(referenceA);
        long bIdOrder = getPartyIdFromExpectedConfirmationReference(referenceB);


        if (aNisOrder < bNisOrder)
        {
            return false;
        }
        else if (aNisOrder > bNisOrder)
        {
            return true;
        }

        if (aIdOrder < bIdOrder)
        {
            return true;
        }
    }

    return false;
}

/**
 * @brief  Sorts a sequence of a ReportingUnitVotes selection.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  reportingUnitVotes The sequence to sort.
 */
static void sortReportingUnitVotesSelection(Contest::ReportingUnitVotesSequence& reportingUnitVotes)
{
    Contest::ReportingUnitVotesIterator it;

    for (it = reportingUnitVotes.begin(); it != reportingUnitVotes.end(); ++it)
    {
        ReportingUnitVotes::SelectionSequence& selections = it->getSelection();

        std::sort(selections.begin(), selections.end(), compareSelections);
    }
}

/**
 * @brief  Sorts a sequence of ReportingUnitVotes.
 * @author manuel.arguelles
 * @date   Jan 22, 2018
 * @param  contest The contest with the ReportingUnitVotesSequence to sort.
 */
static void sortReportingUnitVotes(Contests::ContestSequence& contest)
{
    Contests::ContestIterator it;

    for (it = contest.begin(); it != contest.end(); ++it)
    {
        Contest::ReportingUnitVotesSequence& sequence = it->getReportingUnitVotes();

        std::sort(sequence.begin(), sequence.end(), compareReportingUnitVotes);
    }
}

/**
 * @brief  Use the ExpectedConfirmationReference value to fill the IdNumber,
 * ShortCode and DisplayOrder attributes.
 * @author manuel.arguelles
 * @date   Apr 27, 2018
 * @param  reportingUnitVotes The sequence of ReportingUnitVotes elements.
 */
static void expandExpectedConfirmationReference(Contest::ReportingUnitVotesSequence& reportingUnitVotes)
{
    Contest::ReportingUnitVotesIterator it;

    for (it = reportingUnitVotes.begin(); it != reportingUnitVotes.end(); ++it)
    {
        ReportingUnitVotes::SelectionSequence& selections = it->getSelection();
        ReportingUnitVotes::SelectionIterator sIt;

        for (sIt = selections.begin(); sIt != selections.end(); ++sIt)
        {
            if (sIt->getAffiliationIdentifier().present() &&
                sIt->getAffiliationIdentifier().get().getExpectedConfirmationReference().present())
            {
                std::string reference = sIt->getAffiliationIdentifier().get().getExpectedConfirmationReference().get();
                long id = getPartyIdFromExpectedConfirmationReference(reference);

                it->getReportingUnitIdentifier().setIdNumber(StringUtils::longToString(id));
            }
            else if (sIt->getCandidate().present() && sIt->getCandidate().get().getCandidateIdentifier().present() &&
                     sIt->getCandidate().get().getCandidateIdentifier().get().getExpectedConfirmationReference().present())
            {
                std::string reference = sIt->getCandidate().get().getCandidateIdentifier().get().getExpectedConfirmationReference().get();
                std::string type = getCandidateTypeFromExpectedConfirmationReference(reference);
                long id = getCandidateIdFromExpectedConfirmationReference(reference);

                sIt->getCandidate().get().getCandidateIdentifier().get().setShortCode1(type);
                sIt->getCandidate().get().getCandidateIdentifier().get().setDisplayOrder(id);
            }
        }
    }
}

/**
 * @brief  Gets the voter type as described in the XSD from the type (letter)
 * described in the NationalityCode mapping.
 * @author manuel.arguelles
 * @date   May 11, 2018
 * @param  letterType The letter of the type as described in the NationalityCode.
 * @return The EML voter type literal string.
 */
static std::string getEmlVoterTypeByLetterType(const std::string& letterType)
{
    int typeSize = sizeof extensions::ElectronicVotingCountersType::_xsd_ElectronicVotingCountersType_indexes_ /
        sizeof extensions::ElectronicVotingCountersType::_xsd_ElectronicVotingCountersType_indexes_[0];
    std::string result;

    for (int i = 0; i < typeSize; ++i)
    {
        std::string type = extensions::ElectronicVotingCountersType::_xsd_ElectronicVotingCountersType_literals_[i];

        if (type.compare(0, letterType.size(), letterType) == 0)
        {
            result = type;
            break;
        }
    }

    return result;
}

EmlWrapper::EmlWrapper(EML& eml)
    : m_eml(eml)
{
}

EmlWrapper::~EmlWrapper()
{
}

void EmlWrapper::increaseBlankInvalid(const std::string& customCode)
{
    std::string nisCode = EmlInfo::getInputNisCode(customCode);

    try
    {
        Contest&    contest   = getContest(nisCode);
        std::string category  = EmlInfo::getTotalVotesSelectionCategory(customCode);
        Selection&  selection = getTotalVotesSelection(contest, category);

        TotalVotes::CountMetricSequence& metrics = selection.getCountMetric();
        TotalVotes::CountMetricIterator metricIt;

        for (metricIt = metrics.begin(); metricIt != metrics.end(); ++metricIt)
        {
            if (metricIt->getType() == "VotesDeposited")
            {
                ++(*metricIt);
            }

            if (metricIt->getType() == "BlankAndInvalidVotes")
            {
                ++(*metricIt);
            }
        }
    }
    catch (const EmlElementNotFoundException& exception)
    {
        logger->Debug("Not found: " + exception.getMessage());
        throw;
    }
}

void EmlWrapper::increasePartyListBallotCount(const std::string& customCode,
                                              const std::string& partyCode)
{
    std::string nisCode = EmlInfo::getInputNisCode(customCode);

    try
    {
        Contest&   contest   = getContest(nisCode);
        Selection& selection = getPartyReportingUnitVotesSelection(contest, partyCode);

        if (increaseReportingUnitVotesMetricByType(selection, "NumberOfHeaderVotes"))
        {
            std::string category = EmlInfo::getTotalVotesSelectionCategory(customCode);
            ++selection.getValidVotes();
            increaseTotalVotesDeposit(contest, category);
            increaseTotalValidVotes(contest, category);
        }
    }
    catch (const EmlElementNotFoundException& exception)
    {
        logger->Debug("Not found: " + exception.getMessage());
        throw;
    }
}

void EmlWrapper::increasePartyPrincipalBallotCount(const std::string& customCode,
                                                   const std::string& partyCode)
{
    std::string nisCode = EmlInfo::getInputNisCode(customCode);

    try
    {
        Contest&   contest   = getContest(nisCode);
        Selection& selection = getPartyReportingUnitVotesSelection(contest, partyCode);

        if (increaseReportingUnitVotesMetricByType(selection, "NumberOfBallotsWithOnlyTitularVotes"))
        {
            std::string category = EmlInfo::getTotalVotesSelectionCategory(customCode);
            ++selection.getValidVotes();
            increaseTotalVotesDeposit(contest, category);
            increaseTotalValidVotes(contest, category);
        }
    }
    catch (const EmlElementNotFoundException& exception)
    {
        logger->Debug("Not found: " + exception.getMessage());
        throw;
    }
}

void EmlWrapper::increasePartyStandInBallotCount(const std::string& customCode,
                                                 const std::string& partyCode)
{
    std::string nisCode = EmlInfo::getInputNisCode(customCode);

    try
    {
        Contest&   contest   = getContest(nisCode);
        Selection& selection = getPartyReportingUnitVotesSelection(contest, partyCode);

        if (increaseReportingUnitVotesMetricByType(selection, "NumberOfBallotsWithOnlySubstitutesVotes"))
        {
            std::string category = EmlInfo::getTotalVotesSelectionCategory(customCode);
            ++selection.getValidVotes();
            increaseTotalVotesDeposit(contest, category);
            increaseTotalValidVotes(contest, category);
        }
    }
    catch (const EmlElementNotFoundException& exception)
    {
        logger->Debug("Not found: " + exception.getMessage());
        throw;
    }
}

void EmlWrapper::increasePartyPrincipalStandInBallotCount(const std::string& customCode,
                                                          const std::string& partyCode)
{
    std::string nisCode = EmlInfo::getInputNisCode(customCode);

    try
    {
        Contest&   contest   = getContest(nisCode);
        Selection& selection = getPartyReportingUnitVotesSelection(contest, partyCode);

        if (increaseReportingUnitVotesMetricByType(selection, "NumberOfBallotsWithTitularAndSubstitutesVotes"))
        {
            std::string category = EmlInfo::getTotalVotesSelectionCategory(customCode);
            ++selection.getValidVotes();
            increaseTotalVotesDeposit(contest, category);
            increaseTotalValidVotes(contest, category);
        }
    }
    catch (const EmlElementNotFoundException& exception)
    {
        logger->Debug("Not found: " + exception.getMessage());
        throw;
    }
}

void EmlWrapper::increaseCandidateVote(const std::string& customCode,
                                       const std::string& candidateCode)
{
    std::string nisCode = EmlInfo::getInputNisCode(customCode);

    try
    {
        Contest&   contest   = getContest(nisCode);
        Selection& selection = getCandidateReportingUnitVotesSelection(contest, candidateCode);

        ++selection.getValidVotes();
    }
    catch (const EmlElementNotFoundException& exception)
    {
        logger->Debug("Candidate " + candidateCode + " not found: " + exception.getMessage());
        throw;
    }
}

Contest& EmlWrapper::getContest(const std::string& idNumber)
{
    Count::ElectionSequence& elections = m_eml.getCount().getElection();
    Count::ElectionIterator electionIt;

    for (electionIt = elections.begin(); electionIt != elections.end(); ++electionIt)
    {
        Contests::ContestSequence& contests = electionIt->getContests().getContest();
        Contests::ContestIterator contestIt;

        for (contestIt = contests.begin(); contestIt != contests.end(); ++contestIt)
        {
            if (contestIt->getContestIdentifier().getIdNumber() == idNumber)
            {
                return *contestIt;
            }
        }
    }

    throw EmlElementNotFoundException("Contest not found with id: " + idNumber,
                                      "Contest not found");
}

void EmlWrapper::updateTotalVotesSelectionCategory(const std::string& customCode)
{
    std::string nisCode = EmlInfo::getInputNisCode(customCode);
    Contest& contest = getContest(nisCode);

    if (contest.getTotalVotes().present())
    {
        TotalVotes::SelectionSequence& selections = contest.getTotalVotes().get().getSelection();
        TotalVotes::SelectionIterator it;

        for (it = selections.begin(); it != selections.end(); ++it)
        {
            if (it->getCategory().present() && it->getCategory().get() != "GlobalSituation")
            {
                std::string category = EmlInfo::getTotalVotesSelectionCategory(customCode);

                it->getCategory().get() = category;
            }
        }
    }
}

void EmlWrapper::mergeEml(const EML& eml)
{
    Count::ElectionSequence& elections = m_eml.getCount().getElection();
    Contest& sourceContest = elections.front().getContests().getContest().front();

    if (sourceContest.getTotalVotes().present())
    {
        TotalVotes::SelectionSequence newSelections = eml.getCount().getElection().front().getContests().getContest().front().getTotalVotes().get().getSelection();
        TotalVotes::SelectionSequence& selections = sourceContest.getTotalVotes().get().getSelection();

        mergeTotalVotesSelections(selections, newSelections);
        updateValidVotesGlobalSituation(selections);
    }
    else
    {
        TotalVotes totalVotes = eml.getCount().getElection().front().getContests().getContest().front().getTotalVotes().get();

        sourceContest.setTotalVotes(totalVotes);
    }

    Contest::ReportingUnitVotesSequence& sourceReportingUnitVotes = sourceContest.getReportingUnitVotes();
    sourceReportingUnitVotes.insert(sourceReportingUnitVotes.end(),
                                    eml.getCount().getElection().front().getContests().getContest().front().getReportingUnitVotes().begin(),
                                    eml.getCount().getElection().front().getContests().getContest().front().getReportingUnitVotes().end());
}

bool EmlWrapper::serialize(const std::string& outputFile)
{
    std::ofstream file(outputFile.c_str(), std::ios_base::trunc);
    bool result = true;

    if (file)
    {
        try
        {
            xml_schema::NamespaceInfomap map;

            map[""].name = "urn:oasis:names:tc:evs:schema:eml";
            map[""].schema = "schema/510-count-v7-0.xsd";

            // map["eml"].name = "urn:oasis:names:tc:evs:schema:eml";
            // map["eml"].schema = "schema/510-count-v7-0.xsd";

            map["martine"].name = "http://www.martine.be/extensions";
            map["martine"].schema = "schema/martine-eml-extensions.xsd";
            map["xal"].name = "urn:oasis:names:tc:ciq:xal:4";
            map["xnl"].name = "urn:oasis:names:tc:ciq:xnl:4";
            map["xsi"].name = "http://www.w3.org/2001/XMLSchema-instance";

            serializeEML(file, m_eml, map);
        }
        catch (...)
        {
            logger->Debug("Error serializing EML data to file " + outputFile);
            result = false;
        }

        file.close();
    }
    else
    {
        logger->Debug("Error while opening file: " + outputFile);
        result = false;
    }

    return result;
}

long EmlWrapper::calculateChecksums()
{
    long checksum = 0;

    Count::ElectionSequence& elections = m_eml.getCount().getElection();

    if (elections.size() == 1)
    {
        Contests::ContestSequence& contests = elections.front().getContests().getContest();

        if (contests.size() == 1)
        {
            Contest::ReportingUnitVotesSequence& parties = contests.front().getReportingUnitVotes();
            Contest::ReportingUnitVotesIterator it;

            for (it = parties.begin(); it != parties.end(); ++it)
            {
                calculatePartyChecksum(*it);
            }

            checksum = calculateGlobalChecksum(contests.front());
        }
    }

    return checksum;
}

void EmlWrapper::sort()
{
    Count::ElectionSequence& elections = m_eml.getCount().getElection();

    if (elections.size() == 1)
    {
        Contests::ContestSequence& contests = elections.front().getContests().getContest();

        if (contests.size() == 1)
        {
            Contest::ReportingUnitVotesSequence& sequence = contests.front().getReportingUnitVotes();

            sortReportingUnitVotesSelection(sequence);
        }
        else
        {
            logger->Debug("ERROR: Malformed EML file: Contests number mismatch");
        }

        sortReportingUnitVotes(contests);
    }
    else
    {
        logger->Debug("ERROR: Malformed EML file: Election number mismatch");
    }
}

void EmlWrapper::removePartiesOfDifferentLinguisticRegime(const std::string& customCode)
{
    int linguisticRegime = EmlInfo::getLinguisticRegime(customCode);
    std::string nisCode  = EmlInfo::getInputNisCode(customCode);
    std::string category = EmlInfo::getTotalVotesSelectionCategory(customCode);

    if (linguisticRegime == 0)
    {
        logger->Debug("ERROR: Linguistic regime for " + customCode + " was not found");
        return;
    }

    Election::ElectionIdentifierType& electionType = m_eml.getCount().getElection().front().getElectionIdentifier();

    electionType.getIdNumber() = EmlInfo::getElectionIdentifierId(customCode);
    electionType.setElectionName("");

    if (electionType.getElectionCategory().present())
    {
        electionType.getElectionCategory().get() = EmlInfo::getElectionType(customCode);
    }
    else
    {
        electionType.setElectionCategory(EmlInfo::getElectionType(customCode));
    }

    Contest& contest = getContest(nisCode);

    try
    {
        Selection& selection = getTotalVotesSelection(contest, category);

        addSeatsToBeFilledFromElectionParam(selection, linguisticRegime);
        addSubstitutesMaxFromElectionParam(selection, linguisticRegime);
        addLinguisticRegimeFromElectionParam(selection, linguisticRegime);

        removeAllElectionParam(selection);
    }
    catch (const EmlElementNotFoundException& exception)
    {
        logger->Debug("ERROR: Failed to update linguisting regime data for the total votes selection");
    }

    Contest::ReportingUnitVotesSequence& parties = contest.getReportingUnitVotes();
    Contest::ReportingUnitVotesIterator it;


    for (it = parties.begin(); it != parties.end();)
    {
        ReportingUnitVotes::CountMetricType& metric = getCountMetricByType(*it, "LinguisticRegime");

        if (metric != linguisticRegime)
        {
            std::auto_ptr<ReportingUnitVotes> removed;

            it = parties.detach(it, removed);
        }
        else
        {
            ++it;
        }
    }
}

void EmlWrapper::prepareForOutput(
    const std::string& filename,
    const std::string& contestShortCode,
    const std::string& pollingStationId)
{
    Count::EventIdentifierType& identifier = m_eml.getCount().getEventIdentifier();
    Count::ElectionSequence& elections = m_eml.getCount().getElection();
    Count::ElectionIterator it;
    std::string eventQualifier = EmlInfo::getEventQualifier(filename);
    std::string nisCode = EmlInfo::getOutputNisCode(filename);

    identifier.getEventQualifier() = eventQualifier;

    for (it = elections.begin(); it != elections.end(); ++it)
    {
        Contests::ContestSequence& contests = it->getContests().getContest();
        Contests::ContestIterator cIt;

        for (cIt = contests.begin(); cIt != contests.end(); ++cIt)
        {
            ContestIdentifierStructure& contestIdentifier = cIt->getContestIdentifier();

            contestIdentifier.getIdNumber() = nisCode;
            contestIdentifier.getShortCode() = contestShortCode;
            contestIdentifier.setContestName("");

            if (cIt->getTotalVotes().present())
            {
                addLevelDepositListToCategory(cIt->getTotalVotes().get().getSelection());
                addPollingStationIdToGlobalSituation(
                    cIt->getTotalVotes().get().getSelection(),
                    StringUtils::stringToLong(pollingStationId));
            }

            expandExpectedConfirmationReference(cIt->getReportingUnitVotes());
        }
    }
}

void EmlWrapper::setTotalsByType(const std::map<std::string, long>& totalsByType)
{
    std::map<std::string, long>::const_iterator it;

    for (it = totalsByType.begin(); it != totalsByType.end(); ++it)
    {
        Count::ElectronicVotingCountersType total;

        total = StringUtils::longToString(it->second);
        total.setType(getEmlVoterTypeByLetterType(it->first));

        m_eml.getCount().getElectronicVotingCounters().push_back(total);
    }
}

void EmlWrapper::setElectionDate(const std::string& dateTime)
{
    Glib::TimeVal electionDateTime;

    electionDateTime.assign_from_iso8601(dateTime);

    if (electionDateTime.valid())
    {
        Glib::Date electionDate;

        electionDate.set_time(electionDateTime);

        std::string date = electionDate.format_string(DATE_FORMAT);

        m_eml.getEMLHeader().setIssueDate(date);
        m_eml.getEMLHeader().getOfficialStatusDetail().getStatusDate().year(electionDate.get_year());
        m_eml.getEMLHeader().getOfficialStatusDetail().getStatusDate().month(electionDate.get_month());
        m_eml.getEMLHeader().getOfficialStatusDetail().getStatusDate().day(electionDate.get_day());
    }
}

void EmlWrapper::updatePartyAndCandidateNames(const std::string& customCode, const ElectoralConfiguration& electoralConfiguration)
{
    std::string nisCode = EmlInfo::getInputNisCode(customCode);
    Contest& contest    = getContest(nisCode);

    Contest::ReportingUnitVotesSequence& reportingUnitVotes = contest.getReportingUnitVotes();
    Contest::ReportingUnitVotesIterator it;

    for (it = reportingUnitVotes.begin(); it != reportingUnitVotes.end(); ++it)
    {
        ReportingUnitVotes::SelectionSequence& selections = it->getSelection();
        ReportingUnitVotes::SelectionIterator sIt;

        for (sIt = selections.begin(); sIt != selections.end(); ++sIt)
        {
            if (sIt->getAffiliationIdentifier().present() &&
                sIt->getAffiliationIdentifier().get().getExpectedConfirmationReference().present())
            {
                std::string reference = sIt->getAffiliationIdentifier().get().getExpectedConfirmationReference().get();
                std::string name = electoralConfiguration.getPartyNameByExpectedConfirmationReference(reference);

                sIt->getAffiliationIdentifier().get().setRegisteredName(name);
                it->setReportingUnitIdentifier(name);
            }
            else if (sIt->getCandidate().present() &&
                     sIt->getCandidate().get().getCandidateIdentifier().present() &&
                     sIt->getCandidate().get().getCandidateIdentifier().get().getExpectedConfirmationReference().present())
            {
                std::string reference = sIt->getCandidate().get().getCandidateIdentifier().get().getExpectedConfirmationReference().get();
                std::string name = electoralConfiguration.getCandidateNameByExpectedConfirmationReference(reference);

                sIt->getCandidate().get().getCandidateFullName().get().setPersonFullName(name);
            }
            else
            {
                throw EmlElementNotFoundException("Selection with AffilitionIdentifier or Candidate not found",
                                                  "Selection not found");
            }
        }
    }
}
