/**
 * @file   EmlWrapper.hxx
 * @brief  Declaration of EmlWrapper class.
 * @author manuel.arguelles
 * @date   Jan 11, 2018
 */

#ifndef EML_WRAPPER_HXX
#define EML_WRAPPER_HXX

#include <Voting/Eml/510.hxx>
#include <string>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            class ElectoralConfiguration;

            namespace Eml
            {
                /**
                 * @class  EmlWrapper
                 * @brief  Wrapper around EML objects.
                 * @author manuel.arguelles
                 * @date   Jan 11, 2018
                 */
                class EmlWrapper
                {
                    public:
                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   Jan 11, 2018
                         * @param  eml The EML object.
                         */
                        explicit EmlWrapper(EML& eml);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Jan 11, 2018
                         */
                        ~EmlWrapper();

                    public:

                        /**
                         * @brief  Increase a blank or invalid vote.
                         * @author manuel.arguelles
                         * @date   Jan 12, 2018
                         * @param  customCode The customCode of the contest.
                         * @throws EmlElementNotFoundException If the contest was not found.
                         */
                        void increaseBlankInvalid(const std::string& customCode);

                        /**
                         * @brief  Increase the amount of ballots who have a list
                         * vote for a party.
                         * @author manuel.arguelles
                         * @date   Jan 15, 2018
                         * @param  customCode The custom code of the contest.
                         * @param  partyCode  The code of the party.
                         * @throws EmlElementNotFoundException If the contest or party were not found.
                         */
                        void increasePartyListBallotCount(const std::string& customCode,
                                                          const std::string& partyCode);


                        /**
                         * @brief  Increase the amount of ballots who have
                         * principal votes only for a party.
                         * @author manuel.arguelles
                         * @date   Jan 15, 2018
                         * @param  customCode The custom code of the contest.
                         * @param  partyCode  The code of the party.
                         */
                        void increasePartyPrincipalBallotCount(const std::string& customCode,
                                                               const std::string& partyCode);


                        /**
                         * @brief  Increase the amount of ballots who have stand-in
                         * votes only for a party.
                         * @author manuel.arguelles
                         * @date   Jan 15, 2018
                         * @param  customCode The custom code of the contest.
                         * @param  partyCode  The code of the party.
                         */
                        void increasePartyStandInBallotCount(const std::string& customCode,
                                                             const std::string& partyCode);

                        /**
                         * @brief  Increase the amount of ballots who have
                         * principals and stand-in votes for a party.
                         * @author manuel.arguelles
                         * @date   Jan 15, 2018
                         * @param  customCode The custom code of the contest.
                         * @param  partyCode  The code of the party.
                         */
                        void increasePartyPrincipalStandInBallotCount(const std::string& customCode,
                                                                      const std::string& partyCode);

                        /**
                         * @brief  Increase the votes for a given candidate.
                         * @author manuel.arguelles
                         * @date   Jan 15, 2018
                         * @param  customCode    The custom code of the contest.
                         * @param  candidateCode The code of the candidate.
                         */
                        void increaseCandidateVote(const std::string& customCode,
                                                   const std::string& candidateCode);

                        /**
                         * @brief  Updates the category of the TotalVotes selection.
                         * @author manuel.arguelles
                         * @date   Jan 19, 2018
                         * @param  customCode The custom code.
                         */
                        void updateTotalVotesSelectionCategory(const std::string& customCode);

                        /**
                         * @brief  Merge the data from the given EML.
                         * @author manuel.arguelles
                         * @date   Jan 18, 2018
                         * @param  eml The EML to merge data from.
                         */
                        void mergeEml(const EML& eml);

                        /**
                         * @brief  Serialize the EML to a file.
                         * @author manuel.arguelles
                         * @date   Jan 12, 2018
                         * @param  outputFile The path of the output file.
                         * @return True if the operation succeed, false if not.
                         */
                        bool serialize(const std::string& outputFile);

                        /**
                         * @brief  Calculates the checksum of the EML file.
                         * @author manuel.arguelles
                         * @date   Jan 22, 2018
                         * @return The checksum of the EML FILE.
                         */
                        long calculateChecksums();

                        /**
                         * @brief  Sorts the EML file.
                         * @author manuel.arguelles
                         * @date   Jan 22, 2018
                         */
                        void sort();

                        /**
                         * @brief  Removes all the parties of a different linguistic regime from the
                         * specified in the given custom code.
                         * @author manuel.arguelles
                         * @date   Feb 10, 2018
                         * @param  customCode The custom code.
                         */
                        void removePartiesOfDifferentLinguisticRegime(const std::string& customCode);

                        /**
                         * @brief  Perform final modifications to the EML.
                         * @author manuel.arguelles
                         * @date   Feb 12, 2018
                         * @param  filename         The filename of the output.
                         * @param  contestShortCode The short code of the contest.
                         * @param  pollingStationId The polling station id.
                         */
                        void prepareForOutput(
                            const std::string& filename,
                            const std::string& contestShortCode,
                            const std::string& pollingStationId);

                        /**
                         * @brief  Sets the total votes by elector type.
                         * @author manuel.arguelles
                         * @date   Mar 20, 2018
                         * @param  totalsByType A map with the type and their totals.
                         */
                        void setTotalsByType(const std::map<std::string, long>& totalsByType);

                        /**
                         * @brief  Sets the date of the election.
                         * @author manuel.arguelles
                         * @date   Mar 20, 2018
                         * @param  dateTime The date and time in ISO8601 format.
                         */
                        void setElectionDate(const std::string& dateTime);

                        /**
                         * @brief  Updates the name of the party and candidates according to the electoral configuration.
                         * @author manuel.arguelles
                         * @date   Mar 27, 2018
                         * @param  customCode             The custom code of the EML file.
                         * @param  electoralConfiguration The electoral configuration.
                         * @throws EmlElementNotFoundException If the contest, party of candidates were not found.
                         * @throws PartyNotFoundException      If the EML party was not found in the electoral
                         * configuration.
                         * @throws CandidateNotFoundException  If the EML candidate was not found in the electoral
                         * configuration.
                         */
                        void updatePartyAndCandidateNames(const std::string& customCode,
                                                          const ElectoralConfiguration& electoralConfiguration);
                    private:

                        /**
                         * @brief  Returns a reference to the given contest.
                         * @author manuel.arguelles
                         * @date   Jan 12, 2018
                         * @return A reference to the contest.
                         * @throws EmlElementNotFoundException if the contest can't
                         * be found.
                         */
                        Contest& getContest(const std::string& idNumber);

                    private:

                        EML& m_eml;
                };

            } /* namespace Eml */
        } /* namespace Voting */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* EML_WRAPPER_HXX */
