/**
 * @file   X7SEmlFileWrapper.cxx
 * @brief  Implementation of X7SEmlFileWrapper class.
 * @author manuel.arguelles
 * @date   Feb 14, 2018
 */

#include "X7SEmlFileWrapper.hxx"
#include "X7SSecurity.hxx"
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Runtime/Environment/saes-directories.h>
#include <System/Exception/XmlException.hxx>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/SafeBuffer.h>
#include <System/Exception/FileSystemException.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/Eml/x7s-eml-file-schema.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif
#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::EmlX7S;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System;

static ISMTTLog* logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Voting.Eml.X7SEmlFileWrapper",
    GETTEXT_PACKAGE);

X7SEmlFileWrapper::X7SEmlFileWrapper(ElectoralConfiguration* configuration,
                                     SaesDirectories* directories)
    : m_electoralConfiguration(configuration)
    , m_directories(directories)
{
}

X7SEmlFileWrapper::~X7SEmlFileWrapper()
{
}

void X7SEmlFileWrapper::persistX7SEmlFile(const std::string& emlFile,
                                          const std::string& emlX7SFile,
                                          const std::string& pvFormFile)
{
    std::string outputDirDom = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_DOM);
    std::string outputDirFlash = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_FLASH);
    std::string emlTempFolder = m_directories->getEmlTempFolderFullPath();
    std::string inputFileDom = Path::Combine(m_directories->getDocsFolderFullPath(BasePathType::CONFIGURATION_DOM), pvFormFile);
    std::string inputFileFlash = Path::Combine(m_directories->getDocsFolderFullPath(BasePathType::CONFIGURATION_FLASH), pvFormFile);
    std::string pvreportDom = Path::Combine(outputDirDom, emlX7SFile + ".PDF" );
    std::string pvreportFlash = Path::Combine(outputDirFlash, emlX7SFile + ".PDF" );
    std::string emlTempFile = Path::Combine(emlTempFolder, emlFile);
    X7SEmlFile x7sEml;
    bool addPv = SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getX7SAddPVReport();

    logger->Debug("Persist EML file " + emlFile + " to " + emlX7SFile + " with " + emlX7SFile + ".PDF");

    File::getInstance()->copyFile(inputFileDom, pvreportDom);
    File::getInstance()->copyFile(inputFileFlash, pvreportFlash);

    if (addPv)
    {
        SafeBuffer pvBuffer;

        if (!File::getInstance()->openFileToBuffer(pvreportDom, pvBuffer))
        {
            logger->Debug("ERROR: Load pv report " + pvreportDom + " not found.");
            throw FileSystemException("PV report file " + pvreportDom + " not found",
                                      "Error loading pv report",
                                      logger);
        }

        PVFile pvFile;
        Base64Binary pvBinary(pvBuffer.GetData(), pvBuffer.GetSize());

        pvFile.setPVType(PVType::PDF);
        pvFile.setOutputFileName(pvFormFile);
        pvFile.setData(pvBinary);
        x7sEml.setPVFile(pvFile);
    }
    else
    {
        PVFile pvFile;
        Base64Binary pvBinary(0);

        pvFile.setPVType(PVType::PDF);
        pvFile.setOutputFileName("");
        pvFile.setData(pvBinary);
        x7sEml.setPVFile(pvFile);
    }

    if (!SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getPersistenceEml())
    {
        File::getInstance()->DeleteFile(pvreportDom);
        File::getInstance()->DeleteFile(pvreportFlash);
    }

    SafeBuffer emlBuffer;

    if (!File::getInstance()->openFileToBuffer(emlTempFile, emlBuffer))
    {
        logger->Debug("ERROR: Load EML file " + emlTempFile + " not found.");
        throw FileSystemException("Error reading file: " + emlTempFile,
                                  "Error reading file",
                                  logger);
    }

    FormatEML emlFormat;
    Base64Binary emlBinary(emlBuffer.GetData(), emlBuffer.GetSize());

    emlFormat.setOutputFileName(emlFile);
    emlFormat.setData(emlBinary);
    x7sEml.setFormatEML(emlFormat);

    std::string dataXml = x7sEmlToString(x7sEml);
    SafeBuffer fileBuffer(dataXml);
    SafeBuffer sign;
    SafeBuffer crypto;
    X7SSecurity x7sSecurity;

    x7sSecurity.init(m_electoralConfiguration, m_directories);
    x7sSecurity.signBuffer(fileBuffer, sign);
    x7sSecurity.encryptBuffer(sign, crypto);

    std::string x7sDom = Path::Combine(outputDirDom, emlX7SFile);
    std::string x7sFlash = Path::Combine(outputDirFlash, emlX7SFile);

    File::getInstance()->saveBufferToFile(crypto, x7sDom);
    File::getInstance()->copyFile(x7sDom, x7sFlash);

    logger->Debug("Success persistence X7S EML file " + emlFile);
}

std::string X7SEmlFileWrapper::x7sEmlToString(const X7SEmlFile& x7sEml) const
{
    std::ostringstream str;
    NamespaceInfomap mapv;

    mapv[""].name = NAMESPACE_VOTING_EMLX7S;
    mapv[""].schema = "";

    try
    {
        serializeFinalData(str, x7sEml, mapv, "UTF-8", Flags::dont_validate);
    }
    catch (const ::xsd::cxx::tree::exception<char>& e)
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__ << e;
        logger->Error(ss.str());
        throw XmlException(
            _("Smartmatic.SAES.Voting.Eml.X7SEmlFileWrapper.ErrorSerializaData"),
            N_("Smartmatic.SAES.Voting.Eml.X7SEmlFileWrapper.ErrorSerializaData"),
            logger);
    }
    catch (...)
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__;
        logger->Error(ss.str());
        throw XmlException(
            _("Smartmatic.SAES.Voting.Eml.X7SEmlFileWrapper.ErrorSerializaData"),
            N_("Smartmatic.SAES.Voting.Eml.X7SEmlFileWrapper.ErrorSerializaData"),
            logger);
    }

    return str.str();
}
