/*
 * NationalityTypeManager.cxx
 *
 *  Created on: 22/11/2013
 *      Author: soto
 */

#include "NationalityTypeManager.hxx"
#include <Environment/Configurations/resource-path-provider.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Resources/election-string-resources-manager.h>
#include <Runtime/Environment/saes-directories.h>
#include <System/Exception/FileSystemException.hxx>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/SafeBuffer.h>
#include <System/Utils/StringUtils.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/PBReports/VoteStatisticsWrapper.hxx>
#include <Voting/Recount/RecountVoteStatistics.h>
#include <Voting/Eml/EmlFileManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

NationalityTypeManager * NationalityTypeManager::instance = NULL;
ISMTTLog * NationalityTypeManager::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.NationalityTypeManager", GETTEXT_PACKAGE );

NationalityTypeManager::NationalityTypeManager ()
{
    directories = NULL;
    voteStatistics = NULL;
    recountVoteStatistics = NULL;
}

NationalityTypeManager::~NationalityTypeManager()
{
    instance = NULL;
}

void NationalityTypeManager::init(SAES::Runtime::Environment::SaesDirectories * directories,
                                  PBReports::VoteStatisticsWrapper * voteStatistics,
                                  RecountVoteStatistics * recountVoteStatistics,
                                  Eml::EmlFileManager* emlFiles)
{
    this->directories = directories;
    this->voteStatistics = voteStatistics;
    this->recountVoteStatistics = recountVoteStatistics;

    if (emlFiles != NULL)
    {
        this->machineCode = emlFiles->getMachineCode();
        this->customNisCode = emlFiles->getCustomNisCode();
    }

    if (this->directories != NULL)
    {
        SafeBuffer buffer;
        std::string file = directories->getNationalityDefinitionTypeFileFullPath ();
        std::string resource = ResourcePathProvider::getInstance()->getResourcePath ( file );

        if (File::getInstance()->openFileToBuffer(resource, buffer))
        {
            getNationalityMap(buffer, typeCodeMap);
        }
        else
        {
            throw FileSystemException(
                _("Error open nationality code file"),
                N_("Error open nationality code file"),
                logger);
        }
    }
}

void NationalityTypeManager::persistence()
{
    logger->Debug ( "Generate nationality type results" );

    SafeBuffer buffer;

    std::string file = directories->getNationalityDefinitionTypeFileFullPath ();
    std::string resource = ResourcePathProvider::getInstance()->getResourcePath ( file );

    if ( File::getInstance()->openFileToBuffer ( resource, buffer ) )
    {
        std::string votingResultFilePlainDom   = getVotingResultsDomFile();
        std::string votingResultFilePlainFlash = getVotingResultsFlashFile();

        std::map < std::string, std::string > map;
        getNationalityMap ( buffer, map );

        logger->Debug ( "Persist nationality results " + votingResultFilePlainDom );
        logger->Debug ( "Persist nationality results " + votingResultFilePlainFlash );

        std::string data = getStringResult ( map );
        SafeBuffer result ( data );


        if ( !File::getInstance()->saveBufferToFile ( result, votingResultFilePlainDom ) )
        {
            throw FileSystemException (
                _ ( "Error save nationality results file" ),
                N_ ( "Error save nationality results file" ),
                logger );
        }

        File::getInstance()->copyFile ( votingResultFilePlainDom, votingResultFilePlainFlash );
    }
    else
    {
        throw FileSystemException (
            _ ( "Error open nationality code file" ),
            N_ ( "Error open nationality code file" ),
            logger );
    }
}

bool NationalityTypeManager::validate()
{
    bool validate = true;

    SafeBuffer buffer;

    std::string file = directories->getNationalityDefinitionTypeFileFullPath ();
    std::string resource = ResourcePathProvider::getInstance()->getResourcePath ( file );

    if ( File::getInstance()->openFileToBuffer ( resource, buffer ) )
    {
        std::string votingResultFilePlainDom   = getVotingResultsDomFile();
        std::string votingResultFilePlainFlash = getVotingResultsFlashFile();

        std::map < std::string, std::string > map;
        getNationalityMap ( buffer, map );

        std::string data = getStringResult ( map );
        SafeBuffer result ( data );
        SafeBuffer expected;

        File::getInstance()->openFileToBuffer ( votingResultFilePlainDom, expected );
        validate = validate && File::getInstance()->compareFiles ( votingResultFilePlainDom, votingResultFilePlainFlash ) && result == expected;
    }

    return validate;
}

void NationalityTypeManager::deleteFiles()
{
    std::string votingResultFilePlainDom   = getVotingResultsDomFile();
    std::string votingResultFilePlainFlash = getVotingResultsFlashFile();

    File::getInstance()->DeleteFile ( votingResultFilePlainDom );
    File::getInstance()->DeleteFile ( votingResultFilePlainFlash );
}

void NationalityTypeManager::getNationalityMap(const SafeBuffer & buffer, std::map<std::string, std::string>& map)
{
    map.clear();

    std::string content;

    if ( buffer.GetData() != NULL )
    {
        content.append ( ( const char * ) buffer.GetData(), buffer.GetSize() );
    }

    std::list < std::string > lines;
    std::list < std::string >::iterator it;
    StringUtils::splitToList ( content, "\n", lines );

    for ( it = lines.begin(); it != lines.end(); ++it )
    {
        std::vector < std::string > codes;
        StringUtils::splitToVector ( ( *it ), ",", codes );

        if ( codes.size() == 2 )
        {
            map[codes[1]] = codes[0];
        }
    }
}

std::string NationalityTypeManager::getStringResult ( const std::map < std::string, std::string > & map )
{
    ElectionStringResourcesManager * resource = ElectionStringResourcesManager::getInstance();
    std::list < std::string > types = recountVoteStatistics->GetVoterTypes();
    std::vector < PBReports::VoteCounterByCA > votes = voteStatistics->getVoteCountersByCA();

    std::stringstream ss;
    std::list < std::string >::iterator ittypes;

    ss << "T=" << voteStatistics->getOfficialVotes();

    for ( ittypes = types.begin();
            ittypes != types.end();
            ++ittypes )
    {
        std::string key = resource->getResource ( *ittypes );

        if ( map.find ( key ) != map.end() )
        {
            std::string type = map.at ( key );

            for ( std::vector< PBReports::VoteCounterByCA>::iterator it = votes.begin(); it != votes.end(); ++it )
            {
                if ( *ittypes == ( *it ).getCAValue() )
                {
                    ss << "|" << type << "=" << ( *it ).getCounter();
                    break;
                }
            }
        }
        else
        {
            ss << "|" << key;
        }
    }

    return ss.str();
}

std::string NationalityTypeManager::getStringResult() const
{
    ElectionStringResourcesManager* resource = ElectionStringResourcesManager::getInstance();
    std::list<std::string> types = recountVoteStatistics->GetVoterTypes();
    std::list<std::string>::iterator ittypes;
    std::stringstream ss;
    std::vector<PBReports::VoteCounterByCA> votes = voteStatistics->getVoteCountersByCA();

    ss << "T=" << voteStatistics->getOfficialVotes();

    for (ittypes = types.begin(); ittypes != types.end(); ++ittypes)
    {
        std::string key = resource->getResource(*ittypes);

        if (typeCodeMap.find(key) != typeCodeMap.end())
        {
            std::string type = typeCodeMap.at(key);

            for (std::vector<PBReports::VoteCounterByCA>::iterator it = votes.begin(); it != votes.end(); ++it)
            {
                if (*ittypes == it->getCAValue())
                {
                    ss << "|" << type << "=" << it->getCounter();
                    break;
                }
            }
        }
        else
        {
            ss << "|" << key;
        }
    }

    return ss.str();
}

std::map<std::string, long> NationalityTypeManager::getTotalsByType() const
{
    std::map<std::string, long> result;
    ElectionStringResourcesManager* resource = ElectionStringResourcesManager::getInstance();
    std::list<std::string> types = recountVoteStatistics->GetVoterTypes();
    std::vector<PBReports::VoteCounterByCA> votes = voteStatistics->getVoteCountersByCA();
    std::list<std::string>::iterator ittypes;

    result["T"] = voteStatistics->getOfficialVotes();

    for (ittypes = types.begin(); ittypes != types.end(); ++ittypes)
    {
        std::string key = resource->getResource(*ittypes);

        if (typeCodeMap.find(key) != typeCodeMap.end())
        {
            std::string type = typeCodeMap.at(key);

            for (std::vector<PBReports::VoteCounterByCA>::iterator it = votes.begin(); it != votes.end(); ++it)
            {
                if (*ittypes == it->getCAValue())
                {
                    result[type] = it->getCounter();
                    break;
                }
            }
        }
    }

    return result;
}

NationalityTypeManager * NationalityTypeManager::getInstance()
{
    if ( instance == NULL )
    {
        instance = new NationalityTypeManager();
    }

    return instance;
}

std::string NationalityTypeManager::getVotingResultsDomFile() const
{
    std::string votingResults = directories->getVotingResultsFilePlainFullPath(BasePathType::CONFIGURATION_DOM);

    votingResults = StringUtils::replace ( votingResults, "[n]", machineCode );
    votingResults = StringUtils::replace ( votingResults, "[nis]", customNisCode );

    return votingResults;
}

std::string NationalityTypeManager::getVotingResultsFlashFile() const
{
    std::string votingResults = directories->getVotingResultsFilePlainFullPath(BasePathType::CONFIGURATION_FLASH);

    votingResults = StringUtils::replace ( votingResults, "[n]", machineCode );
    votingResults = StringUtils::replace ( votingResults, "[nis]", customNisCode );

    return votingResults;
}
