/**
 * @file	PVFormWrapper.hxx
 * @brief	pv form wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#ifndef _PVFORM_WRAPPER_H_
#define _PVFORM_WRAPPER_H_

#include <Voting/PBReports/vote-results-schema.hxx>
#include <string>
#include <list>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace XSLFO
        {
            class XSLFOTransformer;
        }
    }

    namespace SAES
    {
        namespace Runtime
        {
            namespace Environment
            {
                class SaesDirectories;
            }
        }

        namespace Voting
        {
            class ElectoralConfiguration;
            class OperationStatus;

            namespace Vote
            {
                class StationTallyWrapper;
                class TransmissionPackageWrapper;
            }

            namespace Eml
            {
                class EmlFileManager;
            }

            namespace PBReports
            {
                class VoteStatisticsWrapper;
                class ActivatedCardStatisticsWrapper;
                class PVLanguage;

                /**
                 * @class	PVFormWrapper
                 * @brief	Transmission package wrapper.
                 * @author	Yadickson Soto yadickson.soto@smartmatic.com
                 * @date	05/05/2011
                 */
                class PVFormWrapper
                {
                    public:

                        /**
                         * @brief	Default constructor.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	05/05/2011
                         */

                        PVFormWrapper();

                        /**
                         * @brief	Finaliser.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	05/05/2011
                         */

                        virtual ~PVFormWrapper();

                        /**
                         * @brief  Initialises this object.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date   05/05/2011
                         * @throws CryptoException Thrown when a crypto error condition occurs.
                         * @throws SignedException Thrown when a signed error condition occurs.
                         * @throws XmlException	   Thrown when an XML error condition occurs.
                         * @param  directories                    SAES directories
                         * @param  electoralConfiguration         The electoral configuration.
                         * @param  operationStatus                The operation status.
                         * @param  activatedCardStatisticsWrapper The activated card statistics wrapper.
                         * @param  voteStatisticsWrapper          The vote statistics wrapper.
                         * @param  stationTallyWrapper            The station tally wrapper
                         * @param  transmissionPackageWrapper     The transmission package wrapper
                         * @param  codiFileWrapper                The codi file wrapper
                         */
                        void init (
                            SAES::Runtime::Environment::SaesDirectories * directories,
                            Voting::ElectoralConfiguration * electoralConfiguration,
                            Voting::OperationStatus * operationStatus,
                            ActivatedCardStatisticsWrapper * activatedCardStatisticsWrapper,
                            VoteStatisticsWrapper * voteStatisticsWrapper,
                            Voting::Vote::StationTallyWrapper * stationTallyWrapper,
                            Voting::Vote::TransmissionPackageWrapper * transmissionPackageWrapper,
                            Voting::Eml::EmlFileManager * emlFiles);

                        /**
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	05/05/2011
                         */
                        void dispose();

                        /**
                         * @brief	Persistence pv form.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @throws	CryptoException	Thrown when a crypto error condition occurs.
                         * @throws	SignedException	Thrown when a signed error condition occurs.
                         * @throws	XmlException	Thrown when an XML error condition occurs.
                         * @param       transformer     The transformer.
                         * @return      True if it succeeds, false if it fails.
                         */
                        bool persistencePVForm(Smartmatic::System::XSLFO::XSLFOTransformer* transformer);

                        /**
                         * @brief	Persistences.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @throws	CryptoException	Thrown when a crypto error condition occurs.
                         * @throws	SignedException	Thrown when a signed error condition occurs.
                         * @throws	XmlException    Thrown when an XML error condition occurs.
                         * @param       pvformFileDom     The pvform file dom.
                         * @param       pvformFileFlash	  The pvform file flash.
                         * @param       pvreportFileDom	  The pvreport file dom.
                         * @param       pvreportFileFlash The pvreport file flash.
                         * @param       pvtemplateFile    The pvtemplate file.
                         * @return	True if it succeeds, false if it fails.
                         */
                        bool persistence ( std::string & pvformFileDom,
                                           std::string & pvformFileFlash,
                                           std::string & pvreportFileDom,
                                           std::string & pvreportFileFlash,
                                           std::string & pvtemplateFile );

                        /**
                         * @brief Signs the pv report pdf file.
                         * @author Eduardo Fernández
                         * @date   11/10/2018
                         * @param  pvReportFilePath path to the pv report pdf file.
                         * @param  signatureFilePath the signature file path.
                         * @return true if the signing process is successful.
                         */
                        bool signPVReport(const std::string& pvReportFilePath, std::string& signatureFilePath);

                        /**
                         * @brief	Persist the transmission package.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	05/05/2011
                         * @throws	CryptoException Thrown when a crypto error condition occurs.
                         * @throws	SignedException Thrown when a signed error condition occurs.
                         * @throws	XmlException    Thrown when an XML error condition occurs.
                         * @param       language The PV language.
                         */
                        void loadPVForm( PVLanguage &language );

                        /**
                         * @brief	Method setter file.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	13/05/2011
                         * @param	fileFullPath				 	file full path dom.
                         * ### param	fileFullPathFlash	file full path flash.
                         */
                        void setFileFullPath ( const std::string& fileFullPath );

                        /**
                         * @brief	Method setter duration election.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	07/05/2011
                         * @param	durationElection	duration election.
                         */
                        void setDurationElection ( const std::string& durationElection );

                        /**
                         * @brief	Method getter duration election.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	07/05/2011
                         * @return	duration election.
                         */
                        std::string getDurationElection();

                        /**
                         * @brief	Validates all codi files.
                         * @author	Yadickson Soto
                         * @date	31/10/2012
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool validate();

                        /**
                         * @brief	Method getter languages.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	16/06/2011
                         * @return	languages.
                         */
                        std::list <std::string> getLanguages();

                        /**
                         * @brief	Method getter pv file names.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	02/03/2012
                         * @param	file		 	file name
                         * @param [in,out]	fileNames	List of names of the files.
                         */
                        void getPVFileNames ( std::string file, std::list <std::string> & fileNames );

                        /**
                         * @brief   Method getter vote results.
                         * @author  eduardo.fernandez
                         * @date    15/02/2018
                         * @return  vote results.
                         */
                        VoteResults getVoteResults();

                        /**
                         * @brief   Method getter total votes.
                         * @author  eduardo.fernandez
                         * @date    15/02/2018
                         * @return  total votes.
                         */
                        size_t getTotalVotes();

                        /**
                         * @brief	Method delete all files.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	02/03/2012
                         * @param directories saes directories
                         */
                        void deleteFiles();

                    private:

                        std::string durationElection; /**< duration electionduration of the election day*/
                        SAES::Runtime::Environment::SaesDirectories * directories; ///< saes directories
                        Voting::ElectoralConfiguration * electoralConfiguration; /**< electoral configuration*/
                        Voting::OperationStatus * operationStatus; /**< operation status*/
                        ActivatedCardStatisticsWrapper * activatedCardStatisticsWrapper; /**< activated card statistics*/
                        VoteStatisticsWrapper * voteStatisticsWrapper; /**< vote statistics wrapper*/
                        Voting::Vote::StationTallyWrapper * stationTallyWrapper; ///< The station tally wrapper
                        Voting::Vote::TransmissionPackageWrapper * transmissionPackageWrapper; ///< The transmission package wrapper
                        Voting::Eml::EmlFileManager * emlFiles; ///< EML file manager
                        VoteResults m_voteResults;
                        std::string fileFullPath; ///< file full path dom
                        Smartmatic::System::XSLFO::XSLFOTransformer * transformer;	///< The transformer
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif /* _PVFORM_WRAPPER_H_ */
