/**
 * @file	activated-card-statistics-wrapper.h
 * @brief	Declares the activated card statistics wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef _ACTIVATED_CARD_STATISTICS_WRAPPER_H_
#define _ACTIVATED_CARD_STATISTICS_WRAPPER_H_

#include <System/Exception/CryptoException.hxx>
#include <System/Exception/SignedException.hxx>
#include <System/Exception/XmlException.hxx>
#include <Voting/PBReports/activated-card-statistics-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/CardData/card-data-schema.hxx>
#include <Runtime/Environment/saes-directories.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            namespace PBReports
            {
                /**
                 * @class	ActivatedCardStatisticsWrapper
                 * @brief	Header class ActivatedCardStatisticsWrapper.
                 * @author	Marcel Mancini &lt;marcel.mancini@smartmatic.com&gt;
                 * @date	29-03-2011
                 */

                class ActivatedCardStatisticsWrapper
                {
                    private:
                        ActivatedCardStatistics * cardStatistics;	///< The card statistics
                        Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories; ///< saes directories

                        /**
                         * @fn	bool ActivatedCardStatisticsWrapper::Load(std::string xmlFile,
                         * 			Smartmatic::SAES::Voting::ElectoralConfiguration * elecConf) throw (Smartmatic::System::Exception::CryptoException,
                         * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                         * @brief	Load the Activated Card Statistics.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                         * 																														occurs.
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                         * 																														occurs.
                         * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                         * 																														occurs.
                         * @param	xmlFile 	The std::string to load.
                         * @param	elecConf	If non-null, the elec conf.
                         * @return	true is load file.
                         */

                        bool Load ( std::string xmlFile, Smartmatic::SAES::Voting::ElectoralConfiguration * elecConf )
                        throw ( Smartmatic::System::Exception::CryptoException,
                                Smartmatic::System::Exception::SignedException,
                                Smartmatic::System::Exception::XmlException );

                        Smartmatic::SAES::Voting::VotingDevice::VotingDeviceWrapper * votingDeviceWrapper; ///< The voting device wrapper
                        Smartmatic::SAES::Voting::VotingDevice::Voting_device vd;   ///< The voting device
                    public:

                        /**
                         * @fn	ActivatedCardStatisticsWrapper::ActivatedCardStatisticsWrapper();
                         * @brief	Class constructor.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        ActivatedCardStatisticsWrapper();

                        /**
                         * @fn	virtual ActivatedCardStatisticsWrapper::~ActivatedCardStatisticsWrapper();
                         * @brief	Class destructorer.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        virtual ~ActivatedCardStatisticsWrapper();

                        /**
                         * @fn	void ActivatedCardStatisticsWrapper::Persist() throw (Smartmatic::System::Exception::CryptoException,
                         * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                         * @brief	Persists.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                         * 																														occurs.
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                         * 																														occurs.
                         * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                         * 																														occurs.
                         */

                        void Persist()
                        throw ( Smartmatic::System::Exception::CryptoException,
                                Smartmatic::System::Exception::SignedException,
                                Smartmatic::System::Exception::XmlException );

                        /**
                         * @fn	bool ActivatedCardStatisticsWrapper::init (Smartmatic::SAES::Voting::ElectoralConfiguration * elecConf) throw (Smartmatic::System::Exception::CryptoException,
                         * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                         * @brief	Initialises this object.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                         * 																														occurs.
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                         * 																														occurs.
                         * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                         * 																														occurs.
                         * @param directories saes directories
                         * @param [in,out]	elecConf	If non-null, the elec conf.
                         * @return	true is load file.
                         */

                        bool init ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories, Smartmatic::SAES::Voting::ElectoralConfiguration * elecConf )
                        throw ( Smartmatic::System::Exception::CryptoException,
                                Smartmatic::System::Exception::SignedException,
                                Smartmatic::System::Exception::XmlException );

                        /**
                         * @fn	ActivatedCardStatistics* ActivatedCardStatisticsWrapper::getCardStatistics();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	null if it fails, else the card statistics.
                         */
                        ActivatedCardStatistics * getCardStatistics();

                        /**
                        * @fn    void ActivatedCardStatisticsWrapper::RefreshStats(Smartmatic::SAES::Voting::CardData::Card_data & card) throw (Smartmatic::System::Exception::CryptoException,
                        *             Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                        * @brief    Refresh stats.
                        * @author    Youssef.tourkmani
                        * @date    31/10/2012
                        * @exception    Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                        *                         																								occurs.
                        * @exception    Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                        *                         																								occurs.
                        * @exception    Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                        *                         																								occurs.
                        * @param    card    The card.
                        */
                        void RefreshStats ( Smartmatic::SAES::Voting::CardData::Card_data & card )
                        throw ( Smartmatic::System::Exception::CryptoException,
                                Smartmatic::System::Exception::SignedException,
                                Smartmatic::System::Exception::XmlException );

                        /**
                         * @fn	bool ActivatedCardStatisticsWrapper::RestoreFile(Smartmatic::SAES::Voting::ElectoralConfiguration * elecConf);
                         * @brief	Restore file.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param [in,out]	elecConf	If non-null, the elec conf.
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool RestoreFile ( Smartmatic::SAES::Voting::ElectoralConfiguration * elecConf );


                        /**
                         * @fn	void ActivatedCardStatisticsWrapper::Dispose ();
                         * @brief	Performs application-defined tasks associated with freeing, releasing, or resetting
                         * 				unmanaged resources.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void Dispose ();

                        /**
                         * @fn	void ActivatedCardStatisticsWrapper::IncrementTotalActivated();
                         * @brief	Increment total activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void IncrementTotalActivated();
                        /**
                         * @fn	void ActivatedCardStatisticsWrapper::IncrementTotalNonAudioActivated();
                         * @brief	Increment total non audio activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void IncrementTotalNonAudioActivated();
                        /**
                         * @fn	void ActivatedCardStatisticsWrapper::IncrementTotalAudioActivated();
                         * @brief	Increment total audio activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void IncrementTotalAudioActivated();
                        /**
                         * @fn	void ActivatedCardStatisticsWrapper::IncrementTotalOfficialActivated();
                         * @brief	Increment total official activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void IncrementTotalOfficialActivated();
                        /**
                         * @fn	void ActivatedCardStatisticsWrapper::IncrementTotalDemoActivated();
                         * @brief	Increment total demo activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void IncrementTotalDemoActivated();
                        /**
                         * @fn	void ActivatedCardStatisticsWrapper::IncrementTotalAudioOfficialActivated();
                         * @brief	Increment total audio official activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void IncrementTotalAudioOfficialActivated();
                        /**
                         * @fn	void ActivatedCardStatisticsWrapper::IncrementTotalAudioDemoActivated();
                         * @brief	Increment total audio demo activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void IncrementTotalAudioDemoActivated();
                        /**
                         * @fn	void ActivatedCardStatisticsWrapper::IncrementTotalNonAudioOfficialActivated();
                         * @brief	Increment total non audio official activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void IncrementTotalNonAudioOfficialActivated();
                        /**
                         * @fn	void ActivatedCardStatisticsWrapper::IncrementTotalNonAudioDemoActivated();
                         * @brief	Increment total non audio demo activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        void IncrementTotalNonAudioDemoActivated();
                        /**
                         * @fn	unsigned int ActivatedCardStatisticsWrapper::getTotalActivated();
                         * @brief	Gets total activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	The total activated.
                         */
                        unsigned int getTotalActivated();
                        /**
                         * @fn	unsigned int ActivatedCardStatisticsWrapper::getTotalNonAudioActivated();
                         * @brief	Gets total non audio activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	The total non audio activated.
                         */
                        unsigned int getTotalNonAudioActivated();
                        /**
                         * @fn	unsigned int ActivatedCardStatisticsWrapper::getTotalAudioActivated();
                         * @brief	Gets total audio activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	The total audio activated.
                         */
                        unsigned int getTotalAudioActivated();
                        /**
                         * @fn	unsigned int ActivatedCardStatisticsWrapper::getTotalOfficialActivated();
                         * @brief	Gets total official activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	The total official activated.
                         */
                        unsigned int getTotalOfficialActivated();
                        /**
                         * @fn	unsigned int ActivatedCardStatisticsWrapper::getTotalDemoActivated();
                         * @brief	Gets total demo activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	The total demo activated.
                         */
                        unsigned int getTotalDemoActivated();
                        /**
                         * @fn	unsigned int ActivatedCardStatisticsWrapper::getTotalAudioOfficialActivated();
                         * @brief	Gets total audio official activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	The total audio official activated.
                         */
                        unsigned int getTotalAudioOfficialActivated();
                        /**
                         * @fn	unsigned int ActivatedCardStatisticsWrapper::getTotalAudioDemoActivated();
                         * @brief	Gets total audio demo activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	The total audio demo activated.
                         */
                        unsigned int getTotalAudioDemoActivated();
                        /**
                         * @fn	unsigned int ActivatedCardStatisticsWrapper::getTotalNonAudioOfficialActivated();
                         * @brief	Gets total non audio official activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	The total non audio official activated.
                         */
                        unsigned int getTotalNonAudioOfficialActivated();
                        /**
                         * @fn	unsigned int ActivatedCardStatisticsWrapper::getTotalNonAudioDemoActivated();
                         * @brief	Gets total non audio demo activated.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	The total non audio demo activated.
                         */
                        unsigned int getTotalNonAudioDemoActivated();

                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };

            }
        }
    }
}

#endif // _ACTIVATED_CARD_STATISTICS_WRAPPER_H_
