/**
 * @file	VMProcessStatusWrapper.cxx
 * @brief	Body voting machine process status wrapper class.
 * @author	Yadickson Soto
 * @date	07/11/2012
 */

#include "VMProcessStatusWrapper.hxx"
#include <Voting/set-language-functionality.h>
#include <Resources/election-string-resources-manager.h>
#include <System/IO/File.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::Log;

VMProcessStatusWrapper * VMProcessStatusWrapper::instance = NULL;
ISMTTLog * VMProcessStatusWrapper::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.Status.VMProcessStatusWrapper", GETTEXT_PACKAGE );


/**
 * @fn static bool sortProcess ( MachineProcessStatus * p1, MachineProcessStatus * p2 )
 * @brief method sort registers
 * @param p1 process one
 * @param p2 process two
 * @return true is p1 < p2
 * @date 20/11/2012
 * @author Yadickson Soto
 */
static bool sortProcess ( MachineProcessStatus * p1, MachineProcessStatus * p2 )
{
    std::string t1 = p1->getName();
    std::string t2 = p2->getName();
    return ( t1.compare ( t2 ) < 0 );
}

VMProcessStatusWrapper::VMProcessStatusWrapper ( )
{
    currentMachine = NULL;
    countFail = 0;
    countSuccess = 0;
}

VMProcessStatusWrapper::~VMProcessStatusWrapper()
{

}

VMProcessStatusWrapper * VMProcessStatusWrapper::getInstance()
{
    if ( instance == NULL )
    {
        instance = new VMProcessStatusWrapper();
    }

    return instance;
}

void VMProcessStatusWrapper::init ( const std::string & currentLanguage, const std::string & currentCountry, const std::string & currentMachine )
{
    persistencePartial ();

    this->currentLanguage = currentLanguage;
    this->currentCountry = currentCountry;
    this->currentMachine = new MachineProcessStatus ( currentMachine, true );
}

void VMProcessStatusWrapper::add ( const std::string & domain, const std::string & key, const std::string & resource, bool status )
{
    if ( currentMachine )
    {
        InfoStatus info ( "", key, resource, status, domain );
        translate ( info );
        currentMachine->getInfoStatus().push_back ( info );
        currentMachine->setStatus ( currentMachine->getStatus() && status );
        logger->Debug ( key );
    }
}

void VMProcessStatusWrapper::addSuccess ( const std::string & domain, const std::string & key, const std::string resource )
{
    add ( domain, key, resource, true );
}

void VMProcessStatusWrapper::addFail ( const std::string & key )
{
    add ( "", key, "", false );
}

void VMProcessStatusWrapper::addFail ( const std::string & domain, const std::string & key, std::string resource )
{
    add ( domain, key, resource, false );
}

bool VMProcessStatusWrapper::isFail()
{
    return countFail > 0;
}

void VMProcessStatusWrapper::getAllByStatus ( std::list < MachineProcessStatus * > & list, bool status )
{
    list.clear();

    std::list < MachineProcessStatus * >::iterator it;

    for ( it = listMachineStatus.begin();
            it != listMachineStatus.end();
            ++it )
    {
        if ( ( *it )->getStatus() == status )
        {
            list.push_back ( *it );
        }
    }

    list.sort ( sortProcess );
}

void VMProcessStatusWrapper::getAllSuccess ( std::list < MachineProcessStatus * > & list )
{
    getAllByStatus ( list, true );
}

void VMProcessStatusWrapper::getAllFail ( std::list < MachineProcessStatus * > & list )
{
    getAllByStatus ( list, false );
}

void VMProcessStatusWrapper::dispose()
{
    logger->Debug ( __func__ );

    std::list < MachineProcessStatus * >::iterator it;

    for ( it = listMachineStatus.begin();
            it != listMachineStatus.end();
            ++it )
    {
        MachineProcessStatus * pointer = ( *it );
        delete pointer;
        pointer = NULL;
    }

    listMachineStatus.clear();

    if ( currentMachine )
    {
        delete currentMachine;
        currentMachine = NULL;
    }

    countFail = 0;
    countSuccess = 0;
}

void VMProcessStatusWrapper::translate ( InfoStatus & info )
{
    std::string key = info.getKey();
    std::string domain = info.getDomain();

    if ( !domain.empty() )
    {
        std::string language, country;
        SetLanguageFunctionality::getCurrentLanguageCountry ( language, country );
        SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( currentLanguage, currentCountry );

        std::string text;
        std::string resource;

        text = dgettext ( domain.c_str(), key.c_str() );
        resource = ElectionStringResourcesManager::getInstance()->getResource ( info.getResource() );
        info.setText ( text + " " + resource );

        SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( language, country );
    }
    else
    {
        info.setText ( key );
    }
}

int VMProcessStatusWrapper::getFailCount()
{
    return countFail;
}

int VMProcessStatusWrapper::getSuccessCount()
{
    return countSuccess;
}

void VMProcessStatusWrapper::start()
{
    safeTime.start();
    countFail = 0;
    countSuccess = 0;
}

void VMProcessStatusWrapper::stop()
{
    safeTime.stop();
}

std::string VMProcessStatusWrapper::getTime ()
{
    return safeTime.getStringTime();
}

void VMProcessStatusWrapper::persistencePartial()
{
    SaesDirectories directories;
    std::string file = directories.getProcessStatusResultsFileFullPath();

    if ( !currentMachine )
    {
        File::getInstance()->DeleteFile ( file );
        return;
    }

    std::stringstream ss;

    ss << ( currentMachine->getStatus() ? "(OK)   : " : "(FAIL) : " );
    ss << currentMachine->getName() << std::endl;

    MachineProcessStatus *pointer = new MachineProcessStatus ( currentMachine->getName(), currentMachine->getStatus() );

    MachineProcessStatus::InfoStatusIterator infoIt;
    MachineProcessStatus::InfoStatusSequence & seq ( currentMachine->getInfoStatus() );

    for ( infoIt = seq.begin();
            infoIt != seq.end();
            ++infoIt )
    {
        if ( ( *infoIt ).getStatus() )
        {
            ss << " - " << "OK " << " - " << ( *infoIt ).getText() << std::endl;
        }
        else
        {
            ss << " - " << "NOK" << " - " << ( *infoIt ).getText() << std::endl;
            pointer->getInfoStatus().push_back ( ( *infoIt ) );
        }

        pointer->setStatus ( ( *infoIt ).getStatus() );
    }

    if ( currentMachine->getStatus() )
    {
        countSuccess++;
    }
    else
    {
        countFail++;
    }

    ss << std::endl;

    listMachineStatus.push_back ( pointer );

    Smartmatic::System::SafeBuffer buffer ( ss.str() );
    File::getInstance()->addBufferToFile ( buffer, file );

    delete currentMachine;
    currentMachine = NULL;
}

void VMProcessStatusWrapper::persistence()
{
    logger->Debug ( __func__ );

    persistencePartial ();

    SaesDirectories directories;
    std::string file = directories.getProcessStatusResultsFileFullPath();

    std::stringstream ss;

    ss << std::endl;
    ss << std::endl;
    ss << "OK     : " << countSuccess << std::endl;
    ss << "FAIL   : " << countFail << std::endl;
    ss << "TOTAL  : " << countSuccess + countFail << std::endl;
    ss << "TIME   : " << getTime() << std::endl;
    ss << std::endl;

    Smartmatic::System::SafeBuffer buffer ( ss.str() );
    File::getInstance()->addBufferToFile ( buffer, file );
}
