/**
 * @file	VoteCode.cxx
 * @brief	vote code class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include "VoteCode.hxx"
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Log/SMTTLogManager.h>
#include <Security/Encryption.hxx>
#include <Security/KeyManager.hxx>
#include <System/Encoding/Base64Encoder.h>
#include <System/Exception/ElectionException.hxx>
#include <System/Security/CRC32Calculator.hxx>
#include <System/Security/Cryptography/MsgDigestUtil.h>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;

static const size_t EXPECTED_MESSAGE_SIZE = 48;

ISMTTLog* VoteCode::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Voting.Vote.VoteCode",
    GETTEXT_PACKAGE);

VoteCode::VoteCode()
{
}

VoteCode::~VoteCode()
{
}

std::string VoteCode::generateCbcMac(const Vote& vote)
{
    SafeBuffer code;
    SafeBuffer checksum;
    SafeBuffer cbcMac;
    std::string cbcMacBase64;

    m_logger->Debug("Generate cbc-mac");

    Base64Encoder::ConvertFromBase64(vote.getBallotID(), code);
    generateChecksum(vote, checksum);

    getCbcMac(code, checksum, cbcMac);
    cbcMacBase64 = cbcMac.GetBase64Data();

    m_logger->Debug("code = " + code.GetBase64Data());
    m_logger->Debug("checksum = " + checksum.GetBase64Data());
    m_logger->Debug("cbc-mac = " + cbcMacBase64);

    return cbcMacBase64;
}

void VoteCode::getSelections(const Vote& vote, SafeBuffer& selections)
{
    std::string message;
    Vote::SSequence sSequence = vote.getSelections();
    std::vector<Selection> selection(sSequence.begin(), sSequence.end());
    std::vector<Selection>::iterator its;

    std::sort(selection.begin(), selection.end(), &VoteCode::sortSelectionVote);

    for (its = selection.begin(); its != selection.end(); ++its)
    {
        Selection::BSequence bSequence = its->getBallotOptions();
        std::vector<std::string> ballots(bSequence.begin(), bSequence.end());
        std::vector<std::string>::iterator itb;

        std::sort(ballots.begin(), ballots.end(), &VoteCode::sortBallotVote);

        for (itb = ballots.begin(); itb != ballots.end(); ++itb)
        {
            message = message.append(itb->c_str());
        }
    }

    selections.Clear();
    selections.AddData(message);
}

void VoteCode::generateChecksum(const Vote& vote, SafeBuffer& checksum)
{
    SafeBuffer selections;
    getSelections(vote, selections);

    Cryptography::MsgDigestUtil digest(selections);
    digest.getDigest(checksum, Cryptography::MsgDigestUtil::getDefaultVoteDigest());
}

void VoteCode::getCbcMac(const SafeBuffer& code, const SafeBuffer& checksum,
                              SafeBuffer& dataOut)
{
    SafeBuffer message;

    message.AddData(code);
    message.AddData(checksum);

    if (message.GetSize() != EXPECTED_MESSAGE_SIZE)
    {
        const std::string errorMessage = StringUtils::compose("Message size (%1) is different from the expected (%2).", message.GetSize(), EXPECTED_MESSAGE_SIZE);
        throw ElectionException ( errorMessage, errorMessage, m_logger );
    }

    cryptoCode(message, dataOut);
}

bool VoteCode::validateCbcMac(const Vote& vote)
{
    m_logger->Debug("Validate code");

    SafeBuffer code;
    SafeBuffer checksum;

    Base64Encoder::ConvertFromBase64(vote.getBallotID(), code);
    generateChecksum(vote, checksum);

    bool isValid = false;

    try
    {
        isValid = generateCbcMac(vote) == vote.getCode() && code.GetSize() + checksum.GetSize() == EXPECTED_MESSAGE_SIZE;
    }
    catch (ElectionException& electionException)
    {
        m_logger->Error("Exception caught while validating the vote cbc-mac: " + electionException.getMessage());
    }

    m_logger->Debug("session id    = " + vote.getSessionID());
    m_logger->Debug("ballot id     = " + vote.getBallotID());
    m_logger->Debug(std::string("Vote code is: ") + (isValid ? "Valid" : "Invalid"));

    return isValid;
}

bool VoteCode::sortSelectionVote(const Selection& first, const Selection& second)
{
    std::string contestFirst(first.getContest());
    std::string contestSecond(second.getContest());

    return contestFirst < contestSecond;
}

bool VoteCode::sortBallotVote(const std::string& first, const std::string& second)
{
    return first < second;
}

void VoteCode::cryptoCode(const SafeBuffer& buffer, SafeBuffer& dataOut)
{
    try
    {
        Crypto obj;
        obj.init(Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true);

        SafeBuffer cbcMacKeyBuffer;
        SafeBuffer keyBuffer(Encryption::getInstance()->getCbcMacKey());

        Cryptography::MsgDigestUtil digest(keyBuffer);
        digest.getDigest(cbcMacKeyBuffer, Cryptography::MsgDigestUtil::getDefaultVoteDigest());

        obj.encryptBufferToBufferAndCreateIV(buffer, dataOut, cbcMacKeyBuffer.GetData(), true);
    }
    catch (const Smartmatic::System::Exception::CryptoException& ex)
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__ << " " << ex.what();
        m_logger->Debug(ss.str());
    }
    catch (...)
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__;
        m_logger->Debug(ss.str());
    }
}

void VoteCode::decryptoCode(const std::string& composeCode, SafeBuffer& dataOut)
{
    try
    {
        SafeBuffer key;
        SafeBuffer data;
        Crypto obj;

        Base64Encoder::ConvertFromBase64(composeCode, data);
        obj.init(Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, false);

        Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getKey(
            Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY, key);

        obj.decryptBufferToBufferAndGetIV(data, dataOut, key.GetData(), true);
    }
    catch (const Smartmatic::System::Exception::CryptoException& ex)
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__ << " " << ex.what();
        m_logger->Debug(ss.str());
    }
    catch (...)
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__;
        m_logger->Debug(ss.str());
    }
}


void VoteCode::encode(std::string& cbcMac)
{
    unsigned int group = SAESConfigurationManager::getInstance()->getVoteCodeConfig().getCountGroupVoteCode();

    if (cbcMac.length() > group && group > 0)
    {
        std::string character = SAESConfigurationManager::getInstance()->getVoteCodeConfig().getCharSeparatorVoteCode();
        std::string code;
        size_t parts = cbcMac.length() / group - 1;

        for (size_t i = 0; i <= parts; ++i)
        {
            if (i != 0)
            {
                code.append(character);
            }

            code.append(cbcMac.substr(0, group));
            cbcMac.erase(0, group);
        }

        if (cbcMac.length() > 0 && code.length() > 0)
        {
            code.append(character);
        }

        code.append(cbcMac);
        cbcMac = code;
    }
}

void VoteCode::decode(std::string& cbcMac)
{
    unsigned int group = SAESConfigurationManager::getInstance()->getVoteCodeConfig().getCountGroupVoteCode();

    if (group > 0)
    {
        std::string find = SAESConfigurationManager::getInstance()->getVoteCodeConfig().getCharSeparatorVoteCode();
        cbcMac = Smartmatic::System::Utils::StringUtils::replaceAll(cbcMac, find, "");
    }
}
