#ifndef TEST_HASH_VOTE_MANAGER
#define TEST_HASH_VOTE_MANAGER

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Voting/Vote/VoteFactory.hxx>
#include <Voting/Vote/VoteWrapper.hxx>
#include <System/Security/OpenSSLSecurity.hxx>
#include "MockHashVoteManager.hxx"

class HashVoteManagerTest : public CPPUNIT_NS::TestFixture
{
    private:

        Smartmatic::SAES::Voting::Vote::Vote * vote;

    public:

        HashVoteManagerTest ()
        {
            vote = NULL;
        }

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();

            vote = Smartmatic::SAES::Voting::Vote::VoteFactory::createVote();
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();

            if ( vote )
            {
                delete vote;
                vote = NULL;
            }
        }

        void testProcessVoteAndCheckListSize()
        {
            MockHashVoteManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockHashVoteManager, getHash, ext ) getHash_mock ( &manager );

            getHash_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub< std::string > ( "xxyyzz" ) );

            manager.processVote ( *vote );

            CPPUNIT_ASSERT ( false == manager.getFileNameList().empty() );
            manager.verify();
        }

        void testProcessVoteAndValidateOtherFileName()
        {
            MockHashVoteManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockHashVoteManager, getHash, ext ) getHash_mock ( &manager );

            getHash_mock
            .expects ( MOCKPP_NS::atLeast ( 1 ) )
            .will ( new MOCKPP_NS::ReturnStub< std::string > ( "xxyyzz" ) );

            manager.processVote ( *vote );

            CPPUNIT_ASSERT ( false == manager.validateVote ( "xxmm" ) );
            manager.verify();
        }

        void testProcessVoteAndValidateHash()
        {
            MockHashVoteManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockHashVoteManager, getHash, ext ) getHash_mock ( &manager );

            getHash_mock
            .expects ( MOCKPP_NS::atLeast ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub< std::string > ( "xxyyzz" ),
                        new MOCKPP_NS::ReturnStub< std::string > ( "xxyyzz" ) ) );

            manager.processVote ( *vote );
            std::string filename = Smartmatic::SAES::Voting::Vote::VoteWrapper::getFileName ( *vote );

            CPPUNIT_ASSERT ( true == manager.validateVote ( filename ) );
            manager.verify();
        }

        void testProcessVoteAndValidateChangeHash()
        {
            MockHashVoteManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockHashVoteManager, getHash, ext ) getHash_mock ( &manager );

            getHash_mock
            .expects ( MOCKPP_NS::atLeast ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub< std::string > ( "xxyyzz" ),
                        new MOCKPP_NS::ReturnStub< std::string > ( "mnop" ) ) );

            manager.processVote ( *vote );
            std::string filename = Smartmatic::SAES::Voting::Vote::VoteWrapper::getFileName ( *vote );

            CPPUNIT_ASSERT ( false == manager.validateVote ( filename ) );
            manager.verify();
        }

        void testProcessVoteAndClear()
        {
            MockHashVoteManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockHashVoteManager, getHash, ext ) getHash_mock ( &manager );

            getHash_mock
            .expects ( MOCKPP_NS::atLeast ( 1 ) )
            .will ( new MOCKPP_NS::ReturnStub< std::string > ( "xxyyzz" ) );

            manager.processVote ( *vote );
            manager.clear();

            CPPUNIT_ASSERT ( true == manager.getFileNameList().empty() );
            manager.verify();
        }

        CPPUNIT_TEST_SUITE ( HashVoteManagerTest );
        CPPUNIT_TEST ( testProcessVoteAndCheckListSize );
        CPPUNIT_TEST ( testProcessVoteAndValidateOtherFileName );
        CPPUNIT_TEST ( testProcessVoteAndValidateHash );
        CPPUNIT_TEST ( testProcessVoteAndValidateChangeHash );
        CPPUNIT_TEST ( testProcessVoteAndClear );
        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_HASH_VOTE_MANAGER */
