/**
 * @file ConnectionBehavior.hxx
 * @brief Declaration of the high-level behaviour of the connection to the smartcard reader and provides operations of connection, commands, and manages error cases.
 * @author Yadickson Soto
 * @date 28/03/2014
 */

#ifndef _CONNECTION_BEHAVIOR_HXX_
#define _CONNECTION_BEHAVIOR_HXX_

#include <Card/ConnectionDriverInterface.hxx>
#include <memory>
#include <ctime>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Card
        {
            class ISmartcardReaderPowerManager;

            /**
             * @class ConnectionBehavior
             * @brief This class defines the behavior of the connection to the smartcard reader's driver.
             * It manages the connection, sends commands, gets responses and is in charge of the behavior in error cases.
             * It will in particular be in charge of restarting the reader (via power control), the daemon, when needed.
             * @author Yadickson Soto
             * @date 28/03/2014
             */
            class ConnectionBehavior
            {
                public:

                    /**
                     * @fn ConnectionBehavior ( unsigned int numberOfTriesToRead = 1 );
                     * @brief Class constructor.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param numberOfTriesToRead The maximum number of tries to read from the smartcard
                     */
                    explicit ConnectionBehavior ( unsigned int numberOfTriesToRead = 1,
                                                  ISmartcardReaderPowerManager * powerManager = NULL );

                    /**
                     * @fn virtual ~ConnectionBehavior();
                     * @brief Class destructor.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     */
                    virtual ~ConnectionBehavior();

                    /**
                     * @fn virtual bool establishContext();
                     * @brief Establishes context with the smartcard reader device.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return true if success
                     */
                    virtual bool establishContext ();

                    /**
                     * @fn virtual bool releaseContext() = 0;
                     * @brief Releases context with the smartcard reader device.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return true if success
                     */
                    virtual bool releaseContext();

                    /**
                     * @fn virtual bool isCardPresent() = 0;
                     * @brief Whether a card is inserted in the reader.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @return true if present
                     */
                    virtual bool isCardPresent () = 0;

                    /**
                     * @fn virtual bool connect () = 0;
                     * @brief Opens a connection to the smartcard reader.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return true if successful.
                     */
                    virtual bool connect () = 0;

                    /**
                     * @fn virtual bool disconnect() = 0;
                     * @brief Disconnects from the device.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return true if successful.
                     */
                    virtual bool disconnect() = 0;

                    /**
                     * @fn virtual bool selectAID ( const Smartmatic::System::SafeBuffer & aid );
                     * @brief Gets the card's applet by ID.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param aid The aid.
                     * @return true if successful.
                     */
                    virtual bool selectAID ( const Smartmatic::System::SafeBuffer & aid );

                    /**
                     * @fn virtual bool sendCommand ( const Smartmatic::System::SafeBuffer & command );
                     * @brief Sends a command to the reader.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param command Command is sent by the reader to the card.
                     * @return true if successful.
                     */
                    virtual bool sendCommand ( const Smartmatic::System::SafeBuffer & command );

                    /**
                     * @fn virtual Smartmatic::System::SafeBuffer sendCommandWithResponse ( const Smartmatic::System::SafeBuffer & command );
                     * @brief Sends a command to reader which will return a response.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param command Command to be sent by the reader to the card.
                     * @return the response buffer
                     */
                    virtual Smartmatic::System::SafeBuffer sendCommandWithResponse ( const Smartmatic::System::SafeBuffer & command );

                protected:

                    /**
                     * @fn virtual bool processConnectionStatus ( unsigned long status );
                     * @brief Acts on the connection based on a status code (restarts the reader when needed).
                     * @author Yadickson Soto
                     * @date 28/05/2014
                     * @param status The status response to check.
                     * @return true if status is zero "0"
                     */
                    virtual bool processConnectionStatus ( unsigned long status );

                    /**
                     * @fn virtual bool checkConnectionStatus ( unsigned long status );
                     * @brief Checks whether a status code is "good" or "bad".
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param status The status response to check.
                     * @return true if status is a "good" status.
                     */
                    virtual bool checkConnectionStatus ( unsigned long status ) = 0;

                protected:

                    ConnectionDriverInterface * connectionDriver; ///< The instance of the driver for the reader which implements elementary connection operations.

                private:

                    unsigned int                                numberOfTriesToRead;
                    static Smartmatic::Log::ISMTTLog *          logger;
                    std::auto_ptr<ISmartcardReaderPowerManager> m_powerManager;
                    std::time_t                                 m_lastRestart;
            };
        }
    }
}

#endif /* _CONNECTION_BEHAVIOR_HXX_ */
