/**
 * @file GlobalPlatformConnectionBehavior.cxx
 * @brief GlobalPlatform-specific behavior for managing the connection when using this type of connection. GlobalPlatform enables secure channels.
 * @author Yadickson Soto
 * @date 28/03/2014
 */

#include <sstream>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include "GlobalPlatformConnectionBehavior.hxx"
#include "GlobalPlatformConnectionDriver.hxx"
#include <Card/ISmartcardReaderPowerManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * GlobalPlatformConnectionBehavior::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Card.GlobalPlatformConnectionBehavior", GETTEXT_PACKAGE );

GlobalPlatformConnectionBehavior::GlobalPlatformConnectionBehavior ( unsigned int numberOfTriesToRead,
                                                                     ConnectionDriverInterface * connectionDriver,
                                                                     ISmartcardReaderPowerManager * powerManager)
    : ConnectionBehavior ( numberOfTriesToRead, powerManager )
{
    this->connectionDriver = ( connectionDriver == NULL ? new GlobalPlatformConnectionDriver() : connectionDriver );
}

GlobalPlatformConnectionBehavior::~GlobalPlatformConnectionBehavior()
{

}

bool GlobalPlatformConnectionBehavior::isCardPresent()
{
    unsigned long status = connectionDriver->getReaderStatus();

    switch ( status )
    {
        case SCARD_S_SUCCESS:
            return true;

        case SCARD_E_NO_SMARTCARD:
            return false;

        default:
            return processConnectionStatus ( status );
    }
}

bool GlobalPlatformConnectionBehavior::connect ()
{
    return processConnectionStatus ( connectionDriver->connect () );
}

bool GlobalPlatformConnectionBehavior::disconnect()
{
    unsigned long status = connectionDriver->disconnect();

    switch ( status )
    {
        case SCARD_E_INVALID_HANDLE:
            return true;

        default:
            return processConnectionStatus ( status );
    }
}

bool GlobalPlatformConnectionBehavior::checkConnectionStatus ( unsigned long status )
{
    std::stringstream ss;
    ss << "Error code (" << std::hex << status << ") - ";

    switch ( status )
    {
        case SCARD_S_SUCCESS:
            return true;

        case SCARD_F_INTERNAL_ERROR:
            ss << "An internal consistency check failed.";
            break;

        case SCARD_E_CANCELLED:
            ss << "The action was cancelled by an SCardCancel request.";
            break;

        case SCARD_E_INVALID_HANDLE:
            ss << "The supplied handle was invalid.";
            break;

        case SCARD_E_INVALID_PARAMETER:
            ss << "One or more of the supplied parameters could not be properly interpreted.";
            break;

        case SCARD_E_INVALID_TARGET:
            ss << "Registry startup information is missing or invalid.";
            break;

        case SCARD_E_NO_MEMORY:
            ss << "Not enough memory available to complete this command.";
            break;

        case SCARD_F_WAITED_TOO_LONG:
            ss << "An internal consistency timer has expired.";
            break;

        case SCARD_E_INSUFFICIENT_BUFFER:
            ss << "The data buffer to receive returned data is too small for the returned data.";
            break;

        case SCARD_E_UNKNOWN_READER:
            ss << "The specified reader name is not recognized.";
            break;

        case SCARD_E_TIMEOUT:
            ss << "The user-specified timeout value has expired.";
            break;

        case SCARD_E_SHARING_VIOLATION:
            ss << "The smart card cannot be accessed because of other connections outstanding.";
            break;

        case SCARD_E_NO_SMARTCARD:
            ss << "The operation requires a Smart Card, but no Smart Card is currently in the device.";
            break;

        case SCARD_E_UNKNOWN_CARD:
            ss << "The specified smart card name is not recognized.";
            break;

        case SCARD_E_CANT_DISPOSE:
            ss << "The system could not dispose of the media in the requested manner.";
            break;

        case SCARD_E_PROTO_MISMATCH:
            ss << "The requested protocols are incompatible with the protocol currently in use with the smart card.";
            break;

        case SCARD_E_NOT_READY:
            ss << "The reader or smart card is not ready to accept commands.";
            break;

        case SCARD_E_INVALID_VALUE:
            ss << "One or more of the supplied parameters values could not be properly interpreted.";
            break;

        case SCARD_E_SYSTEM_CANCELLED:
            ss << "The action was cancelled by the system, presumably to log off or shut down.";
            break;

        case SCARD_F_COMM_ERROR:
            ss << "An internal communications error has been detected.";
            break;

        case SCARD_F_UNKNOWN_ERROR:
            ss << "An internal error has been detected, but the source is unknown.";
            break;

        case SCARD_E_INVALID_ATR:
            ss << "An ATR obtained from the registry is not a valid ATR string.";
            break;

        case SCARD_E_NOT_TRANSACTED:
            ss << "An attempt was made to end a non-existent transaction.";
            break;

        case SCARD_E_READER_UNAVAILABLE:
            ss << "The specified reader is not currently available for use.";
            break;

        case SCARD_P_SHUTDOWN:
            ss << "The operation has been aborted to allow the server application to exit.";
            break;

        case SCARD_E_PCI_TOO_SMALL:
            ss << "The PCI Receive buffer was too small.";
            break;

        case SCARD_E_READER_UNSUPPORTED:
            ss << "The reader driver does not meet minimal requirements for support.";
            break;

        case SCARD_E_DUPLICATE_READER:
            ss << "The reader driver did not produce a unique reader name.";
            break;

        case SCARD_E_CARD_UNSUPPORTED:
            ss << "The smart card does not meet minimal requirements for support.";
            break;

        case SCARD_E_NO_SERVICE:
            ss << "The Smart card resource manager is not running.";
            break;

        case SCARD_E_SERVICE_STOPPED:
            ss << "The Smart card resource manager has shut down.";
            break;

        case SCARD_E_UNEXPECTED:
            ss << "An unexpected card error has occurred.";
            break;

        case SCARD_E_ICC_INSTALLATION:
            ss << "No primary provider can be found for the smart card.";
            break;

        case SCARD_E_ICC_CREATEORDER:
            ss << "The requested order of object creation is not supported.";
            break;

        case SCARD_E_NO_READERS_AVAILABLE:
            ss << "Cannot find a smart card reader.";
            break;

        case SCARD_E_COMM_DATA_LOST:
            ss << "A communications error with the smart card has been detected. Retry the operation.";
            break;

        case SCARD_W_UNSUPPORTED_CARD:
            ss << "The reader cannot communicate with the card, due to ATR string configuration conflicts.";
            break;

        case SCARD_W_UNRESPONSIVE_CARD:
            ss << "The smart card is not responding to a reset.";
            break;

        case SCARD_W_UNPOWERED_CARD:
            ss << "Power has been removed from the smart card, so that further communication is not possible.";
            break;

        case SCARD_W_RESET_CARD:
            ss << "The smart card has been reset, so any shared state information is invalid.";
            break;

        case SCARD_W_REMOVED_CARD:
            ss << "The smart card has been removed, so further communication is not possible.";
            break;

        default:
            ss << "Unknown";
            break;
    }

    logger->Debug ( ss.str() );

    return false;
}
