/**
 * @file GlobalPlatformConnectionBehavior.hxx
 * @brief GlobalPlatform-specific behavior for managing the connection when using this type of connection. GlobalPlatform enables secure channels.
 * @author Yadickson Soto
 * @date 28/03/2014
 */

#ifndef _GLOBAL_PLATFORM_CONNECTION_BEHAVIOR_HXX_
#define _GLOBAL_PLATFORM_CONNECTION_BEHAVIOR_HXX_

#include <Card/ConnectionBehavior.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Card
        {
            class ISmartcardReaderPowerManager;

            /**
             * @class GlobalPlatformConnectionBehavior
             * @brief GlobalPlatform-specific behavior for managing the connection when using this type of connection. GlobalPlatform enables secure channels.
             * @author Yadickson Soto
             * @date 28/03/2014
             */
            class GlobalPlatformConnectionBehavior : public ConnectionBehavior
            {
                public:

                    /**
                     * @fn GlobalPlatformConnectionBehavior ( unsigned int numberOfTriesToRead, ConnectionDriverInterface * connectionDriver = NULL );
                     * @brief Class constructor.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param numberOfTriesToRead The maximum number of tries to read from the smartcard before considering it a failure.
                     * @param connectionDriver The instance of the driver for the reader which implements elementary connection operations.
                     */
                    GlobalPlatformConnectionBehavior (
                        unsigned int numberOfTriesToRead,
                        ConnectionDriverInterface * connectionDriver = NULL,
                        ISmartcardReaderPowerManager * powerManager = NULL);

                    /**
                     * @fn virtual ~GlobalPlatformConnectionBehavior();
                     * @brief Class destructor.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     */
                    virtual ~GlobalPlatformConnectionBehavior();

                    /**
                     * @fn virtual bool isCardPresent();
                     * @brief Whether a card is inserted in the reader.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @return true if present
                     */
                    virtual bool isCardPresent ();

                    /**
                     * @fn virtual bool connect ();
                     * @brief Opens a connection to the smartcard reader.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return true if successful.
                     */
                    virtual bool connect ();

                    /**
                     * @fn virtual bool disconnect();
                     * @brief Disconnects from the device.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return true if successful.
                     */
                    virtual bool disconnect();

                protected:

                    /**
                     * @fn virtual bool checkConnectionStatus ( unsigned long status );
                     * @brief Checks whether a status code is "good" or "bad".
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param status The status response to check.
                     * @return true if status is a "good" status.
                     */
                    virtual bool checkConnectionStatus ( unsigned long status );

                private:

                    static Smartmatic::Log::ISMTTLog * logger;  ///<pointer to the logger instance to use.
            };
        }
    }
}

#endif /* _GLOBAL_PLATFORM_CONNECTION_BEHAVIOR_HXX_ */
