/**
 * @file GlobalPlatformConnectionDriver.cxx
 * @brief Implementation of the GlobalPlatform-specific driver, which implements low-level operations of connection, sending commands and getting responses.
 * @date 20/05/2014
 * @author Yadickson Soto
 */

#include <Configuration/SmartcardConfigurationManager.hxx>
#include <Security/KeyManager.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Card/GlobalPlatformConnectionDriver.hxx>
#include <Card/GlobalPlatformModeOP201.hxx>
#include <Card/GlobalPlatformModeGP211.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define BUFLEN 1024 ///< max buffer length
#define DDES_KEY_LEN 16 ///< key length

using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Common::Configuration;

/**
 * PC/SC specific context information. Used in OPGP_CARD_CONTEXT.librarySpecific.
 */
typedef struct
{
    SCARDCONTEXT cardContext; ///<  Reference to the reader resource manager.
} PCSC_CARD_CONTEXT_SPECIFIC;

ISMTTLog * GlobalPlatformConnectionDriver::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Card.GlobalPlatformConnectionDriver", GETTEXT_PACKAGE );

GlobalPlatformConnectionDriver::GlobalPlatformConnectionDriver()
{
    PlatformMode type = SmartcardConfigurationManager::getInstance()->getPlatformMode();

    switch ( type )
    {
        case PlatformMode::CONFIGURATION_OP_201:
            platformMode = new GlobalPlatformModeOP201 ( cardInfo, cardContext );
            break;

        default:
        case PlatformMode::CONFIGURATION_GP_211:
            platformMode = new GlobalPlatformModeGP211 ( cardInfo, cardContext );
            break;
    }

}

GlobalPlatformConnectionDriver::~GlobalPlatformConnectionDriver()
{

}

unsigned long GlobalPlatformConnectionDriver::establishContext ()
{
    logger->Debug ( "Establish Context " );

    std::string nameLib = SmartcardConfigurationManager::getInstance()->getConnectionPluginName();
    std::string nameVersion = SmartcardConfigurationManager::getInstance()->getConnectionPluginVersion();

    OPGP_ERROR_STATUS status;

    memcpy ( cardContext.libraryName, nameLib.c_str(), nameLib.length() + 1 );
    memcpy ( cardContext.libraryVersion, nameVersion.c_str(), nameVersion.length() + 1 );

    cardContext.libraryHandle = NULL;
    cardContext.librarySpecific = NULL;

    status = OPGP_establish_context ( &cardContext );

    if ( !OPGP_ERROR_CHECK ( status ) )
    {
        char buf[BUFLEN + 1];
        unsigned long readerStrLen = BUFLEN;

        status = OPGP_list_readers ( cardContext, buf, &readerStrLen );

        if ( !OPGP_ERROR_CHECK ( status ) )
        {
            readerName.assign ( buf, readerStrLen );
            readerName.resize ( readerStrLen );
        }
    }

    return checkStatus ( status );
}

unsigned long GlobalPlatformConnectionDriver::releaseContext()
{
    logger->Debug ( "Release Context" );

    OPGP_ERROR_STATUS status;

    status = OPGP_release_context ( &cardContext );

    return checkStatus ( status );
}

unsigned long GlobalPlatformConnectionDriver::getReaderStatus ()
{
    SCARDCONTEXT hContext = ( static_cast < PCSC_CARD_CONTEXT_SPECIFIC * > ( cardContext.librarySpecific ) )->cardContext;
    SCARD_READERSTATE rgReaderStates;

    rgReaderStates.szReader =  readerName.c_str();
    rgReaderStates.dwCurrentState = SCARD_STATE_UNAWARE;
    rgReaderStates.dwEventState = 0;

    unsigned long retVal = SCardGetStatusChange ( hContext, 500, &rgReaderStates, 1 );

    if ( rgReaderStates.dwEventState & 0x10 )
    {
        return SCARD_E_NO_SMARTCARD;
    }
    else if ( rgReaderStates.dwEventState & 0x20 )
    {
        return SCARD_S_SUCCESS;
    }

    return retVal;
}

unsigned long GlobalPlatformConnectionDriver::connect ()
{
    logger->Debug ( "Connect" );

    OPGP_ERROR_STATUS status;

    status = OPGP_card_connect ( cardContext, readerName.c_str(), &cardInfo, OPGP_CARD_PROTOCOL_T0 | OPGP_CARD_PROTOCOL_T1 );
    cardInfo.specVersion = platformMode->getPlatformMode();

    return checkStatus ( status );
}

unsigned long GlobalPlatformConnectionDriver::disconnect()
{
    logger->Debug ( "Disconnect" );

    OPGP_ERROR_STATUS status;

    status = OPGP_card_disconnect ( cardContext, &cardInfo );

    return checkStatus ( status );
}

unsigned long GlobalPlatformConnectionDriver::openSecureChannel()
{
    return checkStatus ( platformMode->openSecureChannel() );
}

unsigned long GlobalPlatformConnectionDriver::selectAID ( const Smartmatic::System::SafeBuffer & aid )
{
    logger->Debug ( "Select AID" );

    OPGP_ERROR_STATUS status;

    status = OPGP_select_application ( cardContext, cardInfo, ( unsigned char * ) aid.GetData(), aid.GetSize() );

    if ( !OPGP_ERROR_CHECK ( status ) )
    {
        return openSecureChannel ();
    }

    return checkStatus ( status );
}

unsigned long GlobalPlatformConnectionDriver::sendCommand ( const Smartmatic::System::SafeBuffer & command )
{
    logger->Debug ( "Send command" );
    return checkStatus ( platformMode->sendCommand ( command, responseBuffer ) );
}

Smartmatic::System::SafeBuffer GlobalPlatformConnectionDriver::getResponse ()
{
    return responseBuffer;
}

unsigned long GlobalPlatformConnectionDriver::checkStatus ( const OPGP_ERROR_STATUS & status )
{
    if ( !OPGP_ERROR_CHECK ( status ) )
    {
        return SCARD_S_SUCCESS;
    }

    return ( unsigned long ) status.errorCode;
}
