/**
 * @file JavaCardOperations.hxx
 * @brief Standard generic JavaCard operations
 * @author Yadickson Soto
 * @date 28/03/2014
 */

#ifndef _JAVA_CARD_OPERATIONS_HXX_
#define _JAVA_CARD_OPERATIONS_HXX_

#include <System/SafeBuffer.h>
#include <Card/ConnectionBehavior.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Card
        {

            /**
             * @class JavaCardOperations
             * @brief Standard generic JavaCard operations
             * @author Yadickson Soto
             * @date 28/03/2014
             */
            class JavaCardOperations
            {
                public:

                    /**
                     * @fn JavaCardOperations( ConnectionBehavior * connectionBehavior );
                     * @brief Class constructor.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param	connectionBehavior
                     */
                    explicit JavaCardOperations ( ConnectionBehavior * connectionBehavior );

                    /**
                     * @fn virtual ~JavaCardOperations();
                     * @brief Class destructor.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     */
                    virtual ~JavaCardOperations();

                    /**
                     * @fn virtual bool verifyGlobalPIN( const Smartmatic::System::SafeBuffer & pin );
                     * @brief Verifies the global pin in the smartcard.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param pin Contains the global PIN to verify.
                     * @return true if the global pin coincides; otherwise false.
                     */
                    virtual bool verifyGlobalPIN ( const Smartmatic::System::SafeBuffer & pin );

                    /**
                     * @fn virtual bool setPasswordPIN ( Smartmatic::System::SafeBuffer & pin );
                     * @brief Sets password pin in the smartcard.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param pin Contains the password PIN to set.
                     * @return true if the sets password was successful; otherwise false.
                     */
                    virtual bool setPasswordPIN ( const Smartmatic::System::SafeBuffer & pin );

                    /**
                     * @fn virtual bool verifyPasswordPIN ( Smartmatic::System::SafeBuffer & pin );
                     * @brief Verify password pin in the smartcard.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param pin Contains the password PIN to verify.
                     * @return true if the password pin coincides; otherwise false.
                     */
                    virtual bool verifyPasswordPIN ( const Smartmatic::System::SafeBuffer & pin );

                    /**
                     * @fn virtual bool setPasswordFile ( const Smartmatic::System::SafeBuffer & password );
                     * @brief Sets password file.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param password Contains the password file to set.
                     * @return true if successful.
                     */
                    virtual bool setPasswordFile ( const Smartmatic::System::SafeBuffer & password );

                    /**
                     * @fn virtual Smartmatic::System::SafeBuffer getPasswordFile ();
                     * @brief Gets password file.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return The password file
                     */
                    virtual Smartmatic::System::SafeBuffer getPasswordFile ();

                    /**
                     * @fn virtual bool openWrite();
                     * @brief Starts a write operation in the smartcard. Does not write any data to the card at this point.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return true if the opening was successful; otherwise false.
                     */
                    virtual bool openWrite ();

                    /**
                     * @fn virtual bool continueWrite( const Smartmatic::System::SafeBuffer & buffer );
                     * @brief To be called after an openWrite, to write a buffer to the card.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param buffer The reference to the buffer to be written.
                     * @return true if the writing was successful; otherwise false.
                     */
                    virtual bool continueWrite ( const Smartmatic::System::SafeBuffer & buffer );

                    /**
                     * @fn virtual bool closeWrite();
                     * @brief Closes a write operation on the card.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return true if the closing was successful; otherwise false.
                     */
                    virtual bool closeWrite();

                    /**
                     * @fn virtual bool abortWrite();
                     * @brief Abort write operation.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return true if the abort write was successful; otherwise false.
                     */
                    virtual bool abortWrite();

                    /**
                     * @fn virtual unsigned int openRead ();
                     * @brief Starts a read operation on the smartcard.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return zero "0" if end
                     */
                    virtual unsigned int openRead ();

                    /**
                     * @fn virtual Smartmatic::System::SafeBuffer continueRead ( unsigned char size, unsigned int & left );
                     * @brief Reads from the smartcard.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param size       The requested size to be read, limited by the maximum length of the standard.
                     * @param [in,out] left  The reference to a integer used to store the length of the remaining data.
                     * @return the data read.
                     */
                    virtual Smartmatic::System::SafeBuffer continueRead ( unsigned char size, unsigned int & left );

                    /**
                     * @fn virtual Smartmatic::System::SafeBuffer getDataFile ();
                     * @brief Gets data contained in the smartcard.
                     * @author Yadickson Soto
                     * @date 26/05/2014
                     * @return The data contained in the smartcard
                     */
                    virtual Smartmatic::System::SafeBuffer getDataFile ();

                    /**
                     * @fn virtual bool setDataFile ( const Smartmatic::System::SafeBuffer & buffer );
                     * @brief Sets data file on the smartcard.
                     * @author Yadickson Soto
                     * @date 26/05/2014
                     * @param buffer The buffer to set inside of the smartcard.
                     * @return true if it succeeds, false if it fails.
                     */
                    virtual bool setDataFile ( const Smartmatic::System::SafeBuffer & buffer );

                private:

                    ConnectionBehavior * connectionBehavior; ///<pointer to the ConnectionBehavior to be used.
                    static Smartmatic::Log::ISMTTLog * logger;  ///< The pointer to the logger instance.
            };
        }
    }
}

#endif /* _JAVA_CARD_OPERATIONS_HXX_ */
