/**
 * @file SimulatorSmartcard.cxx
 * @brief	Implementation of the smartcard reader controller which implements operations of
 * reading and writing the device.
 * @date 21/05/2014
 * @author Yadickson Soto
 */

#include "SimulatorSmartcard.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/IO/File.h>
#include <unistd.h>
#include <System/Utils/StringUtils.hxx>
#include <System/Utils/SafeBufferUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;

ISMTTLog * SimulatorSmartcard::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Card.SimulatorSmartcard", GETTEXT_PACKAGE );

SimulatorSmartcard::SimulatorSmartcard ( const std::string & eventFile, const std::string & contentFile,
        unsigned int timeAfterRead, unsigned int timeBeforeWrite )
    : eventFile ( eventFile ), contentFile ( contentFile ),
      timeAfterRead ( timeAfterRead ), timeBeforeWrite ( timeBeforeWrite )
{

}

SimulatorSmartcard::~SimulatorSmartcard()
{

}

bool SimulatorSmartcard::open ( const std::string & device )
{
    logger->Debug ( "Open simulator smartcard device" );
    return File::getInstance()->canOpenFile ( device );

}

bool SimulatorSmartcard::close()
{
    logger->Debug ( "Clse simulator smartcard device" );
    return true;
}

bool SimulatorSmartcard::isCardPresent()
{
    bool status = false;
    SafeBuffer buffer;

    if ( File::getInstance()->openFileToBuffer ( eventFile, buffer ) )
    {
        std::string event;

        SafeBufferUtils buffUtil ( buffer );
        std::vector < SafeBuffer > result = buffUtil.splitByEndLine();

        if ( ! result.empty() )
        {
            SafeBuffer item ( result.front() );
            event.assign ( ( const char * ) item.GetData(), item.GetSize() );
            status = StringUtils::toUpper ( StringUtils::trim ( event ) ) == "I";
        }
    }
    else
    {
        logger->Debug ( "Error load file " + eventFile );
    }

    return status;
}

bool SimulatorSmartcard::read ( SafeBuffer & buffer, CryptoData & cryptoData )
{
    logger->Debug ( "Read SimulatorSmartcard" );

    bool status = false;

    buffer.Clear();
    SafeBuffer fullFile;

    if ( File::getInstance()->openFileToBuffer ( contentFile, fullFile ) )
    {
        if ( fullFile.GetSize() >= PASSWORD_FILE_KEY )
        {
            SafeBuffer key ( ( unsigned char * ) ( fullFile.GetData() ), PASSWORD_FILE_KEY );
            int size = fullFile.GetSize() - PASSWORD_FILE_KEY;

            if ( size > 0 )
            {
                buffer.AddData ( ( unsigned char * ) ( fullFile.GetData() + PASSWORD_FILE_KEY ), size );
                cryptoData.setKey ( key );
                status = true;
            }
        }
    }
    else
    {
        logger->Debug ( "Error load file " + contentFile );
    }

    sleep ( timeAfterRead );
    logger->Debug ( "End Read" );

    return status;
}

bool SimulatorSmartcard::write ( SafeBuffer & buffer, CryptoData & cryptoData )
{
    SafeBuffer fullFile;
    bool status;

    logger->Debug ( "Write SimulatorSmartcard" );

    sleep ( timeBeforeWrite );

    fullFile.AddData ( cryptoData.getKey() );
    fullFile.AddData ( buffer );

    status = File::getInstance()->saveBufferToFile ( fullFile, contentFile );

    logger->Debug ( "End Write" );
    return status;
}

bool SimulatorSmartcard::test()
{
    SafeBuffer bufferInput ( "InputString" );
    CryptoData cryptoInput;

    SafeBuffer inputKey ( ( unsigned char * ) "012345678901234567890123456789ab", PASSWORD_FILE_KEY );

    cryptoInput.setKey ( inputKey );

    SafeBuffer bufferOutput;
    CryptoData cryptoOutput;

    bool status = write ( bufferInput, cryptoInput ) && read ( bufferOutput, cryptoOutput );

    status = status && bufferInput == bufferOutput;
    status = status && inputKey == cryptoOutput.getKey();

    return status;
}
