/**
 * @file Smartcard.cxx
 * @brief	Implementation of the smartcard reader controller which implements operations of activation,
 * reading, writing, and restarting the device.
 * @date 21/05/2014
 * @author Yadickson Soto
 */

#include "Smartcard.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Security;

ISMTTLog * Smartcard::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Card.Smartcard", GETTEXT_PACKAGE );

Smartcard::Smartcard ( ConnectionBehavior * connectionBehavior, JavaCardOperations * operations, const SafeBuffer & appletID, const SafeBuffer & globalPIN )
    : connectionBehavior ( connectionBehavior ), operations ( operations ), appletID ( appletID ), globalPIN ( globalPIN )
{
    initStatus = false;
}

Smartcard::~Smartcard()
{
    if ( operations )
    {
        delete operations;
    }

    if ( connectionBehavior )
    {
        delete connectionBehavior;
    }
}

bool Smartcard::open ( const std::string & device )
{
    if ( !initStatus && connectionBehavior->establishContext() )
    {
        initStatus = true;
    }

    return initStatus;
}

bool Smartcard::isCardPresent()
{
    return initStatus && connectionBehavior->isCardPresent();
}

bool Smartcard::close()
{
    bool status = true;

    if ( initStatus )
    {
        initStatus = false;
        status = connectionBehavior->releaseContext();
    }

    return status;
}

bool Smartcard::read ( SafeBuffer & buffer, CryptoData & cryptoData )
{
    logger->Debug ( "Read Smartcard" );

    bool status = false;

    buffer.Clear();

    if ( initStatus )
    {
        status = connectionBehavior->connect() &&
                 connectionBehavior->selectAID ( appletID ) &&
                 operations->verifyGlobalPIN ( globalPIN ) &&
                 operations->verifyPasswordPIN ( cryptoData.getPin() );

        if ( status )
        {
            SafeBuffer password = operations->getPasswordFile();

            if ( password.GetSize() >= PASSWORD_FILE_KEY )
            {
                buffer = operations->getDataFile ();
                SafeBuffer key ( ( unsigned char * ) ( password.GetData() ), PASSWORD_FILE_KEY );
                cryptoData.setKey ( key );
            }
        }

        status = connectionBehavior->disconnect() && status;
    }

    logger->Debug ( "End Read" );
    return status;
}

bool Smartcard::write ( SafeBuffer & buffer, CryptoData & cryptoData )
{
    logger->Debug ( "Write Smartcard" );

    bool status = false;

    if ( initStatus )
    {
        SafeBuffer password;

        password.AddData ( cryptoData.getKey() );

        status = connectionBehavior->connect() &&
                 connectionBehavior->selectAID ( appletID ) &&
                 operations->verifyGlobalPIN ( globalPIN ) &&
                 operations->setPasswordPIN ( cryptoData.getPin() ) &&
                 operations->verifyPasswordPIN ( cryptoData.getPin() ) &&
                 operations->setPasswordFile ( password ) &&
                 operations->setDataFile ( buffer );

        status = connectionBehavior->disconnect() && status;
    }

    logger->Debug ( "End Write" );
    return status;
}

bool Smartcard::test()
{
    bool status = false;

    if ( initStatus )
    {
        status = connectionBehavior->connect() &&
                 connectionBehavior->selectAID ( appletID );

        status = connectionBehavior->disconnect() && status;
    }

    return status;
}
