/**
 * @file	SmartcardInterface.hxx
 * @brief	Declaration of the smartcard reader controller which implements operations of
 * reading and writing the device.
 * @author	Yadickson Soto
 * @date	13/05/2015
 */

#ifndef _SMARTCARD_INTERFACE_HXX_
#define _SMARTCARD_INTERFACE_HXX_

#include <Security/CryptoData.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Card
        {
            /**
             * @class	SmartcardInterface
             * @brief Represents the controller of a smartcard reader device which implements of operations
             *        the reading and writing the device.
             * @author	Yadickson Soto
             * @date	13/05/2015
             */
            class SmartcardInterface
            {
                public:

                    /**
                     * @fn	virtual ~SmartcardInterface();
                     * @brief	class destructor
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     */
                    virtual ~SmartcardInterface() {};

                    /**
                     * @fn	virtual bool open ( const std::string & device ) = 0;
                     * @brief	Abstract method to establishes the context.
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @param 	device device path
                     * @return true if success
                     */
                    virtual bool open ( const std::string & device ) = 0;

                    /**
                     * @fn	virtual bool close() = 0;
                     * @brief	Abstract method to release context
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @return true if success
                     */
                    virtual bool close() = 0;

                    /**
                     * @fn	virtual bool isCardPresent() = 0;
                     * @brief	Abstract method to know whether a card is present.
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @return	true if a smartcard is present; otherwise false.
                     */
                    virtual bool isCardPresent() = 0;

                    /**
                     * @fn	virtual bool read(Smartmatic::System::SafeBuffer & buffer,, Smartmatic::SAES::Security::CryptoData & cryptoData ) = 0;
                     * @brief	Abstract method to read the contents of the smartcard
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @param	buffer			The reference to the buffer used to store the information received.
                     * @param	cryptoData	Contains common context information for the cryptographic checking of the operation.
                     * @return	true if the reading was successful; otherwise false.
                     */
                    virtual bool read ( Smartmatic::System::SafeBuffer & buffer, Smartmatic::SAES::Security::CryptoData & cryptoData ) = 0;

                    /**
                     * @fn	virtual bool write(Smartmatic::System::SafeBuffer & buffer,, Smartmatic::SAES::Security::CryptoData & cryptoData ) = 0;
                     * @brief	Abstract method to write the contents of the buffer and verifies the data in the smartcard.
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @param [in,out]	buffer			Contains information that should to be written.
                     * @param [in,out]	cryptoData	Contains common context information for the cryptographic checking of the operation.
                     * @return	true if the writing was successful; otherwise false.
                     */
                    virtual bool write ( Smartmatic::System::SafeBuffer & buffer, Smartmatic::SAES::Security::CryptoData & cryptoData ) = 0;

                    /**
                     * @fn	virtual bool test() = 0;
                     * @brief	Abstract method to test the smartcard.
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @return	true if the test was successful; otherwise false.
                     */
                    virtual bool test() = 0;
            };
        }
    }
}

#endif /* _SMARTCARD_INTERFACE_HXX_ */
