/**
 * @file	SmartcardConfigurationManager.hxx
 * @brief	Declaration of the smartcard configuration manager.
 * @author	Yadickson Soto
 * @date	28/03/2014
 */

#ifndef _SMARTCARD_CONFIGURATION_MANAGER_HXX_
#define _SMARTCARD_CONFIGURATION_MANAGER_HXX_

#include <string>
#include <Configuration/smartcard-configuration-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Common
        {
            namespace Configuration
            {
                /**
                 * @class	SmartcardConfigurationManager
                 * @brief	The smartcard configuration manager.
                 * @author	Yadickson Soto
                 * @date	28/03/2014
                 */
                class SmartcardConfigurationManager
                {
                    public:

                        /**
                         * @fn	static SmartcardConfigurationManager * getInstance( SmartcardConfigurationManager * externManager = NULL );
                         * @brief	Gets the current Smartcard Configuration Manager.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @param 	externManager external manager to unit test
                         * @return	instance of the class.
                         */
                        static SmartcardConfigurationManager * getInstance ( SmartcardConfigurationManager * externManager = NULL );

                        /**
                         * @fn	virtual std::string getAppletID();
                         * @brief	Gets applet ID configuration value.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	the applet ID value
                         */
                        virtual std::string getAppletID();

                        /**
                         * @fn	virtual std::string getGlobalPIN();
                         * @brief	Gets global PIN configuration value.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	the global PIN value
                         */
                        virtual std::string getGlobalPIN();

                        /**
                         * @fn	virtual Smartmatic::SAES::Common::Configuration::PlatformMode getPlatformMode();
                         * @brief	Gets platform mode configuration value.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	the platform mode value
                         */
                        virtual Smartmatic::SAES::Common::Configuration::PlatformMode getPlatformMode();

                        /**
                         * @fn	virtual bool hasSecureChannel();
                         * @brief	Gets if security channel is enabled.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	true if security channel is enabled.
                         */
                        virtual bool hasSecureChannel();

                        /**
                         * @fn	virtual int getSecurityKeySetVersion();
                         * @brief	Gets security key set version configuration value.
                         * The key set version is used during mutual authentication.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	the security key set version value
                         */
                        virtual int getSecurityKeySetVersion();

                        /**
                         * @fn	virtual int getSecurityKeyIndex();
                         * @brief	Gets security key index configuration value.
                         * The key index is used during mutual authentication.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	the security key index value
                         */
                        virtual int getSecurityKeyIndex();

                        /**
                         * @fn	virtual Smartmatic::SAES::Common::Configuration::SecurityChannelProtocol getSecurityChannelProtocol();
                         * @brief	Gets security channel protocol configuration value.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	the security channel protocol value
                         */
                        virtual Smartmatic::SAES::Common::Configuration::SecurityChannelProtocol getSecurityChannelProtocol();

                        /**
                         * @fn	virtual Smartmatic::SAES::Common::Configuration::SecurityChannelProtocolImpl getSecurityChannelProtocolImpl();
                         * @brief	Gets security channel protocol implementation configuration value.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	the security channel protocol implementation value
                         */
                        virtual Smartmatic::SAES::Common::Configuration::SecurityChannelProtocolImpl getSecurityChannelProtocolImpl();

                        /**
                         * @fn	virtual Smartmatic::SAES::Common::Configuration::SecurityLevel getSecurityLevel();
                         * @brief	Gets security level configuration value.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	the security level value
                         */
                        virtual Smartmatic::SAES::Common::Configuration::SecurityLevel getSecurityLevel();

                        /**
                         * @fn	virtual Smartmatic::SAES::Common::Configuration::SecurityKeyDerivation getSecurityKeyDerivation();
                         * @brief	Gets security key derivation configuration value.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	the security key derivation value
                         */
                        virtual Smartmatic::SAES::Common::Configuration::SecurityKeyDerivation getSecurityKeyDerivation();

                        /**
                         * @fn	virtual bool getForceRestartSmartcardReader();
                         * @brief	Gets suspend/resume smartcard reader configuration value.
                         * @author	Yadickson Soto
                         * @date	30/05/2014
                         * @return	true to enable power cycle of smartcard reader device. This power cycle is done via the smtt-smartcard script.
                         */
                        virtual bool getForceRestartSmartcardReader();

                        /**
                         * @fn	virtual bool getForceRestartPCSCD();
                         * @brief	Gets suspend/resume pcscd service.
                         * @author	Yadickson Soto
                         * @date	30/05/2014
                         * @return	true to enable power cycle of pcscd service.
                         */
                        virtual bool getForceRestartPCSCD();

                        /**
                         * @fn	virtual unsigned int getDelayAfterOn();
                         * @brief	Gets delay to wait after turning the smartcard reader ON.
                         * @author	Yadickson Soto
                         * @date	30/05/2014
                         * @return	the delay to wait after turning the smartcard reader ON.
                         */
                        virtual unsigned int getDelayAfterOn();

                        /**
                         * @fn	virtual unsigned int getWaitStopPCSCD();
                         * @brief	Gets delay to wait after stopping the PCSCD.
                         * @author	Yadickson Soto
                         * @date	30/05/2014
                         * @return	the delay to wait after stopping the PCSCD.
                         */
                        virtual unsigned int getWaitStopPCSCD();

                        /**
                         * @fn	virtual unsigned int getWaitStartPCSCD();
                         * @brief	Gets delay to wait after starting the PCSCD.
                         * @author	Yadickson Soto
                         * @date	30/05/2014
                         * @return	the delay to wait after starting the PCSCD.
                         */
                        virtual unsigned int getWaitStartPCSCD();

                        /**
                         * @fn	virtual bool getCallRestartBeforeInitialize();
                         * @brief	Gets if should be power-cycled before being initialized.
                         * @author	Yadickson Soto
                         * @date	30/05/2014
                         * @return	true if the smartcard reader should be power-cycled before being initialized in the software.
                         */
                        virtual bool getCallRestartBeforeInitialize();

                        /**
                         * @fn	virtual unsigned int getNumberOfTriesToRead();
                         * @brief	Gets the maximum number of tries to read from the smarcard.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	the maximum number of tries to read from the smartcard.
                         */
                        virtual unsigned int getNumberOfTriesToRead();

                        /**
                         * @fn	virtual std::string getConnectionPluginName();
                         * @brief	Gets the connection plugin name.
                         * @author	Yadickson Soto
                         * @date	05/06/2014
                         * @return	the connection plugin name.
                         */
                        virtual std::string getConnectionPluginName();

                        /**
                         * @fn	virtual std::string getConnectionPluginVersion();
                         * @brief	Gets the connection plugin version.
                         * @author	Yadickson Soto
                         * @date	05/06/2014
                         * @return	the connection plugin version.
                         */
                        virtual std::string getConnectionPluginVersion();

                        /**
                         * @fn	virtual bool isErrorCode( unsigned long code );
                         * @brief	Whether the code is an error code. Checks in the error codes list.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @param	code code to check
                         * @return	true if the code is an error code.
                         */
                        virtual bool isErrorCode ( unsigned long code );

                        /**
                         * @fn	bool simulationEnabled ();
                         * @brief	Get if simulation mode is enabled
                         * @author	Yadickson Soto
                         * @date	13/05/2015
                         * @return	true if the simulation mode is enabled.
                         */
                        bool simulationEnabled ();

                        /**
                         * @fn	std::string getSimulatorEventFile ();
                         * @brief	Gets simulator event path file to used from SimulatorSmardcard class
                         * @author	Yadickson Soto
                         * @date	13/05/2015
                         * @return	simulator event path file
                         */
                        std::string getSimulatorEventFile ();

                        /**
                         * @fn	std::string getSimulatorContentFile ();
                         * @brief	Gets simulator content path file to used from SimulatorSmardcard class
                         * @author	Yadickson Soto
                         * @date	13/05/2015
                         * @return	simulator content path file
                         */
                        std::string getSimulatorContentFile ();

                        /**
                         * @fn	unsigned int getSimulationTimeAfterReadContentFile ();
                         * @brief	Get simulation time after read the content file
                         * @author	Yadickson Soto
                         * @date	14/05/2015
                         * @return	simulation time in seconds
                         */
                        unsigned int getSimulationTimeAfterReadContentFile ();

                        /**
                         * @fn	unsigned int getSimulationTimeBeforeWriteContentFile ();
                         * @brief	Get simulation time before write the content file
                         * @author	Yadickson Soto
                         * @date	14/05/2015
                         * @return	simulation time in seconds
                         */
                        unsigned int getSimulationTimeBeforeWriteContentFile ();

                        /**
                         * @fn	virtual ~SmartcardConfigurationManager();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         */
                        virtual ~SmartcardConfigurationManager();

                    protected:

                        /**
                         * @fn	SmartcardConfigurationManager();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         */
                        SmartcardConfigurationManager();

                        /**
                         * @fn	virtual SmartcardConfiguration & getSmartcardConfiguration ();
                         * @brief	Gets a reference to the common context of the smartcard which contains the cryptographic data and the reader device settings.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         * @return	The reference to the common context of the smartcard device
                         */
                        virtual SmartcardConfiguration & getSmartcardConfiguration ();

                        /**
                         * @fn	virtual SmartcardSimulation & getSmartcardSimulationConfiguration ();
                         * @brief	Gets a reference to the common context of the smartcard simulation configuration
                         * @author	Yadickson Soto
                         * @date	08/05/2015
                         * @return	The reference to the common context of the smartcard simulation configuration
                         */
                        virtual SmartcardSimulation & getSmartcardSimulationConfiguration ();

                    private:

                        static SmartcardConfigurationManager * instance; ///< The pointer to the singleton instance of the smartcard configuration manager
                };
            }
        }
    }
}

#endif // _SMARTCARD_CONFIGURATION_MANAGER_HXX_
