/**
 * @file	CheckResourceFiles.hxx
 * @brief	Declaration of the class that contains the verify authenticity  for the directories SaesData, ElectoralImages and it's mirrors.
 * @author	Yadickson Soto
 * @date	31/10/2012
 */

#ifndef _CHECK_RESOURCE_FILES_H_
#define _CHECK_RESOURCE_FILES_H_

#include "ICheckCondition.hxx"
#include <Runtime/Environment/saes-directories.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Operation
        {
            namespace Initiator
            {
                /**
                 * @class	CheckResourceFiles
                 * @brief	Class validator to analyze the authenticity for the directories <tt>SaesData</tt>, <tt>ElectoralImages</tt> and it's mirrors.
                 * @author	Yadickson Soto
                 * @date	03-03-2011
                 */
                class CheckResourceFiles: public ICheckCondition
                {
                    public:
                        /**
                         * @fn	CheckResourceFiles::CheckResourceFiles(Smartmatic::SAES::Runtime::Environment::SaesDirectories & directories,
                         * 			Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_FLASH);
                         * @brief	Default constructor initializes an instance of the CheckResourceFiles class.
                         * @author	Yadickson Soto
                         * @date	31/10/2012
                         * @param	directories	The pointer reference to environment context of the directories hierarchy.
                         * @param	type (optional) The type is the selection of the primary hierarchy which contains the file resources.
                         */
                        CheckResourceFiles ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories,
                                             Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                 Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn     CheckResourceFiles::CheckResourceFiles(Smartmatic::SAES::Runtime::Environment::SaesDirectories & directories,
                         *         Smartmatic::SAES::Common::Configuration::BasePathType type,
                         *         const std::string& message);
                         * @brief  Default constructor initializes an instance of the CheckResourceFiles class.
                         * @author manuel.arguelles
                         * @date   21/07/2022
                         * @param  directories The pointer reference to environment context of the directories hierarchy.
                         * @param  type        The type is the selection of the primary hierarchy which contains the file resources.
                         * @param  message     The instrution to display on screen.
                         */
                        CheckResourceFiles ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories,
                                             Smartmatic::SAES::Common::Configuration::BasePathType type,
                                             const std::string& message);

                        /**
                         * @fn	virtual CheckResourceFiles::~CheckResourceFiles();
                         * @brief	Finalizes the instance of the CheckResourceFiles class.
                         * @author	Yadickson Soto
                         * @date	31/10/2012
                         */
                        virtual ~CheckResourceFiles();

                        /**
                         * @fn	virtual TestStatus::Tests CheckResourceFiles::check ();
                         * @brief	Checks the authenticity the directories against the security keys.
                         * @author	Yadickson Soto
                         * @date	31/10/2012
                         * @return <tt>TestStatus::Passed</tt> if it checks successful; otherwise <tt>TestStatus::Failed</tt>.
                         */
                        virtual TestStatus::Tests check ();

                    protected:

                        /**
                         * @fn	bool CheckResourceFiles::validate ( const std::string & file );
                         * @brief	Method validate signature file in others path.
                         * @author	Yadickson Soto
                         * @date	31/10/2012
                         * @param file signature file name
                         * @return	<tt>true</tt> if it validates successful; otherwise <tt>false</tt>.
                         */

                        bool validate ( const std::string & file );

                    private:
                        Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories; /**< The pinter reference to environment context of the directory hierarchy.*/
                        Smartmatic::SAES::Common::Configuration::BasePathType type; ///< The base selection of the primary hierarchy.
                        static Smartmatic::Log::ISMTTLog * logger; /**< The reference pointer of a logger instance.*/
                };
            }
        }
    }
}

#endif // _CHECK_RESOURCE_FILES_H_
