/**
 * @file DeviceTypeMonitor.hxx
 * @brief Declaration of the common interface that abstracts any type of monitored device.
 * @date 12/12/2012
 * @author Yadickson Soto
 */

#ifndef _DEVICE_TYPE_MONITOR_HXX_
#define _DEVICE_TYPE_MONITOR_HXX_

#include <string>
#include <Configuration/device-type-schema.hxx>
#include <set>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Common
        {
            namespace Operation
            {
                namespace Monitor
                {
                    /**
                     * @class	DeviceTypeMonitor
                     * @brief	Interface that abstracts any type of monitored device.
                     * @author	Yadickson Soto
                     * @date	12/12/2012
                     *
                     * A DeviceTypeMonitor keeps track of all devices of the type it monitors.
                     * These devices are tracked by their UDEV UDI.
                     * A Device Type is characterized by a UDI regex, which is supposed to match
                     * with the UDI given by UDEV. This regex is found in configuration. Each of
                     * the monitored devices must additionally match the properties given in
                     * configuration.
                     * Therefore configuration provides a UDI, a set of expected properties, and
                     * all devices reported by UDEV which match on all these criteria will be
                     * added to that DeviceTypeMonitor.
                     * This monitor will trigger events when devices of the monitored type are
                     * added, removed, or modified as reported by UDEV.
                     */
                    class DeviceTypeMonitor
                    {
                        public:

                            /**
                             * @fn	Device( Smartmatic::SAES::Common::Configuration::DeviceType type );
                             * @brief	Default constructor for the interface. Subclasses are encouraged to call it.
                             * @author	Yadickson Soto
                             * @date	12/12/2012
                             * @param 	type device type
                             */
                            explicit DeviceTypeMonitor ( Smartmatic::SAES::Common::Configuration::DeviceType type );

                            /**
                             * @fn	virtual ~Device();
                             * @brief	Destructor.
                             * @author	Yadickson Soto
                             * @date	12/12/2012
                             */
                            virtual ~DeviceTypeMonitor();

                            /**
                             * @fn	virtual Smartmatic::SAES::Common::Configuration::DeviceType getType();
                             * @brief	Gets the device type, used to group the devices in this DeviceTypeMonitor .
                             * @author	Yadickson Soto
                             * @date	09/04/2014
                             * @return	device type
                             */
                            virtual Smartmatic::SAES::Common::Configuration::DeviceType getType();

                            /**
                             * @fn	virtual unsigned int getNumberOfDevices();
                             * @brief	Gets the current number of devices detected in the instance.
                             * @author	Yadickson Soto
                             * @date	09/04/2014
                             * @return	the amount of the devices monitored.
                             */
                            virtual unsigned int getNumberOfDevices();

                            /**
                             * @fn	virtual bool addNode ( const std::string & node );
                             * @brief	Add new node to list of devices matching the type.
                             * @author	Yadickson Soto
                             * @date	06/05/2014
                             * @param	@param node The UDI (Unique Device Identifier) of the device object.
                             * @return	true if node is added t o list
                             */
                            virtual bool addNode ( const std::string & node );

                            /**
                             * @fn	virtual bool removeNode ( const std::string & node );
                             * @brief	Remove node from list of devices of this type.
                             * @author	Yadickson Soto
                             * @date	06/05/2014
                             * @param	@param node The UDI (Unique Device Identifier) of the device object.
                             * @return	true if found node in the list and node is removed
                             */
                            virtual bool removeNode ( const std::string & node );

                            /**
                             * @fn	virtual void added ( const std::string & node ) = 0;
                             * @brief	Notification callback for when a device is added.
                             * @author	Yadickson Soto
                             * @date	09/04/2014
                             * @param 	node event device path
                             */
                            virtual void added ( const std::string & node ) = 0;

                            /**
                             * @fn	virtual void removed ( const std::string & node ) = 0;
                             * @brief	Notification callback for when a device is removed.
                             * @author	Yadickson Soto
                             * @date	09/04/2014
                             * @param 	node event device path
                             */
                            virtual void removed ( const std::string & node ) = 0;

                            /**
                             * @fn	virtual std::string getDeviceMounted ( const std::string & node, const std::string & customNode ) = 0;
                             * @brief	Gets when device mounted.
                             * @author	Yadickson Soto
                             * @date	09/07/2015
                             * @param 	node the node to check
                             * @param	customNode the custom node to check device
                             * @return  path device mounted
                             */
                            virtual std::string getDeviceMounted ( const std::string & node, const std::string & customNode ) = 0;

                        private:

                            Smartmatic::SAES::Common::Configuration::DeviceType type; ///< device type
                            std::set < std::string > nodeList; ///< The collection of node identifiers of currently detected devices matching this type.
                    };
                }
            }
        }
    }
}

#endif /* _DEVICE_TYPE_MONITOR_HXX_ */
