/**
 * @file	DirectPrintingBackend.h
 * @brief	Declaration of direct printing strategy which provides the functionalities to render and print documents.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef DIRECTPRINTINGBACKEND_H_
#define DIRECTPRINTINGBACKEND_H_

#include <gtkmm.h>
#include <queue>
#include <System/SafeBuffer.h>
#include "Printing/IPrintingBackend.h"
#include "Printing/LPSpooler.h"
#include "Printing/DocumentOptions.h"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {

        namespace Printing
        {
            /**
             * @class	DirectPrintingBackend
             * @brief	Direct printing strategy which provides methods for render and print document.
             * @author	Cesar.bonilla
             * @date	4/28/2011
             */

            class DirectPrintingBackend : public IPrintingBackend
            {
                public:

                    /**
                     * @enum ApsControlCodeEnumType
                     * @brief	Type of the aps control code enum.
                     */
                    typedef enum
                    {
                        NUL     = 0,
                        SOH     = 1,
                        STX     = 2,
                        ETX     = 3,
                        EOT     = 4,
                        ENQ     = 5,
                        ACK     = 6,
                        BEL     = 7,
                        BS      = 8,
                        TAB     = 9,
                        LF      = 10,
                        VT      = 11,
                        FF      = 12,
                        CR      = 13,
                        SO      = 14,
                        SI      = 15,
                        DLE     = 16,
                        DC1     = 17,
                        DC2     = 18,
                        DC3     = 19,
                        DC4     = 20,
                        NAK     = 21,
                        SYN     = 22,
                        ETB     = 23,
                        CAN     = 24,
                        EM      = 25,
                        SUB     = 26,
                        ESC     = 27,
                        FS      = 28,
                        GS      = 29,
                        RS      = 30,
                        US      = 31
                    } ApsControlCodeEnumType;

                    /**
                     * @enum QRPrintingModeEnumType
                     * @brief	Type of the qr printing mode enum
                     */
                    typedef enum
                    {
                        DBCKND_IMAGE,
                        DBCKND_CMD

                    } QRPrintingModeEnumType;

                    /**
                     * @fn	DirectPrintingBackend::DirectPrintingBackend();
                     * @brief	Default constructor initializes an instance of the DirectPrintingBackend class.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    DirectPrintingBackend();

                    /**
                     * @fn	virtual DirectPrintingBackend::~DirectPrintingBackend();
                     * @brief	Finalizes the instance of the DirectPrintingBackend class.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    virtual ~DirectPrintingBackend();

                    /**
                     * @fn	void DirectPrintingBackend::Init();
                     * @brief	Initializes the internal stream buffer and create temporary file.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void Init();

                    /**
                     * @fn	void DirectPrintingBackend::End();
                     * @brief	Ends this object.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void End();

                    /**
                     * @fn	void DirectPrintingBackend::DrawString(std::list<Glib::ustring>& textContent,
                     * 			std::list< DocumentTextOptions>& textContentOptionList);
                     * @brief	Draw string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	textContent					 	The text content.
                     * @param	textContentOptionList	List of text content options.
                     */

                    void DrawString ( std::list<Glib::ustring> & textContent, std::list <
                                      DocumentTextOptions > & textContentOptionList );

                    /**
                     * @fn	void DirectPrintingBackend::DrawImage(Glib::RefPtr<Gdk::Pixbuf> image);
                     * @brief	Draw image.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	image	The image.
                     */

                    void DrawImage ( Glib::RefPtr<Gdk::Pixbuf> image );

                    /**
                     * @fn	void DirectPrintingBackend::DrawQRCode (QRCodeContent& qr);
                     * @brief	Draw qr code.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	qr	The qr.
                     */
                    virtual void DrawQRCode ( QRCodeContent & qr );

                    //void NewLine();


                    // Access

                    /**
                     * @fn	PrintDocumentResult DirectPrintingBackend::getPrintResult ();
                     * @brief	Gets the print result.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The print result.
                     */

                    virtual PrintDocumentResult getPrintResult ();


                    /**
                     * @fn	void DirectPrintingBackend::cleanPrintQueue();
                     * @brief	Clean print queue.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void cleanPrintQueue();

                private:

                    /**
                     * @fn	void DirectPrintingBackend::ChangeFile();
                     * @brief	Change file.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void ChangeFile();

                    std::string currentFilename;	///< Filename of the current file
                    DocumentTextOptions currentOptions; ///< Options for controlling the current
                    std::queue<std::string> filequeue;  ///< The filequeue

                    /*std::map<int, char> iso_8859_1_2_aps_french;*/

                    std::map<int, unsigned char> utf8_to_aps;   ///< The utf 8 to aps

                    PrintDocumentResult mOverallResult; ///< The overall result

                    bool deleteFiles;   ///< true to delete the files
                    bool binaryOutput;	///< true to binary output

                    /**
                     * @fn	void DirectPrintingBackend::DrawCell(DocumentTextOptions& options, std::string& text,
                     * 			int columns);
                     * @brief	Draw cell.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	options	Options for controlling the operation.
                     * @param	text	 	The text.
                     * @param	columns	The columns.
                     */

                    void DrawCell ( DocumentTextOptions & options, std::string & text,
                                    int columns );

                    /**
                     * @fn	void DirectPrintingBackend::UpdateParameters(DocumentTextOptions& newParams,
                     * 			bool includeJustification);
                     * @brief	Updates the parameters.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	newParams							Options for controlling the new.
                     * @param	includeJustification	true to include, false to exclude the justification.
                     */

                    void UpdateParameters ( DocumentTextOptions & newParams,
                                            bool includeJustification );

                    /**
                     * @fn	int DirectPrintingBackend::getCharactersPerline(DocumentFontSize fontSize);
                     * @brief	Gets the characters perline.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	fontSize	Size of the font.
                     * @return	The characters perline.
                     */

                    int getCharactersPerline ( DocumentFontSize fontSize );

                    /**
                     * @fn	void DirectPrintingBackend::ChangeFontSize(DocumentFontSize newFont);
                     * @brief	Change font size.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	newFont	The new font.
                     */

                    void ChangeFontSize ( DocumentFontSize newFont );

                    /**
                     * @fn	void DirectPrintingBackend::ChangeFontStyle(DocumentFontStyle newFontStyle);
                     * @brief	Change font style.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	newFontStyle	The new font style.
                     */

                    void ChangeFontStyle ( DocumentFontStyle newFontStyle );

                    /**
                     * @fn	void DirectPrintingBackend::ChangeJustification(DocumentJustifitacion justification);
                     * @brief	Change justification.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	justification	The justification.
                     */

                    void ChangeJustification ( DocumentJustifitacion justification );

                    /**
                     * @fn	void DirectPrintingBackend::UtfToPrinterCharSet (const Glib::ustring& utf8_string,
                     * 			std::string& decodeResult);
                     * @brief	Utf to printer character set.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	utf8_string 	The utf 8 string.
                     * @param	decodeResult	The decode result.
                     */

                    void UtfToPrinterCharSet ( const Glib::ustring & utf8_string, std::string & decodeResult );

                    /**
                     * @fn	void DirectPrintingBackend::AddNewFontCommand(unsigned char code);
                     * @brief	Adds a new font command.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	code	The code.
                     */

                    void AddNewFontCommand ( unsigned char code );

                    /**
                     * @fn	void DirectPrintingBackend::AddNewCharSetCommand(unsigned char code);
                     * @brief	Adds a new character set command.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	code	The code.
                     */

                    void AddNewCharSetCommand ( unsigned char code );

                    /**
                     * @fn	void DirectPrintingBackend::AddNewJustificationCommand(unsigned char code);
                     * @brief	Adds a new justification command.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	code	The code.
                     */

                    void AddNewJustificationCommand ( unsigned char code );

                    /**
                     * @fn	void DirectPrintingBackend::AddNewTextModeCommand(unsigned char underlined);
                     * @brief	Adds a new text mode command.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	underlined	The underlined.
                     */

                    void AddNewTextModeCommand ( unsigned char underlined );

                    /**
                     * @fn	void DirectPrintingBackend::AddNewFullCutCommand();
                     * @brief	Adds new full cut command.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void AddNewFullCutCommand();

                    /**
                     * @fn	void DirectPrintingBackend::AddNewPartialCutCommand();
                     * @brief	Adds new partial cut command.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void AddNewPartialCutCommand();

                    /**
                     * @fn	void DirectPrintingBackend::AddLineFeedCommand();
                     * @brief	Adds line feed command.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void AddLineFeedCommand();

                    /**
                     * @fn	void DirectPrintingBackend::AddFeedPaperCommand(unsigned char val);
                     * @brief	Adds a feed paper command.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	val	The value.
                     */

                    void AddFeedPaperCommand ( unsigned char val );

                    /**
                     * @fn	void DirectPrintingBackend::AddCodePageCommand(unsigned char val);
                     * @brief	Adds a code page command.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	val	The value.
                     */

                    void AddCodePageCommand ( unsigned char val );

                    /**
                     * @fn	void DirectPrintingBackend::DrawQRCodeText (QRCodeContent qr);
                     * @brief	Draw qr code text.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	qr	The qr.
                     */
                    void DrawQRCodeText ( QRCodeContent qr );

                    /**
                     * @fn	void DirectPrintingBackend::DrawQRCodeBin (QRCodeContent qr);
                     * @brief	Draw qr code bin.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	qr	The qr.
                     */
                    void DrawQRCodeBin ( QRCodeContent qr );

                    /**
                     * @fn	void DirectPrintingBackend::JobNotifListenerThreadBody();
                     * @brief	Job notif listener thread body.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void JobNotifListenerThreadBody();

                    // Signal handlers

                    /**
                     * @fn	void DirectPrintingBackend::onPrintJobStateChanged(LPSpooler& sender);
                     * @brief	Executes the print job state changed action.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	sender	The sender.
                     */

                    void onPrintJobStateChanged ( LPSpooler & sender );

                    /**
                     * @fn	void DirectPrintingBackend::setDeleteFiles(bool arg);
                     * @brief	Sets a delete files.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	arg	true to argument.
                     */

                    void setDeleteFiles ( bool arg );

                    /**
                     * @fn	PrintDocumentResult::PrintingResultEnumType DirectPrintingBackend::SpoolerResToPrintingRes ( LPSpooler::JobStateEnumType spoolerRes);
                     * @brief	Spooler resource to printing resource.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	spoolerRes	The spooler resource.
                     * @return	.
                     */

                    PrintDocumentResult::PrintingResultEnumType SpoolerResToPrintingRes (
                        LPSpooler::JobStateEnumType spoolerRes );

                    static Smartmatic::Log::ISMTTLog * logger; ///< The logger

            };

        }
    }
}
#endif /* DIRECTPRINTINGBACKEND_H_ */
