/**
 * @file PrinterStatusNotifier.cpp
 * @brief Body class printer status notifier
 * @date 19/03/2011
 * @author Marcel Cordovi
 */

#include <cups/cups.h>
#include "PrinterStatusNotifier.h"
#include "IB31K1PrintingBackend.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/OptParser.hxx>
#include <System/OptParser.hxx>
#include <System/Runtime/ProcessLauncher.h>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;

ISMTTLog * PrinterStatusNotifier::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Printing.PrinterStatusNotifier", GETTEXT_PACKAGE );


bool PrinterStatusNotifier::CheckCurrentState = true;

PrinterStatusNotifier::PrinterStatusNotifier() :
    mPhState ( PSTATE_UNKNOWN )
    , mStatusMessage ( "" )
{
    printerBackendType = SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getPrinterBackendType();

    if ( printerBackendType == PrinterBackendType::CONFIGURATION_DIRECTPRINTING )
    {
        externalStatusTrigger = true;
    }
    else
    {
        externalStatusTrigger = false;
    }
}

PrinterStatusNotifier::~PrinterStatusNotifier()
{

}

PrinterStatusNotifier::PhysicalStateEnumType PrinterStatusNotifier::getPrinterState ( bool useCachedValue )
{
    if ( SAESCommonConfigurationManager::getInstance()->getConfigurationPrinter().getPrinterBackendType() ==
            PrinterBackendType::CONFIGURATION_DIRECTPRINTING )
    {
        PrinterStatusNotifier::runPrintBlank();
        usleep ( 250000 );
    }

    if ( PrinterStatusNotifier::CheckCurrentState )
    {
        mRefreshStatusMessage ( useCachedValue );
    }
    else
    {
        mPhState = PSTATE_READY;
        mStatusMessage = "printer ok";
    }

    return mPhState;
}

void PrinterStatusNotifier::CheckPrinterState ( bool check )
{
    PrinterBackendType printerBackendType =
        SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getPrinterBackendType();

    // Allow to be set only when GtkPrint backend is being used

    if ( printerBackendType == PrinterBackendType::CONFIGURATION_DIRECTPRINTING )
    {
        PrinterStatusNotifier::CheckCurrentState = true;
    }
    else
    {
        PrinterStatusNotifier::CheckCurrentState = check;
    }
}

const std::string & PrinterStatusNotifier::getStatusMessage ( bool useCachedValue )
{
    return mStatusMessage;
}

void PrinterStatusNotifier::setExternalStatusTrigger ( bool arg )
{
    externalStatusTrigger = arg;
}

int PrinterStatusNotifier::getErrorCondition () const
{
    int printer_failure_cond =
        ( PSTATE_OUT_OF_PAPER | PSTATE_PROCESSING | PSTATE_PAPER_JAM
          | PSTATE_COVER_OPEN | PSTATE_FEEDER_OPEN | PSTATE_OFFLINE | PSTATE_REJECTING_JOBS
          | PSTATE_OUT_OF_TONER | PSTATE_CANCELING | PSTATE_CARTRIDGE_NOT_PRESENT );
    return printer_failure_cond;
}

void PrinterStatusNotifier::mRefreshStatusMessage ( bool skipBlankPage )
{
    int i;
    bool paper_present = false;
    bool paper_jam = false;
    bool processing = false;
    bool online = false;
    bool rejecting = false;
    bool cover_open = false;
    bool neop = false;
    bool supply_low = false;
    bool out_of_toner = false;
    bool cartridge_not_present = false;
    bool printer_canceling = false;
    bool other = false;
    int printer_failure_cond;

    if ( externalStatusTrigger && skipBlankPage == false )
    {
        logger->Debug ( "Running printblank" );
        runPrintBlank();
    }

    mPhState = PSTATE_UNKNOWN;
    mStatusMessage.clear();
    // Get default destination printer
    cups_dest_t * dests = NULL;
    cups_dest_t * dest = NULL;
    int ndests = cupsGetDests ( &dests );

    if ( dests )
    {
        dest = cupsGetDest ( NULL, NULL, ndests, dests );
    }

    if ( dest == NULL )
    {
        mStatusMessage.assign ( _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.ServiceFailureOnNotificationOp" ) );
        logger->Debug ( "Error - impossible to get destination printer for status notification" );

        if ( dests )
        {
            cupsFreeDests ( ndests, dests );
        }

        return;
    }

    for ( i = 0; i < dest->num_options; i++ )
    {
        if ( strcmp ( dest->options[i].name, "printer-state" ) == 0 )
        {
            online = strcmp ( dest->options[i].value, "5" ) != 0;
            // The IPP specification's processing state value
            // processing = strcmp (dflt_dest->options[i].value, "4") == 0;
        }
        else if ( strcmp ( dest->options[i].name, "printer-state-reasons" ) == 0 )
        {
            paper_present = ! strstr ( dest->options[i].value, "media-empty" );
            paper_jam     =   strstr ( dest->options[i].value, "media-jam" );
            processing    =   strstr ( dest->options[i].value, "processing" );
            cover_open    =   strstr ( dest->options[i].value, "cover-open" );
            neop          =   strstr ( dest->options[i].value, "media-low" );
            out_of_toner  =   strstr ( dest->options[i].value, "toner-empty-error" );
            supply_low    =   strstr ( dest->options[i].value, "marker-supply-low-warning" );
            other         =   strstr ( dest->options[i].value, "other" );
            printer_canceling = strstr ( dest->options[i].value, "cancel-error" );
            cartridge_not_present = cartridge_not_present || strstr ( dest->options[i].value, "pen-failure-error" );
            cartridge_not_present = cartridge_not_present || strstr ( dest->options[i].value, "cartridge-not-present-error" );
            logger->Debug ( "printer-state-reasons : " + std::string ( dest->options[i].value ) );
        }
        else if ( strcmp ( dest->options[i].name, "printer-is-accepting-jobs" ) == 0 )
        {
            rejecting = ! ( strstr ( dest->options[i].value, "false" ) == 0 );
        }
    }

    if ( ! online )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_OFFLINE ) );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_OFFLINE );
        cupsFreeDests ( ndests, dests );
        return;
    }

    if ( ! paper_present )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_OUT_OF_PAPER ) );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_OUT_OF_PAPER );
    }

    if ( cover_open )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        if ( printerBackendType ==  PrinterBackendType::CONFIGURATION_DIRECTPRINTING )
        {
            mStatusMessage.append ( mapStatusMessage ( PSTATE_FEEDER_OPEN ) );
            mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_FEEDER_OPEN );
        }
        else
        {
            mStatusMessage.append ( mapStatusMessage ( PSTATE_COVER_OPEN ) );
            mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_COVER_OPEN );
        }
    }

    if ( paper_jam )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_PAPER_JAM ) );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_PAPER_JAM );
    }

    if ( processing )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_PROCESSING ) + "\n" );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_PROCESSING );
    }

    if ( rejecting )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_REJECTING_JOBS ) );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_REJECTING_JOBS );
    }

    if ( neop )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_NEAR_END_OF_PAPER ) );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_NEAR_END_OF_PAPER );
    }

    if ( supply_low )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_SUPPLYLOW_WARN ) );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_SUPPLYLOW_WARN );
    }

    if ( out_of_toner )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_OUT_OF_TONER ) );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_OUT_OF_TONER );
    }

    if ( printer_canceling )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_CANCELING ) );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_CANCELING );
    }

    if ( cartridge_not_present )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_CARTRIDGE_NOT_PRESENT ) );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_CARTRIDGE_NOT_PRESENT );
    }

    if ( other )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_UNKNOWN ) );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_UNKNOWN );
    }

    printer_failure_cond = mPhState & this->getErrorCondition ();

    if ( ! printer_failure_cond && !neop )
    {
        if ( ! mStatusMessage.empty() )
        {
            mStatusMessage.append ( "\n" );
        }

        mStatusMessage.append ( mapStatusMessage ( PSTATE_READY ) );
        mPhState = ( PhysicalStateEnumType ) ( mPhState | PSTATE_READY );
    }

    logger->Debug ( "Printer Status Message : " + mStatusMessage );
    cupsFreeDests ( ndests, dests );
}

const std::string PrinterStatusNotifier::getStatusMessage ( PhysicalStateEnumType state )
{
    return this->mStatusMessage;
}

const std::string PrinterStatusNotifier::getCurrentStatusMessage ()
{
    return this->mStatusMessage;
}

const std::string PrinterStatusNotifier::mapStatusMessage ( PhysicalStateEnumType state )
{
    std::string message;
    std::string machineModel;
    OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);

    switch ( state )
    {
        case PSTATE_UNKNOWN:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.UnknownState" );
            break;

        case PSTATE_READY:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.PrinterReady" );
            break;

        case PSTATE_OUT_OF_PAPER:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.OutOfPaper" );
            break;

        case PSTATE_PROCESSING:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.Processing" );
            break;

        case PSTATE_PAPER_JAM:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.PaperJam" );
            break;

        case PSTATE_COVER_OPEN:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.CoverOpen" );
            break;

        case PSTATE_FEEDER_OPEN:

            if (machineModel == EnumToString(VM_A4517))
            {
                message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.A4517.FeederOpen" );
            }
            else
            {
                message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.SAES3370.FeederOpen" );
            }

            break;

        case PSTATE_OFFLINE:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.Offline" );
            break;

        case PSTATE_REJECTING_JOBS:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.RejectingPrintJobs" );
            break;

        case PSTATE_NEAR_END_OF_PAPER:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.NearEndOfPaper" );
            break;

        case PSTATE_SUPPLYLOW_WARN:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.TonnerLowWarning" );
            break;

        case PSTATE_OUT_OF_TONER:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.OutOfTonner" );
            break;

        case PSTATE_CANCELING:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.Cancel" );
            break;

        case PSTATE_CARTRIDGE_NOT_PRESENT:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.CartridgeNotPresent" );
            break;

        case PSTATE_NONE:
        default:
            message = _ ( "Smartmatic.SAES.Printing.PrinterStatusNotifier.UnknownStateCode" );
            break;
    }

    return message;
}

void PrinterStatusNotifier::runPrintBlank()
{
    std::string script = SAESCommonConfigurationManager::getInstance()->getOSConfigurationFiles().getScriptRunPrintBlank();
    ( void ) ProcessLauncher::getInstance()->RunProcessThroughSystem ( script );
}

void PrinterStatusNotifier::runForcedPrintBlank()
{
    std::string machineModel;
    OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);

    if (StringUtils::toUpper(machineModel) == "VM-A4517")
    {
        IB31K1PrintingBackend printer;

        printer.updateStatus();
        printer.End();
    }
    else
    {
        runPrintBlank();
    }
}
