/**
 * @file EnvironmentBuilder.cxx
 * @brief Implementation of the class that contains the useful functions for file system manipulation.
 * @date 15/11/2011
 * @author Yadickson Soto
 */

#include "EnvironmentBuilder.hxx"
#include "DirectoryManager.hxx"
#include <Log/SMTTLogManager.h>
#include <System/IO/Directory.h>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/Runtime/ProcessLauncher.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::Runtime;

ISMTTLog * EnvironmentBuilder::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Runtime.Environment.EnvironmentBuilder", GETTEXT_PACKAGE );

EnvironmentBuilder::EnvironmentBuilder()
{
}

EnvironmentBuilder::~EnvironmentBuilder()
{
}

bool EnvironmentBuilder::install (
    SaesDirectories * orig,
    SaesDirectories * dest )
{
    DirectoryManager::copyDir ( orig->getBasePath(), dest->getBasePath(), false );

    installElectionFile(orig, dest);
    installDeviceParametersFile(orig, dest);
    installGeneralAudioResources(orig, dest);
    installCertToEncrypt(orig, dest);

    return true;
}

void EnvironmentBuilder::installElectionFile(SaesDirectories* orig, SaesDirectories* dest)
{
    std::string fileOrig = orig->getElectionFileFullPath();
    std::string fileDest = dest->getElectionFileFullPath();

    if ( File::getInstance()->canOpenFile ( fileOrig ) && !File::getInstance()->canOpenFile ( fileDest ) )
    {
        File::getInstance()->copyFile ( fileOrig, fileDest );
        logger->Debug ( "Copy file " + fileOrig + " to " + fileDest );
    }
}

void EnvironmentBuilder::installDeviceParametersFile(SaesDirectories* orig, SaesDirectories* dest)
{
    std::string fileOrig = orig->getDeviceParametersFileFullPath();
    std::string fileDest = dest->getDeviceParametersFileFullPath();

    if ( File::getInstance()->canOpenFile ( fileOrig ) && !File::getInstance()->canOpenFile ( fileDest ) )
    {
        File::getInstance()->copyFile ( fileOrig, fileDest );
        logger->Debug ( "Copy file " + fileOrig + " to " + fileDest );
    }
}

void EnvironmentBuilder::installGeneralAudioResources(SaesDirectories*  orig, SaesDirectories* dest)
{
    const std::string resourceFile = Path::GetFileNameWithOutExtension(orig->getGeneralAudioResourcesFileName());
    const std::string electionFileDirectory = Path::GetDirectory(orig->getElectionFileFullPath());
    const std::string sourceDirectory = Path::Combine(electionFileDirectory, orig->getElectoralResourcesFolderName());
    std::vector<std::string> files;
    std::vector<std::string>::const_iterator it;

    Directory::getInstance()->GetFiles(sourceDirectory, files);

    for (it = files.begin(); it != files.end(); ++it)
    {
        std::string filePresent = *it;

        if (filePresent.find(resourceFile) != std::string::npos)
        {
            std::string sourceFile = Path::Combine(sourceDirectory, *it);
            std::string destinationFile = Path::Combine(dest->getElectoralResourceFullFolderPath(), *it);

            File::getInstance()->copyFile(sourceFile, destinationFile);
            logger->Debug("Copy file " + sourceFile + " to " + destinationFile);
        }
    }
}

void EnvironmentBuilder::installCertToEncrypt(SaesDirectories* const origin, SaesDirectories* const destination)
{
    const std::string certToEncryptOrigin      = origin->getX7SCryptoKeystoreInstallationFileFullPath();
    const std::string certToEncryptDestination = destination->getX7SCryptoKeystoreFileFullPath();

    if (File::getInstance()->canOpenFile(certToEncryptOrigin) && File::getInstance()->canOpenFile(certToEncryptDestination) == false)
    {
        File::getInstance()->copyFile(certToEncryptOrigin, certToEncryptDestination);
        logger->Debug("Copy file " + certToEncryptOrigin + " to " + certToEncryptDestination);
    }
}
