/**
 * @file EnvironmentBuilder.hxx
 * @brief Declaration of the class that contains the useful functions for file system manipulation.
 * @date 15/11/2011
 * @author Yadickson Soto
 */

#ifndef ENVIRONMENTBUILDER_HXX_
#define ENVIRONMENTBUILDER_HXX_

#include <Runtime/Environment/saes-directories.h>
#include <Runtime/Environment/SaesInstallDirectories.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Runtime
        {
            namespace Environment
            {
                /**
                 * @class	EnvironmentBuilder
                 * @brief Generator of environment necessary for voting machine.
                 * @author	Youssef.tourkmani
                 * @date	30/10/2012
                 */
                class EnvironmentBuilder
                {
                    public:
                        /**
                         * @fn	EnvironmentBuilder::EnvironmentBuilder();
                         * @brief	Default constructor initializes an instance of the EnvironmentBuilder class.
                         * This builder of SAES unit has a large storage repository as a medium for the exchange of data
                         * internal.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         */
                        EnvironmentBuilder();
                        /**
                         * @fn	virtual EnvironmentBuilder::~EnvironmentBuilder();
                         * @brief	Finalizes the instance of the EnvironmentBuilder class.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         */
                        virtual ~EnvironmentBuilder();
                        /**
                         * @fn	virtual bool EnvironmentBuilder::install( SaesDirectories * orig,
                         * 			SaesDirectories * dest);
                         * @brief	Installs the files list in the specified directory.
                         * This method does not support null reference.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @param [in,out]	orig	The reference pointer to context of the original directory.
                         * @param [in,out]	dest	The reference pointer to context of the destiny directory.
                         * @return	<tt>true</tt> if it installs successful; otherwise <tt>false</tt>.
                         */
                        virtual bool install (
                            SaesDirectories * orig,
                            SaesDirectories * dest );

                    private:

                        /**
                         * @brief  Installs the election file.
                         * @author angel.ramirez
                         * @date   Jul 05, 2022
                         * @param  orig The source directory.
                         * @param  dest The destination directory.
                         */
                        void installElectionFile(SaesDirectories* orig, SaesDirectories* dest);
                        
                        /**
                         * @brief  Installs the device parameteres file.
                         * @author angel.ramirez
                         * @date   Jul 05, 2022
                         * @param  orig The source directory.
                         * @param  dest The destination directory.
                         */
                        void installDeviceParametersFile(SaesDirectories* orig, SaesDirectories* dest);

                        /**
                         * @brief  Install the general audio resources.
                         * @author manuel.arguelles
                         * @date   Dec 19, 2017
                         * @param  orig The source directory context.
                         * @param  dest The destination directory context.
                         */
                        void installGeneralAudioResources(SaesDirectories*  orig, SaesDirectories* dest);

                        /**
                         * @brief  Install the certificate to encrypt the voting results.
                         * @author eduardo.fernandez
                         * @date   Dec 12, 2018
                         * @param [in,out]  origin      The reference pointer to context of the original directory.
                         * @param [in,out]  destination The reference pointer to context of the destination directory.
                         */
                        void installCertToEncrypt(SaesDirectories* const origin, SaesDirectories* const destination);

                    private:
                        static Smartmatic::Log::ISMTTLog * logger; /**< The reference pointer of a logger instance.*/
                };
            }
        }
    }
}

#endif /* ENVIRONMENTBUILDER_HXX_ */
