/*
 * CertificateStore.h
 *
 *  Created on: Oct 25, 2018
 *      Author: eduardo.fernandez
 */

#ifndef CERTIFICATE_STORE_H
#define CERTIFICATE_STORE_H

#include <Runtime/Environment/saes-directories.h>
#include <Security/CertificateValidatorWrapper.hxx>
#include <Security/Encryption.hxx>

#include <memory>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                class CertificateExtensionBase;
            }
        }
    }

    namespace SAES
    {
        namespace Security
        {
           /**
            * @brief Class to handle digital certificates.
            * @author eduardo.fernandez
            * @date   25/10/2018
            */
            class CertificateStore
            {
                public:

                    /**
                     * @brief Signature file types.
                     */
                    enum SignatureFileTypes
                    {
                        SIGNATURE_FILE_TYPE_RESOURCE,
                        SIGNATURE_FILE_TYPE_ROOT,
                        SIGNATURE_FILE_TYPE_EML,
                        SIGNATURE_FILE_TYPE_SAES_LOCAL
                    };

                public:

                    /**
                     * @brief Gets the class current instance.
                     * @author eduardo.fernandez
                     * @date   25/10/2018
                     * @param  directories SAES directories.
                     * @param  externalInstance Any external instance with testing purposes.
                     * @return Current instance of the class.
                     */
                    static CertificateStore *getInstance(
                        Smartmatic::SAES::Runtime::Environment::SaesDirectories * const directories,
                        CertificateStore * const externalInstance = NULL);

                    /**
                     * @brief Class destructor.
                     * @author eduardo.fernandez
                     * @date   25/10/2018
                     */
                    virtual ~CertificateStore();

                public:

                    /**
                     * @brief Loads issuer certificates involved in the chain of trust validation.
                     * @author eduardo.fernandez
                     * @date   29/10/2018
                     * @param  rootFingerprint Root certificate fingerprint.
                     */
                    void loadIssuerCertificates(const std::string& rootFingerprint);

                    /**
                     * @brief Gets the current root certificate.
                     * @author eduardo.fernandez
                     * @date   04/12/2018
                     * @return The current root certificate.
                     */
                    OpenSSLCryptoX509 * getRootCertificate();

                    /**
                     * @brief Gets the current intermediate certificate.
                     * @author eduardo.fernandez
                     * @date   04/12/2018
                     * @return The current root certificate.
                     */
                    OpenSSLCryptoX509 * getIntermediateCertificate();

                    /**
                     * @brief Gets the client certificate full path.
                     * @author eduardo.fernandez
                     * @date   07/11/2018
                     * @param  resourceFileType Type of the resource file.
                     * @param  protectionKey Client certificate protection key.
                     * @return The client certificate full path.
                     */
                    Smartmatic::System::Security::Cryptography::CertificateExtensionBase *getClientCertificate(
                        const Smartmatic::SAES::Security::Encryption::ResourceFileType& resourceFileType,
                        const std::string& protectionKey);

                private:

                    /**
                     * @brief Class constructor.
                     * @author eduardo.fernandez
                     * @date   25/10/2018
                     * @param  directories SAES directories.
                     */
                    CertificateStore(
                        Smartmatic::SAES::Runtime::Environment::SaesDirectories * const directories);

                private:

                    /**
                     * @brief Loads root certificate.
                     * @author eduardo.fernandez
                     * @date   29/10/2018
                     * @param  rootFingerprint Root certificate fingerprint.
                     */
                    void loadRootCertificate(const std::string& rootFingerprint);

                    /**
                     * @brief Loads intermediate certificate.
                     * @author eduardo.fernandez
                     * @date   25/10/2018
                     */
                    void loadIntermediateCertificate();

                private:

                    void dispose();

                private:

                    static CertificateStore *m_instance;
                    static Smartmatic::Log::ISMTTLog *m_logger;

                    Smartmatic::SAES::Runtime::Environment::SaesDirectories *m_directories;
                    Smartmatic::SAES::Security::CertificateValidatorWrapper *m_validatorWrapper;

                    bool m_isRootCertificateValid;
                    bool m_isIntermediateCertificateValid;
                    std::map<std::string, System::Security::Cryptography::CertificateExtensionBase*> m_store;
            };
        }
    }
}



#endif /* CERTIFICATE_STORE_H */
