/**
 * @file	CryptoData.hxx
 * @brief	Declaration of the abstract facade which mediates with crypto to encode and decode data.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef CRYPTODATA_HXX_
#define CRYPTODATA_HXX_

#include <System/SafeBuffer.h>
#include <System/Security/Crypto.hxx>

/**
 * @def	PASSWORD_PIN_LEN
 * @brief	A macro that defines password pin length.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */
#define PASSWORD_PIN_LEN 0x08
/**
 * @def	PASSWORD_FILE_KEY
 * @brief	A macro that defines password file key.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */
#define PASSWORD_FILE_KEY 0x20

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Security
        {
            /**
             * @class	CryptoData
             * @brief	Abstract facade that interact with the crypto provider for encode and decode data.
             * @author	Youssef.tourkmani
             * @date	30/10/2012
             */
            class CryptoData
            {
                public:

                    /**
                     * @fn	CryptoData::CryptoData();
                     * @brief	Default constructor initializes an instance of the CryptoData class.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    CryptoData();

                    /**
                     * @fn	CryptoData::CryptoData( Smartmatic::System::SafeBuffer & pin,
                     * 			Smartmatic::System::SafeBuffer & key, Smartmatic::System::SafeBuffer & iv);
                     * @brief	Constructor initializes an instance of the CryptoData class.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	pin	The pin.
                     * @param [in,out]	key	The key.
                     * @param [in,out]	iv 	The iv.
                     */
                    CryptoData (
                        Smartmatic::System::SafeBuffer & pin,
                        Smartmatic::System::SafeBuffer & key );

                    /**
                     * @fn	virtual CryptoData::~CryptoData();
                     * @brief	Finalizes the instance of the CryptoData class.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    virtual ~CryptoData();

                    /**
                     * @fn	virtual bool CryptoData::crypto( Smartmatic::System::SafeBuffer & dataIn,
                     * 			Smartmatic::System::SafeBuffer & dataOut);
                     * @brief	Encodes the input data using the cryto provider.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	dataIn 	The reference to input buffer which contains the readable data.
                     * @param [in,out]	dataOut The reference to buffer used to retrieve the result.
                     * @return	true if it succeeds, false if it fails.
                     */
                    virtual bool crypto (
                        Smartmatic::System::SafeBuffer & dataIn,
                        Smartmatic::System::SafeBuffer & dataOut );

                    /**
                     * @fn	virtual bool CryptoData::decrypto( Smartmatic::System::SafeBuffer & dataIn,
                     * 			Smartmatic::System::SafeBuffer & dataOut);
                     * @brief	Decodes the input buffer using the cryto provider.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	dataIn 	The reference to input buffer which contains the encrypt data.
                     * @param [in,out]	dataOut	The reference to buffer used to retrieve the result.
                     * @return	true if it succeeds, false if it fails.
                     */
                    virtual bool decrypto (
                        Smartmatic::System::SafeBuffer & dataIn,
                        Smartmatic::System::SafeBuffer & dataOut );

                    /**
                     * @fn	const Smartmatic::System::SafeBuffer & CryptoData::getPin();
                     * @brief	Gets the pin.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The pin.
                     */
                    const Smartmatic::System::SafeBuffer & getPin();

                    /**
                     * @fn	const Smartmatic::System::SafeBuffer & CryptoData::getKey();
                     * @brief	Gets the key.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The key.
                     */
                    const Smartmatic::System::SafeBuffer & getKey();

                    /**
                     * @fn	void CryptoData::setPin(Smartmatic::System::SafeBuffer & pin);
                     * @brief	Sets a pin.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	pin	The pin.
                     */
                    void setPin ( Smartmatic::System::SafeBuffer & pin );

                    /**
                     * @fn	void CryptoData::setKey(Smartmatic::System::SafeBuffer & key);
                     * @brief	Sets a key.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	key	The key.
                     */
                    void setKey ( Smartmatic::System::SafeBuffer & key );

                private:

                    Smartmatic::System::SafeBuffer                m_iv;
                    Smartmatic::System::SafeBuffer                m_key;
                    Smartmatic::System::SafeBuffer                m_pin;
                    Smartmatic::System::Security::Crypto::KeySize m_keySize;
                    Smartmatic::System::Security::Crypto::Mode    m_mode;
                    static Smartmatic::Log::ISMTTLog*             m_logger;
            };
        }
    }
}

#endif /* CRYPTODATA_HXX_ */
