/**
 * @file KeyManager.cxx
 * @brief Body class for key manager
 * @date 06/05/2013
 * @author Yadickson Soto
 */

#include "KeyManager.hxx"
#include "Configuration/s-a-e-s-common-conf-manager.h"
#include <System/Encoding/Base64Encoder.h>
#include <System/Security/ComposeKey.hxx>
#include <System/Compression/compression-class.h>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Common::Security;
using namespace Smartmatic::System::Compression;
using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System;

ISMTTLog * KeyManager::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Security.KeyManager", GETTEXT_PACKAGE );
KeyManager * KeyManager::instance = NULL;

KeyManager::KeyManager()
{
    ConfigurationElectionFilesPath& filePaths = \
        SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationElectionFilesPath();
    SafeBuffer initKey;
    SafeBuffer zippedInitKey;
    std::string swapFile;
    std::string swapFileContent;

    keySize = Crypto::KEY_AES_256;
    modeType = Crypto::Crypto::MODE_AES_CBC;
    swapFile = Path::Combine(
        filePaths.getSwapBasePath(),
        filePaths.getSwapFileName());

    if (!File::getInstance()->readFile(swapFile, swapFileContent) ||
        !Base64Encoder::ConvertFromBase64(swapFileContent, zippedInitKey) ||
        !CompressionClass::getInstance()->Inflate(zippedInitKey, initKey))
    {
        throw CryptoException(
            _("Swap file not found or invalid"),
            N_("Swap file not found or invalid"),
            logger);
    }

    addKeyByType ( initKey, KeyType::CONFIGURATION_INIT_KEY );
}

KeyManager::KeyManager( Crypto::KeySize size, Crypto::Mode mode )
    : keySize(size)
    , modeType(mode)
{
}

KeyManager::~KeyManager()
{
    keysMap.clear();
    instance = NULL;
}

void KeyManager::initialize (
    const Crypto::KeySize & keySize,
    const Crypto::Mode & modeType )
{
    this->keySize = keySize;
    this->modeType = modeType;
}

unsigned int KeyManager::getKeyLength()
{
    unsigned int leng = 0;

    switch ( keySize )
    {
        default:
        case Crypto::KEY_AES_256:
            leng = 32;
            break;

        case Crypto::KEY_AES_192:
            leng = 24;
            break;

        case Crypto::KEY_AES_128:
            leng = 16;
            break;
    }

    return leng;
}

Crypto::KeySize KeyManager::getKeySize()
{
    return keySize;
}

Crypto::Mode KeyManager::getModeType()
{
    return modeType;
}

void KeyManager::addKeyByType ( const SafeBuffer & bufferIn, const KeyType & type )
{
    keysMap [ type ].Clear();
    keysMap [ type ].AddData ( bufferIn ) ;
}

void KeyManager::getKey ( const KeyType & type, SafeBuffer & key )
{
    key.Clear();

    if ( type == KeyType::CONFIGURATION_UP_KEY &&
         !isKeyByType ( KeyType::CONFIGURATION_UP_KEY ) )
    {
        key.AddData ( keysMap [ KeyType::CONFIGURATION_SECRET_KEY ] );
    }
    else if ( type == KeyType::CONFIGURATION_SECURITY_KEY &&
              !isKeyByType ( KeyType::CONFIGURATION_SECURITY_KEY ) )
    {
        key.AddData ( keysMap [ KeyType::CONFIGURATION_INIT_KEY ] );
    }
    else
    {
        key.AddData ( keysMap [ type ] );
    }
}

bool KeyManager::isKeyByType ( const KeyType & type )
{
    return keysMap.find ( type ) != keysMap.end();
}

KeyManager * KeyManager::getInstance ( KeyManager * externKeyManager )
{
    if ( !instance )
    {
        instance = ( externKeyManager == NULL ? new KeyManager() : externKeyManager );
    }

    return instance;
}
