/**
 * @file	XmlSecResource.hxx
 * @brief	Declaration of the singleton entity that validates the security resources.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */


#ifndef __XML_SEC_RESOURCE__H__
#define __XML_SEC_RESOURCE__H__

#include <Runtime/Environment/saes-directories.h>
#include <System/Exception/XmlException.hxx>
#include <System/Exception/SignedException.hxx>
#include <System/Exception/CryptoException.hxx>
#include <System/Exception/FileSystemException.hxx>
#include <Configuration/key-type-schema.hxx>
#include <Security/Encryption.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
}

namespace Smartmatic
{
    namespace SAES
    {
        namespace Security
        {
            /**
             * @class	XmlSecResource
             * @brief	Validator of the security resources.
             * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
             * @date	04-06-2011
             */
            class XmlSecResource
            {
                private:

                    /**
                     * @fn	XmlSecResource::XmlSecResource();
                     * @brief	Default constructor initializes an instance of the XmlSecResource class.
                     * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                     * @date	04-06-2011
                     */

                    XmlSecResource();

                public:

                    /**
                     * @fn	XmlSecResource::~XmlSecResource();
                     * @brief	Finalizes the instance of the XmlSecResource class.
                     * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                     * @date	04-06-2011
                     */

                    virtual ~XmlSecResource();

                    /**
                     * @fn	bool XmlSecResource::validateResource(std::string path,
                     * 			std::string signature,
                     * 			Smartmatic::SAES::Security::Encryption::ResourceFileType type,
                     *          Smartmatic::SAES::Common::Configuration::KeyType ukt )
                     * 			 throw (Smartmatic::System::Exception::CryptoException,
                     * 					Smartmatic::System::Exception::XmlException,
                     * 					Smartmatic::System::Exception::FileSystemException);
                     * @brief	method to open and validate resource list.
                     * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                     * @date	04-06-2011
                     * @exception	Smartmatic::System::Exception::CryptoException			Thrown when a crypto error condition occurs.
                     * @exception	Smartmatic::System::Exception::XmlException					Thrown when an XML error condition occurs.
                     * @exception	Smartmatic::System::Exception::FileSystemException	Thrown when a file system error condition occurs.
                     * @param	path		 	path base resources.
                     * @param	signature	signature file.
                     * @param type the type resource file
                     * @param ukt user key type
                     * @return	true is validate resources.
                     */
                    bool validateResource ( std::string path,
                                            std::string signature,
                                            Smartmatic::SAES::Security::Encryption::ResourceFileType type,
                                            Smartmatic::SAES::Common::Configuration::KeyType ukt )
                    throw ( Smartmatic::System::Exception::CryptoException,
                            Smartmatic::System::Exception::XmlException,
                            Smartmatic::System::Exception::FileSystemException );

                    /**
                     * @fn	static XmlSecResource * XmlSecResource::getInstance();
                     * @brief	Gets the only instance of the XmlSecResource. If this instance does not exist, then it creates a new one.
                     * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                     * @date	04-06-2011
                     * @return	The pointer reference to XmlSecResource required to access to the instance.
                     */

                    static XmlSecResource * getInstance();

                private:

                    static XmlSecResource * instance; ///< The pointer reference to XmlSecResource required to access to the instance.
                    static Smartmatic::Log::ISMTTLog * logger; ///< The reference pointer of a logger instance.
            };
        }
    }
}

#endif /*__XML_SEC_RESOURCE__H__*/
