#ifndef CERTIFICATE_STORE_TEST
#define CERTIFICATE_STORE_TEST

#ifdef ECUT_MAIN

#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <cppunit/extensions/HelperMacros.h>
#include <Log/SMTTLogManager.h>
#include <MockKeyManager.hxx>
#include <MockSaesCommonConfigurationManager.hxx>
#include <Security/CertificateValidatorWrapper.hxx>
#include <Security/KeyManager.hxx>
#include <System/Security/Cryptography/CertUtil.h>
#include <System/Encoding/Base64Encoder.h>
#include <System/Exception/CryptoException.hxx>
#include <System/IO/File.h>
#include <System/SafeBuffer.h>
#include <System/Security/OpenSSLSecurity.hxx>
#include <System/Security/XmlSecurityC.hxx>
#include <tests/StoreMockFile.hxx>

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Common::Security;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Security::Cryptography;

static const std::string              ALTERNATIVE_ROOT_CERTIFICATE_FINGERPRINT  = "icm0afdzOJC/NrIlSbCsXWwyVLFFtfKWcYBV5fIwNuI=";
static const std::string              EMPTY_CHAR                                = "";
static const std::string              FILE_PATH                                 = "file-path";
static const CertUtil::CertFileFormat INVALID_CLIENT_CERTIFICATE_FORMAT         = CertUtil::CertFormat_PKCS12;
static const std::string              INVALID_CLIENT_CERTIFICATE_PROTECTION_KEY = "vnLWZtxTtXHk27rfpDQ8UbkHSZhaUMz/vD1FGERjojA=";
static const std::string              INVALID_ROOT_CERTIFICATE_FINGERPRINT      = "YsTjQHoNNWrUMGWqt9M/FTDd/vFgo=";
static const std::string              ROOT_CERTIFICATE_TEMP_PATH                = "/tmp/root-ca.crt";
static const std::string              VALID_CLIENT_CERTIFICATE_PROTECTION_KEY   = "ArDlHrjXix4OBaoQW4dNILulUAzJSmFQBx9uyMEt0AI=";
static const CertUtil::CertFileFormat VALID_CLIENT_CERTIFICATE_FORMAT           = CertUtil::CertFormat_PEM;
static const std::string              VALID_ROOT_CERTIFICATE_FINGERPRINT        = "YsTjQHoNNWrUrs5aFTIR/Wn7FwMGWqt9M/FTDd/vFgo=";
static const double                   VALID_TIMESTAMP                           = 1543527334;

static std::string ALTERNATIVE_INTERMEDIATE_CERTIFICATE =
    "-----BEGIN CERTIFICATE-----\n"
    "MIIF9TCCA92gAwIBAgICEAAwDQYJKoZIhvcNAQELBQAwgZYxCzAJBgNVBAYTAkJFMQswCQYDVQQI"
    "DAJCUjEYMBYGA1UEBwwPU2FpbnQtR2lsbGVzMDAxMRMwEQYDVQQKDApTbWFydG1hdGljMQ0wCwYD"
    "VQQLDARFU0JVMQswCQYDVQQDDAJCVTEvMC0GCSqGSIb3DQEJARYgZWR1YXJkby5mZXJuYW5kZXpA"
    "c21hcnRtYXRpYy5jb20wHhcNMTgxMDI1MTk1MzA1WhcNMjgxMDIyMTk1MzA1WjB8MQswCQYDVQQG"
    "EwJCRTELMAkGA1UECAwCQlIxEzARBgNVBAoMClNtYXJ0bWF0aWMxDTALBgNVBAsMBEVTQlUxCzAJ"
    "BgNVBAMMAkJVMS8wLQYJKoZIhvcNAQkBFiBlZHVhcmRvLmZlcm5hbmRlekBzbWFydG1hdGljLmNv"
    "bTCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAMEbE41DCQ21i6I4pV2SmMChNV3aP+Tt"
    "N7n8dTko0S5UfyFwdpCxqd+x+uhaZ2T/jMngy/Ugw6qgefdgylCrm0cW0/LCvFuOA25dBRLfskrI"
    "0hzyQRu/oi9C8zB59elGdq2febYIzBxT3LlQDP4tLhyq/5wOuxac2VltF4m1WqlAcKKYCnbVUF3L"
    "F5gZKOW5rrkmJztdM8WHk2yEleq1ZcP36YPrAYDSn8mOSvL/bMpUNwnam0oVVatQp22JHV+0KVl/"
    "C6mTweSRO5lMuI1NDLqVSMyDOQJYB690fz1tpURq9f4jkz9HFGh4NOuDe8xEJzWD9F1XSLwjh3wl"
    "IU+0ZzVromA2rsNf/u9E7pVp5c1zY7Vu4ROQwo63N8ON4ETzFNENkz2ysIi56RQbGy4iwtpRcylQ"
    "idhXeqzrrgP5gErDdV0PpVstLsWYOC9FKV8fHfyJA6a1R8v5I1s6ejhbkBqHkjFhClzaXCPkw7Sx"
    "I9PHvny4vwOxeJqBdoqygxiMwR9tE75n58f7U7v8cPD7kujGut11tReNd4vpW8PNfF8ZwPYS2R7/"
    "ufljKuxIAeVK2cLI5CD4iZLVboCZO0s53hvGiC/4genUbJvzKj5rLcpnSMZ+CpY2LR/ZCutcXK69"
    "dO3BKXc0siSu5hrBRjUZj+uOtliNWxmcoKzBjDN6yhBJAgMBAAGjZjBkMB0GA1UdDgQWBBSVCIF3"
    "tFXEyidYJNAVVQ13ig2WPzAfBgNVHSMEGDAWgBRkL9xJirvnq3Vex8qcmkxanfle2DASBgNVHRMB"
    "Af8ECDAGAQH/AgEAMA4GA1UdDwEB/wQEAwIBhjANBgkqhkiG9w0BAQsFAAOCAgEANe2F8gcpkaW/"
    "SWUVk4J5+cxFtt31+PZ5mcwkfxmNHhdY3Fxxqk5HMKecERriGYQl3jmw/ctvQoyxXOvxY/cV2k7t"
    "bHiKvxBY8ZKSvvU9XhKwfK7RPB65nwz7bO2kbCFFYKhOLJFIz4Bm+MdRHBJQ/k/JYg8xRmNwaqAe"
    "PdABNc9RRMcZoB3eGATa+PQkOFq7OqFCR5l8fRbn0gEP9Qa457HB3I/95Py5+aECJqRMJHGeOzVA"
    "G/4/s2yTJf1DsfSH5130voe+E6GfktYwKqc6q3Vf1ijR4SrVAndUcXSXLckN2OaJkoV9X90Q+Lef"
    "/18IT4C7HbSITduZyZe171daI3AAMlleMGVH3C6sDAm6uR4Q+3VrDADYYUXl08iNCfTGVmhSuxP3"
    "T1WSEaqvMAypCEEQdDzAXn8+oiYMK43pNkHvU1PAxOr+mHAT/bG4niBEvauE3EM1kBhiINSigmZL"
    "lWnrWy4LfQrLc3jmkle1I5hNBqgtRcmEBkjvQ1IPqlENQQuj3oMi/BYk8B3ODGEy6RFSYD0YXPfs"
    "a4WKdRGaT6IfDt8WzoUEVdEw2j2l5ZFDN51saxpF4pQfnglTOXlDbQN/xhQwZCl7ewzym6v/14Ci"
    "7UF25rpD/HkrmChpO9sLez7YUIlYuTjXQfj83FRj7+EFmkE0c5q0Eyu73hD4R0Y=\n"
    "-----END CERTIFICATE-----\n";

static const std::string ALTERNATIVE_ROOT_CERTIFICATE_BASE64 =
    "-----BEGIN CERTIFICATE-----\n"
    "MIIGFDCCA/ygAwIBAgIJAOWq6ynlFm5HMA0GCSqGSIb3DQEBCwUAMIGWMQswCQYDVQQGEwJCRTEL"
    "MAkGA1UECAwCQlIxGDAWBgNVBAcMD1NhaW50LUdpbGxlczAwMTETMBEGA1UECgwKU21hcnRtYXRp"
    "YzENMAsGA1UECwwERVNCVTELMAkGA1UEAwwCQlUxLzAtBgkqhkiG9w0BCQEWIGVkdWFyZG8uZmVy"
    "bmFuZGV6QHNtYXJ0bWF0aWMuY29tMB4XDTE4MTAyNTE2NTMxNloXDTM4MTAyMDE2NTMxNlowgZYx"
    "CzAJBgNVBAYTAkJFMQswCQYDVQQIDAJCUjEYMBYGA1UEBwwPU2FpbnQtR2lsbGVzMDAxMRMwEQYD"
    "VQQKDApTbWFydG1hdGljMQ0wCwYDVQQLDARFU0JVMQswCQYDVQQDDAJCVTEvMC0GCSqGSIb3DQEJ"
    "ARYgZWR1YXJkby5mZXJuYW5kZXpAc21hcnRtYXRpYy5jb20wggIiMA0GCSqGSIb3DQEBAQUAA4IC"
    "DwAwggIKAoICAQDMKAmxjP4t10ceb3yQVgO/HxeJC03TU8YJyqs1ePyanY3TEEN/PLZXsMFNT2OY"
    "2JNvrTlsmOPKaXrrlGdtgcPEP/IVyWAL/tqdwYGzw93kCty/ADPf9IhTMDwx0+7usuULRXvpqQ90"
    "VrzB7YrXCrj1myQvcwHEfP6iWcRoWK+ZS+OCcFdBdBxh7BzqXUk7AuvkXSxGMkXDykm5++KBqSQ5"
    "tOHHJUHAKHdzSDiVGsRGpqNDB0nHwH550uq0w7LpkZWxNCUdJar0F0i+SqpxcE7nwQpsgve2K6GD"
    "ZBQt9SVs6dE/tfH56wXPCTCe2M5qyWB1PH/qlYdwFn1bHNiEX1Kcrt67bMK0PQ9VvHgzmVanrgT2"
    "3FNGJvV28j+wXYhSVRE8kOKkamSM2jgqhld2Mlu7gbPTFFnubOsVg+3jX8QGly6PBsgQZITOvmUu"
    "FciaEz9v+QKvFFmj6rERAGu4aVf7GP+6LC/fCw0jRvykOqNkdMbJuW+80TcPhHEzaRq7LjqqnXMg"
    "zPEqHBJU++6QFVygfhUOPUwTqR/VbV2kGbZ5xLyIvnVOTcynjXWNnRA3LDgLntlDsoGKRKrIS55w"
    "vf8AxF16+8ZG4vjVRWwBySGqsyjGMCIfP0TQqcfz5HJ8/Apc+B+UwWOhKue5unylvabFTjCYllJJ"
    "C/V9FJPH3c09bwIDAQABo2MwYTAdBgNVHQ4EFgQUZC/cSYq756t1XsfKnJpMWp35XtgwHwYDVR0j"
    "BBgwFoAUZC/cSYq756t1XsfKnJpMWp35XtgwDwYDVR0TAQH/BAUwAwEB/zAOBgNVHQ8BAf8EBAMC"
    "AYYwDQYJKoZIhvcNAQELBQADggIBAAsTVMHh7gNRxLl6LvwsjjbWv70skUBE6P8AOBobw23YbeN0"
    "SzLYjIl7Eh3KhEzNAjRc+S2JL124oCgFcrAgOuNORJDP91w8S0QbGGD6GIH1RJ0SfYJxgx0TJUJu"
    "u0eQTw4SI3/RqLRI2HRsHrpF5QoLWWtsHoK+DraMZgSeJuv3xxPMHn23Gdl+ijMIArLZSbGK6nht"
    "kLaV5oMV5K2EHCGESmrrv1NIAW30cVtJssDE6OSxZNtjW+v67KI8yP61YlS7meiu2ggV5+jHsfn9"
    "3nimu9/MIhE7oPX+JLQC3qmCtvk3zmXWbrSI+6N450AYByXEHy4hsD6edTr3nGzYyenKtEbBJUiq"
    "iQUofXnBqSVnw12bIHm+DjNL41OnRb/SeRVcSBfCnHzWC6N2zqRD9yjyFNT9DO/1fPaN7WPyF8jk"
    "ZTEFkhsqBIKCK4vb8qS+CVp6ILi+uUblgsV84umQNFXiuAAfEykZjUBqHqDDNSTPxbm40KYQzzen"
    "EeBpi+3Zr2s10ignxl5iOAC0guhKwDHaNgEsTyx9xHi+0bRyJnTxaT1NfHO7/3xnvUU7JsYErIDp"
    "FKNLbu90ALBPDK/MPxM1gxqUDcsRsWiYn5IKz9l+zbPH8AnEiEs/SunrknhZPH/esHuchhF0/dDI"
    "DgZC34+YDichGiYyqCbeRwwzhg68\n"
    "-----END CERTIFICATE-----";

static const std::string INVALID_ROOT_CERTIFICATE_BASE64 =
    "-----BEGIN CERTIFICATE-----\n"
    "MIIC0jCCAnigAwIBAgIJAJsXLtrWFaN6MAoGCCqGSM49BAMCMG4xCzAJBgNVBAYTAkJFMREwDwYD"
    "VQQIEwhCcnVzc2VsczERMA8GA1UEBxMIQnJ1c3NlbHMxEzARBgNVBAoTClNtYXJ0bWF0aWMxDTAL"
    "BgNVBAsTBFNBRVMxFTATBgNVBAMTDFJPT1QtQ0EtMjAxODAeFw0xODAxMDIxOTQ4MjZaFw0xOTAx"
    "QgENBBwWGkNlcnRpZmljYXRlIGlzc3VlZCBieSBTbXR0MB0GA1UdDgQWBBRs6EvAcFne+CDTNYT7"
    "K0A+y7eLLjCBoAYDVR0jBIGYMIGVgBRs6EvAcFne+CDTNYT7K0A+y7eLLqFypHAwbjELMAkGA1UE"
    "BhMCQkUxETAPBgNVBAgTCEJydXNzZWxzMREwDwYDVQQHEwhCcnVzc2VsczETMBEGA1UEChMKU21h"
    "cnRtYXRpYzENMAsGA1UECxMEU0FFUzEVMBMGA1UEAxMMUk9PVC1DQS0yMDE4ggkAmxcu2tYVo3ow"
    "DAYDVR0TBAUwAwEB/zAKBggqhkjOPQQDAgNIADBFAiEAx9zrGjP/RaRsXEm22Qnf6ENVhkFJiNRp"
    "RVULxlRBlYkCIAGX9GydKOvNuI0oZFpRSfkx2g9nbLIe+66T1Q5oeuau\n"
    "-----END CERTIFICATE-----";
static const std::string SELF_SIGNED_INTERMEDIATE_CERTIFICATE =
    "-----BEGIN CERTIFICATE-----\n"
    "MIIGFDCCA/ygAwIBAgIJAOWq6ynlFm5HMA0GCSqGSIb3DQEBCwUAMIGWMQswCQYDVQQGEwJCRTEL"
    "MAkGA1UECAwCQlIxGDAWBgNVBAcMD1NhaW50LUdpbGxlczAwMTETMBEGA1UECgwKU21hcnRtYXRp"
    "YzENMAsGA1UECwwERVNCVTELMAkGA1UEAwwCQlUxLzAtBgkqhkiG9w0BCQEWIGVkdWFyZG8uZmVy"
    "bmFuZGV6QHNtYXJ0bWF0aWMuY29tMB4XDTE4MTAyNTE2NTMxNloXDTM4MTAyMDE2NTMxNlowgZYx"
    "CzAJBgNVBAYTAkJFMQswCQYDVQQIDAJCUjEYMBYGA1UEBwwPU2FpbnQtR2lsbGVzMDAxMRMwEQYD"
    "VQQKDApTbWFydG1hdGljMQ0wCwYDVQQLDARFU0JVMQswCQYDVQQDDAJCVTEvMC0GCSqGSIb3DQEJ"
    "ARYgZWR1YXJkby5mZXJuYW5kZXpAc21hcnRtYXRpYy5jb20wggIiMA0GCSqGSIb3DQEBAQUAA4IC"
    "DwAwggIKAoICAQDMKAmxjP4t10ceb3yQVgO/HxeJC03TU8YJyqs1ePyanY3TEEN/PLZXsMFNT2OY"
    "2JNvrTlsmOPKaXrrlGdtgcPEP/IVyWAL/tqdwYGzw93kCty/ADPf9IhTMDwx0+7usuULRXvpqQ90"
    "VrzB7YrXCrj1myQvcwHEfP6iWcRoWK+ZS+OCcFdBdBxh7BzqXUk7AuvkXSxGMkXDykm5++KBqSQ5"
    "tOHHJUHAKHdzSDiVGsRGpqNDB0nHwH550uq0w7LpkZWxNCUdJar0F0i+SqpxcE7nwQpsgve2K6GD"
    "ZBQt9SVs6dE/tfH56wXPCTCe2M5qyWB1PH/qlYdwFn1bHNiEX1Kcrt67bMK0PQ9VvHgzmVanrgT2"
    "3FNGJvV28j+wXYhSVRE8kOKkamSM2jgqhld2Mlu7gbPTFFnubOsVg+3jX8QGly6PBsgQZITOvmUu"
    "FciaEz9v+QKvFFmj6rERAGu4aVf7GP+6LC/fCw0jRvykOqNkdMbJuW+80TcPhHEzaRq7LjqqnXMg"
    "zPEqHBJU++6QFVygfhUOPUwTqR/VbV2kGbZ5xLyIvnVOTcynjXWNnRA3LDgLntlDsoGKRKrIS55w"
    "vf8AxF16+8ZG4vjVRWwBySGqsyjGMCIfP0TQqcfz5HJ8/Apc+B+UwWOhKue5unylvabFTjCYllJJ"
    "C/V9FJPH3c09bwIDAQABo2MwYTAdBgNVHQ4EFgQUZC/cSYq756t1XsfKnJpMWp35XtgwHwYDVR0j"
    "BBgwFoAUZC/cSYq756t1XsfKnJpMWp35XtgwDwYDVR0TAQH/BAUwAwEB/zAOBgNVHQ8BAf8EBAMC"
    "AYYwDQYJKoZIhvcNAQELBQADggIBAAsTVMHh7gNRxLl6LvwsjjbWv70skUBE6P8AOBobw23YbeN0"
    "SzLYjIl7Eh3KhEzNAjRc+S2JL124oCgFcrAgOuNORJDP91w8S0QbGGD6GIH1RJ0SfYJxgx0TJUJu"
    "u0eQTw4SI3/RqLRI2HRsHrpF5QoLWWtsHoK+DraMZgSeJuv3xxPMHn23Gdl+ijMIArLZSbGK6nht"
    "kLaV5oMV5K2EHCGESmrrv1NIAW30cVtJssDE6OSxZNtjW+v67KI8yP61YlS7meiu2ggV5+jHsfn9"
    "3nimu9/MIhE7oPX+JLQC3qmCtvk3zmXWbrSI+6N450AYByXEHy4hsD6edTr3nGzYyenKtEbBJUiq"
    "iQUofXnBqSVnw12bIHm+DjNL41OnRb/SeRVcSBfCnHzWC6N2zqRD9yjyFNT9DO/1fPaN7WPyF8jk"
    "ZTEFkhsqBIKCK4vb8qS+CVp6ILi+uUblgsV84umQNFXiuAAfEykZjUBqHqDDNSTPxbm40KYQzzen"
    "EeBpi+3Zr2s10ignxl5iOAC0guhKwDHaNgEsTyx9xHi+0bRyJnTxaT1NfHO7/3xnvUU7JsYErIDp"
    "FKNLbu90ALBPDK/MPxM1gxqUDcsRsWiYn5IKz9l+zbPH8AnEiEs/SunrknhZPH/esHuchhF0/dDI"
    "DgZC34+YDichGiYyqCbeRwwzhg68\n"
    "-----END CERTIFICATE-----\n";

static const std::string VALID_CLIENT_CERTIFICATE_BASE64 =
    "-----BEGIN CERTIFICATE-----\n"
    "MIICzzCCAnWgAwIBAgICOU0wCgYIKoZIzj0EAwIwaTELMAkGA1UEBhMCQkUxETAPBgNVBAgTCEJy"
    "dXNzZWxzMREwDwYDVQQHEwhCcnVzc2VsczETMBEGA1UEChMKU21hcnRtYXRpYzENMAsGA1UECxME"
    "U0FFUzEQMA4GA1UEAxMHQ0EtMjAxODAeFw0xODAxMDIxOTQ4MjZaFw0xODEyMzEwMDAwMDBaMHYx"
    "HTAbBgNVBAMMFEFuZGVybGVjaHRfMDAxXzIxMDAxMQ0wCwYDVQQLDARTQUVTMRMwEQYDVQQKDApT"
    "bWFydG1hdGljMREwDwYDVQQHDAhCcnVzc2VsczERMA8GA1UECAwIQnJ1c3NlbHMxCzAJBgNVBAYT"
    "AkJFMIGbMBAGByqGSM49AgEGBSuBBAAjA4GGAAQBeAM79xzDMu1XjnrRsm+KOQ1NnlutSOkbzYSL"
    "W06eNDcgwwC/4ey9F70pA4UXHx/3D80DviwEdx2pIXt4utmAZJsAzG29+I5d+iKMpL9qFkiZkjFL"
    "WSlhe/2728XQv009HHUbhAAzkiyAhTMqz8Q3eV5+aDJ7nUF9B3PRPIDpHoilK6WjgbwwgbkwgZgG"
    "A1UdIwSBkDCBjYAUmPGkneaZ7UFyFhRgpnvAfC3wJCKhcqRwMG4xCzAJBgNVBAYTAkJFMREwDwYD"
    "VQQIEwhCcnVzc2VsczERMA8GA1UEBxMIQnJ1c3NlbHMxEzARBgNVBAoTClNtYXJ0bWF0aWMxDTAL"
    "BgNVBAsTBFNBRVMxFTATBgNVBAMTDFJPT1QtQ0EtMjAxOIIBATAMBgNVHRMBAf8EAjAAMA4GA1Ud"
    "DwEB/wQEAwIFoDAKBggqhkjOPQQDAgNIADBFAiA/gZvOyulE783wsrt9+6qiQ7LZQE/GBuT75BJ8"
    "R9UGdgIhALg0a2qUGyvKDo7kZU6vkXviwQlFftVhrBVWQGXlucK/\n"
    "-----END CERTIFICATE-----";

static const std::string VALID_ROOT_CERTIFICATE_BASE64 =
    "-----BEGIN CERTIFICATE-----\n"
    "MIIC0jCCAnigAwIBAgIJAJsXLtrWFaN6MAoGCCqGSM49BAMCMG4xCzAJBgNVBAYTAkJFMREwDwYD"
    "VQQIEwhCcnVzc2VsczERMA8GA1UEBxMIQnJ1c3NlbHMxEzARBgNVBAoTClNtYXJ0bWF0aWMxDTAL"
    "BgNVBAsTBFNBRVMxFTATBgNVBAMTDFJPT1QtQ0EtMjAxODAeFw0xODAxMDIxOTQ4MjZaFw0xOTAx"
    "MDExOTQ4MjZaMG4xCzAJBgNVBAYTAkJFMREwDwYDVQQIEwhCcnVzc2VsczERMA8GA1UEBxMIQnJ1"
    "c3NlbHMxEzARBgNVBAoTClNtYXJ0bWF0aWMxDTALBgNVBAsTBFNBRVMxFTATBgNVBAMTDFJPT1Qt"
    "Q0EtMjAxODBZMBMGByqGSM49AgEGCCqGSM49AwEHA0IABGMF39EXPHz2+gqiALpQJ8Uc6nS1DIM9"
    "Oh8J4a1VjgujIkuhfVl+Z5l+pRMDdsDobrbH46BJIA6J4mvjb4eAeBujgf4wgfswKQYJYIZIAYb4"
    "QgENBBwWGkNlcnRpZmljYXRlIGlzc3VlZCBieSBTbXR0MB0GA1UdDgQWBBRs6EvAcFne+CDTNYT7"
    "K0A+y7eLLjCBoAYDVR0jBIGYMIGVgBRs6EvAcFne+CDTNYT7K0A+y7eLLqFypHAwbjELMAkGA1UE"
    "BhMCQkUxETAPBgNVBAgTCEJydXNzZWxzMREwDwYDVQQHEwhCcnVzc2VsczETMBEGA1UEChMKU21h"
    "cnRtYXRpYzENMAsGA1UECxMEU0FFUzEVMBMGA1UEAxMMUk9PVC1DQS0yMDE4ggkAmxcu2tYVo3ow"
    "DAYDVR0TBAUwAwEB/zAKBggqhkjOPQQDAgNIADBFAiEAx9zrGjP/RaRsXEm22Qnf6ENVhkFJiNRp"
    "RVULxlRBlYkCIAGX9GydKOvNuI0oZFpRSfkx2g9nbLIe+66T1Q5oeuau\n"
    "-----END CERTIFICATE-----";

static std::string VALID_INTERMEDIATE_CERTIFICATE =
    "-----BEGIN CERTIFICATE-----\n"
    "MIICxDCCAmugAwIBAgIBATAKBggqhkjOPQQDAjBuMQswCQYDVQQGEwJCRTERMA8GA1UECBMIQnJ1"
    "c3NlbHMxETAPBgNVBAcTCEJydXNzZWxzMRMwEQYDVQQKEwpTbWFydG1hdGljMQ0wCwYDVQQLEwRT"
    "QUVTMRUwEwYDVQQDEwxST09ULUNBLTIwMTgwHhcNMTgwMTAyMTk0ODI2WhcNMTgxMjMxMDAwMDAw"
    "WjBpMQswCQYDVQQGEwJCRTERMA8GA1UECBMIQnJ1c3NlbHMxETAPBgNVBAcTCEJydXNzZWxzMRMw"
    "EQYDVQQKEwpTbWFydG1hdGljMQ0wCwYDVQQLEwRTQUVTMRAwDgYDVQQDEwdDQS0yMDE4MFkwEwYH"
    "KoZIzj0CAQYIKoZIzj0DAQcDQgAEsTms1BGDUOBIxelOatCJyUTP1s9Iz1kco4E39o+EodiaPUsQ"
    "d+dOasJUWtXq6jBLdnJ6hWuQMDA6NjQ6RvX06aOB/jCB+zApBglghkgBhvhCAQ0EHBYaQ2VydGlm"
    "aWNhdGUgaXNzdWVkIGJ5IFNtdHQwHQYDVR0OBBYEFJjxpJ3mme1BchYUYKZ7wHwt8CQiMIGgBgNV"
    "HSMEgZgwgZWAFGzoS8BwWd74INM1hPsrQD7Lt4suoXKkcDBuMQswCQYDVQQGEwJCRTERMA8GA1UE"
    "CBMIQnJ1c3NlbHMxETAPBgNVBAcTCEJydXNzZWxzMRMwEQYDVQQKEwpTbWFydG1hdGljMQ0wCwYD"
    "VQQLEwRTQUVTMRUwEwYDVQQDEwxST09ULUNBLTIwMTiCCQCbFy7a1hWjejAMBgNVHRMEBTADAQH/"
    "MAoGCCqGSM49BAMCA0cAMEQCIFoTE0PD50sGKx8JfzUyJeRvlyl1irIfbRVEfDb9t5+9AiBiPjBs"
    "v/6rTCplY6a4H9q987mlzwXFckjz1MdePhl1iA==\n"
    "-----END CERTIFICATE-----\n";

class CertificateValidatorWrapperTest : public CPPUNIT_NS::TestFixture
{
    private:

        CertificateValidatorWrapper *m_certificateValidatorWrapper;

    public:

        CertificateValidatorWrapperTest()
        : m_certificateValidatorWrapper(NULL)
        {

        }

        void setUp()
        {
            m_certificateValidatorWrapper = CertificateValidatorWrapper::getInstance();

            MockSaesCommonConfigurationManager* mockConfigurationManager = new MockSaesCommonConfigurationManager();
            SAESCommonConfigurationManager::getInstance(mockConfigurationManager);

            MockKeyManager* mockKeyManager = new MockKeyManager();
            KeyManager::getInstance(mockKeyManager);

            OpenSSLSecurity::init();
            XmlSecurityC::init();

            File::getInstance()->~File();
        }

        void tearDown()
        {
            OpenSSLSecurity::clear();
            XmlSecurityC::clear();

            File::getInstance()->~File();
            File::getInstance()->DeleteFile(ROOT_CERTIFICATE_TEMP_PATH);
            File::getInstance()->~File();
        }

        void validateRootCertificate_rootEmptyPath_failedValidation()
        {
            CPPUNIT_ASSERT_THROW(
                m_certificateValidatorWrapper->validateRootCertificate(EMPTY_CHAR, VALID_ROOT_CERTIFICATE_FINGERPRINT),
                CryptoException);
        }

        void validateRootCertificate_invalidRootFingerprint_failedValidation()
        {
            SafeBuffer buffer(VALID_ROOT_CERTIFICATE_BASE64);

            StoreMockFile file;
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(buffer, true));

            CPPUNIT_ASSERT_THROW(
                m_certificateValidatorWrapper->validateRootCertificate(FILE_PATH, INVALID_ROOT_CERTIFICATE_FINGERPRINT),
                CryptoException);
        }

        void validateRootCertificate_invalidRootCertificate_failedValidation()
        {
            SafeBuffer buffer(INVALID_ROOT_CERTIFICATE_BASE64);

            StoreMockFile file;
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(buffer, true));

            CPPUNIT_ASSERT_THROW(
                m_certificateValidatorWrapper->validateRootCertificate(FILE_PATH, VALID_ROOT_CERTIFICATE_FINGERPRINT),
                CryptoException);
        }

        void validateRootCertificate_validRootCertificate_successfulValidation()
        {
            SafeBuffer buffer(VALID_ROOT_CERTIFICATE_BASE64);

            StoreMockFile file;
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(buffer, true));

            CPPUNIT_ASSERT_NO_THROW(
                m_certificateValidatorWrapper->validateRootCertificate(FILE_PATH, VALID_ROOT_CERTIFICATE_FINGERPRINT));
        }

        void validateIntermediateCertificate_intermediateEmptyPath_failedValidation()
        {
            std::string emptyChar = EMPTY_CHAR;

            CPPUNIT_ASSERT_THROW(
                m_certificateValidatorWrapper->validateIntermediateCertificate(emptyChar),
                CryptoException);
        }

        void validateIntermediateCertificate_invalidIntermediateCertificate_failedValidation()
        {
            SafeBuffer rootBuffer(ALTERNATIVE_ROOT_CERTIFICATE_BASE64);
            File::getInstance()->addBufferToFile(rootBuffer, ROOT_CERTIFICATE_TEMP_PATH);
            File::getInstance()->~File();

            StoreMockFile file;
            SafeBuffer emptyBuffer;
            SafeBuffer intermediateBuffer(VALID_INTERMEDIATE_CERTIFICATE);

            std::string   filePath = FILE_PATH;
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(rootBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(emptyBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(intermediateBuffer, true));

            CPPUNIT_ASSERT_NO_THROW(m_certificateValidatorWrapper->validateRootCertificate(
                ROOT_CERTIFICATE_TEMP_PATH, ALTERNATIVE_ROOT_CERTIFICATE_FINGERPRINT));

            setSecurityConfiguration();

            CPPUNIT_ASSERT_THROW(
                m_certificateValidatorWrapper->validateIntermediateCertificate(filePath),
                CryptoException);
        }

        void validateIntermediateCertificate_selfSignedIntermediateCertificate_failedValidation()
        {
            SafeBuffer rootBuffer(VALID_ROOT_CERTIFICATE_BASE64);
            File::getInstance()->addBufferToFile(rootBuffer, ROOT_CERTIFICATE_TEMP_PATH);
            File::getInstance()->~File();

            StoreMockFile file;
            SafeBuffer    intermediateBuffer(SELF_SIGNED_INTERMEDIATE_CERTIFICATE);

            std::string   filePath = FILE_PATH;
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(rootBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(intermediateBuffer, true));

            CPPUNIT_ASSERT_NO_THROW(m_certificateValidatorWrapper->validateRootCertificate(
                ROOT_CERTIFICATE_TEMP_PATH, VALID_ROOT_CERTIFICATE_FINGERPRINT));

            setSecurityConfiguration();

            CPPUNIT_ASSERT_THROW(
                m_certificateValidatorWrapper->validateIntermediateCertificate(filePath),
                CryptoException);
        }

        void validateIntermediateCertificate_validIntermediateCertificate_successfulValidation()
        {
            SafeBuffer rootBuffer(VALID_ROOT_CERTIFICATE_BASE64);
            File::getInstance()->addBufferToFile(rootBuffer, ROOT_CERTIFICATE_TEMP_PATH);
            File::getInstance()->~File();

            StoreMockFile file;
            SafeBuffer    intermediateBuffer(VALID_INTERMEDIATE_CERTIFICATE);

            std::string   filePath = FILE_PATH;
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(rootBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(intermediateBuffer, true));

            CPPUNIT_ASSERT_NO_THROW(m_certificateValidatorWrapper->validateRootCertificate(
                ROOT_CERTIFICATE_TEMP_PATH, VALID_ROOT_CERTIFICATE_FINGERPRINT));

            setSecurityConfiguration();

            CPPUNIT_ASSERT_NO_THROW(m_certificateValidatorWrapper->validateIntermediateCertificate(filePath));
        }

        void validateClientCertificate_invalidFormat_failedValidation()
        {
            SafeBuffer rootBuffer(VALID_ROOT_CERTIFICATE_BASE64);
            File::getInstance()->addBufferToFile(rootBuffer, ROOT_CERTIFICATE_TEMP_PATH);
            File::getInstance()->~File();

            StoreMockFile file;
            SafeBuffer    intermediateBuffer(VALID_INTERMEDIATE_CERTIFICATE);
            SafeBuffer    clientBuffer(VALID_CLIENT_CERTIFICATE_BASE64);

            std::string   filePath = FILE_PATH;
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(rootBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(intermediateBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(clientBuffer, true));

            CPPUNIT_ASSERT_NO_THROW(m_certificateValidatorWrapper->validateRootCertificate(
                ROOT_CERTIFICATE_TEMP_PATH, VALID_ROOT_CERTIFICATE_FINGERPRINT));

            setSecurityConfiguration();

            CPPUNIT_ASSERT_NO_THROW(m_certificateValidatorWrapper->validateIntermediateCertificate(filePath));

            CPPUNIT_ASSERT_THROW(m_certificateValidatorWrapper->validateClientCertificate(
                filePath, INVALID_CLIENT_CERTIFICATE_FORMAT, VALID_CLIENT_CERTIFICATE_PROTECTION_KEY),
                CryptoException);
        }

        void validateClientCertificate_invalidIssuerCertificates_failedValidation()
        {
            SafeBuffer rootBuffer(ALTERNATIVE_ROOT_CERTIFICATE_BASE64);
            File::getInstance()->addBufferToFile(rootBuffer, ROOT_CERTIFICATE_TEMP_PATH);
            File::getInstance()->~File();

            StoreMockFile file;
            SafeBuffer    intermediateBuffer(ALTERNATIVE_INTERMEDIATE_CERTIFICATE);
            SafeBuffer    clientBuffer(VALID_CLIENT_CERTIFICATE_BASE64);

            std::string   filePath = FILE_PATH;
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(rootBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(intermediateBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(clientBuffer, true));

            CPPUNIT_ASSERT_NO_THROW(m_certificateValidatorWrapper->validateRootCertificate(
                ROOT_CERTIFICATE_TEMP_PATH, ALTERNATIVE_ROOT_CERTIFICATE_FINGERPRINT));

            setSecurityConfiguration();

            CPPUNIT_ASSERT_NO_THROW(m_certificateValidatorWrapper->validateIntermediateCertificate(filePath));

            CPPUNIT_ASSERT_THROW(m_certificateValidatorWrapper->validateClientCertificate(
                filePath, VALID_CLIENT_CERTIFICATE_FORMAT, VALID_CLIENT_CERTIFICATE_PROTECTION_KEY),
                CryptoException);
        }

        void validateClientCertificate_validClientCertificate_successfulValidation()
        {
            SafeBuffer rootBuffer(VALID_ROOT_CERTIFICATE_BASE64);
            File::getInstance()->addBufferToFile(rootBuffer, ROOT_CERTIFICATE_TEMP_PATH);
            File::getInstance()->~File();

            StoreMockFile file;
            SafeBuffer    intermediateBuffer(VALID_INTERMEDIATE_CERTIFICATE);
            SafeBuffer    clientBuffer(VALID_CLIENT_CERTIFICATE_BASE64);

            std::string   filePath = FILE_PATH;
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(rootBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(intermediateBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(clientBuffer, true));

            CPPUNIT_ASSERT_NO_THROW(m_certificateValidatorWrapper->validateRootCertificate(
                ROOT_CERTIFICATE_TEMP_PATH, VALID_ROOT_CERTIFICATE_FINGERPRINT));

            setSecurityConfiguration();

            CPPUNIT_ASSERT_NO_THROW(m_certificateValidatorWrapper->validateIntermediateCertificate(filePath));

            CPPUNIT_ASSERT_NO_THROW(m_certificateValidatorWrapper->validateClientCertificate(
                filePath, VALID_CLIENT_CERTIFICATE_FORMAT, VALID_CLIENT_CERTIFICATE_PROTECTION_KEY));
        }

        void setSecurityConfiguration()
        {
            SecurityFileConfiguration::ValidateCertifiedConfigSequence sequence;
            sequence.push_back(ValidateCertifiedConfiguration::CONFIGURATION_SET_CHECK_TIME);

            SecurityConfiguration& securityConfiguration(
                SAESCommonConfigurationManager::getInstance()->getConfiguration()->getMachineSecuritySchema());
            securityConfiguration.getInstallationFiles().setEncryptedData(false);
            securityConfiguration.setValidateCertifiedTime(TimeConfiguration::CONFIGURATION_MACHINE_TIME);
            securityConfiguration.getInstallationFiles().setValidateCertifiedConfig(sequence);

            Encryption::getInstance()->setMachineTime(VALID_TIMESTAMP);
        }

        CPPUNIT_TEST_SUITE(CertificateValidatorWrapperTest);

        CPPUNIT_TEST(validateRootCertificate_rootEmptyPath_failedValidation);
        CPPUNIT_TEST(validateRootCertificate_invalidRootFingerprint_failedValidation);
        CPPUNIT_TEST(validateRootCertificate_invalidRootCertificate_failedValidation);
        CPPUNIT_TEST(validateRootCertificate_validRootCertificate_successfulValidation);

        CPPUNIT_TEST(validateIntermediateCertificate_intermediateEmptyPath_failedValidation);
        CPPUNIT_TEST(validateIntermediateCertificate_invalidIntermediateCertificate_failedValidation);
        CPPUNIT_TEST(validateIntermediateCertificate_selfSignedIntermediateCertificate_failedValidation);
        CPPUNIT_TEST(validateIntermediateCertificate_validIntermediateCertificate_successfulValidation);

        CPPUNIT_TEST(validateClientCertificate_invalidFormat_failedValidation);
        CPPUNIT_TEST(validateClientCertificate_invalidIssuerCertificates_failedValidation);
        CPPUNIT_TEST(validateClientCertificate_validClientCertificate_successfulValidation);

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* CERTIFICATE_STORE_TEST */
