#ifndef TEST_KEY_MANAGER_TEST
#define TEST_KEY_MANAGER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Security/KeyManager.hxx>
#include <System/Security/OpenSSLSecurity.hxx>
#include "MockKeyManager.hxx"

class KeyManagerTest : public CPPUNIT_NS::TestFixture
{
    private:

        Smartmatic::SAES::Common::Security::KeyManager * keyManager;

    public:

        KeyManagerTest ()
        {
            keyManager = NULL;
        }

        void setUp()
        {
            MockKeyManager* mockKeyManager = new MockKeyManager();
            Smartmatic::System::Security::OpenSSLSecurity::init();
            keyManager = Smartmatic::SAES::Common::Security::KeyManager::getInstance(mockKeyManager);
        }

        void tearDown()
        {
            if ( keyManager )
            {
                delete keyManager;
                keyManager = NULL;
            }

            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

        void testFindSecretKeyIfNotAdd()
        {
            Smartmatic::System::SafeBuffer key;
            keyManager->getKey ( Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY, key );

            CPPUNIT_ASSERT ( 0 == key.GetSize() );
        }

        void testFindSecretKeyIfAdd()
        {
            Smartmatic::System::SafeBuffer key ( "abcd1234567890" );
            keyManager->addKeyByType ( key, Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );

            Smartmatic::System::SafeBuffer resultKey;
            keyManager->getKey ( Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY, resultKey );

            CPPUNIT_ASSERT ( 0 != key.GetSize() );
            CPPUNIT_ASSERT ( key.GetBase64Data() == resultKey.GetBase64Data() );
        }

        void testFindUPKeyIfNotAdd ()
        {
            Smartmatic::System::SafeBuffer key;
            keyManager->getKey ( Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_UP_KEY, key );

            CPPUNIT_ASSERT ( 0 == key.GetSize() );
        }

        void testFindUPKeyIfAdd ()
        {
            Smartmatic::System::SafeBuffer key ( "abcd1234567890" );
            keyManager->addKeyByType ( key, Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_UP_KEY );

            Smartmatic::System::SafeBuffer resultKey;
            keyManager->getKey ( Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_UP_KEY, resultKey );

            CPPUNIT_ASSERT ( key.GetBase64Data() == resultKey.GetBase64Data() );
        }

        void testFindUPKeyIfNotAddAndSecretKeyAdd ()
        {
            Smartmatic::System::SafeBuffer key ( "abcd1234567890" );
            keyManager->addKeyByType ( key, Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );

            Smartmatic::System::SafeBuffer resultKey;
            keyManager->getKey ( Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_UP_KEY, resultKey );

            CPPUNIT_ASSERT ( key.GetBase64Data() == resultKey.GetBase64Data() );
        }

        void testFindSecurityKeyIfNotAdd()
        {
            Smartmatic::System::SafeBuffer key;
            Smartmatic::System::SafeBuffer initkey;

            keyManager->getKey ( Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECURITY_KEY, key );
            keyManager->getKey ( Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_INIT_KEY, initkey );

            CPPUNIT_ASSERT ( 0 != key.GetSize() );
            CPPUNIT_ASSERT ( key.GetBase64Data() == initkey.GetBase64Data() );
        }

        void testFindSecurityKeyIfAdd()
        {
            Smartmatic::System::SafeBuffer key ( "abcd1234567890" );
            keyManager->addKeyByType ( key, Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECURITY_KEY );

            Smartmatic::System::SafeBuffer resultKey;
            keyManager->getKey ( Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECURITY_KEY, resultKey );

            CPPUNIT_ASSERT ( 0 != key.GetSize() );
            CPPUNIT_ASSERT ( key.GetBase64Data() == resultKey.GetBase64Data() );
        }

        CPPUNIT_TEST_SUITE ( KeyManagerTest );
        CPPUNIT_TEST ( testFindSecretKeyIfNotAdd );
        CPPUNIT_TEST ( testFindSecretKeyIfAdd );
        CPPUNIT_TEST ( testFindUPKeyIfNotAdd );
        CPPUNIT_TEST ( testFindUPKeyIfAdd );
        CPPUNIT_TEST ( testFindUPKeyIfNotAddAndSecretKeyAdd );
        CPPUNIT_TEST ( testFindSecurityKeyIfNotAdd );
        CPPUNIT_TEST ( testFindSecurityKeyIfAdd );
        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_KEY_MANAGER_TEST */
