#ifndef TEST_SECRETKEY_TEST
#define TEST_SECRETKEY_TEST

#ifdef ECUT_MAIN

#include "SecretKeyTestData.h"

#include <cppunit/extensions/HelperMacros.h>
#include <tests/MockSaesCommonConfigurationManager.hxx>
#include <tests/MockKeyManager.hxx>
#include <System/IO/File.h>
#include <System/Exception/SecretKeyException.h>
#include <System/Security/OpenSSLSecurity.hxx>
#include <Security/SecretKey.h>
#include <Security/KeyManager.hxx>
#include <System/SafeBuffer.h>
#include <tests/StoreMockFile.hxx>
#include <string>

using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Common::Security;
using namespace Smartmatic::SAES::Common::Configuration;

class SecretKeyTest : public CPPUNIT_NS::TestFixture
{
    private:

        std::string                         m_random;
        std::string                         m_code;
        std::string                         m_officialPassphrase;
        std::string                         m_recountPassphrase;
        MockSaesCommonConfigurationManager* m_configurationManager;
        MockKeyManager*                     m_keyManager;
        StoreMockFile*                      m_mockFile;

    public:

        SecretKeyTest()
        {
            m_configurationManager = NULL;
            m_keyManager = NULL;
            m_mockFile = NULL;
        }

        ~SecretKeyTest()
        {
            delete m_configurationManager;
            delete m_keyManager;
            delete m_mockFile;
        }

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();

            m_random             = "WgCXasNFDsbkkcPpKwKYQOYmlWRicgFc";
            m_code               = "saint-gilles015";
            m_officialPassphrase = "gcabpdpgytufxvbj";
            m_recountPassphrase  = "ykzjtfjupfqfdtzu";

            m_configurationManager = new MockSaesCommonConfigurationManager();
            SAESCommonConfigurationManager::getInstance(m_configurationManager);

            m_keyManager = new MockKeyManager();
            KeyManager::getInstance(m_keyManager);

            m_mockFile = new StoreMockFile();
            IO::File::getInstance(m_mockFile);
        }

        void tearDown()
        {
            delete m_keyManager;
            delete m_configurationManager;
            delete m_mockFile;

            m_keyManager = NULL;
            m_configurationManager = NULL;
            m_mockFile = NULL;

            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

    public:

        void test_valid_official()
        {
            m_mockFile->canOpenFileReturnValues.push_back(true);

            m_mockFile->openFileValues.push_back (
                StoreMockFile::OpenFileValues ( SafeBuffer ( ), false ));

            m_mockFile->openFileValues.push_back (
                StoreMockFile::OpenFileValues ( SafeBuffer ( VALID_SECRET_KEY, VALID_SECRET_KEY_LEN ), true ));

            SecretKey sk("dummy-file", m_officialPassphrase, m_code, m_random);

            std::string b64Key = sk.getKeyBase64();

            CPPUNIT_ASSERT(!b64Key.empty());
        }

        void test_valid_recount()
        {
            m_mockFile->canOpenFileReturnValues.push_back(true);

            m_mockFile->openFileValues.push_back (
                StoreMockFile::OpenFileValues ( SafeBuffer ( VALID_RECOUNT_KEY, VALID_RECOUNT_KEY_LEN ), true ));

            SecretKey sk("dummy-file", m_recountPassphrase, m_code, m_random);

            std::string b64Key = sk.getKeyBase64();

            CPPUNIT_ASSERT(!b64Key.empty());
        }

        void test_invalid_passphrase_official()
        {
            m_mockFile->canOpenFileReturnValues.push_back(true);

            m_mockFile->openFileValues.push_back (
                StoreMockFile::OpenFileValues ( SafeBuffer ( VALID_SECRET_KEY, VALID_SECRET_KEY_LEN ), true ));

            CPPUNIT_ASSERT_THROW (
                SecretKey("dummy-file", m_officialPassphrase + "invalidate", m_code, m_random),
                Exception::SecretKeyException);
        }

        void test_invalid_passphrase_recount()
        {
            m_mockFile->canOpenFileReturnValues.push_back(true);

            m_mockFile->openFileValues.push_back (
                StoreMockFile::OpenFileValues ( SafeBuffer ( VALID_RECOUNT_KEY, VALID_RECOUNT_KEY_LEN ), true ));

            CPPUNIT_ASSERT_THROW (
                SecretKey("dummy-file", m_recountPassphrase + "invalidate", m_code, m_random),
                Exception::SecretKeyException);
        }

        void test_invalid_mac_code_official()
        {
            m_mockFile->canOpenFileReturnValues.push_back(true);

            m_mockFile->openFileValues.push_back (
                StoreMockFile::OpenFileValues (
                    SafeBuffer ( INVALID_MAC_CODE_SECRET_KEY, INVALID_MAC_CODE_SECRET_KEY_LEN ),
                    true ));

            CPPUNIT_ASSERT_THROW (
                SecretKey("dummy-file", m_officialPassphrase, m_code, m_random),
                Exception::SecretKeyException);
        }

        void test_invalid_mac_code_recount()
        {
            m_mockFile->canOpenFileReturnValues.push_back(true);

            m_mockFile->openFileValues.push_back (
                StoreMockFile::OpenFileValues (
                    SafeBuffer ( INVALID_MAC_CODE_RECOUNT_KEY, INVALID_MAC_CODE_RECOUNT_KEY_LEN ),
                    true ));

            CPPUNIT_ASSERT_THROW (
                SecretKey("dummy-file", m_recountPassphrase, m_code, m_random),
                Exception::SecretKeyException);
        }

        void test_invalid_mac_official()
        {
            m_mockFile->canOpenFileReturnValues.push_back(true);

            m_mockFile->openFileValues.push_back (
                StoreMockFile::OpenFileValues (
                    SafeBuffer ( INVALID_MAC_SECRET_KEY, INVALID_MAC_SECRET_KEY_LEN ),
                    true ));

            CPPUNIT_ASSERT_THROW (
                SecretKey("dummy-file", m_officialPassphrase, m_code, m_random),
                Exception::SecretKeyException);
        }

        void test_invalid_mac_recount()
        {
            m_mockFile->canOpenFileReturnValues.push_back(true);

            m_mockFile->openFileValues.push_back (
                StoreMockFile::OpenFileValues (
                    SafeBuffer ( INVALID_MAC_RECOUNT_KEY, INVALID_MAC_RECOUNT_KEY_LEN ),
                    true ));

            CPPUNIT_ASSERT_THROW (
                SecretKey("dummy-file", m_recountPassphrase, m_code, m_random),
                Exception::SecretKeyException);
        }

        void test_no_mac_official()
        {
            m_mockFile->canOpenFileReturnValues.push_back(true);

            m_mockFile->openFileValues.push_back (
                StoreMockFile::OpenFileValues (
                    SafeBuffer ( NO_MAC_SECRET_KEY, NO_MAC_SECRET_KEY_LEN ),
                    true ));

            CPPUNIT_ASSERT_THROW (
                SecretKey("dummy-file", m_officialPassphrase, m_code, m_random),
                Exception::SecretKeyException);
        }

        void test_no_mac_recount()
        {
            m_mockFile->canOpenFileReturnValues.push_back(true);

            m_mockFile->openFileValues.push_back (
                StoreMockFile::OpenFileValues (
                    SafeBuffer ( NO_MAC_RECOUNT_KEY, NO_MAC_RECOUNT_KEY_LEN ),
                    true ));

            CPPUNIT_ASSERT_THROW (
                SecretKey("dummy-file", m_recountPassphrase, m_code, m_random),
                Exception::SecretKeyException);
        }

    public:

        CPPUNIT_TEST_SUITE ( SecretKeyTest );

        CPPUNIT_TEST ( test_valid_official );
        CPPUNIT_TEST ( test_valid_recount );
        CPPUNIT_TEST ( test_invalid_passphrase_official );
        CPPUNIT_TEST ( test_invalid_passphrase_recount );
        CPPUNIT_TEST ( test_invalid_mac_code_official );
        CPPUNIT_TEST ( test_invalid_mac_code_recount );
        CPPUNIT_TEST ( test_invalid_mac_official );
        CPPUNIT_TEST ( test_invalid_mac_recount );
        CPPUNIT_TEST ( test_no_mac_official );
        CPPUNIT_TEST ( test_no_mac_recount );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_SECRETKEY_TEST */
