/**
 * @file s-a-e-s-functionality-conf-manager.cc
 * @brief Implementation of the SAESFunctionalityConfigurationManager class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */


#include "s-a-e-s-functionality-conf-manager.h"
#include <System/ZenityWrapper.hxx>
#include <Configuration/ConfigurationBase-schema.hxx>
#include "Configuration/configuration-persistence-manager.h"
#include <System/Runtime/ExitProcess.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Runtime;

SAESFunctionalityConfigurationManager * SAESFunctionalityConfigurationManager::instance = NULL;

SAESFunctionalityConfigurationManager::SAESFunctionalityConfigurationManager()
{
    configuration = NULL;
}

SAESFunctionalityConfigurationManager::~SAESFunctionalityConfigurationManager()
{
    if ( configuration != NULL )
    {
        delete ( configuration );
        configuration = NULL;
    }

    instance = NULL;
}

void SAESFunctionalityConfigurationManager::initialize()
{
    try
    {
        std::string semiFile ( ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "SAESFunctionality" ) );
        std::istringstream ss ( semiFile );
        configuration = new SaesFunctionalityConfiguration ( * parseSaesFunctionalityConfiguration ( ss, ConfigurationTypes::Flags::dont_validate ).get() );
    }
    catch ( const ::xsd::cxx::tree::exception< char > & e )
    {
        std::stringstream ss;
        ss << e;
        ZenityWrapper::showError ( "Config SAESFUNCTIONALITY Error \n\n" + ss.str(), "Error" );
        ExitProcess::getInstance()->exitProcess();
    }
}

SaesFunctionalityConfiguration  * SAESFunctionalityConfigurationManager::getConfiguration()
{
    if ( configuration == NULL )
    {
        initialize();
    }

    return configuration;
}

void SAESFunctionalityConfigurationManager::loadDefault()
{
    SaesFunctionalityConfiguration configSAESFunctionality;
    AlarmsConfiguration  alarmConf;

    Alarm alarmUnableToRead;
    Alarm alarmUnableToWrite;
    Alarm alarmCryptoError;
    Alarm alarmInvalidContent;
    Alarm alarmCardUsed;
    Alarm alarmCardValidityPeriodExpired;
    Alarm alarmInvalidVotingData;
    Alarm alarmPaperJam;
    Alarm alarmPaperOut;
    Alarm alarmFailedVoteCompliance;
    Alarm alarmNeop;
    Alarm alarmTimeout;
    Alarm alarmUnexpectedError;
    Alarm alarmCardNotRemovedAfterVoting;

    AlarmCommand alarmBuzzCommand;
    AlarmCommand alarmGreenCommand;
    AlarmCommand alarmRedLedCommand;

    AlarmsConfiguration::AlarmSequence alarmsSeq;
    Alarm::AlarmCommandsSequence alarmCommandSequence;

    RunFunctionalityConfiguration runFuncConfig;

    RunFunctionalityConfiguration::FunctionalityDescriptorSequence functionalityDescriptors;
    functionalityDescriptors.push_back ( FunctionalityDescriptorType ( MenuOptionType::CONFIGURATION_XMLMVOTINGEXPERIENCEVALIDATIONFUNCTIONALITY ) );
    runFuncConfig.setFunctionalityDescriptor ( functionalityDescriptors );

    configSAESFunctionality.setRunFunctionalityConfiguration ( runFuncConfig );

    MachineInitiatorConfiguration machineConf;
    Alarm cardExtractionCounter;

    VotingExperienceControllerConfiguration vecc;
    VotingExperienceControllerOperationType vecot = VotingExperienceControllerOperationType::CONFIGURATION_PRIORITY_BY_CONTEST_CLASS;
    VotingExperienceControllerValidateCardDataType vcdt = VotingExperienceControllerValidateCardDataType::CONFIGURATION_ALL;

    vecc.setOperationType ( vecot );
    vecc.setValidateCardDataType ( vcdt );
    vecc.setSkipCustomAttributeAutomatic ( true );
    vecc.setNavigateBetweenContest ( false );

    AudioVotingExperienceControllerConfiguration avecc;
    avecc.setDeselectAudioOptionsWhenReturningToPrincipalList(false);
    avecc.setVotingMachineExpectedSerialNumber("P51");

    configSAESFunctionality.setVotingExperienceController ( vecc );
    configSAESFunctionality.setAudioVotingExperienceController( avecc );

    SystemRoleConfiguration src;
    SystemRoleConfiguration::RoleSequence & sequence ( src.getRole() );

    Role roleMenuActivation;
    roleMenuActivation.setKey ( "260780" );
    roleMenuActivation.setNameKey ( "MenuActivation" );
    roleMenuActivation.setDescriptionKey ( "MenuActivation" );
    roleMenuActivation.setType ( SystemRoleType::CONFIGURATION_MENU );

    Role roleVotingActivation;
    roleVotingActivation.setKey ( "210580" );
    roleVotingActivation.setNameKey ( "VotingActivation" );
    roleVotingActivation.setDescriptionKey ( "VotingActivation" );
    roleVotingActivation.setType ( SystemRoleType::CONFIGURATION_VOTING );

    Role roleElectoralActivation;
    roleElectoralActivation.setKey ( "abcdef" );
    roleElectoralActivation.setNameKey ( "ElectoralActivation" );
    roleElectoralActivation.setDescriptionKey ( "ElectoralActivation" );
    roleElectoralActivation.setType ( SystemRoleType::CONFIGURATION_ELECTORAL );

    Role roleVotingAudioActivation;
    roleVotingAudioActivation.setKey ( "abcde" );
    roleVotingAudioActivation.setNameKey ( "VotingAudioActivation" );
    roleVotingAudioActivation.setDescriptionKey ( "VotingAudioActivation" );
    roleVotingAudioActivation.setType ( SystemRoleType::CONFIGURATION_VOTING_AUDIO );

    Role roleClearAlarm;
    roleClearAlarm.setKey ( "00000" );
    roleClearAlarm.setNameKey ( "ClearAlarm" );
    roleClearAlarm.setDescriptionKey ( "ClearAlarm" );
    roleClearAlarm.setType ( SystemRoleType::CONFIGURATION_CLEAR_ALARM );

    sequence.push_back ( roleMenuActivation );
    sequence.push_back ( roleVotingActivation );
    sequence.push_back ( roleElectoralActivation );
    sequence.push_back ( roleVotingAudioActivation );
    sequence.push_back ( roleClearAlarm );

    alarmBuzzCommand.setTypeOfCommand ( AlarmCommandEnum::CONFIGURATION_BUZZER );
    alarmBuzzCommand.setPeriod ( 100 );
    alarmBuzzCommand.setIsOff ( false );

    alarmRedLedCommand.setTypeOfCommand ( AlarmCommandEnum::CONFIGURATION_REDLED );
    alarmRedLedCommand.setPeriod ( 100 );
    alarmRedLedCommand.setIsOff ( false );

    alarmGreenCommand.setTypeOfCommand ( AlarmCommandEnum::CONFIGURATION_GREENLED );
    alarmGreenCommand.setPeriod ( 100 );
    alarmGreenCommand.setIsOff ( false );

    alarmCommandSequence.push_back ( alarmBuzzCommand );
    alarmCommandSequence.push_back ( alarmRedLedCommand );
    alarmCommandSequence.push_back ( alarmGreenCommand );

    alarmUnableToRead.setAlarmCommands ( alarmCommandSequence );
    alarmUnableToRead.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_UNABLETOREAD );
    alarmUnableToRead.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    alarmUnableToRead.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmUnableToWrite.setAlarmCommands ( alarmCommandSequence );
    alarmUnableToWrite.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_UNABLETOWRITE );
    alarmUnableToWrite.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    alarmUnableToWrite.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmCryptoError.setAlarmCommands ( alarmCommandSequence );
    alarmCryptoError.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_CRYPTOERROR );
    alarmCryptoError.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    alarmCryptoError.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmInvalidContent.setAlarmCommands ( alarmCommandSequence );
    alarmInvalidContent.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_INVALIDCONTENT );
    alarmInvalidContent.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    alarmInvalidContent.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmCardUsed.setAlarmCommands ( alarmCommandSequence );
    alarmCardUsed.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_CARDUSED );
    alarmCardUsed.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_CRITICALALARM );
    alarmCardUsed.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmCardValidityPeriodExpired.setAlarmCommands ( alarmCommandSequence );
    alarmCardValidityPeriodExpired.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_CARDVALIDITYPERIODEXPIRED );
    alarmCardValidityPeriodExpired.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    alarmCardValidityPeriodExpired.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmInvalidVotingData.setAlarmCommands ( alarmCommandSequence );
    alarmInvalidVotingData.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_INVALIDVOTINGDATA );
    alarmInvalidVotingData.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    alarmInvalidVotingData.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmPaperJam.setAlarmCommands ( alarmCommandSequence );
    alarmPaperJam.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_PRINTERNEOPAFTERVOTE );
    alarmPaperJam.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    alarmPaperJam.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmPaperOut.setAlarmCommands ( alarmCommandSequence );
    alarmPaperOut.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_VOTEPRINTINGFAILURE );
    alarmPaperOut.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    alarmPaperOut.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmFailedVoteCompliance.setAlarmCommands ( alarmCommandSequence );
    alarmFailedVoteCompliance.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_VOTECOMPLIANCEFAILURE );
    alarmFailedVoteCompliance.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    alarmFailedVoteCompliance.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmNeop.setAlarmCommands ( alarmCommandSequence );
    alarmNeop.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_PRINTERNOTREADYFORVOTE );
    alarmNeop.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    alarmNeop.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmTimeout.setAlarmCommands ( alarmCommandSequence );
    alarmTimeout.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_TIMEOUT );
    alarmTimeout.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    alarmTimeout.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmUnexpectedError.setAlarmCommands ( alarmCommandSequence );
    alarmUnexpectedError.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_UNEXPECTED );
    alarmUnexpectedError.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_CRITICALALARM );
    alarmUnexpectedError.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_WAIT );

    alarmCardNotRemovedAfterVoting.setAlarmCommands ( alarmCommandSequence );
    alarmCardNotRemovedAfterVoting.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_CARDNOTREMOVEDAFTERVOTING );
    alarmCardNotRemovedAfterVoting.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_NOTALARM );
    alarmCardNotRemovedAfterVoting.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_WAIT );

    cardExtractionCounter.setAlarmCommands ( alarmCommandSequence );
    cardExtractionCounter.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_CARDEXTRACTIONCOUNTER );
    cardExtractionCounter.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_CRITICALALARM );
    cardExtractionCounter.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    Alarm printerNeopBeforeVote;
    printerNeopBeforeVote.setAlarmCommands ( alarmCommandSequence );
    printerNeopBeforeVote.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_PRINTERNEOPBEFOREVOTE );
    printerNeopBeforeVote.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_REGULARALARM );
    printerNeopBeforeVote.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    Alarm alarmDisconnected;
    alarmDisconnected.setAlarmCommands ( alarmCommandSequence );
    alarmDisconnected.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_ALARMBOXDISCONNECTED );
    alarmDisconnected.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_CRITICALALARM );
    alarmDisconnected.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    Alarm alarmConnected;
    alarmConnected.setAlarmCommands ( alarmCommandSequence );
    alarmConnected.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_ALARMBOXCONNECTED );
    alarmConnected.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_CRITICALALARM );
    alarmConnected.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    Alarm cardReaderUnresponsive;
    cardReaderUnresponsive.setAlarmCommands ( alarmCommandSequence );
    cardReaderUnresponsive.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_CARDREADERUNRESPONSIVE );
    cardReaderUnresponsive.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_CRITICALALARM );
    cardReaderUnresponsive.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    Alarm audioCardDetectedButPollingPlaceNotConfiguredForAudio;
    audioCardDetectedButPollingPlaceNotConfiguredForAudio.setAlarmCommands ( alarmCommandSequence );
    audioCardDetectedButPollingPlaceNotConfiguredForAudio.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOTCONFIGUREDFORAUDIO );
    audioCardDetectedButPollingPlaceNotConfiguredForAudio.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_CRITICALALARM );
    audioCardDetectedButPollingPlaceNotConfiguredForAudio.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    Alarm audioCardDetectedButNoAble;
    audioCardDetectedButNoAble.setAlarmCommands ( alarmCommandSequence );
    audioCardDetectedButNoAble.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOABLEPRESENT );
    audioCardDetectedButNoAble.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_CRITICALALARM );
    audioCardDetectedButNoAble.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    Alarm audioCardDetectedButNoVoices;
    audioCardDetectedButNoVoices.setAlarmCommands ( alarmCommandSequence );
    audioCardDetectedButNoVoices.setTypeOfAlarm ( AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOVOICESPRESENT );
    audioCardDetectedButNoVoices.setAlarmCategory ( AlarmCategoryEnum::CONFIGURATION_CRITICALALARM );
    audioCardDetectedButNoVoices.setAlarmBehavior ( AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION );

    alarmsSeq.push_back ( alarmUnableToRead );
    alarmsSeq.push_back ( alarmUnableToWrite );
    alarmsSeq.push_back ( alarmCryptoError );
    alarmsSeq.push_back ( alarmInvalidContent );
    alarmsSeq.push_back ( alarmCardUsed );
    alarmsSeq.push_back ( alarmCardValidityPeriodExpired );
    alarmsSeq.push_back ( alarmInvalidVotingData );
    alarmsSeq.push_back ( alarmPaperJam );
    alarmsSeq.push_back ( alarmPaperOut );
    alarmsSeq.push_back ( alarmNeop );
    alarmsSeq.push_back ( alarmTimeout );
    alarmsSeq.push_back ( alarmUnexpectedError );
    alarmsSeq.push_back ( alarmCardNotRemovedAfterVoting );
    alarmsSeq.push_back ( cardExtractionCounter );
    alarmsSeq.push_back ( printerNeopBeforeVote );
    alarmsSeq.push_back ( alarmDisconnected );
    alarmsSeq.push_back ( alarmConnected );
    alarmsSeq.push_back ( cardReaderUnresponsive );
    alarmsSeq.push_back ( audioCardDetectedButPollingPlaceNotConfiguredForAudio );
    alarmsSeq.push_back ( audioCardDetectedButNoAble );
    alarmsSeq.push_back ( audioCardDetectedButNoVoices );

    alarmConf.setAlarm ( alarmsSeq );

    machineConf.setSuccessMessageTime ( 2500 );

    OperationConfiguration operationConfiguration;
    operationConfiguration.setStartOpenElection ( false );
    operationConfiguration.setConfigurationReset ( false );
    operationConfiguration.setDiagnosesConsideredApproved ( false );
    operationConfiguration.setSimulateVotes ( false );

    PVFormTransformationConfig transformationConfig;
    transformationConfig.setType ( PVFormTransformationType::CONFIGURATION_FOP );
    transformationConfig.setFopConfigFile ( "fop.xconf" );

    ScanVoteConfiguration scanVoteConfiguration;
    ScanVoteConfiguration::StateSequence statesSeq;

    statesSeq.push_back ( ProcessStatus::ELECTION_STATUS_INITIATED );
    statesSeq.push_back ( ProcessStatus::ELECTION_STATUS_CLOSED );

    scanVoteConfiguration.setState ( statesSeq );
    scanVoteConfiguration.setBarcodeInParams ( "LightsON" );
    scanVoteConfiguration.setBarcodeOutParams ( "LightsOFF" );
    scanVoteConfiguration.setBarcodeReconfigureParams ( "baseconfiguration" );
    scanVoteConfiguration.setSendBarcodeConfigurationOnReconnect ( true );

    configSAESFunctionality.setOperationConfiguration ( operationConfiguration );
    configSAESFunctionality.setMachineInitiator ( machineConf );
    configSAESFunctionality.setPVFormTransformation ( transformationConfig );
    configSAESFunctionality.setStructureInstallation ( StructureInstallationType::CONFIGURATION_GROUPED );
    configSAESFunctionality.setAlarmsConfiguration ( alarmConf );
    configSAESFunctionality.setSystemRole ( src );

    FlowStatesConfiguration flowStatesConfiguration;
    FlowStatesConfiguration::ProcessStatusSequence processStatusSequence;

    processStatusSequence.push_back ( ProcessStatus::ELECTION_STATUS_NOTINSTALLED );
    processStatusSequence.push_back ( ProcessStatus::ELECTION_STATUS_NOTDEMOSTRATIONVOTE );
    processStatusSequence.push_back ( ProcessStatus::ELECTION_STATUS_NOTINITIATED );
    processStatusSequence.push_back ( ProcessStatus::ELECTION_STATUS_INITIATED );
    processStatusSequence.push_back ( ProcessStatus::ELECTION_STATUS_CLOSED );
    processStatusSequence.push_back ( ProcessStatus::ELECTION_STATUS_RESULTSTRANSMITTED );

    flowStatesConfiguration.setProcessStatus ( processStatusSequence );

    PrinterControllerConfiguration printerControllerConfiguration;
    printerControllerConfiguration.setSkipPrinter ( false );
    printerControllerConfiguration.setTimeoutPrint ( 4 );
    printerControllerConfiguration.setResultPrintFile ( "ResultPrintFile" );
    printerControllerConfiguration.setStatusPrintFile ( "StatusPrintFile" );

    configSAESFunctionality.setPrinterControllerConfiguration ( printerControllerConfiguration );

    ResultsPackageTransmissionConfiguration resultsPackageTransmission;
    FileListToArchiveConfiguration fileListToArchive;
    FileListToArchiveConfiguration::FileNameSequence fileNameSequence;

    fileNameSequence.push_back ( "export" );
    fileNameSequence.push_back ( "SAESLocal/SAES/Docs" );

    fileListToArchive.setFileName ( fileNameSequence );

    resultsPackageTransmission.setNetworkManagerScript ( "smtt-network-manager" );
    resultsPackageTransmission.setResultsTransmissionScript ( "smtt-send-results" );
    resultsPackageTransmission.setForceUpDown ( false );
    resultsPackageTransmission.setSendResults ( false );
    resultsPackageTransmission.setConfigFileName ( "SAESLocal/send-results-cfg.sh" );
    resultsPackageTransmission.setDebugFileName ( "etc/send-results-cfg.sh" );
    resultsPackageTransmission.setFileListToArchive ( fileListToArchive );
    resultsPackageTransmission.setResultsFileName ( "/tmp/results.tgz" );
    resultsPackageTransmission.setResponseFileName ( "/tmp/response" );
    resultsPackageTransmission.setDHCP ( true );

    VoteComplianceVerificatorConfiguration voteComplianceVerificatorConfiguration;

    BeforePrintingVoteComplianceVerificationConfiguration beforePrintingVoteComplianceVerificationConfiguration;
    beforePrintingVoteComplianceVerificationConfiguration.setCheckMinimumNumberOfSelections(true);
    beforePrintingVoteComplianceVerificationConfiguration.setCheckSelectionsBelongToSameParty(true);
    beforePrintingVoteComplianceVerificationConfiguration.setCheckSelectionsApplicableToConstituency(true);
    beforePrintingVoteComplianceVerificationConfiguration.setCheckUniquenessInSelections(true);
    beforePrintingVoteComplianceVerificationConfiguration.setCheckListHeaderResourceCorrectness(true);
    beforePrintingVoteComplianceVerificationConfiguration.setCheckCumulativeNumberOfSelections(true);

    AfterScanningVoteComplianceVerificationConfiguration afterScanningVoteComplianceVerificationConfiguration;
    afterScanningVoteComplianceVerificationConfiguration.setCheckMinimumNumberOfScannings(true);
    afterScanningVoteComplianceVerificationConfiguration.setCheckScanningsBelongToSameParty(true);
    afterScanningVoteComplianceVerificationConfiguration.setCheckScanningsApplicableToConstituency(true);
    afterScanningVoteComplianceVerificationConfiguration.setCheckUniquenessInScannings(true);
    afterScanningVoteComplianceVerificationConfiguration.setCheckCumulativeNumberOfScannings(true);

    BeforeGeneratingResultsComplianceVerificationConfiguration beforeGeneratingResultsComplianceVerificationConfiguration;
    beforeGeneratingResultsComplianceVerificationConfiguration.setCheckTotalCumulativeSelections(true);
    beforeGeneratingResultsComplianceVerificationConfiguration.setCheckIndividualCumulativeSelections(true);
    beforeGeneratingResultsComplianceVerificationConfiguration.setCheckEmlVotesDeposited(true);
    beforeGeneratingResultsComplianceVerificationConfiguration.setCheckEmlVotesByVoterType(true);
    beforeGeneratingResultsComplianceVerificationConfiguration.setCheckEmlVotesCounter(true);

    voteComplianceVerificatorConfiguration.setBeforePrintingVoteComplianceVerificationConfiguration(beforePrintingVoteComplianceVerificationConfiguration);
    voteComplianceVerificatorConfiguration.setAfterScanningVoteComplianceVerificationConfiguration(afterScanningVoteComplianceVerificationConfiguration);
    voteComplianceVerificatorConfiguration.setBeforeGeneratingResultsComplianceVerificationConfiguration(beforeGeneratingResultsComplianceVerificationConfiguration);

    configSAESFunctionality.setResultsPackageTransmission ( resultsPackageTransmission );
    configSAESFunctionality.setVoteComplianceVerificator(voteComplianceVerificatorConfiguration);

    configSAESFunctionality.setFlowStatesConfiguration ( flowStatesConfiguration );
    configSAESFunctionality.setEnqueueVoteConfiguration ( EnqueueVoteConfiguration ( false ) );
    configSAESFunctionality.setScanVoteConfiguration ( scanVoteConfiguration );

    Smartmatic::ConfigurationTypes::NamespaceInfomap map;
#ifdef NAMESPACE_SAES_FUNCTIONALITY_CONFIGURATION
    map[""].name = NAMESPACE_SAES_FUNCTIONALITY_CONFIGURATION;
#endif
    map[""].schema = "";

    std::stringstream ss;
    serializeSaesFunctionalityConfiguration ( ss, configSAESFunctionality, map, "UTF-8" );
    ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "SAESFunctionality", ss.str() );
}

int SAESFunctionalityConfigurationManager::getSuccesMessageTime()
{
    return getConfiguration()->getMachineInitiator().getSuccessMessageTime();
}

PVFormTransformationConfig & SAESFunctionalityConfigurationManager::getPVFormTransformation()
{
    return getConfiguration()->getPVFormTransformation();
}

StructureInstallationType & SAESFunctionalityConfigurationManager::getStructureInstallationType()
{
    return getConfiguration()->getStructureInstallation();
}

EnqueueVoteConfiguration & SAESFunctionalityConfigurationManager::getEnqueueVoteConfiguration()
{
    return getConfiguration()->getEnqueueVoteConfiguration();
}

BeforePrintingVoteComplianceVerificationConfiguration & SAESFunctionalityConfigurationManager::getBeforePrintingVoteComplianceVerificationConfiguration()
{
    return getConfiguration()->getVoteComplianceVerificator().getBeforePrintingVoteComplianceVerificationConfiguration();
}

AfterScanningVoteComplianceVerificationConfiguration & SAESFunctionalityConfigurationManager::getAfterScanningVoteComplianceVerificationConfiguration()
{
    return getConfiguration()->getVoteComplianceVerificator().getAfterScanningVoteComplianceVerificationConfiguration();
}

BeforeGeneratingResultsComplianceVerificationConfiguration & SAESFunctionalityConfigurationManager::getBeforeGeneratingResultsComplianceVerificationConfiguration()
{
    return getConfiguration()->getVoteComplianceVerificator().getBeforeGeneratingResultsComplianceVerificationConfiguration();
}

SystemRoleConfiguration SAESFunctionalityConfigurationManager::getSystemRoleConfiguration()
{
    return getConfiguration()->getSystemRole();
}

VotingExperienceControllerConfiguration SAESFunctionalityConfigurationManager::getVotingExperienceControllerConfiguration()
{
    return getConfiguration()->getVotingExperienceController();
}

AudioVotingExperienceControllerConfiguration SAESFunctionalityConfigurationManager::getAudioVotingExperienceControllerConfiguration()
{
    return getConfiguration()->getAudioVotingExperienceController();
}

OperationConfiguration SAESFunctionalityConfigurationManager::getOperationConfiguration()
{
    return getConfiguration()->getOperationConfiguration();
}

ScanVoteConfiguration & SAESFunctionalityConfigurationManager::getScanVoteConfiguration()
{
    return getConfiguration()->getScanVoteConfiguration();
}

FlowStatesConfiguration & SAESFunctionalityConfigurationManager::getFlowStatesConfiguration()
{
    return getConfiguration()->getFlowStatesConfiguration();
}

ProcessStatus SAESFunctionalityConfigurationManager::getNextProcessStatus ( ProcessStatus status )
{
    ProcessStatus nextStatus = status;
    FlowStatesConfiguration::ProcessStatusSequence & seq ( getFlowStatesConfiguration().getProcessStatus() );
    FlowStatesConfiguration::ProcessStatusIterator it;

    bool found = false;

    for ( it = seq.begin();
            it != seq.end();
            ++it )
    {
        if ( found )
        {
            nextStatus = ( *it );
            break;
        }

        if ( ( *it ) == status )
        {
            found = true;
        }
    }

    return nextStatus;
}

std::list<Smartmatic::SAES::Functionality::Configuration::MenuOptionType> SAESFunctionalityConfigurationManager::getRunFunctionalityDescriptors()
{
    std::list<Smartmatic::SAES::Functionality::Configuration::MenuOptionType> descriptors;
    RunFunctionalityConfiguration::FunctionalityDescriptorSequence & runFunctionalities ( getConfiguration()->getRunFunctionalityConfiguration().getFunctionalityDescriptor() );

    for ( unsigned int i = 0; i < runFunctionalities.size(); ++i )
    {
        if ( !runFunctionalities[i].getMode().present() )
        {
            Smartmatic::SAES::Functionality::Configuration::MenuOptionType descriptor ( runFunctionalities[i].getName() );
            descriptors.push_back ( descriptor );
        }
    }

    return descriptors;
}

std::list<Smartmatic::SAES::Functionality::Configuration::MenuOptionType> SAESFunctionalityConfigurationManager::getRunFunctionalityDescriptorsByMode ( Smartmatic::SAES::Voting::VotingCenters::RunningMode mode )
{
    std::list<Smartmatic::SAES::Functionality::Configuration::MenuOptionType> descriptors;
    RunFunctionalityConfiguration::FunctionalityDescriptorSequence & runFunctionalities ( getConfiguration()->getRunFunctionalityConfiguration().getFunctionalityDescriptor() );

    for ( unsigned int i = 0; i < runFunctionalities.size(); ++i )
    {
        if ( runFunctionalities[i].getMode().present() && runFunctionalities[i].getMode().get() == mode )
        {
            Smartmatic::SAES::Functionality::Configuration::MenuOptionType descriptor ( runFunctionalities[i].getName() );
            descriptors.push_back ( descriptor );
        }
    }

    return descriptors;
}

SAESFunctionalityConfigurationManager * SAESFunctionalityConfigurationManager::getInstance ( SAESFunctionalityConfigurationManager * externManager )
{
    if ( ! instance )
    {
        instance = ( externManager == NULL ? new SAESFunctionalityConfigurationManager() : externManager );
    }

    return instance;
}
