/**
 * @file IAuthenticator.hxx
 * @brief Declares the IAuthenticator interface.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */

#ifndef _IAUTHENTICATOR_H_
#define _IAUTHENTICATOR_H_

#include <string>
#include <Voting/ElectoralConfiguration.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            namespace Authenticator
            {
                /**
                 * @class IAuthenticator
                 * @brief Abstract class for an authenticator.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 02-03-2011
                 */

                class IAuthenticator
                {
                    public:

                        /**
                         * @enum AuthenticationResultType
                         * @brief enum for authentication result.
                         */
                        enum AuthenticationResultType
                        {
                            VOTING_PROCESS /**<type voting process*/
                            , ELECTORAL_PROCESS /**<type electoral process*/
                            , AUDIO_VOTING_PROCESS /**<type audio voting process*/
                            , MENU /**<type menu*/
                            , WIZARD_MENU /**<type wizard menu*/
                            , CLEAR_ALARM /**<type clear alarm*/
                            , ERROR /**<type error*/
                            , NO_MATCH /**<type no match*/
                        };

                        /**
                         * @enum MachineType
                         * @brief enum for machine type.
                         */
                        enum MachineType
                        {
                            VOTING_MACHINE /**<type voting machine*/
                            , PRESIDENT_MACHINE /**<type president machine*/
                            , DIAGNOSTICS_ONLY /**<type diagnostics machine*/
                        };

                        /**
                         * @fn IAuthenticator::IAuthenticator ( MachineType machineType );
                         * @brief Default constructor for the IAuthenticator class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @param machineType machine type.
                         */

                        explicit IAuthenticator ( MachineType machineType );

                        /**
                         * @fn virtual IAuthenticator::~IAuthenticator();
                         * @brief Finalizes an instance of the IAuthenticator class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */
                        virtual ~IAuthenticator();

                        /**
                         * @fn MachineType IAuthenticator::getMachineType();
                         * @brief Gets the machine type.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return machine type.
                         */

                        MachineType getMachineType();

                        /**
                         * @fn void IAuthenticator::init ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                         * @brief Initializes this authenticator.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @param electoralConfiguration electoral configuration.
                         */

                        void init ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );

                        /**
                         * @fn virtual AuthenticationResultType IAuthenticator::validate ( std::string password ) = 0;
                         * @brief Validates a given password.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @param password The password to validate.
                         * @return Response for the validation of the given password.
                         */

                        virtual AuthenticationResultType validate ( std::string password ) = 0;

                        /**
                         * @fn virtual bool IAuthenticator::ValidateRole ( const std::string& password ) = 0;
                         * @brief Validates if the given password match the password for the menu.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @param password The password to validate.
                         * @return true if the given password match the password for the menu. Otherwise, false.
                         */
                        virtual bool ValidateRole ( const std::string& password ) = 0;

                        /**
                         * @fn virtual void IAuthenticator::dispose() = 0;
                         * @brief Disposes this instance of the IAuthenticator class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */

                        virtual void dispose() = 0;

                    private:
                        MachineType machineType; ///< Type of the machine

                    protected:
                        Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration; ///< The electoral configuration
                };
            }
        }
    }
}

#endif // _IAUTHENTICADOR_H_
