/**
 * @file   A4517AbledController.cxx
 * @brief  Implementation of A4517AbledController class.
 * @author eduardo.fernandez
 * @date   Jan 11, 2018
 */

#include "A4517AbledController.hxx"
#include <Devices/AbledDevice.hxx>
#include <Devices/FactoryDevice.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Connections;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;

A4517AbledController * A4517AbledController::m_instance = NULL;
ISMTTLog* A4517AbledController::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.A4517AbledController",
    GETTEXT_PACKAGE);

A4517AbledController::A4517AbledController()
{
    m_present = false;
}

A4517AbledController::~A4517AbledController()
{
}

A4517AbledController * A4517AbledController::getInstance(A4517AbledController * externInstance)
{
    if (externInstance != NULL)
    {
        m_instance = externInstance;
    }
    else if (m_instance == NULL)
    {
        m_instance = new A4517AbledController();
    }

    return m_instance;
}

bool A4517AbledController::open ( const std::string & device )
{
    Glib::Mutex::Lock lock ( m_mutex );
    bool connected;

    m_logger->Debug ( "Open abled device" );

    if (m_device.get() == NULL)
    {
        m_device.reset(FactoryDevice::getAbledDevice());
    }

    connected = m_device->open (device);

    if (connected)
        m_present = true;

    return connected;
}

bool A4517AbledController::close()
{
    Glib::Mutex::Lock lock ( m_mutex );

    m_logger->Debug ( "Close abled device" );
    m_present = false;

    return m_device.get() != NULL && m_device->close();
}

bool A4517AbledController::isPresent() const
{
    return m_present;
}

bool A4517AbledController::enableNotifications(const bool enable)
{
    Glib::Mutex::Lock lock(m_mutex);

    bool success = false;

    if (m_device.get() != NULL)
    {
        success = m_device->enableNotifications(enable);
    }

    return success;
}

std::string A4517AbledController::getFWVersion()
{
    Glib::Mutex::Lock lock(m_mutex);

    std::string firmwareVersion = "";

    if (m_device.get() != NULL)
    {
        firmwareVersion = m_device->getFWVersion();
    }

    return firmwareVersion;
}

std::string A4517AbledController::getSerialNumber()
{
    Glib::Mutex::Lock lock(m_mutex);

    std::string serialNumber = "";

    if (m_device.get() != NULL)
    {
        m_logger->Debug ( "Obtaining Serial number from abled" );

        serialNumber = StringUtils::toUpper ( m_device->getSerialNumber() );

        MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setAbled20SerialDetected ( serialNumber );
        m_logger->Debug ( "Get serial number : " + serialNumber );
    }

    return serialNumber;
}

void A4517AbledController::setSerialNumber ( const std::string & serial, bool save )
{
    Glib::Mutex::Lock lock(m_mutex);

    if (m_device.get() == NULL)
    {
        return;
    }

    std::string text = StringUtils::toUpper ( serial );

    if ( save )
    {
        m_logger->Debug ( "Save serial number to abled : " + text );
        m_device->setSerialNumber ( text );
    }
    else
    {
        m_logger->Debug ( "Skip set serial number : " + text );
    }

    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setAbled20SerialDetected ( text );
}

AbleDButtonState* A4517AbledController::getA4517AbleDButtonState ()
{
    Glib::Mutex::Lock lock(m_mutex);

    return m_device.get() != NULL ? m_device->getA4517AbleDButtonState() : NULL;
}
