/**
 * @file AlarmController.hxx
 * @brief Declaration of the AlarmController class.
 * @date 07/06/2015
 * @author Yadickson Soto
 */

#ifndef ALARMBOXCONTROLLER_HXX_
#define ALARMBOXCONTROLLER_HXX_

#include <Configuration/alarms-configuration-schema.hxx>
#include <Devices/AlarmBoxInterface.hxx>
#include <glibmm.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Devices
            {
                /**
                 * @class AlarmController
                 * @brief Class alarm controller.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 07/06/2015
                 */
                class AlarmController
                {
                    private:

                        std::vector<Smartmatic::SAES::Functionality::Configuration::AlarmEnum> alarmList;  ///< The alarm list
                        std::string message; ///< message alarm

                        Glib::Mutex mutex;	///< The mutex
                        Smartmatic::SAES::HW::Devices::AlarmBoxInterface * alarmBoxDevice; ///< Alarm box device

                        static AlarmController * instance; ///< instance
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                    protected:

                        /**
                         * @fn AlarmController::AlarmController();
                         * @brief Default constructor for the AlarmController class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 07/06/2015
                         */
                        AlarmController();

                    public:

                        /**
                         * @fn virtual AlarmController::~AlarmController();
                         * @brief Finalizes an instance of the AlarmController class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 07/06/2015
                         */
                        virtual ~AlarmController();

                        /**
                         * @fn	virtual bool open( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn	virtual bool close();
                         * @brief	Closes the connections.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @return	true if successful.
                         */
                        virtual bool close();

                        /**
                         * @fn	virtual bool sendPing();
                         * @brief	Sends a ping.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @return	true if it succeeds, false if it fails.
                         */
                        virtual bool sendPing();

                        /**
                         * @fn	virtual bool turnOnRedLed(unsigned char period);
                         * @brief	Turn on the red led.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @param	period	The period.
                         */
                        virtual bool turnOnRedLed ( unsigned char period );

                        /**
                         * @fn	virtual bool turnOffRedLed();
                         * @brief	Turn off the red led.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         */
                        virtual bool turnOffRedLed();

                        /**
                         * @fn	virtual bool turnOnGreenLed(unsigned char period);
                         * @brief	Turn on the green led.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @param	period	The period.
                         */
                        virtual bool turnOnGreenLed ( unsigned char period );

                        /**
                         * @fn	virtual bool turnOffGreenLed();
                         * @brief	Turn off the green led.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         */
                        virtual bool turnOffGreenLed();

                        /**
                         * @fn	virtual bool turnOnBuzzer(unsigned char period);
                         * @brief	Turn on the buzzer.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @param	period	The period.
                         */
                        virtual bool turnOnBuzzer ( unsigned char period );

                        /**
                         * @fn	virtual bool turnOffBuzzer();
                         * @brief	Turn off the buzzer.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         */
                        virtual bool turnOffBuzzer();

                        /**
                         * @fn	virtual bool isButtonPressed ();
                         * @brief	Check if button is pressed.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @return	true if button is pressed.
                         */
                        virtual bool isButtonPressed ();

                        /**
                         * @fn void AlarmController::addAlarm ( const Smartmatic::SAES::Functionality::Configuration::AlarmEnum & alarm );
                         * @brief Adds the given alarm type to the pending alarms list.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 07/06/2015
                         * @param alarm Type of the alarm to add.
                         */
                        void addAlarm ( const Smartmatic::SAES::Functionality::Configuration::AlarmEnum & alarm );

                        /**
                         * @fn void AlarmController::soundAlarm ( const Smartmatic::SAES::Functionality::Configuration::AlarmEnum & alarm );
                         * @brief Sounds the alarm of the given type.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 07/06/2015
                         * @param alarm Type of the alarm to sound.
                         */
                        void soundAlarm ( const Smartmatic::SAES::Functionality::Configuration::AlarmEnum & alarm );

                        /**
                         * @fn void AlarmController::clearAlarms();
                         * @brief Clears the list of pending alarms.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 07/06/2015
                         */
                        void clearAlarms();

                        /**
                         * @fn bool AlarmController::isAlarmPending();
                         * @brief Verifies if there's at least one alarm in the pending alarm list.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 07/06/2015
                         * @return true if there's at least one pending alarm. Otherwise, false.
                         */
                        bool isAlarmPending();

                        /**
                         * @fn bool AlarmController::isCriticalAlarmPending();
                         * @brief Verifies if there's at least one critical alarm in the pending alarm list.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 07/06/2015
                         * @return true if there's at least one critical alarm. Otherwise, false.
                         */
                        bool isCriticalAlarmPending();

                        /**
                         * @fn bool AlarmController::isGoToAuthentication();
                         * @brief Verifies if at least one alarm in the pending alarm list requires to go to the
                         * authentication.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 07/06/2015
                         * @return true if at least one alarm requires to go the authentication. Otherwise, false.
                         */
                        bool isGoToAuthentication();

                        /**
                         * @fn void AlarmController::setMessage ( const std::string &  message );
                         * @brief Sets the message for the alarm.
                         * @author Yadickson Soto
                         * @date 07/06/2015
                         * @param message Message to set.
                         */
                        void setMessage ( const std::string  & message );

                        /**
                         * @fn std::string AlarmController::getMessage();
                         * @brief Gets the message for the alarm.
                         * @author Yadickson Soto
                         * @date 07/06/2015
                         * @return Message set for the alarm.
                         */

                        std::string getMessage();

                        /**
                         * @fn	void shutDownAlarm();
                         * @brief	Shut down the alarm.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         */
                        void shutDownAlarm();

                        /**
                         * @fn	bool test( unsigned int wait );
                         * @brief	Method test.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @param wait the wait time in milliseconds
                         * @return	true is test success.
                         */
                        bool test ( unsigned int wait = 1000 );

                        /**
                         * @fn static AlarmController * AlarmController::getInstance();
                         * @brief Gets the current instance of the AlarmController.
                         * @author Yadickson Soto
                         * @date 07/06/2015
                         * @return pointer to the current instance of the AlarmController class.
                         */

                        static AlarmController * getInstance();
                };
            }
        }
    }
}

#endif /* ALARMBOXCONTROLLER_HXX_ */
