/**
 * @file BCRController.cxx
 * @brief Body class BCRController
 * @date 09/02/2012
 * @author Yadickson Soto
 */

#include "BCRController.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Devices/FactoryDevice.hxx>
#include <Configuration/BarcodeDeviceConfigurationManager.hxx>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Configuration;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::System;

BCRController * BCRController::instance = NULL;
ISMTTLog * BCRController::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.BCR.BCRController", GETTEXT_PACKAGE );

BCRController::BCRController()
{
    this->barcodeDevice = NULL;
    allowReconfigureCommand();
}

BCRController::~BCRController()
{

}

BCRController * BCRController::getInstance(BCRController * externInstance)
{
    if ( !instance || externInstance)
    {
        instance = ( externInstance == NULL ? new BCRController() : externInstance );
    }

    return instance;
}

bool BCRController::open ( const std::string & device )
{
    Glib::Mutex::Lock lock ( mutex );

    logger->Debug ( "Open barcode device" );

    barcodeDevice = barcodeDevice == NULL ? FactoryDevice::getBarcodeReaderDevice() : barcodeDevice;

    bool connected = barcodeDevice->open ( device );
    bool factorySerial = false;

    deviceModel = barcodeDevice->getDeviceModel();

    if (deviceModel == "hr200v3")
    {
        factorySerial = true;
    }

    deviceSerial = barcodeDevice->getDeviceSerial(factorySerial);

    logger->Debug ( std::string ( "Open barcode device : " ) + ( connected ? "Success" : "Fail" ) );
    logger->Debug ( "Barcode model (" + deviceModel + ")" );
    logger->Debug ( "Barcode serial (" + deviceSerial + ")" );

    return connected;
}

bool BCRController::close()
{
    Glib::Mutex::Lock lock ( mutex );

    logger->Debug ( "Close barcode device" );

    return barcodeDevice != NULL && barcodeDevice->close();
}

std::string BCRController::getDeviceModel()
{
    return deviceModel;
}

std::string BCRController::getDeviceSerial()
{
    return deviceSerial;
}

bool BCRController::setDeviceSerial ( const std::string & serial )
{
    Glib::Mutex::Lock lock ( mutex );

    if (deviceModel == "hr200v3") return true;

    return barcodeDevice != NULL && barcodeDevice->setDeviceSerial ( serial );
}

bool BCRController::sendReconfigureCommand ( const std::string & command )
{
    Glib::Mutex::Lock lock ( mutex );

    bool status = true;

    logger->Debug ( "Send command (" + command  + ") to barcode device" );

    if ( barcodeDevice )
    {
        BarcodeCommands::CommandSequence seq = BarcodeDeviceConfigurationManager::getInstance()->getBarcodeConfigurationCommands ( command, deviceModel );
        BarcodeCommands::CommandIterator it;

        for ( it = seq.begin(); it != seq.end() && status && !this->stopReconfiguration; ++it )
        {
            status = barcodeDevice->sendReconfigureCommand ( ( *it ).getCommand(), ( *it ).getDelayResponse(), ( *it ).getRetries(), ( *it ).getMandatorySent() );
        }
    }
    else
    {
        status = false;
    }

    if ( this->stopReconfiguration )
        this->stopReconfiguration = false;

    logger->Debug ( std::string ( "Send command to barcode device : " ) + ( status ? "Success" : "Fail" ) );

    return status;
}

void BCRController::allowReconfigureCommand()
{
    this->stopReconfiguration = false;
}

void BCRController::stopReconfigureCommand()
{
    this->stopReconfiguration = true;
}

SafeBuffer BCRController::readBuffer ()
{
    Glib::Mutex::Lock lock ( mutex );

    SafeBuffer buffer;

    if ( barcodeDevice )
    {
        buffer = barcodeDevice->readBuffer();
    }

    return buffer;
}
