/**
 * @file BCRController.hxx
 * @brief Declaration of the BCRController.
 * @date 09/02/2012
 * @author Yadickson Soto
 */

#ifndef BCRCONTROLLER_HXX_
#define BCRCONTROLLER_HXX_

#include <System/SafeBuffer.h>
#include <glibmm.h>
#include <Devices/BarcodeInterface.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Diagnosis
            {
                class BarCodeDiagnosis;
            }

            namespace Devices
            {
                /**
                 * @class BCRController
                 * @brief Barcode reader Controller is the singleton instance which provide the support to the device.
                 * @author Yadickson Soto
                 * @date 09/02/2012
                 */
                class BCRController
                {
                    public:

                        /**
                         * @fn virtual BCRController::~BCRController();
                         * @brief Finalizes an instance of the BCRController class.
                         * @author Yadickson Soto
                         * @date 09/02/2012
                         */
                        virtual ~BCRController();

                        /**
                         * @fn static BCRController * BCRController::getInstance();
                         * @brief Gets the current instance of the BCRController class.
                         * @author Juan Valera
                         * @date 01/02/2017
                         * @return instance class.
                         */
                        static BCRController * getInstance(BCRController * externInstance = NULL);

                        /**
                         * @fn	virtual bool open( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	09/07/2015
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn	virtual bool close();
                         * @brief	Closes the connections.
                         * @author	Yadickson Soto
                         * @date	09/07/2015
                         * @return	true if successful.
                         */
                        virtual bool close();

                        /**
                         * @fn	virtual std::string getDeviceModel();
                         * @brief	Getter device model.
                         * @author	Yadickson Soto
                         * @date	31/07/2015
                         * @return	device model
                         */
                        virtual std::string getDeviceModel();

                        /**
                         * @fn	virtual std::string getDeviceSerial();
                         * @brief	Getter device serial ESN.
                         * @author	Yadickson Soto
                         * @date	31/07/2015
                         * @return	device serial ESN
                         */
                        virtual std::string getDeviceSerial();

                        /**
                         * @fn	virtual bool setDeviceSerial ( const std::string & serial );
                         * @brief	Setter new serial to device.
                         * @author	Yadickson Soto
                         * @date	31/07/2015
                         * @param	serial new serial
                         * @return true if successfully
                         */
                        virtual bool setDeviceSerial ( const std::string & serial );

                        /**
                         * @fn	virtual bool sendReconfigureCommand ( const std::string & command );
                         * @brief	Send command to device.
                         * @author	Yadickson Soto
                         * @date	10/07/2015
                         * @param	command command to send
                         * @return true if successfully
                         */
                        virtual bool sendReconfigureCommand ( const std::string & command );

                        /**
                         * @fn  virtual void allowReconfigureCommand ();
                         * @brief Allow device reconfigure command.
                         * @author  Eduardo Fernández
                         * @date  07/08/2017
                         */
                        virtual void allowReconfigureCommand ();

                        /**
                         * @fn  virtual void stopReconfigureCommand ();
                         * @brief Stop device reconfigure command.
                         * @author  Eduardo Fernández
                         * @date  04/08/2017
                         */
                        virtual void stopReconfigureCommand ();

                        /**
                         * @fn	virtual Smartmatic::System::SafeBuffer readBuffer ();
                         * @brief	Read device buffer.
                         * @author	Yadickson Soto
                         * @date	13/07/2015
                         * @return  data buffer from device
                         */
                        virtual Smartmatic::System::SafeBuffer readBuffer ();

                    protected:

                        /**
                         * @fn BCRController::BCRController();
                         * @brief Default constructor for the BCRController class.
                         * @author Yadickson Soto
                         * @date 09/02/2012
                         */
                        BCRController();

                    private:

                        Glib::Mutex mutex;  ///< The mutex
                        Smartmatic::SAES::HW::Devices::BarcodeInterface * barcodeDevice; ///< barcode device interface

                        bool stopReconfiguration;

                        std::string deviceModel; ///< The device model
                        std::string deviceSerial; ///< The device serial

                        static BCRController * instance; ///< Instance
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}
#endif /* BCRCONTROLLER_HXX_ */
