/**
 * @file CardController.hxx
 * @brief  Declaration of smartcard reader controller which implements operations of activation, reading, writing and restarting of the reader.
 * @date 15/08/2011
 * @author Yadickson Soto
 */

#ifndef CARDCONTROLLER_HXX_
#define CARDCONTROLLER_HXX_

#include <Card/SmartcardInterface.hxx>
#include <Voting/CardData/card-data-schema.hxx>
#include <Operation/Security/CryptoSmartcard.hxx>
#include <glibmm.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Devices
            {
                /**
                 * @class CardController
                 * @brief  Represent the controller of a smartcard reader device which implements operations
                 * of activation, reading, writing, and restarting of the reader.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 15/08/2011
                 * @see Smartcard
                 */
                class CardController
                {
                    public:

                        /**
                         * @brief  Definition of power cycled signal.
                         * @author manuel.arguelles
                         * @date   Nov 11, 2017
                         */
                        typedef sigc::signal<void> PowerCycledSignal;

                    protected:

                        /**
                         * @fn CardController( Smartmatic::SAES::Card::SmartcardInterface * smartcard );
                         * @brief Constructor
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 15/08/2011
                         * @param smartcard The external smartcard, only for test
                         */
                        explicit CardController ( Smartmatic::SAES::Card::SmartcardInterface * smartcard );

                        Smartmatic::SAES::Card::SmartcardInterface * smartcard; ///< The smartcard interface
                        Smartmatic::SAES::Operation::Security::CryptoSmartcard * cryptoCardData; ///< crypto card data

                    public:

                        /**
                         * @fn virtual CardController::~CardController();
                         * @brief Destructor
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 15/08/2011
                         */
                        virtual ~CardController();

                        /**
                         * @fn static CardController * CardController::getInstance( CardController * externManager = NULL );
                         * @brief Gets the current instance of the CardController class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 15/08/2011
                         * @param 	externManager external manager (for unit tests purposes only)
                         * @return current instance of the CardController class.
                         */
                        static CardController * getInstance ( CardController * externManager = NULL );

                        /**
                         * @fn virtual bool open ( const std::string & device );
                         * @brief Initializes this card controller.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 15/08/2011
                         * @param 	device device path
                         * @return true if the initialization was completed. Otherwise, false.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn virtual bool CardController::isCardPresent();
                         *
                         * @brief Verifies if a smartcard is present.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 15/08/2011
                         * @return true if the smartcard is present. Otherwise, false.
                         */
                        virtual bool isCardPresent();

                        /**
                         * @fn virtual bool CardController::test();
                         * @brief Determines whether the smartcard is OK (tests select, read, write).
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 15/08/2011
                         * @return <tt>true</tt> if the test was successful; otherwise <tt>false</tt>.
                         */
                        virtual bool test();

                        /**
                         * @fn virtual bool close();
                         * @brief Disposes of an instance of the CardController class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 07/06/2015
                         * @return true if success
                         */
                        virtual bool close();

                        /**
                         * @fn virtual bool CardController::readCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData );
                         * @brief Reads data from the smartcard.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 15/08/2011
                         * @param cardData result card data.
                         * @return true if the read card data is valid. Otherwise, false.
                         */
                        virtual bool readCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData );

                        /**
                         * @fn virtual bool CardController::writeCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData );
                         * @brief Writes data to the smartcard.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 15/08/2011
                         * @param cardData data to write to the smartcard.
                         * @return true if the data was written correctly. Otherwise, false.
                         */
                        virtual bool writeCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData );

                        /**
                         * @brief  Emitted on power restart of the smartcard reader.
                         * @author manuel.arguelles
                         * @date   Nov 11, 2017
                         */
                        PowerCycledSignal signalPowerCycled();

                    protected:

                        /**
                         * @fn virtual bool testSelect();
                         * @brief Tests whether the ID from the smartcard can be read.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return true if the smartcard device test was successful. Otherwise, false.
                         */
                        virtual bool testSelect();

                        /**
                         * @fn virtual bool testWriteRead();
                         * @brief Test write and read from a smartcard device.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return true if the smartcard device test was successful. Otherwise, false.
                         */
                        virtual bool testWriteRead();

                        /**
                         * @fn virtual bool CardController::read ( Smartmatic::System::SafeBuffer & buffer,
                         *    Smartmatic::SAES::Security::CryptoData & cryptoData );
                         * @brief Reads and decrypts data from the smartcard.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 15/08/2011
                         * @param buffer data read from the smartcard.
                         * @param cryptoData cryptographic data used to decrypt the data from the smartcard.
                         * @return true if the read data is valid. Otherwise, false.
                         */
                        virtual bool read ( Smartmatic::System::SafeBuffer & buffer, Smartmatic::SAES::Security::CryptoData & cryptoData );

                        /**
                         * @fn virtual bool CardController::write ( Smartmatic::System::SafeBuffer & buffer,
                         *    Smartmatic::SAES::Security::CryptoData & cryptoData );
                         * @brief Writes encrypted data to the smartcard.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 15/08/2011
                         * @param buffer data to write to the smartcard.
                         * @param cryptoData cryptographic data used to encrypt the data from the smartcard.
                         * @return true if the data was written correctly. Otherwise, false.
                         */
                        virtual bool write ( Smartmatic::System::SafeBuffer & buffer, Smartmatic::SAES::Security::CryptoData & cryptoData );

                    private:

                        /**
                         * @brief  Called on smartcard power cycled.
                         * @author manuel.arguelles
                         * @date   Nov 11, 2017
                         */
                        void onPowerCycled();

                    private:

                        Glib::Mutex mutex; ///< mutex

                        static CardController * instance; ///< instance
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                        sigc::connection powerCycleConnection;
                        PowerCycledSignal m_powerCycledSignal;
                };
            }
        }
    }
}

#endif /* CARDCONTROLLER_HXX_ */
