/**
 * @file   IOBController.cxx
 * @brief  Implementation of IOBController class.
 * @author eduardo.fernandez
 * @date   Jan 2, 2018
 */

#include "IOBController.hxx"
#include <Devices/IOBDevice.hxx>
#include <Devices/FactoryDevice.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;

IOBController* IOBController::m_instance = NULL;

ISMTTLog* IOBController::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.IOBController",
    GETTEXT_PACKAGE);

IOBController::IOBController()
    : m_device(FactoryDevice::getIOBDevice())
{
    m_device->open();
}

IOBController::~IOBController()
{
    m_instance = NULL;
}

void IOBController::disableSmartCardReader()
{
    Glib::Mutex::Lock lock(m_mutex);

    if (m_device->disableSmartcardReader())
    {
        m_logger->Debug("Smartcard reader disabled.");
    }
    else
    {
        m_logger->Debug("Error disabling smartcard reader.");
        reconnect();
    }
}

void IOBController::enableSmartCardReader()
{
    Glib::Mutex::Lock lock(m_mutex);

    if (m_device->enableSmartcardReader())
    {
        m_logger->Debug("Smartcard reader enabled.");
    }
    else
    {
        m_logger->Debug("Error enabling smartcard reader.");
        reconnect();
    }
}

std::string IOBController::getFWVersion()
{
    Glib::Mutex::Lock lock(m_mutex);
    m_logger->Debug ( "Obtaining FW version from IOB" );
    std::string version = m_device->getFWVersion();
    m_logger->Debug ( "IOB FW VERSION: " + version );
    return version;
}

std::string IOBController::getSerialNumber()
{
    Glib::Mutex::Lock lock(m_mutex);
    m_logger->Debug ( "Obtaining Serial number from IOB" );
    std::string serial = MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->getSerialDetected();
    if ( serial.empty())
    {
        serial = m_device->getSerialNumber();
    }
    serial = StringUtils::toUpper ( serial );
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setSerialDetected ( serial );
    m_logger->Debug ( "Get serial number : " + serial );
    return serial;
}

void IOBController::setSerialNumber ( const std::string & serial, bool save )
{
    Glib::Mutex::Lock lock(m_mutex);
    std::string text = StringUtils::toUpper ( serial );
    if ( save )
    {
        m_logger->Debug ( "Save serial number to IOB : " + text );
        m_device->setSerialNumber ( text );
    }
    else
    {
        m_logger->Debug ( "Skip set serial number : " + text );
    }
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setSerial ( text );
}

void IOBController::reconnect()
{
    m_logger->Debug("Reconnecting to the device.");
    m_device->close();

    if (!m_device->open())
    {
        m_logger->Debug("Failed to reconnect to device.");
    }
}

IOBController* IOBController::getInstance()
{
    if (m_instance == NULL)
    {
        m_instance = new IOBController();
    }

    return m_instance;
}
