/**
 * @file   IOBController.hxx
 * @brief  Declaration of IOBController class.
 * @author eduardo.fernandez
 * @date   Jan 2, 2018
 */

#ifndef IOB_CONTROLLER_HXX
#define IOB_CONTROLLER_HXX

#include <glibmm.h>
#include <memory>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Devices
            {
                class IOBDevice;
            }
        }

        namespace Operation
        {
            namespace Devices
            {
                /**
                 * @class  IOBController
                 * @brief  Controls the IOB device.
                 * @author eduardo.fernandez
                 * @date   Jan 2, 2018
                 */
                class IOBController
                {
                    public:

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        ~IOBController();

                    public:

                        /**
                         * @brief  Returns the instance of controller.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         * @return The instance of the controller.
                         */
                        static IOBController* getInstance();

                    public:

                        /**
                         * @brief  Disable smartcard reader.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        void disableSmartCardReader();

                        /**
                         * @brief  Enable smartcard reader.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        void enableSmartCardReader();

                        /**
                         * @brief  Gets the FW version from the IOB's.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                        */
                        std::string getFWVersion();

                        /**
                         * @brief  Gets the serial number from the IOB's EEPROM.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                        */
                        std::string getSerialNumber();

                        /**
                         * @brief  Sets the serial number to the IOB's EEPROM.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        void setSerialNumber ( const std::string & serial, bool save );

                    private:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        IOBController();

                        /**
                         * @brief  Reconnects to the PDB device.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        void reconnect();

                    private:

                        Glib::Mutex                                 m_mutex;
                        static IOBController*                       m_instance;
                        static Smartmatic::Log::ISMTTLog*           m_logger;
                        std::auto_ptr<SAES::HW::Devices::IOBDevice> m_device;
                };

            } /* namespace Devices */
        } /* namespace Operation */
    } /*  namespace SAES */
} /* namespace Smartmatic */

#endif /* IOB_CONTROLLER_HXX */
