/**
 * @file P168AController.cxx
 * @brief Body class controller for serial machine and able.
 * @date 06/07/2015
 * @author Yadickson Soto
 */

#include "P168AController.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Devices/FactoryDevice.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * P168AController::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.P168AController", GETTEXT_PACKAGE );

P168AController * P168AController::interface = NULL;

P168AController::P168AController()
{
    p168aDevice = NULL;
}

P168AController::~P168AController()
{
    if ( p168aDevice )
    {
        delete p168aDevice;
        p168aDevice = NULL;
    }

    interface = NULL;
}

bool P168AController::open ( const std::string & device )
{
    Glib::Mutex::Lock lock ( mutex );
    p168aDevice = ( p168aDevice == NULL ? FactoryDevice::getP168ADevice() : p168aDevice );
    return p168aDevice->open ( device );
}

bool P168AController::close()
{
    Glib::Mutex::Lock lock ( mutex );
    return p168aDevice != NULL && p168aDevice->close();
}

std::string P168AController::getSerialNumber()
{
    Glib::Mutex::Lock lock ( mutex );
    std::string serial = MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->getSerialDetected();

    if ( serial.empty() && p168aDevice )
    {
        serial = p168aDevice->getSerialMachine();
    }

    if (serial.empty() == false)
    {
        serial = StringUtils::toUpper ( serial );
        MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setSerialDetected ( serial );
        logger->Debug ( "Get P168 serial number : " + serial );
    }
    else
    {
        logger->Debug ( "Get P168 serial number : empty" );
    }

    return serial;
}

std::string P168AController::getSerialMachine()
{
    Glib::Mutex::Lock lock ( mutex );

    std::string machineSerialNumber = "";

    if ( p168aDevice )
    {
        machineSerialNumber = p168aDevice->getSerialMachine();
    }

    return machineSerialNumber;
}

void P168AController::setSerialNumber ( const std::string & serial, bool save )
{
    Glib::Mutex::Lock lock ( mutex );
    std::string text = StringUtils::toUpper ( serial );

    if ( save )
    {
        logger->Debug ( "Save serial machine : " + text );

        if ( p168aDevice )
        {
            p168aDevice->setSerialMachine ( text );
        }
    }
    else
    {
        logger->Debug ( "Skip set serial machine : " + text );
    }

    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setSerial ( text );
}

bool P168AController::enableVolumeControl()
{
    Glib::Mutex::Lock lock ( mutex );

    return p168aDevice != NULL ? p168aDevice->enableVolumeControl() : false;
}

AbleDButtonState * P168AController::getAbleDButtonState ()
{
    Glib::Mutex::Lock lock ( mutex );
    return p168aDevice != NULL ? p168aDevice->getAbleDButtonState() : NULL;
}

P168AController * P168AController::getInstance()
{
    if ( !interface )
    {
        interface = new P168AController ();
    }

    return interface;
}
