/**
 * @file P168AController.hxx
 * @brief Header class controller for serial machine and able.
 * @date 06/07/2015
 * @author Yadickson Soto
 */

#ifndef P168A_CONTROLLER_HXX_
#define P168A_CONTROLLER_HXX_

#include <string>
#include <Devices/P168AInterface.hxx>
#include <glibmm.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Devices
            {
                /**
                 * @class P168AController
                 * @brief Class controller for serial machine and able.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 06/07/2015
                 */
                class P168AController
                {
                    private:
                        /**
                         * @fn P168AController::P168AController();
                         * @brief Default constructor for the P168AController class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 06/07/2015
                         */
                        P168AController();

                        Glib::Mutex mutex;	///< The mutex
                        static P168AController * interface; ///< interface card controller
                        Smartmatic::SAES::HW::Devices::P168AInterface * p168aDevice; ///< p168a device

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                    public:
                        /**
                         * @fn virtual P168AController::~P168AController();
                         * @brief Finalizes an instance of the P168AController class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 06/07/2015
                         */
                        virtual ~P168AController();

                        /**
                         * @fn	virtual bool open( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn	virtual bool close();
                         * @brief	Closes the connections.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return	true if successful.
                         */
                        virtual bool close();

                        /**
                         * @fn	std::string SerialNumberManager::getSerialNumber();
                         * @brief	Method getter serial machine.
                         * @author	Yadickson Soto
                         * @date	06/07/2015
                         * @return	serial machine.
                         */
                        std::string getSerialNumber();

                        /**
                         * @fn  std::string SerialNumberManager::getSerialMachine();
                         * @brief   Method getter serial machine.
                         * @author  eduardo.fernandez
                         * @date    25/04/2018
                         * @return  serial machine.
                         */
                        std::string getSerialMachine();

                        /**
                         * @fn	void SerialNumberManager::setSerialNumber ( const std::string & serial,
                         * 			bool save = true );
                         * @brief	Method setter serial machine.
                         * @author	Yadickson Soto
                         * @date	06/07/2015
                         * @param	serial	serial machine.
                         * @param	save		force save.
                         */
                        void setSerialNumber ( const std::string & serial, bool save = true );

                        /**
                         * @brief  Enables able-d volume control.
                         * @author eduardo.fernandez
                         * @date   24/01/2018
                         * @return Whether the volume control was correctly enabled.
                         */
                        bool enableVolumeControl();

                        /**
                         * @fn	Smartmatic::SAES::HW::Operation::AbleButtonState * getAbleDButtonState();
                         * @brief	Method getter able-d button status.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return AbleD button status
                         */
                        Smartmatic::SAES::HW::Operation::AbleDButtonState * getAbleDButtonState ();

                        /**
                         * @fn static P168AController * getInstance();
                         * @brief Gets the current instance
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 06/07/2015
                         * @return current instance of P168AController.
                         */
                        static P168AController * getInstance();
                };
            }
        }
    }
}
#endif /* P168A_CONTROLLER_HXX_ */
