/**
 * @file   PDBController.cxx
 * @brief  Implementation of PDBController class.
 * @author manuel.arguelles
 * @date   Nov  2, 2017
 */

#include "PDBController.hxx"
#include <Devices/PDBDevice.hxx>
#include <Devices/FactoryDevice.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;

PDBController* PDBController::m_instance = NULL;

ISMTTLog* PDBController::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.PDBController",
    GETTEXT_PACKAGE);

PDBController::PDBController()
    : m_device(FactoryDevice::getPDBDevice())
{
    m_device->open();
}

PDBController::~PDBController()
{
    m_instance = NULL;
}

void PDBController::disableSmartCardReader()
{
    Glib::Mutex::Lock lock(m_mutex);

    if (m_device->disableSmartcardReader())
    {
        m_logger->Debug("Smartcard reader disabled.");
    }
    else
    {
        m_logger->Debug("Error disabling smartcard reader.");
        reconnect();
    }
}

void PDBController::enableSmartCardReader()
{
    Glib::Mutex::Lock lock(m_mutex);

    if (m_device->enableSmartcardReader())
    {
        m_logger->Debug("Smartcard reader enabled.");
    }
    else
    {
        m_logger->Debug("Error enabling smartcard reader.");
        reconnect();
    }
}

std::string PDBController::getFWVersion()
{
    Glib::Mutex::Lock lock(m_mutex);

    m_logger->Debug ( "Obtaining FW version from PDB" );

    std::string version = m_device->getFWVersion();

    m_logger->Debug ( "PDB FW VERSION: " + version );

    return version;
}

std::string PDBController::getSerialNumber()
{
    Glib::Mutex::Lock lock(m_mutex);
    m_logger->Debug ( "Obtaining Serial number from PDB" );
    std::string serial = MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->getSerialDetected();
    if ( serial.empty())
    {
        serial = m_device->getSerialNumber();
    }
    serial = StringUtils::toUpper ( serial );
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setSerialDetected ( serial );
    m_logger->Debug ( "Get serial number : " + serial );
    return serial;
}

void PDBController::setSerialNumber ( const std::string & serial, bool save )
{
    Glib::Mutex::Lock lock(m_mutex);
    std::string text = StringUtils::toUpper ( serial );
    if ( save )
    {
        m_logger->Debug ( "Save serial number to PDB : " + text );
        m_device->setSerialNumber ( text );
    }
    else
    {
        m_logger->Debug ( "Skip set serial number : " + text );
    }
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setSerial ( text );
}

bool PDBController::turnAllLedsOn()
{
    Glib::Mutex::Lock lock(m_mutex);
    bool success = m_device->turnAllLedsOn();
    if (success)
    {
        m_logger->Debug("PDB LEDs turned ON");
    }
    else
    {
        m_logger->Debug("Error turning PDB LEDs ON.");
        reconnect();
    }
    return success;
}

bool PDBController::turnAllLedsOff()
{
    Glib::Mutex::Lock lock(m_mutex);
    bool success = m_device->turnAllLedsOff();
    if (success)
    {
        m_logger->Debug("PDB Leds turned OFF");
    }
    else
    {
        m_logger->Debug("Error turning PDB LEDs OFF.");
        reconnect();
    }
    return success;
}

void PDBController::reconnect()
{
    m_logger->Debug("Reconnecting to the device.");
    m_device->close();

    if (!m_device->open())
    {
        m_logger->Debug("Failed to reconnect to device.");
    }
}

PDBController* PDBController::getInstance()
{
    if (m_instance == NULL)
    {
        m_instance = new PDBController();
    }

    return m_instance;
}
