/**
 * @file BCRPolling.cxx
 * @brief Body class barcode reader monitor polling
 * @date 13/07/2015
 * @author Yadickson Soto
 */

#include "BCRPolling.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/BCRController.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * BCRPolling::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Card.BCRPolling", GETTEXT_PACKAGE );
BCRPolling * BCRPolling::instance = NULL;

BCRPolling::BCRPolling()
    : flagPause(true)
    , inCommand(SAESFunctionalityConfigurationManager::getInstance()->getScanVoteConfiguration().getBarcodeInParams())
    , outCommand(SAESFunctionalityConfigurationManager::getInstance()->getScanVoteConfiguration().getBarcodeOutParams())
    , sendInCommand(false)
    , sendOutCommand(false)
{
    safeSync.getSyncSignal().connect(
        sigc::mem_fun(*this, &BCRPolling::onRawObject));
}

BCRPolling::~BCRPolling()
{
    instance = NULL;
}

void BCRPolling::preStart ()
{
    logger->Debug ( "Start" );
    flagPause = false;
    sendInCommand = true;
    sendOutCommand = false;
}

bool BCRPolling::process ()
{
    if ( sendInCommand )
    {
        sendInCommand = false;
        BCRController::getInstance()->sendReconfigureCommand ( inCommand );
    }

    if ( !flagPause )
    {
        SafeBuffer buffer = BCRController::getInstance()->readBuffer();

        if ( buffer.GetSize() > 0 )
        {
            std::stringstream ss;
            ss << "Emit barcode data size (";
            ss << buffer.GetSize();
            ss << ").";

            logger->Debug ( ss.str() );
            safeSync.emit ( new SafeBuffer ( buffer ) );
        }
    }

    if ( sendOutCommand )
    {
        sendOutCommand = false;
        BCRController::getInstance()->sendReconfigureCommand ( outCommand );
    }

    return true;
}

void BCRPolling::postStop ()
{
    logger->Debug ( "Stop" );
    BCRController::getInstance()->sendReconfigureCommand ( outCommand );
}

void BCRPolling::pause ()
{
    if ( isRunning() )
    {
        logger->Debug ( "Pause" );
        flagPause = true;
        sendInCommand = false;
        sendOutCommand = true;
    }
}

void BCRPolling::resume ()
{
    bool isClosedElection = MachineOperationManager::getInstance()->isCloseElection();
    bool isAboutToClose = MachineOperationManager::getInstance()->isWaitingConfirmationToCloseElection();

    if ( isRunning() && !isClosedElection && !isAboutToClose)
    {
        logger->Debug ( "Resume" );
        flagPause = false;
        sendInCommand = true;
        sendOutCommand = false;
    }
}

void BCRPolling::onRawObject ( void * object )
{
    SafeBuffer * buffer = static_cast < SafeBuffer * > ( object );
    getBufferSignal().emit ( buffer );
    delete buffer;
}

BCRPolling::BufferSignal BCRPolling::getBufferSignal()
{
    return bufferSignal;
}

bool BCRPolling::getIsPaused()
{
    return flagPause;
}

BCRPolling * BCRPolling::getInstance()
{
    if ( !instance )
    {
        instance = new BCRPolling();
    }

    return instance;
}
