/**
 * @file CardPolling.cxx
 * @brief Implementation of the CardPolling class.
 * @date 06/03/2012
 * @author Yadickson Soto
 */

#include "CardPolling.hxx"
#include <Card/ISmartcardReaderPowerManager.hxx>
#include <Configuration/SmartcardConfigurationManager.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/CardController.hxx>
#include <Operation/Devices/SmartcardReaderPowerManagerFactory.hxx>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Common::Configuration;

ISMTTLog * CardPolling::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Card.CardPolling", GETTEXT_PACKAGE );
CardPolling * CardPolling::instance = NULL;

CardPolling::CardPolling()
{
    this->present = false;
    this->flagPause = false;
    this->certainty = false;

    signalDispatcher.getSyncSignal().connect ( sigc::mem_fun ( *this, &CardPolling::onRawObject ) );
}

CardPolling::~CardPolling()
{

}

void CardPolling::preStart ()
{
    logger->Debug ( "Start" );

    if ( SmartcardConfigurationManager::getInstance()->getCallRestartBeforeInitialize() )
    {
        ISmartcardReaderPowerManager* powerManager =
            SmartcardReaderPowerManagerFactory::getSmartcardReaderPowerManager();

        powerManager->callRestartSmartcard();
        sleep ( 2 );
    }
}

bool CardPolling::process()
{
    if ( !flagPause )
    {
        bool current = CardController::getInstance()->isCardPresent();

        if ( current != present )
        {
            present = current;

            if ( present )
            {
                logger->Debug ( "Card Insert" );
                signalDispatcher.emit ( new CardPollingStatus ( INSERTED ) );
            }
            else
            {
                logger->Debug ( "Card Remove" );
                signalDispatcher.emit ( new CardPollingStatus ( REMOVED ) );
            }
        }

        certainty = true;
    }

    return true;
}

void CardPolling::postStop ()
{
    logger->Debug ( "Stop" );
}

bool CardPolling::isCardPresent() const
{
    return present;
}

bool CardPolling::certaintyOfCardPresence() const
{
    return certainty;
}

void CardPolling::pause ()
{
    if ( isRunning() )
    {
        logger->Debug ( "Pause" );
        flagPause = true;
    }
}

void CardPolling::resume ()
{
    present = true;
    certainty = false;

    if ( isRunning() )
    {
        logger->Debug ( "Resume" );
        flagPause = false;
    }
}

CardPolling * CardPolling::getInstance()
{
    if ( !instance )
    {
        instance = new CardPolling();
    }

    return instance;
}

void CardPolling::onRawObject ( void * object )
{
    CardPollingStatus * status = static_cast < CardPollingStatus * > ( object );
    getSignalCard().emit ( *status );
    delete status;
}

CardPolling::SignalCard CardPolling::getSignalCard()
{
    return signalCardChange;
}
