/**
 * @file CardPolling.hxx
 * @brief Declaration of the CardPolling class.
 * @date 06/03/2012
 * @author Yadickson Soto
 */

#ifndef CARD_POLLING_HXX_
#define CARD_POLLING_HXX_

#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Operation
        {
            namespace Devices
            {
                /**
                 * @class CardPolling
                 * @brief Represents a smartcard monitor.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 06/03/2012
                 */
                class CardPolling : public Smartmatic::System::SafeThread
                {
                    protected:

                        /**
                         * @fn CardPolling::CardPolling();
                         * @brief Constructor for the CardPolling class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 06/03/2012
                         */
                        CardPolling();

                        /**
                         * @fn	    virtual void preStart ();
                         * @brief	Callback from thread before start thread.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         */
                        virtual void preStart ();

                        /**
                         * @fn virtual bool process();
                         * @brief Runs the smartcard monitoring process.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 06/03/2012
                         * @return true to continue process
                         */
                        virtual bool process();

                        /**
                         * @fn	    virtual void postStop ();
                         * @brief	Callback from thread after stop thread.
                         * @author	Yadickson Soto
                         * @date	04/08/2015
                         */
                        virtual void postStop ();

                        /**
                         * @fn	    void onRawObject ( void * object );
                         * @brief	Callback when emit sync signal.
                         * @author	Yadickson Soto
                         * @date	01/08/2015
                         * @param 	object The object to process
                         */
                        void onRawObject ( void * object );

                    public:

                        /**
                         * @enum CardPollingStatus
                         * @brief Represents the smartcard monitor status.
                         */

                        enum CardPollingStatus
                        {
                            /**
                             * @brief Smart card inserted
                             */
                            INSERTED,

                            /**
                             * @brief Smart card removed
                             */
                            REMOVED,

                            /**
                             * @brief Smart card reader error
                             */
                            ERROR_SCR
                        };

                        /**
                         * @typedef sigc::signal<void, CardPollingStatus > signalCard
                         * @brief Defines an alias representing the signal for any smartcard event.
                         */

                        typedef sigc::signal<void, CardPollingStatus > SignalCard; ///< typedef signal card

                        /**
                         * @fn virtual CardPolling::~CardPolling();
                         * @brief Finalizes an instance of the CardPolling class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 06/03/2012
                         */
                        virtual ~CardPolling();

                        /**
                         * @fn bool CardPolling::isCardPresent();
                         * @brief Verifies if a smartcard is presnet in the smartcard reader.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 08/03/2012
                         * @return true if a smartcard is present. Otherwise, false.
                         */
                        bool isCardPresent() const;

                        /**
                         * @brief  Certainty of the card presence state.
                         * @author manuel.arguelles
                         * @date   Mar  1, 2018
                         * @return True if the presence can be confirmed without doubt,
                         * false if the status is a guess.
                         */
                        bool certaintyOfCardPresence() const;

                        /**
                         * @fn void pause ();
                         * @brief Pause read device.
                         * @author Yadickson Soto
                         * @date 04/08/2015
                         */
                        void pause ();

                        /**
                         * @fn void resume ();
                         * @brief Resume read device.
                         * @author Yadickson Soto
                         * @date 04/08/2015
                         */
                        void resume ();

                        /**
                         * @fn static CardPolling * CardPolling::getInstance();
                         * @brief Gets the current instance of the CardPolling monitor class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 06/03/2012
                         * @return instance.
                         */
                        static CardPolling * getInstance();

                        /**
                         * @fn SignalCard getSignalCard();
                         * @brief Gets the signal for any event related to a smartcard.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 26/03/2013
                         * @return signal signalCard object.
                         */
                        SignalCard getSignalCard();

                    private:

                        static CardPolling * instance; ///< instance
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                        bool present; ///< flag card present
                        bool flagPause; ///< The flag pause
                        bool certainty; ///< Certainty of card presence status
                        Smartmatic::System::SafeDispatcher signalDispatcher; ///< The sync signal
                        SignalCard signalCardChange;   ///< The signal card
                };
            }
        }
    }
}

#endif /* CARD_POLLING_HXX_ */
