/**
 * @file PrinterController.cxx
 * @brief Implementation of the PrinterController class.
 * @date 17/12/2012
 * @author Yadickson Soto
 */

#include "PrinterController.hxx"
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Printing/PrinterStatusNotifier.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Configuration/PrinterControllerConfigurationManager.hxx>
#include <unistd.h>
#include <System/IO/File.h>
#include <System/Utils/StringUtils.hxx>
#include <System/Utils/SafeBufferUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation::Printer;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;

ISMTTLog * PrinterController::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Printer.PrinterController", GETTEXT_PACKAGE );
unsigned long PrinterController::countJobs = 0;

PrinterController::PrinterController()
{
    neop = true;
    status = false;
    saveStatus = false;

    skipPrinter = PrinterControllerConfigurationManager::getInstance()->shouldSkipPrinter();
    timeout = PrinterControllerConfigurationManager::getInstance()->getTimeoutPrint();
    bufferFile = PrinterControllerConfigurationManager::getInstance()->getResultPrintFile ();
    statusFile = PrinterControllerConfigurationManager::getInstance()->getStatusPrintFile ();

    signalDistpacherEnd.getSyncSignal().connect ( sigc::mem_fun ( *this, &PrinterController::onRawObject ) );

    printerStatus = new PrinterDiagnoseStatus ( MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper() );
}

PrinterController::~PrinterController()
{
    if ( printerStatus )
    {
        delete printerStatus;
        printerStatus = NULL;
    }
}

void PrinterController::onDocumentPrinted ( Smartmatic::SAES::Printing::PrintDocumentResult res )
{
    status = res.getSuccess ();
    msg = res.getPrinterStatusDetail ();

    if ( !status && saveStatus )
    {
        if ( res.getPrinterStatusNotifierResult () == PrinterStatusNotifier::PSTATE_NONE )
        {
            printerStatus->addPrintDocumentResult ( res.getPrintingResult() );
        }
        else
        {
            printerStatus->addPrinterStatusNotifier ( res.getPrinterStatusNotifierResult() );
        }
    }

    logger->Debug ( "Message : " + msg );

    signalDistpacherEnd.emit ( new bool ( status ) );
}

bool PrinterController::checkStatusPrinter ( std::string & message, bool neop )
{
    bool validate = false;

    if ( saveStatus )
    {
        printerStatus->clear();
    }

    unsigned int numberOfDevices = DeviceTypeMonitorPool::getInstance()->getNumberOfDevicesByType ( DeviceType::CONFIGURATION_PRINTER );

    if ( numberOfDevices == 0 )
    {
        message = _ ( "Smartmatic.SAES.Operation.Diagnosis.PrinterDiagnosis.PrinterNotFound" );
        logger->Debug ( message );

        if ( saveStatus )
        {
            printerStatus->addPrinterStatusNotifier ( PrinterStatusNotifier::PSTATE_OFFLINE );
        }

        return validate;
    }

    PrinterStatusNotifier printNotifier;
    PrinterStatusNotifier::CheckPrinterState ( true );
    PrinterStatusNotifier::PhysicalStateEnumType printerState = printNotifier.getPrinterState ();

    if ( ( printerState & printNotifier.getErrorCondition () ) != 0 )
    {
        message = printNotifier.getStatusMessage ( printerState );
        validate = false;

        if ( saveStatus )
        {
            printerStatus->addPrinterStatusNotifier ( printerState );
        }
    }
    else if ( neop && ( printerState & PrinterStatusNotifier::PSTATE_NEAR_END_OF_PAPER ) )
    {
        message = printNotifier.getStatusMessage ( PrinterStatusNotifier::PSTATE_NEAR_END_OF_PAPER );
        validate = false;

        if ( saveStatus )
        {
            printerStatus->addPrinterStatusNotifier ( printerState );
        }
    }
    else if ( printerState & PrinterStatusNotifier::PSTATE_SUPPLYLOW_WARN )
    {
        validate = true;

        if ( saveStatus )
        {
            printerStatus->addPrinterStatusNotifier ( printerState );
        }
    }
    else
    {
        validate = true;
    }

    logger->Debug ( std::string ( validate ? "Success " : "Fail " ) + message );

    return validate;
}

void PrinterController::start ( Smartmatic::SAES::Printing::IBasePrintDocument * printObj, bool neop, bool saveStatus )
{
    this->printObj = printObj;
    this->neop = neop;
    this->saveStatus = saveStatus;

    SafeThread::start ( 0 );
}

bool PrinterController::process()
{
    countJobs++;

    if ( skipPrinter )
    {
        logger->Debug ( "Skip printing" );

        sleep ( timeout );

        if ( printObj && !bufferFile.empty() )
        {
            SafeBuffer buffer = printObj->getBuffer();
            std::string file = bufferFile + "." + StringUtils::longToString ( countJobs, 4 );

            logger->Debug ( "BufferToFile (" + file + ")" );

            File::getInstance()->saveBufferToFile ( buffer, file );
        }

        PrintDocumentResult::PrintingResultEnumType status = PrintDocumentResult::PRINTING_COMPLETED;

        SafeBuffer data;

        if ( File::getInstance()->openFileToBuffer ( statusFile, data ) && data.GetSize() > 0 )
        {
            std::string event;

            SafeBufferUtils buffUtil ( data );
            std::vector < SafeBuffer > result = buffUtil.splitByEndLine();

            if ( ! result.empty() )
            {
                SafeBuffer item ( result.front() );
                event.assign ( ( const char * ) item.GetData(), item.GetSize() );

                if ( StringUtils::toUpper ( StringUtils::trim ( event ) ) == "OK" )
                {
                    status = PrintDocumentResult::PRINTING_COMPLETED;
                    msg = _ ( "Printer OK" );
                }
                else
                {
                    status = PrintDocumentResult::PRINTING_CANCELED;
                    msg = _ ( "Printer NOK" );
                }
            }
        }

        printerStatus->addPrintDocumentResult ( status );
        signalDistpacherEnd.emit ( new bool ( status == PrintDocumentResult::PRINTING_COMPLETED ) );

        return false;
    }

    if ( ! ( status = checkStatusPrinter ( msg, neop ) ) )
    {
        signalDistpacherEnd.emit ( new bool ( false ) );
        return false;
    }

    if ( printObj )
    {
        printObj->setClient ( this );
        printObj->Print();
    }
    else
    {
        signalDistpacherEnd.emit ( new bool ( false ) );
    }

    return false;
}

sigc::signal<void> PrinterController::getSignalOk()
{
    return signalOk;
}

sigc::signal<void, std::string> PrinterController::getSignalError()
{
    return signalError;
}

void PrinterController::onRawObject ( void * object )
{
    bool * status = static_cast < bool * > ( object );

    if ( *status )
    {
        getSignalOk().emit();
    }
    else
    {
        getSignalError().emit ( msg );
    }

    delete status;
}
