/**
 * @file PrinterController.hxx
 * @brief Declaration of the PrinterController class.
 * @date 17/12/2012
 * @author Yadickson Soto
 */

#ifndef PRINTERCONTROLLER_HXX_
#define PRINTERCONTROLLER_HXX_

#include <Printing/PrintDocumentClient.h>
#include <Printing/IBasePrintDocument.hxx>
#include <Operation/Printer/PrinterDiagnoseStatus.hxx>
#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Devices
            {
                /**
                 * @class PrinterController
                 * @brief Exposes the common functionality related to printing.
                 * @date 17/12/2012
                 * @author Yadickson Soto
                 */
                class PrinterController: public Smartmatic::SAES::Printing::PrintDocumentClient, private Smartmatic::System::SafeThread
                {
                    public:

                        /**
                         * @fn PrinterController();
                         * @brief Default constructor for the PrinterController class.
                         * @date 17/12/2012
                         * @author Yadickson Soto
                         */
                        PrinterController();

                        /**
                         * @fn virtual ~PrinterController();
                         * @brief Finalizes an instance of the PrinterController class.
                         * @date 17/12/2012
                         * @author Yadickson Soto
                         */

                        virtual ~PrinterController();

                        /**
                         * @fn void start ( Smartmatic::SAES::Printing::IBasePrintDocument * printObj, bool neop = false, bool saveStatus = false );
                         * @brief Start the printing process of the given document.
                         * @author Yadickson Soto
                         * @date 03/04/2013
                         * @param  printObj document to print.
                         * @param  neop check if the printer device is on end of paper status.
                         * @param  saveStatus save status inside voting machine status wrapper
                         */

                        void start ( Smartmatic::SAES::Printing::IBasePrintDocument * printObj, bool neop = false, bool saveStatus = false );

                        /**
                         * @fn sigc::signal<void> getSignalOk();
                         * @brief Gets the signal that occurs when the printing finished successfully.
                         * @author Yadickson Soto
                         * @date 03/04/2013
                         * @return signal sigc::signal<void>
                         */

                        sigc::signal<void> getSignalOk();

                        /**
                         * @fn sigc::signal<void, std::string> getSignalError();
                         * @brief Gets the signal that occurs when the printing finished with errors.
                         * @author Yadickson Soto
                         * @date 03/04/2013
                         * @return signal sigc::signal<void, std::string>
                         */

                        sigc::signal<void, std::string> getSignalError();

                    protected:

                        /**
                         * @fn bool checkStatusPrinter( std::string & message, bool neop = true );
                         * @brief Verifies if the printer device is ready to print.
                         * @author Yadickson Soto
                         * @date 03/04/2013
                         * @param  message messages status
                         * @param  neop check if the printer device is on end of paper status.
                         * @return true if the printer device is ready to print. Otherwise, false.
                         */

                        bool checkStatusPrinter ( std::string & message, bool neop = false );

                        /**
                         * @fn	    virtual bool process ();
                         * @brief	Callback from thread any time.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @return true for continue thread
                         */
                        virtual bool process ();

                        /**
                         * @fn	    void onRawObject ( void * object );
                         * @brief	Callback when emit sync signal. Executes when the printing process ends
                         * @author	Yadickson Soto
                         * @date	01/08/2015
                         * @param 	object The object to process
                         */
                        void onRawObject ( void * object );

                    private:

                        /**
                         * @fn virtual void onDocumentPrinted ( Smartmatic::SAES::Printing::PrintDocumentResult res );
                         * @brief Signals the end of the printing process.
                         * @author Yadickson Soto
                         * @date 17/12/2012
                         * @param res printing process result.
                         */

                        virtual void onDocumentPrinted ( Smartmatic::SAES::Printing::PrintDocumentResult res );

                        Smartmatic::SAES::Printing::IBasePrintDocument * printObj; ///< object printing
                        std::string msg; ///< end message status

                        bool status; ///< status
                        bool neop; ///< flag neop
                        bool saveStatus; ///< flag save status voting machine

                        Smartmatic::System::SafeDispatcher signalDistpacherEnd; ///< signal dispatcher end printing

                        sigc::signal<void> signalOk; ///< signal ok
                        sigc::signal<void, std::string> signalError; ///< signal error

                        bool skipPrinter; ///< Skip all printing job requests and simulate successful printing every time.

                        unsigned int timeout; ///< The timeout

                        std::string bufferFile; ///< The buffer file to save print result
                        std::string statusFile; ///< The status file to end print result

                        Smartmatic::SAES::Operation::Printer::PrinterDiagnoseStatus * printerStatus; ///< The printer diagnose status
                        static unsigned long countJobs; ///< The printer count jobs
                        static Smartmatic::Log::ISMTTLog * logger;  ///< logger
                };
            }
        }
    }
}

#endif /* PRINTERCONTROLLER_HXX_ */
