/**
 * @file FlashDiagnosis.hxx
 * @brief Declaration of the FlashDiagnosis class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */

#ifndef FLASHDIAGNOSIS_HXX_
#define FLASHDIAGNOSIS_HXX_

#include "IDiagnosis.hxx"

namespace Smartmatic
{
    namespace System
    {
        namespace Runtime
        {
            class StorageDevice;
        }
    }
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Operation
        {
            namespace Diagnosis
            {
                /**
                 * @class FlashDiagnosis
                 * @brief Represents a storage device diagnostic.
                 * @author Youssef.tourkmani
                 * @date 01/11/2012
                 */
                class FlashDiagnosis : public IDiagnosis
                {
                    public:

                        /**
                         * @fn FlashDiagnosis::FlashDiagnosis ( bool checkBoth = true );
                         * @brief Constructor for the FlashDiagnosis class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @param checkBoth (optional) the check both.
                         */

                        explicit FlashDiagnosis ( bool checkBoth = true );

                        /**
                         * @fn virtual FlashDiagnosis::~FlashDiagnosis();
                         * @brief Finalizes an instance of the FlashDiagnosis.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */

                        virtual ~FlashDiagnosis();

                    protected:

                        /**
                         * @fn virtual void FlashDiagnosis::run();
                         * @brief Runs the storage device diagnostic.
                         * @author Yadickson Soto
                         * @date 28/09/2012
                         */

                        virtual void run();

                        /**
                         * @fn virtual void FlashDiagnosis::updateMessage();
                         * @brief Updates the status message of this diagnostic.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */

                        virtual void updateMessage();

                    private:

                        /**
                         * @fn bool FlashDiagnosis::getDeviceStatus ( std::string device );
                         * @brief Gets the storage device status.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @param device The device.
                         * @return true if it succeeds, false if it fails.
                         */

                        bool getDeviceStatus ( std::string device );

                        /**
                         * @brief  Checks the presence and accessibility of FAT partitions.
                         * @author manuel.arguelles
                         * @date   Aug 17, 2017
                         * @return True if it succeeded, false otherwise.
                         */
                        bool getFat32Status();

                        /**
                         * @brief  Check if a given Fat 32 StorageDevice is writable.
                         * @author manuel.arguelles
                         * @date   Aug 17, 2017
                         * @param  device The Fat 32 device to check.
                         * @return True if the device could be mounted and written to,
                         * false otherwise.
                         */
                        bool checkFat32Device(System::Runtime::StorageDevice& device);

                    private:

                        bool checkBoth; ///< true to check both
                        std::string errorMessage; ///< Message describing the error
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                };
            }
        }
    }
}

#endif /* FLASHDIAGNOSIS_HXX_ */
