/**
 * @file PMachineInitiator.hxx
 * @brief Declaration of the PMachineInitiator class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */

#ifndef _PMACHINE_INITIATOR_H_
#define _PMACHINE_INITIATOR_H_

#include "IMachineInitiator.hxx"

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            namespace Initiator
            {

                class CheckDirectoryStructure;
                class CheckFilesPresence;
            }
        }
    }
}

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            namespace Initiator
            {
                /**
                 * @class PMachineInitiator
                 * @brief Represents a president machine initiator.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 03-03-2011
                 */

                class PMachineInitiator: public IMachineInitiator
                {

                    public:

                        /**
                         * @fn PMachineInitiator::PMachineInitiator ();
                         * @brief Constructor for the PMachineInitiator class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 03-03-2011
                         */

                        PMachineInitiator ();

                        /**
                         * @fn virtual PMachineInitiator::~PMachineInitiator ();
                         * @brief Finalizes an instance of the PMachineInitiator class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 03-03-2011
                         */

                        virtual ~PMachineInitiator ();

                        /**
                         * @fn virtual void PMachineInitiator::dispose();
                         * @brief Disposes an instance of the PMachineInitiator class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 03-03-2011
                         */

                        virtual void dispose();

                        /**
                         * @fn bool PMachineInitiator::checkData();
                         * @brief Verifies that the data on the internal and external storage devices is correct.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date 25/05/2011
                         * @return true if the data in the internal and external storage devices is correct. Otherwise,
                         * false.
                         */

                        bool checkData();

                        /**
                         * @fn bool PMachineInitiator::checkRunFiles();
                         * @brief Verifies that the files and directory structure on the internal and external storage
                         * devices are correct.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date 31/05/2013
                         * @return true if the files and directory structure in the internal and external storage
                         * devices are correct. Otherwise, false.
                         */

                        bool checkRunFiles();

                        /**
                         * @fn bool PMachineInitiator::runFirstTime();
                         * @brief Runs the initialization process that correspond to the first time the machine
                         * initiates.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date 25/05/2011
                         * @return true if the initialization was successful. Otherwise, false.
                         */

                        bool runFirstTime();

                        /**
                         * @fn bool PMachineInitiator::runNormalMode();
                         * @brief Runs the initialization process that correspond to the normal operation of the
                         * machine.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date 25/05/2011
                         * @return true if the initialization was successful. Otherwise, false.
                         */

                        bool runNormalMode();

                        /**
                         * @fn bool PMachineInitiator::runRecoveryMode ( bool copyDomToFlash );
                         * @brief Runs the initialization process that must be done when the machine is in recovery
                         * mode.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date 25/05/2011
                         * @param copyDomToFlash true to copy dom to flash.
                         * @return true if the initialization was successful. Otherwise, false.
                         */

                        bool runRecoveryMode ( bool copyDomToFlash );

                        /**
                         * @fn bool PMachineInitiator::runReplaceMachineMode();
                         * @brief Runs the initialization process that must be done when the current machine is
                         * replacing another.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date 14/07/2011
                         * @return true if the initialization was successful. Otherwise, false.
                         */

                        bool runReplaceMachineMode();

                        /**
                         * @brief Checks the issuer certificates involved in the election.
                         * @author eduardo.fernandez
                         * @date   08/11/2018
                         * @return true if the validation is successful.
                         */
                        bool checkIssuerCertificates();

                        /**
                         * @fn bool PMachineInitiator::checkBasicConfig();
                         * @brief Verifies that the configuration for this machine is valid.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date 14/07/2011
                         * @return true if the configuration is valid. Otherwise, false.
                         */

                        bool checkBasicConfig();

                        /**
                         * @fn virtual int PMachineInitiator::getSuccessMessageTime();
                         * @brief Gets success message time.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return The success message time.
                         */
                        virtual int getSuccessMessageTime();

                        /**
                         * @fn bool PMachineInitiator::checkResources();
                         * @brief Verifies that the resource files exists.
                         * @author Yadickson Soto
                         * @date 01/11/2012
                         * @return true if resource files exists.
                         */

                        bool checkResources();

                    protected:

                        /**
                         * @fn virtual Smartmatic::Log::ISMTTLog * getLogger();
                         * @brief Getter logger class
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date 09/09/2013
                         * @return logger
                         */
                        virtual Smartmatic::Log::ISMTTLog * getLogger();

                    private:

                        /**
                         * @fn	    virtual void preStart ();
                         * @brief	Callback from thread before start thread.
                         * @author	Yadickson Soto
                         * @date	03/08/2015
                         */
                        virtual void preStart ();

                        /**
                         * @fn	    virtual bool process ();
                         * @brief	Callback from thread any time.
                         * @author	Yadickson Soto
                         * @date	03/08/2015
                         * @return true for continue thread
                         */
                        virtual bool process ();

                        /**
                         * @fn bool PMachineInitiator::CheckConfigurationLoad();
                         * @brief Verifies that the machine configuration was loaded correctly.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 03-03-2011
                         * @return true if the configuration was loaded correctly. Otherwise, false.
                         */

                        bool CheckConfigurationLoad();

                        bool runNormal; ///< emit signal start application
                        RunModeType runMode; ///< The run mode
                        CheckDirectoryStructure * dirCondition; ///< Conditional iterator
                        CheckFilesPresence * filesCondition;    ///< Conditional iterator

                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}

#endif // _PMACHINE_INITIATOR_H_
