/**
 * @file ResultsPackageTransmissionManager.cxx
 * @brief Implementation of the entity that checks the network interface and sends the results package.
 * @date 09/04/2014
 * @author Yadickson Soto
 */

#include "ResultsPackageTransmissionManager.hxx"
#include <Configuration/ResultsPackageTransmissionConfManager.hxx>
#include <Configuration/device-monitor-configuration-schema.hxx>
#include <System/Runtime/ProcessLauncher.h>
#include <Operation/Monitor/EthernetMonitor.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::Log;

ISMTTLog * ResultsPackageTransmissionManager::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.ResultsPackageTransmissionManager", GETTEXT_PACKAGE );

ResultsPackageTransmissionManager::ResultsPackageTransmissionManager ( DeviceTypeMonitor * networkInterface )
    : networkInterface ( networkInterface )
{

}

ResultsPackageTransmissionManager::~ResultsPackageTransmissionManager()
{

}

bool ResultsPackageTransmissionManager::mountNetworkDevice ()
{
    if ( !ResultsPackageTransmissionConfManager::getInstance()->isUpDownInterfaceScriptNeeded() )
    {
        logger->Debug ( "Skip mount network device" );
        return true;
    }

    logger->Debug ( "Mount network device" );

    bool validate = false;

    if ( networkInterface == NULL )
    {
        return false;
    }

    if ( networkInterface->getNumberOfDevices() == 0 )
    {
        std::stringstream command;
        std::string interface = ( static_cast < EthernetMonitor * > ( networkInterface ) )->getInterface ();

        command << ResultsPackageTransmissionConfManager::getInstance()->getNetworkManagerScript();
        command << " " << interface << " up";

        validate = ProcessLauncher::getInstance()->RunProcessThroughSystem ( command.str() ) == 0 && networkInterface->getNumberOfDevices() != 0;
    }
    else
    {
        validate = true;
    }

    logger->Debug ( std::string ( "Mount network device " ) + ( validate ? "success" : "fail" ) );

    return validate;
}

bool ResultsPackageTransmissionManager::umountNetworkDevice ()
{
    if ( !ResultsPackageTransmissionConfManager::getInstance()->isUpDownInterfaceScriptNeeded() )
    {
        logger->Debug ( "Skip umount network device" );
        return true;
    }

    logger->Debug ( "Umount network device" );

    bool validate = false;

    if ( networkInterface != NULL )
    {
        std::stringstream command;
        std::string interface = ( static_cast < EthernetMonitor * > ( networkInterface ) )->getInterface ();

        command << ResultsPackageTransmissionConfManager::getInstance()->getNetworkManagerScript();
        command << " " << interface << " down";

        validate = ProcessLauncher::getInstance()->RunProcessThroughSystem ( command.str() ) == 0 && networkInterface->getNumberOfDevices() == 0;
    }

    logger->Debug ( std::string ( "Umount network device " ) + ( validate ? "success" : "fail" ) );

    return validate;
}

bool ResultsPackageTransmissionManager::sendResultsPackages ( const std::string & centerName, const std::string & devicePath, std::string & messageResponse )
{
    logger->Debug ( "Start of results package transmission" );

    bool validate = false;

    if ( ResultsPackageTransmissionConfManager::getInstance()->isSendResultsPackageScriptNeeded() )
    {
        std::string interface = networkInterface != NULL ? ( static_cast < EthernetMonitor * > ( networkInterface ) )->getInterface () : "device";
        std::string command = ResultsPackageTransmissionConfManager::getInstance()->getFullCommandToSendResults ( centerName, interface, devicePath );

        switch ( ProcessLauncher::getInstance()->RunProcessThroughSystem ( command ) )
        {
            case 0:
                validate = true;
                break;

            case 1:
                messageResponse = _ ( "Connection error during the results package transmission." );
                break;

            case 2:
                messageResponse = _ ( "Package error during the results package transmission." );
                break;

            default:
                messageResponse = _ ( "Script error during the results package transmission." );
                break;
        }
    }
    else
    {
        validate = true;
    }

    if ( !umountNetworkDevice () )
    {
        messageResponse.append ( !validate ? "\n" : "" );
        messageResponse.append ( _ ( "Error bringing network interface down." ) );
        validate = false;
    }

    return validate;
}
