/**
 * @file   AudioVotingCustomSelection.cxx
 * @brief  Implementation of AudioVotingCustomSelection class.
 * @author manuel.arguelles
 * @date   Oct 14, 2017
 */

#include "AudioVotingCustomSelection.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Resources/ElectionAudioResourcesManager.hxx>
#include <System/Audio/TextToSpeech/ITextToSpeech.hxx>
#include <System/OptParser.hxx>
#include <Voting/VotingDevice/CustomAttributeDefinitionWrapper.hxx>
#include <Voting/set-language-functionality.h>

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Audio::TextToSpeech;
using namespace Smartmatic::System;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const unsigned int IDLE_INTERVAL_S                  = 10;
static const unsigned int INTERVAL_MS                      = 10;
static const unsigned int SLEEP_AFTER_STOP_US              = 200000;
static const std::string  GENERAL_INSTRUCTIONS_MESSAGE_KEY = "smartmatic.saes.operation.audiovoting.AudioVotingCustomSelection.GeneralInstructionsMessage";
static const std::string  PRESELECTION_MESSAGE_KEY         = "smartmatic.saes.operation.audiovoting.AudioVotingCustomSelection.PreselectionMessage";
static const std::string  CONFIRMATION_MESSAGE_KEY         = "smartmatic.saes.operation.audiovoting.AudioVotingCustomSelection.ConfirmationMessage";

ISMTTLog* AudioVotingCustomSelection::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.AudioVoting.AudioVotingCustomSelection",
    GETTEXT_PACKAGE);

AudioVotingCustomSelection::AudioVotingCustomSelection(ITextToSpeech* textToSpeech)
    : m_textToSpeech(textToSpeech)
    , m_electoralAudioResources(ElectionAudioResourcesManager::getInstance())
    , m_canInstructionsBeInterrupted(false)
{
}

AudioVotingCustomSelection::~AudioVotingCustomSelection()
{
}

void AudioVotingCustomSelection::start(Custom_attribute_definition customAttribute)
{
    m_customAttribute = customAttribute;
    m_canInstructionsBeInterrupted = false;

    setInitialOption();

    m_currentStage = INITIAL_STAGE;

    SafeThread::start(100);
}

void AudioVotingCustomSelection::preStart()
{
    std::string currentMachineModel;
    OptParser::getInstance()->getArg( OptParser::MACHINE_MODEL, currentMachineModel );

    m_selectConnection = m_abled.signalSelectButton().connect(
        sigc::mem_fun(*this, &AudioVotingCustomSelection::handleSelectPress));
    m_helpConnection = m_abled.signalHelpButton().connect(
        sigc::mem_fun(*this, &AudioVotingCustomSelection::handleHelpPress));
    m_leftConnection = m_abled.signalLeftButton().connect(
        sigc::mem_fun(*this, &AudioVotingCustomSelection::handleLeftPress));
    m_rightConnection = m_abled.signalRightButton().connect(
        sigc::mem_fun(*this, &AudioVotingCustomSelection::handleRightPress));
    m_moreSpeedConnection = m_abled.signalSpeedUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingCustomSelection::handleSpeedPress));
    m_lessSpeedConnection = m_abled.signalSpeedDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingCustomSelection::handleSpeedPress));
}

void AudioVotingCustomSelection::postStop()
{
    m_selectConnection.disconnect();
    m_helpConnection.disconnect();
    m_leftConnection.disconnect();
    m_rightConnection.disconnect();
    m_moreSpeedConnection.disconnect();
    m_lessSpeedConnection.disconnect();
}

bool AudioVotingCustomSelection::process()
{
    if (m_currentStage == INITIAL_STAGE)
    {
        playInstructions();
        playSelection();
        m_currentStage = SELECTION_STAGE;
        m_idleTime.reset();
    }
    else if (m_currentStage == SELECTION_STAGE && m_textToSpeech->isSpeaking())
    {
        m_idleTime.reset();
    }
    else if (m_currentStage == CONFIRMATION_STAGE)
    {
        confirmSelection();
        m_idleTime.reset();
    }
    else if (m_currentStage == END_STAGE)
    {
        Glib::signal_timeout().connect_once(sigc::mem_fun(*this, &AudioVotingCustomSelection::end), INTERVAL_MS);
    }

    if (m_idleTime.elapsed() > IDLE_INTERVAL_S)
    {
        m_currentStage = INITIAL_STAGE;
    }

    return true;
}

void AudioVotingCustomSelection::setInitialOption()
{
    std::string currentLanguage;
    std::string currentCountry;
    SetLanguageFunctionality::getCurrentLanguageCountry(currentLanguage, currentCountry);
    CustomAttributeDefinitionWrapper::sortByLanguage(m_customAttribute, currentLanguage);

    Custom_attribute_definition_values::Custom_attribute_definition_valueSequence attribute =
        m_customAttribute.getCustom_attribute_definition_values().getCustom_attribute_definition_value();
    Custom_attribute_definition_values::Custom_attribute_definition_valueIterator it;

    m_options.clear();

    for (it = attribute.begin(); it != attribute.end(); ++it)
    {
        m_options.push_back(it->getValue());
    }

    m_currentOption = m_options.end();
}

void AudioVotingCustomSelection::playInstructions()
{
    std::string customAttributeNameKey = m_customAttribute.getName_key();
    std::string customAttributeName = m_electoralAudioResources->getResource(customAttributeNameKey);
    std::string generalInstructions;

    generalInstructions = Glib::ustring::compose(
        m_generalAudioResources.getResource(GENERAL_INSTRUCTIONS_MESSAGE_KEY),
        customAttributeName);

    m_logger->Debug(generalInstructions);
    m_textToSpeech->stop();
    m_textToSpeech->speak(generalInstructions);
    m_textToSpeech->waitWhileSpeaking();
    m_canInstructionsBeInterrupted = true;
}

void AudioVotingCustomSelection::end()
{
    m_logger->Debug("Custom selection ended");

    stop();
    m_customSelectedSignal.emit(*m_currentOption);
}

void AudioVotingCustomSelection::playSelection()
{
    if (m_currentOption != m_options.end())
    {
        std::string customAttributeName = m_electoralAudioResources->getResource(*m_currentOption);
        std::string audioText = Glib::ustring::compose(m_generalAudioResources.getResource(PRESELECTION_MESSAGE_KEY), customAttributeName);

        m_logger->Debug(audioText);
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_textToSpeech->speak(audioText);
    }
}

void AudioVotingCustomSelection::nextSelection()
{
    if (m_currentOption == m_options.end())
    {
        m_currentOption = m_options.begin();
    }
    else
    {
        ++m_currentOption;

        if (m_currentOption == m_options.end())
        {
            m_currentOption = m_options.begin();
        }
    }

    playSelection();
}

void AudioVotingCustomSelection::previousSelection()
{
    if (m_currentOption == m_options.begin())
    {
        m_currentOption = m_options.end();
    }

    --m_currentOption;

    playSelection();
}

void AudioVotingCustomSelection::confirmSelection()
{
    std::string optionAudioText = m_electoralAudioResources->getResource(*m_currentOption);
    std::string confirmation = Glib::ustring::compose(m_generalAudioResources.getResource(CONFIRMATION_MESSAGE_KEY), optionAudioText);

    m_logger->Debug(confirmation);
    m_textToSpeech->stop();
    m_textToSpeech->speak(confirmation);
    m_textToSpeech->waitWhileSpeaking();

    m_currentStage = END_STAGE;
}

void AudioVotingCustomSelection::handleLeftPress()
{
    m_idleTime.reset();

    if (m_currentStage == SELECTION_STAGE ||
        (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted))
    {
        previousSelection();
    }
}

void AudioVotingCustomSelection::handleRightPress()
{
    m_idleTime.reset();

    if (m_currentStage == SELECTION_STAGE ||
        (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted))
    {
        nextSelection();
    }
}

void AudioVotingCustomSelection::handleHelpPress()
{
    m_idleTime.reset();
    m_currentStage = INITIAL_STAGE;
}

void AudioVotingCustomSelection::handleSelectPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE && m_currentOption != m_options.end())
    {
        m_currentStage = CONFIRMATION_STAGE;
    }
}

void AudioVotingCustomSelection::handleSpeedPress()
{
    m_idleTime.reset();

    if (m_currentOption == m_options.end())
    {
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_currentStage = INITIAL_STAGE;
    }
    else
    {
        playSelection();
    }
}

AudioVotingCustomSelection::CustomSelectedSignal AudioVotingCustomSelection::signalCustomSelected()
{
    return m_customSelectedSignal;
}
