/**
 * @file   AudioVotingCustomSelection.hxx
 * @brief  Declaration of AudioVotingCustomSelection class.
 * @author manuel.arguelles
 * @date   Oct 14, 2017
 */

#ifndef AUDIO_VOTING_CUSTOM_SELECTION_HXX
#define AUDIO_VOTING_CUSTOM_SELECTION_HXX

#include "AbledController.hxx"
#include "AudioVotingResources.hxx"
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include <System/SafeThread.hxx>
#include <sigc++/sigc++.h>
#include <glibmm.h>
#include <string>
#include <list>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Audio
        {
            namespace TextToSpeech
            {
                class ITextToSpeech;
            }
        }
    }

    namespace SAES
    {
        namespace Resources
        {
            class ElectionAudioResourcesManager;
        }

        namespace Operation
        {
            namespace AudioVoting
            {
                /**
                 * @class  AudioVotingCustomSelection
                 * @brief  Class to handle custom attributes selection.
                 * @author manuel.arguelles
                 * @date   Oct 14, 2017
                 */
                class AudioVotingCustomSelection : public System::SafeThread
                {
                    public:

                        typedef sigc::signal<void, const std::string&> CustomSelectedSignal;

                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         * @param  textToSpeech The text to speech engine to use.
                         * @remark Ownership of the textToSpeech pointer is not transferred.
                         */
                        explicit AudioVotingCustomSelection(System::Audio::TextToSpeech::ITextToSpeech* textToSpeech);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        ~AudioVotingCustomSelection();

                    public:

                        /**
                         * @brief  Starts the selection process.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        void start(SAES::Voting::VotingDevice::Custom_attribute_definition customAttribute);

                        /**
                         * @brief  Emitted when the selection was done.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        CustomSelectedSignal signalCustomSelected();

                    private:

                        /**
                         * @brief  Main loop of the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         * @return True while the selecton is in progress, false when not.
                         */
                        virtual bool process();

                        /**
                         * @brief  Called before starting the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        virtual void preStart();

                        /**
                         * @brief  Called right after the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        virtual void postStop();

                        /**
                         * @brief  Finalizes the selection process.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        void end();

                        /**
                         * @brief  Plays current stage instructions.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void playInstructions();

                        /**
                         * @brief  Goes to the next selection.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        void nextSelection();

                        /**
                         * @brief  Goes to the previous selection.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        void previousSelection();

                        /**
                         * @brief  Plays the current party.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        void playSelection();

                        /**
                         * @brief  Plays current selection audio.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void confirmSelection();

                        /**
                         * @brief  Sets the initial option.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void setInitialOption();

                        /**
                         * @brief  Handles the left button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleLeftPress();

                        /**
                         * @brief  Handles the help button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleHelpPress();

                        /**
                         * @brief  Handles the right button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleRightPress();

                        /**
                         * @brief  Handles the select button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleSelectPress();

                        /**
                         * @brief  Handles the speed button press.
                         * @author manuel.arguelles
                         * @date   Mar 28, 2018
                         */
                        void handleSpeedPress();

                    private:

                        enum Stage
                        {
                            INITIAL_STAGE,
                            SELECTION_STAGE,
                            CONFIRMATION_STAGE,
                            END_STAGE
                        };

                    private:

                        System::Audio::TextToSpeech::ITextToSpeech*             m_textToSpeech;
                        CustomSelectedSignal                                    m_customSelectedSignal;
                        sigc::connection                                        m_abledConnection;
                        SAES::Voting::VotingDevice::Custom_attribute_definition m_customAttribute;
                        SAES::Resources::ElectionAudioResourcesManager*         m_electoralAudioResources;
                        AudioVotingResources                                    m_generalAudioResources;
                        std::list<std::string>                                  m_options;
                        std::list<std::string>::iterator                        m_currentOption;
                        Glib::Timer                                             m_idleTime;
                        AbledController                                         m_abled;
                        Stage                                                   m_currentStage;
                        sigc::connection                                        m_selectConnection;
                        sigc::connection                                        m_helpConnection;
                        sigc::connection                                        m_leftConnection;
                        sigc::connection                                        m_rightConnection;
                        sigc::connection                                        m_moreSpeedConnection;
                        sigc::connection                                        m_lessSpeedConnection;
                        bool                                                    m_canInstructionsBeInterrupted;
                        static Smartmatic::Log::ISMTTLog*                       m_logger;
                };

            } /* namespace AudioVoting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* AUDIO_VOTING_CUSTOM_SELECTION_HXX */

