/**
 * @file   AudioVotingExperienceController.cxx
 * @brief  Implementation of AudioVotingExperienceController class.
 * @author manuel.arguelles
 * @date   Oct 11, 2017
 */

#include "AudioVotingExperienceController.hxx"
#include "TextToSpeechFactory.hxx"
#include "AudioVotingCheckUtility.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/P168AController.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Voting/VotingExperienceController.hxx>
#include <Resources/GeneralAudioResourcesManager.hxx>
#include <System/Audio/TextToSpeech/ITextToSpeech.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Voting/Election/election-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Audio::TextToSpeech;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const unsigned int INTERVAL_MS                 = 10;
static const std::string  VOTE_PRINTING_MESSAGE_KEY   = "smartmatic.saes.operation.audiovoting.AudioVotingExperienceController.VotePrintingMessage";
static const std::string  INSERT_CARD_MESSAGE_KEY     = "smartmatic.saes.gui.widgets.AudioVoting.AudioVotingExperienceWidget.CardExtractedMessage";
static const std::string  VOTING_CANCELED_MESSAGE_KEY = "smartmatic.saes.gui.widgets.AudioVoting.AudioVotingExperienceWidget.VotingCanceledMessage";
static const std::string  DEFAULT_CARD_NAME           = "HDA";
static const std::string  USB_CARD_NAME               = "USB";
static const size_t       ABLE_10_DEFAULT_VOLUME      = 28;
static const size_t       ABLE_20_DEFAULT_VOLUME      = 10;

ISMTTLog* AudioVotingExperienceController::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.AudioVoting.AudioVotingExperienceController",
    GETTEXT_PACKAGE);

AudioVotingExperienceController::AudioVotingExperienceController()
    : m_textToSpeech(TextToSpeechFactory::createTextToSpeech())
    , m_languageSelection(m_textToSpeech.get())
    , m_partySelection(m_textToSpeech.get())
    , m_principalSelection(m_textToSpeech.get())
    , m_standInSelection(m_textToSpeech.get())
    , m_selectionsReview(m_textToSpeech.get())
    , m_reviewConfirmation(m_textToSpeech.get())
    , m_selectionsEdition(m_textToSpeech.get())
    , m_customSelection(m_textToSpeech.get())
    , m_partySelected(false)
    , m_generalAudioResources(GeneralAudioResourcesManager::getInstance())
    , m_previousStage(NEW_VOTING_EXPERIENCE)
    , m_currentStage(LANGUAGE_SELECTION)
    , m_mixer(DEFAULT_CARD_NAME)
    , m_goToReview(false)
    , m_customAttributeSelected(false)
{
    AudioVotingCheckUtility checkUtility;
    AudioVotingExperienceControllerConfiguration configuration =
            SAESFunctionalityConfigurationManager::getInstance()->getAudioVotingExperienceControllerConfiguration();

    if (checkUtility.ableIsConnected() && checkUtility.getAbledConnected() == AudioVotingCheckUtility::ABLED10)
    {
        m_audioVotingDefaultVolume = configuration.getAble10DefaultVolume().present() ?
            configuration.getAble10DefaultVolume().get() :
            ABLE_10_DEFAULT_VOLUME;

        m_logger->Debug("Setting able 1.0 default volume to " + StringUtils::intToString(m_audioVotingDefaultVolume));

        Audio::MixerControl mixer(DEFAULT_CARD_NAME, m_audioVotingDefaultVolume);

        m_mixer = mixer;
    }
    else if (checkUtility.ableIsConnected() && checkUtility.getAbledConnected() == AudioVotingCheckUtility::ABLED20)
    {
        m_audioVotingDefaultVolume = configuration.getAble20DefaultVolume().present() ?
            configuration.getAble20DefaultVolume().get() :
            ABLE_20_DEFAULT_VOLUME;

        m_logger->Debug("Setting able 2.0 default volume to " + StringUtils::intToString(m_audioVotingDefaultVolume));

        Audio::MixerControl mixer(USB_CARD_NAME, m_audioVotingDefaultVolume);

        m_mixer = mixer;
    }

    m_languageSelection.signalLanguageSelected().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::languageSelected));

    m_customSelection.signalCustomSelected().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onCustomAttributeSelected));

    m_partySelection.signalPartySelected().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onPartySelected));
    m_partySelection.signalEmptyPartySelected().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onEmptyPartySelected));
    m_partySelection.signalReturnRequested().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onPartyReturnRequested));

    m_principalSelection.signalOptionSelected().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onBallotSelected));
    m_principalSelection.signalDone().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onPrincipalSelected));
    m_principalSelection.signalReturnRequested().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onPrincipalReturnRequested));

    m_standInSelection.signalOptionSelected().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onBallotSelected));
    m_standInSelection.signalDone().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onStandInSelected));
    m_standInSelection.signalReturnRequested().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onStandInReturnRequested));

    m_selectionsReview.signalDone().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::getReviewConfirmationStage));

    m_reviewConfirmation.signalDone().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onReviewConfirmationEnded));
    m_reviewConfirmation.signalReturn().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onReviewConfirmationReturnRequested));

    m_selectionsEdition.signalOptionSelected().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onReviewOptionSelected));
    m_selectionsEdition.signalDone().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::getReviewConfirmationStage));

    m_abled.signalVolumeDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::decreaseVolume));
    m_abled.signalVolumeUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingExperienceController::increaseVolume));
}

AudioVotingExperienceController::~AudioVotingExperienceController()
{
    stop();
    m_abled.stop();
}

bool AudioVotingExperienceController::isNewStage()
{
    bool changed = (m_previousStage != m_currentStage);

    if (changed && m_currentStage != INTERRUPTED && m_currentStage != CANCELED_VOTING_EXPERIENCE)
    {
        m_previousStage = m_currentStage;
    }

    return changed;
}

void AudioVotingExperienceController::checkRunningStages()
{
    bool running = m_languageSelection.isRunning() ||
        m_customSelection.isRunning() ||
        m_partySelection.isRunning() ||
        m_principalSelection.isRunning() ||
        m_standInSelection.isRunning() ||
        m_selectionsReview.isRunning() ||
        m_reviewConfirmation.isRunning() ||
        m_selectionsEdition.isRunning();

    if (running)
    {
        m_logger->Debug("There are stages running");
    }
    else
    {
        m_logger->Debug("No stages running");
    }
}

void AudioVotingExperienceController::startNewStage()
{
    checkRunningStages();

    switch (m_currentStage)
    {
        case LANGUAGE_SELECTION:
            m_logger->Debug("Starting the language selection stage");
            m_languageSelection.start();
            break;

        case CUSTOM_ATTRIBUTE_SELECTION:
            m_logger->Debug("Starting the custom attribute selection stage");
            m_currentCustomAttribute = m_votingExperienceController->getNextCustomAttributeDefinition();
            m_customAttributeSelected = true;
            m_customSelection.start(m_currentCustomAttribute);
            break;

        case PARTY_SELECTION:
            m_logger->Debug("Starting the party selection stage");
            m_partySelection.start(m_contest, m_goToReview == false && m_contestIndex == 1 && m_customAttributeSelected);
            break;

        case EMPTY_PARTY_SELECTION:
            break;

        case PRINCIPAL_SELECTION:
            m_logger->Debug("Starting the principal selection stage");
            m_principalSelection.start(m_contest, m_party.getCode());
            break;

        case STAND_IN_SELECTION:
            m_logger->Debug("Starting the stand in selection stage");
            m_standInSelection.start(m_contest, m_party.getCode(), m_anySelection);
            break;

        case REVIEW_STAGE:
            m_logger->Debug("Starting the selections review stage");
            m_selectionsReview.start();
            break;

        case REVIEW_CONFIRMATION_STAGE:
            m_logger->Debug("Starting the review confirmation stage");
            m_reviewConfirmation.start();
            break;

        case EDIT_SELECTIONS:
            m_logger->Debug("Starting the selections edition stage");
            m_selectionsEdition.start();
            break;

        case INTERRUPTED:
            m_logger->Debug("The voting experience is currently interrupted");
            m_textToSpeech->speak(m_generalAudioResources->getResource(INSERT_CARD_MESSAGE_KEY));
            m_textToSpeech->waitWhileSpeaking();
            break;

        case CANCELED_VOTING_EXPERIENCE:
            m_logger->Debug("The voting experience has been canceled");
            m_textToSpeech->speak(m_generalAudioResources->getResource(VOTING_CANCELED_MESSAGE_KEY));
            m_textToSpeech->waitWhileSpeaking();
            break;

        case NEW_VOTING_EXPERIENCE:
        default:
            break;
    }
}

bool AudioVotingExperienceController::process()
{
    if (isNewStage())
    {
        startNewStage();
    }

    return true;
}

void AudioVotingExperienceController::postStop()
{
    stopAllStages();
    checkRunningStages();
}

void AudioVotingExperienceController::stopAllStages()
{
    if (m_languageSelection.isRunning())
    {
        m_textToSpeech->stop();
        m_languageSelection.stop();
    }

    if (m_customSelection.isRunning())
    {
        m_textToSpeech->stop();
        m_customSelection.stop();
    }

    if (m_partySelection.isRunning())
    {
        m_textToSpeech->stop();
        m_partySelection.stop();
    }

    if (m_principalSelection.isRunning())
    {
        m_textToSpeech->stop();
        m_principalSelection.stop();
    }

    if (m_standInSelection.isRunning())
    {
        m_textToSpeech->stop();
        m_standInSelection.stop();
    }

    if (m_selectionsReview.isRunning())
    {
        m_textToSpeech->stop();
        m_selectionsReview.stop();
    }

    if (m_reviewConfirmation.isRunning())
    {
        m_textToSpeech->stop();
        m_reviewConfirmation.stop();
    }

    if (m_selectionsEdition.isRunning())
    {
        m_textToSpeech->stop();
        m_selectionsEdition.stop();
    }
}

void AudioVotingExperienceController::dispose()
{
    if (m_textToSpeech.get() != NULL)
    {
        m_previousStage = NEW_VOTING_EXPERIENCE;
        m_textToSpeech->stop();
        stop();
        checkRunningStages();
    }
}

void AudioVotingExperienceController::languageSelected()
{
    m_signalLanguageSelected.emit();

    Glib::signal_timeout().connect_once(
        sigc::mem_fun(*this, &AudioVotingExperienceController::nextStep),
        INTERVAL_MS);
}

void AudioVotingExperienceController::start()
{
    if (m_textToSpeech.get() != NULL)
    {
        m_partySelected = false;
        m_goToReview = false;
        m_customAttributeSelected = false;
        m_currentStage = LANGUAGE_SELECTION;

        m_votingExperienceController = MachineOperationManager::getInstance()->getVotingExperienceController();
        m_mixer.setPlaybackVolume(m_audioVotingDefaultVolume);

        m_abled.start();

        SafeThread::start(500);
    }
    else
    {
        m_logger->Debug("ERROR: Text to speech engine not initialized");
    }
}

sigc::signal<void> AudioVotingExperienceController::getSignalEndAudioVotingExperience()
{
    return m_endAudioVotingExperienceSignal;
}

void AudioVotingExperienceController::playMessage(const std::string& messageKey, const bool waitUntilFinish)
{
    const std::string message = m_generalAudioResources->getResource(messageKey);

    m_textToSpeech->stop();
    m_logger->Debug(message);
    m_textToSpeech->speak(message);

    if (waitUntilFinish)
    {
        m_textToSpeech->waitWhileSpeaking();
    }
}

void AudioVotingExperienceController::interrupt()
{
    if (isRunning())
    {
        m_textToSpeech->stop();
        m_currentStage = INTERRUPTED;
        stopAllStages();
        checkRunningStages();
    }
}

void AudioVotingExperienceController::resume()
{
    if ((m_currentStage == INTERRUPTED) || (m_currentStage == CANCELED_VOTING_EXPERIENCE))
    {
        m_textToSpeech->stop();
        m_currentStage = m_previousStage;

        if (m_currentStage == INTERRUPTED)
        {
            m_previousStage = INTERRUPTED;
        }
        else
        {
            m_previousStage = CANCELED_VOTING_EXPERIENCE;
        }
    }
    else
    {
        m_logger->Debug("Error: Trying to resume while not in interrupted stage");
    }
}

void AudioVotingExperienceController::cancel()
{
    m_textToSpeech->stop();
    m_currentStage = CANCELED_VOTING_EXPERIENCE;
}

void AudioVotingExperienceController::killVotingExperience()
{
    if (isRunning())
    {
        stopAllStages();
        checkRunningStages();
        stop();
        cancel();
    }
}

std::string AudioVotingExperienceController::getActivationCode()
{
    return m_votingExperienceController->getActivationCode();
}

Smartmatic::SAES::Voting::Vote::VoteMode AudioVotingExperienceController::getVotingMode()
{
    return m_votingExperienceController->getVotingMode();
}

CurrentStage AudioVotingExperienceController::getCurrentStage()
{
    return m_currentStage;
}

AudioVotingExperienceController::SignalLanguageSelected AudioVotingExperienceController::getSignalLanguageSelected()
{
    return m_signalLanguageSelected;
}

void AudioVotingExperienceController::nextStep()
{
    VotingExperienceController::VotingExperienceType nextStep = m_votingExperienceController->getNextTypeAction();

    switch (nextStep)
    {
        case VotingExperienceController::VOTING_CONTROLLER_CONTEST:
            m_logger->Debug("Loading the contest list");
            loadContestList();
            break;

        case VotingExperienceController::VOTING_CONTROLLER_CUSTOM_ATTRIBUTE_DEFINITION:
            m_logger->Debug("Loading the custom attributes");
            askCustomAttribute();
            break;

        default:
        case VotingExperienceController::VOTING_CONTROLLER_CLOSE:
            m_logger->Debug("Voting process completed, loading review stage");
            getReviewStage();
            break;
    }
}

void AudioVotingExperienceController::loadContestList()
{
    m_contestList = m_votingExperienceController->getCurrentContestList();

    if (!m_contestList.empty())
    {
        m_contestIterator = m_contestList.begin();
        m_contestIndex = 0;
        m_totalContests = static_cast<int>(m_contestList.size());
        nextContest();
    }
}

void AudioVotingExperienceController::nextContest()
{
    if (m_contestIndex < m_totalContests)
    {
        ++m_contestIndex;
        getPartySelection();
    }
    else
    {
        m_votingExperienceController->moveContestListForward();
        nextStep();
    }
}

void AudioVotingExperienceController::setActualContestFromIndex(const int index)
{
    m_contestIterator = m_contestList.begin();

    for (int i = 1; i < index; ++i)
    {
        ++m_contestIterator;
    }

    m_contest = *m_contestIterator;
}

void AudioVotingExperienceController::getPartySelection()
{
    m_currentStage = PARTY_SELECTION;
    setActualContestFromIndex(m_contestIndex);
}

void AudioVotingExperienceController::onPartySelected(Party& party)
{
    m_party = party;
    m_partySelected = true;

    if (m_contest->getCode() != "")
    {
        if (m_votingExperienceController->isThereAnyPartySelectedFromContest(*m_contest))
        {
            Party previousParty = m_votingExperienceController->getPartySelectedFromContest(*m_contest);

            if (m_party.getCode() != previousParty.getCode())
            {
                if (previousParty.getCode() != "")
                {
                    m_votingExperienceController->clearBallotOptionsSelectedByContestParty(*m_contest, previousParty);
                }
            }
        }
    }

    getPrincipalSelection();
}

void AudioVotingExperienceController::onEmptyPartySelected()
{
    m_partySelected = false;
    m_votingExperienceController->clearBallotOptionsSelectedByContest(*m_contest);
    m_votingExperienceController->setNullPartyResponse(*m_contest);

    m_currentStage = EMPTY_PARTY_SELECTION;

    Glib::signal_timeout().connect_once(
         sigc::mem_fun(*this, &AudioVotingExperienceController::getPrincipalSelection),
         INTERVAL_MS + 500);
}

void AudioVotingExperienceController::onPartyReturnRequested()
{
    if (m_votingExperienceController->moveContestListBack())
    {
        nextStep();
    }
    else
    {
        m_contestIterator = m_contestList.begin();
        m_contestIndex = 0;
        m_totalContests = static_cast<int>(m_contestList.size());
        nextContest();
    }
}

void AudioVotingExperienceController::onPrincipalReturnRequested()
{
    m_principalSelection.deselectAllOptions();
    m_standInSelection.deselectAllOptions();

    --m_contestIndex;
    nextContest();
}

void AudioVotingExperienceController::onStandInReturnRequested()
{
    AudioVotingExperienceControllerConfiguration configuration =
        SAESFunctionalityConfigurationManager::getInstance()->getAudioVotingExperienceControllerConfiguration();

    if (configuration.getDeselectAudioOptionsWhenReturningToPrincipalList())
    {
        m_principalSelection.deselectAllOptions();
        m_standInSelection.deselectAllOptions();
    }

    getPrincipalSelection();
}

void AudioVotingExperienceController::getPrincipalSelection()
{
    if (m_partySelected == false)
    {
        if (m_goToReview)
        {
            m_votingExperienceController->setFinalMemento();
            getReviewStage();
        }
        else
        {
            nextContest();
        }
    }
    else
    {
        m_currentStage = PRINCIPAL_SELECTION;
    }
}

void AudioVotingExperienceController::getReviewConfirmationStage()
{
    m_currentStage = REVIEW_CONFIRMATION_STAGE;
}

void AudioVotingExperienceController::onReviewConfirmationEnded(const bool editSelections)
{
    if (editSelections)
    {
        m_currentStage = EDIT_SELECTIONS;
    }
    else
    {
        endVotingExperience();
    }
}

void AudioVotingExperienceController::onReviewConfirmationReturnRequested()
{
    getReviewStage();
}

void AudioVotingExperienceController::getReviewStage()
{
    m_currentStage = REVIEW_STAGE;
}

void AudioVotingExperienceController::onPrincipalSelected(bool anySelection)
{
    m_anySelection = anySelection;

    m_currentStage = STAND_IN_SELECTION;
}

void AudioVotingExperienceController::onStandInSelected()
{
    Glib::signal_timeout().connect_once(
        sigc::mem_fun(*this, &AudioVotingExperienceController::onStandInDone),
        INTERVAL_MS);
}

void AudioVotingExperienceController::onBallotSelected(
    const std::string& ballotOptionCode,
    const bool isSelected)
{
    m_votingExperienceController->setBallotOptionState(
        m_contest->getCode(),
        m_party.getCode(),
        ballotOptionCode,
        isSelected);
}

void AudioVotingExperienceController::onReviewOptionSelected(const int mementoPosition, const int contestsIndex)
{
    m_goToReview = true;
    m_votingExperienceController->returnToSpecificContest(mementoPosition);
    m_votingExperienceController->deselectOptions(contestsIndex);

    m_contestList = m_votingExperienceController->getCurrentContestList(mementoPosition);

    setActualContestFromIndex(contestsIndex);
    m_contestIndex = contestsIndex;
    m_totalContests = static_cast<int>(m_contestList.size());

    nextContest();
}

void AudioVotingExperienceController::onStandInDone()
{
    m_principalSelection.clearSelectionsArray();

    if (m_goToReview)
    {
        m_votingExperienceController->setFinalMemento();
        getReviewStage();
    }
    else if (m_contestIndex != m_totalContests)
    {
        nextContest();
    }
    else
    {
        m_votingExperienceController->moveContestListForward();
        nextStep();
    }
}

void AudioVotingExperienceController::askCustomAttribute()
{
    m_currentStage = CUSTOM_ATTRIBUTE_SELECTION;
}

void AudioVotingExperienceController::onCustomAttributeSelected(const std::string& code)
{
    m_votingExperienceController->setCustomAttributeDefinitionResponse(
        m_currentCustomAttribute.getCode(),
        code);
    nextStep();
}

void AudioVotingExperienceController::endVotingExperience()
{
    m_anySelection = false;

    const std::string votePrintingMessage = m_generalAudioResources->getResource(VOTE_PRINTING_MESSAGE_KEY);

    m_textToSpeech->stop();
    m_logger->Debug(votePrintingMessage);
    m_textToSpeech->speak(votePrintingMessage);
    m_textToSpeech->waitWhileSpeaking();

    m_abled.stop();
    stop();
    m_endAudioVotingExperienceSignal.emit();
}

void AudioVotingExperienceController::increaseVolume()
{
    m_logger->Debug("Increasing volume");
    m_mixer.increasePlaybackVolume();
}

void AudioVotingExperienceController::decreaseVolume()
{
    m_logger->Debug("Decreasing volume");
    m_mixer.decreasePlaybackVolume();
}
