/**
 * @file   AudioVotingLanguageSelection.hxx
 * @brief  Declaration of AudioVotingLanguageSelection class.
 * @author manuel.arguelles
 * @date   Oct 11, 2017
 */

#ifndef AUDIO_VOTING_LANGUAGE_SELECTION_HXX
#define AUDIO_VOTING_LANGUAGE_SELECTION_HXX

#include <glibmm.h>
#include <sigc++/sigc++.h>
#include <Voting/Election/election-schema.hxx>
#include <System/SafeThread.hxx>
#include "AbledController.hxx"
#include "AudioVotingResources.hxx"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Audio
        {
            namespace TextToSpeech
            {
                class ITextToSpeech;
            }
        }
    }

    namespace SAES
    {
        namespace Resources
        {
            class ElectionAudioResourcesManager;
        }

        namespace Voting
        {
            class ElectoralConfiguration;
        }

        namespace Operation
        {
            namespace AudioVoting
            {

                struct by_precedence {
                    bool operator()(SAES::Voting::Election::Language const &a, SAES::Voting::Election::Language const &b) {
                        return a.getPrecedence() < b.getPrecedence();
                    }
                };

                /**
                 * @class  AudioVotingLanguageSelection
                 * @brief  Select the language of the voting experience.
                 * @author manuel.arguelles

                 * @date   Oct 11, 2017
                 */
                class AudioVotingLanguageSelection : public System::SafeThread
                {
                    public:

                        typedef sigc::signal<void> LanguageSelectedSignal;
                        typedef std::map< std::string, std::string > LanguagesMap;

                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 11, 2017
                         * @param  textToSpeech The text to speech engine. The
                         * pointer is not owned by the class.
                         */
                        explicit AudioVotingLanguageSelection(
                            System::Audio::TextToSpeech::ITextToSpeech* textToSpeech);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 11, 2017
                         */
                        ~AudioVotingLanguageSelection();

                    public:

                        /**
                         * @brief  Starts the language selection process.
                         * @author manuel.arguelles
                         * @date   Oct 12, 2017
                         */
                        void start();

                        /**
                         * @brief  Emitted when the language is selected.
                         * @author manuel.arguelles
                         * @date   Oct 12, 2017
                         * @return The language selected signal.
                         */
                        LanguageSelectedSignal signalLanguageSelected();

                    private:

                        /**
                         * @brief  Language selection process entry point.
                         * @author manuel.arguelles
                         * @date   Oct 12, 2017
                         * @return True while the selection is in progress, false when not.
                         */
                        virtual bool process();

                        /**
                         * @brief  Called before starting the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         */
                        virtual void preStart();

                        /**
                         * @brief  Called right after the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         */
                        virtual void postStop();

                        /**
                         * @brief  Sorts current languages.
                         * @author eduardo.fernandez
                         * @date   Nov 16, 2017
                         */
                        void setLanguagesMap();

                        /**
                         * @brief  Goes to the next selection.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        void nextSelection();

                        /**
                         * @brief  Goes to the previous selection.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        void previousSelection();

                        /**
                         * @brief  Plays the confirmation of selection.
                         * @author eduardo.fernandez
                         * @date   Nov 16, 2017
                         */
                        void confirmSelection();

                        /**
                         * @brief  Gets the current region name.
                         * @author eduardo.fernandez
                         * @date   Nov 17, 2017
                         * @param  electoralConfiguration A pointer to the electoral configuration
                         */
                        std::string getRegionName(Smartmatic::SAES::Voting::ElectoralConfiguration *electoralConfiguration);

                        /**
                         * @brief  Gets the current election's welcome message.
                         * @author eduardo.fernandez
                         * @date   Nov 17, 2017
                         */
                        std::string getWelcomeMessage();

                        /**
                         * @brief  Plays the current selected language.
                         * @author eduardo.fernandez
                         * @date   Nov 16, 2017
                         */
                        void playSelection();

                        /**
                         * @brief  Handles right press user event.
                         * @author eduardo.fernandez
                         * @date   Nov 17, 2017
                         */
                        void handleRightPress();

                        /**
                         * @brief  Handles left press user event.
                         * @author eduardo.fernandez
                         * @date   Nov 17, 2017
                         */
                        void handleLeftPress();

                        /**
                         * @brief  Handles down press user event.
                         * @author eduardo.fernandez
                         * @date   Nov 17, 2017
                         */
                        void handleDownPress();

                        /**
                         * @brief  Handles up press user event.
                         * @author eduardo.fernandez
                         * @date   Nov 17, 2017
                         */
                        void handleUpPress();

                        /**
                         * @brief  Handles select press user event.
                         * @author eduardo.fernandez
                         * @date   Nov 17, 2017
                         */
                        void handleSelectPress();

                        /**
                         * @brief  Handles help button press event.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         */
                        void handleHelpPress();

                        /**
                         * @brief  Handles speed button press event.
                         * @author manuel.arguelles
                         * @date   Mar 28, 2018
                         */
                        void handleSpeedPress();

                        /**
                         * @brief  Finalizes the selection process.
                         * @author eduardo.fernandez
                         * @date   Nov 16, 2017
                         */
                        void end();

                        /**
                         * @brief  Plays the initial message.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         */
                        void playInitialMessage();

                    private:

                        enum Stage
                        {
                            INITIAL_STAGE,
                            INITIAL_WAIT_STAGE,
                            LANGUAGE_SELECT_STAGE,
                            CONFIRMATION_STAGE,
                            WAIT_USER_STAGE
                        };

                    private:

                        System::Audio::TextToSpeech::ITextToSpeech* const     m_textToSpeech;
                        LanguageSelectedSignal                                m_languageSelectedSignal;
                        std::list<SAES::Voting::Election::Language>           m_languages;
                        LanguagesMap                                          m_languagesMap;
                        LanguagesMap::iterator                                m_currentLanguage;
                        SAES::Resources::ElectionAudioResourcesManager*       m_electoralAudioResources;
                        AudioVotingResources                                  m_generalAudioResources;
                        std::list<SAES::Voting::Election::Language>           m_options;
                        std::list<SAES::Voting::Election::Language>::iterator m_currentOption;
                        AbledController                                       m_abled;
                        Stage                                                 m_currentStage;
                        Glib::Timer                                           m_idleTime;
                        sigc::connection                                      m_selectConnection;
                        sigc::connection                                      m_helpConnection;
                        sigc::connection                                      m_leftConnection;
                        sigc::connection                                      m_rightConnection;
                        sigc::connection                                      m_downConnection;
                        sigc::connection                                      m_upConnection;
                        sigc::connection                                      m_moreSpeedConnection;
                        sigc::connection                                      m_lessSpeedConnection;
                        bool                                                  m_canInstructionsBeInterrupted;
                        bool                                                  m_canWelcomeBeInterrupted;
                        static Smartmatic::Log::ISMTTLog*                     m_logger;
                };

            } /* namespace AudioVoting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* AUDIO_VOTING_LANGUAGE_SELECTION_HXX */

