/**
 * @file   AudioVotingPrincipalSelection.cxx
 * @brief  Implementation of AudioVotingPrincipalSelection class.
 * @author manuel.arguelles
 * @date   Oct 14, 2017
 */

#include "AudioVotingPrincipalSelection.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Resources/ElectionAudioResourcesManager.hxx>
#include <System/Audio/TextToSpeech/ITextToSpeech.hxx>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Voting/Election/election-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <algorithm>

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Audio::TextToSpeech;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const unsigned int IDLE_INTERVAL_S                      = 10;
static const unsigned int INTERVAL_MS                          = 10;
static const unsigned int SLEEP_AFTER_STOP_US                  = 200000;
static const std::string  GENERAL_INSTRUCTIONS_MESSAGE_KEY     = "smartmatic.saes.operation.audiovoting.AudioVotingPrincipalSelection.GeneralInstructionsMessage";
static const std::string  PRESELECTION_MESSAGE_KEY             = "smartmatic.saes.operation.audiovoting.AudioVotingPrincipalSelection.PreselectionMessage";
static const std::string  LIST_VOTE_MESSAGE_KEY                = "smartmatic.saes.operation.AudioVoting.AudioVotingPrincipalSelection.ListVoteMessage";
static const std::string  SELECTION_INSTRUCTIONS_MESSAGE_KEY   = "smartmatic.saes.operation.audiovoting.AudioVotingPrincipalSelection.SelectionInstructionsMessage";
static const std::string  DESELECTION_INSTRUCTIONS_MESSAGE_KEY = "smartmatic.saes.operation.audiovoting.AudioVotingPrincipalSelection.DeselectionInstructionsMessage";
static const std::string  DESELECTION_MESSAGE_KEY              = "smartmatic.saes.operation.audiovoting.AudioVotingPrincipalSelection.DeselectionMessage";
static const std::string  SELECTION_MESSAGE_KEY                = "smartmatic.saes.operation.audiovoting.AudioVotingPrincipalSelection.SelectionMessage";

ISMTTLog* AudioVotingPrincipalSelection::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.AudioVoting.AudioVotingPrincipalSelection",
    GETTEXT_PACKAGE);

AudioVotingPrincipalSelection::AudioVotingPrincipalSelection(ITextToSpeech* textToSpeech)
    : m_textToSpeech(textToSpeech)
    , m_electoralAudioResources(ElectionAudioResourcesManager::getInstance())
    , m_contest()
    , m_canInstructionsBeInterrupted(false)
    , m_optionPlayed(false)
{
}

AudioVotingPrincipalSelection::~AudioVotingPrincipalSelection()
{
}

void AudioVotingPrincipalSelection::start(Contest* contest, const std::string& partyCode)
{
    m_contest = contest;
    m_partyCode = partyCode;
    m_canInstructionsBeInterrupted = false;
    m_currentStage = INITIAL_STAGE;

    setInitialOption();

    SafeThread::start(100);
}

void AudioVotingPrincipalSelection::preStart()
{
    std::string currentMachineModel;
    OptParser::getInstance()->getArg( OptParser::MACHINE_MODEL, currentMachineModel );

    m_selectConnection = m_abled.signalSelectButton().connect(
        sigc::mem_fun(*this, &AudioVotingPrincipalSelection::handleSelectPress));
    m_helpConnection = m_abled.signalHelpButton().connect(
        sigc::mem_fun(*this, &AudioVotingPrincipalSelection::handleHelpPress));
    m_leftConnection = m_abled.signalLeftButton().connect(
        sigc::mem_fun(*this, &AudioVotingPrincipalSelection::handleLeftPress));
    m_rightConnection = m_abled.signalRightButton().connect(
        sigc::mem_fun(*this, &AudioVotingPrincipalSelection::handleRightPress));
    m_upConnection = m_abled.signalUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingPrincipalSelection::handleUpPress));
    m_downConnection = m_abled.signalDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingPrincipalSelection::handleDownPress));
    m_moreSpeedConnection = m_abled.signalSpeedUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingPrincipalSelection::handleSpeedPress));
    m_lessSpeedConnection = m_abled.signalSpeedDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingPrincipalSelection::handleSpeedPress));
}

void AudioVotingPrincipalSelection::postStop()
{
    m_selectConnection.disconnect();
    m_helpConnection.disconnect();
    m_leftConnection.disconnect();
    m_rightConnection.disconnect();
    m_upConnection.disconnect();
    m_downConnection.disconnect();
    m_moreSpeedConnection.disconnect();
    m_lessSpeedConnection.disconnect();
}

bool AudioVotingPrincipalSelection::process()
{
    if (m_currentStage == INITIAL_STAGE)
    {
        playInstructions();
        m_currentStage = SELECTION_STAGE;
        m_idleTime.reset();
    }
    else if (m_currentStage == SELECTION_STAGE && m_textToSpeech->isSpeaking())
    {
        m_idleTime.reset();
    }
    else if (m_currentStage == RETURN_STAGE)
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(*this, &AudioVotingPrincipalSelection::returnToPartySelection),
            INTERVAL_MS);
    }
    else if (m_currentStage == END_STAGE)
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(*this, &AudioVotingPrincipalSelection::end),
            INTERVAL_MS);
    }

    if (m_idleTime.elapsed() > IDLE_INTERVAL_S)
    {
        m_currentStage = INITIAL_STAGE;
    }

    return true;
}

void AudioVotingPrincipalSelection::clearSelectionsArray()
{
    m_selected.clear();
}

void AudioVotingPrincipalSelection::playInstructions()
{
    std::string ballotNameKey = (*m_currentOption)->getBallot_name_key();
    std::string ballotName = m_electoralAudioResources->getResource(ballotNameKey);
    std::string instructions;

    m_optionPlayed = false;

    instructions = Glib::ustring::compose(
        m_generalAudioResources.getResource(GENERAL_INSTRUCTIONS_MESSAGE_KEY),
        ballotName);

    m_logger->Debug(instructions);
    m_textToSpeech->stop();
    m_textToSpeech->speak(instructions);
    m_textToSpeech->waitWhileSpeaking();
    m_canInstructionsBeInterrupted = true;
}

void AudioVotingPrincipalSelection::setInitialOption()
{
    ElectoralConfiguration* electoralConfiguration = \
        MachineOperationManager::getInstance()->getElectoralConfiguration();

    m_ballotList = electoralConfiguration->getEligibleOptionsListByContestParty(m_contest->getCode(), m_partyCode);
    m_principalList =
        electoralConfiguration->getEligibleOptionsPrincipalByContestParty(m_contest->getCode(), m_partyCode);

    m_logger->Debug("List options: " + StringUtils::intToString(m_ballotList.size()));
    m_logger->Debug("Candidate options: " + StringUtils::intToString(m_principalList.size()));

    m_allOptions = m_ballotList;
    std::copy(m_principalList.begin(), m_principalList.end(), std::back_inserter(m_allOptions));
    m_logger->Debug("All options: " + StringUtils::intToString(m_allOptions.size()));

    m_currentOption = m_allOptions.begin();
}

void AudioVotingPrincipalSelection::end()
{
    stop();
    m_doneSignal.emit(!m_selected.empty());
}

void AudioVotingPrincipalSelection::nextSelection()
{
    if (m_currentOption == m_allOptions.end())
    {
        m_currentOption = m_allOptions.begin();
    }
    else
    {
        ++m_currentOption;

        if (m_currentOption == m_allOptions.end())
        {
            m_currentOption = m_allOptions.begin();
        }
    }

    playSelection();
}

void AudioVotingPrincipalSelection::previousSelection()
{
    if (m_currentOption == m_allOptions.begin())
    {
        m_currentOption = m_allOptions.end();
    }

    --m_currentOption;

    playSelection();
}

bool AudioVotingPrincipalSelection::isSelected()
{
    return (std::find(m_selected.begin(), m_selected.end(), *m_currentOption) != m_selected.end());
}

void AudioVotingPrincipalSelection::deselectAllOptions()
{
    std::list<Eligible_option*>::const_iterator it = m_selected.begin();

    for (; it != m_selected.end(); ++it)
    {
        m_optionSelectedSignal.emit((*it)->getBallot_options().getBallot_option().begin()->getCode(), false);
    }

    m_selected.clear();
}

void AudioVotingPrincipalSelection::selectOption()
{
    Ballot_option::CodeType ballotOptionCode =
        (*m_currentOption)->getBallot_options().getBallot_option().begin()->getCode();
    std::string confirmation = "";

    if (isSelected())
    {
        m_selected.erase(std::find(m_selected.begin(), m_selected.end(), *m_currentOption));
        m_logger->Debug("Deselected: " + ballotOptionCode);

        confirmation = m_generalAudioResources.getResource(DESELECTION_MESSAGE_KEY);

        m_optionSelectedSignal.emit(ballotOptionCode, false);
    }
    else
    {
        m_logger->Debug("Selected: " + ballotOptionCode);

        confirmation = m_generalAudioResources.getResource(SELECTION_MESSAGE_KEY);

        m_selected.push_back(*m_currentOption);
        m_optionSelectedSignal.emit(ballotOptionCode, true);
    }

    m_logger->Debug(confirmation);
    m_textToSpeech->stop();
    m_textToSpeech->speak(confirmation);
}

void AudioVotingPrincipalSelection::playSelection()
{
    std::string ballotOptionName = m_electoralAudioResources->getResource((*m_currentOption)->getBallot_name_key());
    std::string deselectionInstructionsMessage = m_generalAudioResources.getResource(DESELECTION_INSTRUCTIONS_MESSAGE_KEY);
    std::string selectionInstructionsMessage = m_generalAudioResources.getResource(SELECTION_INSTRUCTIONS_MESSAGE_KEY);

    if ((*m_currentOption)->getParent_code().empty())
    {
        ballotOptionName = Glib::ustring::compose(m_generalAudioResources.getResource(LIST_VOTE_MESSAGE_KEY), ballotOptionName);
    }

    std::string audioText = Glib::ustring::compose(m_generalAudioResources.getResource(PRESELECTION_MESSAGE_KEY), ballotOptionName);
    audioText += " ";
    audioText += isSelected() ? deselectionInstructionsMessage : selectionInstructionsMessage;

    m_logger->Debug(audioText);
    m_textToSpeech->stop();
    Glib::usleep(SLEEP_AFTER_STOP_US);
    m_textToSpeech->speak(audioText);
    m_optionPlayed = true;
}

void AudioVotingPrincipalSelection::returnToPartySelection()
{
    stop();
    m_returnRequestedSignal.emit();
}

AudioVotingPrincipalSelection::OptionSelectedSignal AudioVotingPrincipalSelection::signalOptionSelected()
{
    return m_optionSelectedSignal;
}

AudioVotingPrincipalSelection::DoneSignal AudioVotingPrincipalSelection::signalDone()
{
    return m_doneSignal;
}

AudioVotingPrincipalSelection::ReturnRequestedSignal AudioVotingPrincipalSelection::signalReturnRequested()
{
    return m_returnRequestedSignal;
}

void AudioVotingPrincipalSelection::handleLeftPress()
{
    m_idleTime.reset();

    if  (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        previousSelection();
    }
}

void AudioVotingPrincipalSelection::handleHelpPress()
{
    m_idleTime.reset();
    m_currentStage = INITIAL_STAGE;
}

void AudioVotingPrincipalSelection::handleRightPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        nextSelection();
    }
}

void AudioVotingPrincipalSelection::handleSelectPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        selectOption();
    }
}

void AudioVotingPrincipalSelection::handleUpPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        m_currentStage = RETURN_STAGE;
    }
}

void AudioVotingPrincipalSelection::handleDownPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        m_currentStage = END_STAGE;
    }
}

void AudioVotingPrincipalSelection::handleSpeedPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE)
    {
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_currentStage = INITIAL_STAGE;
    }
    else if (m_currentStage == SELECTION_STAGE && m_optionPlayed)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE && !m_optionPlayed)
    {
        m_currentStage = INITIAL_STAGE;
    }
}
