/**
 * @file   AudioVotingPrincipalSelection.hxx
 * @brief  Declaration of AudioVotingPrincipalSelection class.
 * @author manuel.arguelles
 * @date   Oct 14, 2017
 */

#ifndef AUDIO_VOTING_PRINCIPAL_SELECTION_HXX
#define AUDIO_VOTING_PRINCIPAL_SELECTION_HXX

#include "AbledController.hxx"
#include "AudioVotingResources.hxx"
#include <System/SafeThread.hxx>
#include <string>
#include <list>
#include <sigc++/sigc++.h>
#include <glibmm.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Audio
        {
            namespace TextToSpeech
            {
                class ITextToSpeech;
            }
        }
    }

    namespace SAES
    {
        namespace Voting
        {
            namespace Election
            {
                class Eligible_option;
                class Contest;
            }
        }

        namespace Resources
        {
            class ElectionAudioResourcesManager;
        }

        namespace Operation
        {
            namespace AudioVoting
            {

                /**
                 * @class  AudioVotingPrincipalSelection
                 * @brief  Class to handle selection of principal candidates.
                 * @author manuel.arguelles
                 * @date   Oct 14, 2017
                 */
                class AudioVotingPrincipalSelection : public System::SafeThread
                {
                    public:

                        typedef sigc::signal<void, const std::string&, bool> OptionSelectedSignal;
                        typedef sigc::signal<void, bool> DoneSignal;
                        typedef sigc::signal<void> ReturnRequestedSignal;

                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         * @param  textToSpeech The text to speech engien to use.
                         * Ownership of the pointer is not transferred.
                         */
                        explicit AudioVotingPrincipalSelection(System::Audio::TextToSpeech::ITextToSpeech* textToSpeech);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        ~AudioVotingPrincipalSelection();

                    public:

                        /**
                         * @brief  Starts the selection process.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         * @param  contest   The selected contest.
                         * @param  partyCode The selected party code.
                         * @remark Ownership of contest pointer is not transferred.
                         */
                        void start(SAES::Voting::Election::Contest* contest, const std::string& partyCode);

                        /**
                         * @brief  Clears current selections array.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void clearSelectionsArray();

                        /**
                         * @brief  Deselects currently selected options.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void deselectAllOptions();

                        /**
                         * @brief  Emitted when a selection is selected or
                         * deselected.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        OptionSelectedSignal signalOptionSelected();

                        /**
                         * @brief  Emitted when the selection process is
                         * completed.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        DoneSignal signalDone();

                        /**
                         * @brief  Emitted when the user presses the back button.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         * @return The return requested signal.
                         */
                        ReturnRequestedSignal signalReturnRequested();

                    private:

                        /**
                         * @brief  Main loop of the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        virtual bool process();

                        /**
                         * @brief  Called before starting the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        virtual void preStart();

                        /**
                         * @brief  Called right after the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        virtual void postStop();

                        /**
                         * @brief  Finalizes the selection process.
                         * @author manuel.arguelles
                         * @date   Oct 14, 2017
                         */
                        void end();

                        /**
                         * @brief  Returns to the party selection stage.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void returnToPartySelection();

                        /**
                         * @brief  Plays the general instructions.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void playInstructions();

                        /**
                         * @brief  Goes to the next selection.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void nextSelection();

                        /**
                         * @brief  Goes to the previous selection.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void previousSelection();

                        /**
                         * @brief  Selects current option.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void selectOption();

                        /**
                         * @brief  Plays current option.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void playSelection();

                        /**
                         * @brief  Gets whether the current option is selected.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         * @return Whether the current option is selected.
                         */
                        bool isSelected();

                        /**
                         * @brief  Sets the initial option.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void setInitialOption();

                        /**
                         * @brief  Handles the left button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleLeftPress();

                        /**
                         * @brief  Handles the help button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleHelpPress();

                        /**
                         * @brief  Handles the right button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleRightPress();

                        /**
                         * @brief  Handles the select button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleSelectPress();

                        /**
                         * @brief  Handles the up button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleUpPress();

                        /**
                         * @brief  Handles the down button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleDownPress();

                        /**
                         * @brief  Handles the speed button press.
                         * @author manuel.arguelles
                         * @date   Mar 28, 2018
                         */
                        void handleSpeedPress();

                    private:

                        enum Stage
                        {
                            INITIAL_STAGE,
                            SELECTION_STAGE,
                            RETURN_STAGE,
                            END_STAGE
                        };

                    private:

                        System::Audio::TextToSpeech::ITextToSpeech*                   m_textToSpeech;
                        SAES::Resources::ElectionAudioResourcesManager*               m_electoralAudioResources;
                        AudioVotingResources                                          m_generalAudioResources;
                        sigc::connection                                              m_abledConnection;
                        std::list<SAES::Voting::Election::Eligible_option*>           m_ballotList;
                        std::list<SAES::Voting::Election::Eligible_option*>           m_principalList;
                        std::list<SAES::Voting::Election::Eligible_option*>           m_allOptions;
                        std::list<SAES::Voting::Election::Eligible_option*>::iterator m_currentOption;
                        std::list<SAES::Voting::Election::Eligible_option*>           m_selected;
                        SAES::Voting::Election::Contest*                              m_contest;
                        std::string                                                   m_partyCode;
                        OptionSelectedSignal                                          m_optionSelectedSignal;
                        DoneSignal                                                    m_doneSignal;
                        ReturnRequestedSignal                                         m_returnRequestedSignal;
                        Glib::Timer                                                   m_idleTime;
                        AbledController                                               m_abled;
                        Stage                                                         m_currentStage;
                        sigc::connection                                              m_selectConnection;
                        sigc::connection                                              m_helpConnection;
                        sigc::connection                                              m_leftConnection;
                        sigc::connection                                              m_rightConnection;
                        sigc::connection                                              m_upConnection;
                        sigc::connection                                              m_downConnection;
                        sigc::connection                                              m_moreSpeedConnection;
                        sigc::connection                                              m_lessSpeedConnection;
                        bool                                                          m_canInstructionsBeInterrupted;
                        bool                                                          m_optionPlayed;
                        static Smartmatic::Log::ISMTTLog*                             m_logger;
                };

            } /* namespace AudioVoting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* AUDIO_VOTING_PRINCIPAL_SELECTION_HXX */

