/**
 * @file   AudioVotingStandInSelection.cxx
 * @brief  Implementation of AudioVotingStandInSelection class.
 * @author eduardo.fernandez
 * @date   Oct 31, 2017
 */

#include "AudioVotingStandInSelection.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Resources/ElectionAudioResourcesManager.hxx>
#include <System/Audio/TextToSpeech/ITextToSpeech.hxx>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Voting/Election/election-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <algorithm>

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Audio::TextToSpeech;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const unsigned int IDLE_INTERVAL_S                      = 10;
static const unsigned int INTERVAL_MS                          = 10;
static const unsigned int SLEEP_AFTER_STOP_US                  = 200000;
static const std::string  GENERAL_INSTRUCTIONS_MESSAGE_KEY     = "smartmatic.saes.operation.audiovoting.AudioVotingStandInSelection.GeneralInstructionsMessage";
static const std::string  PRESELECTION_MESSAGE_KEY             = "smartmatic.saes.operation.audiovoting.AudioVotingStandInSelection.PreselectionMessage";
static const std::string  SELECTION_INSTRUCTIONS_MESSAGE_KEY   = "smartmatic.saes.operation.audiovoting.AudioVotingStandInSelection.SelectionInstructionsMessage";
static const std::string  DESELECTION_INSTRUCTIONS_MESSAGE_KEY = "smartmatic.saes.operation.audiovoting.AudioVotingStandInSelection.DeselectionInstructionsMessage";
static const std::string  DESELECTION_MESSAGE_KEY              = "smartmatic.saes.operation.audiovoting.AudioVotingStandInSelection.DeselectionMessage";
static const std::string  SELECTION_MESSAGE_KEY                = "smartmatic.saes.operation.audiovoting.AudioVotingStandInSelection.SelectionMessage";
static const std::string  VALIDATION_MESSAGE_KEY               = "smartmatic.saes.operation.audiovoting.AudioVotingStandInSelection.ValidationMessage";

ISMTTLog* AudioVotingStandInSelection::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.AudioVoting.AudioVotingStandInSelection",
    GETTEXT_PACKAGE);

AudioVotingStandInSelection::AudioVotingStandInSelection(ITextToSpeech* textToSpeech)
    : m_textToSpeech(textToSpeech)
    , m_electoralAudioResources(ElectionAudioResourcesManager::getInstance())
    , m_contest()
    , m_anyPrincipalSelected(false)
    , m_canInstructionsBeInterrupted(false)
    , m_optionPlayed(false)
{
}

AudioVotingStandInSelection::~AudioVotingStandInSelection()
{
}

void AudioVotingStandInSelection::start(Contest* contest, const std::string& partyCode, bool anySelection)
{
    m_contest = contest;
    m_partyCode = partyCode;
    m_anyPrincipalSelected = anySelection;
    m_canInstructionsBeInterrupted = false;

    setInitialOption();

    if (!m_standInList.empty())
    {
        m_currentStage = INITIAL_STAGE;
    }
    else if (m_anyPrincipalSelected)
    {
        m_currentStage = END_STAGE;
    }
    else
    {
        std::string audioText = m_generalAudioResources.getResource(VALIDATION_MESSAGE_KEY);

        m_logger->Debug(audioText);
        m_textToSpeech->stop();
        m_textToSpeech->speak(audioText);
        m_textToSpeech->waitWhileSpeaking();

        m_currentStage = RETURN_STAGE;
    }

    SafeThread::start(100);
}

void AudioVotingStandInSelection::preStart()
{
    std::string currentMachineModel;
    OptParser::getInstance()->getArg( OptParser::MACHINE_MODEL, currentMachineModel );

    m_selectConnection = m_abled.signalSelectButton().connect(
        sigc::mem_fun(*this, &AudioVotingStandInSelection::handleSelectPress));
    m_helpConnection = m_abled.signalHelpButton().connect(
        sigc::mem_fun(*this, &AudioVotingStandInSelection::handleHelpPress));
    m_leftConnection = m_abled.signalLeftButton().connect(
        sigc::mem_fun(*this, &AudioVotingStandInSelection::handleLeftPress));
    m_rightConnection = m_abled.signalRightButton().connect(
        sigc::mem_fun(*this, &AudioVotingStandInSelection::handleRightPress));
    m_upConnection = m_abled.signalUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingStandInSelection::handleUpPress));
    m_downConnection = m_abled.signalDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingStandInSelection::handleDownPress));
    m_moreSpeedConnection = m_abled.signalSpeedUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingStandInSelection::handleSpeedPress));
    m_lessSpeedConnection = m_abled.signalSpeedDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingStandInSelection::handleSpeedPress));
}

void AudioVotingStandInSelection::postStop()
{
    m_selectConnection.disconnect();
    m_helpConnection.disconnect();
    m_leftConnection.disconnect();
    m_rightConnection.disconnect();
    m_upConnection.disconnect();
    m_downConnection.disconnect();
    m_moreSpeedConnection.disconnect();
    m_lessSpeedConnection.disconnect();
}

bool AudioVotingStandInSelection::process()
{
    if (m_currentStage == INITIAL_STAGE)
    {
        playInstructions();
        m_currentStage = SELECTION_STAGE;
        m_idleTime.reset();
    }
    else if (m_currentStage == SELECTION_STAGE && m_textToSpeech->isSpeaking())
    {
        m_idleTime.reset();
    }
    else if (m_currentStage == RETURN_STAGE)
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(*this, &AudioVotingStandInSelection::returnToPrincipalSelection),
            INTERVAL_MS);
    }
    else if (m_currentStage == END_STAGE)
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(*this, &AudioVotingStandInSelection::end),
            INTERVAL_MS);
    }

    if (m_idleTime.elapsed() > IDLE_INTERVAL_S)
    {
        m_currentStage = INITIAL_STAGE;
    }

    return true;
}

void AudioVotingStandInSelection::playInstructions()
{
    std::string ballotNameKey = (*m_currentOption)->getBallot_name_key();
    std::string ballotName = m_electoralAudioResources->getResource(ballotNameKey);
    std::string instructions;

    m_optionPlayed = false;

    instructions = Glib::ustring::compose(
        m_generalAudioResources.getResource(GENERAL_INSTRUCTIONS_MESSAGE_KEY),
        ballotName);

    m_logger->Debug(instructions);
    m_textToSpeech->stop();
    m_textToSpeech->speak(instructions);
    m_textToSpeech->waitWhileSpeaking();
    m_canInstructionsBeInterrupted = true;
}

void AudioVotingStandInSelection::setInitialOption()
{
    ElectoralConfiguration* electoralConfiguration = \
        MachineOperationManager::getInstance()->getElectoralConfiguration();

    m_standInList = electoralConfiguration->getEligibleOptionsStandInByContestParty(m_contest->getCode(), m_partyCode);
    m_logger->Debug("Candidate options: " + StringUtils::intToString(m_standInList.size()));
    m_currentOption = m_standInList.begin();
}

void AudioVotingStandInSelection::end()
{
    stop();
    m_doneSignal.emit();
    m_selected.clear();
}

void AudioVotingStandInSelection::nextSelection()
{
    if (m_currentOption == m_standInList.end())
    {
        m_currentOption = m_standInList.begin();
    }
    else
    {
        ++m_currentOption;

        if (m_currentOption == m_standInList.end())
        {
            m_currentOption = m_standInList.begin();
        }
    }

    playSelection();
}

void AudioVotingStandInSelection::previousSelection()
{
    if (m_currentOption == m_standInList.begin())
    {
        m_currentOption = m_standInList.end();
    }

    --m_currentOption;

    playSelection();
}

bool AudioVotingStandInSelection::isSelected()
{
    return (std::find(m_selected.begin(), m_selected.end(), *m_currentOption) != m_selected.end());
}

void AudioVotingStandInSelection::endStandInSelection()
{
    if (m_selected.empty() && !m_anyPrincipalSelected)
    {
        std::string audioText = m_generalAudioResources.getResource(VALIDATION_MESSAGE_KEY);

        m_logger->Debug(audioText);
        m_textToSpeech->stop();
        m_textToSpeech->speak(audioText);
    }
    else
    {
        m_currentStage = END_STAGE;
    }
}

void AudioVotingStandInSelection::deselectAllOptions()
{
    std::list<Eligible_option*>::const_iterator it = m_selected.begin();

    for (; it != m_selected.end(); ++it)
    {
        m_optionSelectedSignal.emit((*it)->getBallot_options().getBallot_option().begin()->getCode(), false);
    }

    m_selected.clear();
}

void AudioVotingStandInSelection::selectOption()
{
    Ballot_option::CodeType ballotOptionCode = (*m_currentOption)->getBallot_options().getBallot_option().begin()->getCode();
    std::string confirmation = "";

    if (isSelected())
    {
        m_selected.erase(std::find(m_selected.begin(), m_selected.end(), *m_currentOption));
        m_logger->Debug("Deselected: " + ballotOptionCode);
        confirmation = m_generalAudioResources.getResource(DESELECTION_MESSAGE_KEY);
        m_optionSelectedSignal.emit(ballotOptionCode, false);
    }
    else
    {
        m_logger->Debug("Selected: " + ballotOptionCode);
        confirmation = m_generalAudioResources.getResource(SELECTION_MESSAGE_KEY);
        m_selected.push_back(*m_currentOption);
        m_optionSelectedSignal.emit(ballotOptionCode, true);
    }

    m_logger->Debug(confirmation);
    m_textToSpeech->stop();
    m_textToSpeech->speak(confirmation);
}

void AudioVotingStandInSelection::playSelection()
{
    std::string ballotOptionName = m_electoralAudioResources->getResource((*m_currentOption)->getBallot_name_key());
    std::string deselectionInstructionsMessage = m_generalAudioResources.getResource(DESELECTION_INSTRUCTIONS_MESSAGE_KEY);
    std::string selectionInstructionsMessage = m_generalAudioResources.getResource(SELECTION_INSTRUCTIONS_MESSAGE_KEY);

    std::string audioText = Glib::ustring::compose(m_generalAudioResources.getResource(PRESELECTION_MESSAGE_KEY), ballotOptionName);
    audioText += " ";
    audioText += isSelected() ? deselectionInstructionsMessage : selectionInstructionsMessage;

    m_logger->Debug(audioText);
    m_textToSpeech->stop();
    Glib::usleep(SLEEP_AFTER_STOP_US);
    m_textToSpeech->speak(audioText);
    m_optionPlayed = true;
}

void AudioVotingStandInSelection::returnToPrincipalSelection()
{
    stop();
    m_returnRequestedSignal.emit();
}

AudioVotingStandInSelection::OptionSelectedSignal AudioVotingStandInSelection::signalOptionSelected()
{
    return m_optionSelectedSignal;
}

AudioVotingStandInSelection::DoneSignal AudioVotingStandInSelection::signalDone()
{
    return m_doneSignal;
}

AudioVotingStandInSelection::ReturnRequestedSignal AudioVotingStandInSelection::signalReturnRequested()
{
    return m_returnRequestedSignal;
}

void AudioVotingStandInSelection::handleLeftPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        previousSelection();
    }
}

void AudioVotingStandInSelection::handleHelpPress()
{
    m_idleTime.reset();
    m_currentStage = INITIAL_STAGE;
}

void AudioVotingStandInSelection::handleRightPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        nextSelection();
    }
}

void AudioVotingStandInSelection::handleSelectPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        selectOption();
    }
}

void AudioVotingStandInSelection::handleUpPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        m_currentStage = RETURN_STAGE;
    }
}

void AudioVotingStandInSelection::handleDownPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        endStandInSelection();
    }
}

void AudioVotingStandInSelection::handleSpeedPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE)
    {
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_currentStage = INITIAL_STAGE;
    }
    else if (m_currentStage == SELECTION_STAGE && m_optionPlayed)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE && !m_optionPlayed)
    {
        m_currentStage = INITIAL_STAGE;
    }
}
