/*
 * VoteFromSafeBuffer.cxx
 *
 *  Created on: Aug 5, 2015
 *      Author: soto
 */

#include "VoteFromSafeBuffer.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Exception/CryptoException.hxx>
#include <System/Exception/SignedException.hxx>
#include <Voting/Vote/VoteWrapper.hxx>
#include <Exceptions/Voting/Vote/QRVoteInvalidFormatException.h>
#include <Exceptions/Voting/Vote/QRVoteSignatureException.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::Log;

ISMTTLog * VoteFromSafeBuffer::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Voting.VoteFromSafeBuffer", GETTEXT_PACKAGE );

VoteFromSafeBuffer::VoteFromSafeBuffer ( ElectoralConfiguration * electoralConfiguration )
    : electoralConfiguration ( electoralConfiguration )
{

}

VoteFromSafeBuffer::~VoteFromSafeBuffer()
{

}

VoteFromSafeBuffer::VoteFromSafeBufferResult VoteFromSafeBuffer::getVote ( SafeBuffer * safeBuffer, Vote::Vote *& vote )
{
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.GettingVoteFromSafeBuffer" ) );
    logger->Debug ( "Start vote from safe buffer" );

    vote = NULL;

    VoteFromSafeBuffer::VoteFromSafeBufferResult result = VOTE_FROM_BUFFER_SUCCESS;

    try
    {
        Vote::Vote voteTmp;

        if ( safeBuffer != NULL && Vote::VoteWrapper::unpackVoteFromQR ( electoralConfiguration, *safeBuffer, voteTmp, false ) )
        {
            vote = new Vote::Vote ( voteTmp );
            VotingDevice::Voting_device  vd = electoralConfiguration->getVotingDevice ()->getFirstVotingDevice ();

            if ( !Vote::VoteWrapper::validateLogicalVoterGroupVote ( electoralConfiguration, *vote ) )
            {
                logger->Debug ( "Error logical voter group of vote" );
                logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorLogicalVoterGroupInVote" ) );
                result = VOTE_FROM_BUFFER_INVALID_LVG;
            }
            else if ( !Vote::VoteWrapper::validateVotingDeviceCodeVote ( electoralConfiguration, *vote ) )
            {
                logger->Debug ( "Error voting device code of vote" );
                logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorVotingDeviceInVote" ) );
                result = VOTE_FROM_BUFFER_INVALID_VOTING_DEVICE;
            }
            else if ( !Vote::VoteWrapper::validateCbcMac ( *vote ) )
            {
                logger->Debug ( "Error code of vote" );
                logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorVoteCode" ) );
                result = VOTE_FROM_BUFFER_INVALID_PROCESS_INSTANCE;
            }
        }
        else
        {
            result = VOTE_FROM_BUFFER_DECOMPRESSION_ERROR;
            logger->Debug ( "Error Buffer decompression" );
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorUnpackVote" ) );
        }
    }
    catch ( const ::xsd::cxx::tree::exception< char > & e )
    {
        result = VOTE_FROM_BUFFER_UNEXPECTED_ERROR;
        std::stringstream ss;
        ss << __func__ << " " << e;
        logger->Debug ( ss.str() );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorVoteData" ) );
    }
    catch ( Exception::SignedException & ex )
    {
        result = VOTE_FROM_BUFFER_SIGNATURE_ERROR;
        std::stringstream ss;
        ss << __func__ << " " << ex.what();
        logger->Debug ( ss.str() );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorVoteData" ) );
    }
    catch ( Exception::CryptoException & cex )
    {
        result = VOTE_FROM_BUFFER_DECRYPTO_ERROR;
        std::stringstream ss;
        ss << __func__ << " " << cex.what();
        logger->Debug ( ss.str() );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorVoteData" ) );
    }
    catch ( Smartmatic::SAES::Exceptions::Voting::Vote::QRVoteInvalidFormatException & cex )
    {
        result = VOTE_FROM_BUFFER_UNEXPECTED_ERROR;
        std::stringstream ss;
        ss << __func__ << " " << cex.what();
        logger->Debug ( ss.str() );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorVoteData" ) );
    }
    catch ( Smartmatic::SAES::Exceptions::Voting::Vote::QRVoteSignatureException & cex )
    {
        result = VOTE_FROM_BUFFER_SIGNATURE_ERROR;
        std::stringstream ss;
        ss << __func__ << " " << cex.what();
        logger->Debug ( ss.str() );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorVoteData" ) );
    }
    catch ( ... )
    {
        result = VOTE_FROM_BUFFER_UNEXPECTED_ERROR;
        std::stringstream ss;
        ss << __func__;
        logger->Debug ( ss.str() );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorVoteData" ) );
    }

    logger->Debug ( "End vote from safe buffer" );
    return result;
}
