/**
 * @file	VotingEnqueuer.cxx
 * @brief	voting enqueuer class.
 * @author	Gustavo Garcia
 * @date	30/01/2017
 */

#include "VotingEnqueuer.hxx"
#include <System/Compression/compression-class.h>
#include <Operation/Devices/Polling/BCRPolling.hxx>
#include <Operation/Voting/VoteCompliance/VoteComplianceVerificator.hxx>
#include <Voting/IDManager/BallotIDManager.hxx>
#include <Voting/IDManager/SessionIDManager.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <System/Sounds/smartmatic-sounds-manager.h>
#include <Operation/Voting/VoteFromSafeBuffer.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Sounds;
using namespace Smartmatic::System::Compression;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Operation::VoteCompliance;

ISMTTLog * VotingEnqueuer::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.VotingEnqueuer", GETTEXT_PACKAGE );
VotingEnqueuer * VotingEnqueuer::instance = NULL;

VotingEnqueuer::VotingEnqueuer()
{
    flagPause = true;
    flagSaveVote = false;
    cleanVoteRepeated = SAESFunctionalityConfigurationManager::getInstance()->getEnqueueVoteConfiguration().getCleanVoteRepeated();
    validateBallotAndSession = true;

    scanNormal = "ScanNormalVoteSound";
    scanDuplicated = "ScanDuplicatedVoteSound";
    scanBad = "ScanBadVoteSound";
    scanReceivedData = "BarcodeDataReceivedVoteSound";

    BCRPolling::getInstance()->getBufferSignal().connect ( sigc::mem_fun ( *this, &VotingEnqueuer::onProcess ) );
}

VotingEnqueuer::~VotingEnqueuer()
{
    instance = NULL;
}

void VotingEnqueuer::pause()
{
    logger->Debug ( "Pause" );
    flagPause = true;
}

void VotingEnqueuer::resume()
{
    logger->Debug ( "Resume" );
    flagPause = false;
}

void VotingEnqueuer::enableSaveVote ( bool enable )
{
    flagSaveVote = enable;

    if(!flagSaveVote)
    {
       logger->Debug ( "Save vote disabled" );
    }
}

void VotingEnqueuer::validateID ( bool validate )
{
    validateBallotAndSession = validate;

    if(!validateBallotAndSession)
    {
       logger->Debug ( "Validating bid and sid disabled" );
    }
}

VotingEnqueuer::SignalVote VotingEnqueuer::getSignalVote()
{
    return signalVote;
}

VotingEnqueuer::SignalPersistenceStatusVote VotingEnqueuer::getSignalStatus()
{
    return signalStatus;
}

VotingEnqueuer::SignalEurnInteraction VotingEnqueuer::getSignalEurnInteration()
{
    return signalEurnInteration;
}

VotingEnqueuer * VotingEnqueuer::getInstance()
{
    if ( instance == NULL )
    {
        instance = new VotingEnqueuer();
    }

    return instance;
}

void VotingEnqueuer::onProcess ( Smartmatic::System::SafeBuffer * buffer )
{
    if ( flagPause )
    {
        logger->Debug ( "Skip process buffer" );
        return;
    }

    logger->Debug ( "Processing vote" );

    SMTTSoundsManager::getInstance()->Sound ( scanReceivedData );

    std::list <SafeBuffer *> list;
    std::list <SafeBuffer *>::iterator it;

    CompressionClass::getInstance()->Inflate ( *buffer, list );

    for ( it = list.begin(); it != list.end(); ++it )
    {
        SafeBuffer * item = ( *it );
        Vote::Vote * vote = NULL;

        VoteFromSafeBuffer voteFromBuffer ( MachineOperationManager::getInstance()->getElectoralConfiguration() );
        VoteFromSafeBuffer::VoteFromSafeBufferResult parseResult = voteFromBuffer.getVote ( item, vote );

        if ( parseResult == VoteFromSafeBuffer::VOTE_FROM_BUFFER_SUCCESS )
        {
            if ( cleanVoteRepeated && currentVoteCode == vote->getCode() )
            {
                currentVoteCode = "";
                logger->Debug ( "Discard Duplicates Vote" );
            }
            else
            {
                currentVoteCode = vote->getCode();
                getSignalVote().emit ( vote );

                logger->Audit ( N_ ( "Smartmatic.SAES.Operation.BCR.BCRController.ScannedVote" ) );
                if ( ( validateVote ( vote ) ) && ( voteMeetsCompliance( vote ) ) )
                {
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.BCR.BCRController.SuccessVote" ) );

                    if(!saveVote ( vote ))
                    {
                        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.BCR.BCRController.FailVote" ) );
                    }
                    else
                    {
                        BallotIDManager::getInstance()->removeID ( vote->getBallotID() );
                    }
                }
                else
                {
                    getSignalEurnInteration().emit(false);
                    SMTTSoundsManager::getInstance()->Sound ( scanBad );
                    getSignalStatus().emit ( VotePersistence::PERSISTENCE_VOTE_FAIL );
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.BCR.BCRController.FailVote" ) );
                }
            }
        }
        else
        {
            getSignalEurnInteration().emit(false);
            currentVoteCode = "";
            SMTTSoundsManager::getInstance()->Sound ( scanBad );
            getSignalStatus().emit ( VotePersistence::PERSISTENCE_VOTE_FAIL );
        }

        if ( vote )
        {
            delete vote;
            vote = NULL;
        }

        if ( item )
        {
            delete item;
            item = NULL;
        }
    }

    logger->Debug ( "End process vote" );
}

bool VotingEnqueuer::validateVote ( Vote::Vote * vote )
{
    bool validateBallotID = true;
    bool validateSessionID = true;

    if ( validateBallotAndSession )
    {
        if ( !MachineOperationManager::getInstance()->getVoteStatistics()->IsVoteProcessed ( *vote ) )
        {
            // Validate Ballot ID and Session ID

            validateBallotID = BallotIDManager::getInstance()->verifyID ( vote->getBallotID() );
            validateSessionID = SessionIDManager::getInstance()->verifyID ( MachineOperationManager::getInstance()->getOperationStatus(), vote->getSessionID() );

            if ( !validateBallotID )
            {
                logger->Debug ( "Invalid Ballot ID" );
            }

            if ( !validateSessionID )
            {
                logger->Debug ( "Invalid Session ID" );
            }
        }
    }else
    {
        logger->Debug ( "Skip bid and sid validation" );
    }

    return validateBallotID && validateSessionID;
}

bool VotingEnqueuer::voteMeetsCompliance( Vote::Vote * vote )
{
    ElectoralConfiguration *electoralConfiguration = MachineOperationManager::getInstance()->getElectoralConfiguration();

    return VoteComplianceVerificator::getInstance(electoralConfiguration)->checkAllAfterScanningVoteConditions(*vote);
}

bool VotingEnqueuer::saveVote ( Vote::Vote * vote )
{
    if ( !flagSaveVote )
    {
        logger->Debug ( "Skip save vote" );
        return true;
    }
    VotePersistence persistence (vote);
    return saveVote ( persistence );
}

bool VotingEnqueuer::saveVote ( VotePersistence& persistence )
{
    VotePersistence::PersistenceStatusVote status = persistence.saveVote ();

    switch ( status )
    {
        case VotePersistence::PERSISTENCE_VOTE_SUCCESS:
            SMTTSoundsManager::getInstance()->Sound ( scanNormal );
            break;

        case VotePersistence::PERSISTENCE_VOTE_DUPLICATE:
            SMTTSoundsManager::getInstance()->Sound ( scanDuplicated );
            break;

        case VotePersistence::PERSISTENCE_VOTE_FAIL:
        default:
            SMTTSoundsManager::getInstance()->Sound ( scanBad );
            break;
    }

    getSignalEurnInteration().emit(status != VotePersistence::PERSISTENCE_VOTE_FAIL);
    getSignalStatus().emit ( status );

    return status != VotePersistence::PERSISTENCE_VOTE_FAIL;
}
