/**
 * @file	VotingEnqueuer.hxx
 * @brief	Declares the voting enqueuer class.
 * @author	Gustavo Garcia
 * @date	30/01/2017
 */

#ifndef _VOTING_ENQUEUER_H_
#define _VOTING_ENQUEUER_H_

#include <glibmm.h>
#include <System/SafeBuffer.h>
#include <Operation/Voting/VotePersistence.hxx>
#include <Voting/Vote/vote-schema.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Voting
            {
                /**
                 * @class	VotingEnqueuer
                 * @brief	VotingEnqueuer class.
                 * @author	Yadickson Soto
                 * @date	05/08/2015
                 */
                class VotingEnqueuer
                {
                    public:

                        /**
                         * @typedef sigc::signal<void, Smartmatic::SAES::Voting::Vote::Vote * > SignalVote
                         * @brief Defines an alias representing the signal vote.
                         */
                        typedef sigc::signal<void, Smartmatic::SAES::Voting::Vote::Vote * > SignalVote; ///< typedef signal vote
                        /**
                         * @typedef sigc::signal<void, Smartmatic::SAES::Operation::VotePersistence::PersistenceStatusVote > SignalPersistenceStatusVote
                         * @brief Defines an alias representing the signal persistence status vote.
                         */
                        typedef sigc::signal<void, Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote > SignalPersistenceStatusVote; ///< typedef signal status

                        /**
                         * @typedef sigc::signal<void, bool > SignalEurnInteraction
                         * @brief Defines an alias representing the signal Eurn interaction.
                         */
                        typedef sigc::signal<void, bool > SignalEurnInteraction; ///< typedef signal status

                        /**
                         * @fn	virtual ~VotingEnqueuer();
                         * @brief	Finaliser.
                         * @author	Yadickson Soto
                         * @date	05/08/2015
                         */
                        virtual ~VotingEnqueuer();

                        /**
                         * @fn void pause();
                         * @brief Pauses the voting process.
                         * @author Yadickson Soto
                         * @date 05/08/2015
                         */
                        void pause();

                        /**
                         * @fn void resume ( bool force = false );
                         * @brief Resume the voting process.
                         * @author Yadickson Soto
                         * @date 05/08/2015
                         * @param force force resume.
                         */
                        void resume();

                        /**
                         * @fn void enableSaveVote ( bool enable = true );
                         * @brief Enables the saving of a vote.
                         * @author Yadickson Soto
                         * @date 05/08/2015
                         * @param enable enable any save vote.
                         */
                        void enableSaveVote ( bool enable = true );

                        /**
                         * @fn void validateID ( bool validate = true );
                         * @brief Sets if ballot id and the session id of a vote must be validated.
                         * @author Yadickson Soto
                         * @date 05/08/2015
                         * @param validate set validate id.
                         */
                        void validateID ( bool validate = true );

                        /**
                         * @fn SignalVote BCRController::getSignalVote();
                         * @brief Gets the signal that occurs when the voting process has finished.
                         * @author Yadickson Soto
                         * @date 05/08/2015
                         * @return signal.
                         */
                        SignalVote getSignalVote();

                        /**
                         * @fn SignalPersistenceStatusVote BCRController::getSignalStatus();
                         * @brief Gets the signal that occurs when the voting process status has been updated.
                         * @author Yadickson Soto
                         * @date 05/08/2015
                         * @return signal.
                         */
                        SignalPersistenceStatusVote getSignalStatus();

                        /**
                         * @fn SignalEurnInteraction BCRController::getSignalEurnInteration();
                         * @brief Gets the signal that occurs when the interation whith Eurn is required.
                         * @author Juan Valera
                         * @date 27/10/2017
                         * @return signal.
                         */
                        SignalEurnInteraction getSignalEurnInteration();

                        /**
                         * @fn	static CheckDataManager * getInstance();
                         * @brief	Getter instance of the class
                         * @date 05/08/2015
                         * @author Yadickson Soto
                         * @return The instance of the class
                         */
                        static VotingEnqueuer * getInstance();

                        /**
                         * @fn	bool saveVote ( Smartmatic::SAES::Voting::Vote::Vote * vote );
                         * @brief	Save the Votes
                         * @date 27/01/2016
                         * @author Gustavo Garcia
                         * @return FALSE if there was an error
                         */
                        bool saveVote ( Smartmatic::SAES::Voting::Vote::Vote * vote );
                        /**
                         * @fn	bool saveVote ( Smartmatic::SAES::Operation::Voting::VotePersistence& persistence );
                         * @brief	Save the Votes
                         * @date 27/01/2016
                         * @author Gustavo Garcia
                         * @return FALSE if there was an error
                         */
                        bool saveVote ( Smartmatic::SAES::Operation::Voting::VotePersistence& persistence );
                    protected:

                        /**
                         * @fn	VotingEnqueuer();
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	05/08/2015
                         */
                        VotingEnqueuer();

                        /**
                         * @fn	void onProcess ( Smartmatic::System::SafeBuffer * buffer );
                         * @brief	Process the given buffer.
                         * @author	Yadickson Soto
                         * @date	05/08/2015
                         * @param 	buffer	Read buffer from barcode reader
                         */
                        void onProcess ( Smartmatic::System::SafeBuffer * buffer );

                    public:
                        bool validateVote ( Smartmatic::SAES::Voting::Vote::Vote * vote );

                        bool voteMeetsCompliance ( Smartmatic::SAES::Voting::Vote::Vote * vote );

                    private:

                        bool flagPause; ///<The flag pause emit signals
                        bool flagSaveVote; ///< flag enable save vote
                        bool cleanVoteRepeated; ///< flag clean vote
                        bool validateBallotAndSession; ///< flag validate ballot id and session id

                        std::string scanNormal; ///< sound scan normal vote
                        std::string scanDuplicated; ///< sound scan duplicate vote
                        std::string scanBad; ///< sound scan bad vote
                        std::string scanReceivedData; ///< sound scan received data

                        SignalVote signalVote; ///< signal emit vote
                        SignalPersistenceStatusVote signalStatus; ///< signal status
                        SignalEurnInteraction signalEurnInteration; ///< signal Eurn

                        std::string currentVoteCode; ///< current vote code

                        static VotingEnqueuer * instance; ///< The instance of the class
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif // _VOTING_ENQUEUER_H_
