/**
 * @file	VotingExperienceController.hxx
 * @brief	voting experience controller class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#ifndef __VOTING_EXPERIENCE_CONTROLLER_H__
#define __VOTING_EXPERIENCE_CONTROLLER_H__

#include <string>
#include <list>
#include <vector>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/CardData/card-data-schema.hxx>
#include <Voting/Vote/vote-schema.hxx>
#include <Log/ISMTTLog.h>
#include "VotingState.hxx"

namespace Smartmatic
{
    namespace SAES
    {
        namespace Resources
        {
            class ElectionStringResourcesManager;
            class GeneralAudioResourcesManager;
            class ElectionAudioResourcesManager;
        }

        namespace Operation
        {
            namespace Voting
            {
                /**
                 * @class	VotingOriginator
                 * @brief	forward class declaration VotingOriginator.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */

                class VotingOriginator;

                /**
                 * @class	VotingMemento
                 * @brief	forward class declaration VotingMemento.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */

                class VotingMemento;

                /**
                 * @class	FillVote
                 * @brief	forward class declaration FillVote.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */

                class FillVote;

                /**
                 * @class	VotingExperienceController
                 * @brief	Header class VotingExperienceController.
                 * @author	Yadickson Soto
                 * @date	15-02-2011
                 */
                class VotingExperienceController
                {
                    public:

                        /**
                         * @enum VotingExperienceType
                         * @brief	define type actions
                         */

                        typedef enum
                        {
                            VOTING_CONTROLLER_CONTEST /**<action contest*/
                            , VOTING_CONTROLLER_CUSTOM_ATTRIBUTE_DEFINITION /**<action custom attribute definition*/
                            , VOTING_CONTROLLER_CLOSE /**<action close*/
                        } VotingExperienceType;

                        /**
                         * @fn	VotingExperienceController::VotingExperienceController ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                         * @brief	Class constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	electoralConfiguration	electoral configuration.
                         */

                        explicit VotingExperienceController ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );

                        /**
                         * @fn	virtual VotingExperienceController::~VotingExperienceController();
                         * @brief	Finaliser.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */

                        virtual ~VotingExperienceController();

                        /**
                         * @fn	bool VotingExperienceController::init ( Smartmatic::SAES::Voting::CardData::Card_data & cardData ) throw ( Smartmatic::System::Exception::ElectionException,
                         * 			Smartmatic::System::Exception::VotingDeviceException );
                         * @brief	Initialises this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::ElectionException			Thrown when an election error
                         * 																																	condition occurs.
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @param	cardData	Information describing the card.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool init ( Smartmatic::SAES::Voting::CardData::Card_data & cardData )
                        throw ( Smartmatic::System::Exception::ElectionException,
                                Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	void VotingExperienceController::dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */

                        void dispose();

                        /**
                         * @fn	VotingExperienceType VotingExperienceController::getNextTypeAction();
                         * @brief	Gets the type of next action.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	The next type action.
                         */

                        VotingExperienceType getNextTypeAction();

                        /**
                         * @fn	Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition & VotingExperienceController::getPreviousCustomAttributeDefinition () throw ( Smartmatic::System::Exception::VotingDeviceException );
                         * @brief	method to obtain the previous custom attribute definition.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @return	a reference to the previous custom attribute definition
                         * 					@throw VotingDeviceException.
                         */

                        Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition &
                        getPreviousCustomAttributeDefinition ()
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition & VotingExperienceController::getNextCustomAttributeDefinition () throw ( Smartmatic::System::Exception::VotingDeviceException );
                         * @brief	method to obtain the next custom attribute definition.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @return	a reference to the next custom attribute definition
                         * 					@throw VotingDeviceException.
                         */

                        Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition &
                        getNextCustomAttributeDefinition ()
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition & VotingExperienceController::getCurrentCustomAttributeDefinition ();
                         * @brief	method to obtain the current custom attribute definition.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	a reference to the current custom attribute definition.
                         */

                        Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition &
                        getCurrentCustomAttributeDefinition ();

                        /**
                         * @fn	void VotingExperienceController::setCustomAttributeDefinitionResponse ( std::string code,
                         * 			std::string value ) throw ( Smartmatic::System::Exception::VotingDeviceException );
                         * @brief	method to set the current custom attribute definition response.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @param	code 	code response.
                         * @param	value	value response
                         * 							@throw VotingDeviceException.
                         */

                        void setCustomAttributeDefinitionResponse ( const std::string& code, const std::string& value )
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	void VotingExperienceController::setNullPartyResponse ( Smartmatic::SAES::Voting::Election::Contest & contest );
                         * @brief	method to set the null eligible option response.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contest	current contest.
                         */

                        void setNullPartyResponse ( Smartmatic::SAES::Voting::Election::Contest & contest );

                        /**
                         * @fn	Smartmatic::SAES::Voting::Vote::Vote * VotingExperienceController::getVote () throw ( Smartmatic::System::Exception::ElectionException );
                         * @brief	method to get the vote.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @return	vote
                         * 					@throw ElectionException.
                         */

                        Smartmatic::SAES::Voting::Vote::Vote * getVote ()
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void VotingExperienceController::setBallotOptionState ( std::string contestCode,
                         * 			std::string partyCode, std::string ballotOptionCode, bool state = true );
                         * @brief	method to set the ballot option state.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestCode				contest.
                         * @param	partyCode					party.
                         * @param	ballotOptionCode	ballot option.
                         * @param	state							enw state ballot option.
                         */

                        void setBallotOptionState (
                            std::string contestCode,
                            std::string partyCode,
                            std::string ballotOptionCode,
                            bool state = true );

                        /**
                         * @fn	std::list < Smartmatic::SAES::Voting::Election::Contest > VotingExperienceController::getCurrentContestList () throw ( Smartmatic::System::Exception::ElectionException );
                         * @brief	method to get the current contest list.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @return	current contest list.
                         */

                        std::list < Smartmatic::SAES::Voting::Election::Contest * >
                        getCurrentContestList ()
                        throw ( Smartmatic::System::Exception::ElectionException );

                        std::list < Smartmatic::SAES::Voting::Election::Contest * >
                        getCurrentContestList (const int mementoPosition)
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	std::list < Smartmatic::SAES::Voting::Election::Ballot_option > VotingExperienceController::getBallotOptionsSelectedByContest ( Smartmatic::SAES::Voting::Election::Contest & contest );
                         * @brief	Gets the ballot options selected by contest.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contest	The contest.
                         * @return	The ballot options selected by contest.
                         */

                        std::list < Smartmatic::SAES::Voting::Election::Ballot_option * >
                        getBallotOptionsSelectedByContest (
                            Smartmatic::SAES::Voting::Election::Contest & contest );

                        /**
                         * @fn	std::list < Smartmatic::SAES::Voting::Election::Ballot_option > VotingExperienceController::getBallotOptionsSelectedByContestParty ( Smartmatic::SAES::Voting::Election::Contest & contest,
                         * 			Smartmatic::SAES::Voting::Election::Party & party ) throw ( Smartmatic::System::Exception::ElectionException );
                         * @brief	Gets the ballot options selected by contest party.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	contest	The contest.
                         * @param	party	 	The party.
                         * @return	The ballot options selected by contest party.
                         */

                        std::list < Smartmatic::SAES::Voting::Election::Ballot_option * >
                        getBallotOptionsSelectedByContestParty (
                            Smartmatic::SAES::Voting::Election::Contest & contest,
                            Smartmatic::SAES::Voting::Election::Party & party )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void VotingExperienceController::clearBallotOptionsSelectedByContest ( Smartmatic::SAES::Voting::Election::Contest & contest );
                         * @brief	Clears the ballot options selected by contest.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contest	The contest.
                         */

                        void clearBallotOptionsSelectedByContest (
                            Smartmatic::SAES::Voting::Election::Contest & contest );

                        /**
                         * @fn	void VotingExperienceController::resetVotingExperience ();
                         * @brief	Reset the voting experience. used when a recovery vote is done bad.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */

                        void resetVotingExperience ();

                        /**
                         * @fn	void VotingExperienceController::clearBallotOptionsSelectedByContestParty ( Smartmatic::SAES::Voting::Election::Contest & contest,
                         * 			Smartmatic::SAES::Voting::Election::Party & party );
                         * @brief	Clears the ballot options selected by contest and party.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contest	The contest.
                         * @param	party	 	The party.
                         */

                        void clearBallotOptionsSelectedByContestParty (
                            Smartmatic::SAES::Voting::Election::Contest & contest,
                            Smartmatic::SAES::Voting::Election::Party & party );

                        /**
                         * @fn	Smartmatic::SAES::Voting::Election::Party VotingExperienceController::getPartySelectedFromContest ( Smartmatic::SAES::Voting::Election::Contest & contest ) throw ( Smartmatic::System::Exception::ElectionException );
                         * @brief	Gets the party selected from contest.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	contest	The contest.
                         * @return	The party selected from contest.
                         */

                        Smartmatic::SAES::Voting::Election::Party
                        getPartySelectedFromContest (
                            Smartmatic::SAES::Voting::Election::Contest & contest )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	bool VotingExperienceController::isThereAnyPartySelectedFromContest ( Smartmatic::SAES::Voting::Election::Contest & contest );
                         * @brief	Query if there is any party selected from contest.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contest	The contest.
                         * @return	true if there is any party selected from contest, false if not.
                         */

                        bool isThereAnyPartySelectedFromContest (
                            Smartmatic::SAES::Voting::Election::Contest & contest );

                        /**
                         * @fn	bool VotingExperienceController::moveContestListForward();
                         * @brief	moves the contest list forward.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool moveContestListForward();

                        /**
                         * @fn	bool VotingExperienceController::canMoveBack ( const std::string& contestCode );
                         * @brief	Queries if it can move back.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestCode	current contest code.
                         * @return	true if it can move, false if not.
                         */

                        bool canMoveBack ( const std::string& contestCode );

                        /**
                         * @fn	bool VotingExperienceController::firstContest ( std::string contestCode );
                         * @brief	Queries if it can move contest back.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestCode	current contest code.
                         * @return	true if it can move, false if not.
                         */

                        bool firstContest ( std::string contestCode );

                        /**
                         * @fn	bool VotingExperienceController::canMoveContestListBack();
                         * @brief	Queries if it can move back.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	true if it can move, false if not.
                         */

                        bool canMoveContestListBack();

                        void deleteVote();

                        /**
                         * @fn  bool VotingExperienceController::returnToSpecificContest(const std::string contestCode);
                         * @brief   Returns to a specific contest.
                         * @author  eduardo.fernandez
                         * @date    12/12/2017
                         * @param   mementoPosition the memento position to return to.
                         */

                        void returnToSpecificContest(const int mementoPosition);

                        /**
                         * @fn  bool VotingExperienceController::setFinalMemento();
                         * @brief   Sets current memento as the last one.
                         * @author  eduardo.fernandez
                         * @date    16/01/2018
                         */

                        void setFinalMemento();

                        /**
                         * @fn  std::list<std::string> VotingExperienceController::getOrderedContestsList(VotingMemento *currentMemento);
                         * @brief   Gets a list containing the contests ordered by contest class given the current memento.
                         * @author  eduardo.fernandez
                         * @date    16/01/2018
                         * @param   currentMemento Currently selected memento.
                         * @return  A list containing the contests ordered by contest class given the current memento.
                         */

                        std::list<std::string> getOrderedContestsList(VotingMemento *currentMemento);

                        /**
                         * @fn  std::vector<std::string> VotingExperienceController::getSelectedOptions();
                         * @brief   Gets a vector containing currently selected options.
                         * @author  eduardo.fernandez
                         * @date    16/01/2018
                         * @param   listOnlyContests Whether to add only the contests involved in the vote. It should
                         *          only be activated when entering the selections edition stage.
                         */

                        std::vector<std::string> getSelectedOptions(const bool listOnlyContests);

                        /**
                         * @fn  void VotingExperienceController::deselectOptions(int currentContestsIndex);
                         * @brief   Deselects given contest's selections.
                         * @author  eduardo.fernandez
                         * @date    16/01/2018
                         * @param   currentContestsIndex The index of the contest whose selections will be deselected.
                         */

                        void deselectOptions(int currentContestsIndex);

                        /**
                         * @fn	bool VotingExperienceController::moveContestListBack();
                         * @brief	moves the contest list back.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool moveContestListBack();

                        /**
                         * @fn	std::string VotingExperienceController::getChecksumVote();
                         * @brief	method to get checksum of vote.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	the checksum of vote.
                         */

                        std::string getChecksumVote();

                        /**
                         * @fn	Smartmatic::SAES::Voting::Vote::VoteMode VotingExperienceController::getVotingMode();
                         * @brief	method to get the voting mode.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	voting mode.
                         */

                        Smartmatic::SAES::Voting::Vote::VoteMode getVotingMode();

                        /**
                         * @fn	void VotingExperienceController::setVotingMode ( Smartmatic::SAES::Voting::Vote::VoteMode mode );
                         * @brief	Sets a voting mode.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	mode	The mode.
                         */

                        void setVotingMode ( Smartmatic::SAES::Voting::Vote::VoteMode mode );

                        /**
                         * @fn	std::string VotingExperienceController::getActivationCode();
                         * @brief	method to get the activation code.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	activation card data code.
                         */

                        std::string getActivationCode();

                        /**
                         * @fn	std::list < std::string > VotingExperienceController::getFixedContests()
                         * @brief	method to get fixed contest.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	fixed contests.
                         */

                        std::list < std::string > getFixedContests()
                        {
                            return allContestsFixed;
                        }

                        /**
                         * @brief Method to get the next contest code from a given custom attribute.
                         * @param code the contest code.
                         * @param value the value.
                         * @author eduardo.fernandez
                         * @date 12/09/2018
                         * @return the next contest code from a given custom attribute.
                         */
                        std::string getNextContestCodeFromCustomAttribute(
                            const std::string& code,
                            const std::string& value);

                        /**
                         * @brief  Gets the next contest from a given custom attribute.
                         * @author manuel.arguelles
                         * @date   sep 21, 2018
                         * @param  code  The custom attribute code.
                         * @param  value The custom attribute value.
                         */
                        Smartmatic::SAES::Voting::Election::Contest* getNextContestFromCustomAttribute(
                            const std::string& code,
                            const std::string& value);

                    private:
                        static Smartmatic::Log::ISMTTLog * logger;	///< the logger

                        VotingOriginator * originator;   ///< object originator from design pattern memento
                        VotingMemento * memento; ///< object state from design pattern memento
                        FillVote * fillVote; ///< fill vote
                        std::vector < VotingMemento * > caretaker;   ///< object caretaker from design pattern memento

                        Smartmatic::SAES::Voting::Vote::VoteMode votingMode;   ///< flag for voting mode
                        std::string checksumCode;	///< checksum code
                        std::string activationCode;  ///< activation card data code
                        std::string voteCode;	///< vote code
                        std::string operationModeCode;   ///< operation mode code
                        std::string votingBoothCode; ///< voting booth code
                        std::string regionCode;  ///< region code
                        std::string vdCode; ///< The Voting Device Code
                        Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration;   ///< electoral configuration
                        Smartmatic::SAES::Voting::Vote::Vote * vote;   ///< vote
                        Smartmatic::SAES::Voting::Election::Contest currentContest;  ///< The current contest
                        Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition currentCustomAttributeDefinition; ///< The current custom attribute definition

                        std::list < std::string > allContestsFixed;  ///< Contest fixed list
                        std::list < std::string > allContests;   ///< Contest list
                        Smartmatic::SAES::Voting::CardData::Card_data  cardData;	///< Information describing the card
                        SAES::Resources::ElectionStringResourcesManager* m_electoralResources;
                        SAES::Resources::GeneralAudioResourcesManager* m_generalAudioResources;
                        SAES::Resources::ElectionAudioResourcesManager* m_electoralAudioResources;
                        /**
                         * @fn	bool VotingExperienceController::validateCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData );
                         * @brief	validates elements from card data.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	cardData	voting activation card data.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool validateCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData );

                        /**
                         * @fn	bool VotingExperienceController::validateLogicalVoterGroupCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData );
                         * @brief	validates the logical voter group from card data.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	cardData	voting activation card data.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool validateLogicalVoterGroupCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData );

                        /**
                         * @fn	bool VotingExperienceController::validateCustomAttributeDefinitionCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData );
                         * @brief	validates the custom attribute definitions from card data.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	cardData	voting activation card data.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool validateCustomAttributeDefinitionCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData );

                        /**
                         * @fn	bool VotingExperienceController::filterFromCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData ) throw ( Smartmatic::System::Exception::VotingDeviceException );
                         * @brief	filters the elements from the card data response.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @param	cardData	voting activation card data.
                         * @return	true if validate
                         * 					@throw VotingDeviceException.
                         */

                        bool filterFromCardData ( Smartmatic::SAES::Voting::CardData::Card_data & cardData )
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	void VotingExperienceController::filterEmptyCustomAttributeDefinition();
                         * @brief	Filters empty custom attribute definition.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */

                        void filterEmptyCustomAttributeDefinition();

                        /**
                         * @fn	void VotingExperienceController::findFixedContests() throw ( Smartmatic::System::Exception::ElectionException );
                         * @brief	Searches for fixed contests.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         */

                        void findFixedContests()
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	std::list<Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group> VotingExperienceController::filterLogicalVoterGroup ( const std::string& code, const std::string& value );
                         * @brief	method to filter the logical voter group.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	code 	code response.
                         * @param	value	value response.
                         * @return	list of logical voter group.
                         */

                        std::list<Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * > filterLogicalVoterGroup ( const std::string& code, const std::string& value );

                        /**
                         * @fn	std::list < Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition > VotingExperienceController::filterCustomAttributeDefinitionByLogicalVoterGroup ( std::list<Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group> & logicalVoterGroups,
                         * 			std::string code );
                         * @brief	method to filter custom attribute definition by logical voter group.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	logicalVoterGroups	list logical voter groups.
                         * @param	code								code response.
                         * @return	list of custom attribute definitions.
                         */

                        std::list < Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition > filterCustomAttributeDefinitionByLogicalVoterGroup (
                            std::list<Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * > & logicalVoterGroups,
                            std::string code );

                        /**
                         * @fn	bool VotingExperienceController::findCustomAttribute ( std::list<Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group> & logicalVoterGroups,
                         * 			std::string code );
                         * @brief	searches for a custom attribute definition.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	logicalVoterGroups	list of logical voter groups.
                         * @param	code								code response.
                         * @return	true if finds a custom attribute definition.
                         */

                        bool findCustomAttribute ( std::list<Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * > & logicalVoterGroups, std::string code );

                        /**
                         * @fn  bool VotingExperienceController::setCustomAttributeInformation (std::string &selectedOption, VotingMemento *currentMemento, const VotingState::TypeResponse &response);
                         * @brief   sets the custom attribute selection information.
                         * @author  eduardo.fernandez
                         * @date    20/12/2017
                         * @param   selectedOption  current contest selections information.
                         * @param   currentMemento  memento currently being read.
                         * @return  response        voting state type response.
                         */

                        void setCustomAttributeInformation(std::string &selectedOption, VotingMemento *currentMemento, const VotingState::TypeResponse &response);

                        /**
                         * @fn  bool VotingExperienceController::setContestInformation(std::string &selectedOption, const std::list<std::string>::iterator &contestsIterator);
                         * @brief   sets the contest selection information.
                         * @author  eduardo.fernandez
                         * @date    20/12/2017
                         * @param   selectedOption   current contest selections information.
                         * @param   contestCode      current contest code.
                         * @param   listOnlyContests Whether to add only the contests involved in the vote. It should
                         *                           only be activated when entering the selections edition stage.
                         *
                         */

                        void setContestInformation(std::string &selectedOption, const std::string &contestCode, const bool listOnlyContests);

                        /**
                         * @fn  bool VotingExperienceController::setPartyInformation(std::string &selectedOption, const std::list<std::string>::iterator &partiesIterator);
                         * @brief   sets the party selection information.
                         * @author  eduardo.fernandez
                         * @date    20/12/2017
                         * @param   selectedOption  current contest selections information.
                         * @param   partiesIterator current party.
                         */

                        void setPartyInformation(std::string &selectedOption, const std::list<std::string>::iterator &partiesIterator);

                        /**
                         * @fn  bool VotingExperienceController::setListVoteInformation(
                                    std::string &selectedOption,
                                    bool &anyListVote,
                                    const std::list<std::string> &ballotOptions,
                                    const std::string &contestsIterator,
                                    const std::string &partiesIterator);
                         * @brief   sets the list vote selection information.
                         * @author  eduardo.fernandez
                         * @date    20/02/2017
                         * @param   selectedOption   current contest selections information.
                         * @param   anyListVote      whether a list vote has been selected.
                         * @param   ballotOptions    current memento ballot options selection.
                         * @param   contestsIterator current contest.
                         * @param   partiesIterator  current party.
                         */

                        void setListVoteInformation(
                            std::string &selectedOption,
                            bool &anyListVote,
                            const std::list<std::string> &ballotOptions,
                            const std::string &contestsIterator,
                            const std::string &partiesIterator);

                        /**
                         * @fn  bool VotingExperienceController::setCandidatesInformation(
                                    std::string &selectedOption,
                                    bool &anyPrincipal,
                                    const std::list<std::string> &ballotOptions,
                                    const std::list<Smartmatic::SAES::Voting::Election::Ballot_option*> &configurationBallotOptions,
                                    const std::string &candidatesListMessageKey,
                                    const std::string &contestsIterator,
                                    const std::string &partiesIterator);
                         * @brief   sets the principal candidates selection information.
                         * @author  eduardo.fernandez
                         * @date    20/12/2017
                         * @param   selectedOption             current contest selections information.
                         * @param   anyPrincipal               whether at least a principal candidate has been selected.
                         * @param   ballotOptions              current memento ballot options selection.
                         * @param   configurationBallotOptions electoral configuration ballot options.
                         * @param   candidatesListMessageKey   list message key.
                         * @param   contestsIterator           current contest.
                         * @param   partiesIterator            current party.
                         */

                        void setCandidatesInformation(
                            std::string &selectedOption,
                            bool &anyPrincipal,
                            const std::list<std::string> &ballotOptions,
                            const std::list<Smartmatic::SAES::Voting::Election::Ballot_option*> &configurationBallotOptions,
                            const std::string &candidatesListMessageKey,
                            const std::string &contestsIterator,
                            const std::string &partiesIterator);

                        /**
                         * @fn	void VotingExperienceController::findNewContests() throw ( Smartmatic::System::Exception::ElectionException );
                         * @brief	Searches for new contests.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         */

                        void findNewContests()
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	void VotingExperienceController::findContests ( bool fixeds = true ) throw ( Smartmatic::System::Exception::ElectionException );
                         * @brief	Searches for contests.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @exception	Smartmatic::System::Exception::ElectionException	Thrown when an election error
                         * 																															condition occurs.
                         * @param	fixeds	true to fixeds.
                         */

                        void findContests ( bool fixeds = true )
                        throw ( Smartmatic::System::Exception::ElectionException );

                        /**
                         * @fn	VotingExperienceType VotingExperienceController::alternateFirstQuestions();
                         * @brief	alternates first questions, first questions and last contests fixed.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	type action execute.
                         */
                        VotingExperienceType alternateFirstQuestions();

                        /**
                         * @fn	VotingExperienceType VotingExperienceController::alternateFirstContestsFixed();
                         * @brief	alternates questions, first contests fixed.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	type action execute.
                         */
                        VotingExperienceType alternateFirstContestsFixed();

                        /**
                         * @fn	VotingExperienceType VotingExperienceController::allQuestionsFirstQuestions();
                         * @brief	all questions, first questions and last contests.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	type action execute.
                         */
                        VotingExperienceType allQuestionsFirstQuestions();

                        /**
                         * @fn	VotingExperienceType VotingExperienceController::allQuestionsFirstContestsFixed();
                         * @brief	all questions, first contests fixed, all questions and last constest.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	type action execute.
                         */
                        VotingExperienceType allQuestionsFirstContestsFixed();

                        /**
                         * @fn	VotingExperienceType VotingExperienceController::priorityByContestClass();
                         * @brief	all questions and first contests by priority contest class.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	type action execute.
                         */
                        VotingExperienceType priorityByContestClass();

                        /**
                         * @fn	void VotingExperienceController::setResponse ( const std::string& code, const std::string& value ,
                         * 			VotingState::TypeResponse response );
                         * @brief	Sets a response.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	code			The code response.
                         * @param	value			The value response.
                         * @param	response	response of user or automatic.
                         */
                        void setResponse ( const std::string& code, const std::string& value , VotingState::TypeResponse response );

                        /**
                         * @brief method to add the contests by contest group into given voting state.
                         * @author eduardo.fernandez
                         * @date   12/09/2018
                         * @param[out] state              The voting state.
                         * @param[in]  logicalVoterGroups The logical voter groups.
                         */
                        void addContestsByContestGroup(
                            VotingState* state,
                            const std::list<SAES::Voting::VotingDevice::Logical_voter_group*>& logicalVoterGroups);

                        /**
                         * @brief method to create a voting state.
                         * @author eduardo.fernandez
                         * @date 12/09/2018
                         * @param code  The contest code.
                         * @param value The value.
                         * @return The voting state.
                         */
                        VotingState* createVotingState(
                            const std::string& code,
                            const std::string& value,
                            VotingState::TypeResponse response);

                };
            }
        }
    }
}

#endif /*__VOTING_EXPERIENCE_CONTROLLER_H__*/
