/**
 * @file VotingEnqueuerTest.h
 * @date 30/01/2017
 * @author Gustavo Garcia
 */
#ifndef TEST_VOTING_ENQUEUER_TEST
#define TEST_VOTING_ENQUEUER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Voting/Vote/VoteFactory.hxx>
#include <tests/MockSaesFunctionalityConfigurationManager.hxx>
#include "Operation/MachineOperationManager.hxx"
#include "Operation/Voting/VotingEnqueuer.hxx"
#include "Operation/Voting/MockVotePersistence.hxx"
#include <tests/MockSmartmaticSoundsManager.hxx>
#include <tests/MockMachineOperationManager.hxx>
#include <tests/MockSessionIDManager.hxx>

using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting;

class VotingEnqueuerTest : public CPPUNIT_NS::TestFixture
{
    private:
        typedef Smartmatic::SAES::Voting::Vote::Vote Vote;
        typedef Smartmatic::SAES::Operation::Voting::VotingEnqueuer VotingEnqueuer;
        typedef Smartmatic::SAES::Functionality::Configuration::SAESFunctionalityConfigurationManager SAESFunctionalityConfigurationManager;
        typedef Smartmatic::SAES::Operation::Voting::VotePersistence VotePersistence;
        typedef Smartmatic::System::Sounds::SMTTSoundsManager SMTTSoundsManager;

        VotingEnqueuer* votingEnqueuer;

    public:

        VotingEnqueuerTest ()
            : votingEnqueuer(NULL)
        {

        }

        void setUp()
        {
            MockSaesFunctionalityConfigurationManager configurationManager;
            SAESFunctionalityConfigurationManager::getInstance ( &configurationManager );

            votingEnqueuer = VotingEnqueuer::getInstance();
        }

        void tearDown()
        {
            if ( votingEnqueuer )
            {
                delete votingEnqueuer;
                votingEnqueuer = NULL;
            }

        }

        void testSaveVoteFailed()
        {
             Vote vote;
             MockVotePersistence mock(&vote);

             MOCKPP_CHAINER_FOR ( MockVotePersistence, saveVote ) saveVote_mock ( &mock );
             saveVote_mock
                  .expects ( MOCKPP_NS::once() )
                  .will ( new MOCKPP_NS::ReturnStub<VotePersistence::PersistenceStatusVote> ( VotePersistence::PERSISTENCE_VOTE_FAIL ) );

             MockSmartmaticSoundsManager soundsMock;
             SMTTSoundsManager::getInstance(&soundsMock);

             votingEnqueuer->enableSaveVote ( true );
             CPPUNIT_ASSERT ( !votingEnqueuer->saveVote (mock) );
        }

        void testSaveVoteSuccess()
        {
            Vote vote;
            MockVotePersistence mock(&vote);

            MOCKPP_CHAINER_FOR ( MockVotePersistence, saveVote ) saveVote_mock ( &mock );
            saveVote_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub <VotePersistence::PersistenceStatusVote> ( VotePersistence::PERSISTENCE_VOTE_SUCCESS ),
                        new MOCKPP_NS::ReturnStub <VotePersistence::PersistenceStatusVote> ( VotePersistence::PERSISTENCE_VOTE_DUPLICATE ) ) );

            MockSmartmaticSoundsManager soundsMock;
            SMTTSoundsManager::getInstance(&soundsMock);

            votingEnqueuer->enableSaveVote ( true );

            CPPUNIT_ASSERT ( votingEnqueuer->saveVote (mock) );
            CPPUNIT_ASSERT ( votingEnqueuer->saveVote (mock) );

        }

        void testValidateVoteWithValidationOfBallotAndSessionEnabled()
        {
            Vote vote;
            votingEnqueuer->validateID ( true );

            MockSessionIDManager mockIdManager(false);
            SessionIDManager::getInstance(&mockIdManager);

            MockMachineOperationManager mockOperationManager;
            MachineOperationManager::getInstance(&mockOperationManager);

            CPPUNIT_ASSERT ( !votingEnqueuer->validateVote(&vote) );
        }

        void testValidateVoteWithValidationOfBallotAndSessionDisabled()
        {
           Vote vote;
           votingEnqueuer->validateID ( false );

           MockSessionIDManager mockIdManager(false);
           SessionIDManager::getInstance(&mockIdManager);

           MockMachineOperationManager mockOperationManager;
           MachineOperationManager::getInstance(&mockOperationManager);

          CPPUNIT_ASSERT ( votingEnqueuer->validateVote(&vote) );
        }

        CPPUNIT_TEST_SUITE ( VotingEnqueuerTest );

        CPPUNIT_TEST ( testSaveVoteFailed );
        CPPUNIT_TEST ( testSaveVoteSuccess );
//        CPPUNIT_TEST ( testValidateVoteWithValidationOfBallotAndSessionEnabled );
        CPPUNIT_TEST ( testValidateVoteWithValidationOfBallotAndSessionDisabled );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_VOTING_ENQUEUER_TEST */
