/**
 * @file	print-preview-functionality.h
 * @brief	Declares the print preview functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef _PRINT_PREVIEW_FUNCTIONALITY_H_
#define _PRINT_PREVIEW_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include "Voting/Vote/vote-schema.hxx"
#include <Functionalities/graphic-display-functionality.h>
#include "Printing/PrintPreviewWidget.h"
#include "Printing/VotePrintDocument.h"
#include <System/SafeBuffer.h>
#include <Log/ISMTTLog.h>
#include <Operation/Voting/VotePersistence.hxx>
#include <Widgets/SmartmaticBasicScrollContainer.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	PrintPreviewFunctionality
                 * @brief	Print preview functionality.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class PrintPreviewFunctionality: public GraphicDisplayFunctionality
                {

                    public:

                        /**
                         * @fn	bool PrintPreviewFunctionality::DontAllowHelp();
                         * @brief	Gets the dont allow help.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowHelp();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> PrintPreviewFunctionality::getHelpSeqKey();
                         * @brief	Gets the help sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The help sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getHelpSeqKey();

                        /**
                         * @fn	bool PrintPreviewFunctionality::DontAllowExit();
                         * @brief	Gets the dont allow exit.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowExit();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> PrintPreviewFunctionality::getSeqKey();
                         * @brief	Gets the sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getSeqKey();

                        /**
                         * @fn	static PrintPreviewFunctionality * PrintPreviewFunctionality::getInstance();
                         * @brief	Gets the current.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else.
                         */

                        static PrintPreviewFunctionality * getInstance();
                        /**
                         * @fn	virtual void PrintPreviewFunctionality::refresh();
                         * @brief	refresh widget method.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         */
                        virtual void refresh();

                    protected:

                        Glib::Mutex mutex;	///< The mutex

                        /**
                         * @fn	virtual void PrintPreviewFunctionality::block();
                         * @brief	method block functionality.
                         * @author	Yadickson Soto
                         * @date	19/04/2012
                         */
                        virtual void block();
                        /**
                         * @fn	virtual void PrintPreviewFunctionality::partial();
                         * @brief	method partial functionality.
                         * @author	Yadickson Soto
                         * @date	19/04/2012
                         */
                        virtual void partial();
                        /**
                         * @fn	virtual void PrintPreviewFunctionality::resume();
                         * @brief	method resume functionality.
                         * @author	Yadickson Soto
                         * @date	19/04/2012
                         */
                        virtual void resume();

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * PrintPreviewFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the logger.
                         */

                        Smartmatic::Log::ISMTTLog * getLogger();

                        /**
                         * @fn	Glib::ustring PrintPreviewFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The start log message.
                         */

                        Glib::ustring getStartLogMessage();

                        /**
                         * @fn	Glib::ustring PrintPreviewFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The finish log message.
                         */

                        Glib::ustring getFinishLogMessage();

                        /**
                         * @fn	virtual void PrintPreviewFunctionality::deleteElements();
                         * @brief	virtual Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();

                    private:

                        sigc::connection signalVote;	///< The signal vote
                        sigc::connection signalStatusVote;	///< The signal status vote

                        Smartmatic::SAES::Voting::Vote::Vote vote;	///< The vote
                        bool validVote; ///< flag valid vote
                        Gtk::Widget * normalFrameWidget;	///< The normal frame widget

                        bool wasPresent;	///< true to was present
                        std::vector<unsigned char> dataBinarizedToWrite;	///< The data binarized to write
                        sigc::connection conn;  ///< The connection


                        Smartmatic::SAES::Printing::PrintPreviewWidget * printWidgetMagnify; ///< The print widget magnify
                        Smartmatic::SAES::Printing::VotePrintDocument * docMagnify; ///< The document magnify
                        Gtk::Label * labelMessage;  ///< Message describing the label
                        Gtk::Frame * frameMagnify;  ///< The frame magnify
                        Gtk::VBox * vboxMagnify; ///< The vbox magnify
                        Smartmatic::GUI::Widgets::SmartmaticBasicScrollContainer * scrolledMagnify;  ///< The scrolled magnify
                        Gtk::Viewport  * viewMagnify;   ///< The view magnify

                        /**
                         * @fn	void PrintPreviewFunctionality::CreateNormalFrame();
                         * @brief	Create the normal frame.
                         * @author	Juan.Delgado
                         * @date	23/07/2011
                         */
                        void  CreateNormalFrame();

                        static PrintPreviewFunctionality * current_instance; ///< The current instance

                        /**
                         * @fn	PrintPreviewFunctionality::PrintPreviewFunctionality();
                         * @brief	Default constructor initialize all variables in NULL.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        PrintPreviewFunctionality();

                        /**
                         * @fn	virtual PrintPreviewFunctionality::~PrintPreviewFunctionality();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        virtual ~PrintPreviewFunctionality();

                        Smartmatic::SAES::Voting::Vote::VoteMode voteMode;	///< The vote mode

                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType PrintPreviewFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The menu option type.
                         */

                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();

                        /**
                         * @fn	void PrintPreviewFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Dispose();

                        /**
                         * @fn	void PrintPreviewFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Initialize();

                        /**
                         * @fn	Gtk::Widget * PrintPreviewFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the frame.
                         */

                        Gtk::Widget * GetFrame();

                        /**
                         * @fn	bool PrintPreviewFunctionality::Refresh ( int time_num );
                         * @brief	Refreshs.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	time_num	The time number.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool Refresh ( int time_num );

                        /**
                         * @fn	bool PrintPreviewFunctionality::BarcodeRefreshVoteMode ( int time_num );
                         * @brief	Barcode refresh demo.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	time_num	The time number.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool BarcodeRefreshVoteMode ( int time_num );

                        /**
                         * @fn	void PrintPreviewFunctionality::BarcodeError();
                         * @brief	Barcode error.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void BarcodeError();

                        /**
                         * @fn	std::string PrintPreviewFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The window title.
                         */

                        std::string GetWindowTitle();

                        /**
                         * @fn	void PrintPreviewFunctionality::onSignalVote ( Smartmatic::SAES::Voting::Vote::Vote * vote );
                         * @brief	Method getter vote process.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         * @param	vote	vote.
                         */
                        void onSignalVote ( Smartmatic::SAES::Voting::Vote::Vote * vote );
                        /**
                         * @fn	void PrintPreviewFunctionality::onSignalState ( Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status );
                         * @brief	Method getter status vote process.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         * @param	status	status vote process.
                         */
                        void onSignalState ( Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status );
                        /**
                         * @fn	virtual void PrintPreviewFunctionality::onBeforeExit();
                         * @brief	virtual method before exit.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         */
                        virtual void onBeforeExit();

                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}


#endif // _PRINT_PREVIEW_FUNCTIONALITY_H_
