/**
 * @file	ActivationStatsWidget.h
 * @brief	Declares the activation stats widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef ACTIVATIONSTATSWIDGET_H_
#define ACTIVATIONSTATSWIDGET_H_

#include <gtkmm.h>
#include <Log/ISMTTLog.h>
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include <Configuration/SaesGUIConfiguration-schema.hxx>
#include <Widgets/StatisticWidget.h>
#include <Operation/Devices/Polling/CardPolling.hxx>
#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicImage.hxx>
#include <Widgets/SmartmaticBasicEventBox.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Configuration
        {
            class LabelConfiguration;
        }

        namespace Widgets
        {
            class SmartmaticBasicCheckButton;
        }
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	ActivationStatsWidget
                 * @brief	Activation stats widget.
                 * @author	Youssef.tourkmani
                 * @date	05/11/2012
                 */
                class ActivationStatsWidget : public Smartmatic::GUI::Widgets::SmartmaticBasicVBox
                {
                    public:

                        typedef sigc::signal<void> SignalAudioVotingButtonChecked;

                        typedef sigc::signal<void> SignalDone;

                    public:
                        /**
                         * @fn	ActivationStatsWidget::ActivationStatsWidget ( Smartmatic::SAES::Configuration::CountConfiguration & totalCountType,
                         * 			bool showStatistic );
                         * @brief	Constructor.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @param [in,out]	totalCountType	Type of the total count.
                         * @param	showStatistic							true to show, false to hide the statistic.
                         */
                        ActivationStatsWidget ( Smartmatic::SAES::Configuration::CountConfiguration & totalCountType, bool showStatistic );

                        /**
                         * @fn	virtual ActivationStatsWidget::~ActivationStatsWidget();
                         * @brief	finalizer.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         */
                        virtual ~ActivationStatsWidget();

                        /**
                         * @fn	void ActivationStatsWidget::Initialize();
                         * @brief	Initialize widget.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         */
                        void Initialize();

                        /**
                         * @fn	void ActivationStatsWidget::Dispose();
                         * @brief	Dispose widget.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         */
                        void Dispose();

                        /**
                         * @fn	void ActivationStatsWidget::SetAttributes ( std::vector<Smartmatic::SAES::Voting::VotingDevice::Custom_attribute> attributes );
                         * @brief	Set attributes for card.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @param	attributes	with custom attributes.
                         */
                        void SetAttributes ( std::vector<Smartmatic::SAES::Voting::VotingDevice::Custom_attribute> attributes );

                        /**
                         * @fn	void ActivationStatsWidget::AllowCardActivation ( bool allow );
                         * @brief	Allow Card Activation.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @param	allow	true if allow activation.
                         */
                        void AllowCardActivation ( bool allow );

                        /**
                         * @fn	static bool ActivationStatsWidget::WriteVotingCard ( std::vector<Smartmatic::SAES::Voting::VotingDevice::Custom_attribute> attributes );
                         * @brief	Writes voting info in card.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @param [in,out]	attributes	with custom attributes.
                         * @return	true if succesfully.
                         */
                        bool WriteVotingCard ( std::vector<Smartmatic::SAES::Voting::VotingDevice::Custom_attribute> attributes );

                        /**
                         * @fn  void cardInsert();
                         * @brief   method card insert.
                         * @author  Yadickson Soto
                         * @date    28/03/2013
                         */
                        void cardInsert();
                        /**
                         * @fn  void cardRemove();
                         * @brief   method card remove.
                         * @author  Yadickson Soto
                         * @date    28/03/2013
                         */
                        void cardRemove();
                        /**
                         * @fn  void cardReaderUnresponsive();
                         * @brief   Method signal card reader unresponsive
                         * @author  Yadickson Soto
                         * @date    14/12/2012
                         */
                        void cardReaderUnresponsive();

                        /**
                         * @fn	static void ActivationStatsWidget::SetOneMemoryMode ( bool oneMemory );
                         * @brief	Set one memory mode operation.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @param	oneMemory	if only one memory.
                         */
                        static void SetOneMemoryMode ( bool oneMemory );

                        /**
                         * @fn	void ActivationStatsWidget::ContinueOperation();
                         * @brief	Continue operation.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void ContinueOperation();

                        /**
                         * @fn	void ActivationStatsWidget::StopOperation();
                         * @brief	Stop operation.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void StopOperation();

                        /**
                         * @fn	sigc::signal< void, bool > ActivationStatsWidget::getSignalStatusActivated();
                         * @brief	Method getter signal status activated
                         * @author	Yadickson Soto
                         * @date	13/11/2012
                         * @return signal
                         */
                        sigc::signal< void, bool > getSignalStatusActivated();

                        void setIsHelpShown( bool isHelpShown );

                        /**
                         * @fn  SignalAudioVotingButtonChecked ActivationStatsWidget::getSignalAudioVotingButtonChecked();
                         * @brief   Method to get the audio voting button checked signal.
                         * @author  Eduardo Fernández
                         * @date    08/01/2018
                         * @return  The audio voting button checked signal.
                         */
                        SignalAudioVotingButtonChecked getSignalAudioVotingButtonChecked();

                        /**
                         * @fn  SignalDone ActivationStatsWidget::getSignalDone();
                         * @brief   Method to get the activation done signal.
                         * @author  Eduardo Fernández
                         * @date    04/01/2018
                         * @return  The activation done signal.
                         */
                        SignalDone getSignalDone();

                        /**
                         * @brief  Disconnect audio voting checkbox events.
                         * @author manuel.arguelles
                         * @date   Mar  1, 2018
                         */
                        void disconnectAudioVotingCheckBox();

                        /**
                         * @brief  Reconnect audio voting checkbox events.
                         * @author manuel.arguelles
                         * @date   Mar  1, 2018
                         */
                        void reconnectAudioVotingCheckBox();

                        /**
                         * @brief  Resets the audio voting selection.
                         * @author manuel.arguelles
                         * @date   Mar  1, 2018
                         */
                        void resetAudioVotingSelection();

                    private:
                        /**
                         * @fn	void ActivationStatsWidget::WriteCard();
                         * @brief	Run thread for write voting card.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         */
                        void WriteCard();

                        /**
                         * @fn	void ActivationStatsWidget::UpdateLabel( void * p );
                         * @brief	Update label instruction text.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         */
                        void UpdateLabel ( void * p );

                        /**
                         * @fn	bool ActivationStatsWidget::UpdateStat();
                         * @brief	Update stat instruction text.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @return	true if continue signal.
                         */
                        bool UpdateStat();

                        /**
                         * @fn	bool ActivationStatsWidget::showMessageSleepWriteCard();
                         * @brief	Shows the message sleep write card.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool showMessageSleepWriteCard();

                        /**
                         * @fn  void ActivationStatsWidget::refreshRemovedAfterPowerCycle();
                         * @brief   Checks if the card is removed after refreshing from a power cycle to show the correct message and image
                         * @author  Youssef.tourkmani
                         * @date    01/11/2012
                         */
                        void refreshRemovedAfterPowerCycle();

                        /**
                         * @fn	void deviceChange( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );
                         * @brief	getter device monitor change
                         * @author	Yadickson Soto
                         * @date	14/12/2012
                         * @param 	deviceTypeMonitor device change
                         */
                        void deviceChange ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );
                        /**
                         * @fn	void onSignalCardChange( Smartmatic::SAES::Operation::Devices::CardPolling::CardPollingStatus status );
                         * @brief	Method on signal card change
                         * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date	26/03/2013
                         * @param 	status	card status
                         */
                        void onSignalCardChange ( Smartmatic::SAES::Operation::Devices::CardPolling::CardPollingStatus status );
                        /**
                         * @fn  void setVotingExperienceTypeWidgets(Smartmatic::GUI::Configuration::LabelConfiguration labelConfiguration);
                         * @brief   Method set voting experience type widgets
                         * @author  Eduardo Fernández &lt;eduardo.fernandez@smartmatic.com&gt;
                         * @date    24/11/2017
                         * @param   labelConfiguration  label configuration
                         */
                        void setVotingExperienceTypeWidgets(Smartmatic::GUI::Configuration::LabelConfiguration labelConfiguration);
                        /**
                         * @fn  void onPressAudioVotingType(GdkEventButton * event);
                         * @brief   Method on press audio voting type
                         * @author  Eduardo Fernández &lt;eduardo.fernandez@smartmatic.com&gt;
                         * @date    24/11/2017
                         * @param   event  gdk event button
                         */
                        bool onPressAudioVotingType(GdkEventButton * event);

                        /**
                         * @brief  Selects an audio voting experience attribute.
                         * @author manuel.arguelles
                         * @date   Mar  1, 2018
                         */
                        void selectAudioVotingAttribute();

                        /**
                         * @brief  Deselects an audio voting experience attribute.
                         * @author manuel.arguelles
                         * @date   Mar  1, 2018
                         */
                        void deselectAudioVotingAttribute();

                        sigc::connection signalChange; ///< The signal change device
                        std::vector<Smartmatic::SAES::Voting::VotingDevice::Custom_attribute> m_attributes;  ///< The attributes
                        Smartmatic::GUI::Widgets::StatisticWidget * statsWidget;	///< The statistic widget
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * instructionLabel; ///< The instruction label
                        Smartmatic::GUI::Widgets::SmartmaticBasicImage * cardImage; ///< The card image

                        sigc::connection conn_checkCard;	///< The check card connection
                        sigc::connection conn_updateLabel;	///< The update label connection
                        sigc::connection conn_updateStat;	///< The update stat connection
                        sigc::connection connSleepWriteCard;	///< The sleep write card connection

                        Smartmatic::System::SafeDispatcher signalUpdate; ///< signal update text
                        Glib::Thread * m_thread; ///< The thread pointer
                        Glib::Mutex m_insertCardMutex;
                        Glib::Mutex m_attributesMutex;

                        static bool oneMemory;	///< True if operating with one memory
                        bool disposeExecuted;///< True if dispose was executed

                        std::string updateLabel;	///< Text for update label
                        std::string updateImage;	///< image

                        std::string imageInsert;	///< Path of insert image
                        std::string imageExtract;	///< Path of extract image
                        std::string imageExtractError;	///< Path of extract image error
                        std::string imageDeviceNotFound;	///< Path of device not found
                        std::string imageCardReaderUnresponsive; ///< Path image card reader unresponsive

                        bool error; ///< flag error
                        bool forceUpdate; ///< force update
                        bool smartcardError; ///< flag smartcard error
                        bool m_isHelpShown;

                        sigc::signal< void, bool > signalStatusActivated; ///< signal status

                        Smartmatic::SAES::Configuration::CountConfiguration totalCountType; ///< Type of the total count
                        int timerRefresh; ///< time refresh
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                        bool m_powerCycled;

                        Smartmatic::GUI::Widgets::SmartmaticBasicImage m_audioVotingCheckBoxImage;
                        Smartmatic::GUI::Widgets::SmartmaticBasicEventBox m_audioVotingCheckBox;
                        sigc::connection m_audioVotingCheckBoxConnection;

                        bool m_writingCard;

                        SignalAudioVotingButtonChecked m_signalAudioVotingButtonChecked;
                        SignalDone m_signalDone;

                        bool m_audioVotingEnabled;
                        bool m_audioVotingChecked;
                };
            }
        }
    }
}

#endif /* ACTIVATIONSTATSWIDGET_H_ */
