/**
 * @file EndAudioVotingSequenceWidget.cxx
 * @brief Body class end audio voting sequence widget
 * @date 17/11/2017
 * @author Eduardo Fernández
 */

#include "EndAudioVotingSequenceWidget.hxx"
#include <Gui/Widgets/Voting/RemoveCardWidget.hxx>
#include <Gui/Widgets/Voting/PrintVoteWidget.hxx>
#include <Gui/Widgets/Voting/FoldPrintVoteWidget.hxx>
#include <Gui/Widgets/Voting/PrintVoteErrorWidget.hxx>
#include <Gui/Widgets/Voting/PrintAndFoldVoteWidget.hxx>
#include <Gui/Widgets/Voting/PrintAudioVoteWidget.hxx>
#include <Gui/Widgets/Voting/FactoryVotingWidget.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::Log;

ISMTTLog * EndAudioVotingSequenceWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.Voting.EndAudioVotingSequenceWidget", GETTEXT_PACKAGE );

EndAudioVotingSequenceWidget::EndAudioVotingSequenceWidget()
    : StepWidget()
{
    errorDetected = false;
    manager = NULL;
}

EndAudioVotingSequenceWidget::~EndAudioVotingSequenceWidget()
{
    remove();
    dispose();
}

void EndAudioVotingSequenceWidget::refresh()
{
    if ( manager )
    {
        manager->refresh();
    }
}

void EndAudioVotingSequenceWidget::dispose()
{
    std::vector < WizardWidgetBase * >::iterator it;

    for ( it = wizardVector.begin();
            it != wizardVector.end();
            ++it )
    {
        WizardWidgetBase * widget = ( *it );
        delete widget;
        widget = NULL;
    }

    wizardVector.clear();

    if ( manager )
    {
        delete manager;
        manager = NULL;
    }
}

void EndAudioVotingSequenceWidget::start()
{
    errorDetected = false;
    dispose();

    VotingExperienceConfiguration::EndStepSequence & seq ( SAESGUIConfigurationManager::getInstance()->getVotingExperienceConfiguration().getEndStep() );
    VotingExperienceConfiguration::EndStepIterator iter;

    for ( iter = seq.begin(); iter != seq.end(); iter++ )
    {
        WizardWidgetBase * widget = NULL;

        switch ( ( *iter ) )
        {
            default:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_WAITING_CARD:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_READ_CARD:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_USED:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD_AID:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD_PIN:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD_GENERIC:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_LANGUAGE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_LANGUAGE_CONFIRMATION:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CUSTOM_ATTRIBUTE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CUSTOM_ATTRIBUTE_CONFIRMATION:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PARTY:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PARTY_CONFIRMATION:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_NULL_PARTY:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_BALLOT:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_PRINT_VOTE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_REMOVED:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_REMOVED_COUNTER:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_REMOVED_CANCEL_PROCESS:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_INTERRUPT_VOTING_EXPERIENCE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_NULL_VOTING_EXPERIENCE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_TIME_OUT_VOTING_EXPERIENCE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_SCAN_VOTE:
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PRINT_FOLD_VOTE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_FOLD_VOTE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PRINT_VOTE:
                widget = FactoryVotingWidget::createPrintAudioVoteWidget();
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_REMOVE_CARD:
                widget = FactoryVotingWidget::createRemoveCardWidget();
                break;
        }

        if ( widget )
        {
            widget->on_stop_signal_emit().connect ( sigc::mem_fun ( *this, &EndAudioVotingSequenceWidget::errorSequence ) );
            widget->on_play_audio_signal_emit().connect ( sigc::mem_fun ( *this, &EndAudioVotingSequenceWidget::playAudio ) );
            wizardVector.push_back ( widget );
        }
    }

    if ( !wizardVector.empty() )
    {
        manager = new WizardWidgetManager ( wizardVector );
        manager->on_finish_signal_emit().connect ( sigc::mem_fun ( *this, &EndAudioVotingSequenceWidget::endSequence ) );
        manager->Start();
        setWidget ( manager );
    }
    else
    {
        endSequence();
    }
}

void EndAudioVotingSequenceWidget::errorSequence()
{
    errorDetected = true;
    signalError.emit();
}

void EndAudioVotingSequenceWidget::endSequence()
{
    if ( !errorDetected )
    {
        emit();
    }
}

void EndAudioVotingSequenceWidget::playAudio(const std::string& messageKey)
{
    m_signalPlayAudio.emit(messageKey);
}

sigc::signal<void> EndAudioVotingSequenceWidget::getSignalError()
{
    return signalError;
}

EndAudioVotingSequenceWidget::SignalPlayAudio EndAudioVotingSequenceWidget::getSignalPlayAudio()
{
    return m_signalPlayAudio;
}
