/**
 * @file AbleButtonDiagnosticWidget.cxx
 * @brief Body class able button diagnostic widget
 * @date 05/09/2012
 * @author Yadickson Soto
 */

#include "AbleButtonDiagnosticWidget.hxx"
#include <Operation/AbleDButtonState.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Configuration/P168AConfigurationManager.hxx>
#include <Operation/Devices/P168AController.hxx>
#include <Configuration/A4517AbledConfigurationManager.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Configuration/MachineModelEnum.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::HW::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;

static const std::string HELP_BUTTON_PRESS_MESSAGE = _("Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.HelpPushButton");
static const std::string UP_BUTTON_PRESS_MESSAGE = _("Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.UpPushButton");
static const std::string DOWN_BUTTON_PRESS_MESSAGE = _("Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.DownPushButton");
static const std::string LEFT_BUTTON_PRESS_MESSAGE = _("Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.LeftPushButton");
static const std::string RIGHT_BUTTON_PRESS_MESSAGE = _("Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.RightPushButton");
static const std::string SELECT_BUTTON_PRESS_MESSAGE = _("Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.SelectPushButton");
static const std::string VOLUME_DOWN_BUTTON_PRESS_MESSAGE = _("Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.VolumeDownPushButton");
static const std::string SPEED_DOWN_BUTTON_PRESS_MESSAGE = _("Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.SpeedDownPushButton");
static const std::string VOLUME_UP_BUTTON_PRESS_MESSAGE = _("Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.VolumeUpPushButton");
static const std::string SPEED_UP_BUTTON_PRESS_MESSAGE = _("Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.SpeedUpPushButton");

ISMTTLog * AbleButtonDiagnosticWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget", GETTEXT_PACKAGE );

AbleButtonDiagnosticWidget::AbleButtonDiagnosticWidget()
    : StepWidget()
{
    keepButtonsPressed = SAESGUIConfigurationManager::getInstance()->getAbleWidgetConfiguration().getKeepButtonsPressed();
    checkButtonsInSequence = SAESGUIConfigurationManager::getInstance()->getAbleWidgetConfiguration().getCheckButtonsInSequence();
    ableButtonWidget = new AbleButtonWidget();
    ableButtonWidget->getSignalCancel().connect ( sigc::mem_fun ( *this, &AbleButtonDiagnosticWidget::skipDiagnostic ) );
    setWidget ( ableButtonWidget );

    helpButtonPressed = false;
    volumeDownButtonPressed = false;
    downButtonPressed = false;
    speedDownButtonPressed = false;
    leftButtonPressed = false;
    selectButtonPressed = false;
    rightButtonPressed = false;
    volumeUpButtonPressed = false;
    upButtonPressed = false;
    speedUpButtonPressed = false;
}

AbleButtonDiagnosticWidget::~AbleButtonDiagnosticWidget()
{
    remove();
    dispose();

    if ( ableButtonWidget )
    {
        delete ableButtonWidget;
        ableButtonWidget = NULL;
    }
}

void AbleButtonDiagnosticWidget::initialize()
{
    status = true;
    currentButton = 0;
    buttonsToPress.clear();
    a4517AbledButtonsToPress.clear();
    conn.disconnect();

    audioVotingCheck = MachineOperationManager::getInstance()->getAudioVotingCheckUtility();

    if ((audioVotingCheck.ableIsConnected(true)) &&
        (audioVotingCheck.getAbledConnected() == AudioVotingCheckUtility::ABLED10))
    {
        speedDownButtonPressed = true;
    }

    connectAbleSignals();
    abledController.start();

    startCheckButton();
}

void AbleButtonDiagnosticWidget::connectAbleSignals()
{
    helpButtonConnection = abledController.signalHelpButton().connect(
        sigc::mem_fun(*this, &AbleButtonDiagnosticWidget::handleHelpPress));
    volumeDownButtonConnection = abledController.signalVolumeDownButton().connect(
        sigc::mem_fun(*this, &AbleButtonDiagnosticWidget::handleVolumeDownPress));
    downButtonConnection = abledController.signalDownButton().connect(
        sigc::mem_fun(*this, &AbleButtonDiagnosticWidget::handleDownPress));
    leftButtonConnection = abledController.signalLeftButton().connect(
        sigc::mem_fun(*this, &AbleButtonDiagnosticWidget::handleLeftPress));
    selectButtonConnection = abledController.signalSelectButton().connect(
        sigc::mem_fun(*this, &AbleButtonDiagnosticWidget::handleSelectPress));
    rightButtonConnection = abledController.signalRightButton().connect(
        sigc::mem_fun(*this, &AbleButtonDiagnosticWidget::handleRightPress));
    volumeUpButtonConnection = abledController.signalVolumeUpButton().connect(
        sigc::mem_fun(*this, &AbleButtonDiagnosticWidget::handleVolumeUpPress));
    upButtonConnection = abledController.signalUpButton().connect(
        sigc::mem_fun(*this, &AbleButtonDiagnosticWidget::handleUpPress));
    speedUpButtonConnection = abledController.signalSpeedUpButton().connect(
        sigc::mem_fun(*this, &AbleButtonDiagnosticWidget::handleSpeedUpPress));
    speedDownButtonConnection = abledController.signalSpeedDownButton().connect(
        sigc::mem_fun(*this, &AbleButtonDiagnosticWidget::handleSpeedDownPress));
}

void AbleButtonDiagnosticWidget::handleHelpPress()
{
    logger->Debug ( "Button Help pressed" );
    helpButtonPressed = true;
    helpButtonConnection.disconnect();

    (audioVotingCheck.getAbledConnected() == AudioVotingCheckUtility::ABLED20) ?
        handleButtonPress(A4517AbleButtonType::A4517_ABLED_DEVICE_HELP) :
        handleButtonPress(AbleButtonType::P168A_DEVICE_HELP);
}

void AbleButtonDiagnosticWidget::handleVolumeDownPress()
{
    logger->Debug ( "Button Volume Down pressed" );
    volumeDownButtonPressed = true;
    volumeDownButtonConnection.disconnect();
    (audioVotingCheck.getAbledConnected() == AudioVotingCheckUtility::ABLED20) ?
        handleButtonPress(A4517AbleButtonType::A4517_ABLED_DEVICE_VOLUME_DOWN) :
        handleButtonPress(AbleButtonType::P168A_DEVICE_VOLUME_DOWN);
}

void AbleButtonDiagnosticWidget::handleDownPress()
{
    logger->Debug ( "Button Down pressed" );
    downButtonPressed = true;
    downButtonConnection.disconnect();
    (audioVotingCheck.getAbledConnected() == AudioVotingCheckUtility::ABLED20) ?
        handleButtonPress(A4517AbleButtonType::A4517_ABLED_DEVICE_DOWN) :
        handleButtonPress(AbleButtonType::P168A_DEVICE_DOWN);
}

void AbleButtonDiagnosticWidget::handleSpeedDownPress()
{
    logger->Debug ( "Button Speed Down pressed" );
    speedDownButtonPressed = true;
    speedDownButtonConnection.disconnect();
    handleButtonPress(A4517AbleButtonType::A4517_ABLED_DEVICE_SPEED_DOWN);
}

void AbleButtonDiagnosticWidget::handleLeftPress()
{
    logger->Debug ( "Button Left pressed" );
    leftButtonPressed = true;
    leftButtonConnection.disconnect();
    (audioVotingCheck.getAbledConnected() == AudioVotingCheckUtility::ABLED20) ?
        handleButtonPress(A4517AbleButtonType::A4517_ABLED_DEVICE_LEFT) :
        handleButtonPress(AbleButtonType::P168A_DEVICE_LEFT);
}

void AbleButtonDiagnosticWidget::handleSelectPress()
{
    logger->Debug ( "Button Select pressed" );
    selectButtonPressed = true;
    selectButtonConnection.disconnect();
    (audioVotingCheck.getAbledConnected() == AudioVotingCheckUtility::ABLED20) ?
        handleButtonPress(A4517AbleButtonType::A4517_ABLED_DEVICE_SELECT) :
        handleButtonPress(AbleButtonType::P168A_DEVICE_SELECT);
}

void AbleButtonDiagnosticWidget::handleRightPress()
{
    logger->Debug ( "Button Right pressed" );
    rightButtonPressed = true;
    rightButtonConnection.disconnect();
    (audioVotingCheck.getAbledConnected() == AudioVotingCheckUtility::ABLED20) ?
        handleButtonPress(A4517AbleButtonType::A4517_ABLED_DEVICE_RIGHT) :
        handleButtonPress(AbleButtonType::P168A_DEVICE_RIGHT);
}

void AbleButtonDiagnosticWidget::handleVolumeUpPress()
{
    logger->Debug ( "Button Volume Up pressed" );
    volumeUpButtonPressed = true;
    volumeUpButtonConnection.disconnect();
    (audioVotingCheck.getAbledConnected() == AudioVotingCheckUtility::ABLED20) ?
        handleButtonPress(A4517AbleButtonType::A4517_ABLED_DEVICE_VOLUME_UP) :
        handleButtonPress(AbleButtonType::P168A_DEVICE_VOLUME_UP);
}

void AbleButtonDiagnosticWidget::handleUpPress()
{
    logger->Debug ( "Button Up pressed" );
    upButtonPressed = true;
    upButtonConnection.disconnect();
    (audioVotingCheck.getAbledConnected() == AudioVotingCheckUtility::ABLED20) ?
        handleButtonPress(A4517AbleButtonType::A4517_ABLED_DEVICE_UP) :
        handleButtonPress(AbleButtonType::P168A_DEVICE_UP);
}

void AbleButtonDiagnosticWidget::handleSpeedUpPress()
{
    logger->Debug ( "Button Speed Up pressed" );
    speedUpButtonPressed = true;
    speedUpButtonConnection.disconnect();
    (audioVotingCheck.getAbledConnected() == AudioVotingCheckUtility::ABLED20) ?
        handleButtonPress(A4517AbleButtonType::A4517_ABLED_DEVICE_SPEED_UP) :
        handleButtonPress(AbleButtonType::P168A_DEVICE_SPEED_UP);
}

void AbleButtonDiagnosticWidget::handleButtonPress(A4517AbleButtonType button)
{
    ableButtonWidget->pressButton(button);
    showMessage(button);
    validateButtonsPressing();
}

void AbleButtonDiagnosticWidget::handleButtonPress(AbleButtonType button)
{
    ableButtonWidget->pressButton(button);
    showMessage(button);
    validateButtonsPressing();
}

void AbleButtonDiagnosticWidget::validateButtonsPressing()
{
    if (helpButtonPressed && volumeDownButtonPressed && downButtonPressed && speedDownButtonPressed &&
        leftButtonPressed && selectButtonPressed && rightButtonPressed && volumeUpButtonPressed && upButtonPressed &&
        speedUpButtonPressed)
    {
        signalAllButtonsPressed.emit();
    }
}

AbleButtonDiagnosticWidget::SignalAllButtonsPressed AbleButtonDiagnosticWidget::getSignalAllButtonsPressed()
{
    return signalAllButtonsPressed;
}

void AbleButtonDiagnosticWidget::startCheckButton()
{
    currentButton = 0;
    buttonsToPress = P168AConfigurationManager::getInstance()->getButtons();
    a4517AbledButtonsToPress = A4517AbledConfigurationManager::getInstance()->getButtons();

    (audioVotingCheck.getAbledConnected() == AudioVotingCheckUtility::ABLED20) ?
        setA4517AbledButtonWidgetText() :
        setAbledButtonWidgetText();
}

void AbleButtonDiagnosticWidget::endDiagnosis()
{
    dispose();

    if ( status )
    {
        logger->Debug ( "End button diagnosis success" );
        emit();
    }
    else
    {
        logger->Debug ( "Any errors" );
        errorMessage = _ ( "Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.ButtonDiagnosisError" );
        errorButton();
    }
}

void AbleButtonDiagnosticWidget::dispose()
{
    abledController.stop();

    if ( ableButtonWidget )
    {
        ableButtonWidget->viewButton ( false );
    }

    conn.disconnect();
    connTimeout.disconnect();

    currentButton = 0;
    buttonsToPress.clear();
}

void AbleButtonDiagnosticWidget::showMessage(AbleButtonType button)
{
    if ( !checkButtonsInSequence )
    {
        logger->Debug ( "Get any button" );
        setAbledButtonWidgetText();
    }
    else if ( currentButton <= buttonsToPress.size() && buttonsToPress[currentButton - 1] == button )
    {
        logger->Debug ( "Get next button" );
        setAbledButtonWidgetText();
    }
    else if ( SAESGUIConfigurationManager::getInstance()->getAbleWidgetConfiguration().getContinueButtonFailsExpected() )
    {
        logger->Debug ( "Force get next button" );
        setAbledButtonWidgetText();
    }
    else if ( SAESGUIConfigurationManager::getInstance()->getAbleWidgetConfiguration().getFailButtonNotExpected() )
    {
        logger->Debug ( "Expected Error Button" );
        errorMessage = _ ( "Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.ExpectedErrorButton" );
        errorButton();
    }
}

void AbleButtonDiagnosticWidget::showMessage(A4517AbleButtonType button)
{
    if ( !checkButtonsInSequence )
    {
        logger->Debug ( "Get any button" );
        setA4517AbledButtonWidgetText();
    }
    else if ( currentButton <= a4517AbledButtonsToPress.size() && a4517AbledButtonsToPress[currentButton - 1] == button )
    {
        logger->Debug ( "Get next button" );
        setA4517AbledButtonWidgetText();
    }
    else if ( SAESGUIConfigurationManager::getInstance()->getAbleWidgetConfiguration().getContinueButtonFailsExpected() )
    {
        logger->Debug ( "Force get next button" );
        setA4517AbledButtonWidgetText();
    }
    else if ( SAESGUIConfigurationManager::getInstance()->getAbleWidgetConfiguration().getFailButtonNotExpected() )
    {
        logger->Debug ( "Expected Error Button" );
        errorMessage = _ ( "Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.ExpectedErrorButton" );
        errorButton();
    }
}

sigc::signal<void> AbleButtonDiagnosticWidget::getSignalError()
{
    return signalError;
}

void AbleButtonDiagnosticWidget::errorButton()
{
    dispose();
    ableButtonWidget->setText ( errorMessage );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &AbleButtonDiagnosticWidget::emitSignalError ), 1000 );
}

void AbleButtonDiagnosticWidget::emitSignalError()
{
    signalError.emit();
}

void AbleButtonDiagnosticWidget::setAbledButtonWidgetText()
{
    if ( currentButton < buttonsToPress.size() )
    {
        if ( checkButtonsInSequence )
        {
            AbleButtonType button = buttonsToPress[currentButton];

            switch ( button )
            {
                case AbleButtonType::P168A_DEVICE_HELP:
                    ableButtonWidget->setText ( HELP_BUTTON_PRESS_MESSAGE );
                    break;

                case AbleButtonType::P168A_DEVICE_UP:
                    ableButtonWidget->setText ( UP_BUTTON_PRESS_MESSAGE );
                    break;

                case AbleButtonType::P168A_DEVICE_DOWN:
                    ableButtonWidget->setText ( DOWN_BUTTON_PRESS_MESSAGE );
                    break;

                case AbleButtonType::P168A_DEVICE_LEFT:
                    ableButtonWidget->setText ( LEFT_BUTTON_PRESS_MESSAGE );
                    break;

                case AbleButtonType::P168A_DEVICE_RIGHT:
                    ableButtonWidget->setText ( RIGHT_BUTTON_PRESS_MESSAGE );
                    break;

                case AbleButtonType::P168A_DEVICE_SELECT:
                    ableButtonWidget->setText ( SELECT_BUTTON_PRESS_MESSAGE );
                    break;

                case AbleButtonType::P168A_DEVICE_VOLUME_DOWN:
                    ableButtonWidget->setText ( VOLUME_DOWN_BUTTON_PRESS_MESSAGE );
                    break;

                case AbleButtonType::P168A_DEVICE_VOLUME_UP:
                    ableButtonWidget->setText ( VOLUME_UP_BUTTON_PRESS_MESSAGE );
                    break;

                case AbleButtonType::P168A_DEVICE_SPEED_UP:
                    ableButtonWidget->setText ( SPEED_UP_BUTTON_PRESS_MESSAGE );
                    break;

                case AbleButtonType::P168A_DEVICE_PAUSE:
                case AbleButtonType::P168A_DEVICE_SPEED_DOWN:
                case AbleButtonType::P168A_DEVICE_UNKNOWN:
                default:
                    break;
            }

            currentButton++;
        }
        else
        {
            ableButtonWidget->setText ( _ ( "Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.PressAllButtons" ) );
        }
    }
}

void AbleButtonDiagnosticWidget::setA4517AbledButtonWidgetText()
{
    if ( currentButton < a4517AbledButtonsToPress.size() )
    {
        if ( checkButtonsInSequence )
        {
            A4517AbleButtonType button = a4517AbledButtonsToPress[currentButton];

            switch ( button )
            {
                case A4517AbleButtonType::A4517_ABLED_DEVICE_HELP:
                    ableButtonWidget->setText ( HELP_BUTTON_PRESS_MESSAGE );
                    break;

                case A4517AbleButtonType::A4517_ABLED_DEVICE_VOLUME_DOWN:
                    ableButtonWidget->setText ( VOLUME_DOWN_BUTTON_PRESS_MESSAGE );
                    break;

                case A4517AbleButtonType::A4517_ABLED_DEVICE_DOWN:
                    ableButtonWidget->setText ( DOWN_BUTTON_PRESS_MESSAGE );
                    break;

                case A4517AbleButtonType::A4517_ABLED_DEVICE_SPEED_DOWN:
                    ableButtonWidget->setText ( SPEED_DOWN_BUTTON_PRESS_MESSAGE );
                    break;

                case A4517AbleButtonType::A4517_ABLED_DEVICE_LEFT:
                    ableButtonWidget->setText ( LEFT_BUTTON_PRESS_MESSAGE );
                    break;

                case A4517AbleButtonType::A4517_ABLED_DEVICE_SELECT:
                    ableButtonWidget->setText ( SELECT_BUTTON_PRESS_MESSAGE );
                    break;

                case A4517AbleButtonType::A4517_ABLED_DEVICE_RIGHT:
                    ableButtonWidget->setText ( RIGHT_BUTTON_PRESS_MESSAGE );
                    break;

                case A4517AbleButtonType::A4517_ABLED_DEVICE_VOLUME_UP:
                    ableButtonWidget->setText ( VOLUME_UP_BUTTON_PRESS_MESSAGE );
                    break;

                case A4517AbleButtonType::A4517_ABLED_DEVICE_UP:
                    ableButtonWidget->setText ( UP_BUTTON_PRESS_MESSAGE );
                    break;

                case A4517AbleButtonType::A4517_ABLED_DEVICE_SPEED_UP:
                    ableButtonWidget->setText ( SPEED_UP_BUTTON_PRESS_MESSAGE );
                    break;

                case A4517AbleButtonType::A4517_ABLED_DEVICE_UNKNOWN:
                default:
                    break;
            }

            currentButton++;
        }
        else
        {
            ableButtonWidget->setText ( _ ( "Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.PressAllButtons" ) );
        }
    }
}

void AbleButtonDiagnosticWidget::skipDiagnostic()
{
    status = false;
    ableButtonWidget->setText ( _ ( "Smartmatic.SAES.GUI.Widgets.AbleButtonDiagnosticWidget.SkipDiagnostic" ) );
    dispose();
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &AbleButtonDiagnosticWidget::endDiagnosis ), 1000 );
}
