/**
 * @file	AlarmBoxActionsWidget.cxx
 * @brief	alarm box actions diagnostic widget class.
 * @author	Yadickson Soto
 * @date	06/09/2013
 */


#include "AlarmBoxActionsWidget.hxx"
#include "Configuration/label-configuration.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Environment/Configurations/resource-path-provider.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/AlarmController.hxx>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::Log;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::GUI::Widgets;

ISMTTLog * AlarmBoxActionsWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.AlarmBoxActionsWidget", GETTEXT_PACKAGE );

AlarmBoxActionsWidget::AlarmBoxActionsWidget ()
    : BasicDiagnosticWidget ( true, true )
{
    setOkButtonSensitive ( false );

    buzzerState = OFF;
    redLedState = OFF;
    greenLedState = OFF;
    buttonState = OFF;

    Smartmatic::SAES::Configuration::AlarmBoxDiagnosisConfiguration & config ( SAESGUIConfigurationManager::getInstance()->getAlarmBoxDiagnosisConfiguration() );

    timeOk = config.getActionsTimeOk();
    timeFail = config.getActionsTimeFail();

    periodBuzzer = config.getPeriodBuzzer();
    periodRedLed = config.getPeriodRedLed();
    periodGreenLed = config.getPeriodGreenLed();

    showBuzzerButton = config.getShowBuzzerButtonTest();
    showRedLedButton = config.getShowRedLedButtonTest();
    showGreenLedButton = config.getShowGreenLedButtonTest();

    endWidget = AbstractFactoryWidget::createAlarmBoxEndDiagnoseWidget();

    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile;
    gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/AlarmBoxActionsWidget.glade" );

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( "FATAL - ProgressWidget Initialize" );
    }

    builder->get_widget_derived ( "frame", frame );
    builder->get_widget ( "alignment", alignment );
    builder->get_widget_derived ( "vbox-options", vboxOptions );

    buttonBuzzer = new SmartmaticButton();
    buttonRedLed = new SmartmaticButton();
    buttonGreenLed = new SmartmaticButton();

    colorFrame = new SmartmaticBasicFrame();
    colorBox = new SmartmaticBasicEventBox();

    colorFrame->set_shadow_type ( Gtk::SHADOW_IN );
    colorFrame->add ( *colorBox );

    ButtonConfiguration buttonConf = config.getButtonConfiguration();
    LabelConfiguration labelText = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->getGeneralDiagnosisConfiguration().getDiagnosisLabelConfiguration() );

    buttonBuzzer->ModifyFontSize ( buttonConf.getFontSize() );
    buttonRedLed->ModifyFontSize ( buttonConf.getFontSize() );
    buttonGreenLed->ModifyFontSize ( buttonConf.getFontSize() );

    buttonBuzzer->set_size_request ( buttonConf.getButtonWidth(), buttonConf.getButtonHeight() );
    buttonRedLed->set_size_request ( buttonConf.getButtonWidth(), buttonConf.getButtonHeight() );
    buttonGreenLed->set_size_request ( buttonConf.getButtonWidth(), buttonConf.getButtonHeight() );
    colorFrame->set_size_request ( config.getWidthBoxColor(), config.getHeightBoxColor() );

    buttonBuzzer->signal_clicked().connect ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::onBuzzerAction ) );
    buttonRedLed->signal_clicked().connect ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::onRedLedAction ) );
    buttonGreenLed->signal_clicked().connect ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::onGreenLedAction ) );

    buttonBuzzer->set_no_show_all();
    buttonRedLed->set_no_show_all();
    buttonGreenLed->set_no_show_all();

    buttonBuzzer->set_visible ( showBuzzerButton );
    buttonRedLed->set_visible ( showRedLedButton );
    buttonGreenLed->set_visible ( showGreenLedButton );

    buzzerOption = manage ( new AlarmBoxDiagnoseStepWidget ( buttonBuzzer ) );
    buzzerOption->setLabelConfiguration ( labelText );

    redLedOption = manage ( new AlarmBoxDiagnoseStepWidget ( buttonRedLed ) );
    redLedOption->setLabelConfiguration ( labelText );

    greenLedOption = manage ( new AlarmBoxDiagnoseStepWidget ( buttonGreenLed ) );
    greenLedOption->setLabelConfiguration ( labelText );

    buttonOption = manage ( new AlarmBoxDiagnoseStepWidget ( colorFrame ) );
    buttonOption->setLabelConfiguration ( labelText );

    instructionLabel = manage ( new SmartmaticBasicLabel() );
    LabelConfiguration::SetLabelConfiguration ( instructionLabel, labelText );

    vboxOptions->pack_start ( *buzzerOption );
    vboxOptions->pack_start ( *redLedOption );
    vboxOptions->pack_start ( *greenLedOption );
    vboxOptions->pack_start ( *instructionLabel );
    vboxOptions->pack_start ( *buttonOption );

    buzzerOption->getSignalChange().connect ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::validate ) );
    redLedOption->getSignalChange().connect ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::validate ) );
    greenLedOption->getSignalChange().connect ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::validate ) );
    buttonOption->getSignalChange().connect ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::validate ) );

    oneColor = SAESGUIConfigurationManager::ConvertToGdkColor ( config.getUnselectedBoxColor() );
    twoColor = SAESGUIConfigurationManager::ConvertToGdkColor ( config.getSelectedBoxColor() );

    setWidget ( frame );
    signalEnableButtons.getSyncSignal().connect ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::onRawObject ) );
    setAccessibleName ( "AlarmBoxActionsWidget" );
}

AlarmBoxActionsWidget::~AlarmBoxActionsWidget()
{
    remove();

    dispose();

    if ( frame )
    {
        delete frame;
        frame = NULL;
    }

    if ( alignment )
    {
        delete alignment;
        alignment = NULL;
    }

    if ( vboxOptions )
    {
        delete vboxOptions;
        vboxOptions = NULL;
    }

    if ( buttonBuzzer )
    {
        delete buttonBuzzer;
        buttonBuzzer = NULL;
    }

    if ( buttonRedLed )
    {
        delete buttonRedLed;
        buttonRedLed = NULL;
    }

    if ( buttonGreenLed )
    {
        delete buttonGreenLed;
        buttonGreenLed = NULL;
    }

    if ( endWidget )
    {
        delete endWidget;
        endWidget = NULL;
    }

    if ( colorFrame )
    {
        delete colorFrame;
        colorFrame = NULL;
    }

    if ( colorBox )
    {
        delete colorBox;
        colorBox = NULL;
    }
}

void AlarmBoxActionsWidget::initialize()
{
    connAlarm.disconnect();
    connOk.disconnect();
    connError.disconnect();

    setOkButtonSensitive ( true );
    setCancelButtonSensitive ( true );

    buzzerState = OFF;
    redLedState = OFF;
    greenLedState = OFF;
    buttonState = OFF;

    if ( !AlarmPolling::getInstance()->isRunning() )
    {
        AlarmPolling::getInstance()->start ( 100 );
    }
    else
    {
        AlarmPolling::getInstance()->resume ();
    }

    connAlarm = AlarmPolling::getInstance()->getSignalPressButton().connect ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::onButtonPressed ) );

    connOk = signalOkDetected.connect ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::viewSuccess ) );
    connError = signalCancelDetected.connect ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::viewCancel ) );

    buzzerOption->setTextConteiner ( _ ( "Buzzer test" ) );
    redLedOption->setTextConteiner ( _ ( "Red Led test" ) );
    greenLedOption->setTextConteiner ( _ ( "Green Led test" ) );

    instructionLabel->set_text ( _ ( "Please press on the button on the Alarmbox device to test. The box\non the left should change color." ) );

    buzzerOption->setTextOptions ( _ ( "Does the buzzer work correctly?" ), _ ( "yes" ), _ ( "no" ) );
    redLedOption->setTextOptions ( _ ( "Does the Red Led work correctly?" ), _ ( "yes" ), _ ( "no" ) );
    greenLedOption->setTextOptions ( _ ( "Does the Green Led work correctly?" ), _ ( "yes" ), _ ( "no" ) );
    buttonOption->setTextOptions ( _ ( "Does the Box change color with\neach press of the button?" ), _ ( "yes" ), _ ( "no" ) );

    buttonOption->setSensitive ( true );

    buttonBuzzer->set_label ( _ ( "Buzzer ON" ) );
    buttonRedLed->set_label ( _ ( "Red Led ON" ) );
    buttonGreenLed->set_label ( _ ( "Green Led ON" ) );
    colorBox->modify_bg ( Gtk::STATE_NORMAL, buttonState == OFF ? oneColor : twoColor );

    buzzerOption->clearOption();
    redLedOption->clearOption();
    greenLedOption->clearOption();
    buttonOption->clearOption();

    buzzerOption->setTextOptions ( _ ( "Please press the button on the left to\nturn on the alarmbox buzzer." ), _ ( "yes" ), _ ( "no" ) );
    buzzerOption->viewOptions ( false );

    buttonRedLed->set_sensitive ( false );
    buttonGreenLed->set_sensitive ( false );
    buttonBuzzer->set_sensitive ( false );

    validate();
    start ( 100 );
}

bool AlarmBoxActionsWidget::process ()
{
    bool enableButton = !showBuzzerButton || !showRedLedButton || !showGreenLedButton;

    if ( !showBuzzerButton )
    {
        showBuzzerButton = true;

        if ( buzzerState == OFF )
        {
            buzzerState = ON;
            AlarmController::getInstance()->turnOnBuzzer ( periodBuzzer );
        }
        else
        {
            buzzerState = OFF;
            AlarmController::getInstance()->turnOffBuzzer();
        }
    }

    if ( !showRedLedButton )
    {
        showRedLedButton = true;

        if ( redLedState == OFF )
        {
            redLedState = ON;
            AlarmController::getInstance()->turnOnRedLed ( periodRedLed );
        }
        else
        {
            redLedState = OFF;
            AlarmController::getInstance()->turnOffRedLed();
        }
    }

    if ( !showGreenLedButton )
    {
        showGreenLedButton = true;

        if ( greenLedState == OFF )
        {
            greenLedState = ON;
            AlarmController::getInstance()->turnOnGreenLed ( periodGreenLed );
        }
        else
        {
            greenLedState = OFF;
            AlarmController::getInstance()->turnOffGreenLed();
        }
    }

    if ( enableButton )
    {
        signalEnableButtons.emit ( NULL );
    }

    return true;
}

void AlarmBoxActionsWidget::dispose()
{
    buttonBuzzer->set_sensitive ( false );
    buttonRedLed->set_sensitive ( false );
    buttonGreenLed->set_sensitive ( false );

    showButtons ( false, false, false );
    AlarmPolling::getInstance()->pause();

    connAlarm.disconnect();
    connOk.disconnect();
    connError.disconnect();

    stop();
}

void AlarmBoxActionsWidget::onRawObject ( void * object )
{
    buttonRedLed->set_sensitive ( true );
    buttonGreenLed->set_sensitive ( true );
    buttonBuzzer->set_sensitive ( true );
}

void AlarmBoxActionsWidget::onButtonPressed ( AlarmPolling::AlarmButtonStatus status )
{
    if ( status == AlarmPolling::PRESSED )
    {
        buttonOption->setSensitive();
        buttonState = buttonState == OFF ? ON : OFF;

        colorBox->modify_bg ( Gtk::STATE_NORMAL, buttonState == OFF ? oneColor : twoColor );

        validate();
    }
}

void AlarmBoxActionsWidget::onRedLedAction ()
{
    buttonRedLed->set_label ( redLedState == OFF ? _ ( "Red Led OFF" ) : _ ( "Red Led ON" ) );
    buttonRedLed->set_sensitive ( false );
    showRedLedButton = false;
}

void AlarmBoxActionsWidget::onGreenLedAction ()
{
    buttonGreenLed->set_label ( buzzerState == OFF ? _ ( "Green Led OFF" ) : _ ( "Green Led ON" ) );
    buttonGreenLed->set_sensitive ( false );
    showGreenLedButton = false;
}

void AlarmBoxActionsWidget::onBuzzerAction ()
{
    buzzerOption->viewOptions();
    buzzerOption->setTextOptions ( _ ( "Does the buzzer work correctly?" ), _ ( "yes" ), _ ( "no" ) );
    buttonBuzzer->set_label ( buzzerState == OFF ? _ ( "Buzzer OFF" ) : _ ( "Buzzer ON" ) );
    buttonBuzzer->set_sensitive ( false );

    showBuzzerButton = false;
}

void AlarmBoxActionsWidget::validate()
{
    bool result = buzzerOption->getOption() &&
                  redLedOption->getOption() &&
                  greenLedOption->getOption() &&
                  buttonOption->getOption();

    bool check = buzzerOption->isCheck() &&
                 redLedOption->isCheck() &&
                 greenLedOption->isCheck() &&
                 buttonOption->isCheck();

    showButtons ( result, !result, false );

    setCancelButtonSensitive ( check );
}

void AlarmBoxActionsWidget::end ()
{
    AlarmController::getInstance()->shutDownAlarm();
    emit();
}

void AlarmBoxActionsWidget::error ()
{
    AlarmController::getInstance()->shutDownAlarm();
    on_stop_signal_emit().emit();
}

void AlarmBoxActionsWidget::viewSuccess ()
{
    showButtons ( false, false, false );
    setWidget ( endWidget );
    endWidget->showSuccess();
    setStatus();

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::end ), timeOk );
}

void AlarmBoxActionsWidget::viewCancel ()
{
    showButtons ( false, false, false );
    setWidget ( endWidget );
    endWidget->showError();
    setStatus();

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &AlarmBoxActionsWidget::error ), timeFail );
}

void AlarmBoxActionsWidget::setStatus()
{
    bool buzzer = buzzerOption->getOption();
    bool redLed = redLedOption->getOption();
    bool greenLed = greenLedOption->getOption();
    bool button = buttonOption->getOption();

    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setAlarmBoxFullStatus (
        true,
        buzzer,
        redLed,
        greenLed,
        button );
}
