/**
 * @file	CalibrationDiagnosisWidget.cxx
 * @brief	calibration diagnosis widget class.
 * @author	Yadickson Soto
 * @date	10/09/2013
 */

#include "CalibrationDiagnosisWidget.hxx"
#include "System/Sounds/smartmatic-sounds-manager.h"
#include "Configuration/label-configuration.h"
#include <Configuration/SaesGUIConfiguration-schema.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Devices/P168AController.hxx>
#include <Operation/Devices/PDBController.hxx>
#include <System/Runtime/ProcessLauncher.h>
#include <System/Runtime/PowerDown.hxx>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>
#include <Gui/Widgets/FactoryMethodWidget.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Voting::Status;

ISMTTLog * CalibrationDiagnosisWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUi.Widgets.CalibrationDiagnosisWidget", GETTEXT_PACKAGE );

CalibrationDiagnosisWidget::CalibrationDiagnosisWidget ()
    : BasicDiagnosticWidget ( true, true, true )
{
    reports = 0;
    labelCal = NULL;
    framePrincipal = NULL;
    bottomBox = NULL;
    side1 = NULL;
    side2 = NULL;
    middle = NULL;
    button1 = NULL;
    button2 = NULL;
    button3 = NULL;
    button4 = NULL;
    button5 = NULL;
    hbuttonbox1 = NULL;
    hbuttonbox2 = NULL;
    messageLabel = NULL;
    labelResult = NULL;
    imageCalib = NULL;
    infoCalibrationWidget = NULL;
    m_exportThread = NULL;
    printer = NULL;
    printerWidgetSet = false;

    printerWidget = AbstractFactoryWidget::createGenericPrintingWidget();
    printerWidget->getSignalEnd().connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::reportPrinting ) );
    printerWidget->getSignalError().connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::endPrinting ) );
    printerWidget->getSignalErrorPrinting().connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::errorPrinting ) );

    infoCalibrationWidget = FactoryMethodWidget::createProgressWidget();

    signalOkDetected.connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::onCalibrate ) );
    signalRetryDetected.connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::onRetry ) );

    TouchScreenCalibrationConfiguration & config ( SAESGUIConfigurationManager::getInstance()->getTouchScreenCalibrationConfiguration() );

    diagnostic = AbstractFactoryWidget::createAppCalibrationWidget();
    diagnostic->getSignalEnd().connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::onEndCalibration ) );

    std::string machineModel;
    OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);
    if (StringUtils::toUpper(machineModel) == "VM-A4517")
    {
        enableRetry = false;
    }
    else
    {
        enableRetry = config.getForceCalibration();
    }
    timerOk = config.getTimeOk();
    timerFail = config.getTimeFail();
    timerAutoCalibration = config.getTimeAutoCalibration();
    timerTimeOut = 10;
    timerInfoMessage = 5000;

    Glib::RefPtr<Gtk::Builder> builder;

    try
    {
        builder = Gtk::Builder::create_from_file ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/calibration_diagnosis.glade" ) );
    }
    catch ( const Glib::FileError & ex )
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__ << " " << ex.what();
        logger->Debug ( ss.str() );
    }

    builder->get_widget ( "frame", framePrincipal );
    builder->get_widget ( "hbox1", bottomBox );
    builder->get_widget ( "vbox1", vbox1 );
    builder->get_widget ( "vbox2", vbox2 );
    builder->get_widget ( "side1", side1 );
    builder->get_widget ( "side2", side2 );
    builder->get_widget ( "middle", middle );
    builder->get_widget ( "labelCal", labelCal );
    builder->get_widget ( "hbuttonbox1", hbuttonbox1 );
    builder->get_widget ( "hbuttonbox2", hbuttonbox2 );
    builder->get_widget_derived ( "button1", button1 );
    builder->get_widget_derived ( "button2", button2 );
    builder->get_widget_derived ( "button3", button3 );
    builder->get_widget_derived ( "button4", button4 );
    builder->get_widget_derived ( "button5", button5 );

    button1->set_no_show_all();
    button2->set_no_show_all();
    button3->set_no_show_all();
    button4->set_no_show_all();
    button5->set_no_show_all();

    button1->SetSound ( Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->getDiagnosisSound() );
    button2->SetSound ( Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->getDiagnosisSound() );
    button3->SetSound ( Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->getDiagnosisSound() );
    button4->SetSound ( Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->getDiagnosisSound() );
    button5->SetSound ( Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->getDiagnosisSound() );

    button1->signal_clicked().connect ( sigc::mem_fun ( this, &CalibrationDiagnosisWidget::on_button1_clicked ) );
    button2->signal_clicked().connect ( sigc::mem_fun ( this, &CalibrationDiagnosisWidget::on_button2_clicked ) );
    button3->signal_clicked().connect ( sigc::mem_fun ( this, &CalibrationDiagnosisWidget::on_button3_clicked ) );
    button4->signal_clicked().connect ( sigc::mem_fun ( this, &CalibrationDiagnosisWidget::on_button4_clicked ) );
    button5->signal_clicked().connect ( sigc::mem_fun ( this, &CalibrationDiagnosisWidget::on_button5_clicked ) );

    button1->SetLabel ( "" );
    button2->SetLabel ( "" );
    button3->SetLabel ( "" );
    button4->SetLabel ( "" );
    button5->SetLabel ( "" );

    int HeightOfButton = config.getHeightOfButtons();
    int WidthOfButton = config.getWidthOfButtons();

    vbox1->set_size_request ( WidthOfButton, -1 );
    vbox2->set_size_request ( WidthOfButton, -1 );

    button1->set_size_request ( WidthOfButton, HeightOfButton );
    button2->set_size_request ( WidthOfButton, HeightOfButton );
    button3->set_size_request ( WidthOfButton, HeightOfButton );
    button4->set_size_request ( WidthOfButton, HeightOfButton );
    button5->set_size_request ( WidthOfButton, HeightOfButton );

    side1->set_events ( Gdk::BUTTON_PRESS_MASK );
    side2->set_events ( Gdk::BUTTON_PRESS_MASK );
    middle->set_events ( Gdk::BUTTON_PRESS_MASK );

    hbuttonbox2->set_size_request ( 40, 40 );
    middle->show_all_children();
    middle->set_visible ( true );

    side1->signal_button_release_event().connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::on_zone_clicked ) );
    side2->signal_button_release_event().connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::on_zone_clicked ) );
    middle->signal_button_release_event().connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::on_zone_clicked ) );

    messageLabel = new Smartmatic::GUI::Widgets::SmartmaticBasicLabel ();

    LabelConfiguration diagConf =  SAESGUIConfigurationManager::getInstance()->GetDiagnosisLabelConfiguration();
    LabelConfiguration::SetLabelConfiguration ( messageLabel, diagConf );
    LabelConfiguration::SetLabelConfiguration ( labelCal, diagConf );

    setWidget ( framePrincipal );
}

CalibrationDiagnosisWidget::~CalibrationDiagnosisWidget()
{
    remove();

    if ( framePrincipal != NULL )
    {
        delete ( framePrincipal );
        framePrincipal = NULL;
    }

    if ( labelCal != NULL )
    {
        delete ( labelCal );
        labelCal = NULL;
    }

    if ( labelResult != NULL )
    {
        delete ( labelResult );
        labelResult = NULL;
    }

    if ( hbuttonbox1 != NULL )
    {
        delete ( hbuttonbox1 );
        hbuttonbox1 = NULL;
    }

    if ( hbuttonbox2 != NULL )
    {
        delete ( hbuttonbox2 );
        hbuttonbox2 = NULL;
    }

    if ( imageCalib != NULL )
    {
        delete ( imageCalib );
        imageCalib = NULL;
    }

    if ( bottomBox != NULL )
    {
        delete ( bottomBox );
        bottomBox = NULL;
    }

    if ( side1 != NULL )
    {
        delete ( side1 );
        side1 = NULL;
    }

    if ( side2 != NULL )
    {
        delete ( side2 );
        side2 = NULL;
    }

    if ( middle != NULL )
    {
        delete ( middle );
        middle = NULL;
    }

    if ( button1 != NULL )
    {
        delete ( button1 );
        button1 = NULL;
    }

    if ( button2 != NULL )
    {
        delete ( button2 );
        button2 = NULL;
    }

    if ( button3 != NULL )
    {
        delete ( button3 );
        button3 = NULL;
    }

    if ( button4 != NULL )
    {
        delete ( button4 );
        button4 = NULL;
    }

    if ( button5 != NULL )
    {
        delete ( button5 );
        button5 = NULL;
    }

    if ( messageLabel != NULL )
    {
        delete ( messageLabel );
        messageLabel = NULL;
    }

    if ( printer != NULL )
    {
        delete ( printer );
        printer = NULL;
    }

    if ( infoCalibrationWidget != NULL )
    {
        delete ( infoCalibrationWidget );
        infoCalibrationWidget = NULL;
    }

    if ( m_exportThread )
    {
        m_exportThread->join();
        m_exportThread = NULL;
    }
}

void CalibrationDiagnosisWidget::initialize()
{
    dispose();

    TouchScreenCalibrationConfiguration & config ( SAESGUIConfigurationManager::getInstance()->getTouchScreenCalibrationConfiguration() );

    if ( config.getNumberReportPrinting().present() )
    {
        reports = config.getNumberReportPrinting().get();
    }
    else
    {
        reports = 1;
    }

    if ( config.getTimeOutCalibrationFailInSeconds().present() )
    {
        timerTimeOut = config.getTimeOutCalibrationFailInSeconds().get();
    }
    else
    {
        timerTimeOut = 10; ///< 10 seconds
    }

    if ( config.getTimeInfoCalibration().present() )
    {
        timerInfoMessage = config.getTimeInfoCalibration().get();
    }
    else
    {
        timerInfoMessage = 5000; ///< 5 seconds
    }

    showButtons ( false, false, false );

    labelCal->set_label ( _ ( "Smartmatic.SAES.GUI.Widgets.CalibrationDiagnosisWidget.PressButtonToTestCalibration" ) );
    setOkButtonLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.CalibrationDiagnosisWidget.Calibrate" ) );
    setRetryButtonLabel ( _ ( "Retry Calibration Test" ) );

    infoCalibrationWidget->setInfoTitle ( N_ ( "Calibration Instructions" ) );
    infoCalibrationWidget->setInfoText ( N_ ( "Please press the target on the screen, just once in each corner.\nBe patient after each press, the target takes some\ntime before moving." ) );
    infoCalibrationWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SCREEN_CALIBRATION_INSTRUCTION );

    button1->set_visible ( true );
    button2->set_visible ( false );
    button3->set_visible ( false );
    button4->set_visible ( false );
    button5->set_visible ( false );

    printerWidgetSet = false;

    restartTimeOut();

    setWidget ( framePrincipal );
}

void CalibrationDiagnosisWidget::getFrameFail()
{
    dispose();

    showButtons ( false, false, true );
    setOkButtonSensitive ( true );
    setRetryButtonSensitive ( false );

    if ( enableRetry && timerTimeOut > 0 )
    {
        setRetryButtonSensitive ( true );
        counterTimeOut = timerTimeOut;

        timeOutMessage();
        connTimeout = Glib::signal_timeout().connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::timeOutMessage ), 1000, 0 );
    }
    else
    {
        messageLabel->set_label ( _ ( "Calibration Failed, please press retry button or shutdown machine" ) );
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::error ) , timerFail );
    }

    setWidget ( messageLabel );
}

void CalibrationDiagnosisWidget::getFrameSuccess()
{
    dispose();

    showButtons ( false, false, false );

    messageLabel->set_label ( _ ( "Smartmatic.SAES.GUI.Widgets.CalibrationDiagnosisWidget.CalibrationSucceded" ) );

    setWidget ( messageLabel );

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::end ) , timerOk );
}

void CalibrationDiagnosisWidget::on_button1_clicked()
{
    restartTimeOut();
    button1->set_visible ( false );
    button2->set_visible ( true );
}
void CalibrationDiagnosisWidget::on_button2_clicked()
{
    restartTimeOut();
    button2->set_visible ( false );
    button3->set_visible ( true );
}
void CalibrationDiagnosisWidget::on_button3_clicked()
{
    restartTimeOut();
    button3->set_visible ( false );
    button4->set_visible ( true );
}

void CalibrationDiagnosisWidget::on_button4_clicked()
{
    restartTimeOut();
    button4->set_visible ( false );
    button5->set_visible ( true );
}

void  CalibrationDiagnosisWidget::on_button5_clicked()
{
    button5->set_visible ( false );
    getFrameSuccess();
}

bool CalibrationDiagnosisWidget::on_zone_clicked ( GdkEventButton * event )
{
    getFrameFail();
    return true;
}

void CalibrationDiagnosisWidget::dispose()
{
    connInfoMessage.disconnect();
    connTimeout.disconnect();
    connAuto.disconnect();
}

void CalibrationDiagnosisWidget::end()
{
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setDiagnosisStatus ( DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION, VMStatus::VOTING_MACHINE_STATUS_OK, false );
    emit();
}

void CalibrationDiagnosisWidget::exportResults()
{
    MachineOperationManager::getInstance()->getOperationStatus()->persistDiagnosticFile(false);

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::reportPrinting ), 1000 );
}

void CalibrationDiagnosisWidget::error()
{
    std::string machineModel;
    std::string serial;
    OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);
    if (StringUtils::toUpper(machineModel) == "VM-A4517")
    {
        serial = PDBController::getInstance()->getSerialNumber();
    }
    else
    {
        serial = P168AController::getInstance()->getSerialNumber();
    }
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setSerialDetected ( serial );
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setDiagnosisStatus ( DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION, VMStatus::VOTING_MACHINE_STATUS_ERROR, false );
    if (m_exportThread != NULL)
    {
        m_exportThread->join();
        m_exportThread = NULL;
    }

    m_exportThread = Glib::Thread::create(sigc::mem_fun(*this, &CalibrationDiagnosisWidget::exportResults));

}

void CalibrationDiagnosisWidget::reportPrinting()
{
    reports--;

    if ( reports >= 0 )
    {
        if (printerWidgetSet == false)
        {
            setWidget( printerWidget );
            printerWidgetSet = true;
        }

        printer = new CalibrationPrinting ( MachineOperationManager::getInstance()->getOperationStatus() );
        printerWidget->start ( printer, false, false );
    }
    else
    {
        endPrinting();
    }
}

void CalibrationDiagnosisWidget::endPrinting()
{
    setRetryButtonSensitive ( true );
}

void CalibrationDiagnosisWidget::errorPrinting()
{
    reports = 0;
}

void CalibrationDiagnosisWidget::onCalibrate()
{
    logger->Debug ( "Show info" );
    showButtons ( false, false, false );
    setWidget ( infoCalibrationWidget );

    dispose();
    connInfoMessage = Glib::signal_timeout().connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::calibrate ), timerInfoMessage, 0 );
}

void CalibrationDiagnosisWidget::onRetry()
{
    initialize();
}

bool CalibrationDiagnosisWidget::calibrate()
{
    showButtons ( false, false, false );
    setWidget ( diagnostic );

    dispose();

    diagnostic->calibrate();
    return false;
}

void CalibrationDiagnosisWidget::onEndCalibration()
{
    initialize();
}

void CalibrationDiagnosisWidget::restartTimeOut()
{
    if ( enableRetry && timerAutoCalibration > 0 )
    {
        dispose();
        connAuto = Glib::signal_timeout().connect ( sigc::mem_fun ( *this, &CalibrationDiagnosisWidget::calibrate ), timerAutoCalibration );
    }
}

bool CalibrationDiagnosisWidget::timeOutMessage()
{
    Glib::ustring composed = Glib::ustring::compose (
                                 _ ( "Calibration test failed.\nIf you think the screen is properly configured, please press\non the 'Retry' button to retry the calibration test.\n\nOtherwise, please wait, the recalibration tool will\nstart automatically in %1 seconds." ),
                                 counterTimeOut );

    messageLabel->set_label ( composed );

    counterTimeOut--;

    if ( counterTimeOut < 0 )
    {
        logger->Debug ( "Start Calibration" );
        onCalibrate();
        return false;
    }

    return true;
}

bool CalibrationDiagnosisWidget::timeInfoMessage()
{
    calibrate();
    return false;
}
