/**
 * @file	CalibrationDiagnosisWidget.hxx
 * @brief	Declares the calibration diagnosis widget class.
 * @author	Yadickson Soto
 * @date	10/09/2013
 */

#ifndef _CALIBRATION_DIAGNOSIS_WIDGET_H_
#define _CALIBRATION_DIAGNOSIS_WIDGET_H_

#include <Gui/Widgets/Diagnose/BasicDiagnosticWidget.hxx>
#include <gtkmm.h>
#include <string>
#include "Widgets/smartmatic-button.h"
#include <Voting/OperationStatus.hxx>
#include <Log/ISMTTLog.h>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Gui/Widgets/Diagnose/AppCalibrationWidget.hxx>
#include <Printing/CalibrationPrinting.hxx>
#include <Gui/Widgets/GenericPrintingWidget.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	CalibrationDiagnosisWidget
                 * @brief	Calibration diagnosis widget.
                 * @author	Yadickson Soto
                 * @date	10/09/2013
                 */

                class CalibrationDiagnosisWidget: public BasicDiagnosticWidget
                {
                    public:

                        /**
                         * @fn	CalibrationDiagnosisWidget::CalibrationDiagnosisWidget ();
                         * @brief	Constructor.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */
                        CalibrationDiagnosisWidget ();

                        /**
                         * @fn	virtual CalibrationDiagnosisWidget::~CalibrationDiagnosisWidget();
                         * @brief	Finaliser.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */

                        virtual ~CalibrationDiagnosisWidget();

                        /**
                         * @fn	void CalibrationDiagnosisWidget::dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */

                        void dispose();

                        /**
                         * @fn	void CalibrationDiagnosisWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */

                        void initialize();

                    protected:

                        /**
                         * @fn	void end();
                         * @brief	End diagnose successfully.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */
                        void end();
                        /**
                         * @fn	void error();
                         * @brief	End diagnose fail.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */
                        void error();
                        /**
                         * @fn	void onRetry();
                         * @brief	Handle button retry clicked to retry test.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */
                        void onRetry();
                        /**
                         * @fn	void onCalibrate();
                         * @brief	Handle button ok clicked to calibrate.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void onCalibrate();
                        /**
                         * @fn	bool retry();
                         * @brief	Retry and call gCal calibration.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         * @return always false, execute one time
                         */
                        bool calibrate();
                        /**
                         * @fn	void restartTimeOut();
                         * @brief	Restart calibration timeout
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */
                        void restartTimeOut();
                        /**
                         * @fn	void onOkClicked();
                         * @brief	Handles button end calibration.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */
                        void onEndCalibration();
                        /**
                         * @fn	void endPrinting();
                         * @brief	End printing error report.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */
                        void endPrinting();
                        /**
                         * @fn	void reportPrinting();
                         * @brief	Start report printing.
                         * @author	Yadickson Soto
                         * @date	03/10/2013
                         */
                        void reportPrinting();

                        bool timeOutMessage ();
                        bool timeInfoMessage();

                    private:

                        Gtk::Frame * framePrincipal; ///< The frame principal
                        Gtk::HBox * bottomBox;  ///< The bottom box
                        Gtk::VBox * vbox1;  ///< The first vbox
                        Gtk::VBox * vbox2;  ///< The second vbox
                        Gtk::HButtonBox * hbuttonbox1;  ///< The first hbuttonbox
                        Gtk::HButtonBox * hbuttonbox2;  ///< The second hbuttonbox
                        Gtk::EventBox * side1;  ///< The first side
                        Gtk::EventBox * side2;  ///< The second side
                        Gtk::EventBox * middle; ///< The middle
                        Gtk::Label * labelCal;  ///< The label cal
                        Gtk::Image * imageCalib; ///< The image calib
                        Gtk::Label * labelResult;	///< The label result
                        Glib::Thread* m_exportThread; ///< Export thread

                        int reports; ///< reports to print
                        bool enableRetry; ///< The flag enable retry calibration
                        int timerOk; ///< The timer diagnose successfully
                        int timerFail; ///< The timer diagnose fail
                        int timerAutoCalibration; ///< The timer force auto diagnose
                        int timerTimeOut; ///< timer time out in seconds
                        int timerInfoMessage; ///< timer info message
                        int counterTimeOut; ///< timer counter time out

                        sigc::connection connAuto; ///< The connection auto diagnostic
                        sigc::connection connTimeout; ///< The connection time out
                        sigc::connection connInfoMessage; ///< The connection time info message

                        Smartmatic::GUI::Widgets::SmartmaticButton * button1;	///< The first button
                        Smartmatic::GUI::Widgets::SmartmaticButton * button2;	///< The second button
                        Smartmatic::GUI::Widgets::SmartmaticButton * button3;	///< The third button
                        Smartmatic::GUI::Widgets::SmartmaticButton * button4;	///< The fourth button
                        Smartmatic::GUI::Widgets::SmartmaticButton * button5;	///< The fifth button
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * messageLabel; ///< The message question

                        Smartmatic::SAES::GUI::Widgets::AppCalibrationWidget * diagnostic; ///< The calibration tool
                        Smartmatic::SAES::GUI::Widgets::GenericPrintingWidget * printerWidget; ///< The printer widget
                        Smartmatic::SAES::GUI::Widgets::ProgressWidget * infoCalibrationWidget; ///< The info calibration widget
                        Smartmatic::SAES::Printing::CalibrationPrinting * printer; ///< printer

                        /**
                         * @fn	bool CalibrationDiagnosisWidget::on_zone_clicked ( GdkEventButton * event );
                         * @brief	Executes the zone clicked action.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         * @param	event	If non-null, the event.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool  on_zone_clicked ( GdkEventButton * event );

                        /**
                         * @fn	void CalibrationDiagnosisWidget::on_button1_clicked();
                         * @brief	Handles button 1 clicked signals.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */

                        void  on_button1_clicked();

                        /**
                         * @fn	void CalibrationDiagnosisWidget::on_button2_clicked();
                         * @brief	Handles button 2 clicked signals.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */

                        void  on_button2_clicked();

                        /**
                         * @fn	void CalibrationDiagnosisWidget::on_button3_clicked();
                         * @brief	Handles button 3 clicked signals.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */

                        void  on_button3_clicked();

                        /**
                         * @fn	void CalibrationDiagnosisWidget::on_button4_clicked();
                         * @brief	Handles button 4 clicked signals.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */

                        void  on_button4_clicked();

                        /**
                         * @fn	void CalibrationDiagnosisWidget::on_button5_clicked();
                         * @brief	Handles button 5 clicked signals.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */

                        void  on_button5_clicked();

                        /**
                         * @fn	void CalibrationDiagnosisWidget::getFrameSuccess();
                         * @brief	Gets the frame success.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */

                        void getFrameSuccess();

                        /**
                         * @fn	void CalibrationDiagnosisWidget::getFrameFail();
                         * @brief	Gets the frame fail.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */

                        void getFrameFail();

                        /**
                         * @brief  Thread where the results are exported.
                         * @author manuel.arguelles
                         * @date   Dec  4, 2017
                         */
                        void exportResults();

                        /**
                         * @fn void CalibrationDiagnosisWidget::errorPrinting();
                         * @brief  Method to show the error printing message.
                         * @author eduardo.fernandez
                         * @date   Mar 5, 2018
                         */
                        void errorPrinting();

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                        bool printerWidgetSet;
                };

            }
        }
    }
}

#endif // _CALIBRATION_DIAGNOSIS_WIDGET_H_
