/**
 * @file DateDiagnosticWidget.cxx
 * @brief Body class Login widget
 * @date 19/09/2012
 * @author Daniel Ampuero
 */

#include <string>
#include <vector>
#include <gtkmm.h>
#include <ctime>
#include <cctype>
#include <cstring>
#include <cstdlib>

#include "DateDiagnosticWidget.hxx"
#include <Environment/Configurations/resource-path-provider.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineDate.hxx>
#include <System/Utils/StringUtils.hxx>
#include "Configuration/label-configuration.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Configuration/s-m-t-t-g-u-i-conf-manager.h"
#include <System/Utils/DateTimeUtils.hxx>
#include <Configuration/MachineModelEnum.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::Configuration;

static const std::string INITIAL_DAY   = "01";
static const std::string INITIAL_MONTH = "01";

ISMTTLog * DateDiagnosticWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.DateDiagnosticWidget", GETTEXT_PACKAGE );

DateDiagnosticWidget::DateDiagnosticWidget ( Smartmatic::SAES::Configuration::DateDiagnosticWidgetConfiguration & _config )
    : BasicDiagnosticWidget ( true )
{
    this->config = _config;
    this->mainFrame = NULL;
    this->tableDate = NULL;
    this->labelTitle = NULL;
    this->labelDay = NULL;
    this->labelMonth = NULL;
    this->labelYear = NULL;
    this->entryDay = NULL;
    this->entryMonth = NULL;
    this->entryYear = NULL;
    this->buttonDayUp = NULL;
    this->buttonDayDown = NULL;
    this->buttonMonthUp = NULL;
    this->buttonMonthDown = NULL;
    this->buttonYearUp = NULL;
    this->buttonYearDown = NULL;
    this->buttonArrowWidth = config.getButtonArrowWidth();
    this->buttonArrowHeight = config.getButtonArrowHeight();
    this->entryWidth = config.getEntryWidth();
    this->entryHeight = config.getEntryHeight();
    this->minYear = config.getMinYear();
    this->maxYear = config.getMaxYear();
    this->inputFontSize = config.getInputFontSize();

    this->initialize();
    setWidget ( mainFrame );
}

DateDiagnosticWidget::~DateDiagnosticWidget()
{
    remove();

    dispose();
}

void DateDiagnosticWidget::setCurrentDate()
{
    std::string strDate = StringUtils::trim ( Smartmatic::SAES::Operation::MachineDate::getMachineDate() );

    if ( strDate == "" )
    {
        entryDay->set_text ( INITIAL_DAY );
        entryMonth->set_text ( INITIAL_MONTH );
        entryYear->set_text ( StringUtils::intToString ( minYear ) );
    }
    else
    {
        entryDay->set_text ( strDate.substr(0, 2) );
        entryMonth->set_text ( strDate.substr(3, 2) );
        entryYear->set_text ( strDate.substr(6, 4) );
    }
}

void DateDiagnosticWidget::initialize()
{
    dispose();
    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile;
    gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/DateDiagnosticWidget.glade" );

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Error ( "FATAL - DateDiagnosticWidget Initialize" + ex.what() );
    }

    builder->get_widget ( "main-frame", mainFrame );
    builder->get_widget ( "table-date", tableDate );
    builder->get_widget_derived ( "entryDay", entryDay );
    builder->get_widget_derived ( "entryMonth", entryMonth );
    builder->get_widget_derived ( "entryYear", entryYear );
    builder->get_widget_derived ( "buttonDayUp", buttonDayUp );
    builder->get_widget_derived ( "buttonDayDown", buttonDayDown );
    builder->get_widget_derived ( "buttonMonthUp", buttonMonthUp );
    builder->get_widget_derived ( "buttonMonthDown", buttonMonthDown );
    builder->get_widget_derived ( "buttonYearUp", buttonYearUp );
    builder->get_widget_derived ( "buttonYearDown", buttonYearDown );
    builder->get_widget ( "label-title", labelTitle );
    builder->get_widget ( "label-day", labelDay );
    builder->get_widget ( "label-month", labelMonth );
    builder->get_widget ( "label-year", labelYear );

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        double guiReconfigurationFactor = SMTTGUIConfigurationManager::getInstance()->getConfiguration()->getSmartmaticGUIMachineModelConfiguration().getGuiReconfigurationFactor();

        this->buttonArrowWidth *= guiReconfigurationFactor;
        this->buttonArrowHeight *= guiReconfigurationFactor;
        this->entryWidth *= guiReconfigurationFactor;
        this->entryHeight *= guiReconfigurationFactor;
    }

    //  Buttons Up and Down configuration
    buttonDayUp->SetButtonType ( SmartmaticButton::ArrowTop );
    buttonDayUp->set_size_request ( buttonArrowWidth, buttonArrowHeight );
    buttonDayUp->signal_clicked().connect ( sigc::mem_fun ( this, &DateDiagnosticWidget::on_buttonDayUp_clicked ) );
    buttonDayDown->SetButtonType ( SmartmaticButton::ArrowBottom );
    buttonDayDown->set_size_request ( buttonArrowWidth, buttonArrowHeight );
    buttonDayDown->signal_clicked().connect ( sigc::mem_fun ( this, &DateDiagnosticWidget::on_buttonDayDown_clicked ) );
    buttonMonthUp->SetButtonType ( SmartmaticButton::ArrowTop );
    buttonMonthUp->set_size_request ( buttonArrowWidth, buttonArrowHeight );
    buttonMonthUp->signal_clicked().connect ( sigc::mem_fun ( this, &DateDiagnosticWidget::on_buttonMonthUp_clicked ) );
    buttonMonthDown->SetButtonType ( SmartmaticButton::ArrowBottom );
    buttonMonthDown->set_size_request ( buttonArrowWidth, buttonArrowHeight );
    buttonMonthDown->signal_clicked().connect ( sigc::mem_fun ( this, &DateDiagnosticWidget::on_buttonMonthDown_clicked ) );
    buttonYearUp->SetButtonType ( SmartmaticButton::ArrowTop );
    buttonYearUp->set_size_request ( buttonArrowWidth, buttonArrowHeight );
    buttonYearUp->signal_clicked().connect ( sigc::mem_fun ( this, &DateDiagnosticWidget::on_buttonYearUp_clicked ) );
    buttonYearDown->SetButtonType ( SmartmaticButton::ArrowBottom );
    buttonYearDown->set_size_request ( buttonArrowWidth, buttonArrowHeight );
    buttonYearDown->signal_clicked().connect ( sigc::mem_fun ( this, &DateDiagnosticWidget::on_buttonYearDown_clicked ) );

    labelTitle->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.DateDiagnosticWidget.ComfirmDate" ) );
    labelDay->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.DateDiagnosticWidget.Day" ) );
    labelMonth->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.DateDiagnosticWidget.Month" ) );
    labelYear->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.DateDiagnosticWidget.Year" ) );

    LabelConfiguration titleConf = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getWidgetTitleLabelConfiguration() );
    LabelConfiguration::SetLabelConfiguration ( labelTitle, titleConf );
    LabelConfiguration dateEntryConf = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getMonthYearLabelConfiguration() );
    LabelConfiguration::SetLabelConfiguration ( labelDay, dateEntryConf );
    LabelConfiguration::SetLabelConfiguration ( labelMonth, dateEntryConf );
    LabelConfiguration::SetLabelConfiguration ( labelYear, dateEntryConf );

    setCurrentDate();

    Pango::FontDescription font;
    font.set_size ( inputFontSize * Pango::SCALE );

    entryDay->set_size_request( entryWidth, entryHeight );
    entryDay->modify_font( font );
    entryMonth->set_size_request( entryWidth, entryHeight );
    entryMonth->modify_font( font );
    entryYear->set_size_request( entryWidth, entryHeight );
    entryYear->modify_font( font );

    signalOkDetected.connect ( sigc::mem_fun ( *this, &DateDiagnosticWidget::end ) );

    validate();
    show_all_children();
}

void DateDiagnosticWidget::dispose()
{
    if ( mainFrame )
    {
        delete mainFrame;
        mainFrame = NULL;
    }

    if ( tableDate )
    {
        delete tableDate;
        tableDate = NULL;
    }

    if ( labelTitle )
    {
        delete labelTitle;
        labelTitle = NULL;
    }

    if ( labelDay )
    {
        delete labelDay;
        labelDay = NULL;
    }

    if ( labelMonth )
    {
        delete labelMonth;
        labelMonth = NULL;
    }

    if ( labelYear )
    {
        delete labelYear;
        labelYear = NULL;
    }

    if ( entryDay )
    {
        delete entryDay;
        entryDay = NULL;
    }

    if ( entryMonth )
    {
        delete entryMonth;
        entryMonth = NULL;
    }

    if ( entryYear )
    {
        delete entryYear;
        entryYear = NULL;
    }

    if ( buttonDayUp )
    {
        delete buttonDayUp;
        buttonDayUp = NULL;
    }

    if ( buttonDayDown )
    {
        delete buttonDayDown;
        buttonDayDown = NULL;
    }

    if ( buttonMonthUp )
    {
        delete buttonMonthUp;
        buttonMonthUp = NULL;
    }

    if ( buttonMonthDown )
    {
        delete buttonMonthDown;
        buttonMonthDown = NULL;
    }

    if ( buttonYearUp )
    {
        delete buttonYearUp;
        buttonYearUp = NULL;
    }

    if ( buttonYearDown )
    {
        delete buttonYearDown;
        buttonYearDown = NULL;
    }
}

void DateDiagnosticWidget::getDayText ( std::string & text )
{
    text = entryDay->getText();
}

void DateDiagnosticWidget::getMonthText ( std::string & text )
{
    text = entryMonth->getText();
}

void DateDiagnosticWidget::getYearText ( std::string & text )
{
    text = entryYear->getText();
}

std::string DateDiagnosticWidget::checkStringAndAddZero ( std::string input )
{
    if ( input.size() < 2 )
    {
        input.insert ( 0, "0" );
    }

    return input;
}

void DateDiagnosticWidget::on_buttonDayUp_clicked()
{
    int month = StringUtils::stringToInt ( entryMonth->get_text() );
    int day = StringUtils::stringToInt ( entryDay->get_text() );
    int year = StringUtils::stringToInt ( entryYear->get_text() );
    bool isEvenYear = false;
    bool alreadySet = false;

    if ( ( year % 4 == 0 ) && ( ( year % 100 != 0 ) || ( year % 400 == 0 ) ) )
    {
        isEvenYear = true;
    }

    switch ( month )
    {
        case 1:
        case 3:
        case 5:
        case 7:
        case 8:
        case 10:
        case 12:

            if ( day == 31 )
            {
                day = 1;
                alreadySet = true;
            }

            break;

        case 2:

            if ( isEvenYear )
            {
                if ( day == 29 )
                {
                    day = 1;
                    alreadySet = true;
                }
            }
            else
            {
                if ( day == 28 )
                {
                    day = 1;
                    alreadySet = true;
                }
            }

            break;

        case 4:
        case 6:
        case 9:
        case 11:

            if ( day == 30 )
            {
                day = 1;
                alreadySet = true;
            }

            break;

        default:
            break;
    }

    if ( !alreadySet )
    {
        day++;
    }

    entryDay->set_text ( checkStringAndAddZero ( StringUtils::intToString ( day ) ) );
}

void DateDiagnosticWidget::on_buttonDayDown_clicked()
{
    int month = StringUtils::stringToInt ( entryMonth->get_text() );
    int day = StringUtils::stringToInt ( entryDay->get_text() );
    int year = StringUtils::stringToInt ( entryYear->get_text() );
    bool isEvenYear = false;
    bool alreadySet = false;

    if ( ( year % 4 == 0 ) && ( ( year % 100 != 0 ) || ( year % 400 == 0 ) ) )
    {
        isEvenYear = true;
    }

    switch ( month )
    {
        case 1:
        case 3:
        case 5:
        case 7:
        case 8:
        case 10:
        case 12:

            if ( day == 1 )
            {
                day = 31;
                alreadySet = true;
            }

            break;

        case 2:

            if ( isEvenYear )
            {
                if ( day == 1 )
                {
                    day = 29;
                    alreadySet = true;
                }
            }
            else
            {
                if ( day == 1 )
                {
                    day = 28;
                    alreadySet = true;
                }
            }

            break;

        case 4:
        case 6:
        case 9:
        case 11:

            if ( day == 1 )
            {
                day = 30;
                alreadySet = true;
            }

            break;

        default:
            break;
    }

    if ( !alreadySet )
    {
        day--;
    }

    entryDay->set_text ( checkStringAndAddZero ( StringUtils::intToString ( day ) ) );
}

void DateDiagnosticWidget::on_buttonMonthUp_clicked()
{
    int month = StringUtils::stringToInt ( entryMonth->get_text() );

    if ( month == 12 )
    {
        month = 1;
    }
    else
    {
        month++;
    }

    entryMonth->set_text ( checkStringAndAddZero ( StringUtils::intToString ( month ) ) );
    int day = StringUtils::stringToInt ( entryDay->get_text() );

    if ( month == 2 )
    {
        bool isEvenYear = false;
        int year = StringUtils::stringToInt ( entryYear->get_text() );

        if ( ( year % 4 == 0 ) && ( ( year % 100 != 0 ) || ( year % 400 == 0 ) ) )
        {
            isEvenYear = true;
        }

        if ( day == 30 || day == 31 || day == 29 )
        {
            if ( isEvenYear )
            {
                day = 29;
            }
            else
            {
                day = 28;
            }
        }
    }
    else if ( ( month == 4 || month == 6 || month == 9 || month == 11 ) && ( day == 31 ) )
    {
        day = 30;
    }

    entryDay->set_text ( checkStringAndAddZero ( StringUtils::intToString ( day ) ) );
}

void DateDiagnosticWidget::on_buttonMonthDown_clicked()
{
    int month = StringUtils::stringToInt ( entryMonth->get_text() );

    if ( month == 1 )
    {
        month = 12;
    }
    else
    {
        month--;
    }

    entryMonth->set_text ( checkStringAndAddZero ( StringUtils::intToString ( month ) ) );
    int day = StringUtils::stringToInt ( entryDay->get_text() );

    if ( month == 2 )
    {
        bool isEvenYear = false;
        int year = StringUtils::stringToInt ( entryYear->get_text() );

        if ( ( year % 4 == 0 ) && ( ( year % 100 != 0 ) || ( year % 400 == 0 ) ) )
        {
            isEvenYear = true;
        }

        if ( day == 30 || day == 31 || day == 29 )
        {
            if ( isEvenYear )
            {
                day = 29;
            }
            else
            {
                day = 28;
            }
        }
    }
    else if ( ( month == 4 || month == 6 || month == 9 || month == 11 ) && ( day == 31 ) )
    {
        day = 30;
    }

    entryDay->set_text ( checkStringAndAddZero ( StringUtils::intToString ( day ) ) );
}

void DateDiagnosticWidget::on_buttonYearUp_clicked()
{
    int year = StringUtils::stringToInt ( entryYear->get_text() );

    if ( year >= maxYear )
    {
        year = minYear;
    }
    else
    {
        year++;
    }

    entryYear->set_text ( StringUtils::intToString ( year ) );
    int day = StringUtils::stringToInt ( entryDay->get_text() );
    int month = StringUtils::stringToInt ( entryMonth->get_text() );

    if ( month == 2 )
    {
        bool isEvenYear = false;

        if ( ( year % 4 == 0 ) && ( ( year % 100 != 0 ) || ( year % 400 == 0 ) ) )
        {
            isEvenYear = true;
        }

        if ( day == 30 || day == 31 || day == 29 )
        {
            if ( isEvenYear )
            {
                day = 29;
            }
            else
            {
                day = 28;
            }
        }
    }
    else if ( ( month == 4 || month == 6 || month == 9 || month == 11 ) && ( day == 31 ) )
    {
        day = 30;
    }

    entryDay->set_text ( checkStringAndAddZero ( StringUtils::intToString ( day ) ) );
}

void DateDiagnosticWidget::on_buttonYearDown_clicked()
{
    int year = StringUtils::stringToInt ( entryYear->get_text() );

    if ( year <= minYear )
    {
        year = maxYear;
    }
    else
    {
        year--;
    }

    entryYear->set_text ( StringUtils::intToString ( year ) );
    int day = StringUtils::stringToInt ( entryDay->get_text() );
    int month = StringUtils::stringToInt ( entryMonth->get_text() );

    if ( month == 2 )
    {
        bool isEvenYear = false;

        if ( ( year % 4 == 0 ) && ( ( year % 100 != 0 ) || ( year % 400 == 0 ) ) )
        {
            isEvenYear = true;
        }

        if ( day == 30 || day == 31 || day == 29 )
        {
            if ( isEvenYear )
            {
                day = 29;
            }
            else
            {
                day = 28;
            }
        }
    }
    else if ( ( month == 4 || month == 6 || month == 9 || month == 11 ) && ( day == 31 ) )
    {
        day = 30;
    }

    entryDay->set_text ( checkStringAndAddZero ( StringUtils::intToString ( day ) ) );
}

void DateDiagnosticWidget::validate()
{
    DateTimeUtils dataTime ( entryDay->getText(), entryMonth->getText(), entryYear->getText() );
    setOkButtonSensitive ( dataTime.validate() );
}

void DateDiagnosticWidget::end ()
{
    emit();
}
