/**
 * @file	DeviceResultWidget.cxx
 * @brief	device result widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "DeviceResultWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Environment/Configurations/resource-path-provider.h>
#include <Configuration/label-configuration.h>
#include <Configuration/MachineModelEnum.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Configuration;

ISMTTLog * DeviceResultWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.DeviceResultWidget", GETTEXT_PACKAGE );

DeviceResultWidget::DeviceResultWidget ( Status::DeviceStatus & diagnosis, int labelWidth, int labelHeight )
    : StepWidget(), diagnosis ( diagnosis )
{
    this->labelWidth = labelWidth;
    this->labelHeight = labelHeight;
    this->mainFrame = NULL;
    this->label = NULL;
    this->image = NULL;
    this->initialize();
    setWidget ( mainFrame );
}

DeviceResultWidget::~DeviceResultWidget()
{
    dispose();
}

void DeviceResultWidget::initialize()
{
    dispose();
    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile;
    gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/DeviceResultWidget.glade" );

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( "FATAL - DeviceResultWidget Initialize" );
    }

    builder->get_widget ( "main-frame", mainFrame );
    builder->get_widget ( "label", label );
    builder->get_widget ( "image", image );

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        double guiReconfigurationFactor = SMTTGUIConfigurationManager::getInstance()->getConfiguration()->getSmartmaticGUIMachineModelConfiguration().getGuiReconfigurationFactor();

        labelWidth  *= guiReconfigurationFactor;
        labelHeight *= guiReconfigurationFactor;
    }

    label->set_size_request ( labelWidth, labelHeight );
    LabelConfiguration labelConf = SAESGUIConfigurationManager::ConvertToLabelConfiguration (
                                       SAESGUIConfigurationManager::getInstance()->getDeviceDiagnosticResultConfiguration().getDiagnosticReviewLabelConfiguration() );
    LabelConfiguration::SetLabelConfiguration ( label, labelConf );
    update();
}

void DeviceResultWidget::dispose()
{
    this->remove();

    if ( image )
    {
        delete image;
        image = NULL;
    }

    if ( label )
    {
        delete label;
        label = NULL;
    }
}

Status::DeviceStatus & DeviceResultWidget::getDiagnosis()
{
    return diagnosis;
}

void DeviceResultWidget::update()
{
    std::string iconFile;
    std::string text;
    std::string accesibleIdentifierResult;
    std::string accesibleIdentifierTypeOfDiagnostic;
    std::ostringstream ossToConvertAccesibleIdentifier;

    switch ( diagnosis.getVMStatus().getStatus().get() )
    {
        case Status::VMStatus::VOTING_MACHINE_STATUS_OK:
            accesibleIdentifierResult="OK";
            iconFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/Ok_30.png" );
            break;

        case Status::VMStatus::VOTING_MACHINE_STATUS_ERROR:
            accesibleIdentifierResult="ERROR";
            iconFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/Error_30.png" );
            break;

        case Status::VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
        case Status::VMStatus::VOTING_MACHINE_STATUS_RUN:
        default:
            accesibleIdentifierResult="NOTDIAGNOSED";
            iconFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/Interrogation_30.png" );
            break;
    }

    switch ( diagnosis.getVMType() )
    {
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ALARMBOX:
            accesibleIdentifierTypeOfDiagnostic="AB";
            text = _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.AlarmBoxDevice" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_BARCODE:
            accesibleIdentifierTypeOfDiagnostic="BCR";
            text = _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.BarcodeReader" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_PRINTER:
            accesibleIdentifierTypeOfDiagnostic="PRINTER";
            text = _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.PrinterDevice" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_SMARTCARD:
            accesibleIdentifierTypeOfDiagnostic="SCR";
            text = _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.SmartcardReader" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_FLASH:
            accesibleIdentifierTypeOfDiagnostic="FLASH";
            text = _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.FlashDevice" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION:
            accesibleIdentifierTypeOfDiagnostic="TOUCHSCREEN";
            text = _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.TouchScreenCalibration" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_P168A:
            accesibleIdentifierTypeOfDiagnostic="P168";
            text = _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.AbleDevice" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_EURN:
            accesibleIdentifierTypeOfDiagnostic="eUrn";
            text = _ ( "eUrn Device:" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ABLED:
            accesibleIdentifierTypeOfDiagnostic="Abled";
            text = _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.AbleDevice" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_MOUSE:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_KEYBOARD:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_USBSTORAGEMOUNTED:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ETHERNET:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_UNKNOWN:
        default:
            accesibleIdentifierTypeOfDiagnostic="NOTCONFIGURED";
            break;
    }
    ossToConvertAccesibleIdentifier << "deviceStatusFor:" << accesibleIdentifierTypeOfDiagnostic << ":" << accesibleIdentifierResult;
    this->setAccessibleName (ossToConvertAccesibleIdentifier.str());
    image->set ( iconFile );
    label->set_text ( text );
}
