/**
 * @file	PrinterDiagnosisWidget.cxx
 * @brief	printer diagnosis widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "PrinterDiagnosisWidget.hxx"
#include "Configuration/label-configuration.h"
#include "Printing/PrintDocumentResult.h"
#include "Printing/PrinterStatusNotifier.h"
#include <Configuration/SaesGUIConfiguration-schema.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Configuration/s-a-e-s-functionality-conf-manager.h"
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/ImageResourcePath.hxx>
#include "Configuration/s-a-e-s-conf-manager.h"
#include <Graphics/Encoding2D/QREncoding.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Gui/Widgets/FactoryMethodWidget.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Diagnosis;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Voting::Status;

ISMTTLog * PrinterDiagnosisWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.PrinterDiagnosisWidget", GETTEXT_PACKAGE );

PrinterDiagnosisWidget::PrinterDiagnosisWidget ()
    : BasicDiagnosticWidget ( true, true, true )
{
    progressWidget = FactoryMethodWidget::createProgressWidget();

    std::string encode = SAESGUIConfigurationManager::getInstance()->getBarcodeDiagnosisConfiguration().getStringToEncode();
    bool checkPrinter = !SAESGUIConfigurationManager::getInstance()->getPrinterDiagnosisConfiguration().getDoNotCheckBeforePrintDiagnostic();

    timeOK = SAESGUIConfigurationManager::getInstance()->getPrinterDiagnosisConfiguration().getPrinterTimeOk();
    timeFail = SAESGUIConfigurationManager::getInstance()->getPrinterDiagnosisConfiguration().getPrinterTimeFail();

    diagnosis = new PrinterDiagnosis ( encode, checkPrinter );

    setWidget ( progressWidget );
}

PrinterDiagnosisWidget::~PrinterDiagnosisWidget()
{
    remove();

    connOk.disconnect();
    connError.disconnect();
    connRetry.disconnect();
    connDiag.disconnect();

    if ( diagnosis )
    {
        delete diagnosis;
        diagnosis = NULL;
    }
}

void PrinterDiagnosisWidget::initialize()
{
    progressWidget->setImage ( Operation::ImageResourcePath::PRINTING_TEST );

    connOk.disconnect();
    connError.disconnect();
    connRetry.disconnect();
    connDiag.disconnect();

    connOk = signalCancelDetected.connect ( sigc::mem_fun ( this, &PrinterDiagnosisWidget::on_signalCancel ) );
    connError = signalOkDetected.connect ( sigc::mem_fun ( this, &PrinterDiagnosisWidget::on_signalOk ) );
    connRetry = signalRetryDetected.connect ( sigc::mem_fun ( this, &PrinterDiagnosisWidget::on_signalRetry ) );
    connDiag = diagnosis->getSignalEnd().connect ( sigc::mem_fun ( *this, &PrinterDiagnosisWidget::checkEndDiagnosis ) );

    diagnosis->setStatus ( VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED );
    progressWidget->setInfoText ( diagnosis->getMessage() );

    setCancelButtonLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PrinterDiagnosisWidget.Error" ) );
    setOkButtonSensitive ( false );
    setCancelButtonSensitive ( false );

    showButtons ( false, false, false );

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &PrinterDiagnosisWidget::start ), 1000 );
}

void PrinterDiagnosisWidget::on_signalCancel()
{
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.PrinterFinishedError" ) );
    progressWidget->setInfoText ( _ ( "Smartmatic.SAES.GUI.Widgets.PrinterDiagnosisWidget.DiagnosticFailed" ) );
    progressWidget->hideInfoFooter();

    showButtons ( false, false, false );

    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setDiagnosisStatus ( DeviceType::CONFIGURATION_PRINTER, VMStatus::VOTING_MACHINE_STATUS_ERROR, false );

    if ( diagnosis->getStatus() == VMStatus::VOTING_MACHINE_STATUS_OK )
    {
        MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setStatusPrinterManualError();
    }

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &PrinterDiagnosisWidget::EmitContinueSignal ), timeFail );
}

void PrinterDiagnosisWidget::on_signalOk()
{
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.PrinterFinishedOk" ) );
    progressWidget->setInfoText ( _ ( "Smartmatic.SAES.GUI.Widgets.PrinterDiagnosisWidget.DiagnosticSuccessful" ) );
    progressWidget->hideInfoFooter();

    showButtons ( false, false, false );

    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setDiagnosisStatus ( DeviceType::CONFIGURATION_PRINTER, VMStatus::VOTING_MACHINE_STATUS_OK, false );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &PrinterDiagnosisWidget::EmitContinueSignal ), timeOK );
}

void PrinterDiagnosisWidget::start()
{
    PrinterStatusNotifier::runForcedPrintBlank();
    sleep ( 1 );
    showButtons ( false, false, false );
    diagnosis->setStatus ( VMStatus::VOTING_MACHINE_STATUS_RUN );
    progressWidget->setInfoText ( diagnosis->getMessage() );
    progressWidget->hideInfoFooter();
    diagnosis->test();
}

void PrinterDiagnosisWidget::checkEndDiagnosis ()
{
    if ( diagnosis->getStatus() == VMStatus::VOTING_MACHINE_STATUS_OK )
    {
        showButtons ( true, true, true );
        setOkButtonSensitive ( true );
        setCancelButtonSensitive ( true );
        setRetryButtonSensitive ( true );
        progressWidget->setInfoText ( _ ( "Smartmatic.SAES.GUI.Widgets.PrinterDiagnosisWidget.PressButtonAccordingResult" ) );
    }
    else
    {
        showButtons ( false, true, true );
        progressWidget->setInfoText ( diagnosis->getMessage() );

        std::string message;

        switch ( SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getInfoMachine() )
        {
            case InfoMachineConfiguration::CONFIGURATION_PM:
                message = _ ( "Smartmatic.SAES.GUI.Widgets.PrinterDiagnosisWidget.AditionalMessagePM" );
                break;

            default:
            case InfoMachineConfiguration::CONFIGURATION_VM:
                message = _ ( "Smartmatic.SAES.GUI.Widgets.PrinterDiagnosisWidget.AditionalMessageVM" );
                break;
        }

        if ( !message.empty() && message != " " )
        {
            progressWidget->showInfoFooter();
            progressWidget->setInfoFooter ( message );
        }

        setOkButtonSensitive ( false );
        setCancelButtonSensitive ( true );
        setRetryButtonSensitive ( true );
    }
}

void PrinterDiagnosisWidget::EmitContinueSignal()
{
    emit();
}

void PrinterDiagnosisWidget::on_signalRetry()
{
    setRetryButtonSensitive ( false );
    start();
}
