/**
 * @file	SmartcardActionsWidget.cxx
 * @brief	smartcard reader diagnosis widget class.
 * @author	Yadickson Soto
 * @date	11/09/2013
 */

#include "SmartcardActionsWidget.hxx"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Operation/Devices/CardController.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>
#include <Voting/CardData/CardDataFactory.hxx>
#include <Voting/VotingDevice/LogicalVoterGroupFactory.hxx>
#include <Security/KeyManager.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting::CardData;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Common::Security;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Configuration;

ISMTTLog * SmartcardActionsWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.SmartcardActionsWidget", GETTEXT_PACKAGE );

SmartcardActionsWidget::SmartcardActionsWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progressWidget )
    : BasicDiagnosticWidget ( false, true, true ), progressWidget ( progressWidget )
{
    currentStep = INSERT;
    checkRemoveInError = false;

    timeOk = SAESGUIConfigurationManager::getInstance()->getSmartcardReaderDiagnosisConfiguration().getTimeOk();
    timeFail = SAESGUIConfigurationManager::getInstance()->getSmartcardReaderDiagnosisConfiguration().getTimeFail();
    timeStep = SAESGUIConfigurationManager::getInstance()->getSmartcardReaderDiagnosisConfiguration().getTimeStep();
    timeRetry = SAESGUIConfigurationManager::getInstance()->getSmartcardReaderDiagnosisConfiguration().getWaitTimeAfterRetry();
    numberOfRetries = SAESGUIConfigurationManager::getInstance()->getSmartcardReaderDiagnosisConfiguration().getNumberOfRetry();
    currentRetries = 0;

    LabelConfiguration labelConfig = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->getSmartcardReaderDiagnosisConfiguration().getWarningMessageLabelConfiguration() );
    progressWidget->setInfoFooterConfig ( labelConfig );
    progressWidget->showInfoFooter ( true );

    signalCancelDetected.connect ( sigc::mem_fun ( *this, &SmartcardActionsWidget::viewError ) );
    signalRetryDetected.connect ( sigc::mem_fun ( *this, &SmartcardActionsWidget::viewWaitRetry ) );

    signalThread.getSyncSignal().connect ( sigc::mem_fun ( *this, &SmartcardActionsWidget::onRawObject ) );

    setWidget ( progressWidget );
}

SmartcardActionsWidget::~SmartcardActionsWidget()
{
    remove();

    if ( progressWidget )
    {
        delete progressWidget;
        progressWidget = NULL;
    }
}

void SmartcardActionsWidget::initialize()
{
    showButtons ( false, true, false );

    Smartmatic::System::SafeBuffer key;
    KeyManager::getInstance()->getKey ( KeyType::CONFIGURATION_SECRET_KEY, key );

    if ( key.GetData() == NULL )
    {
        key.AddData ( "ABCD$%&/" );
        KeyManager::getInstance()->addKeyByType ( key, KeyType::CONFIGURATION_SECRET_KEY );
    }

    currentStep = INSERT;
    checkRemoveInError = false;

    messageError = N_ ( "Smartcard Reader Diagnose Fail" );

    connChangue.disconnect();
    connChangue = CardPolling::getInstance()->getSignalCard().connect ( sigc::mem_fun ( *this, &SmartcardActionsWidget::onCardPollingStatus ) );

    if ( !CardPolling::getInstance()->isRunning() )
    {
        CardPolling::getInstance()->start ( 100 );
    }
    else
    {
        CardPolling::getInstance()->resume ();
    }

    nextStep();
}

void SmartcardActionsWidget::nextStep()
{
    switch ( ( SmartcardDiagnoseStep ) currentStep )
    {
        case INSERT:
            insert();
            break;

        case WRITE:
            writeCardData();
            break;

        case READ:
            readCardData();
            break;

        case REMOVE:
            remove();
            break;

        default:
            success();
            break;
    }
}

void SmartcardActionsWidget::insert()
{
    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_INSERT );
    progressWidget->setInfoText ( _ ( "Please Insert The Card" ) );
    progressWidget->setInfoFooter ( " " );

    showButtons ( false, true, false );
    setCancelButtonSensitive ( true );
}

void SmartcardActionsWidget::writeCardData()
{
    setCancelButtonSensitive ( false );

    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PROGRESS );
    progressWidget->setInfoText ( _ ( "Write The Card, please wait" ) );
    progressWidget->setInfoFooter ( " " );

    start ( 0 );
}

void SmartcardActionsWidget::readCardData()
{
    setCancelButtonSensitive ( false );

    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PROGRESS );
    progressWidget->setInfoText ( _ ( "Read The Card, please wait" ) );
    progressWidget->setInfoFooter ( " " );

    start ( 0 );
}

bool SmartcardActionsWidget::process()
{
    bool validate = false;

    if ( currentStep == WRITE )
    {
        Logical_voter_group clvg;
        Card_data cardData;

        clvg = LogicalVoterGroupFactory::createLogicalVoterGroup ();
        cardData = CardDataFactory::createCardData ( clvg, NULL );

        try
        {
            validate = CardController::getInstance()->writeCardData ( cardData );
        }
        catch ( ... )
        {

        }

        if ( validate )
        {
            logger->Debug ( "Write successfully" );
        }
        else
        {
            messageError = N_ ( "Smartcard Reader Write Failed" );
            messageErrorInfo = N_ ( "There was an error during the write operations." );

            logger->Debug ( "Write fail" );
        }

        signalThread.emit ( new bool ( validate ) );
    }
    else if ( currentStep == READ )
    {
        Logical_voter_group clvg;
        Card_data cardDataExpected;

        clvg = LogicalVoterGroupFactory::createLogicalVoterGroup ();
        cardDataExpected = CardDataFactory::createCardData ( clvg, NULL );

        try
        {
            Card_data cardData;

            validate = CardController::getInstance()->readCardData ( cardData );
        }
        catch ( ... )
        {

        }

        if ( validate )
        {
            logger->Debug ( "Read successfully" );
        }
        else
        {
            messageError = N_ ( "Smartcard Reader Read Failed" );
            messageErrorInfo = N_ ( "There was an error during the read operations." );

            logger->Debug ( "Read fail" );
        }

        signalThread.emit ( new bool ( validate ) );
    }

    return false;
}

void SmartcardActionsWidget::onRawObject ( void * object )
{
    bool * status = static_cast < bool * > ( object );

    if ( *status )
    {
        currentStep++;
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardActionsWidget::nextStep ), timeStep );
    }
    else
    {
        viewMessageError();
    }

    delete status;
}

void SmartcardActionsWidget::remove ()
{
    setCancelButtonSensitive ( true );

    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_REMOVE );
    progressWidget->setInfoText ( _ ( "Please Remove The Card" ) );
    progressWidget->setInfoFooter ( " " );
}

void SmartcardActionsWidget::success()
{
    showButtons ( false, false, false );

    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_DETECT );
    progressWidget->setInfoText ( _ ( "Write and Read Process Successfully" ) );
    progressWidget->setInfoFooter ( " " );

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardActionsWidget::emit ), timeOk );
}

void SmartcardActionsWidget::viewMessageError()
{
    showButtons ( false, false, false );

    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_DISABLED );
    progressWidget->setInfoText ( _ ( messageError.c_str() ) );
    progressWidget->setInfoFooter ( " " );

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardActionsWidget::checkDevices ), timeFail );
}

void SmartcardActionsWidget::checkDevices()
{
    if ( CardPolling::getInstance()->isCardPresent() )
    {
        checkRemoveInError = true;
        progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_REMOVE );
        progressWidget->setInfoText ( _ ( messageErrorInfo.c_str() ) );
        progressWidget->setInfoFooter ( " " );
    }
    else
    {
        viewRetry ();
    }
}

void SmartcardActionsWidget::viewWaitRetry()
{
    showButtons ( false, false, false );

    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PROGRESS );
    progressWidget->setInfoText ( _ ( "Please wait until the smartcard reader device is being readied to retry the test." ) );
    progressWidget->setInfoFooter ( _ ( "Do not reinsert the card at this moment." ) );

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardActionsWidget::initialize ), timeRetry );
}

void SmartcardActionsWidget::viewRetry()
{
    bool cancelCondition = currentRetries >= numberOfRetries;

    showButtons ( false, true, true );
    setCancelButtonSensitive ( cancelCondition );
    setRetryButtonSensitive ( true );

    Glib::ustring compose = cancelCondition ? _ ( "Press Fail Message" ) : _ ( "Press Retry Message %1" );
   int retriesBeforeContinuing = cancelCondition ? 0 : numberOfRetries - currentRetries;
   std::string message = Glib::ustring::compose ( compose, retriesBeforeContinuing );
   currentRetries++;

    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_INSERT );
    progressWidget->setInfoText ( message );
    progressWidget->setInfoFooter ( _ ( "Do not reinsert the card at this moment." ) );
}

void SmartcardActionsWidget::viewError()
{
    switch ( ( SmartcardDiagnoseStep ) currentStep )
    {
        case INSERT:
            MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setStatusSmartcardReaderInsertionError();
            break;

        case WRITE:
            MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setStatusSmartcardReaderWriteError();
            break;

        case READ:
            MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setStatusSmartcardReaderReadError();
            break;

        case REMOVE:
            MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setStatusSmartcardReaderExtractionError();
            break;

        default:
            break;
    }

    dispose();

    showButtons ( false, false, false );

    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_DISABLED );
    progressWidget->setInfoText ( _ ( "Smartcard Reader Diagnose Fail" ) );
    progressWidget->setInfoFooter ( " " );

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardActionsWidget::error ), timeFail );
}

void SmartcardActionsWidget::error()
{
    on_stop_signal_emit().emit();
}

void SmartcardActionsWidget::dispose()
{
    connChangue.disconnect();

    CardPolling::getInstance()->pause();
}

void SmartcardActionsWidget::onCardPollingStatus ( CardPolling::CardPollingStatus status )
{
    switch ( status )
    {
        case CardPolling::INSERTED:

            if ( currentStep == INSERT )
            {
                currentStep++;
                progressWidget->setInfoText ( _ ( "Card Inserted" ) );
                progressWidget->setInfoFooter ( " " );

                setCancelButtonSensitive ( false );
                Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardActionsWidget::nextStep ), timeStep );
            }

            break;

        case CardPolling::REMOVED:

            if ( checkRemoveInError )
            {
                checkRemoveInError = false;
                Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardActionsWidget::viewRetry ), 0 );
                return;
            }

            if ( currentStep == REMOVE )
            {
                currentStep++;
                progressWidget->setInfoText ( _ ( "Card Removed" ) );
                progressWidget->setInfoFooter ( " " );
                setCancelButtonSensitive ( false );
                Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardActionsWidget::nextStep ), timeStep );
            }

            break;

        default:
        case CardPolling::ERROR_SCR:
            break;
    }
}
