/**
 * @file GenericPrintingWidget.hxx
 * @brief Header class generic printing widget
 * @date 29/11/2012
 * @author Yadickson Soto
 */

#ifndef GENERICPRINTINGWIDGET_HXX_
#define GENERICPRINTINGWIDGET_HXX_

#include <Widgets/wizard-widget-base.h>
#include <Printing/IBasePrintDocument.hxx>
#include <Operation/Devices/PrinterController.hxx>
#include "GenericWidget.hxx"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	GenericPrintingWidget
                 * @brief	Class generic printing widget.
                 * @author	Yadickson Soto
                 * @see StepWidget
                 * @date	29/11/2012
                 */
                class GenericPrintingWidget: public Smartmatic::GUI::Widgets::WizardWidgetBase
                {
                    public:

                        typedef sigc::signal<void> SignalErrorPrinting;

                    public:

                        /**
                         * @fn	GenericPrintingWidget::GenericPrintingWidget( GenericWidget * centerWidget );
                         * @brief	Class constructor.
                         * @param 	centerWidget center widget
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         */
                        explicit GenericPrintingWidget ( GenericWidget * centerWidget );
                        /**
                         * @fn	virtual GenericPrintingWidget::~GenericPrintingWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         */
                        virtual ~GenericPrintingWidget();
                        /**
                         * @fn	void GenericPrintingWidget::setInfoTitle ( std::string text );
                         * @brief	Method setter title text.
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         * @param	text	new title text.
                         */
                        void setInfoTitle ( std::string text );
                        /**
                         * @fn	void GenericPrintingWidget::setInfoText ( std::string text );
                         * @brief	Method setter bottom text.
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         * @param	text	new bottom text.
                         */
                        void setInfoText ( std::string text );
                        /**
                         * @fn	void GenericPrintingWidget::setTextButtons( std::string ok, std::string cancel );
                         * @brief	Method setter text buttons.
                         * @param 	ok text ok buttons
                         * @param 	cancel text cancel buttons
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         */
                        void setTextButtons ( std::string ok, std::string cancel );
                        /**
                         * @fn	void GenericPrintingWidget::start ( Smartmatic::SAES::Printing::IBasePrintDocument * printObj, bool showRetry, bool showCancel, Glib::RefPtr<Gdk::Pixbuf> pixbuff = Glib::RefPtr<Gdk::Pixbuf>(NULL) );
                         * @brief	Method start print process
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         * @param	printObj object printing
                         * @param 	showRetry show retry
                         * @param 	showCancel show cancel
                         * @param 	pixbuff image to view in start process printing
                         */
                        void start ( Smartmatic::SAES::Printing::IBasePrintDocument * printObj, bool showRetry, bool showCancel, Glib::RefPtr<Gdk::Pixbuf> pixbuff = Glib::RefPtr<Gdk::Pixbuf> ( NULL ) );
                        /**
                         * @fn	sigc::signal<void> getSignalError();
                         * @brief	method getter signal error.
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         * @return	signal error.
                         */
                        sigc::signal<void> getSignalError();
                        /**
                         * @fn	bool getStatus();
                         * @brief	Gets the status.
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool getStatus();
                        /**
                         * @fn  SignalErrorPrinting getSignalErrorPrinting();
                         * @brief   Gets error printing signal.
                         * @author  eduardo.fernandez
                         * @date    05/03/2018
                         * @return  Error printing signal.
                         */
                        SignalErrorPrinting getSignalErrorPrinting();

                    protected:
                        /**
                         * @fn	void GenericPrintingWidget::startPrinting();
                         * @brief	Method start print process
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         */
                        void startPrinting();
                        /**
                         * @fn	void GenericPrintingWidget::endPrinting();
                         * @brief	Method end print process
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         */
                        void endPrinting();
                        /**
                         * @fn	void GenericPrintingWidget::errorPrint( std::string errorMessage );
                         * @brief	Method error check printer
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         * @param 	errorMessage error message
                         */
                        void errorPrint ( std::string errorMessage );
                        /**
                         * @fn	void GenericPrintingWidget::end();
                         * @brief	Method end process
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         */
                        void end();

                        /**
                         * @fn	sigc::signal<void> GenericPrintingWidget::onSignalCancel();
                         * @brief	method on signal cancel button clicked.
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         */
                        void onSignalCancel();
                        /**
                         * @fn	sigc::signal<void> GenericPrintingWidget::onSignalAccept();
                         * @brief	method on signal accept button clicked.
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         */
                        void onSignalAccept();

                    private:

                        bool status; ///< flag status printing
                        std::string title; ///< title process
                        std::string message; ///< message process
                        Smartmatic::SAES::Operation::Devices::PrinterController * printer; ///< The printer controller
                        bool showRetry; ///< flag show retry
                        bool showCancel; ///< flag show cancel

                        Glib::RefPtr<Gdk::Pixbuf> pixbuff; ///< The pifbuff image
                        GenericWidget * centerWidget; ///< center widget
                        Smartmatic::SAES::Printing::IBasePrintDocument * printObj; ///< object printing
                        sigc::signal<void> signalError; ///< signal error

                        sigc::connection connOk; ///< The connection signal dispatcher ok
                        sigc::connection connError; ///< The connection signal dispatcher error

                        static Smartmatic::Log::ISMTTLog * logger;  ///< logger

                        SignalErrorPrinting signalErrorPrinting;
                };
            }
        }
    }
}

#endif /* GENERICPRINTINGWIDGET_HXX_ */
