/**
 * @file InitiatorWidget.cxx
 * @brief Body class initiator widget
 * @date 24/10/2011
 * @author Yadickson Soto
 */

#include "InitiatorWidget.hxx"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Log/SMTTLogManager.h>
#include <Operation/functionality-initiator.h>
#include <Operation/MachineOperationManager.hxx>
#include <Voting/Status/election-status-schema.hxx>
#include <Voting/Status/ElectionStatusWrapper.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Initiator;
using namespace Smartmatic::SAES::Voting::Status;

ISMTTLog * InitiatorWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Windows.InitiatorWidget", GETTEXT_PACKAGE );

InitiatorWidget::InitiatorWidget ( ProgressWidget * progress, IMachineInitiator * initiator )
    : Smartmatic::GUI::Widgets::StepWidget(), initiator ( initiator ), progress ( progress )
{
    initialize();
    this->setWidget ( progress );
}

InitiatorWidget::~InitiatorWidget()
{
    dispose();
}

void InitiatorWidget::start()
{
    progress->setInfoTitle ( _ ( "Smartmatic.SAES.GUI.Windows.InitializationWindow.Initializing" ) );
    progress->setInfoText ( " " );

    initiator->start ( 0 );
}

void InitiatorWidget::initialize()
{
    conn.disconnect();
    conn = initiator->getSignalDetected().connect ( sigc::mem_fun ( *this, &InitiatorWidget::statusReport ) );

    progress->setImage ( "ui/check-directory-structure.gif" );
}

void InitiatorWidget::dispose()
{
    this->remove();
    conn.disconnect();

    if ( progress )
    {
        delete progress;
        progress = NULL;
    }
}

void InitiatorWidget::end()
{
    dispose();
    emit();
}

void InitiatorWidget::statusReport ( InitiatorSignal * status )
{
    if ( !status->isFinished() )
    {
        showMessage ( status );
    }
    else
    {
        conn.disconnect();
        showEndMessage ( status );
    }
}

void InitiatorWidget::showMessage ( InitiatorSignal * status )
{
    logger->Debug ( status->getStatus().getMessage() );

    progress->setInfoText ( status->getStatus().getMessage() );

    switch ( status->getStatus().getStatus () )
    {
        default:
        case TestStatus::NotTested:
        case TestStatus::Passed:
            progress->setImage ( "ui/check-directory-structure.gif" );
            break;

        case TestStatus::Failed:
            progress->setImage ( "ui/check-directory-structure-failed.gif" );
            break;

        case TestStatus::BlockDevice:
            progress->setImage ( "ui/check-flash-presence.gif" );
            break;
    }
}

void InitiatorWidget::showEndMessage ( InitiatorSignal * status )
{
    logger->Debug ( "End initiator" );

    if (MachineOperationManager::getInstance()->isClosingElection())
    {
        shutdownWidget.setLabel(_("An error occurred during election closing.\nMachine must be restarted in Recount mode"));
        setWidget(&shutdownWidget);
    }
    else if ( status->getStatus().getStatus() == TestStatus::Passed )
    {
        progress->hideImage();
        progress->hideInfoText();
        progress->hideInfoFooter();
        progress->setInfoTitle ( status->getStatus().getMessage() );
        int timeout = initiator->getSuccessMessageTime();
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &InitiatorWidget::end ), timeout );
    }
    else
    {
        progress->setImage ( "ui/check-directory-structure-failed.gif" );
        progress->setInfoTitle ( status->getStatus().getMessage() );
        progress->setInfoText ( status->getStatus().getFooterMessage() );
    }
}

std::string InitiatorWidget::getTitleText()
{
    std::string message;

    switch ( SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getInfoMachine() )
    {
        case InfoMachineConfiguration::CONFIGURATION_PM:
            message = _ ( "Smartmatic.SAES.GUI.Windows.InitializationWindow.PMTitle" );
            break;

        default:
        case InfoMachineConfiguration::CONFIGURATION_VM:
            message = _ ( "Smartmatic.SAES.GUI.Windows.InitializationWindow.VMTitle" );
            break;
    }

    return message;
}
