/**
 * @file MachineVotingWidget.hxx
 * @brief Header of the voting experience widget class
 * @date 08/03/2012
 * @author Yadickson Soto
 */

#ifndef MACHINEVOTINGWIDGET_HXX_
#define MACHINEVOTINGWIDGET_HXX_

#include "RunMachineWidget.hxx"
#include <Voting/CardData/card-data-schema.hxx>
#include <Widgets/StepWidget.hxx>
#include <System/SafeBuffer.h>
#include <Operation/Voting/VotePersistence.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>
#include <Operation/Devices/Polling/AlarmPolling.hxx>
#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Configuration/alarms-configuration-schema.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                namespace Voting
                {
                    class WaitingVotingWidget;
                    class DualWaitingForCardWidget;
                    class WaitingForCardWidget;
                    class PreviewWidget;
                    class ElectoralTypeSelectionBoxWidget;
                    class ILanguageSelectionWidget;
                    class VotingExperienceWidget;
                }

                namespace AudioVoting
                {
                    class AudioVotingExperienceWidget;
                }

                /**
                 * @class	MachineVotingWidget
                 * @brief	Class voting experience widget.
                 * @author	Yadickson Soto
                 * @date	08/03/2012
                 */
                class MachineVotingWidget : public RunMachineWidget
                {
                    public:
                        /**
                         * @fn	MachineVotingWidget::MachineVotingWidget();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        MachineVotingWidget();
                        /**
                         * @fn	virtual MachineVotingWidget::~MachineVotingWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual ~MachineVotingWidget();
                        /**
                         * @fn	virtual std::string MachineVotingWidget::getTitleText();
                         * @brief	virtual method getter title.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         * @return	title.
                         */
                        virtual std::string getTitleText();
                        /**
                         * @fn	virtual bool MachineVotingWidget::isTitle();
                         * @brief	virtual method getter is visible help button.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         * @return	true is help button visible.
                         */
                        virtual bool isTitle();
                        /**
                         * @fn	virtual void MachineVotingWidget::start();
                         * @brief	virtual method start widget.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void start();
                        /**
                         * @fn	virtual void MachineVotingWidget::dispose();
                         * @brief	virtual method stop.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void dispose();
                        /**
                         * @fn	virtual void MachineVotingWidget::refresh();
                         * @brief	virtual method refresh.
                         * @author	Yadickson Soto
                         * @date	11/04/2012
                         */
                        virtual void refresh();
                        /**
                         * @fn	void MachineVotingWidget::stopAll();
                         * @brief	method stop all process.
                         * @author	Yadickson Soto
                         * @date	25/05/2012
                         */
                        void stopAll();

                    protected:

                        /**
                         * @enum	StepVoting
                         * @brief	step voting.
                         */
                        enum StepVoting
                        {
                            NONE ///< none process
                            , WAITING ///< waiting for card data
                            , VALIDATE ///< validate card data
                            , ERROR_VALIDATE ///< error validate card data
                            , SHOW_LANGUAGE ///< show selection language
                            , VOTING ///< process voting
                            , AUDIO_VOTING_LANGUAGE_SELECTION ///< audio voting language selection
                            , AUDIO_VOTING ///< process audio voting
                        };

                        /**
                         * @fn	virtual void MachineVotingWidget::initialize();
                         * @brief	Initializer method.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void initialize();
                        /**
                         * @fn	void MachineVotingWidget::cardInsert();
                         * @brief	method card insert.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        void cardInsert();
                        /**
                         * @fn	void MachineVotingWidget::cardRemove();
                         * @brief	method card remove.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        void cardRemove();
                        /**
                         * @fn	void MachineVotingWidget::cardReaderUnresponsive();
                         * @brief	method card reader unresponsive.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        void cardReaderUnresponsive();
                        /**
                         * @fn	void MachineVotingWidget::startCheckCardPresent();
                         * @brief	method start check card data.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        void startCheckCardPresent();
                        /**
                         * @fn	void MachineVotingWidget::stopCheckCardPresent();
                         * @brief	method stop check card data.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        void stopCheckCardPresent();
                        /**
                         * @fn	void MachineVotingWidget::startCheckBarcode();
                         * @brief	method start check barcode.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        void startCheckBarcode();
                        /**
                         * @fn	void MachineVotingWidget::stopCheckBarcode();
                         * @brief	method stop check barcode.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        void stopCheckBarcode();
                        /**
                         * @fn	void MachineVotingWidget::startCheckAlarmButton();
                         * @brief	method start check alarm button.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         */
                        void startCheckAlarmButton();
                        /**
                         * @fn	void MachineVotingWidget::stopCheckAlarmButton();
                         * @brief	method stop check alarm button.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         */
                        void stopCheckAlarmButton();
                        /**
                         * @fn	void MachineVotingWidget::connectBarcode();
                         * @brief	method connect barcode.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        void connectBarcode();
                        /**
                         * @fn	void MachineVotingWidget::disconnectBarcode();
                         * @brief	method disconnect barcode.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        void disconnectBarcode();
                        /**
                         * @fn	void MachineVotingWidget::simCardData();
                         * @brief	method simulation insert card data.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         */
                        void simCardData();
                        /**
                         * @brief  Enables audio voting.
                         * @author manuel.arguelles
                         * @date   Oct 11, 2017
                         */
                        void setAudioVoting();
                        /**
                         * @fn	void MachineVotingWidget::startValidateCardData();
                         * @brief	method start voting experience.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         */
                        void startValidateCardData();
                        /**
                         * @fn	void MachineVotingWidget::showSelectionLanguage( void * p );
                         * @brief	method show selection language.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         */
                        void showSelectionLanguage ( void * p );
                        /**
                         * @fn	void MachineVotingWidget::startVotingExperience();
                         * @brief	method start voting experience.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         * @param p null pointer
                         */
                        void startVotingExperience();
                        /**
                         * @fn	void MachineVotingWidget::stopVotingExperience();
                         * @brief	method stop voting experience.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         */
                        void stopVotingExperience();

                        /**
                         * @brief  Starts the audio voting experience.
                         * @author manuel.arguelles
                         * @date   Oct 11, 2017
                         */
                        void startAudioVotingExperience(void*);

                        /**
                         * @brief  Audio voting language selected.
                         * @author eduardo.fernandez
                         * @date   Nov 27, 2017
                         */
                        void audioVotingLanguageSelected();

                        /**
                         * @fn	void MachineVotingWidget::readCardData();
                         * @brief	method read card data.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         */
                        void readCardData();
                        /**
                         * @fn	void MachineVotingWidget::validateCardData();
                         * @brief	method validate card data.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         */
                        void validateCardData();
                        /**
                         * @fn	void MachineVotingWidget::createCardData();
                         * @brief	method create card data.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         */
                        void createCardData();
                        /**
                         * @fn	void MachineVotingWidget::languageSelected();
                         * @brief	method on language selected.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         */
                        void languageSelected();
                        /**
                         * @fn	void MachineVotingWidget::setAlarm ( Smartmatic::SAES::Functionality::Configuration::AlarmEnum alarmType );
                         * @brief	method setter alarm message.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         * @param	alarmType	type of the alarm.
                         */
                        void setAlarm ( Smartmatic::SAES::Functionality::Configuration::AlarmEnum alarmType );
                        /**
                         * @fn	void MachineVotingWidget::safeAlarm ( void * alarmType );
                         * @brief	method setter alarm message.
                         * @author	Yadickson Soto
                         * @date	23/10/2015
                         * @param	alarmType	type of the alarm.
                         */
                        void safeAlarm ( void * alarmType );
                        /**
                         * @fn	void MachineVotingWidget::cardError( void * p );
                         * @brief	method view message error read card.
                         * @author	Yadickson Soto
                         * @date	07/03/2013
                         * @param p null pointer
                         */
                        void cardError ( void * p );
                        /**
                         * @fn	void MachineVotingWidget::cardErrorAID( void * p );
                         * @brief	method view message error read card AID.
                         * @author	Yadickson Soto
                         * @date	07/03/2013
                         * @param p null pointer
                         */
                        void cardErrorAID ( void * p );
                        /**
                         * @fn	void MachineVotingWidget::cardErrorPIN( void * p );
                         * @brief	method view message error read card PIN.
                         * @author	Yadickson Soto
                         * @date	07/03/2013
                         * @param p null pointer
                         */
                        void cardErrorPIN ( void * p );
                        /**
                         * @fn	void MachineVotingWidget::cardErrorGeneric( void * p );
                         * @brief	method view message error read card.
                         * @author	Yadickson Soto
                         * @date	07/03/2013
                         * @param p null pointer
                         */
                        void cardErrorGeneric ( void * p );
                        /**
                         * @fn	bool MachineVotingWidget::checkPrinter();
                         * @brief	method check printer.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         * @return	true is printer success.
                         */
                        bool checkPrinter();
                        /**
                         * @fn	bool MachineVotingWidget::markUsedCardData();
                         * @brief	method mark used card data.
                         * @author	Yadickson Soto
                         * @date	12/04/2012
                         * @return	true is success.
                         */
                        bool markUsedCardData();
                        /**
                         * @fn	void MachineVotingWidget::back();
                         * @brief	method back.
                         * @author	Yadickson Soto
                         * @date	03/05/2012
                         */
                        void back();
                        /**
                         * @fn	void MachineVotingWidget::restoreWidget();
                         * @brief	method restore widget.
                         * @author	Yadickson Soto
                         * @date	25/05/2012
                         */
                        void restoreWidget();
                        /**
                         * @fn	void MachineVotingWidget::refreshAll();
                         * @brief	method restore widget.
                         * @author	Yadickson Soto
                         * @date	25/05/2012
                         */
                        void refreshAll();
                        /**
                         * @fn	void MachineVotingWidget::onSignalVote ( Smartmatic::SAES::Voting::Vote::Vote & vote );
                         * @brief	Method getter vote process.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         * @param	vote	vote.
                         */
                        void onSignalVote ( Smartmatic::SAES::Voting::Vote::Vote * vote );
                        /**
                         * @fn	void MachineVotingWidget::onSignalState ( Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status );
                         * @brief	Method getter status vote process.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         * @param	status	status vote process.
                         * @param	count 	count status.
                         */
                        void onSignalState ( Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status );
                        /**
                         * @fn	void MachineVotingWidget::viewWidget ( Smartmatic::GUI::Widgets::StepWidget * widget );
                         * @brief	Method view widget.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         * @param	widget	new widget.
                         */
                        void viewWidget ( Smartmatic::GUI::Widgets::StepWidget * widget );
                        /**
                         * @fn	void MachineVotingWidget::detectAlarm();
                         * @brief	method detect alarm.
                         * @author	Yadickson Soto
                         * @date	25/05/2012
                         * @param time
                         * @return false
                         */
                        bool detectAlarm ( int time );
                        /**
                         * @fn	void MachineVotingWidget::showCardUsed( void * p );
                         * @brief	method show message card used
                         * @author	Yadickson Soto
                         * @date	25/05/2012
                         * @param p null pointer
                         */
                        void showCardUsed ( void * p );
                        /**
                         * @fn	void onSignalCardChange( Smartmatic::SAES::Operation::Devices::CardPolling::CardPolling::CardPollingStatus status );
                         * @brief	Method on signal card change
                         * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date	06/03/2013
                         * @param 	status	card status
                         */
                        void onSignalCardChange ( Smartmatic::SAES::Operation::Devices::CardPolling::CardPolling::CardPollingStatus status );
                        /**
                         * @fn	void onButtonPresed( Smartmatic::SAES::Operation::Devices::AlarmPolling::AlarmButtonStatus status );
                         * @brief	Method on signal button presed
                         * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date	06/03/2013
                         * @param 	status	alarm box status
                         */
                        void onButtonPresed ( Smartmatic::SAES::Operation::Devices::AlarmPolling::AlarmButtonStatus status );
                        /**
                         * @fn	void deviceChange( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );
                         * @brief	getter device monitor change
                         * @author	Yadickson Soto
                         * @date	14/12/2012
                         * @param 	deviceTypeMonitor device change
                         */
                        void deviceChange ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );
                        /**
                         * @fn	void clearCardData();
                         * @brief	Clear card data values (code, ballot id and session id).
                         * @author	Yadickson Soto
                         * @date	10/05/2013
                         */
                        void clearCardData();

                        /**
                         * @fn  bool pollingPlaceConfiguredForAudioVoting();
                         * @brief   verify if the polling place is configured for audio voting.
                         * @author  Marcel Mancini
                         * @date    23/01/2018
                         */
                        bool pollingPlaceConfiguredForAudioVoting();
                        /**
                         * @fn  bool ableIsConnected();
                         * @brief   verify if the able is connected.
                         * @author  Marcel Mancini
                         * @date    23/01/2018
                         */
                        bool ableIsConnected();
                        /**
                         * @fn  bool voicesArePresent();
                         * @brief   verify if the voices of audio voting are present.
                         * @author  Marcel Mancini
                         * @date    23/01/2018
                         */
                        bool voicesArePresent();

                    private:

                        void stopAudioVotingExperience();

                    private:

                        Smartmatic::SAES::GUI::Widgets::Voting::WaitingVotingWidget * waitingVotingWidget;	///< Contains the simulation buttons.
                        Smartmatic::SAES::GUI::Widgets::Voting::WaitingForCardWidget * waitingForCardWidget; ///< Show the voting instructions to the elector at the beginning of the voting experience, error screens, and card reading and validation.
                        Smartmatic::SAES::GUI::Widgets::Voting::PreviewWidget * previewWidget; ///< print preview widget
                        Smartmatic::SAES::GUI::Widgets::Voting::ElectoralTypeSelectionBoxWidget * simulationWidget; ///< nationality widget
                        Smartmatic::SAES::GUI::Widgets::Voting::ILanguageSelectionWidget * languageSelectionWidget; ///< language selection widget
                        Smartmatic::SAES::GUI::Widgets::Voting::VotingExperienceWidget * votingExperienceWidget; ///< voting experience widget
                        Smartmatic::SAES::GUI::Widgets::Voting::DualWaitingForCardWidget * mainWidget; ///< The start of the voter experience is divided into two parts, election information and voting instructions.
                        Smartmatic::SAES::GUI::Widgets::AudioVoting::AudioVotingExperienceWidget * audioVotingExperienceWidget;

                        Smartmatic::System::SafeDispatcher signalCardError; ///< signal card error
                        Smartmatic::System::SafeDispatcher signalCardErrorAID; ///< signal card error AID
                        Smartmatic::System::SafeDispatcher signalCardErrorPIN; ///< signal card error PIN
                        Smartmatic::System::SafeDispatcher signalCardErrorGeneric; ///< signal card not properly inserted
                        Smartmatic::System::SafeDispatcher signalCardUsed; ///< signal card used
                        Smartmatic::System::SafeDispatcher signalShowLanguage; ///< signal show language
                        Smartmatic::System::SafeDispatcher signalAlarm; ///< signal set alarm inside thread
                        Smartmatic::System::SafeDispatcher signalStartAudioVoting;

                        sigc::signal<void> signalBack; ///< signal back
                        sigc::signal<void> signalRefresh; ///< signal refresh

                        sigc::connection signalVote;	///< The signal vote
                        sigc::connection signalStatusVote;	///< The signal status vote

                        StepVoting stepVoting; ///< step voting process
                        Smartmatic::SAES::Voting::CardData::Card_data currentCardData; ///< current card data
                        std::string currentCardDataCode; ///< current card data code
                        std::string currentBallotIDCardData; ///< current ballot id card data code
                        std::string currentSessionIDCardData; ///< current session id card data code

                        sigc::connection connLanguage;  ///< The connection language
                        sigc::connection connCardPollingEvent;  ///< The connection card insert or remove event
                        sigc::connection signalDeviceChange; ///< signal device change

                        sigc::connection connAlarm; ///< The connection alarm
                        sigc::connection connAlarmMonitor; ///< The connection alarm monitor

                        Glib::Mutex syncVoting; ///< sync voting

                        bool simulateProcess;	///< true if simulate process
                        bool simulateVote;	///< true if simulate votes
                        bool audioVoting;
                        bool isInterrupted;

                        Smartmatic::SAES::Voting::Vote::Vote vote;	///< The vote

                        Glib::Thread * m_thread; ///< thread read card
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}

#endif /* MACHINEVOTINGWIDGET_HXX_ */
