/**
 * @file ProgressWidget.cxx
 * @brief Body class progress widget
 * @date 18/10/2011
 * @author Yadickson Soto
 */

#include "ProgressWidget.hxx"
#include <Environment/Configurations/resource-path-provider.h>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::GUI::Operation;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Operation;

ISMTTLog * ProgressWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.ProgressWidget", GETTEXT_PACKAGE );

ProgressWidget::ProgressWidget()
    : StepWidget()
{
    this->mainFrame = NULL;
    this->mainVBox = NULL;
    this->progressImage = NULL;
    this->progressTitle = NULL;
    this->progressLabel = NULL;
    this->progressFooter = NULL;
    this->initialize();
    this->setWidget ( mainFrame );
    setAccessibleName ( "ProgressWidget" );
}

ProgressWidget::~ProgressWidget()
{
    dispose();
}

void ProgressWidget::initialize()
{
    dispose();

    mainFrame      = new SmartmaticBasicEventBox();
    mainVBox       = new SmartmaticBasicVBox();
    progressImage  = new SmartmaticBasicImage();
    progressTitle  = new SmartmaticBasicLabel();
    progressLabel  = new SmartmaticBasicLabel();
    progressFooter = new SmartmaticBasicLabel();

    mainVBox->set_homogeneous(false);
    mainVBox->pack_start(*progressTitle, true, true, 20);
    mainVBox->pack_start(*progressImage, true, true, 0);
    mainVBox->pack_start(*progressLabel, true, true, 10);
    mainVBox->pack_start(*progressFooter, false, true, 2);

    mainFrame->add(*mainVBox);

    progressTitle->set_visible ( false );
    progressTitle->set_no_show_all();
    progressFooter->set_visible ( false );
    progressFooter->set_no_show_all();

    LabelConfiguration labelTitle = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->getGenericWidgetConfiguration().getTitleConfiguration() );
    LabelConfiguration labelText = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->getGenericWidgetConfiguration().getInfoConfiguration() );
    LabelConfiguration labelFooter = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->getGenericWidgetConfiguration().getFooterConfiguration() );

    setInfoTitleConfig ( labelTitle );
    setInfoTextConfig ( labelText );
    setInfoFooterConfig ( labelFooter );

    mainFrame->setAccessibleName ( "ProgressWidgetContainer" );
    mainVBox->setAccessibleName ( "ProgressWidgetVerticalContainer" );
    progressImage->setAccessibleName ( "ProgressWidget" );
    progressTitle->setAccessibleName ( "ProgressWidgetTitle" );
    progressLabel->setAccessibleName ( "ProgressWidgetInfo" );
    progressFooter->setAccessibleName ( "ProgressWidgetFooter" );
}

void ProgressWidget::dispose()
{
    this->remove();

    if ( mainFrame )
    {
        delete mainFrame;
        mainFrame = NULL;
    }

    if ( mainVBox )
    {
        delete mainVBox;
        mainVBox = NULL;
    }

    if ( progressImage )
    {
        delete progressImage;
        progressImage = NULL;
    }

    if ( progressLabel )
    {
        delete progressLabel;
        progressLabel = NULL;
    }

    if ( progressTitle )
    {
        delete progressTitle;
        progressTitle = NULL;
    }

    if ( progressFooter )
    {
        delete progressFooter;
        progressFooter = NULL;
    }
}

void ProgressWidget::setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::Resource image )
{
    std::string iconFile = ImageResourcePath::getPath ( image );

    try
    {
        progressImage->clear();
        progressImage->set ( iconFile );
    }
    catch ( ... )
    {
        logger->Debug ( "ERROR - load image " + iconFile );
    }
}

void ProgressWidget::setImage ( std::string image )
{
    std::string iconFile = ImageResourcePath::getPath ( image );

    try
    {
        progressImage->clear();
        progressImage->set ( iconFile  );
    }
    catch ( ... )
    {
        ImageResourcePath::getPath ( ImageResourcePath::NOT_FOUND );
        logger->Debug ( "ERROR - load image " + iconFile );
    }
}

void ProgressWidget::setImage ( Glib::RefPtr<Gdk::Pixbuf> pixbuff )
{
    progressImage->clear();
    progressImage->set ( pixbuff );
}

void ProgressWidget::setInfoTitle ( std::string text )
{
    if ( text.empty() )
    {
        text = " ";
    }

    progressTitle->set_visible ( text.compare ( " " ) != 0 );
    progressTitle->set_text ( _ ( text.c_str() ) );
}

void ProgressWidget::setInfoText ( std::string text )
{
    if ( text.empty() )
    {
        text = " ";
    }

    progressLabel->set_text ( _ ( text.c_str() ) );
}

void ProgressWidget::setInfoFooter ( std::string text )
{
    if ( text.empty() )
    {
        text = " ";
    }

    progressFooter->set_visible ( text.compare ( " " ) != 0 );
    progressFooter->set_text ( _ ( text.c_str() ) );
}

void ProgressWidget::setInfoTitleConfig ( LabelConfiguration labelConf )
{
    LabelConfiguration::SetLabelConfiguration ( progressTitle, labelConf );
}

void ProgressWidget::setInfoTextConfig ( LabelConfiguration labelConf )
{
    LabelConfiguration::SetLabelConfiguration ( progressLabel, labelConf );
}

void ProgressWidget::setInfoFooterConfig ( LabelConfiguration labelConf )
{
    LabelConfiguration::SetLabelConfiguration ( progressFooter, labelConf );
}

void ProgressWidget::hideInfoTitle()
{
    progressTitle->hide();
}

void ProgressWidget::showInfoTitle ( bool flag )
{
    progressTitle->set_visible ( flag );
}

void ProgressWidget::hideImage()
{
    progressImage->hide();
}

void ProgressWidget::showImage ( bool flag )
{
    progressImage->set_visible ( flag );
}

void ProgressWidget::hideInfoText()
{
    progressLabel->hide();
}

void ProgressWidget::showInfoText ( bool flag )
{
    progressLabel->set_visible ( flag );
}

void ProgressWidget::hideInfoFooter()
{
    progressFooter->hide();
}

void ProgressWidget::showInfoFooter ( bool flag )
{
    progressFooter->set_visible ( flag );
}

Smartmatic::GUI::Widgets::SmartmaticBasicLabel * ProgressWidget::getLabelTitle()
{
    return progressTitle;
}

Smartmatic::GUI::Widgets::SmartmaticBasicLabel * ProgressWidget::getLabelInfo()
{
    return progressLabel;
}

Smartmatic::GUI::Widgets::SmartmaticBasicLabel * ProgressWidget::getLabelFooter()
{
    return progressFooter;
}
