/**
 * @file   ReportPrintingVMWidget.cxx
 * @brief  Implementation of ReportPrintingVMWidget class.
 * @author manuel.arguelles
 * @date   Jun 13, 2017
 */

#include "ReportPrintingVMWidget.hxx"
#include <Gui/Widgets/ProgressWidget.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Printing/ReportPrintingController.hxx>
#include <System/Runtime/PowerDown.hxx>
#include <Environment/Configurations/resource-path-provider.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String)
#else
#define _(String) (String)
#endif

#define N_(String) (String)

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::Environment;

ReportPrintingVMWidget::ReportPrintingVMWidget(ProgressWidget* progress)
    : StepWidget(), m_progress(progress)
{
    m_printButton.set_size_request(-1, 60);
    m_printButton.set_label(_("Print Report"));

    m_shutdownButton.SetImage(
        ResourcePathProvider::getInstance()->GetResourcePath(PACKAGE_NAME, "ui/screen-shut-down.png"));

    m_shutdownButton.WidgetsPosition(0, 5, 0);
    m_shutdownButton.set_size_request(90, 90);

    m_printButton.signal_clicked().connect(
        sigc::mem_fun(*this, &ReportPrintingVMWidget::onPrintButtonClicked));
    m_shutdownButton.signal_clicked().connect(
        sigc::mem_fun(*this, &ReportPrintingVMWidget::onShutdownButtonClicked));
    m_printerController.getSignalError().connect(
        sigc::mem_fun(*this, &ReportPrintingVMWidget::onPrintError));
    m_printerController.getSignalOk().connect(
        sigc::mem_fun(*this, &ReportPrintingVMWidget::onPrintDone));

    m_progress->hideInfoTitle();
    m_progress->hideInfoFooter();

    m_report.loadReports();
    m_report.setReportsList();

    if (m_report.verifySignature())
    {
        m_progress->setInfoText(N_("Press button to print"));
        m_progress->setImage("ui/printer-vm.png");
    }
    else
    {
        m_progress->setImage("ui/printer-error.img");
        m_progress->setInfoText(_("Report signature could not be verified before printing the report."));
        m_printButton.set_sensitive(false);
    }

    m_buttonHBox.pack_start(m_printButton, true, false);
    m_imageHBox.pack_start(m_shutdownButton, true, false);

    m_mainVBox.pack_start(*m_progress, false, false);
    m_mainVBox.pack_start(m_buttonHBox, false, false, 50);

    m_centralVBox.pack_start(m_mainVBox, true, false);
    m_centralVBox.pack_end(m_imageHBox, false, false, 10);

    setWidget(&m_centralVBox);
    set_visible(true);
}

ReportPrintingVMWidget::~ReportPrintingVMWidget()
{
}

void ReportPrintingVMWidget::onPrintButtonClicked()
{
    PrinterStatusNotifier::runForcedPrintBlank();
    sleep ( 1 );

    m_printButton.set_sensitive(false);
    m_shutdownButton.set_sensitive(false);

    m_progress->setImage("ui/printer-ok.img");
    m_progress->setInfoText(N_("Printing report, please wait"));

    m_printerController.start (&m_report, true);
}

void ReportPrintingVMWidget::onPrintError(const std::string& message)
{
    m_printButton.set_sensitive(true);
    m_shutdownButton.set_sensitive(true);
    m_progress->setImage("ui/printer-error.img");
    m_progress->setInfoText(message);
    m_printButton.set_label(_("Retry Print"));
}

void ReportPrintingVMWidget::onPrintDone()
{
    m_progress->setInfoText(N_("Report printed successfully"));
    m_printButton.set_sensitive(true);
    m_shutdownButton.set_sensitive(true);
    m_progress->setImage("ui/printer-ok.img");
    m_printButton.set_label(_("Reprint Report"));
}

void ReportPrintingVMWidget::onShutdownButtonClicked()
{
    PowerDown::shutdown();
}
