/**
 * @file EndVotingSequenceWidget.cxx
 * @brief Body class end voting sequence widget
 * @date 02/04/2013
 * @author Yadickson Soto
 */

#include "EndVotingSequenceWidget.hxx"
#include <Gui/Widgets/Voting/RemoveCardWidget.hxx>
#include <Gui/Widgets/Voting/PrintVoteWidget.hxx>
#include <Gui/Widgets/Voting/FoldPrintVoteWidget.hxx>
#include <Gui/Widgets/Voting/PrintVoteErrorWidget.hxx>
#include <Gui/Widgets/Voting/PrintAndFoldVoteWidget.hxx>
#include <Gui/Widgets/Voting/FactoryVotingWidget.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::Log;

ISMTTLog * EndVotingSequenceWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.Voting.EndVotingSequenceWidget", GETTEXT_PACKAGE );

EndVotingSequenceWidget::EndVotingSequenceWidget()
    : StepWidget()
{
    errorDetected = false;
    manager = NULL;
}

EndVotingSequenceWidget::~EndVotingSequenceWidget()
{
    remove();
    dispose();
}

void EndVotingSequenceWidget::refresh()
{
    if ( manager )
    {
        manager->refresh();
    }
}

void EndVotingSequenceWidget::dispose()
{
    std::vector < WizardWidgetBase * >::iterator it;

    for ( it = wizardVector.begin();
            it != wizardVector.end();
            ++it )
    {
        WizardWidgetBase * widget = ( *it );
        delete widget;
        widget = NULL;
    }

    wizardVector.clear();

    if ( manager )
    {
        delete manager;
        manager = NULL;
    }
}

void EndVotingSequenceWidget::start()
{
    errorDetected = false;
    dispose();

    VotingExperienceConfiguration::EndStepSequence & seq ( SAESGUIConfigurationManager::getInstance()->getVotingExperienceConfiguration().getEndStep() );
    VotingExperienceConfiguration::EndStepIterator iter;

    for ( iter = seq.begin(); iter != seq.end(); iter++ )
    {
        WizardWidgetBase * widget = NULL;

        switch ( ( *iter ) )
        {
            default:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_WAITING_CARD:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_READ_CARD:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_USED:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD_AID:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD_PIN:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD_GENERIC:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_LANGUAGE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_LANGUAGE_CONFIRMATION:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CUSTOM_ATTRIBUTE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CUSTOM_ATTRIBUTE_CONFIRMATION:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PARTY:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PARTY_CONFIRMATION:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_NULL_PARTY:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_BALLOT:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_PRINT_VOTE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_REMOVED:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_REMOVED_COUNTER:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_REMOVED_CANCEL_PROCESS:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_INTERRUPT_VOTING_EXPERIENCE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_NULL_VOTING_EXPERIENCE:
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_TIME_OUT_VOTING_EXPERIENCE:
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_SCAN_VOTE:
                widget = FactoryVotingWidget::createScanVoteWidget();
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PRINT_VOTE:
                widget = FactoryVotingWidget::createPrintVoteWidget();
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_FOLD_VOTE:
                widget = FactoryVotingWidget::createFoldPrintVoteWidget();
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_REMOVE_CARD:
                widget = FactoryVotingWidget::createRemoveCardWidget();
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PRINT_FOLD_VOTE:
                widget = FactoryVotingWidget::createPrintAndFoldVoteWidget();
                break;
        }

        if ( widget )
        {
            widget->on_stop_signal_emit().connect ( sigc::mem_fun ( *this, &EndVotingSequenceWidget::errorSequence ) );
            wizardVector.push_back ( widget );
        }
    }

    if ( !wizardVector.empty() )
    {
        manager = new WizardWidgetManager ( wizardVector );
        manager->on_finish_signal_emit().connect ( sigc::mem_fun ( *this, &EndVotingSequenceWidget::endSequence ) );
        manager->Start();
        setWidget ( manager );
    }
    else
    {
        endSequence();
    }
}

void EndVotingSequenceWidget::errorSequence()
{
    errorDetected = true;
    signalError.emit();
}

void EndVotingSequenceWidget::endSequence()
{
    if ( !errorDetected )
    {
        emit();
    }
}

sigc::signal<void> EndVotingSequenceWidget::getSignalError()
{
    return signalError;
}
