/**
 * @file   PartyPreview.cxx
 * @brief  Implementation of PartyPreview class.
 * @author manuel.arguelles
 * @date   Aug 30, 2018
 */

#include "PartyPreview.hxx"

#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Gui/Widgets/party-widget.h"

#include <Configuration/voting-experience-configuration-schema.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Voting/Election/ElectionWrapper.hxx>
#include <Voting/Election/election-schema.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/TableCenterWidget.hxx>

#include <list>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const int BORDER_SIZE  = 10;
static const int TITLE_HEIGHT = 40;

using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::Election;

PartyPreview::PartyPreview(const int width, const int height, const size_t maxParties)
    : m_width(width - BORDER_SIZE)
    , m_height(height - BORDER_SIZE - TITLE_HEIGHT)
    , m_maxParties(maxParties)
{
}

PartyPreview::~PartyPreview()
{
}

void PartyPreview::loadParties(const std::string& contestCode)
{
    MachineOperationManager* const     operationManager     = MachineOperationManager::getInstance();
    SAESGUIConfigurationManager* const configurationManager = SAESGUIConfigurationManager::getInstance();
    const PartySelectionConfiguration  partyConfiguration   = configurationManager->GetPartySelectionConfiguration();
    ElectionWrapper* const             electionWrapper      = operationManager->getElectoralConfiguration()->getElection();

    int         columnsNumber = partyConfiguration.getSizeToColumns();
    const int   rowSpacing    = partyConfiguration.getPartiesMargin();
    const int   colSpacing    = partyConfiguration.getPartiesMargin();
    const float numberWidth   = configurationManager->GetPartyWidgetConfiguration().getNumberWidth();
    const float boxWidth      = m_width / (float) columnsNumber;

    Contest                   contest = electionWrapper->getContest(contestCode);
    std::list<Ballot_option*> partiesList;
    electionWrapper->getBallotOptionPartyByContest(contest, partiesList);

    const float widgetWidth  = boxWidth - colSpacing;
    const float imageHeight  = (widgetWidth - numberWidth) / partyConfiguration.getImageRelation();
    float       widgetHeight = imageHeight;
    float       boxHeight    = imageHeight + rowSpacing;

    m_height -= (partyConfiguration.getPartyButtonsSpacing() + partyConfiguration.getFixBorder());

    int rowsNumber = m_height / boxHeight;
    int totalBoxes = static_cast<int>(m_maxParties) + 1;

    updatePartyWidgetHeight(columnsNumber, totalBoxes, widgetHeight, rowsNumber, rowSpacing);

    int actualTotalBoxes = static_cast<int>(partiesList.size()) + 1;

    createTableWidget(partiesList, actualTotalBoxes, rowSpacing, colSpacing, widgetWidth, widgetHeight, columnsNumber);
}

void PartyPreview::setLabel(SmartmaticBasicLabel* label)
{
    label->set_size_request(-1, TITLE_HEIGHT);
    set_label_widget(*label);
    set_label_align(Gtk::ALIGN_CENTER);
    set_shadow_type(Gtk::SHADOW_NONE);
}

void PartyPreview::addPartyWidgetToTable(
    TableCenterWidget* const tableWidget,
    PartyWidget* const partyWidget,
    int columnsNumber,
    int rowsToFill,
    int& rowNumber,
    int& columnNumber)
{
    tableWidget->put(partyWidget, rowNumber, columnNumber);

    if (SAESGUIConfigurationManager::getInstance()->GetPartyWidgetConfiguration().getUseFillByRows())
    {
        columnNumber++;

        if (columnNumber == columnsNumber)
        {
            columnNumber = 0;
            rowNumber++;
        }
    }
    else
    {
        rowNumber++;

        if (rowNumber == rowsToFill)
        {
            rowNumber = 0;
            columnNumber++;
        }

        if (columnNumber == columnsNumber)
        {
            columnNumber = 0;
        }
    }
}

void PartyPreview::createPartiesWidgets(
    const std::list<Ballot_option*>& partiesList,
    const float widgetWidth,
    const float widgetHeight,
    TableCenterWidget* const tableWidget,
    int columnsNumber,
    int rowsToFill)
{
    int rowNumber    = 0;
    int columnNumber = 0;

    std::list<Ballot_option * >::const_iterator it = partiesList.begin();

    for (; it != partiesList.end(); ++it)
    {
        SaesDirectories* const saesDir = MachineOperationManager::getInstance()->getSaesDirectories();
        PartyWidget* const partyWidget = Gtk::manage(new PartyWidget(*(*it), *saesDir, widgetWidth, widgetHeight));
        partyWidget->show();

        addPartyWidgetToTable(tableWidget, partyWidget, columnsNumber, rowsToFill, rowNumber, columnNumber);
    }

    if (SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getShowNullParty())
    {
        PartyWidget* const partyWidget = Gtk::manage(new PartyWidget(_("Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.NullParty"), widgetWidth, widgetHeight));
        partyWidget->show();

        tableWidget->put(partyWidget, rowNumber, columnNumber);
    }
}

void PartyPreview::createTableWidget(
    const std::list<Ballot_option*>& partiesList,
    const int totalBoxes,
    const int rowSpacing,
    const int colSpacing,
    const float widgetWidth,
    const float widgetHeight,
    const int columnsNumber)
{
    int rowsToFill = totalBoxes / columnsNumber;
    rowsToFill     = totalBoxes % columnsNumber > 0.0 ? rowsToFill + 1 : rowsToFill;

    TableCenterWidget* const tableWidget = Gtk::manage(new TableCenterWidget());
    tableWidget->setSpacings(rowSpacing, colSpacing);

    createPartiesWidgets(partiesList, widgetWidth, widgetHeight, tableWidget, columnsNumber, rowsToFill);

    tableWidget->show_all();
    tableWidget->show();
    set_border_width(10);
    add(*tableWidget);
}

void PartyPreview::updatePartyWidgetHeight(
    int& columnsNumber,
    int& totalBoxes,
    float& widgetHeight,
    int rowsNumber,
    const int rowSpacing)
{
    const PartySelectionConfiguration partyConfiguration =
        SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration();

    columnsNumber = partyConfiguration.getColumns();

    if (partyConfiguration.getUseMaxPartyNumerFromAllContest())
    {
        totalBoxes  = MachineOperationManager::getInstance()->getElectoralConfiguration()->getMaxPartiesByAllContest();
        totalBoxes += 1;
    }

    if (rowsNumber * columnsNumber < totalBoxes)
    {
        rowsNumber    = totalBoxes / columnsNumber;
        rowsNumber    = totalBoxes % columnsNumber > 0.0 ? rowsNumber + 1 : rowsNumber;
        int boxHeight = m_height / (float) rowsNumber;
        widgetHeight  = boxHeight - rowSpacing;
    }
}

int PartyPreview::getSmallestFontSize()
{
    int fontSize = -1;

    TableCenterWidget*    tableWidget = static_cast<TableCenterWidget*>(get_child());
    SmartmaticBasicTable* table       = tableWidget->getTable();
    std::vector<Widget*>  parties     = table->get_children();

    for (std::vector<Widget*>::iterator it = parties.begin(); it != parties.end(); ++it)
    {
        PartyWidget* party         = static_cast<PartyWidget*>(*it);
        int          partyFontSize = party->getSmallestFontSize();

        if (partyFontSize != -1 && (fontSize == -1 || partyFontSize < fontSize))
        {
            fontSize = partyFontSize;
        }
    }

    return fontSize;
}

void PartyPreview::setFontSize(const int fontSize)
{
    TableCenterWidget*    tableWidget = static_cast<TableCenterWidget*>(get_child());
    SmartmaticBasicTable* table       = tableWidget->getTable();
    std::vector<Widget*>  parties     = table->get_children();

    for (std::vector<Widget*>::iterator it = parties.begin(); it != parties.end(); ++it)
    {
        PartyWidget* party = static_cast<PartyWidget*>(*it);

        party->setFontSize(fontSize);
    }
}
