/**
 * @file   PartyPreview.hxx
 * @brief  Declaration of PartyPreview class.
 * @author manuel.arguelles
 * @date   Aug 30, 2018
 */

#ifndef PARTY_PREVIEW_H_
#define PARTY_PREVIEW_H_

#include <Widgets/SmartmaticBasicFrame.hxx>

#include <string>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {
            class SmartmaticBasicLabel;
            class TableCenterWidget;
        }
    }

    namespace SAES
    {
        namespace Voting
        {
            namespace Election
            {
                class Ballot_option;
            }
        }

        namespace GUI
        {
            namespace Widgets
            {
                class PartyWidget;
            }

            namespace Widgets
            {
                namespace Voting
                {
                    class PartyPreview : public Smartmatic::GUI::Widgets::SmartmaticBasicFrame
                    {
                        public:

                            /**
                             * @brief  Creates an instance of the party preview class.
                             * @author manuel.arguelles
                             * @date   sep 17, 2018
                             * @param  width      The width of the widget.
                             * @param  height     The height of the widget.
                             * @param  maxParties The maximum amount of parties.
                             */
                            PartyPreview(int width, int height, size_t maxParties);

                            /**
                             * @brief  Destroys an instance of the class.
                             * @author manuel.arguelles
                             * @date   sep 17, 2018
                             */
                            virtual ~PartyPreview();

                        public:

                            /**
                             * @brief Loads the parties to be previewed.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             * @param contestCode The contest code.
                             */
                            void loadParties(const std::string& contestCode);

                            /**
                             * @brief Sets the label widget.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             * @param label The label to be set.
                             */
                            void setLabel(Smartmatic::GUI::Widgets::SmartmaticBasicLabel* label);

                            /**
                             * @brief  Sets the font size of the parties. The font size
                             * can be reduced if needed to fit the screen/widget.
                             * @author manuel.arguelles
                             * @date   oct  2, 2018
                             * @param  fontSize The font size to use.
                             */
                            void setFontSize(int fontSize);

                            /**
                             * @brief  Gets the smallest font size in use.
                             * @author manuel.arguelles
                             * @date   oct  2, 2018
                             * @return The smallest font size of the party preview.
                             */
                            int getSmallestFontSize();

                        private:

                            /**
                             * @brief Adds the party widget to the main table.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             * @param tableWidget[in]    The main table widget.
                             * @param partyWidget[in]    The party widget to be added.
                             * @param columnsNumber[in]  The columns number.
                             * @param rowsToFill[in]     The rows to fill.
                             * @param rowNumber[out]     Current row number.
                             * @param columnNumber[out]  Current column number.
                             */
                            void addPartyWidgetToTable(
                                Smartmatic::GUI::Widgets::TableCenterWidget* const tableWidget,
                                PartyWidget* const partyWidget,
                                int columnsNumber,
                                int rowsToFill,
                                int& rowNumber,
                                int& columnNumber);

                            /**
                             * @brief Creates the parties widgets.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             * @param partiesList   The parties list.
                             * @param widgetWidth   The party widget width.
                             * @param widgetHeight  The party widget height.
                             * @param tableWidget   The main table widget.
                             * @param columnsNumber The columns number.
                             * @param rowsToFill    The rows to fill.
                             */
                            void createPartiesWidgets(
                                const std::list<Smartmatic::SAES::Voting::Election::Ballot_option*>& partiesList,
                                const float widgetWidth,
                                const float widgetHeight,
                                Smartmatic::GUI::Widgets::TableCenterWidget* const tableWidget,
                                int columnsNumber,
                                int rowsToFill);

                            /**
                             * @brief Creates the main table widget.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             * @param partiesList the parties list.
                             * @param totalBoxes    The number of total boxes.
                             * @param rowSpacing    The rows spacing.
                             * @param colSpacing    The columns spacing.
                             * @param widgetWidth   The party widget width.
                             * @param widgetHeight  The party widget height.
                             * @param columnsNumber The columns number.
                             */
                            void createTableWidget(
                                const std::list<Smartmatic::SAES::Voting::Election::Ballot_option*>& partiesList,
                                const int totalBoxes,
                                const int rowSpacing,
                                const int colSpacing,
                                const float widgetWidth,
                                const float widgetHeight,
                                const int columnsNumber);

                            /**
                             * @brief Updates the party widget height.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             * @param columnsNumber[out] The columns number.
                             * @param totalBoxes[out]    The number of total boxes.
                             * @param widgetHeight[out]  The party widget height.
                             * @param rowsNumber[in]     The rows number.
                             * @param rowSpacing[in]     The rows spacing.
                             */
                            void updatePartyWidgetHeight(
                                int& columnsNumber,
                                int& totalBoxes,
                                float& widgetHeight,
                                int rowsNumber,
                                const int rowSpacing);


                        private:

                            int    m_width;
                            int    m_height;
                            size_t m_maxParties;
                    };

                } /* namespace Voting */
            } /* namespace Widgets */
        } /* namespace GUI */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif
