/**
 * @file	VotingExperienceWidget.cxx
 * @brief	voting experience widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#include "VotingExperienceWidget.hxx"
#include "System/Sounds/smartmatic-sounds-manager.h"
#include <Configuration/configuration-persistence-manager.h>
#include <sstream>
#include <Operation/MachineOperationManager.hxx>
#include "Resources/election-string-resources-manager.h"
#include <Voting/CardData/card-data-schema.hxx>
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include "Printing/TimeoutPrintDocument.h"
#include "Printing/SmartcardRemovedPrintDocument.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Configuration/label-configuration.h"
#include <Operation/Devices/CardController.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Voting/Vote/VoteCode.hxx>
#include <Voting/Vote/VoteWrapper.hxx>
#include <Printing/PrinterStatusNotifier.h>
#include <Gui/Widgets/Voting/FactoryVotingWidget.hxx>
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Operation/AlarmMessage.hxx>
#include <Operation/Voting/VotingOperation.hxx>
#include <Operation/FormatLabel.hxx>
#include <Operation/WindowsManager.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>
#include <Windows/basic-window.h>
#include <Operation/Devices/AlarmController.hxx>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Gtk;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Operation;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Operation::Voting;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::CardData;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Sounds;


ISMTTLog * VotingExperienceWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.VotingExperienceWidget", GETTEXT_PACKAGE );

VotingExperienceWidget::VotingExperienceWidget()
    : Smartmatic::GUI::Widgets::StepWidget()
{
    partySelectionWidget = NULL;
    ballotOptionSelectionWidget = NULL;
    customAttrWidget = NULL;
    interruptWidget = NULL;
    referendumSelectionWidget = NULL;
    currentWidget = NULL;
    currentStep = VOTING_EXPERIENCE_STEP_NO_INIT;

    cardRemovedWidget = FactoryVotingWidget::createCardRemovedWidget();
    printTimeOutWidget = FactoryVotingWidget::createPrintTimeOutWidget();
    interruptWidget = FactoryVotingWidget::createInterruptWidget();
    endVotingSequenceWidget = FactoryVotingWidget::createEndVotingSequenceWidget();

    interruptWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::nullVoting ) );
    endVotingSequenceWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::endVoting ) );
    endVotingSequenceWidget->getSignalError().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::errorVoting ) );
    signalDispatcherShowLoadingBallot.connect ( sigc::mem_fun ( this, &VotingExperienceWidget::onShowLoadingBallot ) );

    timeConnector = SAESGUIConfigurationManager::getInstance()->GetVotingWindowConfiguration().getConnectTimer();
    timeConnectorLoading = SAESGUIConfigurationManager::getInstance()->GetVotingWindowConfiguration().getConnectTimerLoading();
    loadingBallotsNumber = SAESGUIConfigurationManager::getInstance()->GetVotingWindowConfiguration().getNumberOfBallotsToDisplayLoadingBallotMessage();

    votingExperienceController = Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getVotingExperienceController();
    votingWidgetManager = VotingWidgetManager::getInstance();
    electionWrapper = Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getElectoralConfiguration()->getElection();
    setName ( VotingOperation::getWindowName() );

    dispose();

    logger->Debug ( "end" );
}

VotingExperienceWidget::~VotingExperienceWidget()
{
    dispose();

    if ( cardRemovedWidget )
    {
        delete cardRemovedWidget;
        cardRemovedWidget = NULL;
    }

    if ( printTimeOutWidget )
    {
        delete printTimeOutWidget;
        printTimeOutWidget = NULL;
    }

    if ( interruptWidget )
    {
        delete interruptWidget;
        interruptWidget = NULL;
    }

    if ( endVotingSequenceWidget )
    {
        delete endVotingSequenceWidget;
        endVotingSequenceWidget = NULL;
    }
}

void VotingExperienceWidget::dispose()
{
    IsInInterrupt = false;
    alreadyInTransition = false;
    detectCardExtractionEnable = true;
    isBallotOptionSelection = false;
    timeConnector = 10;
    timeConnectorLoading = 50;
    currentContestIndex = 0;
    totalContests = 0;
    showBackInParty = true;
    loadingBallotsNumber = 0;
    success = false;
    showPrintImage = true;
    simulationMode = false;

    connAlarmAndBack.disconnect();
    connRefresh.disconnect();
    connAlarmNotRemoveCard.disconnect ();
    connectTimeout.disconnect ();
    connection_partySelectionWidget_null_clicked.disconnect ();
    connection_partySelectionWidget_back_clicked.disconnect ();
    connection_partySelectionWidget_party_clicked.disconnect ();
    connection_ballotOptionSelectionWidget_accept_clicked.disconnect ();
    connection_ballotOptionSelectionWidget_clear_clicked.disconnect();
    connection_ballotOptionSelectionWidget_back_clicked.disconnect ();
    connection_ballotOptionSelectionWidget_ballotOption_clicked.disconnect ();
    possibleWelcomeTimeout.disconnect ();
    connRefresh.disconnect ();
    connectionShowFinalMessage.disconnect();

    if ( customAttrWidget )
    {
        delete customAttrWidget;
        customAttrWidget = NULL;
    }

    if ( referendumSelectionWidget )
    {
        delete referendumSelectionWidget;
        referendumSelectionWidget = NULL;
    }

}

void VotingExperienceWidget::start()
{
    remove();

    int timer = electionWrapper->getElection().getVoting_time_out();

    if ( timer > 0 )
    {
        connectTimeout = Glib::signal_timeout().connect_seconds ( sigc::bind ( sigc::mem_fun ( *this, &VotingExperienceWidget::onTimeout ), 0 ), timer );
    }

    startVoting ();

    if ( detectCardExtractionEnable && !simulationMode )
    {
        if ( SAESGUIConfigurationManager::getInstance()->getDetectCardExtraction() )
        {
            logger->Debug ( "Detect event card data" );
            InvalidCard = false;
            IsInInterrupt = false;
            extractionCounter = 0;
        }
    }
    else
    {
        logger->Debug ( "No detect event card data" );
    }
}

/*******************************************************************************
********************************************************************************
********************************************************************************
********************************************************************************
							VOTING WINDOW LOGIC
********************************************************************************
********************************************************************************
********************************************************************************
*******************************************************************************/

void VotingExperienceWidget::PerformBackStep()
{
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &VotingExperienceWidget::PerformBackStepInternal ), timeConnector );
}

void VotingExperienceWidget::PerformBackStepInternal()
{
    bool canMoveBack = votingExperienceController->moveContestListBack();

    if ( canMoveBack )
    {
        alreadyInTransition = false;
    }

    PerformNextStep();
}

void VotingExperienceWidget::PerformNextStep()
{
    if ( !alreadyInTransition )
    {
        logger->Debug ( "entered transition" );
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &VotingExperienceWidget::PerformNextStepInternal ), timeConnector );
        alreadyInTransition = true;
    }
    else
    {
        logger->Debug ( "already in transition" );
    }
}

void VotingExperienceWidget::PerformNextStepInternal()
{
    VotingExperienceController::VotingExperienceType nextStep = votingExperienceController->getNextTypeAction();

    setCurrentStep(nextStep);

    switch ( nextStep )
    {
        case VotingExperienceController::VOTING_CONTROLLER_CONTEST:
            logger->Debug ( "Get next contest list" );
            GetContestList();
            break;

        case VotingExperienceController::VOTING_CONTROLLER_CUSTOM_ATTRIBUTE_DEFINITION:
            logger->Debug ( "Get Custom Attribute definition" );
            PerformQuestionAction();
            break;

        default:
        case VotingExperienceController::VOTING_CONTROLLER_CLOSE:
            logger->Debug ( "controller close" );
            endVotingSequence();
            break;
    }

    alreadyInTransition = false;
}

void VotingExperienceWidget::setCurrentStep(const VotingExperienceController::VotingExperienceType step)
{
    switch (step)
    {
      case VotingExperienceController::VOTING_CONTROLLER_CONTEST:
          if (currentStep == VotingExperienceWidget::VOTING_EXPERIENCE_STEP_CUSTOM_ATTRIBUTE_WIDGET)
            currentStep = VotingExperienceWidget::VOTING_EXPERIENCE_STEP_CONTEST_WIDGET_CHILD;
          else
            currentStep = VotingExperienceWidget::VOTING_EXPERIENCE_STEP_CONTEST_WIDGET;
        break;
      case VotingExperienceController::VOTING_CONTROLLER_CUSTOM_ATTRIBUTE_DEFINITION:
        currentStep = VotingExperienceWidget::VOTING_EXPERIENCE_STEP_CUSTOM_ATTRIBUTE_WIDGET;
        break;
      case VotingExperienceController::VOTING_CONTROLLER_CLOSE:
        currentStep = VotingExperienceWidget::VOTING_EXPERIENCE_STEP_CLOSE;
        break;
    }
}

void VotingExperienceWidget::GetContestList()
{
    contestList = votingExperienceController->getCurrentContestList();

    if ( !contestList.empty() )
    {
        iterContest = contestList.begin();
        currentContestIndex = 0;
        totalContests = ( int ) contestList.size();
        GetNextContest();
    }
}

void VotingExperienceWidget::GetNextContest()
{
    if ( currentContestIndex < totalContests )
    {
        if (currentContestIndex != 0)
          setCurrentStep(VotingExperienceController::VOTING_CONTROLLER_CONTEST);

        currentContestIndex++;
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &VotingExperienceWidget::ShowWelcomeContestMessage ), timeConnector );
    }
    else
    {
        votingExperienceController->moveContestListForward();
        PerformNextStep();
    }
}

void VotingExperienceWidget::GetPreviousContest()
{
    currentContestIndex--;

    if ( currentContestIndex <= 0 )
    {
        currentContestIndex = 0;
        PerformBackStep();
    }
    else
    {
        SetActualContestFromIndex ( currentContestIndex );
        GetPartySelection();

        if ( SAESGUIConfigurationManager::getInstance()->GetVotingWindowConfiguration().getShowBallotOptionsInContestBack() &&
                votingExperienceController->isThereAnyPartySelectedFromContest ( *actualContest ) )
        {
            actualParty = previousParty;
            ConstructAndShowBallotOptionSelection();
        }
        else
        {
            GetAndShowPartySelection();
        }
    }
}

void VotingExperienceWidget::SetActualContestFromIndex ( int index )
{
    iterContest = contestList.begin();

    for ( int i = 1; i < index; i++ )
    {
        ++iterContest;
    }

    actualContest = *iterContest;

    if ( index > 1 )
    {
        showBackInParty = false;
    }
    else
    {
        showBackInParty = true;
    }
}

void VotingExperienceWidget::setActualContest ( Smartmatic::SAES::Voting::Election::Contest * contestA )
{
    actualContest = contestA;
}

void VotingExperienceWidget::ShowErrorChecksum()
{
    connectTimeout.disconnect();
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &VotingExperienceWidget::ShowErrorChecksumInternal ), timeConnector );
}

void VotingExperienceWidget::ShowErrorChecksumInternal()
{
    sleep ( SAESGUIConfigurationManager::getInstance()->getCheckSumFailTimerMessage() );

    switch ( SAESGUIConfigurationManager::getInstance()->getCheckSumFailOption() )
    {
        default:
        case CheckSumFailOption::CONFIGURATION_CLEANSELECTIONS:
            {
                votingWidgetManager->clearBallotOptionSelectionWidget();
                currentContestIndex = 0;
                GetNextContest();
            }
            break;

        case CheckSumFailOption::CONFIGURATION_MAINTAINSELECTIONS:
            votingWidgetManager->clearBallotOptionSelectionWidget();
            currentContestIndex = 0;
            GetNextContest();
            break;

        case CheckSumFailOption::CONFIGURATION_CLOSEVOTINGEXPERIENCE:
            endVotingSequence();
            break;
    }
}

void VotingExperienceWidget::showStepWidget ( StepWidget * widget )
{
    remove();
    currentWidget = widget;
    widget->refresh();
    setWidget ( widget );
}

void VotingExperienceWidget::GetCurrentContestReources ( Glib::ustring & contestClassName,
        Glib::ustring & contestClassDescription,
        Glib::ustring & contestTitle,
        Glib::ustring & contestAbstract,
        Glib::ustring & contestHelp )
{
    Contest_class contestClass =  electionWrapper->getContestClass ( actualContest->getContest_class_code() );
    contestClassName = ElectionStringResourcesManager::getInstance()->getResource ( contestClass.getName_key() );
    contestClassDescription = ElectionStringResourcesManager::getInstance()->getResource ( contestClass.getDescription_key() );
    contestTitle = ElectionStringResourcesManager::getInstance()->getResource ( actualContest->getTitle_key() );
    contestAbstract = ElectionStringResourcesManager::getInstance()->getResource ( actualContest->getAbstract_key() );
    contestHelp = ElectionStringResourcesManager::getInstance()->getResource ( actualContest->getHelp_key() );
}

void VotingExperienceWidget::startVoting()
{
    PerformNextStep();
}

/*******************************************************************************
********************************************************************************
						WELCOME QUESTION DIALOG
********************************************************************************
*******************************************************************************/

void VotingExperienceWidget::PerformQuestionAction()
{
    currentcustomAttribute = votingExperienceController->getNextCustomAttributeDefinition();
    logger->Debug("Current Custom Attribute Definition Code: " + currentcustomAttribute.getCode());

    CreateQuestionDialog();
}

/*******************************************************************************
********************************************************************************
							 QUESTION DIALOG
********************************************************************************
*******************************************************************************/

void VotingExperienceWidget::CreateQuestionDialog()
{
    if ( customAttrWidget )
    {
        delete customAttrWidget;
    }

    customAttrWidget = FactoryVotingWidget::createCustomAttributeWidget ( currentcustomAttribute );
    connection_question = customAttrWidget->signal_Answer().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::on_QuestionAnswered ) );
    showStepWidget ( customAttrWidget );
}

void VotingExperienceWidget::on_QuestionAnswered ( std::string code )
{
    connection_question.disconnect();
    votingExperienceController->setCustomAttributeDefinitionResponse ( currentcustomAttribute.getCode(), code );
    PerformNextStep();
}

/*******************************************************************************
********************************************************************************
							WELCOME CONTEST
********************************************************************************
*******************************************************************************/

void VotingExperienceWidget::ShowWelcomeContestMessage()
{
    GetReferendumOrPartySelection();
}

void VotingExperienceWidget::on_TimeoutContestWelcome()
{
    GetReferendumOrPartySelection();
}

/*******************************************************************************
********************************************************************************
						REFERENDUM WIDGET
********************************************************************************
*******************************************************************************/

void VotingExperienceWidget::GetReferendumOrPartySelection()
{
    SetActualContestFromIndex ( currentContestIndex );

    if ( actualContest->getIs_question() )
    {
        GetAndShowReferendum();
    }
    else
    {
        GetAndShowPartySelection();
    }
}

void VotingExperienceWidget::GetAndShowReferendum()
{
    GetReferendum();
    ShowReferendum();
}

void VotingExperienceWidget::GetReferendum()
{
    if ( referendumSelectionWidget )
    {
        delete referendumSelectionWidget;
    }

    connection_referendumSelectionWidget_null_clicked.disconnect();
    connection_referendumSelectionWidget_back_clicked.disconnect();
    connection_referendumSelectionWidget_accept_clicked.disconnect();
    referendumSelectionWidget = new ReferendumSelectionWidget ( actualContest );
    referendumSelectionWidget->SetWidgetEnabled ( true );
    connection_referendumSelectionWidget_null_clicked = referendumSelectionWidget->null_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::referendumSelection_on_null_clicked ) );
    connection_referendumSelectionWidget_back_clicked = referendumSelectionWidget->back_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::referendumSelection_on_back_clicked ) );
    connection_referendumSelectionWidget_accept_clicked = referendumSelectionWidget->accept_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::referendumSelection_on_accept_clicked ) );
    referendumSelectionWidget->ballotOption_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::referendumSelection_on_ballotOption_clicked ) );
    referendumSelectionWidget->show();
    referendumSelectionWidget->show_all_children();
    static bool showBackInReferendum = SAESGUIConfigurationManager::getInstance()->GetReferendumConfiguration().getShowBackButton();
    bool showBack = votingExperienceController->canMoveBack ( actualContest->getCode() ) && showBackInReferendum;
    referendumSelectionWidget->SetBackButton ( showBack );
    std::list<Eligible_option * > principalBallotList;

    try
    {
        electionWrapper->fillAllEligibleOptionsByContest ( *actualContest, principalBallotList );
    }
    catch ( ... )
    {
        logger->Debug ( "ERROR - electionWrapper->fillAllEligibleOptionsByContest" );
    }

    referendumSelectionWidget->Initialize ( principalBallotList );
}

void VotingExperienceWidget::ShowReferendum()
{
    isBallotOptionSelection = false;
    showStepWidget ( referendumSelectionWidget );
}

void VotingExperienceWidget::referendumSelection_on_back_clicked()
{
    connection_referendumSelectionWidget_back_clicked.disconnect();
    GetPreviousContest();
}

void VotingExperienceWidget::referendumSelection_on_accept_clicked()
{
    connection_referendumSelectionWidget_accept_clicked.disconnect();
    GetNextContest();
}

void VotingExperienceWidget::referendumSelection_on_null_clicked()
{
    if ( !simulationMode && !CardPolling::getInstance()->isCardPresent() )
    {
        logger->Debug ( "Card not present in - referendumSelection_on_null_clicked" );
        return;
    }

    on_ReferendumNormalVote();
}

void VotingExperienceWidget::on_ReferendumNormalVote()
{
    ShowReferendum();
}

void VotingExperienceWidget::referendumSelection_on_ballotOption_clicked ( std::string ballotOptionCode, bool isSelected )
{
    votingExperienceController->setBallotOptionState ( actualContest->getCode(), "x", ballotOptionCode, isSelected );
}

/*******************************************************************************
********************************************************************************
						PARTY SELECTION WIDGET
********************************************************************************
*******************************************************************************/
void VotingExperienceWidget::GetAndShowPartySelection()
{
    GetPartySelection();
    ShowPartySelection();
}

void VotingExperienceWidget::GetPartySelection()
{
    SetActualContestFromIndex ( currentContestIndex );

    if ( partySelectionWidget )
    {
        connection_partySelectionWidget_null_clicked.disconnect();
        connection_partySelectionWidget_back_clicked.disconnect();
        connection_partySelectionWidget_party_clicked.disconnect();
    }

    partySelectionWidget = votingWidgetManager->getPartySelectionWidgetByContest ( actualContest->getCode() );
    connection_partySelectionWidget_null_clicked = partySelectionWidget->null_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::partySelection_on_null_clicked ) );
    connection_partySelectionWidget_back_clicked = partySelectionWidget->back_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::partySelection_on_back_clicked ) );
    connection_partySelectionWidget_party_clicked = partySelectionWidget->party_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::partySelection_on_party_clicked ) );
    partySelectionWidget->SetWidgetEnabled ( true );

    partySelectionWidget->SetChildCustomAttribute(currentStep == VotingExperienceWidget::VOTING_EXPERIENCE_STEP_CONTEST_WIDGET_CHILD);

    bool showBack = votingExperienceController->canMoveBack ( actualContest->getCode() ) && showBackInParty;
    partySelectionWidget->SetBackButton ( showBack );
    partySelectionWidget->refresh();
}

void VotingExperienceWidget::ShowPartySelection()
{
    isBallotOptionSelection = false;
    connection_partySelectionWidget_back_clicked.disconnect();
    connection_partySelectionWidget_back_clicked = partySelectionWidget->back_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::partySelection_on_back_clicked ) );
    connection_partySelectionWidget_null_clicked.disconnect();
    connection_partySelectionWidget_null_clicked = partySelectionWidget->null_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::partySelection_on_null_clicked ) );

    if ( actualContest->getIs_question() )
    {
        ShowReferendum();
    }
    else
    {
        showStepWidget ( partySelectionWidget );
    }
}

void VotingExperienceWidget::partySelection_on_back_clicked()
{
    if ( !simulationMode && !CardPolling::getInstance()->isCardPresent() )
    {
        logger->Debug ( "Card not present in - partySelection_on_back_clicked" );
        return;
    }

    connection_partySelectionWidget_back_clicked.disconnect();
    logger->Debug ( "partySelection_on_back_clicked ENTER" );
    GetPreviousContest();
    logger->Debug ( "partySelection_on_back_clicked EXIT" );
}

void VotingExperienceWidget::partySelection_on_party_clicked ( Party partySelected )
{
    if ( !simulationMode && !CardPolling::getInstance()->isCardPresent() )
    {
        logger->Debug ( "Card not present in - partySelection_on_party_clicked" );
        return;
    }

    logger->Debug ( "partySelection_on_party_clicked ENTER" );

    actualParty = partySelected;

    onProcessPartyClicked();
}

void VotingExperienceWidget::onProcessPartyClicked()
{
    try
    {
        if ( SAESGUIConfigurationManager::getInstance()->GetVotingWindowConfiguration().getEraseBallotOptionsInPartyChanged() )
        {
            if ( actualContest->getCode() != "" )
            {
                if ( votingExperienceController->isThereAnyPartySelectedFromContest ( *actualContest ) )
                {
                    previousParty = votingExperienceController->getPartySelectedFromContest ( *actualContest );

                    if ( actualParty.getCode() !=  previousParty.getCode() )
                    {
                        if ( previousParty.getCode() != "" )
                        {
                            votingExperienceController->clearBallotOptionsSelectedByContestParty ( *actualContest, previousParty );
                        }
                    }
                }
            }
        }

        signalDispatcherShowLoadingBallot.emit();
    }
    catch ( ... )
    {
        logger->Debug ( "FATAL - party selection on party clicked" );
    }

    logger->Debug ( "partySelection_on_party_clicked EXIT" );
}

void VotingExperienceWidget::onShowLoadingBallot()
{
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &VotingExperienceWidget::ShowLoadingBallot ), 0 );
}

/*******************************************************************************
********************************************************************************
							NULL PARTY SELECTION
********************************************************************************
*******************************************************************************/

void VotingExperienceWidget::partySelection_on_null_clicked()
{
    if ( !simulationMode && !CardPolling::getInstance()->isCardPresent() )
    {
        logger->Debug ( "Card not present in - partySelection_on_null_clicked" );
        return;
    }

    connection_partySelectionWidget_null_clicked.disconnect();
    logger->Debug ( "partySelection_on_null_clicked ENTER" );

    on_ContestNullVote();

    logger->Debug ( "partySelection_on_null_clicked EXIT" );
}

void VotingExperienceWidget::on_ContestNormalVote()
{
    logger->Debug ( "on_ContestNormalVote ENTER, null cancelled" );
    partySelectionWidget->SetWidgetEnabled ( true );
    ShowPartySelection();
    logger->Debug ( "on_ContestNormalVote EXIT, null cancelled" );
}

void VotingExperienceWidget::on_ContestNullVote()
{
    connection_null_voteclicked.disconnect();
    logger->Debug ( "on_ContestNullVote ENTER" );
    votingExperienceController->clearBallotOptionsSelectedByContest ( *actualContest );
    votingExperienceController->setNullPartyResponse ( *actualContest );
    GetNextContest();
    logger->Debug ( "on_ContestNullVote EXIT" );
}

/*******************************************************************************
********************************************************************************
					LOADING BALLOT OPTION SELECTION WIDGET
********************************************************************************
*******************************************************************************/

void VotingExperienceWidget::ShowLoadingBallot()
{
    ConstructAndShowBallotOptionSelection();
}

/*******************************************************************************
********************************************************************************
						BALLOT OPTION SELECTION WIDGET
********************************************************************************
*******************************************************************************/

void VotingExperienceWidget::GetBallotOptionSelection()
{
    if ( ballotOptionSelectionWidget )
    {
        connection_ballotOptionSelectionWidget_accept_clicked.disconnect();
        connection_ballotOptionSelectionWidget_back_clicked.disconnect();
        connection_ballotOptionSelectionWidget_clear_clicked.disconnect();
        connection_ballotOptionSelectionWidget_ballotOption_clicked.disconnect();
    }

    ballotOptionSelectionWidget = votingWidgetManager->getBallotOptionSelectionWidgetByContestParty ( actualContest->getCode(), actualParty.getCode() );

    ballotOptionSelectionWidget->SetChildCustomAttribute(currentStep == VotingExperienceWidget::VOTING_EXPERIENCE_STEP_CONTEST_WIDGET_CHILD);

    ballotOptionSelectionWidget->RefreshLabels();
    ballotOptionSelectionWidget->ClearVisualSelection();

    connection_ballotOptionSelectionWidget_accept_clicked = ballotOptionSelectionWidget->accept_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::ballotOptionSelection_on_accept_clicked ) );
    connection_ballotOptionSelectionWidget_back_clicked = ballotOptionSelectionWidget->back_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::ballotOptionSelection_on_back_clicked ) );
    connection_ballotOptionSelectionWidget_clear_clicked = ballotOptionSelectionWidget->clear_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::ballotOptionSelection_on_clear_clicked ) );
    connection_ballotOptionSelectionWidget_ballotOption_clicked = ballotOptionSelectionWidget->ballotOption_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::ballotOptionSelection_on_ballotOption_clicked ) );
    ballotOptionSelectionWidget->SetWidgetEnabled ( true );
}

void VotingExperienceWidget::ShowBallotOptionSelection()
{
    isBallotOptionSelection = true;
    connection_ballotOptionSelectionWidget_back_clicked.disconnect();
    connection_ballotOptionSelectionWidget_back_clicked = ballotOptionSelectionWidget->back_clicked().connect ( sigc::mem_fun ( this, &VotingExperienceWidget::ballotOptionSelection_on_back_clicked ) );
    showStepWidget ( ballotOptionSelectionWidget );
}

void VotingExperienceWidget::ConstructAndShowBallotOptionSelection()
{
    GetBallotOptionSelection();
    ShowBallotOptionSelection();
}

void VotingExperienceWidget::ballotOptionSelection_on_clear_clicked()
{
    votingExperienceController->clearBallotOptionsSelectedByContest ( *actualContest );
}

void VotingExperienceWidget::ballotOptionSelection_on_back_clicked()
{
    if ( !simulationMode && !CardPolling::getInstance()->isCardPresent() )
    {
        logger->Debug ( "Card not present in - ballotOptionSelection_on_back_clicked" );
        return;
    }

    connection_ballotOptionSelectionWidget_back_clicked.disconnect();
    logger->Debug ( "ballotOptionSelection_on_back_clicked ENTER" );
    //TimeFunctionality::GetStartTimeDifferencePrintOut();
    partySelectionWidget->SetWidgetEnabled ( true );
    ShowPartySelection();
    logger->Debug ( "ballotOptionSelection_on_back_clicked EXIT" );
    //TimeFunctionality::GetStartTimeDifferencePrintOut();
}

void VotingExperienceWidget::ballotOptionSelection_on_ballotOption_clicked ( std::string ballotOptionCode, bool isSelected )
{
    votingExperienceController->setBallotOptionState ( actualContest->getCode(), actualParty.getCode(), ballotOptionCode, isSelected );
}

/*******************************************************************************
********************************************************************************
						VOTE CONFIRMATION DIALOG
********************************************************************************
*******************************************************************************/

void VotingExperienceWidget::ballotOptionSelection_on_accept_clicked()
{
    if ( !simulationMode && !CardPolling::getInstance()->isCardPresent() )
    {
        logger->Debug ( "Card not present in - ballotOptionSelection_on_accept_clicked" );
        return;
    }

    connection_ballotOptionSelectionWidget_accept_clicked.disconnect();
    logger->Debug ( "ballotOptionSelection_on_accept_clicked ENTER" );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &VotingExperienceWidget::on_AcceptVote ), timeConnector );
    logger->Debug ( "ballotOptionSelection_on_accept_clicked EXIT" );
}

void VotingExperienceWidget::on_AcceptVote()
{
    if ( currentContestIndex != totalContests )
    {
        logger->Debug ( "on_AcceptVote NEXT CONTEST" );
        GetNextContest();
    }
    else
    {
        logger->Debug ( "on_AcceptVote NEXT STEP" );
        votingExperienceController->moveContestListForward();
        PerformNextStep();
    }

    logger->Debug ( "on_AcceptVote EXIT" );
}

void VotingExperienceWidget::on_BackVote()
{
    ShowBallotOptionSelection();
}

void VotingExperienceWidget::ShowTimeOutEndVotingExperience()
{
    showStepWidget ( printTimeOutWidget );
    unsigned int time = SAESGUIConfigurationManager::getInstance()->getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_TIME_OUT_VOTING_EXPERIENCE );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &VotingExperienceWidget::PerformTimeOutEndVotingExperience ), time );
}

void VotingExperienceWidget::PerformTimeOutEndVotingExperience()
{
    detectCardExtractionEnable = false;
    AlarmMessage::alarm ( AlarmEnum::CONFIGURATION_TIMEOUT );
    sleep ( 2 );

    try
    {
        TimeoutPrintDocument timeoutPrintDocument ( Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getElectoralConfiguration(), ""/*votingExperienceController->getVoteCode()*/ );
        // timeoutPrintDocument.setClient( this );
        timeoutPrintDocument.Print();
        PrintDocumentResult res = timeoutPrintDocument.getDocumentResult ();

        if ( !res.getSuccess() )
        {
            AlarmMessage::alarm ( AlarmEnum::CONFIGURATION_VOTEPRINTINGFAILURE );
        }
    }
    catch ( Smartmatic::System::Exception::ElectionException & ex )
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__ << " " << ex.what();
        logger->Debug ( ss.str() );
    }

    endVoting();
}

bool VotingExperienceWidget::onTimeout ( int timer )
{
    ShowTimeOutEndVotingExperience();
    return false;
}

void VotingExperienceWidget::SetLabelsText()
{
    switch ( VotingOperation::getVoteMode() )
    {
        case Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_S:
        case Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_O:
            s_CustomAttributeQuestionFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.CustomAttributeQuestionFormat" );
            s_PartyHeaderFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PartyHeaderFormat" );
            s_PartyHeaderFormatNoQuestion = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PartyHeaderFormatNoQuestion" );
            s_PartyFooterFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PartyFooterFormat" );
            s_FooterPartySelection = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.FooterPartySelection" );
            s_ContestWelcomeFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ContestWelcomeFormat" );
            s_BallotHeaderFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.BallotHeaderFormat" );
            s_BallotHeaderFormatNoQuestion = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.BallotHeaderFormatNoQuestion" );
            s_BallotFooterFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.BallotFooterFormat" );
            s_FooterBallotSelection = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.FooterBallotSelection" );
            s_ConfirmSelectionMessage = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ConfirmSelectionMessage" );
            s_ConfirmSelectionMessageHeader = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ConfirmSelectionMessageHeader" );
            s_ConfirmSelectionMessageFooter = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ConfirmSelectionMessageFooter" );
            s_ConfirmSelectionMessageBackButton = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ConfirmSelectionMessageBackButton" );
            s_ConfirmSelectionMessageAcceptButton = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ConfirmSelectionMessageAcceptButton" );
            s_TimeOutVotingExperience = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.TimeOutVotingExperience" );
            s_PrinterErrorOnVote = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PrinterErrorOnVote" );
            s_TimeOutEndVotingExperience = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.TimeOutEndVotingExperience" );
            s_endVotExpLabel = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.WaitPrintingTicket" );
            s_finalVotExpLabel = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.BendPrintedTicket" );
            s_timeOutEndVotExpLabel = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.TimeOutVotingExperience" );
            s_PrinterErrorOnSmartcardRemoved = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PrinterErrorOnSmartcardRemoved" );
            s_LoadingBallotOption = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.LoadingBallotOption" );
            s_referendumHeaderFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ReferendumHeaderFormat" );
            s_referendumHeaderFormatNoQuestion = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ReferendumHeaderFormatNoQuestion" );
            s_referendumFooterFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ReferendumFooterFormat" );
            break;

        default:
        case Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_D:
            s_CustomAttributeQuestionFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.CustomAttributeQuestionFormat.Demo" );
            s_PartyHeaderFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PartyHeaderFormat.Demo" );
            s_PartyHeaderFormatNoQuestion = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PartyHeaderFormatNoQuestion.Demo" );
            s_PartyFooterFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PartyFooterFormat.Demo" );
            s_FooterPartySelection = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.FooterPartySelection.Demo" );
            s_ContestWelcomeFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ContestWelcomeFormat.Demo" );
            s_BallotHeaderFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.BallotHeaderFormat.Demo" );
            s_BallotHeaderFormatNoQuestion = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.BallotHeaderFormatNoQuestion.Demo" );
            s_BallotFooterFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.BallotFooterFormat.Demo" );
            s_FooterBallotSelection = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.FooterBallotSelection.Demo" );
            s_ConfirmSelectionMessage = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ConfirmSelectionMessage.Demo" );
            s_ConfirmSelectionMessageHeader = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ConfirmSelectionMessageHeader.Demo" );
            s_ConfirmSelectionMessageFooter = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ConfirmSelectionMessageFooter.Demo" );
            s_ConfirmSelectionMessageBackButton = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ConfirmSelectionMessageBackButton.Demo" );
            s_ConfirmSelectionMessageAcceptButton = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ConfirmSelectionMessageAcceptButton.Demo" );
            s_TimeOutVotingExperience = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.TimeOutVotingExperience.Demo" );
            s_PrinterErrorOnVote = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PrinterErrorOnVote.Demo" );
            s_TimeOutEndVotingExperience = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.TimeOutEndVotingExperience.Demo" );
            s_endVotExpLabel = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.WaitPrintingTicket.Demo" );
            s_finalVotExpLabel = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.BendPrintedTicket.Demo" );
            s_timeOutEndVotExpLabel = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.TimeOutVotingExperience.Demo" );
            s_PrinterErrorOnSmartcardRemoved = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PrinterErrorOnSmartcardRemoved.Demo" );
            s_LoadingBallotOption = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.LoadingBallotOption.Demo" );
            s_referendumHeaderFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ReferendumHeaderFormat.Demo" );
            s_referendumHeaderFormatNoQuestion = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ReferendumHeaderFormatNoQuestion.Demo" );
            s_referendumFooterFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ReferendumFooterFormat.Demo" );
            break;
    }

    s_errorWritingCard = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ErrorWritingCard" );;
    printTimeOutWidget->setText ( _ ( s_timeOutEndVotExpLabel.c_str() ) );
}

/*******************************************************************************
********************************************************************************
********************************************************************************
********************************************************************************
							CARD EXTRACTION DETECTION LOGIC
********************************************************************************
********************************************************************************
********************************************************************************
*******************************************************************************/

bool VotingExperienceWidget::CardIsValid()
{
    Card_data cardDataCheck;
    bool isGood = false;

    try
    {
        if ( CardController::getInstance()->readCardData ( cardDataCheck ) )
        {
            if ( cardDataCheck.getCard_status().compare ( "" ) == 0 )
            {
                logger->Debug ( "invalid data: card status could not be resolved" );
                isGood = false;
            }
            else
            {
                bool cardUsed = !SAESGUIConfigurationManager::getInstance()->getMarkCardAsUsed () ||
                                ( cardDataCheck.getCard_status() == Smartmatic::SAES::Voting::CardData::Card_status::CARD_DATA_USED ) ||
                                SAESFunctionalityConfigurationManager::getInstance()->getOperationConfiguration().getSimulateVotes();

                if ( cardUsed && cardDataCheck.getActivation_code().compare ( votingExperienceController->getActivationCode() ) == 0 )
                {
                    logger->Debug ( "card is Good" );
                    isGood = true;
                }
                else if ( votingExperienceController->getVotingMode() != Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_S )
                {
                    logger->Debug ( "error: card not used" );
                    isGood = false;
                }
                else
                {
                    logger->Debug ( "Card simulate" );
                    isGood = true;
                }
            }
        }
    }
    catch ( ... )
    {
        isGood = false;
        logger->Debug ( "Error read card data" );
    }

    return isGood;
}

void VotingExperienceWidget::cardInsertEvent()
{
    logger->Debug ( "Card insert event." );

    if ( detectCardExtractionEnable )
    {
        if ( IsInInterrupt && !InvalidCard )
        {
            bool cardValid = CardIsValid();

            if ( cardValid )
            {
                if ( ballotOptionSelectionWidget )
                {
                    ballotOptionSelectionWidget->SetWidgetEnabled ( true );
                }

                if ( partySelectionWidget )
                {
                    partySelectionWidget->SetWidgetEnabled ( true );
                }

                if ( referendumSelectionWidget )
                {
                    referendumSelectionWidget->SetWidgetEnabled ( true );
                }

                currentWidget->refresh();
                setWidget ( currentWidget );
                InvalidCard = false;
                IsInInterrupt = false;
            }
            else
            {
                InvalidCard = true;
                interruptWidget->showCancelProcess();
            }
        }
        else if ( IsInInterrupt && InvalidCard )
        {
            bool cardValid = CardIsValid();

            if ( cardValid )
            {
                InvalidCard = false;
                IsInInterrupt = false;
                showStepWidget ( currentWidget );
            }
        }
    }
}

void VotingExperienceWidget::cardRemoveEvent()
{
    logger->Debug ( "Card remove event." );

    if ( detectCardExtractionEnable )
    {
        if ( !IsInInterrupt )
        {
            possibleWelcomeTimeout.disconnect();
            extractionCounter++;

            if ( ( SAESGUIConfigurationManager::getInstance()->getNumberOfExtractionsBeforeStop() != 0 ) && ( extractionCounter == SAESGUIConfigurationManager::getInstance()->getNumberOfExtractionsBeforeStop() ) )
            {
                detectCardExtractionEnable = false;
                RefreshByTooManyExtracts();
            }
            else
            {
                IsInInterrupt = true;
                interruptWidget->showCardCounter();
                interruptWidget->refresh();
                setWidget ( interruptWidget );
            }
        }
    }
}

void VotingExperienceWidget::nullVoting()
{
    detectCardExtractionEnable = false;
    showStepWidget ( interruptWidget );
    interruptWidget->showEndVoting();
    showPrintImage = false;
    InvalidCard = false;
    IsInInterrupt = false;

    unsigned int time = SAESGUIConfigurationManager::getInstance()->getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_NULL_VOTING_EXPERIENCE );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &VotingExperienceWidget::endVoting ), time );
}

void VotingExperienceWidget::endVoting()
{
    logger->Debug ( "End voting" );
    votingExperienceController->dispose();

    PrinterStatusNotifier notifier;
    PrinterStatusNotifier::PhysicalStateEnumType state = notifier.getPrinterState();

    bool neop_state = ( ( state &  PrinterStatusNotifier::PSTATE_NEAR_END_OF_PAPER ) == PrinterStatusNotifier::PSTATE_NEAR_END_OF_PAPER );
    bool is_neop_blocking = SAESGUIConfigurationManager::getInstance ()->getWaitingForCardWindowConfiguration ().getIsNeopBlockingCondition ();

    if ( neop_state && is_neop_blocking )
    {
        AlarmMessage::alarm ( AlarmEnum::CONFIGURATION_PRINTERNEOPAFTERVOTE );
    }

    emit();
}

void VotingExperienceWidget::errorVoting()
{
    logger->Debug ( "Error voting" );
    votingExperienceController->dispose();
    emit();
}

void VotingExperienceWidget::RefreshByTooManyExtracts ()
{
    showPrintImage = false;
    showStepWidget ( cardRemovedWidget );

    unsigned int time = SAESGUIConfigurationManager::getInstance()->getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_REMOVED_COUNTER );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &VotingExperienceWidget::stopByExtractProcedure ), time );
}

void VotingExperienceWidget::stopByExtractProcedure ()
{
    try
    {
        SmartcardRemovedPrintDocument smartcardRemovedPrintDocument (
            Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getElectoralConfiguration(),
            "",
            extractionCounter );
        //smartcardRemovedPrintDocument.setClient( this );
        smartcardRemovedPrintDocument.Print();

        PrintDocumentResult res = smartcardRemovedPrintDocument.getDocumentResult ();

        if ( !res.getSuccess() )
        {
            AlarmMessage::alarm ( AlarmEnum::CONFIGURATION_VOTEPRINTINGFAILURE );
        }
    }
    catch ( Smartmatic::System::Exception::ElectionException & ex )
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__ << " " << ex.what();
        logger->Debug ( ss.str() );
    }

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &VotingExperienceWidget::AlarmExtractionCounter ), 100 );
}

void VotingExperienceWidget::AlarmExtractionCounter()
{
    logger->Debug ( "AlarmExtractionCounter" );
    AlarmMessage::alarm ( AlarmEnum::CONFIGURATION_CARDEXTRACTIONCOUNTER );
    votingExperienceController->dispose();
    emit();
}

void VotingExperienceWidget::setSimulationMode ( bool enable )
{
    simulationMode = enable;
}

void VotingExperienceWidget::setDetectCardExtractionEnable ( bool enable )
{
    detectCardExtractionEnable = enable;
}

void VotingExperienceWidget::refresh()
{
    SetLabelsText();
}

void VotingExperienceWidget::endVotingSequence()
{
    connectTimeout.disconnect();
    detectCardExtractionEnable = false;
    endVotingSequenceWidget->start();
    showStepWidget ( endVotingSequenceWidget );
}
