/**
 * @file voting-machine-installation-wizard-widget.h
 * @brief Declares the voting machine installation wizard widget class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */


#ifndef _VOTING_MACHINE_INSTALLATION_WIZARD_WIDGET_H_
#define _VOTING_MACHINE_INSTALLATION_WIZARD_WIDGET_H_

#include <Operation/MemoryEventManager.hxx>
#include <Widgets/SmartmaticBasicEventBox.hxx>
#include <Widgets/SmartmaticBasicHBox.hxx>
#include <Widgets/SmartmaticBasicImage.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/StepWidget.hxx>
#include <Widgets/smartmatic-button.h>
#include <list>
#include <string>
#include <sigc++/sigc++.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Common
        {
            namespace Operation
            {
                namespace Monitor
                {
                    class DeviceTypeMonitor;
                }
            }
        }
        namespace GUI
        {
            namespace Widgets
            {
                class ProgressWidget;

                /**
                 * @class VotingMachineInstallationWizardWidget
                 * @brief Voting machine installation wizard widget.
                 * @author Cesar.bonilla
                 * @date 4/29/2011
                 */
                class VotingMachineInstallationWizardWidget : public Smartmatic::GUI::Widgets::StepWidget
                {

                    /**
                     * @enum WizardStepType
                     * @brief Defines an alias representing the enum.
                     */
                    typedef enum
                    {
                        WIZ_EXTRACT_MEDIA,
                        WIZ_INSTALL_MEDIA,
                        WIZ_VERIFY_MEDIA,
                        WIZ_FINALIZE,
                        WIZ_END
                    } WizardStepType; ///< Type of the wizard step

                    public:
                        static Smartmatic::Log::ISMTTLog* logger; ///< The logger

                        /**
                         * @brief Default constructor.
                         * @author Cesar.bonilla
                         * @date 25/03/2014
                         * @param progress The progress widget information
                         */
                        explicit VotingMachineInstallationWizardWidget(ProgressWidget* progress);

                        /**
                         * @brief Class destructor.
                         * @author Yadickson Soto
                         * @date 25/03/2014
                         */
                        virtual ~VotingMachineInstallationWizardWidget();

                        /**
                         * @brief Initializes this object.
                         * @author Cesar.bonilla
                         * @date 4/29/2011
                         */
                        void Initialize();

                        /**
                         * @brief Dispose of this object, cleaning up any resources it uses.
                         * @author Cesar.bonilla
                         * @date 4/29/2011
                         */
                        void Dispose();

                        sigc::signal<void> signalHide_detected; /**< The signal hide detected */
                        sigc::signal<void> signalFinish_detected; /**< The signal finish detected */

                    protected:
                        /**
                         * @brief Synchronize USB
                         * @author Yadickson Soto
                         * @date 19/06/2013
                         */
                        void synchronize();

                        /**
                         * @brief End synchronize USB
                         * @author Yadickson Soto
                         * @date 19/06/2013
                         */
                        void onEndSynchronize();

                        /**
                         * @brief Executes the button next clicked signal action.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return true if it succeeds, false if it fails.
                         */
                        bool on_buttonNext_clicked_signal();

                        /**
                         * @brief Determines if we can refresh label.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return true if it succeeds, false if it fails.
                         */
                        bool RefreshLabel();

                    private:
                        /**
                         * @brief Calls on_next_button_clicked after a delay timer.
                         * @author Juan Delgado
                         * @date 2/09/2011
                         * @param timer delay for next step.
                         */
                        void NextStep (int timer);

                        /**
                         * @brief Disconnect al connections.
                         * @author Juan Delgado
                         * @date 2/09/2011
                         */
                        void DisconnectConnections();

                        /**
                         * @brief Detect if devices is mount.
                         * @author Yadickson Soto
                         * @date 14/06/2013
                         * @return true if not all device mount.
                         */
                        bool DetectAllMount();

                        /**
                         * @brief Extracts the given time_num.
                         * @author Cesar.bonilla
                         * @date 4/29/2011
                         * @return true if it succeeds, false if it fails.
                         */
                        bool Extract();

                        /**
                         * @brief Forward process.
                         * @author Cesar.bonilla
                         * @date 4/29/2011
                         */
                        void mForwardProcess();

                        /**
                         * @brief Handles button next clicked signals.
                         * @author Cesar.bonilla
                         * @date 4/29/2011
                         */
                        void on_buttonNext_clicked();

                        /**
                         * @brief Check data in memory path.
                         * @author Juan Delgado
                         * @date 2/09/2011
                         */
                        void checkData();

                        /**
                         * @brief Procces the check data button.
                         * @author Juan Delgado
                         * @date 2/09/2011
                         */
                        void processCheckDataOk();

                        /**
                         * @brief Process the check data error.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */
                        void processCheckDataError();

                        /**
                         * @brief Set top image by step.
                         * @author Yadickson Soto
                         * @date 24/03/2014
                         */
                        void setImageByStep();

                        /**
                         * @brief Sets accessible name to labels and images by step.
                         * @author Yadickson Soto
                         * @date 24/03/2014
                         */
                        void setAccessibleNameByStep();

                        /**
                         * @brief Callback signal end synchronize
                         * @author Yadickson Soto
                         * @date 24/03/2014
                         */
                        void endSynchronize();

                        /**
                         * @brief  Process the error when memory is removed before time.
                         * @author manuel.arguelles
                         * @date   06/07/2017
                         */
                        void processMemoryRemovedError();

                        /**
                         * @brief  Wait until memory media is ready.
                         * @author manuel.arguelles
                         * @date   Jul 12, 2017
                         * @return true when ready, false on error.
                         */
                        bool waitForMemoryMount();

                        /**
                         * @brief  Process the warning when the memory was removed while synchronizing.
                         * @author manuel.arguelles
                         * @date   26/05/2022
                         */
                        void processMemoryRemovedWhileSynchronizing();

                        /**
                         * @brief  Process the warning when the memory was removed while checking data.
                         * @author manuel.arguelles
                         * @date   26/05/2022
                         */
                        void processMemoryRemovedWhileChecking();

                        /**
                         * @brief  Checks if both memories are connected to resume synchronization.
                         * @author manuel.arguelles
                         * @date   26/05/2022
                         * @return true if the memories are not connected, false if they are.
                         */
                        bool shouldResumeSynchronizing();

                        /**
                         * @brief  Checks if both memories are connected to resume checking data.
                         * @author manuel.arguelles
                         * @date   26/05/2022
                         * @return true if the memories are not connected, false if they are.
                         */
                        bool  shouldResumeChecking();

                    private:
                        ProgressWidget*                                   m_progress;
                        Common::Operation::Monitor::DeviceTypeMonitor*    m_usbMonitor;
                        unsigned int                                      m_imageCounter;
                        Glib::Thread*                                     m_threadCheckData;
                        Glib::Thread*                                     m_threadSync;
                        Gdk::Color                                        m_hboxStepColor;
                        Gdk::Color                                        m_imageStepColor;
                        Glib::Dispatcher                                  m_signalEndSynchronize;
                        Gtk::HButtonBox                                   m_hboxButton;
                        Smartmatic::GUI::Widgets::SmartmaticBasicEventBox m_eventBoxImageStep;
                        Smartmatic::GUI::Widgets::SmartmaticBasicEventBox m_eventBoxStep;
                        Smartmatic::GUI::Widgets::SmartmaticBasicHBox     m_hboxStep;
                        Smartmatic::GUI::Widgets::SmartmaticBasicImage    m_imageStep;
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel    m_labelStep;
                        Smartmatic::GUI::Widgets::SmartmaticBasicVBox     m_vboxPrincipal;
                        Smartmatic::GUI::Widgets::SmartmaticButton        m_nextButton;
                        sigc::connection                                  m_connClickButton;
                        sigc::connection                                  m_connDetect;
                        sigc::connection                                  m_connExtract;
                        sigc::connection                                  m_connNextButton;
                        std::list<WizardStepType>                         m_stepSequence;
                        std::list<WizardStepType>::const_iterator         m_processTracker;
                };
            }
        }
    }
}

#endif /* _VOTING_MACHINE_INSTALLATION_WIZARD_WIDGET_H_ */
