/**
 * @file WizardWidget.cpp
 * @brief Body wizard widget class
 * @date 28/05/2011
 * @author Gerardo Hernandez
 */

#include "WizardWidget.h"
#include "Functionalities/saes-functionality-manager.h"
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>
#include <Configuration/MachineModelEnum.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Configuration;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets;

WizardWidget::WizardWidget ( std::vector<WizardConfiguration> configuration, std::string currentState )
    : SmartmaticBasicFrame()
{
    set_shadow_type ( Gtk::SHADOW_NONE );
    buttonIndex = 0;
    state = currentState;
    wizardWidgetConfiguration = configuration;

    currentWizardConfiguration = WizardConfiguration::GetWizardWidgetConfigurationByState ( configuration, currentState );
    boxPrincipal = manage ( new SmartmaticBasicVBox() );
    toolbox = manage ( new WizardToolkit ( WizardConfiguration::GetCurrentToolbox ( configuration, currentState ), currentWizardConfiguration ) );
    toolBoxConnection = toolbox->WizardToolkitElementClickSigc().connect ( sigc::mem_fun ( this, &WizardWidget::on_toolbox_element_clicked ) );

    timeline = manage ( new WizardTimeline ( currentWizardConfiguration.getTimelineRelativeImagesPath(), currentWizardConfiguration.getTimelineCodes() ) );
    boxCentral = manage ( new SmartmaticBasicHBox() );
    principalButtonBox = manage ( new SmartmaticBasicVBox() );
    helpTitle = manage ( new SmartmaticBasicLabel() );
    helpDetail = manage ( new SmartmaticBasicLabel() );
    eventBoxTimeLine = manage ( new SmartmaticBasicEventBox() );
    eventBoxPrincipalButtons = manage ( new SmartmaticBasicEventBox() );
    eventBoxToolbox = manage ( new SmartmaticBasicEventBox() );
    eventBoxHelpPanel = manage ( new SmartmaticBasicEventBox() );
    eventBoxToolbox->add ( *toolbox );
    eventBoxTimeLine->add ( *timeline );
    principalButtonTable = new SmartmaticBasicTable();
    std::vector<MenuOption> principalMenuOption = WizardConfiguration::GetCurrentPrincipaButtonBox ( configuration, currentState );

    for ( unsigned int i = 0; i < principalMenuOption.size(); i++ )
    {
        Smartmatic::GUI::Widgets::SmartmaticButton * button;
        button = manage ( new Smartmatic::GUI::Widgets::SmartmaticButton ( SmartmaticButton::Menu ) );

        button->set_name ( GetButtonName ( "theme_menu_widget_secondary_button" ) );
        //agregamos el boton mapeado con su opcion
        optionsBybuttonList.insert ( std::pair<SmartmaticButton *, MenuOption> ( button, principalMenuOption[i] ) );
        button->signal_clicked().connect ( sigc::bind ( sigc::mem_fun ( *this, &WizardWidget::on_button_clicked ), button, principalMenuOption[i] ) );
        //asignamos el tamaño del boton
        button->set_size_request ( principalMenuOption[i].width, principalMenuOption[i].height );
        principalButtonTable->attach ( *button, 0, 1, i, i + 1, Gtk::SHRINK, Gtk::SHRINK, 3, 3 );

        button->setAccessibleName ( principalMenuOption[i].accessibleName );
    }
    boxCentral->pack_start ( *principalButtonTable, true, false, 5 );
    helpBox = manage ( new SmartmaticBasicVBox ( false, 10 ) );
    helpTitle->set_line_wrap ( true );
    helpTitle->set_line_wrap_mode ( Pango::WRAP_WORD );
    Pango::FontDescription fontBack = helpTitle->get_style()->get_font();
    fontBack.set_size ( 16 * Pango::SCALE );
    fontBack.set_weight ( Pango::WEIGHT_BOLD );
    helpTitle->modify_font ( fontBack );
    helpDetail->set_alignment ( 0.50, 0.10 );
    helpDetail->set_line_wrap ( true );
    helpDetail->set_line_wrap_mode ( Pango::WRAP_WORD );
    helpBox->pack_start ( *helpTitle, false, false, 3 );
    helpBox->pack_start ( *helpDetail, false, false, 3 );
    eventBoxHelpPanel->add ( *helpBox );
    boxCentral->pack_end ( *eventBoxHelpPanel, false, false, 5 );

    Gtk::Alignment * aligmentTimeLine = manage ( new Gtk::Alignment() );
    aligmentTimeLine->set ( 0.5, 0.5, 0, 0 );
    aligmentTimeLine->add ( *eventBoxTimeLine );

    boxPrincipal->pack_start ( *aligmentTimeLine, false, false, 0 );

    Gtk::HSeparator * separator = manage ( new Gtk::HSeparator() );
    boxPrincipal->pack_start ( *separator, false, true, 0 );

    boxPrincipal->pack_start ( *boxCentral, true, true, 0 );
    Gtk::Alignment * aligment = manage ( new Gtk::Alignment() );
    aligment->set ( 0.5, 0.5, 0, 0 );
    aligment->add ( *eventBoxToolbox );
    boxPrincipal->pack_start ( *aligment, false, false, 0 );
    this->add ( *boxPrincipal );
    show_all_children ( true );

    setAccessibleName ( currentWizardConfiguration.getAccessibleName() );
    timeline->setAccessibleName ( currentWizardConfiguration.getTimelineAccessibleName() );
    boxCentral->setAccessibleName ( "WizardCentralContainer" );
    principalButtonBox->setAccessibleName ( "VerticalButtonContainer" );
    principalButtonTable->setAccessibleName ( "VerticalButtonContainer" );
    eventBoxHelpPanel->setAccessibleName ( "VerticalInfoContainer" );
    helpTitle->setAccessibleName ( currentWizardConfiguration.getHelpTitleAccessibleName () );
    helpDetail->setAccessibleName ( currentWizardConfiguration.getHelpContentAccessibleName() );
}

std::string WizardWidget::GetButtonName ( const std::string& suffix )
{
    std::stringstream out;
    out << buttonIndex++;
    std::string buttonName = suffix + out.str();
    return buttonName;
}

void WizardWidget::RefreshLanguage()
{
    std::map<SmartmaticButton *, MenuOption>::iterator buttonIter;

    for ( buttonIter = optionsBybuttonList.begin(); buttonIter != optionsBybuttonList.end(); ++buttonIter )
    {
        helpTitle->set_label ( _ ( currentWizardConfiguration.getHelpTitle().c_str() ) );
        helpDetail->set_label ( _ ( currentWizardConfiguration.getHelpContent().c_str() ) );
        buttonIter->first->set_label ( _ ( buttonIter->second.nameKey.c_str() ) );

        int buttonWidth = buttonIter->second.width;
        int buttonHeight = buttonIter->second.height;

        if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
        {
            double guiReconfigurationFactor = SMTTGUIConfigurationManager::getInstance()->getConfiguration()->getSmartmaticGUIMachineModelConfiguration().getGuiReconfigurationFactor();

            buttonWidth  *= guiReconfigurationFactor;
            buttonHeight *= guiReconfigurationFactor;
        }

        buttonIter->first->set_size_request ( buttonWidth,  buttonHeight );
    }
}

void WizardWidget::SetKey ( KeysActionsCode value )
{
}

void WizardWidget::on_button_clicked ( SmartmaticButton * widget, MenuOption _option )
{
    MenuOption currentMenuOption;

    if ( widget == NULL )
    {
        currentMenuOption = _option;
    }
    else
    {
        std::map<SmartmaticButton *, MenuOption>::iterator iter;
        iter = optionsBybuttonList.find ( widget );

        if ( iter != optionsBybuttonList.end() )
        {
            currentMenuOption = iter->second;
        }
    }

    Smartmatic::SAES::Functionality::SaesFunctionalityManager::getInstance()->ExecuteOptionSelected ( currentMenuOption.optionDescriptor );

    m_functionality_executed.emit();
}

void WizardWidget::on_toolbox_element_clicked ( MenuOption option )
{
    Smartmatic::SAES::Functionality::SaesFunctionalityManager::getInstance()->ExecuteOptionSelected ( option.optionDescriptor );

    m_functionality_executed.emit();
}

WizardWidget::~WizardWidget()
{
    if ( boxPrincipal )
    {
        delete ( boxPrincipal );
    }

    if ( toolbox )
    {
        delete ( toolbox );
    }

    if ( timeline )
    {
        delete ( timeline );
    }

    if ( boxCentral )
    {
        delete ( boxCentral );
    }

    if ( principalButtonBox )
    {
        delete ( principalButtonBox );
    }

    if ( helpTitle )
    {
        delete ( helpTitle );
    }

    if ( helpDetail )
    {
        delete ( helpDetail );
    }

    if ( eventBoxTimeLine )
    {
        delete ( eventBoxTimeLine );
    }

    if ( eventBoxPrincipalButtons )
    {
        delete ( eventBoxPrincipalButtons );
    }

    if ( eventBoxToolbox )
    {
        delete ( eventBoxToolbox );
    }

    if ( eventBoxHelpPanel )
    {
        delete ( eventBoxHelpPanel );
    }

    if ( principalButtonTable )
    {
        delete ( principalButtonTable );
    }

    if ( helpBox )
    {
        delete ( helpBox );
    }
}
