/**
 * @file	VotingWidgetManager.hxx
 * @brief	voting widget manager class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef __VOTING_WIDGET_MANAGER_H__
#define __VOTING_WIDGET_MANAGER_H__

#include <string>
#include <map>
#include <Voting/ElectoralConfiguration.hxx>
#include "Gui/Widgets/party-selection-widget.h"
#include "Gui/Widgets/ballot-option-selection-widget.h"
#include "Runtime/Environment/saes-directories.h"
#include <Operation/Initiator/ICheckCondition.hxx>
#include <Log/ISMTTLog.h>
#include <System/Exception/HashException.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Operation
            {
                namespace Voting
                {
                    /**
                     * @class	VotingWidgetManager
                     * @brief	Header class VotingWidgetManager.
                     * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                     * @date	23-03-2011
                     */

                    class VotingWidgetManager : public Smartmatic::SAES::Operation::Initiator::ICheckCondition
                    {
                        public:

                            typedef std::map < std::string, Smartmatic::SAES::GUI::Widgets::BallotOptionSelectionWidget * > BallotOptionWidgetMap;
                            typedef std::map < std::string , Smartmatic::SAES::GUI::Widgets::PartySelectionWidget * >      PartyWidgetMap;

                        private:

                            /**
                             * @fn	VotingWidgetManager::VotingWidgetManager();
                             * @brief	Class constructor.
                             * @author	Fsilva
                             * @date	28/04/2011
                             */
                            VotingWidgetManager();

                        public:
                            /**
                             * @fn	Smartmatic::SAES::Operation::Initiator::TestStatus::Tests VotingWidgetManager::check ();
                             * @brief	Gets the check.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @return	.
                             */
                            Smartmatic::SAES::Operation::Initiator::TestStatus::Tests check ();

                            /**
                             * @fn	void VotingWidgetManager::init() throw ( Smartmatic::System::Exception::ElectionException,
                             * 			Smartmatic::System::Exception::VotingDeviceException );
                             * @brief	method initialization.
                             * @author	Fsilva
                             * @date	28/04/2011
                             *
                             * 				@throw ElectionException.
                             * @exception	Smartmatic::System::Exception::ElectionException			Thrown when an election error
                             * 																																	condition occurs.
                             * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                             * 																																	error condition occurs.
                             */

                            void init()
                            throw ( Smartmatic::System::Exception::ElectionException,
                                    Smartmatic::System::Exception::VotingDeviceException,
                                    Smartmatic::System::Exception::HashException);

                            /**
                             * @fn	void VotingWidgetManager::SetElectoralConfiguration ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                             * @brief	Set electoral cofnfiguration.
                             * @author	Juan.Delgado
                             * @date	09/08/2011
                             * @param	electoralConfiguration	electoral configuration.
                             */

                            void SetElectoralConfiguration ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );

                            /**
                             * @fn	void VotingWidgetManager::SetSaesDirectories ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * saesDir );
                             * @brief	Set Saes directories.
                             * @author	Juan.Delgado
                             * @date	09/08/2011
                             * @param	saesDir	Saes directories.
                             */

                            void SetSaesDirectories ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * saesDir );

                            /**
                             * @fn	virtual VotingWidgetManager::~VotingWidgetManager();
                             * @brief	Class destructorer.
                             * @author	Fsilva
                             * @date	28/04/2011
                             */

                            virtual ~VotingWidgetManager();

                            /**
                             * @fn	void VotingWidgetManager::dispose();
                             * @brief	method dispose.
                             * @author	Fsilva
                             * @date	28/04/2011
                             */

                            void dispose();

                            /**
                             * @fn	Smartmatic::SAES::GUI::Widgets::PartySelectionWidget * VotingWidgetManager::getPartySelectionWidgetByContest ( std::string contestCode );
                             * @brief	method getter party selection widget.
                             * @author	Fsilva
                             * @date	28/04/2011
                             * @param	contestCode	contest code.
                             * @return	party selection widget.
                             */

                            Smartmatic::SAES::GUI::Widgets::PartySelectionWidget * getPartySelectionWidgetByContest ( std::string contestCode );

                            /**
                             * @fn	Smartmatic::SAES::GUI::Widgets::BallotOptionSelectionWidget * VotingWidgetManager::getBallotOptionSelectionWidgetByContestParty ( std::string contestCode,
                             * 			std::string partyCode );
                             * @brief	method getter ballot option selection widget.
                             * @author	Fsilva
                             * @date	28/04/2011
                             * @param	contestCode	contest code.
                             * @param	partyCode	 	The party code.
                             * @return	ballot option selection widget.
                             */

                            Smartmatic::SAES::GUI::Widgets::BallotOptionSelectionWidget * getBallotOptionSelectionWidgetByContestParty ( std::string contestCode, std::string partyCode );

                            /**
                             * @fn	unsigned int VotingWidgetManager::getBallotOptionSelectionTotalBallotsWidgetByContestParty ( std::string contestCode,
                             * 			std::string partyCode );
                             * @brief	method getter number of ballot option displayed in selection widget.
                             * @author	Fsilva
                             * @date	28/04/2011
                             * @param	contestCode	contest code.
                             * @param	partyCode	 	The party code.
                             * @return	number of ballots.
                             */

                            unsigned int getBallotOptionSelectionTotalBallotsWidgetByContestParty ( std::string contestCode, std::string partyCode );

                            /**
                             * @fn	void VotingWidgetManager::clearBallotOptionSelectionWidget();
                             * @brief	method clear ballot option selection widget.
                             * @author	Fsilva
                             * @date	28/04/2011
                             */

                            void clearBallotOptionSelectionWidget();

                            /**
                             * @fn	PartyWidgetMap VotingWidgetManager::getMapPartySelectionWidget();
                             * @brief	Gets the map party selection widget.
                             * @author	Fsilva
                             * @date	28/04/2011
                             * @return	The map party selection widget.
                             */

                            PartyWidgetMap & getMapPartySelectionWidget();

                            /**
                             * @fn	std::map < std::string , BallotOptionWidgetMap > VotingWidgetManager::getMapBallotOptionSelectionWidget();
                             * @brief	Gets the map ballot option selection widget.
                             * @author	Fsilva
                             * @date	28/04/2011
                             * @return	The map ballot option selection widget.
                             */

                            std::map < std::string , BallotOptionWidgetMap > & getMapBallotOptionSelectionWidget();

                            /**
                             * @fn	void VotingWidgetManager::update();
                             * @brief	Sets a theme.
                             * @author	Fsilva
                             * @date	28/04/2011
                             */

                            void update();

                            /**
                             * @fn	static VotingWidgetManager * VotingWidgetManager::getInstance();
                             * @brief	Singleton instance getter method
                             * @author	Fsilva
                             * @date	28/04/2011
                             * @return	null if it fails, else the instance.
                             */

                            static VotingWidgetManager * getInstance();

                        private:

                            Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration; ///<The electoral configuration
                            Smartmatic::SAES::Runtime::Environment::SaesDirectories * saesDir;	///< The saes directories
                            static VotingWidgetManager * instance; ///< The instance of voting widget manager
                            PartyWidgetMap mapPartySelectionWidget; ///< The map party selection widget
                            std::map < std::string , BallotOptionWidgetMap > mapBallotOptionSelectionWidget; ///< The map ballot option selection widget

                            static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                    };
                }
            }
        }
    }
}

#endif /*__VOTING_WIDGET_MANAGER_H__*/
