/**
 * @file   CommandBaseEurn.cxx
 * @brief  Implementation of CommandBaseEurn class.
 * @author manuel.arguelles
 * @date   Aug 25, 2017
 */

#include "CommandBaseEurn.hxx"
#include "EurnHeaderCommandElement.hxx"
#include "DataCommandElement.hxx"
#include "LengthCommandElement.hxx"
#include "ChecksumCommandElement.hxx"
#include "EurnFooterCommandElement.hxx"
#include <Connections/IConnection.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Configuration/EurnDeviceSchema.hxx>
#include <Configuration/EurnDeviceConfigurationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const unsigned char ERROR_COMMAND    = 0x20;
static const unsigned char ERROR_MASK       = 0x20;

static const std::string::size_type DATA0_INDEX     = 3;
static const std::string::size_type DATA1_INDEX     = 4;
static const std::string::size_type DATA_SIZE_INDEX = 2;
static const std::string::size_type FOOTER_INDEX    = 5;
static const std::string::size_type HEADER_INDEX    = 0;
static const std::string::size_type PACKAGE_SIZE    = 6;
static const std::string::size_type STATUS_INDEX    = 4;

static const unsigned char DATA_SIZE = 0x02;
static const unsigned char FOOTER    = 0x0A;
static const unsigned char HEADER    = 0x77;

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Commands;
using namespace Smartmatic::SAES::HW::Connections;
using namespace Smartmatic::SAES::HW::Configuration;

ISMTTLog* CommandBaseEurn::logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.HW.Commands.CommandBaseEurn",
    GETTEXT_PACKAGE);

CommandBaseEurn::CommandBaseEurn(IConnection* connection)
    : CommandStructure(connection)
{
    EurnDeviceConfiguration& configuration = EurnDeviceConfigurationManager::getInstance()->getConfiguration();

    retriesToSend        = configuration.getRetriesToSendCommands();
    retriesToReceive     = configuration.getRetriesToReceiveAnswer();
    timeoutBeforeReceive = configuration.getTimeoutBeforeReceiveAnswer();
    hasResponse_         = true;
}

CommandBaseEurn::~CommandBaseEurn()
{
}

CommandStructure::CommandList CommandBaseEurn::getCommandList()
{
    CommandList commands;

    commands.push_back(new EurnHeaderCommandElement());
    commands.push_back(new DataCommandElement(commandByte));
    commands.push_back(new LengthCommandElement(data.size()));
    commands.push_back(new DataCommandElement(data));
    commands.push_back(new EurnFooterCommandElement());

    return commands;
}

EurnError CommandBaseEurn::getError(const std::vector<unsigned char>& response)
{
    EurnError error = EURN_ERROR_FORMAT;

    if (response.size() == PACKAGE_SIZE &&
        response.at(HEADER_INDEX) == HEADER &&
        response.at(FOOTER_INDEX) == FOOTER &&
        response.at(DATA_SIZE_INDEX) == DATA_SIZE)
    {
        switch (getStatus() & ERROR_MASK)
        {
            case ERROR_COMMAND:
                logger->Debug(_("Unknown command."));
                error = EURN_ERROR_UNKNOWN_COMMAND;
                break;
            default:
                error = EURN_ERROR_NO_ERROR;
                break;
        }
    }
    else
    {
        logger->Debug(_("Malformed response packet"));
    }

    return error;
}

unsigned char CommandBaseEurn::getStatus()
{
    if (response.size() < STATUS_INDEX)
    {
        logger->Debug(_("Response is empty or incomplete."));
        return 0x00;
    }

    return response.at(STATUS_INDEX);
}

unsigned char CommandBaseEurn::getVersionByte()
{
    if (response.size() < DATA0_INDEX)
    {
        logger->Debug(_("Response is empty or incomplete."));
        return 0x00;
    }

    return response.at(DATA0_INDEX);
}

bool CommandBaseEurn::processResponse(const std::vector<unsigned char>& response)
{
    return getError(response) == EURN_ERROR_NO_ERROR;
}
