/**
 * @file   CommandBasePDB.cxx
 * @brief  Implementation of CommandBasePDB class.
 * @author manuel.arguelles
 * @date   Oct 31, 2017
 */

#include "CommandBasePDB.hxx"
#include "DataCommandElement.hxx"
#include <stdint.h>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>

using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;

static const std::size_t   HEADER_SIZE   = 24;
static const std::size_t   ID_POS_END    = 4;
static const std::size_t   LEN_POS_BEGIN = 20;
static const std::size_t   LEN_POS_END   = 24;
static const std::size_t   MAC_SIZE      = 16;
static const std::size_t   NOISE_SIZE    = 2;
static const uint32_t      PDB_ID        = 1454003626;
static const uint32_t      SENDER_ID     = 1420449194;
static const unsigned char NOISE_BYTE    = 0xFF;

/**
 * @brief  Converts an integer represented in little-endian as a vector of 4 bytes into an unsigned integer.
 * @author manuel.arguelles
 * @date   Oct 31, 2017
 * @param  value The 4 bytes vector of unsigned char.
 * @return The unsigned 32 bits integer.
 */
static uint32_t vectorToUint32(const std::vector<unsigned char>& value)
{
    uint32_t result = 0;

    result += value[0];
    result += value[1] << 8;
    result += value[2] << 16;
    result += value[3] << 24;

    return result;
}

/**
 * @brief  Verify if the given id represented in little-endian as a vector of 4 bytes corresponds with a valid PDB sender.
 * @author gustavo.garcia
 * @date   Nov 21, 2017
 * @param  value The 4 bytes vector of unsigned char.
 * @return bool value.
 */
static bool verifyCurrentPDBid(uint32_t id)
{
    return id == PDB_ID;
}

/**
 * @brief  Represent a 32 bits unsigned integer into a little-endian vector of bytes.
 * @author manuel.arguelles
 * @date   Oct 31, 2017
 * @param  value The 32 bits unsigned integer.
 * @return A vector of 4 bytes with the integer value.
 */
static std::vector<unsigned char> uint32ToVector(const uint32_t value)
{
    std::vector<unsigned char> result;

    result.push_back(value & 0xFF);
    result.push_back((value >> 8) & 0xFF);
    result.push_back((value >> 16) & 0xFF);
    result.push_back((value >> 24) & 0xFF);

    return result;
}

using namespace Smartmatic::SAES::HW::Commands;
using namespace Smartmatic::SAES::HW::Connections;

CommandBasePDB::CommandBasePDB(IConnection* connection)
    : CommandStructure(connection)
{
}

CommandBasePDB::~CommandBasePDB()
{
}

void CommandBasePDB::addSenderId()
{
    std::vector<unsigned char> sender = uint32ToVector(SENDER_ID);

    commandByte.assign(sender.begin(), sender.end());
}

void CommandBasePDB::addMAC()
{
    commandByte.insert(commandByte.end(), MAC_SIZE, 0x00);
}

void CommandBasePDB::addNoise()
{
    data.insert(data.begin(), NOISE_SIZE, NOISE_BYTE);
}

void CommandBasePDB::addCommandLength()
{
    std::vector<unsigned char> length;

    length = uint32ToVector(static_cast<uint32_t>(data.size()));

    commandByte.insert(commandByte.end(), length.begin(), length.end());
}

CommandStructure::CommandList CommandBasePDB::getCommandList()
{
    CommandList commands;

    addSenderId();
    addMAC();
    addNoise();
    addCommandLength();

    commands.push_back(new DataCommandElement(commandByte));
    commands.push_back(new DataCommandElement(data));

    return commands;
}

bool CommandBasePDB::processResponse(const std::vector<unsigned char>& response)
{
    bool valid = false;

    if (response.size() > HEADER_SIZE)
    {
        std::vector<unsigned char> id(response.begin(), response.begin() + ID_POS_END);
        std::vector<unsigned char> len(response.begin() + LEN_POS_BEGIN, response.begin() + LEN_POS_END);

        if ((verifyCurrentPDBid(vectorToUint32(id))) && (vectorToUint32(len) == (response.size() - HEADER_SIZE)))
        {
            valid = true;
        }
    }

    return valid;
}
