/**
 * @file	command-structure.cc
 * @brief	command structure class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include "command-structure.h"
#include <algorithm>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate
using namespace Smartmatic::SAES::HW::Commands;
using namespace Smartmatic::SAES::HW::Connections;

CommandStructure::CommandStructure ( Smartmatic::SAES::HW::Connections::IConnection * connection )
    : connection ( connection )
{
    retriesToSend = 3;
    retriesToReceive = 3;
    timeoutBeforeReceive = 0;
    hasResponse_ = false;
    responseIsNotEmpty = false;
    responseIsValid = false;
}

CommandStructure::~CommandStructure()
{

}

void CommandStructure::clearCommandList ( CommandList & commands )
{
    CommandList::iterator it;

    for ( it = commands.begin(); it != commands.end(); ++it )
    {
        CommandElement * command = ( *it );

        if ( command )
        {
            delete command;
            command = NULL;
        }
    }

    commands.clear();
}

bool CommandStructure::analyzeResponse ()
{
    bool result = false;

    for ( int retries = 0; retries < getRetriesToReceive() && !result; retries++ )
    {
        usleep ( timeoutBeforeReceive + ( 3 * connection->getTimeOut() ) );
        response = connection->readBuffer ();
        result = !response.empty();
    }

    return result;
}

bool CommandStructure::processResponse ( const std::vector<unsigned char> & response )
{
    return true;
}

bool CommandStructure::sendCommand ( const std::vector<unsigned char> & command )
{
    bool result = false;

    for ( int index = 0; index < getRetriesToSend() && !result; index++ )
    {
        if ( !connection->sendCommand ( command ) )
        {
            continue;
        }
        else
        {
            result = true;
        }
    }

    return result;
}

std::vector<unsigned char> CommandStructure::getCommand ( const CommandList & commands )
{
    std::vector<unsigned char> byteLevelFullCommand;
    CommandList::const_iterator it;

    for ( it = commands.begin(); it != commands.end(); ++it )
    {
        CommandElement * command = ( *it );

        if ( command )
        {
            command->addToCommand ( byteLevelFullCommand );
        }
    }

    return byteLevelFullCommand;
}

bool CommandStructure::execute ()
{
    bool result = false;
    response.clear();

    if ( connection )
    {
        CommandList commands;

        commandByte.clear();
        data.clear();

        initialize();

        commands = getCommandList ();
        std::vector<unsigned char> byteLevelFullCommand = getCommand ( commands );
        clearCommandList ( commands );

        result = sendCommand ( byteLevelFullCommand );

        if ( result && hasResponse() )
        {
            responseIsNotEmpty = analyzeResponse();
            responseIsValid    = true;

            if (responseIsNotEmpty)
            {
                responseIsValid = processResponse(response);
            }

            result = responseIsNotEmpty && responseIsValid;
        }

        if ( !result )
        {
            response.clear();
        }
    }

    return result;
}

const std::vector<unsigned char> & CommandStructure::getResponse () const
{
    return response;
}

bool CommandStructure::hasResponse()
{
    return hasResponse_;
}

int CommandStructure::getRetriesToReceive()
{
    return retriesToReceive;
}

int CommandStructure::getRetriesToSend()
{
    return retriesToSend;
}

bool CommandStructure::isResponseNotEmpty() const
{
	return responseIsNotEmpty;
}

bool CommandStructure::isResponseValid() const
{
	return responseIsValid;
}
