/**
 * @file	command-structure.h
 * @brief	Declares the command structure class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#ifndef _COMMAND_STRUCTURE_H_
#define _COMMAND_STRUCTURE_H_

#include <Connections/IConnection.hxx>
#include <Commands/CommandElement.hxx>
#include <list>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Commands
            {
                /**
                 * @class	CommandStructure
                 * @brief	Base class that defines the basic structure of all commands. It contains sections Sender,
                 * 				MAC, Lenght, Noise, MultiByteIdentifier, CommandByte and Data. Each section can be of any
                 * 				lenght or non-existant. The class receives the providers for some sections, and the
                 * 				children should define other specific sections like MultiByteIdentifier, CommandByte and
                 * 				Data.
                 * @author	Fsilva
                 * @date	27/04/2011
                 */
                class CommandStructure
                {
                    public:

                        typedef std::list < CommandElement * > CommandList; ///< Type definition for CommandElement list

                        /**
                         * @fn	CommandStructure::CommandStructure( Smartmatic::SAES::HW::Connections::IConnection * connection );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	18/02/2014
                         * @param 	connection The connection provider
                         */
                        explicit CommandStructure ( Smartmatic::SAES::HW::Connections::IConnection * connection );

                        /**
                         * @fn	virtual ~CommandStructure();
                         * @brief	Constructor destructorer.
                         * @author	Yadickson Soto
                         * @date	14/02/2014
                         */
                        virtual ~CommandStructure();

                        /**
                         * @fn	bool execute ();
                         * @brief	Executes the command on the connection provider.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         * @return	true is successfully.
                         */
                        bool execute ();

                        /**
                         * @fn	const std::vector<unsigned char> & getResponse () const;
                         * @brief	Gets the response.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         * @return	The result response.
                         */
                        const std::vector<unsigned char> & getResponse () const;

                        /**
                         * @fn	virtual bool isResponseNotEmpty() const;
                         * @brief	Gets whether the response is not empty.
                         * @author	Eduardo Fernández
                         * @date	01/10/2018
                         * @return	Whether the response is not empty.
                         */
                        virtual bool isResponseNotEmpty() const;

                       /**
                        * @fn  virtual bool isResponseValid() const;
                        * @brief	Gets whether the response is valid.
                        * @author	Eduardo Fernández
                        * @date	    01/10/2018
                        * @return	Whether the response is not empty.
                        */
                        virtual bool isResponseValid() const;

                    protected:

                        // These vectors define the sub-sections of the data that is sent to the MTD. It is implicitly understood
                        // that most commands independently of the MTD will also have this sub-sections

                        std::vector<unsigned char> commandByte; ///< The command byte
                        std::vector<unsigned char> data;	///< The data
                        std::vector<unsigned char> response;	///< this vector represents the response received after sending the command, if such a response is available

                        int retriesToSend ; ///< The retries to send
                        int retriesToReceive;	///< The retries to receive
                        int timeoutBeforeReceive; ///< The time between retries
                        bool hasResponse_;   ///< true if has response
                        bool responseIsNotEmpty;   ///< true if the response is not empty
                        bool responseIsValid;   ///< true if the response is valid

                    private:

                        Smartmatic::SAES::HW::Connections::IConnection * connection; ///< The connection provider

                    protected:

                        /**
                         * @fn	virtual void CommandStructure::initialize() = 0;
                         * @brief	Initializes this object.
                         * @author	Yadickson Soto
                         * @date	18/02/2014
                         */

                        virtual void initialize() = 0;

                        /**
                         * @fn	virtual bool hasResponse();
                         * @brief	Gets whether a response is available.
                         * @author	Yadickson Soto
                         * @date	19/02/2014
                         * @return	true if has response
                         */

                        virtual bool hasResponse();

                        /**
                         * @fn	virtual int getRetriesToReceive();
                         * @brief	Gets number of Retries to receive.
                         * @author	Yadickson Soto
                         * @date	19/02/2014
                         * @return	retries to receive.
                         */

                        virtual int getRetriesToReceive();

                        /**
                         * @fn	virtual int getRetriesToSend();
                         * @brief	Gets number of Retries to send.
                         * @author	Yadickson Soto
                         * @date	19/02/2014
                         * @return	retries to send.
                         */

                        virtual int getRetriesToSend();

                        /**
                         * @fn	virtual void clearCommandList( CommandList & commands );
                         * @brief	Resets the command list.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         * @param 	commands The command element list
                         */

                        virtual void clearCommandList ( CommandList & commands );

                        /**
                         * @fn	virtual CommandList getCommandList () = 0;
                         * @brief	Gets the complete command list (list of command elements) to perform the command.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         * @return	The command elements to make full command result.
                         */
                        virtual CommandList getCommandList () = 0;

                        /**
                         * @fn	virtual bool analyzeResponse ();
                         * @brief	Analyze response from the connection provider, implement retry read from connection provider.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         * @return true if analyze response if successfully
                         */
                        virtual bool analyzeResponse ();

                        /**
                         * @fn	virtual bool CommandStructure::processResponse(const std::vector<unsigned char> & response);
                         * @brief	Processes the response and checks for its validity.
                         * @author	Fsilva
                         * @date	27/04/2011
                         * @param	response	The response.
                         * @return	true if it succeeds, false if it fails.
                         */

                        virtual bool processResponse ( const std::vector<unsigned char> & response );

                        /**
                         * @fn	virtual bool sendCommand ( const std::vector<unsigned char> & fullCommand );
                         * @brief	Sends data in a synchronous way.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         * @param	command The full command to send
                         * @return	true if successfully send command.
                         */
                        virtual bool sendCommand ( const std::vector<unsigned char> & command );

                        /**
                         * @fn	virtual std::vector<unsigned char> getCommand ( const CommandList & commands );
                         * @brief	Transforms the list of command elements objects into a byte-level command ready to be sent to a device.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         * @param	commands	The command elements to make full command result.
                         * @return	The full command result.
                         */
                        virtual std::vector<unsigned char> getCommand ( const CommandList & commands );
                };
            }
        }
    }
}

#endif // _COMMAND_STRUCTURE_H_

