/**
 * @file   UsbConnection.hxx
 * @brief  Declaration of UsbConnection class.
 * @author manuel.arguelles
 * @date   Aug 30, 2017
 */

#ifndef USB_CONNECTION_HXX
#define USB_CONNECTION_HXX

#include <Connections/IConnection.hxx>
#include <System/SafeVector.hxx>
#include <libusb.h>
#include <stdint.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {

                /**
                 * @class  UsbConnection
                 * @brief  Connection class for the e-urn 3.0.
                 * @author manuel.arguelles
                 * @date   Aug 30, 2017
                 */
                class UsbConnection : public IConnection
                {
                    public:

                        /**
                         * @brief  Creates and object of UsbConnection class.
                         * @author manuel.arguelles
                         * @date   Aug 30, 2017
                         * @param  vendor  The vendor id.
                         * @param  product The product id.
                         */
                        UsbConnection(const std::string& vendor, const std::string& product, int interface);

                        /**
                         * @brief  Destroy the object.
                         * @author manuel.arguelles
                         * @date   Aug 30, 2017
                         */
                        virtual ~UsbConnection();

                    public:

                        /**
                         * @brief  Connects to the usb device.
                         * @author manuel.arguelles
                         * @date   Aug 30, 2017
                         * @param  device The device name (not used).
                         * @return True if connected, false if not.
                         */
                        virtual bool openConnection(const std::string& device);

                        /**
                         * @brief  Send a command to the device.
                         * @author manuel.arguelles
                         * @date   Aug 30, 2017
                         * @param  command The array of bytes to send.
                         * @return True if succeed, false if not.
                         */
                        virtual bool sendCommand(const std::vector<unsigned char>& command);

                        /**
                         * @brief  Reads from the device.
                         * @author manuel.arguelles
                         * @date   Aug 30, 2017
                         * @return The array of bytes read.
                         */
                        virtual Smartmatic::System::SafeVector<unsigned char> readBuffer();

                        /**
                         * @brief  Gets the timeout to wait for data.
                         * @author manuel.arguelles
                         * @date   Aug 30, 2017
                         * @return The timeout.
                         */
                        virtual unsigned int getTimeOut();

                        /**
                         * @brief  Set the timeout to wait for data.
                         * @author juan.valera
                         * @date   01/11/2017
                         * @param  timeout The timeout in milliseconds.
                         */
                        virtual void setTimeOut(unsigned int timeout);

                        /**
                         * @brief  Closes the connection
                         * @author manuel.arguelles
                         * @date   Aug 30, 2017
                         * @return True if the connection was closed, false if not.
                         */
                        virtual bool closeConnection();

                        /**
                         * @brief  Resets the usb device.
                         * @author manuel.arguelles
                         * @date   Oct 17, 2017
                         * @return True if the connection was reset, false if not.
                         */
                        bool resetDevice();

                    private:

                        /**
                         * @brief  Claim the interface
                         * @author manuel.arguelles
                         * @date   Aug 30, 2017
                         * @return True if the interface was claimed, false if not.
                         */
                        bool claimInterface();

                        /**
                         * @brief  Transfer data to or from the device.
                         * @author manuel.arguelles
                         * @date   Aug 30, 2017
                         * @param  endpoint The endpoint to use.
                         * @param  data     The data to send or where to receive.
                         * @param  size     The mount of data to send or receive.
                         * @return The actual amount of transferred bytes.
                         */
                        size_t transfer(uint8_t endpoint, unsigned char* data, int size);

                    private:

                        int                               m_interface;
                        libusb_context*                   m_context;
                        libusb_device_handle*             m_device;
                        static Smartmatic::Log::ISMTTLog* m_logger;
                        uint16_t                          m_product;
                        uint16_t                          m_vendor;
                        unsigned int                      m_timeout;
                };

            } /* namespace Connections */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* USB_CONNECTION_HXX */
