/**
 * @file   AbledDevice.cxx
 * @brief  Implementation of AbledDevice class.
 * @author eduardo.fernandez
 * @date   Jan 11, 2018
 */

#include "AbledDevice.hxx"
#include <Commands/AbledEnableNotificationsCommand.hxx>
#include <Commands/AbledFWVersionCommand.hxx>
#include <Commands/AbledGetSerialNumberCommand.hxx>
#include <Commands/AbledSetSerialNumberCommand.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/AbleDButtonState.hxx>
#include <System/Utils/StringUtils.hxx>

using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::HW::Connections;
using namespace Smartmatic::SAES::HW::Commands;
using namespace Smartmatic::SAES::HW::Operation;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

ISMTTLog * AbledDevice::m_logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.HW.Devices.AbledDevice", GETTEXT_PACKAGE );

static AbleDButtonState::ButtonType getButtonType(const unsigned char value)
{
    AbleDButtonState::ButtonType type;

    switch (value)
    {
        case 0x30:
            type = AbleDButtonState::HELP;
            break;

        case 0x35:
            type = AbleDButtonState::SELECT;
            break;

        case 0x38:
            type = AbleDButtonState::UP;
            break;

        case 0x32:
            type = AbleDButtonState::DOWN;
            break;

        case 0x36:
            type = AbleDButtonState::RIGHT;
            break;

        case 0x34:
            type = AbleDButtonState::LEFT;
            break;

        case 0x39:
            type = AbleDButtonState::SPEED_UP;
            break;

        case 0x33:
            type = AbleDButtonState::SPEED_DOWN;
            break;

        case 0x37:
            type = AbleDButtonState::VOLUME_UP;
            break;

        case 0x31:
            type = AbleDButtonState::VOLUME_DOWN;
            break;

        default:
        case 0xFF:
            type = AbleDButtonState::UNKNOW;
            break;
    }

    return type;
}

static AbleDButtonState::ButtonState getButtonState(const unsigned char value)
{
    AbleDButtonState::ButtonState state;

    switch (value)
    {
        case 0x01:
            state = AbleDButtonState::PRESS;
            break;

        case 0x02:
            state = AbleDButtonState::RELEASE;
            break;

        case 0x03:
            state = AbleDButtonState::HOLD;
            break;

        default:
        case 0x00:
            state = AbleDButtonState::NONE;
            break;
    }

    return state;
}

static const size_t HEADER_SIZE                    = 24;
static const size_t NOISE_SIZE                     = 2;
static const size_t MB_COMMAND_ID_SIZE             = 1;
static const size_t COMMAND_ID_SIZE                = 1;
static const size_t NOTIFICATION_STATUS_COMMAND_ID = 0xAB;

AbledDevice::AbledDevice(IConnection* connection)
    : m_connection(connection)
    , m_buttonStatus(new AbleDButtonState())
{
}

AbledDevice::~AbledDevice()
{
    delete m_buttonStatus;
}

bool AbledDevice::open(const std::string & device)
{
    bool result = m_connection->openConnection("");

    if (result)
    {
        enableNotifications(true);
    }

    return result;
}

bool AbledDevice::close()
{
    return m_connection->closeConnection();
}

bool AbledDevice::enableNotifications(const bool enable)
{
    enable ? m_logger->Debug ( "Setting abled notifications to true" ) : m_logger->Debug ( "Setting abled notifications to false" );

    AbledEnableNotificationsCommand command(m_connection.get(), enable);
    bool commandSuccess = false;

    if (command.execute())
    {
        std::vector<unsigned char> response(
            command.getResponse().begin() + (HEADER_SIZE + NOISE_SIZE + MB_COMMAND_ID_SIZE + COMMAND_ID_SIZE),
            command.getResponse ().end());

        commandSuccess = response.size() == 1 && response[0] == 0x00;
    }

    std::string resultMessage = "";

    if (commandSuccess)
    {
        resultMessage = "Abled notifications set to ";
        resultMessage += enable ? "true" : "false";
        resultMessage += " with success";
    }
    else
    {
        resultMessage = "Error setting abled notifications to ";
        resultMessage += enable ? "true" : "false";
    }

    m_logger->Debug(resultMessage);

    return commandSuccess;
}

std::string AbledDevice::getFWVersion()
{
    std::string version;
    AbledFWVersionCommand command(m_connection.get());

    m_logger->Debug("Get version from abled");

    if (command.execute())
    {
        std::vector<unsigned char> response(
            command.getResponse().begin() + (HEADER_SIZE + NOISE_SIZE + COMMAND_ID_SIZE),
            command.getResponse ().end());

        version = StringUtils::arrayHexToString(response);
    }

    m_logger->Debug("Result : " + version);

    return version;
}

std::string AbledDevice::getSerialNumber()
{
    std::string serial;
    AbledGetSerialNumberCommand command(m_connection.get());

    m_logger->Debug("Get serial number from abled");

    if (command.execute())
    {
        std::vector<unsigned char> response(
            command.getResponse().begin() + (HEADER_SIZE + NOISE_SIZE + COMMAND_ID_SIZE),
            command.getResponse ().end());

        serial = StringUtils::arrayHexToAsciiString(response);
    }

    m_logger->Debug("Result : " + serial);

    return serial;
}

bool AbledDevice::setSerialNumber(const std::string& serial)
{
    std::vector <unsigned char> data;
    StringUtils::asciiStringToArrayHex(serial, data);
    AbledSetSerialNumberCommand command(m_connection.get(), data);

    m_logger->Debug("Set serial number to abled : " + serial);

    return command.execute ();
}

AbleDButtonState* AbledDevice::getA4517AbleDButtonState ()
{
    std::vector<unsigned char> notification = m_connection->readBuffer();

    if (notification.size() > 0)
    {
        std::vector<unsigned char> response(
            notification.begin() + (HEADER_SIZE + NOISE_SIZE),
            notification.end());

        if ((response[0] == NOTIFICATION_STATUS_COMMAND_ID) && ((response[2] == 0x00) || (response[2] == 0x01)))
        {
            AbleDButtonState::ButtonType  type   = getButtonType(response[1]);
            AbleDButtonState::ButtonState status = getButtonState(response[2]);

            m_logger->Debug ( "Button Event ( " + StringUtils::toHexString ( type, "", 2 ) + " - " + StringUtils::toHexString ( status, "", 2 ) + " )" );

            m_buttonStatus->updateStatus ( type, status );
        }
    }

    return m_buttonStatus;
}
