/**
 * @file   AbledDevice.hxx
 * @brief  Declaration of AbledDevice class.
 * @author eduardo.fernandez
 * @date   Jan 11, 2018
 */

#ifndef ABLED_DEVICE_HXX
#define ABLED_DEVICE_HXX

#include <string>
#include <memory>

#include <Devices/AbledInterface.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                class IConnection;
            }

            namespace Operation
            {
                class AbleDButtonState;
            }

            namespace Devices
            {
                /**
                 * @class  AbledDevice
                 * @brief  Abled Device controller.
                 * @author eduardo.fernandez
                 * @date   Jan 11, 2018
                 */
                class AbledDevice : public AbledInterface
                {
                    public:

                        /**
                         * @brief  Create an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Jan 11, 2018
                         * @param  connection The connection to use.
                         * @remark Takes ownership of the IConnection pointer
                         */
                        explicit AbledDevice(HW::Connections::IConnection* connection);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Jan 11, 2018
                         */
                        virtual ~AbledDevice();

                    public:

                        /**
                         * @brief  Opens a connection with the device.
                         * @author eduardo.fernandez
                         * @date   Jan 11, 2018
                         * @return True if the connection was established.
                         */
                        virtual bool open(const std::string & device);

                        /**
                         * @brief  Closes a connection with the device.
                         * @author eduardo.fernandez
                         * @date   Jan 11, 2018
                         * @return True if the connection was closed.
                         */
                        virtual bool close();

                        /**
                         * @brief  Enables or disables abled automatic notifications.
                         * @author eduardo.fernandez
                         * @date   Jan 12, 2018
                         * @return Whether the automatic notifications were successfully enabled or disabled.
                         */
                        virtual bool enableNotifications(const bool enable);

                        /**
                         * @brief  Gets the FW version from the PDB.
                         * @author eduardo.fernandez
                         * @date   Jan 11, 2018
                         * @return The FW version is successful, empty string if not.
                         */
                        virtual std::string getFWVersion();

                        /**
                         * @brief  Gets the serial number from the PDB's EEPROM.
                         * @author eduardo.fernandez
                         * @date   Jan 11, 2018
                         * @return The serial number is successful, empty string if not.
                         */
                        virtual std::string getSerialNumber();

                        /**
                         * @brief  Sets the serial number to the PDB's EEPROM.
                         * @author eduardo.fernandez
                         * @date   Jan 11, 2018
                         * @param  serial The serial value to set.
                         * @return True if the operation succeed, false if not.
                        */
                        virtual bool setSerialNumber(const std::string& serial);

                        /**
                         * @fn  virtual Smartmatic::SAES::HW::Operation::A4517AbleDButtonState * getA4517AbleDButtonState();
                         * @brief   Method getter able-d button status.
                         * @author  eduardo.fernandez
                         * @date    Jan 15, 2018
                         * @return AbleD button status
                         */
                        virtual SAES::HW::Operation::AbleDButtonState* getA4517AbleDButtonState ();

                    private:

                        std::auto_ptr<HW::Connections::IConnection> m_connection;
                        SAES::HW::Operation::AbleDButtonState* m_buttonStatus; ///< The Able-D button status
                        static Smartmatic::Log::ISMTTLog * m_logger;  ///< The logger
                };

            } /* namespace Devices */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* ABLED_DEVICE_HXX */
