/**
 * @file	AlarmBoxDevice.hxx
 * @brief	Declares the alarm device class.
 * @author	Yadickson Soto
 * @date	30/06/2015
 */

#ifndef _ALARM_BOX_DEVICE_H_
#define _ALARM_BOX_DEVICE_H_

#include <Connections/IConnection.hxx>
#include <Devices/AlarmBoxInterface.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Devices
            {
                /**
                 * @class	AlarmBoxDevice
                 * @brief	Manage all the elements for configuring, activating and deactivating an alarm.
                 * @author	Yadickson Soto
                 * @date	30/06/2015
                 */
                class AlarmBoxDevice: public AlarmBoxInterface
                {

                    public:

                        /**
                         * @fn	AlarmBoxDevice( Smartmatic::SAES::HW::Connections::IConnection * connection );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @param connection The serial connection to device
                         */
                        explicit AlarmBoxDevice ( Smartmatic::SAES::HW::Connections::IConnection * connection );

                        /**
                         * @fn	virtual ~AlarmBoxDevice();
                         * @brief	Class destroyer.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         */
                        virtual ~AlarmBoxDevice();

                        /**
                         * @fn	virtual bool open( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn	virtual bool close();
                         * @brief	Closes the connections.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @return	true if successful.
                         */
                        virtual bool close();

                        /**
                         * @fn	virtual bool sendPing();
                         * @brief	Sends a ping.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @return	true if it succeeds, false if it fails.
                         */
                        virtual bool sendPing();

                        /**
                         * @fn	virtual bool turnOnRedLed(unsigned char period);
                         * @brief	Turn on the red led.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @param	period	The period.
                         */
                        virtual bool turnOnRedLed ( unsigned char period );

                        /**
                         * @fn	virtual bool turnOffRedLed();
                         * @brief	Turn off the red led.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         */
                        virtual bool turnOffRedLed();

                        /**
                         * @fn	virtual bool turnOnGreenLed(unsigned char period);
                         * @brief	Turn on the green led.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @param	period	The period.
                         */
                        virtual bool turnOnGreenLed ( unsigned char period );

                        /**
                         * @fn	virtual bool turnOffGreenLed();
                         * @brief	Turn off the green led.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         */
                        virtual bool turnOffGreenLed();

                        /**
                         * @fn	virtual bool turnOnBuzzer(unsigned char period);
                         * @brief	Turn on the buzzer.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @param	period	The period.
                         */
                        virtual bool turnOnBuzzer ( unsigned char period );

                        /**
                         * @fn	virtual bool turnOffBuzzer();
                         * @brief	Turn off the buzzer.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         */
                        virtual bool turnOffBuzzer();

                        /**
                         * @fn	virtual bool isButtonPressed ();
                         * @brief	Check if button is pressed.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @return	true if button is pressed.
                         */
                        virtual bool isButtonPressed ();

                    private:

                        Smartmatic::SAES::HW::Connections::IConnection * connection; ///< The connection provider
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                };
            }
        }
    }
}

#endif // _ALARM_BOX_DEVICE_H_

