/**
 * @file	BarcodeInterface.hxx
 * @brief	Declares the barcode device interface.
 * @author	Yadickson Soto
 * @date	15/06/2015
 */


#ifndef _BARCODE_INTERFACE_H_
#define _BARCODE_INTERFACE_H_

#include <string>
#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Devices
            {
                /**
                 * @class	BarcodeInterface
                 * @brief	Manage the connections with the barcode.
                 * @author	Yadickson Soto
                 * @date	15/06/2015
                 */
                class BarcodeInterface
                {
                    public:

                        /**
                         * @fn	virtual ~BarcodeInterface();
                         * @brief	Class destroyer.
                         * @author	Yadickson Soto
                         * @date	15/06/2015
                         */
                        virtual ~BarcodeInterface() {};

                        /**
                         * @fn	virtual bool open( const std::string & device ) = 0;
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	15/06/2015
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device ) = 0;

                        /**
                         * @fn	virtual bool close() = 0;
                         * @brief	Closes the connections.
                         * @author	Yadickson Soto
                         * @date	15/06/2015
                         * @return	true if successful.
                         */
                        virtual bool close() = 0;

                        /**
                         * @fn	virtual std::string getDeviceModel() = 0;
                         * @brief	Getter device model.
                         * @author	Yadickson Soto
                         * @date	15/06/2015
                         * @return	device model
                         */
                        virtual std::string getDeviceModel() = 0;

                        /**
                         * @fn	virtual std::string getDeviceSerial() = 0;
                         * @brief	Getter device serial ESN.
                         * @author	Yadickson Soto
                         * @date	15/06/2015
                         * @return	device serial ESN
                         */
                        virtual std::string getDeviceSerial(bool factorySerial = false) = 0;

                        /**
                         * @fn	virtual bool setDeviceSerial ( const std::string & serial ) = 0;
                         * @brief	Setter new serial to device.
                         * @author	Yadickson Soto
                         * @date	15/06/2015
                         * @param	serial new serial
                         * @return true if successfully
                         */
                        virtual bool setDeviceSerial ( const std::string & serial ) = 0;

                        /**
                         * @fn	virtual bool sendReconfigureCommand ( const std::string & command,
                         * 		unsigned int timeout,
                         * 		unsigned int retries,
                         * 		bool mandatory ) = 0;
                         * @brief	Send command to device.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @param	command command to send
                         * @param	command The string command to send
                         * @param	timeout The waiting time to get response command
                         * @param	retries The tries to get good response
                         * @param	mandatory If command isn't mandatory, the execute is true
                         * @return true if successfully
                         */
                        virtual bool sendReconfigureCommand ( const std::string & command,
                                                              unsigned int timeout,
                                                              unsigned int retries,
                                                              bool mandatory ) = 0;

                        /**
                         * @fn	virtual Smartmatic::System::SafeBuffer readBuffer () = 0;
                         * @brief	Read device buffer.
                         * @author	Yadickson Soto
                         * @date	15/06/2015
                         * @return  data buffer from device
                         */
                        virtual Smartmatic::System::SafeBuffer readBuffer () = 0;
                };
            }
        }
    }
}

#endif // _BARCODE_INTERFACE_H_
