/**
 * @file	BarcodeReaderDevice.cxx
 * @brief	Body of the barcode reader device class.
 * @author	Yadickson Soto
 * @date	26/06/2015
 */

#include "BarcodeReaderDevice.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>
#include <Commands/BarcodeModelCommand.hxx>
#include <Commands/BarcodeGetSerialNumberCommand.hxx>
#include <Commands/BarcodeGetFactorySerialNumberCommand.hxx>
#include <Commands/BarcodeSetSerialNumberCommand.hxx>
#include <Commands/BarcodeSendConfigCommand.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::HW::Connections;
using namespace Smartmatic::SAES::HW::Commands;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;

ISMTTLog * BarcodeReaderDevice::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.HW.Devices.BarcodeReaderDevice", GETTEXT_PACKAGE );

BarcodeReaderDevice::BarcodeReaderDevice ( IConnection * connection )
    : connection ( connection ), processSendCommand ( false ), processSendConfigurationCommand ( false )
{

}

BarcodeReaderDevice::~BarcodeReaderDevice()
{
    if ( connection )
    {
        delete ( connection );
        connection = NULL;
    }
}

bool BarcodeReaderDevice::open ( const std::string & device )
{
    logger->Debug ( "Open barcode reader device" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.BarcodeDevice.BarcodeConnected" ) );
    return connection->openConnection ( device );
}

bool BarcodeReaderDevice::close()
{
    logger->Debug ( "Close barcode reader device" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.BarcodeDevice.BarcodeDisconnected" ) );
    return connection->closeConnection();
}

std::string BarcodeReaderDevice::getDeviceModel()
{
    logger->Debug ( "Get device model from barcode reader device" );

    this->processSendCommand = true;

    BarcodeModelCommand modelCommand ( connection );

    modelCommand.execute();

    this->processSendCommand = false;

    return StringUtils::arrayHexToAsciiString ( modelCommand.getResponse() );
}

std::string BarcodeReaderDevice::getDeviceSerial(const bool factorySerial)
{
    std::string serial;

    logger->Debug ( "Get serial number from barcode reader device" );

    if (factorySerial)
    {
        this->processSendCommand = true;

        BarcodeGetFactorySerialNumberCommand getSerialCommand ( connection );

        getSerialCommand.execute();

        this->processSendCommand = false;

        serial = StringUtils::arrayHexToAsciiString ( getSerialCommand.getResponse() );

    }
    else
    {
        this->processSendCommand = true;

        BarcodeGetSerialNumberCommand getSerialCommand ( connection );

        getSerialCommand.execute();

        this->processSendCommand = false;

        serial = StringUtils::arrayHexToAsciiString ( getSerialCommand.getResponse() );
    }

    return serial;
}

bool BarcodeReaderDevice::setDeviceSerial ( const std::string & serial )
{
    logger->Debug ( "Set serial number (" + serial + ") to barcode reader device" );

    this->processSendCommand = true;

    BarcodeSetSerialNumberCommand setSerialCommand ( connection, serial );

    bool result = setSerialCommand.execute();

    this->processSendCommand = false;

    return result;
}

bool BarcodeReaderDevice::sendReconfigureCommand ( const std::string & command,
        unsigned int timeout,
        unsigned int retries,
        bool mandatory )
{
    logger->Debug ( "Send command (" + command + ") to barcode reader device" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.BarcodeDevice.sendConfigurationCommands" ) );

    this->processSendConfigurationCommand = true;

    BarcodeSendConfigCommand sendConfigCommand ( connection, command, timeout, retries, mandatory );

    bool result = sendConfigCommand.execute();

    if (result == false)
    {
        if (sendConfigCommand.isResponseValid() == false)
        {
            for (size_t i = 0; i < retries - 1 && result == false && sendConfigCommand.isResponseValid() == false; ++i)
            {
                result = sendConfigCommand.execute();
            }
        }
    }

    this->processSendConfigurationCommand = false;

    return result;
}

SafeBuffer BarcodeReaderDevice::readBuffer ()
{
    connection->readBuffer();

    SafeBuffer localBuffer ( buffer );

    buffer.Clear();

    return localBuffer;
}

void BarcodeReaderDevice::peekEvent ( const std::vector<unsigned char> & data )
{
    if ( processSendCommand )
    {
        buffer.Clear();
        return;
    }

    if ( data.size() <= 1 )
    {
        buffer.Clear();
        return;
    }

    logger->Debug ( "Start signal barcode data" );

    if ( processSendConfigurationCommand )
    {
        logger->Debug ( "Sending configuration commands. Size response > 1." );

        if ( data[0] == 0x06 || data[0] == 0x15 )  // command response, success or fail
        {
            buffer.AddData ( ( unsigned char * ) ( &data[1] ), data.size() - 1 );
        }
        else
        {
            buffer.AddData ( ( unsigned char * ) ( &data[0] ), data.size() );
        }

        return;
    }

    buffer.AddData ( ( unsigned char * ) ( &data[0] ), data.size() );

    logger->Debug ( "End signal barcode data" );
}
