/*
 * EurnDevice.cxx
 *
 *  Created on: Oct 26, 2017
 *      Author: juanvalera
 */
#include "EurnDevice.hxx"
#include <Commands/EurnDoorCommand.hxx>
#include <Commands/EurnDoorWithLedCommand.hxx>
#include <Commands/EurnGreenBlinkLedCommand.hxx>
#include <Commands/EurnGreenLedCommand.hxx>
#include <Commands/EurnPingCommand.hxx>
#include <Commands/EurnRedBlinkLedCommand.hxx>
#include <Commands/EurnRedLedCommand.hxx>
#include <Configuration/EurnDeviceConfigurationManager.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>

static const unsigned char INFINITE_TIME    = 0x00;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::HW::Configuration;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::HW::Connections;
using namespace Smartmatic::SAES::HW::Commands;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;

ISMTTLog * EurnDevice::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.HW.Devices.EurnDevice", GETTEXT_PACKAGE );

EurnDevice::EurnDevice ( IConnection * connection )
    : connection ( connection )
    , processSendCommand ( false )
    , configuration ( EurnDeviceConfigurationManager::getInstance()->getConfiguration() )
{
}

EurnDevice::~EurnDevice()
{
    if ( connection )
    {
        delete ( connection );
        connection = NULL;
    }
}

bool EurnDevice::open ( const std::string & device )
{
    logger->Debug ( "Open Eurn device" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.EurnConnected" ) );

    bool connected = connection->openConnection (device);

    if (connected)
    {
        turnRedLedOnIndefinitely();
    }

    return connected;
}

bool EurnDevice::close()
{
    logger->Debug ( "Close Eurn device" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.EurnDisconnected" ) );

    return connection->closeConnection();
}

std::string EurnDevice::getVersion()
{
    EurnPingCommand command (connection);
    std::string versionPretty = "";
    bool result = command.execute();

    logger->Debug ( "Get eURN add on version" );

    if(result)
    {
        unsigned char version = command.getVersion();

        versionPretty = StringUtils::toHexString(version);
        logger->Debug ("EURN ADD ON VERSION: " + versionPretty);
    }

    return versionPretty;
}

bool EurnDevice::turnGreenLedOn()
{
    EurnGreenLedCommand command(connection, true, configuration.getTimeToTurnOnGreenLed());

    logger->Debug ( "Turn green led on" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.TurnGreenLedOn" ) );

    return command.execute();
}

bool EurnDevice::turnGreenLedOff()
{
    EurnGreenLedCommand command(connection, false, INFINITE_TIME);

    logger->Debug ( "Turn green led off" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.TurnGreenLedOff" ) );

    return command.execute();
}

bool EurnDevice::turnRedLedOn()
{
    EurnRedLedCommand command(connection, true, configuration.getTimeToTurnOnRedLed());

    logger->Debug ( "Turn red led on" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.TurnRedLedOn" ) );

    return command.execute();
}

bool EurnDevice::turnRedLedOff()
{
    EurnRedLedCommand command(connection, false, INFINITE_TIME);

    logger->Debug ( "Turn red led off" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.TurnRedLedOff" ) );

    return command.execute();
}

bool EurnDevice::turnRedLedOnIndefinitely()
{
    EurnRedLedCommand command(connection, true, INFINITE_TIME);

    logger->Debug ( "Turn red led on" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.TurnRedLedOn" ) );

    return command.execute();
}

bool EurnDevice::blinkGreenLedOnWithRedLedTurnOnAfter()
{
    const size_t timeToBlinkGreenLed = configuration.getTimeToBlinkOnGreenLed();
    const std::string auditMessage = StringUtils::compose(
        _("Smartmatic.SAES.HW.Devices.EurnDevice.BlinkGreenLedOnWithRedLedTurnOnAfter"),
        timeToBlinkGreenLed);

    EurnGreenBlinkLedCommand command(connection, true, timeToBlinkGreenLed, 0x10);

    logger->Debug ( "Blink green led on with red led on after for an infinite time" );
    logger->Audit ( auditMessage );

    return command.execute();
}

bool EurnDevice::blinkGreenLedOn()
{
    EurnGreenBlinkLedCommand command(connection, true, INFINITE_TIME);

    logger->Debug ( "Blink green led on" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.BlinkGreenLedOn" ) );

    return command.execute();
}

bool EurnDevice::blinkGreenLedOff()
{
    EurnGreenBlinkLedCommand command(connection, false, INFINITE_TIME);

    logger->Debug ( "Blink green led off" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.BlinkGreenLedOff" ) );

    return command.execute();
}

bool EurnDevice::blinkRedLedOn()
{
    EurnRedBlinkLedCommand command(connection, true, configuration.getTimeToBlinkOnRedLed(), 0x10);

    logger->Debug ( "Blink red led on" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.BlinkRedLedOn" ) );

    return command.execute();
}

bool EurnDevice::blinkRedLedOff()
{
    EurnRedBlinkLedCommand command (connection, false, INFINITE_TIME);

    logger->Debug ( "Blink red led off" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.BlinkRedLedOff" ) );

    return command.execute();
}

bool EurnDevice::openDoorForAFiniteTime()
{
    EurnDoorCommand command(connection, true, configuration.getTimeToOpenDoor());

    logger->Debug ( "Open door" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.OpenDoor" ) );

    return command.execute();
}

bool EurnDevice::openDoorForAFiniteTimeWithLedActions()
{
    unsigned char duringExecution = 0x00 ;
    unsigned char afterExecution = 0x10;
    if(configuration.getBlinkGreenLed())
    {
        duringExecution = 0x0A;
    }
    else
    {
        duringExecution = 0x02;
    }
    EurnDoorWithLedCommand command(connection,configuration.getTimeToOpenDoor(), duringExecution, afterExecution);

    logger->Debug ( "Open door with led actions" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.OpenDoor" ) );

    return command.execute();
}

bool EurnDevice::openDoor()
{
    EurnDoorCommand command(connection, true, 0);

    logger->Debug ( "Open door" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.OpenDoor" ) );

    return command.execute();
}

bool EurnDevice::closeDoor()
{
    EurnDoorCommand command(connection, false, INFINITE_TIME);

    logger->Debug ( "Close door" );
    logger->Audit ( N_ ( "Smartmatic.SAES.HW.Devices.EurnDevice.CloseDoor" ) );

    return command.execute();
}

