/**
 * @file	SimulatorBarcodeReaderDevice.hxx
 * @brief	Declares the simulator barcode reader device class.
 * @author	Yadickson Soto
 * @date	08/07/2015
 */

#ifndef _SIMULATOR_BARCODE_READER_DEVICE_H_
#define _SIMULATOR_BARCODE_READER_DEVICE_H_

#include <System/SafeBuffer.h>
#include <Devices/BarcodeInterface.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Devices
            {
                /**
                 * @class	SimulatorBarcodeReaderDevice
                 * @brief	Manage the connections with the barcode.
                 * @author	Yadickson Soto
                 * @date	08/07/2015
                 */
                class SimulatorBarcodeReaderDevice: public BarcodeInterface
                {
                    public:

                        /**
                         * @fn	SimulatorBarcodeReaderDevice( const std::string & modelFile, const std::string & serialFile,
                         *  const std::string & commandFile, const std::string & bufferFile );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @param modelFile The model file to get model device
                         * @param serialFile The serial file to get and set serial device
                         * @param commandFile The command response list state
                         * @param bufferFile The buffer file to read device
                         */
                        SimulatorBarcodeReaderDevice (
                            const std::string & modelFile,
                            const std::string & serialFile,
                            const std::string & commandFile,
                            const std::string & bufferFile );

                        /**
                         * @fn	virtual ~SimulatorBarcodeReaderDevice();
                         * @brief	Class destroyer.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         */
                        virtual ~SimulatorBarcodeReaderDevice();

                        /**
                         * @fn	virtual bool open( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn	virtual bool close();
                         * @brief	Closes the connections.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return	true if successful.
                         */
                        virtual bool close();

                        /**
                         * @fn	virtual std::string getDeviceModel();
                         * @brief	Getter device model.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return	device model
                         */
                        virtual std::string getDeviceModel();
                        /**
                         * @fn	virtual std::string getDeviceSerial();
                         * @brief	Getter device serial ESN.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return	device serial ESN
                         */
                        virtual std::string getDeviceSerial(bool factorySerial = false);
                        /**
                         * @fn	virtual bool setDeviceSerial ( const std::string & serial );
                         * @brief	Setter new serial to device.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @param	serial new serial
                         * @return true if successfully
                         */
                        virtual bool setDeviceSerial ( const std::string & serial );

                        /**
                         * @fn	virtual bool sendReconfigureCommand ( const std::string & command,
                         * 		unsigned int timeout,
                         * 		unsigned int retries,
                         * 		bool mandatory );
                         * @brief	Send command to device.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @param	command command to send
                         * @param	command The string command to send
                         * @param	timeout The waiting time to get response command
                         * @param	retries The tries to get good response
                         * @param	mandatory If command isn't mandatory, the execute is true
                         * @return true if successfully
                         */
                        virtual bool sendReconfigureCommand ( const std::string & command,
                                                              unsigned int timeout,
                                                              unsigned int retries,
                                                              bool mandatory );

                        /**
                         * @fn	virtual Smartmatic::System::SafeBuffer readBuffer ();
                         * @brief	Read device buffer.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return  data buffer from device
                         */
                        virtual Smartmatic::System::SafeBuffer readBuffer ();

                    private:

                        std::string modelFile; ///< The model file to get model device
                        std::string serialFile; ///< The serial file to get and set serial device
                        std::string commandFile; ///< The command response list state
                        std::string bufferFile; ///< The buffer file to read device

                        Smartmatic::System::SafeBuffer currentHash; ///< Current hash

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif // _SIMULATOR_BARCODE_READER_DEVICE_H_
