/**
 * @file AbleDButtonState.hxx
 * @brief Header manager for all able-d button status
 * @date 30/06/2015
 * @author Yadickson Soto
 */

#ifndef ABLED_BUTTON_STATE_HXX_
#define ABLED_BUTTON_STATE_HXX_

#include <map>
#include <string>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Operation
            {
                /**
                 * @class AbleDButtonState
                 * @brief manager for all able-d button status
                 * @author Yadickson Soto
                 * @date 30/06/2015
                 */
                class AbleDButtonState
                {
                    public:

                        /**
                         * @enum ButtonState
                         * @brief	enum able button state.
                         */
                        typedef enum
                        {
                            NONE = 1,
                            PRESS,
                            RELEASE,
                            HOLD
                        } ButtonState;

                        /**
                         * @enum ButtonType
                         * @brief	enum able P168 button type.
                         */
                        typedef enum
                        {
                            UNKNOW = 1,
                            HELP,
                            SELECT,
                            UP,
                            DOWN,
                            RIGHT,
                            LEFT,
                            PAUSE,
                            SPEED_UP,
                            SPEED_DOWN,
                            VOLUME_UP,
                            VOLUME_DOWN,
                            ACTIVATE
                        } ButtonType;

                        /**
                         * @fn AbleDButtonState ();
                         * @brief Class constructor
                         * @author Yadickson Soto
                         * @date 30/06/2015
                         */
                        AbleDButtonState ();

                        /**
                         * @fn virtual ~AbleDButtonState();
                         * @brief Class destroyer
                         * @author Yadickson Soto
                         * @date 30/06/2015
                         */
                        virtual ~AbleDButtonState();

                        /**
                         * @fn void updateStatus ( ButtonType type, ButtonState state );
                         * @brief Update new button status
                         * @author Yadickson Soto
                         * @date 30/06/2015
                         * @param type type of button
                         * @param state new state of the button
                         */
                        void updateStatus ( ButtonType type, ButtonState state );

                        /**
                         * @fn ButtonState getStatus ( ButtonType type );
                         * @brief Gets the button status by type
                         * @author Yadickson Soto
                         * @date 30/06/2015
                         * @param type type of button
                         * @return state of the button
                         */
                        ButtonState getStatus ( ButtonType type );

                        /**
                         * @fn bool isChanged ();
                         * @brief Gets if any button change
                         * @author Yadickson Soto
                         * @date 30/06/2015
                         * @return true if any button change
                         */
                        bool isChanged ();

                        /**
                         * @fn ButtonType stringToButtonType ( const std::string & strType );
                         * @brief Gets representation of the button type from string
                         * @author Yadickson Soto
                         * @date 30/06/2015
                         * @param strType input string
                         * @return representation of the button type from string
                         */
                        ButtonType stringToButtonType ( const std::string & strType );

                        /**
                         * @fn ButtonState stringToButtonState ( const std::string & strStatus );
                         * @brief Gets representation of the button state from string
                         * @author Yadickson Soto
                         * @date 30/06/2015
                         * @param strStatus input string
                         * @return representation of the button state from string
                         */
                        ButtonState stringToButtonState ( const std::string & strStatus );

                    private:

                        /**
                         * @brief  Resets all button states.
                         * @author manuel.arguelles
                         * @date   Apr 24, 2018
                         */
                        void resetAllButtonStates();

                    private:

                        bool changed; ///< Flag to check if any button change of state
                        std::map < ButtonType, ButtonState > buttonMap; ///< The button status map
                };

            } /* namespace Operation */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* ABLED_BUTTON_STATE_HXX_ */
