/**
 * @file	SmartmaticSoundConfManager.hxx
 * @brief	Declares the smartmatic sound conf manager class.
 */

#ifndef _SMTT_SOUND_CONFIGURATION_MANAGER_H_
#define _SMTT_SOUND_CONFIGURATION_MANAGER_H_

#include <string>
#include <System/SafeVector.hxx>
#include <System/Sounds/beep-descriptor.h>
#include <Configuration/SmartmaticConfiguration-schema.hxx>

namespace Smartmatic
{
    namespace Configuration
    {
        /**
         * @class	SmartmaticSoundConfManager
         * @brief	Manager for smartmatic sound configurations class.
         * @author	Yadickson Soto
         * @date	06/02/2014
         */
        class SmartmaticSoundConfManager
        {
            public:

                /**
                 * @fn	static SmartmaticSoundConfManager *getInstance(SmartmaticSoundConfManager * externManager = NULL);
                 * @brief	Gets the current Smartmatic Sound Configuration Manager.
                 * @author	Yadickson Soto
                 * @date	06/02/2014
                 * @param 	externManager external manager to unit test
                 * @return	instance of the class
                 */
                static SmartmaticSoundConfManager * getInstance ( SmartmaticSoundConfManager * externManager = NULL );

                /**
                 * @fn	virtual Smartmatic::System::SafeVector<Smartmatic::System::Sounds::BeepDescriptor> getSoundConfiguration ( const std::string & sound );
                 * @brief	Gets the configuration associated with a sound (BeepDescriptor)
                 * @author	Yadickson Soto
                 * @date	06/02/2014
                 * @param	sound	The sound name configuration.
                 * @return	The sound configuration result.
                 */
                virtual Smartmatic::System::SafeVector<Smartmatic::System::Sounds::BeepDescriptor> getSoundConfiguration ( const std::string & sound );

                /**
                 * @fn	virtual bool isUsingBeep();
                 * @brief	method to know if the application should use beep or revert to application sound.
                 * @author	Yadickson Soto
                 * @date	06/02/2014
                 * @return	true is use beep.
                 */
                virtual bool isUsingBeep();

                /**
                 * @fn	virtual std::string getAppSound();
                 * @brief	gets the default application sound
                 * @author	Yadickson Soto
                 * @date	06/02/2014
                 * @return	app sound.
                 */
                virtual std::string getAppSound();

                /**
                 * @fn	virtual std::string getSoundFile(const std::string & soundConfig);
                 * @brief	method getter sound file
                 * @author	Yadickson Soto
                 * @date	06/02/2014
                 * @param	soundConfig	The sound configuration name.
                 * @return	name of the sound configuration file.
                 */
                virtual std::string getSoundFile ( const std::string & soundConfig );

                /**
                 * @fn	virtual ~SmartmaticSoundConfManager();
                 * @brief	Class destructorer
                 * @author	Yadickson Soto
                 * @date	06/02/2014
                 */
                virtual ~SmartmaticSoundConfManager();

            protected:

                /**
                 * @fn	virtual Smartmatic::Configuration::SoundOutput & getSoundOutputConfiguration();
                 * @brief	Gets sound out configuration
                 * @author	Yadickson Soto
                 * @date	19/02/2014
                 * @return	sound out configuration.
                 */
                virtual Smartmatic::Configuration::SoundOutput & getSoundOutputConfiguration();

                /**
                 * @fn	SmartmaticSoundConfManager();
                 * @brief	Default constructor.
                 * @author	Yadickson Soto
                 * @date	06/02/2014
                 */
                SmartmaticSoundConfManager();

                static SmartmaticSoundConfManager * instance; ///< The singleton instance
        };
    }
}

#endif // _SMTT_SOUND_CONFIGURATION_MANAGER_H_
