/**
 * @file	LanguageController.hxx
 * @brief	Header file to the language controller class. This class list and set one language
 * @author	Yadickson Soto
 * @date	27/11/2013
 */

#ifndef _LANGUAGE_CONTROLLER_HXX_
#define _LANGUAGE_CONTROLLER_HXX_

#include <set>
#include <string>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace Environment
    {
        /**
         * @class	LanguageController
         * @brief	Base language controller class.
         * @author	Yadickson Soto
         * @date	27/11/2013
         */
        class LanguageController
        {

            protected:

                /**
                 * @fn	LanguageController();
                 * @brief	Default constructor.
                 * @author	Yadickson Soto
                 * @date	27/11/2013
                 */
                LanguageController();

                /**
                 * @fn	virtual void loadLangSupports();
                 * @brief	Load the languages supports inside locales.
                 * @author	Yadickson Soto
                 * @date	27/11/2013
                 */
                virtual void loadLangSupports();

                /**
                 * @fn	virtual std::string getLangSystem();
                 * @brief	Gets the current language inside Operatin System.
                 * @author	Yadickson Soto
                 * @date	27/11/2013
                 * @return	set language in operating system
                 */
                virtual std::string getLangSystem();

                /**
                 * @fn	virtual void setLangSystem ( const std::string & lang );
                 * @brief	Sets the language inside Operatin System.
                 * @author	Yadickson Soto
                 * @date	27/11/2013
                 * @param	lang the new language to set in operating system
                 */
                virtual void setLangSystem ( const std::string & lang );

            public:

                /**
                 * @fn	virtual ~LanguageController();
                 * @brief	Destroy
                 * @author	Yadickson Soto
                 * @date	27/11/2013
                 */
                virtual ~LanguageController();

                /**
                 * @fn	bool setLanguage ( const std::string & newLanguage, const std::string & newCountry );
                 * @brief	Sets the new language contry to environment LANG.
                 * @author	Yadickson Soto
                 * @date	27/11/2013
                 * @param	newLanguage new language
                 * @param	newCountry new contry language
                 * @return 	true if successfull
                 */
                bool setLanguage ( const std::string & newLanguage, const std::string & newCountry );

                /**
                 * @fn	std::string getCurrentLanguage ();
                 * @brief	Gets the current language.
                 * @author	Yadickson Soto
                 * @date	27/11/2013
                 * @return 	The current language
                 */
                std::string getCurrentLanguage ();
                /**
                 * @fn	std::string getCurrentCountry ();
                 * @brief	Gets the current country.
                 * @author	Yadickson Soto
                 * @date	27/11/2013
                 * @return 	The current country
                 */
                std::string getCurrentCountry ();
                /**
                 * @fn	std::string getFullLanguage();
                 * @brief	Gets the current full language.
                 * @author	Yadickson Soto yadickson.soto@gmail.com
                 * @date	07/05/2011
                 * @return	current language in format ll_CC.
                 */
                std::string getCurrentFullLanguage();
                /**
                 * @fn	bool isSetLang();
                 * @brief	Gets if set language, default is false preview set language.
                 * @author	Yadickson Soto yadickson.soto@gmail.com
                 * @date	27/11/2013
                 * @return	true if set environment LANG
                 */
                bool isSetLang();

                /**
                 * @fn	 static LanguageController * getInstance ( LanguageController * externController = NULL );
                 * @brief	Getter method instance of class.
                 * @author	Yadickson Soto
                 * @date	27/11/2013
                 * @param 	externController external Language Controller to unit test
                 * @return	instance of class.
                 */
                static LanguageController * getInstance ( LanguageController * externController = NULL );

            protected:

                std::set < std::string > languagesSupports; ///< locale support list

            private:

                std::string currentLanguage; ///< The act language
                std::string currentCountry; ///< The act country

                bool flagLanguage; ///< The flag set language

                static LanguageController * instance; ///< instance of class
                static Smartmatic::Log::ISMTTLog * logger; ///< The logger
        };
    }
}

#endif // _LANGUAGE_CONTROLLER_HXX_
