/**
 * @file   MixerControlImpl.hxx
 * @brief  Declaration of MixerControlImpl class.
 * @author manuel.arguelles
 * @date   Dec 13, 2017
 */

#ifndef MIXER_CONTROL_IMPL_HXX
#define MIXER_CONTROL_IMPL_HXX

#include <alsa/asoundlib.h>
#include <string>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Audio
        {

            /**
             * @class  MixerControlImpl
             * @brief  Controls the audio mixer.
             * @author manuel.arguelles
             * @date   Dec 13, 2017
             */
            class MixerControlImpl
            {
                public:

                    /**
                     * @brief  Creates an instance of the class.
                     * @author manuel.arguelles
                     * @date   Dec 13, 2017
                     * @param  cardName Audio card name.
                     * @param  defaultVolume Audio default volume.
                     */
                    explicit MixerControlImpl(const std::string& cardName, const int defaultVolume);

                    /**
                     * @brief  Destroys an instance of the class.
                     * @author manuel.arguelles
                     * @date   Dec 13, 2017
                     */
                    virtual ~MixerControlImpl();

                public:

                    /**
                     * @brief  Sets the playback volume.
                     * @author manuel.arguelles
                     * @date   Dec 13, 2017
                     * @param  volume The volume value (0 to 255).
                     */
                    void setPlaybackVolume(int volume);

                    /**
                     * @brief  Gets the playback volume.
                     * @author manuel.arguelles
                     * @date   Dec 13, 2017
                     * @return The current playback volume.
                     */
                    int getPlaybackVolume();

                    /**
                     * @brief  Gets the playback device of the card.
                     * @author manuel.arguelles
                     * @date   Jan 31, 2018
                     * @return The device name.
                     */
                    std::string getPlaybackDeviceName() const;

                private:

                    /**
                     * @brief  Sets the volume element.
                     * @author manuel.arguelles
                     * @date   Jan 23, 2018
                     * @param  name The name of the element.
                     * @return True if the element was found, false if not.
                     */
                    bool setVolumeElement(const std::string& name);

                    /**
                     * @brief  Opens the mixer device.
                     * @author manuel.arguelles
                     * @date   Feb 17, 2018
                     */
                    void open();

                    /**
                     * @brief  Closes the mixer device.
                     * @author manuel.arguelles
                     * @date   Feb 17, 2018
                     */
                    void close();

                private:

                    static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger

                    snd_mixer_t*      m_mixer;
                    snd_mixer_elem_t* m_element;
                    long              m_volumeMax;
                    long              m_volumeMin;
                    long              m_volume;
                    int               m_defaultVolume;
                    int               m_cardId;
            };

        } /* namespace Audio */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* MIXER_CONTROL_IMPL_HXX */
