/**
 * @file B64.cxx
 * @brief Body base 64 encoder/decoder class.
 * @date 30/08/2013
 * @author Yadickson Soto
 */

#include "B64.hxx"
#include <stdlib.h>
#include <stdint.h>

using namespace Smartmatic::System::Encoding;

static unsigned char encoding_table[] = {'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
                                         'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
                                         'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
                                         'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
                                         'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
                                         'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
                                         'w', 'x', 'y', 'z', '0', '1', '2', '3',
                                         '4', '5', '6', '7', '8', '9', '+', '/'
                                        }; ///< base 64 table definitions

static int mod_table[] = {0, 2, 1}; ///< mod table definitions

B64::B64()
{
    decodingTable = static_cast<unsigned char *> ( malloc ( 256 ) );

    for ( int i = 0; i < 64; i++ )
    {
        decodingTable[encoding_table[i]] = i;
    }
}

B64::~B64()
{
    free ( decodingTable );
    decodingTable = NULL;
}

unsigned char * B64::encode ( const unsigned char * data,
                              unsigned int inputLength,
                              unsigned int & outputLength )
{

    outputLength = 4 * ( ( inputLength + 2 ) / 3 );

    unsigned char * encoded_data = static_cast<unsigned char *> ( malloc ( outputLength ) );

    for ( unsigned int i = 0, j = 0; i < inputLength; )
    {

        uint32_t octet_a = i < inputLength ? data[i++] : 0;
        uint32_t octet_b = i < inputLength ? data[i++] : 0;
        uint32_t octet_c = i < inputLength ? data[i++] : 0;

        uint32_t triple = ( octet_a << 0x10 ) + ( octet_b << 0x08 ) + octet_c;

        encoded_data[j++] = encoding_table[ ( triple >> 3 * 6 ) & 0x3F];
        encoded_data[j++] = encoding_table[ ( triple >> 2 * 6 ) & 0x3F];
        encoded_data[j++] = encoding_table[ ( triple >> 1 * 6 ) & 0x3F];
        encoded_data[j++] = encoding_table[ ( triple >> 0 * 6 ) & 0x3F];
    }

    for ( int i = 0; i < mod_table[inputLength % 3]; i++ )
    {
        encoded_data[outputLength - 1 - i] = '=';
    }

    return encoded_data;
}

unsigned char * B64::decode ( const char * data,
                              unsigned int inputLength,
                              unsigned int & outputLength )
{
    outputLength = 0;

    if ( inputLength % 4 != 0 )
    {
        return NULL;
    }

    outputLength = inputLength / 4 * 3;

    if ( data[inputLength - 1] == '=' )
    {
        ( outputLength )--;
    }

    if ( data[inputLength - 2] == '=' )
    {
        ( outputLength )--;
    }

    unsigned char * decoded_data = static_cast<unsigned char *> ( malloc ( outputLength ) );

    for ( unsigned int i = 0, j = 0; i < inputLength; )
    {
        uint32_t sextet_a = data[i] == '=' ? 0 & i++ : decodingTable[ ( unsigned char ) data[i++]];
        uint32_t sextet_b = data[i] == '=' ? 0 & i++ : decodingTable[ ( unsigned char ) data[i++]];
        uint32_t sextet_c = data[i] == '=' ? 0 & i++ : decodingTable[ ( unsigned char ) data[i++]];
        uint32_t sextet_d = data[i] == '=' ? 0 & i++ : decodingTable[ ( unsigned char ) data[i++]];

        uint32_t triple = ( sextet_a << 3 * 6 )
                          + ( sextet_b << 2 * 6 )
                          + ( sextet_c << 1 * 6 )
                          + ( sextet_d << 0 * 6 );

        if ( j < outputLength )
        {
            decoded_data[j++] = ( triple >> 2 * 8 ) & 0xFF;
        }

        if ( j < outputLength )
        {
            decoded_data[j++] = ( triple >> 1 * 8 ) & 0xFF;
        }

        if ( j < outputLength )
        {
            decoded_data[j++] = ( triple >> 0 * 8 ) & 0xFF;
        }
    }

    return decoded_data;
}
