/**
 * @file   FatFileManager.cxx
 * @brief  Implementation of FatFileManager class.
 * @author manuel.arguelles
 * @date   Aug  3, 2017
 */

#include "FatFileManager.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/IO/Directory.h>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/Runtime/ProcessLauncher.h>
#include <System/Runtime/StorageDevice.hxx>
#include <algorithm>
#include <stdlib.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Runtime;

static const int         MAX_DEPTH = 30;
static const std::string FS_TYPE   = "vfat";

ISMTTLog* FatFileManager::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.System::IO::FatFileManager",
    GETTEXT_PACKAGE);

/**
 * @brief  Strip a path from another path.
 * @author manuel.arguelles
 * @date   Aug  7, 2017
 * @param  path The complete path.
 * @param  pathToStrip The part of the path to be removed.
 */
static void stripPath(std::string& path, const std::string& pathToStrip)
{
    std::string::size_type position;

    if (!pathToStrip.empty())
    {
        position = path.find(pathToStrip);

        if (position != std::string::npos)
            path.erase(position, pathToStrip.size());
    }
}

/**
 * @brief  Checks if the entry starts with a dot.
 * @author manuel.arguelles
 * @date   Aug  9, 2017
 * @param  directory The directory to check.
 */
static bool removeDotStarting(const std::string& directory)
{
    bool toRemove = false;

    if (directory[0] == '.')
    {
        toRemove = true;
    }

    return toRemove;
}

/**
 * @brief  Remove entries starting with dot.
 * @author manuel.arguelles
 * @date   Aug  9, 2017
 * @param  directories The vector with the directories to remove.
 */
static void removeHiddenDirectories(std::vector<std::string>& directories)
{
    directories.erase(std::remove_if(directories.begin(), directories.end(),
                                     removeDotStarting),
                      directories.end());
}

FatFileManager::FatFileManager()
    : m_directoryInterface(*Directory::getInstance())
    , m_fileInterface(*File::getInstance())
{
}

FatFileManager::FatFileManager(IDirectory& directoryInterface, IFile& fileInterface)
    : m_directoryInterface(directoryInterface)
    , m_fileInterface(fileInterface)
{
}

FatFileManager::~FatFileManager()
{
    freeStorageDevices();
}

void FatFileManager::freeStorageDevices()
{
    std::vector<IStorageDevice*>::iterator it;

    for (it = m_storageDevices.begin(); it != m_storageDevices.end(); ++it)
    {
        delete *it;
    }

    m_storageDevices.clear();
}

void FatFileManager::addStorageDevice(IStorageDevice* device)
{
    m_storageDevices.push_back(device);
}

void FatFileManager::removeStorageDevice(System::Runtime::IStorageDevice* device)
{
    std::vector<IStorageDevice*>::iterator it;

    for (it = m_storageDevices.begin(); it != m_storageDevices.end(); ++it)
    {
        if (*it == device)
        {
            m_storageDevices.erase(it);
            break;
        }
    }
}

void FatFileManager::addFatDevices()
{
    std::vector<StorageDevice> fatDevices = StorageDevice::getUniqueDevicesByFileSystem(FS_TYPE);
    std::vector<StorageDevice>::const_iterator it;

    freeStorageDevices();

    for (it = fatDevices.begin(); it != fatDevices.end(); ++it)
    {
        IStorageDevice* duplicated = new StorageDevice(*it);
        m_storageDevices.push_back(duplicated);
    }
}

std::string::size_type FatFileManager::getNumberOfDevices() const
{
    return m_storageDevices.size();
}

bool FatFileManager::copyFiles(const std::string& sourcePath, const std::string& stripFinalPath)
{
    bool result = mountDevices();

    m_pathToStrip = stripFinalPath;

    if (result && !copyWalker(sourcePath, MAX_DEPTH))
    {
        result = false;
    }

    if (!unmountDevices())
    {
        result = false;
    }

    return result;
}

bool FatFileManager::verifyFiles(const std::string& sourcePath, const std::string& stripFinalPath)
{
    bool result = mountDevices();

    m_pathToStrip = stripFinalPath;

    if (result && !verifyWalker(sourcePath, MAX_DEPTH))
    {
        result = false;
    }

    if (!unmountDevices())
    {
        result = false;
    }

    return result;
}

bool FatFileManager::deleteDirectory(const std::string& directory)
{
    bool result = mountDevices();

    if (result)
    {
        std::vector<IStorageDevice*>::const_iterator       it  = m_storageDevices.begin();
        const std::vector<IStorageDevice*>::const_iterator end = m_storageDevices.end();

        for (; it != end && result; ++it)
        {
            const std::string directoryToDelete = Path::Combine((*it)->getMountPoint(), directory);

            if (m_directoryInterface.DirectoryExists(directoryToDelete))
            {
                result = m_directoryInterface.Delete(directoryToDelete);
            }
        }

        result = result && unmountDevices();
    }

    return result;
}

bool FatFileManager::mountDevices()
{
    bool result = true;
    std::vector<IStorageDevice*>::iterator it;

    m_logger->Debug ( "Mounting devices" );

    for (it = m_storageDevices.begin(); it != m_storageDevices.end(); ++it)
    {
        if (!(*it)->isMounted() && !(*it)->mount())
        {
            result = false;
        }
    }

    m_logger->Debug ( "Devices mounted" );

    return result;
}

bool FatFileManager::unmountDevices()
{
    ProcessLauncher::getInstance()->RunProcessThroughSystem ( "sync" );

    bool result = true;
    std::vector<IStorageDevice*>::iterator it;

    m_logger->Debug ( "Unmounting devices" );

    for (it = m_storageDevices.begin(); it != m_storageDevices.end(); ++it)
    {
        if ((*it)->isMounted())
        {
            std::string mountPoint = (*it)->getMountPoint();

            if ((*it)->unmount())
            {
                m_directoryInterface.Delete(mountPoint);
            }
            else
            {
                result = false;
            }
        }
    }

    m_logger->Debug ( "Devices unmounted" );

    return result;
}

bool FatFileManager::copyWalker(const std::string& path, int maxDepth)
{
    std::vector<std::string> files;
    std::vector<std::string> directories;
    std::vector<std::string> filesAndDirectories;
    std::vector<std::string>::const_iterator it;

    if (maxDepth <= 0 || !m_directoryInterface.DirectoryExists(path))
        return false;

    m_directoryInterface.GetFiles(path, files);
    random_shuffle(files.begin(), files.end());
    m_directoryInterface.GetDirectories(path, directories);
    removeHiddenDirectories(directories);

    filesAndDirectories.reserve(files.size() + directories.size());
    filesAndDirectories.insert(filesAndDirectories.end(), files.begin(), files.end());
    filesAndDirectories.insert(filesAndDirectories.end(), directories.begin(), directories.end());
    filesAndDirectories.push_back(".");

    for (it = filesAndDirectories.begin(); it != filesAndDirectories.end(); ++it)
    {
        if (!copyToDevice(Path::Combine(path, *it)))
            return false;
    }

    for (it = directories.begin(); it != directories.end(); ++it)
    {
        if (!copyWalker(Path::Combine(path, *it), --maxDepth))
            return false;
    }

    return true;
}

bool FatFileManager::verifyWalker(const std::string& path, int maxDepth)
{
    std::vector<std::string> files;
    std::vector<std::string> directories;
    std::vector<std::string> filesAndDirectories;
    std::vector<std::string>::const_iterator it;

    if (maxDepth <= 0 || !m_directoryInterface.DirectoryExists(path))
        return false;

    m_directoryInterface.GetFiles(path, files);
    m_directoryInterface.GetDirectories(path, directories);
    removeHiddenDirectories(directories);

    filesAndDirectories.reserve(files.size() + directories.size());
    filesAndDirectories.insert(filesAndDirectories.end(), files.begin(), files.end());
    filesAndDirectories.insert(filesAndDirectories.end(), directories.begin(), directories.end());
    filesAndDirectories.push_back(".");

    for (it = filesAndDirectories.begin(); it != filesAndDirectories.end(); ++it)
    {
        if (!verifyWithDevice(Path::Combine(path, *it)))
            return false;
    }

    for (it = directories.begin(); it != directories.end(); ++it)
    {
        if (!verifyWalker(Path::Combine(path, *it), --maxDepth))
            return false;
    }

    return true;
}

bool FatFileManager::copyToDevice(const std::string& sourceFile)
{
    bool result = true;
    std::string destinationFile;
    std::vector<IStorageDevice*>::const_iterator it;

    destinationFile = sourceFile;
    stripPath(destinationFile, m_pathToStrip);

    if (m_storageDevices.empty())
        return false;

    for (it = m_storageDevices.begin(); it != m_storageDevices.end(); ++it)
    {
        std::string destinationPath = (*it)->getMountPoint();

        if (destinationPath.empty() || !result)
            return false;

        if (m_directoryInterface.DirectoryExists(sourceFile))
        {
            m_logger->Debug(_("Creating directory: ") + Path::Combine(destinationPath, destinationFile));
            result = m_directoryInterface.Mkdir(Path::Combine(destinationPath, destinationFile));
        }
        else
        {
            m_logger->Debug(_("Copying file: ") + sourceFile);
            result = m_fileInterface.copyFile(sourceFile, Path::Combine(destinationPath, destinationFile));
        }
    }

    return result;
}

bool FatFileManager::verifyWithDevice(const std::string& sourceFile)
{
    bool result = true;
    std::string destinationFile;
    std::vector<IStorageDevice*>::const_iterator it;

    destinationFile = sourceFile;
    stripPath(destinationFile, m_pathToStrip);

    if (m_storageDevices.empty())
        return false;

    for (it = m_storageDevices.begin(); it != m_storageDevices.end(); ++it)
    {
        std::string destinationPath = (*it)->getMountPoint();

        if (destinationPath.empty() || !result)
            return false;

        if (m_directoryInterface.DirectoryExists(sourceFile))
        {
            std::vector<std::string> filesInSource;
            std::vector<std::string> filesInDevice;
            std::vector<std::string> directoriesInSource;
            std::vector<std::string> directoriesInDevice;

            m_directoryInterface.GetFiles(sourceFile, filesInSource);
            m_directoryInterface.GetDirectories(sourceFile, directoriesInSource);
            m_directoryInterface.GetFiles(Path::Combine(destinationPath, destinationFile), filesInDevice);
            m_directoryInterface.GetDirectories(Path::Combine(destinationPath, destinationFile), directoriesInDevice);

            m_logger->Debug(_("Comparing directory: ") + sourceFile);
            result = m_directoryInterface.DirectoryExists(Path::Combine(destinationPath, destinationFile)) &&
                filesInSource.size() == filesInDevice.size() && directoriesInSource.size() == directoriesInDevice.size();
        }
        else
        {
            m_logger->Debug(_("Comparing file: ") + sourceFile);
            result = m_fileInterface.compareFiles(sourceFile, Path::Combine(destinationPath, destinationFile));
        }
    }

    return result;
}
