/**
 * @file File.cc
 * @brief Body class File
 */

#include "File.h"
#include <fstream>
#include <System/Security/Cryptography/MsgDigestUtil.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Runtime/ProcessLauncher.h>
#include "Directory.h"
#include "Path.h"
#include <sstream>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::Runtime;

ISMTTLog * File::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.IO.File", GETTEXT_PACKAGE );
File * File::instance = NULL;

File::File()
{

}

File::~File()
{
    instance = NULL;
}

bool File::canOpenFile ( const std::string & file )
{
    try
    {
        if ( Directory::getInstance()->DirectoryExists ( file ) || !Directory::getInstance()->FileExists ( file ) )
        {
            return false;
        }

        std::ifstream ifile ( file.c_str() );

        if ( ifile )
        {
            std::ifstream ifs;
            ifs.exceptions ( std::ifstream::badbit | std::ifstream::failbit );
            ifs.open ( file.c_str() );
            ifs.close();
            return true;
        }
    }
    catch ( std::ifstream::failure & ex )
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__ << " " << ex.what();
        logger->Debug ( ss.str() );
    }

    return false;
}

bool File::readFile ( const std::string & file, std::string & content )
{
    bool validate = false;

    SafeBuffer buffer;
    content.clear();

    if ( openFileToBuffer ( file, buffer ) )
    {
        content.assign ( ( const char * ) buffer.GetData(), buffer.GetSize() );
        validate = true;
    }

    return validate;
}

bool File::DeleteFile ( const std::string & fullPath )
{
    if ( remove ( fullPath.c_str() ) != 0 )
    {
        return false;
    }
    else
    {
        return true;
    }
}

bool File::copyFile ( const std::string & inputFile, const std::string & outputFile )
{
    std::string path = Path::GetDirectory ( outputFile );
    Directory::getInstance()->Mkdir ( path );

    SafeBuffer buffer;
    return openFileToBuffer ( inputFile, buffer ) && saveBufferToFile ( buffer, outputFile );
}

bool File::saveBufferToFile ( const SafeBuffer & buffer, const std::string & file )
{
    bool success = false;

    if ( buffer.GetSize() > 0 )
    {
        std::string path = Path::GetDirectory ( file );
        Directory::getInstance()->Mkdir ( path );

        std::ofstream ofile ( file.c_str(), std::ios::out | std::ios::binary | std::ios::trunc );

        if ( ofile.is_open() )
        {
            ofile.write ( ( char * ) buffer.GetData(), buffer.GetSize() );
            ofile.close();
            success = true;
        }
    }
    else
    {
        success = true;
    }

    return success;
}

bool File::addBufferToFile ( const SafeBuffer & buffer, const std::string & file )
{
    bool success = false;

    if ( buffer.GetSize() > 0 )
    {
        std::string path = Path::GetDirectory ( file );
        Directory::getInstance()->Mkdir ( path );

        std::ofstream ofile ( file.c_str(), std::ios::out | std::ios::binary | std::ios::app | std::ios::ate );

        if ( ofile.is_open() )
        {
            ofile.write ( ( char * ) buffer.GetData(), buffer.GetSize() );
            ofile.close();
            success = true;
        }
    }
    else
    {
        success = true;
    }

    return success;
}

bool File::openFileToBuffer ( const std::string & file, Smartmatic::System::SafeBuffer & dataOut )
{
    bool validate = false;

    if ( canOpenFile ( file ) )
    {
        std::ifstream ifFile ( file.c_str(), std::ios::in | std::ios::binary | std::ios::ate );

        if ( ifFile.is_open() )
        {
            int size;
            size = ifFile.tellg();
            unsigned char * data = new unsigned char [size];
            ifFile.seekg ( 0, std::ios::beg );
            ifFile.read ( ( char * ) data, size );
            ifFile.close();
            dataOut.AddData ( data, size );
            delete[] data;
            validate = true;
        }
    }

    return validate;
}

bool File::compareFiles ( const std::string & fileOne, const std::string & fileTwo )
{
    bool validate = false;
    logger->Debug ( "Compare files " + fileOne + " - " + fileTwo );

    SafeBuffer digOne;
    SafeBuffer digTwo;

    if ( getHashFile ( fileOne, digOne ) && getHashFile ( fileTwo, digTwo ) )
    {
        validate = ( digOne.GetData() != NULL && digOne.GetBase64Data() == digTwo.GetBase64Data() );
    }

    logger->Debug ( "Compare files " + std::string ( validate ? "Success" : "Fail" ) );
    return validate;
}

bool File::getHashFile ( const std::string & file, Smartmatic::System::SafeBuffer & dataOut )
{
    bool validate = false;

    SafeBuffer buffer;
    dataOut.Clear();

    if ( openFileToBuffer ( file, buffer ) )
    {
        try
        {
            MsgDigestUtil digest ( buffer );
            digest.getDigest ( dataOut, MsgDigestUtil::getDefaultFileDigest());
            validate = true;
        }
        catch ( ... )
        {
        }
    }

    return validate;
}

bool File::getHashFileByChunks(const std::string &file, int chunkSize, SafeBuffer& dataOut)
{
    bool validate = false;

    SafeBuffer buffer;
    dataOut.Clear();

    if ( canOpenFile ( file ) )
    {
        std::ifstream ifFile ( file.c_str(), std::ios::in | std::ios::binary | std::ios::ate );

        if ( ifFile.is_open() )
        {
            int size;
            size = ifFile.tellg();
            int dataRead = 0;

            MsgDigestUtil digest ( MsgDigestUtil::getDefaultFileDigest() );
            SafeBuffer partialBuffer;

            unsigned char * data = new unsigned char [chunkSize];

            while (dataRead < size)
            {
                if (dataRead + chunkSize > size)
                {
                    chunkSize = size - dataRead;
                }

                ifFile.seekg ( dataRead, std::ios::beg );
                ifFile.read ( ( char * ) data, chunkSize );

                partialBuffer.Clear();
                partialBuffer.AddData ( data, chunkSize );

                try
                {
                    digest.updateDigest ( partialBuffer );
                }
                catch ( ... )
                {
                }

                dataRead += chunkSize;
            }

            digest.obtainResult(dataOut);

            ifFile.close();
            delete[] data;
            validate = true;
        }
    }

    return validate;
}

bool File::renameFile ( const std::string & oldName, const std::string & newName )
{
    return rename ( oldName.c_str() , newName.c_str() ) == 0;
}

bool File::copyDirectory ( const std::string & origin, const std::string & destination )
{
    bool success = true;

    if ( ! Directory::getInstance()->DirectoryExists ( origin ) ||
         ! Directory::getInstance()->Mkdir ( destination) )
    {
        logger->Debug ( "Failed to copy directory " + origin + " to " + destination );
        return false;
    }

    std::vector<std::string> files;

    Directory::getInstance()->GetFiles( origin, files );

    for ( std::vector<std::string>::const_iterator it = files.begin(); it != files.end(); ++it )
    {
        if ( *it != "." && *it != ".." )
        {
            if ( ! copyFile ( Path::Combine(origin, *it), Path::Combine(destination, *it) ) )
            {
                logger->Debug ( "Coping file " + Path::Combine(origin, *it) + " to " + Path::Combine(destination, *it) + " failed!" );
                success = false;
                break;
            }
        }
    }

    std::vector<std::string> directories;

    Directory::getInstance()->GetDirectories ( origin, directories );

    for ( std::vector<std::string>::const_iterator it = directories.begin(); it != directories.end(); ++it )
    {
        if ( *it != "." && *it != ".." )
        {
            if ( ! copyDirectory ( Path::Combine(origin, *it), Path::Combine(destination, *it) ) )
            {
                success = false;
                break;
            }
        }
    }

    return success;
}

File * File::getInstance ( File * externFile )
{
    if ( instance == NULL )
    {
        instance = ( externFile == NULL ? new File() : externFile );
    }

    return instance;
}
