/**
 * @file	INotifyFile.cxx
 * @brief	Body file to the watch file class
 * @date 	27/04/2015
 * @author 	Yadickson Soto
 */

#include "INotifyFile.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/inotify.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define EVENT_SIZE			( sizeof (struct inotify_event) )
#define EVENT_BUF_LEN		( 1024 * ( EVENT_SIZE + 16 ) )

using namespace Smartmatic::System::IO;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;

ISMTTLog * INotifyFile::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.IO.INotifyFile", GETTEXT_PACKAGE );

INotifyFile::INotifyFile()
{
    handle = 0;
}

INotifyFile::~INotifyFile()
{
    dispose();
}

void INotifyFile::initialize()
{
    if ( handle == 0 )
    {
#ifdef IN_NONBLOCK
        handle = inotify_init1 ( IN_NONBLOCK );
#else
        handle = inotify_init();
#endif
    }
}

void INotifyFile::dispose()
{
    std::list < int >::iterator it;

    for ( it = watchDogList.begin();
            it != watchDogList.end();
            ++it )
    {
        inotify_rm_watch ( handle, ( *it ) );
    }

    handle = 0;
}

void INotifyFile::addPath ( const std::string & path )
{
    if ( handle != 0 )
    {
        int wd = inotify_add_watch ( handle, path.c_str(), IN_CREATE | IN_MODIFY | IN_DELETE );
        watchDogList.push_back ( wd );
    }
}

SafeList < INotifyFile::StatusNotify > INotifyFile::getStatus ()
{
    SafeList < StatusNotify > events;
    char buffer[EVENT_BUF_LEN];

    fd_set fds;
    struct timeval tv;
    int ret;

    FD_ZERO ( &fds );
    FD_SET ( handle, &fds );
    tv.tv_sec = 1;
    tv.tv_usec = 0;

    ret = select ( handle + 1, &fds, NULL, NULL, &tv );

    if ( ret > 0 && FD_ISSET ( handle, &fds ) )
    {
        int length = read ( handle, buffer, EVENT_BUF_LEN );
        int i = 0;

        while ( i < length )
        {
            struct inotify_event * event = reinterpret_cast<struct inotify_event *>(&buffer[ i ]);

            if ( event->len )
            {
                int status = NONE;

                if ( event->mask & IN_CREATE )
                {
                    logger->Debug ( "Create file " + std::string ( event->name ) );
                    status |= CREATE;
                }

                if ( event->mask & IN_MODIFY )
                {
                    logger->Debug ( "Modify file " + std::string ( event->name ) );
                    status |= MODIFY;
                }

                if ( event->mask & IN_DELETE )
                {
                    logger->Debug ( "Delete file " + std::string ( event->name ) );
                    status |= DELETE;
                }

                if ( status != NONE )
                {
                    events.push_back ( StatusNotify ( event->name, status ) );
                }
            }

            i += EVENT_SIZE + event->len;
        }
    }

    return events;
}
