/**
 * @file OptParser.cxx
 * @brief Body class OptParser
 * @author Yadickson Soto
 * @date 23/08/2011
 */

#include "OptParser.hxx"
#include "Configuration/configuration-persistence-manager.h"
#include <getopt.h>
#include <Log/SMTTLogManager.h>
#include <System/IO/Directory.h>
#include <System/IO/Path.h>
#include <System/ZenityWrapper.hxx>
#include <System/XSLFO/FOPTransformer.hxx>
#include <System/Utils/StringUtils.hxx>
#include <System/Runtime/ExitProcess.hxx>
#include <iostream>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< Macro gettext
#else
#define _(String) (String) ///< Macro not gettext
#endif

#define CONFIG_FILENAME "configuration-miscellaneous.xml" ///< config file base
#define LOG_CONFIG_FILENAME "log4-configuration.xml" ///< logger file
#define XSLFO_CONFIG_FILENAME "fop.xconf" ///< fop config file

using namespace Smartmatic::System;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::XSLFO;
using namespace Smartmatic::System::Runtime;

static struct option long_options[] =
{
    {"config", 1, 0, OptParser::CONFIG },
    {"sharedir", 1, 0, OptParser::SHARE_DIR },
    {"localedir", 1, 0, OptParser::LOCALE_DIR },
    {"logger", 1, 0, OptParser::LOGGER },
    {"xslfo", 1, 0, OptParser::XSLFO },
    {"mode", 1, 0, OptParser::MODE },
    {"login", 1, 0, OptParser::LOGIN },
    {"password", 1, 0, OptParser::PASSWORD },
    {"disable-bid-sid-validation", 0, 0, OptParser::DISABLE_BID_SID_VALIDATE },
    {"dbus-info", 0, 0, OptParser::DBUS_INFO },
    {"help", 0, 0, OptParser::HELP },
    {"version", 0, 0, OptParser::VERS },
    {"disable-save-vote", 0, 0, OptParser::DISABLE_SAVE_VOTE },
    {"machine-model", 1, 0, OptParser::MACHINE_MODEL},
    {0, 0, 0, 0}
}; ///< Option values

ISMTTLog * OptParser::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.OptParser", GETTEXT_PACKAGE );
OptParser * OptParser::instance = NULL;

OptParser::OptParser ()
{
    this->argc = 0;
    this->argv = NULL;

    std::stringstream ss;

    for ( int i = 0 ; i < ( int ) ( sizeof ( long_options ) / sizeof ( option ) - 1 ); i++ )
    {
        ss << ( char ) long_options [i].val << ( long_options[i].has_arg ? ":" : "" );
    }

    shortopts = ss.str();
}

OptParser::~OptParser ()
{
    instance = NULL;
}

void OptParser::initialize ( int argc, char ** argv )
{
    this->argc = argc;
    this->argv = argv;

    setMap ( CONFIG );
    setMap ( SHARE_DIR );
    setMap ( LOCALE_DIR );
    setMap ( LOGGER );
    setMap ( XSLFO );
    setMap ( MODE );
    setMap ( LOGIN );
    setMap ( PASSWORD );
    setMap ( DISABLE_BID_SID_VALIDATE );
    setMap ( DBUS_INFO );
    setMap ( HELP );
    setMap ( VERS );
    setMap ( DISABLE_SAVE_VOTE );
    setMap ( MACHINE_MODEL );
}

void OptParser::setMap ( OptParser::Options opt )
{
    if ( argc <= 1 )
    {
        return;
    }

    optarg = NULL;
    optind = opterr = optopt = 0;

    while ( true )
    {
        int c;
        int option_index = 0;
        c = getopt_long_only ( argc, argv, shortopts.c_str(), long_options, &option_index );

        if ( c == -1 )
        {
            break;
        }

        if ( c == ( int ) opt )
        {
            if ( optarg )
            {
                mapOptions [ opt ] = optarg;
            }
            else
            {
                mapOptions [ opt ] = "";
            }

            return;
        }
    }
}

bool OptParser::isSet ( OptParser::Options opt )
{
    return mapOptions.find ( opt ) != mapOptions.end();
}

bool OptParser::getArg ( OptParser::Options opt, std::string & str )
{
    str.clear();

    if ( ! isSet ( opt ) || mapOptions [ opt ].empty() )
    {
        return false;
    }

    str = mapOptions [ opt ];

    return true;
}

bool OptParser::getHelpOption ( std::string & help )
{
    bool validate = isSet ( OptParser::HELP );

    help.clear();

    if ( validate )
    {
        std::stringstream ss;
        ss << "  -" << ( char ) OptParser::CONFIG << ", --" << long_options[0].name << "    <config-file>            " << "set path config file" << std::endl;
        ss << "  -" << ( char ) OptParser::SHARE_DIR << ", --" << long_options[1].name << "   <sharedir>               " << "set sharedir path (" << DATADIR << " defaults)" << std::endl;
        ss << "  -" << ( char ) OptParser::LOCALE_DIR << ", --" << long_options[2].name << "   <localedir>              " << "set localedir path (" << GNOMELOCALEDIR << " )" << std::endl;
        ss << "  -" << ( char ) OptParser::LOGGER << ", --" << long_options[3].name << "    <logger-file>            " << "set path logger file" << std::endl;
        ss << "  -" << ( char ) OptParser::XSLFO << ", --" << long_options[4].name << "     <xslfo-config-file>      " << "set path xslfo file" << std::endl;
        ss << "  -" << ( char ) OptParser::MODE << ", --" << long_options[5].name << "      <mp|mv|sg|dmp|dmv|duv>   " << "set mode to load run machine" << std::endl;
        ss << "  -" << ( char ) OptParser::LOGIN << ", --" << long_options[6].name << "     <login>                  " << "set authentication login" << std::endl;
        ss << "  -" << ( char ) OptParser::PASSWORD << ", --" << long_options[7].name << "  <password>               " << "set authentication password" << std::endl;
        ss << "  -" << ( char ) OptParser::DISABLE_BID_SID_VALIDATE << ", --" << long_options[8].name << "         " << "disable id validation" << std::endl;
        ss << "  -" << ( char ) OptParser::DBUS_INFO << ", --" << long_options[9].name << "                          " << "view dbus info devices" << std::endl;
        ss << "  -" << ( char ) OptParser::HELP << ", --" << long_options[10].name << "                               " << "this help application" << std::endl;
        ss << "  -" << ( char ) OptParser::VERS << ", --" << long_options[11].name << "                            " << "version application" << std::endl;
        ss << "  -" << ( char ) OptParser::DISABLE_SAVE_VOTE << ", --" << long_options[12].name << "                 " << "disable save vote" << std::endl;
        ss << "  -" << ( char ) OptParser::MACHINE_MODEL << ", --" << long_options[13].name << "                 " << "machine model" << std::endl;

        help = ss.str();
    }

    return validate;
}

bool OptParser::getLoginOption ( std::string & login )
{
    return getArg ( OptParser::LOGIN, login );
}

bool OptParser::getPasswordOption ( std::string & password )
{
    return getArg ( OptParser::PASSWORD, password );
}

void OptParser::runApplicationProcess ( const std::string & package, std::string basepath )
{
    std::string configFile = "";
    std::string loggerFile = "";
    std::string xslfoFile = "";
    std::string helpStr = "";
    std::string mode;

    if ( getHelpOption ( helpStr ) )
    {
        std::cout << std::endl;
        std::cout << "Program name: " << package << std::endl;
        std::cout << "Version: " << VERSION << std::endl;
        std::cout << std::endl;
        std::cout << " " << package << " " << "[options]" << std::endl;
        std::cout << " " << "options" << std::endl;
        std::cout << helpStr << std::endl;
        std::cout << std::endl;
        ExitProcess::getInstance()->exitProcessWithCode ( EXIT_SUCCESS );
        return;
    }
    else if ( isSet ( OptParser::VERS ) )
    {
        std::cout << std::endl;
        std::cout << "Program name: " << package << std::endl;
        std::cout << "Version: " << VERSION << std::endl;
        std::cout << std::endl;
        ExitProcess::getInstance()->exitProcessWithCode ( EXIT_SUCCESS );
        return;
    }
    else if ( isSet ( OptParser::MODE ) )
    {
        getArg ( OptParser::MODE, mode );
        mode = Utils::StringUtils::toUpper ( mode );
    }

    if ( ! getArg ( OptParser::CONFIG, configFile ) )
    {
        configFile = Path::Combine ( Path::Combine ( basepath, mode ), CONFIG_FILENAME );
    }

    if ( ! getArg ( OptParser::LOGGER, loggerFile ) )
    {
        loggerFile = Path::Combine ( Path::Combine ( basepath, mode ), LOG_CONFIG_FILENAME );
    }

    if ( ! getArg ( OptParser::XSLFO, xslfoFile ) )
    {
        xslfoFile = Path::Combine ( Path::Combine ( basepath, mode ), XSLFO_CONFIG_FILENAME );
    }

    if ( Directory::getInstance()->FileExists ( CONFIG_FILENAME ) )
    {
        ConfigurationPersistenceManager::filePath = CONFIG_FILENAME;
    }
    else if ( Directory::getInstance()->FileExists ( configFile ) )
    {
        ConfigurationPersistenceManager::filePath = configFile;
    }
    else
    {
        ZenityWrapper::showError ( "Error load config file (" + std::string ( CONFIG_FILENAME ) + ")", "Error" );
        ExitProcess::getInstance()->exitProcess();
        return;
    }

    if ( Directory::getInstance()->FileExists ( LOG_CONFIG_FILENAME ) )
    {
        SMTTLogManager::getInstance()->setConfigFile ( LOG_CONFIG_FILENAME );
    }
    else if ( Directory::getInstance()->FileExists ( loggerFile ) )
    {
        SMTTLogManager::getInstance()->setConfigFile ( loggerFile );
    }
    else
    {
        ZenityWrapper::showError ( "Error load logger file (" + std::string ( LOG_CONFIG_FILENAME ) + ")", "Error" );
        ExitProcess::getInstance()->exitProcess();
        return;
    }

    if ( Directory::getInstance()->FileExists ( XSLFO_CONFIG_FILENAME ) )
    {
        FOPTransformer::setConfigFile ( XSLFO_CONFIG_FILENAME );
    }
    else if ( Directory::getInstance()->FileExists ( xslfoFile ) )
    {
        FOPTransformer::setConfigFile ( xslfoFile );
    }
}

OptParser * OptParser::getInstance()
{
    if ( !instance )
    {
        instance = new OptParser();
    }

    return instance;
}
