/**
 * @file CMSBase.cxx
 * @brief Body CMS extension base class to crypto and signed
 * @date 12/07/207
 * @author Marcel Mancini
 */

#include "CMSBase.hxx"
#include <System/Security/Cryptography/CertificateExtensionBase.hxx>
#include <openssl/cms.h>
#include <openssl/ssl.h>

using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System;

CMSBase::CMSBase ( int flags )
    : SecurityBase()
{
    this->memoryBIO = NULL;
    this->flags = flags;
}

CMSBase::~CMSBase()
{

}

bool CMSBase::processBuffer ( const SafeBuffer & bufferIn, SafeBuffer & bufferOut)
{
    bufferOut.Clear();

    if ( certificate == NULL )
    {
        return false;
    }

    memoryBIO = getMemoryBIO ( bufferIn );

    CMS_ContentInfo * cms = static_cast < CMS_ContentInfo * > ( CMSProcess() );

    if ( cms == NULL )
    {
        BIO_free ( memoryBIO );
        return false;
    }

    BIO * message = BIO_new ( BIO_s_mem() );

    int resp = 0;

    resp = SMIME_write_CMS ( message, cms, memoryBIO, 0 );


    if ( resp > 0 )
    {
        safeToBuffer ( message, bufferOut );
    }

    CMS_ContentInfo_free ( cms );
    BIO_free ( memoryBIO );
    BIO_free ( message );

    memoryBIO = NULL;

    return resp > 0;
}

bool CMSBase::reprocessBuffer ( const SafeBuffer & bufferIn, SafeBuffer & bufferOut)
{
    bufferOut.Clear();

    if ( certificate == NULL )
    {
        return false;
    }

    memoryBIO = getMemoryBIO ( bufferIn );

    CMS_ContentInfo * cms;

    cms = SMIME_read_CMS( memoryBIO, NULL);


    if ( cms == NULL )
    {
        BIO_free ( memoryBIO );
        return false;
    }

    BIO * message = BIO_new ( BIO_s_mem() );

    int resp = CMSProcess ( cms, message );

    if ( resp > 0 )
    {
        safeToBuffer ( message, bufferOut );
    }

    CMS_ContentInfo_free ( cms );
    BIO_free ( memoryBIO );
    BIO_free ( message );

    return resp > 0;
}

BIO * CMSBase::getMemoryBIO ( const Smartmatic::System::SafeBuffer & bufferIn )
{
    return BIO_new_mem_buf ( ( void * ) bufferIn.GetData(), bufferIn.GetSize() );
}

void CMSBase::safeToBuffer ( BIO * message, Smartmatic::System::SafeBuffer & bufferOut )
{
    char * bioDataPtr = NULL;
    long   bioDataLen = 0;

    bioDataLen = BIO_get_mem_data ( message, &bioDataPtr );
    bufferOut.AddData ( ( unsigned char * ) bioDataPtr, bioDataLen );
}
