/**
 * @file CMSSigner.cxx
 * @brief Body class CMSSigner
 * @date 12/07/2017
 * @author Marcel Mancini
 */

#include "CMSSigner.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <openssl/evp.h>
#include <openssl/err.h>
#include <openssl/cms.h>
#include <openssl/x509.h>
#include <openssl/safestack.h>
#include <vector>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;

ISMTTLog * CMSSigner::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.CMSSigner", GETTEXT_PACKAGE );

static void remove_duplicates(STACK_OF(X509)* stack, const X509* mainCertificate)
{
    std::vector<X509*> certificates;
    std::vector<X509*>::const_iterator it;
    X509* certificate;

    while ((certificate = sk_X509_pop(stack)) != NULL)
    {
        certificates.push_back(certificate);
    }

    for (it = certificates.begin(); it < certificates.end(); ++it)
    {
        if (X509_cmp(mainCertificate, *it) != 0)
        {
            sk_X509_push(stack, *it);
        }
        else
        {
            X509_free(*it);
        }
    }
}

CMSSigner::CMSSigner ()
    : CMSBase ( CMS_BINARY )
{

}

CMSSigner::~CMSSigner()
{

}

void CMSSigner::signBuffer ( const SafeBuffer & bufferIn, SafeBuffer & bufferOut) throw ( SignedException )
{
    if ( ! processBuffer ( bufferIn, bufferOut ) )
    {
        throw SignedException (
            _ ( "Smartmatic.System.Security.Cryptography.CMSSigner.ErrorSignedData" ),
            N_ ( "Smartmatic.System.Security.Cryptography.CMSSigner.ErrorSignedData" ), logger );
    }
}

void CMSSigner::verifyBuffer ( const SafeBuffer & bufferIn, Smartmatic::System::SafeBuffer & signedBuffer) throw ( SignedException )
{
    if ( ! reprocessBuffer ( bufferIn, signedBuffer ) )
    {
        throw SignedException ( _ ( "Smartmatic.System.Security.Cryptography.CMSSigner.ErrorValidateData" ),
                                N_ ( "Smartmatic.System.Security.Cryptography.CMSSigner.ErrorValidateData" ), logger );
    }
}

void * CMSSigner::CMSProcess()
{
    X509 * x509Certificate;
    STACK_OF ( X509 ) *certificateAuthorities;

    x509Certificate = certificate->LoadCertifiedX509();
    certificateAuthorities = certificate->getCertificateAuthorities();

    remove_duplicates(certificateAuthorities, x509Certificate);

    void * cms = ( void * ) CMS_sign ( x509Certificate,
                                       certificate->LoadCertifiedPrivateKey(),
                                       certificateAuthorities,
                                       memoryBIO,
                                       flags );

    return cms;
}

int CMSSigner::CMSProcess ( void * cms, BIO * result )
{
    X509 * cert = certificate->LoadCertifiedX509();
    X509_STORE *st = X509_STORE_new();
    int resultValue;

    if ( cert == NULL || !X509_STORE_add_cert(st, cert))
    {
        X509_STORE_free(st);
        return -1;
    }

    resultValue = CMS_verify ( ( CMS_ContentInfo * ) cms, NULL, st, NULL, result, flags  | CMS_NO_SIGNER_CERT_VERIFY);

    X509_STORE_free(st);
    return resultValue;
}

