/**
 * @file CertValidationResult.h
 * @brief Header class certificate validation result
 * @date 08/01/2014
 * @author Beatriz Avila
 */

#ifndef CERTVALIDATIONRESULT_H_
#define CERTVALIDATIONRESULT_H_

#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {

                /**
                 * @enum	TrustResultEnum
                 * @brief	Values that represent TrustResultEnum.
                 */
                enum TrustResultEnum
                {
                    TRUST_OK = 0,
                    TRUST_CANNOT_READ_TARGET = 1,
                    TRUST_INVALID_DATE = 2,
                    TRUST_CANNOT_CREATE_STORE = 3,
                    TRUST_CANNOT_READ_INTERMEDIATE_CA_CERTS = 4,
                    TRUST_CANNOT_READ_TRUSTED_CERTS = 5,
                    TRUST_ERR_UNABLE_TO_GET_CRL = 6,
                    TRUST_UNABLE_TO_GET_ISSUER_CERT = 7,
                    TRUST_UNABLE_TO_DECRYPT_CERT_SIGNATURE,
                    TRUST_UNABLE_TO_DECRYPT_CRL_SIGNATURE,
                    TRUST_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY,
                    TRUST_CERT_SIGNATURE_FAILURE,
                    TRUST_CRL_SIGNATURE_FAILURE,
                    TRUST_CERT_NOT_YET_VALID,
                    TRUST_CRL_NOT_YET_VALID,
                    TRUST_CERT_HAS_EXPIRED,
                    TRUST_ERR_CRL_HAS_EXPIRED,
                    TRUST_ERROR_IN_CERT_NOT_BEFORE_FIELD,
                    TRUST_ERROR_IN_CERT_NOT_AFTER_FIELD,
                    TRUST_ERROR_IN_CRL_LAST_UPDATE_FIELD,
                    TRUST_ERROR_IN_CRL_NEXT_UPDATE_FIELD,
                    TRUST_OUT_OF_MEM,
                    TRUST_DEPTH_ZERO_SELF_SIGNED_CERT,
                    TRUST_SELF_SIGNED_CERT_IN_CHAIN,
                    TRUST_UNABLE_TO_GET_ISSUER_CERT_LOCALLY,
                    TRUST_UNABLE_TO_VERIFY_LEAF_SIGNATURE,
                    TRUST_CERT_CHAIN_TOO_LONG,
                    TRUST_ERR_CERT_REVOKED,
                    TRUST_ERR_INVALID_CA,
                    TRUST_INVALID_NON_CA,
                    TRUST_ERR_PATH_LENGTH_EXCEEDED,
                    TRUST_ERR_PROXY_PATH_LENGTH_EXCEEDED,
                    TRUST_ERR_PROXY_CERTIFICATES_NOT_ALLOWED,
                    TRUST_ERR_INVALID_PURPOSE,
                    TRUST_ERR_CERT_UNTRUSTED,
                    TRUST_ERR_CERT_REJECTED,
                    TRUST_ERR_APPLICATION_VERIFICATION,
                    TRUST_ERR_SUBJECT_ISSUER_MISMATCH,
                    TRUST_ERR_AKID_SKID_MISMATCH,
                    TRUST_ERR_AKID_ISSUER_SERIAL_MISMATCH,
                    TRUST_ERR_KEYUSAGE_NO_CERTSIGN,
                    TRUST_ERR_UNABLE_TO_GET_CRL_ISSUER,
                    TRUST_ERR_UNHANDLED_CRITICAL_EXTENSION,
                    TRUST_ERR_KEYUSAGE_NO_CRL_SIGN,
                    TRUST_ERR_KEYUSAGE_NO_DIGITAL_SIGNATURE,
                    TRUST_ERR_UNHANDLED_CRITICAL_CRL_EXTENSION,
                    TRUST_ERR_INVALID_EXTENSION,
                    TRUST_ERR_INVALID_POLICY_EXTENSION,
                    TRUST_ERR_NO_EXPLICIT_POLICY,
                    TRUST_ERR_DIFFERENT_CRL_SCOPE,
                    TRUST_ERR_UNSUPPORTED_EXTENSION_FEATURE,
                    TRUST_ERR_UNNESTED_RESOURCE,
                    TRUST_ERR_PERMITTED_VIOLATION,
                    TRUST_ERR_EXCLUDED_VIOLATION,
                    TRUST_ERR_SUBTREE_MINMAX,
                    TRUST_ERR_UNSUPPORTED_CONSTRAINT_TYPE,
                    TRUST_ERR_UNSUPPORTED_CONSTRAINT_SYNTAX,
                    TRUST_ERR_UNSUPPORTED_NAME_SYNTAX,
                    TRUST_ERR_CRL_PATH_VALIDATION_ERROR
                };

                /**
                 * @class	CertValidationResult
                 * @brief	Cert validation result class.
                 * @author	freddyg
                 * @date	28/04/2011
                 */
                class CertValidationResult
                {
                    public:

                        /**
                         * @fn	CertValidationResult::CertValidationResult();
                         * @brief	Default constructor.
                         * @author	freddyg
                         * @date	28/04/2011
                         */
                        CertValidationResult();

                        /**
                         * @fn	CertValidationResult::CertValidationResult(CertValidationResult& x);
                         * @brief	Copy constructor.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @param	x	The x coordinate.
                         */
                        CertValidationResult ( const CertValidationResult & x );

                        /**
                         * @fn	CertValidationResult::CertValidationResult(int depthA, TrustResultEnum detailA);
                         * @brief	Constructor.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @param	depthA 	The depth .
                         * @param	detailA	The detail.
                         */
                        CertValidationResult ( int depthA, TrustResultEnum detailA );

                        /**
                         * @fn	virtual CertValidationResult::~CertValidationResult();
                         * @brief	Finaliser.
                         * @author	freddyg
                         * @date	28/04/2011
                         */
                        virtual ~CertValidationResult();

                        /**
                         * @fn	bool CertValidationResult::GetSucces();
                         * @brief	Gets the succes.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool GetSucces();

                        /**
                         * @fn	TrustResultEnum CertValidationResult::GetDetail();
                         * @brief	Gets the detail.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @return	The detail.
                         */
                        TrustResultEnum GetDetail();

                        /**
                         * @fn	void CertValidationResult::SetDetail(TrustResultEnum val);
                         * @brief	Sets a detail.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @param	val	The value.
                         */
                        void SetDetail ( TrustResultEnum val );

                        /**
                         * @fn	int CertValidationResult::GetDepth();
                         * @brief	Gets the depth.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @return	The depth.
                         */
                        int GetDepth();

                        /**
                         * @fn	void CertValidationResult::SetDepth(int newDepth);
                         * @brief	Sets a depth.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @param	newDepth	new Depth.
                         */
                        void SetDepth ( int newDepth );

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                    private:

                        int depth;  ///< The depth
                        TrustResultEnum detail;  ///< The detail
                };
            }
        }
    }
}
#endif /* CERTVALIDATIONRESULT_H_ */
