/**
 * @file CertificateExtensionBase.h
 * @brief Header class certificate extension.
 * @date 18/12/2013
 * @author Beatriz Avila
 */

#ifndef CERTIFICATEEXTENSIONBASE_HXX_
#define CERTIFICATEEXTENSIONBASE_HXX_

#include <System/SafeBuffer.h>

typedef struct x509_st X509;
typedef struct evp_pkey_st EVP_PKEY;
struct stack_st_X509;
typedef struct bio_st BIO;

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	CertificateExtensionBase
                 * @brief	Class certificate extension.
                 * @author	Beatriz Avila
                 * @date	18/12/2013
                 */
                class CertificateExtensionBase
                {
                    public:

                        /**
                         * @fn	CertificateExtensionBase ( const Smartmatic::System::SafeBuffer & certificateBuffer, const std::string & certPassword = "" );
                         * @brief	Class constructor.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @param	certificateBuffer	SafeBuffer with certificate.
                         * @param	certPassword		password.
                         */
                        CertificateExtensionBase ( const Smartmatic::System::SafeBuffer & certificateBuffer, const std::string & certPassword = "" );

                        /**
                         * @fn	virtual CertificateExtensionBase::~CertificateExtensionBase();
                         * @brief	Class destructor.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         */
                        virtual ~CertificateExtensionBase();

                        /**
                         * @fn	X509 * CertificateExtensionBase::LoadCertifiedX509();
                         * @brief	Method load certified file from certificateBuffer.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @return	certificate.
                         */
                        X509 * LoadCertifiedX509 ( );

                        /**
                         * @fn	EVP_PKEY * CertificateExtensionBase::LoadCertifiedPrivateKey();
                         * @brief	Method load private key from certificateBuffer.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @return	private key.
                         */
                        EVP_PKEY * LoadCertifiedPrivateKey ( );

                        /**
                         * @fn	EVP_PKEY * CertificateExtensionBase::LoadCertifiedPublicKey();
                         * @brief	Method load public key from certificateBuffer.
                         * @author	Yadickson Soto
                         * @date	21/01/2014
                         * @return	private key.
                         */
                        EVP_PKEY * LoadCertifiedPublicKey ( );

                        /**
                         * @fn	stack_st_X509 * CertificateExtensionBase::getCertificateAuthorities();
                         * @brief	Method get Certificate Authorities.
                         * @author	Beatriz Avila
                         * @date	19/12/2013
                         * @return	certificate authorities.
                         */
                        stack_st_X509 * getCertificateAuthorities ( );

                        /**
                         * @fn	void releaseCertifiedX509();
                         * @brief	Method to release Certified X509.
                         * @author	Yadickson Soto
                         * @date	08/01/2014
                         */
                        void releaseCertifiedX509();
                        /**
                         * @fn	void releaseCertifiedPrivateKey();
                         * @brief	Method to release Certified Private Key.
                         * @author	Yadickson Soto
                         * @date	08/01/2014
                         */
                        void releaseCertifiedPrivateKey();
                        /**
                         * @fn	void releaseCertifiedPublicKey();
                         * @brief	Method to release Certified Public Key.
                         * @author	Yadickson Soto
                         * @date	21/01/2014
                         */
                        void releaseCertifiedPublicKey();
                        /**
                         * @fn	void releaseCertificateAuthorities();
                         * @brief	Method to release Certificate Authorities.
                         * @author	Yadickson Soto
                         * @date	08/01/2014
                         */
                        void releaseCertificateAuthorities();
                        /**
                         * @fn	void releaseAll();();
                         * @brief	Method to release Certified X509, Certified Private Key and Certificate Authorities.
                         * @author	Yadickson Soto
                         * @date	08/01/2014
                         */
                        void releaseAll();

                        /**
                         * @fn     CertificateExtensionBase* clone() const;
                         * @brief  Clone the CertificateExtensionBase object.
                         * @return The new instance of the class.
                         * @author manuel.arguelles
                         * @date   08/08/2022
                         */
                        virtual CertificateExtensionBase* clone() const = 0;

                    protected:

                        Smartmatic::System::SafeBuffer certificateBuffer; ///< The SafeBuffer with the certificate
                        std::string certPassword; ///< The password

                        X509 * x509; ///< certificate
                        EVP_PKEY * privateKey; ///< private key;
                        EVP_PKEY * publicKey; ///< public key;
                        stack_st_X509 * certificateAuthorities; /**<certificate authorities*/
                        BIO * memoryBIO; ///< The BIO

                        /**
                         * @fn void CertificateExtensionBase::WriteSafeBufferToMemoryBIO();
                         * @brief	Method load SafeBuffer to memory BIO.
                         * @return true if success load
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         */
                        bool WriteSafeBufferToMemoryBIO ( ) ;

                        /**
                         * @fn virtual	X509 * CertificateExtensionBase::LoadCertifiedX509FromBIO();
                         * @brief	Method load certified file from BIO.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @return	certificate.
                         */
                        virtual X509 * LoadCertifiedX509FromBIO ( ) = 0;

                        /**
                         * @fn virtual	EVP_PKEY * CertificateExtensionBase::LoadCertifiedPrivateKeyFromBIO();
                         * @brief	Method load private key from BIO.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @return	private key.
                         */
                        virtual EVP_PKEY * LoadCertifiedPrivateKeyFromBIO ( ) = 0;

                        /**
                         * @fn virtual	STACK_OF ( X509 ) * CertificateExtensionBase::getCertificateAuthoritiesFromBIO();
                         * @brief	Method get certificate authorities from BIO
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @return	certificate authorities.
                         */
                        virtual stack_st_X509 * getCertificateAuthoritiesFromBIO ( ) = 0;

                    protected:

                        /**
                         * @fn void releaseMemoryBIO();
                         * @brief	Method release OpenSSL memoryBIO
                         * @author	Yadickson Soto
                         * @date	24/01/2014
                         */
                        void releaseMemoryBIO();

                    private:

                        static Smartmatic::Log::ISMTTLog * logger; ///<logger
                };
            }
        }
    }
}

#endif /* CERTIFICATEEXTENSIONBASE_HXX_ */
