/**
 * @file CertificateExtensionPKCS12.cxx
 * @brief Body class certificate PKCS12 extension.
 * @date 18/12/2013
 * @author Beatriz Avila
 */

#include "CertificateExtensionPKCS12.hxx"
#include <openssl/pkcs12.h>
#include <openssl/err.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::Log;

ISMTTLog * CertificateExtensionPKCS12::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.CertificateExtensionBase", GETTEXT_PACKAGE );

CertificateExtensionPKCS12::CertificateExtensionPKCS12 ( const Smartmatic::System::SafeBuffer & certificateBuffer, std::string certPassword ) :
    CertificateExtensionBase::CertificateExtensionBase ( certificateBuffer, certPassword )
{

}

CertificateExtensionPKCS12::~CertificateExtensionPKCS12 ( )
{
}

CertificateExtensionPKCS12 * CertificateExtensionPKCS12::clone() const
{
    CertificateExtensionPKCS12* cloned = new CertificateExtensionPKCS12(certificateBuffer, certPassword);

    if (x509 != NULL)
        cloned->x509 = X509_dup(x509);

    if (privateKey != NULL)
    {
        CRYPTO_add(&privateKey->references, 1, CRYPTO_LOCK_EVP_PKEY);
        cloned->privateKey = privateKey;
    }

    if (publicKey != NULL)
    {
        CRYPTO_add(&publicKey->references, 1, CRYPTO_LOCK_EVP_PKEY);
        cloned->publicKey = publicKey;
    }

    return cloned;
}

X509 * CertificateExtensionPKCS12::LoadCertifiedX509FromBIO ( )
{
    if ( x509 == NULL && privateKey == NULL && certificateAuthorities == NULL )
    {
        LoadP12 ();
    }

    return x509;
}

EVP_PKEY * CertificateExtensionPKCS12::LoadCertifiedPrivateKeyFromBIO ( )
{
    if ( x509 == NULL && privateKey == NULL && certificateAuthorities == NULL )
    {
        LoadP12 ();
    }

    return privateKey;
}

STACK_OF ( X509 ) * CertificateExtensionPKCS12::getCertificateAuthoritiesFromBIO ( )
{
    if ( x509 == NULL && privateKey == NULL && certificateAuthorities == NULL )
    {
        LoadP12 ();
    }

    return certificateAuthorities;
}

void CertificateExtensionPKCS12::LoadP12 ()
{
    PKCS12 * p12 = d2i_PKCS12_bio ( memoryBIO, NULL );

    if ( !p12 )
    {
        logger->Debug ( "Unable to load PKCS12 to memory BIO" );
        return;
    }

    if ( !PKCS12_parse ( p12, certPassword.c_str(), &privateKey, &x509, &certificateAuthorities ) )
    {
        logger->Debug ( "Unable to load certificate" );
    }

    if ((privateKey == NULL) && ( sk_X509_num(certificateAuthorities) == 1))
    {
        logger->Debug ( "Using certificate without private key." );
        x509 = sk_X509_pop(certificateAuthorities);
    }

    if ( p12 )
    {
        PKCS12_free ( p12 );
        p12 = NULL;
    }
}
