/**
 * @file CertificateValidator.cpp
 * @brief Body class certificate validator
 * @date 07/04/2011
 * @author Freddy Gomez
 */

#include "System/Security/Cryptography/CertificateValidator.h"
#include <System/SafeBuffer.h>
#include <System/IO/File.h>

#include <openssl/x509v3.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security::Cryptography;

/**
 * @fn static int cb(int ok, X509_STORE_CTX *ctx)
 * @brief Method for verify the store created
 * @param ok flag status
 * @param ctx pointer
 * @return 0 is true
 * @date 07/04/2011
 * @author Freddy Gomez
 */
static int cb ( int ok, X509_STORE_CTX * ctx );

/**
 * @fn static void check(X509_STORE *ctx, X509* target, DEFINE_STACK *uchain, DEFINE_STACK *tchain,
 *   TrustResultEnum& error, int& depth)
 * @brief Method check certificate
 * @param ctx context
 * @param target target
 * @param uchain certificate authorities
 * @param tchain certificate authorities
 * @param error result error
 * @param depth result depth
 * @return 1 is check success
 * @date 07/04/2011
 * @author Freddy Gomez
 */
static void check ( X509_STORE * ctx, X509 * target, DEFINE_STACK * uchain, DEFINE_STACK * tchain,
                    TrustResultEnum & error, int & depth );

/**
 * @fn X509_STORE * CreateAndConfigureStore(X509_VERIFY_PARAM *vpm )
 * @brief Method create and configure store
 * @param vpm x509 verify param
 * @return x509 store
 * @date 07/04/2011
 * @author Freddy Gomez
 */
X509_STORE * CreateAndConfigureStore ( X509_VERIFY_PARAM * vpm );

/**
 * @fn X509_VERIFY_PARAM * CreateAndInitializeParameters(CertificateValidationFlags flags, CertificatePurpose purpose, time_t checkTime)
 * @brief Method create and initialize parameters
 * @param flags certificate validation flags
 * @param purpose certificate purpose
 * @param checkTime check time
 * @return x509 verify param
 * @date 07/04/2011
 * @author Freddy Gomez
 */
X509_VERIFY_PARAM * CreateAndInitializeParameters ( CertificateValidationFlags flags, CertificatePurpose purpose, time_t checkTime );

/**
 * @fn void LookupCA(X509_STORE * cert_ctx, const std::string & path, CertUtil::CertFileFormat format)
 * @brief Method lookup certificate authorities
 * @param cert_ctx certificate context
 * @param path certificate path
 * @param format format
 * @date 07/04/2011
 * @author Freddy Gomez
 */
void LookupCA ( X509_STORE * cert_ctx, const std::string & path, CertUtil::CertFileFormat format );

ISMTTLog * CertificateValidator::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.CertificateValidator", GETTEXT_PACKAGE );

CertificateValidator::CertificateValidator()
: m_cert_ctx(NULL)
, m_untrusted(NULL)
, m_certAuthorityPath("")
, m_certAuthorityFormat(CertUtil::CertFormat_PEM)
, m_validationFlags(VERIFY_NO_FLAGS)
, m_checkTime(0)
{
}

CertificateValidator::~CertificateValidator()
{
    if ( m_cert_ctx != NULL )
    {
        X509_STORE_free ( m_cert_ctx );
    }
}

void CertificateValidator::SetOptions ( CertificateValidationFlags flags )
{
    m_validationFlags = ( CertificateValidationFlags ) ( m_validationFlags | flags );
}

void CertificateValidator::ClearOptions ( CertificateValidationFlags flags )
{
    m_validationFlags = ( CertificateValidationFlags ) ( m_validationFlags & ( ~flags ) );
}


void CertificateValidator::SetVerificationTime ( time_t time )
{
    m_checkTime = time;
}

void CertificateValidator::ClearVerificationTime()
{
    m_validationFlags = ( CertificateValidationFlags ) ( m_validationFlags & ( ~VERIFY_SET_CHECK_TIME ) );
}


//This would be the Root CA
void CertificateValidator::SetCACertificate ( const std::string & certPath, CertUtil::CertFileFormat format )
{
    m_certAuthorityPath = certPath;
    m_certAuthorityFormat = format;
}

//These would be the certs in the chain, untrusted cert for open ssl
void CertificateValidator::SetIntermediateCACertificate ( SafeBuffer & certBuffer, CertUtil::CertFileFormat format, const std::string & password )
{
    m_untrusted = CertUtil::LoadCertificateAuthorities ( certBuffer, format, password );
}

CertValidationResult CertificateValidator::VerifyCertificate(X509 * cert)
{
    CertValidationResult retVal ( 0, TRUST_OK );

    try
    {
        if (m_untrusted == NULL)
        {
            retVal.SetDetail ( TRUST_CANNOT_READ_INTERMEDIATE_CA_CERTS );
        }

        int depth = 1;
        TrustResultEnum detail = TRUST_CANNOT_CREATE_STORE;
        //Check the certificate
        check ( m_cert_ctx, cert, m_untrusted, NULL, detail, depth );
        retVal.SetDepth ( depth );
        retVal.SetDetail ( detail );
    }
    catch (std::exception & exc)
    {
        retVal.SetDetail ( TRUST_CANNOT_READ_TARGET );
    }

    return retVal;
}

CertValidationResult CertificateValidator::VerifyX509Certificate ( const std::string & certPath, CertUtil::CertFileFormat format, CertificatePurpose purpose, const std::string & password )
{
    CertValidationResult retVal ( 0, TRUST_OK );
    X509 * cert = CertUtil::LoadCertifiedX509 ( certPath, format, password );

    if (cert == NULL)
    {
        retVal.SetDetail ( TRUST_CANNOT_READ_TARGET );
    }
    else
    {
        retVal = VerifyCertificate(cert);
    }

    X509_free ( cert );

    return retVal;
}

CertValidationResult CertificateValidator::VerifyX509Certificate ( OpenSSLCryptoX509 * cert, CertificatePurpose purpose, const bool loadIntermediateCertificate )
{
    CertValidationResult retVal ( 0, TRUST_OK );

    if (cert == NULL)
    {
        retVal.SetDetail ( TRUST_CANNOT_READ_TARGET );
    }
    else
    {
        X509 * x = cert->getOpenSSLX509();

        retVal = loadIntermediateCertificate ? VerifyX509Certificate ( x, purpose ) : VerifyCertificate( x );
    }

    return retVal;
}

CertValidationResult CertificateValidator::VerifyX509Certificate ( X509 * cert, CertificatePurpose purpose )
{
    CertValidationResult retVal ( 0, TRUST_OK );

    if ( cert == NULL )
    {
        retVal.SetDetail ( TRUST_CANNOT_READ_TARGET );
        return retVal;
    }

    if ( ( m_validationFlags &  VERIFY_DO_NOT_VALIDATE ) != 0 )
    {
        return retVal;
    }

    X509_VERIFY_PARAM * vp = CreateAndInitializeParameters ( m_validationFlags, purpose, m_checkTime );
    m_cert_ctx = CreateAndConfigureStore ( vp );

    if ( m_cert_ctx != NULL )
    {
        //Load the CA
        LookupCA ( m_cert_ctx, m_certAuthorityPath, m_certAuthorityFormat );

        retVal = VerifyCertificate(cert);
    }
    else
    {
        retVal.SetDetail ( TRUST_CANNOT_CREATE_STORE );
    }

    if ( vp )
    {
        X509_VERIFY_PARAM_free ( vp );
    }

    return retVal;
}

X509_STORE * CreateAndConfigureStore ( X509_VERIFY_PARAM * vpm )
{
    X509_STORE *cert_ctx = X509_STORE_new();
    cert_ctx->verify_cb = cb;
    //Set parameters
    X509_STORE_set1_param ( cert_ctx, vpm );
    return cert_ctx;
}

X509_VERIFY_PARAM * CreateAndInitializeParameters ( CertificateValidationFlags flags, CertificatePurpose purpose, time_t checkTime )
{
    unsigned long verifyFlags = 0;
    //set purpose
    X509_VERIFY_PARAM * vp =  X509_VERIFY_PARAM_new();

    //set purpose
    if ( purpose != DefaultPurpose )
    {
        X509_VERIFY_PARAM_set_purpose ( vp, ( int ) purpose );
    }

    //set flags
    if ( ( flags & VERIFY_IGNORE_CRITICAL ) != 0 )
    {
        verifyFlags |= X509_V_FLAG_IGNORE_CRITICAL;
    }

    if ( ( flags & VERIFY_CB_ISSUER_CHECK ) != 0 )
    {
        verifyFlags |= X509_V_FLAG_CB_ISSUER_CHECK;
    }

    if ( ( flags & VERIFY_CRL_CHECK ) != 0 )
    {
        verifyFlags |= X509_V_FLAG_CRL_CHECK;
    }

    if ( ( flags & VERIFY_CRL_CHECK_ALL ) != 0 )
    {
        verifyFlags |= X509_V_FLAG_CRL_CHECK_ALL;
    }

    if ( ( flags & VERIFY_POLICY_CHECK ) != 0 )
    {
        verifyFlags |= X509_V_FLAG_POLICY_CHECK;
    }

    if ( ( flags & VERIFY_EXPLICIT_POLICY ) != 0 )
    {
        verifyFlags |= X509_V_FLAG_EXPLICIT_POLICY;
    }

    if ( ( flags & VERIFY_INHIBIT_ANY ) != 0 )
    {
        verifyFlags |= X509_V_FLAG_INHIBIT_ANY;
    }

    if ( ( flags & VERIFY_X509_STRICT ) != 0 )
    {
        verifyFlags |= X509_V_FLAG_X509_STRICT;
    }

    if ( ( flags & VERIFY_NOTIFY_POLICY ) != 0 )
    {
        verifyFlags |= X509_V_FLAG_NOTIFY_POLICY;
    }

#ifdef X509_V_FLAG_CHECK_SS_SIGNATURE

    if ( ( flags & VERIFY_CHECK_SS_SIGNATURE ) != 0 )
    {
        verifyFlags |= X509_V_FLAG_CHECK_SS_SIGNATURE;
    }

#endif

    if ( ( flags & VERIFY_ALLOW_PROXY_CERTS ) != 0 )
    {
        verifyFlags |= X509_V_FLAG_ALLOW_PROXY_CERTS;
    }

#ifdef X509_V_FLAG_EXTENDED_CRL_SUPPORT

    if ( ( flags & VERIFY_EXTENDED_CRL_SUPPORT ) == 1 ) //Not supported in version 0.9.8 openssl
    {
        verifyFlags |= X509_V_FLAG_EXTENDED_CRL_SUPPORT;
    }

#endif
#ifdef X509_V_FLAG_USE_DELTAS

    if ( ( flags & VERIFY_USE_DELTAS ) == 1 )        //Not supported in version 0.9.8 openssl
    {
        verifyFlags |= X509_V_FLAG_USE_DELTAS;
    }

#endif

    if ( ( flags & VERIFY_SET_CHECK_TIME ) != 0 )
    {
        X509_VERIFY_PARAM_set_time ( vp, checkTime );
    }

    X509_VERIFY_PARAM_set_flags ( vp, verifyFlags );
    return vp;
}

void LookupCA ( X509_STORE * cert_ctx, const std::string & path, CertUtil::CertFileFormat format )
{
    X509_LOOKUP * lookup = X509_STORE_add_lookup ( cert_ctx, X509_LOOKUP_file() );
    int fileformat = 0;

    if ( format == CertUtil::CertFormat_PEM )
    {
        fileformat = X509_FILETYPE_PEM;
    }
    else if ( format == CertUtil::CertFormat_DER )
    {
        fileformat = X509_FILETYPE_ASN1;
    }
    else
    {
        fileformat = X509_FILETYPE_DEFAULT;
    }

    X509_LOOKUP_load_file ( lookup, path.c_str(), fileformat );
}

static int cb ( int ok, X509_STORE_CTX * ctx )
{
    /* Exonerate certificate date constraint */
    const int error = X509_STORE_CTX_get_error(ctx);

    if ((error == X509_V_ERR_CERT_NOT_YET_VALID) ||
        (error == X509_V_ERR_CRL_NOT_YET_VALID) ||
        (error == X509_V_ERR_CERT_HAS_EXPIRED) ||
        (error == X509_V_ERR_CRL_HAS_EXPIRED))
    {
        return 1;
    }

    /* Otherwise don't override */
    return ok;
}

static void check ( X509_STORE * ctx, X509 * target, DEFINE_STACK * uchain, DEFINE_STACK * tchain,
                    TrustResultEnum & error, int & depth )
{
    X509_STORE_CTX * csc;
    int trustError = 0;
    X509 *x = target;

    csc = X509_STORE_CTX_new();

    if ( csc == NULL )
    {
        return;
    }

    if ( !X509_STORE_CTX_init ( csc, ctx, x, uchain ) )
    {
        return;
    }

    if ( tchain )
    {
        X509_STORE_CTX_trusted_stack ( csc, tchain );
    }

    if (X509_verify_cert ( csc ))
    {
        trustError = X509_V_OK;
    }
    else
    {
        trustError = X509_STORE_CTX_get_error ( csc );
    }

    depth =  X509_STORE_CTX_get_error_depth ( csc );
    X509_STORE_CTX_free ( csc );

    switch ( trustError )
    {
        case X509_V_OK:
            error = TRUST_OK;
            CertificateValidator::logger->Debug ( "Certificate is trusted" );
            break;

        case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
            error = TRUST_UNABLE_TO_GET_ISSUER_CERT;
            CertificateValidator::logger->Debug ( "unable to get issuer certificate" );
            break;

        case X509_V_ERR_UNABLE_TO_GET_CRL:
            error = TRUST_ERR_UNABLE_TO_GET_CRL;
            break;

        case X509_V_ERR_UNABLE_TO_DECRYPT_CERT_SIGNATURE:
            error = TRUST_UNABLE_TO_DECRYPT_CERT_SIGNATURE;
            CertificateValidator::logger->Debug ( "unable to decrypt certificate's signature" );
            break;

        case X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE:
            CertificateValidator::logger->Debug ( "unable to decrypt CRL's signature" );
            error = TRUST_UNABLE_TO_DECRYPT_CERT_SIGNATURE;
            break;

        case X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY:
            CertificateValidator::logger->Debug ( "unable to decode issuer public key" );
            error = TRUST_UNABLE_TO_DECRYPT_CRL_SIGNATURE;
            break;

        case X509_V_ERR_CERT_SIGNATURE_FAILURE:
            CertificateValidator::logger->Debug ( "certificate signature failure" );
            error = TRUST_CERT_SIGNATURE_FAILURE;
            break;

        case X509_V_ERR_CRL_SIGNATURE_FAILURE:
            CertificateValidator::logger->Debug ( "CRL signature failure" );
            error = TRUST_CERT_SIGNATURE_FAILURE;
            break;

        case X509_V_ERR_CERT_NOT_YET_VALID:
            CertificateValidator::logger->Debug ( "certificate is not yet valid" );
            error = TRUST_CERT_NOT_YET_VALID;
            break;

        case X509_V_ERR_CRL_NOT_YET_VALID:
            CertificateValidator::logger->Debug ( "CRL is not yet valid" );
            error = TRUST_CRL_NOT_YET_VALID;
            break;

        case X509_V_ERR_CERT_HAS_EXPIRED:
            CertificateValidator::logger->Debug ( "certificate has expired" );
            error = TRUST_CERT_HAS_EXPIRED;
            break;

        case X509_V_ERR_CRL_HAS_EXPIRED:
            CertificateValidator::logger->Debug ( "CRL has expired" );
            error = TRUST_ERR_CRL_HAS_EXPIRED;
            break;

        case X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD:
            CertificateValidator::logger->Debug ( "format error in certificate's notBefore field" );
            error = TRUST_ERROR_IN_CERT_NOT_BEFORE_FIELD;
            break;

        case X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD:
            CertificateValidator::logger->Debug ( "format error in certificate's notAfter field" );
            error = TRUST_ERROR_IN_CERT_NOT_AFTER_FIELD;
            break;

        case X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD:
            CertificateValidator::logger->Debug ( "format error in CRL's lastUpdate field" );
            error = TRUST_ERROR_IN_CRL_LAST_UPDATE_FIELD;
            break;

        case X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD:
            CertificateValidator::logger->Debug ( "format error in CRL's nextUpdate field" );
            error = TRUST_ERROR_IN_CRL_NEXT_UPDATE_FIELD;
            break;

        case X509_V_ERR_OUT_OF_MEM:
            CertificateValidator::logger->Debug ( "out of memory" );
            error = TRUST_OUT_OF_MEM;
            break;

        case X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT:
            CertificateValidator::logger->Debug ( "self signed certificate" );
            error = TRUST_DEPTH_ZERO_SELF_SIGNED_CERT;
            break;

        case X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN:
            CertificateValidator::logger->Debug ( "self signed certificate in certificate chain" );
            error = TRUST_SELF_SIGNED_CERT_IN_CHAIN;
            break;

        case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
            CertificateValidator::logger->Debug ( "unable to get local issuer certificate" );
            error = TRUST_UNABLE_TO_GET_ISSUER_CERT_LOCALLY;
            break;

        case X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE:
            CertificateValidator::logger->Debug ( "unable to verify the first certificate" );
            error = TRUST_UNABLE_TO_VERIFY_LEAF_SIGNATURE;
            break;

        case X509_V_ERR_CERT_CHAIN_TOO_LONG:
            CertificateValidator::logger->Debug ( "certificate chain too long" );
            error = TRUST_CERT_CHAIN_TOO_LONG;
            break;

        case X509_V_ERR_CERT_REVOKED:
            CertificateValidator::logger->Debug ( "certificate revoked" );
            error = TRUST_ERR_CERT_REVOKED;
            break;

        case X509_V_ERR_INVALID_CA:
            CertificateValidator::logger->Debug ( "invalid CA certificate" );
            error = TRUST_ERR_INVALID_CA;
            break;

        case X509_V_ERR_INVALID_NON_CA:
            CertificateValidator::logger->Debug ( "invalid non-CA certificate (has CA markings)" );
            error = TRUST_INVALID_NON_CA;
            break;

        case X509_V_ERR_PATH_LENGTH_EXCEEDED:
            CertificateValidator::logger->Debug ( "path length constraint exceeded" );
            error = TRUST_ERR_PATH_LENGTH_EXCEEDED;
            break;

        case X509_V_ERR_PROXY_PATH_LENGTH_EXCEEDED:
            CertificateValidator::logger->Debug ( "proxy path length constraint exceeded" );
            error = TRUST_ERR_PROXY_PATH_LENGTH_EXCEEDED;
            break;

        case X509_V_ERR_PROXY_CERTIFICATES_NOT_ALLOWED:
            CertificateValidator::logger->Debug ( "proxy certificates not allowed, please set the appropriate flag" );
            error = TRUST_ERR_PROXY_CERTIFICATES_NOT_ALLOWED;
            break;

        case X509_V_ERR_INVALID_PURPOSE:
            CertificateValidator::logger->Debug ( "unsupported certificate purpose" );
            error = TRUST_ERR_INVALID_PURPOSE;
            break;

        case X509_V_ERR_CERT_UNTRUSTED:
            CertificateValidator::logger->Debug ( "certificate not trusted" );
            error = TRUST_ERR_CERT_UNTRUSTED;
            break;

        case X509_V_ERR_CERT_REJECTED:
            CertificateValidator::logger->Debug ( "certificate rejected" );
            error = TRUST_ERR_CERT_REJECTED;
            break;

        case X509_V_ERR_APPLICATION_VERIFICATION:
            CertificateValidator::logger->Debug ( "application verification failure" );
            error = TRUST_ERR_APPLICATION_VERIFICATION;
            break;

        case X509_V_ERR_SUBJECT_ISSUER_MISMATCH:
            CertificateValidator::logger->Debug ( "subject issuer mismatch" );
            error = TRUST_ERR_SUBJECT_ISSUER_MISMATCH;
            break;

        case X509_V_ERR_AKID_SKID_MISMATCH:
            CertificateValidator::logger->Debug ( "authority and subject key identifier mismatch" );
            error = TRUST_ERR_AKID_SKID_MISMATCH;
            break;

        case X509_V_ERR_AKID_ISSUER_SERIAL_MISMATCH:
            CertificateValidator::logger->Debug ( "authority and issuer serial number mismatch" );
            error = TRUST_ERR_AKID_ISSUER_SERIAL_MISMATCH;
            break;

        case X509_V_ERR_KEYUSAGE_NO_CERTSIGN:
            CertificateValidator::logger->Debug ( "key usage does not include certificate signing" );
            error = TRUST_ERR_KEYUSAGE_NO_CERTSIGN;
            break;

        case X509_V_ERR_UNABLE_TO_GET_CRL_ISSUER:
            CertificateValidator::logger->Debug ( "unable to get CRL issuer certificate" );
            error = TRUST_ERR_UNABLE_TO_GET_CRL_ISSUER;
            break;

        case X509_V_ERR_UNHANDLED_CRITICAL_EXTENSION:
            CertificateValidator::logger->Debug ( "unhandled critical extension" );
            error = TRUST_ERR_UNHANDLED_CRITICAL_EXTENSION;
            break;

        case X509_V_ERR_KEYUSAGE_NO_CRL_SIGN:
            CertificateValidator::logger->Debug ( "key usage does not include CRL signing" );
            error = TRUST_ERR_KEYUSAGE_NO_CRL_SIGN;
            break;

        case X509_V_ERR_KEYUSAGE_NO_DIGITAL_SIGNATURE:
            CertificateValidator::logger->Debug ( "key usage does not include digital signature" );
            error = TRUST_ERR_KEYUSAGE_NO_DIGITAL_SIGNATURE;
            break;

        case X509_V_ERR_UNHANDLED_CRITICAL_CRL_EXTENSION:
            CertificateValidator::logger->Debug ( "unhandled critical CRL extension" );
            error = TRUST_ERR_UNHANDLED_CRITICAL_CRL_EXTENSION;
            break;

        case X509_V_ERR_INVALID_EXTENSION:
            CertificateValidator::logger->Debug ( "invalid or inconsistent certificate extension" );
            error = TRUST_ERR_INVALID_EXTENSION;
            break;

        case X509_V_ERR_INVALID_POLICY_EXTENSION:
            CertificateValidator::logger->Debug ( "invalid or inconsistent certificate policy extension" );
            error = TRUST_ERR_INVALID_POLICY_EXTENSION;
            break;

        case X509_V_ERR_NO_EXPLICIT_POLICY:
            CertificateValidator::logger->Debug ( "no explicit policy" );
            error = TRUST_ERR_NO_EXPLICIT_POLICY;
            break;

        case X509_V_ERR_UNNESTED_RESOURCE:
            error = TRUST_ERR_UNNESTED_RESOURCE;
            CertificateValidator::logger->Debug ( "RFC 3779 resource not subset of parent's resources" );
            break;

        default:
            CertificateValidator::logger->Debug ( "Unknown" );
            break;
    }
}
