/**
 * @file CryptoCMS.cxx
 * @brief Body class crypto CMS
 * @date 12/07/2017
 * @author Marcel Mancini
 */

#include "CryptoCMS.hxx"
#include <openssl/evp.h>
#include <openssl/cms.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;

ISMTTLog * CryptoCMS::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.CryptoCMS", GETTEXT_PACKAGE );

CryptoCMS::CryptoCMS ( const EVP_CIPHER * cipherMode )
    : CMSBase ( CMS_BINARY )
{
    if ( cipherMode == NULL )
    {
        this->cipherMode = EVP_aes_256_cbc();
    }
    else
    {
        this->cipherMode = cipherMode;
    }
}

CryptoCMS::~CryptoCMS()
{

}

void CryptoCMS::encryptBuffer ( const SafeBuffer & bufferIn, SafeBuffer & bufferOut) throw ( CryptoException )
{
    if ( ! processBuffer ( bufferIn, bufferOut) )
    {
        throw CryptoException ( _ ( "Smartmatic.System.Security.Cryptography.CryptoCMS.ErrorCryptoData" ),
                                N_ ( "Smartmatic.System.Security.Cryptography.CryptoCMS.ErrorCryptoData" ), logger );
    }
}

void CryptoCMS::decryptBuffer ( const SafeBuffer & bufferIn, SafeBuffer & bufferOut) throw ( CryptoException )
{
    if ( ! reprocessBuffer ( bufferIn, bufferOut) )
    {
        throw CryptoException ( _ ( "Smartmatic.System.Security.Cryptography.CryptoCMS.ErrorDecryptoData" ),
                                N_ ( "Smartmatic.System.Security.Cryptography.CryptoCMS.ErrorDecryptoData" ), logger );
    }
}

void * CryptoCMS::CMSProcess()
{
    X509 * cert = certificate->LoadCertifiedX509();

    if ( cert == NULL )
    {
        return NULL;
    }

    STACK_OF ( X509 ) * ca = NULL;
    ca = sk_X509_new_null();
    sk_X509_push ( ca, cert );

    void * cms = ( void * ) CMS_encrypt ( ca, memoryBIO, cipherMode, flags );

    sk_X509_free ( ca );

    return cms;
}

int CryptoCMS::CMSProcess ( void * cms, BIO * result )
{
    EVP_PKEY * pkey = certificate->LoadCertifiedPrivateKey();

    if ( pkey == NULL )
    {
        return -1;
    }

    return CMS_decrypt ( ( CMS_ContentInfo * ) cms, pkey, certificate->LoadCertifiedX509(), NULL, result, flags );
}

