/**
 * @file CryptoCMS.hxx
 * @brief HEader class crypto CMS
 * @date 12/07/2017
 * @author Marcel Mancini
 */

#ifndef CRYPTOCMS_HXX_
#define CRYPTOCMS_HXX_

#include <System/Exception/CryptoException.hxx>
#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/CMSBase.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	CryptoCMS
                 * @brief	Class crypto CMS.
                 * @author	Marcel Mancini
                 * @date	12/07/2017
                 */
                class CryptoCMS : public CMSBase
                {
                    public:

                        /**
                         * @fn	CryptoCMS::CryptoCMS( const EVP_CIPHER * cipherMode = NULL );
                         * @brief	Class constructor.
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @param	cipherMode		cipher mode crypto.
                         */
                        explicit CryptoCMS ( const EVP_CIPHER * cipherMode = NULL );

                        /**
                         * @fn	virtual CryptoCMS::~CryptoCMS();
                         * @brief	Class destructor.
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         */
                        virtual ~CryptoCMS();

                        /**
                         * @fn	void CryptoCMS::encryptBuffer(const Smartmatic::System::SafeBuffer & bufferIn,
                         * 			Smartmatic::System::SafeBuffer & bufferOut) throw (Smartmatic::System::Exception::CryptoException);
                         * @brief	method crypto buffer.
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * 				    @throw CryptoException
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                         * @param	bufferIn 	buffer in.
                         * @param	bufferOut	buffer out result.
                         * @param	type		 	crypto type.
                         */
                        void encryptBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                             Smartmatic::System::SafeBuffer & bufferOut)
                        throw ( Smartmatic::System::Exception::CryptoException );

                        /**
                        * @fn    void CryptoCMS::decryptBuffer(const Smartmatic::System::SafeBuffer & bufferIn,
                        *             Smartmatic::System::SafeBuffer & bufferOut) throw (Smartmatic::System::Exception::CryptoException);
                        * @brief    method decrypto buffer.
                        * @author    Marcel Mancini
                        * @date    12/07/2017
                        *                     @throw CryptoException
                        * @exception    Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                        * @param    bufferIn 	buffer in.
                        * @param    bufferOut	buffer out result.
                        */
                        void decryptBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                             Smartmatic::System::SafeBuffer & bufferOut)
                        throw ( Smartmatic::System::Exception::CryptoException );

                    protected:

                        /**
                         * @fn	virtual void * CMSProcess() = 0;
                         * @brief	Method to process CMS object
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @return	CMS pointer
                         */
                        virtual void * CMSProcess();

                        /**
                         * @fn	virtual int CMSProcess ( void * cms, BIO * result ) = 0;
                         * @brief	Method to process CMS object and getter result value
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @param	cms CMS pointer to process
                         * @param 	result result buffer
                         * @return	successfully if return value > 0
                         */
                        virtual int CMSProcess ( void * cms, BIO * result );

                    private:

                        const EVP_CIPHER * cipherMode; ///< chipher mode
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/
                };
            }
        }
    }
}

#endif /* CRYPTOCMS_HXX_ */
