/**
 * @file CryptoPKCS7.hxx
 * @brief HEader class crypto PKCS7
 * @date 19/03/2012
 * @author Yadickson Soto
 */

#ifndef CRYPTOPKCS7_HXX_
#define CRYPTOPKCS7_HXX_

#include <System/Exception/CryptoException.hxx>
#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/PKCS7Base.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	CryptoPKCS7
                 * @brief	Class crypto PKCS7.
                 * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date	19/03/2012
                 */
                class CryptoPKCS7 : public PKCS7Base
                {
                    public:

                        /**
                         * @fn	CryptoPKCS7::CryptoPKCS7( const EVP_CIPHER * cipherMode = NULL );
                         * @brief	Class constructor.
                         * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date	19/03/2012
                         * @param	cipherMode		cipher mode crypto.
                         */
                        explicit CryptoPKCS7 ( const EVP_CIPHER * cipherMode = NULL );

                        /**
                         * @fn	virtual CryptoPKCS7::~CryptoPKCS7();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date	19/03/2012
                         */
                        virtual ~CryptoPKCS7();

                        /**
                         * @fn	void CryptoPKCS7::encryptBuffer(const Smartmatic::System::SafeBuffer & bufferIn,
                         * 			Smartmatic::System::SafeBuffer & bufferOut,
                         * 			CryptoBase::PKCS7BaseType type) throw (Smartmatic::System::Exception::CryptoException);
                         * @brief	method crypto buffer.
                         * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date	19/03/2012
                         * 				    @throw CryptoException
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                         * @param	bufferIn 	buffer in.
                         * @param	bufferOut	buffer out result.
                         * @param	type		 	crypto type.
                         */
                        void encryptBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                             Smartmatic::System::SafeBuffer & bufferOut,
                                             PKCS7BaseType type )
                        throw ( Smartmatic::System::Exception::CryptoException );

                        /**
                        * @fn    void CryptoPKCS7::decryptBuffer(const Smartmatic::System::SafeBuffer & bufferIn,
                        *             Smartmatic::System::SafeBuffer & bufferOut,
                        *             CryptoBase::PKCS7BaseType type) throw (Smartmatic::System::Exception::CryptoException);
                        * @brief    method decrypto buffer.
                        * @author    Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                        * @date    19/03/2012
                        *                     @throw CryptoException
                        * @exception    Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                        * @param    bufferIn 	buffer in.
                        * @param    bufferOut	buffer out result.
                        * @param    type    	 	crypto type.
                        */
                        void decryptBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                             Smartmatic::System::SafeBuffer & bufferOut,
                                             PKCS7BaseType type )
                        throw ( Smartmatic::System::Exception::CryptoException );

                    protected:

                        /**
                         * @fn	virtual void * PKCS7Process() = 0;
                         * @brief	Method to process pkcs7 object
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         * @return	PKCS7 pointer
                         */
                        virtual void * PKCS7Process();

                        /**
                         * @fn	virtual int PKCS7Process ( void * p7, BIO * result ) = 0;
                         * @brief	Method to process pkcs7 object and getter result value
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         * @param	p7 PKCS7 pointer to process
                         * @param 	result result buffer
                         * @return	successfully if return value > 0
                         */
                        virtual int PKCS7Process ( void * p7, BIO * result );

                    private:

                        const EVP_CIPHER * cipherMode; ///< chipher mode
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/
                };
            }
        }
    }
}

#endif /* CRYPTOPKCS7_HXX_ */
