/**
 * @file ECDSASigner.cxx
 * @brief Body ECDSA signer class
 * @date 10/11/2011
 * @author Freddy Gomez
 * @author manuel.arguelles
 */

#include "ECDSASigner.hxx"
#include <Log/SMTTLogManager.h>
#include <System/Security/Cryptography/DigestAlgorithmLookup.h>
#include <openssl/evp.h>
#include <openssl/safestack.h>
#include <openssl/x509.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System;

Smartmatic::Log::ISMTTLog* ECDSASigner::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.System.Security.Cryptography.ECDSASigner",
    GETTEXT_PACKAGE);

ECDSASigner::ECDSASigner()
{
}

ECDSASigner::~ECDSASigner()
{
}

void ECDSASigner::SignData(const SafeBuffer& inSafeBuffer,
                           SafeBuffer& outSafeBuffer,
                           MsgDigestUtil::DigestAlgType hashKind)
    throw(SignedException)
{
    EVP_PKEY* pkey = GetPrivateKey();
    EVP_MD_CTX* mdCtx = NULL;
    size_t resultSize;
    int result;

    if (pkey == NULL)
    {
        throw SignedException(_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              N_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              m_logger);
    }

    if (pkey->type != EVP_PKEY_EC)
    {
        throw SignedException(_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              N_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              m_logger);
    }

    mdCtx = EVP_MD_CTX_create();

    if (mdCtx == NULL)
    {
        throw SignedException(_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              N_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              m_logger);
    }

    result = EVP_DigestSignInit(mdCtx, NULL, DigestAlgorithmLookup::LookupDigest(hashKind), NULL, pkey);

    if (result == 1)
    {
        result = EVP_DigestSignUpdate(mdCtx, inSafeBuffer.GetData(), inSafeBuffer.GetSize());
    }

    if (result == 1)
    {
        result = EVP_DigestSignFinal(mdCtx, NULL, &resultSize);
    }

    if (result == 1)
    {
        unsigned char* resultBuffer = static_cast<unsigned char*>(OPENSSL_malloc(resultSize));

        if (resultBuffer == NULL)
        {
            EVP_MD_CTX_destroy(mdCtx);
            throw SignedException(_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                                  N_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                                  m_logger);
        }

        result = EVP_DigestSignFinal(mdCtx, resultBuffer, &resultSize);

        if (result == 1)
        {
            outSafeBuffer.Clear();
            outSafeBuffer.AddData(resultBuffer, resultSize);
        }

        OPENSSL_free(resultBuffer);
    }

    EVP_MD_CTX_destroy(mdCtx);

    if (result != 1)
    {
        throw SignedException(_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              N_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              m_logger);
    }
}

bool ECDSASigner::VerifySignature(const SafeBuffer& inSafeBuffer,
                                  const SafeBuffer& signature,
                                  MsgDigestUtil::DigestAlgType hashKind)
    throw(SignedException)
{
    EVP_PKEY* pkey = GetPublicKey();
    EVP_MD_CTX* mdCtx = NULL;
    int result;

    if (pkey == NULL)
    {
        throw SignedException(_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              N_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              m_logger);
    }

    if (pkey->type != EVP_PKEY_EC)
    {
        throw SignedException(_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              N_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              m_logger);
    }

    mdCtx = EVP_MD_CTX_create();

    if (mdCtx == NULL)
    {
        throw SignedException(_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              N_("Smartmatic.System.Security.Cryptography.ECDSASigner.CannotComputeSignature"),
                              m_logger);
    }

    result = EVP_DigestVerifyInit(mdCtx, NULL, DigestAlgorithmLookup::LookupDigest(hashKind), NULL, pkey);

    if (result == 1)
    {
        result = EVP_DigestVerifyUpdate(mdCtx, inSafeBuffer.GetData(), inSafeBuffer.GetSize());
    }

    if (result == 1)
    {
        result = EVP_DigestVerifyFinal(mdCtx, (unsigned char *) signature.GetData(), signature.GetSize());
    }

    EVP_MD_CTX_destroy(mdCtx);
    return (result == 1);
}
