/**
 * @file   HmacSha256.cxx
 * @brief  Implementation of HmacSha256 class.
 * @author manuel.arguelles
 * @date   sep 20, 2018
 */

#include "HmacSha256.hxx"

#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Exception/CryptoException.hxx>

#include <openssl/evp.h>
#include <openssl/sha.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System;

static ISMTTLog * logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.HmacSha256", GETTEXT_PACKAGE );

HmacSha256::HmacSha256(const SafeBuffer& key)
    : m_key(key)
{
}

HmacSha256::~HmacSha256()
{
}

void HmacSha256::update(const SafeBuffer& data)
{
    m_data.AddData(data);
}

SafeBuffer HmacSha256::getDigest() const
{
    EVP_MD_CTX* context = EVP_MD_CTX_create();
    EVP_PKEY*   pkey    = EVP_PKEY_new_mac_key(EVP_PKEY_HMAC, NULL, m_key.GetData(), m_key.GetSize());

    EVP_DigestInit(context, EVP_sha256());
    EVP_DigestSignInit(context, NULL, EVP_sha256(), NULL, pkey);
    EVP_DigestSignUpdate(context, m_data.GetData(), m_data.GetSize());

    unsigned char digest[EVP_MAX_MD_SIZE];
    size_t        digestSize = 0;

    EVP_DigestSignFinal(context, digest, &digestSize);

    EVP_PKEY_free(pkey);
    EVP_MD_CTX_destroy(context);

    SafeBuffer output(digest, digestSize);

    if (static_cast<long>(output.GetSize()) != EVP_MD_size(EVP_sha256()))
    {
        throw CryptoException(
            _( "Smartmatic.System.Security.Cryptography.HmacSha256.Error" ),
            N_( "Smartmatic.System.Security.Cryptography.HmacSha256.Error" ),
            logger);
    }

    return output;
}

bool Smartmatic::System::Security::Cryptography::HmacSha256Validate(const HmacSha256& hmac, const SafeBuffer& code)
{
    bool       valid  = false;
    SafeBuffer digest = hmac.getDigest();

    if (code.GetSize() == digest.GetSize() &&
        CRYPTO_memcmp(code.GetData(), digest.GetData(), digest.GetSize()) == 0)
    {
        valid = true;
    }

    return valid;
}
