/**
 * @file PBEKeyDerivatorHMAC.cpp
 * @brief Body class PBE Key derivator hmac
 * @date  05/04/2011
 * @author Freddy Gomez
 */

#include "PBEKeyDerivatorHMAC.h"

#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Exception/CryptoException.hxx>

#include <openssl/evp.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System;

static ISMTTLog * logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.PBEKeyDerivatorHMAC", GETTEXT_PACKAGE );

PBEKeyDerivatorHMAC::PBEKeyDerivatorHMAC()
{

}

PBEKeyDerivatorHMAC::~PBEKeyDerivatorHMAC()
{

}

void PBEKeyDerivatorHMAC::SetSalt ( const SafeBuffer & salt )
{
    this->salt.Clear();
    this->salt.AddData ( salt );
}

void PBEKeyDerivatorHMAC::GenerateKey ( const SafeBuffer & passPhrase, SafeBuffer & outKey, MsgDigestUtil::DigestAlgType mdAlg, unsigned int keyLen, unsigned int iterations )
{
    outKey.Clear();

    if ( passPhrase.GetSize() == 0 )
    {
        throw CryptoException(
            _( "Smartmatic.System.Security.Cryptography.PBEKeyDerivatorHMAC.EmptyPassphrase" ),
            N_( "Smartmatic.System.Security.Cryptography.PBEKeyDerivatorHMAC.EmptyPassphrase" ),
            logger);
    }

    const EVP_MD * digest = DigestAlgorithmLookup::LookupDigest ( mdAlg );

    keyLen = keyLen > 0 ? keyLen : 32;

    unsigned char * ptr = new unsigned char [ keyLen ];

    int success =  PKCS5_PBKDF2_HMAC (
        reinterpret_cast<const char *>(passPhrase.GetData()),
        passPhrase.GetSize(),
        salt.GetData(),
        salt.GetSize(),
        iterations,
        digest,
        keyLen,
        ptr );

    if (success != 1)
    {
        delete [] ptr;
        throw CryptoException(
            _( "Smartmatic.System.Security.Cryptography.PBEKeyDerivatorHMAC.Error" ),
            N_( "Smartmatic.System.Security.Cryptography.PBEKeyDerivatorHMAC.Error" ),
            logger);
    }

    outKey.AddData ( ptr, keyLen );

    delete [] ptr;
}
