/**
 * @file PKCS7Base.hxx
 * @brief Header PKCS7 extension base class to crypto and signed
 * @date 23/01/2014
 * @author Yadickson Soto
 */

#ifndef PKCS7_BASE_HXX_
#define PKCS7_BASE_HXX_

#include <System/Security/SecurityBase.hxx>
#include <openssl/bio.h>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	PKCS7Base
                 * @brief	Class crypto PKCS7.
                 * @author	Yadickson Soto
                 * @date	23/01/2014
                 */
                class PKCS7Base: public Smartmatic::System::Security::SecurityBase
                {
                    public:

                        /**
                         * @enum	PKCS7BaseType
                         * @brief	pkcs7 base type.
                         */
                        enum PKCS7BaseType
                        {
                            PEM ///< crypto and signed PEM type
                            , SMIME ///< crypto and signed S/MIME type
                            , DER ///< crypto and signed DER type
                        };

                        /**
                         * @fn	PKCS7Base( int flags );
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         * @param 	flags flags to process PKCS7 object
                         */
                        explicit PKCS7Base ( int flags );

                        /**
                         * @fn	virtual ~PKCS7Base();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         */
                        virtual ~PKCS7Base();

                    protected:

                        /**
                         * @fn	void processBuffer(
                         * 				const Smartmatic::System::SafeBuffer & bufferIn,
                         *				Smartmatic::System::SafeBuffer & bufferOut,
                         *				CryptoBase::PKCS7BaseType type);
                         *
                         * @brief	method to encrypt or sign a buffer.
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         *
                         * @param	bufferIn 	buffer in.
                         * @param	bufferOut	buffer out result.
                         * @param	type		pkcs7 base type.
                         * @return true if successfully
                         */
                        bool processBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                             Smartmatic::System::SafeBuffer & bufferOut,
                                             PKCS7BaseType type );

                        /**
                         * @fn    void reprocessBuffer(
                         * 				const Smartmatic::System::SafeBuffer & bufferIn,
                         *				Smartmatic::System::SafeBuffer & bufferOut,
                         *				CryptoBase::PKCS7BaseType type);
                         *
                         * @brief	method to decrypt of verify the signature of the buffer.
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         *
                         * @param    bufferIn 	buffer in.
                         * @param    bufferOut	buffer out result.
                         * @param	type		pkcs7 base type.
                         * @return true if successfully
                         */
                        bool reprocessBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                               Smartmatic::System::SafeBuffer & bufferOut,
                                               PKCS7BaseType type );

                        /**
                         * @fn	virtual void * PKCS7Process() = 0;
                         * @brief	Method to process pkcs7 object
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         * @return	PKCS7 pointer
                         */
                        virtual void * PKCS7Process() = 0;

                        /**
                         * @fn	virtual int PKCS7Process ( void * p7, BIO * result ) = 0;
                         * @brief	Method to process pkcs7 object and get a result value
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         * @param	p7 PKCS7 pointer to process
                         * @param 	result result buffer
                         * @return	successfully if return value > 0
                         */
                        virtual int PKCS7Process ( void * p7, BIO * result ) = 0;

                        BIO * memoryBIO; ///< The BIO memory
                        int flags; ///< flag to process message

                    private:

                        /**
                         * @fn	BIO * getMemoryBIO ( const Smartmatic::System::SafeBuffer & bufferIn );
                         * @brief	Method to get a BIO pointer from a buffer.
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         * @param    bufferIn	buffer in.
                         * @return	memory OpenSSL BIO
                         */
                        BIO * getMemoryBIO ( const Smartmatic::System::SafeBuffer & bufferIn );

                        /**
                         * @fn	void safeToBuffer( BIO * message, Smartmatic::System::SafeBuffer & bufferOut );
                         * @brief	Method to convert a BIO to a safeBuffer
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         * @param    message 	OpenSSL BIO message.
                         * @param    bufferOut	buffer out result.
                         */
                        void safeToBuffer ( BIO * message, Smartmatic::System::SafeBuffer & bufferOut );
                };
            }
        }
    }
}

#endif /* PKCS7_BASE_HXX_ */
