/**
 * @file RsaUtil.h
 * @brief Header RSA util class
 * @date 24/05/2011
 * @author Marcel Cordovi
 */

#ifndef RSAUTIL_H_
#define RSAUTIL_H_

#include <string>
#include "System/SafeBuffer.h"
#include "System/Exception/RSAException.h"
#include <System/Exception/SignedException.hxx>
#include "System/Security/Cryptography/MsgDigestUtil.h"
#include "System/Security/Cryptography/SignatureAlgorithm.h"

#define DEFAULT_MOD 1024 ///< default mod
#define DEFAULT_EXP 0X10001 ///< default exp

namespace Smartmatic
{
    namespace Log
    {

        /**
         * @class	ISMTTLog
         * @brief	Ismtt log.
         * @author	Youssef.tourkmani
         * @date	24/10/2012
         */
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	RsaUtil
                 * @brief	Rsa utility.
                 * @author	Youssef.tourkmani
                 * @date	24/10/2012
                 */
                class RsaUtil : public SignatureAlgorithm
                {
                    public:

                        /**
                         * @typedef	enum RsaPaddingEnum
                         * @brief	Message padding type for use in rsa encyption/decryption computations.
                         */
                        typedef enum RsaPaddingEnum
                        {
                            RSAPAD_PKCS1, // default
                            RSAPAD_OAEP,
                            RSAPAD_SSL,
                            RSAPAD_RAW

                        } RsaPaddingEnumType; ///< Message padding type for use in rsa encyption/decryption computations

                        /**
                         * @fn	RsaUtil::RsaUtil ();
                         * @brief	Default constructor.
                         * @author	Youssef.tourkmani
                         * @date	24/10/2012
                         */
                        RsaUtil ();

                        /**
                         * @fn	virtual RsaUtil::~RsaUtil();
                         * @brief	Destructor.
                         * @author	mcordovi
                         * @date	May 24, 2011
                         */
                        virtual ~RsaUtil();

                        /**
                         * @fn	Smartmatic::System::SafeBuffer RsaUtil::PublicEncrypt ( const Smartmatic::System::SafeBuffer& plaintext) throw (Smartmatic::System::Exception::RSAException);
                         * @brief	Public key encryption.
                         * @author	mcordovi
                         * @date	May 24, 2011
                         * @exception	Smartmatic::System::Exception::RSAException	Thrown when a rsa error condition occurs.
                         * @param	plaintext	buffer containing the message to be encrypted.
                         * @return	a buffer containing the ciphertext obtained from the encryption of the plaintext using
                         * 					this instance's public key and padding type.
                         */
                        Smartmatic::System::SafeBuffer PublicEncrypt (
                            const Smartmatic::System::SafeBuffer & plaintext )
                        throw ( Smartmatic::System::Exception::RSAException );

                        /**
                         * @fn	Smartmatic::System::SafeBuffer RsaUtil::PrivateDecrypt ( const Smartmatic::System::SafeBuffer& ciphertext) throw (Smartmatic::System::Exception::RSAException);
                         * @brief	Private key decryption.
                         * @author	mcordovi
                         * @date	May 24, 2011
                         * @exception	Smartmatic::System::Exception::RSAException	Thrown when a rsa error condition occurs.
                         * @param	ciphertext	buffer containing the encrypted message.
                         * @return	a buffer containing the message recovered from the ciphertext. This instance's private
                         * 					key and padding are used for decryption computations. Padding must be set to the padding
                         * 					type used to encrypt the original message.
                         */
                        Smartmatic::System::SafeBuffer PrivateDecrypt (
                            const Smartmatic::System::SafeBuffer & ciphertext )
                        throw ( Smartmatic::System::Exception::RSAException );

                        /**
                         * @fn	Smartmatic::System::SafeBuffer RsaUtil::PrivateEncrypt ( const Smartmatic::System::SafeBuffer& plaintext) throw (Smartmatic::System::Exception::RSAException);
                         * @brief	Private key encryption.
                         * @author	mcordovi
                         * @date	May 24, 2011
                         * @exception	Smartmatic::System::Exception::RSAException	Thrown when a rsa error condition occurs.
                         * @param	plaintext	buffer containing the message to be encrypted.
                         * @return	a buffer containing the ciphertext obtained from the encryption of the plaintext using
                         * 					this instance's private key and padding type.
                         */
                        Smartmatic::System::SafeBuffer PrivateEncrypt (
                            const Smartmatic::System::SafeBuffer & plaintext )
                        throw ( Smartmatic::System::Exception::RSAException );

                        /**
                         * @fn	Smartmatic::System::SafeBuffer RsaUtil::PublicDecrypt ( const Smartmatic::System::SafeBuffer& ciphertext) throw (Smartmatic::System::Exception::RSAException);
                         * @brief	Public key decryption.
                         * @author	mcordovi
                         * @date	May 24, 2011
                         * @exception	Smartmatic::System::Exception::RSAException	Thrown when a rsa error condition occurs.
                         * @param	ciphertext	safe buffer containing the encrypted message.
                         * @return	a buffer containing the message recovered from the ciphertext. This instance's public
                         * 					key and padding are used for decryption computations. Padding must be set to the padding
                         * 					type used to encrypt the original message.
                         */
                        Smartmatic::System::SafeBuffer PublicDecrypt (
                            const Smartmatic::System::SafeBuffer & ciphertext )
                        throw ( Smartmatic::System::Exception::RSAException );

                        /**
                         * @fn	void RsaUtil::SignData ( const Smartmatic::System::SafeBuffer& inSafeBuffer,
                         * 			Smartmatic::System::SafeBuffer& outSafeBuffer ,
                         * 			MsgDigestUtil::DigestAlgType hashKind= MsgDigestUtil::DGSTUTIL_SHA256) throw (Smartmatic::System::Exception::SignedException);
                         * @brief	Signature calculation.
                         * @author	fgomez
                         * @date	Oct 27, 2011
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a rsa error condition occurs.
                         * @param [in,out]	inSafeBuffer 	reference to data.
                         * @param [in,out]	outSafeBuffer	output buffer.
                         * @param	hashKind							 	hash algorithm to use.
                         */
                        void SignData ( const Smartmatic::System::SafeBuffer & inSafeBuffer,
                                        Smartmatic::System::SafeBuffer & outSafeBuffer,
                                        MsgDigestUtil::DigestAlgType hashKind )
                        throw ( Smartmatic::System::Exception::SignedException );

                        /**
                         * @fn	bool RsaUtil::VerifySignature (
                         * 			const Smartmatic::System::SafeBuffer& inSafeBuffer,
                         * 			const Smartmatic::System::SafeBuffer& signature ,
                         * 			MsgDigestUtil::DigestAlgType hashKind= MsgDigestUtil::DGSTUTIL_SHA256) throw (Smartmatic::System::Exception::SignedException);
                         * @brief	Signature verification.
                         * @author	fgomez
                         * @date	Oct 27, 2011
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a rsa error condition occurs.
                         * @param [in,out]	inSafeBuffer	reference to data.
                         * @param [in,out]	signature			to verify.
                         * @param	hashKind								hash algorithm to use.
                         * 																@return: true if validation succeed. Otherwise, false.
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool VerifySignature (
                            const Smartmatic::System::SafeBuffer & inSafeBuffer,
                            const Smartmatic::System::SafeBuffer & signature,
                            MsgDigestUtil::DigestAlgType hashKind )
                        throw ( Smartmatic::System::Exception::SignedException );


                        /**
                         * @fn	RsaPaddingEnumType RsaUtil::GetPaddingType () const;
                         * @brief	Gets the current padding.
                         * @author	mcordovi
                         * @date	May 24, 2011
                         * @return	the current padding type to be used in cryptographic computations.
                         */
                        RsaPaddingEnumType GetPaddingType () const;

                        /**
                         * @fn	void RsaUtil::SetPaddingType (RsaPaddingEnumType pad);
                         * @brief	Sets the padding to be used for cryptographic computations.
                         * @author	mcordovi
                         * @date	May 24, 2011
                         * @param	pad	new padding.
                         */
                        void SetPaddingType ( RsaPaddingEnumType pad );

                        /**
                         * @fn	bool RsaUtil::IsValidPublicKey () const;
                         * @brief	Checks whether a public or private key is currently being used.
                         * @author	mcordovi
                         * @date	May 24, 2011
                         * @return	true if the key hold by this instance for use in cryptographic computations is a public
                         * 					key.
                         */
                        bool IsValidPublicKey () const;

                        /**
                         * @fn	bool RsaUtil::IsValidPrivateKey () const;
                         * @brief	Checks this instance's key for integrity.
                         * @author	mcordovi
                         * @date	May 24, 2011
                         * @return	false if the key pair or public key hold by this instance is invalid or has not been
                         * 					set yet (has a NULL value).
                         */
                        bool IsValidPrivateKey () const;

                    protected:


                    private:

                        static Smartmatic::Log::ISMTTLog * logger; ///< logger

                        RsaPaddingEnumType m_Padding; ///< padding type

                        /**
                         * @fn	bool RsaUtil::mCheckPadding (int buffLen, void* key);
                         * @brief	Checks for the buffer intended for encryption to be consistent with this instance's
                         * 				padding type and RSA combination.
                         * @author	mcordovi
                         * @date	May 25, 2011
                         * @param	buffLen	length of buffer intended for encryption.
                         * @param	key		 	RSA key.
                         * @return	false if buffer length is not consistent with the current padding and RSA (see
                         * 					http://www.openssl.org/docs/crypto/RSA_public_encrypt.html# or manual page (3) for
                         * 					RSA_public_encrypt).
                         */
                        bool mCheckPadding ( int buffLen, void * key );

                        /**
                         * @fn	int RsaUtil::mGetNativePadding ();
                         * @brief	method getter native padding.
                         * @author	mcordovi
                         * @date	May 25, 2011
                         * @return	integer value corresponding to the RSA paddign value of the wrapped library.
                         */
                        int mGetNativePadding ();
                };
            }
        }
    }
}

#endif /* RSAUTIL_H_ */
