/**
 * @file SignatureAlgorithm.h
 * @brief Header signature algorithm class
 * @date 10/11/2011
 * @author Freddy Gomez
 */

#ifndef SIGNATUREALGORITHM_H_
#define SIGNATUREALGORITHM_H_

#include <string>
#include <System/SafeBuffer.h>
#include <System/Exception/SignedException.hxx>
#include <System/Security/Cryptography/MsgDigestUtil.h>
#include <System/Security/Cryptography/CertUtil.h>
#include <System/Security/SecurityBase.hxx>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	SignatureAlgorithm
                 * @brief	Signature algorithm.
                 * @author	Youssef.tourkmani
                 * @date	24/10/2012
                 */
                class SignatureAlgorithm: public Smartmatic::System::Security::SecurityBase
                {
                    public:

                        /**
                         * @fn	SignatureAlgorithm::SignatureAlgorithm();
                         * @brief	Default constructor.
                         * @author	Youssef.tourkmani
                         * @date	24/10/2012
                         */
                        SignatureAlgorithm ();

                        /**
                         * @fn	virtual SignatureAlgorithm::~SignatureAlgorithm();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	24/10/2012
                         */
                        virtual ~SignatureAlgorithm();

                        /**
                         * @fn	virtual void SignatureAlgorithm::SignData ( const Smartmatic::System::SafeBuffer& inSafeBuffer,
                         * 			Smartmatic::System::SafeBuffer& outSafeBuffer ,
                         * 			MsgDigestUtil::DigestAlgType hashKind= MsgDigestUtil::DGSTUTIL_SHA256)
                         * 			throw (Smartmatic::System::Exception::SignedException) = 0;
                         * @brief	Signature calculation.
                         * @author	Freddy Gomez, Yadickson Soto
                         * @date	Nov 10, 2011
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a rsa error condition occurs.
                         * @param	inSafeBuffer 	reference to data.
                         * @param	[in,out]	outSafeBuffer	output buffer.
                         * @param	hashKind							 	hash algorithm to use.
                         */
                        virtual void SignData (
                            const Smartmatic::System::SafeBuffer & inSafeBuffer,
                            Smartmatic::System::SafeBuffer & outSafeBuffer ,
                            MsgDigestUtil::DigestAlgType hashKind )
                        throw ( Smartmatic::System::Exception::SignedException ) = 0;

                        /**
                         * @fn	virtual bool SignatureAlgorithm::VerifySignature (
                         *          const Smartmatic::System::SafeBuffer& inSafeBuffer,
                         * 			const Smartmatic::System::SafeBuffer& signature ,
                         * 			MsgDigestUtil::DigestAlgType hashKind= MsgDigestUtil::DGSTUTIL_SHA256)
                         * 			throw (Smartmatic::System::Exception::SignedException) = 0;
                         * @brief	Signature verification.
                         * @author	fgomez
                         * @date	Nov 10, 2011
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a rsa error condition occurs.
                         * @param	inSafeBuffer	reference to data.
                         * @param	signature to verify.
                         * @param	hashKind hash algorithm to use.
                         * @return	true if it succeeds, false if it fails.
                         */
                        virtual bool VerifySignature (
                            const Smartmatic::System::SafeBuffer & inSafeBuffer,
                            const Smartmatic::System::SafeBuffer & signature,
                            MsgDigestUtil::DigestAlgType hashKind )
                        throw ( Smartmatic::System::Exception::SignedException ) = 0;

                    protected:

                        /**
                         * @fn	EVP_PKEY * SignatureAlgorithm::GetPublicKey() const;
                         * @brief	Gets public key.
                         * @author	Youssef.tourkmani
                         * @date	24/10/2012
                         * @return	null if it fails, else the public key.
                         */
                        EVP_PKEY * GetPublicKey() const;

                        /**
                         * @fn	EVP_PKEY * SignatureAlgorithm::GetPrivateKey() const;
                         * @brief	Gets private key.
                         * @author	Youssef.tourkmani
                         * @date	24/10/2012
                         * @return	null if it fails, else the private key.
                         */
                        EVP_PKEY * GetPrivateKey() const;
                };
            } /* namespace Cryptography */
        } /* namespace Security */
    } /* namespace System */
} /* namespace Smartmatic */
#endif /* SIGNATUREALGORITHM_H_ */
