/**
 * @file XmlSignature.hxx
 * @brief Header class for XML signatures.
 * @author	Eduardo Fernández
 * @date	10/10/2018
 */

#ifndef XML_SIGNATURE_HXX
#define XML_SIGNATURE_HXX

#include <System/Security/XmlDigitalSignature/XmlSignatureAlgorithm.hxx>

#include <memory>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            /**
             * @class	XmlSignature
             * @brief	Class for XML signatures.
             * @author	Eduardo Fernández
             * @date	10/10/2018
             */
            class XmlSignature
            {
                public:

                    /**
                     * @fn	virtual ~XmlSignature();
                     * @brief	Class destructor.
                     * @author	Eduardo Fernández
                     * @date	10/10/2018
                     */
                    virtual ~XmlSignature();

                public:

                    /**
                     * @fn void sign(
                               const Smartmatic::System::SafeBuffer& bufferToSign,
                               Smartmatic::System::SafeBuffer& signature,
                               const Signed::SignedType& signatureType,
                               Smartmatic::System::Security::Cryptography::CertificateExtensionBase * const certificate = NULL,
                               const hashMethod& hash = HASH_SHA256,
                               const std::string& basePath = "");
                     * @brief Signs buffer.
                     * @author Eduardo Fernández
                     * @date   10/10/2018
                     * @param  bufferToSign buffer to sign.
                     * @param  basePath base path of the file to sign.
                     * @param  signature signature buffer.
                     * @param  signatureType signature type.
                     * @param  certificate complete certificate information.
                     * @param  hash hash method.
                     */
                    bool sign(
                        const Smartmatic::System::SafeBuffer& bufferToSign,
                        Smartmatic::System::SafeBuffer& signature,
                        const Signed::SignedType& signatureType,
                        Smartmatic::System::Security::Cryptography::CertificateExtensionBase * const certificate = NULL,
                        const hashMethod& hash = HASH_SHA256,
                        const std::string& basePath = "");

                    /**
                     * @fn bool verify(
                               const Smartmatic::System::SafeBuffer& bufferToVerify,
                               X509 * const certificate = NULL,
                               const std::string& basePath = "");
                     * @brief Verifies a signature.
                     * @author Eduardo Fernández
                     * @date   10/10/2018
                     * @param  bufferToVerify Buffer including XML signature.
                     * @param  certificate X509 certificate.
                     * @param  basePath reference information.
                     * @return true if the verification is successful.
                     */
                    bool verify(
                        const Smartmatic::System::SafeBuffer& bufferToVerify,
                        X509 * const certificate = NULL,
                        const std::string& basePath = "");

                protected:

                    /**
                     * @fn XmlSignature();
                     * @brief Class constructor.
                     * @author Eduardo Fernández
                     * @date   10/10/2018
                     */
                    XmlSignature();

                protected:

                    std::auto_ptr<XmlSignatureAlgorithm> m_xmlSignatureAlgorithm;
            };
        }
    }
}

#endif /* XML_SIGNATURE_HXX */
