/**
 * @file StringUtils.cxx
 * @brief Body class string utils
 */

#include "StringUtils.hxx"
#include <algorithm>
#include <iomanip>
#include <stdio.h>
#include <regex.h>
#include <sstream>
#include <glibmm/ustring.h>

using namespace Smartmatic::System::Utils;

std::string StringUtils::remove ( const std::string & input, const std::string & find )
{
    return replace ( input, find, "" );
}

std::string StringUtils::removeAll ( const std::string & input, const std::string & find )
{
    return replaceAll ( input, find, "" );
}

std::string StringUtils::replace ( const std::string & input, const std::string & find,
                                   const std::string & dest )
{
    std::string result = input;

    if ( !find.empty() )
    {
        std::string::size_type pos;
        pos = result.find ( find );

        if ( pos != std::string::npos )
        {
            result.replace ( pos, find.length(), dest );
        }
    }

    return result;
}

std::string StringUtils::replaceLast ( const std::string & input, const std::string & find,
                                       const std::string & dest )
{
    std::string result = input;

    if ( !find.empty() )
    {
        std::string::size_type pos;
        pos = result.rfind ( find );

        if ( pos != std::string::npos )
        {
            result.replace ( pos, find.length(), dest );
        }
    }

    return result;
}

std::string StringUtils::replaceAll ( const std::string & input, const std::string & find,
                                      const std::string & dest )
{
    std::string result;

    if ( !find.empty() )
    {
        std::string::const_iterator substart = input.begin(), subend;

        while ( true )
        {
            subend = search ( substart, input.end(), find.begin(), find.end() );
            std::string temp ( substart, subend );

            result.append ( temp );

            if ( subend == input.end() )
            {
                break;
            }

            if ( find.size() == 0 )
            {
                break;
            }

            result.append ( dest );
            substart = subend + find.size();
        }
    }
    else
    {
        result = input;
    }

    return result;
}

void StringUtils::splitToList ( const std::string & input, const std::string & find, std::list<std::string> & list )
{
    list.clear();

    if ( input.empty() || find.empty() )
    {
        list.push_back ( input );
        return;
    }

    std::string::const_iterator substart = input.begin(), subend;

    while ( true )
    {
        subend = search ( substart, input.end(), find.begin(), find.end() );
        std::string temp ( substart, subend );

        list.push_back ( temp );

        if ( subend == input.end() )
        {
            break;
        }

        if ( find.size() == 0 )
        {
            break;
        }

        substart = subend + find.size();
    }
}

void StringUtils::splitToVector ( const std::string & input, const std::string & find, std::vector <std::string> & vector )
{
    vector.clear();
    std::list<std::string> result;
    splitToList ( input, find, result );
    vector.insert ( vector.end(), result.begin(), result.end() );
}

std::string StringUtils::arrayHexToString (
    const std::vector <unsigned char> & input,
    const std::string & prefix,
    const std::string & separator )
{
    std::stringstream ss;
    int group = 2;

    for ( size_t index = 0; index < input.size(); )
    {
        ss << toHexString ( input[index], prefix, group );

        if ( ++index < input.size() )
        {
            ss << separator;
        }
    }

    return ss.str();
}

std::string StringUtils::arrayHexToAsciiString ( const std::vector <unsigned char> & input )
{
    std::stringstream ss;

    for ( size_t index = 0; index < input.size(); ++index )
    {
        ss << ( char ) input[index];
    }

    return ss.str();
}

void StringUtils::asciiStringToArrayHex (
    const std::string & input,
    std::vector <unsigned char> & output )
{
    std::stringstream ss;

    for ( size_t index = 0; index < input.size(); ++index )
    {
        ss << toHexString ( input[index], "", 2 );
    }

    stringToArrayHex ( ss.str(), output );
}

void StringUtils::stringToArrayHex (
    const std::string & input,
    std::vector <unsigned char> & output,
    const std::string & prefix,
    const std::string & separator )
{
    std::string tmp;
    unsigned int number;
    std::string str;

    tmp = replaceAll ( input, prefix, "" );
    tmp = replaceAll ( tmp, separator, "" );

    output.clear();

    for ( unsigned int i = 0; i < tmp.length(); i += 2 )
    {
        str = tmp.substr ( i, 2 );

        if ( sscanf ( str.c_str(), "%02x", &number ) > 0 )
        {
            output.push_back ( number );
        }
    }
}

std::string StringUtils::bufferHexToString (
    const Smartmatic::System::SafeBuffer & input,
    const std::string & prefix,
    const std::string & separator )
{
    std::vector <unsigned char> data;
    std::string result;

    if ( input.GetData() != NULL )
    {
        data.insert ( data.end(), input.GetData(), input.GetData() + input.GetSize() );
        result = arrayHexToString ( data, prefix, separator );
    }

    return result;
}

void StringUtils::stringToBufferHex (
    const std::string & input,
    Smartmatic::System::SafeBuffer & output,
    const std::string & prefix,
    const std::string & separator )
{
    std::vector <unsigned char> dataOut;
    stringToArrayHex ( input, dataOut, prefix, separator );

    if ( !dataOut.empty() )
    {
        output.AddData ( ( const unsigned char * ) &dataOut[0], dataOut.size() );
    }
}

bool StringUtils::isFound ( const std::string & text, char character )
{
    bool f = false;

    if ( !text.empty() )
    {
        std::string::size_type pos;
        pos = text.find ( character );
        f = pos != std::string::npos;
    }

    return f;
}

bool StringUtils::regex ( const std::string & re, const std::string & input )
{
    bool f = false;
    regex_t regex;
    int reti;
    reti = regcomp ( &regex, re.c_str(), REG_EXTENDED | REG_NOSUB );

    if ( reti )
    {
        return f;
    }

    reti = regexec ( &regex, input.c_str(), 0, NULL, 0 );
    f = !reti;
    regfree ( &regex );

    return f;
}

std::string StringUtils::trim ( const std::string & input )
{
    std::string::size_type pos = input.find_last_not_of ( ' ' );
    std::string tmp = input;

    if ( pos != std::string::npos )
    {
        tmp.erase ( pos + 1 );
        pos = input.find_first_not_of ( ' ' );

        if ( pos != std::string::npos )
        {
            tmp.erase ( 0, pos );
        }
    }
    else
    {
        tmp.erase ( tmp.begin(), tmp.end() );
    }

    return tmp;
}

int StringUtils::stringToInt ( const std::string & str )
{
    int value = 0;
    std::istringstream iss ( str );
    iss >> std::dec >> value;
    return value;
}

long StringUtils::stringToLong ( const std::string & str )
{
    long value = 0;
    std::istringstream iss ( str );
    iss >> std::dec >> value;
    return value;
}

long StringUtils::stringToDouble ( const std::string & str )
{
    double value = 0.0;
    std::istringstream iss ( str );
    iss >> std::dec >> value;
    return value;
}

std::string StringUtils::intToString ( int value, unsigned int group )
{
    std::stringstream iss;
    iss << std::setw ( group ) << std::setfill ( '0' ) << value;
    return iss.str();
}

std::string StringUtils::longToString ( long long value, unsigned int group )
{
    std::stringstream iss;
    iss << std::setw ( group ) << std::setfill ( '0' ) << value;
    return iss.str();
}

std::string StringUtils::toHexString ( unsigned long long value, const std::string & prefix, unsigned int group )
{
    std::stringstream iss;
    iss << prefix;
    iss << std::hex << std::setw ( group ) << std::setfill ( '0' ) << value;
    return iss.str();
}

std::string StringUtils::toUpper ( const std::string & text )
{
    std::string tmp = text;
    std::transform ( tmp.begin(), tmp.end(), tmp.begin(), ::toupper );
    return tmp;
}

std::string StringUtils::toLower ( const std::string & text )
{
    std::string tmp = text;
    std::transform ( tmp.begin(), tmp.end(), tmp.begin(), ::tolower );
    return tmp;
}

std::string StringUtils::convertEndLine ( const std::string & text )
{
    return replaceAll ( text, "\\n", "\n" );
}

std::string StringUtils::capitalizeUtf8(const std::string& text)
{
    std::string result;
    std::vector<std::string> words;
    std::vector<std::string>::iterator it;

    splitToVector(text, " ", words);

    for (it = words.begin(); it != words.end(); ++it)
    {
        Glib::ustring lowerCase = Glib::ustring(*it).lowercase();

        if (!lowerCase.empty())
        {
            lowerCase.replace(lowerCase.begin(), ++lowerCase.begin(), 1, Glib::Unicode::toupper(lowerCase[0]));
            result += lowerCase + " ";
        }
    }

    result = trim(result);

    return result;
}

std::string StringUtils::toLowerUtf8(const std::string& text)
{
    return Glib::ustring(text).lowercase();
}
