/**
 * @file StringUtils.hxx
 * @brief Header class string utils
 */

#ifndef __STRING_UTILS__
#define __STRING_UTILS__

#include <string>
#include <list>
#include <vector>
#include <glibmm.h>
#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace System
    {
        namespace Utils
        {
            /**
             * @class	StringUtils
             * @brief	Class utils for string manager.
             * @author	Yadickson Soto yadickson.soto@smartmatic.com
             * @date	13-05-2011
             */
            class StringUtils
            {

                public:
                    /**
                     * @fn	static std::string StringUtils::remove(const std::string & input, const std::string & find);
                     * @brief	Method remove first element found.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13-05-2011
                     * @param	input	string orig.
                     * @param	find 	string find.
                     * @return	string result.
                     */
                    static std::string remove ( const std::string & input, const std::string & find );
                    /**
                     * @fn	static std::string StringUtils::removeAll(const std::string & input, const std::string & find);
                     * @brief	Method remove all element found.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13-05-2011
                     * @param	input	string orig.
                     * @param	find 	string find.
                     * @return	string result.
                     */
                    static std::string removeAll ( const std::string & input, const std::string & find );
                    /**
                     * @fn	static std::string StringUtils::replace(const std::string & input, const std::string & find,
                     * 			const std::string & dest);
                     * @brief	Method replace first element found.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13-05-2011
                     * @param	input	string orig.
                     * @param	find 	string find.
                     * @param	dest 	element to replace.
                     * @return	string result.
                     */
                    static std::string replace ( const std::string & input, const std::string & find, const std::string & dest );
                    /**
                     * @fn	static std::string StringUtils::replaceLast(const std::string & input, const std::string & find,
                     * 			const std::string & dest);
                     * @brief	Method replace last element found.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13-06-2013
                     * @param	input	string orig.
                     * @param	find 	string find.
                     * @param	dest 	element to replace.
                     * @return	string result.
                     */
                    static std::string replaceLast ( const std::string & input, const std::string & find, const std::string & dest );

                    /**
                     * @fn	static std::string StringUtils::replaceAll(const std::string & input, const std::string & find,
                     * 			const std::string & dest);
                     * @brief	Method replace all element found.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13-05-2011
                     * @param	input	string orig.
                     * @param	find 	string find.
                     * @param	dest 	element to replace.
                     * @return	string result.
                     */
                    static std::string replaceAll ( const std::string & input, const std::string & find, const std::string & dest );
                    /**
                     * @fn	static void splitToList ( const std::string & input, const std::string & find, std::list <std::string> & list );
                     * @brief	Method split string to list object.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13-05-2011
                     * @param	input	string orig.
                     * @param	find 	string find.
                     * @param	list	all element of split.
                     */
                    static void splitToList ( const std::string & input, const std::string & find, std::list <std::string> & list );
                    /**
                     * @fn	static void splitToVector ( const std::string & input, const std::string & find, std::vector <std::string> & vector );
                     * @brief	Method split string to vector object.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13-05-2011
                     * @param	input 	string orig.
                     * @param	find		string find.
                     * @param	vector	result element of split.
                     */
                    static void splitToVector ( const std::string & input, const std::string & find, std::vector <std::string> & vector );
                    /**
                     * @fn	static std::string StringUtils::arrayHexToString( const std::vector <unsigned char> & input,
                     * 			std::string prefix = "", std::string separator = "");
                     * @brief	Method convert array hex to string.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	18-08-2011
                     * @param	input		 	array.
                     * @param	prefix	 	string prefix.
                     * @param	separator	string separator.
                     * @return	string result.
                     */
                    static std::string arrayHexToString (
                        const std::vector <unsigned char> & input,
                        const std::string & prefix = "",
                        const std::string & separator = "" );
                    /**
                     * @fn	static std::string StringUtils::arrayHexToAsciiString( const std::vector <unsigned char> & input);
                     * @brief	Method convert array hex to string.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	13-09-2012
                     * @param	input	array.
                     * @return	string result.
                     */
                    static std::string arrayHexToAsciiString (
                        const std::vector <unsigned char> & input );
                    /**
                     * @fn	static void StringUtils::asciiStringToArrayHex( const std::string & input,
                     * 			std::vector <unsigned char> & output);
                     * @brief	Method convert ascii string to array hex.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	18-08-2011
                     * @param	input 	array.
                     * @param	output	vector result.
                     */
                    static void asciiStringToArrayHex (
                        const std::string & input,
                        std::vector <unsigned char> & output );
                    /**
                     * @fn	static void StringUtils::stringToArrayHex( const std::string & input,
                     * 			std::vector <unsigned char> & output, const std::string & prefix = "", const std::string & separator = "");
                     * @brief	Method convert string to array hex.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	18-08-2011
                     * @param	input		 	array.
                     * @param	output	 	vector result.
                     * @param	prefix	 	string prefix.
                     * @param	separator	string separator.
                     */
                    static void stringToArrayHex (
                        const std::string & input,
                        std::vector <unsigned char> & output,
                        const std::string & prefix = "",
                        const std::string & separator = "" );
                    /**
                     * @fn	static std::string StringUtils::bufferHexToString( const Smartmatic::System::SafeBuffer & input,
                     * 			const std::string & prefix = "", const std::string & separator = "");
                     * @brief	Method convert array hex to string.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	18-08-2011
                     * @param	input		 	array.
                     * @param	prefix	 	string prefix.
                     * @param	separator	string separator.
                     * @return	string result.
                     */
                    static std::string bufferHexToString (
                        const Smartmatic::System::SafeBuffer & input,
                        const std::string & prefix = "",
                        const std::string & separator = "" );
                    /**
                     * @fn	static void StringUtils::stringToBufferHex( const std::string & input,
                     * 			Smartmatic::System::SafeBuffer & output, const std::string & prefix = "", const std::string & separator = "");
                     * @brief	Method convert string to array hex.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	18-08-2011
                     * @param	input		 	array.
                     * @param	output	 	buffer result.
                     * @param	prefix	 	string prefix.
                     * @param	separator	string separator.
                     */
                    static void stringToBufferHex (
                        const std::string & input,
                        Smartmatic::System::SafeBuffer & output,
                        const std::string & prefix = "",
                        const std::string & separator = "" );

                    /**
                     * @fn	static bool StringUtils::isFound(const std::string & text, char character);
                     * @brief	Method is found character in string.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	02/09/2013
                     * @param	text		 	input string.
                     * @param	character	character to find.
                     * @return	result.
                     */
                    static bool isFound ( const std::string & text, char character );
                    /**
                     * @fn	static bool StringUtils::regex( const std::string & re, const std::string & input );
                     * @brief	Method use regex for match.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	02/09/2013
                     * @param	re	regular expression to match.
                     * @param	input	input to match.
                     * @return	result.
                     */
                    static bool regex ( const std::string & re, const std::string & input );

                    /**
                     * @fn	static std::string StringUtils::trim(const std::string & input);
                     * @brief	Method trim to string.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	18-08-2011
                     * @param	input	input string.
                     * @return	result.
                     */
                    static std::string trim ( const std::string & input );

                    /**
                     * @fn	static int StringUtils::stringToInt(const std::string & str);
                     * @brief	Method string to integer.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	03-05-2012
                     * @param	str	input string.
                     * @return	result.
                     */
                    static int stringToInt ( const std::string & str );

                    /**
                     * @fn	static long StringUtils::stringToLong(const std::string & str);
                     * @brief	Method string to long.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	02/09/2013
                     * @param	str	input string.
                     * @return	result.
                     */
                    static long stringToLong ( const std::string & str );

                    /**
                     * @fn	static long StringUtils::stringToDouble(const std::string & str);
                     * @brief	Method string to double.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	02/09/2013
                     * @param	str	input string.
                     * @return	result.
                     */
                    static long stringToDouble ( const std::string & str );

                    /**
                     * @fn	static std::string StringUtils::intToString(int value, unsigned int group = 1);
                     * @brief	Method integer to string.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	03-05-2012
                     * @param	value	input value.
                     * @param	group	group number
                     * @return	result.
                     */
                    static std::string intToString ( int value, unsigned int group = 1 );

                    /**
                     * @fn	static std::string StringUtils::longToString(long long value, unsigned int group = 1);
                     * @brief	Method long long to string.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	03-05-2012
                     * @param	value	input value.
                     * @param	group	group number
                     * @return	result.
                     */
                    static std::string longToString ( long long value, unsigned int group = 1 );

                    /**
                     * @fn	static std::string toHexString ( unsigned long long value, const std::string & prefix = "", unsigned int group = 8 );
                     * @brief	Method conver to hexadecimal string.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	10-01-2014
                     * @param	value	input value.
                     * @prefix	prefix	result prefix value
                     * @param	group	result group number
                     * @return	hexadecimal result.
                     */
                    static std::string toHexString ( unsigned long long value, const std::string & prefix = "", unsigned int group = 8 );
                    /**
                     * @fn	static std::string StringUtils::toUpper(const std::string & text);
                     * @brief	Method to upper.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	21-09-2012
                     * @param	text	text to upper.
                     * @return	to upper string.
                     */
                    static std::string toUpper ( const std::string & text );
                    /**
                     * @fn	static std::string StringUtils::toLower(const std::string & text);
                     * @brief	Method to lower.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	21-09-2012
                     * @param	text	text to lower.
                     * @return	to lower string.
                     */
                    static std::string toLower ( const std::string & text );
                    /**
                     * @fn	static std::string convertEndLine ( const std::string & text );
                     * @brief	Method to convert all \\n to end line.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	25-10-2013
                     * @param	text	text to lower.
                     * @return	new text.
                     */
                    static std::string convertEndLine ( const std::string & text );

                    /**
                     * @brief  Capitalize the first letter of each word, utf-8 aware.
                     * @author manuel.arguelles
                     * @date   Feb 27, 2018
                     * @param  text The input text to capitalize in utf-8.
                     * @return A string with the first letter of each word capitalized.
                     */
                    static std::string capitalizeUtf8(const std::string& text);

                    /**
                     * @brief  Method to convert string to lower case.
                     * @author manuel.arguelles
                     * @date   sep 26, 2018
                     * @param  text The text to convert.
                     * @return A lower cased text.
                     */
                    static std::string toLowerUtf8(const std::string& text);

                    /**
                     * @brief  Builds a composed message.
                     * @author eduardo.fernandez
                     * @date   Mar 26, 2018
                     * @param  text The base text message.
                     * @return A composed message
                     */
                    template <class T1>
                    inline // static
                    static std::string compose(const Glib::ustring& text, const T1& arg1)
                    {
                        return Glib::ustring::compose(text, arg1);
                    }

                    /**
                     * @brief  Builds a composed message.
                     * @author eduardo.fernandez
                     * @date   Mar 26, 2018
                     * @param  text The base text message.
                     * @return A composed message
                     */
                    template <class T1, class T2>
                    inline // static
                    static std::string compose(const Glib::ustring& text, const T1& arg1, const T2& arg2)
                    {
                        return Glib::ustring::compose(text, arg1, arg2);
                    }

                    /**
                     * @brief  Builds a composed message.
                     * @author eduardo.fernandez
                     * @date   Mar 26, 2018
                     * @param  text The base text message.
                     * @return A composed message
                     */
                    template <class T1, class T2, class T3>
                    inline // static
                    static std::string compose(const Glib::ustring& text,
                        const T1& arg1, const T2& arg2, const T3& arg3)
                    {
                        return Glib::ustring::compose(text, arg1, arg2, arg3);
                    }

                    /**
                     * @brief  Builds a composed message.
                     * @author eduardo.fernandez
                     * @date   Mar 26, 2018
                     * @param  text The base text message.
                     * @return A composed message
                     */
                    template <class T1, class T2, class T3, class T4>
                    inline // static
                    static std::string compose(const Glib::ustring& text,
                        const T1& arg1, const T2& arg2, const T3& arg3, const T4& arg4)
                    {
                        return Glib::ustring::compose(text, arg1, arg2, arg3, arg4);
                    }

                    /**
                     * @brief  Builds a composed message.
                     * @author eduardo.fernandez
                     * @date   Mar 26, 2018
                     * @param  text The base text message.
                     * @return A composed message
                     */
                    template <class T1, class T2, class T3, class T4, class T5>
                    inline // static
                    static std::string compose(const Glib::ustring& text,
                        const T1& arg1, const T2& arg2, const T3& arg3, const T4& arg4, const T5& arg5)
                    {
                        return Glib::ustring::compose(text, arg1, arg2, arg3, arg4, arg5);
                    }

                    /**
                     * @brief  Builds a composed message.
                     * @author eduardo.fernandez
                     * @date   Mar 26, 2018
                     * @param  text The base text message.
                     * @return A composed message
                     */
                    template <class T1, class T2, class T3, class T4, class T5, class T6>
                    inline // static
                    static std::string compose(const Glib::ustring& text,
                        const T1& arg1, const T2& arg2, const T3& arg3, const T4& arg4,
                        const T5& arg5, const T6& arg6)
                    {
                        return Glib::ustring::compose(text, arg1, arg2, arg3, arg4, arg5, arg6);
                    }

                    /**
                     * @brief  Builds a composed message.
                     * @author eduardo.fernandez
                     * @date   Mar 26, 2018
                     * @param  text The base text message.
                     * @return A composed message
                     */
                    template <class T1, class T2, class T3, class T4, class T5, class T6, class T7>
                    inline // static
                    static std::string compose(const Glib::ustring& text,
                        const T1& arg1, const T2& arg2, const T3& arg3, const T4& arg4,
                        const T5& arg5, const T6& arg6, const T7& arg7)
                    {
                        return Glib::ustring::compose(text, arg1, arg2, arg3, arg4, arg5, arg6, arg7);
                    }

                    /**
                     * @brief  Builds a composed message.
                     * @author eduardo.fernandez
                     * @date   Mar 26, 2018
                     * @param  text The base text message.
                     * @return A composed message
                     */
                    template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8>
                    inline // static
                    static std::string compose(const Glib::ustring& text,
                        const T1& arg1, const T2& arg2, const T3& arg3, const T4& arg4,
                        const T5& arg5, const T6& arg6, const T7& arg7, const T8& arg8)
                    {
                        return Glib::ustring::compose(text, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
                    }

                    /**
                     * @brief  Builds a composed message.
                     * @author eduardo.fernandez
                     * @date   Mar 26, 2018
                     * @param  text The base text message.
                     * @return A composed message
                     */
                    template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9>
                    inline // static
                    static std::string compose(const Glib::ustring& text,
                        const T1& arg1, const T2& arg2, const T3& arg3, const T4& arg4,
                        const T5& arg5, const T6& arg6, const T7& arg7, const T8& arg8,
                        const T9& arg9)
                    {
                        return Glib::ustring::compose(text, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);
                    }
            };
        }
    }
}

#endif
