/**
 * @file   CMSSignerTest.h
 * @brief  Declaration of tests for CMS signatures.
 * @author manuel.arguelles
 * @date   Jul 26, 2017
 */

#ifndef TEST_CMSSIGNERTEST
#define TEST_CMSSIGNERTEST

#ifdef ECUT_MAIN

#include <System/Encoding/Base64Encoder.h>
#include <System/IO/File.h>
#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/CMSSigner.hxx>
#include <System/Security/OpenSSLSecurity.hxx>
#include <cppunit/extensions/HelperMacros.h>
#include <tests/StoreMockFile.hxx>

using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System;

static const std::string certificatePkcs12 = \
    "MIIFtgIBAzCCBXAGCSqGSIb3DQEHAaCCBWEEggVdMIIFWTCCAaYGCSqGSIb3DQEHAaCCAZcEggGT"
    "MIIBjzCCAYsGCyqGSIb3DQEMCgECoIIBMjCCAS4wKAYKKoZIhvcNAQwBAzAaBBT3hx/JLAHFSFRE"
    "zfS/wS55m8cR3wICBAAEggEARe7Zm4KtWkDP92UcCV9VdNRtSEHABMgxQ/0tNHO+gI+efhtpXJN1"
    "ze4AKS+D+3JWYIiRykuHWxRxrJjLNZ+JAzqCx3dSkpAuVYmBYyMw/omnBChr/WDpt1hyh840h8tv"
    "mTOXxFL0bidqHS2X8FY20Qgz8UkL/DHHRZbVPKo+hqez7Qo59KTzU0xQonAJBbXl+r8V9b+9uWP2"
    "UJ/sBpuIEyA6ifMCocQmI0Za0Z34ZAmlOpUPs/ZkcVoGaicblBuogGwNBUpsSrCT3+BHr+uK0zM4"
    "W/ly47BBar+oMUNC5SPtnOFsZQwxl2/pwZ/lVXxVLAsNJY319LxvJqW1pS3ObDFGMB8GCSqGSIb3"
    "DQEJFDESHhAAYQBuAHkAQQBsAGkAYQBzMCMGCSqGSIb3DQEJFTEWBBQ+BNOR/4EGCTH/CW80zAIH"
    "BsqI/zCCA6sGCSqGSIb3DQEHBqCCA5wwggOYAgEAMIIDkQYJKoZIhvcNAQcBMCgGCiqGSIb3DQEM"
    "AQYwGgQUP1aCnx4+t10r85ntQin81t8IoVMCAgQAgIIDWHG1zHyRK1piFiMNPGiwSZoaVBEDqIon"
    "VLqM57TW0jlqkEuCSjWGMI7fYOwkvMn3/e7ad1Grfy0D+3URksG0nyfeq2aRHm24zLolFtS1vB1J"
    "/mOEuOMeaGx3p8ctegsZRObzeiAbZ6ZEokRpbg1Qb2mtBPLm2IyjKBktZa9bP/xnBjqXfjaYO5GY"
    "s/myheXw2ej+I8Kw6ijuGqA4dE/1JIvFSTy3XoLcAeJe0fXHc9NL8xBbw3062VYrMCmnRC3Qat6f"
    "ZHDMlnY5UdOmMGHlwBDtcnJiO+NUWcrOJSWuVk0qdolCky7Oqc1aIoYbMNe+6aOOPYumAbkkL+nB"
    "xPno0rraOPYX6xu/YUJJa1Fortg0Le2oB2N4oWz/fhhzrtMiHSsHOD9il18SCdV1X0CzYhwaN/mB"
    "69hiTvBia6SL0xO3tIjMyQB4w0ypa5QoHMYKAjc/r2kQza6s77N2TU/KSZzBn31YTGL1jBOHs5Rj"
    "maj3YxXLYcdTsBx+TG4nVZPQ5G7JdylJWMy7jtgJfqFPvBJmwmbh1b4j7jPeMok5i3403+0o9T+n"
    "Z7ORtup/GADYDljxAunJLXEaOs8bQtkQElf6PT3/T3UolDeHqUGNMD3dZLKXRIlzQMYZ9RSWg3vm"
    "q2HdnM0PfflinqokYfz9cHZyWKXwJ+2AqGE2BR5C3TfnHQnYGAARmvCUhG/dJtlf8DN46p2YcpbO"
    "8h07BPUQ6cteazNGr2bKS0CwZ5Hi6bcYx0K7zOfXsjWknjry2iqc44kYih8keMOmJ2Z8aLUx42hl"
    "4IACL37Urch4jR4+CrrnFylduoz6HvOCxdLHcbVvymMToaizlL5tDU6zQVt5i2rjly29DHnYtu9K"
    "tGPfjExmXn1EG13NyVn0ZwADic9JsNlSzAnIoBF1KfCo9VMWpBWzUSgAv1DiJkedf94/uTf92q6p"
    "saGKngD97TFFsfE5gjjrTMR25AqN/0JGlOUG1/t+j8yt98G0sPaNokGXEim6sZyHRd72kLyoSFu/"
    "HAbRohT3H1MVM9ETT4+aPlNm3yXny74OQIHz9LynQqraFcSUBNkEU5dD63/HBfYhrAO6sbPjSfsV"
    "CAswbW2gXJOx9+6ZdHASds7GpYumdbbCPRlm9kVpOvF0h9wwPTAhMAkGBSsOAwIaBQAEFPqw6frp"
    "ESC3dyt0JCFx+xZJojsnBBS+d2pPC5If7Yzma8nqACdcLOXxcwICBAA=";

static const std::string certificatePem = \
    "Bag Attributes\n"
    "    friendlyName: anyAlias\n"
    "    localKeyID: 3E 04 D3 91 FF 81 06 09 31 FF 09 6F 34 CC 02 07 06 CA 88 FF\n"
    "Key Attributes: <No Attributes>\n"
    "-----BEGIN ENCRYPTED PRIVATE KEY-----\n"
    "MIIBPTBABgkqhkiG9w0BBQ0wMzAbBgkqhkiG9w0BBQwwDgQI8xxuGLgQJxkCAggA\n"
    "MBQGCCqGSIb3DQMHBAgQBwzbvXZeFQSB+DjYHcsHnqlU/ySYxuQl/YyH+Ab1W2cZ\n"
    "IUb/RrwG43zKKzSApFzPA7uKB2tYKRFhZihZ3KFKE4QB3a2x3Plm+giz0qjfr8E9\n"
    "fjOGDN77awn/FJHFttNGVbKxV69P9jLHdv/gJEu4L6yKFlP1U/khVMSEmnBGbSQ6\n"
    "92Fy8VDrhk8qJOAOSyz9lBf90h01J1qxaH73ms/XpNWSQTr2MQ9OjBkG6rc64Owf\n"
    "PFCv1qsTXSk1EozbeaQmBCgMAqeJ/wBmmzRQVHG0RKHFno6ChalGiAQTJACa6Vws\n"
    "Wbr0GJdSG62H1jVU/AWdlj6cpG7g6wEZwgTHiqHFqplz\n"
    "-----END ENCRYPTED PRIVATE KEY-----\n"
    "Bag Attributes\n"
    "    friendlyName: anyAlias\n"
    "    localKeyID: 3E 04 D3 91 FF 81 06 09 31 FF 09 6F 34 CC 02 07 06 CA 88 FF\n"
    "subject=/CN=SAINT-GILLES_017_21013/OU=SAES/O=Smartmatic/L=Brussels/ST=Brussels/C=BE\n"
    "issuer=/C=BE/ST=Brussels/L=Brussels/O=Smartmatic/OU=SAES/CN=intermediate_ca\n"
    "-----BEGIN CERTIFICATE-----\n"
    "MIIC1DCCAnqgAwIBAgICBUEwCgYIKoZIzj0EAwIwcTELMAkGA1UEBhMCQkUxETAP\n"
    "BgNVBAgTCEJydXNzZWxzMREwDwYDVQQHEwhCcnVzc2VsczETMBEGA1UEChMKU21h\n"
    "cnRtYXRpYzENMAsGA1UECxMEU0FFUzEYMBYGA1UEAxQPaW50ZXJtZWRpYXRlX2Nh\n"
    "MB4XDTE3MDYyMDIwMzgyOFoXDTE3MTIzMTAwMDAwMFoweDEfMB0GA1UEAwwWU0FJ\n"
    "TlQtR0lMTEVTXzAxN18yMTAxMzENMAsGA1UECwwEU0FFUzETMBEGA1UECgwKU21h\n"
    "cnRtYXRpYzERMA8GA1UEBwwIQnJ1c3NlbHMxETAPBgNVBAgMCEJydXNzZWxzMQsw\n"
    "CQYDVQQGEwJCRTCBmzAQBgcqhkjOPQIBBgUrgQQAIwOBhgAEAQhYZ61siKN94QmV\n"
    "Jg3hVa/UffvsLrq9taodRdSwN92sw5KR0JB++dXoew01wGGg5QOAYokoRBmBLP72\n"
    "egIyRKU1ALdu113E2zdfyVd5j9g70cdnNk5ee00zsP+3GUNtSoZk3SdQninKXngQ\n"
    "t3oAkj8RmUE1b9n0BwZ+fYtG+rQJNwoYo4G3MIG0MIGTBgNVHSMEgYswgYiAFNgO\n"
    "I9lmen9JDkZc31DDa3Sgua+boW2kazBpMQswCQYDVQQGEwJCRTERMA8GA1UECBMI\n"
    "QnJ1c3NlbHMxETAPBgNVBAcTCEJydXNzZWxzMRMwEQYDVQQKEwpTbWFydG1hdGlj\n"
    "MQ0wCwYDVQQLEwRTQUVTMRAwDgYDVQQDFAdyb290X2NhggEBMAwGA1UdEwEB/wQC\n"
    "MAAwDgYDVR0PAQH/BAQDAgWgMAoGCCqGSM49BAMCA0gAMEUCIBU7mYrPiPycjCgV\n"
    "25Z27EKVPoY3cpDX26c5LS5Lr9KbAiEA2h+Rbcc2MQBaIanSho2dsZR66dhdKSuc\n"
    "jvqQzeNhiHk=\n"
    "-----END CERTIFICATE-----\n";

static const std::string certificateOnlyPem = \
    "-----BEGIN CERTIFICATE-----\n"
    "MIIC1DCCAnqgAwIBAgICBUEwCgYIKoZIzj0EAwIwcTELMAkGA1UEBhMCQkUxETAP\n"
    "BgNVBAgTCEJydXNzZWxzMREwDwYDVQQHEwhCcnVzc2VsczETMBEGA1UEChMKU21h\n"
    "cnRtYXRpYzENMAsGA1UECxMEU0FFUzEYMBYGA1UEAxQPaW50ZXJtZWRpYXRlX2Nh\n"
    "MB4XDTE3MDYyMDIwMzgyOFoXDTE3MTIzMTAwMDAwMFoweDEfMB0GA1UEAwwWU0FJ\n"
    "TlQtR0lMTEVTXzAxN18yMTAxMzENMAsGA1UECwwEU0FFUzETMBEGA1UECgwKU21h\n"
    "cnRtYXRpYzERMA8GA1UEBwwIQnJ1c3NlbHMxETAPBgNVBAgMCEJydXNzZWxzMQsw\n"
    "CQYDVQQGEwJCRTCBmzAQBgcqhkjOPQIBBgUrgQQAIwOBhgAEAQhYZ61siKN94QmV\n"
    "Jg3hVa/UffvsLrq9taodRdSwN92sw5KR0JB++dXoew01wGGg5QOAYokoRBmBLP72\n"
    "egIyRKU1ALdu113E2zdfyVd5j9g70cdnNk5ee00zsP+3GUNtSoZk3SdQninKXngQ\n"
    "t3oAkj8RmUE1b9n0BwZ+fYtG+rQJNwoYo4G3MIG0MIGTBgNVHSMEgYswgYiAFNgO\n"
    "I9lmen9JDkZc31DDa3Sgua+boW2kazBpMQswCQYDVQQGEwJCRTERMA8GA1UECBMI\n"
    "QnJ1c3NlbHMxETAPBgNVBAcTCEJydXNzZWxzMRMwEQYDVQQKEwpTbWFydG1hdGlj\n"
    "MQ0wCwYDVQQLEwRTQUVTMRAwDgYDVQQDFAdyb290X2NhggEBMAwGA1UdEwEB/wQC\n"
    "MAAwDgYDVR0PAQH/BAQDAgWgMAoGCCqGSM49BAMCA0gAMEUCIBU7mYrPiPycjCgV\n"
    "25Z27EKVPoY3cpDX26c5LS5Lr9KbAiEA2h+Rbcc2MQBaIanSho2dsZR66dhdKSuc\n"
    "jvqQzeNhiHk=\n"
    "-----END CERTIFICATE-----\n";

static const std::string privateKeyPem = \
    "-----BEGIN ENCRYPTED PRIVATE KEY-----\n"
    "MIIBPTBABgkqhkiG9w0BBQ0wMzAbBgkqhkiG9w0BBQwwDgQI8xxuGLgQJxkCAggA\n"
    "MBQGCCqGSIb3DQMHBAgQBwzbvXZeFQSB+DjYHcsHnqlU/ySYxuQl/YyH+Ab1W2cZ\n"
    "IUb/RrwG43zKKzSApFzPA7uKB2tYKRFhZihZ3KFKE4QB3a2x3Plm+giz0qjfr8E9\n"
    "fjOGDN77awn/FJHFttNGVbKxV69P9jLHdv/gJEu4L6yKFlP1U/khVMSEmnBGbSQ6\n"
    "92Fy8VDrhk8qJOAOSyz9lBf90h01J1qxaH73ms/XpNWSQTr2MQ9OjBkG6rc64Owf\n"
    "PFCv1qsTXSk1EozbeaQmBCgMAqeJ/wBmmzRQVHG0RKHFno6ChalGiAQTJACa6Vws\n"
    "Wbr0GJdSG62H1jVU/AWdlj6cpG7g6wEZwgTHiqHFqplz\n"
    "-----END ENCRYPTED PRIVATE KEY-----\n";

static const std::string passPhrase = \
    "gQnN6L1GSLgN/Bzt0sWtz+ZAlB5JQPJP03aILQ9oS1Q=";

static const std::string wrongPassPhrase = \
    "jahdgfyatgd6e718bfakdrfr65a67dtgvb6g5ad6flag";

static const std::string message = "Hello World!";

static const std::string filePath = "filePath";

static const unsigned long DATA_OFFSET = 190;

class CMSSignerTest : public CPPUNIT_NS::TestFixture
{
    public:
        CMSSignerTest()
        {
        }

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

        void testSignWithoutCertificate()
        {
            CMSSigner  cmsSigner;
            SafeBuffer bufferToSign(message);
            SafeBuffer signedBuffer("invalid");

            CPPUNIT_ASSERT_THROW(cmsSigner.signBuffer(bufferToSign, signedBuffer),
                                 SignedException);
            CPPUNIT_ASSERT (signedBuffer.GetSize() == 0);
        }

        void testVerifyWithoutCertificate()
        {
            CMSSigner  cmsSigner;
            SafeBuffer bufferToVerify(message);
            SafeBuffer signedBuffer("invalid");

            CPPUNIT_ASSERT_THROW(cmsSigner.verifyBuffer(signedBuffer, bufferToVerify),
                                 SignedException);
        }

        void testSignAndVerifyWithPkcs12()
        {
            CMSSigner     cmsSigner;
            SafeBuffer    bufferToSign(message);
            SafeBuffer    certificate;
            SafeBuffer    resultBuffer;
            SafeBuffer    signedBuffer;
            StoreMockFile file;

            Base64Encoder::ConvertFromBase64(certificatePkcs12, certificate);
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificate, true));

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PKCS12, passPhrase);
            cmsSigner.signBuffer(bufferToSign, signedBuffer);

            CPPUNIT_ASSERT(signedBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(signedBuffer != bufferToSign);

            cmsSigner.verifyBuffer(signedBuffer, resultBuffer);
            CPPUNIT_ASSERT(resultBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(resultBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(resultBuffer == bufferToSign);
        }

        void testSignAndVerifyWithPEM()
        {
            CMSSigner     cmsSigner;
            SafeBuffer    bufferToSign(message);
            SafeBuffer    resultBuffer;
            SafeBuffer    signedBuffer;
            StoreMockFile file;

            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificatePem, true));

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PEM, passPhrase);
            cmsSigner.signBuffer(bufferToSign, signedBuffer);

            CPPUNIT_ASSERT(signedBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(signedBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(signedBuffer != bufferToSign);

            cmsSigner.verifyBuffer(signedBuffer, resultBuffer);
            CPPUNIT_ASSERT(resultBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(resultBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(resultBuffer == bufferToSign);
        }

        void testSignWithoutPemPrivateKey()
        {
            CMSSigner     cmsSigner;
            SafeBuffer    bufferToSign(message);
            SafeBuffer    signedBuffer;
            StoreMockFile file;

            File::getInstance(&file);
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificateOnlyPem, true));

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PEM, passPhrase);

            CPPUNIT_ASSERT_THROW(cmsSigner.signBuffer(bufferToSign, signedBuffer),
                                 SignedException);
            CPPUNIT_ASSERT(signedBuffer.GetSize() == 0);
            CPPUNIT_ASSERT(signedBuffer.GetData() == NULL);
        }

        void testSignAndVerifyWithoutPemPrivateKey()
        {
            CMSSigner     cmsSigner;
            SafeBuffer    bufferToSign(message);
            SafeBuffer    signedBuffer;
            SafeBuffer    resultBuffer;
            StoreMockFile file;

            File::getInstance(&file);
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificatePem, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificateOnlyPem, true));

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PEM, passPhrase);
            cmsSigner.signBuffer(bufferToSign, signedBuffer);

            CPPUNIT_ASSERT(signedBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(signedBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(signedBuffer != bufferToSign);

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PEM, passPhrase);
            cmsSigner.verifyBuffer(signedBuffer, resultBuffer);

            CPPUNIT_ASSERT(resultBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(resultBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(resultBuffer == bufferToSign);
        }

        void testSignAndVerityModifiedBufferWithPemCertificate()
        {
            CMSSigner     cmsSigner;
            SafeBuffer    bufferToSign(message);
            SafeBuffer    signedBuffer;
            SafeBuffer    resultBuffer;
            StoreMockFile file;
            const unsigned char bug = 0x62;

            File::getInstance(&file);
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificatePem, true));

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PEM, passPhrase);
            cmsSigner.signBuffer(bufferToSign, signedBuffer);

            CPPUNIT_ASSERT(signedBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(signedBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(signedBuffer != bufferToSign);

            signedBuffer.insertData(DATA_OFFSET, &bug, 1);

            CPPUNIT_ASSERT_THROW(cmsSigner.verifyBuffer(signedBuffer, resultBuffer),
                                 SignedException);

            CPPUNIT_ASSERT(resultBuffer.GetSize() == 0);
            CPPUNIT_ASSERT(resultBuffer.GetData() == NULL);
        }

        void testSignAndVerifyModifiedBufferWithPkcs12Certificate()
        {
            CMSSigner     cmsSigner;
            SafeBuffer    bufferToSign(message);
            SafeBuffer    certificate;
            SafeBuffer    resultBuffer;
            SafeBuffer    signedBuffer;
            StoreMockFile file;
            const unsigned char bug = 0x62;

            Base64Encoder::ConvertFromBase64(certificatePkcs12, certificate);
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificate, true));

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PKCS12, passPhrase);
            cmsSigner.signBuffer(bufferToSign, signedBuffer);

            CPPUNIT_ASSERT(signedBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(signedBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(signedBuffer != bufferToSign);

            signedBuffer.insertData(DATA_OFFSET, &bug, 1);

            CPPUNIT_ASSERT_THROW(cmsSigner.verifyBuffer(signedBuffer, resultBuffer),
                                 SignedException);
            CPPUNIT_ASSERT(resultBuffer.GetSize() == 0);
            CPPUNIT_ASSERT(resultBuffer.GetData() == NULL);
        }

        void testSignWithPasswordAndVerifyWithoutPasswordPemCertificate()
        {
            CMSSigner     cmsSigner;
            SafeBuffer    bufferToSign(message);
            SafeBuffer    resultBuffer;
            SafeBuffer    signedBuffer;
            StoreMockFile file;

            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificatePem, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificatePem, true));

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PEM, passPhrase);
            cmsSigner.signBuffer(bufferToSign, signedBuffer);

            CPPUNIT_ASSERT(signedBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(signedBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(signedBuffer != bufferToSign);

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PEM, wrongPassPhrase);
            cmsSigner.verifyBuffer(signedBuffer, resultBuffer);
            CPPUNIT_ASSERT(resultBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(resultBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(resultBuffer == bufferToSign);
        }

        void testSignWithPasswordAndVerifyWithoutPasswordPkcs12Certificate()
        {
            CMSSigner     cmsSigner;
            SafeBuffer    bufferToSign(message);
            SafeBuffer    certificate;
            SafeBuffer    resultBuffer;
            SafeBuffer    signedBuffer;
            StoreMockFile file;

            Base64Encoder::ConvertFromBase64(certificatePkcs12, certificate);
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificate, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificate, true));

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PKCS12, passPhrase);
            cmsSigner.signBuffer(bufferToSign, signedBuffer);

            CPPUNIT_ASSERT(signedBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(signedBuffer != bufferToSign);

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PKCS12, wrongPassPhrase);
            CPPUNIT_ASSERT_THROW(cmsSigner.verifyBuffer(signedBuffer, resultBuffer),
                                 SignedException);
            CPPUNIT_ASSERT(resultBuffer.GetSize() == 0);
            CPPUNIT_ASSERT(resultBuffer.GetData() == NULL);
        }

        void testSignWithPkcs12AndVerifyWithPemCertificate()
        {
            CMSSigner     cmsSigner;
            SafeBuffer    bufferToSign(message);
            SafeBuffer    certificate;
            SafeBuffer    resultBuffer;
            SafeBuffer    signedBuffer;
            StoreMockFile file;

            Base64Encoder::ConvertFromBase64(certificatePkcs12, certificate);
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificate, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificatePem, true));

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PKCS12, passPhrase);
            cmsSigner.signBuffer(bufferToSign, signedBuffer);

            CPPUNIT_ASSERT(signedBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(signedBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(signedBuffer != bufferToSign);

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PEM, passPhrase);
            cmsSigner.verifyBuffer(signedBuffer, resultBuffer);
            CPPUNIT_ASSERT(resultBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(resultBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(resultBuffer == bufferToSign);
        }

        void testSignWithPkcs12AndVerityWithNullCertificate()
        {
            CMSSigner     cmsSigner;
            SafeBuffer    bufferToSign(message);
            SafeBuffer    certificate;
            SafeBuffer    resultBuffer;
            SafeBuffer    signedBuffer;
            StoreMockFile file;

            Base64Encoder::ConvertFromBase64(certificatePkcs12, certificate);
            File::getInstance(&file);

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(certificate, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(privateKeyPem, true));

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PKCS12, passPhrase);
            cmsSigner.signBuffer(bufferToSign, signedBuffer);

            CPPUNIT_ASSERT(signedBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(signedBuffer.GetData() != NULL);
            CPPUNIT_ASSERT(signedBuffer != bufferToSign);

            cmsSigner.SetCertificate(filePath, CertUtil::CertFormat_PEM, passPhrase);
            CPPUNIT_ASSERT_THROW(cmsSigner.verifyBuffer(signedBuffer, resultBuffer),
                                 SignedException);
            CPPUNIT_ASSERT(resultBuffer.GetSize() == 0);
            CPPUNIT_ASSERT(resultBuffer.GetData() == NULL);
        }

        CPPUNIT_TEST_SUITE(CMSSignerTest);

        CPPUNIT_TEST(testSignWithoutCertificate);
        CPPUNIT_TEST(testVerifyWithoutCertificate);
        CPPUNIT_TEST(testSignAndVerifyWithPkcs12);
        CPPUNIT_TEST(testSignAndVerifyWithPEM);
        CPPUNIT_TEST(testSignWithoutPemPrivateKey);
        CPPUNIT_TEST(testSignAndVerifyWithoutPemPrivateKey);
        CPPUNIT_TEST(testSignAndVerityModifiedBufferWithPemCertificate);
        CPPUNIT_TEST(testSignAndVerifyModifiedBufferWithPkcs12Certificate);
        CPPUNIT_TEST(testSignWithPasswordAndVerifyWithoutPasswordPemCertificate);
        CPPUNIT_TEST(testSignWithPasswordAndVerifyWithoutPasswordPkcs12Certificate);
        CPPUNIT_TEST(testSignWithPkcs12AndVerifyWithPemCertificate);
        CPPUNIT_TEST(testSignWithPkcs12AndVerityWithNullCertificate);

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_CMSSIGNERTEST */
