#ifndef TEST_PBE_KEY_DERIVATOR_TEST
#define TEST_PBE_KEY_DERIVATOR_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/Security/Cryptography/PBEKeyDerivatorHMAC.h>
#include <System/Security/OpenSSLSecurity.hxx>
#include <System/Exception/CryptoException.hxx>
#include <System/SafeBuffer.h>

class PBEKeyDerivatorHMACTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::Security::Cryptography::PBEKeyDerivatorHMAC PBEKeyDerivatorHMAC;
        typedef Smartmatic::System::Exception::CryptoException CryptoException;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;

        PBEKeyDerivatorHMAC * manager;

    public:

        PBEKeyDerivatorHMACTest()
        {
            manager = NULL;
        }

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();
            manager = new PBEKeyDerivatorHMAC();
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();

            if ( manager != NULL )
            {
                delete ( manager );
                manager = NULL;
            }
        }

        void testCreateKeyWithoutSaltEmptyBuffer()
        {
            SafeBuffer inKey;
            SafeBuffer outKey;

            CPPUNIT_ASSERT_THROW (
                manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256 ),
                CryptoException );

            CPPUNIT_ASSERT ( 0 == outKey.GetSize() );
        }

        void testCreateKeyWithoutSalt()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;

            std::string expected = "KRs1xLXKpYlMvWpsDz6gtbhMOpi7LoY9tSpQ6m3z8bM=";

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithoutSaltEmptyBufferTwoTime()
        {
            SafeBuffer inKey;
            SafeBuffer outKey;

            CPPUNIT_ASSERT_THROW (
                manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256 ),
                CryptoException );

            CPPUNIT_ASSERT ( 0 == outKey.GetSize() );

            CPPUNIT_ASSERT_THROW (
                manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256 ),
                CryptoException );

            CPPUNIT_ASSERT ( 0 == outKey.GetSize() );
        }

        void testCreateKeyWithoutSaltTwoTime()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;

            std::string expected = "KRs1xLXKpYlMvWpsDz6gtbhMOpi7LoY9tSpQ6m3z8bM=";

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithoutSaltKeyLenZero()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;

            std::string expected = "KRs1xLXKpYlMvWpsDz6gtbhMOpi7LoY9tSpQ6m3z8bM=";

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256, 0 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithoutSaltKeyLen16()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;

            std::string expected = "KRs1xLXKpYlMvWpsDz6gtQ==";

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256, 16 );

            CPPUNIT_ASSERT ( 16 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithoutSaltKeyLen8()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;

            std::string expected = "KRs1xLXKpYk=";

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256, 8 );

            CPPUNIT_ASSERT ( 8 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSalt()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "eb8xGt4kcIr5dV/Q9FVEQi9Y1NYOBRQY8gT/L9QPOkg=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeySetSaltTwoTime()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "eb8xGt4kcIr5dV/Q9FVEQi9Y1NYOBRQY8gT/L9QPOkg=";

            manager->SetSalt ( salt );
            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltTwoTime()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "eb8xGt4kcIr5dV/Q9FVEQi9Y1NYOBRQY8gT/L9QPOkg=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterationsZero()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "ClE4GkepdT1SVcNJxzcJNhFTG9hGbwdho9nckHjrcPA=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256, 32, 0 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "VDv+wZXb8+oO271a+5LYwGhfJMjYVmi3rnM8Z8nk+5U=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024AndSHA224()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "XxUJarDzTrvLzJZuU7RDcbIMBvXBDQ0OvzsYpywELAU=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA224, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024AndSHA256()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "VDv+wZXb8+oO271a+5LYwGhfJMjYVmi3rnM8Z8nk+5U=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024AndSHA384()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "qafPgYIFVEu2+S0YagbuNMG2dFn2xCx7Tbo7MfuSXyc=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA384, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024AndSHA512()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "wUzIS8GIdfXe/n9fLtmDg4zFwzomP2YCEQjjmQfqLic=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA512, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        CPPUNIT_TEST_SUITE ( PBEKeyDerivatorHMACTest );

        CPPUNIT_TEST ( testCreateKeyWithoutSaltEmptyBuffer );
        CPPUNIT_TEST ( testCreateKeyWithoutSalt );
        CPPUNIT_TEST ( testCreateKeyWithoutSaltEmptyBufferTwoTime );
        CPPUNIT_TEST ( testCreateKeyWithoutSaltTwoTime );

        CPPUNIT_TEST ( testCreateKeyWithoutSaltKeyLenZero );
        CPPUNIT_TEST ( testCreateKeyWithoutSaltKeyLen16 );
        CPPUNIT_TEST ( testCreateKeyWithoutSaltKeyLen8 );

        CPPUNIT_TEST ( testCreateKeyWithSalt );
        CPPUNIT_TEST ( testCreateKeySetSaltTwoTime );
        CPPUNIT_TEST ( testCreateKeyWithSaltTwoTime );

        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterationsZero );
        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024 );

        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024AndSHA224 );
        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024AndSHA256 );
        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024AndSHA384 );
        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024AndSHA512 );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_PBE_KEY_DERIVATOR_TEST */
