#ifndef TEST_MSGDIGESTTEST
#define TEST_MSGDIGESTTEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/Security/Cryptography/MsgDigestUtil.h>
#include <System/Security/OpenSSLSecurity.hxx>
#include "MockMsgDigestUtil.h"

class MsgDigestUtilTest : public CPPUNIT_NS::TestFixture
{
    public:

        typedef Smartmatic::System::Exception::MessageDigestException MessageDigestException;
        typedef Smartmatic::System::Security::Cryptography::MsgDigestUtil MsgDigestUtil;

        MsgDigestUtilTest() {}

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

        void testSHA224Name()
        {
            Smartmatic::System::SafeBuffer buffer;
            MockMsgDigestUtil digest ( buffer );
            std::string expected = "sha224";
            std::string result = digest.getName ( MsgDigestUtil::DGSTUTIL_SHA224 );

            CPPUNIT_ASSERT ( expected == result );
        }

        void testSHA256Name()
        {
            Smartmatic::System::SafeBuffer buffer;
            MockMsgDigestUtil digest ( buffer );
            std::string expected = "sha256";
            std::string result = digest.getName ( MsgDigestUtil::DGSTUTIL_SHA256 );

            CPPUNIT_ASSERT ( expected == result );
        }

        void testSHA384Name()
        {
            Smartmatic::System::SafeBuffer buffer;
            MockMsgDigestUtil digest ( buffer );
            std::string expected = "sha384";
            std::string result = digest.getName ( MsgDigestUtil::DGSTUTIL_SHA384 );

            CPPUNIT_ASSERT ( expected == result );
        }

        void testSHA512Name()
        {
            Smartmatic::System::SafeBuffer buffer;
            MockMsgDigestUtil digest ( buffer );
            std::string expected = "sha512";
            std::string result = digest.getName ( MsgDigestUtil::DGSTUTIL_SHA512 );

            CPPUNIT_ASSERT ( expected == result );
        }

        void testDigestNullBufferSHA256()
        {
            Smartmatic::System::SafeBuffer buffer;
            Smartmatic::System::SafeBuffer result;
            MsgDigestUtil digest ( buffer );
            digest.getDigest ( result, MsgDigestUtil::DGSTUTIL_SHA256 );

            CPPUNIT_ASSERT ( NULL != result.GetData() );
            CPPUNIT_ASSERT ( 32 == result.GetSize() );
        }

        void testDigestNullBufferSHA512()
        {
            Smartmatic::System::SafeBuffer buffer;
            Smartmatic::System::SafeBuffer result;
            MsgDigestUtil digest ( buffer );
            digest.getDigest ( result, MsgDigestUtil::DGSTUTIL_SHA512 );

            CPPUNIT_ASSERT ( NULL != result.GetData() );
            CPPUNIT_ASSERT ( 64 == result.GetSize() );
        }

        void testDigestSHA224()
        {
            Smartmatic::System::SafeBuffer buffer ( "ABCDEFG" );
            Smartmatic::System::SafeBuffer result;

            // echo -n "ABCDEFG" | openssl dgst -binary -sha224 | openssl enc -base64
            std::string expected = "uuNzXlgi2MMPr9cHNjFueAf3zM9l5uc8FfMqYA==";

            MsgDigestUtil digest ( buffer );
            digest.getDigest ( result, MsgDigestUtil::DGSTUTIL_SHA224 );

            CPPUNIT_ASSERT ( NULL != result.GetData() );
            CPPUNIT_ASSERT ( 28 == result.GetSize() );
            CPPUNIT_ASSERT ( expected == result.GetBase64Data() );
        }

        void testDigestSHA256()
        {
            Smartmatic::System::SafeBuffer buffer ( "ABCDEFG" );
            Smartmatic::System::SafeBuffer result;

            // echo -n "ABCDEFG" | openssl dgst -binary -sha256 | openssl enc -base64
            std::string expected = "6akqLtDVNzKsE7AxonsHGBQjHIYzyfQYRMy6iE1IKxY=";

            MsgDigestUtil digest ( buffer );
            digest.getDigest ( result, MsgDigestUtil::DGSTUTIL_SHA256 );

            CPPUNIT_ASSERT ( NULL != result.GetData() );
            CPPUNIT_ASSERT ( 32 == result.GetSize() );
            CPPUNIT_ASSERT ( expected == result.GetBase64Data() );
        }

        void testDigestSHA384()
        {
            Smartmatic::System::SafeBuffer buffer ( "ABCDEFG" );
            Smartmatic::System::SafeBuffer result;

            // echo -n "ABCDEFG" | openssl dgst -binary -sha384 | openssl enc -base64
            std::string expected = "fHJ5tvS74vOWl+fgHn43lU8rV2bV/H/BsZzOvvs5tUOZormnT/LwihqBjXrqSIcl";

            MsgDigestUtil digest ( buffer );
            digest.getDigest ( result, MsgDigestUtil::DGSTUTIL_SHA384 );

            CPPUNIT_ASSERT ( NULL != result.GetData() );
            CPPUNIT_ASSERT ( 48 == result.GetSize() );
            CPPUNIT_ASSERT ( expected == result.GetBase64Data() );
        }

        void testDigestSHA512()
        {
            Smartmatic::System::SafeBuffer buffer ( "ABCDEFG" );
            Smartmatic::System::SafeBuffer result;

            // echo -n "ABCDEFG" | openssl dgst -binary -sha512 | openssl enc -base64
            std::string expected = "Pni0zYiE8VYioAUmMIhxyAXa6KmssGUr94yXfPUVDKheCujeiL5mjRPiVCZU2+E5" \
                                   "r6bJFHg/jdbXDni0fUGt3A==";

            MsgDigestUtil digest ( buffer );
            digest.getDigest ( result, MsgDigestUtil::DGSTUTIL_SHA512 );

            CPPUNIT_ASSERT ( NULL != result.GetData() );
            CPPUNIT_ASSERT ( 64 == result.GetSize() );
            CPPUNIT_ASSERT ( expected == result.GetBase64Data() );
        }

        void testDigestErrorName()
        {
            Smartmatic::System::SafeBuffer buffer ( "ABCDEFG" );
            Smartmatic::System::SafeBuffer result;

            MockMsgDigestUtil digest ( buffer );

            MOCKPP_CHAINER_FOR_EXT ( MockMsgDigestUtil, mMapAlgName, ext ) mMapAlgName_mock ( &digest );

            mMapAlgName_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub<const char *> ( "null" ) );

            CPPUNIT_ASSERT_THROW ( digest.getDigest ( result, MsgDigestUtil::DGSTUTIL_SHA512 ), MessageDigestException );

            digest.verify();
        }


        void testNullAlgoritmo()
        {
            Smartmatic::System::SafeBuffer buffer ( "ABCDEFG" );
            Smartmatic::System::SafeBuffer result;

            MockMsgDigestUtil digest ( buffer );

            MOCKPP_CHAINER_FOR_EXT ( MockMsgDigestUtil, mMapAlgName, ext ) mMapAlgName_mock ( &digest );

            mMapAlgName_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub<const char *> ( "null" ) );

            CPPUNIT_ASSERT_THROW ( digest.getDigest ( result, MsgDigestUtil::DGSTUTIL_SHA512 ), MessageDigestException );

            digest.verify();
        }

        CPPUNIT_TEST_SUITE ( MsgDigestUtilTest );

        CPPUNIT_TEST ( testSHA224Name );
        CPPUNIT_TEST ( testSHA256Name );
        CPPUNIT_TEST ( testSHA384Name );
        CPPUNIT_TEST ( testSHA512Name );

        CPPUNIT_TEST ( testDigestNullBufferSHA256 );
        CPPUNIT_TEST ( testDigestNullBufferSHA512 );
        CPPUNIT_TEST ( testDigestSHA224 );
        CPPUNIT_TEST ( testDigestSHA256 );
        CPPUNIT_TEST ( testDigestSHA384 );
        CPPUNIT_TEST ( testDigestSHA512 );

        CPPUNIT_TEST ( testDigestErrorName );
        CPPUNIT_TEST ( testNullAlgoritmo );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_MSGDIGESTTEST */
