#ifndef TEST_FOP_TRANSFORMER_TEST
#define TEST_FOP_TRANSFORMER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/XSLFO/XSLFOTransformer.h>
#include <System/XSLFO/FOPTransformer.hxx>
#include <System/Runtime/ProcessLauncher.h>
#include <System/IO/Directory.h>
#include "MockProcessLauncher.hxx"
#include "StoreMockProcessLauncher.hxx"
#include "MockDirectory.hxx"

class FOPTransformerTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::Runtime::ProcessLauncher ProcessLauncher;
        typedef Smartmatic::System::IO::Directory Directory;

        typedef Smartmatic::System::XSLFO::FOPTransformer FOPTransformer;
        typedef Smartmatic::System::XSLFO::XSLFOTransformer XSLFOTransformer;

        XSLFOTransformer * trasnformer;

    public:

        FOPTransformerTest ()
        {
            trasnformer = NULL;
        }

        void setUp()
        {
            trasnformer = new FOPTransformer();
        }

        void tearDown()
        {
            delete trasnformer;
            trasnformer = NULL;
        }

        void testBasicTransformerSuccessful()
        {
            MockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            MOCKPP_CHAINER_FOR_EXT ( MockProcessLauncher, RunProcessThroughSystem, ext ) RunProcessThroughSystem_mock ( &launcher );

            RunProcessThroughSystem_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<int> ( 0 ) );

            MockDirectory directory;
            Directory::getInstance ( &directory );

            MOCKPP_CHAINER_FOR_EXT ( MockDirectory, FileExists, ext ) FileExists_mock ( &directory );

            FileExists_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            bool expected = true;
            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( expected == result );

            launcher.verify();
            directory.verify();
        }

        void testBasicTransformerFail()
        {
            MockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            MOCKPP_CHAINER_FOR_EXT ( MockProcessLauncher, RunProcessThroughSystem, ext ) RunProcessThroughSystem_mock ( &launcher );

            RunProcessThroughSystem_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<int> ( 1 ) );

            MockDirectory directory;
            Directory::getInstance ( &directory );

            MOCKPP_CHAINER_FOR_EXT ( MockDirectory, FileExists, ext ) FileExists_mock ( &directory );

            FileExists_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            bool expected = false;
            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( expected == result );

            launcher.verify();
            directory.verify();
        }

        void testBasicTransformerCommandExpectedNotConfigFile()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            MockDirectory directory;
            Directory::getInstance ( &directory );

            MOCKPP_CHAINER_FOR_EXT ( MockDirectory, FileExists, ext ) FileExists_mock ( &directory );

            FileExists_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            std::string expected = "fop -xml input -xsl template -pdf output";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );

            directory.verify();
        }

        void testBasicTransformerCommandExpectedWithDefaultConfigFile()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            MockDirectory directory;
            Directory::getInstance ( &directory );

            MOCKPP_CHAINER_FOR_EXT ( MockDirectory, FileExists, ext ) FileExists_mock ( &directory );

            FileExists_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            std::string expected = "fop -xml input -xsl template -pdf output -c fop.xconf";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );

            directory.verify();
        }

        void testWithConfigFileTransformerCommandExpectedWithConfigFile()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            MockDirectory directory;
            Directory::getInstance ( &directory );

            MOCKPP_CHAINER_FOR_EXT ( MockDirectory, FileExists, ext ) FileExists_mock ( &directory );

            FileExists_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            std::string expected = "fop -xml input -xsl template -pdf output -c config";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";
            std::string configFile = "config";

            ( static_cast < FOPTransformer * > ( trasnformer ) )->setConfigFile ( configFile );

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );

            directory.verify();
        }

        CPPUNIT_TEST_SUITE ( FOPTransformerTest );

        CPPUNIT_TEST ( testBasicTransformerSuccessful );
        CPPUNIT_TEST ( testBasicTransformerFail );
        CPPUNIT_TEST ( testBasicTransformerCommandExpectedNotConfigFile );
        CPPUNIT_TEST ( testBasicTransformerCommandExpectedWithDefaultConfigFile );
        CPPUNIT_TEST ( testWithConfigFileTransformerCommandExpectedWithConfigFile );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_FOP_TRANSFORMER_TEST */
