/**
 * @file	label-configuration.h
 * @brief	Declares the label configuration class.
 */
/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * smartmaticgui
 * Copyright (C)  2011 <>
 *
 * smartmaticgui is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * smartmaticgui is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _LABEL_CONFIGURATION_H_

#define _LABEL_CONFIGURATION_H_

#include <gtkmm.h>
#include <string>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Configuration
        {
            /**
             * @enum	LabelAlignmentEnum
             * @brief	Values that represent LabelAlignmentEnum.
             */
            enum LabelAlignmentEnum
            {
                LeftAlignment,  ///< Value that represent a left alignment.
                CenterAlignment,	///< Value that represent a center alignment.
                RigthAlignment  ///< Value that represent a rigth alignment.
            };

            /**
             * @enum	LabelStyleEnum
             * @brief	Values that represent LabelStyleEnum.
             */
            enum LabelStyleEnum
            {
                NormalStyle,	///< Value that represent a normal style.
                ObliqueStyle,   ///< Value that represent a oblique style.
                ItalicStyle ///< Value that represent a Italic style.
            };

            /**
             * @enum	LabelWeightEnum
             * @brief	Values that represent LabelWeightEnum.
             */
            enum LabelWeightEnum
            {
                LightWeight,	///< Value that represent a light weight.
                NormalWeight,   ///< Value that represent a normal weight.
                BoldWeight  ///< Value that represent a bold weight.
            };


            /**
             * @class	LabelConfiguration
             * @brief	Label configuration.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class LabelConfiguration
            {
                public:

                    /**
                     * @fn	LabelConfiguration::LabelConfiguration();
                     * @brief	Default constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    LabelConfiguration();

                    /**
                     * @fn	LabelConfiguration::LabelConfiguration(LabelAlignmentEnum labelAlignmentA,
                     * 			int verticalPaddingA, int horizontalPaddingA, const std::string& fontFamilyA, LabelStyleEnum styleA,
                     * 			LabelWeightEnum weightA, int sizeA);
                     * @brief	Constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	labelAlignmentA			The label alignment a.
                     * @param	verticalPaddingA		The vertical padding a.
                     * @param	horizontalPaddingA	The horizontal padding a.
                     * @param	fontFamilyA					The font family a.
                     * @param	styleA							The style a.
                     * @param	weightA							The weight a.
                     * @param	sizeA								The size a.
                     */
                    LabelConfiguration ( LabelAlignmentEnum labelAlignmentA, int verticalPaddingA, int horizontalPaddingA,
                                         const std::string& fontFamilyA, LabelStyleEnum styleA, LabelWeightEnum weightA,	int sizeA ); ///< The size a)

                    /**
                     * @fn	LabelAlignmentEnum LabelConfiguration::GetLabelAlignment();
                     * @brief	Gets the label alignment.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	The label alignment.
                     */
                    LabelAlignmentEnum GetLabelAlignment();

                    /**
                     * @fn	int LabelConfiguration::GetVerticalPadding();
                     * @brief	Gets the vertical padding.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	The vertical padding.
                     */
                    int GetVerticalPadding();

                    /**
                     * @fn	int LabelConfiguration::GetHorizontalPadding();
                     * @brief	Gets the horizontal padding.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	The horizontal padding.
                     */
                    int GetHorizontalPadding();

                    /**
                     * @fn	std::string LabelConfiguration::GetFontFamily();
                     * @brief	Gets the font family.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	The font family.
                     */
                    std::string GetFontFamily();

                    /**
                     * @fn	LabelStyleEnum LabelConfiguration::GetStyle();
                     * @brief	Gets the style.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	The style.
                     */
                    LabelStyleEnum GetStyle();

                    /**
                     * @fn	LabelWeightEnum LabelConfiguration::GetWeight();
                     * @brief	Gets the weight.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	The weight.
                     */
                    LabelWeightEnum GetWeight();

                    /**
                     * @fn	int LabelConfiguration::GetSize();
                     * @brief	Gets the size.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	The size.
                     */
                    int GetSize();

                    /**
                     * @fn	void LabelConfiguration::SetLabelAlignment(LabelAlignmentEnum labelAlignmentA);
                     * @brief	Sets a label alignment.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	labelAlignmentA	The label alignment a.
                     */
                    void SetLabelAlignment ( LabelAlignmentEnum labelAlignmentA );

                    /**
                     * @fn	void LabelConfiguration::SetVerticalPadding(int verticalPaddingA);
                     * @brief	Sets a vertical padding.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	verticalPaddingA	The vertical padding a.
                     */
                    void SetVerticalPadding ( int verticalPaddingA );

                    /**
                     * @fn	void LabelConfiguration::SetHorizontalPadding(int horizontalPaddingA);
                     * @brief	Sets a horizontal padding.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	horizontalPaddingA	The horizontal padding a.
                     */
                    void SetHorizontalPadding ( int horizontalPaddingA );

                    /**
                     * @fn	void LabelConfiguration::SetFontFamily( const std::string& forntFamilyA );
                     * @brief	Sets a font family.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	forntFamilyA	The fornt family a.
                     */
                    void SetFontFamily ( const std::string& forntFamilyA );

                    /**
                     * @fn	void LabelConfiguration::SetStyle(LabelStyleEnum styleA);
                     * @brief	Sets a style.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	styleA	The style a.
                     */
                    void SetStyle ( LabelStyleEnum styleA );

                    /**
                     * @fn	void LabelConfiguration::SetWeight(LabelWeightEnum weightA);
                     * @brief	Sets a weight.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	weightA	The weight a.
                     */
                    void SetWeight ( LabelWeightEnum weightA );

                    /**
                     * @fn	void LabelConfiguration::SetSize(int sizeA);
                     * @brief	Sets a size.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	sizeA	The size a.
                     */
                    void SetSize ( int sizeA );

                    /**
                     * @fn	static void LabelConfiguration::SetLabelConfiguration(Gtk::Label* label,
                     * 			Smartmatic::GUI::Configuration::LabelConfiguration& configuration);
                     * @brief	Sets a label configuration.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	label				 	If non-null, the label.
                     * @param	configuration	The configuration.
                     */
                    static void SetLabelConfiguration ( Gtk::Label * label, Smartmatic::GUI::Configuration::LabelConfiguration & configuration );

                    /**
                     * @fn	static void LabelConfiguration::SetLabelConfigurationWithoutAlignment(Gtk::Label* label,
                     * 			Smartmatic::GUI::Configuration::LabelConfiguration& configuration);
                     * @brief	Sets a label configuration without alignment.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	label				 	If non-null, the label.
                     * @param	configuration	The configuration.
                     */
                    static void SetLabelConfigurationWithoutAlignment ( Gtk::Label * label, Smartmatic::GUI::Configuration::LabelConfiguration & configuration );
                    //static void SetLabelConfiguration(Gtk::Label& label, Smartmatic::GUI::Configuration::LabelConfiguration& configuration);

                protected:

                private:
                    LabelAlignmentEnum labelAlignment;  ///< The label alignment
                    int verticalPadding;	///< The vertical padding
                    int horizontalPadding;  ///< The horizontal padding
                    std::string fontFamily; ///< The font family
                    LabelStyleEnum style;   ///< The style
                    LabelWeightEnum weight; ///< The weight
                    int size;   ///< The size
            };
        }
    }
}

#endif // _LABEL_CONFIGURATION_H_
