/**
 * @file BoxSelectionWidget.cxx
 * @brief Body box selection widget class
 * @date 14/11/2012
 * @author Yadickson Soto
 */

#include "BoxSelectionWidget.hxx"

using namespace Smartmatic::GUI::Widgets;

BoxSelectionWidget::BoxSelectionWidget ( const std::string & code, bool enableClick, bool showBorder, bool setColors )
    : SmartmaticBasicFrame ( )
{
    initialize ( code, "", enableClick, showBorder, setColors );
}

BoxSelectionWidget::BoxSelectionWidget ( const std::string & code, const std::string & optionalValue, bool enableClick, bool showBorder, bool setColors )
    : SmartmaticBasicFrame ( )
{
    initialize ( code, optionalValue, enableClick, showBorder, setColors );
}

void BoxSelectionWidget::initialize ( const std::string & code, const std::string & optionalValue, bool enableClick, bool showBorder, bool setColors )
{
    this->code = code;
    this->optionalValue = optionalValue;
    this->setColors = setColors;

    flagForce = false;
    flagSelected = false;
    eventbox = new SmartmaticBasicEventBox();
    eventbox->set_events ( Gdk::BUTTON_PRESS_MASK );

    if ( showBorder )
    {
        set_shadow_type ( Gtk::SHADOW_IN );
    }
    else
    {
        set_shadow_type ( Gtk::SHADOW_NONE );
    }

    if ( enableClick )
    {
        eventbox->signal_button_press_event().connect ( sigc::mem_fun ( *this, &BoxSelectionWidget::onClicked ) );
    }

    add ( *eventbox );
}

BoxSelectionWidget::~BoxSelectionWidget()
{
    remove();

    if ( eventbox )
    {
        delete eventbox;
        eventbox = NULL;
    }
}

void BoxSelectionWidget::addWidget ( Gtk::Widget * widget )
{
    eventbox->remove();
    eventbox->add ( *widget );
    eventbox->show_all_children();
}

void BoxSelectionWidget::setSelectedColor ( Gdk::Color color )
{
    this->selectedColor = color;
}

void BoxSelectionWidget::setUnselectedColor ( Gdk::Color color )
{
    this->unselectedColor = color;
}

void BoxSelectionWidget::forceClick()
{
    flagForce = true;
    click();
}

void BoxSelectionWidget::click()
{
    flagSelected = true;

    if ( setColors )
    {
        eventbox->modify_bg ( Gtk::STATE_NORMAL, selectedColor );
        SmartmaticBasicFrame::setAccessibleName ( accessibleName + "*" );
    }

    signalClick.emit ( this );
}

void BoxSelectionWidget::clear()
{
    if ( !flagForce )
    {
        flagSelected = false;

        if ( setColors )
        {
            eventbox->modify_bg ( Gtk::STATE_NORMAL, unselectedColor );
            SmartmaticBasicFrame::setAccessibleName ( accessibleName );
        }
    }
}

sigc::signal<void, BoxSelectionWidget *> BoxSelectionWidget::getSignalClick()
{
    return signalClick;
}

bool BoxSelectionWidget::onClicked ( GdkEventButton * event )
{
    if ( flagSelected )
    {
        clear();
        signalClick.emit ( this );
    }
    else
    {
        click();
    }

    return true;
}

std::string BoxSelectionWidget::getCode()
{
    return code;
}

bool BoxSelectionWidget::getStatus()
{
    return flagSelected;
}

void BoxSelectionWidget::setName ( const std::string & name )
{
    eventbox->set_name ( name );
}

SmartmaticBasicEventBox * BoxSelectionWidget::getEventBox()
{
    return eventbox;
}

std::string BoxSelectionWidget::getOptionalValue()
{
    return optionalValue;
}

void BoxSelectionWidget::setSize ( int width, int height )
{
    set_size_request ( width, height );
    clear();
    show();
}

void BoxSelectionWidget::setAccessibleName ( const std::string & name )
{
    this->accessibleName = name;
    SmartmaticBasicFrame::setAccessibleName ( name );
}
