/**
 * @file EventboxSelectionWidget.h
 * @brief Header class event box selection widget
 * @date 05/09/2011
 * @author Juan Delgado
 */

#ifndef EVENTBOXSELECTIONWIDGET_H_
#define EVENTBOXSELECTIONWIDGET_H_

#include <gtkmm.h>
#include <Configuration/label-configuration.h>
#include <Widgets/SmartmaticBasicVBox.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {

            /**
             * @class	EventboxSelectionWidget
             * @brief	Eventbox selection widget.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class EventboxSelectionWidget : public SmartmaticBasicVBox
            {
                public:

                    typedef std::map<std::string, sigc::connection> ConnectionMap;

                    typedef sigc::signal<void, bool> DisconnectionSignal;

                public:

                    /**
                     * @fn	EventboxSelectionWidget::EventboxSelectionWidget( std::list< std::map < std::string, std::list < std::map < std::string, std::string > > > > & selectionMap, std::string titleAccessibleName, const std::string& boxAccessibleName );
                     * @brief	Constructor.
                     * @author	Juan Delgado, Yadickson Soto
                     * @date	29/10/2012
                     * @param	selectionMap	The selection map.
                     */
                    EventboxSelectionWidget ( std::list< std::map < std::string, std::list < std::map < std::string, std::string > > > > & selectionMap, std::string titleAccessibleName, const std::string& boxAccessibleName );

                    /**
                     * @fn	virtual EventboxSelectionWidget::~EventboxSelectionWidget();
                     * @brief	Destructor.
                     * @author	Juan Delgado
                     * @date	30/10/2012
                     */
                    virtual ~EventboxSelectionWidget();

                    /**
                     * @fn	std::list< std::string > EventboxSelectionWidget::GetNamesSelected();
                     * @brief	Get names of option selected.
                     * @author	Juan Delgado
                     * @date	30/10/2012
                     * @return	list with name selected.
                     */
                    std::list< std::string > GetNamesSelected();

                    /**
                     * @fn	std::list< std::string > EventboxSelectionWidget::GetValuesSelected();
                     * @brief	Get values of option selected.
                     * @author	Juan Delgado
                     * @date	30/10/2012
                     * @return	list with value selected.
                     */
                    std::list< std::string > GetValuesSelected();

                    /**
                     * @fn	void EventboxSelectionWidget::SetSelectedBoxColor(Gdk::Color color);
                     * @brief	Sets selected box color.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @param	color	The color.
                     */
                    void SetSelectedBoxColor ( Gdk::Color color );

                    /**
                     * @fn	void EventboxSelectionWidget::SetUnselectedBoxColor(Gdk::Color color);
                     * @brief	Sets unselected box color.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @param	color	The color.
                     */
                    void SetUnselectedBoxColor ( Gdk::Color color );

                    /**
                     * @fn	void EventboxSelectionWidget::SetSelectedFontColor(Gdk::Color color);
                     * @brief	Sets selected font color.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @param	color	The color.
                     */
                    void SetSelectedFontColor ( Gdk::Color color );

                    /**
                     * @fn	void EventboxSelectionWidget::SetUnselectedFontColor(Gdk::Color color);
                     * @brief	Sets unselected font color.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @param	color	The color.
                     */
                    void SetUnselectedFontColor ( Gdk::Color color );

                    /**
                     * @fn	void EventboxSelectionWidget::SetSelectedFontDescription(Smartmatic::GUI::Configuration::LabelConfiguration& labelConfiguration);
                     * @brief	Sets selected font description.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @param	labelConfiguration	The label configuration.
                     */
                    void SetSelectedFontDescription ( Smartmatic::GUI::Configuration::LabelConfiguration & labelConfiguration );

                    /**
                     * @fn	void EventboxSelectionWidget::SetUnselectedFontDescription(Smartmatic::GUI::Configuration::LabelConfiguration& labelConfiguration);
                     * @brief	Sets unselected font description.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @param	labelConfiguration	The label configuration.
                     */
                    void SetUnselectedFontDescription ( Smartmatic::GUI::Configuration::LabelConfiguration & labelConfiguration );

                    /**
                     * @fn	void EventboxSelectionWidget::SetTitlesFontDescription(Smartmatic::GUI::Configuration::LabelConfiguration& labelConfiguration);
                     * @brief	Sets titles font description.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @param	labelConfiguration	The label configuration.
                     */
                    void SetTitlesFontDescription ( Smartmatic::GUI::Configuration::LabelConfiguration & labelConfiguration );

                    /**
                     * @fn	void EventboxSelectionWidget::SetTitlesHeight(int height);
                     * @brief	Sets titles height.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @param	height	The height.
                     */
                    void SetTitlesHeight ( int height );


                    /**
                     * @fn	void EventboxSelectionWidget::SetFirstSelected();
                     * @brief	Set first element selected.
                     * @author	Juan Delgado
                     * @date	30/10/2012
                     */
                    void SetFirstSelected();

                    /**
                     * @fn  void EventboxSelectionWidget::disconnectSignals();
                     * @brief   Disconnects voter types signals.
                     * @author  Eduardo Fernández
                     * @date    05/01/2018
                     * @param   emitDisconnectionSignal Whether to emit the disconnection signal.
                     */
                    void disconnectSignals(const bool emitDisconnectionSignal = true);

                    /**
                     * @fn  void EventboxSelectionWidget::reconnectSignals();
                     * @brief   Reconnects voter types signals.
                     * @author  Eduardo Fernández
                     * @date    05/01/2018
                     */
                    void reconnectSignals();

                    /**
                     * @fn  EventboxSelectionWidget::DisconnectionSignal EventboxSelectionWidget::reconnectSignals();
                     * @brief   Gets the disconnection signal.
                     * @author  Eduardo Fernández
                     * @date    05/01/2018
                     */
                    DisconnectionSignal getDisconnectionSignal();

                    /**
                     * @fn	virtual void EventboxSelectionWidget::set_size_request(int width, int height);
                     * @brief	set_size_request.
                     * @author	Juan Delgado
                     * @date	29/10/2012
                     * @param	width 	width request.
                     * @param	height	height request.
                     */
                    virtual void set_size_request ( int width, int height );

                    /**
                     * @typedef	sigc::signal< void, std::list< std::string > > type_box_clicked
                     * @brief	Signal returns list of boxs selected.
                     */
                    typedef sigc::signal< void, std::list< std::string > > type_box_clicked;

                    /**
                     * @fn	type_box_clicked EventboxSelectionWidget::signal_box_clicked();
                     * @brief	Signal to receive when a box is clicked.
                     * @author	Juan Delgado
                     * @date	30/10/2012
                     * @return	connection signal.
                     */
                    type_box_clicked signal_box_clicked();

                    /**
                     * @fn  type_box_clicked EventboxSelectionWidget::setInitializing(const bool initializing);
                     * @brief   Sets the variable to initialize the event box.
                     * @author  Eduardo Fernández
                     * @date    05/01/2018
                     * @return  initializing The variable to initialize the event box.
                     */
                    void setInitializing(const bool initializing);

                protected:

                    type_box_clicked m_type_box_clicked;	///< The type box clicked

                    /**
                     * @fn	bool EventboxSelectionWidget::on_event_selected( GdkEventButton* event, const std::string& value );
                     * @brief	Handles option selected.
                     * @author	Juan Delgado
                     * @date	29/10/2012
                     * @param	event	If non-null, the event.
                     * @param	value	The value.
                     * @return	true if handle.
                     */
                    bool on_event_selected ( GdkEventButton * event, const std::string& value );

                    /**
                     * @fn	void EventboxSelectionWidget::SetEventboxSelected(bool isSelected, Gtk::Label* label,
                     * 			Gtk::EventBox* eventbox);
                     * @brief	Set event box selected.
                     * @author	Juan Delgado
                     * @date	29/10/2012
                     * @param	isSelected	true if is selected.
                     * @param	label				to be changed.
                     * @param	eventbox		to be change.
                     */
                    void SetEventboxSelected ( bool isSelected, Gtk::Label * label, Gtk::EventBox * eventbox );

                    /**
                     * @brief  Adjust the size of the options in the selection widget.
                     * @author manuel.arguelles
                     * @date   May 16, 2018
                     */
                    void adjustOptionSize();

                    int widthRequested;	///< the width requested
                    Gdk::Color selectedBoxColor;	///< the selected box color
                    Gdk::Color unselectedBoxColor;	///< the unselected box color
                    Gdk::Color selectedFontColor;	///< the selected font color
                    Gdk::Color unselectedFontColor;	///< the unselected font color
                    Smartmatic::GUI::Configuration::LabelConfiguration selectedLabelConfiguration; ///< the selected label configuration
                    Smartmatic::GUI::Configuration::LabelConfiguration unselectedLabelConfiguration; ///< the unselected label configuration
                    std::list< std::map < std::string, std::list < std::map < std::string, std::string > > > > selectionMap;	///< the selection map
                    std::map < std::string, std::string > namesSelected;	///< the map of names selected
                    std::map < std::string, std::string > valuesSelected; ///< the map of values selected
                    std::map < std::string, Gtk::EventBox * > eventboxMap;	///< the eventbox map
                    std::map < std::string, Gtk::Label * > labelMap;		///< the label map
                    std::list < Gtk::Label * > titleLabelList;	///< the title label list
                    ConnectionMap m_connectionMap;
                    std::string m_currentVoterType;
                    DisconnectionSignal m_disconnectionSignal;
                    bool m_initializing;
            };
        }
    }
}

#endif /* EVENTBOXSELECTIONWIDGET_H_ */
