/**
 * @file	keypad-widget.h
 * @brief	Declares the keypad widget class.
 */
/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * MachineApplication
 * Copyright (C)  2010 <>
 *
 * MachineApplication is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MachineApplication is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _KEYPAD_WIDGET_H_
#define _KEYPAD_WIDGET_H_

#include <gtkmm.h>
#include <string>
#include "Widgets/key-pad-widget-interface.h"
#include "Widgets/smartmatic-button.h"
#include "Environment/Configurations/resource-path-provider.h"
#include <Widgets/SmartmaticBasicTable.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace GUI
    {
        namespace Widgets
        {
            /**
             * @enum	KeypadEnum
             * @brief	Values that represent KeypadEnum.
             */
            enum KeypadEnum
            {
                Numeric,	///< Value that represent KeypadEnum Numeric.
                Complete,   ///< Values that represent KeypadEnum Complete.
                CorrigeConfirma,	///< Values that represent KeypadEnum CorrigeConfirma.
                T9  ///< Values that represent KeypadEnum T9.
            };


            /**
             * @class	KeypadWidget
             * @brief	Keypad widget.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class KeypadWidget : public SmartmaticBasicTable, public KeypadCompleteInterface
            {
                public:

                    /**
                     * @fn	KeypadWidget::KeypadWidget(KeypadEnum type, bool sound = false);
                     * @brief	Constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	type 	The type.
                     * @param	sound	true to sound.
                     */
                    KeypadWidget ( KeypadEnum type, bool sound = false, int tableMaxHeight = -1, int buttonsMaxWidth = -1, int buttonsMaxHeight = -1, bool useSmallImages = false );

                    /**
                     * @fn	virtual KeypadWidget::~KeypadWidget();
                     * @brief	Finaliser.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    virtual ~KeypadWidget();

                    //Signal Handlers

                    /**
                     * @fn	KeypadWidget::void (*button_clicked)(int data);
                     * @brief	Button clicked.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	data	The data.
                     */
                    void ( *button_clicked ) ( int data );
                    KeypadInterface * keypadinterface;  ///< The keypadinterface

                    /**
                     * @fn	void KeypadWidget::buttonSetNames( const std::string& prefix );
                     * @brief	Button set names.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	prefix	The prefix.
                     */
                    void buttonSetNames ( const std::string& prefix );

                    /**
                     * @fn	void setSensitiveButton ( Smartmatic::GUI::Widgets::Keys key, bool sensitive = false );
                     * @brief	set sensitive to button
                     * @author	Yadiockson Soto
                     * @date	06/09/2013
                     * @param	key	The key value from button
                     * @param sensitive new value
                     */
                    void setSensitiveButton ( Smartmatic::GUI::Widgets::Keys key, bool sensitive = false );

                protected:
                    //Signal Handlers

                    /**
                     * @fn	void KeypadWidget::on_button_numbered(Keys data);
                     * @brief	Handles button numbered signals.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	data	The data.
                     */
                    void on_button_numbered ( Keys data );

                    /**
                     * @fn	void KeypadWidget::on_button_numbered_T9(Keys data);
                     * @brief	Handles button numbered t 9 signals.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	data	The data.
                     */
                    void on_button_numbered_T9 ( Keys data );

                private:

                    //Widgets of the Glade file
                    SmartmaticBasicTable * keyPadTable;	///< The key pad table
                    SmartmaticButton * button[numberKeys];  ///< The button

                    static Smartmatic::Log::ISMTTLog * logger; ///< The logger

                    /**
                     * @fn	void KeypadWidget::SetSound(bool sound);
                     * @brief	Sets a sound.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	sound	true to sound.
                     */
                    void SetSound ( bool sound );

                    /**
                     * @fn	void KeypadWidget::AllButtonImages();
                     * @brief	All button images.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    void AllButtonImages();

                    /**
                     * @fn	void KeypadWidget::SetNormalImages(bool normal);
                     * @brief	Sets a normal images.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	normal	true to normal.
                     */
                    void SetNormalImages ( bool normal, bool useSmallImages );

                    /**
                     * @fn	void KeypadWidget::SubscribeNormalSignals(bool normal);
                     * @brief	Subscribe normal signals.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	normal	true to normal.
                     */

                    void SubscribeNormalSignals ( bool normal );

                    //T9
                    std::string T9_strings[numberKeys]; ///< The 9 strings
                    Keys keyLastPressed;	///< The key last pressed
                    int timesPressed;   ///< The times pressed
                    bool timeElapsed;   ///< true to time elapsed
                    std::string numericSound;	///< The numeric sound
                    std::string cancelSound;	///< The cancel sound
                    std::string acceptSound;	///< The accept sound
                    sigc::connection connector; ///< The connector
                    sigc::slot<bool> slot;  ///< The slot

                    /**
                     * @fn	bool KeypadWidget::OnTimerTimeout(int timer_number);
                     * @brief	Executes the timer timeout action.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	timer_number	The timer number.
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool OnTimerTimeout ( int timer_number );
            };
        }
    }
}

#endif // _KEYPAD_WIDGET_H_
