/**
 * @file	notebook-container.h
 * @brief	Declares the notebook container class.
 */

/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * smartmaticgui
 * Copyright (C) Juan Delgado 2011 <juan.delgado@smartmatic.com>
 *
 * smartmaticgui is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * smartmaticgui is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _NOTEBOOK_CONTAINER_H_
#define _NOTEBOOK_CONTAINER_H_

#include <gtkmm.h>
#include <Widgets/smartmatic-button.h>
#include <Widgets/SmartmaticBasicEventBox.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {

            /**
             * @class	NotebookContainer
             * @brief	Notebook container.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class NotebookContainer: public SmartmaticBasicEventBox
            {
                public:

                    /**
                     * @fn	NotebookContainer::NotebookContainer();
                     * @brief	class constructor.
                     * @author	Yadickson Soto
                     * @date	18/05/2012
                     */
                    NotebookContainer();


                    /**
                     * @fn	NotebookContainer::NotebookContainer(std::vector<Gtk::Widget*> & pageWidgets,
                     * 			std::vector<std::string> & titlesTabs);
                     * @brief	Constructor.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param [in,out]	pageWidgets	If non-null, the page widgets.
                     * @param	titlesTabs					 	The titles tabs.
                     */
                    NotebookContainer ( std::vector<Gtk::Widget *> & pageWidgets, std::vector<std::string> & titlesTabs );

                    /**
                     * @fn	virtual NotebookContainer::~NotebookContainer();
                     * @brief	Destructor.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    virtual ~NotebookContainer();

                    /**
                     * @fn	void NotebookContainer::ShowTabs(bool showTabs);
                     * @brief	Shows the tabs.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	showTabs	true to show, false to hide the tabs.
                     */
                    void ShowTabs ( bool showTabs );

                    /**
                     * @fn	void NotebookContainer::AppendPage(Gtk::Widget* widget, std::string titleTab = "");
                     * @brief	Appends a page.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	widget		If non-null, the widget.
                     * @param	titleTab	(optional) the title tab.
                     */
                    void AppendPage ( Gtk::Widget * widget, std::string titleTab = "" );

                    /**
                     * @fn	void NotebookContainer::SetCurrentPage(int page_num);
                     * @brief	Sets current page.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	page_num	The page number.
                     */
                    void SetCurrentPage ( int page_num );

                    /**
                     * @fn	int NotebookContainer::getCurrentPage();
                     * @brief	Method getter current page.
                     * @author	Yadickson Soto
                     * @date	18/05/2012
                     * @return	current page.
                     */
                    int getCurrentPage();

                    /**
                     * @fn	int NotebookContainer::getTotalPages();
                     * @brief	Method getter total pages.
                     * @author	Yadickson Soto
                     * @date	18/05/2012
                     * @return	total pages.
                     */
                    int getTotalPages();


                    /**
                     * @fn	void NotebookContainer::AddBottomWidget(Gtk::Widget* widget);
                     * @brief	Adds a bottom widget.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	widget	If non-null, the widget.
                     */
                    void AddBottomWidget ( Gtk::Widget * widget );


                    /**
                     * @typedef	sigc::signal<void> void_signal
                     * @brief	Defines an alias representing the void signal.
                     */
                    typedef sigc::signal<void> void_signal;

                    /**
                     * @fn	void_signal NotebookContainer::signal_all_pages_viewed();
                     * @brief	Raises all pages viewed event.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @return	.
                     */
                    void_signal signal_all_pages_viewed();

                    /**
                     * @fn	void NotebookContainer::EnableMapRefresh();
                     * @brief	Enables the map refresh.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    void EnableMapRefresh();

                    /**
                     * @fn	Gtk::Widget * NotebookContainer::getCurrentPageWidget();
                     * @brief	Method getter current page widget.
                     * @author	Yadickson Soto
                     * @date	18/05/2012
                     * @return	current page widget.
                     */
                    Gtk::Widget * getCurrentPageWidget();

                    /**
                     * @fn	void NotebookContainer::hideControls();
                     * @brief	Method hide controls
                     * @author	Yadickson Soto
                     * @date	06/11/2012
                     */
                    void hideControls();

                protected:
                    /**
                     * @fn	void NotebookContainer::initialize(std::vector<Gtk::Widget*> & pageWidgets,
                     * 			std::vector<std::string> & titlesTabs);
                     * @brief	initialize.
                     * @author	Yadickson Soto
                     * @date	18/05/2012
                     * @param	pageWidgets	widgets.
                     * @param	titlesTabs 	titles.
                     */
                    void initialize ( std::vector<Gtk::Widget *> & pageWidgets, std::vector<std::string> & titlesTabs );

                    /**
                     * @fn	void NotebookContainer::on_nextButton_clicked();
                     * @brief	Handles next button clicked signals.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    void on_nextButton_clicked();


                    /**
                     * @fn	void NotebookContainer::on_backButton_clicked();
                     * @brief	Handles back button clicked signals.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    void on_backButton_clicked();

                    /**
                     * @fn	void NotebookContainer::on_notebook_switch_page(GtkNotebookPage* page, guint page_num);
                     * @brief	Executes the notebook switch page action.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	page			If non-null, the page.
                     * @param	page_num	The page number.
                     */
                    void on_notebook_switch_page ( GtkNotebookPage * page, guint page_num );

                    void_signal m_pages_viewed;   ///< The pages viewed signal

                    /**
                     * @fn	void NotebookContainer::switch_page_connect();
                     * @brief	Switch page connect.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    void switch_page_connect();

                private:

                    Gtk::Notebook * notebook;	///< The notebook
                    Gtk::Label * pageLabel;	///< The page label
                    Gtk::VBox * notebox; ///< The notebox
                    Gtk::HBox * bottomBox; ///< The bottom box
                    Gtk::HBox * buttonsBox;	///< The buttons box
                    Smartmatic::GUI::Widgets::SmartmaticButton * nextButton; ///< The next button
                    Smartmatic::GUI::Widgets::SmartmaticButton * backButton; ///< The back button
                    std::vector<Gtk::Widget *> pageList; ///< List of pages

                    /**
                     * @fn	void NotebookContainer::SetPageLabelText();
                     * @brief	Sets page label text.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    void SetPageLabelText();

                    /**
                     * @fn	void NotebookContainer::RefreshMap();
                     * @brief	Refresh map.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    void RefreshMap();

                    std::string pageLabelText;	///< The page label text

                    std::map<int, int> pagesNotShown; ///< The pages not shown
                    bool enableMap; ///< true to enable, false to disable the map
            };
        }
    }
}

#endif // _NOTEBOOK_CONTAINER_H_
