/**
 * @file smartmatic-entry-divided.cc
 * @brief Body smartmatic entry divided
 * @date 27/04/2011
 * @author Juan Delgado
 */

#include "smartmatic-entry-divided.h"
#include "smartmatic-entry.h"
#include "Configuration/s-m-t-t-g-u-i-conf-manager.h"

using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::GUI::Operation;

#define GDK_Key_Shift_L_ 0xffe1
#define GDK_Key_ISO_Left_Tab_ 0xfe20
#define GDK_Key_Tab_ 0xff09
#define GDK_Key_Return_ 0xff0d

SmartmaticEntryDivided::SmartmaticEntryDivided ( int divisions, int maxChar, int width, int height, bool visibility )
    : Glib::ObjectBase ( "SmartmaticEntryDivided" ), Smartmatic::GUI::Widgets::SmartmaticBasicHBox ( )
{
    this->visibility=visibility;
    this->divisions = divisions;
    this->maxChar = maxChar;
    this->currentEntry = 0;
    this->m_hasFocus = false;
    current.clear();
    entriesVector.clear();
    inputType = Smartmatic::GUI::Configuration::SMTTGUIConfigurationManager::getInstance()->getEntryConfiguration();

    for ( int i = 0; i < divisions; i++ )
    {
        SmartmaticEntry * smttEntry = manage ( new SmartmaticEntry() );
        smttEntry->setMax ( maxChar );
        smttEntry->signal_key_release_event().connect ( sigc::mem_fun ( *this, &SmartmaticEntryDivided::on_my_insert_at_cursor ) );
        smttEntry->signal_focus_in_event().connect ( sigc::mem_fun ( *this, &SmartmaticEntryDivided::on_my_focus_cursor ) );
        smttEntry->signal_focus_out_event().connect ( sigc::mem_fun ( *this, &SmartmaticEntryDivided::on_my_focus_out_cursor ) );
        smttEntry->set_visibility ( visibility );
        smttEntry->set_size_request ( width, height );

        g_signal_connect ( ( gpointer ) smttEntry->gobj(), "paste-clipboard", G_CALLBACK ( SmartmaticEntryDivided::on_clipboard_received ), this );

        if ( inputType != Smartmatic::GUI::Configuration::EntryPassPhraseConfiguration::CONFIGURATION_CASESENSITIVE )
        {
            smttEntry->property_caps_lock_warning().set_value ( false );
        }

        pack_start ( *smttEntry, false, false );
        entriesVector.push_back ( smttEntry );
    }

    if ( divisions > 0 )
    {
        iter = entriesVector.begin();
        ( *iter )->grab_focus();
    }

    set_spacing ( 7 );
    show_all_children();
}

bool SmartmaticEntryDivided::on_my_insert_at_cursor ( GdkEventKey * event )
{
    if(event->keyval == GDK_Key_Tab_ || event->keyval == GDK_Key_ISO_Left_Tab_ || event->keyval == GDK_Key_Shift_L_)
    {
       return true;
    }

    if (event->keyval == GDK_Key_Return_ && isEntryFull())
    {
        m_signal.emit();
    }

    if ( currentEntry < divisions - 1 )
    {
        Glib::ustring toCompare = entriesVector[currentEntry]->get_text();

        if ( ( int ) toCompare.size() == maxChar )
        {
            entriesVector[currentEntry + 1]->grab_focus();
        }
    }

    return true;
}

bool SmartmaticEntryDivided::on_my_focus_cursor ( GdkEventFocus * event )
{
    for ( unsigned int i = 0; i < entriesVector.size(); i++ )
    {
        if ( entriesVector[i]->is_focus() )
        {
            currentEntry = i;
            entriesVector[i]->set_visibility(true);
            m_hasFocus = true;
            break;
        }
    }

    return true;
}

bool SmartmaticEntryDivided::on_my_focus_out_cursor ( GdkEventFocus * event )
{
    entriesVector[currentEntry]->set_visibility(visibility);
    return false;
}

SmartmaticEntryDivided::~SmartmaticEntryDivided()
{
}

Glib::ustring SmartmaticEntryDivided::GetText( bool keepCase )
{
    Glib::ustring toReturn = "";

    for ( iter = entriesVector.begin(); iter != entriesVector.end(); ++iter )
    {
        toReturn.append ( ( *iter )->get_text() );
    }

    if ( keepCase == false )
    {
        if ( inputType == Smartmatic::GUI::Configuration::EntryPassPhraseConfiguration::CONFIGURATION_LOWERCASE )
        {
            toReturn = toReturn.lowercase();
        }
        else if ( inputType == Smartmatic::GUI::Configuration::EntryPassPhraseConfiguration::CONFIGURATION_UPPERCASE )
        {
            toReturn = toReturn.uppercase();
        }
    }

    return toReturn;
}

void SmartmaticEntryDivided::AddData( Glib::ustring data )
{
    if ( currentEntry < divisions )
    {
        entriesVector[ currentEntry ]->set_visibility( true );
        Glib::ustring currentEntryData = entriesVector[ currentEntry ]->get_text();

        if ( currentEntryData.size() < ( unsigned int ) maxChar )
        {
            currentEntryData.append( data );
            entriesVector[ currentEntry ]->set_text( currentEntryData );

            if ( ( currentEntryData.size() == ( unsigned int ) maxChar ) && ( currentEntry < divisions - 1 ) )
                entriesVector[ currentEntry ]->set_visibility( false );
        }
        else
        {
            entriesVector[ currentEntry ]->set_visibility( false );
            ++currentEntry;

            if ( currentEntry < divisions )
            {
                entriesVector[ currentEntry ]->set_visibility( true );
                entriesVector[ currentEntry ]->set_text( data );
            }
        }
    }
}

void SmartmaticEntryDivided::Clean()
{
    for ( iter = entriesVector.begin(); iter != entriesVector.end(); ++iter )
    {
        ( *iter )->set_text ( "" );
    }
}

int SmartmaticEntryDivided::getCurrentPos()
{
    return currentEntry;
}

int SmartmaticEntryDivided::getMaxChar()
{
    return maxChar;
}

std::vector <SmartmaticEntry * > & SmartmaticEntryDivided::getEntriesVector()
{
    return entriesVector;
}

void SmartmaticEntryDivided::on_clipboard_received ( GtkEntry * entry, gpointer user_data )
{
    if ( Gtk::Clipboard::get()->wait_is_text_available() )
    {
        SmartmaticEntryDivided * parent = static_cast< SmartmaticEntryDivided * > ( user_data );
        std::string text = Gtk::Clipboard::get()->wait_for_text();
        unsigned int current = parent->getCurrentPos();

        for ( unsigned int i = current; i < parent->getEntriesVector().size(); i++ )
        {
            if ( ( unsigned int ) text.length() >= ( unsigned int ) parent->getMaxChar() )
            {
                std::string input = text.substr ( 0, parent->getMaxChar() );
                gtk_entry_set_text ( parent->getEntriesVector() [i]->gobj(), input.c_str() );
                parent->getEntriesVector() [i]->grab_focus();
                text = text.substr ( parent->getMaxChar() );
            }
        }
    }
}

void SmartmaticEntryDivided::OnKeyDeletePress()
{
    if(currentEntry >= (int)entriesVector.size())
    {
        currentEntry = entriesVector.size() - 1;
    }
    entriesVector[ currentEntry ]->set_visibility( true );
    std::string currentData = entriesVector[ currentEntry ]->get_text();

    if ( currentData.size() > 0 )
        currentData.erase( currentData.begin() + currentData.size() - 1 );

    entriesVector[ currentEntry ]->set_text( currentData );
}

void SmartmaticEntryDivided::focusFirstEntry()
{
    entriesVector[0]->focus();
}

bool SmartmaticEntryDivided::isEntryFull()
{
    return (GetText().size() == (maxChar * entriesVector.size()));
}

SmartmaticEntryDivided::EntryCompleteSignal SmartmaticEntryDivided::signalEntryComplete()
{
    return m_signal;
}

bool SmartmaticEntryDivided::hasFocus()
{
    return m_hasFocus;
}

void SmartmaticEntryDivided::setHasFocus( bool hasFocus )
{
    m_hasFocus = hasFocus;
}
