/**
 * @file smartmatic-entry-divided.h
 * @brief Header smartmatic entry divided
 * @date 27/04/2011
 * @author Juan Delgado
 */

#ifndef _SMARTMATIC_ENTRY_DIVIDED_H_
#define _SMARTMATIC_ENTRY_DIVIDED_H_

#include <gtkmm.h>
#include <Configuration/SmartmaticGUIConfiguration-schema.hxx>
#include <Widgets/SmartmaticBasicHBox.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {
            class SmartmaticEntry;

            /**
             * @class	SmartmaticEntryDivided
             * @brief	Class entry divided.
             * @author	Yadickson Soto
             * @date	27/03/2012
             */
            class SmartmaticEntryDivided: public Smartmatic::GUI::Widgets::SmartmaticBasicHBox
            {
                public:

                    /**
                     * @brief  Signal type when user press return key and the entry is full.
                     * @author manuel.arguelles
                     * @date   Jul  5, 2017
                     */
                    typedef sigc::signal<void> EntryCompleteSignal;

                public:

                    /**
                     * @fn	SmartmaticEntryDivided::SmartmaticEntryDivided(int divisions, int maxChar, int width = -1,
                     * 			bool visibility = false);
                     * @brief	Constructor.
                     * @author	Yadickson Soto
                     * @date	27/03/2012
                     * @param	divisions 	Total divisions.
                     * @param	maxChar			maximun chars in every entry.
                     * @param	width				width of every entry.
                     * * @param height              height of every entry.
                     * @param	visibility	flag visibility char.
                     */
                    SmartmaticEntryDivided ( int divisions, int maxChar, int width = -1, int height = -1, bool visibility = false );
                    /**
                     * @fn	virtual SmartmaticEntryDivided::~SmartmaticEntryDivided();
                     * @brief	Class destructor.
                     * @author	Yadickson Soto
                     * @date	27/03/2012
                     */
                    virtual ~SmartmaticEntryDivided();
                    /**
                     * @fn	Glib::ustring SmartmaticEntryDivided::GetText();
                     * @brief	Method getter text.
                     * @author	Yadickson Soto
                     * @date	27/03/2012
                     * @return	text.
                     * @param keepCase Whether to keep the characters case.
                     */
                    Glib::ustring GetText( bool keepCase = false );
                    /**
                     * @fn  Glib::ustring SmartmaticEntryDivided::AddData( Glib::ustring data );
                     * @brief   Method to add data.
                     * @author  Eduardo Fernández
                     * @date    25/09/2017
                     */
                    void AddData( Glib::ustring data );
                    /**
                     * @fn	void SmartmaticEntryDivided::Clean();
                     * @brief	Method clear all entry.
                     * @author	Yadickson Soto
                     * @date	27/03/2012
                     */
                    void Clean();
                    /**
                     * @fn	int SmartmaticEntryDivided::getCurrentPos();
                     * @brief	Method getter current entry pos.
                     * @author	Yadickson Soto
                     * @date	27/03/2012
                     * @return	current entry pos.
                     */
                    int getCurrentPos();
                    /**
                     * @fn	int SmartmaticEntryDivided::getMaxChar();
                     * @brief	Method getter max char.
                     * @author	Yadickson Soto
                     * @date	27/03/2012
                     * @return	max char.
                     */
                    int getMaxChar();
                    /**
                     * @fn	std::vector<SmartmaticEntry*> & SmartmaticEntryDivided::getEntriesVector();
                     * @brief	Method getter entries.
                     * @author	Yadickson Soto
                     * @date	27/03/2012
                     * @return	entries.
                     */
                    std::vector<SmartmaticEntry *> & getEntriesVector();
                    /**
                     * @fn	void SmartmaticEntryDivided::focusFirstEntry();
                     * @brief	focus the first entry.
                     * @author	Juan Valera
                     * @date	23/02/2017
                     */
                    void focusFirstEntry();

                    /**
                     * @brief  Emitted when the user pressed return key and the entry is full.
                     * @author manuel.arguelles
                     * @date   Jul  5, 2017
                     */
                    EntryCompleteSignal signalEntryComplete();

                    /**
                     * @brief  Determines whether the password entry has the focus.
                     * @author eduardo.fernandez
                     * @date   25/09/2017
                     */
                    bool hasFocus();

                    /**
                     * @brief  Sets the password focus.
                     * @author eduardo.fernandez
                     * @date   25/09/2017
                     */
                    void setHasFocus( bool hasFocus );

                    /**
                     * @brief  Method manager event key delete press received.
                     * @author eduardo.fernandez
                     * @date   25/09/2017
                     */
                    void OnKeyDeletePress();

                protected:

                    Glib::ustring current; ///< current text
                    bool visibility;	///< The visibility
                    int divisions;	///< The maximum
                    int maxChar;	///< The minimum
                    int currentEntry; ///< current cursor pos

                    std::vector<SmartmaticEntry *> entriesVector; ///< entries
                    std::vector<SmartmaticEntry *>::iterator iter; ///< iterator

                    Smartmatic::GUI::Configuration::EntryPassPhraseConfiguration::Value inputType; ///< input type

                    /**
                     * @fn	bool SmartmaticEntryDivided::on_my_insert_at_cursor(GdkEventKey * event);
                     * @brief	Method manager insert cursor in entry.
                     * @author	Yadickson Soto
                     * @date	27/03/2012
                     * @param	event	event cursor entry pos.
                     * @return	result event (true)
                     */
                    bool on_my_insert_at_cursor ( GdkEventKey * event );
                    /**
                     * @fn	bool SmartmaticEntryDivided::on_my_focus_cursor(GdkEventFocus * event);
                     * @brief	Method manager getter focus entry.
                     * @author	Yadickson Soto
                     * @date	27/03/2012
                     * @param	event	event cursor entry pos.
                     * @return	result event (true)
                     */
                    bool on_my_focus_cursor ( GdkEventFocus * event );
                    /**
                     * @fn	bool SmartmaticEntryDivided::on_my_focus_out_cursor(GdkEventFocus * event);
                     * @brief	Method manager out focus entry.
                     * @author	Juan Valera
                     * @date	24/02/2017
                     * @param	event	event cursor entry pos.
                     * @return	result event (true)
                     */
                    bool on_my_focus_out_cursor ( GdkEventFocus * event );
                    /**
                     * @fn	static void SmartmaticEntryDivided::on_clipboard_received(GtkEntry *entry,
                     * 			gpointer user_data);
                     * @brief	Method manager event clipboard received.
                     * @author	Yadickson Soto
                     * @date	27/03/2012
                     * @param	entry		 	element entry.
                     * @param	user_data	user data.
                     */
                    static void on_clipboard_received ( GtkEntry * entry, gpointer user_data );

                private:

                    /**
                     * @brief  Check if the entry text is full.
                     * @author manuel.arguelles
                     * @date   Jul  5, 2017
                     * @return true if the entry is full, false otherwise.
                     */
                    bool isEntryFull();

                private:

                    EntryCompleteSignal m_signal;
                    bool m_hasFocus;
            };
        }
    }
}

#endif // _SMARTMATIC_ENTRY_DIVIDED_H_
