/**
 * @file	smartmatic-entry.h
 * @brief	Declares the smartmatic entry class.
 */

/* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * smartmaticgui
 * Copyright (C) Juan Delgado 2011 <juan.delgado@smartmatic.com>
 *
 * smartmaticgui is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * smartmaticgui is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _SMARTMATIC_ENTRY_H_

/**
 * @def	_SMARTMATIC_ENTRY_H_
 * @brief	Gets the smartmatic entry h.
 * @author	Luiggi.mendez
 * @date	27/04/2011
 */

#define _SMARTMATIC_ENTRY_H_

#include <gtkmm.h>
#include "ISmartmaticText.hxx"
#include <Operation/AccessibleNameWidget.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {

            /**
             * @class	SmartmaticEntry
             * @brief	Smartmatic entry.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class SmartmaticEntry: public Gtk::Entry, public ISmartmaticText,
                public Smartmatic::GUI::Operation::AccessibleNameWidget
            {
                public:

                    /**
                     * @enum	EntryType
                     * @brief	Values that represent EntryType.
                     */
                    enum EntryType
                    {
                        Alphabetic, ///< Value that represent an alphabetic entry.
                        Decimal,	///< Value that represent a decimal entry.
                        AlphaNumeric,   ///< Value that represent an alphanumeric entry.
                        All, ///< Value that represent whatever entry.
                        AllWithoutNumber, ///< Value that represent whatever entry without number.
                        AllWithoutAlphabetic, ///< Value that represent whatever entry without alphabetic.
                        Mask ///< Mask
                    };

                    /**
                     * @fn	SmartmaticEntry::SmartmaticEntry(EntryType type = All);
                     * @brief	Constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	type	The type.
                     */
                    explicit SmartmaticEntry ( EntryType type = All );

                    /**
                     * @fn	SmartmaticEntry::SmartmaticEntry(BaseObjectType* cobject,
                     * 			const Glib::RefPtr<Gtk::Builder>& refBuilder=Gtk::Builder::create());
                     * @brief	Constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	cobject			If non-null, the cobject.
                     * @param	refBuilder	The reference builder.
                     */
                    SmartmaticEntry ( BaseObjectType * cobject, const Glib::RefPtr<Gtk::Builder> & refBuilder = Gtk::Builder::create() );

                    /**
                     * @fn	virtual SmartmaticEntry::~SmartmaticEntry();
                     * @brief	Finaliser.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    virtual ~SmartmaticEntry();

                    /**
                     * @fn	void SmartmaticEntry::SetType(EntryType type);
                     * @brief	Set entry type.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	type	Type of.
                     */
                    void SetType ( EntryType type );

                    /**
                     * @fn	void SmartmaticEntry::SetNumber(int number);
                     * @brief	Sets a number.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	number	Number of.
                     */
                    void SetNumber ( int number );

                    /**
                     * @fn	bool SmartmaticEntry::GetNumber(int& number);
                     * @brief	Gets a number.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	number	Number of.
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool GetNumber ( int & number );

                    /**
                     * @typedef	sigc::signal<void> text_inserted
                     * @brief	Defines an alias representing the text inserted.
                     */
                    typedef sigc::signal<void> text_inserted;

                    /**
                     * @fn	text_inserted SmartmaticEntry::signal_inserted();
                     * @brief	Raises the inserted event.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @return	.
                     */
                    text_inserted signal_inserted();

                    /**
                     * @fn	void SmartmaticEntry::setPosition(unsigned int pos);
                     * @brief	Sets a position.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	pos	The position.
                     */
                    void setPosition ( unsigned int pos );

                    /**
                     * @fn	unsigned int SmartmaticEntry::getPosition();
                     * @brief	Gets the position.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @return	The position.
                     */
                    unsigned int getPosition();

                    /**
                     * @fn	virtual void SmartmaticEntry::setText(std::string text);
                     * @brief	Method setter text.
                     * @author	Yadickson Soto
                     * @date	03/06/2012
                     * @param	text	text.
                     */
                    virtual void setText ( std::string text );
                    /**
                     * @fn	virtual std::string SmartmaticEntry::getText();
                     * @brief	Method getter text.
                     * @author	Yadickson Soto
                     * @date	03/06/2012
                     * @return	text.
                     */
                    virtual std::string getText();
                    /**
                     * @fn	virtual void SmartmaticEntry::setMax(int max);
                     * @brief	Set entry max elements.
                     * @author	Yadickson Soto
                     * @date	04/05/2012
                     * @param	max	max.
                     */
                    virtual void setMax ( int max );
                    /**
                     * @fn	virtual void SmartmaticEntry::setBGError();
                     * @brief	Method set background error.
                     * @author	Yadickson Soto
                     * @date	03/06/2012
                     */
                    virtual void setBGError();
                    /**
                     * @fn	virtual void SmartmaticEntry::setBGNormal();
                     * @brief	method set bg normal.
                     * @author	Yadickson Soto
                     * @date	03/06/2012
                     */
                    virtual void setBGNormal();
                    /**
                     * @fn	virtual void focus();
                     * @brief	Focus component
                     * @author	Yadickson Soto
                     * @date	13/09/2013
                     */
                    virtual void focus();

                protected:
                    text_inserted m_signal_inserted;   ///< The ballot clicked

                    /**
                     * @fn	void SmartmaticEntry::initialize();
                     * @brief	method initialize.
                     * @author	Yadickson Soto
                     * @date	03/06/2012
                     */
                    void initialize();
                    /**
                     * @fn	virtual void SmartmaticEntry::on_insert_text(const Glib::ustring& text, int* position);
                     * @brief	Executes the insert text action.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	text			The text.
                     * @param	position	If non-null, the position.
                     */

                    virtual void on_insert_text ( const Glib::ustring & text, int * position );

                    /**
                     * @fn	bool SmartmaticEntry::DeleteLeftZero();
                     * @brief	Deletes the left zero.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool DeleteLeftZero();

                    EntryType type; ///< The type

                    /**
                     * @property	unsigned int pos
                     * @brief	Gets the position.
                     * @return	The position.
                     */
                    unsigned int pos;

                    /**
                     * @fn	bool SmartmaticEntry::on_focus_in_event(GdkEventFocus * event);
                     * @brief	Focus in event.
                     * @author	Juan.Delgado
                     * @date	08/07/2011
                     * @param	event	The event.
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool on_focus_in_event ( GdkEventFocus * event );
                    /**
                     * @fn	virtual bool SmartmaticEntry::on_button_press_event(GdkEventButton* event);
                     * @brief	Handles on button press event.
                     * @author	Juan.Delgado
                     * @date	23/07/2011
                     * @param	event	The event.
                     * @return	true if it succeeds, false if it fails.
                     */
                    virtual bool on_button_press_event ( GdkEventButton * event );

                    /**
                     * @fn	virtual bool SmartmaticEntry::on_key_press_event(GdkEventKey* event);
                     * @brief	Executes the key press event action.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @param	event	If non-null, the event.
                     * @return	true if it succeeds, false if it fails.
                     */
                    virtual bool on_key_press_event ( GdkEventKey * event );

                    /**
                     * @fn	void SmartmaticEntry::changeText();
                     * @brief	method validate change text.
                     * @author	Yadickson Soto
                     * @date	03/06/2012
                     */
                    void changeText();
                    /**
                     * @fn	void SmartmaticEntry::insertText(const Glib::ustring& text, int* position);
                     * @brief	method validate insert text.
                     * @author	Yadickson Soto
                     * @date	03/06/2012
                     * @param	text			text.
                     * @param	position	position.
                     */
                    void insertText ( const Glib::ustring & text, int * position );
                    /**
                     * @fn	void SmartmaticEntry::deleteText(int i, int j);
                     * @brief	method validate delete text.
                     * @author	Yadickson Soto
                     * @date	03/06/2012
                     * @param	i	position i
                     * @param	j	position j
                     */
                    void deleteText ( int i, int j );
            };
        }
    }
}

#endif // _SMARTMATIC_ENTRY_H_
