/**
 * @file	pop-up-message-window.cc
 * @brief	POP up message Windows Form.
 */

/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * smartmaticgui
 * Copyright (C) Juan Delgado 2011 <juan.delgado@smartmatic.com>
 *
 * smartmaticgui is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * smartmaticgui is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "pop-up-message-window.h"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/WindowsManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::GUI::Windows;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Operation;

#define LOGGER_PROJECT "Smartmatic.GUI.Windows.PopUpMessageWindow" ///< logger project name

#ifdef GETTEXT_PACKAGE
ISMTTLog * PopUpMessageWindow::logger = SMTTLogManager::GetLogger ( LOGGER_PROJECT, GETTEXT_PACKAGE ); ///< The logger
#else
ISMTTLog * PopUpMessageWindow::logger = SMTTLogManager::GetLogger ( LOGGER_PROJECT, "" ); ///< The logger
#endif

PopUpMessageWindow::PopUpMessageWindow()
    : BasicWindow ( false, Gtk::WINDOW_POPUP )
{
    child = NULL;
    timerToClose = 0;
    label = NULL;
    eventbox = NULL;
}

PopUpMessageWindow::PopUpMessageWindow (
    Glib::ustring text,
    LabelConfiguration & labelConf,
    int timerToClose,
    int opacity )
    : BasicWindow ( false, Gtk::WINDOW_POPUP )
{
    child = NULL;
    timerToClose = 0;
    setText ( text );
    hideTitle();
    setLabelConfig ( labelConf );
    refreshText ( text );
    setTimerToClose ( timerToClose );
    setOpacity ( opacity );
}

PopUpMessageWindow::PopUpMessageWindow ( Gtk::Widget * child, int timerToClose )
    : BasicWindow ( false, Gtk::WINDOW_POPUP )
{
    timerToClose = 0;
    eventbox = NULL;
    label = NULL;
    setTimerToClose ( timerToClose );
    setContainer ( child );
}

PopUpMessageWindow::~PopUpMessageWindow()
{
    conn_hide.disconnect();
    this->remove();

    if ( eventbox )
    {
        delete eventbox;
        eventbox = NULL;
    }

    if ( label )
    {
        delete label;
        label = NULL;
    }

    BasicWindow::dispose();
}

void PopUpMessageWindow::setContainer ( Gtk::Widget * child )
{
    this->child = child;
    setWidget ( child );
}

void PopUpMessageWindow::setText ( Glib::ustring text )
{
    if ( !eventbox )
    {
        eventbox = new Gtk::EventBox();
        eventbox->signal_button_press_event().connect ( sigc::mem_fun ( *this, &PopUpMessageWindow::hideWindow ) );
        eventbox->set_events ( Gdk::BUTTON_PRESS_MASK );
        eventbox->show();
    }

    if ( !label )
    {
        label = new Gtk::Label ( text );
        eventbox->remove();
        eventbox->add ( *label );
        setContainer ( eventbox );
    }

    label->set_text ( text );
}

void PopUpMessageWindow::refreshText ( Glib::ustring text )
{
    setText ( text );
    restart ();
}

void PopUpMessageWindow::setLabelConfig ( Smartmatic::GUI::Configuration::LabelConfiguration & labelConf )
{
    Smartmatic::GUI::Configuration::LabelConfiguration::SetLabelConfiguration ( label, labelConf );
}

void PopUpMessageWindow::setFontColorLabel ( Gdk::Color & color )
{
    label->modify_fg ( Gtk::STATE_NORMAL, color );
}

void PopUpMessageWindow::setBoxColorLabel ( Gdk::Color & color )
{
    eventbox->modify_bg ( Gtk::STATE_NORMAL, color );
}

void PopUpMessageWindow::on_show()
{
    set_modal ( false );
    set_keep_above ( true );
    set_can_focus ( true );
    restart ();
    BasicWindow::on_show();
}

bool PopUpMessageWindow::hideWindow ( GdkEventButton * event )
{
    conn_hide.disconnect();
    QuitDialog();
    return true;
}

void PopUpMessageWindow::ShowDialog()
{
    logger->Debug ( "Show popup window" );

    restart ();

    Gtk::Window * parent = WindowsManager::getInstance()->current();

    if ( parent )
    {
        logger->Debug ( "Set parent window" );
        set_parent_window ( parent->get_window() );
    }

    this->show();
}

void PopUpMessageWindow::QuitDialog()
{
    logger->Debug ( "Hide popup window" );

    hide();
}

bool PopUpMessageWindow::myHide()
{
    QuitDialog();
    return false;
}

void PopUpMessageWindow::setTimerToClose ( int timerToClose )
{
    this->timerToClose = timerToClose;
}

void PopUpMessageWindow::setOpacity ( int opacity )
{
    this->set_opacity ( opacity / 100.0 );
}

void PopUpMessageWindow::restart()
{
    conn_hide.disconnect();

    if ( timerToClose > 0 )
    {
        conn_hide = Glib::signal_timeout().connect ( sigc::mem_fun ( *this, &PopUpMessageWindow::myHide ), timerToClose );
    }
}
