/**
 * @file	s-a-e-s-g-u-i-conf-manager.cc
 * @brief	saes gui conf manager class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "Environment/Configurations/resource-path-provider.h"
#include "s-a-e-s-g-u-i-conf-manager.h"
#include <System/ZenityWrapper.hxx>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Configuration/BasicWindowConfManager.hxx>
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <System/IO/File.h>
#include <Configuration/ConfigurationBase-schema.hxx>
#include "Configuration/configuration-persistence-manager.h"
#include <System/Runtime/ExitProcess.hxx>
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>
#include <System/OptParser.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::VotingCenters;
using namespace Smartmatic::System;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Runtime;

/**
 * @typedef	MenuOption XMLMenuOption
 * @brief	Defines an alias representing the XML menu option.
 */
typedef Smartmatic::SAES::Configuration::MenuOption XMLMenuOption;

SAESGUIConfigurationManager * SAESGUIConfigurationManager::instance = NULL;

ISMTTLog * SAESGUIConfigurationManager::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager", GETTEXT_PACKAGE );

SAESGUIConfigurationManager::SAESGUIConfigurationManager()
{
    configuration = NULL;
}

SAESGUIConfigurationManager::~SAESGUIConfigurationManager()
{
    if ( configuration != NULL )
    {
        delete ( configuration );
        configuration = NULL;
    }

    instance = NULL;
}

SaesGUIConfiguration * SAESGUIConfigurationManager::getConfiguration()
{
    if ( configuration == NULL )
    {
        initialize();
    }

    return configuration;
}

void SAESGUIConfigurationManager::initialize()
{
    try
    {
        std::string semiFile ( ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "SAESGUI" ) );
        std::istringstream ss ( semiFile );
        configuration = new SaesGUIConfiguration ( * parseSaesGUIConfiguration ( ss, ConfigurationTypes::Flags::dont_validate ).get() );
    }
    catch ( const ::xsd::cxx::tree::exception< char > & e )
    {
        std::stringstream ss;
        ss << e;
        ZenityWrapper::showError ( "Config SAESGUI Error \n\n" + ss.str(), "Error" );
        ExitProcess::getInstance()->exitProcess();
    }
}

void SAESGUIConfigurationManager::loadDefault()
{
    FunctionalityWizardFlowConfiguration functionalityWizardConfig = createFunctionalityWizardConfig();
    FormsCommonControlsConfigurations formCommonControl = createCommonControlsConfigurations();

    DeviceConfiguration barcodeDevice ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_BARCODE );
    DeviceConfiguration smartcardDevice ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_SMARTCARD );
    DeviceConfiguration printerDevice ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_PRINTER );
    Smartmatic::SAES::Configuration::EventSoundsConfiguration eventSounds;

    SaesGUIConfiguration configSAESGUI;

    RecountFunctionalityConfiguration recountFunctionalityConfiguration;
    recountFunctionalityConfiguration.setExtendedView ( false );
    recountFunctionalityConfiguration.setLevelOfSelection ( 3 );
    recountFunctionalityConfiguration.setZerosFollowSelection ( true );
    recountFunctionalityConfiguration.setZerosReplacementInTotals ( "X" );
    recountFunctionalityConfiguration.setRecountExitButtonXSize ( 220 );
    recountFunctionalityConfiguration.setRecountStatTableColSpacing ( 10 );
    recountFunctionalityConfiguration.setRecountStatTableRowSpacing ( 10 );
    recountFunctionalityConfiguration.setRecountStatTableColHeadersExtendedViewSize ( 82 );
    recountFunctionalityConfiguration.setRecountStatTableColHeadersSimpleViewSize ( 120 );
    recountFunctionalityConfiguration.setRecountStatSessionNameToFoundDataSpacing ( 50 );
    recountFunctionalityConfiguration.setRecountFoundToNotFoundDataSpacing ( 50 );
    recountFunctionalityConfiguration.setRecountNotFoundDataToTotalSpacing ( 50 );

    CustomRemarkDiagnoseConfiguration customRemarkDiagnoseConfiguration;
    customRemarkDiagnoseConfiguration.setTextBoxWidth ( 800 );
    customRemarkDiagnoseConfiguration.setTextBoxMaxLength ( 1024 );
    customRemarkDiagnoseConfiguration.setTextBoxMaxLines ( 0 );

    SaesGUIMachineModelConfiguration::MachineModelSequence machineModelSequence;
    Smartmatic::SAES::Configuration::MachineModelType machineModelVIU;
    machineModelVIU.setId( "PM-VIU805" );
    Smartmatic::SAES::Configuration::LoginWidgetEntryConfigurationType loginWidgetEntryConfigurationVIU;
    loginWidgetEntryConfigurationVIU.setEntryWidth( 66 );
    loginWidgetEntryConfigurationVIU.setEntryHeight( 20 );
    machineModelVIU.setLoginWidgetEntryConfiguration( loginWidgetEntryConfigurationVIU );
    machineModelVIU.setCustomRemarkDiagnoseTextBoxHeight( 50 );
    machineModelSequence.push_back( machineModelVIU );

    Smartmatic::SAES::Configuration::MachineModelType machineModelPMA;
    machineModelPMA.setId( "PM-NOTEBOOK-A" );
    Smartmatic::SAES::Configuration::LoginWidgetEntryConfigurationType loginWidgetEntryConfigurationPMA;
    loginWidgetEntryConfigurationPMA.setEntryWidth( 55 );
    loginWidgetEntryConfigurationPMA.setEntryHeight( 16 );
    machineModelPMA.setLoginWidgetEntryConfiguration( loginWidgetEntryConfigurationPMA );
    machineModelPMA.setCustomRemarkDiagnoseTextBoxHeight( 300 );
    machineModelSequence.push_back( machineModelPMA );

    Smartmatic::SAES::Configuration::MachineModelType machineModelPMB;
    machineModelPMB.setId( "PM-NOTEBOOK-B" );
    Smartmatic::SAES::Configuration::LoginWidgetEntryConfigurationType loginWidgetEntryConfigurationPMB;
    loginWidgetEntryConfigurationPMB.setEntryWidth( 55 );
    loginWidgetEntryConfigurationPMB.setEntryHeight( 16 );
    machineModelPMB.setLoginWidgetEntryConfiguration( loginWidgetEntryConfigurationPMB );
    machineModelPMB.setCustomRemarkDiagnoseTextBoxHeight( 300 );
    machineModelSequence.push_back( machineModelPMB );

    Smartmatic::SAES::Configuration::MachineModelType machineModelVM;
    machineModelVM.setId( "VM-SAES3377" );
    Smartmatic::SAES::Configuration::LoginWidgetEntryConfigurationType loginWidgetEntryConfigurationVM;
    loginWidgetEntryConfigurationVM.setEntryWidth( 55 );
    loginWidgetEntryConfigurationVM.setEntryHeight( 16 );
    machineModelVM.setLoginWidgetEntryConfiguration( loginWidgetEntryConfigurationVM );
    machineModelVM.setCustomRemarkDiagnoseTextBoxHeight( 300 );
    machineModelSequence.push_back( machineModelVM );

    SaesGUIMachineModelConfiguration saesGUIMachineModelConfiguration;
    saesGUIMachineModelConfiguration.setMachineModel( machineModelSequence );

    // reusable - down
    XMLLabelConfiguration recountLabelsDefaultConfig;
    recountLabelsDefaultConfig.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    recountLabelsDefaultConfig.setLabel_VerticalPadding ( 1 );
    recountLabelsDefaultConfig.setLabel_HorizontalPadding ( 1 );
    recountLabelsDefaultConfig.setLabel_Family ( "Verdana" );
    recountLabelsDefaultConfig.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    recountLabelsDefaultConfig.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    recountLabelsDefaultConfig.setLabel_Size ( 12 );

    recountFunctionalityConfiguration.setRecountScanPageTotalLabelConfig ( recountLabelsDefaultConfig );
    recountFunctionalityConfiguration.setRecountScanInfoHeader ( recountLabelsDefaultConfig );
    recountFunctionalityConfiguration.setRecountScanInfoLabel ( recountLabelsDefaultConfig );
    recountFunctionalityConfiguration.setRecountStatExtendedViewHeader ( recountLabelsDefaultConfig );
    recountFunctionalityConfiguration.setRecountStatColumnHeader ( recountLabelsDefaultConfig );
    recountFunctionalityConfiguration.setRecountStatSessionLabel ( recountLabelsDefaultConfig );
    recountFunctionalityConfiguration.setRecountStatSessionTotalsLabel ( recountLabelsDefaultConfig );
    recountFunctionalityConfiguration.setRecountConfirmInfoEntryLabel ( recountLabelsDefaultConfig );
    recountFunctionalityConfiguration.setRecountConfirmInfoLabel ( recountLabelsDefaultConfig );

    // reusable - down
    ButtonConfiguration recountButtonsDefault;
    recountButtonsDefault.setButtonWidth ( 200 );
    recountButtonsDefault.setButtonHeight ( 60 );
    recountButtonsDefault.setFontSize ( 18 );

    ButtonConfiguration externalPrintButton;
    externalPrintButton.setButtonWidth ( 200 );
    externalPrintButton.setButtonHeight ( 60 );
    externalPrintButton.setFontSize ( 18 );

    ExternalPrintWizardWidgetConfiguration externalPrintWizardWidgetDefault;
    externalPrintWizardWidgetDefault.setButtonConf ( externalPrintButton );

    recountFunctionalityConfiguration.setRecountConfirmButton ( recountButtonsDefault );

    configSAESGUI.setCustomRemarkDiagnoseConfiguration ( customRemarkDiagnoseConfiguration );

    configSAESGUI.setSaesGUIMachineModelConfiguration( saesGUIMachineModelConfiguration );

    configSAESGUI.setRecountFunctionalityConfiguration ( recountFunctionalityConfiguration );

    configSAESGUI.setExternalPrintWizardWidgetConfiguration ( externalPrintWizardWidgetDefault );

    XMLMenuOption reportProcess;
    XMLMenuOption votingMachine;
    XMLMenuOption mainteinanceMenu;
    XMLMenuOption validationMenu;
    XMLMenuOption openElection;
    XMLMenuOption closeElection;
    XMLMenuOption logReport;
    XMLMenuOption diagnosis;
    XMLMenuOption cardActivators;
    XMLMenuOption alarmBoxDiagnosis;
    XMLMenuOption smartcardReaderDiagnosis;
    XMLMenuOption calibrationDiagnosis;
    XMLMenuOption barcodeDiagnosis;
    XMLMenuOption allDiagnosis;
    XMLMenuOption flashDiagnosis;
    XMLMenuOption ableDiagnosis;
    XMLMenuOption printerDiagnosis;
    XMLMenuOption changeHour;
    XMLMenuOption changeLanguage;
    XMLMenuOption vmInstallation;
    XMLMenuOption voteProcessDemo;
    XMLMenuOption printPreview;
    XMLMenuOption votingExperienceValidation;
    XMLMenuOption closeVotation;
    XMLMenuOption reportPrinting;
    XMLMenuOption shutDownMenuOption;
    XMLMenuOption activationConsolidation;
    XMLMenuOption shutDownParentMenuOption;

    MenuOptions optionsProcessReport;
    MenuOptions optionsVotingMachine;
    MenuOptions optionsMaintenance;
    MenuOptions optionsValidations;
    MenuOptions optionsDiagnosis;
    MenuOptions optionsCardActivators;

    MenuOptions::OptionSequence seqOptionsReportingProcess;
    MenuOptions::OptionSequence seqOptionsVotingMachine;
    MenuOptions::OptionSequence seqOptionsMaintenance;
    MenuOptions::OptionSequence seqOptionsValidations;
    MenuOptions::OptionSequence seqOptionsDiagnosis;
    MenuOptions::OptionSequence seqOptionsCardActivators;

    MenuOptions optionsRoot;
    MenuOptions::OptionSequence seqRoot;
    XMLMenuOption rootPresidentMachineOption;
    XMLMenuOption rootVotingMachineOption;
    MenuWidgetConfiguration pollBookMachineConfiguration;
    MenuWidgetConfiguration votingMachineConfiguration;
    VotingExperienceConfiguration votingExperienceConfiguration;
    VotingWindowConfiguration votingWindowConfiguration;
    CustomAtributtesWindowConfiguration customAtributtesWindowConfiguration;

    VotingExperienceLabelConfiguration votingExperienceLabelConfiguration;
    InstallationWizardConfiguration installWizardConfig;
    PMWidgetConfiguration pmWidgetConfig;
    FontConfiguration votingWindowFont;
    PartySelectionConfiguration partySelection;
    PartyWidgetConfiguration partyWidget;
    WidgetDisplayType displayTypeImage ( DisplayType::CONFIGURATION_IMAGE_RESIZE, 0, 0.5 );
    BallotOptionSelectionConfiguration ballotOptionSelection;
    BallotOptionWidgetConfiguration ballotOptionWidget;
    FontConfiguration ballotOptionFont;
    WidgetDisplayType displayTypeText ( DisplayType::CONFIGURATION_TEXT, 0, 0 );
    ColorConfiguration selectedColor;
    ColorConfiguration unSelectedColor;
    GeneralDiagnosisConfiguration generalDiag;
    SmartcardReaderDiagnosisConfiguration screaderDiagnosisConf;
    PrinterDiagnosisConfiguration printerDiagnosisConf;
    BarcodeDiagnosisConfiguration barcodeDiagnosisConf;

    TouchScreenCalibrationConfiguration touchDiagConfig;
    AllDiagnosisConfiguration allDiagnosisConf;
    AuthenticationWindowConfiguration authWindowConf;

    XMLLabelConfiguration diagnosisLabelConfiguration;
    XMLLabelConfiguration exitMessageLabelConfiguration;

    AutomaticWindowConfigurationForOfficialMode::DiagnosticToApplySequence seqDiagnosticToApplyOM;
    AutomaticWindowConfigurationForOfficialMode::DiagnosticToApplySequence seqAutoDiagnosticToApplyOM;
    AutomaticWindowConfigurationForOfficialMode autoDiagnosisConfOM;
    AutomaticWindowConfigurationForRecountMode::DiagnosticToApplySequence seqDiagnosticToApplyRM;
    AutomaticWindowConfigurationForRecountMode::DiagnosticToApplySequence seqAutoDiagnosticToApplyRM;
    AutomaticWindowConfigurationForRecountMode autoDiagnosisConfRM;
    CardActivatorFunctionalityConfiguration activationCardConf;
    VoteDemoFunctionalityConfiguration voteDemoConf;

    ColorConfiguration selectedColorCardActiv;
    ColorConfiguration unSelectedColorCardActiv;
    ReferendumConfiguration referendumConfiguration;

    WaitingForCardWindowConfiguration waitingCardWindow;
    FunctionalitiesConfiguration funcConfiguration;
    FunctionalitiesConfiguration::FunctionalityConfigurationSequence funcConfigSequence;
    OpenElectionFunctionalityConfiguration openElectionConfig;
    CloseElectionFunctionalityConfiguration closeElectionconfig;
    LogReportFunctionalityConfiguration logReportFunconfig;
    VoteConsolidationFunctionalityConfiguration voteConsolidationFuncConfig;
    PrintPreviewFunctionalityConfiguration printPreviewFuncConfig;
    XMLLabelConfiguration resultErrorLabelConfiguration;
    MemoryStateWidgetConfiguration memoryStateWidgetConfiguration;
    ButtonConfiguration memoryStateButtonConf;
    memoryStateButtonConf.setButtonWidth ( 1000 );
    memoryStateButtonConf.setButtonHeight ( 65 );
    memoryStateButtonConf.setFontSize ( 16 );
    XMLLabelConfiguration memoryStateTitleFontConf;
    memoryStateTitleFontConf.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    memoryStateTitleFontConf.setLabel_VerticalPadding ( 1 );
    memoryStateTitleFontConf.setLabel_HorizontalPadding ( 1 );
    memoryStateTitleFontConf.setLabel_Family ( "Arial" );
    memoryStateTitleFontConf.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    memoryStateTitleFontConf.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    memoryStateTitleFontConf.setLabel_Size ( 20 );
    memoryStateWidgetConfiguration.setButtonDelay ( 4000 );
    memoryStateWidgetConfiguration.setButtonConf ( memoryStateButtonConf );
    memoryStateWidgetConfiguration.setTextFontConf ( memoryStateTitleFontConf );
    HelpWidgetConfiguration helpWidgetConfiguration;
    ButtonConfiguration helpButtonConf;
    helpButtonConf.setButtonWidth ( 200 );
    helpButtonConf.setButtonHeight ( 60 );
    helpButtonConf.setFontSize ( 18 );
    helpWidgetConfiguration.setPrintButtonConf ( helpButtonConf );
    configSAESGUI.setMemoryStateWidgetConfiguration ( memoryStateWidgetConfiguration );
    configSAESGUI.setHelpWidgetConfiguration ( helpWidgetConfiguration );

    ButtonBoxConfiguration buttonBoxConfig ( 170, 50, 20 );
    ThemesConfiguration themesConfiguration;
    OfficialThemeConfiguration officialThemeConfiguration;
    DemoThemeConfiguration demoThemeConfiguration;

    XMLLabelConfiguration PartyLabelConfiguration;
    XMLLabelConfiguration PartyNumberLabelConfiguration;
    XMLLabelConfiguration PartyNullLabelConfiguration;
    XMLLabelConfiguration BallotSubstituteLabelConfiguration;
    XMLLabelConfiguration BallotListLabelConfiguration;
    XMLLabelConfiguration BallotListNumberLabelConfiguration;
    XMLLabelConfiguration BallotNumberLabelConfiguration;
    XMLLabelConfiguration BallotTopLabelConfiguration;
    XMLLabelConfiguration BallotBottomLabelConfiguration;
    XMLLabelConfiguration selectionLanguageHeaderLabelConfiguration;
    XMLLabelConfiguration selectionLanguageBottomLabelConfiguration;
    XMLLabelConfiguration selectionLanguageConfirmationHeaderLabelConfiguration;
    XMLLabelConfiguration selectionLanguageConfirmationBottomLabelConfiguration;
    XMLLabelConfiguration selectionLanguageConfirmationCenterLabelConfiguration;
    XMLLabelConfiguration welcomeContestCenterLabelConfiguration;
    XMLLabelConfiguration votingWidgetLabelChildLabelConfiguration;
    XMLLabelConfiguration votingWidgetTopChildLabelConfiguration;
    XMLLabelConfiguration votingWidgetBottomChildLabelConfiguration;
    XMLLabelConfiguration votingWidgetNumberChildLabelConfiguration;
    XMLLabelConfiguration votingWidgetPartySelectionLabelChildLabelConfiguration;
    XMLLabelConfiguration votingWindowEndMessageLabelConfiguration;
    XMLLabelConfiguration loadingBallotsLabelConfiguration;
    XMLLabelConfiguration checkSumLabelConfiguration;
    XMLLabelConfiguration interrupVotingExperienceMessageLabelConfiguration;
    XMLLabelConfiguration interrupVotingExperienceCancelLabelConfiguration;

    ElectionBlockStatusList listElectionNotInitiated;
    ElectionBlockStatusList listElectionInitiated;
    ElectionBlockStatusList listElectionClosed;
    ElectionBlockStatusList listElectionResults;

    HelpConfiguration helpStandardFunctionality;
    helpStandardFunctionality.setImageHelp ( "ui/standard-help-image-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/standard-help-pdf-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.Generic" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    ElectionBlockStatusList::ElectionStatusSequence seq1;
    seq1.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_INITIATED );
    seq1.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_CLOSED );
    seq1.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_RESULTSTRANSMITTED );
    listElectionNotInitiated.setElectionStatus ( seq1 );
    ElectionBlockStatusList::ElectionStatusSequence seq2;
    seq2.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINITIATED );
    seq2.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_CLOSED );
    seq2.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_RESULTSTRANSMITTED );
    listElectionInitiated.setElectionStatus ( seq2 );
    ElectionBlockStatusList::ElectionStatusSequence seq3;
    seq3.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINITIATED );
    seq3.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_INITIATED );
    seq3.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_RESULTSTRANSMITTED );
    listElectionClosed.setElectionStatus ( seq3 );
    ElectionBlockStatusList::ElectionStatusSequence seq4;
    seq4.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINITIATED );
    seq4.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_INITIATED );
    seq4.push_back ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_CLOSED );
    listElectionResults.setElectionStatus ( seq4 );

    /************* ReportPrintingWidget Label Configuration *****************************/
    ReportPrintingWidgetConfiguration reportPrintingWidgetConfiguration;

    XMLLabelConfiguration compositeDescriptionLabelConfiguration;
    XMLLabelConfiguration reportListTitleLabelConfiguration;
    XMLLabelConfiguration progressMessageLabelConfiguration;
    XMLLabelConfiguration reportListItemLabelConfiguration;

    compositeDescriptionLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    compositeDescriptionLabelConfiguration.setLabel_VerticalPadding ( 0 );
    compositeDescriptionLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    compositeDescriptionLabelConfiguration.setLabel_Family ( "Verdana" );
    compositeDescriptionLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    compositeDescriptionLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    compositeDescriptionLabelConfiguration.setLabel_Size ( 15 );

    reportListTitleLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    reportListTitleLabelConfiguration.setLabel_VerticalPadding ( 0 );
    reportListTitleLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    reportListTitleLabelConfiguration.setLabel_Family ( "Verdana" );
    reportListTitleLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    reportListTitleLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    reportListTitleLabelConfiguration.setLabel_Size ( 15 );

    progressMessageLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    progressMessageLabelConfiguration.setLabel_VerticalPadding ( 0 );
    progressMessageLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    progressMessageLabelConfiguration.setLabel_Family ( "Verdana" );
    progressMessageLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    progressMessageLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    progressMessageLabelConfiguration.setLabel_Size ( 15 );

    reportListItemLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_LEFT );
    reportListItemLabelConfiguration.setLabel_VerticalPadding ( 0 );
    reportListItemLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    reportListItemLabelConfiguration.setLabel_Family ( "Verdana" );
    reportListItemLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    reportListItemLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    reportListItemLabelConfiguration.setLabel_Size ( 15 );

    reportPrintingWidgetConfiguration.setCompositeDescriptionLabelConfiguration ( compositeDescriptionLabelConfiguration );
    reportPrintingWidgetConfiguration.setReportListTitleLabelConfiguration ( reportListTitleLabelConfiguration );
    reportPrintingWidgetConfiguration.setProgressMessageLabelConfiguration ( progressMessageLabelConfiguration );
    reportPrintingWidgetConfiguration.setReportListItemLabelConfiguration ( reportListItemLabelConfiguration );

    configSAESGUI.setReportPrintingWidget ( reportPrintingWidgetConfiguration );
    configSAESGUI.setForceToLoadFrame ( true );
    configSAESGUI.setPMInstallationWidgetShowPrintPasswordButton ( true );

    /************* Start Voting Installation Window Configuration *****************************/
    VMInstallationWidgetConfiguration vMInstallationWidgetConfiguration ( 5000, 270, 100 );
    configSAESGUI.setVMInstallationConfiguration ( vMInstallationWidgetConfiguration );
    /************* End Voting Installation Window Configuration *****************************/
    exitMessageLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    exitMessageLabelConfiguration.setLabel_VerticalPadding ( 1 );
    exitMessageLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    exitMessageLabelConfiguration.setLabel_Family ( "Verdana" );
    exitMessageLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    exitMessageLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    exitMessageLabelConfiguration.setLabel_Size ( 20 );

    configSAESGUI.setExitMessageLabelConfiguration ( exitMessageLabelConfiguration );
    /************* End ReportPrintingWidget Label Configuration *****************************/
    /************* Start Close Voting Configuration *****************************/
    CloseVotationFunctionalityConfiguration * closeVotationConfig = new CloseVotationFunctionalityConfiguration ();
    closeVotationConfig->setMinimumOfficialsActivationForConfirmation ( 1 );
    closeVotationConfig->setShowConfirmationMessageWhenClosing ( true );
    configSAESGUI.setCloseVotationFunctionalityConfiguration ( *closeVotationConfig );
    /************* End Close Voting Configuration *****************************/

    PollingCardConfiguration pollingCardConfiguration;
    pollingCardConfiguration.setPMInstallationWindow ( 1000 );
    pollingCardConfiguration.setVMInstallationWindow ( 1000 );
    pollingCardConfiguration.setVotingExperienceWindow ( 1000 );
    pollingCardConfiguration.setEndVotingExperienceWindow ( 1000 );
    pollingCardConfiguration.setVoteProcessDemoFunctionality ( 2000 );
    pollingCardConfiguration.setActivationStatsWidget ( 1000 );
    pollingCardConfiguration.setWaitingCardWidget ( 1000 );
    pollingCardConfiguration.setSmartcardReaderDiagnosisWidget ( 1000 );
    pollingCardConfiguration.setVotingMachineInstallationWidgetInsert ( 1500 );
    pollingCardConfiguration.setVotingMachineInstallationWidgetExtract ( 1500 );
    AlarmBoxMonitorConfiguration alarmBoxMonitorConfiguration;
    alarmBoxMonitorConfiguration.setEnable ( false );
    alarmBoxMonitorConfiguration.setMonitoringTime ( 5000 );
    alarmBoxMonitorConfiguration.setSearchTime ( 2000 );

    std::stringstream ss;

    //voting Machine Menu
    votingMachine.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotingProcess" ) );
    votingMachine.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotingProcess" ) );
    votingMachine.setAccessibleName ( "VotingProcessMenu" );

    votingMachine.setImageKey ( "votingmachine.jpg" );
    votingMachine.setCode ( 0 );
    votingMachine.setMappingKey ( 1 );
    votingMachine.setMappingKeyName ( "A" );
    votingMachine.setOptionType ( MenuOptionType::CONFIGURATION_XMLMENU );
    mainteinanceMenu.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.Maintenance" ) );
    mainteinanceMenu.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.Maintenance" ) );
    mainteinanceMenu.setAccessibleName ( "MaintenanceMenu" );
    mainteinanceMenu.setImageKey ( "Maintenance.png" );
    mainteinanceMenu.setCode ( 1 );
    mainteinanceMenu.setMappingKey ( 2 );
    mainteinanceMenu.setMappingKeyName ( "B" );
    mainteinanceMenu.setOptionType ( MenuOptionType::CONFIGURATION_XMLMENU );
    validationMenu.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.validationMenu" ) );
    validationMenu.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.validationMenu" ) );
    validationMenu.setAccessibleName ( "ValidationMenu" );

    validationMenu.setImageKey ( "Maintenance.png" );
    validationMenu.setCode ( 2 );
    validationMenu.setMappingKey ( 3 );
    validationMenu.setMappingKeyName ( "C" );
    validationMenu.setOptionType ( MenuOptionType::CONFIGURATION_XMLMENU );
    reportProcess.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ReportingProcess" ) );
    reportProcess.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ReportingProcess" ) );
    reportProcess.setAccessibleName ( "ReportingProcessMenu" );

    reportProcess.setImageKey ( "ReportingProcess.jpg" );
    reportProcess.setCode ( 3 );
    reportProcess.setMappingKey ( 4 );
    reportProcess.setMappingKeyName ( "D" );
    reportProcess.setOptionType ( MenuOptionType::CONFIGURATION_XMLMENU );
    //option openElection
    openElection.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.OpenElection" ) );
    openElection.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.OpenElection" ) );
    openElection.setAccessibleName ( "OpenElectionMenu" );

    openElection.setImageKey ( "" );
    openElection.setCode ( 11 );
    openElection.setMappingKey ( 3 );
    openElection.setMappingKeyName ( "HOLA" );
    openElection.setOptionType ( MenuOptionType::CONFIGURATION_XMLMOPENELECTIONFUNCTIONALITY );
    openElection.setMenuOptions ( MenuOptions () );
    //option closeElection
    closeElection.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.CloseElection" ) );
    closeElection.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.CloseElection" ) );
    closeElection.setAccessibleName ( "CloseElectionMenu" );
    closeElection.setImageKey ( "" );
    closeElection.setCode ( 12 );
    closeElection.setMappingKey ( 4 );
    closeElection.setMappingKeyName ( "@" );
    closeElection.setOptionType ( MenuOptionType::CONFIGURATION_XMLMCLOSEELECTIONFUNCTIONALITY );
    closeElection.setMenuOptions ( MenuOptions () );
    //option voting machine installation
    vmInstallation.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotingMachineInstallation" ) );
    vmInstallation.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotingMachineInstallation" ) );
    vmInstallation.setAccessibleName ( "VotingMachineInstallationMenu" );
    vmInstallation.setImageKey ( "" );
    vmInstallation.setCode ( 13 );
    vmInstallation.setMappingKey ( 5 );
    vmInstallation.setMappingKeyName ( "F" );
    vmInstallation.setOptionType ( MenuOptionType::CONFIGURATION_XMLMVOTINGMACHINEINSTALLATIONFUNCTIONALITY );
    vmInstallation.setMenuOptions ( MenuOptions () );
    //option card activator
    cardActivators.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ActivationAndDemoProcess" ) );
    cardActivators.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ActivationAndDemoProcess" ) );
    cardActivators.setAccessibleName ( "ActivationAndDemoProcessMenu" );
    cardActivators.setImageKey ( "" );
    cardActivators.setCode ( 14 );
    cardActivators.setMappingKey ( 6 );
    cardActivators.setMappingKeyName ( "G" );
    cardActivators.setOptionType ( MenuOptionType::CONFIGURATION_XMLMENU );
    //option voting process demo
    voteProcessDemo.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VoteProcessDemo" ) );
    voteProcessDemo.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VoteProcessDemo" ) );
    voteProcessDemo.setAccessibleName ( "VoteProcessDemoMenu" );
    voteProcessDemo.setImageKey ( "" );
    voteProcessDemo.setCode ( 141 );
    voteProcessDemo.setMappingKey ( 3 );
    voteProcessDemo.setMappingKeyName ( "H" );
    voteProcessDemo.setOptionType ( MenuOptionType::CONFIGURATION_XMLMVOTEPROCESSDEMOFUNCTIONALITY );
    voteProcessDemo.setMenuOptions ( MenuOptions () );
    // Vote Print Preview
    printPreview.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotePrintPreview" ) );
    printPreview.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotePrintPreview" ) );
    printPreview.setAccessibleName ( "VotePrintPreviewMenu" );
    printPreview.setImageKey ( "" );
    printPreview.setCode ( 143 );
    printPreview.setMappingKey ( 5 );
    printPreview.setMappingKeyName ( "I" );
    printPreview.setOptionType ( MenuOptionType::CONFIGURATION_XMLMPRINTPREVIEWFUNCTIONALITY );
    printPreview.setMenuOptions ( MenuOptions () );
    //option diagnosis
    diagnosis.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.Diagnosis" ) );
    diagnosis.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.Diagnosis" ) );
    diagnosis.setAccessibleName ( "DiagnosisMenu" );
    diagnosis.setImageKey ( "" );
    diagnosis.setCode ( 21 );
    diagnosis.setMappingKey ( 3 );
    diagnosis.setMappingKeyName ( "J" );
    diagnosis.setOptionType ( MenuOptionType::CONFIGURATION_XMLMENU );
    //diagnosis.setMenuOptions (MenuOptions ());
    //option alarmBoxDiagnosis
    alarmBoxDiagnosis.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.AlarmBoxDiagnosis" ) );
    alarmBoxDiagnosis.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.AlarmBoxDiagnosis" ) );
    alarmBoxDiagnosis.setAccessibleName ( "AlarmBoxDiagnosisMenu" );
    alarmBoxDiagnosis.setImageKey ( "" );
    alarmBoxDiagnosis.setCode ( 211 );
    alarmBoxDiagnosis.setMappingKey ( 3 );
    alarmBoxDiagnosis.setMappingKeyName ( "K" );
    alarmBoxDiagnosis.setOptionType ( MenuOptionType::CONFIGURATION_XMLMALARMBOXDIAGNOSISFUNCTIONALITY );
    alarmBoxDiagnosis.setMenuOptions ( MenuOptions () );
    //option smartcardReaderDiagnosis
    smartcardReaderDiagnosis.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.SmartcardReaderDiagnosis" ) );
    smartcardReaderDiagnosis.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.SmartcardReaderDiagnosis" ) );
    smartcardReaderDiagnosis.setAccessibleName ( "SmartcardReaderDiagnosisMenu" );
    smartcardReaderDiagnosis.setImageKey ( "" );
    smartcardReaderDiagnosis.setCode ( 212 );
    smartcardReaderDiagnosis.setMappingKey ( 4 );
    smartcardReaderDiagnosis.setMappingKeyName ( "L" );
    smartcardReaderDiagnosis.setOptionType ( MenuOptionType::CONFIGURATION_XMLMSCREADERDIAGNOSISFUNCTIONALITY );
    smartcardReaderDiagnosis.setMenuOptions ( MenuOptions () );
    //option printerDiagnosis
    printerDiagnosis.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.PrinterDiagnosis" ) );
    printerDiagnosis.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.PrinterDiagnosis" ) );
    printerDiagnosis.setAccessibleName ( "PrinterDiagnosisMenu" );
    printerDiagnosis.setImageKey ( "" );
    printerDiagnosis.setCode ( 213 );
    printerDiagnosis.setMappingKey ( 5 );
    printerDiagnosis.setMappingKeyName ( "M" );
    printerDiagnosis.setOptionType ( MenuOptionType::CONFIGURATION_XMLMPRINTERDIAGNOSISFUNCTIONALITY );
    printerDiagnosis.setMenuOptions ( MenuOptions () );
    //option calibrationDiagnosis
    calibrationDiagnosis.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.CalibrationDiagnosis" ) );
    calibrationDiagnosis.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.CalibrationDiagnosis" ) );
    calibrationDiagnosis.setAccessibleName ( "CalibrationDiagnosisMenu" );
    calibrationDiagnosis.setImageKey ( "" );
    calibrationDiagnosis.setCode ( 214 );
    calibrationDiagnosis.setMappingKey ( 6 );
    calibrationDiagnosis.setMappingKeyName ( "N" );
    calibrationDiagnosis.setOptionType ( MenuOptionType::CONFIGURATION_XMLMCALIBRATIONDIAGNOSISFUNCTIONALITY );
    calibrationDiagnosis.setMenuOptions ( MenuOptions () );
    //option barcodeDiagnosis
    barcodeDiagnosis.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.BarcodeDiagnosis" ) );
    barcodeDiagnosis.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.BarcodeDiagnosis" ) );
    barcodeDiagnosis.setAccessibleName ( "BarcodeDiagnosisMenu" );
    barcodeDiagnosis.setImageKey ( "" );
    barcodeDiagnosis.setCode ( 216 );
    barcodeDiagnosis.setMappingKey ( 8 );
    barcodeDiagnosis.setMappingKeyName ( "P" );
    barcodeDiagnosis.setOptionType ( MenuOptionType::CONFIGURATION_XMLMBARCODEDIAGNOSISFUNCTIONALITY );
    barcodeDiagnosis.setMenuOptions ( MenuOptions () );
    //option allDiagnosis
    allDiagnosis.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.AllDiagnosis" ) );
    allDiagnosis.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.AllDiagnosis" ) );
    allDiagnosis.setAccessibleName ( "AllDiagnosisMenu" );
    allDiagnosis.setImageKey ( "" );
    allDiagnosis.setCode ( 217 );
    allDiagnosis.setMappingKey ( 9 );
    allDiagnosis.setMappingKeyName ( "Q" );
    allDiagnosis.setOptionType ( MenuOptionType::CONFIGURATION_XMLMALLDIAGNOSISFUNCTIONALITY );
    allDiagnosis.setMenuOptions ( MenuOptions () );
    //option flashDiagnosis
    flashDiagnosis.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.FlashDiagnosis" ) );
    flashDiagnosis.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.FlashDiagnosis" ) );
    flashDiagnosis.setAccessibleName ( "FlashDiagnosisMenu" );
    flashDiagnosis.setImageKey ( "" );
    flashDiagnosis.setCode ( 218 );
    flashDiagnosis.setMappingKey ( 3 );
    flashDiagnosis.setMappingKeyName ( "R" );
    flashDiagnosis.setOptionType ( MenuOptionType::CONFIGURATION_XMLMFLASHDIAGNOSISFUNCTIONALITY );
    flashDiagnosis.setMenuOptions ( MenuOptions () );
    ableDiagnosis.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.AbleDiagnosis" ) );
    ableDiagnosis.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.AbleDiagnosis" ) );
    ableDiagnosis.setAccessibleName ( "AbleDiagnosisMenu" );
    ableDiagnosis.setImageKey ( "" );
    ableDiagnosis.setCode ( 218 );
    ableDiagnosis.setMappingKey ( 3 );
    ableDiagnosis.setMappingKeyName ( "X" );
    ableDiagnosis.setOptionType ( MenuOptionType::CONFIGURATION_XMLMABLEDIAGNOSTICFUNCTIONALITY );
    ableDiagnosis.setMenuOptions ( MenuOptions () );
    //option report printing
    reportPrinting.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ReportPrinting" ) );
    reportPrinting.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ReportPrinting" ) );
    reportPrinting.setAccessibleName ( "ReportPrintingMenu" );
    reportPrinting.setImageKey ( "" );
    reportPrinting.setCode ( 219 );
    reportPrinting.setMappingKey ( 3 );
    reportPrinting.setMappingKeyName ( "S" );
    reportPrinting.setOptionType ( MenuOptionType::CONFIGURATION_XMLMEXTERNALPRINTFUNCTIONALITY );
    reportPrinting.setMenuOptions ( MenuOptions () );
    //option shutdown menu
    shutDownMenuOption.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.Shutdowm" ) );
    shutDownMenuOption.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.Shutdowm" ) );
    shutDownMenuOption.setAccessibleName ( "ShutdowmMenu" );
    shutDownMenuOption.setImageKey ( "" );
    shutDownMenuOption.setCode ( 888 );
    shutDownMenuOption.setMappingKey ( 3 );
    shutDownMenuOption.setMappingKeyName ( "V" );
    shutDownMenuOption.setOptionType ( MenuOptionType::CONFIGURATION_XMLMSHUTDOWMMACHINEFUNCTIONALITY );
    shutDownMenuOption.setMenuOptions ( MenuOptions () );
    //shutdown parent menu
    shutDownParentMenuOption.setNameKey(N_("Smartmatic.SAES.GUI.Configuration.ShutdownMenuOption.Shutdown"));
    shutDownParentMenuOption.setUntraslatedKey(N_("Smartmatic.SAES.GUI.Configuration.ShutdownMenuOption.Shutdown"));
    //option changeHour
    changeHour.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.SetDateHour" ) );
    changeHour.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.SetDateHour" ) );
    changeHour.setAccessibleName ( "SetDateHourMenu" );
    changeHour.setImageKey ( "" );
    changeHour.setCode ( 22 );
    changeHour.setMappingKey ( 4 );
    changeHour.setMappingKeyName ( "T" );
    changeHour.setOptionType ( MenuOptionType::CONFIGURATION_XMLMCHANGEHOURFUNCTIONALITY );
    changeHour.setMenuOptions ( MenuOptions () );
    //Activation Consolidation Functionality
    activationConsolidation.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ActivationConsolidationFunctionality" ) );
    activationConsolidation.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ActivationConsolidationFunctionality" ) );
    activationConsolidation.setAccessibleName ( "ActivationConsolidationMenu" );
    activationConsolidation.setImageKey ( "" );
    activationConsolidation.setCode ( 145 );
    activationConsolidation.setMappingKey ( 2 );
    activationConsolidation.setMappingKeyName ( "W" );
    activationConsolidation.setOptionType ( MenuOptionType::CONFIGURATION_XMLMACTIVATIONCONSOLIDATIONFUNCTIONALITY );
    activationConsolidation.setMenuOptions ( MenuOptions () );
    //Change language
    changeLanguage.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ChangeLanguageFuncionality" ) );
    changeLanguage.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ChangeLanguageFuncionality" ) );
    changeLanguage.setAccessibleName ( "ChangeLanguageMenu" );
    changeLanguage.setImageKey ( "" );
    changeLanguage.setCode ( 24 );
    changeLanguage.setMappingKey ( 5 );
    changeLanguage.setMappingKeyName ( "V" );
    changeLanguage.setOptionType ( MenuOptionType::CONFIGURATION_XMLMCHANGELANGUAGEFUNCTIONALITY );
    changeLanguage.setMenuOptions ( MenuOptions () );

    votingExperienceValidation.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotingExperienceValidationFunctionality" ) );
    votingExperienceValidation.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotingExperienceValidationFunctionality" ) );
    votingExperienceValidation.setAccessibleName ( "VotingExperienceValidationMenu" );
    votingExperienceValidation.setImageKey ( "" );
    votingExperienceValidation.setCode ( 27 );
    votingExperienceValidation.setMappingKey ( 8 );
    votingExperienceValidation.setMappingKeyName ( "Y" );
    votingExperienceValidation.setOptionType ( MenuOptionType::CONFIGURATION_XMLMVOTINGEXPERIENCEVALIDATIONFUNCTIONALITY );
    votingExperienceValidation.setMenuOptions ( MenuOptions () );

    logReport.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.LogReport" ) );
    logReport.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.LogReport" ) );
    logReport.setAccessibleName ( "LogReportMenu" );
    logReport.setImageKey ( "" );
    logReport.setCode ( 285 );
    logReport.setMappingKey ( 99 );
    logReport.setMappingKeyName ( "<" );
    logReport.setOptionType ( MenuOptionType::CONFIGURATION_XMLMLOGREPORTFUNCTIONALITY );
    logReport.setMenuOptions ( MenuOptions () );

    closeVotation.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.CloseVotation" ) );
    closeVotation.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.CloseVotation" ) );
    closeVotation.setAccessibleName ( "CloseVotationMenu" );
    closeVotation.setImageKey ( "" );
    closeVotation.setCode ( 282 );
    closeVotation.setMappingKey ( 3 );
    closeVotation.setMappingKeyName ( "+" );
    closeVotation.setOptionType ( MenuOptionType::CONFIGURATION_XMLMCLOSEVOTATIONFUNCTIONALITY );
    closeVotation.setMenuOptions ( MenuOptions () );

    seqOptionsDiagnosis.push_back ( alarmBoxDiagnosis );
    seqOptionsDiagnosis.push_back ( allDiagnosis );
    seqOptionsDiagnosis.push_back ( barcodeDiagnosis );
    seqOptionsDiagnosis.push_back ( smartcardReaderDiagnosis );
    seqOptionsDiagnosis.push_back ( printerDiagnosis );
    seqOptionsDiagnosis.push_back ( calibrationDiagnosis );
    seqOptionsDiagnosis.push_back ( flashDiagnosis );
    seqOptionsDiagnosis.push_back ( ableDiagnosis );

    optionsDiagnosis.setOption ( seqOptionsDiagnosis );
    diagnosis.setMenuOptions ( optionsDiagnosis );
    seqOptionsMaintenance.push_back ( diagnosis );
    seqOptionsMaintenance.push_back ( changeHour );
    seqOptionsMaintenance.push_back ( changeLanguage );
    seqOptionsMaintenance.push_back ( votingExperienceValidation );

    optionsMaintenance.setOption ( seqOptionsMaintenance );
    optionsValidations.setOption ( seqOptionsValidations );
    mainteinanceMenu.setMenuOptions ( optionsMaintenance );
    validationMenu.setMenuOptions ( optionsValidations );

    //MenuOptions VotingMachineOption
    seqOptionsVotingMachine.push_back ( reportPrinting );
    seqOptionsVotingMachine.push_back ( shutDownMenuOption );
    seqOptionsVotingMachine.push_back ( openElection );
    seqOptionsVotingMachine.push_back ( closeElection );
    seqOptionsVotingMachine.push_back ( closeVotation );
    seqOptionsReportingProcess.push_back ( logReport );
    seqOptionsDiagnosis.push_back ( alarmBoxDiagnosis );
    seqOptionsCardActivators.push_back ( activationConsolidation );
    seqOptionsCardActivators.push_back ( voteProcessDemo );
    seqOptionsCardActivators.push_back ( printPreview );
    optionsCardActivators.setOption ( seqOptionsCardActivators );
    cardActivators.setMenuOptions ( optionsCardActivators );
    //seqOptionsVotingMachine.push_back(*vmInstallation);
    seqOptionsVotingMachine.push_back ( vmInstallation );
    seqOptionsVotingMachine.push_back ( cardActivators );
    optionsVotingMachine.setOption ( seqOptionsVotingMachine );
    optionsProcessReport.setOption ( seqOptionsReportingProcess );
    votingMachine.setMenuOptions ( optionsVotingMachine );
    reportProcess.setMenuOptions ( optionsProcessReport );
    //Menu Options  Root
    seqRoot.push_back ( votingMachine );
    seqRoot.push_back ( reportProcess );
    seqRoot.push_back ( mainteinanceMenu );

    optionsRoot.setOption ( seqRoot );
    // rootPollBookMachine
    rootPresidentMachineOption.setNameKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.RootPresidentMachineOption" ) );
    rootPresidentMachineOption.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.RootPresidentMachineOption" ) );
    rootPresidentMachineOption.setAccessibleName ( "RootPresidentMenu" );
    rootPresidentMachineOption.setImageKey ( "rootPollBookMachine" );
    rootPresidentMachineOption.setCode ( -1 );
    rootPresidentMachineOption.setMappingKey ( -1 );
    rootPresidentMachineOption.setMappingKeyName ( "rootPollBookMachine" );
    rootPresidentMachineOption.setOptionType ( MenuOptionType::CONFIGURATION_XMLMENU );
    rootPresidentMachineOption.setMenuOptions ( optionsRoot );
    // rootVotingMachine
    rootVotingMachineOption.setNameKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.RootVotingMachineOption" ) );
    rootVotingMachineOption.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.RootVotingMachineOption" ) );
    rootVotingMachineOption.setAccessibleName ( "RootVotingMenu" );
    rootVotingMachineOption.setImageKey ( "rootVotingMachine" );
    rootVotingMachineOption.setCode ( -1 );
    rootVotingMachineOption.setMappingKey ( -1 );
    rootVotingMachineOption.setMappingKeyName ( "rootVotingMachine" );
    rootVotingMachineOption.setOptionType ( MenuOptionType::CONFIGURATION_XMLMENU );
    rootVotingMachineOption.setMenuOptions ( optionsRoot );
    //presidentMachineConfiguration
    pollBookMachineConfiguration.setFirstLevelWidth ( 190 );
    pollBookMachineConfiguration.setFirstLevelHeight ( 109 );
    pollBookMachineConfiguration.setFirstLevelSpacing ( 5 );
    pollBookMachineConfiguration.setFirstLevelBorderWidth ( 5 );
    pollBookMachineConfiguration.setFirstLevelFrameWidth ( 200 );
    pollBookMachineConfiguration.setFirstLevelFrameHeight ( -1 );
    pollBookMachineConfiguration.setSecondLevelWidth ( 520 );
    pollBookMachineConfiguration.setSecondLevelHeight ( 50 );
    pollBookMachineConfiguration.setSecondLevelSpacing ( 5 );
    pollBookMachineConfiguration.setSecondLevelBorderWidth ( 5 );
    pollBookMachineConfiguration.setSecondLevelFramePadding ( 10 );
    pollBookMachineConfiguration.setHasShortcut ( true );
    pollBookMachineConfiguration.setRootOption ( rootPresidentMachineOption );
    //votingMachineConfiguration
    votingMachineConfiguration.setFirstLevelWidth ( 190 );
    votingMachineConfiguration.setFirstLevelHeight ( 109 );
    votingMachineConfiguration.setFirstLevelSpacing ( 5 );
    votingMachineConfiguration.setFirstLevelBorderWidth ( 5 );
    votingMachineConfiguration.setFirstLevelFrameWidth ( 210 );
    votingMachineConfiguration.setFirstLevelFrameHeight ( -1 );
    votingMachineConfiguration.setSecondLevelWidth ( 720 );
    votingMachineConfiguration.setSecondLevelHeight ( 40 );
    votingMachineConfiguration.setSecondLevelSpacing ( 5 );
    votingMachineConfiguration.setSecondLevelBorderWidth ( 5 );
    votingMachineConfiguration.setSecondLevelFramePadding ( 10 );
    votingMachineConfiguration.setHasShortcut ( true );
    votingMachineConfiguration.setRootOption ( rootVotingMachineOption );

    SaesGUIConfiguration::MenuConfigurationSequence menuOptionsConfiguration;

    votingMachineConfiguration.setMachine ( InfoMachineConfiguration::CONFIGURATION_PM );
    menuOptionsConfiguration.push_back ( votingMachineConfiguration );

    votingMachineConfiguration.setMachine ( InfoMachineConfiguration::CONFIGURATION_VM );
    menuOptionsConfiguration.push_back ( votingMachineConfiguration );

    configSAESGUI.setMenuConfiguration ( menuOptionsConfiguration );
    configSAESGUI.setFunctionalityWizardConfiguration ( functionalityWizardConfig );
    //
    //Voting Experience Configuration
    //
    XMLLabelConfiguration customAtributtesWindowBottomLabelConfiguration;
    customAtributtesWindowBottomLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    customAtributtesWindowBottomLabelConfiguration.setLabel_VerticalPadding ( 1 );
    customAtributtesWindowBottomLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    customAtributtesWindowBottomLabelConfiguration.setLabel_Family ( "Verdana" );
    customAtributtesWindowBottomLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    customAtributtesWindowBottomLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    customAtributtesWindowBottomLabelConfiguration.setLabel_Size ( 20 );
    XMLLabelConfiguration customAtributtesWindowHeaderLabelConfiguration;
    customAtributtesWindowHeaderLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    customAtributtesWindowHeaderLabelConfiguration.setLabel_VerticalPadding ( 1 );
    customAtributtesWindowHeaderLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    customAtributtesWindowHeaderLabelConfiguration.setLabel_Family ( "Verdana" );
    customAtributtesWindowHeaderLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    customAtributtesWindowHeaderLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    customAtributtesWindowHeaderLabelConfiguration.setLabel_Size ( 20 );
    XMLLabelConfiguration customAtributtesWindowCenterLabelConfiguration;
    customAtributtesWindowCenterLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    customAtributtesWindowCenterLabelConfiguration.setLabel_VerticalPadding ( 1 );
    customAtributtesWindowCenterLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    customAtributtesWindowCenterLabelConfiguration.setLabel_Family ( "Verdana" );
    customAtributtesWindowCenterLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    customAtributtesWindowCenterLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    customAtributtesWindowCenterLabelConfiguration.setLabel_Size ( 20 );
    customAtributtesWindowConfiguration.setHeaderLabelConfiguration ( customAtributtesWindowHeaderLabelConfiguration );
    customAtributtesWindowConfiguration.setBottomLabelConfiguration ( customAtributtesWindowHeaderLabelConfiguration );
    customAtributtesWindowConfiguration.setCenterLabelConfiguration ( customAtributtesWindowCenterLabelConfiguration );
    //voting experience label configuration
    //<1>
    selectionLanguageHeaderLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    selectionLanguageHeaderLabelConfiguration.setLabel_VerticalPadding ( 1 );
    selectionLanguageHeaderLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    selectionLanguageHeaderLabelConfiguration.setLabel_Family ( "Verdana" );
    selectionLanguageHeaderLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    selectionLanguageHeaderLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    selectionLanguageHeaderLabelConfiguration.setLabel_Size ( 20 );
    //<2>
    selectionLanguageBottomLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    selectionLanguageBottomLabelConfiguration.setLabel_VerticalPadding ( 1 );
    selectionLanguageBottomLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    selectionLanguageBottomLabelConfiguration.setLabel_Family ( "Verdana" );
    selectionLanguageBottomLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    selectionLanguageBottomLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    selectionLanguageBottomLabelConfiguration.setLabel_Size ( 20 );
    //<3>
    selectionLanguageConfirmationHeaderLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    selectionLanguageConfirmationHeaderLabelConfiguration.setLabel_VerticalPadding ( 1 );
    selectionLanguageConfirmationHeaderLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    selectionLanguageConfirmationHeaderLabelConfiguration.setLabel_Family ( "Verdana" );
    selectionLanguageConfirmationHeaderLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    selectionLanguageConfirmationHeaderLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    selectionLanguageConfirmationHeaderLabelConfiguration.setLabel_Size ( 20 );
    //<4>
    selectionLanguageConfirmationBottomLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    selectionLanguageConfirmationBottomLabelConfiguration.setLabel_VerticalPadding ( 1 );
    selectionLanguageConfirmationBottomLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    selectionLanguageConfirmationBottomLabelConfiguration.setLabel_Family ( "Verdana" );
    selectionLanguageConfirmationBottomLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    selectionLanguageConfirmationBottomLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    selectionLanguageConfirmationBottomLabelConfiguration.setLabel_Size ( 20 );
    //<5>
    selectionLanguageConfirmationCenterLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    selectionLanguageConfirmationCenterLabelConfiguration.setLabel_VerticalPadding ( 1 );
    selectionLanguageConfirmationCenterLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    selectionLanguageConfirmationCenterLabelConfiguration.setLabel_Family ( "Verdana" );
    selectionLanguageConfirmationCenterLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    selectionLanguageConfirmationCenterLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    selectionLanguageConfirmationCenterLabelConfiguration.setLabel_Size ( 15 );
    //<15>
    welcomeContestCenterLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    welcomeContestCenterLabelConfiguration.setLabel_VerticalPadding ( 1 );
    welcomeContestCenterLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    welcomeContestCenterLabelConfiguration.setLabel_Family ( "Verdana" );
    welcomeContestCenterLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    welcomeContestCenterLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    welcomeContestCenterLabelConfiguration.setLabel_Size ( 23 );
    //<16>
    votingWidgetLabelChildLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_LEFT );
    votingWidgetLabelChildLabelConfiguration.setLabel_VerticalPadding ( 0 );
    votingWidgetLabelChildLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    votingWidgetLabelChildLabelConfiguration.setLabel_Family ( "Verdana" );
    votingWidgetLabelChildLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    votingWidgetLabelChildLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    votingWidgetLabelChildLabelConfiguration.setLabel_Size ( 20 );
    //<17>
    votingWidgetTopChildLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_LEFT );
    votingWidgetTopChildLabelConfiguration.setLabel_VerticalPadding ( 0 );
    votingWidgetTopChildLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    votingWidgetTopChildLabelConfiguration.setLabel_Family ( "Verdana" );
    votingWidgetTopChildLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    votingWidgetTopChildLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    votingWidgetTopChildLabelConfiguration.setLabel_Size ( 10 );
    //<18>
    votingWidgetBottomChildLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    votingWidgetBottomChildLabelConfiguration.setLabel_VerticalPadding ( 0 );
    votingWidgetBottomChildLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    votingWidgetBottomChildLabelConfiguration.setLabel_Family ( "Verdana" );
    votingWidgetBottomChildLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    votingWidgetBottomChildLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    votingWidgetBottomChildLabelConfiguration.setLabel_Size ( 10 );
    //<19>
    votingWidgetNumberChildLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    votingWidgetNumberChildLabelConfiguration.setLabel_VerticalPadding ( 0 );
    votingWidgetNumberChildLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    votingWidgetNumberChildLabelConfiguration.setLabel_Family ( "Verdana" );
    votingWidgetNumberChildLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    votingWidgetNumberChildLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    votingWidgetNumberChildLabelConfiguration.setLabel_Size ( 15 );
    //<20>
    votingWidgetPartySelectionLabelChildLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_LEFT );
    votingWidgetPartySelectionLabelChildLabelConfiguration.setLabel_VerticalPadding ( 0 );
    votingWidgetPartySelectionLabelChildLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    votingWidgetPartySelectionLabelChildLabelConfiguration.setLabel_Family ( "Verdana" );
    votingWidgetPartySelectionLabelChildLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    votingWidgetPartySelectionLabelChildLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    votingWidgetPartySelectionLabelChildLabelConfiguration.setLabel_Size ( 15 );
    votingWindowEndMessageLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    votingWindowEndMessageLabelConfiguration.setLabel_VerticalPadding ( 0 );
    votingWindowEndMessageLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    votingWindowEndMessageLabelConfiguration.setLabel_Family ( "Verdana" );
    votingWindowEndMessageLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    votingWindowEndMessageLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    votingWindowEndMessageLabelConfiguration.setLabel_Size ( 20 );
    //<18>
    PartyNullLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    PartyNullLabelConfiguration.setLabel_VerticalPadding ( 0 );
    PartyNullLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    PartyNullLabelConfiguration.setLabel_Family ( "Verdana" );
    PartyNullLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    PartyNullLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    PartyNullLabelConfiguration.setLabel_Size ( 15 );

    BallotSubstituteLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    BallotSubstituteLabelConfiguration.setLabel_VerticalPadding ( 0 );
    BallotSubstituteLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    BallotSubstituteLabelConfiguration.setLabel_Family ( "Verdana" );
    BallotSubstituteLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    BallotSubstituteLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    BallotSubstituteLabelConfiguration.setLabel_Size ( 10 );

    loadingBallotsLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    loadingBallotsLabelConfiguration.setLabel_VerticalPadding ( 0 );
    loadingBallotsLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    loadingBallotsLabelConfiguration.setLabel_Family ( "Verdana" );
    loadingBallotsLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    loadingBallotsLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    loadingBallotsLabelConfiguration.setLabel_Size ( 20 );

    checkSumLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    checkSumLabelConfiguration.setLabel_VerticalPadding ( 0 );
    checkSumLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    checkSumLabelConfiguration.setLabel_Family ( "Verdana" );
    checkSumLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    checkSumLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    checkSumLabelConfiguration.setLabel_Size ( 20 );

    interrupVotingExperienceMessageLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    interrupVotingExperienceMessageLabelConfiguration.setLabel_VerticalPadding ( 0 );
    interrupVotingExperienceMessageLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    interrupVotingExperienceMessageLabelConfiguration.setLabel_Family ( "Verdana" );
    interrupVotingExperienceMessageLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    interrupVotingExperienceMessageLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    interrupVotingExperienceMessageLabelConfiguration.setLabel_Size ( 20 );
    interrupVotingExperienceCancelLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    interrupVotingExperienceCancelLabelConfiguration.setLabel_VerticalPadding ( 0 );
    interrupVotingExperienceCancelLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    interrupVotingExperienceCancelLabelConfiguration.setLabel_Family ( "Verdana" );
    interrupVotingExperienceCancelLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    interrupVotingExperienceCancelLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    interrupVotingExperienceCancelLabelConfiguration.setLabel_Size ( 20 );
    votingExperienceLabelConfiguration.setPartyNullLabelConfiguration ( PartyNullLabelConfiguration );
    votingExperienceLabelConfiguration.setBallotSubstituteLabelConfiguration ( BallotSubstituteLabelConfiguration );
    votingExperienceLabelConfiguration.setSelectionLanguageHeaderLabelConfiguration ( selectionLanguageHeaderLabelConfiguration );
    votingExperienceLabelConfiguration.setSelectionLanguageBottomLabelConfiguration ( selectionLanguageBottomLabelConfiguration );
    votingExperienceLabelConfiguration.setSelectionLanguageConfirmationHeaderLabelConfiguration ( selectionLanguageConfirmationHeaderLabelConfiguration );
    votingExperienceLabelConfiguration.setSelectionLanguageConfirmationBottomLabelConfiguration ( selectionLanguageConfirmationBottomLabelConfiguration );
    votingExperienceLabelConfiguration.setSelectionLanguageConfirmationCenterLabelConfiguration ( selectionLanguageConfirmationCenterLabelConfiguration );
    votingExperienceLabelConfiguration.setWelcomeContestCenterLabelConfiguration ( welcomeContestCenterLabelConfiguration );
    votingExperienceLabelConfiguration.setVotingWidgetLabelChildLabelConfiguration ( votingWidgetLabelChildLabelConfiguration );
    votingExperienceLabelConfiguration.setVotingWidgetTopChildLabelConfiguration ( votingWidgetTopChildLabelConfiguration );
    votingExperienceLabelConfiguration.setVotingWidgetBottomChildLabelConfiguration ( votingWidgetBottomChildLabelConfiguration );
    votingExperienceLabelConfiguration.setVotingWidgetNumberChildLabelConfiguration ( votingWidgetNumberChildLabelConfiguration );
    votingExperienceLabelConfiguration.setVotingWindowEndMessageLabelConfiguration ( votingWindowEndMessageLabelConfiguration );
    votingExperienceLabelConfiguration.setLoadingBallotsLabelConfiguration ( loadingBallotsLabelConfiguration );
    votingExperienceLabelConfiguration.setCheckSumLabelConfiguration ( checkSumLabelConfiguration );
    votingExperienceLabelConfiguration.setInterrupVotingExperienceMessageLabelConfiguration ( interrupVotingExperienceMessageLabelConfiguration );
    votingExperienceLabelConfiguration.setInterrupVotingExperienceCancelLabelConfiguration ( interrupVotingExperienceCancelLabelConfiguration );
    votingExperienceLabelConfiguration.setVotingWidgetPartySelectionLabelChildLabelConfiguration ( votingWidgetPartySelectionLabelChildLabelConfiguration );
    //Voting Window Font
    votingWindowFont.setSize ( 16000 );
    votingWindowFont.setFontName ( "Arial" );
    //Voting Window Configuration
    votingWindowConfiguration.setBottomFrameMargin ( 0 );
    votingWindowConfiguration.setBottomFont ( votingWindowFont );
    votingWindowConfiguration.setBottomFrameHeight ( 40 );
    votingWindowConfiguration.setShowBallotOptionFooter ( false );
    votingWindowConfiguration.setShowBallotOptionsInContestBack ( false );
    votingWindowConfiguration.setEraseBallotOptionsInPartyChanged ( true );
    votingWindowConfiguration.setConnectTimer ( 50 );
    votingWindowConfiguration.setConnectTimerLoading ( 1000 );
    votingWindowConfiguration.setConnectTimerLanguage ( 5 );
    votingWindowConfiguration.setNumberOfBallotsToDisplayLoadingBallotMessage ( 50 );
    votingWindowConfiguration.setLoadingBallotMessage ( "Never" );
    //Color Configuration
    selectedColor.setRed ( 40000 );
    selectedColor.setGreen ( 40000 );
    selectedColor.setBlue ( 40000 );
    //Color Configuration
    unSelectedColor.setRed ( 65535 );
    unSelectedColor.setGreen ( 65535 );
    unSelectedColor.setBlue ( 65535 );
    //Party Widget
    partyWidget.setNumberWidth ( 40 );
    partyWidget.setUseAbbreviation ( false );
    partyWidget.setPadding ( 3 );
    partyWidget.setShowFrame ( true );


    partyWidget.setDisplay ( displayTypeImage );
    partyWidget.setUseFillByRows ( false );
    partyWidget.setSelectedColor ( selectedColor );
    partyWidget.setUnSelectedColor ( unSelectedColor );
    //Party Selection Widget

    ViewButtonConfiguration partyButtonConf ( ViewButtonConfiguration::CONFIGURATION_SEPARATE_BUTTONS );
    ExpandButtonConfiguration partyExpandButtonConfig;
    partyExpandButtonConfig.setSpacing ( 0 );
    partyExpandButtonConfig.setFontSize ( 16 );
    PartySeparateButtonConfiguration partySeparateButtonConfig;
    partySeparateButtonConfig.setNullButtonWidth ( 300 );
    partySeparateButtonConfig.setBackButtonWidth ( 600 );
    partySeparateButtonConfig.setClearButtonWidth ( 300 );
    partySeparateButtonConfig.setAcceptButtonWidth ( 300 );
    partySeparateButtonConfig.setFontSize ( 16 );
    partySelection.setEnableClick ( true );
    partySelection.setPartyButton ( partyButtonConf );
    partySelection.setExpandButton ( partyExpandButtonConfig );
    partySelection.setSeparateButton ( partySeparateButtonConfig );
    partySelection.setUseMaxPartyNumerFromAllContest ( false );
    partySelection.setShowNullParty ( true );
    partySelection.setShowNullButton ( false );
    partySelection.setFrameTopHeight ( 80 );
    partySelection.setFrameInfoHeight ( 20 );
    partySelection.setFrameBottomHeight ( 80 );
    partySelection.setColumns ( 4 );
    partySelection.setSizeToColumns ( 4 );
    partySelection.setImageRelation ( 1 );
    partySelection.setPartiesMargin ( 5 );
    partySelection.setHCenter ( false );
    partySelection.setVCenter ( true );
    partySelection.setPartyWidget ( partyWidget );
    partySelection.setAutoSelectParty ( false );
    partySelection.setAutoSelectNullParty ( false );
    partySelection.setPartyButtonsSpacing ( 40 );
    partySelection.setFixBorder ( 10 );

    eventSounds.setPMInstalationSound ( "AzertySound" );
    eventSounds.setVMInstalationSound ( "AzertySound" );
    eventSounds.setAuthenticationValidPasswordSound ( "ValidPasswordSound" );
    eventSounds.setAuthenticationInvalidPasswordSound ( "InvalidPasswordSound" );
    eventSounds.setVotingExperienceSound ( "VotePrintSound" );
    eventSounds.setBallotOptionSelectSound ( "BallotVoteSound" );
    eventSounds.setPartySelectSound ( "PartyVoteSound" );
    eventSounds.setDiagnosisSound ( "DiagnosticSound" );
    configSAESGUI.setEventSoundsConfiguration ( eventSounds );
    XMLLabelConfiguration partySelectionWindowTitleLabelConfiguration;
    partySelectionWindowTitleLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_LEFT );
    partySelectionWindowTitleLabelConfiguration.setLabel_VerticalPadding ( 1 );
    partySelectionWindowTitleLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    partySelectionWindowTitleLabelConfiguration.setLabel_Family ( "Verdana" );
    partySelectionWindowTitleLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    partySelectionWindowTitleLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    partySelectionWindowTitleLabelConfiguration.setLabel_Size ( 20 );
    XMLLabelConfiguration partySelectionWindowSubtitleLabelConfiguration;
    partySelectionWindowSubtitleLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_LEFT );
    partySelectionWindowSubtitleLabelConfiguration.setLabel_VerticalPadding ( 1 );
    partySelectionWindowSubtitleLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    partySelectionWindowSubtitleLabelConfiguration.setLabel_Family ( "Verdana" );
    partySelectionWindowSubtitleLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    partySelectionWindowSubtitleLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    partySelectionWindowSubtitleLabelConfiguration.setLabel_Size ( 16 );
    XMLLabelConfiguration partySelectionWindowBottomLabelConfiguration;
    partySelectionWindowBottomLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_RIGHT );
    partySelectionWindowBottomLabelConfiguration.setLabel_VerticalPadding ( 1 );
    partySelectionWindowBottomLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    partySelectionWindowBottomLabelConfiguration.setLabel_Family ( "Verdana" );
    partySelectionWindowBottomLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    partySelectionWindowBottomLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    partySelectionWindowBottomLabelConfiguration.setLabel_Size ( 20 );
    partySelection.setTitleLabelConfiguration ( partySelectionWindowTitleLabelConfiguration );
    partySelection.setSubtitleLabelConfiguration ( partySelectionWindowSubtitleLabelConfiguration );
    partySelection.setBottomLabelConfiguration ( partySelectionWindowBottomLabelConfiguration );
    //Ballot Option Font
    ballotOptionFont.setSize ( 10 );
    ballotOptionFont.setFontName ( "Arial" );
    //Color Configuration
    selectedColor.setRed ( 40000 );
    selectedColor.setGreen ( 40000 );
    selectedColor.setBlue ( 40000 );
    //Color Configuration
    unSelectedColor.setRed ( 65535 );
    unSelectedColor.setGreen ( 65535 );
    unSelectedColor.setBlue ( 65535 );
    //Ballot Option Widget Configuration
    ballotOptionWidget.setNumberWidth ( 35 );
    ballotOptionWidget.setDisplay ( displayTypeText );
    ballotOptionWidget.setPadding ( 0 );
    ballotOptionWidget.setSelectedColor ( selectedColor );
    ballotOptionWidget.setUnSelectedColor ( unSelectedColor );
    ballotOptionWidget.setShowFrame ( true );
    ballotOptionWidget.setBallotFont ( ballotOptionFont );
    ballotOptionWidget.setSeparatorCharacter ( "#" );
    ballotOptionWidget.setNameIsBeforeSeparator ( true );
    //Ballot Option Selection Widget
    ViewButtonConfiguration ballotOptionButtonConf ( ViewButtonConfiguration::CONFIGURATION_SEPARATE_BUTTONS );
    ViewTableConfiguration ballotOptionTableConf ( ViewTableConfiguration::CONFIGURATION_COLUMN_FIXED );
    ExpandButtonConfiguration ballotOptionExpandButtonConfig;
    ballotOptionExpandButtonConfig.setSpacing ( 0 );
    ballotOptionExpandButtonConfig.setFontSize ( 16 );
    BallotOptionSeparateButtonConfiguration ballotOptionSeparateButtonConfig;
    ballotOptionSeparateButtonConfig.setClearButtonWidth ( 300 );
    ballotOptionSeparateButtonConfig.setAcceptButtonWidth ( 300 );
    ballotOptionSeparateButtonConfig.setBackButtonWidth ( 600 );
    ballotOptionSeparateButtonConfig.setFontSize ( 16 );
    ballotOptionSelection.setEnableClick ( true );
    ballotOptionSelection.setBallotOptionTable ( ballotOptionTableConf );
    ballotOptionSelection.setBallotOptionButton ( ballotOptionButtonConf );
    ballotOptionSelection.setHideStandInLabelIfEmpty ( true );
    ballotOptionSelection.setExpandButton ( ballotOptionExpandButtonConfig );
    ballotOptionSelection.setSeparateButton ( ballotOptionSeparateButtonConfig );
    ballotOptionSelection.setFrameTopHeight ( 80 );
    ballotOptionSelection.setFrameListHeight ( 50 );
    ballotOptionSelection.setFrameBottomHeight ( 80 );
    ballotOptionSelection.setFrameInfoHeight ( 0 );
    ballotOptionSelection.setMaxColumns ( 4 );
    ballotOptionSelection.setMaxRows ( 23 );
    ballotOptionSelection.setFixBorder ( 10 );
    ballotOptionSelection.setResizeBallotBox ( false );
    ballotOptionSelection.setBallotsMargin ( 1 );
    ballotOptionSelection.setOptionListLinked ( false );
    ballotOptionSelection.setSelectListImpliesSelectStandIn ( false );
    ballotOptionSelection.setSelectStandInImpliesSelectList ( false );
    ballotOptionSelection.setAllowBackWithBallotSelected ( false );
    ballotOptionSelection.setCandidatesButtonsSpacing ( 40 );
    ballotOptionSelection.setShowImageInList ( true );
    ballotOptionSelection.setImageInListPadgging ( 10 );
    ballotOptionSelection.setListSelectedColor ( selectedColor );
    ballotOptionSelection.setListUnSelectedColor ( unSelectedColor );
    ballotOptionSelection.setStandInColor ( unSelectedColor );
    ballotOptionSelection.setBallotOptionWidget ( ballotOptionWidget );
    ballotOptionSelection.setImageRelation ( 1 );

    //referendum selection
    referendumConfiguration.setEnableClick ( true );
    referendumConfiguration.setFrameTopHeight ( 80 );
    referendumConfiguration.setFrameBottomHeight ( 80 );
    referendumConfiguration.setQuestionWidth ( 400 );
    referendumConfiguration.setQuestionHeight ( 200 );
    referendumConfiguration.setButtonsVertical ( false );
    referendumConfiguration.setButtonsPivot ( 2 );
    referendumConfiguration.setBallotBoxTopPadding ( 10 );
    referendumConfiguration.setBallotBoxLeftPadding ( 0 );
    referendumConfiguration.setBallotBoxSpacing ( 50 );
    referendumConfiguration.setBallotWidth ( 200 );
    referendumConfiguration.setBallotHeight ( 100 );
    referendumConfiguration.setShowBackButton ( true );
    referendumConfiguration.setShowNullOption ( false );
    referendumConfiguration.setShowClearButton ( true );
    referendumConfiguration.setSelectedColor ( selectedColor );
    referendumConfiguration.setUnSelectedColor ( unSelectedColor );
    referendumConfiguration.setWrapQuestion ( false );

    XMLLabelConfiguration referendumHeaderLabelConfiguration;
    referendumHeaderLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_RIGHT );
    referendumHeaderLabelConfiguration.setLabel_VerticalPadding ( 1 );
    referendumHeaderLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    referendumHeaderLabelConfiguration.setLabel_Family ( "Verdana" );
    referendumHeaderLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    referendumHeaderLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    referendumHeaderLabelConfiguration.setLabel_Size ( 20 );
    XMLLabelConfiguration referendumBottomLabelConfiguration;
    referendumBottomLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    referendumBottomLabelConfiguration.setLabel_VerticalPadding ( 1 );
    referendumBottomLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    referendumBottomLabelConfiguration.setLabel_Family ( "Verdana" );
    referendumBottomLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    referendumBottomLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    referendumBottomLabelConfiguration.setLabel_Size ( 20 );
    XMLLabelConfiguration referendumQuestionLabelConfiguration;
    referendumQuestionLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    referendumQuestionLabelConfiguration.setLabel_VerticalPadding ( 1 );
    referendumQuestionLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    referendumQuestionLabelConfiguration.setLabel_Family ( "Verdana" );
    referendumQuestionLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    referendumQuestionLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    referendumQuestionLabelConfiguration.setLabel_Size ( 30 );
    XMLLabelConfiguration referendumBallotLabelConfiguration;
    referendumBallotLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    referendumBallotLabelConfiguration.setLabel_VerticalPadding ( 1 );
    referendumBallotLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    referendumBallotLabelConfiguration.setLabel_Family ( "Verdana" );
    referendumBallotLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    referendumBallotLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    referendumBallotLabelConfiguration.setLabel_Size ( 14 );
    referendumConfiguration.setHeaderLabelConfiguration ( referendumHeaderLabelConfiguration );
    referendumConfiguration.setBottomLabelConfiguration ( referendumBottomLabelConfiguration );
    referendumConfiguration.setQuestionLabelConfiguration ( referendumQuestionLabelConfiguration );
    referendumConfiguration.setBallotsLabelConfiguration ( referendumBallotLabelConfiguration );
    XMLLabelConfiguration ballotOptionSelectionWindowTitleLabelConfiguration;
    ballotOptionSelectionWindowTitleLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_LEFT );
    ballotOptionSelectionWindowTitleLabelConfiguration.setLabel_VerticalPadding ( 1 );
    ballotOptionSelectionWindowTitleLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    ballotOptionSelectionWindowTitleLabelConfiguration.setLabel_Family ( "Verdana" );
    ballotOptionSelectionWindowTitleLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    ballotOptionSelectionWindowTitleLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    ballotOptionSelectionWindowTitleLabelConfiguration.setLabel_Size ( 20 );
    XMLLabelConfiguration ballotOptionSelectionWindowSubtitleLabelConfiguration;
    ballotOptionSelectionWindowSubtitleLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_LEFT );
    ballotOptionSelectionWindowSubtitleLabelConfiguration.setLabel_VerticalPadding ( 1 );
    ballotOptionSelectionWindowSubtitleLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    ballotOptionSelectionWindowSubtitleLabelConfiguration.setLabel_Family ( "Verdana" );
    ballotOptionSelectionWindowSubtitleLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    ballotOptionSelectionWindowSubtitleLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    ballotOptionSelectionWindowSubtitleLabelConfiguration.setLabel_Size ( 16 );
    XMLLabelConfiguration ballotOptionSelectionWindowBottomLabelConfiguration;
    ballotOptionSelectionWindowBottomLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    ballotOptionSelectionWindowBottomLabelConfiguration.setLabel_VerticalPadding ( 1 );
    ballotOptionSelectionWindowBottomLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    ballotOptionSelectionWindowBottomLabelConfiguration.setLabel_Family ( "Verdana" );
    ballotOptionSelectionWindowBottomLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    ballotOptionSelectionWindowBottomLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    ballotOptionSelectionWindowBottomLabelConfiguration.setLabel_Size ( 20 );
    ballotOptionSelection.setTitleLabelConfiguration ( ballotOptionSelectionWindowTitleLabelConfiguration );
    ballotOptionSelection.setSubtitleLabelConfiguration ( ballotOptionSelectionWindowSubtitleLabelConfiguration );
    ballotOptionSelection.setBottomLabelConfiguration ( ballotOptionSelectionWindowBottomLabelConfiguration );

    ballotOptionSelection.setBorderInfo ( 10 );
    ballotOptionSelection.setInstructionTitle ( ballotOptionSelectionWindowBottomLabelConfiguration );
    ballotOptionSelection.setInstructionInfo ( ballotOptionSelectionWindowBottomLabelConfiguration );
    ballotOptionSelection.setFooterInfo ( ballotOptionSelectionWindowBottomLabelConfiguration );
    ballotOptionSelection.setWarningInstructionTitle ( ballotOptionSelectionWindowBottomLabelConfiguration );
    ballotOptionSelection.setWarningInstructionInfo ( ballotOptionSelectionWindowBottomLabelConfiguration );
    ballotOptionSelection.setBorderWarning ( 2 );
    ballotOptionSelection.setUseWarningColor ( true );
    ballotOptionSelection.setWarningBorderColor ( ColorConfiguration ( 0, 0, 0 ) );
    ballotOptionSelection.setWarningFillColor ( ColorConfiguration ( 0xffff, 0xffff, 0xffff ) );

    //Define start the aplication
    votingExperienceConfiguration.setDetectCardExtraction ( true );
    votingExperienceConfiguration.setCheckCardEndVoting ( true );
    votingExperienceConfiguration.setTimeOutCardEndVoting ( 60 );
    votingExperienceConfiguration.setNumberOfExtractionsBeforeStop ( 3 );
    votingExperienceConfiguration.setVotingWindow ( votingWindowConfiguration );
    votingExperienceConfiguration.setPartySelection ( partySelection );
    votingExperienceConfiguration.setBallotOptionSelection ( ballotOptionSelection );
    votingExperienceConfiguration.setReferendumSelection ( referendumConfiguration );
    votingExperienceConfiguration.setCustomAtributtesWindow ( customAtributtesWindowConfiguration );
    votingExperienceConfiguration.setVotingExperienceLabel ( votingExperienceLabelConfiguration );
    votingExperienceConfiguration.setMarkCardAsUsed ( true );
    votingExperienceConfiguration.setCheckSumFailTimerMessage ( 3 );
    votingExperienceConfiguration.setCheckSumFailOption ( CheckSumFailOption::CONFIGURATION_CLEANSELECTIONS );

    VotingExperienceConfiguration::WidgetStepSequence widgetStepSeq;
    VotingExperienceConfiguration::EndStepSequence endStepSeq;

    widgetStepSeq.push_back ( StepConfiguration ( BasicImageType::GENERATED_IMAGES_PRINT_VOTE, 10000 ) );
    widgetStepSeq.push_back ( StepConfiguration ( BasicImageType::GENERATED_IMAGES_FOLD_VOTE, 10000 ) );
    widgetStepSeq.push_back ( StepConfiguration ( BasicImageType::GENERATED_IMAGES_REMOVE_CARD, 10000 ) );
    widgetStepSeq.push_back ( StepConfiguration ( BasicImageType::GENERATED_IMAGES_PRINT_FOLD_VOTE, 10000 ) );
    widgetStepSeq.push_back ( StepConfiguration ( BasicImageType::GENERATED_IMAGES_ERROR_PRINT_VOTE, 10000 ) );
    widgetStepSeq.push_back ( StepConfiguration ( BasicImageType::GENERATED_IMAGES_NULL_VOTING_EXPERIENCE, 5000 ) );
    widgetStepSeq.push_back ( StepConfiguration ( BasicImageType::GENERATED_IMAGES_CARD_REMOVED_COUNTER, 5000 ) );

    votingExperienceConfiguration.setWidgetStep ( widgetStepSeq );

    endStepSeq.push_back ( BasicImageType::GENERATED_IMAGES_PRINT_VOTE );
    endStepSeq.push_back ( BasicImageType::GENERATED_IMAGES_FOLD_VOTE );
    endStepSeq.push_back ( BasicImageType::GENERATED_IMAGES_REMOVE_CARD );

    votingExperienceConfiguration.setEndStep ( endStepSeq );

    //
    //Change hour funcionality configuration
    //
    ChangeHourFunctionalityConfiguration changeHourFunctionality;
    changeHourFunctionality.setButtonArrowWidth ( 80 );
    changeHourFunctionality.setButtonArrowHeight ( 80 );
    changeHourFunctionality.setButtonValidateWidth ( 150 );
    changeHourFunctionality.setButtonValidateHeight ( 90 );
    changeHourFunctionality.setEntryWidth ( 90 );
    changeHourFunctionality.setEntryHeight ( 65 );
    changeHourFunctionality.setFontEntrySize ( 23 );
    XMLLabelConfiguration timeLabelConfiguration;
    timeLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_RIGHT );
    timeLabelConfiguration.setLabel_VerticalPadding ( 1 );
    timeLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    timeLabelConfiguration.setLabel_Family ( "Verdana" );
    timeLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    timeLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    timeLabelConfiguration.setLabel_Size ( 18 );
    changeHourFunctionality.setTimeLabelConfiguration ( timeLabelConfiguration );
    XMLLabelConfiguration titlesLabelConfiguration;
    titlesLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    titlesLabelConfiguration.setLabel_VerticalPadding ( 1 );
    titlesLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    titlesLabelConfiguration.setLabel_Family ( "Verdana" );
    titlesLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    titlesLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    titlesLabelConfiguration.setLabel_Size ( 20 );
    changeHourFunctionality.setTitlesLabelConfiguration ( titlesLabelConfiguration );
    XMLLabelConfiguration principalTitleLabelConfiguration;
    principalTitleLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    principalTitleLabelConfiguration.setLabel_VerticalPadding ( 1 );
    principalTitleLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    principalTitleLabelConfiguration.setLabel_Family ( "Verdana" );
    principalTitleLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    principalTitleLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    principalTitleLabelConfiguration.setLabel_Size ( 22 );
    changeHourFunctionality.setPrincipalTitleLabelConfiguration ( principalTitleLabelConfiguration );
    changeHourFunctionality.setTableVerticalSpacing ( 30 );
    configSAESGUI.setChangeHourFunctionalityConfiguration ( changeHourFunctionality );

    touchDiagConfig.setForceCalibration ( true );
    touchDiagConfig.setTimeOk ( 3000 );
    touchDiagConfig.setTimeFail ( 5000 );
    touchDiagConfig.setTimeAutoCalibration ( 10000 );

    touchDiagConfig.setNumberOfCalibrationPoints ( 5 );
    touchDiagConfig.setTimeOutCalibrationFailInSeconds ( 10 );
    touchDiagConfig.setTimeInfoCalibration ( 5000 );
    touchDiagConfig.setHeightOfButtons ( 40 );
    touchDiagConfig.setWidthOfButtons ( 40 );
    touchDiagConfig.setNumberReportPrinting ( 2 );
    touchDiagConfig.setWaitingTimeAfterCalibration ( 5000 );

    screaderDiagnosisConf.setPowerDownEnabled ( false );
    screaderDiagnosisConf.setTimeOk ( 3000 );
    screaderDiagnosisConf.setTimeFail ( 5000 );
    screaderDiagnosisConf.setTimeStep ( 1000 );
    screaderDiagnosisConf.setWaitTimeAfterRetry ( 15000 );
    screaderDiagnosisConf.setNumberOfRetry ( 3 );
    screaderDiagnosisConf.setWarningMessageLabelConfiguration ( titlesLabelConfiguration );

    printerDiagnosisConf.setDoNotCheckBeforePrintDiagnostic ( false );
    printerDiagnosisConf.setPrinterTimeOk ( 3000 );
    printerDiagnosisConf.setPrinterTimeFail ( 5000 );

    barcodeDiagnosisConf.setReconfigureTimeOk ( 3000 );
    barcodeDiagnosisConf.setReconfigureTimeFail ( 5000 );
    barcodeDiagnosisConf.setPrinterTimeFail ( 5000 );
    barcodeDiagnosisConf.setScanTimeOk ( 3000 );
    barcodeDiagnosisConf.setScanTimeFail ( 5000 );
    barcodeDiagnosisConf.setStringToEncode ( "BARCODEREADER barcodereader BARCODEREADER barcodereader" );
    barcodeDiagnosisConf.setBarcodeCommandsName ( "baseconfiguration" );
    barcodeDiagnosisConf.setBarcodeInParams ( "LightsON" );
    barcodeDiagnosisConf.setBarcodeOutParams ( "LightsOFF" );
    barcodeDiagnosisConf.setSetSerialNumber ( true );

    DiagnosticType printer ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_PRINTER, true , 3000 );
    DiagnosticType barcode ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_BARCODE, false , 3000 );
    DiagnosticType touchScreen ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION, false , 3000 );
    DiagnosticType alarmbox ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ALARMBOX, false , 3000 );
    DiagnosticType smartcard ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_SMARTCARD, true , 3000 );
    DiagnosticType flash ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_FLASH, true , 3000 );
    DiagnosticType able ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_P168A, false , 3000 );

    seqDiagnosticToApplyOM.push_back ( printer );
    seqDiagnosticToApplyOM.push_back ( barcode );
    seqDiagnosticToApplyOM.push_back ( touchScreen );
    seqDiagnosticToApplyOM.push_back ( smartcard );
    seqDiagnosticToApplyOM.push_back ( flash );
    seqDiagnosticToApplyOM.push_back ( able );
    seqDiagnosticToApplyRM.push_back ( barcode );
    allDiagnosisConf.setDiagnosticToApply ( seqDiagnosticToApplyOM );
    allDiagnosisConf.setDiagnosticReviewWidth ( 500 );
    allDiagnosisConf.setDiagnosticReviewHeight ( 50 );
    seqAutoDiagnosticToApplyOM.push_back ( printer );
    seqAutoDiagnosticToApplyOM.push_back ( smartcard );
    seqAutoDiagnosticToApplyOM.push_back ( flash );
    seqAutoDiagnosticToApplyOM.push_back ( able );
    seqAutoDiagnosticToApplyRM.push_back ( barcode );
    XMLLabelConfiguration diagnosisFooterLabelConfiguration;
    diagnosisLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    diagnosisLabelConfiguration.setLabel_VerticalPadding ( 1 );
    diagnosisLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    diagnosisLabelConfiguration.setLabel_Family ( "Verdana" );
    diagnosisLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    diagnosisLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    diagnosisLabelConfiguration.setLabel_Size ( 20 );
    diagnosisFooterLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    diagnosisFooterLabelConfiguration.setLabel_VerticalPadding ( 1 );
    diagnosisFooterLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    diagnosisFooterLabelConfiguration.setLabel_Family ( "Verdana" );
    diagnosisFooterLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    diagnosisFooterLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    diagnosisFooterLabelConfiguration.setLabel_Size ( 12 );
    resultErrorLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    resultErrorLabelConfiguration.setLabel_VerticalPadding ( 1 );
    resultErrorLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    resultErrorLabelConfiguration.setLabel_Family ( "Verdana" );
    resultErrorLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    resultErrorLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    resultErrorLabelConfiguration.setLabel_Size ( 12 );

    XMLLabelConfiguration diagnosticReviewConfiguration;
    diagnosticReviewConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    diagnosticReviewConfiguration.setLabel_VerticalPadding ( 1 );
    diagnosticReviewConfiguration.setLabel_HorizontalPadding ( 1 );
    diagnosticReviewConfiguration.setLabel_Family ( "Verdana" );
    diagnosticReviewConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    diagnosticReviewConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    diagnosticReviewConfiguration.setLabel_Size ( 20 );
    XMLLabelConfiguration blockDiagnosticTitleConfiguration;
    blockDiagnosticTitleConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    blockDiagnosticTitleConfiguration.setLabel_VerticalPadding ( 1 );
    blockDiagnosticTitleConfiguration.setLabel_HorizontalPadding ( 1 );
    blockDiagnosticTitleConfiguration.setLabel_Family ( "Verdana" );
    blockDiagnosticTitleConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    blockDiagnosticTitleConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    blockDiagnosticTitleConfiguration.setLabel_Size ( 14 );
    ButtonConfiguration blockDiagnosticButtonConf;
    blockDiagnosticButtonConf.setButtonWidth ( 220 );
    blockDiagnosticButtonConf.setButtonHeight ( 60 );
    blockDiagnosticButtonConf.setFontSize ( 12 );
    autoDiagnosisConfOM.setDiagnosticToApply ( seqAutoDiagnosticToApplyOM );
    autoDiagnosisConfOM.setBlockDiagnostic ( true );
    autoDiagnosisConfOM.setBlockDiagnosticTitle ( blockDiagnosticTitleConfiguration );
    autoDiagnosisConfOM.setBlockDiagnosticButton ( blockDiagnosticButtonConf );
    autoDiagnosisConfOM.setUseOkButtonOnSomeNonMandatoryDiagnosticFailed ( true );
    autoDiagnosisConfOM.setUseShutDownButtonOnSomeMandatoryDiagnosticFailed ( true );
    autoDiagnosisConfOM.setDiagnosticReviewWidth ( 500 );
    autoDiagnosisConfOM.setDiagnosticReviewHeight ( 100 );
    autoDiagnosisConfOM.setShowFooterInAutoDiagnosis ( true );
    autoDiagnosisConfRM.setDiagnosticToApply ( seqAutoDiagnosticToApplyRM );
    autoDiagnosisConfRM.setBlockDiagnostic ( false );
    autoDiagnosisConfRM.setBlockDiagnosticTitle ( blockDiagnosticTitleConfiguration );
    autoDiagnosisConfRM.setBlockDiagnosticButton ( blockDiagnosticButtonConf );
    autoDiagnosisConfRM.setUseOkButtonOnSomeNonMandatoryDiagnosticFailed ( true );
    autoDiagnosisConfRM.setUseShutDownButtonOnSomeMandatoryDiagnosticFailed ( true );
    autoDiagnosisConfRM.setDiagnosticReviewWidth ( 500 );
    autoDiagnosisConfRM.setDiagnosticReviewHeight ( 100 );
    autoDiagnosisConfRM.setShowFooterInAutoDiagnosis ( true );
    DeviceDiagnosticResultConfiguration deviceDiagnosticResult;
    deviceDiagnosticResult.setDiagnosticReviewWidth ( 500 );
    deviceDiagnosticResult.setDiagnosticReviewHeight ( 100 );
    deviceDiagnosticResult.setDiagnosticReviewLabelConfiguration ( diagnosticReviewConfiguration );
    XMLLabelConfiguration otherTitleConfiguration;
    otherTitleConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    otherTitleConfiguration.setLabel_VerticalPadding ( 1 );
    otherTitleConfiguration.setLabel_HorizontalPadding ( 1 );
    otherTitleConfiguration.setLabel_Family ( "Verdana" );
    otherTitleConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    SerialNumberConfiguration serialNumberConfiguration;
    serialNumberConfiguration.setPersistence ( false );
    otherTitleConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    otherTitleConfiguration.setLabel_Size ( 16 );
    serialNumberConfiguration.setLabelTitle ( otherTitleConfiguration );
    otherTitleConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    otherTitleConfiguration.setLabel_Size ( 12 );
    serialNumberConfiguration.setLabelInto ( otherTitleConfiguration );
    serialNumberConfiguration.setLabelMessage ( otherTitleConfiguration );

    ZipCodeConfiguration zipCodeConfiguration;
    zipCodeConfiguration.setMax ( 4 );
    zipCodeConfiguration.setMask ( "0123456789" );
    zipCodeConfiguration.setRegex ( "[0-9]{4}" );
    otherTitleConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    otherTitleConfiguration.setLabel_Size ( 16 );
    zipCodeConfiguration.setLabelTitle ( otherTitleConfiguration );
    otherTitleConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    otherTitleConfiguration.setLabel_Size ( 12 );
    zipCodeConfiguration.setLabelInto ( otherTitleConfiguration );
    zipCodeConfiguration.setLabelMessage ( otherTitleConfiguration );

    SerialNumberConfiguration::OptionSequence seqSerialNumber;
    seqSerialNumber.push_back ( SerialNumberOption ( 0, "", "", "", "" ) );
    seqSerialNumber.push_back ( SerialNumberOption ( 5, "0123456789", "[0-9]{5}", "P51VM4AA000*****", "P51VM4AA000" ) );
    seqSerialNumber.push_back ( SerialNumberOption ( 5, "0123456789", "[0-9]{5}", "P50VM3AA000*****", "P50VM3AA000" ) );
    seqSerialNumber.push_back ( SerialNumberOption ( 5, "0123456789", "[0-9]{5}", "P49BEPMA000*****", "P49BEPMA000" ) );
    serialNumberConfiguration.setOption ( seqSerialNumber );

    XMLLabelConfiguration labelsDemoConfiguration;
    labelsDemoConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    labelsDemoConfiguration.setLabel_VerticalPadding ( 1 );
    labelsDemoConfiguration.setLabel_HorizontalPadding ( 1 );
    labelsDemoConfiguration.setLabel_Family ( "Arial" );
    labelsDemoConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    labelsDemoConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    labelsDemoConfiguration.setLabel_Size ( 11 );
    voteDemoConf.setPersistVote ( true );
    voteDemoConf.setShowBarcodeInPreview ( false );
    voteDemoConf.setTitleFontConfiguration ( labelsDemoConfiguration );
    labelsDemoConfiguration.setLabel_Size ( 12 );
    voteDemoConf.setLabelSelectedConfiguration ( labelsDemoConfiguration );
    labelsDemoConfiguration.setLabel_Size ( 12 );
    voteDemoConf.setLabelUnselectedConfiguration ( labelsDemoConfiguration );

    voteDemoConf.setResetDefaultCustomAttribute ( true );

    activationCardConf.setShowAudioCheck ( false );
    activationCardConf.setCustomAttributesPresentation ( "EventBoxes" );
    activationCardConf.setShowConfirmationButtons ( false );
    activationCardConf.setMinimumDemoCounterToAllowActivation ( 1 );
    activationCardConf.setAssistedActivationBySelection ( false );
    activationCardConf.setActivatedAfterInsertingCard ( true );
    CountConfiguration cardActivationTotalCount;
    CountConfiguration::CountTypeSequence countTypeSequence;
    countTypeSequence.push_back ( CountTypeConfig::CONFIGURATION_TOTAL );
    cardActivationTotalCount.setCountType ( countTypeSequence );
    activationCardConf.setCardActivationTotalCountConfiguration ( cardActivationTotalCount );
    //Color Configuration
    selectedColorCardActiv.setRed ( 35500 );
    selectedColorCardActiv.setGreen ( 35500 );
    selectedColorCardActiv.setBlue ( 60000 );
    //Color Configuration
    unSelectedColorCardActiv.setRed ( 65535 );
    unSelectedColorCardActiv.setGreen ( 65535 );
    unSelectedColorCardActiv.setBlue ( 65535 );
    //Color Configuration
    ColorConfiguration activatedCardStatisticsTitleColor;
    activatedCardStatisticsTitleColor.setRed ( 65535 );
    activatedCardStatisticsTitleColor.setGreen ( 65535 );
    activatedCardStatisticsTitleColor.setBlue ( 65535 );
    //Statistics Conf
    ActivatedCardStatisticsConfiguration cardStatisticsConf;
    cardStatisticsConf.setHideOfficial ( false );
    cardStatisticsConf.setHideDemo ( true );
    cardStatisticsConf.setHideAudio ( true );
    cardStatisticsConf.setHideNonAudio ( true );
    cardStatisticsConf.setHideCA ( false );
    XMLLabelConfiguration activatedCardStatisticsTitleLabelConf;
    activatedCardStatisticsTitleLabelConf.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    activatedCardStatisticsTitleLabelConf.setLabel_VerticalPadding ( 1 );
    activatedCardStatisticsTitleLabelConf.setLabel_HorizontalPadding ( 1 );
    activatedCardStatisticsTitleLabelConf.setLabel_Family ( "Verdana" );
    activatedCardStatisticsTitleLabelConf.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    activatedCardStatisticsTitleLabelConf.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    activatedCardStatisticsTitleLabelConf.setLabel_Size ( 20 );
    activationCardConf.setStatisticsTitleLabelConfiguration ( activatedCardStatisticsTitleLabelConf );
    activationCardConf.setCardStatisticsConfiguration ( cardStatisticsConf );
    activationCardConf.setSelectedColor ( selectedColorCardActiv );
    activationCardConf.setUnSelectedColor ( unSelectedColorCardActiv );
    activationCardConf.setStatisticsTitleLabelConfiguration ( activatedCardStatisticsTitleLabelConf );
    activationCardConf.setStatisticsTitleColor ( activatedCardStatisticsTitleColor );

    AbleWidgetConfiguration ableWidgetConfiguration;
    ableWidgetConfiguration.setContinueButtonFailsExpected ( false );
    ableWidgetConfiguration.setContinueTimeOutButtonExpected ( false );
    ableWidgetConfiguration.setSimulationPressButton ( false );
    ableWidgetConfiguration.setFailButtonNotExpected ( false );
    ableWidgetConfiguration.setKeepButtonsPressed ( false );
    ableWidgetConfiguration.setCheckButtonsInSequence ( true );
    ButtonConfiguration skipButtonConf;
    skipButtonConf.setButtonWidth ( 200 );
    skipButtonConf.setButtonHeight ( 60 );
    skipButtonConf.setFontSize ( 18 );

    DateDiagnosticWidgetConfiguration dateDiagnosticWidgetConfiguration;

    time_t t = time ( NULL );
    tm * daTime = localtime ( &t );
    dateDiagnosticWidgetConfiguration.setMinYear ( daTime->tm_year + 1900 );
    dateDiagnosticWidgetConfiguration.setMaxYear ( daTime->tm_year + 1910 );
    dateDiagnosticWidgetConfiguration.setInputFontSize ( 24 );
    XMLLabelConfiguration dateDiagnosticWidgetTitleConfiguration;
    dateDiagnosticWidgetTitleConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    dateDiagnosticWidgetTitleConfiguration.setLabel_VerticalPadding ( 1 );
    dateDiagnosticWidgetTitleConfiguration.setLabel_HorizontalPadding ( 1 );
    dateDiagnosticWidgetTitleConfiguration.setLabel_Family ( "Verdana" );
    dateDiagnosticWidgetTitleConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    dateDiagnosticWidgetTitleConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    dateDiagnosticWidgetTitleConfiguration.setLabel_Size ( 20 );
    dateDiagnosticWidgetConfiguration.setWidgetTitleLabelConfiguration ( dateDiagnosticWidgetTitleConfiguration );
    dateDiagnosticWidgetConfiguration.setMonthYearLabelConfiguration ( dateDiagnosticWidgetTitleConfiguration );
    dateDiagnosticWidgetConfiguration.setButtonArrowWidth ( 50 );
    dateDiagnosticWidgetConfiguration.setButtonArrowHeight ( 50 );
    dateDiagnosticWidgetConfiguration.setEntryWidth ( 84 );
    dateDiagnosticWidgetConfiguration.setEntryHeight ( 72 );
    waitingCardWindow.setShowSimulateButton ( true );
    waitingCardWindow.setShowBarcodeInPreview ( false );
    waitingCardWindow.setShowBackButton ( true );
    waitingCardWindow.setViewPrevious ( false );
    waitingCardWindow.setAllowVotePreview ( true );
    waitingCardWindow.setTimeoutPreviewVote ( 90 );
    waitingCardWindow.setTimeoutPreviewVoteClose ( 3 );
    waitingCardWindow.setBlockOnCardInsertionError ( false );
    waitingCardWindow.setIsNeopBlockingCondition ( true );
    waitingCardWindow.setCallRestartSmartcardIfPressAlarmBoxAndNotAlarm ( false );
    waitingCardWindow.setShowElectoralInformationFrame ( false );
    waitingCardWindow.setElectoralInformationImage ( "Image.png" );

    XMLLabelConfiguration waitingCardPreviewVoteTitleLabelConf;
    waitingCardPreviewVoteTitleLabelConf.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    waitingCardPreviewVoteTitleLabelConf.setLabel_VerticalPadding ( 1 );
    waitingCardPreviewVoteTitleLabelConf.setLabel_HorizontalPadding ( 1 );
    waitingCardPreviewVoteTitleLabelConf.setLabel_Family ( "Verdana" );
    waitingCardPreviewVoteTitleLabelConf.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    waitingCardPreviewVoteTitleLabelConf.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    waitingCardPreviewVoteTitleLabelConf.setLabel_Size ( 20 );
    waitingCardWindow.setPreviewVoteLabelConfiguration ( waitingCardPreviewVoteTitleLabelConf );
    ButtonConfiguration waitingCardPreviewVoteButtonConf;
    waitingCardPreviewVoteButtonConf.setButtonWidth ( 100 );
    waitingCardPreviewVoteButtonConf.setButtonHeight ( 120 );
    waitingCardPreviewVoteButtonConf.setFontSize ( 20 );
    waitingCardWindow.setPreviewVoteButtonConfiguration ( waitingCardPreviewVoteButtonConf );
    FunctionalityConfiguration confOpenElection;
    confOpenElection.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMOPENELECTIONFUNCTIONALITY );
    confOpenElection.setElectionBlockStatus ( listElectionNotInitiated );
    confOpenElection.setShowExitMessage ( true );
    helpStandardFunctionality.setImageHelp ( "ui/open-election-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/open-election-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.OpenElection" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confOpenElection.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confCloseElection;
    confCloseElection.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMCLOSEELECTIONFUNCTIONALITY );
    confCloseElection.setElectionBlockStatus ( listElectionClosed );

    confCloseElection.setShowExitMessage ( true );
    helpStandardFunctionality.setImageHelp ( "ui/close-election-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/close-election-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.CloseElection" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confCloseElection.setHelp ( helpStandardFunctionality );

    FunctionalityConfiguration confSCReaderDiagnosis;
    confSCReaderDiagnosis.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMSCREADERDIAGNOSISFUNCTIONALITY );

    confSCReaderDiagnosis.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/sc-reader-diagnosis-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/sc-reader-diagnosis-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.ReaderDiagnosis" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confSCReaderDiagnosis.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confPrinterDiagnosis;
    confPrinterDiagnosis.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMPRINTERDIAGNOSISFUNCTIONALITY );

    confPrinterDiagnosis.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/printer-diagnosis-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/printer-diagnosis-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.PrinterDiagnosis" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confPrinterDiagnosis.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confAlarmBDiagnosis;
    confAlarmBDiagnosis.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMALARMBOXDIAGNOSISFUNCTIONALITY );

    confAlarmBDiagnosis.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/alarm-box-diagnosis-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/alarm-box-diagnosis-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.AlarmBoxDiagnosis" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confAlarmBDiagnosis.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confCalibDiagnosis;
    confCalibDiagnosis.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMCALIBRATIONDIAGNOSISFUNCTIONALITY );

    confCalibDiagnosis.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/calibration-diagnosis-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/calibration-diagnosis-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.CalibrationDiagnosis" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confCalibDiagnosis.setHelp ( helpStandardFunctionality );

    FunctionalityConfiguration confBarcodeDiagnosis;
    confBarcodeDiagnosis.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMBARCODEDIAGNOSISFUNCTIONALITY );

    confBarcodeDiagnosis.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/barcode-diagnosis-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/barcode-diagnosis-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.BarcodeDiagnosis" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confBarcodeDiagnosis.setHelp ( helpStandardFunctionality );
    NecessaryDevicesConfiguration necessaryDevicesBarcodeDiagnosis;
    NecessaryDevicesConfiguration::DeviceConfigurationSequence ndBarcodeDiagnosis;
    ndBarcodeDiagnosis.push_back ( barcodeDevice );
    necessaryDevicesBarcodeDiagnosis.setDeviceConfiguration ( ndBarcodeDiagnosis );
    confBarcodeDiagnosis.setNecessaryDevicesConfiguration ( necessaryDevicesBarcodeDiagnosis );
    FunctionalityConfiguration confAllDiagnosis;
    confAllDiagnosis.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMALLDIAGNOSISFUNCTIONALITY );

    confAllDiagnosis.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/all-diagnosis-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/all-diagnosis-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.AllDiagnosis" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confAllDiagnosis.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confFlashDiagnosis;
    confFlashDiagnosis.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMFLASHDIAGNOSISFUNCTIONALITY );

    confFlashDiagnosis.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/flash-diagnosis-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/flash-diagnosis-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.FlashDiagnosis" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confFlashDiagnosis.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confChangeHour;
    confChangeHour.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMCHANGEHOURFUNCTIONALITY );

    confChangeHour.setShowExitMessage ( true );
    helpStandardFunctionality.setImageHelp ( "ui/change-hour-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/change-hour-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.ChangeHour" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confChangeHour.setHelp ( helpStandardFunctionality );

    FunctionalityConfiguration confVotingMachineInstallation;
    confVotingMachineInstallation.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMVOTINGMACHINEINSTALLATIONFUNCTIONALITY );

    confVotingMachineInstallation.setShowExitMessage ( true );
    helpStandardFunctionality.setImageHelp ( "ui/voting-machine-installation-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/voting-machine-installation-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.VotingMachineInstallation" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confVotingMachineInstallation.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration skipVotingMachineInstallation;
    skipVotingMachineInstallation.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMSKIPVOTINGMACHINEINSTALLATIONFUNCTIONALITY );

    skipVotingMachineInstallation.setShowExitMessage ( true );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.SkipVotingMachineInstallation" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    skipVotingMachineInstallation.setHelp ( helpStandardFunctionality );
    NecessaryDevicesConfiguration necessaryDevicesVotingMachineInstallation;
    NecessaryDevicesConfiguration::DeviceConfigurationSequence ndVotingMachineInstallation;
    ndVotingMachineInstallation.push_back ( smartcardDevice );
    necessaryDevicesVotingMachineInstallation.setDeviceConfiguration ( ndVotingMachineInstallation );
    confVotingMachineInstallation.setNecessaryDevicesConfiguration ( necessaryDevicesVotingMachineInstallation );
    FunctionalityConfiguration confChangeLanguage;
    confChangeLanguage.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMCHANGELANGUAGEFUNCTIONALITY );

    confChangeLanguage.setShowExitMessage ( true );
    helpStandardFunctionality.setImageHelp ( "ui/change-language-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/change-language-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.ChangeLanguage" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confChangeLanguage.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confVoteProcessDemo;
    confVoteProcessDemo.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMVOTEPROCESSDEMOFUNCTIONALITY );

    confVoteProcessDemo.setShowExitMessage ( true );
    helpStandardFunctionality.setImageHelp ( "ui/vote-process-demo-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/vote-process-demo-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.VoteProcessDemo" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confVoteProcessDemo.setHelp ( helpStandardFunctionality );
    NecessaryDevicesConfiguration necessaryDevicesVoteProcessDemo;
    NecessaryDevicesConfiguration::DeviceConfigurationSequence ndVoteProcessDemo;
    ndVoteProcessDemo.push_back ( smartcardDevice );
    ndVoteProcessDemo.push_back ( barcodeDevice );
    necessaryDevicesVoteProcessDemo.setDeviceConfiguration ( ndVoteProcessDemo );
    confVoteProcessDemo.setNecessaryDevicesConfiguration ( necessaryDevicesVoteProcessDemo );
    FunctionalityConfiguration confPrintPreview;
    confPrintPreview.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMPRINTPREVIEWFUNCTIONALITY );

    confPrintPreview.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/print-preview-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/print-preview-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.PrintPreview" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confPrintPreview.setHelp ( helpStandardFunctionality );
    NecessaryDevicesConfiguration necessaryDevicesPrintPreview;
    NecessaryDevicesConfiguration::DeviceConfigurationSequence ndPrintPreview;
    ndPrintPreview.push_back ( barcodeDevice );
    necessaryDevicesPrintPreview.setDeviceConfiguration ( ndPrintPreview );
    confPrintPreview.setNecessaryDevicesConfiguration ( necessaryDevicesPrintPreview );

    FunctionalityConfiguration skipVoteDemo;
    skipVoteDemo.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMSKIPVOTEPROCESSDEMOFUNCTIONALITY );

    skipVoteDemo.setShowExitMessage ( true );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.SkipVoteDemo" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    skipVoteDemo.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confVotingExperienceValidation;
    confVotingExperienceValidation.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMVOTINGEXPERIENCEVALIDATIONFUNCTIONALITY );

    confVotingExperienceValidation.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/voting-experience-validation-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/voting-experience-validation-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.VotingExperienceValidation" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality .setEnableFilePrinting ( true );

    confVotingExperienceValidation.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confCloseVotation;
    confCloseVotation.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMCLOSEVOTATIONFUNCTIONALITY );
    confCloseVotation.setElectionBlockStatus ( listElectionInitiated );

    confCloseVotation.setShowExitMessage ( true );
    helpStandardFunctionality.setImageHelp ( "ui/close-votation-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/close-votation-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.CloseVotation" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confCloseVotation.setHelp ( helpStandardFunctionality );

    FunctionalityConfiguration confAbleDiagnosticInfo;
    confAbleDiagnosticInfo.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMABLEDIAGNOSTICFUNCTIONALITY );
    confAbleDiagnosticInfo.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/able-info-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/able-info-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.AbleDiagnosticInfo" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confAbleDiagnosticInfo.setHelp ( helpStandardFunctionality );

    FunctionalityConfiguration simulatorVotesFunctionality;
    simulatorVotesFunctionality.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMSIMULATORVOTESFUNCTIONALITY );

    simulatorVotesFunctionality.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.SimulatorVotesInfo" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    simulatorVotesFunctionality.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration authenticatorFunctionality;
    authenticatorFunctionality.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMAUTHENTICATORFUNCTIONALITY );

    authenticatorFunctionality.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.AuthenticatorInfo" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    authenticatorFunctionality.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration initiatorFunctionality;
    initiatorFunctionality.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMINITIATORFUNCTIONALITY );

    initiatorFunctionality.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.InitiatorInfo" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    initiatorFunctionality.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confZipCodeInfo;
    confZipCodeInfo.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMZIPCODEFUNCTIONALITY );

    confZipCodeInfo.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.ZipCodeInfo" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confZipCodeInfo.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confAutoDiagnosticInfo;
    confAutoDiagnosticInfo.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMAUTODIAGNOSTICFUNCTIONALITY );

    confAutoDiagnosticInfo.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( " " );
    helpStandardFunctionality.setPdfHelp ( " " );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.AutoDiagnosticInfo" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confAutoDiagnosticInfo.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confLoginInfo;
    confLoginInfo.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMLOGINFUNCTIONALITY );

    confLoginInfo.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( " " );
    helpStandardFunctionality.setPdfHelp ( " " );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.LoginInfo" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confLoginInfo.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confSerialNumberInfo;
    confSerialNumberInfo.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMSERIALNUMBERFUNCTIONALITY );

    confSerialNumberInfo.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.SerialNumberInfo" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confSerialNumberInfo.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confLanguageInstallFunctionality;
    confLanguageInstallFunctionality.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMLANGUAGEINSTALLFUNCTIONALITY );

    confLanguageInstallFunctionality.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( "" );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confLanguageInstallFunctionality.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confDeviceDiagnosticResutlFunctionality;
    confDeviceDiagnosticResutlFunctionality.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMDEVICEDIAGNOSTICRESULTFUNCTIONALITY );

    confDeviceDiagnosticResutlFunctionality.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( "" );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confDeviceDiagnosticResutlFunctionality.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confDeviceDiagnosticPrintFunctionality;
    confDeviceDiagnosticPrintFunctionality.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMDEVICEDIAGNOSTICPRINTFUNCTIONALITY );

    confDeviceDiagnosticPrintFunctionality.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( "" );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confDeviceDiagnosticPrintFunctionality.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confDeviceDiagnosticSaveFunctionality;
    confDeviceDiagnosticSaveFunctionality.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMDEVICEDIAGNOSTICSAVEFUNCTIONALITY );

    confDeviceDiagnosticSaveFunctionality.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( "" );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confDeviceDiagnosticSaveFunctionality.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confInitialCalibrationFunctionality;
    confInitialCalibrationFunctionality.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMINITIALCALIBRATIONFUNCTIONALITY );

    confInitialCalibrationFunctionality.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( "" );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confInitialCalibrationFunctionality.setHelp ( helpStandardFunctionality );

    FunctionalityConfiguration confDateDiagnosticFunctionality;
    confDateDiagnosticFunctionality.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMDATEDIAGNOSTICFUNCTIONALITY );

    confDateDiagnosticFunctionality.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "" );
    helpStandardFunctionality.setPdfHelp ( "" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.DateDiagnosticInfo" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confDateDiagnosticFunctionality.setHelp ( helpStandardFunctionality );

    FunctionalityConfiguration confActivationConsolidation;
    NecessaryDevicesConfiguration necessaryDevicesActivationConsolidation;
    NecessaryDevicesConfiguration::DeviceConfigurationSequence ndActivationConsolidation;
    ndActivationConsolidation.push_back ( smartcardDevice );
    necessaryDevicesActivationConsolidation.setDeviceConfiguration ( ndActivationConsolidation );
    confActivationConsolidation.setNecessaryDevicesConfiguration ( necessaryDevicesActivationConsolidation );
    confActivationConsolidation.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMACTIVATIONCONSOLIDATIONFUNCTIONALITY );

    confActivationConsolidation.setShowExitMessage ( true );
    confActivationConsolidation.setElectionBlockStatus ( listElectionInitiated );
    helpStandardFunctionality.setImageHelp ( "ui/activation-consolidation-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/activation-consolidation-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.ActivationConsolidation" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confActivationConsolidation.setHelp ( helpStandardFunctionality );
    FunctionalityConfiguration confReportPrinting;
    confReportPrinting.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMEXTERNALPRINTFUNCTIONALITY );

    confReportPrinting.setElectionBlockStatus ( listElectionResults );
    confReportPrinting.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/report-printing-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/report-printing-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.ReportPrinting" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confReportPrinting.setHelp ( helpStandardFunctionality );
    NecessaryDevicesConfiguration necessaryDevicesReportPrinting;
    NecessaryDevicesConfiguration::DeviceConfigurationSequence ndReportPrinting;
    ndReportPrinting.push_back ( printerDevice );
    necessaryDevicesReportPrinting.setDeviceConfiguration ( ndReportPrinting );
    confReportPrinting.setNecessaryDevicesConfiguration ( necessaryDevicesReportPrinting );
    FunctionalityConfiguration confLogReport;
    confLogReport.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMLOGREPORTFUNCTIONALITY );

    confLogReport.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( "ui/log-report-help.gif" );
    helpStandardFunctionality.setPdfHelp ( "ui/log-report-help.pdf" );
    helpStandardFunctionality.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.HelpConfiguration.LogReport" ) );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( true );

    confLogReport.setHelp ( helpStandardFunctionality );

    FunctionalityConfiguration confSearchAndMountFunctionality;
    confSearchAndMountFunctionality.setFunctionalityDescriptor ( MenuOptionType::CONFIGURATION_XMLMSEARCHANDMONITORDEVICESFUNCTIONALITY );

    confSearchAndMountFunctionality.setShowExitMessage ( false );
    helpStandardFunctionality.setImageHelp ( " " );
    helpStandardFunctionality.setPdfHelp ( " " );
    helpStandardFunctionality.setTitleKey ( " " );
    helpStandardFunctionality.setUseHelp ( false );
    helpStandardFunctionality.setEnableFilePrinting ( false );

    confSearchAndMountFunctionality.setHelp ( helpStandardFunctionality );

    funcConfigSequence.push_back ( confSearchAndMountFunctionality );
    funcConfigSequence.push_back ( confOpenElection );
    funcConfigSequence.push_back ( confCloseElection );
    funcConfigSequence.push_back ( confSCReaderDiagnosis );
    funcConfigSequence.push_back ( confPrinterDiagnosis );
    funcConfigSequence.push_back ( confAlarmBDiagnosis );
    funcConfigSequence.push_back ( confCalibDiagnosis );
    funcConfigSequence.push_back ( confBarcodeDiagnosis );
    funcConfigSequence.push_back ( confAllDiagnosis );
    funcConfigSequence.push_back ( confFlashDiagnosis );
    funcConfigSequence.push_back ( confChangeHour );
    funcConfigSequence.push_back ( confVotingMachineInstallation );
    funcConfigSequence.push_back ( confChangeLanguage );
    funcConfigSequence.push_back ( confVoteProcessDemo );
    funcConfigSequence.push_back ( confPrintPreview );
    funcConfigSequence.push_back ( confVotingExperienceValidation );
    funcConfigSequence.push_back ( confCloseVotation );
    funcConfigSequence.push_back ( confAbleDiagnosticInfo );
    funcConfigSequence.push_back ( confActivationConsolidation );
    funcConfigSequence.push_back ( confLogReport );
    funcConfigSequence.push_back ( confReportPrinting );
    funcConfigSequence.push_back ( confLanguageInstallFunctionality );
    funcConfigSequence.push_back ( confDateDiagnosticFunctionality );
    funcConfigSequence.push_back ( confInitialCalibrationFunctionality );
    funcConfigSequence.push_back ( confDeviceDiagnosticResutlFunctionality );
    funcConfigSequence.push_back ( confDeviceDiagnosticPrintFunctionality );
    funcConfigSequence.push_back ( confDeviceDiagnosticSaveFunctionality );
    funcConfigSequence.push_back ( confSerialNumberInfo );
    funcConfigSequence.push_back ( confAutoDiagnosticInfo );
    funcConfigSequence.push_back ( confLoginInfo );
    funcConfigSequence.push_back ( confZipCodeInfo );
    funcConfigSequence.push_back ( simulatorVotesFunctionality );
    funcConfigSequence.push_back ( initiatorFunctionality );
    funcConfigSequence.push_back ( authenticatorFunctionality );
    funcConfigSequence.push_back ( skipVotingMachineInstallation );
    funcConfigSequence.push_back ( skipVoteDemo );
    funcConfiguration.setFunctionalityConfiguration ( funcConfigSequence );

    ButtonConfiguration diagWidgetButtonConf;
    diagWidgetButtonConf.setButtonWidth ( 250 );
    diagWidgetButtonConf.setButtonHeight ( 60 );
    diagWidgetButtonConf.setFontSize ( 12 );

    generalDiag.setDiagnosisLabelConfiguration ( diagnosisLabelConfiguration );
    generalDiag.setDiagnosisFooterLabelConfiguration ( diagnosisFooterLabelConfiguration );
    generalDiag.setResultErrorLabelConfiguration ( resultErrorLabelConfiguration );
    generalDiag.setButtonConfiguration ( diagWidgetButtonConf );

    installWizardConfig.setSaveLoginKey ( true );
    installWizardConfig.setDeleteLoginKey ( false );

    EntryDividedWidgetConfiguration entryPassword;
    pmWidgetConfig.setShowImages ( true );
    pmWidgetConfig.setLoginCaseSensitiveConfiguration ( true );
    entryPassword.setNumberOfDivision ( 4 );
    entryPassword.setMaxCharacters ( 4 );
    entryPassword.setVisibility ( false );
    pmWidgetConfig.setPasswordWidgetConfiguration ( entryPassword );

    XMLLabelConfiguration authenticationPasswordLabelConfiguration;
    authenticationPasswordLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    authenticationPasswordLabelConfiguration.setLabel_VerticalPadding ( 0 );
    authenticationPasswordLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    authenticationPasswordLabelConfiguration.setLabel_Family ( "Verdana" );
    authenticationPasswordLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    authenticationPasswordLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    authenticationPasswordLabelConfiguration.setLabel_Size ( 50 );
    XMLLabelConfiguration authenticationAlarmLabelConfiguration;
    authenticationAlarmLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    authenticationAlarmLabelConfiguration.setLabel_VerticalPadding ( 0 );
    authenticationAlarmLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    authenticationAlarmLabelConfiguration.setLabel_Family ( "Verdana" );
    authenticationAlarmLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    authenticationAlarmLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    authenticationAlarmLabelConfiguration.setLabel_Size ( 14 );
    XMLLabelConfiguration authenticationInfoLabelConfiguration;
    authenticationInfoLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    authenticationInfoLabelConfiguration.setLabel_VerticalPadding ( 0 );
    authenticationInfoLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    authenticationInfoLabelConfiguration.setLabel_Family ( "Verdana" );
    authenticationInfoLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    authenticationInfoLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    authenticationInfoLabelConfiguration.setLabel_Size ( 20 );
    XMLLabelConfiguration authenticationVotingInfoLabelConfiguration;
    authenticationVotingInfoLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    authenticationVotingInfoLabelConfiguration.setLabel_VerticalPadding ( 0 );
    authenticationVotingInfoLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    authenticationVotingInfoLabelConfiguration.setLabel_Family ( "Verdana" );
    authenticationVotingInfoLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    authenticationVotingInfoLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    authenticationVotingInfoLabelConfiguration.setLabel_Size ( 14 );
    authWindowConf.setPasswordLabelConfiguration ( authenticationPasswordLabelConfiguration );
    authWindowConf.setAlarmLabelConfiguration ( authenticationAlarmLabelConfiguration );
    authWindowConf.setInfoLabelConfiguration ( authenticationInfoLabelConfiguration );
    authWindowConf.setVotingInfoLabelConfiguration ( authenticationVotingInfoLabelConfiguration );
    configSAESGUI.setFunctionalitiesConfiguration ( funcConfiguration );

    configSAESGUI.setInstallationWizardConfiguration ( installWizardConfig );
    configSAESGUI.setPMWidgetConfiguration ( pmWidgetConfig );
    configSAESGUI.setTouchScreenCalibrationConfiguration ( touchDiagConfig );
    configSAESGUI.setVotingExperience ( votingExperienceConfiguration );
    configSAESGUI.setGeneralDiagnosisConfiguration ( generalDiag );
    configSAESGUI.setPrinterDiagnosisConfiguration ( printerDiagnosisConf );

    GenericWidgetConfiguration genericWidgetConfiguration;

    XMLLabelConfiguration genericWidgetLabelConfiguration;
    genericWidgetLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    genericWidgetLabelConfiguration.setLabel_VerticalPadding ( 1 );
    genericWidgetLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    genericWidgetLabelConfiguration.setLabel_Family ( "Verdana" );
    genericWidgetLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    genericWidgetLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    genericWidgetLabelConfiguration.setLabel_Size ( 18 );

    genericWidgetConfiguration.setTitleConfiguration ( genericWidgetLabelConfiguration );
    genericWidgetLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    genericWidgetLabelConfiguration.setLabel_Size ( 14 );
    genericWidgetConfiguration.setInfoConfiguration ( genericWidgetLabelConfiguration );
    genericWidgetLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    genericWidgetLabelConfiguration.setLabel_Size ( 12 );
    genericWidgetConfiguration.setFooterConfiguration ( genericWidgetLabelConfiguration );

    ButtonConfiguration genericWidgetButtonConf;
    genericWidgetButtonConf.setButtonWidth ( 250 );
    genericWidgetButtonConf.setButtonHeight ( 60 );
    genericWidgetButtonConf.setFontSize ( 12 );

    genericWidgetConfiguration.setButtonConfiguration ( genericWidgetButtonConf );

    configSAESGUI.setGenericWidgetConfiguration ( genericWidgetConfiguration );

    configSAESGUI.setSmartcardReaderDiagnosisConfiguration ( screaderDiagnosisConf );

    ColorConfiguration abSelectedBoxColor ( 0, 0, 0 );
    ColorConfiguration abUnselectedBoxColor ( 65000, 65000, 65000 );

    LedStripDiagnosisConfiguration ledStripDiagnosisConfiguration;
    ledStripDiagnosisConfiguration.setDetectedTimeOk ( 3000 );
    ledStripDiagnosisConfiguration.setDetectedTimeFail ( 5000 );
    ledStripDiagnosisConfiguration.setActionsTimeOk ( 3000 );
    ledStripDiagnosisConfiguration.setActionsTimeFail ( 5000 );
    genericWidgetButtonConf.setButtonWidth ( 200 );

    ledStripDiagnosisConfiguration.setButtonConfiguration ( genericWidgetButtonConf );
    configSAESGUI.setLedStripDiagnosisConfiguration ( ledStripDiagnosisConfiguration );

    EurnDiagnosisConfiguration eurnDiagnosisConfiguration;
    eurnDiagnosisConfiguration.setDetectedTimeOk ( 3000 );
    eurnDiagnosisConfiguration.setDetectedTimeFail ( 5000 );
    eurnDiagnosisConfiguration.setActionsTimeOk ( 3000 );
    eurnDiagnosisConfiguration.setActionsTimeFail ( 5000 );
    genericWidgetButtonConf.setButtonWidth ( 200 );

    eurnDiagnosisConfiguration.setButtonConfiguration ( genericWidgetButtonConf );
    configSAESGUI.setEurnDiagnosisConfiguration ( eurnDiagnosisConfiguration );


    AlarmBoxDiagnosisConfiguration alarmBoxDiagnosisConfiguration;
    alarmBoxDiagnosisConfiguration.setDetectedTimeOk ( 3000 );
    alarmBoxDiagnosisConfiguration.setDetectedTimeFail ( 5000 );
    alarmBoxDiagnosisConfiguration.setActionsTimeOk ( 3000 );
    alarmBoxDiagnosisConfiguration.setActionsTimeFail ( 5000 );
    alarmBoxDiagnosisConfiguration.setPeriodBuzzer ( 1000 );
    alarmBoxDiagnosisConfiguration.setPeriodRedLed ( 1 );
    alarmBoxDiagnosisConfiguration.setPeriodGreenLed ( 1 );
    alarmBoxDiagnosisConfiguration.setWidthFirstColumn ( 350 );
    alarmBoxDiagnosisConfiguration.setShowBuzzerButtonTest ( true );
    alarmBoxDiagnosisConfiguration.setShowRedLedButtonTest ( true );
    alarmBoxDiagnosisConfiguration.setShowGreenLedButtonTest ( true );
    alarmBoxDiagnosisConfiguration.setSelectedBoxColor ( abSelectedBoxColor );
    alarmBoxDiagnosisConfiguration.setUnselectedBoxColor ( abUnselectedBoxColor );
    alarmBoxDiagnosisConfiguration.setWidthBoxColor ( 200 );
    alarmBoxDiagnosisConfiguration.setHeightBoxColor ( 60 );

    genericWidgetButtonConf.setButtonWidth ( 200 );

    alarmBoxDiagnosisConfiguration.setButtonConfiguration ( genericWidgetButtonConf );

    configSAESGUI.setAlarmBoxDiagnosisConfiguration ( alarmBoxDiagnosisConfiguration );

    configSAESGUI.setBarcodeDiagnosisConfiguration ( barcodeDiagnosisConf );
    configSAESGUI.setAllDiagnosisConfiguration ( allDiagnosisConf );
    configSAESGUI.setAutomaticWindowConfigurationForOfficialMode( autoDiagnosisConfOM );
    configSAESGUI.setAutomaticWindowConfigurationForRecountMode( autoDiagnosisConfRM );
    configSAESGUI.setDeviceDiagnosticResultConfiguration ( deviceDiagnosticResult );
    configSAESGUI.setSerialNumberConfiguration ( serialNumberConfiguration );
    configSAESGUI.setZipCodeConfiguration ( zipCodeConfiguration );
    configSAESGUI.setCardActivatorFunctionalityConfiguration ( activationCardConf );
    configSAESGUI.setVoteDemoFunctionalityConfiguration ( voteDemoConf );
    configSAESGUI.setWaitingForCardWindowConfiguration ( waitingCardWindow );

    configSAESGUI.setAbleWidgetConfiguration ( ableWidgetConfiguration );
    configSAESGUI.setFormsCommonControls ( formCommonControl );
    configSAESGUI.setAuthenticationWindowConfiguration ( authWindowConf );
    configSAESGUI.setDateDiagnosticWidgetConfiguration ( dateDiagnosticWidgetConfiguration );

    openElectionConfig.setDialogLabelConfiguration ( genericWidgetLabelConfiguration );
    configSAESGUI.setOpenElectionFunctionalityConfiguration ( openElectionConfig );

    //close election functionality
    CloseElectionFunctionalityConfiguration::ClosingElectionStepsSequence closeSeq;
    closeSeq.push_back ( CloseElectionSteps::CONFIGURATION_CLOSECONFIRMATION );
    closeSeq.push_back ( CloseElectionSteps::CONFIGURATION_GENERATEALLRESULT );
    closeSeq.push_back ( CloseElectionSteps::CONFIGURATION_CLOSEELECTION );
    closeSeq.push_back ( CloseElectionSteps::CONFIGURATION_EXTERNALPRINTPV );
    closeElectionconfig.setClosingElectionSteps ( closeSeq );
    closeElectionconfig.setButtonConfiguration ( buttonBoxConfig );
    closeElectionconfig.setShowSignQuestion ( false );
    closeElectionconfig.setShowSignConfirmation ( false );
    closeElectionconfig.setContinueCloseOnSignFail ( true );
    closeElectionconfig.setShowConfirmation ( true );

    configSAESGUI.setCloseElectionFunctionalityConfiguration ( closeElectionconfig );
    //Activation consolidation functionality
    ActivationConsolidationFunctionalityConfiguration activationConsolidationFunctionality;
    activationConsolidationFunctionality.setTitleHeight ( 60 );
    activationConsolidationFunctionality.setStatsConsolidationHeight ( 150 );
    activationConsolidationFunctionality.setStatsActivationHeight ( 110 );
    activationConsolidationFunctionality.setActivateCardsWithoutBarcode ( true );
    activationConsolidationFunctionality.setConsolidateVoteWithOneMemory ( false );
    ColorConfiguration acColor1 ( 100, 30000, 30000 );
    activationConsolidationFunctionality.setSelectedBoxColor ( acColor1 );
    ColorConfiguration acColor2 ( 65000, 65000, 65000 );
    activationConsolidationFunctionality.setUnselectedBoxColor ( acColor2 );
    ColorConfiguration acColor3 ( 65000, 65000, 65000 );
    activationConsolidationFunctionality.setSelectedFontColor ( acColor3 );
    ColorConfiguration acColor4 ( 10, 10, 10 );
    activationConsolidationFunctionality.setUnselectedFontColor ( acColor4 );
    activationConsolidationFunctionality.setShowTotalVotes ( false );
    activationConsolidationFunctionality.setShowOfficialVotes ( true );
    activationConsolidationFunctionality.setShowDuplicatesVotes ( true );
    activationConsolidationFunctionality.setShowScanMessage ( true );
    activationConsolidationFunctionality.setShowBarcodeMessage ( true );
    activationConsolidationFunctionality.setShowActivateCards ( true );
    activationConsolidationFunctionality.setTimerSleepWriteCard ( 10000 );
    activationConsolidationFunctionality.setResetDefaultCustomAttribute ( true );

    XMLLabelConfiguration labelsConfiguration;
    labelsConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    labelsConfiguration.setLabel_VerticalPadding ( 1 );
    labelsConfiguration.setLabel_HorizontalPadding ( 1 );
    labelsConfiguration.setLabel_Family ( "Verdana" );
    labelsConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    labelsConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    labelsConfiguration.setLabel_Size ( 13 );
    activationConsolidationFunctionality.setTitleFontConfiguration ( labelsConfiguration );
    labelsConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    labelsConfiguration.setLabel_Size ( 13 );
    activationConsolidationFunctionality.setStatsConsolidationFontConfiguration ( labelsConfiguration );
    activationConsolidationFunctionality.setStatsActivationFontConfiguration ( labelsConfiguration );
    labelsConfiguration.setLabel_Size ( 15 );
    activationConsolidationFunctionality.setInstructionFontConfiguration ( labelsConfiguration );
    labelsConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_LEFT );
    labelsConfiguration.setLabel_Size ( 14 );
    activationConsolidationFunctionality.setDevicesLabelConfiguration ( labelsConfiguration );
    labelsConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    labelsConfiguration.setLabel_Size ( 15 );
    labelsConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    activationConsolidationFunctionality.setLabelSelectedConfiguration ( labelsConfiguration );
    labelsConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    activationConsolidationFunctionality.setLabelUnselectedConfiguration ( labelsConfiguration );
    CountConfiguration cardActivationTotal;
    CountConfiguration::CountTypeSequence countTypeCardSeq;
    countTypeCardSeq.push_back ( CountTypeConfig::CONFIGURATION_TOTAL );
    cardActivationTotal.setCountType ( countTypeCardSeq );
    CountConfiguration cosolidationTotal;
    CountConfiguration::CountTypeSequence countTypeConsolidationSeq;
    countTypeConsolidationSeq.push_back ( CountTypeConfig::CONFIGURATION_TOTAL );
    cosolidationTotal.setCountType ( countTypeConsolidationSeq );
    CountConfiguration cosolidationOfficial;
    CountConfiguration::CountTypeSequence countTypeConsolidationOfficialSeq;
    countTypeConsolidationOfficialSeq.push_back ( CountTypeConfig::CONFIGURATION_OFFICIAL );
    cosolidationOfficial.setCountType ( countTypeConsolidationOfficialSeq );
    activationConsolidationFunctionality.setCardActivationTotalCountConfiguration ( cardActivationTotal );
    activationConsolidationFunctionality.setConsolidationTotalCountConfiguration ( cosolidationTotal );
    activationConsolidationFunctionality.setConsolidationOfficialCountConfiguration ( cosolidationOfficial );
    configSAESGUI.setActivationConsolidationFunctionalityConfiguration ( activationConsolidationFunctionality );

    //LogReport functionality
    logReportFunconfig.setPageSize ( 10 );
    logReportFunconfig.setEnablePrintReport ( true );

    configSAESGUI.setLogReportFunctionalityConfiguration ( logReportFunconfig );

    //vote consolidation functionality

    voteConsolidationFuncConfig.setConsolidateDemoVotes ( true );
    voteConsolidationFuncConfig.setConsolidateSimulatedVotes ( false );

    configSAESGUI.setVoteConsolidationFunctionalityConfiguration ( voteConsolidationFuncConfig );
    ScanVoteProcessConfiguration scanVoteProcessConfiguration;
    ScanConsolidationConfiguration scanConfiguration;
    ColorConfiguration scanBoxColorValid;
    scanBoxColorValid.setRed ( 0 );
    scanBoxColorValid.setGreen ( 65535 );
    scanBoxColorValid.setBlue ( 0 );
    ColorConfiguration scanBoxColorBad;
    scanBoxColorBad.setRed ( 65535 );
    scanBoxColorBad.setGreen ( 0 );
    scanBoxColorBad.setBlue ( 0 );
    ColorConfiguration scanBoxColorDuplicate;
    scanBoxColorDuplicate.setRed ( 65535 );
    scanBoxColorDuplicate.setGreen ( 0 );
    scanBoxColorDuplicate.setBlue ( 0 );
    ColorConfiguration scanFontColorValid;
    scanFontColorValid.setRed ( 65535 );
    scanFontColorValid.setGreen ( 65535 );
    scanFontColorValid.setBlue ( 65535 );
    ColorConfiguration scanFontColorBad;
    scanFontColorBad.setRed ( 0 );
    scanFontColorBad.setGreen ( 0 );
    scanFontColorBad.setBlue ( 0 );
    ColorConfiguration scanFontColorDuplicate;
    scanFontColorDuplicate.setRed ( 0 );
    scanFontColorDuplicate.setGreen ( 0 );
    scanFontColorDuplicate.setBlue ( 0 );
    XMLLabelConfiguration scanLabelConfiguration;
    scanLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    scanLabelConfiguration.setLabel_VerticalPadding ( 1 );
    scanLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    scanLabelConfiguration.setLabel_Family ( "Verdana" );
    scanLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    scanLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    scanLabelConfiguration.setLabel_Size ( 16 );
    ScanPopUpMessageConfiguration scanPopUpMessageConfiguration;
    scanPopUpMessageConfiguration.setVisible ( true );
    scanPopUpMessageConfiguration.setTimeOut ( 3000 );
    scanPopUpMessageConfiguration.setOpacity ( 90 );
    scanPopUpMessageConfiguration.setWidth ( 200 );
    scanPopUpMessageConfiguration.setHeight ( 100 );
    scanPopUpMessageConfiguration.setX ( 0 );
    scanPopUpMessageConfiguration.setY ( 0 );
    scanPopUpMessageConfiguration.setPosition ( Smartmatic::GUI::Configuration::PositionConfiguration ( Smartmatic::GUI::Configuration::PositionConfiguration::CONFIGURATION_BOTTOM_CENTER ) );
    scanConfiguration.setScanPopUpMessageConfiguration ( scanPopUpMessageConfiguration );
    scanConfiguration.setEnabledChangeColor ( true );
    scanConfiguration.setBoxColorValid ( scanBoxColorValid );
    scanConfiguration.setBoxColorDuplicate ( scanBoxColorDuplicate );
    scanConfiguration.setBoxColorBad ( scanBoxColorBad );
    scanConfiguration.setFontColorValid ( scanFontColorValid );
    scanConfiguration.setFontColorDuplicate ( scanFontColorDuplicate );
    scanConfiguration.setFontColorBad ( scanFontColorBad );
    scanConfiguration.setTextConfiguration ( scanLabelConfiguration );
    scanVoteProcessConfiguration.setNormalScanConsolidationConfiguration ( scanConfiguration );
    scanPopUpMessageConfiguration.setPosition ( Smartmatic::GUI::Configuration::PositionConfiguration ( Smartmatic::GUI::Configuration::PositionConfiguration::CONFIGURATION_TOP_LEFT ) );
    scanConfiguration.setScanPopUpMessageConfiguration ( scanPopUpMessageConfiguration );
    scanVoteProcessConfiguration.setReduceScanConsolidationConfiguration ( scanConfiguration );
    printPreviewFuncConfig.setShowBarcodeInPreview ( false );
    configSAESGUI.setPrintPreviewFunctionalityConfiguration ( printPreviewFuncConfig );
    configSAESGUI.setScanVoteProcessConfiguration ( scanVoteProcessConfiguration );

    DeviceTypeMonitorWindowConfiguration deviceMonitorWindowConfiguration;
    deviceMonitorWindowConfiguration.setVisible ( true );
    deviceMonitorWindowConfiguration.setOpacity ( 80 );
    deviceMonitorWindowConfiguration.setTimeOut ( 5000 );
    deviceMonitorWindowConfiguration.setIconHeight ( 64 );
    deviceMonitorWindowConfiguration.setIconWidth ( 64 );
    deviceMonitorWindowConfiguration.setBorder ( 10 );
    deviceMonitorWindowConfiguration.setPosition ( Smartmatic::GUI::Configuration::PositionConfiguration::CONFIGURATION_BOTTOM_RIGHT );

    DevicesToShowConfiguration devicesToShow;
    DevicesToShowConfiguration::DeviceTypeMonitorSequence deviceToShowSeq;

    deviceToShowSeq.push_back ( DeviceConfiguration::CONFIGURATION_BARCODE );
    deviceToShowSeq.push_back ( DeviceConfiguration::CONFIGURATION_PRINTER );
    deviceToShowSeq.push_back ( DeviceConfiguration::CONFIGURATION_FLASH );
    deviceToShowSeq.push_back ( DeviceConfiguration::CONFIGURATION_USBSTORAGEMOUNTED );
    deviceToShowSeq.push_back ( DeviceConfiguration::CONFIGURATION_SMARTCARD );
    deviceToShowSeq.push_back ( DeviceConfiguration::CONFIGURATION_MOUSE );

    devicesToShow.setDeviceTypeMonitor ( deviceToShowSeq );
    deviceMonitorWindowConfiguration.setDevicesToShow ( devicesToShow );

    configSAESGUI.setDeviceTypeMonitorWindowConfiguration ( deviceMonitorWindowConfiguration );

    USBMonitorWindowConfiguration usbMonitorWindowConfiguration;
    usbMonitorWindowConfiguration.setVisible ( true );
    usbMonitorWindowConfiguration.setOpacity ( 100 );
    usbMonitorWindowConfiguration.setTimeOut ( 0 );
    usbMonitorWindowConfiguration.setHeight ( 200 );
    usbMonitorWindowConfiguration.setWidth ( 300 );
    usbMonitorWindowConfiguration.setX ( 0 );
    usbMonitorWindowConfiguration.setY ( 0 );
    usbMonitorWindowConfiguration.setPosition ( Smartmatic::GUI::Configuration::PositionConfiguration::CONFIGURATION_CUSTOM );

    scanLabelConfiguration.setLabel_Size ( 12 );

    usbMonitorWindowConfiguration.setLabelConfiguration ( scanLabelConfiguration );

    configSAESGUI.setUSBMonitorWindowConfiguration ( usbMonitorWindowConfiguration );

    ButtonConfiguration screensButtonConf;
    screensButtonConf.setButtonWidth ( 350 );
    screensButtonConf.setButtonHeight ( 100 );
    screensButtonConf.setFontSize ( 16 );
    //voting experience validation screens conf

    VotingExperienceValidationScreensFunctionalityConfiguration votingExperienceValidationScreensConf;

    votingExperienceValidationScreensConf.setScreenCaptureSound ( "ScreenCaptureSound" );
    votingExperienceValidationScreensConf.setAutorun ( true );
    votingExperienceValidationScreensConf.setTimeForStart ( 500 );
    votingExperienceValidationScreensConf.setAutoclose ( true );
    votingExperienceValidationScreensConf.setTimeForClose ( 1000 );
    votingExperienceValidationScreensConf.setTimeToShow ( 500 );
    votingExperienceValidationScreensConf.setScreensButtonConf ( screensButtonConf );
    votingExperienceValidationScreensConf.setCheckImageFile ( true );
    votingExperienceValidationScreensConf.setScreenOnlyFirstBallot ( false );

    VotingExperienceValidationScreensFunctionalityConfiguration::ImageSequence imageSequence;

    ScreenImageConfiguration waiting ( "ZZ_%1_%2-We.png", BasicImageType::GENERATED_IMAGES_WAITING_CARD );
    ScreenImageConfiguration readCard ( "ZZ_%1_%2-RC.png", BasicImageType::GENERATED_IMAGES_READ_CARD );
    ScreenImageConfiguration cardUsed ( "ZZ_%1_%2-CU.png", BasicImageType::GENERATED_IMAGES_CARD_USED );
    ScreenImageConfiguration errorCard ( "ZZ_%1_%2-EC.png", BasicImageType::GENERATED_IMAGES_ERROR_CARD );
    ScreenImageConfiguration errorCardAID ( "ZZ_%1_%2-ECAID.png", BasicImageType::GENERATED_IMAGES_ERROR_CARD_AID );
    ScreenImageConfiguration errorCardPIN ( "ZZ_%1_%2-ECPIN.png", BasicImageType::GENERATED_IMAGES_ERROR_CARD_PIN );
    ScreenImageConfiguration errorCardGeneric ( "ZZ_%1_%2-ECG.png", BasicImageType::GENERATED_IMAGES_ERROR_CARD_GENERIC );
    ScreenImageConfiguration language ( "ZZ_%1_%2-LS.png", BasicImageType::GENERATED_IMAGES_LANGUAGE );
    ScreenImageConfiguration languageConf ( "ZZ_%1_%2-LSC-%3.png", BasicImageType::GENERATED_IMAGES_LANGUAGE_CONFIRMATION );
    ScreenImageConfiguration customAttribute ( "%1_%2-%4_0_CA.png", BasicImageType::GENERATED_IMAGES_CUSTOM_ATTRIBUTE );
    ScreenImageConfiguration customAttributeConf ( "%ZZ1_%2-CAC-%3-%4.png", BasicImageType::GENERATED_IMAGES_CUSTOM_ATTRIBUTE_CONFIRMATION );
    ScreenImageConfiguration party ( "%1_%2-%4_%3_0000.png", BasicImageType::GENERATED_IMAGES_PARTY );
    ScreenImageConfiguration nullParty ( "ZZ_%1_%2-NC.png", BasicImageType::GENERATED_IMAGES_NULL_PARTY );
    ScreenImageConfiguration ballot ( "%1_%2-%5_%3_%4.png", BasicImageType::GENERATED_IMAGES_BALLOT );
    ScreenImageConfiguration print ( "ZZ_%1_%2-P.png", BasicImageType::GENERATED_IMAGES_PRINT_VOTE );
    ScreenImageConfiguration errorPrint ( "ZZ_%1_%2-EP.png", BasicImageType::GENERATED_IMAGES_ERROR_PRINT_VOTE );
    ScreenImageConfiguration fold ( "ZZ_%1_%2-FV.png", BasicImageType::GENERATED_IMAGES_FOLD_VOTE );
    ScreenImageConfiguration printFold ( "ZZ_%1_%2-PFV.png", BasicImageType::GENERATED_IMAGES_PRINT_FOLD_VOTE );
    ScreenImageConfiguration removeCard ( "ZZ_%1_%2-CRI.png", BasicImageType::GENERATED_IMAGES_REMOVE_CARD );
    ScreenImageConfiguration cardRemoved ( "ZZ_%1_%2-CR.png", BasicImageType::GENERATED_IMAGES_CARD_REMOVED );
    ScreenImageConfiguration cardRemovedCounter ( "ZZ_%1_%2-CRC.png", BasicImageType::GENERATED_IMAGES_CARD_REMOVED_COUNTER );
    ScreenImageConfiguration cardRemovedCancelProcess ( "ZZ_%1_%2-CRCP.png", BasicImageType::GENERATED_IMAGES_CARD_REMOVED_CANCEL_PROCESS );
    ScreenImageConfiguration cardRemovedInterrupt ( "ZZ_%1_%2-CRIVE.png", BasicImageType::GENERATED_IMAGES_INTERRUPT_VOTING_EXPERIENCE );
    ScreenImageConfiguration nullVotingExperience ( "ZZ_%1_%2-NVE.png", BasicImageType::GENERATED_IMAGES_NULL_VOTING_EXPERIENCE );

    imageSequence.push_back ( waiting );
    imageSequence.push_back ( readCard );
    imageSequence.push_back ( cardUsed );
    imageSequence.push_back ( errorCard );
    imageSequence.push_back ( errorCardAID );
    imageSequence.push_back ( errorCardPIN );
    imageSequence.push_back ( errorCardGeneric );
    imageSequence.push_back ( language );
    imageSequence.push_back ( languageConf );
    imageSequence.push_back ( customAttribute );
    imageSequence.push_back ( customAttributeConf );
    imageSequence.push_back ( party );
    imageSequence.push_back ( nullParty );
    imageSequence.push_back ( ballot );
    imageSequence.push_back ( print );
    imageSequence.push_back ( fold );
    imageSequence.push_back ( printFold );
    imageSequence.push_back ( errorPrint );
    imageSequence.push_back ( removeCard );
    imageSequence.push_back ( cardRemoved );
    imageSequence.push_back ( cardRemovedCounter );
    imageSequence.push_back ( cardRemovedCancelProcess );
    imageSequence.push_back ( cardRemovedInterrupt );
    imageSequence.push_back ( nullVotingExperience );

    votingExperienceValidationScreensConf.setImage ( imageSequence );

    configSAESGUI.setVotingExperienceValidationScreensFunctionalityConfiguration ( votingExperienceValidationScreensConf );

    ProcessStatusResultsShutdownFunctionalityConfiguration processStatusResultsShutdownFunctionalityConfiguration;
    processStatusResultsShutdownFunctionalityConfiguration.setShutDownButtonConf ( screensButtonConf );
    configSAESGUI.setProcessStatusResultsShutdownFunctionalityConfiguration ( processStatusResultsShutdownFunctionalityConfiguration );

    ShutdownWidgetConfiguration shutdownWidgetConfiguration;
    XMLLabelConfiguration shutdownWidgetLabelConf;
    shutdownWidgetLabelConf.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    shutdownWidgetLabelConf.setLabel_VerticalPadding ( 1 );
    shutdownWidgetLabelConf.setLabel_HorizontalPadding ( 1 );
    shutdownWidgetLabelConf.setLabel_Family ( "Verdana" );
    shutdownWidgetLabelConf.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    shutdownWidgetLabelConf.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    shutdownWidgetLabelConf.setLabel_Size ( 24 );
    shutdownWidgetConfiguration.setLabelConfiguration ( shutdownWidgetLabelConf );
    ButtonConfiguration shutdownWidgetButtonConf;
    shutdownWidgetButtonConf.setButtonWidth ( 200 );
    shutdownWidgetButtonConf.setButtonHeight ( 60 );
    shutdownWidgetButtonConf.setFontSize ( 14 );
    shutdownWidgetConfiguration.setButtonConfiguration ( shutdownWidgetButtonConf );
    configSAESGUI.setShutdownWidgetConfiguration ( shutdownWidgetConfiguration );

    /// voting for disabled
    ButtonConfiguration addButtonConf;
    addButtonConf.setButtonWidth ( 320 );
    addButtonConf.setButtonHeight ( 50 );
    addButtonConf.setFontSize ( 12 );
    ButtonConfiguration questionButtonConf;
    questionButtonConf.setButtonWidth ( 350 );
    questionButtonConf.setButtonHeight ( 100 );
    questionButtonConf.setFontSize ( 16 );
    ButtonConfiguration addCancelButtonConf;
    addCancelButtonConf.setButtonWidth ( 220 );
    addCancelButtonConf.setButtonHeight ( 50 );
    addCancelButtonConf.setFontSize ( 12 );
    XMLLabelConfiguration questionTitleConfiguration;
    questionTitleConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    questionTitleConfiguration.setLabel_VerticalPadding ( 1 );
    questionTitleConfiguration.setLabel_HorizontalPadding ( 1 );
    questionTitleConfiguration.setLabel_Family ( "Verdana" );
    questionTitleConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    questionTitleConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    questionTitleConfiguration.setLabel_Size ( 14 );
    XMLLabelConfiguration infoTitleConfiguration;
    infoTitleConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    infoTitleConfiguration.setLabel_VerticalPadding ( 1 );
    infoTitleConfiguration.setLabel_HorizontalPadding ( 1 );
    infoTitleConfiguration.setLabel_Family ( "Verdana" );
    infoTitleConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    infoTitleConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    infoTitleConfiguration.setLabel_Size ( 14 );
    XMLLabelConfiguration infoIDConfiguration;
    infoIDConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_LEFT );
    infoIDConfiguration.setLabel_VerticalPadding ( 1 );
    infoIDConfiguration.setLabel_HorizontalPadding ( 1 );
    infoIDConfiguration.setLabel_Family ( "Verdana" );
    infoIDConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    infoIDConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    infoIDConfiguration.setLabel_Size ( 12 );
    XMLLabelConfiguration infoTextConfiguration;
    infoTextConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    infoTextConfiguration.setLabel_VerticalPadding ( 1 );
    infoTextConfiguration.setLabel_HorizontalPadding ( 1 );
    infoTextConfiguration.setLabel_Family ( "Verdana" );
    infoTextConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    infoTextConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    infoTextConfiguration.setLabel_Size ( 12 );
    XMLLabelConfiguration infoErrorConfiguration;
    infoErrorConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    infoErrorConfiguration.setLabel_VerticalPadding ( 1 );
    infoErrorConfiguration.setLabel_HorizontalPadding ( 1 );
    infoErrorConfiguration.setLabel_Family ( "Verdana" );
    infoErrorConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    infoErrorConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    infoErrorConfiguration.setLabel_Size ( 16 );

    DataUtilsValidatorFunctionalityConfiguration dataUtilsValidatorFunctionalityConfiguration;
    dataUtilsValidatorFunctionalityConfiguration.setKeyFile ( "votingDevices" );
    dataUtilsValidatorFunctionalityConfiguration.setSeparator ( "|" );
    dataUtilsValidatorFunctionalityConfiguration.setPositionCode ( 0 );
    dataUtilsValidatorFunctionalityConfiguration.setPositionPassword ( 4 );
    dataUtilsValidatorFunctionalityConfiguration.setResultFolder ( "ResultValidator" );
    dataUtilsValidatorFunctionalityConfiguration.setExportResults ( true );
    configSAESGUI.setDataUtilsValidatorFunctionalityConfiguration ( dataUtilsValidatorFunctionalityConfiguration );

    ProcessResultWidgetConfiguration processResultWidgetConfiguration;
    processResultWidgetConfiguration.setColumns ( 2 );
    processResultWidgetConfiguration.setWidth ( 350 );
    processResultWidgetConfiguration.setHeight ( 40 );
    processResultWidgetConfiguration.setElementsByPage ( 20 );
    processResultWidgetConfiguration.setShowExitButton ( true );
    configSAESGUI.setProcessResultWidgetConfiguration ( processResultWidgetConfiguration );

    XMLLabelConfiguration languageSelectionLabelConfiguration;
    languageSelectionLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    languageSelectionLabelConfiguration.setLabel_VerticalPadding ( 1 );
    languageSelectionLabelConfiguration.setLabel_HorizontalPadding ( 1 );
    languageSelectionLabelConfiguration.setLabel_Family ( "Verdana" );
    languageSelectionLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    languageSelectionLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    languageSelectionLabelConfiguration.setLabel_Size ( 16 );

    SelectionWidgetConfiguration languageSelectionWidgetConfiguration;
    languageSelectionWidgetConfiguration.setAutoSelect ( false );
    languageSelectionWidgetConfiguration.setSortAlphabetical ( false );
    languageSelectionWidgetConfiguration.setEnableClick ( true );
    languageSelectionWidgetConfiguration.setButtonType ( ViewButtonConfiguration::CONFIGURATION_SEPARATE_BUTTONS );

    ExpandButtonConfiguration languageSelectionExpandButtonConfiguration;
    languageSelectionExpandButtonConfiguration.setFontSize ( 14 );
    languageSelectionExpandButtonConfiguration.setSpacing ( 10 );
    languageSelectionWidgetConfiguration.setExpandButton ( languageSelectionExpandButtonConfiguration );

    SelectionWidgetSeparateButtonConfiguration languageSelectionSeparateButtonConfiguration;
    languageSelectionSeparateButtonConfiguration.setAcceptButtonWidth ( 330 );
    languageSelectionSeparateButtonConfiguration.setClearButtonWidth ( 330 );
    languageSelectionSeparateButtonConfiguration.setFontSize ( 14 );
    languageSelectionWidgetConfiguration.setSeparateButton ( languageSelectionSeparateButtonConfiguration );

    languageSelectionWidgetConfiguration.setFrameTopHeight ( 80 );
    languageSelectionWidgetConfiguration.setFrameBottomHeight ( 80 );
    languageSelectionWidgetConfiguration.setShowBorder ( true );
    languageSelectionWidgetConfiguration.setBoxSpacing ( 150 );
    languageSelectionWidgetConfiguration.setBoxWidth ( 450 );
    languageSelectionWidgetConfiguration.setBoxHeight ( 100 );
    languageSelectionWidgetConfiguration.setSelectedColor ( selectedColor );
    languageSelectionWidgetConfiguration.setUnSelectedColor ( unSelectedColor );
    languageSelectionWidgetConfiguration.setBoxLabelConfiguration ( languageSelectionLabelConfiguration );
    languageSelectionWidgetConfiguration.setHeaderLabelConfiguration ( languageSelectionLabelConfiguration );

    configSAESGUI.setLanguageSelectionWidgetConfiguration ( languageSelectionWidgetConfiguration );

    SelectionWidgetConfiguration customAttributeSelectionWidgetConfiguration;
    customAttributeSelectionWidgetConfiguration.setAutoSelect ( false );
    customAttributeSelectionWidgetConfiguration.setSortAlphabetical ( false );
    customAttributeSelectionWidgetConfiguration.setEnableClick ( true );
    customAttributeSelectionWidgetConfiguration.setButtonType ( ViewButtonConfiguration::CONFIGURATION_SEPARATE_BUTTONS );

    ExpandButtonConfiguration customAttributeSelectionExpandButtonConfiguration;
    customAttributeSelectionExpandButtonConfiguration.setFontSize ( 14 );
    customAttributeSelectionExpandButtonConfiguration.setSpacing ( 10 );
    customAttributeSelectionWidgetConfiguration.setExpandButton ( customAttributeSelectionExpandButtonConfiguration );

    SelectionWidgetSeparateButtonConfiguration customSelectionSeparateButtonConfiguration;
    customSelectionSeparateButtonConfiguration.setAcceptButtonWidth ( 330 );
    customSelectionSeparateButtonConfiguration.setClearButtonWidth ( 330 );
    customSelectionSeparateButtonConfiguration.setFontSize ( 14 );
    customAttributeSelectionWidgetConfiguration.setSeparateButton ( customSelectionSeparateButtonConfiguration );

    customAttributeSelectionWidgetConfiguration.setFrameTopHeight ( 80 );
    customAttributeSelectionWidgetConfiguration.setFrameBottomHeight ( 80 );
    customAttributeSelectionWidgetConfiguration.setShowBorder ( true );
    customAttributeSelectionWidgetConfiguration.setBoxSpacing ( 150 );
    customAttributeSelectionWidgetConfiguration.setBoxWidth ( 450 );
    customAttributeSelectionWidgetConfiguration.setBoxHeight ( 100 );
    customAttributeSelectionWidgetConfiguration.setSelectedColor ( selectedColor );
    customAttributeSelectionWidgetConfiguration.setUnSelectedColor ( unSelectedColor );
    customAttributeSelectionWidgetConfiguration.setBoxLabelConfiguration ( languageSelectionLabelConfiguration );
    customAttributeSelectionWidgetConfiguration.setHeaderLabelConfiguration ( languageSelectionLabelConfiguration );

    configSAESGUI.setCustomAttributeSelectionWidgetConfiguration ( customAttributeSelectionWidgetConfiguration );

    //themes configuration
    officialThemeConfiguration.setVotingWindowButton ( "VotingWindowButton" );
    officialThemeConfiguration.setVotingExperiencePartyEventbox ( "votingexperiencepartyeventbox" );
    officialThemeConfiguration.setVotingExperienceWindow ( "votingexperiencewindow" );
    officialThemeConfiguration.setVotingExperienceBallotOptionEventbox ( "votingexperienceballotoptioneventbox" );
    officialThemeConfiguration.setCardActivatorFunctionalityMessage ( "CardActivatorFunctionalityMessage" );
    officialThemeConfiguration.setCardActivatorMessageEventbox ( "CardActivatorMessageEventBox" );
    officialThemeConfiguration.setCardActivatorFunctionalityEventbox ( "cardActivatorFunctionalityEventBox" );
    officialThemeConfiguration.setCardActivatorValuesEventbox ( "cardActivatorValuesEventBox" );
    officialThemeConfiguration.setCardActivatorFunctionalityCenterFrame ( "CardActivatorFunctionalityCenterFrame" );
    officialThemeConfiguration.setEventboxStepBarcode ( "eventboxStepBarcode" );
    officialThemeConfiguration.setEventboxStepBarcodeTitle ( "eventboxStepBarcodeTitle" );
    officialThemeConfiguration.setEventboxStepCardTitle ( "eventboxStepCardTitle" );
    officialThemeConfiguration.setVoteDemoFunctionalityMessage ( "VoteDemoFunctionalityMessage" );
    officialThemeConfiguration.setStandInLabelLabel ( "stand_in_label_label" );
    officialThemeConfiguration.setStandInLabelEvent ( "stand_in_label_event" );
    officialThemeConfiguration.setVotingExperienceSelectBallotOptionWidget ( "voting_experience_select_ballot_option_widget" );
    officialThemeConfiguration.setChangeLanguageMessageWigdetEventbox ( "change_language_message_wigdet_event_box" );
    officialThemeConfiguration.setCustomAttributeComboSelectionsWidgetTitle ( "CustomAttributeComboSelectionsWidgetTitle" );
    officialThemeConfiguration.setCustomAttributeCounterWidgetTitle ( "CustomAttributeCounterWidgetTitle" );
    officialThemeConfiguration.setEventboxChooseWidgetFrame ( "event_box_choose_widget_frame" );
    officialThemeConfiguration.setVotingExperiencePartyWidget ( "voting_experience_party_widget" );
    officialThemeConfiguration.setPartyWidgetEventbox ( "party_widget_event_box" );
    officialThemeConfiguration.setEventboxImageStepInstallationWizard ( "EventBoxImageStepInstallationWizard" );
    officialThemeConfiguration.setEventboxStepInstallationWizard ( "EventBoxStepInstallationWizard" );
    officialThemeConfiguration.setParentKeypadEventbox ( "parentkeypadeventbox" );
    officialThemeConfiguration.setInfoFrame ( "infoFrame" );
    demoThemeConfiguration.setVotingWindowButtonDemo ( "VotingWindowButtonDemo" );
    demoThemeConfiguration.setVotingExperiencePartyEventboxDemo ( "votingexperiencepartyeventboxDemo" );
    demoThemeConfiguration.setVotingExperienceWindowDemo ( "votingexperiencewindowDemo" );
    demoThemeConfiguration.setVotingExperienceBallotOptionEventboxDemo ( "votingexperienceballotoptioneventboxDemo" );
    themesConfiguration.setOfficialThemeConfiguration ( officialThemeConfiguration );
    themesConfiguration.setDemoThemeConfiguration ( demoThemeConfiguration );
    configSAESGUI.setThemesConfiguration ( themesConfiguration );
    configSAESGUI.setPollingCardConfiguration ( pollingCardConfiguration );
    configSAESGUI.setAlarmBoxMonitorConfiguration ( alarmBoxMonitorConfiguration );
    StartConfiguration startConfig;
    startConfig.setLoadSelectionLanguage ( false );
    startConfig.setPersistLanguage ( false );
    startConfig.setInfoMachine ( InfoMachineConfiguration::CONFIGURATION_PM );


    configSAESGUI.setForceExecuteFunctionality ( false );
    configSAESGUI.setStartConfiguration ( startConfig );
    Smartmatic::ConfigurationTypes::NamespaceInfomap map;
#ifdef NAMESPACE_SAES_GUI_CONFIGURATION
    map[""].name = NAMESPACE_SAES_GUI_CONFIGURATION;
#endif
    map[""].schema = "";
    serializeSaesGUIConfiguration ( ss, configSAESGUI, map, "UTF-8" );
    ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "SAESGUI", ss.str() );
}

FunctionalityWizardFlowConfiguration SAESGUIConfigurationManager::createFunctionalityWizardConfig()
{
    FunctionalityWizardFlowConfiguration retVal;

    FunctionalityWizardFlowConfiguration::WizardStatesConfigurationSequence wizardSteps;
    FunctionalityWizardStateConfiguration noInstalled;
    FunctionalityWizardStateConfiguration noDemoVote;
    FunctionalityWizardStateConfiguration noInitiated;
    FunctionalityWizardStateConfiguration initiated;
    FunctionalityWizardStateConfiguration votingClosed;
    FunctionalityWizardStateConfiguration electionClosed;
    WizardStateHelpConfiguration helpConfig;
    WizardStateTimeLineConfiguration timelineConfig;
    WizardStateToolboxConfiguration toolboxConfig;
    FunctionalityWizardStateConfiguration::PrincipalMenuOptionsSequence notInstalledOptions;
    FunctionalityWizardStateConfiguration::ToolbarMenuOptionsSequence toolBars;
    FunctionalityWizardStateConfiguration::PrincipalMenuOptionsSequence noDemoVoteOptions;
    FunctionalityWizardStateConfiguration::PrincipalMenuOptionsSequence notInitiatedOptions;
    FunctionalityWizardStateConfiguration::PrincipalMenuOptionsSequence initiatedOptions;
    FunctionalityWizardStateConfiguration::PrincipalMenuOptionsSequence votingClosedOptions;
    FunctionalityWizardStateConfiguration::PrincipalMenuOptionsSequence electionClosedOptions;
    MenuOptions emptyOptions;
    XMLMenuOption showMenu;

    showMenu.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.MenuToolbar" ) );
    showMenu.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.MenuToolbar" ) );
    showMenu.setAccessibleName ( "ToolbarMenu" );

    showMenu.setImageKey ( "ui/MenuInToolbar.gif" );
    showMenu.setCode ( 11 );
    showMenu.setMappingKey ( 3 );
    showMenu.setMappingKeyName ( "Menu" );
    showMenu.setOptionType ( MenuOptionType::CONFIGURATION_XMLMENU );
    showMenu.setMenuOptions ( emptyOptions );
    //option openElection
    XMLMenuOption openElection;
    openElection.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.OpenElection" ) );
    openElection.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.OpenElection" ) );
    openElection.setAccessibleName ( "OpenElection" );

    openElection.setImageKey ( "" );
    openElection.setCode ( 11 );
    openElection.setMappingKey ( 3 );
    openElection.setMappingKeyName ( "HOLA" );
    openElection.setOptionType ( MenuOptionType::CONFIGURATION_XMLMOPENELECTIONFUNCTIONALITY );
    openElection.setMenuOptions ( emptyOptions );
    //option closeElection
    XMLMenuOption closeElection;
    closeElection.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.CloseElection" ) );
    closeElection.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.CloseElection" ) );
    closeElection.setAccessibleName ( "CloseElection" );

    closeElection.setImageKey ( "" );
    closeElection.setCode ( 12 );
    closeElection.setMappingKey ( 4 );
    closeElection.setMappingKeyName ( "@" );
    closeElection.setOptionType ( MenuOptionType::CONFIGURATION_XMLMCLOSEELECTIONFUNCTIONALITY );
    closeElection.setMenuOptions ( emptyOptions );
    //option voting machine installation
    XMLMenuOption vmInstallation;
    vmInstallation.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotingMachineInstallation" ) );
    vmInstallation.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotingMachineInstallation" ) );
    vmInstallation.setAccessibleName ( "VotingMachineInstallation" );

    vmInstallation.setImageKey ( "ui/InstallMachineToolbar.gif" );
    vmInstallation.setCode ( 13 );
    vmInstallation.setMappingKey ( 5 );
    vmInstallation.setMappingKeyName ( "F" );
    vmInstallation.setOptionType ( MenuOptionType::CONFIGURATION_XMLMVOTINGMACHINEINSTALLATIONFUNCTIONALITY );
    vmInstallation.setMenuOptions ( emptyOptions );
    //option skip voting machine installation
    XMLMenuOption skipInstallation;
    skipInstallation.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.SkipVotingMachineInstallation" ) );
    skipInstallation.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.SkipVotingMachineInstallation" ) );
    skipInstallation.setAccessibleName ( "SkipVotingMachineInstallation" );

    skipInstallation.setImageKey ( "" );
    skipInstallation.setCode ( 13 );
    skipInstallation.setMappingKey ( 5 );
    skipInstallation.setMappingKeyName ( "S" );
    skipInstallation.setOptionType ( MenuOptionType::CONFIGURATION_XMLMSKIPVOTINGMACHINEINSTALLATIONFUNCTIONALITY );
    skipInstallation.setMenuOptions ( MenuOptions () );

    //option voting process demo
    XMLMenuOption voteProcessDemo;
    voteProcessDemo.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VoteProcessDemo" ) );
    voteProcessDemo.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VoteProcessDemo" ) );
    voteProcessDemo.setAccessibleName ( "VoteProcessDemo" );

    voteProcessDemo.setImageKey ( "ui/VotingExperienceDemoToolbar.gif" );
    voteProcessDemo.setCode ( 141 );
    voteProcessDemo.setMappingKey ( 3 );
    voteProcessDemo.setMappingKeyName ( "H" );
    voteProcessDemo.setOptionType ( MenuOptionType::CONFIGURATION_XMLMVOTEPROCESSDEMOFUNCTIONALITY );
    voteProcessDemo.setMenuOptions ( emptyOptions );
    // Vote Print Preview
    XMLMenuOption printPreview;
    printPreview.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotePrintPreview" ) );
    printPreview.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.VotePrintPreview" ) );
    printPreview.setAccessibleName ( "VotePrintPreview" );

    printPreview.setImageKey ( "ui/VotePrintPreviewToolbar.jpg" );
    printPreview.setCode ( 143 );
    printPreview.setMappingKey ( 5 );
    printPreview.setMappingKeyName ( "I" );
    printPreview.setOptionType ( MenuOptionType::CONFIGURATION_XMLMPRINTPREVIEWFUNCTIONALITY );
    printPreview.setMenuOptions ( emptyOptions );

    XMLMenuOption logReport;
    logReport.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.LogReport" ) );
    logReport.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.LogReport" ) );
    logReport.setAccessibleName ( "LogReport" );

    logReport.setImageKey ( "ui/ViewLogToolbar.gif" );
    logReport.setCode ( 285 );
    logReport.setMappingKey ( 99 );
    logReport.setMappingKeyName ( "<" );
    logReport.setOptionType ( MenuOptionType::CONFIGURATION_XMLMLOGREPORTFUNCTIONALITY );
    logReport.setMenuOptions ( emptyOptions );

    // Skip Vote Demo Register
    XMLMenuOption skipVoteDemo;
    skipVoteDemo.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.SkipVoteDemo" ) );
    skipVoteDemo.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.SkipVoteDemo" ) );
    skipVoteDemo.setAccessibleName ( "SkipVoteDemo" );

    skipVoteDemo.setImageKey ( "ui/RegPollworkerToolbar.gif" );
    skipVoteDemo.setCode ( 281 );
    skipVoteDemo.setMappingKey ( 9 );
    skipVoteDemo.setMappingKeyName ( "Z" );
    skipVoteDemo.setOptionType ( MenuOptionType::CONFIGURATION_XMLMSKIPVOTEPROCESSDEMOFUNCTIONALITY );
    skipVoteDemo.setMenuOptions ( emptyOptions );
    XMLMenuOption closeVotation;
    closeVotation.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.CloseVotation" ) );
    closeVotation.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.CloseVotation" ) );
    closeVotation.setAccessibleName ( "CloseVotation" );

    closeVotation.setImageKey ( "" );
    closeVotation.setCode ( 282 );
    closeVotation.setMappingKey ( 3 );
    closeVotation.setMappingKeyName ( "+" );
    closeVotation.setOptionType ( MenuOptionType::CONFIGURATION_XMLMCLOSEVOTATIONFUNCTIONALITY );
    closeVotation.setMenuOptions ( emptyOptions );

    XMLMenuOption reportPrinting;
    reportPrinting.setNameKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ReportPrinting" ) );
    reportPrinting.setUntraslatedKey ( N_ ( "Smartmatic.SAES.GUI.Configuration.SAESGUIConfigurationManager.ReportPrinting" ) );
    reportPrinting.setAccessibleName ( "ReportPrinting" );
    reportPrinting.setImageKey ( "ui/RePrintPVToolbar.gif" );
    reportPrinting.setCode ( 219 );
    reportPrinting.setMappingKey ( 3 );
    reportPrinting.setMappingKeyName ( "S" );
    reportPrinting.setOptionType ( MenuOptionType::CONFIGURATION_XMLMEXTERNALPRINTFUNCTIONALITY );
    reportPrinting.setMenuOptions ( emptyOptions );

    //Set the tool bar, the same for all states for now
    //set toolbar options
    toolBars.push_back ( showMenu );
    toolBars.push_back ( vmInstallation );
    toolBars.push_back ( voteProcessDemo );
    toolBars.push_back ( printPreview );
    toolBars.push_back ( logReport );
    toolBars.push_back ( reportPrinting );
    XMLLabelConfiguration ButtonsLabelConfiguration;
    ButtonsLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    ButtonsLabelConfiguration.setLabel_VerticalPadding ( 0 );
    ButtonsLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    ButtonsLabelConfiguration.setLabel_Family ( "Verdana" );
    ButtonsLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    ButtonsLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    ButtonsLabelConfiguration.setLabel_Size ( 16 );
    XMLLabelConfiguration TitleHelpLabelConfiguration;
    TitleHelpLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    TitleHelpLabelConfiguration.setLabel_VerticalPadding ( 0 );
    TitleHelpLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    TitleHelpLabelConfiguration.setLabel_Family ( "Verdana" );
    TitleHelpLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    TitleHelpLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    TitleHelpLabelConfiguration.setLabel_Size ( 20 );
    XMLLabelConfiguration HelpLabelConfiguration;
    HelpLabelConfiguration.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    HelpLabelConfiguration.setLabel_VerticalPadding ( 0 );
    HelpLabelConfiguration.setLabel_HorizontalPadding ( 0 );
    HelpLabelConfiguration.setLabel_Family ( "Verdana" );
    HelpLabelConfiguration.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    HelpLabelConfiguration.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    HelpLabelConfiguration.setLabel_Size ( 14 );
    /********** Configure Not installed********/
    //Set help for no installed
    helpConfig.setTitleKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.NotInstalledStateTitleKey" ) );
    helpConfig.setContentKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.NotInstalledStateContentKey" ) );
    helpConfig.setRelativeImagePath ( "ui/NotInstalledHelpImage.gif" );
    //set time line config

    timelineConfig.setNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.NotInstalledStateNameKey" ) );
    timelineConfig.setDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.NotInstalledStateDescKey" ) );
    timelineConfig.setAccessibleName ( "NotInstalledState" );

    TimeLineImageConfiguration imageConfiguration;

    TimeLineImageConfiguration::ImageSequence imagesNoInstalled;
    imagesNoInstalled.push_back ( TimeLineStateImageConfiguration ( "ui/NotInstalledTimeline.png" , Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINSTALLED, "NotInstalledTimeline" ) );
    imageConfiguration.setImage ( imagesNoInstalled );
    timelineConfig.setImageConfiguration ( imageConfiguration );
    toolboxConfig.setToolboxElementHeight ( 70 );
    toolboxConfig.setToolboxElementWidth ( 70 );
    //set help configuration
    HelpConfiguration helpNoInstalled;
    helpNoInstalled.setImageHelp ( "ui/wizard-not-installed-help.gif" );
    helpNoInstalled.setPdfHelp ( "ui/wizard-not-installed-help.pdf" );
    helpNoInstalled.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.FunctionalityWizard.HelpConfiguration.helpNoInstalled" ) );
    helpNoInstalled.setUseHelp ( true );
    helpNoInstalled.setEnableFilePrinting ( true );
    //helpNoInstalle.
    //set principal options
    notInstalledOptions.push_back ( vmInstallation );
    notInstalledOptions.push_back ( skipInstallation );
    //Assign options
    noInstalled.setPrincipalMenuOptions ( notInstalledOptions );
    noInstalled.setToolbarMenuOptions ( toolBars );
    //set properties
    noInstalled.setProcessStatus ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINSTALLED );
    noInstalled.setStateNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.NotInstalledStateNameKey" ) );
    noInstalled.setStateDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.NotInstalledStateDeskKey" ) );
    noInstalled.setAccessibleName ( "NotInstalledStepWizard" );

    helpConfig.setHelp ( helpNoInstalled );
    noInstalled.setStateHelp ( helpConfig );
    noInstalled.setTimeLineConfiguration ( timelineConfig );
    noInstalled.setToolboxConfiguration ( toolboxConfig );
    noInstalled.setButtonsLabelConfiguration ( ButtonsLabelConfiguration );
    noInstalled.setButtonWidth ( 500 );
    noInstalled.setButtonHeight ( 100 );

    noInstalled.setTitleHelpLabelConfiguration ( TitleHelpLabelConfiguration );
    noInstalled.setHelpLabelConfiguration ( HelpLabelConfiguration );

    /********** Configure No demo vote********/
    //Set help for no demo
    helpConfig.setTitleKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.NoDemoVoteStateTitleKey" ) );
    helpConfig.setContentKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.NoDemoVoteStateContentKey" ) );
    helpConfig.setRelativeImagePath ( "ui/NotDemonstrationVote.gif" );
    //set time line config
    timelineConfig.setNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.NoDemoVoteStateNameKey" ) );
    timelineConfig.setDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.NoDemoVoteStateDescKey" ) );
    timelineConfig.setAccessibleName ( "NoDemoVoteState" );

    TimeLineImageConfiguration::ImageSequence imagesNotDemoVote;
    imagesNotDemoVote.push_back ( TimeLineStateImageConfiguration ( "ui/NoDemoVoteTimeline.png" , Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTDEMOSTRATIONVOTE, "NoDemoVoteTimeline" ) );
    imageConfiguration.setImage ( imagesNotDemoVote );
    timelineConfig.setImageConfiguration ( imageConfiguration );

    //set principal options
    noDemoVoteOptions.push_back ( voteProcessDemo );
    noDemoVoteOptions.push_back ( skipVoteDemo );
    HelpConfiguration helpNoDemoVote;
    helpNoDemoVote.setImageHelp ( "ui/wizard-not-demonstration-vote-help.gif" );
    helpNoDemoVote.setPdfHelp ( "ui/wizard-not-demonstration-vote-help.pdf" );
    helpNoDemoVote.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.FunctionalityWizard.HelpConfiguration.NoDemoVote" ) );
    helpNoDemoVote.setUseHelp ( true );
    helpNoDemoVote.setEnableFilePrinting ( true );

    //Assign options
    noDemoVote.setPrincipalMenuOptions ( noDemoVoteOptions );
    noDemoVote.setToolbarMenuOptions ( toolBars );
    //set properties
    noDemoVote.setProcessStatus ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTDEMOSTRATIONVOTE );
    noDemoVote.setStateNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.NoDemoVoteStateNameKey" ) );
    noDemoVote.setStateDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.NoDemoVoteDeskKey" ) );
    noDemoVote.setAccessibleName ( "NotDemostrationVoteStepWizard" );

    helpConfig.setHelp ( helpNoDemoVote );
    noDemoVote.setStateHelp ( helpConfig );
    noDemoVote.setTimeLineConfiguration ( timelineConfig );
    noDemoVote.setToolboxConfiguration ( toolboxConfig );
    noDemoVote.setButtonsLabelConfiguration ( ButtonsLabelConfiguration );
    noDemoVote.setButtonWidth ( 500 );
    noDemoVote.setButtonHeight ( 100 );

    noDemoVote.setTitleHelpLabelConfiguration ( TitleHelpLabelConfiguration );
    noDemoVote.setHelpLabelConfiguration ( HelpLabelConfiguration );
    /********** Configure No initiated********/
    //Set help for no demo
    helpConfig.setTitleKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.NoInitiatedStateTitleKey" ) );
    helpConfig.setContentKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.NoInitiatedStateContentKey" ) );
    helpConfig.setRelativeImagePath ( "ui/NotInitiatedHelpImage.gif" );
    //set time line config
    timelineConfig.setNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.NoInitiatedStateNameKey" ) );
    timelineConfig.setDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.NoInitiatedStateDescKey" ) );
    timelineConfig.setAccessibleName ( "NoInitiatedState" );

    TimeLineImageConfiguration::ImageSequence imagesNotInitiated;
    imagesNotInitiated.push_back ( TimeLineStateImageConfiguration ( "ui/NoInitiatedTimeline.png" , Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINITIATED, "NoInitiatedTimeline" ) );
    imageConfiguration.setImage ( imagesNotInitiated );
    timelineConfig.setImageConfiguration ( imageConfiguration );

    //set principal options
    notInitiatedOptions.push_back ( openElection );
    HelpConfiguration helpNoInitiated;
    helpNoInitiated.setImageHelp ( "ui/wizard-not-initiated-help.gif" );
    helpNoInitiated.setPdfHelp ( "ui/wizard-not-initiated-help.pdf" );
    helpNoInitiated.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.FunctionalityWizard.HelpConfiguration.NoInitiated" ) );
    helpNoInitiated.setUseHelp ( true );
    helpNoInitiated.setEnableFilePrinting ( true );

    //Assign options
    noInitiated.setPrincipalMenuOptions ( notInitiatedOptions );
    noInitiated.setToolbarMenuOptions ( toolBars );
    //set properties
    noInitiated.setProcessStatus ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINITIATED );
    noInitiated.setStateNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.NoInitiatedStateNameKey" ) );
    noInitiated.setStateDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.NoInitiatedDeskKey" ) );
    noInitiated.setAccessibleName ( "NotInitiatedStepWizard" );
    helpConfig.setHelp ( helpNoInitiated );
    noInitiated.setStateHelp ( helpConfig );
    noInitiated.setTimeLineConfiguration ( timelineConfig );
    noInitiated.setToolboxConfiguration ( toolboxConfig );
    noInitiated.setButtonsLabelConfiguration ( ButtonsLabelConfiguration );
    noInitiated.setButtonWidth ( 500 );
    noInitiated.setButtonHeight ( 100 );

    noInitiated.setTitleHelpLabelConfiguration ( TitleHelpLabelConfiguration );
    noInitiated.setHelpLabelConfiguration ( HelpLabelConfiguration );
    /********** Configure initiated********/
    //Set help for no demo
    helpConfig.setTitleKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.InitiatedStateTitleKey" ) );
    helpConfig.setContentKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.InitiatedStateContentKey" ) );
    helpConfig.setRelativeImagePath ( "ui/InitiatedHelpImage.gif" );
    //set time line config
    timelineConfig.setNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.InitiatedStateNameKey" ) );
    timelineConfig.setDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.InitiatedStateDescKey" ) );
    timelineConfig.setAccessibleName ( "InitiatedState" );

    TimeLineImageConfiguration::ImageSequence imagesInitiated;
    imagesInitiated.push_back ( TimeLineStateImageConfiguration ( "ui/InitiatedTimeline.png" , Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_INITIATED, "InitiatedTimeline" ) );
    imageConfiguration.setImage ( imagesInitiated );
    timelineConfig.setImageConfiguration ( imageConfiguration );

    //set principal options
    initiatedOptions.push_back ( closeVotation );
    HelpConfiguration helpInitiated;
    helpInitiated.setImageHelp ( "ui/wizard-initiated-help.gif" );
    helpInitiated.setPdfHelp ( "ui/wizard-initiated-help.pdf" );
    helpInitiated.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.FunctionalityWizard.HelpConfiguration.Initiated" ) );
    helpInitiated.setUseHelp ( true );
    helpInitiated.setEnableFilePrinting ( true );

    //Assign options
    initiated.setPrincipalMenuOptions ( initiatedOptions );
    initiated.setToolbarMenuOptions ( toolBars );
    //set properties
    initiated.setProcessStatus ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_INITIATED );
    initiated.setStateNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.InitiatedStateNameKey" ) );
    initiated.setStateDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.InitiatedDeskKey" ) );
    initiated.setAccessibleName ( "InitiatedStepWizard" );
    helpConfig.setHelp ( helpInitiated );
    initiated.setStateHelp ( helpConfig );
    initiated.setTimeLineConfiguration ( timelineConfig );
    initiated.setToolboxConfiguration ( toolboxConfig );
    initiated.setButtonsLabelConfiguration ( ButtonsLabelConfiguration );
    initiated.setButtonWidth ( 500 );
    initiated.setButtonHeight ( 100 );

    initiated.setTitleHelpLabelConfiguration ( TitleHelpLabelConfiguration );
    initiated.setHelpLabelConfiguration ( HelpLabelConfiguration );
    /********** Configure VotingClosed********/
    //Set help for no demo
    helpConfig.setTitleKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.VotingClosedStateTitleKey" ) );
    helpConfig.setContentKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.VotingClosedStateContentKey" ) );
    helpConfig.setRelativeImagePath ( "ui/VotingClosedHelpImage.gif" );
    //set time line config
    timelineConfig.setNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.VotingClosedStateNameKey" ) );
    timelineConfig.setDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.VotingClosedStateDescKey" ) );
    timelineConfig.setAccessibleName ( "VotingClosedState" );

    TimeLineImageConfiguration::ImageSequence imagesVotingClosed;
    imagesVotingClosed.push_back ( TimeLineStateImageConfiguration ( "ui/VotingClosedTimeline.png" , Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_CLOSED, "VotingClosedTimeline" ) );
    imageConfiguration.setImage ( imagesVotingClosed );
    timelineConfig.setImageConfiguration ( imageConfiguration );

    //set principal options
    votingClosedOptions.push_back ( closeElection );
    HelpConfiguration helpVotingClosed;
    helpVotingClosed.setImageHelp ( "ui/wizard-voting-closed-help.gif" );
    helpVotingClosed.setPdfHelp ( "ui/wizard-voting-closed-help.pdf" );
    helpVotingClosed.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.FunctionalityWizard.HelpConfiguration.VotingClosed" ) );
    helpVotingClosed.setUseHelp ( true );
    helpVotingClosed.setEnableFilePrinting ( true );

    //Assign options
    votingClosed.setPrincipalMenuOptions ( votingClosedOptions );
    votingClosed.setToolbarMenuOptions ( toolBars );
    //set properties
    votingClosed.setProcessStatus ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_CLOSED );
    votingClosed.setStateNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.VotingClosedStateNameKey" ) );
    votingClosed.setStateDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.VotingClosedDeskKey" ) );
    votingClosed.setAccessibleName ( "ClosedStepWizard" );
    helpConfig.setHelp ( helpVotingClosed );
    votingClosed.setStateHelp ( helpConfig );
    votingClosed.setTimeLineConfiguration ( timelineConfig );
    votingClosed.setToolboxConfiguration ( toolboxConfig );
    votingClosed.setButtonsLabelConfiguration ( ButtonsLabelConfiguration );
    votingClosed.setButtonWidth ( 500 );
    votingClosed.setButtonHeight ( 100 );

    votingClosed.setTitleHelpLabelConfiguration ( TitleHelpLabelConfiguration );
    votingClosed.setHelpLabelConfiguration ( HelpLabelConfiguration );
    /********** Configure Election Closed********/
    //Set help for no demo
    helpConfig.setTitleKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.ElectionClosedStateTitleKey" ) );
    helpConfig.setContentKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Help.ElectionClosedStateContentKey" ) );
    helpConfig.setRelativeImagePath ( "ui/ElectionClosedHelpImage.gif" );
    //set time line config
    timelineConfig.setNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.ElectionClosedStateNameKey" ) );
    timelineConfig.setDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.Timeline.ElectionClosedStateDescKey" ) );
    timelineConfig.setAccessibleName ( "ElectionClosedState" );

    TimeLineImageConfiguration::ImageSequence imagesElectionClosed;
    imagesElectionClosed.push_back ( TimeLineStateImageConfiguration ( "ui/ElectionClosedTimeline.png" , Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_RESULTSTRANSMITTED, "ElectionClosedTimeline" ) );
    imageConfiguration.setImage ( imagesElectionClosed );
    timelineConfig.setImageConfiguration ( imageConfiguration );

    //set principal options
    electionClosedOptions.push_back ( reportPrinting );
    HelpConfiguration helpElectionClosed;
    helpElectionClosed.setImageHelp ( "ui/wizard-election-closed-help.gif" );
    helpElectionClosed.setPdfHelp ( "ui/wizard-election-closed-help.pdf" );
    helpElectionClosed.setTitleKey ( N_ ( "Smartmatic.SAES.GUI.FunctionalityWizard.HelpConfiguration.ElectionClosed" ) );
    helpElectionClosed.setUseHelp ( true );
    helpElectionClosed.setEnableFilePrinting ( true );

    //Assign options
    electionClosed.setPrincipalMenuOptions ( electionClosedOptions );
    electionClosed.setToolbarMenuOptions ( toolBars );
    //set properties
    electionClosed.setProcessStatus ( Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_RESULTSTRANSMITTED );
    electionClosed.setStateNameKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.ElectionClosedStateNameKey" ) );
    electionClosed.setStateDescriptionKey ( N_ ( "Smartmatic.SAES.Configuration.FunctionalityWizardStateConfiguration.ElectionClosedDeskKey" ) );
    electionClosed.setAccessibleName ( "ResultsTransmittedStepWizard" );
    helpConfig.setHelp ( helpElectionClosed );
    electionClosed.setStateHelp ( helpConfig );
    electionClosed.setTimeLineConfiguration ( timelineConfig );
    electionClosed.setToolboxConfiguration ( toolboxConfig );
    electionClosed.setButtonsLabelConfiguration ( ButtonsLabelConfiguration );
    electionClosed.setButtonWidth ( 500 );
    electionClosed.setButtonHeight ( 100 );

    electionClosed.setTitleHelpLabelConfiguration ( TitleHelpLabelConfiguration );
    electionClosed.setHelpLabelConfiguration ( HelpLabelConfiguration );
    //register the states
    wizardSteps.push_back ( noInstalled );
    wizardSteps.push_back ( noDemoVote );
    wizardSteps.push_back ( noInitiated );
    wizardSteps.push_back ( initiated );
    wizardSteps.push_back ( votingClosed );
    wizardSteps.push_back ( electionClosed );

    retVal.setWizardStatesConfiguration ( wizardSteps );

    return retVal;
}

FormsCommonControlsConfigurations SAESGUIConfigurationManager::createCommonControlsConfigurations()
{
    FormsCommonControlsConfigurations formCommonCotrols;

    FormsTitleLabelConfigurations formTitles;
    FormsMessagesLabelConfigurations formMessages;

    XMLLabelConfiguration emptyWindowTitle;
    XMLLabelConfiguration VMInstallMessage;
    XMLLabelConfiguration UnableFunctionalityMessage;

    emptyWindowTitle.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    emptyWindowTitle.setLabel_VerticalPadding ( 1 );
    emptyWindowTitle.setLabel_HorizontalPadding ( 1 );
    emptyWindowTitle.setLabel_Family ( "Verdana" );
    emptyWindowTitle.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    emptyWindowTitle.setLabel_Weight ( Label_Weight::CONFIGURATION_BOLD );
    emptyWindowTitle.setLabel_Size ( 20 );

    VMInstallMessage.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    VMInstallMessage.setLabel_VerticalPadding ( 1 );
    VMInstallMessage.setLabel_HorizontalPadding ( 1 );
    VMInstallMessage.setLabel_Family ( "Verdana" );
    VMInstallMessage.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    VMInstallMessage.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    VMInstallMessage.setLabel_Size ( 18 );

    UnableFunctionalityMessage.setLabel_Alignment ( Label_Alignment::CONFIGURATION_CENTER );
    UnableFunctionalityMessage.setLabel_VerticalPadding ( 1 );
    UnableFunctionalityMessage.setLabel_HorizontalPadding ( 1 );
    UnableFunctionalityMessage.setLabel_Family ( "Verdana" );
    UnableFunctionalityMessage.setLabel_Style ( Label_Style::CONFIGURATION_NORMAL );
    UnableFunctionalityMessage.setLabel_Weight ( Label_Weight::CONFIGURATION_NORMAL );
    UnableFunctionalityMessage.setLabel_Size ( 30 );

    formTitles.setEmptyWindowTitleLabel ( emptyWindowTitle );
    formMessages.setVMLogingMessageLabel ( VMInstallMessage );

    formCommonCotrols.setFormTitles ( formTitles );
    formCommonCotrols.setFormMesages ( formMessages );
    formCommonCotrols.setUnableFunctionalityMessage ( UnableFunctionalityMessage );
    formCommonCotrols.setUnableMessageCloseTime ( 3000 );
    formCommonCotrols.setExitFunctionalityMessageCloseTime ( 4000 );

    return formCommonCotrols;
}

ReportPrintingWidgetConfiguration SAESGUIConfigurationManager::GetPrintingWidgetConfiguration ()
{
    return getConfiguration()->getReportPrintingWidget();
}

bool SAESGUIConfigurationManager::GetForceToLoadFrame ()
{
    return getConfiguration()->getForceToLoadFrame();
}

bool SAESGUIConfigurationManager::GetPMInstallationWidgetShowPrintPasswordButton()
{
    return getConfiguration()->getPMInstallationWidgetShowPrintPasswordButton();
}

CustomAtributtesWindowConfiguration & SAESGUIConfigurationManager::GetCustomAtributtesWindowConfiguration()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getCustomAtributtesWindow();
}

VotingExperienceLabelConfiguration & SAESGUIConfigurationManager::GetVotingExperienceLabelConfiguration()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getVotingExperienceLabel();
}

VMInstallationWidgetConfiguration SAESGUIConfigurationManager::GetVMInstallationConfiguration()
{
    return getConfiguration()->getVMInstallationConfiguration();
}

BallotOptionWidgetConfiguration & SAESGUIConfigurationManager::GetBallotOptionWidgetConfiguration()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getBallotOptionSelection().getBallotOptionWidget();
}

BallotOptionSelectionConfiguration & SAESGUIConfigurationManager::GetBallotOptionSelectionConfiguration()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getBallotOptionSelection();
}

ReferendumConfiguration & SAESGUIConfigurationManager::GetReferendumConfiguration()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getReferendumSelection();
}

PartyWidgetConfiguration & SAESGUIConfigurationManager::GetPartyWidgetConfiguration()
{
    return GetPartySelectionConfiguration().getPartyWidget();
}

VotingWindowConfiguration & SAESGUIConfigurationManager::GetVotingWindowConfiguration()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getVotingWindow();
}

unsigned int SAESGUIConfigurationManager::getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType type )
{
    VotingExperienceConfiguration::WidgetStepSequence & seq ( getVotingExperienceGUIConfiguration().getVotingExperience().getWidgetStep() );
    VotingExperienceConfiguration::WidgetStepIterator it;

    unsigned int time = 3000;

    for ( it = seq.begin();
            it != seq.end();
            ++it )
    {
        if ( ( *it ).getType() == type )
        {
            time = ( *it ).getWaitTime();
            break;
        }
    }

    return time;
}

bool SAESGUIConfigurationManager::getMarkCardAsUsed()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getMarkCardAsUsed();
}

int SAESGUIConfigurationManager::getCheckSumFailTimerMessage()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getCheckSumFailTimerMessage();
}

CheckSumFailOption::Value SAESGUIConfigurationManager::getCheckSumFailOption()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getCheckSumFailOption();
}

bool SAESGUIConfigurationManager::getDetectCardExtraction()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getDetectCardExtraction();
}

unsigned int SAESGUIConfigurationManager::getNumberOfExtractionsBeforeStop()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getNumberOfExtractionsBeforeStop();
}

bool SAESGUIConfigurationManager::getAssistedActivationBySelection()
{
    return getConfiguration()->getCardActivatorFunctionalityConfiguration().getAssistedActivationBySelection();
}

PrintPreviewFunctionalityConfiguration SAESGUIConfigurationManager::getPrintPreviewFunctionalityConfiguration()
{
    return getConfiguration()->getPrintPreviewFunctionalityConfiguration();
}

PartySelectionConfiguration & SAESGUIConfigurationManager::GetPartySelectionConfiguration()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience().getPartySelection();
}

unsigned int SAESGUIConfigurationManager::getMinimumDemoCounterToAllowActivation()
{
    return getConfiguration()->getCardActivatorFunctionalityConfiguration().getMinimumDemoCounterToAllowActivation();
}

bool SAESGUIConfigurationManager::getShowConfirmationButtons()
{
    return getConfiguration()->getCardActivatorFunctionalityConfiguration().getShowConfirmationButtons();
}

OpenElectionFunctionalityConfiguration & SAESGUIConfigurationManager::GetOpenElectionFunctionalityConfiguration()
{
    return getConfiguration()->getOpenElectionFunctionalityConfiguration();
}

CloseElectionFunctionalityConfiguration & SAESGUIConfigurationManager::GetCloseElectionFunctionalityConfiguration()
{
    return getConfiguration()->getCloseElectionFunctionalityConfiguration();
}

ActivationConsolidationFunctionalityConfiguration SAESGUIConfigurationManager::GetActivationConsolidationFunctionalityConfiguration()
{
    return getConfiguration()->getActivationConsolidationFunctionalityConfiguration();
}

ScanVoteProcessConfiguration & SAESGUIConfigurationManager::getScanVoteProcessConfiguration()
{
    return getConfiguration()->getScanVoteProcessConfiguration();
}

USBMonitorWindowConfiguration & SAESGUIConfigurationManager::getUSBMonitorWindowConfiguration()
{
    return getConfiguration()->getUSBMonitorWindowConfiguration();
}

VoteConsolidationFunctionalityConfiguration SAESGUIConfigurationManager::GetVoteConsolidationFunctionalityConfiguration()
{
    return getConfiguration()->getVoteConsolidationFunctionalityConfiguration();
}

VotingExperienceValidationScreensFunctionalityConfiguration SAESGUIConfigurationManager::GetVotingExperienceValidationScreensFunctionalityConfiguration()
{
    return getConfiguration()->getVotingExperienceValidationScreensFunctionalityConfiguration();
}

LogReportFunctionalityConfiguration SAESGUIConfigurationManager::GetLogReportFunctionalityConfiguration()
{
    return getConfiguration()->getLogReportFunctionalityConfiguration();
}

//Themes Methods

ThemesConfiguration & SAESGUIConfigurationManager::GetThemesConfiguration()
{
    return getVotingExperienceGUIConfiguration().getThemesConfiguration();
}

OfficialThemeConfiguration & SAESGUIConfigurationManager::GetOfficialThemeConfiguration()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration();
}

DemoThemeConfiguration & SAESGUIConfigurationManager::GetDemoThemeConfiguration()
{
    return GetThemesConfiguration().getDemoThemeConfiguration();
}

std::string SAESGUIConfigurationManager::GetVotingWindowButtonName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getVotingWindowButton();
}

std::string SAESGUIConfigurationManager::GetVotingExperiencePartyEventboxName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getVotingExperiencePartyEventbox();
}

std::string SAESGUIConfigurationManager::GetVotingExperienceWindowName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getVotingExperienceWindow();
}

std::string SAESGUIConfigurationManager::GetVotingExperienceBallotOptionEventboxName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getVotingExperienceBallotOptionEventbox();
}

std::string SAESGUIConfigurationManager::GetCardActivatorFunctionalityMessageName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getCardActivatorFunctionalityMessage();
}

std::string SAESGUIConfigurationManager::GetCardActivatorMessageEventboxName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getCardActivatorMessageEventbox();
}

std::string SAESGUIConfigurationManager::GetCardActivatorFunctionalityEventboxName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getCardActivatorFunctionalityEventbox();
}

std::string SAESGUIConfigurationManager::GetCardActivatorValuesEventboxName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getCardActivatorValuesEventbox();
}

std::string SAESGUIConfigurationManager::GetCardActivatorFunctionalityCenterFrameName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getCardActivatorFunctionalityCenterFrame();
}

std::string SAESGUIConfigurationManager::GetEventboxStepBarcodeName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getEventboxStepBarcode();
}

std::string SAESGUIConfigurationManager::GetEventboxStepBarcodeTitleName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getEventboxStepBarcodeTitle();
}

std::string SAESGUIConfigurationManager::GetEventboxStepCardTitleName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getEventboxStepCardTitle();
}

std::string SAESGUIConfigurationManager::GetVoteDemoFunctionalityMessageName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getVoteDemoFunctionalityMessage();
}

std::string SAESGUIConfigurationManager::GetStandInLabelLabelName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getStandInLabelLabel();
}

std::string SAESGUIConfigurationManager::GetStandInLabelEventName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getStandInLabelEvent();
}

std::string SAESGUIConfigurationManager::GetVotingExperienceSelectBallotOptionWidgetName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getVotingExperienceSelectBallotOptionWidget();
}

std::string SAESGUIConfigurationManager::GetChangeLanguageMessageWigdetEventboxName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getChangeLanguageMessageWigdetEventbox();
}

std::string SAESGUIConfigurationManager::GetCustomAttributeComboSelectionsWidgetTitleName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getCustomAttributeComboSelectionsWidgetTitle();
}

std::string SAESGUIConfigurationManager::GetCustomAttributeCounterWidgetTitleName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getCustomAttributeCounterWidgetTitle();
}

std::string SAESGUIConfigurationManager::GetEventboxChooseWidgetFrameName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getEventboxChooseWidgetFrame();
}

std::string SAESGUIConfigurationManager::GetVotingExperiencePartyWidgetName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getVotingExperiencePartyWidget();
}

std::string SAESGUIConfigurationManager::GetPartyWidgetEventboxName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getPartyWidgetEventbox();
}

std::string SAESGUIConfigurationManager::GetEventboxImageStepInstallationWizardName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getEventboxImageStepInstallationWizard();
}

std::string SAESGUIConfigurationManager::GetEventboxStepInstallationWizardName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getEventboxStepInstallationWizard();
}

std::string SAESGUIConfigurationManager::GetParentKeypadEventboxName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getParentKeypadEventbox();
}

std::string SAESGUIConfigurationManager::GetInfoFrameName()
{
    return GetThemesConfiguration().getOfficialThemeConfiguration().getInfoFrame();
}

std::string SAESGUIConfigurationManager::GetVotingWindowButtonDemoName()
{
    return GetThemesConfiguration().getDemoThemeConfiguration().getVotingWindowButtonDemo();
}

std::string SAESGUIConfigurationManager::GetVotingExperiencePartyEventboxDemoName()
{
    return GetThemesConfiguration().getDemoThemeConfiguration().getVotingExperiencePartyEventboxDemo();
}

std::string SAESGUIConfigurationManager::GetVotingExperienceWindowDemoName()
{
    return GetThemesConfiguration().getDemoThemeConfiguration().getVotingExperienceWindowDemo();
}

std::string SAESGUIConfigurationManager::GetVotingExperienceBallotOptionEventboxDemoName()
{
    return GetThemesConfiguration().getDemoThemeConfiguration().getVotingExperienceBallotOptionEventboxDemo();
}

std::vector<Smartmatic::SAES::GUI::Widgets::MenuOption> SAESGUIConfigurationManager::GetMenuOptionConfiguration()
{
    return GetInnerOptions ( getMenuWidgetConfiguration().getRootOption() );
}

std::vector<Smartmatic::SAES::GUI::Widgets::MenuOption> SAESGUIConfigurationManager::GetInnerOptions ( XMLMenuOption xmlMenuOption )
{
    std::vector<Smartmatic::SAES::GUI::Widgets::MenuOption> optionsReturn;
    MenuOptions childrenOptions = xmlMenuOption.getMenuOptions();

    for ( MenuOptions::OptionIterator i ( childrenOptions.getOption().begin() );	i != childrenOptions.getOption().end (); ++i )
    {
        const XMLMenuOption & xmlOption ( *i );
        Smartmatic::SAES::GUI::Widgets::MenuOption option; /// = *new Smartmatic::SAES::GUI::Widgets::MenuOption();
        option.nameKey = _ ( xmlOption.getUntraslatedKey().c_str() ); //FG: this translate the name of the button
        option.accessibleName = xmlOption.getAccessibleName();
        option.imageKey = xmlOption.getImageKey();
        option.code = xmlOption.getCode();
        option.mappingKey = xmlOption.getMappingKey();
        option.mappingKeyName = xmlOption.getMappingKeyName ();
        option.optionDescriptor = xmlOption.getOptionType ();
        option.v_MenuOption = GetInnerOptions ( xmlOption );
        optionsReturn.push_back ( option );
    }

    return optionsReturn;
}

Smartmatic::SAES::GUI::Widgets::MenuOption SAESGUIConfigurationManager::GetSMMTMenuOptionFromXMLMenuOption ( XMLMenuOption xmlOption )
{
    Smartmatic::SAES::GUI::Widgets::MenuOption optReturn;
    optReturn.nameKey = xmlOption.getUntraslatedKey().c_str();
    optReturn.accessibleName = xmlOption.getAccessibleName().c_str();
    optReturn.imageKey = xmlOption.getImageKey();
    optReturn.code = xmlOption.getCode();
    optReturn.mappingKey = xmlOption.getMappingKey();
    optReturn.mappingKeyName = xmlOption.getMappingKeyName ();
    optReturn.optionDescriptor = xmlOption.getOptionType ();
    optReturn.v_MenuOption = GetInnerOptions ( xmlOption );
    return optReturn;
}

std::vector<Smartmatic::SAES::GUI::Widgets::WizardConfiguration> SAESGUIConfigurationManager::GetFunctionalityWizardConfiguration()
{
    std::vector<Smartmatic::SAES::GUI::Widgets::WizardConfiguration> wizardWidgetConfigurationVector;
    FunctionalityWizardFlowConfiguration wizardFlow = getFunctionalityWizardConfiguration();
    FunctionalityWizardFlowConfiguration::WizardStatesConfigurationSequence wizardStatesConfigurationSeq = wizardFlow.getWizardStatesConfiguration();
    FunctionalityWizardFlowConfiguration::WizardStatesConfigurationIterator iterWizardStatesConfiguration;

    for ( iterWizardStatesConfiguration = wizardStatesConfigurationSeq.begin();
            iterWizardStatesConfiguration != wizardStatesConfigurationSeq.end();
            iterWizardStatesConfiguration++ )
    {
        Smartmatic::SAES::GUI::Widgets::WizardConfiguration wizardWidgetConfiguration;
        FunctionalityWizardStateConfiguration functionalityWizardStateConfiguration = *iterWizardStatesConfiguration;
        FunctionalityWizardStateConfiguration::PrincipalMenuOptionsSequence principalSeq =
            functionalityWizardStateConfiguration.getPrincipalMenuOptions();
        FunctionalityWizardStateConfiguration::PrincipalMenuOptionsIterator principalIter;
        std::vector<Smartmatic::SAES::GUI::Widgets::MenuOption> principalVector;

        for ( principalIter = principalSeq.begin(); principalIter != principalSeq.end(); principalIter++ )
        {
            Smartmatic::SAES::GUI::Widgets::MenuOption opt = GetSMMTMenuOptionFromXMLMenuOption ( *principalIter );
            opt.width = ( *iterWizardStatesConfiguration ).getButtonWidth();
            opt.height = ( *iterWizardStatesConfiguration ).getButtonHeight();
            principalVector.push_back ( opt );
        }

        wizardWidgetConfiguration.setPrincipalOptions ( principalVector );
        std::vector<Smartmatic::SAES::GUI::Widgets::MenuOption> toolVector;
        FunctionalityWizardStateConfiguration::ToolbarMenuOptionsSequence toolSeq =
            functionalityWizardStateConfiguration.getToolbarMenuOptions();
        FunctionalityWizardStateConfiguration::ToolbarMenuOptionsIterator toolIter;

        for ( toolIter = toolSeq.begin(); toolIter != toolSeq.end(); toolIter++ )
        {
            Smartmatic::SAES::GUI::Widgets::MenuOption opt = GetSMMTMenuOptionFromXMLMenuOption ( *toolIter );
            toolVector.push_back ( opt );
        }

        wizardWidgetConfiguration.setToolbox ( toolVector );
        wizardWidgetConfiguration.setStateName ( functionalityWizardStateConfiguration.getStateNameKey() );
        wizardWidgetConfiguration.setStateDescription ( functionalityWizardStateConfiguration.getStateDescriptionKey() );
        wizardWidgetConfiguration.setAccessibleName ( functionalityWizardStateConfiguration.getAccessibleName() );

        wizardWidgetConfiguration.setStateHelpTitleKey ( functionalityWizardStateConfiguration.getStateHelp().getTitleKey() );
        wizardWidgetConfiguration.setStateHelpContentKey ( functionalityWizardStateConfiguration.getStateHelp().getContentKey() );
        wizardWidgetConfiguration.setStateHelpRelativeImagePath ( functionalityWizardStateConfiguration.getStateHelp().getRelativeImagePath() );
        wizardWidgetConfiguration.setTimelineNameKey ( functionalityWizardStateConfiguration.getTimeLineConfiguration().getNameKey() );
        wizardWidgetConfiguration.setTimelineDescriptionKey ( functionalityWizardStateConfiguration.getTimeLineConfiguration().getDescriptionKey() );
        wizardWidgetConfiguration.setTimelineAccessibleName ( functionalityWizardStateConfiguration.getTimeLineConfiguration().getAccessibleName() );

        TimeLineImageConfiguration::ImageSequence & imageSeq ( functionalityWizardStateConfiguration.getTimeLineConfiguration().getImageConfiguration().getImage() );
        TimeLineImageConfiguration::ImageIterator itseq;

        Smartmatic::SAES::Functionality::Configuration::FlowStatesConfiguration::ProcessStatusSequence & flowSteps ( Smartmatic::SAES::Functionality::Configuration::SAESFunctionalityConfigurationManager::getInstance()->getFlowStatesConfiguration().getProcessStatus() );
        Smartmatic::SAES::Functionality::Configuration::FlowStatesConfiguration::ProcessStatusIterator itsteps;

        for ( itsteps = flowSteps.begin();
                itsteps != flowSteps.end();
                ++itsteps )
        {
            for ( itseq = imageSeq.begin ();
                    itseq != imageSeq.end ();
                    ++itseq )
            {
                if ( ( *itsteps ) == ( *itseq ).getStatus() )
                {
                    wizardWidgetConfiguration.addTimelineRelativeImagePath ( ( *itseq ).getPath(), ( *itseq ).getAccessibleName() );
                }
            }
        }

        wizardWidgetConfiguration.setToolboxElementWidth ( functionalityWizardStateConfiguration.getToolboxConfiguration().getToolboxElementWidth() );
        wizardWidgetConfiguration.setToolboxElementHeight ( functionalityWizardStateConfiguration.getToolboxConfiguration().getToolboxElementHeight() );
        wizardWidgetConfiguration.setImageHelp ( functionalityWizardStateConfiguration.getStateHelp().getHelp().getImageHelp() );
        wizardWidgetConfiguration.setPdfHelp ( functionalityWizardStateConfiguration.getStateHelp().getHelp().getPdfHelp() );
        wizardWidgetConfiguration.setTitleKey ( functionalityWizardStateConfiguration.getStateHelp().getHelp().getTitleKey() );
        wizardWidgetConfiguration.setUseHelp ( functionalityWizardStateConfiguration.getStateHelp().getHelp().getUseHelp() );

        bool enablePrinting = true;

        if ( functionalityWizardStateConfiguration.getStateHelp().getHelp().getEnableFilePrinting().present() )
        {
            enablePrinting = functionalityWizardStateConfiguration.getStateHelp().getHelp().getEnableFilePrinting().get();
        }

        wizardWidgetConfiguration.setEnableFilePrinting ( enablePrinting );
        wizardWidgetConfiguration.setStateEnum ( functionalityWizardStateConfiguration.getProcessStatus() );
        wizardWidgetConfiguration.setEnumIntState ( ( int ) functionalityWizardStateConfiguration.getProcessStatus() );
        wizardWidgetConfiguration.setHelpLabelConfiguration ( ConvertToLabelConfiguration ( functionalityWizardStateConfiguration.getHelpLabelConfiguration() ) );
        wizardWidgetConfiguration.setTitleHelpLabelConfiguration ( ConvertToLabelConfiguration ( functionalityWizardStateConfiguration.getTitleHelpLabelConfiguration() ) );
        wizardWidgetConfiguration.setButtonsLabelConfiguration ( ConvertToLabelConfiguration ( functionalityWizardStateConfiguration.getButtonsLabelConfiguration() ) );
        wizardWidgetConfigurationVector.push_back ( wizardWidgetConfiguration );
    }

    return wizardWidgetConfigurationVector;
}

ChangeHourFunctionalityConfiguration SAESGUIConfigurationManager::GetChangeHourFunctionalitConfiguration()
{
    return getConfiguration()->getChangeHourFunctionalityConfiguration();
}

ColorConfiguration SAESGUIConfigurationManager::getCardActivatorStatisticsTitleColor()
{
    return getConfiguration()->getCardActivatorFunctionalityConfiguration ().getStatisticsTitleColor();
}

HelpConfiguration SAESGUIConfigurationManager::getHelpConfiguration ( Smartmatic::SAES::Functionality::Configuration::MenuOptionType menuType )
{
    HelpConfiguration config;
    config.setImageHelp ( "xx" );
    config.setPdfHelp ( "yy" );
    config.setTitleKey ( " " );
    config.setUseHelp ( false );
    config.setEnableFilePrinting ( true );

    for ( unsigned int i = 0; i < getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration().size(); i++ )
    {
        MenuOptionType saesMenuOptionType ( menuType );

        if ( saesMenuOptionType == getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration() [i].getFunctionalityDescriptor() )
        {
            if ( getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration() [i].getHelp() )
            {
                config = getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration() [i].getHelp().get();
            }

            break;
        }
    }

    return config;
}

ElectionBlockStatusList::ElectionStatusSequence SAESGUIConfigurationManager::getDisableStateList ( Smartmatic::SAES::Functionality::Configuration::MenuOptionType menuType )
{
    ElectionBlockStatusList::ElectionStatusSequence seqStatus;
    seqStatus.clear();

    for ( unsigned int i = 0; i < getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration().size(); i++ )
    {
        MenuOptionType saesMenuOptionType ( menuType );

        if ( saesMenuOptionType == getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration() [i].getFunctionalityDescriptor() )
        {
            if ( getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration() [i].getElectionBlockStatus() )
            {
                seqStatus = getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration() [i].getElectionBlockStatus().get().getElectionStatus();
            }

            break;
        }
    }

    return seqStatus;
}

NecessaryDevicesConfiguration::DeviceConfigurationSequence SAESGUIConfigurationManager::getDeviceConfiguration ( Smartmatic::SAES::Functionality::Configuration::MenuOptionType menuType )
{
    NecessaryDevicesConfiguration::DeviceConfigurationSequence seq;
    seq.clear();

    for ( unsigned int i = 0; i < getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration().size(); i++ )
    {
        MenuOptionType saesMenuOptionType ( menuType );

        if ( saesMenuOptionType == getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration() [i].getFunctionalityDescriptor() )
        {
            if ( getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration() [i].getNecessaryDevicesConfiguration() != NULL )
            {
                seq = getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration() [i].getNecessaryDevicesConfiguration().get().getDeviceConfiguration();
            }

            break;
        }
    }

    return seq;
}

CardActivatorFunctionalityConfiguration * SAESGUIConfigurationManager::GetCardActivatorWidgetConfiguration()
{
    return & ( getConfiguration()->getCardActivatorFunctionalityConfiguration() );
}

TouchScreenCalibrationConfiguration & SAESGUIConfigurationManager::getTouchScreenCalibrationConfiguration()
{
    return getConfiguration()->getTouchScreenCalibrationConfiguration();
}

DiagnosticType SAESGUIConfigurationManager::getAllDiagnosisDiagnosticType ( const DiagnosticType::DeviceType & type )
{
    DiagnosticType diag ( type, true, 1000 );

    AllDiagnosisConfiguration::DiagnosticToApplySequence & seq ( getConfiguration()->getAllDiagnosisConfiguration ().getDiagnosticToApply() );
    AllDiagnosisConfiguration::DiagnosticToApplyIterator it;

    for ( it = seq.begin();
            it != seq.end();
            ++it )
    {
        if ( ( *it ).getDevice() == type )
        {
            diag = ( *it );
            break;
        }
    }

    return diag;
}

DiagnosticType SAESGUIConfigurationManager::getAutoDiagnosisDiagnosticType ( const DiagnosticType::DeviceType & type )
{
    DiagnosticType diag ( type, true, 1000 );

    AutomaticWindowConfigurationForOfficialMode::DiagnosticToApplySequence & seq ( getConfiguration()->getAutomaticWindowConfigurationForOfficialMode().getDiagnosticToApply() );
    AutomaticWindowConfigurationForOfficialMode::DiagnosticToApplyIterator it;

    for ( it = seq.begin();
            it != seq.end();
            ++it )
    {
        if ( ( *it ).getDevice() == type )
        {
            diag = ( *it );
            break;
        }
    }

    return diag;
}

int SAESGUIConfigurationManager::getAllDiagnosisWaitTime ( const DiagnosticType::DeviceType & type )
{
    DiagnosticType diag = getAllDiagnosisDiagnosticType ( type );
    return diag.getWaitTime();
}

int SAESGUIConfigurationManager::getAutoDiagnosisWaitTime ( const DiagnosticType::DeviceType & type )
{
    DiagnosticType diag = getAutoDiagnosisDiagnosticType ( type );
    return diag.getWaitTime();
}

int SAESGUIConfigurationManager::getAllDiagnosisPrinterWaitTime()
{
    return getAllDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_PRINTER );
}

SmartcardReaderDiagnosisConfiguration & SAESGUIConfigurationManager::getSmartcardReaderDiagnosisConfiguration()
{
    return getConfiguration()->getSmartcardReaderDiagnosisConfiguration();
}

int SAESGUIConfigurationManager::getAllDiagnosisBarcodeWaitTime()
{
    return getAllDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_BARCODE );
}

int SAESGUIConfigurationManager::getAllDiagnosisSmartcardWaitTime()
{
    return getAllDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_SMARTCARD );
}

int SAESGUIConfigurationManager::getAllDiagnosisCalibrationWaitTime()
{
    return getAllDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION );
}

int SAESGUIConfigurationManager::getAllDiagnosisAlarmWaitTime()
{
    return getAllDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_ALARMBOX );
}

int SAESGUIConfigurationManager::getAllDiagnosisFlashWaitTime()
{
    return getAllDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_FLASH );
}

int SAESGUIConfigurationManager::getAllDiagnosisAbleWaitTime()
{
    return getAllDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_P168A );
}

int SAESGUIConfigurationManager::getAutoDiagnosisPrinterWaitTime()
{
    return getAutoDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_PRINTER );
}

int SAESGUIConfigurationManager::getAutoDiagnosisBarcodeWaitTime()
{
    return getAutoDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_BARCODE );
}

int SAESGUIConfigurationManager::getAutoDiagnosisSmartcardWaitTime()
{
    return getAutoDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_SMARTCARD );
}

int SAESGUIConfigurationManager::getAutoDiagnosisAlarmWaitTime()
{
    return getAutoDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_ALARMBOX );
}

int SAESGUIConfigurationManager::getAutoDiagnosisFlashWaitTime()
{
    return getAutoDiagnosisWaitTime ( DiagnosticType::DeviceType::CONFIGURATION_FLASH );
}

std::string  SAESGUIConfigurationManager::getCustomAttributesPresentation()
{
    return getConfiguration()->getCardActivatorFunctionalityConfiguration().getCustomAttributesPresentation();
}

bool SAESGUIConfigurationManager::getShowAudioCheckButton()
{
    return getConfiguration()->getCardActivatorFunctionalityConfiguration().getShowAudioCheck();
}

PrinterDiagnosisConfiguration SAESGUIConfigurationManager::getPrinterDiagnosisConfiguration()
{
    return getConfiguration()->getPrinterDiagnosisConfiguration();
}

InstallationWizardConfiguration & SAESGUIConfigurationManager::getInstallationWizardConfiguration()
{
    return getConfiguration()->getInstallationWizardConfiguration ();
}

PMWidgetConfiguration & SAESGUIConfigurationManager::getPMInstallationWidgetConfig()
{
    return getConfiguration()->getPMWidgetConfiguration();
}

std::vector<DiagnosticType> SAESGUIConfigurationManager::getAllDiagnosisList()
{
    std::vector<DiagnosticType> diagnosisList;
    AllDiagnosisConfiguration::DiagnosticToApplySequence seq = getConfiguration()->getAllDiagnosisConfiguration ().getDiagnosticToApply();

    for ( unsigned int i = 0; i < seq.size(); i++ )
    {
        diagnosisList.push_back ( seq[i] );
    }

    return diagnosisList;
}

AutomaticWindowConfigurationForOfficialMode & SAESGUIConfigurationManager::GetAutomaticWindowConfigurationForOfficialMode()
{
    return getConfiguration()->getAutomaticWindowConfigurationForOfficialMode ();
}

AutomaticWindowConfigurationForRecountMode & SAESGUIConfigurationManager::GetAutomaticWindowConfigurationForRecountMode()
{
    return getConfiguration()->getAutomaticWindowConfigurationForRecountMode();
}

AllDiagnosisConfiguration & SAESGUIConfigurationManager::getAllDiagnosisConfiguration()
{
    return getConfiguration()->getAllDiagnosisConfiguration();
}

DeviceDiagnosticResultConfiguration & SAESGUIConfigurationManager::getDeviceDiagnosticResultConfiguration()
{
    return getConfiguration()->getDeviceDiagnosticResultConfiguration();
}

SerialNumberConfiguration & SAESGUIConfigurationManager::getSerialNumberConfiguration()
{
    return getConfiguration()->getSerialNumberConfiguration();
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getSerialNumberTitleConfiguration()
{
    return ConvertToLabelConfiguration ( getSerialNumberConfiguration().getLabelTitle() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getSerialNumberInfoConfiguration()
{
    return ConvertToLabelConfiguration ( getSerialNumberConfiguration().getLabelInto() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getSerialNumberMessageConfiguration()
{
    return ConvertToLabelConfiguration ( getSerialNumberConfiguration().getLabelMessage() );
}

ZipCodeConfiguration & SAESGUIConfigurationManager::getZipCodeConfiguration()
{
    return getConfiguration()->getZipCodeConfiguration();
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getZipCodeTitleConfiguration()
{
    return ConvertToLabelConfiguration ( getZipCodeConfiguration().getLabelTitle() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getZipCodeInfoConfiguration()
{
    return ConvertToLabelConfiguration ( getZipCodeConfiguration().getLabelInto() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getZipCodeMessageConfiguration()
{
    return ConvertToLabelConfiguration ( getZipCodeConfiguration().getLabelMessage() );
}

std::vector<DiagnosticType> SAESGUIConfigurationManager::getAutoDiagnosisList()
{
    std::vector<DiagnosticType> diagnosisList;
    std::vector<DiagnosticType>::iterator it;
    AllDiagnosisConfiguration::DiagnosticToApplySequence seq = getConfiguration()->getAutomaticWindowConfigurationForOfficialMode ().getDiagnosticToApply();
    RunningMode mode = VotingCentersWrapper::getInstance()->getRunningMode();

    if (mode == RunningMode::VOTING_CENTER_RECOUNT)
    {
        seq.clear();
        seq = getConfiguration()->getAutomaticWindowConfigurationForRecountMode ().getDiagnosticToApply();
    }

    for ( unsigned int i = 0; i < seq.size(); i++ )
    {
        bool found = false;

        for ( it = skipDiagnosis.begin();
                it != skipDiagnosis.end();
                ++it )
        {
            if ( seq[i].getDevice() == ( *it ).getDevice() )
            {
                found = true;
                break;
            }
        }

        if ( !found )
        {
            diagnosisList.push_back ( seq[i] );
        }
    }

    return diagnosisList;
}

void SAESGUIConfigurationManager::addSkipAutoDiagnosis ( DiagnosticType type )
{
    skipDiagnosis.push_back ( type );
}

::ActivatedCardStatisticsConfiguration SAESGUIConfigurationManager::getCardActivationStatisticsConfiguration()
{
    return getConfiguration()->getCardActivatorFunctionalityConfiguration().getCardStatisticsConfiguration();
}

WaitingForCardWindowConfiguration SAESGUIConfigurationManager::getWaitingForCardWindowConfiguration()
{
    return getConfiguration()->getWaitingForCardWindowConfiguration();
}

VoteDemoFunctionalityConfiguration SAESGUIConfigurationManager::getVoteDemoFunctionalityConfiguration()
{
    return getConfiguration()->getVoteDemoFunctionalityConfiguration();
}

BarcodeDiagnosisConfiguration SAESGUIConfigurationManager::getBarcodeDiagnosisConfiguration()
{
    return getConfiguration()->getBarcodeDiagnosisConfiguration ();
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetCardActivatorStatisticsTitleLabelConfiguration()
{
    return ConvertToLabelConfiguration ( getConfiguration()->getCardActivatorFunctionalityConfiguration ().getStatisticsTitleLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetPreviewVoteLabelConfiguration()
{
    return ConvertToLabelConfiguration ( getConfiguration()->getWaitingForCardWindowConfiguration ().getPreviewVoteLabelConfiguration() );
}

ButtonConfiguration SAESGUIConfigurationManager::getPreviewVoteButtonConfiguration()
{
    return getConfiguration()->getWaitingForCardWindowConfiguration ().getPreviewVoteButtonConfiguration();
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetCustomAtributtesWindowHeaderLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetCustomAtributtesWindowConfiguration().getHeaderLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetCustomAtributtesWindowBottomLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetCustomAtributtesWindowConfiguration().getBottomLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetCustomAtributtesWindowCenterLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetCustomAtributtesWindowConfiguration().getCenterLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration  SAESGUIConfigurationManager::GetDiagnosisLabelConfiguration()
{
    return ConvertToLabelConfiguration ( getConfiguration()->getGeneralDiagnosisConfiguration().getDiagnosisLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration  SAESGUIConfigurationManager::getDiagnosisFooterLabelConfiguration()
{
    return ConvertToLabelConfiguration ( getConfiguration()->getGeneralDiagnosisConfiguration().getDiagnosisFooterLabelConfiguration() );
}

bool SAESGUIConfigurationManager::getShowFooterInAutoDiagnosis()
{
    return getConfiguration()->getAutomaticWindowConfigurationForOfficialMode().getShowFooterInAutoDiagnosis();
}

Smartmatic::GUI::Configuration::LabelConfiguration  SAESGUIConfigurationManager::getResultErrorLabelConfiguration()
{
    return ConvertToLabelConfiguration ( getConfiguration()->getGeneralDiagnosisConfiguration().getResultErrorLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration  SAESGUIConfigurationManager::GetExitMessageLabelConfiguration()
{
    return ConvertToLabelConfiguration ( getConfiguration()->getExitMessageLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetPartySelectionWindowBottomLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetPartySelectionConfiguration().getBottomLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetPartySelectionWindowTitleLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetPartySelectionConfiguration().getTitleLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetPartySelectionWindowSubtitleLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetPartySelectionConfiguration().getSubtitleLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetBallotOptionSelectionWindowBottomLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetBallotOptionSelectionConfiguration().getBottomLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetBallotOptionSelectionWindowTitleLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetBallotOptionSelectionConfiguration().getTitleLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetBallotOptionSelectionWindowSubtitleLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetBallotOptionSelectionConfiguration().getSubtitleLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetSelectionLanguageHeaderLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getSelectionLanguageHeaderLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetSelectionLanguageBottomLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getSelectionLanguageBottomLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetSelectionLanguageConfirmationHeaderLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getSelectionLanguageConfirmationHeaderLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetSelectionLanguageConfirmationBottomLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getSelectionLanguageConfirmationBottomLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetSelectionLanguageConfirmationCenterLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getSelectionLanguageConfirmationCenterLabelConfiguration() );
}




Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetReportListTitleLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetPrintingWidgetConfiguration ().getCompositeDescriptionLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetProgressLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetPrintingWidgetConfiguration ().getReportListTitleLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetCompositeMsgLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetPrintingWidgetConfiguration ().getProgressMessageLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetFileItemLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetPrintingWidgetConfiguration ().getReportListItemLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetWelcomeContestCenterLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getWelcomeContestCenterLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetLoadingBallotsLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getLoadingBallotsLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetCheckSumLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getCheckSumLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetInterrupVotingExperienceMessageLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getInterrupVotingExperienceMessageLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetInterrupVotingExperienceCancelLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getInterrupVotingExperienceCancelLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getAuthenticationPasswordLabelConfiguration()
{
    return ConvertToLabelConfiguration ( getConfiguration()->getAuthenticationWindowConfiguration().getPasswordLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getAuthenticationAlarmLabelConfiguration()
{
    return ConvertToLabelConfiguration ( getConfiguration()->getAuthenticationWindowConfiguration().getAlarmLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getAuthenticationInfoLabelConfiguration()
{
    return ConvertToLabelConfiguration ( getConfiguration()->getAuthenticationWindowConfiguration().getInfoLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getAuthenticationVotingInfoLabelConfiguration()
{
    return ConvertToLabelConfiguration ( getConfiguration()->getAuthenticationWindowConfiguration().getVotingInfoLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration & SAESGUIConfigurationManager::GetVotingWidgetLabelChildLabelConfiguration()
{
    static bool load = false;

    if ( !load )
    {
        load = true;
        votingWidgetLabelChildLabelConfiguration = ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getVotingWidgetLabelChildLabelConfiguration() );
    }

    return votingWidgetLabelChildLabelConfiguration;
}

Smartmatic::GUI::Configuration::LabelConfiguration & SAESGUIConfigurationManager::GetVotingWidgetTopChildLabelConfiguration()
{
    static bool load = false;

    if ( !load )
    {
        load = true;
        votingWidgetTopChildLabelConfiguration = ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getVotingWidgetTopChildLabelConfiguration() );
    }

    return votingWidgetTopChildLabelConfiguration;
}

Smartmatic::GUI::Configuration::LabelConfiguration & SAESGUIConfigurationManager::GetVotingWidgetBottomChildLabelConfiguration()
{
    static bool load = false;

    if ( !load )
    {
        load = true;
        votingWidgetBottomChildLabelConfiguration = ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getVotingWidgetBottomChildLabelConfiguration() );
    }

    return votingWidgetBottomChildLabelConfiguration;
}

Smartmatic::GUI::Configuration::LabelConfiguration & SAESGUIConfigurationManager::GetVotingWidgetNumberChildLabelConfiguration()
{
    static bool load = false;

    if ( !load )
    {
        load = true;
        votingWidgetNumberChildLabelConfiguration = ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getVotingWidgetNumberChildLabelConfiguration() );
    }

    return votingWidgetNumberChildLabelConfiguration;
}

Smartmatic::GUI::Configuration::LabelConfiguration & SAESGUIConfigurationManager::GetPartyNullLabelConfiguration()
{
    static bool load = false;

    if ( !load )
    {
        load = true;
        partyNullLabelConfiguration = ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getPartyNullLabelConfiguration() );
    }

    return partyNullLabelConfiguration;
}

Smartmatic::GUI::Configuration::LabelConfiguration & SAESGUIConfigurationManager::GetVotingWidgetPartySelectionLabelChildLabelConfiguration()
{
    static bool load = false;

    if ( !load )
    {
        load = true;
        votingWidgetPartySelectionLabelChildLabelConfiguration = ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getVotingWidgetPartySelectionLabelChildLabelConfiguration() );
    }

    return votingWidgetPartySelectionLabelChildLabelConfiguration;
}

Smartmatic::GUI::Configuration::LabelConfiguration & SAESGUIConfigurationManager::GetVotingWindowEndMessagelabelConfig()
{
    static bool load = false;

    if ( !load )
    {
        load = true;
        votingWindowEndMessagelabelConfig = ConvertToLabelConfiguration ( GetVotingExperienceLabelConfiguration().getVotingWindowEndMessageLabelConfiguration() );
    }

    return votingWindowEndMessagelabelConfig;
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetVMLogingMessageLabel()
{
    return ConvertToLabelConfiguration ( GetFormMessagesLabelsConfiguration().getVMLogingMessageLabel() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetReferendumHeaderLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetReferendumConfiguration().getHeaderLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetReferendumBottomLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetReferendumConfiguration().getBottomLabelConfiguration() );
}
Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetReferendumQuestionLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetReferendumConfiguration().getQuestionLabelConfiguration() );
}
Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetReferendumBallotsLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetReferendumConfiguration().getBallotsLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::ConvertToLabelConfiguration ( XMLLabelConfiguration xmlLabelConfiguration )
{
    Smartmatic::GUI::Configuration::LabelConfiguration labelConfiguration;

    switch ( xmlLabelConfiguration.getLabel_Alignment() )
    {
        default:
        case Label_Alignment::CONFIGURATION_CENTER:
            {
                labelConfiguration.SetLabelAlignment ( Smartmatic::GUI::Configuration::CenterAlignment );
            }
            break;

        case Label_Alignment::CONFIGURATION_LEFT:
            {
                labelConfiguration.SetLabelAlignment ( Smartmatic::GUI::Configuration::LeftAlignment );
            }
            break;

        case Label_Alignment::CONFIGURATION_RIGHT:
            {
                labelConfiguration.SetLabelAlignment ( Smartmatic::GUI::Configuration::RigthAlignment );
            }
            break;
    }

    labelConfiguration.SetVerticalPadding ( xmlLabelConfiguration.getLabel_VerticalPadding() );
    labelConfiguration.SetHorizontalPadding ( xmlLabelConfiguration.getLabel_HorizontalPadding() );
    labelConfiguration.SetFontFamily ( xmlLabelConfiguration.getLabel_Family() );

    switch ( xmlLabelConfiguration.getLabel_Style() )
    {
        default:
        case Label_Style::CONFIGURATION_NORMAL:
            {
                labelConfiguration.SetStyle ( Smartmatic::GUI::Configuration::NormalStyle );
            }
            break;

        case Label_Style::CONFIGURATION_OBLIQUE:
            {
                labelConfiguration.SetStyle ( Smartmatic::GUI::Configuration::ObliqueStyle );
            }
            break;

        case Label_Style::CONFIGURATION_ITALIC:
            {
                labelConfiguration.SetStyle ( Smartmatic::GUI::Configuration::ItalicStyle );
            }
            break;
    }

    switch ( xmlLabelConfiguration.getLabel_Weight() )
    {
        default:
        case Label_Weight::CONFIGURATION_LIGHT:
            {
                labelConfiguration.SetWeight ( Smartmatic::GUI::Configuration::LightWeight );
            }
            break;

        case Label_Weight::CONFIGURATION_NORMAL:
            {
                labelConfiguration.SetWeight ( Smartmatic::GUI::Configuration::NormalWeight );
            }
            break;

        case Label_Weight::CONFIGURATION_BOLD:
            {
                labelConfiguration.SetWeight ( Smartmatic::GUI::Configuration::BoldWeight );
            }
            break;
    }

    labelConfiguration.SetSize ( xmlLabelConfiguration.getLabel_Size() );
    return labelConfiguration;
}

Smartmatic::GUI::Configuration::LabelAlignmentEnum SAESGUIConfigurationManager::ConvertToLabelConfiguration ( AlignmentConfiguration alignmentConfiguration )
{
    Smartmatic::GUI::Configuration::LabelAlignmentEnum labelAlignment;

    switch ( alignmentConfiguration)
    {
        default:
        case AlignmentConfiguration::ELECTION_CENTER:
            {
                labelAlignment = Smartmatic::GUI::Configuration::CenterAlignment;
            }
            break;

        case AlignmentConfiguration::ELECTION_LEFT:
            {
                labelAlignment = Smartmatic::GUI::Configuration::LeftAlignment;
            }
            break;

        case AlignmentConfiguration::ELECTION_RIGHT:
            {
                labelAlignment =  Smartmatic::GUI::Configuration::RigthAlignment;
            }
            break;
    }

    return labelAlignment;
}

Gdk::Color SAESGUIConfigurationManager::ConvertToGdkColor ( ColorConfiguration colorConfiguration )
{
    Gdk::Color color;
    unsigned r = colorConfiguration.getRed();
    unsigned g = colorConfiguration.getGreen();
    unsigned b = colorConfiguration.getBlue();
    color.set_rgb ( r, g, b );
    return color;
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetOpenElectionPollWorkerLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetOpenElectionFunctionalityConfiguration().getDialogLabelConfiguration() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetEmptyWindowTitleFormConfiguration()
{
    return ConvertToLabelConfiguration ( GetFormTitlesLabelsConfiguration().getEmptyWindowTitleLabel() );
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::GetUnableFunctionalityLabelConfiguration()
{
    return ConvertToLabelConfiguration ( GetCommonControlsFormConfiguration().getUnableFunctionalityMessage() );
}

int SAESGUIConfigurationManager::GetUnableMessageCloseTime()
{
    return GetCommonControlsFormConfiguration().getUnableMessageCloseTime();
}

int SAESGUIConfigurationManager::GetExitFunctionalityMessageCloseTime()
{
    return GetCommonControlsFormConfiguration().getExitFunctionalityMessageCloseTime();
}

FormsTitleLabelConfigurations & SAESGUIConfigurationManager::GetFormTitlesLabelsConfiguration()
{
    return GetCommonControlsFormConfiguration().getFormTitles();
}

::FormsMessagesLabelConfigurations & SAESGUIConfigurationManager::GetFormMessagesLabelsConfiguration()
{
    return GetCommonControlsFormConfiguration().getFormMesages();
}

::FormsCommonControlsConfigurations & SAESGUIConfigurationManager::GetCommonControlsFormConfiguration()
{
    return getConfiguration()->getFormsCommonControls();
}

bool SAESGUIConfigurationManager::GetShowExitMessage ( Smartmatic::SAES::Functionality::Configuration::MenuOptionType menuOptionDescriptor )
{
    for ( unsigned int i = 0; i < getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration().size(); i++ )
    {
        MenuOptionType saesMenuOptionType ( menuOptionDescriptor );

        if ( saesMenuOptionType == getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration() [i].getFunctionalityDescriptor() )
        {
            return getConfiguration()->getFunctionalitiesConfiguration().getFunctionalityConfiguration() [i].getShowExitMessage();
        }
    }

    return false;
}

std::string SAESGUIConfigurationManager::getPMInstalationSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getPMInstalationSound();
}

std::string SAESGUIConfigurationManager::getVMInstalationSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getVMInstalationSound();
}

std::string SAESGUIConfigurationManager::getAuthenticationValidPasswordSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getAuthenticationValidPasswordSound();
}

std::string SAESGUIConfigurationManager::getAuthenticationInvalidPasswordSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getAuthenticationInvalidPasswordSound();
}

std::string SAESGUIConfigurationManager::getVotingExperienceSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getVotingExperienceSound();
}

std::string SAESGUIConfigurationManager::getBallotOptionSelectSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getBallotOptionSelectSound();
}

std::string SAESGUIConfigurationManager::getPartySelectSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getPartySelectSound();
}

std::string SAESGUIConfigurationManager::getDiagnosisSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getDiagnosisSound();
}

MemoryStateWidgetConfiguration & SAESGUIConfigurationManager::getMemoryStateWidgetConfiguration()
{
    return getConfiguration()->getMemoryStateWidgetConfiguration();
}

ExternalPrintWizardWidgetConfiguration & SAESGUIConfigurationManager::getExternalPrintWizardWidgetConfiguration()
{
    return getConfiguration()->getExternalPrintWizardWidgetConfiguration();
}

HelpWidgetConfiguration & SAESGUIConfigurationManager::getHelpWidgetConfiguration()
{
    return getConfiguration()->getHelpWidgetConfiguration();
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getMemoryStateWidgetTextConfiguration()
{
    return ConvertToLabelConfiguration ( getMemoryStateWidgetConfiguration().getTextFontConf() );
}

int SAESGUIConfigurationManager::getCloseVotationMinimumOfficialsActivationForConfirmation()
{
    return getConfiguration()->getCloseVotationFunctionalityConfiguration().getMinimumOfficialsActivationForConfirmation();
}

bool SAESGUIConfigurationManager::getCloseVotationShowConfirmationMessageWhenClosing()
{
    return getConfiguration()->getCloseVotationFunctionalityConfiguration().getShowConfirmationMessageWhenClosing();
}

PollingCardConfiguration & SAESGUIConfigurationManager::getPollingCardConfiguration()
{
    return getConfiguration()->getPollingCardConfiguration();
}

AlarmBoxMonitorConfiguration & SAESGUIConfigurationManager::getAlarmBoxMonitorConfiguration()
{
    return getConfiguration()->getAlarmBoxMonitorConfiguration();
}

StartConfiguration & SAESGUIConfigurationManager::getStartConfiguration()
{
    return getConfiguration()->getStartConfiguration();
}

DataUtilsValidatorFunctionalityConfiguration & SAESGUIConfigurationManager::getDataUtilsValidatorFunctionalityConfiguration()
{
    return getConfiguration()->getDataUtilsValidatorFunctionalityConfiguration();
}

Smartmatic::GUI::Configuration::LabelConfiguration SAESGUIConfigurationManager::getBlockDiagnosticTitle()
{
    return ConvertToLabelConfiguration ( getConfiguration()->getAutomaticWindowConfigurationForOfficialMode().getBlockDiagnosticTitle() );
}

VotingExperienceConfiguration & SAESGUIConfigurationManager::getVotingExperienceConfiguration()
{
    return getVotingExperienceGUIConfiguration().getVotingExperience();
}

AbleWidgetConfiguration & SAESGUIConfigurationManager::getAbleWidgetConfiguration()
{
    return getConfiguration()->getAbleWidgetConfiguration();
}

DateDiagnosticWidgetConfiguration & SAESGUIConfigurationManager::getDateDiagnosticWidgetConfiguration()
{
    return getConfiguration()->getDateDiagnosticWidgetConfiguration();
}

ProcessResultWidgetConfiguration & SAESGUIConfigurationManager::getProcessResultWidgetConfiguration()
{
    return getConfiguration()->getProcessResultWidgetConfiguration();
}

ProcessStatusResultsShutdownFunctionalityConfiguration & SAESGUIConfigurationManager::getProcessStatusResultsShutdownFunctionalityConfiguration()
{
    return getConfiguration()->getProcessStatusResultsShutdownFunctionalityConfiguration();
}

SelectionWidgetConfiguration & SAESGUIConfigurationManager::getLanguageSelectionWidgetConfiguration()
{
    return getVotingExperienceGUIConfiguration().getLanguageSelectionWidgetConfiguration();
}

SelectionWidgetConfiguration & SAESGUIConfigurationManager::getCustomAttributeSelectionWidgetConfiguration()
{
    return getVotingExperienceGUIConfiguration().getCustomAttributeSelectionWidgetConfiguration();
}

bool SAESGUIConfigurationManager::getForceExecuteFunctionality()
{
    return getConfiguration()->getForceExecuteFunctionality();
}

GenericWidgetConfiguration & SAESGUIConfigurationManager::getGenericWidgetConfiguration()
{
    return getConfiguration()->getGenericWidgetConfiguration();
}

AlarmBoxDiagnosisConfiguration & SAESGUIConfigurationManager::getAlarmBoxDiagnosisConfiguration()
{
    return getConfiguration()->getAlarmBoxDiagnosisConfiguration();
}

EurnDiagnosisConfiguration & SAESGUIConfigurationManager::getEurnDiagnosisConfiguration()
{
    return getConfiguration()->getEurnDiagnosisConfiguration();
}

LedStripDiagnosisConfiguration & SAESGUIConfigurationManager::getLedStripDiagnosisConfiguration()
{
    return getConfiguration()->getLedStripDiagnosisConfiguration();
}

MenuOptionsConfiguration & SAESGUIConfigurationManager::getMenuOptionsConfiguration()
{
    static bool load = false;

    if ( !load )
    {
        load = true;
        logger->Debug ( "Load menu configuration" );

        Smartmatic::SAES::Runtime::Environment::SaesDirectories saes;
        std::string file = saes.getMenuConfigFileFullPath();
        bool loadConfig = false;

        if ( File::getInstance()->canOpenFile ( file ) )
        {
            try
            {
                Smartmatic::System::SafeBuffer dataOut;
                File::getInstance()->openFileToBuffer ( file, dataOut );
                std::string data;

                if ( dataOut.GetSize() > 0 )
                {
                    data.assign ( ( const char * ) dataOut.GetData(), dataOut.GetSize() );
                }

                std::istringstream istr ( data );
                MenuOptionsConfiguration * config = new MenuOptionsConfiguration ( * ( parseMenuOptionsConfiguration ( istr, Smartmatic::MenudConfigurationTypes::Flags::dont_validate ) ) );
                menuOptionsConfiguration = *config;
                delete config;
                config = NULL;

                loadConfig = true;
            }
            catch ( const ::xsd::cxx::tree::exception< char > & e )
            {
                std::stringstream ss;
                ss  << "FATAL - " << __func__ << " " << e;
                logger->Debug ( ss.str() );
            }
            catch ( ... )
            {
                std::stringstream ss;
                ss  << "FATAL - " << __func__;
                logger->Debug ( ss.str() );
            }
        }

        if ( !loadConfig )
        {
            SaesGUIConfiguration::MenuConfigurationSequence & seq ( getConfiguration()->getMenuConfiguration() );
            SaesGUIConfiguration::MenuConfigurationIterator it;

            MenuOptionsConfiguration::MenuConfigurationSequence menuSeq;

            for ( it = seq.begin();
                    it != seq.end();
                    ++it )
            {
                menuSeq.push_back ( *it );
            }

            menuOptionsConfiguration.setMenuConfiguration ( menuSeq );
        }
    }

    return menuOptionsConfiguration;
}

MenuWidgetConfiguration & SAESGUIConfigurationManager::getMenuWidgetConfiguration()
{
    static bool load = false;

    if ( !load )
    {
        load = true;

        MenuOptionsConfiguration::MenuConfigurationSequence & seq ( getMenuOptionsConfiguration().getMenuConfiguration() );
        MenuOptionsConfiguration::MenuConfigurationIterator it;

        for ( it = seq.begin();
                it != seq.end();
                ++it )
        {
            if ( ( *it ).getMachine() == getStartConfiguration().getInfoMachine() )
            {
                logger->Debug ( "Load menu machine for " + getStartConfiguration().getInfoMachine() );
                menuWidgetConfiguration = *it;
                break;
            }
        }

        if ( seq.empty() )
        {
            ZenityWrapper::showError ( std::string ( "Menu configuration for " ) + getStartConfiguration().getInfoMachine() + " not found", "Error" );
            Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->exitMachine();
        }
    }

    return menuWidgetConfiguration;
}

FunctionalityWizardFlowConfiguration & SAESGUIConfigurationManager::getFunctionalityWizardConfiguration()
{
    static bool load = false;

    if ( !load )
    {
        load = true;
        logger->Debug ( "Load wizard configuration" );

        Smartmatic::SAES::Runtime::Environment::SaesDirectories saes;
        std::string file = saes.getWizardConfigFileFullPath();
        bool loadConfig = false;

        if ( File::getInstance()->canOpenFile ( file ) )
        {
            try
            {
                Smartmatic::System::SafeBuffer dataOut;
                File::getInstance()->openFileToBuffer ( file, dataOut );
                std::string data;

                if ( dataOut.GetSize() > 0 )
                {
                    data.assign ( ( const char * ) dataOut.GetData(), dataOut.GetSize() );
                }

                std::istringstream istr ( data );
                FunctionalityWizardFlowConfiguration * config = new FunctionalityWizardFlowConfiguration ( * ( parseFunctionalityWizardConfiguration ( istr, Smartmatic::WizardConfigurationTypes::Flags::dont_validate ) ) );
                functionalityWizardFlowConfiguration = *config;
                delete config;
                config = NULL;

                loadConfig = true;
            }
            catch ( const ::xsd::cxx::tree::exception< char > & e )
            {
                std::stringstream ss;
                ss  << "FATAL - " << __func__ << " " << e;
                logger->Debug ( ss.str() );
            }
            catch ( ... )
            {
                std::stringstream ss;
                ss  << "FATAL - " << __func__;
                logger->Debug ( ss.str() );
            }
        }

        if ( !loadConfig )
        {
            functionalityWizardFlowConfiguration = getConfiguration()->getFunctionalityWizardConfiguration();
        }
    }

    return functionalityWizardFlowConfiguration;
}

VotingExperienceGUIConfiguration & SAESGUIConfigurationManager::getVotingExperienceGUIConfiguration()
{
    static bool load = false;

    if ( !load )
    {
        load = true;
        logger->Debug ( "Load voting experience configuration" );

        Smartmatic::SAES::Runtime::Environment::SaesDirectories saes;
        std::string file = saes.getVotingExperienceConfigFileFullPath();
        bool loadConfig = false;

        if ( File::getInstance()->canOpenFile ( file ) )
        {
            try
            {
                Smartmatic::System::SafeBuffer dataOut;
                File::getInstance()->openFileToBuffer ( file, dataOut );
                std::string data;

                if ( dataOut.GetSize() > 0 )
                {
                    data.assign ( ( const char * ) dataOut.GetData(), dataOut.GetSize() );
                }

                std::istringstream istr ( data );
                VotingExperienceGUIConfiguration * config = new VotingExperienceGUIConfiguration ( * ( parseVotingExperienceGUIConfiguration ( istr, Smartmatic::VotingExperienceConfigurationTypes::Flags::dont_validate ) ) );
                votingExperienceGUIConfiguration = *config;
                delete config;
                config = NULL;

                loadConfig = true;
            }
            catch ( const ::xsd::cxx::tree::exception< char > & e )
            {
                std::stringstream ss;
                ss  << "FATAL - " << __func__ << " " << e;
                logger->Debug ( ss.str() );
            }
            catch ( ... )
            {
                std::stringstream ss;
                ss  << "FATAL - " << __func__;
                logger->Debug ( ss.str() );
            }
        }

        if ( !loadConfig )
        {
            votingExperienceGUIConfiguration.setCustomAttributeSelectionWidgetConfiguration ( getConfiguration()->getCustomAttributeSelectionWidgetConfiguration() );
            votingExperienceGUIConfiguration.setLanguageSelectionWidgetConfiguration ( getConfiguration()->getLanguageSelectionWidgetConfiguration() );
            votingExperienceGUIConfiguration.setThemesConfiguration ( getConfiguration()->getThemesConfiguration() );
            votingExperienceGUIConfiguration.setVotingExperience ( getConfiguration()->getVotingExperience() );
        }
    }

    return votingExperienceGUIConfiguration;
}

RecountFunctionalityConfiguration & SAESGUIConfigurationManager::getRecountFunctionalityConfiguration()
{
    return getConfiguration()->getRecountFunctionalityConfiguration();
}

CustomRemarkDiagnoseConfiguration & SAESGUIConfigurationManager::getCustomRemarkDiagnoseConfiguration()
{
    return getConfiguration()->getCustomRemarkDiagnoseConfiguration();
}

Smartmatic::SAES::Configuration::MachineModelType SAESGUIConfigurationManager::getCurrentMachineModelConfiguration ()
{
    std::string machineModel;
    OptParser::getInstance()->getArg( OptParser::MACHINE_MODEL, machineModel );

    SaesGUIMachineModelConfiguration::MachineModelSequence machineModels = getConfiguration()->getSaesGUIMachineModelConfiguration().getMachineModel();

    Smartmatic::SAES::Configuration::MachineModelType configuration = machineModels[ 0 ];

    if ( machineModel.empty() == false )
    {
        for ( size_t i = 0; i < machineModels.size(); ++i )
        {
            if ( machineModels[ i ].getId() == machineModel )
            {
                configuration = machineModels[ i ];
                i = machineModels.size();
            }
        }
    }

    return configuration;
}

GeneralDiagnosisConfiguration & SAESGUIConfigurationManager::getGeneralDiagnosisConfiguration()
{
    return getConfiguration()->getGeneralDiagnosisConfiguration();
}

SAESGUIConfigurationManager * SAESGUIConfigurationManager::getInstance ( SAESGUIConfigurationManager * externManager )
{
    if ( ! instance )
    {
        instance = ( externManager == NULL ? new SAESGUIConfigurationManager() : externManager );
    }

    return instance;
}
